#include <tree_sitter/parser.h>

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
#pragma optimize("", off)
#elif defined(__clang__)
#pragma clang optimize off
#elif defined(__GNUC__)
#pragma GCC optimize ("O0")
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 2921
#define LARGE_STATE_COUNT 447
#define SYMBOL_COUNT 448
#define ALIAS_COUNT 23
#define TOKEN_COUNT 211
#define EXTERNAL_TOKEN_COUNT 6
#define FIELD_COUNT 19
#define MAX_ALIAS_SEQUENCE_LENGTH 12
#define PRODUCTION_ID_COUNT 114

enum {
  aux_sym_program_statement_token1 = 1,
  aux_sym_end_program_statement_token1 = 2,
  aux_sym_end_program_statement_token2 = 3,
  aux_sym_module_statement_token1 = 4,
  aux_sym_end_module_statement_token1 = 5,
  aux_sym_submodule_statement_token1 = 6,
  anon_sym_LPAREN = 7,
  anon_sym_COLON = 8,
  anon_sym_RPAREN = 9,
  aux_sym_end_submodule_statement_token1 = 10,
  aux_sym_interface_statement_token1 = 11,
  aux_sym_end_interface_statement_token1 = 12,
  aux_sym_assignment_token1 = 13,
  anon_sym_EQ = 14,
  aux_sym_operator_token1 = 15,
  aux_sym_operator_token2 = 16,
  aux_sym_defined_io_procedure_token1 = 17,
  aux_sym_defined_io_procedure_token2 = 18,
  aux_sym_defined_io_procedure_token3 = 19,
  aux_sym_defined_io_procedure_token4 = 20,
  aux_sym_subroutine_statement_token1 = 21,
  aux_sym_end_subroutine_statement_token1 = 22,
  aux_sym_module_procedure_statement_token1 = 23,
  aux_sym_end_module_procedure_statement_token1 = 24,
  aux_sym_function_statement_token1 = 25,
  aux_sym_language_binding_token1 = 26,
  anon_sym_COMMA = 27,
  aux_sym_procedure_attributes_token1 = 28,
  aux_sym_procedure_attributes_token2 = 29,
  aux_sym_procedure_attributes_token3 = 30,
  aux_sym_procedure_attributes_token4 = 31,
  aux_sym_procedure_attributes_token5 = 32,
  aux_sym_end_function_statement_token1 = 33,
  aux_sym_function_result_token1 = 34,
  aux_sym_contains_statement_token1 = 35,
  aux_sym_use_statement_token1 = 36,
  aux_sym_use_statement_token2 = 37,
  aux_sym_use_statement_token3 = 38,
  anon_sym_COLON_COLON = 39,
  aux_sym_included_items_token1 = 40,
  anon_sym_EQ_GT = 41,
  aux_sym_implicit_statement_token1 = 42,
  aux_sym_implicit_statement_token2 = 43,
  aux_sym_implicit_statement_token3 = 44,
  aux_sym_implicit_statement_token4 = 45,
  aux_sym_save_statement_token1 = 46,
  anon_sym_SLASH = 47,
  aux_sym_private_statement_token1 = 48,
  aux_sym_public_statement_token1 = 49,
  aux_sym_namelist_statement_token1 = 50,
  aux_sym_common_statement_token1 = 51,
  aux_sym_implicit_range_token1 = 52,
  anon_sym_DASH = 53,
  aux_sym_import_statement_token1 = 54,
  aux_sym__import_names_token1 = 55,
  aux_sym_derived_type_definition_token1 = 56,
  aux_sym_abstract_specifier_token1 = 57,
  aux_sym_base_type_specifier_token1 = 58,
  aux_sym_end_type_statement_token1 = 59,
  aux_sym__procedure_kind_token1 = 60,
  aux_sym__procedure_kind_token2 = 61,
  aux_sym__procedure_kind_token3 = 62,
  aux_sym__procedure_kind_token4 = 63,
  aux_sym_procedure_attribute_token1 = 64,
  aux_sym_procedure_attribute_token2 = 65,
  aux_sym_procedure_attribute_token3 = 66,
  aux_sym_procedure_attribute_token4 = 67,
  aux_sym_procedure_attribute_token5 = 68,
  aux_sym_procedure_attribute_token6 = 69,
  aux_sym_variable_attributes_token1 = 70,
  aux_sym_variable_attributes_token2 = 71,
  aux_sym_variable_attributes_token3 = 72,
  aux_sym_variable_attributes_token4 = 73,
  aux_sym_variable_attributes_token5 = 74,
  aux_sym_intrinsic_type_token1 = 75,
  aux_sym_intrinsic_type_token2 = 76,
  aux_sym_intrinsic_type_token3 = 77,
  aux_sym_intrinsic_type_token4 = 78,
  aux_sym_intrinsic_type_token5 = 79,
  aux_sym_intrinsic_type_token6 = 80,
  aux_sym_intrinsic_type_token7 = 81,
  aux_sym_intrinsic_type_token8 = 82,
  aux_sym_intrinsic_type_token9 = 83,
  aux_sym_intrinsic_type_token10 = 84,
  aux_sym_derived_type_token1 = 85,
  anon_sym_STAR = 86,
  aux_sym_size_token1 = 87,
  aux_sym_type_qualifier_token1 = 88,
  aux_sym_type_qualifier_token2 = 89,
  aux_sym_type_qualifier_token3 = 90,
  aux_sym_type_qualifier_token4 = 91,
  aux_sym_type_qualifier_token5 = 92,
  aux_sym_type_qualifier_token6 = 93,
  aux_sym_type_qualifier_token7 = 94,
  aux_sym_type_qualifier_token8 = 95,
  aux_sym_type_qualifier_token9 = 96,
  aux_sym_type_qualifier_token10 = 97,
  aux_sym_type_qualifier_token11 = 98,
  aux_sym_type_qualifier_token12 = 99,
  aux_sym_type_qualifier_token13 = 100,
  aux_sym_type_qualifier_token14 = 101,
  aux_sym_type_qualifier_token15 = 102,
  aux_sym_procedure_qualifier_token1 = 103,
  aux_sym_procedure_qualifier_token2 = 104,
  aux_sym_procedure_qualifier_token3 = 105,
  aux_sym_procedure_qualifier_token4 = 106,
  aux_sym_equivalence_statement_token1 = 107,
  aux_sym_stop_statement_token1 = 108,
  aux_sym_stop_statement_token2 = 109,
  aux_sym_stop_statement_token3 = 110,
  aux_sym_subroutine_call_token1 = 111,
  anon_sym_LT_LT_LT = 112,
  anon_sym_GT_GT_GT = 113,
  aux_sym_keyword_statement_token1 = 114,
  aux_sym_keyword_statement_token2 = 115,
  aux_sym_keyword_statement_token3 = 116,
  aux_sym_keyword_statement_token4 = 117,
  aux_sym_keyword_statement_token5 = 118,
  aux_sym_keyword_statement_token6 = 119,
  aux_sym_keyword_statement_token7 = 120,
  aux_sym_include_statement_token1 = 121,
  aux_sym_data_statement_token1 = 122,
  aux_sym_do_loop_statement_token1 = 123,
  aux_sym_end_do_loop_statement_token1 = 124,
  aux_sym_while_statement_token1 = 125,
  aux_sym_concurrent_header_token1 = 126,
  aux_sym_concurrent_locality_token1 = 127,
  aux_sym_concurrent_locality_token2 = 128,
  aux_sym_concurrent_locality_token3 = 129,
  aux_sym__inline_if_statement_token1 = 130,
  aux_sym__block_if_statement_token1 = 131,
  aux_sym_end_if_statement_token1 = 132,
  aux_sym_elseif_clause_token1 = 133,
  aux_sym_elseif_clause_token2 = 134,
  aux_sym__inline_where_statement_token1 = 135,
  aux_sym_end_where_statement_token1 = 136,
  aux_sym_elsewhere_clause_token1 = 137,
  aux_sym__forall_control_expression_token1 = 138,
  aux_sym_end_forall_statement_token1 = 139,
  aux_sym_select_case_statement_token1 = 140,
  aux_sym_select_case_statement_token2 = 141,
  aux_sym_select_case_statement_token3 = 142,
  aux_sym_select_type_statement_token1 = 143,
  aux_sym_select_rank_statement_token1 = 144,
  aux_sym_select_rank_statement_token2 = 145,
  aux_sym_end_select_statement_token1 = 146,
  aux_sym_type_statement_token1 = 147,
  aux_sym_type_statement_token2 = 148,
  aux_sym_type_statement_token3 = 149,
  aux_sym__class_default_token1 = 150,
  aux_sym_block_construct_token1 = 151,
  aux_sym_end_block_construct_statement_token1 = 152,
  aux_sym_associate_statement_token1 = 153,
  aux_sym_end_associate_statement_token1 = 154,
  aux_sym_format_statement_token1 = 155,
  sym_edit_descriptor = 156,
  aux_sym_hollerith_constant_token1 = 157,
  aux_sym_print_statement_token1 = 158,
  aux_sym_open_statement_token1 = 159,
  aux_sym_close_statement_token1 = 160,
  aux_sym_inquire_statement_token1 = 161,
  aux_sym_enum_statement_token1 = 162,
  aux_sym_enumerator_statement_token1 = 163,
  aux_sym_end_enum_statement_token1 = 164,
  aux_sym_file_position_statement_token1 = 165,
  aux_sym_file_position_statement_token2 = 166,
  aux_sym_file_position_statement_token3 = 167,
  aux_sym_file_position_statement_token4 = 168,
  anon_sym_PERCENT = 169,
  aux_sym_logical_expression_token1 = 170,
  aux_sym_logical_expression_token2 = 171,
  aux_sym_logical_expression_token3 = 172,
  aux_sym_logical_expression_token4 = 173,
  aux_sym_logical_expression_token5 = 174,
  anon_sym_LT = 175,
  aux_sym_relational_expression_token1 = 176,
  anon_sym_GT = 177,
  aux_sym_relational_expression_token2 = 178,
  anon_sym_LT_EQ = 179,
  aux_sym_relational_expression_token3 = 180,
  anon_sym_GT_EQ = 181,
  aux_sym_relational_expression_token4 = 182,
  anon_sym_EQ_EQ = 183,
  aux_sym_relational_expression_token5 = 184,
  anon_sym_SLASH_EQ = 185,
  aux_sym_relational_expression_token6 = 186,
  anon_sym_SLASH_SLASH = 187,
  anon_sym_PLUS = 188,
  anon_sym_STAR_STAR = 189,
  anon_sym_DOT = 190,
  aux_sym_user_defined_operator_token1 = 191,
  sym_assumed_rank = 192,
  anon_sym_LPAREN_SLASH = 193,
  anon_sym_SLASH_RPAREN = 194,
  anon_sym_LBRACK = 195,
  anon_sym_RBRACK = 196,
  sym_boolean_literal = 197,
  aux_sym_null_literal_token1 = 198,
  anon_sym_POUND = 199,
  aux_sym_preproc_file_line_token1 = 200,
  aux_sym_identifier_token1 = 201,
  sym_comment = 202,
  sym__line_continuation = 203,
  sym__semicolon = 204,
  sym__newline = 205,
  sym__integer_literal = 206,
  sym__float_literal = 207,
  sym__boz_literal = 208,
  sym_string_literal = 209,
  sym__end_of_statement = 210,
  sym_translation_unit = 211,
  sym_program = 212,
  sym_program_statement = 213,
  sym_end_program_statement = 214,
  sym_module = 215,
  sym_module_statement = 216,
  sym_end_module_statement = 217,
  sym_submodule = 218,
  sym_submodule_statement = 219,
  sym_end_submodule_statement = 220,
  sym_module_name = 221,
  sym_interface = 222,
  sym_interface_statement = 223,
  sym_end_interface_statement = 224,
  sym_assignment = 225,
  sym_operator = 226,
  sym_defined_io_procedure = 227,
  sym__generic_procedure = 228,
  sym_subroutine = 229,
  sym_subroutine_statement = 230,
  sym_end_subroutine_statement = 231,
  sym_module_procedure = 232,
  sym_module_procedure_statement = 233,
  sym_end_module_procedure_statement = 234,
  sym_function = 235,
  sym_function_statement = 236,
  sym_language_binding = 237,
  aux_sym__callable_interface_qualifers = 238,
  sym_procedure_attributes = 239,
  sym_end_function_statement = 240,
  sym_function_result = 241,
  sym__name = 242,
  sym__parameters = 243,
  sym_parameters = 244,
  sym_internal_procedures = 245,
  sym_contains_statement = 246,
  sym__specification_part = 247,
  sym_use_statement = 248,
  sym_included_items = 249,
  sym_use_alias = 250,
  sym_implicit_statement = 251,
  sym_save_statement = 252,
  sym_private_statement = 253,
  sym_public_statement = 254,
  sym_namelist_statement = 255,
  sym_common_statement = 256,
  sym_variable_group = 257,
  sym_implicit_range = 258,
  sym_import_statement = 259,
  sym__import_names = 260,
  sym_derived_type_definition = 261,
  sym_abstract_specifier = 262,
  sym_access_specifier = 263,
  sym_base_type_specifier = 264,
  sym__derived_type_qualifier = 265,
  sym_derived_type_statement = 266,
  sym_end_type_statement = 267,
  sym__type_name = 268,
  sym_derived_type_procedures = 269,
  sym_procedure_statement = 270,
  sym_binding_name = 271,
  sym_method_name = 272,
  sym__procedure_kind = 273,
  sym_procedure_attribute = 274,
  sym_variable_declaration = 275,
  sym_procedure_declaration = 276,
  sym_variable_modification = 277,
  sym_variable_attributes = 278,
  sym__declaration_targets = 279,
  sym__intrinsic_type = 280,
  sym_intrinsic_type = 281,
  sym_derived_type = 282,
  sym_unlimited_polymorphic = 283,
  sym_size = 284,
  sym_character_length = 285,
  sym_type_qualifier = 286,
  sym_procedure_qualifier = 287,
  sym_parameter_statement = 288,
  sym_parameter_assignment = 289,
  sym_equivalence_statement = 290,
  sym_equivalence_set = 291,
  sym__statements = 292,
  sym_statement_label = 293,
  sym_statement_label_reference = 294,
  sym_stop_statement = 295,
  sym_assignment_statement = 296,
  sym_pointer_association_statement = 297,
  sym_subroutine_call = 298,
  sym_cuda_kernel_argument_list = 299,
  sym_keyword_statement = 300,
  sym_include_statement = 301,
  sym_data_statement = 302,
  sym_data_set = 303,
  sym_data_value = 304,
  sym_do_loop_statement = 305,
  sym_do_label_statement = 306,
  sym_end_do_loop_statement = 307,
  sym_while_statement = 308,
  sym_concurrent_statement = 309,
  sym_concurrent_header = 310,
  sym_concurrent_control = 311,
  sym_concurrent_locality = 312,
  sym_if_statement = 313,
  sym__inline_if_statement = 314,
  sym_arithmetic_if_statement = 315,
  sym__block_if_statement = 316,
  sym_end_if_statement = 317,
  sym_elseif_clause = 318,
  sym_else_clause = 319,
  sym_where_statement = 320,
  sym__inline_where_statement = 321,
  sym__block_where_statement = 322,
  sym_end_where_statement = 323,
  sym_elsewhere_clause = 324,
  sym_forall_statement = 325,
  sym_triplet_spec = 326,
  sym__forall_control_expression = 327,
  sym__inline_forall_statement = 328,
  sym__block_forall_statement = 329,
  sym_end_forall_statement = 330,
  sym_select_case_statement = 331,
  sym_select_type_statement = 332,
  sym_select_rank_statement = 333,
  sym_end_select_statement = 334,
  sym_selector = 335,
  sym_case_statement = 336,
  sym_type_statement = 337,
  sym__class_default = 338,
  sym_case_value_range_list = 339,
  sym_rank_statement = 340,
  sym_block_construct = 341,
  sym_end_block_construct_statement = 342,
  sym_associate_statement = 343,
  sym_association = 344,
  sym_end_associate_statement = 345,
  sym_format_statement = 346,
  sym__transfer_item = 347,
  sym__transfer_items = 348,
  sym_hollerith_constant = 349,
  sym__io_arguments = 350,
  sym_read_statement = 351,
  sym__simple_read_statement = 352,
  sym__parameterized_read_statement = 353,
  sym_print_statement = 354,
  sym_open_statement = 355,
  sym_close_statement = 356,
  sym_write_statement = 357,
  sym_inquire_statement = 358,
  sym_enum = 359,
  sym_enum_statement = 360,
  sym_enumerator_statement = 361,
  sym_end_enum_statement = 362,
  sym_unit_identifier = 363,
  sym_format_identifier = 364,
  sym__file_position_spec = 365,
  sym_file_position_statement = 366,
  sym__io_expressions = 367,
  sym_input_item_list = 368,
  sym_output_item_list = 369,
  sym__expression = 370,
  sym_parenthesized_expression = 371,
  sym_derived_type_member_expression = 372,
  sym_logical_expression = 373,
  sym_relational_expression = 374,
  sym_concatenation_expression = 375,
  sym_math_expression = 376,
  sym_unary_expression = 377,
  sym_user_defined_operator = 378,
  sym_call_expression = 379,
  sym_implied_do_loop_expression = 380,
  sym_argument_list = 381,
  sym_keyword_argument = 382,
  sym_extent_specifier = 383,
  sym_assumed_size = 384,
  sym_assumed_shape = 385,
  sym_block_label_start_expression = 386,
  sym__block_label = 387,
  sym_loop_control_expression = 388,
  sym_array_literal = 389,
  sym__array_constructor_legacy = 390,
  sym__array_constructor_f2003 = 391,
  sym__type_spec = 392,
  sym__ac_value_list = 393,
  sym_complex_literal = 394,
  sym_number_literal = 395,
  sym_null_literal = 396,
  sym_preproc_file_line = 397,
  sym_identifier = 398,
  aux_sym_translation_unit_repeat1 = 399,
  aux_sym_program_repeat1 = 400,
  aux_sym_program_repeat2 = 401,
  aux_sym_interface_repeat1 = 402,
  aux_sym_function_statement_repeat1 = 403,
  aux_sym_procedure_attributes_repeat1 = 404,
  aux_sym_parameters_repeat1 = 405,
  aux_sym_internal_procedures_repeat1 = 406,
  aux_sym_use_statement_repeat1 = 407,
  aux_sym_included_items_repeat1 = 408,
  aux_sym_implicit_statement_repeat1 = 409,
  aux_sym_implicit_statement_repeat2 = 410,
  aux_sym_implicit_statement_repeat3 = 411,
  aux_sym_save_statement_repeat1 = 412,
  aux_sym_private_statement_repeat1 = 413,
  aux_sym_namelist_statement_repeat1 = 414,
  aux_sym_common_statement_repeat1 = 415,
  aux_sym_derived_type_definition_repeat1 = 416,
  aux_sym_derived_type_statement_repeat1 = 417,
  aux_sym_derived_type_procedures_repeat1 = 418,
  aux_sym_procedure_statement_repeat1 = 419,
  aux_sym_procedure_statement_repeat2 = 420,
  aux_sym_variable_declaration_repeat1 = 421,
  aux_sym_variable_modification_repeat1 = 422,
  aux_sym__declaration_targets_repeat1 = 423,
  aux_sym_parameter_statement_repeat1 = 424,
  aux_sym_equivalence_statement_repeat1 = 425,
  aux_sym_equivalence_set_repeat1 = 426,
  aux_sym_cuda_kernel_argument_list_repeat1 = 427,
  aux_sym_keyword_statement_repeat1 = 428,
  aux_sym_data_statement_repeat1 = 429,
  aux_sym_data_set_repeat1 = 430,
  aux_sym_data_value_repeat1 = 431,
  aux_sym_concurrent_statement_repeat1 = 432,
  aux_sym_concurrent_header_repeat1 = 433,
  aux_sym__block_if_statement_repeat1 = 434,
  aux_sym__block_where_statement_repeat1 = 435,
  aux_sym__forall_control_expression_repeat1 = 436,
  aux_sym_select_case_statement_repeat1 = 437,
  aux_sym_select_type_statement_repeat1 = 438,
  aux_sym_select_rank_statement_repeat1 = 439,
  aux_sym_case_value_range_list_repeat1 = 440,
  aux_sym_associate_statement_repeat1 = 441,
  aux_sym__transfer_items_repeat1 = 442,
  aux_sym__io_arguments_repeat1 = 443,
  aux_sym_enum_repeat1 = 444,
  aux_sym_enumerator_statement_repeat1 = 445,
  aux_sym_call_expression_repeat1 = 446,
  aux_sym_argument_list_repeat1 = 447,
  alias_sym_block_label = 448,
  anon_alias_sym_class = 449,
  alias_sym_concurrent_mask = 450,
  anon_alias_sym_default = 451,
  anon_alias_sym_double = 452,
  anon_alias_sym_else = 453,
  anon_alias_sym_end = 454,
  alias_sym_filename = 455,
  anon_alias_sym_label = 456,
  alias_sym_local_name = 457,
  anon_alias_sym_method_name = 458,
  alias_sym_name = 459,
  anon_alias_sym_none = 460,
  alias_sym_preproc_line_number = 461,
  alias_sym_procedure_interface = 462,
  anon_alias_sym_program = 463,
  anon_alias_sym_select = 464,
  anon_alias_sym_sequence = 465,
  anon_alias_sym_statement_label = 466,
  anon_alias_sym_statement_label_reference = 467,
  alias_sym_transfer_items = 468,
  alias_sym_type_member = 469,
  alias_sym_type_name = 470,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [aux_sym_program_statement_token1] = "endprogram",
  [aux_sym_end_program_statement_token1] = "endprogram",
  [aux_sym_end_program_statement_token2] = "endprogram",
  [aux_sym_module_statement_token1] = "module",
  [aux_sym_end_module_statement_token1] = "endmodule",
  [aux_sym_submodule_statement_token1] = "submodule",
  [anon_sym_LPAREN] = "(",
  [anon_sym_COLON] = ":",
  [anon_sym_RPAREN] = ")",
  [aux_sym_end_submodule_statement_token1] = "endsubmodule",
  [aux_sym_interface_statement_token1] = "interface",
  [aux_sym_end_interface_statement_token1] = "endinterface",
  [aux_sym_assignment_token1] = "assignment",
  [anon_sym_EQ] = "=",
  [aux_sym_operator_token1] = "operator",
  [aux_sym_operator_token2] = "operator_token2",
  [aux_sym_defined_io_procedure_token1] = "read",
  [aux_sym_defined_io_procedure_token2] = "write",
  [aux_sym_defined_io_procedure_token3] = "formatted",
  [aux_sym_defined_io_procedure_token4] = "unformatted",
  [aux_sym_subroutine_statement_token1] = "subroutine",
  [aux_sym_end_subroutine_statement_token1] = "endsubroutine",
  [aux_sym_module_procedure_statement_token1] = "procedure",
  [aux_sym_end_module_procedure_statement_token1] = "endprocedure",
  [aux_sym_function_statement_token1] = "function",
  [aux_sym_language_binding_token1] = "bind",
  [anon_sym_COMMA] = ",",
  [aux_sym_procedure_attributes_token1] = "attributes",
  [aux_sym_procedure_attributes_token2] = "global",
  [aux_sym_procedure_attributes_token3] = "device",
  [aux_sym_procedure_attributes_token4] = "host",
  [aux_sym_procedure_attributes_token5] = "grid_global",
  [aux_sym_end_function_statement_token1] = "endfunction",
  [aux_sym_function_result_token1] = "result",
  [aux_sym_contains_statement_token1] = "contains",
  [aux_sym_use_statement_token1] = "use",
  [aux_sym_use_statement_token2] = "intrinsic",
  [aux_sym_use_statement_token3] = "non_intrinsic",
  [anon_sym_COLON_COLON] = "::",
  [aux_sym_included_items_token1] = "only",
  [anon_sym_EQ_GT] = "=>",
  [aux_sym_implicit_statement_token1] = "implicit",
  [aux_sym_implicit_statement_token2] = "none",
  [aux_sym_implicit_statement_token3] = "type",
  [aux_sym_implicit_statement_token4] = "external",
  [aux_sym_save_statement_token1] = "save",
  [anon_sym_SLASH] = "/",
  [aux_sym_private_statement_token1] = "private",
  [aux_sym_public_statement_token1] = "public",
  [aux_sym_namelist_statement_token1] = "namelist",
  [aux_sym_common_statement_token1] = "common",
  [aux_sym_implicit_range_token1] = "implicit_range_token1",
  [anon_sym_DASH] = "-",
  [aux_sym_import_statement_token1] = "import",
  [aux_sym__import_names_token1] = "all",
  [aux_sym_derived_type_definition_token1] = "sequence_statement",
  [aux_sym_abstract_specifier_token1] = "abstract",
  [aux_sym_base_type_specifier_token1] = "extends",
  [aux_sym_end_type_statement_token1] = "endtype",
  [aux_sym__procedure_kind_token1] = "generic",
  [aux_sym__procedure_kind_token2] = "initial",
  [aux_sym__procedure_kind_token3] = "property",
  [aux_sym__procedure_kind_token4] = "final",
  [aux_sym_procedure_attribute_token1] = "deferred",
  [aux_sym_procedure_attribute_token2] = "pass",
  [aux_sym_procedure_attribute_token3] = "nopass",
  [aux_sym_procedure_attribute_token4] = "non_overridable",
  [aux_sym_procedure_attribute_token5] = "family",
  [aux_sym_procedure_attribute_token6] = "pointer",
  [aux_sym_variable_attributes_token1] = "managed",
  [aux_sym_variable_attributes_token2] = "constant",
  [aux_sym_variable_attributes_token3] = "shared",
  [aux_sym_variable_attributes_token4] = "pinned",
  [aux_sym_variable_attributes_token5] = "texture",
  [aux_sym_intrinsic_type_token1] = "byte",
  [aux_sym_intrinsic_type_token2] = "integer",
  [aux_sym_intrinsic_type_token3] = "real",
  [aux_sym_intrinsic_type_token4] = "doubleprecision",
  [aux_sym_intrinsic_type_token5] = "doubleprecision",
  [aux_sym_intrinsic_type_token6] = "doubleprecision",
  [aux_sym_intrinsic_type_token7] = "complex",
  [aux_sym_intrinsic_type_token8] = "doublecomplex",
  [aux_sym_intrinsic_type_token9] = "logical",
  [aux_sym_intrinsic_type_token10] = "character",
  [aux_sym_derived_type_token1] = "derived_type_token1",
  [anon_sym_STAR] = "*",
  [aux_sym_size_token1] = "size_token1",
  [aux_sym_type_qualifier_token1] = "allocatable",
  [aux_sym_type_qualifier_token2] = "automatic",
  [aux_sym_type_qualifier_token3] = "dimension",
  [aux_sym_type_qualifier_token4] = "contiguous",
  [aux_sym_type_qualifier_token5] = "intent",
  [aux_sym_type_qualifier_token6] = "in",
  [aux_sym_type_qualifier_token7] = "out",
  [aux_sym_type_qualifier_token8] = "inout",
  [aux_sym_type_qualifier_token9] = "optional",
  [aux_sym_type_qualifier_token10] = "parameter",
  [aux_sym_type_qualifier_token11] = "protected",
  [aux_sym_type_qualifier_token12] = "static",
  [aux_sym_type_qualifier_token13] = "target",
  [aux_sym_type_qualifier_token14] = "value",
  [aux_sym_type_qualifier_token15] = "volatile",
  [aux_sym_procedure_qualifier_token1] = "elemental",
  [aux_sym_procedure_qualifier_token2] = "impure",
  [aux_sym_procedure_qualifier_token3] = "pure",
  [aux_sym_procedure_qualifier_token4] = "recursive",
  [aux_sym_equivalence_statement_token1] = "equivalence",
  [aux_sym_stop_statement_token1] = "error",
  [aux_sym_stop_statement_token2] = "stop",
  [aux_sym_stop_statement_token3] = "quiet",
  [aux_sym_subroutine_call_token1] = "call",
  [anon_sym_LT_LT_LT] = "<<<",
  [anon_sym_GT_GT_GT] = ">>>",
  [aux_sym_keyword_statement_token1] = "continue",
  [aux_sym_keyword_statement_token2] = "cycle",
  [aux_sym_keyword_statement_token3] = "exit",
  [aux_sym_keyword_statement_token4] = "goto",
  [aux_sym_keyword_statement_token5] = "goto",
  [aux_sym_keyword_statement_token6] = "goto",
  [aux_sym_keyword_statement_token7] = "return",
  [aux_sym_include_statement_token1] = "include",
  [aux_sym_data_statement_token1] = "data",
  [aux_sym_do_loop_statement_token1] = "do",
  [aux_sym_end_do_loop_statement_token1] = "enddo",
  [aux_sym_while_statement_token1] = "while",
  [aux_sym_concurrent_header_token1] = "concurrent",
  [aux_sym_concurrent_locality_token1] = "local",
  [aux_sym_concurrent_locality_token2] = "local_init",
  [aux_sym_concurrent_locality_token3] = "concurrent_locality_token3",
  [aux_sym__inline_if_statement_token1] = "if",
  [aux_sym__block_if_statement_token1] = "then",
  [aux_sym_end_if_statement_token1] = "endif",
  [aux_sym_elseif_clause_token1] = "elsewhere",
  [aux_sym_elseif_clause_token2] = "elseif",
  [aux_sym__inline_where_statement_token1] = "where",
  [aux_sym_end_where_statement_token1] = "endwhere",
  [aux_sym_elsewhere_clause_token1] = "elsewhere",
  [aux_sym__forall_control_expression_token1] = "forall",
  [aux_sym_end_forall_statement_token1] = "endforall",
  [aux_sym_select_case_statement_token1] = "selectcase",
  [aux_sym_select_case_statement_token2] = "case",
  [aux_sym_select_case_statement_token3] = "selectcase",
  [aux_sym_select_type_statement_token1] = "selecttype",
  [aux_sym_select_rank_statement_token1] = "rank",
  [aux_sym_select_rank_statement_token2] = "selectrank",
  [aux_sym_end_select_statement_token1] = "endselect",
  [aux_sym_type_statement_token1] = "typeis",
  [aux_sym_type_statement_token2] = "typeis",
  [aux_sym_type_statement_token3] = "classis",
  [aux_sym__class_default_token1] = "_class_default_token1",
  [aux_sym_block_construct_token1] = "block",
  [aux_sym_end_block_construct_statement_token1] = "endblock",
  [aux_sym_associate_statement_token1] = "associate",
  [aux_sym_end_associate_statement_token1] = "endassociate",
  [aux_sym_format_statement_token1] = "format",
  [sym_edit_descriptor] = "edit_descriptor",
  [aux_sym_hollerith_constant_token1] = "hollerith_constant_token1",
  [aux_sym_print_statement_token1] = "print",
  [aux_sym_open_statement_token1] = "open",
  [aux_sym_close_statement_token1] = "close",
  [aux_sym_inquire_statement_token1] = "inquire",
  [aux_sym_enum_statement_token1] = "enum",
  [aux_sym_enumerator_statement_token1] = "enumerator",
  [aux_sym_end_enum_statement_token1] = "endenum",
  [aux_sym_file_position_statement_token1] = "backspace",
  [aux_sym_file_position_statement_token2] = "endfile",
  [aux_sym_file_position_statement_token3] = "rewind",
  [aux_sym_file_position_statement_token4] = "pause",
  [anon_sym_PERCENT] = "%",
  [aux_sym_logical_expression_token1] = "\\.or\\.",
  [aux_sym_logical_expression_token2] = "\\.and\\.",
  [aux_sym_logical_expression_token3] = "\\.eqv\\.",
  [aux_sym_logical_expression_token4] = "\\.neqv\\.",
  [aux_sym_logical_expression_token5] = "\\.not\\.",
  [anon_sym_LT] = "<",
  [aux_sym_relational_expression_token1] = "\\.lt\\.",
  [anon_sym_GT] = ">",
  [aux_sym_relational_expression_token2] = "\\.gt\\.",
  [anon_sym_LT_EQ] = "<=",
  [aux_sym_relational_expression_token3] = "\\.le\\.",
  [anon_sym_GT_EQ] = ">=",
  [aux_sym_relational_expression_token4] = "\\.ge\\.",
  [anon_sym_EQ_EQ] = "==",
  [aux_sym_relational_expression_token5] = "\\.eq\\.",
  [anon_sym_SLASH_EQ] = "/=",
  [aux_sym_relational_expression_token6] = "\\.ne\\.",
  [anon_sym_SLASH_SLASH] = "//",
  [anon_sym_PLUS] = "+",
  [anon_sym_STAR_STAR] = "**",
  [anon_sym_DOT] = ".",
  [aux_sym_user_defined_operator_token1] = "user_defined_operator_token1",
  [sym_assumed_rank] = "assumed_rank",
  [anon_sym_LPAREN_SLASH] = "(/",
  [anon_sym_SLASH_RPAREN] = "/)",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [sym_boolean_literal] = "boolean_literal",
  [aux_sym_null_literal_token1] = "null",
  [anon_sym_POUND] = "#",
  [aux_sym_preproc_file_line_token1] = "preproc_filename",
  [aux_sym_identifier_token1] = "identifier_token1",
  [sym_comment] = "comment",
  [sym__line_continuation] = "_line_continuation",
  [sym__semicolon] = "_semicolon",
  [sym__newline] = "_newline",
  [sym__integer_literal] = "_integer_literal",
  [sym__float_literal] = "_float_literal",
  [sym__boz_literal] = "_boz_literal",
  [sym_string_literal] = "string_literal",
  [sym__end_of_statement] = "_end_of_statement",
  [sym_translation_unit] = "translation_unit",
  [sym_program] = "program",
  [sym_program_statement] = "program_statement",
  [sym_end_program_statement] = "end_program_statement",
  [sym_module] = "module",
  [sym_module_statement] = "module_statement",
  [sym_end_module_statement] = "end_module_statement",
  [sym_submodule] = "submodule",
  [sym_submodule_statement] = "submodule_statement",
  [sym_end_submodule_statement] = "end_submodule_statement",
  [sym_module_name] = "module_name",
  [sym_interface] = "interface",
  [sym_interface_statement] = "interface_statement",
  [sym_end_interface_statement] = "end_interface_statement",
  [sym_assignment] = "assignment",
  [sym_operator] = "operator",
  [sym_defined_io_procedure] = "defined_io_procedure",
  [sym__generic_procedure] = "_generic_procedure",
  [sym_subroutine] = "subroutine",
  [sym_subroutine_statement] = "subroutine_statement",
  [sym_end_subroutine_statement] = "end_subroutine_statement",
  [sym_module_procedure] = "module_procedure",
  [sym_module_procedure_statement] = "module_procedure_statement",
  [sym_end_module_procedure_statement] = "end_module_procedure_statement",
  [sym_function] = "function",
  [sym_function_statement] = "function_statement",
  [sym_language_binding] = "language_binding",
  [aux_sym__callable_interface_qualifers] = "_callable_interface_qualifers",
  [sym_procedure_attributes] = "procedure_attributes",
  [sym_end_function_statement] = "end_function_statement",
  [sym_function_result] = "function_result",
  [sym__name] = "_name",
  [sym__parameters] = "_parameters",
  [sym_parameters] = "parameters",
  [sym_internal_procedures] = "internal_procedures",
  [sym_contains_statement] = "contains_statement",
  [sym__specification_part] = "_specification_part",
  [sym_use_statement] = "use_statement",
  [sym_included_items] = "included_items",
  [sym_use_alias] = "use_alias",
  [sym_implicit_statement] = "implicit_statement",
  [sym_save_statement] = "save_statement",
  [sym_private_statement] = "private_statement",
  [sym_public_statement] = "public_statement",
  [sym_namelist_statement] = "namelist_statement",
  [sym_common_statement] = "common_statement",
  [sym_variable_group] = "variable_group",
  [sym_implicit_range] = "implicit_range",
  [sym_import_statement] = "import_statement",
  [sym__import_names] = "_import_names",
  [sym_derived_type_definition] = "derived_type_definition",
  [sym_abstract_specifier] = "abstract_specifier",
  [sym_access_specifier] = "access_specifier",
  [sym_base_type_specifier] = "base_type_specifier",
  [sym__derived_type_qualifier] = "_derived_type_qualifier",
  [sym_derived_type_statement] = "derived_type_statement",
  [sym_end_type_statement] = "end_type_statement",
  [sym__type_name] = "_type_name",
  [sym_derived_type_procedures] = "derived_type_procedures",
  [sym_procedure_statement] = "procedure_statement",
  [sym_binding_name] = "binding_name",
  [sym_method_name] = "method_name",
  [sym__procedure_kind] = "_procedure_kind",
  [sym_procedure_attribute] = "procedure_attribute",
  [sym_variable_declaration] = "variable_declaration",
  [sym_procedure_declaration] = "procedure",
  [sym_variable_modification] = "variable_modification",
  [sym_variable_attributes] = "variable_attributes",
  [sym__declaration_targets] = "_declaration_targets",
  [sym__intrinsic_type] = "_intrinsic_type",
  [sym_intrinsic_type] = "intrinsic_type",
  [sym_derived_type] = "derived_type",
  [sym_unlimited_polymorphic] = "unlimited_polymorphic",
  [sym_size] = "size",
  [sym_character_length] = "character_length",
  [sym_type_qualifier] = "type_qualifier",
  [sym_procedure_qualifier] = "procedure_qualifier",
  [sym_parameter_statement] = "parameter_statement",
  [sym_parameter_assignment] = "parameter_assignment",
  [sym_equivalence_statement] = "equivalence_statement",
  [sym_equivalence_set] = "equivalence_set",
  [sym__statements] = "_statements",
  [sym_statement_label] = "statement_label",
  [sym_statement_label_reference] = "statement_label_reference",
  [sym_stop_statement] = "stop_statement",
  [sym_assignment_statement] = "assignment_statement",
  [sym_pointer_association_statement] = "pointer_association_statement",
  [sym_subroutine_call] = "subroutine_call",
  [sym_cuda_kernel_argument_list] = "cuda_kernel_argument_list",
  [sym_keyword_statement] = "keyword_statement",
  [sym_include_statement] = "include_statement",
  [sym_data_statement] = "data_statement",
  [sym_data_set] = "data_set",
  [sym_data_value] = "data_value",
  [sym_do_loop_statement] = "do_loop_statement",
  [sym_do_label_statement] = "do_label_statement",
  [sym_end_do_loop_statement] = "end_do_loop_statement",
  [sym_while_statement] = "while_statement",
  [sym_concurrent_statement] = "concurrent_statement",
  [sym_concurrent_header] = "concurrent_header",
  [sym_concurrent_control] = "concurrent_control",
  [sym_concurrent_locality] = "concurrent_locality",
  [sym_if_statement] = "if_statement",
  [sym__inline_if_statement] = "_inline_if_statement",
  [sym_arithmetic_if_statement] = "arithmetic_if_statement",
  [sym__block_if_statement] = "_block_if_statement",
  [sym_end_if_statement] = "end_if_statement",
  [sym_elseif_clause] = "elseif_clause",
  [sym_else_clause] = "else_clause",
  [sym_where_statement] = "where_statement",
  [sym__inline_where_statement] = "_inline_where_statement",
  [sym__block_where_statement] = "_block_where_statement",
  [sym_end_where_statement] = "end_where_statement",
  [sym_elsewhere_clause] = "elsewhere_clause",
  [sym_forall_statement] = "forall_statement",
  [sym_triplet_spec] = "triplet_spec",
  [sym__forall_control_expression] = "_forall_control_expression",
  [sym__inline_forall_statement] = "_inline_forall_statement",
  [sym__block_forall_statement] = "_block_forall_statement",
  [sym_end_forall_statement] = "end_forall_statement",
  [sym_select_case_statement] = "select_case_statement",
  [sym_select_type_statement] = "select_type_statement",
  [sym_select_rank_statement] = "select_rank_statement",
  [sym_end_select_statement] = "end_select_statement",
  [sym_selector] = "selector",
  [sym_case_statement] = "case_statement",
  [sym_type_statement] = "type_statement",
  [sym__class_default] = "default",
  [sym_case_value_range_list] = "case_value_range_list",
  [sym_rank_statement] = "rank_statement",
  [sym_block_construct] = "block_construct",
  [sym_end_block_construct_statement] = "end_block_construct_statement",
  [sym_associate_statement] = "associate_statement",
  [sym_association] = "association",
  [sym_end_associate_statement] = "end_associate_statement",
  [sym_format_statement] = "format_statement",
  [sym__transfer_item] = "_transfer_item",
  [sym__transfer_items] = "_transfer_items",
  [sym_hollerith_constant] = "hollerith_constant",
  [sym__io_arguments] = "_io_arguments",
  [sym_read_statement] = "read_statement",
  [sym__simple_read_statement] = "_simple_read_statement",
  [sym__parameterized_read_statement] = "_parameterized_read_statement",
  [sym_print_statement] = "print_statement",
  [sym_open_statement] = "open_statement",
  [sym_close_statement] = "close_statement",
  [sym_write_statement] = "write_statement",
  [sym_inquire_statement] = "inquire_statement",
  [sym_enum] = "enum",
  [sym_enum_statement] = "enum_statement",
  [sym_enumerator_statement] = "enumerator_statement",
  [sym_end_enum_statement] = "end_enum_statement",
  [sym_unit_identifier] = "unit_identifier",
  [sym_format_identifier] = "format_identifier",
  [sym__file_position_spec] = "_file_position_spec",
  [sym_file_position_statement] = "file_position_statement",
  [sym__io_expressions] = "_io_expressions",
  [sym_input_item_list] = "input_item_list",
  [sym_output_item_list] = "output_item_list",
  [sym__expression] = "_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_derived_type_member_expression] = "derived_type_member_expression",
  [sym_logical_expression] = "logical_expression",
  [sym_relational_expression] = "relational_expression",
  [sym_concatenation_expression] = "concatenation_expression",
  [sym_math_expression] = "math_expression",
  [sym_unary_expression] = "unary_expression",
  [sym_user_defined_operator] = "user_defined_operator",
  [sym_call_expression] = "call_expression",
  [sym_implied_do_loop_expression] = "implied_do_loop_expression",
  [sym_argument_list] = "argument_list",
  [sym_keyword_argument] = "keyword_argument",
  [sym_extent_specifier] = "extent_specifier",
  [sym_assumed_size] = "assumed_size",
  [sym_assumed_shape] = "assumed_shape",
  [sym_block_label_start_expression] = "block_label_start_expression",
  [sym__block_label] = "_block_label",
  [sym_loop_control_expression] = "loop_control_expression",
  [sym_array_literal] = "array_literal",
  [sym__array_constructor_legacy] = "_array_constructor_legacy",
  [sym__array_constructor_f2003] = "_array_constructor_f2003",
  [sym__type_spec] = "_type_spec",
  [sym__ac_value_list] = "_ac_value_list",
  [sym_complex_literal] = "complex_literal",
  [sym_number_literal] = "number_literal",
  [sym_null_literal] = "null_literal",
  [sym_preproc_file_line] = "preproc_file_line",
  [sym_identifier] = "identifier",
  [aux_sym_translation_unit_repeat1] = "translation_unit_repeat1",
  [aux_sym_program_repeat1] = "program_repeat1",
  [aux_sym_program_repeat2] = "program_repeat2",
  [aux_sym_interface_repeat1] = "interface_repeat1",
  [aux_sym_function_statement_repeat1] = "function_statement_repeat1",
  [aux_sym_procedure_attributes_repeat1] = "procedure_attributes_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym_internal_procedures_repeat1] = "internal_procedures_repeat1",
  [aux_sym_use_statement_repeat1] = "use_statement_repeat1",
  [aux_sym_included_items_repeat1] = "included_items_repeat1",
  [aux_sym_implicit_statement_repeat1] = "implicit_statement_repeat1",
  [aux_sym_implicit_statement_repeat2] = "implicit_statement_repeat2",
  [aux_sym_implicit_statement_repeat3] = "implicit_statement_repeat3",
  [aux_sym_save_statement_repeat1] = "save_statement_repeat1",
  [aux_sym_private_statement_repeat1] = "private_statement_repeat1",
  [aux_sym_namelist_statement_repeat1] = "namelist_statement_repeat1",
  [aux_sym_common_statement_repeat1] = "common_statement_repeat1",
  [aux_sym_derived_type_definition_repeat1] = "derived_type_definition_repeat1",
  [aux_sym_derived_type_statement_repeat1] = "derived_type_statement_repeat1",
  [aux_sym_derived_type_procedures_repeat1] = "derived_type_procedures_repeat1",
  [aux_sym_procedure_statement_repeat1] = "procedure_statement_repeat1",
  [aux_sym_procedure_statement_repeat2] = "procedure_statement_repeat2",
  [aux_sym_variable_declaration_repeat1] = "variable_declaration_repeat1",
  [aux_sym_variable_modification_repeat1] = "variable_modification_repeat1",
  [aux_sym__declaration_targets_repeat1] = "_declaration_targets_repeat1",
  [aux_sym_parameter_statement_repeat1] = "parameter_statement_repeat1",
  [aux_sym_equivalence_statement_repeat1] = "equivalence_statement_repeat1",
  [aux_sym_equivalence_set_repeat1] = "equivalence_set_repeat1",
  [aux_sym_cuda_kernel_argument_list_repeat1] = "cuda_kernel_argument_list_repeat1",
  [aux_sym_keyword_statement_repeat1] = "keyword_statement_repeat1",
  [aux_sym_data_statement_repeat1] = "data_statement_repeat1",
  [aux_sym_data_set_repeat1] = "data_set_repeat1",
  [aux_sym_data_value_repeat1] = "data_value_repeat1",
  [aux_sym_concurrent_statement_repeat1] = "concurrent_statement_repeat1",
  [aux_sym_concurrent_header_repeat1] = "concurrent_header_repeat1",
  [aux_sym__block_if_statement_repeat1] = "_block_if_statement_repeat1",
  [aux_sym__block_where_statement_repeat1] = "_block_where_statement_repeat1",
  [aux_sym__forall_control_expression_repeat1] = "_forall_control_expression_repeat1",
  [aux_sym_select_case_statement_repeat1] = "select_case_statement_repeat1",
  [aux_sym_select_type_statement_repeat1] = "select_type_statement_repeat1",
  [aux_sym_select_rank_statement_repeat1] = "select_rank_statement_repeat1",
  [aux_sym_case_value_range_list_repeat1] = "case_value_range_list_repeat1",
  [aux_sym_associate_statement_repeat1] = "associate_statement_repeat1",
  [aux_sym__transfer_items_repeat1] = "_transfer_items_repeat1",
  [aux_sym__io_arguments_repeat1] = "_io_arguments_repeat1",
  [aux_sym_enum_repeat1] = "enum_repeat1",
  [aux_sym_enumerator_statement_repeat1] = "enumerator_statement_repeat1",
  [aux_sym_call_expression_repeat1] = "call_expression_repeat1",
  [aux_sym_argument_list_repeat1] = "argument_list_repeat1",
  [alias_sym_block_label] = "block_label",
  [anon_alias_sym_class] = "class",
  [alias_sym_concurrent_mask] = "concurrent_mask",
  [anon_alias_sym_default] = "default",
  [anon_alias_sym_double] = "double",
  [anon_alias_sym_else] = "else",
  [anon_alias_sym_end] = "end",
  [alias_sym_filename] = "filename",
  [anon_alias_sym_label] = "label",
  [alias_sym_local_name] = "local_name",
  [anon_alias_sym_method_name] = "method_name",
  [alias_sym_name] = "name",
  [anon_alias_sym_none] = "none",
  [alias_sym_preproc_line_number] = "preproc_line_number",
  [alias_sym_procedure_interface] = "procedure_interface",
  [anon_alias_sym_program] = "program",
  [anon_alias_sym_select] = "select",
  [anon_alias_sym_sequence] = "sequence",
  [anon_alias_sym_statement_label] = "statement_label",
  [anon_alias_sym_statement_label_reference] = "statement_label_reference",
  [alias_sym_transfer_items] = "transfer_items",
  [alias_sym_type_member] = "type_member",
  [alias_sym_type_name] = "type_name",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [aux_sym_program_statement_token1] = aux_sym_program_statement_token1,
  [aux_sym_end_program_statement_token1] = aux_sym_program_statement_token1,
  [aux_sym_end_program_statement_token2] = aux_sym_program_statement_token1,
  [aux_sym_module_statement_token1] = aux_sym_module_statement_token1,
  [aux_sym_end_module_statement_token1] = aux_sym_end_module_statement_token1,
  [aux_sym_submodule_statement_token1] = aux_sym_submodule_statement_token1,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [aux_sym_end_submodule_statement_token1] = aux_sym_end_submodule_statement_token1,
  [aux_sym_interface_statement_token1] = aux_sym_interface_statement_token1,
  [aux_sym_end_interface_statement_token1] = aux_sym_end_interface_statement_token1,
  [aux_sym_assignment_token1] = aux_sym_assignment_token1,
  [anon_sym_EQ] = anon_sym_EQ,
  [aux_sym_operator_token1] = aux_sym_operator_token1,
  [aux_sym_operator_token2] = aux_sym_operator_token2,
  [aux_sym_defined_io_procedure_token1] = aux_sym_defined_io_procedure_token1,
  [aux_sym_defined_io_procedure_token2] = aux_sym_defined_io_procedure_token2,
  [aux_sym_defined_io_procedure_token3] = aux_sym_defined_io_procedure_token3,
  [aux_sym_defined_io_procedure_token4] = aux_sym_defined_io_procedure_token4,
  [aux_sym_subroutine_statement_token1] = aux_sym_subroutine_statement_token1,
  [aux_sym_end_subroutine_statement_token1] = aux_sym_end_subroutine_statement_token1,
  [aux_sym_module_procedure_statement_token1] = aux_sym_module_procedure_statement_token1,
  [aux_sym_end_module_procedure_statement_token1] = aux_sym_end_module_procedure_statement_token1,
  [aux_sym_function_statement_token1] = aux_sym_function_statement_token1,
  [aux_sym_language_binding_token1] = aux_sym_language_binding_token1,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [aux_sym_procedure_attributes_token1] = aux_sym_procedure_attributes_token1,
  [aux_sym_procedure_attributes_token2] = aux_sym_procedure_attributes_token2,
  [aux_sym_procedure_attributes_token3] = aux_sym_procedure_attributes_token3,
  [aux_sym_procedure_attributes_token4] = aux_sym_procedure_attributes_token4,
  [aux_sym_procedure_attributes_token5] = aux_sym_procedure_attributes_token5,
  [aux_sym_end_function_statement_token1] = aux_sym_end_function_statement_token1,
  [aux_sym_function_result_token1] = aux_sym_function_result_token1,
  [aux_sym_contains_statement_token1] = aux_sym_contains_statement_token1,
  [aux_sym_use_statement_token1] = aux_sym_use_statement_token1,
  [aux_sym_use_statement_token2] = aux_sym_use_statement_token2,
  [aux_sym_use_statement_token3] = aux_sym_use_statement_token3,
  [anon_sym_COLON_COLON] = anon_sym_COLON_COLON,
  [aux_sym_included_items_token1] = aux_sym_included_items_token1,
  [anon_sym_EQ_GT] = anon_sym_EQ_GT,
  [aux_sym_implicit_statement_token1] = aux_sym_implicit_statement_token1,
  [aux_sym_implicit_statement_token2] = aux_sym_implicit_statement_token2,
  [aux_sym_implicit_statement_token3] = aux_sym_implicit_statement_token3,
  [aux_sym_implicit_statement_token4] = aux_sym_implicit_statement_token4,
  [aux_sym_save_statement_token1] = aux_sym_save_statement_token1,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [aux_sym_private_statement_token1] = aux_sym_private_statement_token1,
  [aux_sym_public_statement_token1] = aux_sym_public_statement_token1,
  [aux_sym_namelist_statement_token1] = aux_sym_namelist_statement_token1,
  [aux_sym_common_statement_token1] = aux_sym_common_statement_token1,
  [aux_sym_implicit_range_token1] = aux_sym_implicit_range_token1,
  [anon_sym_DASH] = anon_sym_DASH,
  [aux_sym_import_statement_token1] = aux_sym_import_statement_token1,
  [aux_sym__import_names_token1] = aux_sym__import_names_token1,
  [aux_sym_derived_type_definition_token1] = aux_sym_derived_type_definition_token1,
  [aux_sym_abstract_specifier_token1] = aux_sym_abstract_specifier_token1,
  [aux_sym_base_type_specifier_token1] = aux_sym_base_type_specifier_token1,
  [aux_sym_end_type_statement_token1] = aux_sym_end_type_statement_token1,
  [aux_sym__procedure_kind_token1] = aux_sym__procedure_kind_token1,
  [aux_sym__procedure_kind_token2] = aux_sym__procedure_kind_token2,
  [aux_sym__procedure_kind_token3] = aux_sym__procedure_kind_token3,
  [aux_sym__procedure_kind_token4] = aux_sym__procedure_kind_token4,
  [aux_sym_procedure_attribute_token1] = aux_sym_procedure_attribute_token1,
  [aux_sym_procedure_attribute_token2] = aux_sym_procedure_attribute_token2,
  [aux_sym_procedure_attribute_token3] = aux_sym_procedure_attribute_token3,
  [aux_sym_procedure_attribute_token4] = aux_sym_procedure_attribute_token4,
  [aux_sym_procedure_attribute_token5] = aux_sym_procedure_attribute_token5,
  [aux_sym_procedure_attribute_token6] = aux_sym_procedure_attribute_token6,
  [aux_sym_variable_attributes_token1] = aux_sym_variable_attributes_token1,
  [aux_sym_variable_attributes_token2] = aux_sym_variable_attributes_token2,
  [aux_sym_variable_attributes_token3] = aux_sym_variable_attributes_token3,
  [aux_sym_variable_attributes_token4] = aux_sym_variable_attributes_token4,
  [aux_sym_variable_attributes_token5] = aux_sym_variable_attributes_token5,
  [aux_sym_intrinsic_type_token1] = aux_sym_intrinsic_type_token1,
  [aux_sym_intrinsic_type_token2] = aux_sym_intrinsic_type_token2,
  [aux_sym_intrinsic_type_token3] = aux_sym_intrinsic_type_token3,
  [aux_sym_intrinsic_type_token4] = aux_sym_intrinsic_type_token4,
  [aux_sym_intrinsic_type_token5] = aux_sym_intrinsic_type_token4,
  [aux_sym_intrinsic_type_token6] = aux_sym_intrinsic_type_token4,
  [aux_sym_intrinsic_type_token7] = aux_sym_intrinsic_type_token7,
  [aux_sym_intrinsic_type_token8] = aux_sym_intrinsic_type_token8,
  [aux_sym_intrinsic_type_token9] = aux_sym_intrinsic_type_token9,
  [aux_sym_intrinsic_type_token10] = aux_sym_intrinsic_type_token10,
  [aux_sym_derived_type_token1] = aux_sym_derived_type_token1,
  [anon_sym_STAR] = anon_sym_STAR,
  [aux_sym_size_token1] = aux_sym_size_token1,
  [aux_sym_type_qualifier_token1] = aux_sym_type_qualifier_token1,
  [aux_sym_type_qualifier_token2] = aux_sym_type_qualifier_token2,
  [aux_sym_type_qualifier_token3] = aux_sym_type_qualifier_token3,
  [aux_sym_type_qualifier_token4] = aux_sym_type_qualifier_token4,
  [aux_sym_type_qualifier_token5] = aux_sym_type_qualifier_token5,
  [aux_sym_type_qualifier_token6] = aux_sym_type_qualifier_token6,
  [aux_sym_type_qualifier_token7] = aux_sym_type_qualifier_token7,
  [aux_sym_type_qualifier_token8] = aux_sym_type_qualifier_token8,
  [aux_sym_type_qualifier_token9] = aux_sym_type_qualifier_token9,
  [aux_sym_type_qualifier_token10] = aux_sym_type_qualifier_token10,
  [aux_sym_type_qualifier_token11] = aux_sym_type_qualifier_token11,
  [aux_sym_type_qualifier_token12] = aux_sym_type_qualifier_token12,
  [aux_sym_type_qualifier_token13] = aux_sym_type_qualifier_token13,
  [aux_sym_type_qualifier_token14] = aux_sym_type_qualifier_token14,
  [aux_sym_type_qualifier_token15] = aux_sym_type_qualifier_token15,
  [aux_sym_procedure_qualifier_token1] = aux_sym_procedure_qualifier_token1,
  [aux_sym_procedure_qualifier_token2] = aux_sym_procedure_qualifier_token2,
  [aux_sym_procedure_qualifier_token3] = aux_sym_procedure_qualifier_token3,
  [aux_sym_procedure_qualifier_token4] = aux_sym_procedure_qualifier_token4,
  [aux_sym_equivalence_statement_token1] = aux_sym_equivalence_statement_token1,
  [aux_sym_stop_statement_token1] = aux_sym_stop_statement_token1,
  [aux_sym_stop_statement_token2] = aux_sym_stop_statement_token2,
  [aux_sym_stop_statement_token3] = aux_sym_stop_statement_token3,
  [aux_sym_subroutine_call_token1] = aux_sym_subroutine_call_token1,
  [anon_sym_LT_LT_LT] = anon_sym_LT_LT_LT,
  [anon_sym_GT_GT_GT] = anon_sym_GT_GT_GT,
  [aux_sym_keyword_statement_token1] = aux_sym_keyword_statement_token1,
  [aux_sym_keyword_statement_token2] = aux_sym_keyword_statement_token2,
  [aux_sym_keyword_statement_token3] = aux_sym_keyword_statement_token3,
  [aux_sym_keyword_statement_token4] = aux_sym_keyword_statement_token4,
  [aux_sym_keyword_statement_token5] = aux_sym_keyword_statement_token4,
  [aux_sym_keyword_statement_token6] = aux_sym_keyword_statement_token4,
  [aux_sym_keyword_statement_token7] = aux_sym_keyword_statement_token7,
  [aux_sym_include_statement_token1] = aux_sym_include_statement_token1,
  [aux_sym_data_statement_token1] = aux_sym_data_statement_token1,
  [aux_sym_do_loop_statement_token1] = aux_sym_do_loop_statement_token1,
  [aux_sym_end_do_loop_statement_token1] = aux_sym_end_do_loop_statement_token1,
  [aux_sym_while_statement_token1] = aux_sym_while_statement_token1,
  [aux_sym_concurrent_header_token1] = aux_sym_concurrent_header_token1,
  [aux_sym_concurrent_locality_token1] = aux_sym_concurrent_locality_token1,
  [aux_sym_concurrent_locality_token2] = aux_sym_concurrent_locality_token2,
  [aux_sym_concurrent_locality_token3] = aux_sym_concurrent_locality_token3,
  [aux_sym__inline_if_statement_token1] = aux_sym__inline_if_statement_token1,
  [aux_sym__block_if_statement_token1] = aux_sym__block_if_statement_token1,
  [aux_sym_end_if_statement_token1] = aux_sym_end_if_statement_token1,
  [aux_sym_elseif_clause_token1] = aux_sym_elseif_clause_token1,
  [aux_sym_elseif_clause_token2] = aux_sym_elseif_clause_token2,
  [aux_sym__inline_where_statement_token1] = aux_sym__inline_where_statement_token1,
  [aux_sym_end_where_statement_token1] = aux_sym_end_where_statement_token1,
  [aux_sym_elsewhere_clause_token1] = aux_sym_elseif_clause_token1,
  [aux_sym__forall_control_expression_token1] = aux_sym__forall_control_expression_token1,
  [aux_sym_end_forall_statement_token1] = aux_sym_end_forall_statement_token1,
  [aux_sym_select_case_statement_token1] = aux_sym_select_case_statement_token1,
  [aux_sym_select_case_statement_token2] = aux_sym_select_case_statement_token2,
  [aux_sym_select_case_statement_token3] = aux_sym_select_case_statement_token1,
  [aux_sym_select_type_statement_token1] = aux_sym_select_type_statement_token1,
  [aux_sym_select_rank_statement_token1] = aux_sym_select_rank_statement_token1,
  [aux_sym_select_rank_statement_token2] = aux_sym_select_rank_statement_token2,
  [aux_sym_end_select_statement_token1] = aux_sym_end_select_statement_token1,
  [aux_sym_type_statement_token1] = aux_sym_type_statement_token1,
  [aux_sym_type_statement_token2] = aux_sym_type_statement_token1,
  [aux_sym_type_statement_token3] = aux_sym_type_statement_token3,
  [aux_sym__class_default_token1] = aux_sym__class_default_token1,
  [aux_sym_block_construct_token1] = aux_sym_block_construct_token1,
  [aux_sym_end_block_construct_statement_token1] = aux_sym_end_block_construct_statement_token1,
  [aux_sym_associate_statement_token1] = aux_sym_associate_statement_token1,
  [aux_sym_end_associate_statement_token1] = aux_sym_end_associate_statement_token1,
  [aux_sym_format_statement_token1] = aux_sym_format_statement_token1,
  [sym_edit_descriptor] = sym_edit_descriptor,
  [aux_sym_hollerith_constant_token1] = aux_sym_hollerith_constant_token1,
  [aux_sym_print_statement_token1] = aux_sym_print_statement_token1,
  [aux_sym_open_statement_token1] = aux_sym_open_statement_token1,
  [aux_sym_close_statement_token1] = aux_sym_close_statement_token1,
  [aux_sym_inquire_statement_token1] = aux_sym_inquire_statement_token1,
  [aux_sym_enum_statement_token1] = aux_sym_enum_statement_token1,
  [aux_sym_enumerator_statement_token1] = aux_sym_enumerator_statement_token1,
  [aux_sym_end_enum_statement_token1] = aux_sym_end_enum_statement_token1,
  [aux_sym_file_position_statement_token1] = aux_sym_file_position_statement_token1,
  [aux_sym_file_position_statement_token2] = aux_sym_file_position_statement_token2,
  [aux_sym_file_position_statement_token3] = aux_sym_file_position_statement_token3,
  [aux_sym_file_position_statement_token4] = aux_sym_file_position_statement_token4,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [aux_sym_logical_expression_token1] = aux_sym_logical_expression_token1,
  [aux_sym_logical_expression_token2] = aux_sym_logical_expression_token2,
  [aux_sym_logical_expression_token3] = aux_sym_logical_expression_token3,
  [aux_sym_logical_expression_token4] = aux_sym_logical_expression_token4,
  [aux_sym_logical_expression_token5] = aux_sym_logical_expression_token5,
  [anon_sym_LT] = anon_sym_LT,
  [aux_sym_relational_expression_token1] = aux_sym_relational_expression_token1,
  [anon_sym_GT] = anon_sym_GT,
  [aux_sym_relational_expression_token2] = aux_sym_relational_expression_token2,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [aux_sym_relational_expression_token3] = aux_sym_relational_expression_token3,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [aux_sym_relational_expression_token4] = aux_sym_relational_expression_token4,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [aux_sym_relational_expression_token5] = aux_sym_relational_expression_token5,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [aux_sym_relational_expression_token6] = aux_sym_relational_expression_token6,
  [anon_sym_SLASH_SLASH] = anon_sym_SLASH_SLASH,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_STAR_STAR] = anon_sym_STAR_STAR,
  [anon_sym_DOT] = anon_sym_DOT,
  [aux_sym_user_defined_operator_token1] = aux_sym_user_defined_operator_token1,
  [sym_assumed_rank] = sym_assumed_rank,
  [anon_sym_LPAREN_SLASH] = anon_sym_LPAREN_SLASH,
  [anon_sym_SLASH_RPAREN] = anon_sym_SLASH_RPAREN,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [sym_boolean_literal] = sym_boolean_literal,
  [aux_sym_null_literal_token1] = aux_sym_null_literal_token1,
  [anon_sym_POUND] = anon_sym_POUND,
  [aux_sym_preproc_file_line_token1] = aux_sym_preproc_file_line_token1,
  [aux_sym_identifier_token1] = aux_sym_identifier_token1,
  [sym_comment] = sym_comment,
  [sym__line_continuation] = sym__line_continuation,
  [sym__semicolon] = sym__semicolon,
  [sym__newline] = sym__newline,
  [sym__integer_literal] = sym__integer_literal,
  [sym__float_literal] = sym__float_literal,
  [sym__boz_literal] = sym__boz_literal,
  [sym_string_literal] = sym_string_literal,
  [sym__end_of_statement] = sym__end_of_statement,
  [sym_translation_unit] = sym_translation_unit,
  [sym_program] = sym_program,
  [sym_program_statement] = sym_program_statement,
  [sym_end_program_statement] = sym_end_program_statement,
  [sym_module] = sym_module,
  [sym_module_statement] = sym_module_statement,
  [sym_end_module_statement] = sym_end_module_statement,
  [sym_submodule] = sym_submodule,
  [sym_submodule_statement] = sym_submodule_statement,
  [sym_end_submodule_statement] = sym_end_submodule_statement,
  [sym_module_name] = sym_module_name,
  [sym_interface] = sym_interface,
  [sym_interface_statement] = sym_interface_statement,
  [sym_end_interface_statement] = sym_end_interface_statement,
  [sym_assignment] = sym_assignment,
  [sym_operator] = sym_operator,
  [sym_defined_io_procedure] = sym_defined_io_procedure,
  [sym__generic_procedure] = sym__generic_procedure,
  [sym_subroutine] = sym_subroutine,
  [sym_subroutine_statement] = sym_subroutine_statement,
  [sym_end_subroutine_statement] = sym_end_subroutine_statement,
  [sym_module_procedure] = sym_module_procedure,
  [sym_module_procedure_statement] = sym_module_procedure_statement,
  [sym_end_module_procedure_statement] = sym_end_module_procedure_statement,
  [sym_function] = sym_function,
  [sym_function_statement] = sym_function_statement,
  [sym_language_binding] = sym_language_binding,
  [aux_sym__callable_interface_qualifers] = aux_sym__callable_interface_qualifers,
  [sym_procedure_attributes] = sym_procedure_attributes,
  [sym_end_function_statement] = sym_end_function_statement,
  [sym_function_result] = sym_function_result,
  [sym__name] = sym__name,
  [sym__parameters] = sym__parameters,
  [sym_parameters] = sym_parameters,
  [sym_internal_procedures] = sym_internal_procedures,
  [sym_contains_statement] = sym_contains_statement,
  [sym__specification_part] = sym__specification_part,
  [sym_use_statement] = sym_use_statement,
  [sym_included_items] = sym_included_items,
  [sym_use_alias] = sym_use_alias,
  [sym_implicit_statement] = sym_implicit_statement,
  [sym_save_statement] = sym_save_statement,
  [sym_private_statement] = sym_private_statement,
  [sym_public_statement] = sym_public_statement,
  [sym_namelist_statement] = sym_namelist_statement,
  [sym_common_statement] = sym_common_statement,
  [sym_variable_group] = sym_variable_group,
  [sym_implicit_range] = sym_implicit_range,
  [sym_import_statement] = sym_import_statement,
  [sym__import_names] = sym__import_names,
  [sym_derived_type_definition] = sym_derived_type_definition,
  [sym_abstract_specifier] = sym_abstract_specifier,
  [sym_access_specifier] = sym_access_specifier,
  [sym_base_type_specifier] = sym_base_type_specifier,
  [sym__derived_type_qualifier] = sym__derived_type_qualifier,
  [sym_derived_type_statement] = sym_derived_type_statement,
  [sym_end_type_statement] = sym_end_type_statement,
  [sym__type_name] = sym__type_name,
  [sym_derived_type_procedures] = sym_derived_type_procedures,
  [sym_procedure_statement] = sym_procedure_statement,
  [sym_binding_name] = sym_binding_name,
  [sym_method_name] = sym_method_name,
  [sym__procedure_kind] = sym__procedure_kind,
  [sym_procedure_attribute] = sym_procedure_attribute,
  [sym_variable_declaration] = sym_variable_declaration,
  [sym_procedure_declaration] = sym_procedure_declaration,
  [sym_variable_modification] = sym_variable_modification,
  [sym_variable_attributes] = sym_variable_attributes,
  [sym__declaration_targets] = sym__declaration_targets,
  [sym__intrinsic_type] = sym__intrinsic_type,
  [sym_intrinsic_type] = sym_intrinsic_type,
  [sym_derived_type] = sym_derived_type,
  [sym_unlimited_polymorphic] = sym_unlimited_polymorphic,
  [sym_size] = sym_size,
  [sym_character_length] = sym_character_length,
  [sym_type_qualifier] = sym_type_qualifier,
  [sym_procedure_qualifier] = sym_procedure_qualifier,
  [sym_parameter_statement] = sym_parameter_statement,
  [sym_parameter_assignment] = sym_parameter_assignment,
  [sym_equivalence_statement] = sym_equivalence_statement,
  [sym_equivalence_set] = sym_equivalence_set,
  [sym__statements] = sym__statements,
  [sym_statement_label] = sym_statement_label,
  [sym_statement_label_reference] = sym_statement_label_reference,
  [sym_stop_statement] = sym_stop_statement,
  [sym_assignment_statement] = sym_assignment_statement,
  [sym_pointer_association_statement] = sym_pointer_association_statement,
  [sym_subroutine_call] = sym_subroutine_call,
  [sym_cuda_kernel_argument_list] = sym_cuda_kernel_argument_list,
  [sym_keyword_statement] = sym_keyword_statement,
  [sym_include_statement] = sym_include_statement,
  [sym_data_statement] = sym_data_statement,
  [sym_data_set] = sym_data_set,
  [sym_data_value] = sym_data_value,
  [sym_do_loop_statement] = sym_do_loop_statement,
  [sym_do_label_statement] = sym_do_label_statement,
  [sym_end_do_loop_statement] = sym_end_do_loop_statement,
  [sym_while_statement] = sym_while_statement,
  [sym_concurrent_statement] = sym_concurrent_statement,
  [sym_concurrent_header] = sym_concurrent_header,
  [sym_concurrent_control] = sym_concurrent_control,
  [sym_concurrent_locality] = sym_concurrent_locality,
  [sym_if_statement] = sym_if_statement,
  [sym__inline_if_statement] = sym__inline_if_statement,
  [sym_arithmetic_if_statement] = sym_arithmetic_if_statement,
  [sym__block_if_statement] = sym__block_if_statement,
  [sym_end_if_statement] = sym_end_if_statement,
  [sym_elseif_clause] = sym_elseif_clause,
  [sym_else_clause] = sym_else_clause,
  [sym_where_statement] = sym_where_statement,
  [sym__inline_where_statement] = sym__inline_where_statement,
  [sym__block_where_statement] = sym__block_where_statement,
  [sym_end_where_statement] = sym_end_where_statement,
  [sym_elsewhere_clause] = sym_elsewhere_clause,
  [sym_forall_statement] = sym_forall_statement,
  [sym_triplet_spec] = sym_triplet_spec,
  [sym__forall_control_expression] = sym__forall_control_expression,
  [sym__inline_forall_statement] = sym__inline_forall_statement,
  [sym__block_forall_statement] = sym__block_forall_statement,
  [sym_end_forall_statement] = sym_end_forall_statement,
  [sym_select_case_statement] = sym_select_case_statement,
  [sym_select_type_statement] = sym_select_type_statement,
  [sym_select_rank_statement] = sym_select_rank_statement,
  [sym_end_select_statement] = sym_end_select_statement,
  [sym_selector] = sym_selector,
  [sym_case_statement] = sym_case_statement,
  [sym_type_statement] = sym_type_statement,
  [sym__class_default] = sym__class_default,
  [sym_case_value_range_list] = sym_case_value_range_list,
  [sym_rank_statement] = sym_rank_statement,
  [sym_block_construct] = sym_block_construct,
  [sym_end_block_construct_statement] = sym_end_block_construct_statement,
  [sym_associate_statement] = sym_associate_statement,
  [sym_association] = sym_association,
  [sym_end_associate_statement] = sym_end_associate_statement,
  [sym_format_statement] = sym_format_statement,
  [sym__transfer_item] = sym__transfer_item,
  [sym__transfer_items] = sym__transfer_items,
  [sym_hollerith_constant] = sym_hollerith_constant,
  [sym__io_arguments] = sym__io_arguments,
  [sym_read_statement] = sym_read_statement,
  [sym__simple_read_statement] = sym__simple_read_statement,
  [sym__parameterized_read_statement] = sym__parameterized_read_statement,
  [sym_print_statement] = sym_print_statement,
  [sym_open_statement] = sym_open_statement,
  [sym_close_statement] = sym_close_statement,
  [sym_write_statement] = sym_write_statement,
  [sym_inquire_statement] = sym_inquire_statement,
  [sym_enum] = sym_enum,
  [sym_enum_statement] = sym_enum_statement,
  [sym_enumerator_statement] = sym_enumerator_statement,
  [sym_end_enum_statement] = sym_end_enum_statement,
  [sym_unit_identifier] = sym_unit_identifier,
  [sym_format_identifier] = sym_format_identifier,
  [sym__file_position_spec] = sym__file_position_spec,
  [sym_file_position_statement] = sym_file_position_statement,
  [sym__io_expressions] = sym__io_expressions,
  [sym_input_item_list] = sym_input_item_list,
  [sym_output_item_list] = sym_output_item_list,
  [sym__expression] = sym__expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_derived_type_member_expression] = sym_derived_type_member_expression,
  [sym_logical_expression] = sym_logical_expression,
  [sym_relational_expression] = sym_relational_expression,
  [sym_concatenation_expression] = sym_concatenation_expression,
  [sym_math_expression] = sym_math_expression,
  [sym_unary_expression] = sym_unary_expression,
  [sym_user_defined_operator] = sym_user_defined_operator,
  [sym_call_expression] = sym_call_expression,
  [sym_implied_do_loop_expression] = sym_implied_do_loop_expression,
  [sym_argument_list] = sym_argument_list,
  [sym_keyword_argument] = sym_keyword_argument,
  [sym_extent_specifier] = sym_extent_specifier,
  [sym_assumed_size] = sym_assumed_size,
  [sym_assumed_shape] = sym_assumed_shape,
  [sym_block_label_start_expression] = sym_block_label_start_expression,
  [sym__block_label] = sym__block_label,
  [sym_loop_control_expression] = sym_loop_control_expression,
  [sym_array_literal] = sym_array_literal,
  [sym__array_constructor_legacy] = sym__array_constructor_legacy,
  [sym__array_constructor_f2003] = sym__array_constructor_f2003,
  [sym__type_spec] = sym__type_spec,
  [sym__ac_value_list] = sym__ac_value_list,
  [sym_complex_literal] = sym_complex_literal,
  [sym_number_literal] = sym_number_literal,
  [sym_null_literal] = sym_null_literal,
  [sym_preproc_file_line] = sym_preproc_file_line,
  [sym_identifier] = sym_identifier,
  [aux_sym_translation_unit_repeat1] = aux_sym_translation_unit_repeat1,
  [aux_sym_program_repeat1] = aux_sym_program_repeat1,
  [aux_sym_program_repeat2] = aux_sym_program_repeat2,
  [aux_sym_interface_repeat1] = aux_sym_interface_repeat1,
  [aux_sym_function_statement_repeat1] = aux_sym_function_statement_repeat1,
  [aux_sym_procedure_attributes_repeat1] = aux_sym_procedure_attributes_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym_internal_procedures_repeat1] = aux_sym_internal_procedures_repeat1,
  [aux_sym_use_statement_repeat1] = aux_sym_use_statement_repeat1,
  [aux_sym_included_items_repeat1] = aux_sym_included_items_repeat1,
  [aux_sym_implicit_statement_repeat1] = aux_sym_implicit_statement_repeat1,
  [aux_sym_implicit_statement_repeat2] = aux_sym_implicit_statement_repeat2,
  [aux_sym_implicit_statement_repeat3] = aux_sym_implicit_statement_repeat3,
  [aux_sym_save_statement_repeat1] = aux_sym_save_statement_repeat1,
  [aux_sym_private_statement_repeat1] = aux_sym_private_statement_repeat1,
  [aux_sym_namelist_statement_repeat1] = aux_sym_namelist_statement_repeat1,
  [aux_sym_common_statement_repeat1] = aux_sym_common_statement_repeat1,
  [aux_sym_derived_type_definition_repeat1] = aux_sym_derived_type_definition_repeat1,
  [aux_sym_derived_type_statement_repeat1] = aux_sym_derived_type_statement_repeat1,
  [aux_sym_derived_type_procedures_repeat1] = aux_sym_derived_type_procedures_repeat1,
  [aux_sym_procedure_statement_repeat1] = aux_sym_procedure_statement_repeat1,
  [aux_sym_procedure_statement_repeat2] = aux_sym_procedure_statement_repeat2,
  [aux_sym_variable_declaration_repeat1] = aux_sym_variable_declaration_repeat1,
  [aux_sym_variable_modification_repeat1] = aux_sym_variable_modification_repeat1,
  [aux_sym__declaration_targets_repeat1] = aux_sym__declaration_targets_repeat1,
  [aux_sym_parameter_statement_repeat1] = aux_sym_parameter_statement_repeat1,
  [aux_sym_equivalence_statement_repeat1] = aux_sym_equivalence_statement_repeat1,
  [aux_sym_equivalence_set_repeat1] = aux_sym_equivalence_set_repeat1,
  [aux_sym_cuda_kernel_argument_list_repeat1] = aux_sym_cuda_kernel_argument_list_repeat1,
  [aux_sym_keyword_statement_repeat1] = aux_sym_keyword_statement_repeat1,
  [aux_sym_data_statement_repeat1] = aux_sym_data_statement_repeat1,
  [aux_sym_data_set_repeat1] = aux_sym_data_set_repeat1,
  [aux_sym_data_value_repeat1] = aux_sym_data_value_repeat1,
  [aux_sym_concurrent_statement_repeat1] = aux_sym_concurrent_statement_repeat1,
  [aux_sym_concurrent_header_repeat1] = aux_sym_concurrent_header_repeat1,
  [aux_sym__block_if_statement_repeat1] = aux_sym__block_if_statement_repeat1,
  [aux_sym__block_where_statement_repeat1] = aux_sym__block_where_statement_repeat1,
  [aux_sym__forall_control_expression_repeat1] = aux_sym__forall_control_expression_repeat1,
  [aux_sym_select_case_statement_repeat1] = aux_sym_select_case_statement_repeat1,
  [aux_sym_select_type_statement_repeat1] = aux_sym_select_type_statement_repeat1,
  [aux_sym_select_rank_statement_repeat1] = aux_sym_select_rank_statement_repeat1,
  [aux_sym_case_value_range_list_repeat1] = aux_sym_case_value_range_list_repeat1,
  [aux_sym_associate_statement_repeat1] = aux_sym_associate_statement_repeat1,
  [aux_sym__transfer_items_repeat1] = aux_sym__transfer_items_repeat1,
  [aux_sym__io_arguments_repeat1] = aux_sym__io_arguments_repeat1,
  [aux_sym_enum_repeat1] = aux_sym_enum_repeat1,
  [aux_sym_enumerator_statement_repeat1] = aux_sym_enumerator_statement_repeat1,
  [aux_sym_call_expression_repeat1] = aux_sym_call_expression_repeat1,
  [aux_sym_argument_list_repeat1] = aux_sym_argument_list_repeat1,
  [alias_sym_block_label] = alias_sym_block_label,
  [anon_alias_sym_class] = anon_alias_sym_class,
  [alias_sym_concurrent_mask] = alias_sym_concurrent_mask,
  [anon_alias_sym_default] = anon_alias_sym_default,
  [anon_alias_sym_double] = anon_alias_sym_double,
  [anon_alias_sym_else] = anon_alias_sym_else,
  [anon_alias_sym_end] = anon_alias_sym_end,
  [alias_sym_filename] = alias_sym_filename,
  [anon_alias_sym_label] = anon_alias_sym_label,
  [alias_sym_local_name] = alias_sym_local_name,
  [anon_alias_sym_method_name] = anon_alias_sym_method_name,
  [alias_sym_name] = alias_sym_name,
  [anon_alias_sym_none] = anon_alias_sym_none,
  [alias_sym_preproc_line_number] = alias_sym_preproc_line_number,
  [alias_sym_procedure_interface] = alias_sym_procedure_interface,
  [anon_alias_sym_program] = anon_alias_sym_program,
  [anon_alias_sym_select] = anon_alias_sym_select,
  [anon_alias_sym_sequence] = anon_alias_sym_sequence,
  [anon_alias_sym_statement_label] = anon_alias_sym_statement_label,
  [anon_alias_sym_statement_label_reference] = anon_alias_sym_statement_label_reference,
  [alias_sym_transfer_items] = alias_sym_transfer_items,
  [alias_sym_type_member] = alias_sym_type_member,
  [alias_sym_type_name] = alias_sym_type_name,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [aux_sym_program_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_program_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_program_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_module_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_module_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_submodule_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_submodule_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_interface_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_interface_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_assignment_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_operator_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_operator_token2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_defined_io_procedure_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_defined_io_procedure_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_defined_io_procedure_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_defined_io_procedure_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_subroutine_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_subroutine_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_module_procedure_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_module_procedure_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_function_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_language_binding_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attributes_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attributes_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attributes_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attributes_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attributes_token5] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_function_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_function_result_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_contains_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_use_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_use_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_use_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_included_items_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_implicit_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_implicit_statement_token2] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_implicit_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_implicit_statement_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_save_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_private_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_public_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_namelist_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_common_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_implicit_range_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_import_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__import_names_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_derived_type_definition_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_abstract_specifier_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_base_type_specifier_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_type_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__procedure_kind_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__procedure_kind_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__procedure_kind_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__procedure_kind_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attribute_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attribute_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attribute_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attribute_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attribute_token5] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_attribute_token6] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_variable_attributes_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_variable_attributes_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_variable_attributes_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_variable_attributes_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_variable_attributes_token5] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token5] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token6] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token7] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token8] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token9] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_intrinsic_type_token10] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_derived_type_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_size_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_qualifier_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token5] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token6] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token7] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token8] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token9] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token10] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token11] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token12] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token13] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token14] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_qualifier_token15] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_qualifier_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_qualifier_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_qualifier_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_procedure_qualifier_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_equivalence_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_stop_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_stop_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_stop_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_subroutine_call_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token5] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token6] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_keyword_statement_token7] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_include_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_data_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_do_loop_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_do_loop_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_while_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_concurrent_header_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_concurrent_locality_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_concurrent_locality_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_concurrent_locality_token3] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__inline_if_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__block_if_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_if_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_elseif_clause_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_elseif_clause_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__inline_where_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_where_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_elsewhere_clause_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__forall_control_expression_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_forall_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_select_case_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_select_case_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_select_case_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_select_type_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_select_rank_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_select_rank_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_select_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_type_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym__class_default_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_block_construct_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_block_construct_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_associate_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_associate_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_format_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [sym_edit_descriptor] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_hollerith_constant_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_print_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_open_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_close_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_inquire_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_enum_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_enumerator_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_end_enum_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_file_position_statement_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_file_position_statement_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_file_position_statement_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_file_position_statement_token4] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_logical_expression_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_logical_expression_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_logical_expression_token3] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_logical_expression_token4] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_logical_expression_token5] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_relational_expression_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_relational_expression_token2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_relational_expression_token3] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_relational_expression_token4] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_relational_expression_token5] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_relational_expression_token6] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_user_defined_operator_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_assumed_rank] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LPAREN_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [sym_boolean_literal] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_null_literal_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUND] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_file_line_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_identifier_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__line_continuation] = {
    .visible = false,
    .named = true,
  },
  [sym__semicolon] = {
    .visible = false,
    .named = true,
  },
  [sym__newline] = {
    .visible = false,
    .named = true,
  },
  [sym__integer_literal] = {
    .visible = false,
    .named = true,
  },
  [sym__float_literal] = {
    .visible = false,
    .named = true,
  },
  [sym__boz_literal] = {
    .visible = false,
    .named = true,
  },
  [sym_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym__end_of_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_translation_unit] = {
    .visible = true,
    .named = true,
  },
  [sym_program] = {
    .visible = true,
    .named = true,
  },
  [sym_program_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_program_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_module] = {
    .visible = true,
    .named = true,
  },
  [sym_module_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_module_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_submodule] = {
    .visible = true,
    .named = true,
  },
  [sym_submodule_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_submodule_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_module_name] = {
    .visible = true,
    .named = true,
  },
  [sym_interface] = {
    .visible = true,
    .named = true,
  },
  [sym_interface_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_interface_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_defined_io_procedure] = {
    .visible = true,
    .named = true,
  },
  [sym__generic_procedure] = {
    .visible = false,
    .named = true,
  },
  [sym_subroutine] = {
    .visible = true,
    .named = true,
  },
  [sym_subroutine_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_subroutine_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_module_procedure] = {
    .visible = true,
    .named = true,
  },
  [sym_module_procedure_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_module_procedure_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_function] = {
    .visible = true,
    .named = true,
  },
  [sym_function_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_language_binding] = {
    .visible = true,
    .named = true,
  },
  [aux_sym__callable_interface_qualifers] = {
    .visible = false,
    .named = false,
  },
  [sym_procedure_attributes] = {
    .visible = true,
    .named = true,
  },
  [sym_end_function_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_function_result] = {
    .visible = true,
    .named = true,
  },
  [sym__name] = {
    .visible = false,
    .named = true,
  },
  [sym__parameters] = {
    .visible = false,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_procedures] = {
    .visible = true,
    .named = true,
  },
  [sym_contains_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__specification_part] = {
    .visible = false,
    .named = true,
  },
  [sym_use_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_included_items] = {
    .visible = true,
    .named = true,
  },
  [sym_use_alias] = {
    .visible = true,
    .named = true,
  },
  [sym_implicit_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_save_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_private_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_public_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_namelist_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_common_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_group] = {
    .visible = true,
    .named = true,
  },
  [sym_implicit_range] = {
    .visible = true,
    .named = true,
  },
  [sym_import_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__import_names] = {
    .visible = false,
    .named = true,
  },
  [sym_derived_type_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_access_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_base_type_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym__derived_type_qualifier] = {
    .visible = false,
    .named = true,
  },
  [sym_derived_type_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_type_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__type_name] = {
    .visible = false,
    .named = true,
  },
  [sym_derived_type_procedures] = {
    .visible = true,
    .named = true,
  },
  [sym_procedure_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_binding_name] = {
    .visible = true,
    .named = true,
  },
  [sym_method_name] = {
    .visible = true,
    .named = true,
  },
  [sym__procedure_kind] = {
    .visible = false,
    .named = true,
  },
  [sym_procedure_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_procedure_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_modification] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_attributes] = {
    .visible = true,
    .named = true,
  },
  [sym__declaration_targets] = {
    .visible = false,
    .named = true,
  },
  [sym__intrinsic_type] = {
    .visible = false,
    .named = true,
  },
  [sym_intrinsic_type] = {
    .visible = true,
    .named = true,
  },
  [sym_derived_type] = {
    .visible = true,
    .named = true,
  },
  [sym_unlimited_polymorphic] = {
    .visible = true,
    .named = true,
  },
  [sym_size] = {
    .visible = true,
    .named = true,
  },
  [sym_character_length] = {
    .visible = true,
    .named = true,
  },
  [sym_type_qualifier] = {
    .visible = true,
    .named = true,
  },
  [sym_procedure_qualifier] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_equivalence_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_equivalence_set] = {
    .visible = true,
    .named = true,
  },
  [sym__statements] = {
    .visible = false,
    .named = true,
  },
  [sym_statement_label] = {
    .visible = true,
    .named = true,
  },
  [sym_statement_label_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_stop_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_association_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_subroutine_call] = {
    .visible = true,
    .named = true,
  },
  [sym_cuda_kernel_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_keyword_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_include_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_data_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_data_set] = {
    .visible = true,
    .named = true,
  },
  [sym_data_value] = {
    .visible = true,
    .named = true,
  },
  [sym_do_loop_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_do_label_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_do_loop_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_while_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_concurrent_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_concurrent_header] = {
    .visible = true,
    .named = true,
  },
  [sym_concurrent_control] = {
    .visible = true,
    .named = true,
  },
  [sym_concurrent_locality] = {
    .visible = true,
    .named = true,
  },
  [sym_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__inline_if_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_arithmetic_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__block_if_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_end_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_elseif_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_else_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_where_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__inline_where_statement] = {
    .visible = false,
    .named = true,
  },
  [sym__block_where_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_end_where_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_elsewhere_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_forall_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_triplet_spec] = {
    .visible = true,
    .named = true,
  },
  [sym__forall_control_expression] = {
    .visible = false,
    .named = true,
  },
  [sym__inline_forall_statement] = {
    .visible = false,
    .named = true,
  },
  [sym__block_forall_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_end_forall_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_select_case_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_select_type_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_select_rank_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_select_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_selector] = {
    .visible = true,
    .named = true,
  },
  [sym_case_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_type_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__class_default] = {
    .visible = true,
    .named = true,
  },
  [sym_case_value_range_list] = {
    .visible = true,
    .named = true,
  },
  [sym_rank_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_block_construct] = {
    .visible = true,
    .named = true,
  },
  [sym_end_block_construct_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_associate_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_association] = {
    .visible = true,
    .named = true,
  },
  [sym_end_associate_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_format_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__transfer_item] = {
    .visible = false,
    .named = true,
  },
  [sym__transfer_items] = {
    .visible = false,
    .named = true,
  },
  [sym_hollerith_constant] = {
    .visible = true,
    .named = true,
  },
  [sym__io_arguments] = {
    .visible = false,
    .named = true,
  },
  [sym_read_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__simple_read_statement] = {
    .visible = false,
    .named = true,
  },
  [sym__parameterized_read_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_print_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_open_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_close_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_write_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_inquire_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_enum] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_enumerator_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_end_enum_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_unit_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_format_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym__file_position_spec] = {
    .visible = false,
    .named = true,
  },
  [sym_file_position_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__io_expressions] = {
    .visible = false,
    .named = true,
  },
  [sym_input_item_list] = {
    .visible = true,
    .named = true,
  },
  [sym_output_item_list] = {
    .visible = true,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_derived_type_member_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_logical_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_relational_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_concatenation_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_math_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_user_defined_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_implied_do_loop_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_keyword_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_extent_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_assumed_size] = {
    .visible = true,
    .named = true,
  },
  [sym_assumed_shape] = {
    .visible = true,
    .named = true,
  },
  [sym_block_label_start_expression] = {
    .visible = true,
    .named = true,
  },
  [sym__block_label] = {
    .visible = false,
    .named = true,
  },
  [sym_loop_control_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_array_literal] = {
    .visible = true,
    .named = true,
  },
  [sym__array_constructor_legacy] = {
    .visible = false,
    .named = true,
  },
  [sym__array_constructor_f2003] = {
    .visible = false,
    .named = true,
  },
  [sym__type_spec] = {
    .visible = false,
    .named = true,
  },
  [sym__ac_value_list] = {
    .visible = false,
    .named = true,
  },
  [sym_complex_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_number_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_null_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_file_line] = {
    .visible = true,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_translation_unit_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_program_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_program_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_interface_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_function_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_procedure_attributes_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_internal_procedures_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_use_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_included_items_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_implicit_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_implicit_statement_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_implicit_statement_repeat3] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_save_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_private_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_namelist_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_common_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_derived_type_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_derived_type_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_derived_type_procedures_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_procedure_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_procedure_statement_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_variable_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_variable_modification_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__declaration_targets_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameter_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_equivalence_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_equivalence_set_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_cuda_kernel_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_keyword_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_data_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_data_set_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_data_value_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_concurrent_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_concurrent_header_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__block_if_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__block_where_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__forall_control_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_select_case_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_select_type_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_select_rank_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_value_range_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_associate_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__transfer_items_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__io_arguments_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enum_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enumerator_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_call_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_block_label] = {
    .visible = true,
    .named = true,
  },
  [anon_alias_sym_class] = {
    .visible = true,
    .named = false,
  },
  [alias_sym_concurrent_mask] = {
    .visible = true,
    .named = true,
  },
  [anon_alias_sym_default] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_double] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_end] = {
    .visible = true,
    .named = false,
  },
  [alias_sym_filename] = {
    .visible = true,
    .named = true,
  },
  [anon_alias_sym_label] = {
    .visible = true,
    .named = false,
  },
  [alias_sym_local_name] = {
    .visible = true,
    .named = true,
  },
  [anon_alias_sym_method_name] = {
    .visible = true,
    .named = false,
  },
  [alias_sym_name] = {
    .visible = true,
    .named = true,
  },
  [anon_alias_sym_none] = {
    .visible = true,
    .named = false,
  },
  [alias_sym_preproc_line_number] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_procedure_interface] = {
    .visible = true,
    .named = true,
  },
  [anon_alias_sym_program] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_select] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_sequence] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_statement_label] = {
    .visible = true,
    .named = false,
  },
  [anon_alias_sym_statement_label_reference] = {
    .visible = true,
    .named = false,
  },
  [alias_sym_transfer_items] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_member] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_name] = {
    .visible = true,
    .named = true,
  },
};

enum {
  field_access = 1,
  field_ancestor = 2,
  field_argument = 3,
  field_base = 4,
  field_final = 5,
  field_initial = 6,
  field_left = 7,
  field_name = 8,
  field_operator = 9,
  field_parameters = 10,
  field_parent = 11,
  field_path = 12,
  field_repeat = 13,
  field_right = 14,
  field_selector = 15,
  field_step = 16,
  field_subroutine = 17,
  field_type = 18,
  field_value = 19,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_access] = "access",
  [field_ancestor] = "ancestor",
  [field_argument] = "argument",
  [field_base] = "base",
  [field_final] = "final",
  [field_initial] = "initial",
  [field_left] = "left",
  [field_name] = "name",
  [field_operator] = "operator",
  [field_parameters] = "parameters",
  [field_parent] = "parent",
  [field_path] = "path",
  [field_repeat] = "repeat",
  [field_right] = "right",
  [field_selector] = "selector",
  [field_step] = "step",
  [field_subroutine] = "subroutine",
  [field_type] = "type",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [7] = {.index = 0, .length = 1},
  [11] = {.index = 1, .length = 1},
  [14] = {.index = 2, .length = 2},
  [16] = {.index = 4, .length = 1},
  [17] = {.index = 5, .length = 1},
  [18] = {.index = 6, .length = 1},
  [21] = {.index = 7, .length = 2},
  [23] = {.index = 9, .length = 1},
  [24] = {.index = 10, .length = 1},
  [28] = {.index = 11, .length = 1},
  [33] = {.index = 12, .length = 1},
  [38] = {.index = 13, .length = 2},
  [39] = {.index = 15, .length = 3},
  [55] = {.index = 18, .length = 2},
  [58] = {.index = 20, .length = 1},
  [59] = {.index = 21, .length = 2},
  [61] = {.index = 23, .length = 2},
  [62] = {.index = 25, .length = 4},
  [69] = {.index = 29, .length = 2},
  [73] = {.index = 31, .length = 2},
  [74] = {.index = 33, .length = 1},
  [75] = {.index = 34, .length = 2},
  [87] = {.index = 36, .length = 1},
  [90] = {.index = 37, .length = 2},
  [91] = {.index = 39, .length = 4},
  [92] = {.index = 43, .length = 1},
  [93] = {.index = 44, .length = 1},
  [97] = {.index = 45, .length = 2},
  [101] = {.index = 47, .length = 2},
  [103] = {.index = 49, .length = 2},
  [105] = {.index = 44, .length = 1},
  [106] = {.index = 44, .length = 1},
  [107] = {.index = 51, .length = 4},
  [108] = {.index = 44, .length = 1},
  [110] = {.index = 55, .length = 1},
  [111] = {.index = 55, .length = 1},
  [112] = {.index = 44, .length = 1},
  [113] = {.index = 56, .length = 3},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_type, 0, .inherited = true},
  [1] =
    {field_name, 1},
  [2] =
    {field_argument, 1},
    {field_operator, 0},
  [4] =
    {field_subroutine, 1},
  [5] =
    {field_path, 1},
  [6] =
    {field_type, 0},
  [7] =
    {field_name, 1},
    {field_parameters, 2},
  [9] =
    {field_access, 0},
  [10] =
    {field_base, 0},
  [11] =
    {field_type, 1, .inherited = true},
  [12] =
    {field_name, 2},
  [13] =
    {field_left, 0},
    {field_right, 2},
  [15] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [18] =
    {field_name, 2},
    {field_parameters, 3},
  [20] =
    {field_ancestor, 2},
  [21] =
    {field_name, 0},
    {field_value, 2},
  [23] =
    {field_access, 1, .inherited = true},
    {field_base, 1, .inherited = true},
  [25] =
    {field_access, 0, .inherited = true},
    {field_access, 1, .inherited = true},
    {field_base, 0, .inherited = true},
    {field_base, 1, .inherited = true},
  [29] =
    {field_name, 0},
    {field_selector, 2},
  [31] =
    {field_access, 2, .inherited = true},
    {field_base, 2, .inherited = true},
  [33] =
    {field_repeat, 2, .inherited = true},
  [34] =
    {field_repeat, 0, .inherited = true},
    {field_repeat, 1, .inherited = true},
  [36] =
    {field_name, 3},
  [37] =
    {field_ancestor, 2},
    {field_parent, 4},
  [39] =
    {field_access, 2, .inherited = true},
    {field_access, 3, .inherited = true},
    {field_base, 2, .inherited = true},
    {field_base, 3, .inherited = true},
  [43] =
    {field_repeat, 1},
  [44] =
    {field_type, 2},
  [45] =
    {field_access, 3, .inherited = true},
    {field_base, 3, .inherited = true},
  [47] =
    {field_repeat, 1},
    {field_repeat, 4, .inherited = true},
  [49] =
    {field_final, 4},
    {field_initial, 2},
  [51] =
    {field_access, 3, .inherited = true},
    {field_access, 4, .inherited = true},
    {field_base, 3, .inherited = true},
    {field_base, 4, .inherited = true},
  [55] =
    {field_type, 3},
  [56] =
    {field_final, 4},
    {field_initial, 2},
    {field_step, 6},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = anon_alias_sym_statement_label,
  },
  [2] = {
    [0] = anon_alias_sym_end,
  },
  [3] = {
    [0] = anon_alias_sym_sequence,
  },
  [4] = {
    [0] = anon_alias_sym_double,
  },
  [5] = {
    [0] = aux_sym_intrinsic_type_token8,
  },
  [6] = {
    [0] = anon_alias_sym_select,
  },
  [8] = {
    [0] = anon_alias_sym_program,
  },
  [9] = {
    [0] = alias_sym_name,
  },
  [10] = {
    [0] = anon_alias_sym_statement_label_reference,
  },
  [12] = {
    [1] = sym_module_name,
  },
  [13] = {
    [0] = alias_sym_type_name,
  },
  [15] = {
    [0] = aux_sym_intrinsic_type_token8,
    [1] = aux_sym_intrinsic_type_token8,
  },
  [17] = {
    [1] = alias_sym_filename,
  },
  [19] = {
    [0] = aux_sym_end_enum_statement_token1,
  },
  [20] = {
    [0] = anon_alias_sym_label,
  },
  [22] = {
    [2] = sym_module_name,
  },
  [25] = {
    [1] = anon_alias_sym_none,
  },
  [26] = {
    [0] = aux_sym_end_do_loop_statement_token1,
  },
  [27] = {
    [0] = aux_sym_end_block_construct_statement_token1,
  },
  [29] = {
    [0] = aux_sym_end_module_statement_token1,
  },
  [30] = {
    [0] = aux_sym_end_submodule_statement_token1,
  },
  [31] = {
    [0] = aux_sym_end_interface_statement_token1,
  },
  [32] = {
    [0] = anon_alias_sym_method_name,
  },
  [34] = {
    [0] = aux_sym_end_type_statement_token1,
  },
  [35] = {
    [0] = sym_assumed_size,
  },
  [36] = {
    [0] = aux_sym_end_forall_statement_token1,
  },
  [37] = {
    [0] = aux_sym_end_enum_statement_token1,
    [1] = aux_sym_end_enum_statement_token1,
  },
  [40] = {
    [2] = alias_sym_type_member,
  },
  [41] = {
    [1] = alias_sym_preproc_line_number,
  },
  [42] = {
    [2] = alias_sym_procedure_interface,
  },
  [43] = {
    [0] = anon_alias_sym_class,
  },
  [44] = {
    [2] = aux_sym_type_qualifier_token8,
  },
  [45] = {
    [0] = aux_sym_end_do_loop_statement_token1,
    [1] = aux_sym_end_do_loop_statement_token1,
  },
  [46] = {
    [0] = alias_sym_block_label,
  },
  [47] = {
    [0] = aux_sym_end_where_statement_token1,
  },
  [48] = {
    [0] = aux_sym_end_block_construct_statement_token1,
    [1] = aux_sym_end_block_construct_statement_token1,
  },
  [49] = {
    [2] = alias_sym_transfer_items,
  },
  [50] = {
    [0] = aux_sym_end_module_statement_token1,
    [1] = aux_sym_end_module_statement_token1,
  },
  [51] = {
    [0] = aux_sym_end_submodule_statement_token1,
    [1] = aux_sym_end_submodule_statement_token1,
  },
  [52] = {
    [0] = aux_sym_end_interface_statement_token1,
    [1] = aux_sym_end_interface_statement_token1,
  },
  [53] = {
    [0] = aux_sym_end_subroutine_statement_token1,
  },
  [54] = {
    [0] = aux_sym_end_function_statement_token1,
  },
  [56] = {
    [0] = aux_sym_end_type_statement_token1,
    [1] = aux_sym_end_type_statement_token1,
  },
  [57] = {
    [0] = aux_sym_end_forall_statement_token1,
    [1] = aux_sym_end_forall_statement_token1,
  },
  [60] = {
    [4] = sym_module_name,
  },
  [63] = {
    [2] = aux_sym_type_qualifier_token8,
    [3] = aux_sym_type_qualifier_token8,
  },
  [64] = {
    [0] = aux_sym_end_if_statement_token1,
  },
  [65] = {
    [0] = aux_sym_end_where_statement_token1,
    [1] = aux_sym_end_where_statement_token1,
  },
  [66] = {
    [0] = aux_sym_end_select_statement_token1,
  },
  [67] = {
    [0] = aux_sym_select_type_statement_token1,
  },
  [68] = {
    [0] = aux_sym_select_rank_statement_token2,
  },
  [70] = {
    [0] = aux_sym_end_subroutine_statement_token1,
    [1] = aux_sym_end_subroutine_statement_token1,
  },
  [71] = {
    [0] = aux_sym_end_function_statement_token1,
    [1] = aux_sym_end_function_statement_token1,
  },
  [72] = {
    [0] = alias_sym_local_name,
  },
  [76] = {
    [0] = anon_alias_sym_default,
    [2] = anon_alias_sym_none,
  },
  [77] = {
    [0] = aux_sym_end_if_statement_token1,
    [1] = aux_sym_end_if_statement_token1,
  },
  [78] = {
    [0] = anon_alias_sym_else,
  },
  [79] = {
    [1] = aux_sym_elseif_clause_token1,
  },
  [80] = {
    [0] = aux_sym_select_type_statement_token1,
    [1] = aux_sym_select_type_statement_token1,
  },
  [81] = {
    [1] = aux_sym_select_case_statement_token1,
  },
  [82] = {
    [0] = aux_sym_select_rank_statement_token2,
    [1] = aux_sym_select_rank_statement_token2,
  },
  [83] = {
    [1] = sym__class_default,
  },
  [84] = {
    [0] = aux_sym_end_select_statement_token1,
    [1] = aux_sym_end_select_statement_token1,
  },
  [85] = {
    [0] = aux_sym_end_associate_statement_token1,
  },
  [86] = {
    [0] = aux_sym_end_module_procedure_statement_token1,
  },
  [88] = {
    [1] = aux_sym_select_type_statement_token1,
  },
  [89] = {
    [1] = aux_sym_select_rank_statement_token2,
  },
  [94] = {
    [4] = alias_sym_concurrent_mask,
  },
  [95] = {
    [0] = aux_sym_end_associate_statement_token1,
    [1] = aux_sym_end_associate_statement_token1,
  },
  [96] = {
    [0] = aux_sym_end_module_procedure_statement_token1,
    [1] = aux_sym_end_module_procedure_statement_token1,
  },
  [98] = {
    [1] = aux_sym_select_type_statement_token1,
    [2] = aux_sym_select_type_statement_token1,
  },
  [99] = {
    [1] = aux_sym_select_case_statement_token1,
    [2] = aux_sym_select_case_statement_token1,
  },
  [100] = {
    [1] = aux_sym_select_rank_statement_token2,
    [2] = aux_sym_select_rank_statement_token2,
  },
  [102] = {
    [5] = alias_sym_concurrent_mask,
  },
  [104] = {
    [0] = aux_sym_elseif_clause_token2,
  },
  [105] = {
    [0] = aux_sym_type_statement_token1,
  },
  [106] = {
    [0] = aux_sym_type_statement_token3,
  },
  [108] = {
    [6] = alias_sym_concurrent_mask,
  },
  [109] = {
    [0] = aux_sym_elseif_clause_token2,
    [1] = aux_sym_elseif_clause_token2,
  },
  [110] = {
    [0] = aux_sym_type_statement_token1,
  },
  [111] = {
    [0] = aux_sym_type_statement_token3,
    [1] = aux_sym_type_statement_token3,
  },
  [112] = {
    [7] = alias_sym_concurrent_mask,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_statement_label, 2,
    sym_statement_label,
    anon_alias_sym_statement_label_reference,
  sym__transfer_items, 2,
    sym__transfer_items,
    alias_sym_transfer_items,
  sym__expression, 2,
    sym__expression,
    alias_sym_concurrent_mask,
  sym_identifier, 10,
    sym_identifier,
    alias_sym_block_label,
    alias_sym_local_name,
    alias_sym_name,
    alias_sym_procedure_interface,
    alias_sym_type_member,
    alias_sym_type_name,
    anon_alias_sym_label,
    anon_alias_sym_method_name,
    sym_module_name,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 5,
  [7] = 7,
  [8] = 4,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 11,
  [13] = 11,
  [14] = 9,
  [15] = 10,
  [16] = 7,
  [17] = 17,
  [18] = 10,
  [19] = 9,
  [20] = 7,
  [21] = 5,
  [22] = 4,
  [23] = 11,
  [24] = 7,
  [25] = 5,
  [26] = 10,
  [27] = 27,
  [28] = 9,
  [29] = 4,
  [30] = 10,
  [31] = 11,
  [32] = 5,
  [33] = 7,
  [34] = 4,
  [35] = 9,
  [36] = 7,
  [37] = 11,
  [38] = 9,
  [39] = 10,
  [40] = 4,
  [41] = 11,
  [42] = 5,
  [43] = 10,
  [44] = 7,
  [45] = 45,
  [46] = 5,
  [47] = 7,
  [48] = 10,
  [49] = 11,
  [50] = 5,
  [51] = 51,
  [52] = 52,
  [53] = 53,
  [54] = 54,
  [55] = 55,
  [56] = 55,
  [57] = 55,
  [58] = 55,
  [59] = 55,
  [60] = 60,
  [61] = 61,
  [62] = 62,
  [63] = 63,
  [64] = 64,
  [65] = 65,
  [66] = 66,
  [67] = 67,
  [68] = 68,
  [69] = 69,
  [70] = 70,
  [71] = 71,
  [72] = 72,
  [73] = 73,
  [74] = 74,
  [75] = 75,
  [76] = 76,
  [77] = 77,
  [78] = 78,
  [79] = 79,
  [80] = 80,
  [81] = 81,
  [82] = 82,
  [83] = 83,
  [84] = 84,
  [85] = 85,
  [86] = 86,
  [87] = 86,
  [88] = 85,
  [89] = 89,
  [90] = 90,
  [91] = 91,
  [92] = 85,
  [93] = 91,
  [94] = 89,
  [95] = 95,
  [96] = 85,
  [97] = 89,
  [98] = 98,
  [99] = 91,
  [100] = 100,
  [101] = 91,
  [102] = 98,
  [103] = 86,
  [104] = 98,
  [105] = 90,
  [106] = 91,
  [107] = 90,
  [108] = 86,
  [109] = 109,
  [110] = 85,
  [111] = 89,
  [112] = 91,
  [113] = 90,
  [114] = 98,
  [115] = 90,
  [116] = 86,
  [117] = 89,
  [118] = 90,
  [119] = 119,
  [120] = 86,
  [121] = 98,
  [122] = 122,
  [123] = 98,
  [124] = 89,
  [125] = 86,
  [126] = 85,
  [127] = 98,
  [128] = 98,
  [129] = 86,
  [130] = 91,
  [131] = 90,
  [132] = 132,
  [133] = 90,
  [134] = 134,
  [135] = 135,
  [136] = 136,
  [137] = 91,
  [138] = 138,
  [139] = 139,
  [140] = 140,
  [141] = 141,
  [142] = 142,
  [143] = 143,
  [144] = 144,
  [145] = 145,
  [146] = 146,
  [147] = 82,
  [148] = 148,
  [149] = 149,
  [150] = 150,
  [151] = 82,
  [152] = 152,
  [153] = 153,
  [154] = 154,
  [155] = 82,
  [156] = 156,
  [157] = 157,
  [158] = 158,
  [159] = 82,
  [160] = 82,
  [161] = 82,
  [162] = 162,
  [163] = 163,
  [164] = 164,
  [165] = 165,
  [166] = 166,
  [167] = 167,
  [168] = 168,
  [169] = 169,
  [170] = 170,
  [171] = 171,
  [172] = 172,
  [173] = 173,
  [174] = 174,
  [175] = 175,
  [176] = 176,
  [177] = 177,
  [178] = 82,
  [179] = 179,
  [180] = 180,
  [181] = 181,
  [182] = 182,
  [183] = 183,
  [184] = 184,
  [185] = 185,
  [186] = 186,
  [187] = 187,
  [188] = 188,
  [189] = 189,
  [190] = 190,
  [191] = 191,
  [192] = 192,
  [193] = 193,
  [194] = 82,
  [195] = 195,
  [196] = 196,
  [197] = 197,
  [198] = 198,
  [199] = 199,
  [200] = 82,
  [201] = 82,
  [202] = 202,
  [203] = 203,
  [204] = 204,
  [205] = 82,
  [206] = 82,
  [207] = 207,
  [208] = 208,
  [209] = 209,
  [210] = 210,
  [211] = 211,
  [212] = 82,
  [213] = 213,
  [214] = 214,
  [215] = 215,
  [216] = 216,
  [217] = 217,
  [218] = 218,
  [219] = 219,
  [220] = 220,
  [221] = 221,
  [222] = 222,
  [223] = 221,
  [224] = 224,
  [225] = 225,
  [226] = 226,
  [227] = 227,
  [228] = 228,
  [229] = 221,
  [230] = 221,
  [231] = 221,
  [232] = 232,
  [233] = 233,
  [234] = 234,
  [235] = 235,
  [236] = 236,
  [237] = 237,
  [238] = 238,
  [239] = 239,
  [240] = 240,
  [241] = 241,
  [242] = 242,
  [243] = 243,
  [244] = 244,
  [245] = 245,
  [246] = 246,
  [247] = 247,
  [248] = 248,
  [249] = 249,
  [250] = 250,
  [251] = 251,
  [252] = 252,
  [253] = 252,
  [254] = 254,
  [255] = 252,
  [256] = 252,
  [257] = 257,
  [258] = 258,
  [259] = 252,
  [260] = 252,
  [261] = 261,
  [262] = 252,
  [263] = 263,
  [264] = 264,
  [265] = 265,
  [266] = 266,
  [267] = 267,
  [268] = 252,
  [269] = 252,
  [270] = 270,
  [271] = 271,
  [272] = 252,
  [273] = 273,
  [274] = 252,
  [275] = 275,
  [276] = 276,
  [277] = 252,
  [278] = 278,
  [279] = 279,
  [280] = 280,
  [281] = 252,
  [282] = 282,
  [283] = 283,
  [284] = 284,
  [285] = 285,
  [286] = 286,
  [287] = 252,
  [288] = 288,
  [289] = 289,
  [290] = 290,
  [291] = 291,
  [292] = 292,
  [293] = 293,
  [294] = 294,
  [295] = 295,
  [296] = 265,
  [297] = 297,
  [298] = 298,
  [299] = 233,
  [300] = 300,
  [301] = 301,
  [302] = 284,
  [303] = 303,
  [304] = 304,
  [305] = 305,
  [306] = 295,
  [307] = 307,
  [308] = 308,
  [309] = 245,
  [310] = 310,
  [311] = 311,
  [312] = 308,
  [313] = 307,
  [314] = 314,
  [315] = 300,
  [316] = 316,
  [317] = 317,
  [318] = 238,
  [319] = 319,
  [320] = 303,
  [321] = 304,
  [322] = 305,
  [323] = 295,
  [324] = 324,
  [325] = 325,
  [326] = 295,
  [327] = 319,
  [328] = 328,
  [329] = 310,
  [330] = 324,
  [331] = 314,
  [332] = 332,
  [333] = 325,
  [334] = 310,
  [335] = 288,
  [336] = 324,
  [337] = 319,
  [338] = 325,
  [339] = 233,
  [340] = 340,
  [341] = 340,
  [342] = 342,
  [343] = 342,
  [344] = 288,
  [345] = 317,
  [346] = 316,
  [347] = 317,
  [348] = 316,
  [349] = 311,
  [350] = 303,
  [351] = 307,
  [352] = 308,
  [353] = 265,
  [354] = 314,
  [355] = 300,
  [356] = 311,
  [357] = 357,
  [358] = 233,
  [359] = 284,
  [360] = 342,
  [361] = 340,
  [362] = 304,
  [363] = 303,
  [364] = 304,
  [365] = 265,
  [366] = 305,
  [367] = 284,
  [368] = 245,
  [369] = 300,
  [370] = 238,
  [371] = 319,
  [372] = 324,
  [373] = 245,
  [374] = 325,
  [375] = 310,
  [376] = 288,
  [377] = 340,
  [378] = 342,
  [379] = 233,
  [380] = 317,
  [381] = 316,
  [382] = 305,
  [383] = 311,
  [384] = 307,
  [385] = 238,
  [386] = 308,
  [387] = 288,
  [388] = 238,
  [389] = 265,
  [390] = 245,
  [391] = 314,
  [392] = 284,
  [393] = 324,
  [394] = 316,
  [395] = 314,
  [396] = 300,
  [397] = 303,
  [398] = 304,
  [399] = 305,
  [400] = 295,
  [401] = 319,
  [402] = 288,
  [403] = 325,
  [404] = 245,
  [405] = 308,
  [406] = 284,
  [407] = 233,
  [408] = 265,
  [409] = 311,
  [410] = 238,
  [411] = 317,
  [412] = 342,
  [413] = 340,
  [414] = 310,
  [415] = 307,
  [416] = 416,
  [417] = 417,
  [418] = 418,
  [419] = 419,
  [420] = 420,
  [421] = 55,
  [422] = 55,
  [423] = 420,
  [424] = 424,
  [425] = 425,
  [426] = 426,
  [427] = 427,
  [428] = 427,
  [429] = 427,
  [430] = 430,
  [431] = 427,
  [432] = 432,
  [433] = 433,
  [434] = 434,
  [435] = 435,
  [436] = 435,
  [437] = 437,
  [438] = 437,
  [439] = 439,
  [440] = 440,
  [441] = 441,
  [442] = 442,
  [443] = 443,
  [444] = 444,
  [445] = 445,
  [446] = 446,
  [447] = 332,
  [448] = 448,
  [449] = 449,
  [450] = 432,
  [451] = 451,
  [452] = 452,
  [453] = 453,
  [454] = 454,
  [455] = 455,
  [456] = 456,
  [457] = 457,
  [458] = 298,
  [459] = 459,
  [460] = 460,
  [461] = 461,
  [462] = 462,
  [463] = 301,
  [464] = 297,
  [465] = 465,
  [466] = 466,
  [467] = 467,
  [468] = 467,
  [469] = 467,
  [470] = 467,
  [471] = 466,
  [472] = 467,
  [473] = 467,
  [474] = 466,
  [475] = 475,
  [476] = 476,
  [477] = 477,
  [478] = 478,
  [479] = 479,
  [480] = 480,
  [481] = 462,
  [482] = 482,
  [483] = 483,
  [484] = 461,
  [485] = 461,
  [486] = 462,
  [487] = 462,
  [488] = 461,
  [489] = 462,
  [490] = 462,
  [491] = 491,
  [492] = 492,
  [493] = 461,
  [494] = 494,
  [495] = 495,
  [496] = 496,
  [497] = 461,
  [498] = 498,
  [499] = 461,
  [500] = 462,
  [501] = 462,
  [502] = 461,
  [503] = 461,
  [504] = 504,
  [505] = 505,
  [506] = 506,
  [507] = 462,
  [508] = 461,
  [509] = 462,
  [510] = 462,
  [511] = 511,
  [512] = 461,
  [513] = 513,
  [514] = 465,
  [515] = 460,
  [516] = 449,
  [517] = 517,
  [518] = 518,
  [519] = 462,
  [520] = 520,
  [521] = 521,
  [522] = 522,
  [523] = 461,
  [524] = 524,
  [525] = 525,
  [526] = 462,
  [527] = 461,
  [528] = 528,
  [529] = 517,
  [530] = 530,
  [531] = 461,
  [532] = 532,
  [533] = 533,
  [534] = 534,
  [535] = 535,
  [536] = 536,
  [537] = 537,
  [538] = 532,
  [539] = 539,
  [540] = 540,
  [541] = 540,
  [542] = 536,
  [543] = 540,
  [544] = 532,
  [545] = 536,
  [546] = 462,
  [547] = 532,
  [548] = 540,
  [549] = 532,
  [550] = 536,
  [551] = 551,
  [552] = 540,
  [553] = 536,
  [554] = 540,
  [555] = 532,
  [556] = 536,
  [557] = 540,
  [558] = 532,
  [559] = 559,
  [560] = 560,
  [561] = 540,
  [562] = 532,
  [563] = 563,
  [564] = 298,
  [565] = 233,
  [566] = 238,
  [567] = 567,
  [568] = 568,
  [569] = 245,
  [570] = 570,
  [571] = 567,
  [572] = 572,
  [573] = 573,
  [574] = 574,
  [575] = 265,
  [576] = 576,
  [577] = 577,
  [578] = 578,
  [579] = 579,
  [580] = 307,
  [581] = 308,
  [582] = 582,
  [583] = 284,
  [584] = 314,
  [585] = 300,
  [586] = 586,
  [587] = 587,
  [588] = 303,
  [589] = 304,
  [590] = 298,
  [591] = 591,
  [592] = 305,
  [593] = 311,
  [594] = 295,
  [595] = 316,
  [596] = 317,
  [597] = 574,
  [598] = 598,
  [599] = 342,
  [600] = 340,
  [601] = 601,
  [602] = 602,
  [603] = 603,
  [604] = 604,
  [605] = 605,
  [606] = 319,
  [607] = 324,
  [608] = 325,
  [609] = 310,
  [610] = 328,
  [611] = 340,
  [612] = 342,
  [613] = 613,
  [614] = 614,
  [615] = 288,
  [616] = 616,
  [617] = 310,
  [618] = 325,
  [619] = 324,
  [620] = 319,
  [621] = 317,
  [622] = 576,
  [623] = 238,
  [624] = 316,
  [625] = 245,
  [626] = 626,
  [627] = 297,
  [628] = 311,
  [629] = 332,
  [630] = 301,
  [631] = 308,
  [632] = 304,
  [633] = 303,
  [634] = 573,
  [635] = 328,
  [636] = 297,
  [637] = 284,
  [638] = 332,
  [639] = 233,
  [640] = 300,
  [641] = 288,
  [642] = 314,
  [643] = 301,
  [644] = 265,
  [645] = 307,
  [646] = 295,
  [647] = 305,
  [648] = 648,
  [649] = 649,
  [650] = 650,
  [651] = 651,
  [652] = 652,
  [653] = 653,
  [654] = 654,
  [655] = 655,
  [656] = 656,
  [657] = 650,
  [658] = 649,
  [659] = 659,
  [660] = 660,
  [661] = 661,
  [662] = 662,
  [663] = 663,
  [664] = 664,
  [665] = 651,
  [666] = 659,
  [667] = 667,
  [668] = 668,
  [669] = 669,
  [670] = 661,
  [671] = 655,
  [672] = 672,
  [673] = 662,
  [674] = 664,
  [675] = 667,
  [676] = 668,
  [677] = 669,
  [678] = 655,
  [679] = 656,
  [680] = 650,
  [681] = 661,
  [682] = 682,
  [683] = 656,
  [684] = 684,
  [685] = 685,
  [686] = 686,
  [687] = 687,
  [688] = 662,
  [689] = 660,
  [690] = 649,
  [691] = 663,
  [692] = 692,
  [693] = 693,
  [694] = 663,
  [695] = 695,
  [696] = 696,
  [697] = 697,
  [698] = 698,
  [699] = 699,
  [700] = 700,
  [701] = 660,
  [702] = 702,
  [703] = 703,
  [704] = 704,
  [705] = 705,
  [706] = 706,
  [707] = 707,
  [708] = 708,
  [709] = 651,
  [710] = 710,
  [711] = 711,
  [712] = 712,
  [713] = 713,
  [714] = 686,
  [715] = 661,
  [716] = 663,
  [717] = 717,
  [718] = 650,
  [719] = 656,
  [720] = 720,
  [721] = 669,
  [722] = 668,
  [723] = 723,
  [724] = 655,
  [725] = 660,
  [726] = 669,
  [727] = 668,
  [728] = 720,
  [729] = 667,
  [730] = 460,
  [731] = 664,
  [732] = 662,
  [733] = 713,
  [734] = 651,
  [735] = 735,
  [736] = 660,
  [737] = 465,
  [738] = 693,
  [739] = 739,
  [740] = 740,
  [741] = 664,
  [742] = 742,
  [743] = 743,
  [744] = 744,
  [745] = 449,
  [746] = 746,
  [747] = 667,
  [748] = 748,
  [749] = 660,
  [750] = 750,
  [751] = 751,
  [752] = 752,
  [753] = 517,
  [754] = 754,
  [755] = 755,
  [756] = 756,
  [757] = 755,
  [758] = 756,
  [759] = 756,
  [760] = 755,
  [761] = 756,
  [762] = 755,
  [763] = 756,
  [764] = 756,
  [765] = 755,
  [766] = 755,
  [767] = 755,
  [768] = 755,
  [769] = 756,
  [770] = 756,
  [771] = 771,
  [772] = 772,
  [773] = 505,
  [774] = 774,
  [775] = 603,
  [776] = 776,
  [777] = 777,
  [778] = 778,
  [779] = 517,
  [780] = 780,
  [781] = 781,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 443,
  [786] = 577,
  [787] = 578,
  [788] = 604,
  [789] = 789,
  [790] = 790,
  [791] = 791,
  [792] = 792,
  [793] = 793,
  [794] = 460,
  [795] = 459,
  [796] = 796,
  [797] = 684,
  [798] = 798,
  [799] = 799,
  [800] = 800,
  [801] = 801,
  [802] = 802,
  [803] = 803,
  [804] = 804,
  [805] = 697,
  [806] = 806,
  [807] = 807,
  [808] = 808,
  [809] = 809,
  [810] = 810,
  [811] = 455,
  [812] = 812,
  [813] = 696,
  [814] = 814,
  [815] = 815,
  [816] = 816,
  [817] = 685,
  [818] = 449,
  [819] = 819,
  [820] = 708,
  [821] = 821,
  [822] = 822,
  [823] = 706,
  [824] = 452,
  [825] = 825,
  [826] = 456,
  [827] = 827,
  [828] = 828,
  [829] = 829,
  [830] = 653,
  [831] = 451,
  [832] = 695,
  [833] = 453,
  [834] = 700,
  [835] = 465,
  [836] = 836,
  [837] = 837,
  [838] = 838,
  [839] = 839,
  [840] = 840,
  [841] = 841,
  [842] = 842,
  [843] = 843,
  [844] = 844,
  [845] = 845,
  [846] = 846,
  [847] = 847,
  [848] = 848,
  [849] = 849,
  [850] = 850,
  [851] = 851,
  [852] = 849,
  [853] = 853,
  [854] = 854,
  [855] = 855,
  [856] = 856,
  [857] = 857,
  [858] = 856,
  [859] = 859,
  [860] = 856,
  [861] = 859,
  [862] = 862,
  [863] = 851,
  [864] = 855,
  [865] = 857,
  [866] = 849,
  [867] = 847,
  [868] = 857,
  [869] = 856,
  [870] = 857,
  [871] = 850,
  [872] = 857,
  [873] = 856,
  [874] = 854,
  [875] = 859,
  [876] = 851,
  [877] = 848,
  [878] = 856,
  [879] = 849,
  [880] = 857,
  [881] = 849,
  [882] = 849,
  [883] = 883,
  [884] = 883,
  [885] = 297,
  [886] = 886,
  [887] = 887,
  [888] = 784,
  [889] = 887,
  [890] = 890,
  [891] = 839,
  [892] = 890,
  [893] = 893,
  [894] = 783,
  [895] = 890,
  [896] = 890,
  [897] = 897,
  [898] = 841,
  [899] = 886,
  [900] = 840,
  [901] = 838,
  [902] = 837,
  [903] = 843,
  [904] = 844,
  [905] = 846,
  [906] = 845,
  [907] = 836,
  [908] = 298,
  [909] = 909,
  [910] = 887,
  [911] = 911,
  [912] = 912,
  [913] = 890,
  [914] = 332,
  [915] = 915,
  [916] = 890,
  [917] = 301,
  [918] = 842,
  [919] = 839,
  [920] = 836,
  [921] = 921,
  [922] = 922,
  [923] = 923,
  [924] = 838,
  [925] = 921,
  [926] = 840,
  [927] = 927,
  [928] = 928,
  [929] = 921,
  [930] = 930,
  [931] = 841,
  [932] = 932,
  [933] = 933,
  [934] = 921,
  [935] = 922,
  [936] = 936,
  [937] = 933,
  [938] = 938,
  [939] = 939,
  [940] = 922,
  [941] = 941,
  [942] = 933,
  [943] = 933,
  [944] = 944,
  [945] = 945,
  [946] = 933,
  [947] = 842,
  [948] = 948,
  [949] = 949,
  [950] = 839,
  [951] = 951,
  [952] = 842,
  [953] = 944,
  [954] = 922,
  [955] = 955,
  [956] = 956,
  [957] = 957,
  [958] = 958,
  [959] = 944,
  [960] = 845,
  [961] = 837,
  [962] = 846,
  [963] = 922,
  [964] = 964,
  [965] = 965,
  [966] = 966,
  [967] = 844,
  [968] = 843,
  [969] = 921,
  [970] = 837,
  [971] = 838,
  [972] = 972,
  [973] = 840,
  [974] = 841,
  [975] = 836,
  [976] = 845,
  [977] = 846,
  [978] = 978,
  [979] = 933,
  [980] = 922,
  [981] = 939,
  [982] = 844,
  [983] = 843,
  [984] = 923,
  [985] = 985,
  [986] = 957,
  [987] = 987,
  [988] = 955,
  [989] = 956,
  [990] = 930,
  [991] = 921,
  [992] = 839,
  [993] = 933,
  [994] = 922,
  [995] = 995,
  [996] = 921,
  [997] = 933,
  [998] = 958,
  [999] = 921,
  [1000] = 1000,
  [1001] = 460,
  [1002] = 1002,
  [1003] = 1003,
  [1004] = 1003,
  [1005] = 911,
  [1006] = 1006,
  [1007] = 897,
  [1008] = 911,
  [1009] = 792,
  [1010] = 1010,
  [1011] = 449,
  [1012] = 1012,
  [1013] = 1013,
  [1014] = 1014,
  [1015] = 1015,
  [1016] = 1016,
  [1017] = 1017,
  [1018] = 332,
  [1019] = 465,
  [1020] = 783,
  [1021] = 1021,
  [1022] = 1022,
  [1023] = 1023,
  [1024] = 328,
  [1025] = 1025,
  [1026] = 1026,
  [1027] = 1027,
  [1028] = 784,
  [1029] = 301,
  [1030] = 1030,
  [1031] = 1031,
  [1032] = 1032,
  [1033] = 1033,
  [1034] = 298,
  [1035] = 1035,
  [1036] = 297,
  [1037] = 1037,
  [1038] = 1038,
  [1039] = 1039,
  [1040] = 1040,
  [1041] = 1041,
  [1042] = 790,
  [1043] = 1043,
  [1044] = 1044,
  [1045] = 432,
  [1046] = 1046,
  [1047] = 1047,
  [1048] = 1048,
  [1049] = 1049,
  [1050] = 1050,
  [1051] = 1051,
  [1052] = 1052,
  [1053] = 1053,
  [1054] = 1054,
  [1055] = 1055,
  [1056] = 1056,
  [1057] = 1057,
  [1058] = 1058,
  [1059] = 1059,
  [1060] = 1060,
  [1061] = 1061,
  [1062] = 1062,
  [1063] = 1063,
  [1064] = 1064,
  [1065] = 1040,
  [1066] = 1066,
  [1067] = 1067,
  [1068] = 1068,
  [1069] = 1069,
  [1070] = 1070,
  [1071] = 1071,
  [1072] = 1072,
  [1073] = 1073,
  [1074] = 1074,
  [1075] = 1075,
  [1076] = 1073,
  [1077] = 1077,
  [1078] = 1040,
  [1079] = 1079,
  [1080] = 1080,
  [1081] = 1040,
  [1082] = 1016,
  [1083] = 1083,
  [1084] = 1084,
  [1085] = 1085,
  [1086] = 1086,
  [1087] = 1087,
  [1088] = 1088,
  [1089] = 1089,
  [1090] = 1088,
  [1091] = 1091,
  [1092] = 1092,
  [1093] = 1093,
  [1094] = 1094,
  [1095] = 1095,
  [1096] = 1096,
  [1097] = 1073,
  [1098] = 1040,
  [1099] = 1099,
  [1100] = 1100,
  [1101] = 1101,
  [1102] = 1101,
  [1103] = 1040,
  [1104] = 1104,
  [1105] = 1105,
  [1106] = 1096,
  [1107] = 839,
  [1108] = 1040,
  [1109] = 1099,
  [1110] = 1110,
  [1111] = 1111,
  [1112] = 1112,
  [1113] = 1113,
  [1114] = 1114,
  [1115] = 1115,
  [1116] = 1116,
  [1117] = 427,
  [1118] = 1118,
  [1119] = 1073,
  [1120] = 1120,
  [1121] = 1121,
  [1122] = 1122,
  [1123] = 1123,
  [1124] = 1124,
  [1125] = 1120,
  [1126] = 1126,
  [1127] = 460,
  [1128] = 449,
  [1129] = 1129,
  [1130] = 1130,
  [1131] = 1131,
  [1132] = 1132,
  [1133] = 465,
  [1134] = 1134,
  [1135] = 1132,
  [1136] = 1136,
  [1137] = 1137,
  [1138] = 1138,
  [1139] = 806,
  [1140] = 802,
  [1141] = 790,
  [1142] = 807,
  [1143] = 821,
  [1144] = 809,
  [1145] = 1145,
  [1146] = 812,
  [1147] = 815,
  [1148] = 1148,
  [1149] = 1149,
  [1150] = 1150,
  [1151] = 1130,
  [1152] = 1152,
  [1153] = 1153,
  [1154] = 796,
  [1155] = 808,
  [1156] = 803,
  [1157] = 804,
  [1158] = 1158,
  [1159] = 279,
  [1160] = 1160,
  [1161] = 1161,
  [1162] = 1162,
  [1163] = 1163,
  [1164] = 427,
  [1165] = 1161,
  [1166] = 754,
  [1167] = 1167,
  [1168] = 1168,
  [1169] = 1169,
  [1170] = 829,
  [1171] = 1171,
  [1172] = 1172,
  [1173] = 1173,
  [1174] = 1163,
  [1175] = 1175,
  [1176] = 1176,
  [1177] = 1177,
  [1178] = 280,
  [1179] = 267,
  [1180] = 1180,
  [1181] = 1181,
  [1182] = 1182,
  [1183] = 235,
  [1184] = 271,
  [1185] = 261,
  [1186] = 270,
  [1187] = 780,
  [1188] = 1162,
  [1189] = 1189,
  [1190] = 1190,
  [1191] = 240,
  [1192] = 241,
  [1193] = 242,
  [1194] = 244,
  [1195] = 249,
  [1196] = 250,
  [1197] = 275,
  [1198] = 1162,
  [1199] = 276,
  [1200] = 1180,
  [1201] = 289,
  [1202] = 1180,
  [1203] = 1176,
  [1204] = 1204,
  [1205] = 1162,
  [1206] = 1206,
  [1207] = 1176,
  [1208] = 1208,
  [1209] = 1209,
  [1210] = 1210,
  [1211] = 1162,
  [1212] = 1212,
  [1213] = 1169,
  [1214] = 1214,
  [1215] = 1176,
  [1216] = 1216,
  [1217] = 1176,
  [1218] = 1176,
  [1219] = 1162,
  [1220] = 1220,
  [1221] = 1162,
  [1222] = 1222,
  [1223] = 1176,
  [1224] = 1224,
  [1225] = 1225,
  [1226] = 1226,
  [1227] = 1227,
  [1228] = 1225,
  [1229] = 1229,
  [1230] = 1224,
  [1231] = 1231,
  [1232] = 1232,
  [1233] = 1233,
  [1234] = 1234,
  [1235] = 1235,
  [1236] = 1233,
  [1237] = 1234,
  [1238] = 825,
  [1239] = 1239,
  [1240] = 1240,
  [1241] = 1241,
  [1242] = 1227,
  [1243] = 1231,
  [1244] = 1244,
  [1245] = 1245,
  [1246] = 1233,
  [1247] = 1234,
  [1248] = 1241,
  [1249] = 1227,
  [1250] = 1250,
  [1251] = 1250,
  [1252] = 1229,
  [1253] = 1224,
  [1254] = 1233,
  [1255] = 1245,
  [1256] = 1241,
  [1257] = 1225,
  [1258] = 1241,
  [1259] = 1245,
  [1260] = 1250,
  [1261] = 1261,
  [1262] = 1235,
  [1263] = 1245,
  [1264] = 1234,
  [1265] = 1250,
  [1266] = 814,
  [1267] = 1226,
  [1268] = 1268,
  [1269] = 1229,
  [1270] = 1229,
  [1271] = 1224,
  [1272] = 1272,
  [1273] = 1229,
  [1274] = 1224,
  [1275] = 1275,
  [1276] = 1276,
  [1277] = 1233,
  [1278] = 1234,
  [1279] = 1250,
  [1280] = 1224,
  [1281] = 1281,
  [1282] = 1233,
  [1283] = 1283,
  [1284] = 1234,
  [1285] = 1168,
  [1286] = 1286,
  [1287] = 1227,
  [1288] = 1231,
  [1289] = 828,
  [1290] = 1227,
  [1291] = 1231,
  [1292] = 1227,
  [1293] = 1231,
  [1294] = 1272,
  [1295] = 1245,
  [1296] = 1296,
  [1297] = 1006,
  [1298] = 1245,
  [1299] = 1241,
  [1300] = 1245,
  [1301] = 1225,
  [1302] = 1235,
  [1303] = 1303,
  [1304] = 1231,
  [1305] = 1272,
  [1306] = 1241,
  [1307] = 1235,
  [1308] = 1225,
  [1309] = 1250,
  [1310] = 1250,
  [1311] = 1229,
  [1312] = 1227,
  [1313] = 1231,
  [1314] = 1225,
  [1315] = 1235,
  [1316] = 1235,
  [1317] = 1275,
  [1318] = 1229,
  [1319] = 1225,
  [1320] = 1224,
  [1321] = 1272,
  [1322] = 1322,
  [1323] = 1233,
  [1324] = 1272,
  [1325] = 1235,
  [1326] = 1326,
  [1327] = 1272,
  [1328] = 1225,
  [1329] = 1241,
  [1330] = 1234,
  [1331] = 1331,
  [1332] = 1235,
  [1333] = 297,
  [1334] = 1334,
  [1335] = 1335,
  [1336] = 298,
  [1337] = 1337,
  [1338] = 505,
  [1339] = 1339,
  [1340] = 1340,
  [1341] = 1341,
  [1342] = 1342,
  [1343] = 1343,
  [1344] = 1335,
  [1345] = 1345,
  [1346] = 297,
  [1347] = 332,
  [1348] = 301,
  [1349] = 1349,
  [1350] = 1350,
  [1351] = 1351,
  [1352] = 1352,
  [1353] = 1353,
  [1354] = 1352,
  [1355] = 1355,
  [1356] = 1339,
  [1357] = 1357,
  [1358] = 1358,
  [1359] = 1359,
  [1360] = 1360,
  [1361] = 1361,
  [1362] = 1358,
  [1363] = 1363,
  [1364] = 1364,
  [1365] = 1361,
  [1366] = 1366,
  [1367] = 1367,
  [1368] = 1368,
  [1369] = 1369,
  [1370] = 1370,
  [1371] = 1357,
  [1372] = 1372,
  [1373] = 1373,
  [1374] = 298,
  [1375] = 1339,
  [1376] = 1376,
  [1377] = 1341,
  [1378] = 1378,
  [1379] = 1379,
  [1380] = 1335,
  [1381] = 301,
  [1382] = 1341,
  [1383] = 332,
  [1384] = 1370,
  [1385] = 1385,
  [1386] = 1386,
  [1387] = 1357,
  [1388] = 1388,
  [1389] = 1339,
  [1390] = 1335,
  [1391] = 1391,
  [1392] = 1392,
  [1393] = 1393,
  [1394] = 1357,
  [1395] = 1395,
  [1396] = 1358,
  [1397] = 1357,
  [1398] = 1339,
  [1399] = 1370,
  [1400] = 1361,
  [1401] = 1401,
  [1402] = 1358,
  [1403] = 1352,
  [1404] = 1370,
  [1405] = 1361,
  [1406] = 1358,
  [1407] = 1361,
  [1408] = 1033,
  [1409] = 1409,
  [1410] = 1410,
  [1411] = 1411,
  [1412] = 1085,
  [1413] = 1370,
  [1414] = 1414,
  [1415] = 1415,
  [1416] = 1416,
  [1417] = 1370,
  [1418] = 1418,
  [1419] = 1361,
  [1420] = 1420,
  [1421] = 1370,
  [1422] = 1352,
  [1423] = 1341,
  [1424] = 1358,
  [1425] = 1341,
  [1426] = 1426,
  [1427] = 1339,
  [1428] = 1428,
  [1429] = 1341,
  [1430] = 1335,
  [1431] = 1431,
  [1432] = 1335,
  [1433] = 1358,
  [1434] = 1434,
  [1435] = 1352,
  [1436] = 1436,
  [1437] = 1437,
  [1438] = 1357,
  [1439] = 1339,
  [1440] = 1361,
  [1441] = 1358,
  [1442] = 1357,
  [1443] = 1443,
  [1444] = 1335,
  [1445] = 1339,
  [1446] = 1446,
  [1447] = 1447,
  [1448] = 1361,
  [1449] = 1335,
  [1450] = 1352,
  [1451] = 1451,
  [1452] = 1452,
  [1453] = 1453,
  [1454] = 1454,
  [1455] = 1455,
  [1456] = 1456,
  [1457] = 1457,
  [1458] = 1458,
  [1459] = 1459,
  [1460] = 1460,
  [1461] = 1461,
  [1462] = 1462,
  [1463] = 1463,
  [1464] = 1464,
  [1465] = 1465,
  [1466] = 1466,
  [1467] = 1467,
  [1468] = 1468,
  [1469] = 1469,
  [1470] = 1470,
  [1471] = 1460,
  [1472] = 1472,
  [1473] = 1458,
  [1474] = 1461,
  [1475] = 1465,
  [1476] = 1476,
  [1477] = 1477,
  [1478] = 1478,
  [1479] = 1479,
  [1480] = 1476,
  [1481] = 1481,
  [1482] = 1482,
  [1483] = 1483,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 1486,
  [1487] = 1487,
  [1488] = 1488,
  [1489] = 1489,
  [1490] = 1490,
  [1491] = 1145,
  [1492] = 1451,
  [1493] = 1493,
  [1494] = 1494,
  [1495] = 1495,
  [1496] = 1452,
  [1497] = 1497,
  [1498] = 1486,
  [1499] = 1499,
  [1500] = 1500,
  [1501] = 1501,
  [1502] = 1502,
  [1503] = 1503,
  [1504] = 1504,
  [1505] = 1479,
  [1506] = 1138,
  [1507] = 1507,
  [1508] = 1137,
  [1509] = 1136,
  [1510] = 1510,
  [1511] = 1511,
  [1512] = 1512,
  [1513] = 1468,
  [1514] = 1514,
  [1515] = 1510,
  [1516] = 1466,
  [1517] = 1517,
  [1518] = 1518,
  [1519] = 1519,
  [1520] = 1520,
  [1521] = 1484,
  [1522] = 1522,
  [1523] = 1489,
  [1524] = 1524,
  [1525] = 1525,
  [1526] = 1526,
  [1527] = 1527,
  [1528] = 1528,
  [1529] = 1529,
  [1530] = 1530,
  [1531] = 1531,
  [1532] = 1532,
  [1533] = 1533,
  [1534] = 1534,
  [1535] = 578,
  [1536] = 577,
  [1537] = 1537,
  [1538] = 1538,
  [1539] = 1537,
  [1540] = 1540,
  [1541] = 1541,
  [1542] = 1542,
  [1543] = 1526,
  [1544] = 1526,
  [1545] = 1545,
  [1546] = 1527,
  [1547] = 1547,
  [1548] = 1527,
  [1549] = 1549,
  [1550] = 1550,
  [1551] = 1534,
  [1552] = 1552,
  [1553] = 1538,
  [1554] = 604,
  [1555] = 603,
  [1556] = 1556,
  [1557] = 1557,
  [1558] = 1556,
  [1559] = 1525,
  [1560] = 708,
  [1561] = 706,
  [1562] = 700,
  [1563] = 653,
  [1564] = 279,
  [1565] = 235,
  [1566] = 1566,
  [1567] = 1567,
  [1568] = 1567,
  [1569] = 1569,
  [1570] = 1570,
  [1571] = 1571,
  [1572] = 267,
  [1573] = 1573,
  [1574] = 1571,
  [1575] = 1566,
  [1576] = 1576,
  [1577] = 1573,
  [1578] = 1578,
  [1579] = 289,
  [1580] = 279,
  [1581] = 276,
  [1582] = 275,
  [1583] = 280,
  [1584] = 235,
  [1585] = 1585,
  [1586] = 250,
  [1587] = 249,
  [1588] = 244,
  [1589] = 271,
  [1590] = 242,
  [1591] = 241,
  [1592] = 240,
  [1593] = 261,
  [1594] = 270,
  [1595] = 1585,
  [1596] = 695,
  [1597] = 240,
  [1598] = 241,
  [1599] = 242,
  [1600] = 244,
  [1601] = 249,
  [1602] = 250,
  [1603] = 1585,
  [1604] = 270,
  [1605] = 275,
  [1606] = 261,
  [1607] = 276,
  [1608] = 271,
  [1609] = 235,
  [1610] = 289,
  [1611] = 1578,
  [1612] = 1576,
  [1613] = 1566,
  [1614] = 1571,
  [1615] = 1573,
  [1616] = 1570,
  [1617] = 1569,
  [1618] = 280,
  [1619] = 267,
  [1620] = 1567,
  [1621] = 1567,
  [1622] = 267,
  [1623] = 280,
  [1624] = 271,
  [1625] = 261,
  [1626] = 270,
  [1627] = 240,
  [1628] = 241,
  [1629] = 242,
  [1630] = 244,
  [1631] = 249,
  [1632] = 250,
  [1633] = 1585,
  [1634] = 275,
  [1635] = 276,
  [1636] = 279,
  [1637] = 289,
  [1638] = 1578,
  [1639] = 1567,
  [1640] = 1576,
  [1641] = 1566,
  [1642] = 1571,
  [1643] = 1573,
  [1644] = 1570,
  [1645] = 1569,
  [1646] = 280,
  [1647] = 267,
  [1648] = 1567,
  [1649] = 235,
  [1650] = 271,
  [1651] = 261,
  [1652] = 270,
  [1653] = 240,
  [1654] = 241,
  [1655] = 1569,
  [1656] = 1570,
  [1657] = 242,
  [1658] = 244,
  [1659] = 249,
  [1660] = 250,
  [1661] = 1585,
  [1662] = 1573,
  [1663] = 1571,
  [1664] = 1566,
  [1665] = 275,
  [1666] = 276,
  [1667] = 1578,
  [1668] = 1576,
  [1669] = 279,
  [1670] = 289,
  [1671] = 1578,
  [1672] = 1576,
  [1673] = 1576,
  [1674] = 1578,
  [1675] = 1566,
  [1676] = 1571,
  [1677] = 1573,
  [1678] = 1570,
  [1679] = 1569,
  [1680] = 280,
  [1681] = 267,
  [1682] = 235,
  [1683] = 271,
  [1684] = 261,
  [1685] = 270,
  [1686] = 240,
  [1687] = 241,
  [1688] = 242,
  [1689] = 1585,
  [1690] = 244,
  [1691] = 249,
  [1692] = 250,
  [1693] = 275,
  [1694] = 289,
  [1695] = 279,
  [1696] = 276,
  [1697] = 275,
  [1698] = 276,
  [1699] = 250,
  [1700] = 249,
  [1701] = 244,
  [1702] = 1569,
  [1703] = 1570,
  [1704] = 242,
  [1705] = 241,
  [1706] = 240,
  [1707] = 270,
  [1708] = 279,
  [1709] = 261,
  [1710] = 271,
  [1711] = 235,
  [1712] = 267,
  [1713] = 289,
  [1714] = 280,
  [1715] = 684,
  [1716] = 1716,
  [1717] = 697,
  [1718] = 696,
  [1719] = 685,
  [1720] = 1720,
  [1721] = 1721,
  [1722] = 1722,
  [1723] = 1723,
  [1724] = 1724,
  [1725] = 1725,
  [1726] = 1726,
  [1727] = 1727,
  [1728] = 1728,
  [1729] = 1729,
  [1730] = 1730,
  [1731] = 1731,
  [1732] = 1732,
  [1733] = 1733,
  [1734] = 1734,
  [1735] = 1735,
  [1736] = 1736,
  [1737] = 1737,
  [1738] = 1738,
  [1739] = 1739,
  [1740] = 1740,
  [1741] = 1741,
  [1742] = 1033,
  [1743] = 1743,
  [1744] = 1744,
  [1745] = 1745,
  [1746] = 1746,
  [1747] = 1747,
  [1748] = 1748,
  [1749] = 1749,
  [1750] = 1750,
  [1751] = 1055,
  [1752] = 1043,
  [1753] = 1089,
  [1754] = 1754,
  [1755] = 1755,
  [1756] = 1756,
  [1757] = 1757,
  [1758] = 1116,
  [1759] = 1114,
  [1760] = 1113,
  [1761] = 1061,
  [1762] = 1112,
  [1763] = 1111,
  [1764] = 1059,
  [1765] = 1044,
  [1766] = 1067,
  [1767] = 1060,
  [1768] = 1056,
  [1769] = 1083,
  [1770] = 1770,
  [1771] = 1050,
  [1772] = 1049,
  [1773] = 1037,
  [1774] = 1774,
  [1775] = 1063,
  [1776] = 1093,
  [1777] = 1092,
  [1778] = 1091,
  [1779] = 1048,
  [1780] = 1062,
  [1781] = 1781,
  [1782] = 1782,
  [1783] = 1075,
  [1784] = 1784,
  [1785] = 1074,
  [1786] = 1080,
  [1787] = 1072,
  [1788] = 1070,
  [1789] = 1115,
  [1790] = 1790,
  [1791] = 1152,
  [1792] = 1134,
  [1793] = 1793,
  [1794] = 1150,
  [1795] = 1790,
  [1796] = 1796,
  [1797] = 1797,
  [1798] = 1798,
  [1799] = 1799,
  [1800] = 1793,
  [1801] = 1801,
  [1802] = 1802,
  [1803] = 1803,
  [1804] = 1804,
  [1805] = 1799,
  [1806] = 1806,
  [1807] = 1807,
  [1808] = 1808,
  [1809] = 1809,
  [1810] = 1810,
  [1811] = 1811,
  [1812] = 1812,
  [1813] = 1813,
  [1814] = 1814,
  [1815] = 1815,
  [1816] = 1816,
  [1817] = 1817,
  [1818] = 1818,
  [1819] = 1819,
  [1820] = 1820,
  [1821] = 1821,
  [1822] = 1822,
  [1823] = 1823,
  [1824] = 1824,
  [1825] = 1825,
  [1826] = 1826,
  [1827] = 1827,
  [1828] = 1828,
  [1829] = 1829,
  [1830] = 1830,
  [1831] = 1831,
  [1832] = 1832,
  [1833] = 1833,
  [1834] = 1834,
  [1835] = 1835,
  [1836] = 1836,
  [1837] = 1837,
  [1838] = 1838,
  [1839] = 1839,
  [1840] = 1840,
  [1841] = 1834,
  [1842] = 1842,
  [1843] = 1834,
  [1844] = 1844,
  [1845] = 1845,
  [1846] = 1846,
  [1847] = 1847,
  [1848] = 1834,
  [1849] = 1844,
  [1850] = 1850,
  [1851] = 1834,
  [1852] = 1844,
  [1853] = 1844,
  [1854] = 1854,
  [1855] = 1834,
  [1856] = 1844,
  [1857] = 1834,
  [1858] = 1844,
  [1859] = 1844,
  [1860] = 1860,
  [1861] = 1861,
  [1862] = 1862,
  [1863] = 1863,
  [1864] = 1864,
  [1865] = 1865,
  [1866] = 1866,
  [1867] = 1867,
  [1868] = 1868,
  [1869] = 1869,
  [1870] = 1870,
  [1871] = 1871,
  [1872] = 1872,
  [1873] = 1873,
  [1874] = 432,
  [1875] = 1875,
  [1876] = 1876,
  [1877] = 1877,
  [1878] = 1878,
  [1879] = 1879,
  [1880] = 1880,
  [1881] = 1881,
  [1882] = 1882,
  [1883] = 1883,
  [1884] = 1884,
  [1885] = 1885,
  [1886] = 1886,
  [1887] = 1887,
  [1888] = 1888,
  [1889] = 1889,
  [1890] = 1890,
  [1891] = 1891,
  [1892] = 1892,
  [1893] = 1893,
  [1894] = 1894,
  [1895] = 1895,
  [1896] = 1896,
  [1897] = 1897,
  [1898] = 1898,
  [1899] = 1899,
  [1900] = 1899,
  [1901] = 1901,
  [1902] = 1902,
  [1903] = 1903,
  [1904] = 1904,
  [1905] = 328,
  [1906] = 1906,
  [1907] = 1907,
  [1908] = 1276,
  [1909] = 1909,
  [1910] = 1910,
  [1911] = 1911,
  [1912] = 1520,
  [1913] = 1033,
  [1914] = 1914,
  [1915] = 1915,
  [1916] = 1268,
  [1917] = 1006,
  [1918] = 1918,
  [1919] = 1919,
  [1920] = 1920,
  [1921] = 1899,
  [1922] = 1922,
  [1923] = 1923,
  [1924] = 1924,
  [1925] = 1925,
  [1926] = 1926,
  [1927] = 1927,
  [1928] = 1928,
  [1929] = 1929,
  [1930] = 1930,
  [1931] = 1931,
  [1932] = 1932,
  [1933] = 1933,
  [1934] = 1934,
  [1935] = 1935,
  [1936] = 1936,
  [1937] = 1937,
  [1938] = 1938,
  [1939] = 1939,
  [1940] = 1940,
  [1941] = 1941,
  [1942] = 1942,
  [1943] = 1943,
  [1944] = 1944,
  [1945] = 1943,
  [1946] = 1946,
  [1947] = 1947,
  [1948] = 1943,
  [1949] = 1949,
  [1950] = 1950,
  [1951] = 1943,
  [1952] = 1952,
  [1953] = 1933,
  [1954] = 1954,
  [1955] = 1955,
  [1956] = 1956,
  [1957] = 1957,
  [1958] = 1958,
  [1959] = 1959,
  [1960] = 1960,
  [1961] = 1961,
  [1962] = 1962,
  [1963] = 1957,
  [1964] = 1943,
  [1965] = 1965,
  [1966] = 1937,
  [1967] = 1928,
  [1968] = 1954,
  [1969] = 1950,
  [1970] = 1970,
  [1971] = 1971,
  [1972] = 1952,
  [1973] = 1962,
  [1974] = 1974,
  [1975] = 1975,
  [1976] = 1957,
  [1977] = 1977,
  [1978] = 1933,
  [1979] = 1979,
  [1980] = 1961,
  [1981] = 1981,
  [1982] = 1982,
  [1983] = 1928,
  [1984] = 1984,
  [1985] = 1959,
  [1986] = 1986,
  [1987] = 1987,
  [1988] = 1929,
  [1989] = 1989,
  [1990] = 1990,
  [1991] = 1991,
  [1992] = 1992,
  [1993] = 1993,
  [1994] = 1943,
  [1995] = 1995,
  [1996] = 1996,
  [1997] = 1997,
  [1998] = 1954,
  [1999] = 1999,
  [2000] = 2000,
  [2001] = 1952,
  [2002] = 1959,
  [2003] = 2003,
  [2004] = 2004,
  [2005] = 2005,
  [2006] = 1961,
  [2007] = 2007,
  [2008] = 2008,
  [2009] = 2009,
  [2010] = 1970,
  [2011] = 2011,
  [2012] = 2012,
  [2013] = 2013,
  [2014] = 2014,
  [2015] = 2015,
  [2016] = 1941,
  [2017] = 1937,
  [2018] = 2018,
  [2019] = 1950,
  [2020] = 1965,
  [2021] = 2021,
  [2022] = 1965,
  [2023] = 2023,
  [2024] = 2024,
  [2025] = 1958,
  [2026] = 1933,
  [2027] = 2027,
  [2028] = 1984,
  [2029] = 2029,
  [2030] = 1957,
  [2031] = 1095,
  [2032] = 2032,
  [2033] = 2033,
  [2034] = 1941,
  [2035] = 2035,
  [2036] = 2036,
  [2037] = 2037,
  [2038] = 1941,
  [2039] = 2039,
  [2040] = 2040,
  [2041] = 1954,
  [2042] = 2042,
  [2043] = 2043,
  [2044] = 1958,
  [2045] = 1952,
  [2046] = 2046,
  [2047] = 1959,
  [2048] = 2048,
  [2049] = 1961,
  [2050] = 2050,
  [2051] = 1958,
  [2052] = 2052,
  [2053] = 2053,
  [2054] = 2054,
  [2055] = 1962,
  [2056] = 1928,
  [2057] = 2057,
  [2058] = 2058,
  [2059] = 2059,
  [2060] = 2060,
  [2061] = 1929,
  [2062] = 2062,
  [2063] = 2063,
  [2064] = 1959,
  [2065] = 2065,
  [2066] = 2066,
  [2067] = 1933,
  [2068] = 2068,
  [2069] = 1970,
  [2070] = 2070,
  [2071] = 1937,
  [2072] = 2072,
  [2073] = 1961,
  [2074] = 1950,
  [2075] = 1871,
  [2076] = 1485,
  [2077] = 2077,
  [2078] = 2078,
  [2079] = 1965,
  [2080] = 1416,
  [2081] = 2081,
  [2082] = 1970,
  [2083] = 2083,
  [2084] = 1965,
  [2085] = 2085,
  [2086] = 2086,
  [2087] = 2087,
  [2088] = 1958,
  [2089] = 2089,
  [2090] = 2090,
  [2091] = 2091,
  [2092] = 2092,
  [2093] = 2093,
  [2094] = 2094,
  [2095] = 1087,
  [2096] = 2096,
  [2097] = 2097,
  [2098] = 1933,
  [2099] = 1984,
  [2100] = 2100,
  [2101] = 1950,
  [2102] = 1957,
  [2103] = 2103,
  [2104] = 2104,
  [2105] = 1984,
  [2106] = 2106,
  [2107] = 1894,
  [2108] = 1941,
  [2109] = 2109,
  [2110] = 2110,
  [2111] = 2111,
  [2112] = 2112,
  [2113] = 1954,
  [2114] = 2114,
  [2115] = 1929,
  [2116] = 1957,
  [2117] = 1952,
  [2118] = 2118,
  [2119] = 2119,
  [2120] = 1937,
  [2121] = 1929,
  [2122] = 1959,
  [2123] = 2123,
  [2124] = 2066,
  [2125] = 2125,
  [2126] = 2126,
  [2127] = 1961,
  [2128] = 1928,
  [2129] = 1928,
  [2130] = 2130,
  [2131] = 1943,
  [2132] = 2132,
  [2133] = 1929,
  [2134] = 2134,
  [2135] = 1971,
  [2136] = 2136,
  [2137] = 2137,
  [2138] = 2138,
  [2139] = 2139,
  [2140] = 1971,
  [2141] = 2141,
  [2142] = 1962,
  [2143] = 1971,
  [2144] = 2144,
  [2145] = 2145,
  [2146] = 1962,
  [2147] = 1069,
  [2148] = 1068,
  [2149] = 2149,
  [2150] = 2150,
  [2151] = 1058,
  [2152] = 1995,
  [2153] = 1971,
  [2154] = 2154,
  [2155] = 2155,
  [2156] = 2156,
  [2157] = 2157,
  [2158] = 2158,
  [2159] = 1386,
  [2160] = 1512,
  [2161] = 1937,
  [2162] = 2162,
  [2163] = 2163,
  [2164] = 1933,
  [2165] = 2165,
  [2166] = 2166,
  [2167] = 1950,
  [2168] = 1957,
  [2169] = 2169,
  [2170] = 2170,
  [2171] = 2171,
  [2172] = 2172,
  [2173] = 1961,
  [2174] = 1959,
  [2175] = 2175,
  [2176] = 2176,
  [2177] = 1954,
  [2178] = 1979,
  [2179] = 1952,
  [2180] = 1884,
  [2181] = 1952,
  [2182] = 1965,
  [2183] = 2183,
  [2184] = 1970,
  [2185] = 1954,
  [2186] = 2186,
  [2187] = 2187,
  [2188] = 2188,
  [2189] = 1962,
  [2190] = 1941,
  [2191] = 2191,
  [2192] = 2192,
  [2193] = 2193,
  [2194] = 2194,
  [2195] = 2195,
  [2196] = 2196,
  [2197] = 2197,
  [2198] = 1436,
  [2199] = 1984,
  [2200] = 1958,
  [2201] = 2201,
  [2202] = 2202,
  [2203] = 1958,
  [2204] = 1100,
  [2205] = 1984,
  [2206] = 2206,
  [2207] = 2207,
  [2208] = 1933,
  [2209] = 2209,
  [2210] = 1970,
  [2211] = 2211,
  [2212] = 1957,
  [2213] = 1965,
  [2214] = 1941,
  [2215] = 2215,
  [2216] = 2216,
  [2217] = 1954,
  [2218] = 2218,
  [2219] = 2139,
  [2220] = 2220,
  [2221] = 1952,
  [2222] = 2222,
  [2223] = 1959,
  [2224] = 2224,
  [2225] = 1961,
  [2226] = 1950,
  [2227] = 2227,
  [2228] = 2228,
  [2229] = 1971,
  [2230] = 2230,
  [2231] = 2231,
  [2232] = 2232,
  [2233] = 2233,
  [2234] = 1962,
  [2235] = 2235,
  [2236] = 1937,
  [2237] = 2237,
  [2238] = 2238,
  [2239] = 2239,
  [2240] = 2240,
  [2241] = 2241,
  [2242] = 2242,
  [2243] = 2243,
  [2244] = 2244,
  [2245] = 2245,
  [2246] = 2246,
  [2247] = 2247,
  [2248] = 2248,
  [2249] = 2249,
  [2250] = 2250,
  [2251] = 2251,
  [2252] = 1138,
  [2253] = 2253,
  [2254] = 1459,
  [2255] = 1137,
  [2256] = 2256,
  [2257] = 1136,
  [2258] = 2258,
  [2259] = 2259,
  [2260] = 2260,
  [2261] = 2261,
  [2262] = 2262,
  [2263] = 2263,
  [2264] = 2264,
  [2265] = 2265,
  [2266] = 2266,
  [2267] = 2267,
  [2268] = 2268,
  [2269] = 2269,
  [2270] = 2270,
  [2271] = 2271,
  [2272] = 2272,
  [2273] = 2273,
  [2274] = 2274,
  [2275] = 2275,
  [2276] = 2276,
  [2277] = 2277,
  [2278] = 2278,
  [2279] = 2279,
  [2280] = 2280,
  [2281] = 2281,
  [2282] = 2282,
  [2283] = 2283,
  [2284] = 2284,
  [2285] = 2285,
  [2286] = 2286,
  [2287] = 2287,
  [2288] = 2285,
  [2289] = 2289,
  [2290] = 2290,
  [2291] = 2291,
  [2292] = 2292,
  [2293] = 2293,
  [2294] = 2294,
  [2295] = 2295,
  [2296] = 2296,
  [2297] = 2297,
  [2298] = 2298,
  [2299] = 2299,
  [2300] = 2300,
  [2301] = 2301,
  [2302] = 2253,
  [2303] = 2303,
  [2304] = 2304,
  [2305] = 2305,
  [2306] = 2306,
  [2307] = 2307,
  [2308] = 2308,
  [2309] = 2309,
  [2310] = 2310,
  [2311] = 2311,
  [2312] = 2312,
  [2313] = 2313,
  [2314] = 2314,
  [2315] = 2315,
  [2316] = 2316,
  [2317] = 2317,
  [2318] = 1497,
  [2319] = 2319,
  [2320] = 2320,
  [2321] = 2258,
  [2322] = 2322,
  [2323] = 2323,
  [2324] = 2324,
  [2325] = 2325,
  [2326] = 2326,
  [2327] = 2327,
  [2328] = 2328,
  [2329] = 2329,
  [2330] = 2330,
  [2331] = 2331,
  [2332] = 2332,
  [2333] = 2333,
  [2334] = 2334,
  [2335] = 2335,
  [2336] = 2336,
  [2337] = 2337,
  [2338] = 2258,
  [2339] = 2339,
  [2340] = 2340,
  [2341] = 2341,
  [2342] = 2316,
  [2343] = 2343,
  [2344] = 2322,
  [2345] = 2320,
  [2346] = 2346,
  [2347] = 2347,
  [2348] = 1145,
  [2349] = 2349,
  [2350] = 1520,
  [2351] = 2351,
  [2352] = 2265,
  [2353] = 2353,
  [2354] = 2354,
  [2355] = 2355,
  [2356] = 2356,
  [2357] = 513,
  [2358] = 2358,
  [2359] = 2359,
  [2360] = 2360,
  [2361] = 2361,
  [2362] = 2362,
  [2363] = 2363,
  [2364] = 2364,
  [2365] = 2365,
  [2366] = 2366,
  [2367] = 2367,
  [2368] = 2368,
  [2369] = 2369,
  [2370] = 2370,
  [2371] = 2371,
  [2372] = 2372,
  [2373] = 2373,
  [2374] = 2370,
  [2375] = 2375,
  [2376] = 2376,
  [2377] = 2377,
  [2378] = 2378,
  [2379] = 2379,
  [2380] = 2380,
  [2381] = 2381,
  [2382] = 2382,
  [2383] = 2383,
  [2384] = 2384,
  [2385] = 2385,
  [2386] = 2386,
  [2387] = 2387,
  [2388] = 2388,
  [2389] = 2362,
  [2390] = 2390,
  [2391] = 2391,
  [2392] = 2392,
  [2393] = 2393,
  [2394] = 2373,
  [2395] = 2372,
  [2396] = 2396,
  [2397] = 2397,
  [2398] = 2398,
  [2399] = 2386,
  [2400] = 2385,
  [2401] = 2360,
  [2402] = 2402,
  [2403] = 2403,
  [2404] = 2404,
  [2405] = 2405,
  [2406] = 2406,
  [2407] = 2407,
  [2408] = 2408,
  [2409] = 2409,
  [2410] = 2410,
  [2411] = 2411,
  [2412] = 2412,
  [2413] = 2413,
  [2414] = 2414,
  [2415] = 2415,
  [2416] = 2416,
  [2417] = 2417,
  [2418] = 2418,
  [2419] = 2419,
  [2420] = 2420,
  [2421] = 2421,
  [2422] = 2422,
  [2423] = 2423,
  [2424] = 2424,
  [2425] = 2425,
  [2426] = 2426,
  [2427] = 2427,
  [2428] = 2428,
  [2429] = 2429,
  [2430] = 2430,
  [2431] = 2429,
  [2432] = 2426,
  [2433] = 2425,
  [2434] = 2434,
  [2435] = 2413,
  [2436] = 2436,
  [2437] = 2437,
  [2438] = 2438,
  [2439] = 2439,
  [2440] = 2356,
  [2441] = 2441,
  [2442] = 2430,
  [2443] = 2358,
  [2444] = 2444,
  [2445] = 2445,
  [2446] = 2424,
  [2447] = 2447,
  [2448] = 2366,
  [2449] = 2367,
  [2450] = 2450,
  [2451] = 2451,
  [2452] = 2452,
  [2453] = 2453,
  [2454] = 2454,
  [2455] = 2455,
  [2456] = 2456,
  [2457] = 2457,
  [2458] = 2458,
  [2459] = 2370,
  [2460] = 2460,
  [2461] = 2461,
  [2462] = 2462,
  [2463] = 2380,
  [2464] = 2412,
  [2465] = 2465,
  [2466] = 2410,
  [2467] = 2387,
  [2468] = 2388,
  [2469] = 2469,
  [2470] = 2390,
  [2471] = 2471,
  [2472] = 2392,
  [2473] = 2473,
  [2474] = 2474,
  [2475] = 2475,
  [2476] = 2476,
  [2477] = 2477,
  [2478] = 2478,
  [2479] = 2479,
  [2480] = 2480,
  [2481] = 2406,
  [2482] = 2405,
  [2483] = 2386,
  [2484] = 2385,
  [2485] = 2360,
  [2486] = 2486,
  [2487] = 2392,
  [2488] = 2410,
  [2489] = 2489,
  [2490] = 2412,
  [2491] = 2491,
  [2492] = 2492,
  [2493] = 2404,
  [2494] = 2494,
  [2495] = 2495,
  [2496] = 2496,
  [2497] = 2403,
  [2498] = 2360,
  [2499] = 2499,
  [2500] = 2500,
  [2501] = 2385,
  [2502] = 2424,
  [2503] = 2386,
  [2504] = 2504,
  [2505] = 2505,
  [2506] = 2506,
  [2507] = 2430,
  [2508] = 2508,
  [2509] = 2413,
  [2510] = 2510,
  [2511] = 2437,
  [2512] = 2512,
  [2513] = 2513,
  [2514] = 2356,
  [2515] = 2515,
  [2516] = 2327,
  [2517] = 2358,
  [2518] = 2518,
  [2519] = 2519,
  [2520] = 2392,
  [2521] = 2366,
  [2522] = 2367,
  [2523] = 2523,
  [2524] = 2524,
  [2525] = 2494,
  [2526] = 2526,
  [2527] = 2495,
  [2528] = 2496,
  [2529] = 2529,
  [2530] = 2530,
  [2531] = 2531,
  [2532] = 2370,
  [2533] = 2377,
  [2534] = 2390,
  [2535] = 2535,
  [2536] = 2380,
  [2537] = 2388,
  [2538] = 2387,
  [2539] = 2387,
  [2540] = 2388,
  [2541] = 2541,
  [2542] = 2390,
  [2543] = 2543,
  [2544] = 2392,
  [2545] = 2545,
  [2546] = 2546,
  [2547] = 2411,
  [2548] = 2548,
  [2549] = 2549,
  [2550] = 2550,
  [2551] = 2551,
  [2552] = 2552,
  [2553] = 2553,
  [2554] = 2554,
  [2555] = 2386,
  [2556] = 2385,
  [2557] = 2360,
  [2558] = 2558,
  [2559] = 2559,
  [2560] = 2410,
  [2561] = 2561,
  [2562] = 2412,
  [2563] = 2563,
  [2564] = 2564,
  [2565] = 2565,
  [2566] = 2566,
  [2567] = 2567,
  [2568] = 2568,
  [2569] = 2569,
  [2570] = 2570,
  [2571] = 2571,
  [2572] = 2572,
  [2573] = 2573,
  [2574] = 2424,
  [2575] = 2575,
  [2576] = 2381,
  [2577] = 2577,
  [2578] = 522,
  [2579] = 2430,
  [2580] = 2580,
  [2581] = 2413,
  [2582] = 2582,
  [2583] = 2437,
  [2584] = 2584,
  [2585] = 2585,
  [2586] = 2356,
  [2587] = 2587,
  [2588] = 2588,
  [2589] = 2358,
  [2590] = 2403,
  [2591] = 2591,
  [2592] = 2404,
  [2593] = 2366,
  [2594] = 2367,
  [2595] = 2595,
  [2596] = 2596,
  [2597] = 2597,
  [2598] = 1520,
  [2599] = 2599,
  [2600] = 2600,
  [2601] = 2601,
  [2602] = 2602,
  [2603] = 2603,
  [2604] = 2370,
  [2605] = 2605,
  [2606] = 2606,
  [2607] = 2361,
  [2608] = 2380,
  [2609] = 2609,
  [2610] = 2610,
  [2611] = 2387,
  [2612] = 2388,
  [2613] = 520,
  [2614] = 2390,
  [2615] = 2615,
  [2616] = 2616,
  [2617] = 2617,
  [2618] = 1167,
  [2619] = 2486,
  [2620] = 1171,
  [2621] = 2621,
  [2622] = 1172,
  [2623] = 2405,
  [2624] = 2406,
  [2625] = 2625,
  [2626] = 2386,
  [2627] = 2360,
  [2628] = 2628,
  [2629] = 2474,
  [2630] = 2410,
  [2631] = 2473,
  [2632] = 2412,
  [2633] = 2633,
  [2634] = 2634,
  [2635] = 2635,
  [2636] = 2437,
  [2637] = 2637,
  [2638] = 2638,
  [2639] = 2639,
  [2640] = 511,
  [2641] = 2641,
  [2642] = 2642,
  [2643] = 2643,
  [2644] = 2424,
  [2645] = 2420,
  [2646] = 2646,
  [2647] = 2647,
  [2648] = 2648,
  [2649] = 2430,
  [2650] = 2650,
  [2651] = 2413,
  [2652] = 2652,
  [2653] = 2437,
  [2654] = 2654,
  [2655] = 2655,
  [2656] = 2356,
  [2657] = 2657,
  [2658] = 2658,
  [2659] = 2358,
  [2660] = 2660,
  [2661] = 2383,
  [2662] = 2662,
  [2663] = 2366,
  [2664] = 2367,
  [2665] = 2665,
  [2666] = 2666,
  [2667] = 2667,
  [2668] = 2668,
  [2669] = 2669,
  [2670] = 2670,
  [2671] = 2671,
  [2672] = 2672,
  [2673] = 2673,
  [2674] = 2370,
  [2675] = 2675,
  [2676] = 2676,
  [2677] = 2382,
  [2678] = 2380,
  [2679] = 2679,
  [2680] = 2680,
  [2681] = 2388,
  [2682] = 2682,
  [2683] = 2390,
  [2684] = 2684,
  [2685] = 2685,
  [2686] = 2686,
  [2687] = 2687,
  [2688] = 2688,
  [2689] = 2689,
  [2690] = 2690,
  [2691] = 2691,
  [2692] = 2692,
  [2693] = 2693,
  [2694] = 2386,
  [2695] = 2360,
  [2696] = 2410,
  [2697] = 2697,
  [2698] = 2412,
  [2699] = 2699,
  [2700] = 2358,
  [2701] = 2701,
  [2702] = 2702,
  [2703] = 2703,
  [2704] = 2704,
  [2705] = 2705,
  [2706] = 2706,
  [2707] = 2707,
  [2708] = 2708,
  [2709] = 2709,
  [2710] = 2424,
  [2711] = 2711,
  [2712] = 2712,
  [2713] = 2713,
  [2714] = 2714,
  [2715] = 2430,
  [2716] = 2413,
  [2717] = 2717,
  [2718] = 2437,
  [2719] = 2719,
  [2720] = 2720,
  [2721] = 2356,
  [2722] = 2722,
  [2723] = 2358,
  [2724] = 2724,
  [2725] = 2725,
  [2726] = 2726,
  [2727] = 2366,
  [2728] = 2367,
  [2729] = 2729,
  [2730] = 2730,
  [2731] = 2731,
  [2732] = 2732,
  [2733] = 2733,
  [2734] = 2734,
  [2735] = 2735,
  [2736] = 2390,
  [2737] = 2737,
  [2738] = 2738,
  [2739] = 2739,
  [2740] = 2740,
  [2741] = 2386,
  [2742] = 2742,
  [2743] = 2412,
  [2744] = 2744,
  [2745] = 2745,
  [2746] = 2746,
  [2747] = 2747,
  [2748] = 2748,
  [2749] = 2749,
  [2750] = 2750,
  [2751] = 2751,
  [2752] = 2752,
  [2753] = 2753,
  [2754] = 2413,
  [2755] = 2755,
  [2756] = 2437,
  [2757] = 2367,
  [2758] = 2758,
  [2759] = 2356,
  [2760] = 2760,
  [2761] = 2366,
  [2762] = 2367,
  [2763] = 2390,
  [2764] = 2386,
  [2765] = 2390,
  [2766] = 2386,
  [2767] = 2390,
  [2768] = 2386,
  [2769] = 2390,
  [2770] = 2386,
  [2771] = 2390,
  [2772] = 2386,
  [2773] = 2390,
  [2774] = 2386,
  [2775] = 2410,
  [2776] = 2776,
  [2777] = 2777,
  [2778] = 2778,
  [2779] = 2779,
  [2780] = 2780,
  [2781] = 2437,
  [2782] = 2782,
  [2783] = 2783,
  [2784] = 2784,
  [2785] = 2785,
  [2786] = 2786,
  [2787] = 2787,
  [2788] = 2430,
  [2789] = 2465,
  [2790] = 2418,
  [2791] = 2412,
  [2792] = 2390,
  [2793] = 2793,
  [2794] = 2444,
  [2795] = 2531,
  [2796] = 2425,
  [2797] = 2615,
  [2798] = 2388,
  [2799] = 2799,
  [2800] = 2800,
  [2801] = 2801,
  [2802] = 2387,
  [2803] = 2426,
  [2804] = 2429,
  [2805] = 2366,
  [2806] = 2779,
  [2807] = 2778,
  [2808] = 2808,
  [2809] = 2809,
  [2810] = 2434,
  [2811] = 2811,
  [2812] = 2365,
  [2813] = 2364,
  [2814] = 2814,
  [2815] = 2675,
  [2816] = 2816,
  [2817] = 2436,
  [2818] = 2818,
  [2819] = 2819,
  [2820] = 2820,
  [2821] = 2821,
  [2822] = 2822,
  [2823] = 2778,
  [2824] = 2779,
  [2825] = 2825,
  [2826] = 2826,
  [2827] = 2461,
  [2828] = 2828,
  [2829] = 2829,
  [2830] = 2830,
  [2831] = 2465,
  [2832] = 2418,
  [2833] = 2833,
  [2834] = 2444,
  [2835] = 2531,
  [2836] = 2836,
  [2837] = 2800,
  [2838] = 2801,
  [2839] = 2675,
  [2840] = 2840,
  [2841] = 2778,
  [2842] = 2779,
  [2843] = 2843,
  [2844] = 2356,
  [2845] = 2845,
  [2846] = 2465,
  [2847] = 2418,
  [2848] = 2848,
  [2849] = 2800,
  [2850] = 2801,
  [2851] = 2675,
  [2852] = 2852,
  [2853] = 2778,
  [2854] = 2779,
  [2855] = 2855,
  [2856] = 2856,
  [2857] = 2800,
  [2858] = 2465,
  [2859] = 2418,
  [2860] = 2668,
  [2861] = 2800,
  [2862] = 2801,
  [2863] = 2675,
  [2864] = 2864,
  [2865] = 2778,
  [2866] = 2779,
  [2867] = 2867,
  [2868] = 2868,
  [2869] = 2465,
  [2870] = 2418,
  [2871] = 2871,
  [2872] = 2800,
  [2873] = 2801,
  [2874] = 2675,
  [2875] = 2875,
  [2876] = 2778,
  [2877] = 2779,
  [2878] = 2424,
  [2879] = 2879,
  [2880] = 2465,
  [2881] = 2418,
  [2882] = 2409,
  [2883] = 2801,
  [2884] = 2675,
  [2885] = 2885,
  [2886] = 2778,
  [2887] = 2779,
  [2888] = 2465,
  [2889] = 2889,
  [2890] = 2801,
  [2891] = 2891,
  [2892] = 2892,
  [2893] = 2893,
  [2894] = 2894,
  [2895] = 2892,
  [2896] = 2891,
  [2897] = 2829,
  [2898] = 2898,
  [2899] = 2899,
  [2900] = 2744,
  [2901] = 2742,
  [2902] = 2902,
  [2903] = 2500,
  [2904] = 2378,
  [2905] = 2413,
  [2906] = 2906,
  [2907] = 2379,
  [2908] = 1750,
  [2909] = 2909,
  [2910] = 2380,
  [2911] = 2911,
  [2912] = 2912,
  [2913] = 2913,
  [2914] = 2549,
  [2915] = 2915,
  [2916] = 2892,
  [2917] = 2744,
  [2918] = 2742,
  [2919] = 2919,
  [2920] = 2920,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(875);
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '"') ADVANCE(72);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1270);
      if (lookahead == '/') ADVANCE(981);
      if (lookahead == ':') ADVANCE(914);
      if (lookahead == ';') ADVANCE(1992);
      if (lookahead == '<') ADVANCE(1252);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1256);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1006);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(992);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(993);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1000);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1013);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(996);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1007);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1020);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1010);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1018);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1002);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1005);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1017);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(994);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1022);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1004);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(995);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1003);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1016);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(999);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1012);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(0)
      if (('J' <= lookahead && lookahead <= 'Z') ||
          ('j' <= lookahead && lookahead <= 'z')) ADVANCE(991);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1088);
      END_STATE();
    case 1:
      if (lookahead == '\n') ADVANCE(1993);
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '\t' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(1)
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1088);
      END_STATE();
    case 2:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(2)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 3:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(914);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1368);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1787);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1319);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1689);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1552);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1810);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1340);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1305);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1311);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(3)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 4:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(914);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1899);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1574);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1322);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1635);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1783);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1548);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1763);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1973);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1491);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1543);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(4)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 5:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(914);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(5)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 6:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1686);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(6)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 7:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1688);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(7)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 8:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1684);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(8)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 9:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1272);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1684);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(9)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 10:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(981);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1252);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1256);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(166);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(425);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(156);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(101);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(500);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(97);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(470);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(625);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(369);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(598);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(103);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(136);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(649);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(96);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(819);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(109);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(93);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(104);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(100);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(394);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(10)
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1088);
      END_STATE();
    case 11:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(981);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1256);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(489);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(392);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(503);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(397);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(317);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(530);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(630);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(561);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(684);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(822);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(11)
      END_STATE();
    case 12:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(981);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1256);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(751);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(452);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(509);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(603);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(565);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(633);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(370);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(820);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(365);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(389);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(394);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(12)
      END_STATE();
    case 13:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(981);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1256);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(752);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(392);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(502);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(818);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(531);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(849);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(822);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(13)
      END_STATE();
    case 14:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(981);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(923);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(167);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(390);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(102);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(501);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(637);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(519);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(103);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(135);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(648);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(129);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(92);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(105);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(547);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(100);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(14)
      END_STATE();
    case 15:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1252);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(15)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 16:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1783);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(16)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 17:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1861);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(17)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 18:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1865);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(18)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 19:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '%') ADVANCE(1246);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == '*') ADVANCE(1087);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1273);
      if (lookahead == '/') ADVANCE(982);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '<') ADVANCE(1253);
      if (lookahead == '=') ADVANCE(924);
      if (lookahead == '>') ADVANCE(1255);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1543);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(19)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 20:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1086);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(20)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 21:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1086);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1271);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == '_') ADVANCE(1988);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1014);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1024);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1001);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1015);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1019);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1011);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1023);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1008);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1009);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(998);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(997);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1021);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(21)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1025);
      END_STATE();
    case 22:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '*') ADVANCE(1086);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1271);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(22)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 23:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1086);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '/') ADVANCE(73);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == ']') ADVANCE(1280);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(23)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 24:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '*') ADVANCE(1086);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1899);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1574);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1322);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1635);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1783);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1548);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1763);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1973);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1491);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1543);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(24)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 25:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '/') ADVANCE(980);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1368);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1787);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1319);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1689);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1552);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1810);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1340);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1305);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1311);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(25)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 26:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1368);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1787);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1319);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1689);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1555);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1810);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1340);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1305);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1311);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(26)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 27:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1368);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1573);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1320);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1634);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1783);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1549);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1306);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1810);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1339);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1491);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1304);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(27)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 28:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1367);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1297);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1318);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1692);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1547);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1342);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1799);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1290);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1303);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1873);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(28)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 29:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1367);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1297);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1318);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1677);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1547);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1342);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1799);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1290);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1303);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1873);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(29)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 30:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1367);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1297);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1318);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1678);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1547);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1342);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1799);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1290);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1303);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1873);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(30)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 31:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1367);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1297);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1318);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1676);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1547);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1342);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1799);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1290);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1303);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1873);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(31)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 32:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1367);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1298);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1318);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1679);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1547);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1307);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1342);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1799);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1290);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1303);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1873);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(32)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 33:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1301);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1680);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1314);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(33)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 34:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1680);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1341);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(34)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 35:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1693);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(35)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 36:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1694);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(36)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 37:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1685);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(37)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 38:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1686);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(38)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 39:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1687);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(39)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 40:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1688);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(40)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 41:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1684);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1312);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(41)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 42:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1684);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(42)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 43:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1299);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1684);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(43)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 44:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1296);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1680);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(44)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 45:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1300);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1681);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(45)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 46:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1300);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1682);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(46)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 47:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1300);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1683);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(47)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 48:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1874);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1300);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1323);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1691);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1768);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1807);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1332);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1537);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(48)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 49:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1574);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1322);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1554);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(49)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 50:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(50)
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1222);
      if (lookahead == '$' ||
          lookahead == '*' ||
          ('.' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'G') ||
          ('I' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'g') ||
          ('i' <= lookahead && lookahead <= 'z')) ADVANCE(1223);
      END_STATE();
    case 51:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1269);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(51)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 52:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == '=') ADVANCE(88);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(391);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(576);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(396);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(317);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(558);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(668);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(360);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(312);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(52)
      END_STATE();
    case 53:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(911);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1893);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1811);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(53)
      if (('G' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('g' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 54:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '*') ADVANCE(1086);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1575);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1322);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1554);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(54)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 55:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1269);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1892);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1574);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1322);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1636);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1626);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1519);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1550);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1763);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1811);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1841);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1491);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1543);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(55)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 56:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == ':') ADVANCE(86);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1899);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1574);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1322);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1636);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1626);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1519);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1550);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1763);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1841);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1491);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1543);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(56)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 57:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1791);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1579);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(57)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 58:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(752);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(392);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(504);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(818);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(531);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(849);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(822);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(58)
      END_STATE();
    case 59:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(752);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(392);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(505);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(818);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(531);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(849);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(822);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(59)
      END_STATE();
    case 60:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(752);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(392);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(506);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(818);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(531);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(849);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(822);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(60)
      END_STATE();
    case 61:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(752);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(868);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(392);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(507);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(818);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(531);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(608);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(599);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(849);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(309);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(822);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(61)
      END_STATE();
    case 62:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1763);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(62)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 63:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1742);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(63)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 64:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1542);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1315);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(64)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 65:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1647);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1986);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1321);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1690);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1784);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1553);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1964);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1518);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1544);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1313);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1309);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1827);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(65)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 66:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(157);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(577);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(108);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(867);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(66)
      END_STATE();
    case 67:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(512);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(67)
      END_STATE();
    case 68:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(513);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(68)
      END_STATE();
    case 69:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(577);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(640);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(871);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(69)
      END_STATE();
    case 70:
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(70)
      if (('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1275);
      END_STATE();
    case 71:
      if (lookahead == '!') ADVANCE(927);
      if (lookahead == '#') ADVANCE(1286);
      if (lookahead == '&') ADVANCE(1991);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') ADVANCE(928);
      if (lookahead != 0 &&
          lookahead != '(' &&
          lookahead != ')') ADVANCE(929);
      END_STATE();
    case 72:
      if (lookahead == '"') ADVANCE(1287);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(72);
      END_STATE();
    case 73:
      if (lookahead == ')') ADVANCE(1278);
      END_STATE();
    case 74:
      if (lookahead == '.') ADVANCE(1263);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(82);
      END_STATE();
    case 75:
      if (lookahead == '.') ADVANCE(1261);
      END_STATE();
    case 76:
      if (lookahead == '.') ADVANCE(1257);
      END_STATE();
    case 77:
      if (lookahead == '.') ADVANCE(1259);
      END_STATE();
    case 78:
      if (lookahead == '.') ADVANCE(1254);
      END_STATE();
    case 79:
      if (lookahead == '.') ADVANCE(1265);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(855);
      END_STATE();
    case 80:
      if (lookahead == '.') ADVANCE(1247);
      END_STATE();
    case 81:
      if (lookahead == '.') ADVANCE(1248);
      END_STATE();
    case 82:
      if (lookahead == '.') ADVANCE(1249);
      END_STATE();
    case 83:
      if (lookahead == '.') ADVANCE(1251);
      END_STATE();
    case 84:
      if (lookahead == '.') ADVANCE(1250);
      END_STATE();
    case 85:
      if (lookahead == '.') ADVANCE(1281);
      END_STATE();
    case 86:
      if (lookahead == ':') ADVANCE(965);
      END_STATE();
    case 87:
      if (lookahead == '<') ADVANCE(1134);
      END_STATE();
    case 88:
      if (lookahead == '>') ADVANCE(968);
      END_STATE();
    case 89:
      if (lookahead == '>') ADVANCE(1135);
      END_STATE();
    case 90:
      if (lookahead == '_') ADVANCE(420);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(971);
      END_STATE();
    case 91:
      if (lookahead == '_') ADVANCE(382);
      END_STATE();
    case 92:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(857);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(656);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(111);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(114);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(173);
      END_STATE();
    case 93:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(857);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(517);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(111);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(115);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(173);
      END_STATE();
    case 94:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(224);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(826);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(834);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(830);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(428);
      END_STATE();
    case 95:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1151);
      END_STATE();
    case 96:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(671);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(586);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(407);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(319);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(172);
      END_STATE();
    case 97:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(529);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(657);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(562);
      END_STATE();
    case 98:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(729);
      END_STATE();
    case 99:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(729);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(736);
      END_STATE();
    case 100:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(508);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(472);
      END_STATE();
    case 101:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(776);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(371);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(535);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(817);
      END_STATE();
    case 102:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(776);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(854);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(535);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(817);
      END_STATE();
    case 103:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(564);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(243);
      END_STATE();
    case 104:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(658);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(863);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(321);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1144);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(651);
      END_STATE();
    case 105:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(658);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(863);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(651);
      END_STATE();
    case 106:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(510);
      END_STATE();
    case 107:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(524);
      END_STATE();
    case 108:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(560);
      END_STATE();
    case 109:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(560);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(133);
      END_STATE();
    case 110:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(673);
      END_STATE();
    case 111:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(685);
      END_STATE();
    case 112:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(216);
      END_STATE();
    case 113:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(525);
      END_STATE();
    case 114:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(804);
      END_STATE();
    case 115:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(804);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(643);
      END_STATE();
    case 116:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(458);
      END_STATE();
    case 117:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(459);
      END_STATE();
    case 118:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(800);
      END_STATE();
    case 119:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(759);
      END_STATE();
    case 120:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(461);
      END_STATE();
    case 121:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(516);
      END_STATE();
    case 122:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(462);
      END_STATE();
    case 123:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(567);
      END_STATE();
    case 124:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(463);
      END_STATE();
    case 125:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(786);
      END_STATE();
    case 126:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(464);
      END_STATE();
    case 127:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(465);
      END_STATE();
    case 128:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(466);
      END_STATE();
    case 129:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(670);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(586);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(407);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(402);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(172);
      END_STATE();
    case 130:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(468);
      END_STATE();
    case 131:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(457);
      END_STATE();
    case 132:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(457);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(826);
      END_STATE();
    case 133:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(457);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(826);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(834);
      END_STATE();
    case 134:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(810);
      END_STATE();
    case 135:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(532);
      END_STATE();
    case 136:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(532);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(548);
      END_STATE();
    case 137:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(383);
      END_STATE();
    case 138:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(482);
      END_STATE();
    case 139:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(482);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(119);
      END_STATE();
    case 140:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(801);
      END_STATE();
    case 141:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(579);
      END_STATE();
    case 142:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(533);
      END_STATE();
    case 143:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(487);
      END_STATE();
    case 144:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(421);
      END_STATE();
    case 145:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(421);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(380);
      END_STATE();
    case 146:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(421);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(379);
      END_STATE();
    case 147:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(732);
      END_STATE();
    case 148:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(201);
      END_STATE();
    case 149:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(838);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(702);
      END_STATE();
    case 150:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(205);
      END_STATE();
    case 151:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(738);
      END_STATE();
    case 152:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(791);
      END_STATE();
    case 153:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(741);
      END_STATE();
    case 154:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(840);
      END_STATE();
    case 155:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(206);
      END_STATE();
    case 156:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(728);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(110);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(151);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(521);
      END_STATE();
    case 157:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(728);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(98);
      END_STATE();
    case 158:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(793);
      END_STATE();
    case 159:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(208);
      END_STATE();
    case 160:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(796);
      END_STATE();
    case 161:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(176);
      END_STATE();
    case 162:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(808);
      END_STATE();
    case 163:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(814);
      END_STATE();
    case 164:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(802);
      END_STATE();
    case 165:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(178);
      END_STATE();
    case 166:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(744);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(480);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(750);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(780);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(777);
      END_STATE();
    case 167:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(744);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(480);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(780);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(777);
      END_STATE();
    case 168:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(544);
      END_STATE();
    case 169:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(844);
      END_STATE();
    case 170:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(545);
      END_STATE();
    case 171:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(511);
      END_STATE();
    case 172:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(511);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(265);
      END_STATE();
    case 173:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(675);
      END_STATE();
    case 174:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(120);
      END_STATE();
    case 175:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(488);
      END_STATE();
    case 176:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(496);
      END_STATE();
    case 177:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(130);
      END_STATE();
    case 178:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(499);
      END_STATE();
    case 179:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(714);
      END_STATE();
    case 180:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(546);
      END_STATE();
    case 181:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(451);
      END_STATE();
    case 182:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(985);
      END_STATE();
    case 183:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1109);
      END_STATE();
    case 184:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1037);
      END_STATE();
    case 185:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1091);
      END_STATE();
    case 186:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(962);
      END_STATE();
    case 187:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(964);
      END_STATE();
    case 188:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(448);
      END_STATE();
    case 189:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(449);
      END_STATE();
    case 190:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(479);
      END_STATE();
    case 191:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(474);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(785);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(350);
      END_STATE();
    case 192:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(474);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(263);
      END_STATE();
    case 193:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(445);
      END_STATE();
    case 194:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(442);
      END_STATE();
    case 195:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(423);
      END_STATE();
    case 196:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(342);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(679);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(348);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(363);
      END_STATE();
    case 197:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(342);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(348);
      END_STATE();
    case 198:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(342);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(363);
      END_STATE();
    case 199:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(763);
      END_STATE();
    case 200:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(275);
      END_STATE();
    case 201:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(766);
      END_STATE();
    case 202:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(770);
      END_STATE();
    case 203:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(775);
      END_STATE();
    case 204:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(287);
      END_STATE();
    case 205:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(290);
      END_STATE();
    case 206:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(293);
      END_STATE();
    case 207:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(301);
      END_STATE();
    case 208:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(303);
      END_STATE();
    case 209:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(787);
      END_STATE();
    case 210:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(117);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(406);
      END_STATE();
    case 211:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(832);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(784);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(145);
      END_STATE();
    case 212:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(125);
      END_STATE();
    case 213:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(792);
      END_STATE();
    case 214:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(124);
      END_STATE();
    case 215:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(816);
      END_STATE();
    case 216:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(811);
      END_STATE();
    case 217:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(444);
      END_STATE();
    case 218:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(446);
      END_STATE();
    case 219:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(367);
      END_STATE();
    case 220:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(367);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(681);
      END_STATE();
    case 221:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(878);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(522);
      END_STATE();
    case 222:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(946);
      END_STATE();
    case 223:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(91);
      END_STATE();
    case 224:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(930);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1066);
      END_STATE();
    case 225:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1058);
      END_STATE();
    case 226:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1242);
      END_STATE();
    case 227:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1056);
      END_STATE();
    case 228:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1052);
      END_STATE();
    case 229:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1045);
      END_STATE();
    case 230:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1107);
      END_STATE();
    case 231:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(935);
      END_STATE();
    case 232:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(899);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(526);
      END_STATE();
    case 233:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(896);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(526);
      END_STATE();
    case 234:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(934);
      END_STATE();
    case 235:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(895);
      END_STATE();
    case 236:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(882);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(534);
      END_STATE();
    case 237:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(891);
      END_STATE();
    case 238:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(897);
      END_STATE();
    case 239:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(902);
      END_STATE();
    case 240:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(903);
      END_STATE();
    case 241:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(900);
      END_STATE();
    case 242:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(893);
      END_STATE();
    case 243:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(842);
      END_STATE();
    case 244:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(81);
      END_STATE();
    case 245:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(721);
      END_STATE();
    case 246:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(281);
      END_STATE();
    case 247:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(901);
      END_STATE();
    case 248:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(898);
      END_STATE();
    case 249:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(845);
      END_STATE();
    case 250:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(846);
      END_STATE();
    case 251:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(847);
      END_STATE();
    case 252:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(848);
      END_STATE();
    case 253:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(850);
      END_STATE();
    case 254:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(165);
      END_STATE();
    case 255:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(75);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(76);
      END_STATE();
    case 256:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(79);
      END_STATE();
    case 257:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(79);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(783);
      END_STATE();
    case 258:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(549);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(410);
      END_STATE();
    case 259:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(960);
      END_STATE();
    case 260:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1062);
      END_STATE();
    case 261:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1193);
      END_STATE();
    case 262:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1173);
      END_STATE();
    case 263:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(387);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(419);
      END_STATE();
    case 264:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(971);
      END_STATE();
    case 265:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1121);
      END_STATE();
    case 266:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(978);
      END_STATE();
    case 267:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(973);
      END_STATE();
    case 268:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1229);
      END_STATE();
    case 269:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1138);
      END_STATE();
    case 270:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1244);
      END_STATE();
    case 271:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1113);
      END_STATE();
    case 272:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1179);
      END_STATE();
    case 273:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1159);
      END_STATE();
    case 274:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(932);
      END_STATE();
    case 275:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(951);
      END_STATE();
    case 276:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1069);
      END_STATE();
    case 277:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1119);
      END_STATE();
    case 278:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(906);
      END_STATE();
    case 279:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1240);
      END_STATE();
    case 280:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1036);
      END_STATE();
    case 281:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1149);
      END_STATE();
    case 282:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1231);
      END_STATE();
    case 283:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(983);
      END_STATE();
    case 284:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1060);
      END_STATE();
    case 285:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1136);
      END_STATE();
    case 286:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1181);
      END_STATE();
    case 287:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1031);
      END_STATE();
    case 288:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1115);
      END_STATE();
    case 289:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1216);
      END_STATE();
    case 290:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1238);
      END_STATE();
    case 291:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1183);
      END_STATE();
    case 292:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(908);
      END_STATE();
    case 293:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(917);
      END_STATE();
    case 294:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(940);
      END_STATE();
    case 295:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1123);
      END_STATE();
    case 296:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(909);
      END_STATE();
    case 297:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1195);
      END_STATE();
    case 298:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1197);
      END_STATE();
    case 299:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(936);
      END_STATE();
    case 300:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1089);
      END_STATE();
    case 301:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1125);
      END_STATE();
    case 302:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1218);
      END_STATE();
    case 303:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(919);
      END_STATE();
    case 304:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(942);
      END_STATE();
    case 305:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(916);
      END_STATE();
    case 306:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(938);
      END_STATE();
    case 307:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1048);
      END_STATE();
    case 308:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(972);
      END_STATE();
    case 309:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(132);
      END_STATE();
    case 310:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1172);
      END_STATE();
    case 311:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1176);
      END_STATE();
    case 312:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(656);
      END_STATE();
    case 313:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(861);
      END_STATE();
    case 314:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(566);
      END_STATE();
    case 315:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(862);
      END_STATE();
    case 316:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(77);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(78);
      END_STATE();
    case 317:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(568);
      END_STATE();
    case 318:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(193);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(573);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(196);
      END_STATE();
    case 319:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(193);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(856);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(198);
      END_STATE();
    case 320:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(225);
      END_STATE();
    case 321:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(550);
      END_STATE();
    case 322:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(227);
      END_STATE();
    case 323:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(569);
      END_STATE();
    case 324:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(228);
      END_STATE();
    case 325:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(85);
      END_STATE();
    case 326:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(703);
      END_STATE();
    case 327:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(229);
      END_STATE();
    case 328:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(230);
      END_STATE();
    case 329:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(578);
      END_STATE();
    case 330:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(231);
      END_STATE();
    case 331:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(758);
      END_STATE();
    case 332:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(234);
      END_STATE();
    case 333:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(723);
      END_STATE();
    case 334:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(481);
      END_STATE();
    case 335:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(660);
      END_STATE();
    case 336:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(661);
      END_STATE();
    case 337:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(712);
      END_STATE();
    case 338:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(764);
      END_STATE();
    case 339:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(663);
      END_STATE();
    case 340:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(664);
      END_STATE();
    case 341:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(707);
      END_STATE();
    case 342:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(250);
      END_STATE();
    case 343:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(587);
      END_STATE();
    case 344:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(199);
      END_STATE();
    case 345:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(375);
      END_STATE();
    case 346:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(687);
      END_STATE();
    case 347:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(687);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(485);
      END_STATE();
    case 348:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(690);
      END_STATE();
    case 349:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(689);
      END_STATE();
    case 350:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(386);
      END_STATE();
    case 351:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(580);
      END_STATE();
    case 352:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(202);
      END_STATE();
    case 353:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(713);
      END_STATE();
    case 354:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(581);
      END_STATE();
    case 355:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(203);
      END_STATE();
    case 356:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(695);
      END_STATE();
    case 357:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(515);
      END_STATE();
    case 358:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(515);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(180);
      END_STATE();
    case 359:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(697);
      END_STATE();
    case 360:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(131);
      END_STATE();
    case 361:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(537);
      END_STATE();
    case 362:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(537);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(262);
      END_STATE();
    case 363:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(213);
      END_STATE();
    case 364:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(591);
      END_STATE();
    case 365:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(477);
      END_STATE();
    case 366:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(812);
      END_STATE();
    case 367:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(252);
      END_STATE();
    case 368:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(218);
      END_STATE();
    case 369:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1166);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(642);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(192);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1205);
      END_STATE();
    case 370:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1166);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1100);
      END_STATE();
    case 371:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(149);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(404);
      END_STATE();
    case 372:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1170);
      END_STATE();
    case 373:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1170);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(809);
      END_STATE();
    case 374:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1177);
      END_STATE();
    case 375:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(154);
      END_STATE();
    case 376:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(617);
      END_STATE();
    case 377:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(155);
      END_STATE();
    case 378:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(159);
      END_STATE();
    case 379:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(828);
      END_STATE();
    case 380:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(828);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(837);
      END_STATE();
    case 381:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(596);
      END_STATE();
    case 382:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(518);
      END_STATE();
    case 383:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(324);
      END_STATE();
    case 384:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(406);
      END_STATE();
    case 385:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(338);
      END_STATE();
    case 386:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(335);
      END_STATE();
    case 387:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(335);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(761);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(377);
      END_STATE();
    case 388:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(681);
      END_STATE();
    case 389:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(321);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(645);
      END_STATE();
    case 390:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(110);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(151);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(521);
      END_STATE();
    case 391:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(110);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(151);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(542);
      END_STATE();
    case 392:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(110);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(151);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(541);
      END_STATE();
    case 393:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(356);
      END_STATE();
    case 394:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(346);
      END_STATE();
    case 395:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(359);
      END_STATE();
    case 396:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(585);
      END_STATE();
    case 397:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(585);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(562);
      END_STATE();
    case 398:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(381);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(194);
      END_STATE();
    case 399:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(859);
      END_STATE();
    case 400:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(223);
      END_STATE();
    case 401:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(856);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(197);
      END_STATE();
    case 402:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(856);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(198);
      END_STATE();
    case 403:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(169);
      END_STATE();
    case 404:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(200);
      END_STATE();
    case 405:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(754);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(314);
      END_STATE();
    case 406:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(214);
      END_STATE();
    case 407:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(590);
      END_STATE();
    case 408:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(785);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(350);
      END_STATE();
    case 409:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(182);
      END_STATE();
    case 410:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(635);
      END_STATE();
    case 411:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(183);
      END_STATE();
    case 412:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(331);
      END_STATE();
    case 413:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(184);
      END_STATE();
    case 414:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(185);
      END_STATE();
    case 415:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(471);
      END_STATE();
    case 416:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(186);
      END_STATE();
    case 417:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(254);
      END_STATE();
    case 418:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(187);
      END_STATE();
    case 419:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(572);
      END_STATE();
    case 420:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(589);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(860);
      END_STATE();
    case 421:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(574);
      END_STATE();
    case 422:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(588);
      END_STATE();
    case 423:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(768);
      END_STATE();
    case 424:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(773);
      END_STATE();
    case 425:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(563);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(626);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(778);
      END_STATE();
    case 426:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(790);
      END_STATE();
    case 427:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(618);
      END_STATE();
    case 428:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(571);
      END_STATE();
    case 429:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(740);
      END_STATE();
    case 430:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(858);
      END_STATE();
    case 431:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(582);
      END_STATE();
    case 432:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(195);
      END_STATE();
    case 433:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(619);
      END_STATE();
    case 434:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(583);
      END_STATE();
    case 435:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(620);
      END_STATE();
    case 436:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(622);
      END_STATE();
    case 437:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(693);
      END_STATE();
    case 438:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(122);
      END_STATE();
    case 439:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(623);
      END_STATE();
    case 440:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(491);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(705);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(597);
      END_STATE();
    case 441:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(493);
      END_STATE();
    case 442:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(158);
      END_STATE();
    case 443:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(594);
      END_STATE();
    case 444:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(160);
      END_STATE();
    case 445:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(748);
      END_STATE();
    case 446:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(749);
      END_STATE();
    case 447:
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1199);
      END_STATE();
    case 448:
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1212);
      END_STATE();
    case 449:
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1214);
      END_STATE();
    case 450:
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1201);
      END_STATE();
    case 451:
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(727);
      END_STATE();
    case 452:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(626);
      END_STATE();
    case 453:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1030);
      END_STATE();
    case 454:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(864);
      END_STATE();
    case 455:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1132);
      END_STATE();
    case 456:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1283);
      END_STATE();
    case 457:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1066);
      END_STATE();
    case 458:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1043);
      END_STATE();
    case 459:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1163);
      END_STATE();
    case 460:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1185);
      END_STATE();
    case 461:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(950);
      END_STATE();
    case 462:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1039);
      END_STATE();
    case 463:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1078);
      END_STATE();
    case 464:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(976);
      END_STATE();
    case 465:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1103);
      END_STATE();
    case 466:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1117);
      END_STATE();
    case 467:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1187);
      END_STATE();
    case 468:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(954);
      END_STATE();
    case 469:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1029);
      END_STATE();
    case 470:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(605);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(400);
      END_STATE();
    case 471:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(865);
      END_STATE();
    case 472:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(118);
      END_STATE();
    case 473:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(455);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(261);
      END_STATE();
    case 474:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(824);
      END_STATE();
    case 475:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(456);
      END_STATE();
    case 476:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(432);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(683);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(691);
      END_STATE();
    case 477:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(344);
      END_STATE();
    case 478:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(344);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(841);
      END_STATE();
    case 479:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(269);
      END_STATE();
    case 480:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(607);
      END_STATE();
    case 481:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(429);
      END_STATE();
    case 482:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(460);
      END_STATE();
    case 483:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(762);
      END_STATE();
    case 484:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(765);
      END_STATE();
    case 485:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(273);
      END_STATE();
    case 486:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(313);
      END_STATE();
    case 487:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(467);
      END_STATE();
    case 488:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(276);
      END_STATE();
    case 489:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(469);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(780);
      END_STATE();
    case 490:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(278);
      END_STATE();
    case 491:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(279);
      END_STATE();
    case 492:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(774);
      END_STATE();
    case 493:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(288);
      END_STATE();
    case 494:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(292);
      END_STATE();
    case 495:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(296);
      END_STATE();
    case 496:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(300);
      END_STATE();
    case 497:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(315);
      END_STATE();
    case 498:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(305);
      END_STATE();
    case 499:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(307);
      END_STATE();
    case 500:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(232);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(823);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(781);
      END_STATE();
    case 501:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(233);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(823);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(815);
      END_STATE();
    case 502:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(236);
      END_STATE();
    case 503:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(235);
      END_STATE();
    case 504:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(237);
      END_STATE();
    case 505:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(247);
      END_STATE();
    case 506:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(238);
      END_STATE();
    case 507:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(361);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(248);
      END_STATE();
    case 508:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(835);
      END_STATE();
    case 509:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(98);
      END_STATE();
    case 510:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(735);
      END_STATE();
    case 511:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(409);
      END_STATE();
    case 512:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(742);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(242);
      END_STATE();
    case 513:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(743);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(240);
      END_STATE();
    case 514:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(629);
      END_STATE();
    case 515:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(352);
      END_STATE();
    case 516:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(364);
      END_STATE();
    case 517:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(355);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(841);
      END_STATE();
    case 518:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(632);
      END_STATE();
    case 519:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(642);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(192);
      END_STATE();
    case 520:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(528);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(211);
      END_STATE();
    case 521:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(528);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(733);
      END_STATE();
    case 522:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1234);
      END_STATE();
    case 523:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1237);
      END_STATE();
    case 524:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(876);
      END_STATE();
    case 525:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(904);
      END_STATE();
    case 526:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1233);
      END_STATE();
    case 527:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(654);
      END_STATE();
    case 528:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(616);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(486);
      END_STATE();
    case 529:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(415);
      END_STATE();
    case 530:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(644);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(408);
      END_STATE();
    case 531:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(653);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(798);
      END_STATE();
    case 532:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(334);
      END_STATE();
    case 533:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(366);
      END_STATE();
    case 534:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(353);
      END_STATE();
    case 535:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(323);
      END_STATE();
    case 536:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(119);
      END_STATE();
    case 537:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(329);
      END_STATE();
    case 538:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(162);
      END_STATE();
    case 539:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(351);
      END_STATE();
    case 540:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(134);
      END_STATE();
    case 541:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(652);
      END_STATE();
    case 542:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(652);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(788);
      END_STATE();
    case 543:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(163);
      END_STATE();
    case 544:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(639);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(627);
      END_STATE();
    case 545:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(641);
      END_STATE();
    case 546:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(641);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(634);
      END_STATE();
    case 547:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(376);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(259);
      END_STATE();
    case 548:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(90);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(147);
      END_STATE();
    case 549:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1227);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(140);
      END_STATE();
    case 550:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1168);
      END_STATE();
    case 551:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(989);
      END_STATE();
    case 552:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1147);
      END_STATE();
    case 553:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(944);
      END_STATE();
    case 554:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1093);
      END_STATE();
    case 555:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1071);
      END_STATE();
    case 556:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(955);
      END_STATE();
    case 557:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1072);
      END_STATE();
    case 558:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(191);
      END_STATE();
    case 559:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(244);
      END_STATE();
    case 560:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(447);
      END_STATE();
    case 561:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(454);
      END_STATE();
    case 562:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(209);
      END_STATE();
    case 563:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(222);
      END_STATE();
    case 564:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(137);
      END_STATE();
    case 565:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(236);
      END_STATE();
    case 566:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(245);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(592);
      END_STATE();
    case 567:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(450);
      END_STATE();
    case 568:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(326);
      END_STATE();
    case 569:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(747);
      END_STATE();
    case 570:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(821);
      END_STATE();
    case 571:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(226);
      END_STATE();
    case 572:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(745);
      END_STATE();
    case 573:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(757);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(152);
      END_STATE();
    case 574:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(722);
      END_STATE();
    case 575:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(320);
      END_STATE();
    case 576:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(239);
      END_STATE();
    case 577:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(241);
      END_STATE();
    case 578:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(813);
      END_STATE();
    case 579:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(767);
      END_STATE();
    case 580:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(771);
      END_STATE();
    case 581:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(772);
      END_STATE();
    case 582:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(299);
      END_STATE();
    case 583:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(306);
      END_STATE();
    case 584:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(264);
      END_STATE();
    case 585:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(116);
      END_STATE();
    case 586:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(575);
      END_STATE();
    case 587:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(204);
      END_STATE();
    case 588:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(424);
      END_STATE();
    case 589:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(789);
      END_STATE();
    case 590:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(806);
      END_STATE();
    case 591:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(207);
      END_STATE();
    case 592:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(126);
      END_STATE();
    case 593:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(127);
      END_STATE();
    case 594:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(746);
      END_STATE();
    case 595:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(809);
      END_STATE();
    case 596:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(539);
      END_STATE();
    case 597:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(215);
      END_STATE();
    case 598:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(210);
      END_STATE();
    case 599:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(243);
      END_STATE();
    case 600:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1145);
      END_STATE();
    case 601:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1157);
      END_STATE();
    case 602:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(197);
      END_STATE();
    case 603:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1153);
      END_STATE();
    case 604:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(220);
      END_STATE();
    case 605:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(174);
      END_STATE();
    case 606:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(538);
      END_STATE();
    case 607:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(212);
      END_STATE();
    case 608:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(384);
      END_STATE();
    case 609:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(783);
      END_STATE();
    case 610:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(388);
      END_STATE();
    case 611:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(527);
      END_STATE();
    case 612:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(659);
      END_STATE();
    case 613:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(833);
      END_STATE();
    case 614:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(683);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(691);
      END_STATE();
    case 615:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(817);
      END_STATE();
    case 616:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(551);
      END_STATE();
    case 617:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(708);
      END_STATE();
    case 618:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(553);
      END_STATE();
    case 619:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(554);
      END_STATE();
    case 620:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(555);
      END_STATE();
    case 621:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(662);
      END_STATE();
    case 622:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(556);
      END_STATE();
    case 623:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(557);
      END_STATE();
    case 624:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(665);
      END_STATE();
    case 625:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(730);
      END_STATE();
    case 626:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(188);
      END_STATE();
    case 627:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(836);
      END_STATE();
    case 628:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(194);
      END_STATE();
    case 629:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(189);
      END_STATE();
    case 630:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(584);
      END_STATE();
    case 631:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(219);
      END_STATE();
    case 632:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(177);
      END_STATE();
    case 633:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(706);
      END_STATE();
    case 634:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(851);
      END_STATE();
    case 635:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(593);
      END_STATE();
    case 636:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(249);
      END_STATE();
    case 637:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(710);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(562);
      END_STATE();
    case 638:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(217);
      END_STATE();
    case 639:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(251);
      END_STATE();
    case 640:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(711);
      END_STATE();
    case 641:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(253);
      END_STATE();
    case 642:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(476);
      END_STATE();
    case 643:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1129);
      END_STATE();
    case 644:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(614);
      END_STATE();
    case 645:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(267);
      END_STATE();
    case 646:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(150);
      END_STATE();
    case 647:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(280);
      END_STATE();
    case 648:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(782);
      END_STATE();
    case 649:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(782);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(753);
      END_STATE();
    case 650:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(298);
      END_STATE();
    case 651:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(308);
      END_STATE();
    case 652:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(486);
      END_STATE();
    case 653:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(839);
      END_STATE();
    case 654:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(497);
      END_STATE();
    case 655:
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(74);
      END_STATE();
    case 656:
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(841);
      END_STATE();
    case 657:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(139);
      END_STATE();
    case 658:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(385);
      END_STATE();
    case 659:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1127);
      END_STATE();
    case 660:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1064);
      END_STATE();
    case 661:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1050);
      END_STATE();
    case 662:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(925);
      END_STATE();
    case 663:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1080);
      END_STATE();
    case 664:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1105);
      END_STATE();
    case 665:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1236);
      END_STATE();
    case 666:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(831);
      END_STATE();
    case 667:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(612);
      END_STATE();
    case 668:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(401);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(171);
      END_STATE();
    case 669:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(80);
      END_STATE();
    case 670:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(142);
      END_STATE();
    case 671:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(142);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(716);
      END_STATE();
    case 672:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(142);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(716);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(737);
      END_STATE();
    case 673:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(112);
      END_STATE();
    case 674:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(265);
      END_STATE();
    case 675:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(627);
      END_STATE();
    case 676:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(403);
      END_STATE();
    case 677:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(148);
      END_STATE();
    case 678:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(604);
      END_STATE();
    case 679:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(107);
      END_STATE();
    case 680:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(552);
      END_STATE();
    case 681:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(113);
      END_STATE();
    case 682:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(704);
      END_STATE();
    case 683:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(760);
      END_STATE();
    case 684:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(602);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(674);
      END_STATE();
    case 685:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(322);
      END_STATE();
    case 686:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(631);
      END_STATE();
    case 687:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(272);
      END_STATE();
    case 688:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(610);
      END_STATE();
    case 689:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(700);
      END_STATE();
    case 690:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(779);
      END_STATE();
    case 691:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(277);
      END_STATE();
    case 692:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(327);
      END_STATE();
    case 693:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(282);
      END_STATE();
    case 694:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(284);
      END_STATE();
    case 695:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(286);
      END_STATE();
    case 696:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(368);
      END_STATE();
    case 697:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(291);
      END_STATE();
    case 698:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(294);
      END_STATE();
    case 699:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(304);
      END_STATE();
    case 700:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(417);
      END_STATE();
    case 701:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(739);
      END_STATE();
    case 702:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(692);
      END_STATE();
    case 703:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(413);
      END_STATE();
    case 704:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(354);
      END_STATE();
    case 705:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(143);
      END_STATE();
    case 706:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(138);
      END_STATE();
    case 707:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(592);
      END_STATE();
    case 708:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(540);
      END_STATE();
    case 709:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(443);
      END_STATE();
    case 710:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(543);
      END_STATE();
    case 711:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(536);
      END_STATE();
    case 712:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(378);
      END_STATE();
    case 713:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(164);
      END_STATE();
    case 714:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(634);
      END_STATE();
    case 715:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(398);
      END_STATE();
    case 716:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1046);
      END_STATE();
    case 717:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1083);
      END_STATE();
    case 718:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1047);
      END_STATE();
    case 719:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1206);
      END_STATE();
    case 720:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1208);
      END_STATE();
    case 721:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1035);
      END_STATE();
    case 722:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(958);
      END_STATE();
    case 723:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(948);
      END_STATE();
    case 724:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1095);
      END_STATE();
    case 725:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1082);
      END_STATE();
    case 726:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(259);
      END_STATE();
    case 727:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(646);
      END_STATE();
    case 728:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(261);
      END_STATE();
    case 729:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(717);
      END_STATE();
    case 730:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(755);
      END_STATE();
    case 731:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(734);
      END_STATE();
    case 732:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(718);
      END_STATE();
    case 733:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(784);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(146);
      END_STATE();
    case 734:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(638);
      END_STATE();
    case 735:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(325);
      END_STATE();
    case 736:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(268);
      END_STATE();
    case 737:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(270);
      END_STATE();
    case 738:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(725);
      END_STATE();
    case 739:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(430);
      END_STATE();
    case 740:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(769);
      END_STATE();
    case 741:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(297);
      END_STATE();
    case 742:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(310);
      END_STATE();
    case 743:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(311);
      END_STATE();
    case 744:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(799);
      END_STATE();
    case 745:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(416);
      END_STATE();
    case 746:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(418);
      END_STATE();
    case 747:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(433);
      END_STATE();
    case 748:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(435);
      END_STATE();
    case 749:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(439);
      END_STATE();
    case 750:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(628);
      END_STATE();
    case 751:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(750);
      END_STATE();
    case 752:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(780);
      END_STATE();
    case 753:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1101);
      END_STATE();
    case 754:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1140);
      END_STATE();
    case 755:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(953);
      END_STATE();
    case 756:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1102);
      END_STATE();
    case 757:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1225);
      END_STATE();
    case 758:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1131);
      END_STATE();
    case 759:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1220);
      END_STATE();
    case 760:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1027);
      END_STATE();
    case 761:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1097);
      END_STATE();
    case 762:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(957);
      END_STATE();
    case 763:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1191);
      END_STATE();
    case 764:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1111);
      END_STATE();
    case 765:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1165);
      END_STATE();
    case 766:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1033);
      END_STATE();
    case 767:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1054);
      END_STATE();
    case 768:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(969);
      END_STATE();
    case 769:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(987);
      END_STATE();
    case 770:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1203);
      END_STATE();
    case 771:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(921);
      END_STATE();
    case 772:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1161);
      END_STATE();
    case 773:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1164);
      END_STATE();
    case 774:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1210);
      END_STATE();
    case 775:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1189);
      END_STATE();
    case 776:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(95);
      END_STATE();
    case 777:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(606);
      END_STATE();
    case 778:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(260);
      END_STATE();
    case 779:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(866);
      END_STATE();
    case 780:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(676);
      END_STATE();
    case 781:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(314);
      END_STATE();
    case 782:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(410);
      END_STATE();
    case 783:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(83);
      END_STATE();
    case 784:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(141);
      END_STATE();
    case 785:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(438);
      END_STATE();
    case 786:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(161);
      END_STATE();
    case 787:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(427);
      END_STATE();
    case 788:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(144);
      END_STATE();
    case 789:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(709);
      END_STATE();
    case 790:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(274);
      END_STATE();
    case 791:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(283);
      END_STATE();
    case 792:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(328);
      END_STATE();
    case 793:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(289);
      END_STATE();
    case 794:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(333);
      END_STATE();
    case 795:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(330);
      END_STATE();
    case 796:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(302);
      END_STATE();
    case 797:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(332);
      END_STATE();
    case 798:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(350);
      END_STATE();
    case 799:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(677);
      END_STATE();
    case 800:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(441);
      END_STATE();
    case 801:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(621);
      END_STATE();
    case 802:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(624);
      END_STATE();
    case 803:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(843);
      END_STATE();
    case 804:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(411);
      END_STATE();
    case 805:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(431);
      END_STATE();
    case 806:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(336);
      END_STATE();
    case 807:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(434);
      END_STATE();
    case 808:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(414);
      END_STATE();
    case 809:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(337);
      END_STATE();
    case 810:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(795);
      END_STATE();
    case 811:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(339);
      END_STATE();
    case 812:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(340);
      END_STATE();
    case 813:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(128);
      END_STATE();
    case 814:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(797);
      END_STATE();
    case 815:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(341);
      END_STATE();
    case 816:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(436);
      END_STATE();
    case 817:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(175);
      END_STATE();
    case 818:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(562);
      END_STATE();
    case 819:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(412);
      END_STATE();
    case 820:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(753);
      END_STATE();
    case 821:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(523);
      END_STATE();
    case 822:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(173);
      END_STATE();
    case 823:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(399);
      END_STATE();
    case 824:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(246);
      END_STATE();
    case 825:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(170);
      END_STATE();
    case 826:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(701);
      END_STATE();
    case 827:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(756);
      END_STATE();
    case 828:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(613);
      END_STATE();
    case 829:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(437);
      END_STATE();
    case 830:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(680);
      END_STATE();
    case 831:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(325);
      END_STATE();
    case 832:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(682);
      END_STATE();
    case 833:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(724);
      END_STATE();
    case 834:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(483);
      END_STATE();
    case 835:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(271);
      END_STATE();
    case 836:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(805);
      END_STATE();
    case 837:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(285);
      END_STATE();
    case 838:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(484);
      END_STATE();
    case 839:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(691);
      END_STATE();
    case 840:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(492);
      END_STATE();
    case 841:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(343);
      END_STATE();
    case 842:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(490);
      END_STATE();
    case 843:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(694);
      END_STATE();
    case 844:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(794);
      END_STATE();
    case 845:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(494);
      END_STATE();
    case 846:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(698);
      END_STATE();
    case 847:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(495);
      END_STATE();
    case 848:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(699);
      END_STATE();
    case 849:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(674);
      END_STATE();
    case 850:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(498);
      END_STATE();
    case 851:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(807);
      END_STATE();
    case 852:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(597);
      END_STATE();
    case 853:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(179);
      END_STATE();
    case 854:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(404);
      END_STATE();
    case 855:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(84);
      END_STATE();
    case 856:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(152);
      END_STATE();
    case 857:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(266);
      END_STATE();
    case 858:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(295);
      END_STATE();
    case 859:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(121);
      END_STATE();
    case 860:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(349);
      END_STATE();
    case 861:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1074);
      END_STATE();
    case 862:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1076);
      END_STATE();
    case 863:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(803);
      END_STATE();
    case 864:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(966);
      END_STATE();
    case 865:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1049);
      END_STATE();
    case 866:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1041);
      END_STATE();
    case 867:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(645);
      END_STATE();
    case 868:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(778);
      END_STATE();
    case 869:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(647);
      END_STATE();
    case 870:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(650);
      END_STATE();
    case 871:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(651);
      END_STATE();
    case 872:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1282);
      END_STATE();
    case 873:
      if (lookahead != 0 &&
          lookahead != ')' &&
          lookahead != ',' &&
          lookahead != '/') ADVANCE(1224);
      END_STATE();
    case 874:
      if (eof) ADVANCE(875);
      if (lookahead == '!') ADVANCE(1989);
      if (lookahead == '#') ADVANCE(1285);
      if (lookahead == '&') ADVANCE(1990);
      if (lookahead == '(') ADVANCE(912);
      if (lookahead == ')') ADVANCE(915);
      if (lookahead == '+') ADVANCE(1267);
      if (lookahead == ',') ADVANCE(947);
      if (lookahead == '-') ADVANCE(1026);
      if (lookahead == '.') ADVANCE(1274);
      if (lookahead == ':') ADVANCE(913);
      if (lookahead == '[') ADVANCE(1279);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1367);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1288);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1297);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1318);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1633);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1767);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1761);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1546);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1762);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1306);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1342);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1799);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1289);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1444);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1302);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1310);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1873);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1308);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1576);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') SKIP(874)
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 875:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 876:
      ACCEPT_TOKEN(aux_sym_program_statement_token1);
      END_STATE();
    case 877:
      ACCEPT_TOKEN(aux_sym_program_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 878:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(731);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(514);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(601);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(570);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(440);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(373);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(636);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(678);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(358);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(869);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(393);
      END_STATE();
    case 879:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1898);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 880:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1695);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 881:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1765);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 882:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(570);
      END_STATE();
    case 883:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1617);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 884:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1616);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 885:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1838);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 886:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1857);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 887:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1952);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 888:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1540);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 889:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1577);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 890:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1618);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 891:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(852);
      END_STATE();
    case 892:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 893:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(372);
      END_STATE();
    case 894:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1556);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 895:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(595);
      END_STATE();
    case 896:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(636);
      END_STATE();
    case 897:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(686);
      END_STATE();
    case 898:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(688);
      END_STATE();
    case 899:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(825);
      END_STATE();
    case 900:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(357);
      END_STATE();
    case 901:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(853);
      END_STATE();
    case 902:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(869);
      END_STATE();
    case 903:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token1);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(393);
      END_STATE();
    case 904:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token2);
      END_STATE();
    case 905:
      ACCEPT_TOKEN(aux_sym_end_program_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 906:
      ACCEPT_TOKEN(aux_sym_module_statement_token1);
      END_STATE();
    case 907:
      ACCEPT_TOKEN(aux_sym_module_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 908:
      ACCEPT_TOKEN(aux_sym_end_module_statement_token1);
      END_STATE();
    case 909:
      ACCEPT_TOKEN(aux_sym_submodule_statement_token1);
      END_STATE();
    case 910:
      ACCEPT_TOKEN(aux_sym_submodule_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 911:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 912:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      if (lookahead == '/') ADVANCE(1277);
      END_STATE();
    case 913:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 914:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == ':') ADVANCE(965);
      END_STATE();
    case 915:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 916:
      ACCEPT_TOKEN(aux_sym_end_submodule_statement_token1);
      END_STATE();
    case 917:
      ACCEPT_TOKEN(aux_sym_interface_statement_token1);
      END_STATE();
    case 918:
      ACCEPT_TOKEN(aux_sym_interface_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 919:
      ACCEPT_TOKEN(aux_sym_end_interface_statement_token1);
      END_STATE();
    case 920:
      ACCEPT_TOKEN(aux_sym_end_interface_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 921:
      ACCEPT_TOKEN(aux_sym_assignment_token1);
      END_STATE();
    case 922:
      ACCEPT_TOKEN(aux_sym_assignment_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 923:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(1262);
      END_STATE();
    case 924:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(1262);
      if (lookahead == '>') ADVANCE(968);
      END_STATE();
    case 925:
      ACCEPT_TOKEN(aux_sym_operator_token1);
      END_STATE();
    case 926:
      ACCEPT_TOKEN(aux_sym_operator_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 927:
      ACCEPT_TOKEN(aux_sym_operator_token2);
      if (lookahead == '\n') ADVANCE(929);
      if (lookahead == '(' ||
          lookahead == ')') ADVANCE(1989);
      if (lookahead != 0) ADVANCE(927);
      END_STATE();
    case 928:
      ACCEPT_TOKEN(aux_sym_operator_token2);
      if (lookahead == '!') ADVANCE(927);
      if (lookahead == '#') ADVANCE(1286);
      if (lookahead == '&') ADVANCE(1991);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ') ADVANCE(928);
      if (lookahead != 0 &&
          lookahead != '(' &&
          lookahead != ')') ADVANCE(929);
      END_STATE();
    case 929:
      ACCEPT_TOKEN(aux_sym_operator_token2);
      if (lookahead != 0 &&
          lookahead != '(' &&
          lookahead != ')') ADVANCE(929);
      END_STATE();
    case 930:
      ACCEPT_TOKEN(aux_sym_defined_io_procedure_token1);
      END_STATE();
    case 931:
      ACCEPT_TOKEN(aux_sym_defined_io_procedure_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 932:
      ACCEPT_TOKEN(aux_sym_defined_io_procedure_token2);
      END_STATE();
    case 933:
      ACCEPT_TOKEN(aux_sym_defined_io_procedure_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 934:
      ACCEPT_TOKEN(aux_sym_defined_io_procedure_token3);
      END_STATE();
    case 935:
      ACCEPT_TOKEN(aux_sym_defined_io_procedure_token4);
      END_STATE();
    case 936:
      ACCEPT_TOKEN(aux_sym_subroutine_statement_token1);
      END_STATE();
    case 937:
      ACCEPT_TOKEN(aux_sym_subroutine_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 938:
      ACCEPT_TOKEN(aux_sym_end_subroutine_statement_token1);
      END_STATE();
    case 939:
      ACCEPT_TOKEN(aux_sym_end_subroutine_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 940:
      ACCEPT_TOKEN(aux_sym_module_procedure_statement_token1);
      END_STATE();
    case 941:
      ACCEPT_TOKEN(aux_sym_module_procedure_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 942:
      ACCEPT_TOKEN(aux_sym_end_module_procedure_statement_token1);
      END_STATE();
    case 943:
      ACCEPT_TOKEN(aux_sym_end_module_procedure_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 944:
      ACCEPT_TOKEN(aux_sym_function_statement_token1);
      END_STATE();
    case 945:
      ACCEPT_TOKEN(aux_sym_function_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 946:
      ACCEPT_TOKEN(aux_sym_language_binding_token1);
      END_STATE();
    case 947:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 948:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token1);
      END_STATE();
    case 949:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 950:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token2);
      END_STATE();
    case 951:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token3);
      END_STATE();
    case 952:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 953:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token4);
      END_STATE();
    case 954:
      ACCEPT_TOKEN(aux_sym_procedure_attributes_token5);
      END_STATE();
    case 955:
      ACCEPT_TOKEN(aux_sym_end_function_statement_token1);
      END_STATE();
    case 956:
      ACCEPT_TOKEN(aux_sym_end_function_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 957:
      ACCEPT_TOKEN(aux_sym_function_result_token1);
      END_STATE();
    case 958:
      ACCEPT_TOKEN(aux_sym_contains_statement_token1);
      END_STATE();
    case 959:
      ACCEPT_TOKEN(aux_sym_contains_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 960:
      ACCEPT_TOKEN(aux_sym_use_statement_token1);
      END_STATE();
    case 961:
      ACCEPT_TOKEN(aux_sym_use_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 962:
      ACCEPT_TOKEN(aux_sym_use_statement_token2);
      END_STATE();
    case 963:
      ACCEPT_TOKEN(aux_sym_use_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 964:
      ACCEPT_TOKEN(aux_sym_use_statement_token3);
      END_STATE();
    case 965:
      ACCEPT_TOKEN(anon_sym_COLON_COLON);
      END_STATE();
    case 966:
      ACCEPT_TOKEN(aux_sym_included_items_token1);
      END_STATE();
    case 967:
      ACCEPT_TOKEN(aux_sym_included_items_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 968:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 969:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token1);
      END_STATE();
    case 970:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 971:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token2);
      END_STATE();
    case 972:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token3);
      END_STATE();
    case 973:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token3);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(719);
      END_STATE();
    case 974:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token3);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1871);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 975:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 976:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token4);
      END_STATE();
    case 977:
      ACCEPT_TOKEN(aux_sym_implicit_statement_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 978:
      ACCEPT_TOKEN(aux_sym_save_statement_token1);
      END_STATE();
    case 979:
      ACCEPT_TOKEN(aux_sym_save_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 980:
      ACCEPT_TOKEN(anon_sym_SLASH);
      END_STATE();
    case 981:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == ')') ADVANCE(1278);
      if (lookahead == '/') ADVANCE(1266);
      if (lookahead == '=') ADVANCE(1264);
      END_STATE();
    case 982:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '/') ADVANCE(1266);
      if (lookahead == '=') ADVANCE(1264);
      END_STATE();
    case 983:
      ACCEPT_TOKEN(aux_sym_private_statement_token1);
      END_STATE();
    case 984:
      ACCEPT_TOKEN(aux_sym_private_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 985:
      ACCEPT_TOKEN(aux_sym_public_statement_token1);
      END_STATE();
    case 986:
      ACCEPT_TOKEN(aux_sym_public_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 987:
      ACCEPT_TOKEN(aux_sym_namelist_statement_token1);
      END_STATE();
    case 988:
      ACCEPT_TOKEN(aux_sym_namelist_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 989:
      ACCEPT_TOKEN(aux_sym_common_statement_token1);
      END_STATE();
    case 990:
      ACCEPT_TOKEN(aux_sym_common_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 991:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      END_STATE();
    case 992:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(181);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(563);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(626);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(778);
      END_STATE();
    case 993:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(473);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(110);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(99);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(520);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(190);
      END_STATE();
    case 994:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(672);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(586);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(407);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(318);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(172);
      END_STATE();
    case 995:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(857);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(478);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(111);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(115);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(168);
      END_STATE();
    case 996:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(529);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(585);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(657);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(562);
      END_STATE();
    case 997:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1672);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 998:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 999:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(508);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(472);
      END_STATE();
    case 1000:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(776);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(371);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(535);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1154);
      END_STATE();
    case 1001:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1975);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1002:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(564);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(243);
      END_STATE();
    case 1003:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(658);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(863);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(321);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1144);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(645);
      END_STATE();
    case 1004:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(560);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(94);
      END_STATE();
    case 1005:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(532);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(548);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(475);
      END_STATE();
    case 1006:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(744);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(453);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(715);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(780);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(777);
      END_STATE();
    case 1007:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(568);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(605);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1142);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(400);
      END_STATE();
    case 1008:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1294);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1009:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1698);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1771);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1010:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1166);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(642);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1099);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1205);
      END_STATE();
    case 1011:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1818);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1012:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(347);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(426);
      END_STATE();
    case 1013:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(362);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(221);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(823);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(667);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(405);
      END_STATE();
    case 1014:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1789);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1919);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1015:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1432);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1016:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(376);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(259);
      END_STATE();
    case 1017:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(454);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(258);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(753);
      END_STATE();
    case 1018:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(210);
      END_STATE();
    case 1019:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1859);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1020:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(730);
      END_STATE();
    case 1021:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1607);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1022:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(412);
      END_STATE();
    case 1023:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1654);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1024:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1025:
      ACCEPT_TOKEN(aux_sym_implicit_range_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1026:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 1027:
      ACCEPT_TOKEN(aux_sym_import_statement_token1);
      END_STATE();
    case 1028:
      ACCEPT_TOKEN(aux_sym_import_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1029:
      ACCEPT_TOKEN(aux_sym__import_names_token1);
      END_STATE();
    case 1030:
      ACCEPT_TOKEN(aux_sym__import_names_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(212);
      END_STATE();
    case 1031:
      ACCEPT_TOKEN(aux_sym_derived_type_definition_token1);
      END_STATE();
    case 1032:
      ACCEPT_TOKEN(aux_sym_derived_type_definition_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1033:
      ACCEPT_TOKEN(aux_sym_abstract_specifier_token1);
      END_STATE();
    case 1034:
      ACCEPT_TOKEN(aux_sym_abstract_specifier_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1035:
      ACCEPT_TOKEN(aux_sym_base_type_specifier_token1);
      END_STATE();
    case 1036:
      ACCEPT_TOKEN(aux_sym_end_type_statement_token1);
      END_STATE();
    case 1037:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token1);
      END_STATE();
    case 1038:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1039:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token2);
      END_STATE();
    case 1040:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1041:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token3);
      END_STATE();
    case 1042:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1043:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token4);
      END_STATE();
    case 1044:
      ACCEPT_TOKEN(aux_sym__procedure_kind_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1045:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token1);
      END_STATE();
    case 1046:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token2);
      END_STATE();
    case 1047:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token3);
      END_STATE();
    case 1048:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token4);
      END_STATE();
    case 1049:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token5);
      END_STATE();
    case 1050:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token6);
      END_STATE();
    case 1051:
      ACCEPT_TOKEN(aux_sym_procedure_attribute_token6);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1052:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token1);
      END_STATE();
    case 1053:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1054:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token2);
      END_STATE();
    case 1055:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1056:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token3);
      END_STATE();
    case 1057:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1058:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token4);
      END_STATE();
    case 1059:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1060:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token5);
      END_STATE();
    case 1061:
      ACCEPT_TOKEN(aux_sym_variable_attributes_token5);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1062:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token1);
      END_STATE();
    case 1063:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1064:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token2);
      END_STATE();
    case 1065:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1066:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token3);
      END_STATE();
    case 1067:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1068:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token4);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1774);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1851);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1069:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token4);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(611);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(696);
      END_STATE();
    case 1070:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1071:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token5);
      END_STATE();
    case 1072:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token6);
      END_STATE();
    case 1073:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token6);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1074:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token7);
      END_STATE();
    case 1075:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token7);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1076:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token8);
      END_STATE();
    case 1077:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token8);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1078:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token9);
      END_STATE();
    case 1079:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token9);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1080:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token10);
      END_STATE();
    case 1081:
      ACCEPT_TOKEN(aux_sym_intrinsic_type_token10);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1082:
      ACCEPT_TOKEN(aux_sym_derived_type_token1);
      END_STATE();
    case 1083:
      ACCEPT_TOKEN(aux_sym_derived_type_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(345);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(720);
      END_STATE();
    case 1084:
      ACCEPT_TOKEN(aux_sym_derived_type_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1523);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1872);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1085:
      ACCEPT_TOKEN(aux_sym_derived_type_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1086:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 1087:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '*') ADVANCE(1268);
      END_STATE();
    case 1088:
      ACCEPT_TOKEN(aux_sym_size_token1);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1088);
      END_STATE();
    case 1089:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token1);
      END_STATE();
    case 1090:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1091:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token2);
      END_STATE();
    case 1092:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1093:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token3);
      END_STATE();
    case 1094:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1095:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token4);
      END_STATE();
    case 1096:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1097:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token5);
      END_STATE();
    case 1098:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token5);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1099:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token6);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(474);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(785);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(827);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(829);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(263);
      END_STATE();
    case 1100:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token6);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(827);
      END_STATE();
    case 1101:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token7);
      END_STATE();
    case 1102:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token8);
      END_STATE();
    case 1103:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token9);
      END_STATE();
    case 1104:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token9);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1105:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token10);
      END_STATE();
    case 1106:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token10);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1107:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token11);
      END_STATE();
    case 1108:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token11);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1109:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token12);
      END_STATE();
    case 1110:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token12);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1111:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token13);
      END_STATE();
    case 1112:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token13);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1113:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token14);
      END_STATE();
    case 1114:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token14);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1115:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token15);
      END_STATE();
    case 1116:
      ACCEPT_TOKEN(aux_sym_type_qualifier_token15);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1117:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token1);
      END_STATE();
    case 1118:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1119:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token2);
      END_STATE();
    case 1120:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1121:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token3);
      END_STATE();
    case 1122:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1123:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token4);
      END_STATE();
    case 1124:
      ACCEPT_TOKEN(aux_sym_procedure_qualifier_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1125:
      ACCEPT_TOKEN(aux_sym_equivalence_statement_token1);
      END_STATE();
    case 1126:
      ACCEPT_TOKEN(aux_sym_equivalence_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1127:
      ACCEPT_TOKEN(aux_sym_stop_statement_token1);
      END_STATE();
    case 1128:
      ACCEPT_TOKEN(aux_sym_stop_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1129:
      ACCEPT_TOKEN(aux_sym_stop_statement_token2);
      END_STATE();
    case 1130:
      ACCEPT_TOKEN(aux_sym_stop_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1131:
      ACCEPT_TOKEN(aux_sym_stop_statement_token3);
      END_STATE();
    case 1132:
      ACCEPT_TOKEN(aux_sym_subroutine_call_token1);
      END_STATE();
    case 1133:
      ACCEPT_TOKEN(aux_sym_subroutine_call_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1134:
      ACCEPT_TOKEN(anon_sym_LT_LT_LT);
      END_STATE();
    case 1135:
      ACCEPT_TOKEN(anon_sym_GT_GT_GT);
      END_STATE();
    case 1136:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token1);
      END_STATE();
    case 1137:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1138:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token2);
      END_STATE();
    case 1139:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1140:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token3);
      END_STATE();
    case 1141:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1142:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token4);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(600);
      END_STATE();
    case 1143:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token4);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1764);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1144:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token5);
      END_STATE();
    case 1145:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token6);
      END_STATE();
    case 1146:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token6);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1147:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token7);
      END_STATE();
    case 1148:
      ACCEPT_TOKEN(aux_sym_keyword_statement_token7);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1149:
      ACCEPT_TOKEN(aux_sym_include_statement_token1);
      END_STATE();
    case 1150:
      ACCEPT_TOKEN(aux_sym_include_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1151:
      ACCEPT_TOKEN(aux_sym_data_statement_token1);
      END_STATE();
    case 1152:
      ACCEPT_TOKEN(aux_sym_data_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1153:
      ACCEPT_TOKEN(aux_sym_do_loop_statement_token1);
      END_STATE();
    case 1154:
      ACCEPT_TOKEN(aux_sym_do_loop_statement_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(175);
      END_STATE();
    case 1155:
      ACCEPT_TOKEN(aux_sym_do_loop_statement_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1373);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1156:
      ACCEPT_TOKEN(aux_sym_do_loop_statement_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1376);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1157:
      ACCEPT_TOKEN(aux_sym_end_do_loop_statement_token1);
      END_STATE();
    case 1158:
      ACCEPT_TOKEN(aux_sym_end_do_loop_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1159:
      ACCEPT_TOKEN(aux_sym_while_statement_token1);
      END_STATE();
    case 1160:
      ACCEPT_TOKEN(aux_sym_while_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1161:
      ACCEPT_TOKEN(aux_sym_concurrent_header_token1);
      END_STATE();
    case 1162:
      ACCEPT_TOKEN(aux_sym_concurrent_header_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1163:
      ACCEPT_TOKEN(aux_sym_concurrent_locality_token1);
      if (lookahead == '_') ADVANCE(422);
      END_STATE();
    case 1164:
      ACCEPT_TOKEN(aux_sym_concurrent_locality_token2);
      END_STATE();
    case 1165:
      ACCEPT_TOKEN(aux_sym_concurrent_locality_token3);
      END_STATE();
    case 1166:
      ACCEPT_TOKEN(aux_sym__inline_if_statement_token1);
      END_STATE();
    case 1167:
      ACCEPT_TOKEN(aux_sym__inline_if_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1168:
      ACCEPT_TOKEN(aux_sym__block_if_statement_token1);
      END_STATE();
    case 1169:
      ACCEPT_TOKEN(aux_sym__block_if_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1170:
      ACCEPT_TOKEN(aux_sym_end_if_statement_token1);
      END_STATE();
    case 1171:
      ACCEPT_TOKEN(aux_sym_end_if_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1172:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(374);
      END_STATE();
    case 1173:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(374);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(395);
      END_STATE();
    case 1174:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1558);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1578);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1175:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1558);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1176:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token1);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(395);
      END_STATE();
    case 1177:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token2);
      END_STATE();
    case 1178:
      ACCEPT_TOKEN(aux_sym_elseif_clause_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1179:
      ACCEPT_TOKEN(aux_sym__inline_where_statement_token1);
      END_STATE();
    case 1180:
      ACCEPT_TOKEN(aux_sym__inline_where_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1181:
      ACCEPT_TOKEN(aux_sym_end_where_statement_token1);
      END_STATE();
    case 1182:
      ACCEPT_TOKEN(aux_sym_end_where_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1183:
      ACCEPT_TOKEN(aux_sym_elsewhere_clause_token1);
      END_STATE();
    case 1184:
      ACCEPT_TOKEN(aux_sym_elsewhere_clause_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1185:
      ACCEPT_TOKEN(aux_sym__forall_control_expression_token1);
      END_STATE();
    case 1186:
      ACCEPT_TOKEN(aux_sym__forall_control_expression_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1187:
      ACCEPT_TOKEN(aux_sym_end_forall_statement_token1);
      END_STATE();
    case 1188:
      ACCEPT_TOKEN(aux_sym_end_forall_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1189:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token1);
      END_STATE();
    case 1190:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1354);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1336);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1987);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1191:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(153);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(123);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(870);
      END_STATE();
    case 1192:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1193:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token2);
      END_STATE();
    case 1194:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1195:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token3);
      END_STATE();
    case 1196:
      ACCEPT_TOKEN(aux_sym_select_case_statement_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1197:
      ACCEPT_TOKEN(aux_sym_select_type_statement_token1);
      END_STATE();
    case 1198:
      ACCEPT_TOKEN(aux_sym_select_type_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1199:
      ACCEPT_TOKEN(aux_sym_select_rank_statement_token1);
      END_STATE();
    case 1200:
      ACCEPT_TOKEN(aux_sym_select_rank_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1201:
      ACCEPT_TOKEN(aux_sym_select_rank_statement_token2);
      END_STATE();
    case 1202:
      ACCEPT_TOKEN(aux_sym_select_rank_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1203:
      ACCEPT_TOKEN(aux_sym_end_select_statement_token1);
      END_STATE();
    case 1204:
      ACCEPT_TOKEN(aux_sym_end_select_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1205:
      ACCEPT_TOKEN(aux_sym_type_statement_token1);
      END_STATE();
    case 1206:
      ACCEPT_TOKEN(aux_sym_type_statement_token2);
      END_STATE();
    case 1207:
      ACCEPT_TOKEN(aux_sym_type_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1208:
      ACCEPT_TOKEN(aux_sym_type_statement_token3);
      END_STATE();
    case 1209:
      ACCEPT_TOKEN(aux_sym_type_statement_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1210:
      ACCEPT_TOKEN(aux_sym__class_default_token1);
      END_STATE();
    case 1211:
      ACCEPT_TOKEN(aux_sym__class_default_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1212:
      ACCEPT_TOKEN(aux_sym_block_construct_token1);
      END_STATE();
    case 1213:
      ACCEPT_TOKEN(aux_sym_block_construct_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1214:
      ACCEPT_TOKEN(aux_sym_end_block_construct_statement_token1);
      END_STATE();
    case 1215:
      ACCEPT_TOKEN(aux_sym_end_block_construct_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1216:
      ACCEPT_TOKEN(aux_sym_associate_statement_token1);
      END_STATE();
    case 1217:
      ACCEPT_TOKEN(aux_sym_associate_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1218:
      ACCEPT_TOKEN(aux_sym_end_associate_statement_token1);
      END_STATE();
    case 1219:
      ACCEPT_TOKEN(aux_sym_end_associate_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1220:
      ACCEPT_TOKEN(aux_sym_format_statement_token1);
      END_STATE();
    case 1221:
      ACCEPT_TOKEN(aux_sym_format_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1222:
      ACCEPT_TOKEN(sym_edit_descriptor);
      if (lookahead == 'H') ADVANCE(873);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1222);
      if (lookahead == '$' ||
          lookahead == '*' ||
          ('.' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'g') ||
          ('i' <= lookahead && lookahead <= 'z')) ADVANCE(1223);
      END_STATE();
    case 1223:
      ACCEPT_TOKEN(sym_edit_descriptor);
      if (lookahead == '$' ||
          lookahead == '*' ||
          ('.' <= lookahead && lookahead <= ':') ||
          ('A' <= lookahead && lookahead <= 'G') ||
          ('I' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'g') ||
          ('i' <= lookahead && lookahead <= 'z')) ADVANCE(1223);
      END_STATE();
    case 1224:
      ACCEPT_TOKEN(aux_sym_hollerith_constant_token1);
      if (lookahead != 0 &&
          lookahead != ')' &&
          lookahead != ',' &&
          lookahead != '/') ADVANCE(1224);
      END_STATE();
    case 1225:
      ACCEPT_TOKEN(aux_sym_print_statement_token1);
      END_STATE();
    case 1226:
      ACCEPT_TOKEN(aux_sym_print_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1227:
      ACCEPT_TOKEN(aux_sym_open_statement_token1);
      END_STATE();
    case 1228:
      ACCEPT_TOKEN(aux_sym_open_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1229:
      ACCEPT_TOKEN(aux_sym_close_statement_token1);
      END_STATE();
    case 1230:
      ACCEPT_TOKEN(aux_sym_close_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1231:
      ACCEPT_TOKEN(aux_sym_inquire_statement_token1);
      END_STATE();
    case 1232:
      ACCEPT_TOKEN(aux_sym_inquire_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1233:
      ACCEPT_TOKEN(aux_sym_enum_statement_token1);
      END_STATE();
    case 1234:
      ACCEPT_TOKEN(aux_sym_enum_statement_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(713);
      END_STATE();
    case 1235:
      ACCEPT_TOKEN(aux_sym_enum_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1236:
      ACCEPT_TOKEN(aux_sym_enumerator_statement_token1);
      END_STATE();
    case 1237:
      ACCEPT_TOKEN(aux_sym_end_enum_statement_token1);
      END_STATE();
    case 1238:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token1);
      END_STATE();
    case 1239:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1240:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token2);
      END_STATE();
    case 1241:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1242:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token3);
      END_STATE();
    case 1243:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token3);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1244:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token4);
      END_STATE();
    case 1245:
      ACCEPT_TOKEN(aux_sym_file_position_statement_token4);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1246:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 1247:
      ACCEPT_TOKEN(aux_sym_logical_expression_token1);
      END_STATE();
    case 1248:
      ACCEPT_TOKEN(aux_sym_logical_expression_token2);
      END_STATE();
    case 1249:
      ACCEPT_TOKEN(aux_sym_logical_expression_token3);
      END_STATE();
    case 1250:
      ACCEPT_TOKEN(aux_sym_logical_expression_token4);
      END_STATE();
    case 1251:
      ACCEPT_TOKEN(aux_sym_logical_expression_token5);
      END_STATE();
    case 1252:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(87);
      if (lookahead == '=') ADVANCE(1258);
      END_STATE();
    case 1253:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '=') ADVANCE(1258);
      END_STATE();
    case 1254:
      ACCEPT_TOKEN(aux_sym_relational_expression_token1);
      END_STATE();
    case 1255:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(1260);
      END_STATE();
    case 1256:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(1260);
      if (lookahead == '>') ADVANCE(89);
      END_STATE();
    case 1257:
      ACCEPT_TOKEN(aux_sym_relational_expression_token2);
      END_STATE();
    case 1258:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 1259:
      ACCEPT_TOKEN(aux_sym_relational_expression_token3);
      END_STATE();
    case 1260:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 1261:
      ACCEPT_TOKEN(aux_sym_relational_expression_token4);
      END_STATE();
    case 1262:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 1263:
      ACCEPT_TOKEN(aux_sym_relational_expression_token5);
      END_STATE();
    case 1264:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 1265:
      ACCEPT_TOKEN(aux_sym_relational_expression_token6);
      END_STATE();
    case 1266:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      END_STATE();
    case 1267:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 1268:
      ACCEPT_TOKEN(anon_sym_STAR_STAR);
      END_STATE();
    case 1269:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 1270:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(1276);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(559);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(655);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(106);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(255);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(316);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(257);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(669);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(666);
      END_STATE();
    case 1271:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(1276);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(106);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(609);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(666);
      END_STATE();
    case 1272:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(559);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(655);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(106);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(255);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(316);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(257);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(669);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(666);
      END_STATE();
    case 1273:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(559);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(655);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(255);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(316);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(256);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(669);
      END_STATE();
    case 1274:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(106);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(609);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(666);
      END_STATE();
    case 1275:
      ACCEPT_TOKEN(aux_sym_user_defined_operator_token1);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1275);
      END_STATE();
    case 1276:
      ACCEPT_TOKEN(sym_assumed_rank);
      END_STATE();
    case 1277:
      ACCEPT_TOKEN(anon_sym_LPAREN_SLASH);
      END_STATE();
    case 1278:
      ACCEPT_TOKEN(anon_sym_SLASH_RPAREN);
      END_STATE();
    case 1279:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 1280:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 1281:
      ACCEPT_TOKEN(sym_boolean_literal);
      if (lookahead == '_') ADVANCE(872);
      END_STATE();
    case 1282:
      ACCEPT_TOKEN(sym_boolean_literal);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1282);
      END_STATE();
    case 1283:
      ACCEPT_TOKEN(aux_sym_null_literal_token1);
      END_STATE();
    case 1284:
      ACCEPT_TOKEN(aux_sym_null_literal_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1285:
      ACCEPT_TOKEN(anon_sym_POUND);
      END_STATE();
    case 1286:
      ACCEPT_TOKEN(anon_sym_POUND);
      if (lookahead != 0 &&
          lookahead != '(' &&
          lookahead != ')') ADVANCE(929);
      END_STATE();
    case 1287:
      ACCEPT_TOKEN(aux_sym_preproc_file_line_token1);
      END_STATE();
    case 1288:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1378);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1789);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1919);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1289:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1828);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1730);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1591);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1580);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1371);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1290:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1828);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1730);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1591);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1581);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1372);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1291:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1417);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1954);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1955);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1593);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1292:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1417);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1954);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1293:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1417);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1955);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1593);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1294:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1417);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1295:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1152);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1296:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1650);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1788);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1759);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1297:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1651);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1325);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1316);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1700);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1298:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1651);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1325);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1316);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1701);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1299:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1651);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1788);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1759);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1300:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1651);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1788);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1744);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1301:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1651);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1346);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1759);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1302:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1978);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1655);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1328);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1326);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1366);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1303:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1978);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1655);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1328);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1326);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1304:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1978);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1697);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1328);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1326);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1377);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1305:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1978);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1697);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1328);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1326);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1306:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1727);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1433);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1307:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1727);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1308:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1672);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1652);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1309:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1672);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1310:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1983);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1804);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1311:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1983);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1312:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1517);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1313:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1804);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1314:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1808);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1315:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1826);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1316:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1877);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1881);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1317:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1877);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1318:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1976);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1705);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1155);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1319:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1976);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1705);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1975);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1320:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1976);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1705);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1950);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1321:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1975);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1322:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1950);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1323:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1916);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1156);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1324:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1703);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1325:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1831);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1326:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1940);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1801);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1327:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1704);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1328:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1842);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1329:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1640);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1330:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1641);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1331:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1946);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1332:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1965);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1619);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1333:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1642);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1334:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1902);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1335:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1643);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1336:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1728);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1337:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1645);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1338:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1646);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1339:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1829);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1730);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1591);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1608);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1371);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1340:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1829);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1730);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1591);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1608);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1372);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1341:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1731);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1293);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1342:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1707);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1654);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1343:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1935);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1344:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1958);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1345:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1566);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1346:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1884);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1881);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1347:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1924);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1348:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1711);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1349:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1657);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1334);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1350:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1928);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1351:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1411);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1352:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1401);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1353:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1743);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1354:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1887);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1355:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1601);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1562);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1356:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1601);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1752);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1357:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1660);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1358:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1396);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1359:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1402);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1360:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1930);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1361:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1699);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1362:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1932);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1363:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1404);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1364:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1375);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1365:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1943);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1366:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1709);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1367:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1890);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1648);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1891);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1920);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1921);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1368:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1890);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1648);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1920);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1921);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1369:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1710);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1370:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1969);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1371:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1675);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1448);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1372:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1675);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1373:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1663);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1374:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1864);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1375:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1668);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1376:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1670);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1377:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1835);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1378:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1632);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1379:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(986);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1380:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1110);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1381:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1092);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1382:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(963);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1383:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1038);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1384:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1628);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1385:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1630);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1386:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1659);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1387:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1933);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1388:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1329);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1389:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1624);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1390:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1627);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1391:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1521);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1836);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1506);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1392:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1521);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1527);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1393:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1521);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1506);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1394:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1521);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1395:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1905);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1396:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1907);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1397:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1911);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1398:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1457);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1399:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1913);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1400:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1467);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1401:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1470);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1402:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1471);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1403:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1479);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1404:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1493);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1405:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1604);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1406:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1653);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1447);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1407:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1653);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1408:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1347);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1409:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1929);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1410:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1962);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1411:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1939);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1412:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1947);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1413:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1625);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1414:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1545);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1415:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(885);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1416:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(885);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1417:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(931);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1067);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1418:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1059);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1419:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1243);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1420:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1057);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1421:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1053);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1422:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1108);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1423:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(883);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1424:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(883);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1425:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(887);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1426:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(887);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1427:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(880);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1428:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(880);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1429:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(889);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1430:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(879);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1431:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(881);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1432:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(892);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1433:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1967);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1434:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(890);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1435:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(888);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1436:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(894);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1437:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(884);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1438:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1462);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1439:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(886);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1440:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(886);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1441:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1971);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1442:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1970);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1443:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1972);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1444:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1291);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1445:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(961);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1446:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1063);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1447:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1569);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1598);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1448:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1122);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1449:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(979);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1450:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(975);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1451:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1230);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1452:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1139);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1453:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1245);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1454:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1114);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1455:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1180);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1456:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(933);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1457:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(952);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1458:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1068);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1459:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1120);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1460:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(907);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1461:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1241);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1462:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1150);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1463:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1232);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1464:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(984);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1465:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1061);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1466:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1137);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1467:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1032);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1468:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1116);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1469:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1217);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1470:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1239);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1471:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(918);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1472:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(941);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1473:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1124);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1474:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(910);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1475:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1196);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1476:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1198);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1477:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(937);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1478:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1090);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1479:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1126);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1480:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1293);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1481:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(943);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1482:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(939);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1483:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(974);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1484:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1070);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1485:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1174);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1486:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1182);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1487:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1184);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1488:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1175);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1489:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1219);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1490:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1194);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1491:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1292);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1492:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1740);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1598);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1493:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(920);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1494:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1160);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1495:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1981);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1496:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1570);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1598);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1497:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1418);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1498:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1982);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1499:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1718);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1594);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1500:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1718);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1501:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1420);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1502:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1421);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1503:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1732);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1504:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1422);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1505:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1583);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1506:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1409);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1507:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1856);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1508:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1658);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1509:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1868);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1510:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1821);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1511:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1822);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1512:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1823);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1513:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1824);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1514:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1906);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1515:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1858);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1516:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1863);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1517:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1725);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1518:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1294);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1519:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1757);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1520:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1750);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1521:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1442);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1522:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1395);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1523:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1559);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1524:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1390);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1525:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1844);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1526:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1753);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1527:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1848);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1528:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1571);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1529:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1397);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1530:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1845);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1531:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1399);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1532:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1741);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1598);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1533:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1745);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1534:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1854);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1535:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1746);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1536:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1855);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1537:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1656);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1771);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1538:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1712);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1505);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1539:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1755);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1540:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1696);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1541:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1942);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1542:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1698);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1771);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1369);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1543:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1698);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1771);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1377);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1544:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1698);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1771);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1545:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1443);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1546:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1800);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1406);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1547:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1802);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1406);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1548:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1813);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1817);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1549:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1813);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1815);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1550:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1803);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1605);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1551:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1407);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1552:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1814);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1553:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1818);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1554:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1817);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1555:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1167);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1816);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1556:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1171);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1945);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1557:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1171);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1558:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1178);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1559:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1344);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1560:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1359);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1561:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1363);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1562:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1956);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1963);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1563:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1956);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1564:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1514);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1565:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1758);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1566:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1502);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1567:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1609);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1568:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1836);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1569:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1510);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1904);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1560);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1570:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1510);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1904);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1571:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1510);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1572:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1840);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1573:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1325);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1317);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1715);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1574:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1325);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1317);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1716);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1575:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1325);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1716);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1576:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1525);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1607);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1577:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1534);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1578:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1536);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1579:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1623);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1607);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1580:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1736);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1391);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1581:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1736);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1393);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1582:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1562);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1583:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1558);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1584:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1979);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1585:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1370);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1586:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1563);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1587:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1900);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1507);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1588:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1900);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1589:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1565);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1590:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1398);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1591:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1734);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1592:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1405);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1593:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1729);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1594:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1794);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1595:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1379);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1596:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1886);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1597:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1380);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1598:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1733);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1599:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1381);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1600:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1382);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1601:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1735);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1602:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1383);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1603:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1752);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1604:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1909);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1605:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1926);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1528);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1606:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1980);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1607:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1927);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1608:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1977);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1925);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1609:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1388);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1610:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1747);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1611:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1780);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1612:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1748);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1613:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1781);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1614:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1849);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1615:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1782);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1616:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1665);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1843);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1617:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1665);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1760);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1618:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1665);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1619:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1737);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1620:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1785);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1621:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1338);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1622:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1666);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1623:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1671);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1624:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1360);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1625:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1362);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1626:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1756);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1859);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1726);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1627:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1896);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1628:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1213);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1629:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1202);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1630:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1215);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1631:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1200);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1632:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1875);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1633:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1538);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1415);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1951);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1634:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1538);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1432);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1635:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1538);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1432);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1636:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1538);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1436);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1637:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1133);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1638:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1284);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1639:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1186);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1640:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1079);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1641:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(977);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1642:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1104);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1643:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1118);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1644:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1188);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1645:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1044);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1646:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1040);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1647:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1789);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1919);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1648:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1772);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1649:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1985);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1650:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1637);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1490);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1651:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1637);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1652:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1331);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1653:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1949);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1654:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1638);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1655:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1522);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1959);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1656:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1522);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1657:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1639);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1658:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1596);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1659:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1452);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1660:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1644);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1661:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1495);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1662:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1912);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1663:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1458);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1664:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1460);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1665:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1461);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1666:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1468);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1667:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1474);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1668:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1478);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1669:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1498);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1670:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1484);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1671:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1494);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1672:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1961);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1673:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1592);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1839);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1847);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1674:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1592);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1839);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1675:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1595);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1676:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1415);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1951);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1677:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1423);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1951);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1678:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1425);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1951);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1679:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1427);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1951);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1680:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1435);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1681:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1440);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1682:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1424);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1683:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1426);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1684:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1434);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1685:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1430);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1686:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1431);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1687:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1428);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1688:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1437);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1689:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1432);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1690:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1432);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1691:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1416);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1692:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1876);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1439);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1951);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1693:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1888);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1429);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1694:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1889);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1434);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1830);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1695:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1790);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1696:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1529);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1697:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1531);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1959);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1698:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1531);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1699:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1539);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1700:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1708);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1878);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1701:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1708);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1879);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1702:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1235);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1703:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(877);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1704:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(905);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1705:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1503);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1706:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1812);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1707:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1508);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1708:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1779);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1661);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1709:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1795);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1775);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1710:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1795);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1711:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1541);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1712:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1526);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1713:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1334);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1714:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1365);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1715:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1809);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1880);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1716:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1809);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1717:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1533);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1718:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1228);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1719:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(990);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1720:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1148);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1721:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(945);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1722:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1094);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1723:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1073);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1724:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(956);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1725:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1169);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1726:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1387);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1727:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1345);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1728:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1629);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1729:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1419);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1730:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1739);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1731:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1631);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1732:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1895);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1733:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1894);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1734:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1936);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1735:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1867);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1736:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1901);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1350);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1737:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1901);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1738:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1410);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1739:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1497);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1740:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1904);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1560);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1741:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1904);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1742:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1649);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1743:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1908);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1744:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1917);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1745:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1914);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1746:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1915);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1747:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1477);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1748:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1482);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1749:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1880);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1750:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1400);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1751:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1330);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1752:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1963);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1753:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1937);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1754:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1333);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1755:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1403);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1756:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1337);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1757:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1515);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1758:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1717);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1759:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1944);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1760:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1412);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1761:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1143);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1762:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1567);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1763:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1433);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1764:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1146);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1765:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1158);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1766:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1392);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1767:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1819);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1726);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1768:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1819);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1769:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1714);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1770:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1572);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1771:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1801);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1772:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1408);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1773:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1389);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1774:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1706);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1775:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1960);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1776:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1820);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1777:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1957);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1778:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1839);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1847);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1779:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1719);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1780:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1721);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1781:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1722);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1782:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1723);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1783:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1859);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1726);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1784:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1859);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1785:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1724);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1786:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1825);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1787:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1749);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1788:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1881);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1789:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1384);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1790:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1385);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1791:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1738);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1792:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1414);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1793:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1568);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1794:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1754);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1795:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1441);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1796:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1974);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1797:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1413);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1798:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1394);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1799:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1499);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1800:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1673);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1801:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1130);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1802:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1674);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1803:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1778);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1804:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1450);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1805:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1352);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1806:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1476);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1807:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1500);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1808:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1483);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1809:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1661);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1810:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1922);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1811:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1530);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1812:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1669);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1813:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1966);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1814:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1532);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1815:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1496);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1816:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1492);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1817:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1528);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1818:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1953);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1819:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1349);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1820:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1128);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1821:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1065);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1822:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1051);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1823:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1081);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1824:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1106);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1825:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(926);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1826:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1564);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1827:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1607);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1828:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1348);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1883);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1829:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1348);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1830:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1776);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1831:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1351);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1832:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1585);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1833:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1448);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1834:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1882);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1835:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1775);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1836:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1324);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1837:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1720);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1838:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1770);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1839:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1903);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1840:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1327);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1841:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1766);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1833);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1842:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1501);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1843:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1357);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1844:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1455);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1845:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1343);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1846:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1862);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1847:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1459);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1848:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1918);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1849:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1463);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1850:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1465);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1851:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1524);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1852:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1472);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1853:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1481);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1854:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1486);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1855:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1487);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1856:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1751);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1857:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1792);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1858:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1602);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1859:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1713);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1860:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1358);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1861:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1793);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1862:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1535);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1863:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1561);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1864:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1796);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1865:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1798);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1866:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1085);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1867:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(959);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1868:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(949);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1869:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1096);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1870:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1084);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1871:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1207);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1872:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1209);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1873:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1445);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1874:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1891);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1875:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1805);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1876:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1505);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1877:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1866);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1878:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1923);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1355);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1879:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1923);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1582);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1880:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1923);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1586);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1881:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1451);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1882:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1606);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1883:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1453);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1884:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1870);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1885:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1589);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1886:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1910);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1887:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1475);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1888:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1485);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1889:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1488);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1890:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1948);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1891:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1773);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1892:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1885);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1920);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1893:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1885);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1894:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1600);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1895:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1613);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1896:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1615);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1897:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1797);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1898:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1897);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1899:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1920);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1900:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1141);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1901:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1226);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1902:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1221);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1903:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1028);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1904:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1098);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1905:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1190);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1906:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1112);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1907:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1034);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1908:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1055);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1909:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(970);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1910:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(988);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1911:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1204);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1912:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1211);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1913:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1192);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1914:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(922);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1915:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1162);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1916:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1295);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1917:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1356);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1918:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1984);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1919:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1446);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1920:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1832);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1921:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1769);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1922:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1594);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1923:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1353);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1924:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1364);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1925:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1506);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1926:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1621);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1927:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1456);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1928:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1464);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1929:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1504);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1930:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1469);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1931:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1509);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1932:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1489);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1933:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1611);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1934:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1968);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1935:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1786);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1936:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1511);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1937:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1335);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1938:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1610);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1939:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1512);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1940:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1597);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1941:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1612);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1942:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1513);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1943:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1599);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1944:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1603);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1945:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1516);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1946:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1622);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1947:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1620);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1948:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1860);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1949:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1438);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1950:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1373);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1951:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1584);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1952:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1374);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1953:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1614);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1954:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1834);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1955:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1837);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1956:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1777);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1957:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1869);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1958:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1662);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1959:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1520);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1960:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1938);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1961:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1454);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1962:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1846);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1963:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1466);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1964:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1654);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1965:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1883);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1966:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1847);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1967:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1664);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1968:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1850);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1969:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1931);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1970:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1852);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1971:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1667);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1972:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1853);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1973:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1833);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1974:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1941);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1975:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1376);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1976:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1590);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1977:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1350);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1978:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1449);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1979:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1361);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1980:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1473);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1981:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1075);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1982:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1077);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1983:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1934);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1984:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1042);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1985:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(967);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1986:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1987:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1806);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1988:
      ACCEPT_TOKEN(aux_sym_identifier_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(1988);
      END_STATE();
    case 1989:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1989);
      END_STATE();
    case 1990:
      ACCEPT_TOKEN(sym__line_continuation);
      END_STATE();
    case 1991:
      ACCEPT_TOKEN(sym__line_continuation);
      if (lookahead != 0 &&
          lookahead != '(' &&
          lookahead != ')') ADVANCE(929);
      END_STATE();
    case 1992:
      ACCEPT_TOKEN(sym__semicolon);
      END_STATE();
    case 1993:
      ACCEPT_TOKEN(sym__newline);
      if (lookahead == '\n') ADVANCE(1993);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 874, .external_lex_state = 2},
  [2] = {.lex_state = 874, .external_lex_state = 2},
  [3] = {.lex_state = 874, .external_lex_state = 2},
  [4] = {.lex_state = 28, .external_lex_state = 2},
  [5] = {.lex_state = 29, .external_lex_state = 2},
  [6] = {.lex_state = 29, .external_lex_state = 2},
  [7] = {.lex_state = 30, .external_lex_state = 2},
  [8] = {.lex_state = 28, .external_lex_state = 2},
  [9] = {.lex_state = 28, .external_lex_state = 2},
  [10] = {.lex_state = 29, .external_lex_state = 2},
  [11] = {.lex_state = 30, .external_lex_state = 2},
  [12] = {.lex_state = 30, .external_lex_state = 2},
  [13] = {.lex_state = 30, .external_lex_state = 2},
  [14] = {.lex_state = 28, .external_lex_state = 2},
  [15] = {.lex_state = 29, .external_lex_state = 2},
  [16] = {.lex_state = 30, .external_lex_state = 2},
  [17] = {.lex_state = 31, .external_lex_state = 2},
  [18] = {.lex_state = 29, .external_lex_state = 2},
  [19] = {.lex_state = 28, .external_lex_state = 2},
  [20] = {.lex_state = 30, .external_lex_state = 2},
  [21] = {.lex_state = 29, .external_lex_state = 2},
  [22] = {.lex_state = 28, .external_lex_state = 2},
  [23] = {.lex_state = 30, .external_lex_state = 2},
  [24] = {.lex_state = 30, .external_lex_state = 2},
  [25] = {.lex_state = 29, .external_lex_state = 2},
  [26] = {.lex_state = 29, .external_lex_state = 2},
  [27] = {.lex_state = 31, .external_lex_state = 2},
  [28] = {.lex_state = 28, .external_lex_state = 2},
  [29] = {.lex_state = 28, .external_lex_state = 2},
  [30] = {.lex_state = 29, .external_lex_state = 2},
  [31] = {.lex_state = 30, .external_lex_state = 2},
  [32] = {.lex_state = 29, .external_lex_state = 2},
  [33] = {.lex_state = 30, .external_lex_state = 2},
  [34] = {.lex_state = 28, .external_lex_state = 2},
  [35] = {.lex_state = 28, .external_lex_state = 2},
  [36] = {.lex_state = 30, .external_lex_state = 2},
  [37] = {.lex_state = 30, .external_lex_state = 2},
  [38] = {.lex_state = 28, .external_lex_state = 2},
  [39] = {.lex_state = 29, .external_lex_state = 2},
  [40] = {.lex_state = 28, .external_lex_state = 2},
  [41] = {.lex_state = 30, .external_lex_state = 2},
  [42] = {.lex_state = 29, .external_lex_state = 2},
  [43] = {.lex_state = 29, .external_lex_state = 2},
  [44] = {.lex_state = 30, .external_lex_state = 2},
  [45] = {.lex_state = 31, .external_lex_state = 2},
  [46] = {.lex_state = 29, .external_lex_state = 2},
  [47] = {.lex_state = 30, .external_lex_state = 2},
  [48] = {.lex_state = 29, .external_lex_state = 2},
  [49] = {.lex_state = 30, .external_lex_state = 2},
  [50] = {.lex_state = 29, .external_lex_state = 2},
  [51] = {.lex_state = 32, .external_lex_state = 2},
  [52] = {.lex_state = 32, .external_lex_state = 2},
  [53] = {.lex_state = 32, .external_lex_state = 2},
  [54] = {.lex_state = 32, .external_lex_state = 2},
  [55] = {.lex_state = 30, .external_lex_state = 2},
  [56] = {.lex_state = 31, .external_lex_state = 2},
  [57] = {.lex_state = 28, .external_lex_state = 2},
  [58] = {.lex_state = 29, .external_lex_state = 2},
  [59] = {.lex_state = 32, .external_lex_state = 2},
  [60] = {.lex_state = 33, .external_lex_state = 2},
  [61] = {.lex_state = 33, .external_lex_state = 2},
  [62] = {.lex_state = 33, .external_lex_state = 2},
  [63] = {.lex_state = 33, .external_lex_state = 2},
  [64] = {.lex_state = 33, .external_lex_state = 2},
  [65] = {.lex_state = 33, .external_lex_state = 2},
  [66] = {.lex_state = 33, .external_lex_state = 2},
  [67] = {.lex_state = 33, .external_lex_state = 2},
  [68] = {.lex_state = 35, .external_lex_state = 2},
  [69] = {.lex_state = 35, .external_lex_state = 2},
  [70] = {.lex_state = 33, .external_lex_state = 2},
  [71] = {.lex_state = 33, .external_lex_state = 2},
  [72] = {.lex_state = 33, .external_lex_state = 2},
  [73] = {.lex_state = 33, .external_lex_state = 2},
  [74] = {.lex_state = 33, .external_lex_state = 2},
  [75] = {.lex_state = 35, .external_lex_state = 2},
  [76] = {.lex_state = 33, .external_lex_state = 2},
  [77] = {.lex_state = 33, .external_lex_state = 2},
  [78] = {.lex_state = 33, .external_lex_state = 2},
  [79] = {.lex_state = 33, .external_lex_state = 2},
  [80] = {.lex_state = 35, .external_lex_state = 2},
  [81] = {.lex_state = 33, .external_lex_state = 2},
  [82] = {.lex_state = 33, .external_lex_state = 2},
  [83] = {.lex_state = 33, .external_lex_state = 2},
  [84] = {.lex_state = 33, .external_lex_state = 2},
  [85] = {.lex_state = 45, .external_lex_state = 2},
  [86] = {.lex_state = 46, .external_lex_state = 2},
  [87] = {.lex_state = 46, .external_lex_state = 2},
  [88] = {.lex_state = 45, .external_lex_state = 2},
  [89] = {.lex_state = 45, .external_lex_state = 2},
  [90] = {.lex_state = 47, .external_lex_state = 2},
  [91] = {.lex_state = 46, .external_lex_state = 2},
  [92] = {.lex_state = 45, .external_lex_state = 2},
  [93] = {.lex_state = 46, .external_lex_state = 2},
  [94] = {.lex_state = 45, .external_lex_state = 2},
  [95] = {.lex_state = 48, .external_lex_state = 2},
  [96] = {.lex_state = 45, .external_lex_state = 2},
  [97] = {.lex_state = 45, .external_lex_state = 2},
  [98] = {.lex_state = 47, .external_lex_state = 2},
  [99] = {.lex_state = 46, .external_lex_state = 2},
  [100] = {.lex_state = 48, .external_lex_state = 2},
  [101] = {.lex_state = 46, .external_lex_state = 2},
  [102] = {.lex_state = 47, .external_lex_state = 2},
  [103] = {.lex_state = 46, .external_lex_state = 2},
  [104] = {.lex_state = 47, .external_lex_state = 2},
  [105] = {.lex_state = 47, .external_lex_state = 2},
  [106] = {.lex_state = 46, .external_lex_state = 2},
  [107] = {.lex_state = 47, .external_lex_state = 2},
  [108] = {.lex_state = 46, .external_lex_state = 2},
  [109] = {.lex_state = 36, .external_lex_state = 2},
  [110] = {.lex_state = 45, .external_lex_state = 2},
  [111] = {.lex_state = 45, .external_lex_state = 2},
  [112] = {.lex_state = 46, .external_lex_state = 2},
  [113] = {.lex_state = 47, .external_lex_state = 2},
  [114] = {.lex_state = 47, .external_lex_state = 2},
  [115] = {.lex_state = 47, .external_lex_state = 2},
  [116] = {.lex_state = 46, .external_lex_state = 2},
  [117] = {.lex_state = 45, .external_lex_state = 2},
  [118] = {.lex_state = 47, .external_lex_state = 2},
  [119] = {.lex_state = 48, .external_lex_state = 2},
  [120] = {.lex_state = 46, .external_lex_state = 2},
  [121] = {.lex_state = 47, .external_lex_state = 2},
  [122] = {.lex_state = 36, .external_lex_state = 2},
  [123] = {.lex_state = 47, .external_lex_state = 2},
  [124] = {.lex_state = 45, .external_lex_state = 2},
  [125] = {.lex_state = 46, .external_lex_state = 2},
  [126] = {.lex_state = 45, .external_lex_state = 2},
  [127] = {.lex_state = 47, .external_lex_state = 2},
  [128] = {.lex_state = 47, .external_lex_state = 2},
  [129] = {.lex_state = 46, .external_lex_state = 2},
  [130] = {.lex_state = 46, .external_lex_state = 2},
  [131] = {.lex_state = 47, .external_lex_state = 2},
  [132] = {.lex_state = 36, .external_lex_state = 2},
  [133] = {.lex_state = 47, .external_lex_state = 2},
  [134] = {.lex_state = 36, .external_lex_state = 2},
  [135] = {.lex_state = 36, .external_lex_state = 2},
  [136] = {.lex_state = 36, .external_lex_state = 2},
  [137] = {.lex_state = 46, .external_lex_state = 2},
  [138] = {.lex_state = 35, .external_lex_state = 2},
  [139] = {.lex_state = 35, .external_lex_state = 2},
  [140] = {.lex_state = 35, .external_lex_state = 2},
  [141] = {.lex_state = 35, .external_lex_state = 2},
  [142] = {.lex_state = 35, .external_lex_state = 2},
  [143] = {.lex_state = 35, .external_lex_state = 2},
  [144] = {.lex_state = 35, .external_lex_state = 2},
  [145] = {.lex_state = 35, .external_lex_state = 2},
  [146] = {.lex_state = 35, .external_lex_state = 2},
  [147] = {.lex_state = 35, .external_lex_state = 2},
  [148] = {.lex_state = 35, .external_lex_state = 2},
  [149] = {.lex_state = 35, .external_lex_state = 2},
  [150] = {.lex_state = 35, .external_lex_state = 2},
  [151] = {.lex_state = 46, .external_lex_state = 2},
  [152] = {.lex_state = 34, .external_lex_state = 2},
  [153] = {.lex_state = 34, .external_lex_state = 2},
  [154] = {.lex_state = 34, .external_lex_state = 2},
  [155] = {.lex_state = 45, .external_lex_state = 2},
  [156] = {.lex_state = 37, .external_lex_state = 2},
  [157] = {.lex_state = 44, .external_lex_state = 2},
  [158] = {.lex_state = 44, .external_lex_state = 2},
  [159] = {.lex_state = 44, .external_lex_state = 2},
  [160] = {.lex_state = 34, .external_lex_state = 2},
  [161] = {.lex_state = 47, .external_lex_state = 2},
  [162] = {.lex_state = 38, .external_lex_state = 2},
  [163] = {.lex_state = 38, .external_lex_state = 2},
  [164] = {.lex_state = 38, .external_lex_state = 2},
  [165] = {.lex_state = 37, .external_lex_state = 2},
  [166] = {.lex_state = 39, .external_lex_state = 2},
  [167] = {.lex_state = 40, .external_lex_state = 2},
  [168] = {.lex_state = 38, .external_lex_state = 2},
  [169] = {.lex_state = 40, .external_lex_state = 2},
  [170] = {.lex_state = 44, .external_lex_state = 2},
  [171] = {.lex_state = 34, .external_lex_state = 2},
  [172] = {.lex_state = 38, .external_lex_state = 2},
  [173] = {.lex_state = 40, .external_lex_state = 2},
  [174] = {.lex_state = 34, .external_lex_state = 2},
  [175] = {.lex_state = 40, .external_lex_state = 2},
  [176] = {.lex_state = 44, .external_lex_state = 2},
  [177] = {.lex_state = 44, .external_lex_state = 2},
  [178] = {.lex_state = 48, .external_lex_state = 2},
  [179] = {.lex_state = 34, .external_lex_state = 2},
  [180] = {.lex_state = 37, .external_lex_state = 2},
  [181] = {.lex_state = 37, .external_lex_state = 2},
  [182] = {.lex_state = 37, .external_lex_state = 2},
  [183] = {.lex_state = 34, .external_lex_state = 2},
  [184] = {.lex_state = 34, .external_lex_state = 2},
  [185] = {.lex_state = 44, .external_lex_state = 2},
  [186] = {.lex_state = 44, .external_lex_state = 2},
  [187] = {.lex_state = 38, .external_lex_state = 2},
  [188] = {.lex_state = 38, .external_lex_state = 2},
  [189] = {.lex_state = 37, .external_lex_state = 2},
  [190] = {.lex_state = 38, .external_lex_state = 2},
  [191] = {.lex_state = 39, .external_lex_state = 2},
  [192] = {.lex_state = 39, .external_lex_state = 2},
  [193] = {.lex_state = 44, .external_lex_state = 2},
  [194] = {.lex_state = 38, .external_lex_state = 2},
  [195] = {.lex_state = 36, .external_lex_state = 2},
  [196] = {.lex_state = 41, .external_lex_state = 2},
  [197] = {.lex_state = 36, .external_lex_state = 2},
  [198] = {.lex_state = 36, .external_lex_state = 2},
  [199] = {.lex_state = 36, .external_lex_state = 2},
  [200] = {.lex_state = 37, .external_lex_state = 2},
  [201] = {.lex_state = 36, .external_lex_state = 2},
  [202] = {.lex_state = 36, .external_lex_state = 2},
  [203] = {.lex_state = 36, .external_lex_state = 2},
  [204] = {.lex_state = 36, .external_lex_state = 2},
  [205] = {.lex_state = 40, .external_lex_state = 2},
  [206] = {.lex_state = 39, .external_lex_state = 2},
  [207] = {.lex_state = 36, .external_lex_state = 2},
  [208] = {.lex_state = 42, .external_lex_state = 2},
  [209] = {.lex_state = 38, .external_lex_state = 2},
  [210] = {.lex_state = 38, .external_lex_state = 2},
  [211] = {.lex_state = 42, .external_lex_state = 2},
  [212] = {.lex_state = 42, .external_lex_state = 2},
  [213] = {.lex_state = 42, .external_lex_state = 2},
  [214] = {.lex_state = 38, .external_lex_state = 2},
  [215] = {.lex_state = 38, .external_lex_state = 2},
  [216] = {.lex_state = 40, .external_lex_state = 2},
  [217] = {.lex_state = 42, .external_lex_state = 2},
  [218] = {.lex_state = 40, .external_lex_state = 2},
  [219] = {.lex_state = 38, .external_lex_state = 2},
  [220] = {.lex_state = 40, .external_lex_state = 2},
  [221] = {.lex_state = 43, .external_lex_state = 2},
  [222] = {.lex_state = 42, .external_lex_state = 1},
  [223] = {.lex_state = 43, .external_lex_state = 2},
  [224] = {.lex_state = 42, .external_lex_state = 2},
  [225] = {.lex_state = 42, .external_lex_state = 2},
  [226] = {.lex_state = 42, .external_lex_state = 2},
  [227] = {.lex_state = 42, .external_lex_state = 1},
  [228] = {.lex_state = 42, .external_lex_state = 2},
  [229] = {.lex_state = 43, .external_lex_state = 2},
  [230] = {.lex_state = 43, .external_lex_state = 2},
  [231] = {.lex_state = 43, .external_lex_state = 2},
  [232] = {.lex_state = 874, .external_lex_state = 2},
  [233] = {.lex_state = 874, .external_lex_state = 2},
  [234] = {.lex_state = 874, .external_lex_state = 2},
  [235] = {.lex_state = 874, .external_lex_state = 2},
  [236] = {.lex_state = 874, .external_lex_state = 2},
  [237] = {.lex_state = 874, .external_lex_state = 2},
  [238] = {.lex_state = 874, .external_lex_state = 2},
  [239] = {.lex_state = 874, .external_lex_state = 2},
  [240] = {.lex_state = 874, .external_lex_state = 2},
  [241] = {.lex_state = 874, .external_lex_state = 2},
  [242] = {.lex_state = 874, .external_lex_state = 2},
  [243] = {.lex_state = 874, .external_lex_state = 2},
  [244] = {.lex_state = 874, .external_lex_state = 2},
  [245] = {.lex_state = 874, .external_lex_state = 2},
  [246] = {.lex_state = 874, .external_lex_state = 2},
  [247] = {.lex_state = 874, .external_lex_state = 2},
  [248] = {.lex_state = 874, .external_lex_state = 2},
  [249] = {.lex_state = 874, .external_lex_state = 2},
  [250] = {.lex_state = 874, .external_lex_state = 2},
  [251] = {.lex_state = 874, .external_lex_state = 2},
  [252] = {.lex_state = 42, .external_lex_state = 2},
  [253] = {.lex_state = 42, .external_lex_state = 2},
  [254] = {.lex_state = 874, .external_lex_state = 2},
  [255] = {.lex_state = 42, .external_lex_state = 2},
  [256] = {.lex_state = 42, .external_lex_state = 2},
  [257] = {.lex_state = 874, .external_lex_state = 2},
  [258] = {.lex_state = 874, .external_lex_state = 2},
  [259] = {.lex_state = 42, .external_lex_state = 2},
  [260] = {.lex_state = 42, .external_lex_state = 2},
  [261] = {.lex_state = 874, .external_lex_state = 2},
  [262] = {.lex_state = 42, .external_lex_state = 2},
  [263] = {.lex_state = 874, .external_lex_state = 2},
  [264] = {.lex_state = 874, .external_lex_state = 2},
  [265] = {.lex_state = 874, .external_lex_state = 2},
  [266] = {.lex_state = 874, .external_lex_state = 2},
  [267] = {.lex_state = 874, .external_lex_state = 2},
  [268] = {.lex_state = 42, .external_lex_state = 2},
  [269] = {.lex_state = 42, .external_lex_state = 2},
  [270] = {.lex_state = 874, .external_lex_state = 2},
  [271] = {.lex_state = 874, .external_lex_state = 2},
  [272] = {.lex_state = 42, .external_lex_state = 2},
  [273] = {.lex_state = 874, .external_lex_state = 2},
  [274] = {.lex_state = 42, .external_lex_state = 2},
  [275] = {.lex_state = 874, .external_lex_state = 2},
  [276] = {.lex_state = 874, .external_lex_state = 2},
  [277] = {.lex_state = 42, .external_lex_state = 2},
  [278] = {.lex_state = 874, .external_lex_state = 2},
  [279] = {.lex_state = 874, .external_lex_state = 2},
  [280] = {.lex_state = 874, .external_lex_state = 2},
  [281] = {.lex_state = 42, .external_lex_state = 2},
  [282] = {.lex_state = 874, .external_lex_state = 2},
  [283] = {.lex_state = 874, .external_lex_state = 2},
  [284] = {.lex_state = 874, .external_lex_state = 2},
  [285] = {.lex_state = 874, .external_lex_state = 2},
  [286] = {.lex_state = 874, .external_lex_state = 2},
  [287] = {.lex_state = 42, .external_lex_state = 2},
  [288] = {.lex_state = 874, .external_lex_state = 2},
  [289] = {.lex_state = 874, .external_lex_state = 2},
  [290] = {.lex_state = 874, .external_lex_state = 2},
  [291] = {.lex_state = 874, .external_lex_state = 2},
  [292] = {.lex_state = 874, .external_lex_state = 2},
  [293] = {.lex_state = 874, .external_lex_state = 2},
  [294] = {.lex_state = 874, .external_lex_state = 2},
  [295] = {.lex_state = 29, .external_lex_state = 2},
  [296] = {.lex_state = 31, .external_lex_state = 2},
  [297] = {.lex_state = 31, .external_lex_state = 2},
  [298] = {.lex_state = 31, .external_lex_state = 2},
  [299] = {.lex_state = 29, .external_lex_state = 2},
  [300] = {.lex_state = 31, .external_lex_state = 2},
  [301] = {.lex_state = 31, .external_lex_state = 2},
  [302] = {.lex_state = 29, .external_lex_state = 2},
  [303] = {.lex_state = 28, .external_lex_state = 2},
  [304] = {.lex_state = 28, .external_lex_state = 2},
  [305] = {.lex_state = 28, .external_lex_state = 2},
  [306] = {.lex_state = 28, .external_lex_state = 2},
  [307] = {.lex_state = 30, .external_lex_state = 2},
  [308] = {.lex_state = 30, .external_lex_state = 2},
  [309] = {.lex_state = 29, .external_lex_state = 2},
  [310] = {.lex_state = 31, .external_lex_state = 2},
  [311] = {.lex_state = 31, .external_lex_state = 2},
  [312] = {.lex_state = 31, .external_lex_state = 2},
  [313] = {.lex_state = 31, .external_lex_state = 2},
  [314] = {.lex_state = 30, .external_lex_state = 2},
  [315] = {.lex_state = 30, .external_lex_state = 2},
  [316] = {.lex_state = 31, .external_lex_state = 2},
  [317] = {.lex_state = 31, .external_lex_state = 2},
  [318] = {.lex_state = 29, .external_lex_state = 2},
  [319] = {.lex_state = 28, .external_lex_state = 2},
  [320] = {.lex_state = 30, .external_lex_state = 2},
  [321] = {.lex_state = 30, .external_lex_state = 2},
  [322] = {.lex_state = 30, .external_lex_state = 2},
  [323] = {.lex_state = 30, .external_lex_state = 2},
  [324] = {.lex_state = 28, .external_lex_state = 2},
  [325] = {.lex_state = 28, .external_lex_state = 2},
  [326] = {.lex_state = 31, .external_lex_state = 2},
  [327] = {.lex_state = 30, .external_lex_state = 2},
  [328] = {.lex_state = 31, .external_lex_state = 2},
  [329] = {.lex_state = 28, .external_lex_state = 2},
  [330] = {.lex_state = 30, .external_lex_state = 2},
  [331] = {.lex_state = 31, .external_lex_state = 2},
  [332] = {.lex_state = 31, .external_lex_state = 2},
  [333] = {.lex_state = 30, .external_lex_state = 2},
  [334] = {.lex_state = 30, .external_lex_state = 2},
  [335] = {.lex_state = 29, .external_lex_state = 2},
  [336] = {.lex_state = 31, .external_lex_state = 2},
  [337] = {.lex_state = 31, .external_lex_state = 2},
  [338] = {.lex_state = 31, .external_lex_state = 2},
  [339] = {.lex_state = 31, .external_lex_state = 2},
  [340] = {.lex_state = 28, .external_lex_state = 2},
  [341] = {.lex_state = 30, .external_lex_state = 2},
  [342] = {.lex_state = 28, .external_lex_state = 2},
  [343] = {.lex_state = 30, .external_lex_state = 2},
  [344] = {.lex_state = 31, .external_lex_state = 2},
  [345] = {.lex_state = 28, .external_lex_state = 2},
  [346] = {.lex_state = 28, .external_lex_state = 2},
  [347] = {.lex_state = 30, .external_lex_state = 2},
  [348] = {.lex_state = 30, .external_lex_state = 2},
  [349] = {.lex_state = 30, .external_lex_state = 2},
  [350] = {.lex_state = 31, .external_lex_state = 2},
  [351] = {.lex_state = 29, .external_lex_state = 2},
  [352] = {.lex_state = 29, .external_lex_state = 2},
  [353] = {.lex_state = 30, .external_lex_state = 2},
  [354] = {.lex_state = 29, .external_lex_state = 2},
  [355] = {.lex_state = 29, .external_lex_state = 2},
  [356] = {.lex_state = 28, .external_lex_state = 2},
  [357] = {.lex_state = 31, .external_lex_state = 2},
  [358] = {.lex_state = 30, .external_lex_state = 2},
  [359] = {.lex_state = 30, .external_lex_state = 2},
  [360] = {.lex_state = 31, .external_lex_state = 2},
  [361] = {.lex_state = 31, .external_lex_state = 2},
  [362] = {.lex_state = 31, .external_lex_state = 2},
  [363] = {.lex_state = 29, .external_lex_state = 2},
  [364] = {.lex_state = 29, .external_lex_state = 2},
  [365] = {.lex_state = 28, .external_lex_state = 2},
  [366] = {.lex_state = 29, .external_lex_state = 2},
  [367] = {.lex_state = 31, .external_lex_state = 2},
  [368] = {.lex_state = 30, .external_lex_state = 2},
  [369] = {.lex_state = 28, .external_lex_state = 2},
  [370] = {.lex_state = 30, .external_lex_state = 2},
  [371] = {.lex_state = 29, .external_lex_state = 2},
  [372] = {.lex_state = 29, .external_lex_state = 2},
  [373] = {.lex_state = 31, .external_lex_state = 2},
  [374] = {.lex_state = 29, .external_lex_state = 2},
  [375] = {.lex_state = 29, .external_lex_state = 2},
  [376] = {.lex_state = 30, .external_lex_state = 2},
  [377] = {.lex_state = 29, .external_lex_state = 2},
  [378] = {.lex_state = 29, .external_lex_state = 2},
  [379] = {.lex_state = 28, .external_lex_state = 2},
  [380] = {.lex_state = 29, .external_lex_state = 2},
  [381] = {.lex_state = 29, .external_lex_state = 2},
  [382] = {.lex_state = 31, .external_lex_state = 2},
  [383] = {.lex_state = 29, .external_lex_state = 2},
  [384] = {.lex_state = 28, .external_lex_state = 2},
  [385] = {.lex_state = 31, .external_lex_state = 2},
  [386] = {.lex_state = 28, .external_lex_state = 2},
  [387] = {.lex_state = 28, .external_lex_state = 2},
  [388] = {.lex_state = 28, .external_lex_state = 2},
  [389] = {.lex_state = 29, .external_lex_state = 2},
  [390] = {.lex_state = 28, .external_lex_state = 2},
  [391] = {.lex_state = 28, .external_lex_state = 2},
  [392] = {.lex_state = 28, .external_lex_state = 2},
  [393] = {.lex_state = 32, .external_lex_state = 2},
  [394] = {.lex_state = 32, .external_lex_state = 2},
  [395] = {.lex_state = 32, .external_lex_state = 2},
  [396] = {.lex_state = 32, .external_lex_state = 2},
  [397] = {.lex_state = 32, .external_lex_state = 2},
  [398] = {.lex_state = 32, .external_lex_state = 2},
  [399] = {.lex_state = 32, .external_lex_state = 2},
  [400] = {.lex_state = 32, .external_lex_state = 2},
  [401] = {.lex_state = 32, .external_lex_state = 2},
  [402] = {.lex_state = 32, .external_lex_state = 2},
  [403] = {.lex_state = 32, .external_lex_state = 2},
  [404] = {.lex_state = 32, .external_lex_state = 2},
  [405] = {.lex_state = 32, .external_lex_state = 2},
  [406] = {.lex_state = 32, .external_lex_state = 2},
  [407] = {.lex_state = 32, .external_lex_state = 2},
  [408] = {.lex_state = 32, .external_lex_state = 2},
  [409] = {.lex_state = 32, .external_lex_state = 2},
  [410] = {.lex_state = 32, .external_lex_state = 2},
  [411] = {.lex_state = 32, .external_lex_state = 2},
  [412] = {.lex_state = 32, .external_lex_state = 2},
  [413] = {.lex_state = 32, .external_lex_state = 2},
  [414] = {.lex_state = 32, .external_lex_state = 2},
  [415] = {.lex_state = 32, .external_lex_state = 2},
  [416] = {.lex_state = 10, .external_lex_state = 3},
  [417] = {.lex_state = 14, .external_lex_state = 3},
  [418] = {.lex_state = 10, .external_lex_state = 3},
  [419] = {.lex_state = 14, .external_lex_state = 3},
  [420] = {.lex_state = 3, .external_lex_state = 1},
  [421] = {.lex_state = 10, .external_lex_state = 3},
  [422] = {.lex_state = 14, .external_lex_state = 3},
  [423] = {.lex_state = 3, .external_lex_state = 2},
  [424] = {.lex_state = 25, .external_lex_state = 2},
  [425] = {.lex_state = 2, .external_lex_state = 2},
  [426] = {.lex_state = 2, .external_lex_state = 1},
  [427] = {.lex_state = 9, .external_lex_state = 2},
  [428] = {.lex_state = 7, .external_lex_state = 2},
  [429] = {.lex_state = 6, .external_lex_state = 2},
  [430] = {.lex_state = 2, .external_lex_state = 2},
  [431] = {.lex_state = 8, .external_lex_state = 2},
  [432] = {.lex_state = 4, .external_lex_state = 2},
  [433] = {.lex_state = 27, .external_lex_state = 2},
  [434] = {.lex_state = 24, .external_lex_state = 2},
  [435] = {.lex_state = 49, .external_lex_state = 2},
  [436] = {.lex_state = 49, .external_lex_state = 2},
  [437] = {.lex_state = 49, .external_lex_state = 2},
  [438] = {.lex_state = 49, .external_lex_state = 2},
  [439] = {.lex_state = 25, .external_lex_state = 2},
  [440] = {.lex_state = 25, .external_lex_state = 1},
  [441] = {.lex_state = 21, .external_lex_state = 2},
  [442] = {.lex_state = 21, .external_lex_state = 2},
  [443] = {.lex_state = 25, .external_lex_state = 2},
  [444] = {.lex_state = 25, .external_lex_state = 1},
  [445] = {.lex_state = 25, .external_lex_state = 1},
  [446] = {.lex_state = 26, .external_lex_state = 2},
  [447] = {.lex_state = 11, .external_lex_state = 4},
  [448] = {.lex_state = 25, .external_lex_state = 2},
  [449] = {.lex_state = 25, .external_lex_state = 2},
  [450] = {.lex_state = 5, .external_lex_state = 2},
  [451] = {.lex_state = 25, .external_lex_state = 2},
  [452] = {.lex_state = 25, .external_lex_state = 2},
  [453] = {.lex_state = 25, .external_lex_state = 2},
  [454] = {.lex_state = 25, .external_lex_state = 2},
  [455] = {.lex_state = 25, .external_lex_state = 2},
  [456] = {.lex_state = 25, .external_lex_state = 2},
  [457] = {.lex_state = 25, .external_lex_state = 2},
  [458] = {.lex_state = 11, .external_lex_state = 4},
  [459] = {.lex_state = 25, .external_lex_state = 2},
  [460] = {.lex_state = 25, .external_lex_state = 2},
  [461] = {.lex_state = 33, .external_lex_state = 2},
  [462] = {.lex_state = 33, .external_lex_state = 2},
  [463] = {.lex_state = 11, .external_lex_state = 4},
  [464] = {.lex_state = 11, .external_lex_state = 4},
  [465] = {.lex_state = 25, .external_lex_state = 2},
  [466] = {.lex_state = 22, .external_lex_state = 2},
  [467] = {.lex_state = 22, .external_lex_state = 2},
  [468] = {.lex_state = 22, .external_lex_state = 2},
  [469] = {.lex_state = 22, .external_lex_state = 2},
  [470] = {.lex_state = 22, .external_lex_state = 2},
  [471] = {.lex_state = 22, .external_lex_state = 2},
  [472] = {.lex_state = 22, .external_lex_state = 2},
  [473] = {.lex_state = 22, .external_lex_state = 2},
  [474] = {.lex_state = 22, .external_lex_state = 2},
  [475] = {.lex_state = 22, .external_lex_state = 2},
  [476] = {.lex_state = 23, .external_lex_state = 2},
  [477] = {.lex_state = 23, .external_lex_state = 2},
  [478] = {.lex_state = 23, .external_lex_state = 2},
  [479] = {.lex_state = 20, .external_lex_state = 2},
  [480] = {.lex_state = 23, .external_lex_state = 2},
  [481] = {.lex_state = 35, .external_lex_state = 2},
  [482] = {.lex_state = 23, .external_lex_state = 2},
  [483] = {.lex_state = 20, .external_lex_state = 2},
  [484] = {.lex_state = 35, .external_lex_state = 2},
  [485] = {.lex_state = 34, .external_lex_state = 2},
  [486] = {.lex_state = 46, .external_lex_state = 2},
  [487] = {.lex_state = 44, .external_lex_state = 2},
  [488] = {.lex_state = 44, .external_lex_state = 2},
  [489] = {.lex_state = 47, .external_lex_state = 2},
  [490] = {.lex_state = 34, .external_lex_state = 2},
  [491] = {.lex_state = 23, .external_lex_state = 2},
  [492] = {.lex_state = 20, .external_lex_state = 2},
  [493] = {.lex_state = 46, .external_lex_state = 2},
  [494] = {.lex_state = 23, .external_lex_state = 2},
  [495] = {.lex_state = 23, .external_lex_state = 2},
  [496] = {.lex_state = 23, .external_lex_state = 2},
  [497] = {.lex_state = 48, .external_lex_state = 2},
  [498] = {.lex_state = 23, .external_lex_state = 2},
  [499] = {.lex_state = 47, .external_lex_state = 2},
  [500] = {.lex_state = 48, .external_lex_state = 2},
  [501] = {.lex_state = 45, .external_lex_state = 2},
  [502] = {.lex_state = 45, .external_lex_state = 2},
  [503] = {.lex_state = 37, .external_lex_state = 2},
  [504] = {.lex_state = 23, .external_lex_state = 2},
  [505] = {.lex_state = 24, .external_lex_state = 2},
  [506] = {.lex_state = 23, .external_lex_state = 2},
  [507] = {.lex_state = 37, .external_lex_state = 2},
  [508] = {.lex_state = 39, .external_lex_state = 2},
  [509] = {.lex_state = 36, .external_lex_state = 2},
  [510] = {.lex_state = 38, .external_lex_state = 2},
  [511] = {.lex_state = 42, .external_lex_state = 1},
  [512] = {.lex_state = 38, .external_lex_state = 2},
  [513] = {.lex_state = 42, .external_lex_state = 1},
  [514] = {.lex_state = 13, .external_lex_state = 4},
  [515] = {.lex_state = 13, .external_lex_state = 4},
  [516] = {.lex_state = 13, .external_lex_state = 4},
  [517] = {.lex_state = 42, .external_lex_state = 1},
  [518] = {.lex_state = 20, .external_lex_state = 2},
  [519] = {.lex_state = 40, .external_lex_state = 2},
  [520] = {.lex_state = 42, .external_lex_state = 1},
  [521] = {.lex_state = 23, .external_lex_state = 1},
  [522] = {.lex_state = 42, .external_lex_state = 1},
  [523] = {.lex_state = 40, .external_lex_state = 2},
  [524] = {.lex_state = 20, .external_lex_state = 2},
  [525] = {.lex_state = 23, .external_lex_state = 2},
  [526] = {.lex_state = 39, .external_lex_state = 2},
  [527] = {.lex_state = 36, .external_lex_state = 2},
  [528] = {.lex_state = 23, .external_lex_state = 2},
  [529] = {.lex_state = 41, .external_lex_state = 2},
  [530] = {.lex_state = 20, .external_lex_state = 2},
  [531] = {.lex_state = 42, .external_lex_state = 2},
  [532] = {.lex_state = 16, .external_lex_state = 5},
  [533] = {.lex_state = 23, .external_lex_state = 1},
  [534] = {.lex_state = 23, .external_lex_state = 2},
  [535] = {.lex_state = 23, .external_lex_state = 1},
  [536] = {.lex_state = 18, .external_lex_state = 5},
  [537] = {.lex_state = 23, .external_lex_state = 1},
  [538] = {.lex_state = 16, .external_lex_state = 5},
  [539] = {.lex_state = 23, .external_lex_state = 2},
  [540] = {.lex_state = 19, .external_lex_state = 5},
  [541] = {.lex_state = 19, .external_lex_state = 5},
  [542] = {.lex_state = 18, .external_lex_state = 5},
  [543] = {.lex_state = 19, .external_lex_state = 5},
  [544] = {.lex_state = 16, .external_lex_state = 5},
  [545] = {.lex_state = 18, .external_lex_state = 5},
  [546] = {.lex_state = 42, .external_lex_state = 2},
  [547] = {.lex_state = 16, .external_lex_state = 5},
  [548] = {.lex_state = 19, .external_lex_state = 5},
  [549] = {.lex_state = 16, .external_lex_state = 5},
  [550] = {.lex_state = 18, .external_lex_state = 5},
  [551] = {.lex_state = 23, .external_lex_state = 1},
  [552] = {.lex_state = 19, .external_lex_state = 5},
  [553] = {.lex_state = 18, .external_lex_state = 5},
  [554] = {.lex_state = 19, .external_lex_state = 5},
  [555] = {.lex_state = 16, .external_lex_state = 5},
  [556] = {.lex_state = 18, .external_lex_state = 5},
  [557] = {.lex_state = 19, .external_lex_state = 5},
  [558] = {.lex_state = 16, .external_lex_state = 5},
  [559] = {.lex_state = 23, .external_lex_state = 1},
  [560] = {.lex_state = 17, .external_lex_state = 5},
  [561] = {.lex_state = 19, .external_lex_state = 5},
  [562] = {.lex_state = 16, .external_lex_state = 5},
  [563] = {.lex_state = 23, .external_lex_state = 2},
  [564] = {.lex_state = 10, .external_lex_state = 3},
  [565] = {.lex_state = 10, .external_lex_state = 3},
  [566] = {.lex_state = 10, .external_lex_state = 3},
  [567] = {.lex_state = 23, .external_lex_state = 2},
  [568] = {.lex_state = 23, .external_lex_state = 2},
  [569] = {.lex_state = 10, .external_lex_state = 3},
  [570] = {.lex_state = 10, .external_lex_state = 3},
  [571] = {.lex_state = 23, .external_lex_state = 2},
  [572] = {.lex_state = 23, .external_lex_state = 2},
  [573] = {.lex_state = 23, .external_lex_state = 2},
  [574] = {.lex_state = 23, .external_lex_state = 2},
  [575] = {.lex_state = 10, .external_lex_state = 3},
  [576] = {.lex_state = 23, .external_lex_state = 2},
  [577] = {.lex_state = 24, .external_lex_state = 2},
  [578] = {.lex_state = 24, .external_lex_state = 2},
  [579] = {.lex_state = 23, .external_lex_state = 2},
  [580] = {.lex_state = 14, .external_lex_state = 3},
  [581] = {.lex_state = 14, .external_lex_state = 3},
  [582] = {.lex_state = 10, .external_lex_state = 3},
  [583] = {.lex_state = 10, .external_lex_state = 3},
  [584] = {.lex_state = 14, .external_lex_state = 3},
  [585] = {.lex_state = 14, .external_lex_state = 3},
  [586] = {.lex_state = 14, .external_lex_state = 3},
  [587] = {.lex_state = 55, .external_lex_state = 4},
  [588] = {.lex_state = 14, .external_lex_state = 3},
  [589] = {.lex_state = 14, .external_lex_state = 3},
  [590] = {.lex_state = 14, .external_lex_state = 3},
  [591] = {.lex_state = 55, .external_lex_state = 4},
  [592] = {.lex_state = 14, .external_lex_state = 3},
  [593] = {.lex_state = 10, .external_lex_state = 3},
  [594] = {.lex_state = 14, .external_lex_state = 3},
  [595] = {.lex_state = 10, .external_lex_state = 3},
  [596] = {.lex_state = 10, .external_lex_state = 3},
  [597] = {.lex_state = 23, .external_lex_state = 2},
  [598] = {.lex_state = 23, .external_lex_state = 2},
  [599] = {.lex_state = 10, .external_lex_state = 3},
  [600] = {.lex_state = 10, .external_lex_state = 3},
  [601] = {.lex_state = 23, .external_lex_state = 2},
  [602] = {.lex_state = 23, .external_lex_state = 2},
  [603] = {.lex_state = 24, .external_lex_state = 2},
  [604] = {.lex_state = 24, .external_lex_state = 2},
  [605] = {.lex_state = 23, .external_lex_state = 2},
  [606] = {.lex_state = 14, .external_lex_state = 3},
  [607] = {.lex_state = 14, .external_lex_state = 3},
  [608] = {.lex_state = 14, .external_lex_state = 3},
  [609] = {.lex_state = 14, .external_lex_state = 3},
  [610] = {.lex_state = 10, .external_lex_state = 3},
  [611] = {.lex_state = 14, .external_lex_state = 3},
  [612] = {.lex_state = 14, .external_lex_state = 3},
  [613] = {.lex_state = 23, .external_lex_state = 2},
  [614] = {.lex_state = 23, .external_lex_state = 2},
  [615] = {.lex_state = 14, .external_lex_state = 3},
  [616] = {.lex_state = 15, .external_lex_state = 5},
  [617] = {.lex_state = 10, .external_lex_state = 3},
  [618] = {.lex_state = 10, .external_lex_state = 3},
  [619] = {.lex_state = 10, .external_lex_state = 3},
  [620] = {.lex_state = 10, .external_lex_state = 3},
  [621] = {.lex_state = 14, .external_lex_state = 3},
  [622] = {.lex_state = 23, .external_lex_state = 2},
  [623] = {.lex_state = 14, .external_lex_state = 3},
  [624] = {.lex_state = 14, .external_lex_state = 3},
  [625] = {.lex_state = 14, .external_lex_state = 3},
  [626] = {.lex_state = 23, .external_lex_state = 2},
  [627] = {.lex_state = 10, .external_lex_state = 3},
  [628] = {.lex_state = 14, .external_lex_state = 3},
  [629] = {.lex_state = 10, .external_lex_state = 3},
  [630] = {.lex_state = 10, .external_lex_state = 3},
  [631] = {.lex_state = 10, .external_lex_state = 3},
  [632] = {.lex_state = 10, .external_lex_state = 3},
  [633] = {.lex_state = 10, .external_lex_state = 3},
  [634] = {.lex_state = 23, .external_lex_state = 2},
  [635] = {.lex_state = 14, .external_lex_state = 3},
  [636] = {.lex_state = 14, .external_lex_state = 3},
  [637] = {.lex_state = 14, .external_lex_state = 3},
  [638] = {.lex_state = 14, .external_lex_state = 3},
  [639] = {.lex_state = 14, .external_lex_state = 3},
  [640] = {.lex_state = 10, .external_lex_state = 3},
  [641] = {.lex_state = 10, .external_lex_state = 3},
  [642] = {.lex_state = 10, .external_lex_state = 3},
  [643] = {.lex_state = 14, .external_lex_state = 3},
  [644] = {.lex_state = 14, .external_lex_state = 3},
  [645] = {.lex_state = 10, .external_lex_state = 3},
  [646] = {.lex_state = 10, .external_lex_state = 3},
  [647] = {.lex_state = 10, .external_lex_state = 3},
  [648] = {.lex_state = 23, .external_lex_state = 2},
  [649] = {.lex_state = 23, .external_lex_state = 2},
  [650] = {.lex_state = 23, .external_lex_state = 2},
  [651] = {.lex_state = 23, .external_lex_state = 2},
  [652] = {.lex_state = 23, .external_lex_state = 2},
  [653] = {.lex_state = 24, .external_lex_state = 2},
  [654] = {.lex_state = 23, .external_lex_state = 2},
  [655] = {.lex_state = 23, .external_lex_state = 2},
  [656] = {.lex_state = 23, .external_lex_state = 2},
  [657] = {.lex_state = 23, .external_lex_state = 2},
  [658] = {.lex_state = 23, .external_lex_state = 2},
  [659] = {.lex_state = 23, .external_lex_state = 2},
  [660] = {.lex_state = 23, .external_lex_state = 2},
  [661] = {.lex_state = 23, .external_lex_state = 2},
  [662] = {.lex_state = 23, .external_lex_state = 2},
  [663] = {.lex_state = 23, .external_lex_state = 2},
  [664] = {.lex_state = 23, .external_lex_state = 2},
  [665] = {.lex_state = 23, .external_lex_state = 2},
  [666] = {.lex_state = 23, .external_lex_state = 2},
  [667] = {.lex_state = 23, .external_lex_state = 2},
  [668] = {.lex_state = 23, .external_lex_state = 2},
  [669] = {.lex_state = 23, .external_lex_state = 2},
  [670] = {.lex_state = 23, .external_lex_state = 2},
  [671] = {.lex_state = 23, .external_lex_state = 2},
  [672] = {.lex_state = 23, .external_lex_state = 2},
  [673] = {.lex_state = 23, .external_lex_state = 2},
  [674] = {.lex_state = 23, .external_lex_state = 2},
  [675] = {.lex_state = 23, .external_lex_state = 2},
  [676] = {.lex_state = 23, .external_lex_state = 2},
  [677] = {.lex_state = 23, .external_lex_state = 2},
  [678] = {.lex_state = 23, .external_lex_state = 2},
  [679] = {.lex_state = 23, .external_lex_state = 2},
  [680] = {.lex_state = 23, .external_lex_state = 2},
  [681] = {.lex_state = 23, .external_lex_state = 2},
  [682] = {.lex_state = 23, .external_lex_state = 2},
  [683] = {.lex_state = 23, .external_lex_state = 2},
  [684] = {.lex_state = 24, .external_lex_state = 2},
  [685] = {.lex_state = 24, .external_lex_state = 2},
  [686] = {.lex_state = 23, .external_lex_state = 2},
  [687] = {.lex_state = 23, .external_lex_state = 2},
  [688] = {.lex_state = 23, .external_lex_state = 2},
  [689] = {.lex_state = 23, .external_lex_state = 2},
  [690] = {.lex_state = 23, .external_lex_state = 2},
  [691] = {.lex_state = 23, .external_lex_state = 2},
  [692] = {.lex_state = 23, .external_lex_state = 2},
  [693] = {.lex_state = 23, .external_lex_state = 2},
  [694] = {.lex_state = 23, .external_lex_state = 2},
  [695] = {.lex_state = 24, .external_lex_state = 2},
  [696] = {.lex_state = 24, .external_lex_state = 2},
  [697] = {.lex_state = 24, .external_lex_state = 2},
  [698] = {.lex_state = 23, .external_lex_state = 2},
  [699] = {.lex_state = 23, .external_lex_state = 2},
  [700] = {.lex_state = 24, .external_lex_state = 2},
  [701] = {.lex_state = 23, .external_lex_state = 2},
  [702] = {.lex_state = 23, .external_lex_state = 2},
  [703] = {.lex_state = 23, .external_lex_state = 2},
  [704] = {.lex_state = 23, .external_lex_state = 2},
  [705] = {.lex_state = 23, .external_lex_state = 2},
  [706] = {.lex_state = 24, .external_lex_state = 2},
  [707] = {.lex_state = 23, .external_lex_state = 2},
  [708] = {.lex_state = 24, .external_lex_state = 2},
  [709] = {.lex_state = 23, .external_lex_state = 2},
  [710] = {.lex_state = 23, .external_lex_state = 2},
  [711] = {.lex_state = 23, .external_lex_state = 2},
  [712] = {.lex_state = 23, .external_lex_state = 2},
  [713] = {.lex_state = 23, .external_lex_state = 2},
  [714] = {.lex_state = 23, .external_lex_state = 2},
  [715] = {.lex_state = 23, .external_lex_state = 2},
  [716] = {.lex_state = 23, .external_lex_state = 2},
  [717] = {.lex_state = 23, .external_lex_state = 2},
  [718] = {.lex_state = 23, .external_lex_state = 2},
  [719] = {.lex_state = 23, .external_lex_state = 2},
  [720] = {.lex_state = 23, .external_lex_state = 2},
  [721] = {.lex_state = 23, .external_lex_state = 2},
  [722] = {.lex_state = 23, .external_lex_state = 2},
  [723] = {.lex_state = 23, .external_lex_state = 2},
  [724] = {.lex_state = 23, .external_lex_state = 2},
  [725] = {.lex_state = 23, .external_lex_state = 2},
  [726] = {.lex_state = 23, .external_lex_state = 2},
  [727] = {.lex_state = 23, .external_lex_state = 2},
  [728] = {.lex_state = 23, .external_lex_state = 2},
  [729] = {.lex_state = 23, .external_lex_state = 2},
  [730] = {.lex_state = 24, .external_lex_state = 2},
  [731] = {.lex_state = 23, .external_lex_state = 2},
  [732] = {.lex_state = 23, .external_lex_state = 2},
  [733] = {.lex_state = 23, .external_lex_state = 2},
  [734] = {.lex_state = 23, .external_lex_state = 2},
  [735] = {.lex_state = 23, .external_lex_state = 2},
  [736] = {.lex_state = 23, .external_lex_state = 2},
  [737] = {.lex_state = 24, .external_lex_state = 2},
  [738] = {.lex_state = 23, .external_lex_state = 2},
  [739] = {.lex_state = 23, .external_lex_state = 2},
  [740] = {.lex_state = 23, .external_lex_state = 2},
  [741] = {.lex_state = 23, .external_lex_state = 2},
  [742] = {.lex_state = 23, .external_lex_state = 2},
  [743] = {.lex_state = 15, .external_lex_state = 5},
  [744] = {.lex_state = 23, .external_lex_state = 2},
  [745] = {.lex_state = 24, .external_lex_state = 2},
  [746] = {.lex_state = 23, .external_lex_state = 2},
  [747] = {.lex_state = 23, .external_lex_state = 2},
  [748] = {.lex_state = 23, .external_lex_state = 2},
  [749] = {.lex_state = 23, .external_lex_state = 2},
  [750] = {.lex_state = 23, .external_lex_state = 2},
  [751] = {.lex_state = 23, .external_lex_state = 2},
  [752] = {.lex_state = 23, .external_lex_state = 2},
  [753] = {.lex_state = 15, .external_lex_state = 5},
  [754] = {.lex_state = 56, .external_lex_state = 4},
  [755] = {.lex_state = 11, .external_lex_state = 4},
  [756] = {.lex_state = 11, .external_lex_state = 4},
  [757] = {.lex_state = 11, .external_lex_state = 4},
  [758] = {.lex_state = 11, .external_lex_state = 4},
  [759] = {.lex_state = 11, .external_lex_state = 4},
  [760] = {.lex_state = 11, .external_lex_state = 4},
  [761] = {.lex_state = 11, .external_lex_state = 4},
  [762] = {.lex_state = 11, .external_lex_state = 4},
  [763] = {.lex_state = 11, .external_lex_state = 4},
  [764] = {.lex_state = 11, .external_lex_state = 4},
  [765] = {.lex_state = 11, .external_lex_state = 4},
  [766] = {.lex_state = 11, .external_lex_state = 4},
  [767] = {.lex_state = 11, .external_lex_state = 4},
  [768] = {.lex_state = 11, .external_lex_state = 4},
  [769] = {.lex_state = 11, .external_lex_state = 4},
  [770] = {.lex_state = 11, .external_lex_state = 4},
  [771] = {.lex_state = 11, .external_lex_state = 4},
  [772] = {.lex_state = 4, .external_lex_state = 4},
  [773] = {.lex_state = 23, .external_lex_state = 2},
  [774] = {.lex_state = 23, .external_lex_state = 2},
  [775] = {.lex_state = 23, .external_lex_state = 2},
  [776] = {.lex_state = 23, .external_lex_state = 2},
  [777] = {.lex_state = 23, .external_lex_state = 2},
  [778] = {.lex_state = 23, .external_lex_state = 2},
  [779] = {.lex_state = 12, .external_lex_state = 4},
  [780] = {.lex_state = 23, .external_lex_state = 2},
  [781] = {.lex_state = 23, .external_lex_state = 2},
  [782] = {.lex_state = 23, .external_lex_state = 2},
  [783] = {.lex_state = 13, .external_lex_state = 4},
  [784] = {.lex_state = 13, .external_lex_state = 4},
  [785] = {.lex_state = 23, .external_lex_state = 2},
  [786] = {.lex_state = 23, .external_lex_state = 2},
  [787] = {.lex_state = 23, .external_lex_state = 2},
  [788] = {.lex_state = 23, .external_lex_state = 2},
  [789] = {.lex_state = 23, .external_lex_state = 2},
  [790] = {.lex_state = 12, .external_lex_state = 4},
  [791] = {.lex_state = 23, .external_lex_state = 1},
  [792] = {.lex_state = 12, .external_lex_state = 4},
  [793] = {.lex_state = 23, .external_lex_state = 1},
  [794] = {.lex_state = 23, .external_lex_state = 2},
  [795] = {.lex_state = 23, .external_lex_state = 2},
  [796] = {.lex_state = 12, .external_lex_state = 4},
  [797] = {.lex_state = 23, .external_lex_state = 2},
  [798] = {.lex_state = 23, .external_lex_state = 1},
  [799] = {.lex_state = 23, .external_lex_state = 2},
  [800] = {.lex_state = 23, .external_lex_state = 1},
  [801] = {.lex_state = 23, .external_lex_state = 2},
  [802] = {.lex_state = 12, .external_lex_state = 4},
  [803] = {.lex_state = 12, .external_lex_state = 4},
  [804] = {.lex_state = 12, .external_lex_state = 4},
  [805] = {.lex_state = 23, .external_lex_state = 2},
  [806] = {.lex_state = 12, .external_lex_state = 4},
  [807] = {.lex_state = 12, .external_lex_state = 4},
  [808] = {.lex_state = 12, .external_lex_state = 4},
  [809] = {.lex_state = 12, .external_lex_state = 4},
  [810] = {.lex_state = 23, .external_lex_state = 1},
  [811] = {.lex_state = 23, .external_lex_state = 2},
  [812] = {.lex_state = 12, .external_lex_state = 4},
  [813] = {.lex_state = 23, .external_lex_state = 2},
  [814] = {.lex_state = 23, .external_lex_state = 2},
  [815] = {.lex_state = 12, .external_lex_state = 4},
  [816] = {.lex_state = 23, .external_lex_state = 1},
  [817] = {.lex_state = 23, .external_lex_state = 2},
  [818] = {.lex_state = 23, .external_lex_state = 2},
  [819] = {.lex_state = 23, .external_lex_state = 2},
  [820] = {.lex_state = 23, .external_lex_state = 2},
  [821] = {.lex_state = 12, .external_lex_state = 4},
  [822] = {.lex_state = 23, .external_lex_state = 2},
  [823] = {.lex_state = 23, .external_lex_state = 2},
  [824] = {.lex_state = 23, .external_lex_state = 2},
  [825] = {.lex_state = 23, .external_lex_state = 2},
  [826] = {.lex_state = 23, .external_lex_state = 2},
  [827] = {.lex_state = 23, .external_lex_state = 2},
  [828] = {.lex_state = 23, .external_lex_state = 2},
  [829] = {.lex_state = 23, .external_lex_state = 2},
  [830] = {.lex_state = 23, .external_lex_state = 2},
  [831] = {.lex_state = 23, .external_lex_state = 2},
  [832] = {.lex_state = 23, .external_lex_state = 2},
  [833] = {.lex_state = 23, .external_lex_state = 2},
  [834] = {.lex_state = 23, .external_lex_state = 2},
  [835] = {.lex_state = 23, .external_lex_state = 2},
  [836] = {.lex_state = 10, .external_lex_state = 4},
  [837] = {.lex_state = 10, .external_lex_state = 4},
  [838] = {.lex_state = 10, .external_lex_state = 4},
  [839] = {.lex_state = 10, .external_lex_state = 4},
  [840] = {.lex_state = 10, .external_lex_state = 4},
  [841] = {.lex_state = 10, .external_lex_state = 4},
  [842] = {.lex_state = 10, .external_lex_state = 4},
  [843] = {.lex_state = 10, .external_lex_state = 4},
  [844] = {.lex_state = 10, .external_lex_state = 4},
  [845] = {.lex_state = 10, .external_lex_state = 4},
  [846] = {.lex_state = 10, .external_lex_state = 4},
  [847] = {.lex_state = 53, .external_lex_state = 4},
  [848] = {.lex_state = 53, .external_lex_state = 4},
  [849] = {.lex_state = 58, .external_lex_state = 4},
  [850] = {.lex_state = 53, .external_lex_state = 4},
  [851] = {.lex_state = 54, .external_lex_state = 4},
  [852] = {.lex_state = 10, .external_lex_state = 4},
  [853] = {.lex_state = 23, .external_lex_state = 2},
  [854] = {.lex_state = 53, .external_lex_state = 4},
  [855] = {.lex_state = 53, .external_lex_state = 4},
  [856] = {.lex_state = 59, .external_lex_state = 4},
  [857] = {.lex_state = 58, .external_lex_state = 4},
  [858] = {.lex_state = 14, .external_lex_state = 4},
  [859] = {.lex_state = 54, .external_lex_state = 4},
  [860] = {.lex_state = 60, .external_lex_state = 4},
  [861] = {.lex_state = 54, .external_lex_state = 4},
  [862] = {.lex_state = 57, .external_lex_state = 6},
  [863] = {.lex_state = 54, .external_lex_state = 4},
  [864] = {.lex_state = 53, .external_lex_state = 4},
  [865] = {.lex_state = 14, .external_lex_state = 4},
  [866] = {.lex_state = 59, .external_lex_state = 4},
  [867] = {.lex_state = 53, .external_lex_state = 4},
  [868] = {.lex_state = 59, .external_lex_state = 4},
  [869] = {.lex_state = 10, .external_lex_state = 4},
  [870] = {.lex_state = 10, .external_lex_state = 4},
  [871] = {.lex_state = 53, .external_lex_state = 4},
  [872] = {.lex_state = 60, .external_lex_state = 4},
  [873] = {.lex_state = 58, .external_lex_state = 4},
  [874] = {.lex_state = 53, .external_lex_state = 4},
  [875] = {.lex_state = 54, .external_lex_state = 4},
  [876] = {.lex_state = 54, .external_lex_state = 4},
  [877] = {.lex_state = 53, .external_lex_state = 4},
  [878] = {.lex_state = 61, .external_lex_state = 4},
  [879] = {.lex_state = 61, .external_lex_state = 4},
  [880] = {.lex_state = 61, .external_lex_state = 4},
  [881] = {.lex_state = 14, .external_lex_state = 4},
  [882] = {.lex_state = 60, .external_lex_state = 4},
  [883] = {.lex_state = 53, .external_lex_state = 4},
  [884] = {.lex_state = 53, .external_lex_state = 4},
  [885] = {.lex_state = 10, .external_lex_state = 5},
  [886] = {.lex_state = 51, .external_lex_state = 7},
  [887] = {.lex_state = 10, .external_lex_state = 4},
  [888] = {.lex_state = 10, .external_lex_state = 5},
  [889] = {.lex_state = 10, .external_lex_state = 4},
  [890] = {.lex_state = 10, .external_lex_state = 4},
  [891] = {.lex_state = 13, .external_lex_state = 4},
  [892] = {.lex_state = 10, .external_lex_state = 4},
  [893] = {.lex_state = 10, .external_lex_state = 4},
  [894] = {.lex_state = 10, .external_lex_state = 5},
  [895] = {.lex_state = 10, .external_lex_state = 4},
  [896] = {.lex_state = 10, .external_lex_state = 4},
  [897] = {.lex_state = 10, .external_lex_state = 4},
  [898] = {.lex_state = 13, .external_lex_state = 4},
  [899] = {.lex_state = 51, .external_lex_state = 7},
  [900] = {.lex_state = 13, .external_lex_state = 4},
  [901] = {.lex_state = 13, .external_lex_state = 4},
  [902] = {.lex_state = 13, .external_lex_state = 4},
  [903] = {.lex_state = 13, .external_lex_state = 4},
  [904] = {.lex_state = 13, .external_lex_state = 4},
  [905] = {.lex_state = 13, .external_lex_state = 4},
  [906] = {.lex_state = 13, .external_lex_state = 4},
  [907] = {.lex_state = 13, .external_lex_state = 4},
  [908] = {.lex_state = 10, .external_lex_state = 5},
  [909] = {.lex_state = 10, .external_lex_state = 4},
  [910] = {.lex_state = 10, .external_lex_state = 4},
  [911] = {.lex_state = 10, .external_lex_state = 4},
  [912] = {.lex_state = 54, .external_lex_state = 4},
  [913] = {.lex_state = 10, .external_lex_state = 4},
  [914] = {.lex_state = 10, .external_lex_state = 5},
  [915] = {.lex_state = 10, .external_lex_state = 4},
  [916] = {.lex_state = 10, .external_lex_state = 4},
  [917] = {.lex_state = 10, .external_lex_state = 5},
  [918] = {.lex_state = 13, .external_lex_state = 4},
  [919] = {.lex_state = 13, .external_lex_state = 4},
  [920] = {.lex_state = 10, .external_lex_state = 5},
  [921] = {.lex_state = 53, .external_lex_state = 5},
  [922] = {.lex_state = 52, .external_lex_state = 4},
  [923] = {.lex_state = 53, .external_lex_state = 4},
  [924] = {.lex_state = 10, .external_lex_state = 4},
  [925] = {.lex_state = 53, .external_lex_state = 5},
  [926] = {.lex_state = 10, .external_lex_state = 4},
  [927] = {.lex_state = 53, .external_lex_state = 5},
  [928] = {.lex_state = 10, .external_lex_state = 4},
  [929] = {.lex_state = 53, .external_lex_state = 5},
  [930] = {.lex_state = 53, .external_lex_state = 4},
  [931] = {.lex_state = 10, .external_lex_state = 4},
  [932] = {.lex_state = 57, .external_lex_state = 5},
  [933] = {.lex_state = 53, .external_lex_state = 5},
  [934] = {.lex_state = 53, .external_lex_state = 5},
  [935] = {.lex_state = 52, .external_lex_state = 4},
  [936] = {.lex_state = 54, .external_lex_state = 4},
  [937] = {.lex_state = 53, .external_lex_state = 5},
  [938] = {.lex_state = 10, .external_lex_state = 4},
  [939] = {.lex_state = 53, .external_lex_state = 4},
  [940] = {.lex_state = 52, .external_lex_state = 4},
  [941] = {.lex_state = 54, .external_lex_state = 4},
  [942] = {.lex_state = 53, .external_lex_state = 5},
  [943] = {.lex_state = 53, .external_lex_state = 5},
  [944] = {.lex_state = 10, .external_lex_state = 4},
  [945] = {.lex_state = 10, .external_lex_state = 5},
  [946] = {.lex_state = 53, .external_lex_state = 5},
  [947] = {.lex_state = 10, .external_lex_state = 4},
  [948] = {.lex_state = 10, .external_lex_state = 4},
  [949] = {.lex_state = 10, .external_lex_state = 4},
  [950] = {.lex_state = 10, .external_lex_state = 5},
  [951] = {.lex_state = 10, .external_lex_state = 4},
  [952] = {.lex_state = 10, .external_lex_state = 5},
  [953] = {.lex_state = 10, .external_lex_state = 4},
  [954] = {.lex_state = 52, .external_lex_state = 4},
  [955] = {.lex_state = 53, .external_lex_state = 4},
  [956] = {.lex_state = 53, .external_lex_state = 4},
  [957] = {.lex_state = 53, .external_lex_state = 4},
  [958] = {.lex_state = 53, .external_lex_state = 4},
  [959] = {.lex_state = 10, .external_lex_state = 4},
  [960] = {.lex_state = 10, .external_lex_state = 5},
  [961] = {.lex_state = 10, .external_lex_state = 4},
  [962] = {.lex_state = 10, .external_lex_state = 5},
  [963] = {.lex_state = 52, .external_lex_state = 4},
  [964] = {.lex_state = 10, .external_lex_state = 4},
  [965] = {.lex_state = 54, .external_lex_state = 4},
  [966] = {.lex_state = 54, .external_lex_state = 4},
  [967] = {.lex_state = 10, .external_lex_state = 5},
  [968] = {.lex_state = 10, .external_lex_state = 5},
  [969] = {.lex_state = 53, .external_lex_state = 5},
  [970] = {.lex_state = 10, .external_lex_state = 5},
  [971] = {.lex_state = 10, .external_lex_state = 5},
  [972] = {.lex_state = 10, .external_lex_state = 5},
  [973] = {.lex_state = 10, .external_lex_state = 5},
  [974] = {.lex_state = 10, .external_lex_state = 5},
  [975] = {.lex_state = 10, .external_lex_state = 4},
  [976] = {.lex_state = 10, .external_lex_state = 4},
  [977] = {.lex_state = 10, .external_lex_state = 4},
  [978] = {.lex_state = 10, .external_lex_state = 5},
  [979] = {.lex_state = 53, .external_lex_state = 5},
  [980] = {.lex_state = 52, .external_lex_state = 4},
  [981] = {.lex_state = 53, .external_lex_state = 4},
  [982] = {.lex_state = 10, .external_lex_state = 4},
  [983] = {.lex_state = 10, .external_lex_state = 4},
  [984] = {.lex_state = 53, .external_lex_state = 4},
  [985] = {.lex_state = 10, .external_lex_state = 4},
  [986] = {.lex_state = 53, .external_lex_state = 4},
  [987] = {.lex_state = 10, .external_lex_state = 4},
  [988] = {.lex_state = 53, .external_lex_state = 4},
  [989] = {.lex_state = 53, .external_lex_state = 4},
  [990] = {.lex_state = 53, .external_lex_state = 4},
  [991] = {.lex_state = 53, .external_lex_state = 5},
  [992] = {.lex_state = 10, .external_lex_state = 4},
  [993] = {.lex_state = 53, .external_lex_state = 5},
  [994] = {.lex_state = 52, .external_lex_state = 4},
  [995] = {.lex_state = 10, .external_lex_state = 5},
  [996] = {.lex_state = 53, .external_lex_state = 5},
  [997] = {.lex_state = 53, .external_lex_state = 5},
  [998] = {.lex_state = 53, .external_lex_state = 4},
  [999] = {.lex_state = 53, .external_lex_state = 5},
  [1000] = {.lex_state = 10, .external_lex_state = 5},
  [1001] = {.lex_state = 10, .external_lex_state = 5},
  [1002] = {.lex_state = 10, .external_lex_state = 4},
  [1003] = {.lex_state = 10, .external_lex_state = 4},
  [1004] = {.lex_state = 10, .external_lex_state = 5},
  [1005] = {.lex_state = 10, .external_lex_state = 4},
  [1006] = {.lex_state = 11, .external_lex_state = 4},
  [1007] = {.lex_state = 10, .external_lex_state = 5},
  [1008] = {.lex_state = 10, .external_lex_state = 4},
  [1009] = {.lex_state = 10, .external_lex_state = 5},
  [1010] = {.lex_state = 10, .external_lex_state = 5},
  [1011] = {.lex_state = 10, .external_lex_state = 5},
  [1012] = {.lex_state = 10, .external_lex_state = 4},
  [1013] = {.lex_state = 57, .external_lex_state = 5},
  [1014] = {.lex_state = 57, .external_lex_state = 5},
  [1015] = {.lex_state = 10, .external_lex_state = 4},
  [1016] = {.lex_state = 10, .external_lex_state = 5},
  [1017] = {.lex_state = 10, .external_lex_state = 5},
  [1018] = {.lex_state = 13, .external_lex_state = 4},
  [1019] = {.lex_state = 10, .external_lex_state = 5},
  [1020] = {.lex_state = 10, .external_lex_state = 4},
  [1021] = {.lex_state = 10, .external_lex_state = 5},
  [1022] = {.lex_state = 53, .external_lex_state = 4},
  [1023] = {.lex_state = 10, .external_lex_state = 5},
  [1024] = {.lex_state = 11, .external_lex_state = 4},
  [1025] = {.lex_state = 10, .external_lex_state = 4},
  [1026] = {.lex_state = 10, .external_lex_state = 4},
  [1027] = {.lex_state = 10, .external_lex_state = 4},
  [1028] = {.lex_state = 10, .external_lex_state = 4},
  [1029] = {.lex_state = 13, .external_lex_state = 4},
  [1030] = {.lex_state = 10, .external_lex_state = 4},
  [1031] = {.lex_state = 10, .external_lex_state = 4},
  [1032] = {.lex_state = 10, .external_lex_state = 4},
  [1033] = {.lex_state = 11, .external_lex_state = 4},
  [1034] = {.lex_state = 13, .external_lex_state = 4},
  [1035] = {.lex_state = 10, .external_lex_state = 4},
  [1036] = {.lex_state = 13, .external_lex_state = 4},
  [1037] = {.lex_state = 11, .external_lex_state = 4},
  [1038] = {.lex_state = 10, .external_lex_state = 5},
  [1039] = {.lex_state = 10, .external_lex_state = 5},
  [1040] = {.lex_state = 52, .external_lex_state = 4},
  [1041] = {.lex_state = 10, .external_lex_state = 4},
  [1042] = {.lex_state = 10, .external_lex_state = 5},
  [1043] = {.lex_state = 11, .external_lex_state = 4},
  [1044] = {.lex_state = 11, .external_lex_state = 4},
  [1045] = {.lex_state = 14, .external_lex_state = 4},
  [1046] = {.lex_state = 10, .external_lex_state = 4},
  [1047] = {.lex_state = 10, .external_lex_state = 4},
  [1048] = {.lex_state = 11, .external_lex_state = 4},
  [1049] = {.lex_state = 11, .external_lex_state = 4},
  [1050] = {.lex_state = 11, .external_lex_state = 4},
  [1051] = {.lex_state = 10, .external_lex_state = 4},
  [1052] = {.lex_state = 10, .external_lex_state = 4},
  [1053] = {.lex_state = 10, .external_lex_state = 5},
  [1054] = {.lex_state = 10, .external_lex_state = 4},
  [1055] = {.lex_state = 11, .external_lex_state = 4},
  [1056] = {.lex_state = 11, .external_lex_state = 4},
  [1057] = {.lex_state = 10, .external_lex_state = 4},
  [1058] = {.lex_state = 11, .external_lex_state = 4},
  [1059] = {.lex_state = 11, .external_lex_state = 4},
  [1060] = {.lex_state = 11, .external_lex_state = 4},
  [1061] = {.lex_state = 11, .external_lex_state = 4},
  [1062] = {.lex_state = 11, .external_lex_state = 4},
  [1063] = {.lex_state = 11, .external_lex_state = 4},
  [1064] = {.lex_state = 10, .external_lex_state = 4},
  [1065] = {.lex_state = 52, .external_lex_state = 4},
  [1066] = {.lex_state = 10, .external_lex_state = 5},
  [1067] = {.lex_state = 11, .external_lex_state = 4},
  [1068] = {.lex_state = 11, .external_lex_state = 4},
  [1069] = {.lex_state = 11, .external_lex_state = 4},
  [1070] = {.lex_state = 11, .external_lex_state = 4},
  [1071] = {.lex_state = 10, .external_lex_state = 4},
  [1072] = {.lex_state = 11, .external_lex_state = 4},
  [1073] = {.lex_state = 10, .external_lex_state = 4},
  [1074] = {.lex_state = 11, .external_lex_state = 4},
  [1075] = {.lex_state = 11, .external_lex_state = 4},
  [1076] = {.lex_state = 10, .external_lex_state = 4},
  [1077] = {.lex_state = 10, .external_lex_state = 5},
  [1078] = {.lex_state = 52, .external_lex_state = 4},
  [1079] = {.lex_state = 10, .external_lex_state = 5},
  [1080] = {.lex_state = 11, .external_lex_state = 4},
  [1081] = {.lex_state = 52, .external_lex_state = 4},
  [1082] = {.lex_state = 10, .external_lex_state = 4},
  [1083] = {.lex_state = 11, .external_lex_state = 4},
  [1084] = {.lex_state = 10, .external_lex_state = 4},
  [1085] = {.lex_state = 11, .external_lex_state = 4},
  [1086] = {.lex_state = 10, .external_lex_state = 4},
  [1087] = {.lex_state = 11, .external_lex_state = 4},
  [1088] = {.lex_state = 10, .external_lex_state = 4},
  [1089] = {.lex_state = 11, .external_lex_state = 4},
  [1090] = {.lex_state = 10, .external_lex_state = 4},
  [1091] = {.lex_state = 11, .external_lex_state = 4},
  [1092] = {.lex_state = 11, .external_lex_state = 4},
  [1093] = {.lex_state = 11, .external_lex_state = 4},
  [1094] = {.lex_state = 10, .external_lex_state = 4},
  [1095] = {.lex_state = 11, .external_lex_state = 4},
  [1096] = {.lex_state = 53, .external_lex_state = 4},
  [1097] = {.lex_state = 10, .external_lex_state = 4},
  [1098] = {.lex_state = 52, .external_lex_state = 4},
  [1099] = {.lex_state = 53, .external_lex_state = 4},
  [1100] = {.lex_state = 11, .external_lex_state = 4},
  [1101] = {.lex_state = 10, .external_lex_state = 5},
  [1102] = {.lex_state = 10, .external_lex_state = 4},
  [1103] = {.lex_state = 52, .external_lex_state = 4},
  [1104] = {.lex_state = 10, .external_lex_state = 4},
  [1105] = {.lex_state = 10, .external_lex_state = 5},
  [1106] = {.lex_state = 53, .external_lex_state = 4},
  [1107] = {.lex_state = 10, .external_lex_state = 4},
  [1108] = {.lex_state = 52, .external_lex_state = 4},
  [1109] = {.lex_state = 53, .external_lex_state = 4},
  [1110] = {.lex_state = 10, .external_lex_state = 4},
  [1111] = {.lex_state = 11, .external_lex_state = 4},
  [1112] = {.lex_state = 11, .external_lex_state = 4},
  [1113] = {.lex_state = 11, .external_lex_state = 4},
  [1114] = {.lex_state = 11, .external_lex_state = 4},
  [1115] = {.lex_state = 11, .external_lex_state = 4},
  [1116] = {.lex_state = 11, .external_lex_state = 4},
  [1117] = {.lex_state = 10, .external_lex_state = 4},
  [1118] = {.lex_state = 10, .external_lex_state = 4},
  [1119] = {.lex_state = 10, .external_lex_state = 4},
  [1120] = {.lex_state = 53, .external_lex_state = 4},
  [1121] = {.lex_state = 12, .external_lex_state = 4},
  [1122] = {.lex_state = 10, .external_lex_state = 5},
  [1123] = {.lex_state = 10, .external_lex_state = 5},
  [1124] = {.lex_state = 10, .external_lex_state = 4},
  [1125] = {.lex_state = 53, .external_lex_state = 4},
  [1126] = {.lex_state = 10, .external_lex_state = 4},
  [1127] = {.lex_state = 10, .external_lex_state = 4},
  [1128] = {.lex_state = 10, .external_lex_state = 4},
  [1129] = {.lex_state = 10, .external_lex_state = 4},
  [1130] = {.lex_state = 10, .external_lex_state = 4},
  [1131] = {.lex_state = 10, .external_lex_state = 4},
  [1132] = {.lex_state = 10, .external_lex_state = 4},
  [1133] = {.lex_state = 10, .external_lex_state = 4},
  [1134] = {.lex_state = 11, .external_lex_state = 4},
  [1135] = {.lex_state = 10, .external_lex_state = 4},
  [1136] = {.lex_state = 11, .external_lex_state = 4},
  [1137] = {.lex_state = 11, .external_lex_state = 4},
  [1138] = {.lex_state = 11, .external_lex_state = 4},
  [1139] = {.lex_state = 10, .external_lex_state = 5},
  [1140] = {.lex_state = 10, .external_lex_state = 5},
  [1141] = {.lex_state = 10, .external_lex_state = 5},
  [1142] = {.lex_state = 10, .external_lex_state = 5},
  [1143] = {.lex_state = 10, .external_lex_state = 5},
  [1144] = {.lex_state = 10, .external_lex_state = 5},
  [1145] = {.lex_state = 11, .external_lex_state = 4},
  [1146] = {.lex_state = 10, .external_lex_state = 5},
  [1147] = {.lex_state = 10, .external_lex_state = 5},
  [1148] = {.lex_state = 10, .external_lex_state = 4},
  [1149] = {.lex_state = 10, .external_lex_state = 5},
  [1150] = {.lex_state = 11, .external_lex_state = 4},
  [1151] = {.lex_state = 10, .external_lex_state = 4},
  [1152] = {.lex_state = 11, .external_lex_state = 4},
  [1153] = {.lex_state = 10, .external_lex_state = 4},
  [1154] = {.lex_state = 10, .external_lex_state = 5},
  [1155] = {.lex_state = 10, .external_lex_state = 5},
  [1156] = {.lex_state = 10, .external_lex_state = 5},
  [1157] = {.lex_state = 10, .external_lex_state = 5},
  [1158] = {.lex_state = 10, .external_lex_state = 4},
  [1159] = {.lex_state = 11, .external_lex_state = 4},
  [1160] = {.lex_state = 11, .external_lex_state = 4},
  [1161] = {.lex_state = 51, .external_lex_state = 4},
  [1162] = {.lex_state = 52, .external_lex_state = 4},
  [1163] = {.lex_state = 10, .external_lex_state = 4},
  [1164] = {.lex_state = 10, .external_lex_state = 5},
  [1165] = {.lex_state = 51, .external_lex_state = 4},
  [1166] = {.lex_state = 5, .external_lex_state = 5},
  [1167] = {.lex_state = 11, .external_lex_state = 4},
  [1168] = {.lex_state = 10, .external_lex_state = 4},
  [1169] = {.lex_state = 10, .external_lex_state = 4},
  [1170] = {.lex_state = 53, .external_lex_state = 4},
  [1171] = {.lex_state = 11, .external_lex_state = 4},
  [1172] = {.lex_state = 11, .external_lex_state = 4},
  [1173] = {.lex_state = 10, .external_lex_state = 4},
  [1174] = {.lex_state = 10, .external_lex_state = 4},
  [1175] = {.lex_state = 10, .external_lex_state = 4},
  [1176] = {.lex_state = 52, .external_lex_state = 4},
  [1177] = {.lex_state = 11, .external_lex_state = 4},
  [1178] = {.lex_state = 11, .external_lex_state = 4},
  [1179] = {.lex_state = 11, .external_lex_state = 4},
  [1180] = {.lex_state = 10, .external_lex_state = 5},
  [1181] = {.lex_state = 15, .external_lex_state = 5},
  [1182] = {.lex_state = 10, .external_lex_state = 4},
  [1183] = {.lex_state = 11, .external_lex_state = 4},
  [1184] = {.lex_state = 11, .external_lex_state = 4},
  [1185] = {.lex_state = 11, .external_lex_state = 4},
  [1186] = {.lex_state = 11, .external_lex_state = 4},
  [1187] = {.lex_state = 53, .external_lex_state = 4},
  [1188] = {.lex_state = 52, .external_lex_state = 4},
  [1189] = {.lex_state = 10, .external_lex_state = 4},
  [1190] = {.lex_state = 10, .external_lex_state = 4},
  [1191] = {.lex_state = 11, .external_lex_state = 4},
  [1192] = {.lex_state = 11, .external_lex_state = 4},
  [1193] = {.lex_state = 11, .external_lex_state = 4},
  [1194] = {.lex_state = 11, .external_lex_state = 4},
  [1195] = {.lex_state = 11, .external_lex_state = 4},
  [1196] = {.lex_state = 11, .external_lex_state = 4},
  [1197] = {.lex_state = 11, .external_lex_state = 4},
  [1198] = {.lex_state = 52, .external_lex_state = 4},
  [1199] = {.lex_state = 11, .external_lex_state = 4},
  [1200] = {.lex_state = 10, .external_lex_state = 4},
  [1201] = {.lex_state = 11, .external_lex_state = 4},
  [1202] = {.lex_state = 10, .external_lex_state = 4},
  [1203] = {.lex_state = 52, .external_lex_state = 4},
  [1204] = {.lex_state = 10, .external_lex_state = 4},
  [1205] = {.lex_state = 52, .external_lex_state = 4},
  [1206] = {.lex_state = 11, .external_lex_state = 4},
  [1207] = {.lex_state = 52, .external_lex_state = 4},
  [1208] = {.lex_state = 53, .external_lex_state = 4},
  [1209] = {.lex_state = 2, .external_lex_state = 5},
  [1210] = {.lex_state = 10, .external_lex_state = 5},
  [1211] = {.lex_state = 52, .external_lex_state = 4},
  [1212] = {.lex_state = 10, .external_lex_state = 4},
  [1213] = {.lex_state = 10, .external_lex_state = 4},
  [1214] = {.lex_state = 15, .external_lex_state = 5},
  [1215] = {.lex_state = 52, .external_lex_state = 4},
  [1216] = {.lex_state = 10, .external_lex_state = 4},
  [1217] = {.lex_state = 52, .external_lex_state = 4},
  [1218] = {.lex_state = 52, .external_lex_state = 4},
  [1219] = {.lex_state = 52, .external_lex_state = 4},
  [1220] = {.lex_state = 2, .external_lex_state = 5},
  [1221] = {.lex_state = 52, .external_lex_state = 4},
  [1222] = {.lex_state = 53, .external_lex_state = 4},
  [1223] = {.lex_state = 52, .external_lex_state = 4},
  [1224] = {.lex_state = 52, .external_lex_state = 4},
  [1225] = {.lex_state = 19, .external_lex_state = 5},
  [1226] = {.lex_state = 10, .external_lex_state = 4},
  [1227] = {.lex_state = 52, .external_lex_state = 4},
  [1228] = {.lex_state = 19, .external_lex_state = 5},
  [1229] = {.lex_state = 52, .external_lex_state = 4},
  [1230] = {.lex_state = 52, .external_lex_state = 4},
  [1231] = {.lex_state = 52, .external_lex_state = 4},
  [1232] = {.lex_state = 2, .external_lex_state = 4},
  [1233] = {.lex_state = 52, .external_lex_state = 4},
  [1234] = {.lex_state = 52, .external_lex_state = 4},
  [1235] = {.lex_state = 16, .external_lex_state = 5},
  [1236] = {.lex_state = 52, .external_lex_state = 4},
  [1237] = {.lex_state = 52, .external_lex_state = 4},
  [1238] = {.lex_state = 53, .external_lex_state = 4},
  [1239] = {.lex_state = 17, .external_lex_state = 5},
  [1240] = {.lex_state = 10, .external_lex_state = 4},
  [1241] = {.lex_state = 65, .external_lex_state = 5},
  [1242] = {.lex_state = 52, .external_lex_state = 4},
  [1243] = {.lex_state = 52, .external_lex_state = 4},
  [1244] = {.lex_state = 10, .external_lex_state = 4},
  [1245] = {.lex_state = 52, .external_lex_state = 4},
  [1246] = {.lex_state = 52, .external_lex_state = 4},
  [1247] = {.lex_state = 52, .external_lex_state = 4},
  [1248] = {.lex_state = 65, .external_lex_state = 5},
  [1249] = {.lex_state = 52, .external_lex_state = 4},
  [1250] = {.lex_state = 52, .external_lex_state = 4},
  [1251] = {.lex_state = 52, .external_lex_state = 4},
  [1252] = {.lex_state = 52, .external_lex_state = 4},
  [1253] = {.lex_state = 52, .external_lex_state = 4},
  [1254] = {.lex_state = 52, .external_lex_state = 4},
  [1255] = {.lex_state = 52, .external_lex_state = 4},
  [1256] = {.lex_state = 65, .external_lex_state = 5},
  [1257] = {.lex_state = 19, .external_lex_state = 5},
  [1258] = {.lex_state = 65, .external_lex_state = 5},
  [1259] = {.lex_state = 52, .external_lex_state = 4},
  [1260] = {.lex_state = 52, .external_lex_state = 4},
  [1261] = {.lex_state = 5, .external_lex_state = 4},
  [1262] = {.lex_state = 16, .external_lex_state = 5},
  [1263] = {.lex_state = 52, .external_lex_state = 4},
  [1264] = {.lex_state = 52, .external_lex_state = 4},
  [1265] = {.lex_state = 52, .external_lex_state = 4},
  [1266] = {.lex_state = 53, .external_lex_state = 4},
  [1267] = {.lex_state = 10, .external_lex_state = 4},
  [1268] = {.lex_state = 2, .external_lex_state = 5},
  [1269] = {.lex_state = 52, .external_lex_state = 4},
  [1270] = {.lex_state = 52, .external_lex_state = 4},
  [1271] = {.lex_state = 52, .external_lex_state = 4},
  [1272] = {.lex_state = 18, .external_lex_state = 5},
  [1273] = {.lex_state = 52, .external_lex_state = 4},
  [1274] = {.lex_state = 52, .external_lex_state = 4},
  [1275] = {.lex_state = 10, .external_lex_state = 4},
  [1276] = {.lex_state = 2, .external_lex_state = 5},
  [1277] = {.lex_state = 52, .external_lex_state = 4},
  [1278] = {.lex_state = 52, .external_lex_state = 4},
  [1279] = {.lex_state = 52, .external_lex_state = 4},
  [1280] = {.lex_state = 52, .external_lex_state = 4},
  [1281] = {.lex_state = 10, .external_lex_state = 4},
  [1282] = {.lex_state = 52, .external_lex_state = 4},
  [1283] = {.lex_state = 2, .external_lex_state = 5},
  [1284] = {.lex_state = 52, .external_lex_state = 4},
  [1285] = {.lex_state = 10, .external_lex_state = 5},
  [1286] = {.lex_state = 10, .external_lex_state = 4},
  [1287] = {.lex_state = 52, .external_lex_state = 4},
  [1288] = {.lex_state = 52, .external_lex_state = 4},
  [1289] = {.lex_state = 53, .external_lex_state = 4},
  [1290] = {.lex_state = 52, .external_lex_state = 4},
  [1291] = {.lex_state = 52, .external_lex_state = 4},
  [1292] = {.lex_state = 52, .external_lex_state = 4},
  [1293] = {.lex_state = 52, .external_lex_state = 4},
  [1294] = {.lex_state = 18, .external_lex_state = 5},
  [1295] = {.lex_state = 52, .external_lex_state = 4},
  [1296] = {.lex_state = 10, .external_lex_state = 4},
  [1297] = {.lex_state = 2, .external_lex_state = 5},
  [1298] = {.lex_state = 52, .external_lex_state = 4},
  [1299] = {.lex_state = 65, .external_lex_state = 5},
  [1300] = {.lex_state = 52, .external_lex_state = 4},
  [1301] = {.lex_state = 19, .external_lex_state = 5},
  [1302] = {.lex_state = 16, .external_lex_state = 5},
  [1303] = {.lex_state = 64, .external_lex_state = 5},
  [1304] = {.lex_state = 52, .external_lex_state = 4},
  [1305] = {.lex_state = 18, .external_lex_state = 5},
  [1306] = {.lex_state = 65, .external_lex_state = 5},
  [1307] = {.lex_state = 16, .external_lex_state = 5},
  [1308] = {.lex_state = 19, .external_lex_state = 5},
  [1309] = {.lex_state = 52, .external_lex_state = 4},
  [1310] = {.lex_state = 52, .external_lex_state = 4},
  [1311] = {.lex_state = 52, .external_lex_state = 4},
  [1312] = {.lex_state = 52, .external_lex_state = 4},
  [1313] = {.lex_state = 52, .external_lex_state = 4},
  [1314] = {.lex_state = 19, .external_lex_state = 5},
  [1315] = {.lex_state = 16, .external_lex_state = 5},
  [1316] = {.lex_state = 16, .external_lex_state = 5},
  [1317] = {.lex_state = 10, .external_lex_state = 4},
  [1318] = {.lex_state = 52, .external_lex_state = 4},
  [1319] = {.lex_state = 19, .external_lex_state = 5},
  [1320] = {.lex_state = 52, .external_lex_state = 4},
  [1321] = {.lex_state = 18, .external_lex_state = 5},
  [1322] = {.lex_state = 2, .external_lex_state = 5},
  [1323] = {.lex_state = 52, .external_lex_state = 4},
  [1324] = {.lex_state = 18, .external_lex_state = 5},
  [1325] = {.lex_state = 16, .external_lex_state = 5},
  [1326] = {.lex_state = 10, .external_lex_state = 4},
  [1327] = {.lex_state = 18, .external_lex_state = 5},
  [1328] = {.lex_state = 19, .external_lex_state = 5},
  [1329] = {.lex_state = 65, .external_lex_state = 5},
  [1330] = {.lex_state = 52, .external_lex_state = 4},
  [1331] = {.lex_state = 62, .external_lex_state = 5},
  [1332] = {.lex_state = 16, .external_lex_state = 5},
  [1333] = {.lex_state = 52, .external_lex_state = 4},
  [1334] = {.lex_state = 2, .external_lex_state = 5},
  [1335] = {.lex_state = 2, .external_lex_state = 5},
  [1336] = {.lex_state = 52, .external_lex_state = 4},
  [1337] = {.lex_state = 2, .external_lex_state = 5},
  [1338] = {.lex_state = 14, .external_lex_state = 4},
  [1339] = {.lex_state = 2, .external_lex_state = 5},
  [1340] = {.lex_state = 5, .external_lex_state = 4},
  [1341] = {.lex_state = 2, .external_lex_state = 5},
  [1342] = {.lex_state = 2, .external_lex_state = 5},
  [1343] = {.lex_state = 2, .external_lex_state = 5},
  [1344] = {.lex_state = 2, .external_lex_state = 5},
  [1345] = {.lex_state = 2, .external_lex_state = 5},
  [1346] = {.lex_state = 2, .external_lex_state = 5},
  [1347] = {.lex_state = 2, .external_lex_state = 5},
  [1348] = {.lex_state = 2, .external_lex_state = 5},
  [1349] = {.lex_state = 2, .external_lex_state = 4},
  [1350] = {.lex_state = 2, .external_lex_state = 5},
  [1351] = {.lex_state = 2, .external_lex_state = 5},
  [1352] = {.lex_state = 2, .external_lex_state = 5},
  [1353] = {.lex_state = 2, .external_lex_state = 5},
  [1354] = {.lex_state = 2, .external_lex_state = 5},
  [1355] = {.lex_state = 2, .external_lex_state = 5},
  [1356] = {.lex_state = 2, .external_lex_state = 5},
  [1357] = {.lex_state = 2, .external_lex_state = 5},
  [1358] = {.lex_state = 2, .external_lex_state = 5},
  [1359] = {.lex_state = 2, .external_lex_state = 4},
  [1360] = {.lex_state = 2, .external_lex_state = 5},
  [1361] = {.lex_state = 2, .external_lex_state = 5},
  [1362] = {.lex_state = 2, .external_lex_state = 5},
  [1363] = {.lex_state = 2, .external_lex_state = 5},
  [1364] = {.lex_state = 2, .external_lex_state = 5},
  [1365] = {.lex_state = 2, .external_lex_state = 5},
  [1366] = {.lex_state = 2, .external_lex_state = 5},
  [1367] = {.lex_state = 10, .external_lex_state = 4},
  [1368] = {.lex_state = 10, .external_lex_state = 4},
  [1369] = {.lex_state = 2, .external_lex_state = 5},
  [1370] = {.lex_state = 2, .external_lex_state = 5},
  [1371] = {.lex_state = 2, .external_lex_state = 5},
  [1372] = {.lex_state = 2, .external_lex_state = 5},
  [1373] = {.lex_state = 2, .external_lex_state = 5},
  [1374] = {.lex_state = 2, .external_lex_state = 5},
  [1375] = {.lex_state = 2, .external_lex_state = 5},
  [1376] = {.lex_state = 10, .external_lex_state = 4},
  [1377] = {.lex_state = 2, .external_lex_state = 5},
  [1378] = {.lex_state = 2, .external_lex_state = 5},
  [1379] = {.lex_state = 2, .external_lex_state = 5},
  [1380] = {.lex_state = 2, .external_lex_state = 5},
  [1381] = {.lex_state = 52, .external_lex_state = 4},
  [1382] = {.lex_state = 2, .external_lex_state = 5},
  [1383] = {.lex_state = 52, .external_lex_state = 4},
  [1384] = {.lex_state = 2, .external_lex_state = 5},
  [1385] = {.lex_state = 2, .external_lex_state = 5},
  [1386] = {.lex_state = 52, .external_lex_state = 4},
  [1387] = {.lex_state = 2, .external_lex_state = 5},
  [1388] = {.lex_state = 2, .external_lex_state = 5},
  [1389] = {.lex_state = 2, .external_lex_state = 5},
  [1390] = {.lex_state = 2, .external_lex_state = 5},
  [1391] = {.lex_state = 63, .external_lex_state = 4},
  [1392] = {.lex_state = 2, .external_lex_state = 5},
  [1393] = {.lex_state = 2, .external_lex_state = 5},
  [1394] = {.lex_state = 2, .external_lex_state = 5},
  [1395] = {.lex_state = 2, .external_lex_state = 4},
  [1396] = {.lex_state = 2, .external_lex_state = 5},
  [1397] = {.lex_state = 2, .external_lex_state = 5},
  [1398] = {.lex_state = 2, .external_lex_state = 5},
  [1399] = {.lex_state = 2, .external_lex_state = 5},
  [1400] = {.lex_state = 2, .external_lex_state = 5},
  [1401] = {.lex_state = 63, .external_lex_state = 4},
  [1402] = {.lex_state = 2, .external_lex_state = 5},
  [1403] = {.lex_state = 2, .external_lex_state = 5},
  [1404] = {.lex_state = 2, .external_lex_state = 5},
  [1405] = {.lex_state = 2, .external_lex_state = 5},
  [1406] = {.lex_state = 2, .external_lex_state = 5},
  [1407] = {.lex_state = 2, .external_lex_state = 5},
  [1408] = {.lex_state = 52, .external_lex_state = 4},
  [1409] = {.lex_state = 2, .external_lex_state = 5},
  [1410] = {.lex_state = 63, .external_lex_state = 4},
  [1411] = {.lex_state = 2, .external_lex_state = 5},
  [1412] = {.lex_state = 2, .external_lex_state = 5},
  [1413] = {.lex_state = 2, .external_lex_state = 5},
  [1414] = {.lex_state = 2, .external_lex_state = 5},
  [1415] = {.lex_state = 2, .external_lex_state = 5},
  [1416] = {.lex_state = 52, .external_lex_state = 4},
  [1417] = {.lex_state = 2, .external_lex_state = 5},
  [1418] = {.lex_state = 2, .external_lex_state = 5},
  [1419] = {.lex_state = 2, .external_lex_state = 5},
  [1420] = {.lex_state = 2, .external_lex_state = 5},
  [1421] = {.lex_state = 2, .external_lex_state = 5},
  [1422] = {.lex_state = 2, .external_lex_state = 5},
  [1423] = {.lex_state = 2, .external_lex_state = 5},
  [1424] = {.lex_state = 2, .external_lex_state = 5},
  [1425] = {.lex_state = 2, .external_lex_state = 5},
  [1426] = {.lex_state = 2, .external_lex_state = 5},
  [1427] = {.lex_state = 2, .external_lex_state = 5},
  [1428] = {.lex_state = 2, .external_lex_state = 5},
  [1429] = {.lex_state = 2, .external_lex_state = 5},
  [1430] = {.lex_state = 2, .external_lex_state = 5},
  [1431] = {.lex_state = 2, .external_lex_state = 5},
  [1432] = {.lex_state = 2, .external_lex_state = 5},
  [1433] = {.lex_state = 2, .external_lex_state = 5},
  [1434] = {.lex_state = 2, .external_lex_state = 5},
  [1435] = {.lex_state = 2, .external_lex_state = 5},
  [1436] = {.lex_state = 52, .external_lex_state = 4},
  [1437] = {.lex_state = 2, .external_lex_state = 5},
  [1438] = {.lex_state = 2, .external_lex_state = 5},
  [1439] = {.lex_state = 2, .external_lex_state = 5},
  [1440] = {.lex_state = 2, .external_lex_state = 5},
  [1441] = {.lex_state = 2, .external_lex_state = 5},
  [1442] = {.lex_state = 2, .external_lex_state = 5},
  [1443] = {.lex_state = 2, .external_lex_state = 5},
  [1444] = {.lex_state = 2, .external_lex_state = 5},
  [1445] = {.lex_state = 2, .external_lex_state = 5},
  [1446] = {.lex_state = 2, .external_lex_state = 5},
  [1447] = {.lex_state = 2, .external_lex_state = 5},
  [1448] = {.lex_state = 2, .external_lex_state = 5},
  [1449] = {.lex_state = 2, .external_lex_state = 5},
  [1450] = {.lex_state = 2, .external_lex_state = 5},
  [1451] = {.lex_state = 2, .external_lex_state = 4},
  [1452] = {.lex_state = 2, .external_lex_state = 4},
  [1453] = {.lex_state = 2, .external_lex_state = 4},
  [1454] = {.lex_state = 2, .external_lex_state = 4},
  [1455] = {.lex_state = 2, .external_lex_state = 5},
  [1456] = {.lex_state = 2, .external_lex_state = 4},
  [1457] = {.lex_state = 2, .external_lex_state = 4},
  [1458] = {.lex_state = 2, .external_lex_state = 4},
  [1459] = {.lex_state = 52, .external_lex_state = 4},
  [1460] = {.lex_state = 2, .external_lex_state = 4},
  [1461] = {.lex_state = 2, .external_lex_state = 4},
  [1462] = {.lex_state = 2, .external_lex_state = 4},
  [1463] = {.lex_state = 5, .external_lex_state = 4},
  [1464] = {.lex_state = 2, .external_lex_state = 4},
  [1465] = {.lex_state = 2, .external_lex_state = 4},
  [1466] = {.lex_state = 2, .external_lex_state = 4},
  [1467] = {.lex_state = 2, .external_lex_state = 4},
  [1468] = {.lex_state = 2, .external_lex_state = 4},
  [1469] = {.lex_state = 2, .external_lex_state = 4},
  [1470] = {.lex_state = 2, .external_lex_state = 4},
  [1471] = {.lex_state = 2, .external_lex_state = 4},
  [1472] = {.lex_state = 2, .external_lex_state = 4},
  [1473] = {.lex_state = 2, .external_lex_state = 4},
  [1474] = {.lex_state = 2, .external_lex_state = 4},
  [1475] = {.lex_state = 2, .external_lex_state = 4},
  [1476] = {.lex_state = 2, .external_lex_state = 4},
  [1477] = {.lex_state = 2, .external_lex_state = 4},
  [1478] = {.lex_state = 2, .external_lex_state = 4},
  [1479] = {.lex_state = 2, .external_lex_state = 4},
  [1480] = {.lex_state = 2, .external_lex_state = 4},
  [1481] = {.lex_state = 2, .external_lex_state = 4},
  [1482] = {.lex_state = 2, .external_lex_state = 4},
  [1483] = {.lex_state = 2, .external_lex_state = 4},
  [1484] = {.lex_state = 2, .external_lex_state = 4},
  [1485] = {.lex_state = 52, .external_lex_state = 4},
  [1486] = {.lex_state = 2, .external_lex_state = 4},
  [1487] = {.lex_state = 2, .external_lex_state = 4},
  [1488] = {.lex_state = 2, .external_lex_state = 4},
  [1489] = {.lex_state = 2, .external_lex_state = 4},
  [1490] = {.lex_state = 2, .external_lex_state = 4},
  [1491] = {.lex_state = 52, .external_lex_state = 4},
  [1492] = {.lex_state = 2, .external_lex_state = 4},
  [1493] = {.lex_state = 2, .external_lex_state = 4},
  [1494] = {.lex_state = 2, .external_lex_state = 4},
  [1495] = {.lex_state = 2, .external_lex_state = 4},
  [1496] = {.lex_state = 2, .external_lex_state = 4},
  [1497] = {.lex_state = 2, .external_lex_state = 4},
  [1498] = {.lex_state = 2, .external_lex_state = 4},
  [1499] = {.lex_state = 2, .external_lex_state = 4},
  [1500] = {.lex_state = 2, .external_lex_state = 4},
  [1501] = {.lex_state = 2, .external_lex_state = 4},
  [1502] = {.lex_state = 2, .external_lex_state = 4},
  [1503] = {.lex_state = 2, .external_lex_state = 4},
  [1504] = {.lex_state = 2, .external_lex_state = 4},
  [1505] = {.lex_state = 2, .external_lex_state = 4},
  [1506] = {.lex_state = 52, .external_lex_state = 4},
  [1507] = {.lex_state = 52, .external_lex_state = 4},
  [1508] = {.lex_state = 52, .external_lex_state = 4},
  [1509] = {.lex_state = 52, .external_lex_state = 4},
  [1510] = {.lex_state = 2, .external_lex_state = 4},
  [1511] = {.lex_state = 2, .external_lex_state = 4},
  [1512] = {.lex_state = 52, .external_lex_state = 4},
  [1513] = {.lex_state = 2, .external_lex_state = 4},
  [1514] = {.lex_state = 2, .external_lex_state = 4},
  [1515] = {.lex_state = 2, .external_lex_state = 4},
  [1516] = {.lex_state = 2, .external_lex_state = 4},
  [1517] = {.lex_state = 2, .external_lex_state = 4},
  [1518] = {.lex_state = 2, .external_lex_state = 4},
  [1519] = {.lex_state = 2, .external_lex_state = 4},
  [1520] = {.lex_state = 2, .external_lex_state = 4},
  [1521] = {.lex_state = 2, .external_lex_state = 4},
  [1522] = {.lex_state = 2, .external_lex_state = 4},
  [1523] = {.lex_state = 2, .external_lex_state = 4},
  [1524] = {.lex_state = 2, .external_lex_state = 4},
  [1525] = {.lex_state = 2, .external_lex_state = 4},
  [1526] = {.lex_state = 2, .external_lex_state = 4},
  [1527] = {.lex_state = 2, .external_lex_state = 4},
  [1528] = {.lex_state = 2, .external_lex_state = 5},
  [1529] = {.lex_state = 2, .external_lex_state = 4},
  [1530] = {.lex_state = 52, .external_lex_state = 4},
  [1531] = {.lex_state = 2, .external_lex_state = 4},
  [1532] = {.lex_state = 52, .external_lex_state = 4},
  [1533] = {.lex_state = 2, .external_lex_state = 4},
  [1534] = {.lex_state = 2, .external_lex_state = 4},
  [1535] = {.lex_state = 14, .external_lex_state = 4},
  [1536] = {.lex_state = 14, .external_lex_state = 4},
  [1537] = {.lex_state = 2, .external_lex_state = 4},
  [1538] = {.lex_state = 2, .external_lex_state = 4},
  [1539] = {.lex_state = 2, .external_lex_state = 4},
  [1540] = {.lex_state = 2, .external_lex_state = 4},
  [1541] = {.lex_state = 2, .external_lex_state = 4},
  [1542] = {.lex_state = 2, .external_lex_state = 4},
  [1543] = {.lex_state = 2, .external_lex_state = 4},
  [1544] = {.lex_state = 2, .external_lex_state = 4},
  [1545] = {.lex_state = 2, .external_lex_state = 4},
  [1546] = {.lex_state = 2, .external_lex_state = 4},
  [1547] = {.lex_state = 2, .external_lex_state = 5},
  [1548] = {.lex_state = 2, .external_lex_state = 4},
  [1549] = {.lex_state = 2, .external_lex_state = 4},
  [1550] = {.lex_state = 2, .external_lex_state = 4},
  [1551] = {.lex_state = 2, .external_lex_state = 4},
  [1552] = {.lex_state = 2, .external_lex_state = 4},
  [1553] = {.lex_state = 2, .external_lex_state = 4},
  [1554] = {.lex_state = 14, .external_lex_state = 4},
  [1555] = {.lex_state = 14, .external_lex_state = 4},
  [1556] = {.lex_state = 2, .external_lex_state = 4},
  [1557] = {.lex_state = 2, .external_lex_state = 4},
  [1558] = {.lex_state = 2, .external_lex_state = 4},
  [1559] = {.lex_state = 2, .external_lex_state = 4},
  [1560] = {.lex_state = 14, .external_lex_state = 4},
  [1561] = {.lex_state = 14, .external_lex_state = 4},
  [1562] = {.lex_state = 14, .external_lex_state = 4},
  [1563] = {.lex_state = 14, .external_lex_state = 4},
  [1564] = {.lex_state = 14, .external_lex_state = 4},
  [1565] = {.lex_state = 10, .external_lex_state = 4},
  [1566] = {.lex_state = 61, .external_lex_state = 4},
  [1567] = {.lex_state = 61, .external_lex_state = 4},
  [1568] = {.lex_state = 59, .external_lex_state = 4},
  [1569] = {.lex_state = 10, .external_lex_state = 4},
  [1570] = {.lex_state = 10, .external_lex_state = 4},
  [1571] = {.lex_state = 61, .external_lex_state = 4},
  [1572] = {.lex_state = 14, .external_lex_state = 4},
  [1573] = {.lex_state = 10, .external_lex_state = 4},
  [1574] = {.lex_state = 10, .external_lex_state = 4},
  [1575] = {.lex_state = 10, .external_lex_state = 4},
  [1576] = {.lex_state = 10, .external_lex_state = 4},
  [1577] = {.lex_state = 61, .external_lex_state = 4},
  [1578] = {.lex_state = 10, .external_lex_state = 4},
  [1579] = {.lex_state = 61, .external_lex_state = 4},
  [1580] = {.lex_state = 61, .external_lex_state = 4},
  [1581] = {.lex_state = 61, .external_lex_state = 4},
  [1582] = {.lex_state = 61, .external_lex_state = 4},
  [1583] = {.lex_state = 14, .external_lex_state = 4},
  [1584] = {.lex_state = 14, .external_lex_state = 4},
  [1585] = {.lex_state = 10, .external_lex_state = 4},
  [1586] = {.lex_state = 61, .external_lex_state = 4},
  [1587] = {.lex_state = 61, .external_lex_state = 4},
  [1588] = {.lex_state = 61, .external_lex_state = 4},
  [1589] = {.lex_state = 14, .external_lex_state = 4},
  [1590] = {.lex_state = 61, .external_lex_state = 4},
  [1591] = {.lex_state = 61, .external_lex_state = 4},
  [1592] = {.lex_state = 61, .external_lex_state = 4},
  [1593] = {.lex_state = 14, .external_lex_state = 4},
  [1594] = {.lex_state = 14, .external_lex_state = 4},
  [1595] = {.lex_state = 61, .external_lex_state = 4},
  [1596] = {.lex_state = 14, .external_lex_state = 4},
  [1597] = {.lex_state = 14, .external_lex_state = 4},
  [1598] = {.lex_state = 14, .external_lex_state = 4},
  [1599] = {.lex_state = 14, .external_lex_state = 4},
  [1600] = {.lex_state = 14, .external_lex_state = 4},
  [1601] = {.lex_state = 14, .external_lex_state = 4},
  [1602] = {.lex_state = 14, .external_lex_state = 4},
  [1603] = {.lex_state = 59, .external_lex_state = 4},
  [1604] = {.lex_state = 61, .external_lex_state = 4},
  [1605] = {.lex_state = 14, .external_lex_state = 4},
  [1606] = {.lex_state = 61, .external_lex_state = 4},
  [1607] = {.lex_state = 14, .external_lex_state = 4},
  [1608] = {.lex_state = 61, .external_lex_state = 4},
  [1609] = {.lex_state = 61, .external_lex_state = 4},
  [1610] = {.lex_state = 14, .external_lex_state = 4},
  [1611] = {.lex_state = 59, .external_lex_state = 4},
  [1612] = {.lex_state = 59, .external_lex_state = 4},
  [1613] = {.lex_state = 59, .external_lex_state = 4},
  [1614] = {.lex_state = 59, .external_lex_state = 4},
  [1615] = {.lex_state = 59, .external_lex_state = 4},
  [1616] = {.lex_state = 59, .external_lex_state = 4},
  [1617] = {.lex_state = 59, .external_lex_state = 4},
  [1618] = {.lex_state = 10, .external_lex_state = 4},
  [1619] = {.lex_state = 10, .external_lex_state = 4},
  [1620] = {.lex_state = 58, .external_lex_state = 4},
  [1621] = {.lex_state = 10, .external_lex_state = 4},
  [1622] = {.lex_state = 61, .external_lex_state = 4},
  [1623] = {.lex_state = 61, .external_lex_state = 4},
  [1624] = {.lex_state = 10, .external_lex_state = 4},
  [1625] = {.lex_state = 10, .external_lex_state = 4},
  [1626] = {.lex_state = 10, .external_lex_state = 4},
  [1627] = {.lex_state = 10, .external_lex_state = 4},
  [1628] = {.lex_state = 10, .external_lex_state = 4},
  [1629] = {.lex_state = 10, .external_lex_state = 4},
  [1630] = {.lex_state = 10, .external_lex_state = 4},
  [1631] = {.lex_state = 10, .external_lex_state = 4},
  [1632] = {.lex_state = 10, .external_lex_state = 4},
  [1633] = {.lex_state = 58, .external_lex_state = 4},
  [1634] = {.lex_state = 10, .external_lex_state = 4},
  [1635] = {.lex_state = 10, .external_lex_state = 4},
  [1636] = {.lex_state = 10, .external_lex_state = 4},
  [1637] = {.lex_state = 10, .external_lex_state = 4},
  [1638] = {.lex_state = 58, .external_lex_state = 4},
  [1639] = {.lex_state = 14, .external_lex_state = 4},
  [1640] = {.lex_state = 58, .external_lex_state = 4},
  [1641] = {.lex_state = 58, .external_lex_state = 4},
  [1642] = {.lex_state = 58, .external_lex_state = 4},
  [1643] = {.lex_state = 58, .external_lex_state = 4},
  [1644] = {.lex_state = 58, .external_lex_state = 4},
  [1645] = {.lex_state = 58, .external_lex_state = 4},
  [1646] = {.lex_state = 59, .external_lex_state = 4},
  [1647] = {.lex_state = 59, .external_lex_state = 4},
  [1648] = {.lex_state = 60, .external_lex_state = 4},
  [1649] = {.lex_state = 59, .external_lex_state = 4},
  [1650] = {.lex_state = 59, .external_lex_state = 4},
  [1651] = {.lex_state = 59, .external_lex_state = 4},
  [1652] = {.lex_state = 59, .external_lex_state = 4},
  [1653] = {.lex_state = 59, .external_lex_state = 4},
  [1654] = {.lex_state = 59, .external_lex_state = 4},
  [1655] = {.lex_state = 14, .external_lex_state = 4},
  [1656] = {.lex_state = 14, .external_lex_state = 4},
  [1657] = {.lex_state = 59, .external_lex_state = 4},
  [1658] = {.lex_state = 59, .external_lex_state = 4},
  [1659] = {.lex_state = 59, .external_lex_state = 4},
  [1660] = {.lex_state = 59, .external_lex_state = 4},
  [1661] = {.lex_state = 60, .external_lex_state = 4},
  [1662] = {.lex_state = 14, .external_lex_state = 4},
  [1663] = {.lex_state = 14, .external_lex_state = 4},
  [1664] = {.lex_state = 14, .external_lex_state = 4},
  [1665] = {.lex_state = 59, .external_lex_state = 4},
  [1666] = {.lex_state = 59, .external_lex_state = 4},
  [1667] = {.lex_state = 61, .external_lex_state = 4},
  [1668] = {.lex_state = 61, .external_lex_state = 4},
  [1669] = {.lex_state = 59, .external_lex_state = 4},
  [1670] = {.lex_state = 59, .external_lex_state = 4},
  [1671] = {.lex_state = 60, .external_lex_state = 4},
  [1672] = {.lex_state = 60, .external_lex_state = 4},
  [1673] = {.lex_state = 14, .external_lex_state = 4},
  [1674] = {.lex_state = 14, .external_lex_state = 4},
  [1675] = {.lex_state = 60, .external_lex_state = 4},
  [1676] = {.lex_state = 60, .external_lex_state = 4},
  [1677] = {.lex_state = 60, .external_lex_state = 4},
  [1678] = {.lex_state = 60, .external_lex_state = 4},
  [1679] = {.lex_state = 60, .external_lex_state = 4},
  [1680] = {.lex_state = 58, .external_lex_state = 4},
  [1681] = {.lex_state = 58, .external_lex_state = 4},
  [1682] = {.lex_state = 58, .external_lex_state = 4},
  [1683] = {.lex_state = 58, .external_lex_state = 4},
  [1684] = {.lex_state = 58, .external_lex_state = 4},
  [1685] = {.lex_state = 58, .external_lex_state = 4},
  [1686] = {.lex_state = 58, .external_lex_state = 4},
  [1687] = {.lex_state = 58, .external_lex_state = 4},
  [1688] = {.lex_state = 58, .external_lex_state = 4},
  [1689] = {.lex_state = 14, .external_lex_state = 4},
  [1690] = {.lex_state = 58, .external_lex_state = 4},
  [1691] = {.lex_state = 58, .external_lex_state = 4},
  [1692] = {.lex_state = 58, .external_lex_state = 4},
  [1693] = {.lex_state = 58, .external_lex_state = 4},
  [1694] = {.lex_state = 60, .external_lex_state = 4},
  [1695] = {.lex_state = 60, .external_lex_state = 4},
  [1696] = {.lex_state = 60, .external_lex_state = 4},
  [1697] = {.lex_state = 60, .external_lex_state = 4},
  [1698] = {.lex_state = 58, .external_lex_state = 4},
  [1699] = {.lex_state = 60, .external_lex_state = 4},
  [1700] = {.lex_state = 60, .external_lex_state = 4},
  [1701] = {.lex_state = 60, .external_lex_state = 4},
  [1702] = {.lex_state = 61, .external_lex_state = 4},
  [1703] = {.lex_state = 61, .external_lex_state = 4},
  [1704] = {.lex_state = 60, .external_lex_state = 4},
  [1705] = {.lex_state = 60, .external_lex_state = 4},
  [1706] = {.lex_state = 60, .external_lex_state = 4},
  [1707] = {.lex_state = 60, .external_lex_state = 4},
  [1708] = {.lex_state = 58, .external_lex_state = 4},
  [1709] = {.lex_state = 60, .external_lex_state = 4},
  [1710] = {.lex_state = 60, .external_lex_state = 4},
  [1711] = {.lex_state = 60, .external_lex_state = 4},
  [1712] = {.lex_state = 60, .external_lex_state = 4},
  [1713] = {.lex_state = 58, .external_lex_state = 4},
  [1714] = {.lex_state = 60, .external_lex_state = 4},
  [1715] = {.lex_state = 14, .external_lex_state = 4},
  [1716] = {.lex_state = 10, .external_lex_state = 4},
  [1717] = {.lex_state = 14, .external_lex_state = 4},
  [1718] = {.lex_state = 14, .external_lex_state = 4},
  [1719] = {.lex_state = 14, .external_lex_state = 4},
  [1720] = {.lex_state = 10, .external_lex_state = 4},
  [1721] = {.lex_state = 10, .external_lex_state = 4},
  [1722] = {.lex_state = 10, .external_lex_state = 4},
  [1723] = {.lex_state = 52, .external_lex_state = 4},
  [1724] = {.lex_state = 52, .external_lex_state = 4},
  [1725] = {.lex_state = 52, .external_lex_state = 4},
  [1726] = {.lex_state = 52, .external_lex_state = 4},
  [1727] = {.lex_state = 52, .external_lex_state = 4},
  [1728] = {.lex_state = 10, .external_lex_state = 4},
  [1729] = {.lex_state = 10, .external_lex_state = 4},
  [1730] = {.lex_state = 10, .external_lex_state = 4},
  [1731] = {.lex_state = 52, .external_lex_state = 4},
  [1732] = {.lex_state = 52, .external_lex_state = 4},
  [1733] = {.lex_state = 52, .external_lex_state = 4},
  [1734] = {.lex_state = 52, .external_lex_state = 4},
  [1735] = {.lex_state = 66, .external_lex_state = 3},
  [1736] = {.lex_state = 52, .external_lex_state = 4},
  [1737] = {.lex_state = 66, .external_lex_state = 3},
  [1738] = {.lex_state = 66, .external_lex_state = 3},
  [1739] = {.lex_state = 66, .external_lex_state = 3},
  [1740] = {.lex_state = 52, .external_lex_state = 4},
  [1741] = {.lex_state = 52, .external_lex_state = 4},
  [1742] = {.lex_state = 52, .external_lex_state = 4},
  [1743] = {.lex_state = 10, .external_lex_state = 4},
  [1744] = {.lex_state = 12, .external_lex_state = 4},
  [1745] = {.lex_state = 66, .external_lex_state = 3},
  [1746] = {.lex_state = 52, .external_lex_state = 4},
  [1747] = {.lex_state = 52, .external_lex_state = 4},
  [1748] = {.lex_state = 52, .external_lex_state = 4},
  [1749] = {.lex_state = 52, .external_lex_state = 4},
  [1750] = {.lex_state = 52, .external_lex_state = 4},
  [1751] = {.lex_state = 52, .external_lex_state = 4},
  [1752] = {.lex_state = 52, .external_lex_state = 4},
  [1753] = {.lex_state = 52, .external_lex_state = 4},
  [1754] = {.lex_state = 10, .external_lex_state = 4},
  [1755] = {.lex_state = 67, .external_lex_state = 3},
  [1756] = {.lex_state = 12, .external_lex_state = 4},
  [1757] = {.lex_state = 10, .external_lex_state = 4},
  [1758] = {.lex_state = 52, .external_lex_state = 4},
  [1759] = {.lex_state = 52, .external_lex_state = 4},
  [1760] = {.lex_state = 52, .external_lex_state = 4},
  [1761] = {.lex_state = 52, .external_lex_state = 4},
  [1762] = {.lex_state = 52, .external_lex_state = 4},
  [1763] = {.lex_state = 52, .external_lex_state = 4},
  [1764] = {.lex_state = 52, .external_lex_state = 4},
  [1765] = {.lex_state = 52, .external_lex_state = 4},
  [1766] = {.lex_state = 52, .external_lex_state = 4},
  [1767] = {.lex_state = 52, .external_lex_state = 4},
  [1768] = {.lex_state = 52, .external_lex_state = 4},
  [1769] = {.lex_state = 52, .external_lex_state = 4},
  [1770] = {.lex_state = 67, .external_lex_state = 3},
  [1771] = {.lex_state = 52, .external_lex_state = 4},
  [1772] = {.lex_state = 52, .external_lex_state = 4},
  [1773] = {.lex_state = 52, .external_lex_state = 4},
  [1774] = {.lex_state = 10, .external_lex_state = 4},
  [1775] = {.lex_state = 52, .external_lex_state = 4},
  [1776] = {.lex_state = 52, .external_lex_state = 4},
  [1777] = {.lex_state = 52, .external_lex_state = 4},
  [1778] = {.lex_state = 52, .external_lex_state = 4},
  [1779] = {.lex_state = 52, .external_lex_state = 4},
  [1780] = {.lex_state = 52, .external_lex_state = 4},
  [1781] = {.lex_state = 10, .external_lex_state = 4},
  [1782] = {.lex_state = 67, .external_lex_state = 3},
  [1783] = {.lex_state = 52, .external_lex_state = 4},
  [1784] = {.lex_state = 67, .external_lex_state = 3},
  [1785] = {.lex_state = 52, .external_lex_state = 4},
  [1786] = {.lex_state = 52, .external_lex_state = 4},
  [1787] = {.lex_state = 52, .external_lex_state = 4},
  [1788] = {.lex_state = 52, .external_lex_state = 4},
  [1789] = {.lex_state = 52, .external_lex_state = 4},
  [1790] = {.lex_state = 10, .external_lex_state = 4},
  [1791] = {.lex_state = 52, .external_lex_state = 4},
  [1792] = {.lex_state = 52, .external_lex_state = 4},
  [1793] = {.lex_state = 10, .external_lex_state = 4},
  [1794] = {.lex_state = 52, .external_lex_state = 4},
  [1795] = {.lex_state = 10, .external_lex_state = 4},
  [1796] = {.lex_state = 50, .external_lex_state = 8},
  [1797] = {.lex_state = 50, .external_lex_state = 8},
  [1798] = {.lex_state = 10, .external_lex_state = 4},
  [1799] = {.lex_state = 10, .external_lex_state = 4},
  [1800] = {.lex_state = 10, .external_lex_state = 4},
  [1801] = {.lex_state = 10, .external_lex_state = 4},
  [1802] = {.lex_state = 55, .external_lex_state = 7},
  [1803] = {.lex_state = 10, .external_lex_state = 5},
  [1804] = {.lex_state = 55, .external_lex_state = 7},
  [1805] = {.lex_state = 10, .external_lex_state = 4},
  [1806] = {.lex_state = 10, .external_lex_state = 4},
  [1807] = {.lex_state = 10, .external_lex_state = 5},
  [1808] = {.lex_state = 10, .external_lex_state = 4},
  [1809] = {.lex_state = 50, .external_lex_state = 8},
  [1810] = {.lex_state = 55, .external_lex_state = 7},
  [1811] = {.lex_state = 66, .external_lex_state = 3},
  [1812] = {.lex_state = 66, .external_lex_state = 3},
  [1813] = {.lex_state = 52, .external_lex_state = 4},
  [1814] = {.lex_state = 66, .external_lex_state = 3},
  [1815] = {.lex_state = 66, .external_lex_state = 3},
  [1816] = {.lex_state = 12, .external_lex_state = 4},
  [1817] = {.lex_state = 66, .external_lex_state = 3},
  [1818] = {.lex_state = 66, .external_lex_state = 3},
  [1819] = {.lex_state = 12, .external_lex_state = 4},
  [1820] = {.lex_state = 12, .external_lex_state = 4},
  [1821] = {.lex_state = 66, .external_lex_state = 3},
  [1822] = {.lex_state = 12, .external_lex_state = 4},
  [1823] = {.lex_state = 66, .external_lex_state = 3},
  [1824] = {.lex_state = 68, .external_lex_state = 4},
  [1825] = {.lex_state = 68, .external_lex_state = 4},
  [1826] = {.lex_state = 50, .external_lex_state = 8},
  [1827] = {.lex_state = 10, .external_lex_state = 5},
  [1828] = {.lex_state = 50, .external_lex_state = 8},
  [1829] = {.lex_state = 68, .external_lex_state = 4},
  [1830] = {.lex_state = 67, .external_lex_state = 3},
  [1831] = {.lex_state = 10, .external_lex_state = 5},
  [1832] = {.lex_state = 10, .external_lex_state = 5},
  [1833] = {.lex_state = 66, .external_lex_state = 3},
  [1834] = {.lex_state = 13, .external_lex_state = 4},
  [1835] = {.lex_state = 10, .external_lex_state = 5},
  [1836] = {.lex_state = 10, .external_lex_state = 5},
  [1837] = {.lex_state = 13, .external_lex_state = 4},
  [1838] = {.lex_state = 50, .external_lex_state = 8},
  [1839] = {.lex_state = 50, .external_lex_state = 8},
  [1840] = {.lex_state = 68, .external_lex_state = 4},
  [1841] = {.lex_state = 13, .external_lex_state = 4},
  [1842] = {.lex_state = 50, .external_lex_state = 8},
  [1843] = {.lex_state = 13, .external_lex_state = 4},
  [1844] = {.lex_state = 13, .external_lex_state = 4},
  [1845] = {.lex_state = 10, .external_lex_state = 5},
  [1846] = {.lex_state = 10, .external_lex_state = 5},
  [1847] = {.lex_state = 13, .external_lex_state = 4},
  [1848] = {.lex_state = 13, .external_lex_state = 4},
  [1849] = {.lex_state = 13, .external_lex_state = 4},
  [1850] = {.lex_state = 50, .external_lex_state = 8},
  [1851] = {.lex_state = 13, .external_lex_state = 4},
  [1852] = {.lex_state = 13, .external_lex_state = 4},
  [1853] = {.lex_state = 13, .external_lex_state = 4},
  [1854] = {.lex_state = 10, .external_lex_state = 4},
  [1855] = {.lex_state = 13, .external_lex_state = 4},
  [1856] = {.lex_state = 13, .external_lex_state = 4},
  [1857] = {.lex_state = 13, .external_lex_state = 4},
  [1858] = {.lex_state = 13, .external_lex_state = 4},
  [1859] = {.lex_state = 13, .external_lex_state = 4},
  [1860] = {.lex_state = 10, .external_lex_state = 5},
  [1861] = {.lex_state = 10, .external_lex_state = 5},
  [1862] = {.lex_state = 50, .external_lex_state = 8},
  [1863] = {.lex_state = 10, .external_lex_state = 5},
  [1864] = {.lex_state = 50, .external_lex_state = 8},
  [1865] = {.lex_state = 10, .external_lex_state = 5},
  [1866] = {.lex_state = 10, .external_lex_state = 5},
  [1867] = {.lex_state = 66, .external_lex_state = 3},
  [1868] = {.lex_state = 13, .external_lex_state = 4},
  [1869] = {.lex_state = 67, .external_lex_state = 3},
  [1870] = {.lex_state = 13, .external_lex_state = 4},
  [1871] = {.lex_state = 13, .external_lex_state = 4},
  [1872] = {.lex_state = 67, .external_lex_state = 3},
  [1873] = {.lex_state = 10, .external_lex_state = 5},
  [1874] = {.lex_state = 14, .external_lex_state = 4},
  [1875] = {.lex_state = 13, .external_lex_state = 4},
  [1876] = {.lex_state = 67, .external_lex_state = 3},
  [1877] = {.lex_state = 10, .external_lex_state = 5},
  [1878] = {.lex_state = 10, .external_lex_state = 5},
  [1879] = {.lex_state = 10, .external_lex_state = 5},
  [1880] = {.lex_state = 67, .external_lex_state = 3},
  [1881] = {.lex_state = 10, .external_lex_state = 5},
  [1882] = {.lex_state = 10, .external_lex_state = 5},
  [1883] = {.lex_state = 10, .external_lex_state = 5},
  [1884] = {.lex_state = 0, .external_lex_state = 4},
  [1885] = {.lex_state = 10, .external_lex_state = 5},
  [1886] = {.lex_state = 10, .external_lex_state = 5},
  [1887] = {.lex_state = 13, .external_lex_state = 4},
  [1888] = {.lex_state = 13, .external_lex_state = 4},
  [1889] = {.lex_state = 13, .external_lex_state = 4},
  [1890] = {.lex_state = 13, .external_lex_state = 4},
  [1891] = {.lex_state = 10, .external_lex_state = 5},
  [1892] = {.lex_state = 13, .external_lex_state = 4},
  [1893] = {.lex_state = 13, .external_lex_state = 4},
  [1894] = {.lex_state = 13, .external_lex_state = 4},
  [1895] = {.lex_state = 10, .external_lex_state = 5},
  [1896] = {.lex_state = 67, .external_lex_state = 3},
  [1897] = {.lex_state = 67, .external_lex_state = 3},
  [1898] = {.lex_state = 10, .external_lex_state = 5},
  [1899] = {.lex_state = 10, .external_lex_state = 4},
  [1900] = {.lex_state = 10, .external_lex_state = 4},
  [1901] = {.lex_state = 0, .external_lex_state = 4},
  [1902] = {.lex_state = 0, .external_lex_state = 5},
  [1903] = {.lex_state = 0, .external_lex_state = 4},
  [1904] = {.lex_state = 0, .external_lex_state = 4},
  [1905] = {.lex_state = 10, .external_lex_state = 5},
  [1906] = {.lex_state = 10, .external_lex_state = 4},
  [1907] = {.lex_state = 10, .external_lex_state = 4},
  [1908] = {.lex_state = 0, .external_lex_state = 5},
  [1909] = {.lex_state = 10, .external_lex_state = 4},
  [1910] = {.lex_state = 68, .external_lex_state = 4},
  [1911] = {.lex_state = 10, .external_lex_state = 3},
  [1912] = {.lex_state = 69, .external_lex_state = 4},
  [1913] = {.lex_state = 10, .external_lex_state = 5},
  [1914] = {.lex_state = 66, .external_lex_state = 3},
  [1915] = {.lex_state = 66, .external_lex_state = 3},
  [1916] = {.lex_state = 0, .external_lex_state = 5},
  [1917] = {.lex_state = 0, .external_lex_state = 5},
  [1918] = {.lex_state = 0, .external_lex_state = 5},
  [1919] = {.lex_state = 0, .external_lex_state = 5},
  [1920] = {.lex_state = 10, .external_lex_state = 4},
  [1921] = {.lex_state = 10, .external_lex_state = 4},
  [1922] = {.lex_state = 13, .external_lex_state = 4},
  [1923] = {.lex_state = 10, .external_lex_state = 5},
  [1924] = {.lex_state = 10, .external_lex_state = 3},
  [1925] = {.lex_state = 0, .external_lex_state = 5},
  [1926] = {.lex_state = 0, .external_lex_state = 4},
  [1927] = {.lex_state = 13, .external_lex_state = 4},
  [1928] = {.lex_state = 60, .external_lex_state = 4},
  [1929] = {.lex_state = 60, .external_lex_state = 4},
  [1930] = {.lex_state = 10, .external_lex_state = 4},
  [1931] = {.lex_state = 0, .external_lex_state = 4},
  [1932] = {.lex_state = 66, .external_lex_state = 4},
  [1933] = {.lex_state = 59, .external_lex_state = 4},
  [1934] = {.lex_state = 0, .external_lex_state = 5},
  [1935] = {.lex_state = 0, .external_lex_state = 5},
  [1936] = {.lex_state = 0, .external_lex_state = 4},
  [1937] = {.lex_state = 52, .external_lex_state = 4},
  [1938] = {.lex_state = 0, .external_lex_state = 4},
  [1939] = {.lex_state = 0, .external_lex_state = 4},
  [1940] = {.lex_state = 0, .external_lex_state = 4},
  [1941] = {.lex_state = 52, .external_lex_state = 4},
  [1942] = {.lex_state = 0, .external_lex_state = 4},
  [1943] = {.lex_state = 10, .external_lex_state = 4},
  [1944] = {.lex_state = 0, .external_lex_state = 4},
  [1945] = {.lex_state = 10, .external_lex_state = 4},
  [1946] = {.lex_state = 874, .external_lex_state = 4},
  [1947] = {.lex_state = 0, .external_lex_state = 3},
  [1948] = {.lex_state = 10, .external_lex_state = 4},
  [1949] = {.lex_state = 61, .external_lex_state = 4},
  [1950] = {.lex_state = 52, .external_lex_state = 4},
  [1951] = {.lex_state = 10, .external_lex_state = 4},
  [1952] = {.lex_state = 58, .external_lex_state = 4},
  [1953] = {.lex_state = 59, .external_lex_state = 4},
  [1954] = {.lex_state = 59, .external_lex_state = 4},
  [1955] = {.lex_state = 0, .external_lex_state = 4},
  [1956] = {.lex_state = 0, .external_lex_state = 5},
  [1957] = {.lex_state = 58, .external_lex_state = 4},
  [1958] = {.lex_state = 52, .external_lex_state = 4},
  [1959] = {.lex_state = 58, .external_lex_state = 4},
  [1960] = {.lex_state = 0, .external_lex_state = 5},
  [1961] = {.lex_state = 59, .external_lex_state = 4},
  [1962] = {.lex_state = 52, .external_lex_state = 4},
  [1963] = {.lex_state = 58, .external_lex_state = 4},
  [1964] = {.lex_state = 10, .external_lex_state = 4},
  [1965] = {.lex_state = 52, .external_lex_state = 4},
  [1966] = {.lex_state = 52, .external_lex_state = 4},
  [1967] = {.lex_state = 60, .external_lex_state = 4},
  [1968] = {.lex_state = 59, .external_lex_state = 4},
  [1969] = {.lex_state = 52, .external_lex_state = 4},
  [1970] = {.lex_state = 60, .external_lex_state = 4},
  [1971] = {.lex_state = 0, .external_lex_state = 4},
  [1972] = {.lex_state = 58, .external_lex_state = 4},
  [1973] = {.lex_state = 52, .external_lex_state = 4},
  [1974] = {.lex_state = 0, .external_lex_state = 4},
  [1975] = {.lex_state = 0, .external_lex_state = 4},
  [1976] = {.lex_state = 58, .external_lex_state = 4},
  [1977] = {.lex_state = 10, .external_lex_state = 4},
  [1978] = {.lex_state = 59, .external_lex_state = 4},
  [1979] = {.lex_state = 0, .external_lex_state = 4},
  [1980] = {.lex_state = 59, .external_lex_state = 4},
  [1981] = {.lex_state = 0, .external_lex_state = 5},
  [1982] = {.lex_state = 66, .external_lex_state = 4},
  [1983] = {.lex_state = 60, .external_lex_state = 4},
  [1984] = {.lex_state = 0, .external_lex_state = 4},
  [1985] = {.lex_state = 58, .external_lex_state = 4},
  [1986] = {.lex_state = 67, .external_lex_state = 4},
  [1987] = {.lex_state = 61, .external_lex_state = 4},
  [1988] = {.lex_state = 60, .external_lex_state = 4},
  [1989] = {.lex_state = 0, .external_lex_state = 4},
  [1990] = {.lex_state = 0, .external_lex_state = 4},
  [1991] = {.lex_state = 0, .external_lex_state = 4},
  [1992] = {.lex_state = 0, .external_lex_state = 4},
  [1993] = {.lex_state = 0, .external_lex_state = 5},
  [1994] = {.lex_state = 10, .external_lex_state = 4},
  [1995] = {.lex_state = 12, .external_lex_state = 4},
  [1996] = {.lex_state = 10, .external_lex_state = 4},
  [1997] = {.lex_state = 0, .external_lex_state = 4},
  [1998] = {.lex_state = 59, .external_lex_state = 4},
  [1999] = {.lex_state = 14, .external_lex_state = 4},
  [2000] = {.lex_state = 66, .external_lex_state = 4},
  [2001] = {.lex_state = 58, .external_lex_state = 4},
  [2002] = {.lex_state = 58, .external_lex_state = 4},
  [2003] = {.lex_state = 0, .external_lex_state = 4},
  [2004] = {.lex_state = 61, .external_lex_state = 4},
  [2005] = {.lex_state = 0, .external_lex_state = 5},
  [2006] = {.lex_state = 59, .external_lex_state = 4},
  [2007] = {.lex_state = 0, .external_lex_state = 4},
  [2008] = {.lex_state = 10, .external_lex_state = 5},
  [2009] = {.lex_state = 13, .external_lex_state = 4},
  [2010] = {.lex_state = 60, .external_lex_state = 4},
  [2011] = {.lex_state = 0, .external_lex_state = 4},
  [2012] = {.lex_state = 0, .external_lex_state = 5},
  [2013] = {.lex_state = 10, .external_lex_state = 5},
  [2014] = {.lex_state = 0, .external_lex_state = 4},
  [2015] = {.lex_state = 0, .external_lex_state = 4},
  [2016] = {.lex_state = 52, .external_lex_state = 4},
  [2017] = {.lex_state = 52, .external_lex_state = 4},
  [2018] = {.lex_state = 0, .external_lex_state = 4},
  [2019] = {.lex_state = 52, .external_lex_state = 4},
  [2020] = {.lex_state = 52, .external_lex_state = 4},
  [2021] = {.lex_state = 66, .external_lex_state = 4},
  [2022] = {.lex_state = 52, .external_lex_state = 4},
  [2023] = {.lex_state = 0, .external_lex_state = 4},
  [2024] = {.lex_state = 0, .external_lex_state = 4},
  [2025] = {.lex_state = 52, .external_lex_state = 4},
  [2026] = {.lex_state = 59, .external_lex_state = 4},
  [2027] = {.lex_state = 66, .external_lex_state = 4},
  [2028] = {.lex_state = 0, .external_lex_state = 4},
  [2029] = {.lex_state = 0, .external_lex_state = 4},
  [2030] = {.lex_state = 58, .external_lex_state = 4},
  [2031] = {.lex_state = 0, .external_lex_state = 5},
  [2032] = {.lex_state = 0, .external_lex_state = 4},
  [2033] = {.lex_state = 0, .external_lex_state = 4},
  [2034] = {.lex_state = 52, .external_lex_state = 4},
  [2035] = {.lex_state = 10, .external_lex_state = 4},
  [2036] = {.lex_state = 0, .external_lex_state = 5},
  [2037] = {.lex_state = 10, .external_lex_state = 4},
  [2038] = {.lex_state = 52, .external_lex_state = 4},
  [2039] = {.lex_state = 0, .external_lex_state = 4},
  [2040] = {.lex_state = 0, .external_lex_state = 4},
  [2041] = {.lex_state = 59, .external_lex_state = 4},
  [2042] = {.lex_state = 0, .external_lex_state = 4},
  [2043] = {.lex_state = 0, .external_lex_state = 4},
  [2044] = {.lex_state = 52, .external_lex_state = 4},
  [2045] = {.lex_state = 58, .external_lex_state = 4},
  [2046] = {.lex_state = 66, .external_lex_state = 4},
  [2047] = {.lex_state = 58, .external_lex_state = 4},
  [2048] = {.lex_state = 0, .external_lex_state = 4},
  [2049] = {.lex_state = 59, .external_lex_state = 4},
  [2050] = {.lex_state = 0, .external_lex_state = 4},
  [2051] = {.lex_state = 52, .external_lex_state = 4},
  [2052] = {.lex_state = 0, .external_lex_state = 5},
  [2053] = {.lex_state = 0, .external_lex_state = 5},
  [2054] = {.lex_state = 0, .external_lex_state = 4},
  [2055] = {.lex_state = 52, .external_lex_state = 4},
  [2056] = {.lex_state = 60, .external_lex_state = 4},
  [2057] = {.lex_state = 0, .external_lex_state = 4},
  [2058] = {.lex_state = 0, .external_lex_state = 4},
  [2059] = {.lex_state = 0, .external_lex_state = 4},
  [2060] = {.lex_state = 10, .external_lex_state = 5},
  [2061] = {.lex_state = 60, .external_lex_state = 4},
  [2062] = {.lex_state = 10, .external_lex_state = 4},
  [2063] = {.lex_state = 0, .external_lex_state = 4},
  [2064] = {.lex_state = 58, .external_lex_state = 4},
  [2065] = {.lex_state = 0, .external_lex_state = 5},
  [2066] = {.lex_state = 0, .external_lex_state = 4},
  [2067] = {.lex_state = 59, .external_lex_state = 4},
  [2068] = {.lex_state = 66, .external_lex_state = 4},
  [2069] = {.lex_state = 60, .external_lex_state = 4},
  [2070] = {.lex_state = 0, .external_lex_state = 5},
  [2071] = {.lex_state = 52, .external_lex_state = 4},
  [2072] = {.lex_state = 0, .external_lex_state = 5},
  [2073] = {.lex_state = 59, .external_lex_state = 4},
  [2074] = {.lex_state = 52, .external_lex_state = 4},
  [2075] = {.lex_state = 10, .external_lex_state = 5},
  [2076] = {.lex_state = 0, .external_lex_state = 5},
  [2077] = {.lex_state = 10, .external_lex_state = 4},
  [2078] = {.lex_state = 10, .external_lex_state = 4},
  [2079] = {.lex_state = 52, .external_lex_state = 4},
  [2080] = {.lex_state = 0, .external_lex_state = 5},
  [2081] = {.lex_state = 67, .external_lex_state = 4},
  [2082] = {.lex_state = 60, .external_lex_state = 4},
  [2083] = {.lex_state = 0, .external_lex_state = 5},
  [2084] = {.lex_state = 52, .external_lex_state = 4},
  [2085] = {.lex_state = 0, .external_lex_state = 5},
  [2086] = {.lex_state = 66, .external_lex_state = 4},
  [2087] = {.lex_state = 0, .external_lex_state = 4},
  [2088] = {.lex_state = 52, .external_lex_state = 4},
  [2089] = {.lex_state = 0, .external_lex_state = 4},
  [2090] = {.lex_state = 0, .external_lex_state = 5},
  [2091] = {.lex_state = 66, .external_lex_state = 4},
  [2092] = {.lex_state = 0, .external_lex_state = 5},
  [2093] = {.lex_state = 0, .external_lex_state = 5},
  [2094] = {.lex_state = 0, .external_lex_state = 4},
  [2095] = {.lex_state = 0, .external_lex_state = 5},
  [2096] = {.lex_state = 0, .external_lex_state = 4},
  [2097] = {.lex_state = 874, .external_lex_state = 4},
  [2098] = {.lex_state = 59, .external_lex_state = 4},
  [2099] = {.lex_state = 0, .external_lex_state = 4},
  [2100] = {.lex_state = 0, .external_lex_state = 4},
  [2101] = {.lex_state = 52, .external_lex_state = 4},
  [2102] = {.lex_state = 58, .external_lex_state = 4},
  [2103] = {.lex_state = 10, .external_lex_state = 5},
  [2104] = {.lex_state = 0, .external_lex_state = 3},
  [2105] = {.lex_state = 0, .external_lex_state = 4},
  [2106] = {.lex_state = 10, .external_lex_state = 5},
  [2107] = {.lex_state = 10, .external_lex_state = 5},
  [2108] = {.lex_state = 52, .external_lex_state = 4},
  [2109] = {.lex_state = 10, .external_lex_state = 5},
  [2110] = {.lex_state = 0, .external_lex_state = 5},
  [2111] = {.lex_state = 0, .external_lex_state = 5},
  [2112] = {.lex_state = 0, .external_lex_state = 5},
  [2113] = {.lex_state = 59, .external_lex_state = 4},
  [2114] = {.lex_state = 0, .external_lex_state = 4},
  [2115] = {.lex_state = 60, .external_lex_state = 4},
  [2116] = {.lex_state = 58, .external_lex_state = 4},
  [2117] = {.lex_state = 58, .external_lex_state = 4},
  [2118] = {.lex_state = 0, .external_lex_state = 4},
  [2119] = {.lex_state = 0, .external_lex_state = 5},
  [2120] = {.lex_state = 52, .external_lex_state = 4},
  [2121] = {.lex_state = 60, .external_lex_state = 4},
  [2122] = {.lex_state = 58, .external_lex_state = 4},
  [2123] = {.lex_state = 0, .external_lex_state = 5},
  [2124] = {.lex_state = 0, .external_lex_state = 4},
  [2125] = {.lex_state = 66, .external_lex_state = 4},
  [2126] = {.lex_state = 67, .external_lex_state = 4},
  [2127] = {.lex_state = 59, .external_lex_state = 4},
  [2128] = {.lex_state = 60, .external_lex_state = 4},
  [2129] = {.lex_state = 60, .external_lex_state = 4},
  [2130] = {.lex_state = 0, .external_lex_state = 5},
  [2131] = {.lex_state = 10, .external_lex_state = 4},
  [2132] = {.lex_state = 67, .external_lex_state = 4},
  [2133] = {.lex_state = 60, .external_lex_state = 4},
  [2134] = {.lex_state = 14, .external_lex_state = 4},
  [2135] = {.lex_state = 0, .external_lex_state = 4},
  [2136] = {.lex_state = 0, .external_lex_state = 4},
  [2137] = {.lex_state = 0, .external_lex_state = 4},
  [2138] = {.lex_state = 0, .external_lex_state = 4},
  [2139] = {.lex_state = 11, .external_lex_state = 4},
  [2140] = {.lex_state = 0, .external_lex_state = 4},
  [2141] = {.lex_state = 0, .external_lex_state = 3},
  [2142] = {.lex_state = 52, .external_lex_state = 4},
  [2143] = {.lex_state = 0, .external_lex_state = 4},
  [2144] = {.lex_state = 874, .external_lex_state = 4},
  [2145] = {.lex_state = 0, .external_lex_state = 4},
  [2146] = {.lex_state = 52, .external_lex_state = 4},
  [2147] = {.lex_state = 0, .external_lex_state = 5},
  [2148] = {.lex_state = 0, .external_lex_state = 5},
  [2149] = {.lex_state = 0, .external_lex_state = 5},
  [2150] = {.lex_state = 0, .external_lex_state = 4},
  [2151] = {.lex_state = 0, .external_lex_state = 5},
  [2152] = {.lex_state = 12, .external_lex_state = 4},
  [2153] = {.lex_state = 0, .external_lex_state = 4},
  [2154] = {.lex_state = 66, .external_lex_state = 4},
  [2155] = {.lex_state = 0, .external_lex_state = 5},
  [2156] = {.lex_state = 0, .external_lex_state = 4},
  [2157] = {.lex_state = 0, .external_lex_state = 4},
  [2158] = {.lex_state = 0, .external_lex_state = 5},
  [2159] = {.lex_state = 0, .external_lex_state = 5},
  [2160] = {.lex_state = 0, .external_lex_state = 5},
  [2161] = {.lex_state = 52, .external_lex_state = 4},
  [2162] = {.lex_state = 61, .external_lex_state = 4},
  [2163] = {.lex_state = 0, .external_lex_state = 3},
  [2164] = {.lex_state = 59, .external_lex_state = 4},
  [2165] = {.lex_state = 0, .external_lex_state = 4},
  [2166] = {.lex_state = 0, .external_lex_state = 4},
  [2167] = {.lex_state = 52, .external_lex_state = 4},
  [2168] = {.lex_state = 58, .external_lex_state = 4},
  [2169] = {.lex_state = 0, .external_lex_state = 5},
  [2170] = {.lex_state = 0, .external_lex_state = 4},
  [2171] = {.lex_state = 0, .external_lex_state = 4},
  [2172] = {.lex_state = 0, .external_lex_state = 5},
  [2173] = {.lex_state = 59, .external_lex_state = 4},
  [2174] = {.lex_state = 58, .external_lex_state = 4},
  [2175] = {.lex_state = 0, .external_lex_state = 4},
  [2176] = {.lex_state = 0, .external_lex_state = 4},
  [2177] = {.lex_state = 59, .external_lex_state = 4},
  [2178] = {.lex_state = 0, .external_lex_state = 4},
  [2179] = {.lex_state = 58, .external_lex_state = 4},
  [2180] = {.lex_state = 0, .external_lex_state = 5},
  [2181] = {.lex_state = 58, .external_lex_state = 4},
  [2182] = {.lex_state = 52, .external_lex_state = 4},
  [2183] = {.lex_state = 0, .external_lex_state = 4},
  [2184] = {.lex_state = 60, .external_lex_state = 4},
  [2185] = {.lex_state = 59, .external_lex_state = 4},
  [2186] = {.lex_state = 0, .external_lex_state = 4},
  [2187] = {.lex_state = 0, .external_lex_state = 3},
  [2188] = {.lex_state = 0, .external_lex_state = 4},
  [2189] = {.lex_state = 52, .external_lex_state = 4},
  [2190] = {.lex_state = 52, .external_lex_state = 4},
  [2191] = {.lex_state = 0, .external_lex_state = 5},
  [2192] = {.lex_state = 10, .external_lex_state = 4},
  [2193] = {.lex_state = 0, .external_lex_state = 5},
  [2194] = {.lex_state = 0, .external_lex_state = 5},
  [2195] = {.lex_state = 0, .external_lex_state = 4},
  [2196] = {.lex_state = 0, .external_lex_state = 5},
  [2197] = {.lex_state = 67, .external_lex_state = 4},
  [2198] = {.lex_state = 0, .external_lex_state = 5},
  [2199] = {.lex_state = 0, .external_lex_state = 4},
  [2200] = {.lex_state = 52, .external_lex_state = 4},
  [2201] = {.lex_state = 0, .external_lex_state = 5},
  [2202] = {.lex_state = 0, .external_lex_state = 5},
  [2203] = {.lex_state = 52, .external_lex_state = 4},
  [2204] = {.lex_state = 0, .external_lex_state = 5},
  [2205] = {.lex_state = 0, .external_lex_state = 4},
  [2206] = {.lex_state = 0, .external_lex_state = 5},
  [2207] = {.lex_state = 10, .external_lex_state = 5},
  [2208] = {.lex_state = 59, .external_lex_state = 4},
  [2209] = {.lex_state = 0, .external_lex_state = 4},
  [2210] = {.lex_state = 60, .external_lex_state = 4},
  [2211] = {.lex_state = 0, .external_lex_state = 4},
  [2212] = {.lex_state = 58, .external_lex_state = 4},
  [2213] = {.lex_state = 52, .external_lex_state = 4},
  [2214] = {.lex_state = 52, .external_lex_state = 4},
  [2215] = {.lex_state = 0, .external_lex_state = 4},
  [2216] = {.lex_state = 10, .external_lex_state = 5},
  [2217] = {.lex_state = 59, .external_lex_state = 4},
  [2218] = {.lex_state = 0, .external_lex_state = 5},
  [2219] = {.lex_state = 11, .external_lex_state = 4},
  [2220] = {.lex_state = 0, .external_lex_state = 4},
  [2221] = {.lex_state = 58, .external_lex_state = 4},
  [2222] = {.lex_state = 0, .external_lex_state = 4},
  [2223] = {.lex_state = 58, .external_lex_state = 4},
  [2224] = {.lex_state = 0, .external_lex_state = 4},
  [2225] = {.lex_state = 59, .external_lex_state = 4},
  [2226] = {.lex_state = 52, .external_lex_state = 4},
  [2227] = {.lex_state = 10, .external_lex_state = 4},
  [2228] = {.lex_state = 0, .external_lex_state = 5},
  [2229] = {.lex_state = 0, .external_lex_state = 4},
  [2230] = {.lex_state = 0, .external_lex_state = 4},
  [2231] = {.lex_state = 0, .external_lex_state = 4},
  [2232] = {.lex_state = 0, .external_lex_state = 4},
  [2233] = {.lex_state = 0, .external_lex_state = 4},
  [2234] = {.lex_state = 52, .external_lex_state = 4},
  [2235] = {.lex_state = 10, .external_lex_state = 5},
  [2236] = {.lex_state = 52, .external_lex_state = 4},
  [2237] = {.lex_state = 0, .external_lex_state = 4},
  [2238] = {.lex_state = 10, .external_lex_state = 5},
  [2239] = {.lex_state = 10, .external_lex_state = 5},
  [2240] = {.lex_state = 0, .external_lex_state = 5},
  [2241] = {.lex_state = 0, .external_lex_state = 5},
  [2242] = {.lex_state = 0, .external_lex_state = 5},
  [2243] = {.lex_state = 10, .external_lex_state = 4},
  [2244] = {.lex_state = 10, .external_lex_state = 4},
  [2245] = {.lex_state = 10, .external_lex_state = 5},
  [2246] = {.lex_state = 0, .external_lex_state = 4},
  [2247] = {.lex_state = 0, .external_lex_state = 5},
  [2248] = {.lex_state = 10, .external_lex_state = 4},
  [2249] = {.lex_state = 0, .external_lex_state = 5},
  [2250] = {.lex_state = 0, .external_lex_state = 5},
  [2251] = {.lex_state = 10, .external_lex_state = 4},
  [2252] = {.lex_state = 0, .external_lex_state = 5},
  [2253] = {.lex_state = 10, .external_lex_state = 4},
  [2254] = {.lex_state = 0, .external_lex_state = 5},
  [2255] = {.lex_state = 0, .external_lex_state = 5},
  [2256] = {.lex_state = 0, .external_lex_state = 4},
  [2257] = {.lex_state = 0, .external_lex_state = 5},
  [2258] = {.lex_state = 14, .external_lex_state = 4},
  [2259] = {.lex_state = 10, .external_lex_state = 4},
  [2260] = {.lex_state = 10, .external_lex_state = 4},
  [2261] = {.lex_state = 10, .external_lex_state = 4},
  [2262] = {.lex_state = 0, .external_lex_state = 5},
  [2263] = {.lex_state = 0, .external_lex_state = 5},
  [2264] = {.lex_state = 0, .external_lex_state = 5},
  [2265] = {.lex_state = 0, .external_lex_state = 4},
  [2266] = {.lex_state = 874, .external_lex_state = 4},
  [2267] = {.lex_state = 874, .external_lex_state = 4},
  [2268] = {.lex_state = 0, .external_lex_state = 5},
  [2269] = {.lex_state = 0, .external_lex_state = 4},
  [2270] = {.lex_state = 0, .external_lex_state = 4},
  [2271] = {.lex_state = 0, .external_lex_state = 4},
  [2272] = {.lex_state = 0, .external_lex_state = 4},
  [2273] = {.lex_state = 0, .external_lex_state = 4},
  [2274] = {.lex_state = 10, .external_lex_state = 4},
  [2275] = {.lex_state = 0, .external_lex_state = 4},
  [2276] = {.lex_state = 0, .external_lex_state = 5},
  [2277] = {.lex_state = 0, .external_lex_state = 4},
  [2278] = {.lex_state = 10, .external_lex_state = 4},
  [2279] = {.lex_state = 1, .external_lex_state = 4},
  [2280] = {.lex_state = 0, .external_lex_state = 5},
  [2281] = {.lex_state = 0, .external_lex_state = 5},
  [2282] = {.lex_state = 0, .external_lex_state = 4},
  [2283] = {.lex_state = 0, .external_lex_state = 4},
  [2284] = {.lex_state = 0, .external_lex_state = 4},
  [2285] = {.lex_state = 10, .external_lex_state = 4},
  [2286] = {.lex_state = 10, .external_lex_state = 4},
  [2287] = {.lex_state = 0, .external_lex_state = 4},
  [2288] = {.lex_state = 10, .external_lex_state = 4},
  [2289] = {.lex_state = 0, .external_lex_state = 5},
  [2290] = {.lex_state = 0, .external_lex_state = 4},
  [2291] = {.lex_state = 0, .external_lex_state = 5},
  [2292] = {.lex_state = 0, .external_lex_state = 5},
  [2293] = {.lex_state = 10, .external_lex_state = 4},
  [2294] = {.lex_state = 0, .external_lex_state = 4},
  [2295] = {.lex_state = 0, .external_lex_state = 5},
  [2296] = {.lex_state = 0, .external_lex_state = 4},
  [2297] = {.lex_state = 0, .external_lex_state = 5},
  [2298] = {.lex_state = 0, .external_lex_state = 5},
  [2299] = {.lex_state = 0, .external_lex_state = 5},
  [2300] = {.lex_state = 21, .external_lex_state = 4},
  [2301] = {.lex_state = 0, .external_lex_state = 5},
  [2302] = {.lex_state = 10, .external_lex_state = 4},
  [2303] = {.lex_state = 10, .external_lex_state = 4},
  [2304] = {.lex_state = 10, .external_lex_state = 5},
  [2305] = {.lex_state = 0, .external_lex_state = 5},
  [2306] = {.lex_state = 10, .external_lex_state = 4},
  [2307] = {.lex_state = 0, .external_lex_state = 4},
  [2308] = {.lex_state = 10, .external_lex_state = 4},
  [2309] = {.lex_state = 10, .external_lex_state = 4},
  [2310] = {.lex_state = 21, .external_lex_state = 4},
  [2311] = {.lex_state = 0, .external_lex_state = 5},
  [2312] = {.lex_state = 0, .external_lex_state = 5},
  [2313] = {.lex_state = 10, .external_lex_state = 4},
  [2314] = {.lex_state = 10, .external_lex_state = 4},
  [2315] = {.lex_state = 0, .external_lex_state = 5},
  [2316] = {.lex_state = 0, .external_lex_state = 5},
  [2317] = {.lex_state = 0, .external_lex_state = 4},
  [2318] = {.lex_state = 0, .external_lex_state = 5},
  [2319] = {.lex_state = 0, .external_lex_state = 4},
  [2320] = {.lex_state = 0, .external_lex_state = 5},
  [2321] = {.lex_state = 14, .external_lex_state = 4},
  [2322] = {.lex_state = 0, .external_lex_state = 4},
  [2323] = {.lex_state = 21, .external_lex_state = 4},
  [2324] = {.lex_state = 0, .external_lex_state = 9},
  [2325] = {.lex_state = 10, .external_lex_state = 4},
  [2326] = {.lex_state = 10, .external_lex_state = 5},
  [2327] = {.lex_state = 0, .external_lex_state = 4},
  [2328] = {.lex_state = 0, .external_lex_state = 5},
  [2329] = {.lex_state = 10, .external_lex_state = 4},
  [2330] = {.lex_state = 10, .external_lex_state = 4},
  [2331] = {.lex_state = 10, .external_lex_state = 4},
  [2332] = {.lex_state = 10, .external_lex_state = 4},
  [2333] = {.lex_state = 10, .external_lex_state = 4},
  [2334] = {.lex_state = 10, .external_lex_state = 4},
  [2335] = {.lex_state = 0, .external_lex_state = 5},
  [2336] = {.lex_state = 0, .external_lex_state = 5},
  [2337] = {.lex_state = 0, .external_lex_state = 4},
  [2338] = {.lex_state = 14, .external_lex_state = 4},
  [2339] = {.lex_state = 10, .external_lex_state = 4},
  [2340] = {.lex_state = 10, .external_lex_state = 4},
  [2341] = {.lex_state = 0, .external_lex_state = 5},
  [2342] = {.lex_state = 0, .external_lex_state = 4},
  [2343] = {.lex_state = 0, .external_lex_state = 4},
  [2344] = {.lex_state = 0, .external_lex_state = 4},
  [2345] = {.lex_state = 0, .external_lex_state = 4},
  [2346] = {.lex_state = 0, .external_lex_state = 4},
  [2347] = {.lex_state = 10, .external_lex_state = 4},
  [2348] = {.lex_state = 0, .external_lex_state = 5},
  [2349] = {.lex_state = 0, .external_lex_state = 4},
  [2350] = {.lex_state = 67, .external_lex_state = 4},
  [2351] = {.lex_state = 0, .external_lex_state = 4},
  [2352] = {.lex_state = 0, .external_lex_state = 4},
  [2353] = {.lex_state = 0, .external_lex_state = 4},
  [2354] = {.lex_state = 10, .external_lex_state = 4},
  [2355] = {.lex_state = 0, .external_lex_state = 4},
  [2356] = {.lex_state = 0, .external_lex_state = 5},
  [2357] = {.lex_state = 0, .external_lex_state = 5},
  [2358] = {.lex_state = 0, .external_lex_state = 5},
  [2359] = {.lex_state = 0, .external_lex_state = 5},
  [2360] = {.lex_state = 0, .external_lex_state = 5},
  [2361] = {.lex_state = 0, .external_lex_state = 4},
  [2362] = {.lex_state = 0, .external_lex_state = 4},
  [2363] = {.lex_state = 0, .external_lex_state = 5},
  [2364] = {.lex_state = 0, .external_lex_state = 4},
  [2365] = {.lex_state = 0, .external_lex_state = 4},
  [2366] = {.lex_state = 0, .external_lex_state = 5},
  [2367] = {.lex_state = 0, .external_lex_state = 5},
  [2368] = {.lex_state = 0, .external_lex_state = 5},
  [2369] = {.lex_state = 0, .external_lex_state = 5},
  [2370] = {.lex_state = 0, .external_lex_state = 5},
  [2371] = {.lex_state = 0, .external_lex_state = 5},
  [2372] = {.lex_state = 0, .external_lex_state = 4},
  [2373] = {.lex_state = 0, .external_lex_state = 4},
  [2374] = {.lex_state = 0, .external_lex_state = 5},
  [2375] = {.lex_state = 874, .external_lex_state = 4},
  [2376] = {.lex_state = 0, .external_lex_state = 5},
  [2377] = {.lex_state = 0, .external_lex_state = 4},
  [2378] = {.lex_state = 0, .external_lex_state = 4},
  [2379] = {.lex_state = 0, .external_lex_state = 4},
  [2380] = {.lex_state = 0, .external_lex_state = 5},
  [2381] = {.lex_state = 0, .external_lex_state = 4},
  [2382] = {.lex_state = 0, .external_lex_state = 4},
  [2383] = {.lex_state = 0, .external_lex_state = 4},
  [2384] = {.lex_state = 0, .external_lex_state = 5},
  [2385] = {.lex_state = 0, .external_lex_state = 5},
  [2386] = {.lex_state = 0, .external_lex_state = 5},
  [2387] = {.lex_state = 0, .external_lex_state = 5},
  [2388] = {.lex_state = 0, .external_lex_state = 5},
  [2389] = {.lex_state = 0, .external_lex_state = 4},
  [2390] = {.lex_state = 0, .external_lex_state = 5},
  [2391] = {.lex_state = 0, .external_lex_state = 5},
  [2392] = {.lex_state = 0, .external_lex_state = 5},
  [2393] = {.lex_state = 0, .external_lex_state = 5},
  [2394] = {.lex_state = 0, .external_lex_state = 4},
  [2395] = {.lex_state = 0, .external_lex_state = 4},
  [2396] = {.lex_state = 0, .external_lex_state = 4},
  [2397] = {.lex_state = 0, .external_lex_state = 5},
  [2398] = {.lex_state = 0, .external_lex_state = 5},
  [2399] = {.lex_state = 0, .external_lex_state = 5},
  [2400] = {.lex_state = 0, .external_lex_state = 5},
  [2401] = {.lex_state = 0, .external_lex_state = 5},
  [2402] = {.lex_state = 0, .external_lex_state = 5},
  [2403] = {.lex_state = 0, .external_lex_state = 4},
  [2404] = {.lex_state = 0, .external_lex_state = 4},
  [2405] = {.lex_state = 0, .external_lex_state = 4},
  [2406] = {.lex_state = 0, .external_lex_state = 4},
  [2407] = {.lex_state = 0, .external_lex_state = 4},
  [2408] = {.lex_state = 0, .external_lex_state = 5},
  [2409] = {.lex_state = 0, .external_lex_state = 4},
  [2410] = {.lex_state = 0, .external_lex_state = 5},
  [2411] = {.lex_state = 0, .external_lex_state = 4},
  [2412] = {.lex_state = 0, .external_lex_state = 5},
  [2413] = {.lex_state = 0, .external_lex_state = 5},
  [2414] = {.lex_state = 0, .external_lex_state = 5},
  [2415] = {.lex_state = 10, .external_lex_state = 4},
  [2416] = {.lex_state = 0, .external_lex_state = 5},
  [2417] = {.lex_state = 0, .external_lex_state = 5},
  [2418] = {.lex_state = 0, .external_lex_state = 5},
  [2419] = {.lex_state = 0, .external_lex_state = 5},
  [2420] = {.lex_state = 0, .external_lex_state = 4},
  [2421] = {.lex_state = 0, .external_lex_state = 5},
  [2422] = {.lex_state = 0, .external_lex_state = 5},
  [2423] = {.lex_state = 0, .external_lex_state = 5},
  [2424] = {.lex_state = 0, .external_lex_state = 5},
  [2425] = {.lex_state = 0, .external_lex_state = 4},
  [2426] = {.lex_state = 0, .external_lex_state = 4},
  [2427] = {.lex_state = 0, .external_lex_state = 4},
  [2428] = {.lex_state = 0, .external_lex_state = 4},
  [2429] = {.lex_state = 0, .external_lex_state = 4},
  [2430] = {.lex_state = 0, .external_lex_state = 5},
  [2431] = {.lex_state = 0, .external_lex_state = 4},
  [2432] = {.lex_state = 0, .external_lex_state = 4},
  [2433] = {.lex_state = 0, .external_lex_state = 4},
  [2434] = {.lex_state = 0, .external_lex_state = 4},
  [2435] = {.lex_state = 0, .external_lex_state = 5},
  [2436] = {.lex_state = 0, .external_lex_state = 4},
  [2437] = {.lex_state = 0, .external_lex_state = 5},
  [2438] = {.lex_state = 0, .external_lex_state = 5},
  [2439] = {.lex_state = 0, .external_lex_state = 5},
  [2440] = {.lex_state = 0, .external_lex_state = 5},
  [2441] = {.lex_state = 0, .external_lex_state = 5},
  [2442] = {.lex_state = 0, .external_lex_state = 5},
  [2443] = {.lex_state = 0, .external_lex_state = 5},
  [2444] = {.lex_state = 0, .external_lex_state = 4},
  [2445] = {.lex_state = 0, .external_lex_state = 5},
  [2446] = {.lex_state = 0, .external_lex_state = 5},
  [2447] = {.lex_state = 0, .external_lex_state = 4},
  [2448] = {.lex_state = 0, .external_lex_state = 5},
  [2449] = {.lex_state = 0, .external_lex_state = 5},
  [2450] = {.lex_state = 0, .external_lex_state = 5},
  [2451] = {.lex_state = 0, .external_lex_state = 5},
  [2452] = {.lex_state = 0, .external_lex_state = 5},
  [2453] = {.lex_state = 10, .external_lex_state = 4},
  [2454] = {.lex_state = 0, .external_lex_state = 5},
  [2455] = {.lex_state = 10, .external_lex_state = 4},
  [2456] = {.lex_state = 0, .external_lex_state = 5},
  [2457] = {.lex_state = 0, .external_lex_state = 5},
  [2458] = {.lex_state = 0, .external_lex_state = 5},
  [2459] = {.lex_state = 0, .external_lex_state = 5},
  [2460] = {.lex_state = 10, .external_lex_state = 4},
  [2461] = {.lex_state = 0, .external_lex_state = 4},
  [2462] = {.lex_state = 0, .external_lex_state = 5},
  [2463] = {.lex_state = 0, .external_lex_state = 5},
  [2464] = {.lex_state = 0, .external_lex_state = 5},
  [2465] = {.lex_state = 10, .external_lex_state = 4},
  [2466] = {.lex_state = 0, .external_lex_state = 5},
  [2467] = {.lex_state = 0, .external_lex_state = 5},
  [2468] = {.lex_state = 0, .external_lex_state = 5},
  [2469] = {.lex_state = 0, .external_lex_state = 5},
  [2470] = {.lex_state = 0, .external_lex_state = 5},
  [2471] = {.lex_state = 0, .external_lex_state = 5},
  [2472] = {.lex_state = 0, .external_lex_state = 5},
  [2473] = {.lex_state = 0, .external_lex_state = 4},
  [2474] = {.lex_state = 0, .external_lex_state = 4},
  [2475] = {.lex_state = 0, .external_lex_state = 5},
  [2476] = {.lex_state = 0, .external_lex_state = 5},
  [2477] = {.lex_state = 0, .external_lex_state = 5},
  [2478] = {.lex_state = 0, .external_lex_state = 5},
  [2479] = {.lex_state = 0, .external_lex_state = 5},
  [2480] = {.lex_state = 0, .external_lex_state = 5},
  [2481] = {.lex_state = 0, .external_lex_state = 4},
  [2482] = {.lex_state = 0, .external_lex_state = 4},
  [2483] = {.lex_state = 0, .external_lex_state = 5},
  [2484] = {.lex_state = 0, .external_lex_state = 5},
  [2485] = {.lex_state = 0, .external_lex_state = 5},
  [2486] = {.lex_state = 874, .external_lex_state = 4},
  [2487] = {.lex_state = 0, .external_lex_state = 5},
  [2488] = {.lex_state = 0, .external_lex_state = 5},
  [2489] = {.lex_state = 0, .external_lex_state = 5},
  [2490] = {.lex_state = 0, .external_lex_state = 5},
  [2491] = {.lex_state = 10, .external_lex_state = 4},
  [2492] = {.lex_state = 0, .external_lex_state = 5},
  [2493] = {.lex_state = 0, .external_lex_state = 4},
  [2494] = {.lex_state = 0, .external_lex_state = 4},
  [2495] = {.lex_state = 0, .external_lex_state = 4},
  [2496] = {.lex_state = 0, .external_lex_state = 4},
  [2497] = {.lex_state = 0, .external_lex_state = 4},
  [2498] = {.lex_state = 0, .external_lex_state = 5},
  [2499] = {.lex_state = 0, .external_lex_state = 4},
  [2500] = {.lex_state = 10, .external_lex_state = 4},
  [2501] = {.lex_state = 0, .external_lex_state = 5},
  [2502] = {.lex_state = 0, .external_lex_state = 5},
  [2503] = {.lex_state = 0, .external_lex_state = 5},
  [2504] = {.lex_state = 0, .external_lex_state = 5},
  [2505] = {.lex_state = 0, .external_lex_state = 5},
  [2506] = {.lex_state = 0, .external_lex_state = 5},
  [2507] = {.lex_state = 0, .external_lex_state = 5},
  [2508] = {.lex_state = 0, .external_lex_state = 5},
  [2509] = {.lex_state = 0, .external_lex_state = 5},
  [2510] = {.lex_state = 0, .external_lex_state = 4},
  [2511] = {.lex_state = 0, .external_lex_state = 5},
  [2512] = {.lex_state = 55, .external_lex_state = 4},
  [2513] = {.lex_state = 0, .external_lex_state = 5},
  [2514] = {.lex_state = 0, .external_lex_state = 5},
  [2515] = {.lex_state = 10, .external_lex_state = 4},
  [2516] = {.lex_state = 0, .external_lex_state = 5},
  [2517] = {.lex_state = 0, .external_lex_state = 5},
  [2518] = {.lex_state = 0, .external_lex_state = 5},
  [2519] = {.lex_state = 0, .external_lex_state = 5},
  [2520] = {.lex_state = 0, .external_lex_state = 5},
  [2521] = {.lex_state = 0, .external_lex_state = 5},
  [2522] = {.lex_state = 0, .external_lex_state = 5},
  [2523] = {.lex_state = 0, .external_lex_state = 5},
  [2524] = {.lex_state = 0, .external_lex_state = 4},
  [2525] = {.lex_state = 0, .external_lex_state = 4},
  [2526] = {.lex_state = 0, .external_lex_state = 5},
  [2527] = {.lex_state = 0, .external_lex_state = 4},
  [2528] = {.lex_state = 0, .external_lex_state = 4},
  [2529] = {.lex_state = 0, .external_lex_state = 5},
  [2530] = {.lex_state = 0, .external_lex_state = 5},
  [2531] = {.lex_state = 71, .external_lex_state = 4},
  [2532] = {.lex_state = 0, .external_lex_state = 5},
  [2533] = {.lex_state = 0, .external_lex_state = 4},
  [2534] = {.lex_state = 0, .external_lex_state = 5},
  [2535] = {.lex_state = 0, .external_lex_state = 4},
  [2536] = {.lex_state = 0, .external_lex_state = 5},
  [2537] = {.lex_state = 0, .external_lex_state = 5},
  [2538] = {.lex_state = 0, .external_lex_state = 5},
  [2539] = {.lex_state = 0, .external_lex_state = 5},
  [2540] = {.lex_state = 0, .external_lex_state = 5},
  [2541] = {.lex_state = 0, .external_lex_state = 5},
  [2542] = {.lex_state = 0, .external_lex_state = 5},
  [2543] = {.lex_state = 0, .external_lex_state = 5},
  [2544] = {.lex_state = 0, .external_lex_state = 5},
  [2545] = {.lex_state = 0, .external_lex_state = 5},
  [2546] = {.lex_state = 0, .external_lex_state = 5},
  [2547] = {.lex_state = 0, .external_lex_state = 4},
  [2548] = {.lex_state = 0, .external_lex_state = 5},
  [2549] = {.lex_state = 10, .external_lex_state = 4},
  [2550] = {.lex_state = 0, .external_lex_state = 5},
  [2551] = {.lex_state = 0, .external_lex_state = 5},
  [2552] = {.lex_state = 0, .external_lex_state = 4},
  [2553] = {.lex_state = 10, .external_lex_state = 4},
  [2554] = {.lex_state = 0, .external_lex_state = 4},
  [2555] = {.lex_state = 0, .external_lex_state = 5},
  [2556] = {.lex_state = 0, .external_lex_state = 5},
  [2557] = {.lex_state = 0, .external_lex_state = 5},
  [2558] = {.lex_state = 0, .external_lex_state = 4},
  [2559] = {.lex_state = 0, .external_lex_state = 4},
  [2560] = {.lex_state = 0, .external_lex_state = 5},
  [2561] = {.lex_state = 0, .external_lex_state = 5},
  [2562] = {.lex_state = 0, .external_lex_state = 5},
  [2563] = {.lex_state = 0, .external_lex_state = 4},
  [2564] = {.lex_state = 0, .external_lex_state = 5},
  [2565] = {.lex_state = 0, .external_lex_state = 5},
  [2566] = {.lex_state = 0, .external_lex_state = 5},
  [2567] = {.lex_state = 0, .external_lex_state = 5},
  [2568] = {.lex_state = 0, .external_lex_state = 5},
  [2569] = {.lex_state = 10, .external_lex_state = 4},
  [2570] = {.lex_state = 0, .external_lex_state = 5},
  [2571] = {.lex_state = 10, .external_lex_state = 4},
  [2572] = {.lex_state = 0, .external_lex_state = 5},
  [2573] = {.lex_state = 0, .external_lex_state = 5},
  [2574] = {.lex_state = 0, .external_lex_state = 5},
  [2575] = {.lex_state = 0, .external_lex_state = 5},
  [2576] = {.lex_state = 0, .external_lex_state = 4},
  [2577] = {.lex_state = 0, .external_lex_state = 5},
  [2578] = {.lex_state = 0, .external_lex_state = 5},
  [2579] = {.lex_state = 0, .external_lex_state = 5},
  [2580] = {.lex_state = 0, .external_lex_state = 5},
  [2581] = {.lex_state = 0, .external_lex_state = 5},
  [2582] = {.lex_state = 0, .external_lex_state = 4},
  [2583] = {.lex_state = 0, .external_lex_state = 5},
  [2584] = {.lex_state = 0, .external_lex_state = 5},
  [2585] = {.lex_state = 10, .external_lex_state = 4},
  [2586] = {.lex_state = 0, .external_lex_state = 5},
  [2587] = {.lex_state = 0, .external_lex_state = 4},
  [2588] = {.lex_state = 0, .external_lex_state = 5},
  [2589] = {.lex_state = 0, .external_lex_state = 5},
  [2590] = {.lex_state = 0, .external_lex_state = 4},
  [2591] = {.lex_state = 0, .external_lex_state = 5},
  [2592] = {.lex_state = 0, .external_lex_state = 4},
  [2593] = {.lex_state = 0, .external_lex_state = 5},
  [2594] = {.lex_state = 0, .external_lex_state = 5},
  [2595] = {.lex_state = 10, .external_lex_state = 4},
  [2596] = {.lex_state = 0, .external_lex_state = 5},
  [2597] = {.lex_state = 0, .external_lex_state = 5},
  [2598] = {.lex_state = 0, .external_lex_state = 5},
  [2599] = {.lex_state = 0, .external_lex_state = 5},
  [2600] = {.lex_state = 10, .external_lex_state = 4},
  [2601] = {.lex_state = 0, .external_lex_state = 4},
  [2602] = {.lex_state = 0, .external_lex_state = 5},
  [2603] = {.lex_state = 0, .external_lex_state = 4},
  [2604] = {.lex_state = 0, .external_lex_state = 5},
  [2605] = {.lex_state = 0, .external_lex_state = 5},
  [2606] = {.lex_state = 0, .external_lex_state = 4},
  [2607] = {.lex_state = 0, .external_lex_state = 4},
  [2608] = {.lex_state = 0, .external_lex_state = 5},
  [2609] = {.lex_state = 0, .external_lex_state = 5},
  [2610] = {.lex_state = 10, .external_lex_state = 4},
  [2611] = {.lex_state = 0, .external_lex_state = 5},
  [2612] = {.lex_state = 0, .external_lex_state = 5},
  [2613] = {.lex_state = 0, .external_lex_state = 5},
  [2614] = {.lex_state = 0, .external_lex_state = 5},
  [2615] = {.lex_state = 0, .external_lex_state = 4},
  [2616] = {.lex_state = 0, .external_lex_state = 5},
  [2617] = {.lex_state = 0, .external_lex_state = 5},
  [2618] = {.lex_state = 0, .external_lex_state = 5},
  [2619] = {.lex_state = 874, .external_lex_state = 4},
  [2620] = {.lex_state = 0, .external_lex_state = 5},
  [2621] = {.lex_state = 0, .external_lex_state = 5},
  [2622] = {.lex_state = 0, .external_lex_state = 5},
  [2623] = {.lex_state = 0, .external_lex_state = 4},
  [2624] = {.lex_state = 0, .external_lex_state = 4},
  [2625] = {.lex_state = 0, .external_lex_state = 5},
  [2626] = {.lex_state = 0, .external_lex_state = 5},
  [2627] = {.lex_state = 0, .external_lex_state = 5},
  [2628] = {.lex_state = 0, .external_lex_state = 5},
  [2629] = {.lex_state = 0, .external_lex_state = 4},
  [2630] = {.lex_state = 0, .external_lex_state = 5},
  [2631] = {.lex_state = 0, .external_lex_state = 4},
  [2632] = {.lex_state = 0, .external_lex_state = 5},
  [2633] = {.lex_state = 0, .external_lex_state = 4},
  [2634] = {.lex_state = 0, .external_lex_state = 5},
  [2635] = {.lex_state = 0, .external_lex_state = 5},
  [2636] = {.lex_state = 0, .external_lex_state = 5},
  [2637] = {.lex_state = 10, .external_lex_state = 4},
  [2638] = {.lex_state = 0, .external_lex_state = 4},
  [2639] = {.lex_state = 10, .external_lex_state = 4},
  [2640] = {.lex_state = 0, .external_lex_state = 5},
  [2641] = {.lex_state = 0, .external_lex_state = 4},
  [2642] = {.lex_state = 0, .external_lex_state = 5},
  [2643] = {.lex_state = 0, .external_lex_state = 5},
  [2644] = {.lex_state = 0, .external_lex_state = 5},
  [2645] = {.lex_state = 0, .external_lex_state = 4},
  [2646] = {.lex_state = 10, .external_lex_state = 4},
  [2647] = {.lex_state = 0, .external_lex_state = 5},
  [2648] = {.lex_state = 10, .external_lex_state = 4},
  [2649] = {.lex_state = 0, .external_lex_state = 5},
  [2650] = {.lex_state = 0, .external_lex_state = 4},
  [2651] = {.lex_state = 0, .external_lex_state = 5},
  [2652] = {.lex_state = 0, .external_lex_state = 4},
  [2653] = {.lex_state = 0, .external_lex_state = 5},
  [2654] = {.lex_state = 0, .external_lex_state = 5},
  [2655] = {.lex_state = 0, .external_lex_state = 5},
  [2656] = {.lex_state = 0, .external_lex_state = 5},
  [2657] = {.lex_state = 0, .external_lex_state = 5},
  [2658] = {.lex_state = 0, .external_lex_state = 5},
  [2659] = {.lex_state = 0, .external_lex_state = 5},
  [2660] = {.lex_state = 0, .external_lex_state = 5},
  [2661] = {.lex_state = 0, .external_lex_state = 4},
  [2662] = {.lex_state = 0, .external_lex_state = 5},
  [2663] = {.lex_state = 0, .external_lex_state = 5},
  [2664] = {.lex_state = 0, .external_lex_state = 5},
  [2665] = {.lex_state = 10, .external_lex_state = 4},
  [2666] = {.lex_state = 0, .external_lex_state = 5},
  [2667] = {.lex_state = 0, .external_lex_state = 5},
  [2668] = {.lex_state = 0, .external_lex_state = 5},
  [2669] = {.lex_state = 0, .external_lex_state = 5},
  [2670] = {.lex_state = 0, .external_lex_state = 5},
  [2671] = {.lex_state = 0, .external_lex_state = 5},
  [2672] = {.lex_state = 0, .external_lex_state = 5},
  [2673] = {.lex_state = 10, .external_lex_state = 4},
  [2674] = {.lex_state = 0, .external_lex_state = 5},
  [2675] = {.lex_state = 0, .external_lex_state = 5},
  [2676] = {.lex_state = 10, .external_lex_state = 4},
  [2677] = {.lex_state = 0, .external_lex_state = 4},
  [2678] = {.lex_state = 0, .external_lex_state = 5},
  [2679] = {.lex_state = 0, .external_lex_state = 4},
  [2680] = {.lex_state = 0, .external_lex_state = 5},
  [2681] = {.lex_state = 0, .external_lex_state = 5},
  [2682] = {.lex_state = 0, .external_lex_state = 5},
  [2683] = {.lex_state = 0, .external_lex_state = 5},
  [2684] = {.lex_state = 0, .external_lex_state = 5},
  [2685] = {.lex_state = 0, .external_lex_state = 4},
  [2686] = {.lex_state = 0, .external_lex_state = 5},
  [2687] = {.lex_state = 0, .external_lex_state = 5},
  [2688] = {.lex_state = 0, .external_lex_state = 4},
  [2689] = {.lex_state = 0, .external_lex_state = 5},
  [2690] = {.lex_state = 0, .external_lex_state = 5},
  [2691] = {.lex_state = 10, .external_lex_state = 4},
  [2692] = {.lex_state = 0, .external_lex_state = 5},
  [2693] = {.lex_state = 0, .external_lex_state = 5},
  [2694] = {.lex_state = 0, .external_lex_state = 5},
  [2695] = {.lex_state = 0, .external_lex_state = 5},
  [2696] = {.lex_state = 0, .external_lex_state = 5},
  [2697] = {.lex_state = 0, .external_lex_state = 5},
  [2698] = {.lex_state = 0, .external_lex_state = 5},
  [2699] = {.lex_state = 21, .external_lex_state = 4},
  [2700] = {.lex_state = 0, .external_lex_state = 5},
  [2701] = {.lex_state = 0, .external_lex_state = 4},
  [2702] = {.lex_state = 0, .external_lex_state = 5},
  [2703] = {.lex_state = 0, .external_lex_state = 5},
  [2704] = {.lex_state = 0, .external_lex_state = 5},
  [2705] = {.lex_state = 0, .external_lex_state = 5},
  [2706] = {.lex_state = 0, .external_lex_state = 5},
  [2707] = {.lex_state = 0, .external_lex_state = 5},
  [2708] = {.lex_state = 0, .external_lex_state = 5},
  [2709] = {.lex_state = 0, .external_lex_state = 4},
  [2710] = {.lex_state = 0, .external_lex_state = 5},
  [2711] = {.lex_state = 0, .external_lex_state = 4},
  [2712] = {.lex_state = 0, .external_lex_state = 5},
  [2713] = {.lex_state = 0, .external_lex_state = 4},
  [2714] = {.lex_state = 0, .external_lex_state = 4},
  [2715] = {.lex_state = 0, .external_lex_state = 5},
  [2716] = {.lex_state = 0, .external_lex_state = 5},
  [2717] = {.lex_state = 0, .external_lex_state = 5},
  [2718] = {.lex_state = 0, .external_lex_state = 5},
  [2719] = {.lex_state = 0, .external_lex_state = 5},
  [2720] = {.lex_state = 0, .external_lex_state = 5},
  [2721] = {.lex_state = 0, .external_lex_state = 5},
  [2722] = {.lex_state = 874, .external_lex_state = 4},
  [2723] = {.lex_state = 0, .external_lex_state = 5},
  [2724] = {.lex_state = 0, .external_lex_state = 5},
  [2725] = {.lex_state = 0, .external_lex_state = 5},
  [2726] = {.lex_state = 0, .external_lex_state = 5},
  [2727] = {.lex_state = 0, .external_lex_state = 5},
  [2728] = {.lex_state = 0, .external_lex_state = 5},
  [2729] = {.lex_state = 0, .external_lex_state = 4},
  [2730] = {.lex_state = 0, .external_lex_state = 4},
  [2731] = {.lex_state = 0, .external_lex_state = 5},
  [2732] = {.lex_state = 0, .external_lex_state = 5},
  [2733] = {.lex_state = 0, .external_lex_state = 5},
  [2734] = {.lex_state = 0, .external_lex_state = 5},
  [2735] = {.lex_state = 0, .external_lex_state = 4},
  [2736] = {.lex_state = 0, .external_lex_state = 5},
  [2737] = {.lex_state = 0, .external_lex_state = 5},
  [2738] = {.lex_state = 0, .external_lex_state = 5},
  [2739] = {.lex_state = 0, .external_lex_state = 5},
  [2740] = {.lex_state = 10, .external_lex_state = 4},
  [2741] = {.lex_state = 0, .external_lex_state = 5},
  [2742] = {.lex_state = 10, .external_lex_state = 4},
  [2743] = {.lex_state = 0, .external_lex_state = 5},
  [2744] = {.lex_state = 10, .external_lex_state = 4},
  [2745] = {.lex_state = 0, .external_lex_state = 5},
  [2746] = {.lex_state = 0, .external_lex_state = 4},
  [2747] = {.lex_state = 0, .external_lex_state = 5},
  [2748] = {.lex_state = 10, .external_lex_state = 4},
  [2749] = {.lex_state = 0, .external_lex_state = 5},
  [2750] = {.lex_state = 0, .external_lex_state = 5},
  [2751] = {.lex_state = 0, .external_lex_state = 5},
  [2752] = {.lex_state = 0, .external_lex_state = 5},
  [2753] = {.lex_state = 0, .external_lex_state = 4},
  [2754] = {.lex_state = 0, .external_lex_state = 5},
  [2755] = {.lex_state = 10, .external_lex_state = 4},
  [2756] = {.lex_state = 0, .external_lex_state = 5},
  [2757] = {.lex_state = 0, .external_lex_state = 5},
  [2758] = {.lex_state = 0, .external_lex_state = 5},
  [2759] = {.lex_state = 0, .external_lex_state = 5},
  [2760] = {.lex_state = 0, .external_lex_state = 5},
  [2761] = {.lex_state = 0, .external_lex_state = 5},
  [2762] = {.lex_state = 0, .external_lex_state = 5},
  [2763] = {.lex_state = 0, .external_lex_state = 5},
  [2764] = {.lex_state = 0, .external_lex_state = 5},
  [2765] = {.lex_state = 0, .external_lex_state = 5},
  [2766] = {.lex_state = 0, .external_lex_state = 5},
  [2767] = {.lex_state = 0, .external_lex_state = 5},
  [2768] = {.lex_state = 0, .external_lex_state = 5},
  [2769] = {.lex_state = 0, .external_lex_state = 5},
  [2770] = {.lex_state = 0, .external_lex_state = 5},
  [2771] = {.lex_state = 0, .external_lex_state = 5},
  [2772] = {.lex_state = 0, .external_lex_state = 5},
  [2773] = {.lex_state = 0, .external_lex_state = 5},
  [2774] = {.lex_state = 0, .external_lex_state = 5},
  [2775] = {.lex_state = 0, .external_lex_state = 5},
  [2776] = {.lex_state = 0, .external_lex_state = 5},
  [2777] = {.lex_state = 0, .external_lex_state = 5},
  [2778] = {.lex_state = 0, .external_lex_state = 5},
  [2779] = {.lex_state = 0, .external_lex_state = 5},
  [2780] = {.lex_state = 0, .external_lex_state = 5},
  [2781] = {.lex_state = 0, .external_lex_state = 5},
  [2782] = {.lex_state = 0, .external_lex_state = 5},
  [2783] = {.lex_state = 0, .external_lex_state = 5},
  [2784] = {.lex_state = 0, .external_lex_state = 5},
  [2785] = {.lex_state = 0, .external_lex_state = 5},
  [2786] = {.lex_state = 0, .external_lex_state = 5},
  [2787] = {.lex_state = 0, .external_lex_state = 5},
  [2788] = {.lex_state = 0, .external_lex_state = 5},
  [2789] = {.lex_state = 10, .external_lex_state = 4},
  [2790] = {.lex_state = 0, .external_lex_state = 5},
  [2791] = {.lex_state = 0, .external_lex_state = 5},
  [2792] = {.lex_state = 0, .external_lex_state = 5},
  [2793] = {.lex_state = 0, .external_lex_state = 4},
  [2794] = {.lex_state = 0, .external_lex_state = 4},
  [2795] = {.lex_state = 71, .external_lex_state = 4},
  [2796] = {.lex_state = 0, .external_lex_state = 4},
  [2797] = {.lex_state = 0, .external_lex_state = 4},
  [2798] = {.lex_state = 0, .external_lex_state = 5},
  [2799] = {.lex_state = 0, .external_lex_state = 5},
  [2800] = {.lex_state = 0, .external_lex_state = 5},
  [2801] = {.lex_state = 0, .external_lex_state = 5},
  [2802] = {.lex_state = 0, .external_lex_state = 5},
  [2803] = {.lex_state = 0, .external_lex_state = 4},
  [2804] = {.lex_state = 0, .external_lex_state = 4},
  [2805] = {.lex_state = 0, .external_lex_state = 5},
  [2806] = {.lex_state = 0, .external_lex_state = 5},
  [2807] = {.lex_state = 0, .external_lex_state = 5},
  [2808] = {.lex_state = 0, .external_lex_state = 4},
  [2809] = {.lex_state = 70, .external_lex_state = 4},
  [2810] = {.lex_state = 0, .external_lex_state = 4},
  [2811] = {.lex_state = 0, .external_lex_state = 4},
  [2812] = {.lex_state = 0, .external_lex_state = 4},
  [2813] = {.lex_state = 0, .external_lex_state = 4},
  [2814] = {.lex_state = 0, .external_lex_state = 5},
  [2815] = {.lex_state = 0, .external_lex_state = 5},
  [2816] = {.lex_state = 10, .external_lex_state = 4},
  [2817] = {.lex_state = 0, .external_lex_state = 4},
  [2818] = {.lex_state = 0, .external_lex_state = 5},
  [2819] = {.lex_state = 1, .external_lex_state = 4},
  [2820] = {.lex_state = 10, .external_lex_state = 4},
  [2821] = {.lex_state = 10, .external_lex_state = 4},
  [2822] = {.lex_state = 0, .external_lex_state = 5},
  [2823] = {.lex_state = 0, .external_lex_state = 5},
  [2824] = {.lex_state = 0, .external_lex_state = 5},
  [2825] = {.lex_state = 0, .external_lex_state = 5},
  [2826] = {.lex_state = 0, .external_lex_state = 4},
  [2827] = {.lex_state = 0, .external_lex_state = 4},
  [2828] = {.lex_state = 0, .external_lex_state = 5},
  [2829] = {.lex_state = 10, .external_lex_state = 4},
  [2830] = {.lex_state = 0, .external_lex_state = 5},
  [2831] = {.lex_state = 10, .external_lex_state = 4},
  [2832] = {.lex_state = 0, .external_lex_state = 5},
  [2833] = {.lex_state = 0, .external_lex_state = 5},
  [2834] = {.lex_state = 0, .external_lex_state = 4},
  [2835] = {.lex_state = 71, .external_lex_state = 4},
  [2836] = {.lex_state = 0, .external_lex_state = 4},
  [2837] = {.lex_state = 0, .external_lex_state = 5},
  [2838] = {.lex_state = 0, .external_lex_state = 5},
  [2839] = {.lex_state = 0, .external_lex_state = 5},
  [2840] = {.lex_state = 0, .external_lex_state = 4},
  [2841] = {.lex_state = 0, .external_lex_state = 5},
  [2842] = {.lex_state = 0, .external_lex_state = 5},
  [2843] = {.lex_state = 0, .external_lex_state = 5},
  [2844] = {.lex_state = 0, .external_lex_state = 5},
  [2845] = {.lex_state = 0, .external_lex_state = 5},
  [2846] = {.lex_state = 10, .external_lex_state = 4},
  [2847] = {.lex_state = 0, .external_lex_state = 5},
  [2848] = {.lex_state = 0, .external_lex_state = 5},
  [2849] = {.lex_state = 0, .external_lex_state = 5},
  [2850] = {.lex_state = 0, .external_lex_state = 5},
  [2851] = {.lex_state = 0, .external_lex_state = 5},
  [2852] = {.lex_state = 0, .external_lex_state = 8},
  [2853] = {.lex_state = 0, .external_lex_state = 5},
  [2854] = {.lex_state = 0, .external_lex_state = 5},
  [2855] = {.lex_state = 10, .external_lex_state = 4},
  [2856] = {.lex_state = 0, .external_lex_state = 5},
  [2857] = {.lex_state = 0, .external_lex_state = 5},
  [2858] = {.lex_state = 10, .external_lex_state = 4},
  [2859] = {.lex_state = 0, .external_lex_state = 5},
  [2860] = {.lex_state = 0, .external_lex_state = 4},
  [2861] = {.lex_state = 0, .external_lex_state = 5},
  [2862] = {.lex_state = 0, .external_lex_state = 5},
  [2863] = {.lex_state = 0, .external_lex_state = 5},
  [2864] = {.lex_state = 10, .external_lex_state = 4},
  [2865] = {.lex_state = 0, .external_lex_state = 5},
  [2866] = {.lex_state = 0, .external_lex_state = 5},
  [2867] = {.lex_state = 0, .external_lex_state = 5},
  [2868] = {.lex_state = 0, .external_lex_state = 5},
  [2869] = {.lex_state = 10, .external_lex_state = 4},
  [2870] = {.lex_state = 0, .external_lex_state = 5},
  [2871] = {.lex_state = 0, .external_lex_state = 5},
  [2872] = {.lex_state = 0, .external_lex_state = 5},
  [2873] = {.lex_state = 0, .external_lex_state = 5},
  [2874] = {.lex_state = 0, .external_lex_state = 5},
  [2875] = {.lex_state = 0, .external_lex_state = 5},
  [2876] = {.lex_state = 0, .external_lex_state = 5},
  [2877] = {.lex_state = 0, .external_lex_state = 5},
  [2878] = {.lex_state = 0, .external_lex_state = 5},
  [2879] = {.lex_state = 0, .external_lex_state = 5},
  [2880] = {.lex_state = 10, .external_lex_state = 4},
  [2881] = {.lex_state = 0, .external_lex_state = 5},
  [2882] = {.lex_state = 0, .external_lex_state = 4},
  [2883] = {.lex_state = 0, .external_lex_state = 5},
  [2884] = {.lex_state = 0, .external_lex_state = 5},
  [2885] = {.lex_state = 0, .external_lex_state = 5},
  [2886] = {.lex_state = 0, .external_lex_state = 5},
  [2887] = {.lex_state = 0, .external_lex_state = 5},
  [2888] = {.lex_state = 10, .external_lex_state = 4},
  [2889] = {.lex_state = 10, .external_lex_state = 4},
  [2890] = {.lex_state = 0, .external_lex_state = 5},
  [2891] = {.lex_state = 10, .external_lex_state = 4},
  [2892] = {.lex_state = 10, .external_lex_state = 4},
  [2893] = {.lex_state = 0, .external_lex_state = 5},
  [2894] = {.lex_state = 0, .external_lex_state = 5},
  [2895] = {.lex_state = 10, .external_lex_state = 4},
  [2896] = {.lex_state = 10, .external_lex_state = 4},
  [2897] = {.lex_state = 10, .external_lex_state = 4},
  [2898] = {.lex_state = 0, .external_lex_state = 4},
  [2899] = {.lex_state = 0, .external_lex_state = 5},
  [2900] = {.lex_state = 10, .external_lex_state = 4},
  [2901] = {.lex_state = 10, .external_lex_state = 4},
  [2902] = {.lex_state = 0, .external_lex_state = 5},
  [2903] = {.lex_state = 10, .external_lex_state = 4},
  [2904] = {.lex_state = 0, .external_lex_state = 4},
  [2905] = {.lex_state = 0, .external_lex_state = 5},
  [2906] = {.lex_state = 0, .external_lex_state = 5},
  [2907] = {.lex_state = 0, .external_lex_state = 4},
  [2908] = {.lex_state = 0, .external_lex_state = 5},
  [2909] = {.lex_state = 10, .external_lex_state = 4},
  [2910] = {.lex_state = 0, .external_lex_state = 5},
  [2911] = {.lex_state = 0, .external_lex_state = 5},
  [2912] = {.lex_state = 0, .external_lex_state = 5},
  [2913] = {.lex_state = 0, .external_lex_state = 5},
  [2914] = {.lex_state = 10, .external_lex_state = 4},
  [2915] = {.lex_state = 10, .external_lex_state = 4},
  [2916] = {.lex_state = 10, .external_lex_state = 4},
  [2917] = {.lex_state = 10, .external_lex_state = 4},
  [2918] = {.lex_state = 10, .external_lex_state = 4},
  [2919] = {(TSStateId)(-1)},
  [2920] = {(TSStateId)(-1)},
};

enum {
  ts_external_token__line_continuation = 0,
  ts_external_token__integer_literal = 1,
  ts_external_token__float_literal = 2,
  ts_external_token__boz_literal = 3,
  ts_external_token_string_literal = 4,
  ts_external_token__end_of_statement = 5,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__line_continuation] = sym__line_continuation,
  [ts_external_token__integer_literal] = sym__integer_literal,
  [ts_external_token__float_literal] = sym__float_literal,
  [ts_external_token__boz_literal] = sym__boz_literal,
  [ts_external_token_string_literal] = sym_string_literal,
  [ts_external_token__end_of_statement] = sym__end_of_statement,
};

static const bool ts_external_scanner_states[10][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token__integer_literal] = true,
    [ts_external_token__float_literal] = true,
    [ts_external_token__boz_literal] = true,
    [ts_external_token_string_literal] = true,
    [ts_external_token__end_of_statement] = true,
  },
  [2] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token__integer_literal] = true,
    [ts_external_token__float_literal] = true,
    [ts_external_token__boz_literal] = true,
    [ts_external_token_string_literal] = true,
  },
  [3] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token__integer_literal] = true,
  },
  [4] = {
    [ts_external_token__line_continuation] = true,
  },
  [5] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token__end_of_statement] = true,
  },
  [6] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token__integer_literal] = true,
    [ts_external_token__end_of_statement] = true,
  },
  [7] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token__integer_literal] = true,
    [ts_external_token__float_literal] = true,
    [ts_external_token__boz_literal] = true,
  },
  [8] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token_string_literal] = true,
  },
  [9] = {
    [ts_external_token__line_continuation] = true,
    [ts_external_token_string_literal] = true,
    [ts_external_token__end_of_statement] = true,
  },
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [sym_preproc_file_line] = STATE(0),
    [ts_builtin_sym_end] = ACTIONS(1),
    [aux_sym_program_statement_token1] = ACTIONS(1),
    [aux_sym_end_program_statement_token1] = ACTIONS(1),
    [aux_sym_end_program_statement_token2] = ACTIONS(1),
    [aux_sym_module_statement_token1] = ACTIONS(1),
    [aux_sym_end_module_statement_token1] = ACTIONS(1),
    [aux_sym_submodule_statement_token1] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [aux_sym_end_submodule_statement_token1] = ACTIONS(1),
    [aux_sym_interface_statement_token1] = ACTIONS(1),
    [aux_sym_end_interface_statement_token1] = ACTIONS(1),
    [aux_sym_assignment_token1] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [aux_sym_operator_token1] = ACTIONS(1),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1),
    [aux_sym_defined_io_procedure_token4] = ACTIONS(1),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1),
    [aux_sym_function_statement_token1] = ACTIONS(1),
    [aux_sym_language_binding_token1] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1),
    [aux_sym_procedure_attributes_token2] = ACTIONS(1),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1),
    [aux_sym_procedure_attributes_token4] = ACTIONS(1),
    [aux_sym_procedure_attributes_token5] = ACTIONS(1),
    [aux_sym_end_function_statement_token1] = ACTIONS(1),
    [aux_sym_function_result_token1] = ACTIONS(1),
    [aux_sym_contains_statement_token1] = ACTIONS(1),
    [aux_sym_use_statement_token1] = ACTIONS(1),
    [aux_sym_use_statement_token2] = ACTIONS(1),
    [aux_sym_use_statement_token3] = ACTIONS(1),
    [anon_sym_COLON_COLON] = ACTIONS(1),
    [aux_sym_included_items_token1] = ACTIONS(1),
    [anon_sym_EQ_GT] = ACTIONS(1),
    [aux_sym_implicit_statement_token1] = ACTIONS(1),
    [aux_sym_implicit_statement_token2] = ACTIONS(1),
    [aux_sym_implicit_statement_token3] = ACTIONS(1),
    [aux_sym_implicit_statement_token4] = ACTIONS(1),
    [aux_sym_save_statement_token1] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [aux_sym_private_statement_token1] = ACTIONS(1),
    [aux_sym_public_statement_token1] = ACTIONS(1),
    [aux_sym_namelist_statement_token1] = ACTIONS(1),
    [aux_sym_common_statement_token1] = ACTIONS(1),
    [aux_sym_implicit_range_token1] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [aux_sym_import_statement_token1] = ACTIONS(1),
    [aux_sym__import_names_token1] = ACTIONS(1),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1),
    [aux_sym_base_type_specifier_token1] = ACTIONS(1),
    [aux_sym_end_type_statement_token1] = ACTIONS(1),
    [aux_sym__procedure_kind_token1] = ACTIONS(1),
    [aux_sym__procedure_kind_token2] = ACTIONS(1),
    [aux_sym__procedure_kind_token3] = ACTIONS(1),
    [aux_sym__procedure_kind_token4] = ACTIONS(1),
    [aux_sym_procedure_attribute_token1] = ACTIONS(1),
    [aux_sym_procedure_attribute_token2] = ACTIONS(1),
    [aux_sym_procedure_attribute_token3] = ACTIONS(1),
    [aux_sym_procedure_attribute_token4] = ACTIONS(1),
    [aux_sym_procedure_attribute_token5] = ACTIONS(1),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1),
    [aux_sym_variable_attributes_token1] = ACTIONS(1),
    [aux_sym_variable_attributes_token2] = ACTIONS(1),
    [aux_sym_variable_attributes_token3] = ACTIONS(1),
    [aux_sym_variable_attributes_token4] = ACTIONS(1),
    [aux_sym_variable_attributes_token5] = ACTIONS(1),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1),
    [aux_sym_intrinsic_type_token5] = ACTIONS(1),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1),
    [aux_sym_derived_type_token1] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [aux_sym_size_token1] = ACTIONS(1),
    [aux_sym_type_qualifier_token1] = ACTIONS(1),
    [aux_sym_type_qualifier_token2] = ACTIONS(1),
    [aux_sym_type_qualifier_token3] = ACTIONS(1),
    [aux_sym_type_qualifier_token4] = ACTIONS(1),
    [aux_sym_type_qualifier_token5] = ACTIONS(1),
    [aux_sym_type_qualifier_token6] = ACTIONS(1),
    [aux_sym_type_qualifier_token7] = ACTIONS(1),
    [aux_sym_type_qualifier_token8] = ACTIONS(1),
    [aux_sym_type_qualifier_token9] = ACTIONS(1),
    [aux_sym_type_qualifier_token10] = ACTIONS(1),
    [aux_sym_type_qualifier_token11] = ACTIONS(1),
    [aux_sym_type_qualifier_token12] = ACTIONS(1),
    [aux_sym_type_qualifier_token13] = ACTIONS(1),
    [aux_sym_type_qualifier_token14] = ACTIONS(1),
    [aux_sym_type_qualifier_token15] = ACTIONS(1),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1),
    [aux_sym_stop_statement_token1] = ACTIONS(1),
    [aux_sym_stop_statement_token2] = ACTIONS(1),
    [aux_sym_stop_statement_token3] = ACTIONS(1),
    [aux_sym_subroutine_call_token1] = ACTIONS(1),
    [anon_sym_LT_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT_GT] = ACTIONS(1),
    [aux_sym_keyword_statement_token1] = ACTIONS(1),
    [aux_sym_keyword_statement_token2] = ACTIONS(1),
    [aux_sym_keyword_statement_token3] = ACTIONS(1),
    [aux_sym_keyword_statement_token4] = ACTIONS(1),
    [aux_sym_keyword_statement_token5] = ACTIONS(1),
    [aux_sym_keyword_statement_token6] = ACTIONS(1),
    [aux_sym_keyword_statement_token7] = ACTIONS(1),
    [aux_sym_include_statement_token1] = ACTIONS(1),
    [aux_sym_data_statement_token1] = ACTIONS(1),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(1),
    [aux_sym_while_statement_token1] = ACTIONS(1),
    [aux_sym_concurrent_header_token1] = ACTIONS(1),
    [aux_sym_concurrent_locality_token1] = ACTIONS(1),
    [aux_sym_concurrent_locality_token2] = ACTIONS(1),
    [aux_sym_concurrent_locality_token3] = ACTIONS(1),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1),
    [aux_sym__block_if_statement_token1] = ACTIONS(1),
    [aux_sym_end_if_statement_token1] = ACTIONS(1),
    [aux_sym_elseif_clause_token1] = ACTIONS(1),
    [aux_sym_elseif_clause_token2] = ACTIONS(1),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1),
    [aux_sym_end_where_statement_token1] = ACTIONS(1),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(1),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1),
    [aux_sym_end_forall_statement_token1] = ACTIONS(1),
    [aux_sym_select_case_statement_token1] = ACTIONS(1),
    [aux_sym_select_case_statement_token2] = ACTIONS(1),
    [aux_sym_select_case_statement_token3] = ACTIONS(1),
    [aux_sym_select_type_statement_token1] = ACTIONS(1),
    [aux_sym_select_rank_statement_token1] = ACTIONS(1),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1),
    [aux_sym_end_select_statement_token1] = ACTIONS(1),
    [aux_sym_type_statement_token1] = ACTIONS(1),
    [aux_sym_type_statement_token2] = ACTIONS(1),
    [aux_sym_type_statement_token3] = ACTIONS(1),
    [aux_sym__class_default_token1] = ACTIONS(1),
    [aux_sym_block_construct_token1] = ACTIONS(1),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1),
    [aux_sym_associate_statement_token1] = ACTIONS(1),
    [aux_sym_end_associate_statement_token1] = ACTIONS(1),
    [aux_sym_format_statement_token1] = ACTIONS(1),
    [aux_sym_print_statement_token1] = ACTIONS(1),
    [aux_sym_open_statement_token1] = ACTIONS(1),
    [aux_sym_close_statement_token1] = ACTIONS(1),
    [aux_sym_inquire_statement_token1] = ACTIONS(1),
    [aux_sym_enum_statement_token1] = ACTIONS(1),
    [aux_sym_enumerator_statement_token1] = ACTIONS(1),
    [aux_sym_end_enum_statement_token1] = ACTIONS(1),
    [aux_sym_file_position_statement_token1] = ACTIONS(1),
    [aux_sym_file_position_statement_token2] = ACTIONS(1),
    [aux_sym_file_position_statement_token3] = ACTIONS(1),
    [aux_sym_file_position_statement_token4] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [aux_sym_logical_expression_token1] = ACTIONS(1),
    [aux_sym_logical_expression_token2] = ACTIONS(1),
    [aux_sym_logical_expression_token3] = ACTIONS(1),
    [aux_sym_logical_expression_token4] = ACTIONS(1),
    [aux_sym_logical_expression_token5] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [aux_sym_relational_expression_token1] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [aux_sym_relational_expression_token2] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [aux_sym_relational_expression_token3] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [aux_sym_relational_expression_token4] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [aux_sym_relational_expression_token5] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [aux_sym_relational_expression_token6] = ACTIONS(1),
    [anon_sym_SLASH_SLASH] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_STAR_STAR] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [sym_assumed_rank] = ACTIONS(1),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1),
    [anon_sym_SLASH_RPAREN] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [sym_boolean_literal] = ACTIONS(1),
    [aux_sym_null_literal_token1] = ACTIONS(1),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_preproc_file_line_token1] = ACTIONS(1),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__semicolon] = ACTIONS(1),
    [sym__integer_literal] = ACTIONS(1),
    [sym__float_literal] = ACTIONS(1),
    [sym__boz_literal] = ACTIONS(1),
    [sym_string_literal] = ACTIONS(1),
    [sym__end_of_statement] = ACTIONS(1),
  },
  [1] = {
    [sym_translation_unit] = STATE(2808),
    [sym_program] = STATE(294),
    [sym_program_statement] = STATE(27),
    [sym_end_program_statement] = STATE(293),
    [sym_module] = STATE(292),
    [sym_module_statement] = STATE(417),
    [sym_submodule] = STATE(292),
    [sym_submodule_statement] = STATE(418),
    [sym_interface] = STATE(283),
    [sym_interface_statement] = STATE(764),
    [sym_subroutine] = STATE(292),
    [sym_subroutine_statement] = STATE(2807),
    [sym_function] = STATE(292),
    [sym_function_statement] = STATE(2806),
    [aux_sym__callable_interface_qualifers] = STATE(1376),
    [sym_procedure_attributes] = STATE(1729),
    [sym_internal_procedures] = STATE(1987),
    [sym_contains_statement] = STATE(2802),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(434),
    [sym_intrinsic_type] = STATE(505),
    [sym_derived_type] = STATE(434),
    [sym_type_qualifier] = STATE(457),
    [sym_procedure_qualifier] = STATE(1729),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(229),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2785),
    [sym_data_statement] = STATE(2798),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(1),
    [sym_identifier] = STATE(1148),
    [aux_sym_translation_unit_repeat1] = STATE(3),
    [aux_sym_program_repeat1] = STATE(45),
    [aux_sym_program_repeat2] = STATE(119),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [ts_builtin_sym_end] = ACTIONS(7),
    [aux_sym_program_statement_token1] = ACTIONS(9),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [aux_sym_module_statement_token1] = ACTIONS(15),
    [aux_sym_submodule_statement_token1] = ACTIONS(17),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_subroutine_statement_token1] = ACTIONS(27),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_function_statement_token1] = ACTIONS(31),
    [aux_sym_procedure_attributes_token1] = ACTIONS(33),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(43),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(63),
    [aux_sym_intrinsic_type_token2] = ACTIONS(65),
    [aux_sym_intrinsic_type_token3] = ACTIONS(63),
    [aux_sym_intrinsic_type_token4] = ACTIONS(67),
    [aux_sym_intrinsic_type_token6] = ACTIONS(65),
    [aux_sym_intrinsic_type_token7] = ACTIONS(65),
    [aux_sym_intrinsic_type_token8] = ACTIONS(69),
    [aux_sym_intrinsic_type_token9] = ACTIONS(65),
    [aux_sym_intrinsic_type_token10] = ACTIONS(65),
    [aux_sym_derived_type_token1] = ACTIONS(71),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(81),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(81),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(81),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(81),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [2] = {
    [sym_program] = STATE(292),
    [sym_program_statement] = STATE(27),
    [sym_end_program_statement] = STATE(293),
    [sym_module] = STATE(292),
    [sym_module_statement] = STATE(417),
    [sym_submodule] = STATE(292),
    [sym_submodule_statement] = STATE(418),
    [sym_interface] = STATE(283),
    [sym_interface_statement] = STATE(764),
    [sym_subroutine] = STATE(292),
    [sym_subroutine_statement] = STATE(2807),
    [sym_function] = STATE(292),
    [sym_function_statement] = STATE(2806),
    [aux_sym__callable_interface_qualifers] = STATE(1376),
    [sym_procedure_attributes] = STATE(1729),
    [sym_internal_procedures] = STATE(1987),
    [sym_contains_statement] = STATE(2802),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(434),
    [sym_intrinsic_type] = STATE(505),
    [sym_derived_type] = STATE(434),
    [sym_type_qualifier] = STATE(457),
    [sym_procedure_qualifier] = STATE(1729),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(229),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2785),
    [sym_data_statement] = STATE(2798),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(2),
    [sym_identifier] = STATE(1148),
    [aux_sym_translation_unit_repeat1] = STATE(2),
    [aux_sym_program_repeat1] = STATE(45),
    [aux_sym_program_repeat2] = STATE(119),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [ts_builtin_sym_end] = ACTIONS(157),
    [aux_sym_program_statement_token1] = ACTIONS(159),
    [aux_sym_end_program_statement_token1] = ACTIONS(162),
    [aux_sym_end_program_statement_token2] = ACTIONS(165),
    [aux_sym_module_statement_token1] = ACTIONS(168),
    [aux_sym_submodule_statement_token1] = ACTIONS(171),
    [anon_sym_LPAREN] = ACTIONS(174),
    [aux_sym_interface_statement_token1] = ACTIONS(177),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(180),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(183),
    [aux_sym_subroutine_statement_token1] = ACTIONS(186),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(189),
    [aux_sym_function_statement_token1] = ACTIONS(192),
    [aux_sym_procedure_attributes_token1] = ACTIONS(195),
    [aux_sym_procedure_attributes_token3] = ACTIONS(198),
    [aux_sym_contains_statement_token1] = ACTIONS(201),
    [aux_sym_use_statement_token1] = ACTIONS(204),
    [aux_sym_use_statement_token2] = ACTIONS(198),
    [aux_sym_implicit_statement_token1] = ACTIONS(207),
    [aux_sym_implicit_statement_token3] = ACTIONS(210),
    [aux_sym_implicit_statement_token4] = ACTIONS(198),
    [aux_sym_save_statement_token1] = ACTIONS(213),
    [aux_sym_private_statement_token1] = ACTIONS(216),
    [aux_sym_public_statement_token1] = ACTIONS(219),
    [aux_sym_namelist_statement_token1] = ACTIONS(222),
    [aux_sym_common_statement_token1] = ACTIONS(225),
    [anon_sym_DASH] = ACTIONS(228),
    [aux_sym_import_statement_token1] = ACTIONS(231),
    [aux_sym_derived_type_definition_token1] = ACTIONS(234),
    [aux_sym_abstract_specifier_token1] = ACTIONS(237),
    [aux_sym_procedure_attribute_token6] = ACTIONS(198),
    [aux_sym_variable_attributes_token1] = ACTIONS(198),
    [aux_sym_variable_attributes_token2] = ACTIONS(198),
    [aux_sym_variable_attributes_token3] = ACTIONS(198),
    [aux_sym_variable_attributes_token4] = ACTIONS(198),
    [aux_sym_variable_attributes_token5] = ACTIONS(198),
    [aux_sym_intrinsic_type_token1] = ACTIONS(240),
    [aux_sym_intrinsic_type_token2] = ACTIONS(243),
    [aux_sym_intrinsic_type_token3] = ACTIONS(240),
    [aux_sym_intrinsic_type_token4] = ACTIONS(246),
    [aux_sym_intrinsic_type_token6] = ACTIONS(243),
    [aux_sym_intrinsic_type_token7] = ACTIONS(243),
    [aux_sym_intrinsic_type_token8] = ACTIONS(249),
    [aux_sym_intrinsic_type_token9] = ACTIONS(243),
    [aux_sym_intrinsic_type_token10] = ACTIONS(243),
    [aux_sym_derived_type_token1] = ACTIONS(252),
    [aux_sym_type_qualifier_token1] = ACTIONS(198),
    [aux_sym_type_qualifier_token2] = ACTIONS(198),
    [aux_sym_type_qualifier_token3] = ACTIONS(255),
    [aux_sym_type_qualifier_token4] = ACTIONS(198),
    [aux_sym_type_qualifier_token5] = ACTIONS(258),
    [aux_sym_type_qualifier_token9] = ACTIONS(198),
    [aux_sym_type_qualifier_token10] = ACTIONS(261),
    [aux_sym_type_qualifier_token11] = ACTIONS(198),
    [aux_sym_type_qualifier_token12] = ACTIONS(198),
    [aux_sym_type_qualifier_token13] = ACTIONS(264),
    [aux_sym_type_qualifier_token14] = ACTIONS(264),
    [aux_sym_type_qualifier_token15] = ACTIONS(198),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(267),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(267),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(267),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(267),
    [aux_sym_equivalence_statement_token1] = ACTIONS(270),
    [aux_sym_stop_statement_token1] = ACTIONS(273),
    [aux_sym_stop_statement_token2] = ACTIONS(276),
    [aux_sym_subroutine_call_token1] = ACTIONS(279),
    [aux_sym_keyword_statement_token1] = ACTIONS(282),
    [aux_sym_keyword_statement_token2] = ACTIONS(285),
    [aux_sym_keyword_statement_token3] = ACTIONS(285),
    [aux_sym_keyword_statement_token4] = ACTIONS(288),
    [aux_sym_keyword_statement_token6] = ACTIONS(291),
    [aux_sym_keyword_statement_token7] = ACTIONS(282),
    [aux_sym_include_statement_token1] = ACTIONS(294),
    [aux_sym_data_statement_token1] = ACTIONS(297),
    [aux_sym_do_loop_statement_token1] = ACTIONS(300),
    [aux_sym__inline_if_statement_token1] = ACTIONS(303),
    [aux_sym_end_if_statement_token1] = ACTIONS(306),
    [aux_sym_elseif_clause_token2] = ACTIONS(306),
    [aux_sym__inline_where_statement_token1] = ACTIONS(309),
    [aux_sym__forall_control_expression_token1] = ACTIONS(312),
    [aux_sym_select_case_statement_token1] = ACTIONS(315),
    [aux_sym_select_case_statement_token3] = ACTIONS(318),
    [aux_sym_select_type_statement_token1] = ACTIONS(321),
    [aux_sym_select_rank_statement_token2] = ACTIONS(324),
    [aux_sym_block_construct_token1] = ACTIONS(327),
    [aux_sym_associate_statement_token1] = ACTIONS(330),
    [aux_sym_format_statement_token1] = ACTIONS(333),
    [aux_sym_print_statement_token1] = ACTIONS(336),
    [aux_sym_open_statement_token1] = ACTIONS(339),
    [aux_sym_close_statement_token1] = ACTIONS(342),
    [aux_sym_inquire_statement_token1] = ACTIONS(345),
    [aux_sym_enum_statement_token1] = ACTIONS(348),
    [aux_sym_file_position_statement_token1] = ACTIONS(351),
    [aux_sym_file_position_statement_token2] = ACTIONS(351),
    [aux_sym_file_position_statement_token3] = ACTIONS(351),
    [aux_sym_file_position_statement_token4] = ACTIONS(354),
    [aux_sym_logical_expression_token5] = ACTIONS(357),
    [anon_sym_PLUS] = ACTIONS(228),
    [anon_sym_DOT] = ACTIONS(360),
    [anon_sym_LPAREN_SLASH] = ACTIONS(363),
    [anon_sym_LBRACK] = ACTIONS(366),
    [sym_boolean_literal] = ACTIONS(369),
    [aux_sym_null_literal_token1] = ACTIONS(372),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(306),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(375),
    [sym__float_literal] = ACTIONS(378),
    [sym__boz_literal] = ACTIONS(378),
    [sym_string_literal] = ACTIONS(369),
  },
  [3] = {
    [sym_program] = STATE(273),
    [sym_program_statement] = STATE(27),
    [sym_end_program_statement] = STATE(293),
    [sym_module] = STATE(292),
    [sym_module_statement] = STATE(417),
    [sym_submodule] = STATE(292),
    [sym_submodule_statement] = STATE(418),
    [sym_interface] = STATE(283),
    [sym_interface_statement] = STATE(764),
    [sym_subroutine] = STATE(292),
    [sym_subroutine_statement] = STATE(2807),
    [sym_function] = STATE(292),
    [sym_function_statement] = STATE(2806),
    [aux_sym__callable_interface_qualifers] = STATE(1376),
    [sym_procedure_attributes] = STATE(1729),
    [sym_internal_procedures] = STATE(1987),
    [sym_contains_statement] = STATE(2802),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(434),
    [sym_intrinsic_type] = STATE(505),
    [sym_derived_type] = STATE(434),
    [sym_type_qualifier] = STATE(457),
    [sym_procedure_qualifier] = STATE(1729),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(229),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2785),
    [sym_data_statement] = STATE(2798),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(3),
    [sym_identifier] = STATE(1148),
    [aux_sym_translation_unit_repeat1] = STATE(2),
    [aux_sym_program_repeat1] = STATE(45),
    [aux_sym_program_repeat2] = STATE(119),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [ts_builtin_sym_end] = ACTIONS(381),
    [aux_sym_program_statement_token1] = ACTIONS(9),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [aux_sym_module_statement_token1] = ACTIONS(15),
    [aux_sym_submodule_statement_token1] = ACTIONS(17),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_subroutine_statement_token1] = ACTIONS(27),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_function_statement_token1] = ACTIONS(31),
    [aux_sym_procedure_attributes_token1] = ACTIONS(33),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(43),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(63),
    [aux_sym_intrinsic_type_token2] = ACTIONS(65),
    [aux_sym_intrinsic_type_token3] = ACTIONS(63),
    [aux_sym_intrinsic_type_token4] = ACTIONS(67),
    [aux_sym_intrinsic_type_token6] = ACTIONS(65),
    [aux_sym_intrinsic_type_token7] = ACTIONS(65),
    [aux_sym_intrinsic_type_token8] = ACTIONS(69),
    [aux_sym_intrinsic_type_token9] = ACTIONS(65),
    [aux_sym_intrinsic_type_token10] = ACTIONS(65),
    [aux_sym_derived_type_token1] = ACTIONS(71),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(81),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(81),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(81),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(81),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [4] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1661),
    [sym_internal_procedures] = STATE(2082),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(4),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(35),
    [aux_sym_program_repeat2] = STATE(110),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(383),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(385),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [5] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1652),
    [sym_internal_procedures] = STATE(2102),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(5),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(106),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(403),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [6] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(270),
    [sym_internal_procedures] = STATE(1963),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(6),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(91),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(407),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [7] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1650),
    [sym_internal_procedures] = STATE(2098),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(7),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(105),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(409),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(411),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [8] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1595),
    [sym_internal_procedures] = STATE(2010),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(8),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(14),
    [aux_sym_program_repeat2] = STATE(96),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(413),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(415),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [9] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1576),
    [sym_internal_procedures] = STATE(1928),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(9),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_program_repeat2] = STATE(89),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(417),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(419),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [10] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1647),
    [sym_internal_procedures] = STATE(2122),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(10),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(5),
    [aux_sym_program_repeat2] = STATE(103),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(403),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [11] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1680),
    [sym_internal_procedures] = STATE(2049),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(11),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(16),
    [aux_sym_program_repeat2] = STATE(104),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(423),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [12] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1646),
    [sym_internal_procedures] = STATE(2127),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(12),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(7),
    [aux_sym_program_repeat2] = STATE(98),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(409),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(411),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [13] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1583),
    [sym_internal_procedures] = STATE(2173),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(13),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(20),
    [aux_sym_program_repeat2] = STATE(102),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(425),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(427),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [14] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1668),
    [sym_internal_procedures] = STATE(1967),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(14),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_program_repeat2] = STATE(117),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(413),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(415),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [15] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1681),
    [sym_internal_procedures] = STATE(2047),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(15),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(21),
    [aux_sym_program_repeat2] = STATE(129),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(429),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(431),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [16] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1683),
    [sym_internal_procedures] = STATE(2164),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(16),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(131),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(423),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [17] = {
    [sym_end_program_statement] = STATE(232),
    [sym_interface] = STATE(312),
    [sym_interface_statement] = STATE(761),
    [sym_internal_procedures] = STATE(2004),
    [sym_contains_statement] = STATE(2802),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(229),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2487),
    [sym_data_statement] = STATE(2798),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(17),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(56),
    [aux_sym_program_repeat2] = STATE(95),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [18] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1572),
    [sym_internal_procedures] = STATE(2174),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(18),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(25),
    [aux_sym_program_repeat2] = STATE(86),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(433),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(435),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [19] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1673),
    [sym_internal_procedures] = STATE(2129),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(19),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_program_repeat2] = STATE(94),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(437),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(439),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [20] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1589),
    [sym_internal_procedures] = STATE(1953),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(20),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(133),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(425),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(427),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [21] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1685),
    [sym_internal_procedures] = STATE(2168),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(21),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(137),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(429),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(431),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [22] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1689),
    [sym_internal_procedures] = STATE(1970),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(22),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(19),
    [aux_sym_program_repeat2] = STATE(92),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(437),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(439),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [23] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1714),
    [sym_internal_procedures] = STATE(2006),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(23),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(33),
    [aux_sym_program_repeat2] = STATE(127),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(441),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(443),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [24] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1608),
    [sym_internal_procedures] = STATE(1978),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(24),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(115),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(445),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(447),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [25] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1594),
    [sym_internal_procedures] = STATE(1957),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(25),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(130),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(433),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(435),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [26] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1712),
    [sym_internal_procedures] = STATE(2002),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(26),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(50),
    [aux_sym_program_repeat2] = STATE(116),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(449),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(451),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [27] = {
    [sym_end_program_statement] = STATE(243),
    [sym_interface] = STATE(312),
    [sym_interface_statement] = STATE(761),
    [sym_internal_procedures] = STATE(2162),
    [sym_contains_statement] = STATE(2802),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(229),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2487),
    [sym_data_statement] = STATE(2798),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(27),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(17),
    [aux_sym_program_repeat2] = STATE(100),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [28] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1640),
    [sym_internal_procedures] = STATE(2056),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(28),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_program_repeat2] = STATE(97),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(453),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(455),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [29] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1633),
    [sym_internal_procedures] = STATE(2184),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(29),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(28),
    [aux_sym_program_repeat2] = STATE(85),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(453),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(455),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [30] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(267),
    [sym_internal_procedures] = STATE(1959),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(30),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(6),
    [aux_sym_program_repeat2] = STATE(87),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(407),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [31] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1623),
    [sym_internal_procedures] = STATE(1980),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(31),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(24),
    [aux_sym_program_repeat2] = STATE(123),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(445),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(447),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [32] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1626),
    [sym_internal_procedures] = STATE(2030),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(32),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(99),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(457),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(459),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [33] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1710),
    [sym_internal_procedures] = STATE(2208),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(33),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(113),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(441),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(443),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [34] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1603),
    [sym_internal_procedures] = STATE(2210),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(34),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(38),
    [aux_sym_program_repeat2] = STATE(126),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(463),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [35] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1672),
    [sym_internal_procedures] = STATE(2128),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(35),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_program_repeat2] = STATE(111),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(383),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(385),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [36] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1624),
    [sym_internal_procedures] = STATE(2026),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(36),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(107),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(465),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(467),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [37] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1178),
    [sym_internal_procedures] = STATE(2073),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(37),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(47),
    [aux_sym_program_repeat2] = STATE(121),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(469),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(471),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [38] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1612),
    [sym_internal_procedures] = STATE(1983),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(38),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_program_repeat2] = STATE(124),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(463),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [39] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1179),
    [sym_internal_procedures] = STATE(2064),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(39),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(46),
    [aux_sym_program_repeat2] = STATE(120),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(475),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [40] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym_end_module_procedure_statement] = STATE(1585),
    [sym_internal_procedures] = STATE(2069),
    [sym_contains_statement] = STATE(2611),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(231),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2544),
    [sym_data_statement] = STATE(2681),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(40),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(9),
    [aux_sym_program_repeat2] = STATE(88),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(417),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(419),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [41] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(280),
    [sym_internal_procedures] = STATE(1961),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(41),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(44),
    [aux_sym_program_repeat2] = STATE(128),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(477),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(479),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [42] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1604),
    [sym_internal_procedures] = STATE(1976),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(42),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(101),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(481),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(483),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [43] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1619),
    [sym_internal_procedures] = STATE(2223),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(43),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(32),
    [aux_sym_program_repeat2] = STATE(108),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(457),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(459),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [44] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(271),
    [sym_internal_procedures] = STATE(1933),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(44),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(90),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(477),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(479),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [45] = {
    [sym_end_program_statement] = STATE(243),
    [sym_interface] = STATE(312),
    [sym_interface_statement] = STATE(761),
    [sym_internal_procedures] = STATE(2162),
    [sym_contains_statement] = STATE(2802),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(229),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2487),
    [sym_data_statement] = STATE(2798),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(45),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(56),
    [aux_sym_program_repeat2] = STATE(100),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [46] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1186),
    [sym_internal_procedures] = STATE(2116),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(46),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(93),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(475),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [47] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1184),
    [sym_internal_procedures] = STATE(2067),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(47),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_program_repeat2] = STATE(118),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(469),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(471),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [48] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1622),
    [sym_internal_procedures] = STATE(1985),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(48),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(42),
    [aux_sym_program_repeat2] = STATE(125),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(481),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(483),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [49] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym_end_subroutine_statement] = STATE(1618),
    [sym_internal_procedures] = STATE(2225),
    [sym_contains_statement] = STATE(2467),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(230),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2392),
    [sym_data_statement] = STATE(2540),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(49),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(36),
    [aux_sym_program_repeat2] = STATE(114),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(465),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(467),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [50] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym_end_function_statement] = STATE(1707),
    [sym_internal_procedures] = STATE(2212),
    [sym_contains_statement] = STATE(2539),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(221),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2472),
    [sym_data_statement] = STATE(2612),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(50),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_program_repeat2] = STATE(112),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(449),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_end_function_statement_token1] = ACTIONS(451),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [51] = {
    [sym_interface] = STATE(405),
    [sym_interface_statement] = STATE(770),
    [sym__specification_part] = STATE(415),
    [sym_use_statement] = STATE(2468),
    [sym_implicit_statement] = STATE(2468),
    [sym_save_statement] = STATE(2468),
    [sym_private_statement] = STATE(2468),
    [sym_public_statement] = STATE(2468),
    [sym_namelist_statement] = STATE(2468),
    [sym_common_statement] = STATE(2468),
    [sym_import_statement] = STATE(2468),
    [sym_derived_type_definition] = STATE(405),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(922),
    [sym_variable_declaration] = STATE(2468),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2468),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2468),
    [sym_equivalence_statement] = STATE(2468),
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(223),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2520),
    [sym_data_statement] = STATE(2468),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2707),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(405),
    [sym_enum_statement] = STATE(1853),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(51),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(54),
    [aux_sym_program_repeat2] = STATE(192),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [52] = {
    [sym_interface] = STATE(405),
    [sym_interface_statement] = STATE(770),
    [sym__specification_part] = STATE(415),
    [sym_use_statement] = STATE(2468),
    [sym_implicit_statement] = STATE(2468),
    [sym_save_statement] = STATE(2468),
    [sym_private_statement] = STATE(2468),
    [sym_public_statement] = STATE(2468),
    [sym_namelist_statement] = STATE(2468),
    [sym_common_statement] = STATE(2468),
    [sym_import_statement] = STATE(2468),
    [sym_derived_type_definition] = STATE(405),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(922),
    [sym_variable_declaration] = STATE(2468),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2468),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2468),
    [sym_equivalence_statement] = STATE(2468),
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(223),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2520),
    [sym_data_statement] = STATE(2468),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2707),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(405),
    [sym_enum_statement] = STATE(1853),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(52),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(59),
    [aux_sym_program_repeat2] = STATE(192),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [53] = {
    [sym_interface] = STATE(405),
    [sym_interface_statement] = STATE(770),
    [sym__specification_part] = STATE(415),
    [sym_use_statement] = STATE(2468),
    [sym_implicit_statement] = STATE(2468),
    [sym_save_statement] = STATE(2468),
    [sym_private_statement] = STATE(2468),
    [sym_public_statement] = STATE(2468),
    [sym_namelist_statement] = STATE(2468),
    [sym_common_statement] = STATE(2468),
    [sym_import_statement] = STATE(2468),
    [sym_derived_type_definition] = STATE(405),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(922),
    [sym_variable_declaration] = STATE(2468),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2468),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2468),
    [sym_equivalence_statement] = STATE(2468),
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(223),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2520),
    [sym_data_statement] = STATE(2468),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2719),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(405),
    [sym_enum_statement] = STATE(1853),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(53),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(52),
    [aux_sym_program_repeat2] = STATE(191),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [54] = {
    [sym_interface] = STATE(405),
    [sym_interface_statement] = STATE(770),
    [sym__specification_part] = STATE(415),
    [sym_use_statement] = STATE(2468),
    [sym_implicit_statement] = STATE(2468),
    [sym_save_statement] = STATE(2468),
    [sym_private_statement] = STATE(2468),
    [sym_public_statement] = STATE(2468),
    [sym_namelist_statement] = STATE(2468),
    [sym_common_statement] = STATE(2468),
    [sym_import_statement] = STATE(2468),
    [sym_derived_type_definition] = STATE(405),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(922),
    [sym_variable_declaration] = STATE(2468),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2468),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2468),
    [sym_equivalence_statement] = STATE(2468),
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(223),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2520),
    [sym_data_statement] = STATE(2468),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2602),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_enum] = STATE(405),
    [sym_enum_statement] = STATE(1853),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(54),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat1] = STATE(59),
    [aux_sym_program_repeat2] = STATE(166),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_interface_statement_token1] = ACTIONS(21),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(29),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_use_statement_token1] = ACTIONS(39),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [aux_sym_implicit_statement_token1] = ACTIONS(41),
    [aux_sym_implicit_statement_token3] = ACTIONS(389),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(45),
    [aux_sym_private_statement_token1] = ACTIONS(47),
    [aux_sym_public_statement_token1] = ACTIONS(49),
    [aux_sym_namelist_statement_token1] = ACTIONS(51),
    [aux_sym_common_statement_token1] = ACTIONS(53),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_import_statement_token1] = ACTIONS(57),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(61),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(391),
    [aux_sym_intrinsic_type_token2] = ACTIONS(393),
    [aux_sym_intrinsic_type_token3] = ACTIONS(391),
    [aux_sym_intrinsic_type_token4] = ACTIONS(395),
    [aux_sym_intrinsic_type_token6] = ACTIONS(393),
    [aux_sym_intrinsic_type_token7] = ACTIONS(393),
    [aux_sym_intrinsic_type_token8] = ACTIONS(397),
    [aux_sym_intrinsic_type_token9] = ACTIONS(393),
    [aux_sym_intrinsic_type_token10] = ACTIONS(393),
    [aux_sym_derived_type_token1] = ACTIONS(399),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(77),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(79),
    [aux_sym_type_qualifier_token14] = ACTIONS(79),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_equivalence_statement_token1] = ACTIONS(83),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(101),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_enum_statement_token1] = ACTIONS(135),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(153),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [55] = {
    [sym_interface] = STATE(308),
    [sym_interface_statement] = STATE(769),
    [sym__specification_part] = STATE(307),
    [sym_use_statement] = STATE(2540),
    [sym_implicit_statement] = STATE(2540),
    [sym_save_statement] = STATE(2540),
    [sym_private_statement] = STATE(2540),
    [sym_public_statement] = STATE(2540),
    [sym_namelist_statement] = STATE(2540),
    [sym_common_statement] = STATE(2540),
    [sym_import_statement] = STATE(2540),
    [sym_derived_type_definition] = STATE(308),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(980),
    [sym_variable_declaration] = STATE(2540),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2540),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2540),
    [sym_equivalence_statement] = STATE(2540),
    [sym_statement_label] = STATE(1948),
    [sym_include_statement] = STATE(2540),
    [sym_data_statement] = STATE(2540),
    [sym_enum] = STATE(308),
    [sym_enum_statement] = STATE(1852),
    [sym_preproc_file_line] = STATE(55),
    [aux_sym_program_repeat1] = STATE(55),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [anon_sym_LPAREN] = ACTIONS(489),
    [aux_sym_interface_statement_token1] = ACTIONS(491),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(489),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(489),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(489),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(494),
    [aux_sym_procedure_attributes_token1] = ACTIONS(497),
    [aux_sym_procedure_attributes_token3] = ACTIONS(500),
    [aux_sym_contains_statement_token1] = ACTIONS(489),
    [aux_sym_use_statement_token1] = ACTIONS(503),
    [aux_sym_use_statement_token2] = ACTIONS(500),
    [aux_sym_implicit_statement_token1] = ACTIONS(506),
    [aux_sym_implicit_statement_token3] = ACTIONS(509),
    [aux_sym_implicit_statement_token4] = ACTIONS(500),
    [aux_sym_save_statement_token1] = ACTIONS(512),
    [aux_sym_private_statement_token1] = ACTIONS(515),
    [aux_sym_public_statement_token1] = ACTIONS(518),
    [aux_sym_namelist_statement_token1] = ACTIONS(521),
    [aux_sym_common_statement_token1] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(527),
    [aux_sym_import_statement_token1] = ACTIONS(529),
    [aux_sym_derived_type_definition_token1] = ACTIONS(532),
    [aux_sym_abstract_specifier_token1] = ACTIONS(535),
    [aux_sym_procedure_attribute_token6] = ACTIONS(500),
    [aux_sym_variable_attributes_token1] = ACTIONS(500),
    [aux_sym_variable_attributes_token2] = ACTIONS(500),
    [aux_sym_variable_attributes_token3] = ACTIONS(500),
    [aux_sym_variable_attributes_token4] = ACTIONS(500),
    [aux_sym_variable_attributes_token5] = ACTIONS(500),
    [aux_sym_intrinsic_type_token1] = ACTIONS(538),
    [aux_sym_intrinsic_type_token2] = ACTIONS(538),
    [aux_sym_intrinsic_type_token3] = ACTIONS(538),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(538),
    [aux_sym_intrinsic_type_token7] = ACTIONS(538),
    [aux_sym_intrinsic_type_token8] = ACTIONS(544),
    [aux_sym_intrinsic_type_token9] = ACTIONS(538),
    [aux_sym_intrinsic_type_token10] = ACTIONS(538),
    [aux_sym_derived_type_token1] = ACTIONS(547),
    [aux_sym_type_qualifier_token1] = ACTIONS(500),
    [aux_sym_type_qualifier_token2] = ACTIONS(500),
    [aux_sym_type_qualifier_token3] = ACTIONS(550),
    [aux_sym_type_qualifier_token4] = ACTIONS(500),
    [aux_sym_type_qualifier_token5] = ACTIONS(553),
    [aux_sym_type_qualifier_token9] = ACTIONS(500),
    [aux_sym_type_qualifier_token10] = ACTIONS(556),
    [aux_sym_type_qualifier_token11] = ACTIONS(500),
    [aux_sym_type_qualifier_token12] = ACTIONS(500),
    [aux_sym_type_qualifier_token13] = ACTIONS(500),
    [aux_sym_type_qualifier_token14] = ACTIONS(500),
    [aux_sym_type_qualifier_token15] = ACTIONS(500),
    [aux_sym_equivalence_statement_token1] = ACTIONS(559),
    [aux_sym_stop_statement_token1] = ACTIONS(489),
    [aux_sym_stop_statement_token2] = ACTIONS(489),
    [aux_sym_subroutine_call_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token2] = ACTIONS(489),
    [aux_sym_keyword_statement_token3] = ACTIONS(489),
    [aux_sym_keyword_statement_token4] = ACTIONS(489),
    [aux_sym_keyword_statement_token6] = ACTIONS(489),
    [aux_sym_keyword_statement_token7] = ACTIONS(489),
    [aux_sym_include_statement_token1] = ACTIONS(562),
    [aux_sym_data_statement_token1] = ACTIONS(565),
    [aux_sym_do_loop_statement_token1] = ACTIONS(489),
    [aux_sym__inline_if_statement_token1] = ACTIONS(489),
    [aux_sym_end_if_statement_token1] = ACTIONS(489),
    [aux_sym_elseif_clause_token2] = ACTIONS(489),
    [aux_sym__inline_where_statement_token1] = ACTIONS(489),
    [aux_sym__forall_control_expression_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token3] = ACTIONS(489),
    [aux_sym_select_type_statement_token1] = ACTIONS(489),
    [aux_sym_select_rank_statement_token2] = ACTIONS(489),
    [aux_sym_block_construct_token1] = ACTIONS(489),
    [aux_sym_associate_statement_token1] = ACTIONS(489),
    [aux_sym_format_statement_token1] = ACTIONS(489),
    [aux_sym_print_statement_token1] = ACTIONS(489),
    [aux_sym_open_statement_token1] = ACTIONS(489),
    [aux_sym_close_statement_token1] = ACTIONS(489),
    [aux_sym_inquire_statement_token1] = ACTIONS(489),
    [aux_sym_enum_statement_token1] = ACTIONS(568),
    [aux_sym_file_position_statement_token1] = ACTIONS(489),
    [aux_sym_file_position_statement_token2] = ACTIONS(489),
    [aux_sym_file_position_statement_token3] = ACTIONS(489),
    [aux_sym_file_position_statement_token4] = ACTIONS(489),
    [aux_sym_logical_expression_token5] = ACTIONS(527),
    [anon_sym_PLUS] = ACTIONS(527),
    [anon_sym_DOT] = ACTIONS(489),
    [anon_sym_LPAREN_SLASH] = ACTIONS(527),
    [anon_sym_LBRACK] = ACTIONS(527),
    [sym_boolean_literal] = ACTIONS(527),
    [aux_sym_null_literal_token1] = ACTIONS(489),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(489),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
    [sym__float_literal] = ACTIONS(527),
    [sym__boz_literal] = ACTIONS(527),
    [sym_string_literal] = ACTIONS(527),
  },
  [56] = {
    [sym_interface] = STATE(312),
    [sym_interface_statement] = STATE(761),
    [sym__specification_part] = STATE(313),
    [sym_use_statement] = STATE(2798),
    [sym_implicit_statement] = STATE(2798),
    [sym_save_statement] = STATE(2798),
    [sym_private_statement] = STATE(2798),
    [sym_public_statement] = STATE(2798),
    [sym_namelist_statement] = STATE(2798),
    [sym_common_statement] = STATE(2798),
    [sym_import_statement] = STATE(2798),
    [sym_derived_type_definition] = STATE(312),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(963),
    [sym_variable_declaration] = STATE(2798),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2798),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2798),
    [sym_equivalence_statement] = STATE(2798),
    [sym_statement_label] = STATE(1964),
    [sym_include_statement] = STATE(2798),
    [sym_data_statement] = STATE(2798),
    [sym_enum] = STATE(312),
    [sym_enum_statement] = STATE(1859),
    [sym_preproc_file_line] = STATE(56),
    [aux_sym_program_repeat1] = STATE(56),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [aux_sym_end_program_statement_token2] = ACTIONS(489),
    [anon_sym_LPAREN] = ACTIONS(489),
    [aux_sym_interface_statement_token1] = ACTIONS(491),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(489),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(489),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(494),
    [aux_sym_procedure_attributes_token1] = ACTIONS(497),
    [aux_sym_procedure_attributes_token3] = ACTIONS(500),
    [aux_sym_contains_statement_token1] = ACTIONS(489),
    [aux_sym_use_statement_token1] = ACTIONS(503),
    [aux_sym_use_statement_token2] = ACTIONS(500),
    [aux_sym_implicit_statement_token1] = ACTIONS(506),
    [aux_sym_implicit_statement_token3] = ACTIONS(509),
    [aux_sym_implicit_statement_token4] = ACTIONS(500),
    [aux_sym_save_statement_token1] = ACTIONS(512),
    [aux_sym_private_statement_token1] = ACTIONS(515),
    [aux_sym_public_statement_token1] = ACTIONS(518),
    [aux_sym_namelist_statement_token1] = ACTIONS(521),
    [aux_sym_common_statement_token1] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(527),
    [aux_sym_import_statement_token1] = ACTIONS(529),
    [aux_sym_derived_type_definition_token1] = ACTIONS(532),
    [aux_sym_abstract_specifier_token1] = ACTIONS(535),
    [aux_sym_procedure_attribute_token6] = ACTIONS(500),
    [aux_sym_variable_attributes_token1] = ACTIONS(500),
    [aux_sym_variable_attributes_token2] = ACTIONS(500),
    [aux_sym_variable_attributes_token3] = ACTIONS(500),
    [aux_sym_variable_attributes_token4] = ACTIONS(500),
    [aux_sym_variable_attributes_token5] = ACTIONS(500),
    [aux_sym_intrinsic_type_token1] = ACTIONS(538),
    [aux_sym_intrinsic_type_token2] = ACTIONS(538),
    [aux_sym_intrinsic_type_token3] = ACTIONS(538),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(538),
    [aux_sym_intrinsic_type_token7] = ACTIONS(538),
    [aux_sym_intrinsic_type_token8] = ACTIONS(544),
    [aux_sym_intrinsic_type_token9] = ACTIONS(538),
    [aux_sym_intrinsic_type_token10] = ACTIONS(538),
    [aux_sym_derived_type_token1] = ACTIONS(547),
    [aux_sym_type_qualifier_token1] = ACTIONS(500),
    [aux_sym_type_qualifier_token2] = ACTIONS(500),
    [aux_sym_type_qualifier_token3] = ACTIONS(550),
    [aux_sym_type_qualifier_token4] = ACTIONS(500),
    [aux_sym_type_qualifier_token5] = ACTIONS(553),
    [aux_sym_type_qualifier_token9] = ACTIONS(500),
    [aux_sym_type_qualifier_token10] = ACTIONS(556),
    [aux_sym_type_qualifier_token11] = ACTIONS(500),
    [aux_sym_type_qualifier_token12] = ACTIONS(500),
    [aux_sym_type_qualifier_token13] = ACTIONS(500),
    [aux_sym_type_qualifier_token14] = ACTIONS(500),
    [aux_sym_type_qualifier_token15] = ACTIONS(500),
    [aux_sym_equivalence_statement_token1] = ACTIONS(559),
    [aux_sym_stop_statement_token1] = ACTIONS(489),
    [aux_sym_stop_statement_token2] = ACTIONS(489),
    [aux_sym_subroutine_call_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token2] = ACTIONS(489),
    [aux_sym_keyword_statement_token3] = ACTIONS(489),
    [aux_sym_keyword_statement_token4] = ACTIONS(489),
    [aux_sym_keyword_statement_token6] = ACTIONS(489),
    [aux_sym_keyword_statement_token7] = ACTIONS(489),
    [aux_sym_include_statement_token1] = ACTIONS(562),
    [aux_sym_data_statement_token1] = ACTIONS(565),
    [aux_sym_do_loop_statement_token1] = ACTIONS(489),
    [aux_sym__inline_if_statement_token1] = ACTIONS(489),
    [aux_sym_end_if_statement_token1] = ACTIONS(489),
    [aux_sym_elseif_clause_token2] = ACTIONS(489),
    [aux_sym__inline_where_statement_token1] = ACTIONS(489),
    [aux_sym__forall_control_expression_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token3] = ACTIONS(489),
    [aux_sym_select_type_statement_token1] = ACTIONS(489),
    [aux_sym_select_rank_statement_token2] = ACTIONS(489),
    [aux_sym_block_construct_token1] = ACTIONS(489),
    [aux_sym_associate_statement_token1] = ACTIONS(489),
    [aux_sym_format_statement_token1] = ACTIONS(489),
    [aux_sym_print_statement_token1] = ACTIONS(489),
    [aux_sym_open_statement_token1] = ACTIONS(489),
    [aux_sym_close_statement_token1] = ACTIONS(489),
    [aux_sym_inquire_statement_token1] = ACTIONS(489),
    [aux_sym_enum_statement_token1] = ACTIONS(568),
    [aux_sym_file_position_statement_token1] = ACTIONS(489),
    [aux_sym_file_position_statement_token2] = ACTIONS(489),
    [aux_sym_file_position_statement_token3] = ACTIONS(489),
    [aux_sym_file_position_statement_token4] = ACTIONS(489),
    [aux_sym_logical_expression_token5] = ACTIONS(527),
    [anon_sym_PLUS] = ACTIONS(527),
    [anon_sym_DOT] = ACTIONS(489),
    [anon_sym_LPAREN_SLASH] = ACTIONS(527),
    [anon_sym_LBRACK] = ACTIONS(527),
    [sym_boolean_literal] = ACTIONS(527),
    [aux_sym_null_literal_token1] = ACTIONS(489),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(489),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
    [sym__float_literal] = ACTIONS(527),
    [sym__boz_literal] = ACTIONS(527),
    [sym_string_literal] = ACTIONS(527),
  },
  [57] = {
    [sym_interface] = STATE(386),
    [sym_interface_statement] = STATE(756),
    [sym__specification_part] = STATE(384),
    [sym_use_statement] = STATE(2681),
    [sym_implicit_statement] = STATE(2681),
    [sym_save_statement] = STATE(2681),
    [sym_private_statement] = STATE(2681),
    [sym_public_statement] = STATE(2681),
    [sym_namelist_statement] = STATE(2681),
    [sym_common_statement] = STATE(2681),
    [sym_import_statement] = STATE(2681),
    [sym_derived_type_definition] = STATE(386),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(935),
    [sym_variable_declaration] = STATE(2681),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2681),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2681),
    [sym_equivalence_statement] = STATE(2681),
    [sym_statement_label] = STATE(1943),
    [sym_include_statement] = STATE(2681),
    [sym_data_statement] = STATE(2681),
    [sym_enum] = STATE(386),
    [sym_enum_statement] = STATE(1844),
    [sym_preproc_file_line] = STATE(57),
    [aux_sym_program_repeat1] = STATE(57),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [anon_sym_LPAREN] = ACTIONS(489),
    [aux_sym_interface_statement_token1] = ACTIONS(491),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(489),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(489),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(494),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(489),
    [aux_sym_procedure_attributes_token1] = ACTIONS(497),
    [aux_sym_procedure_attributes_token3] = ACTIONS(500),
    [aux_sym_contains_statement_token1] = ACTIONS(489),
    [aux_sym_use_statement_token1] = ACTIONS(503),
    [aux_sym_use_statement_token2] = ACTIONS(500),
    [aux_sym_implicit_statement_token1] = ACTIONS(506),
    [aux_sym_implicit_statement_token3] = ACTIONS(509),
    [aux_sym_implicit_statement_token4] = ACTIONS(500),
    [aux_sym_save_statement_token1] = ACTIONS(512),
    [aux_sym_private_statement_token1] = ACTIONS(515),
    [aux_sym_public_statement_token1] = ACTIONS(518),
    [aux_sym_namelist_statement_token1] = ACTIONS(521),
    [aux_sym_common_statement_token1] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(527),
    [aux_sym_import_statement_token1] = ACTIONS(529),
    [aux_sym_derived_type_definition_token1] = ACTIONS(532),
    [aux_sym_abstract_specifier_token1] = ACTIONS(535),
    [aux_sym_procedure_attribute_token6] = ACTIONS(500),
    [aux_sym_variable_attributes_token1] = ACTIONS(500),
    [aux_sym_variable_attributes_token2] = ACTIONS(500),
    [aux_sym_variable_attributes_token3] = ACTIONS(500),
    [aux_sym_variable_attributes_token4] = ACTIONS(500),
    [aux_sym_variable_attributes_token5] = ACTIONS(500),
    [aux_sym_intrinsic_type_token1] = ACTIONS(538),
    [aux_sym_intrinsic_type_token2] = ACTIONS(538),
    [aux_sym_intrinsic_type_token3] = ACTIONS(538),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(538),
    [aux_sym_intrinsic_type_token7] = ACTIONS(538),
    [aux_sym_intrinsic_type_token8] = ACTIONS(544),
    [aux_sym_intrinsic_type_token9] = ACTIONS(538),
    [aux_sym_intrinsic_type_token10] = ACTIONS(538),
    [aux_sym_derived_type_token1] = ACTIONS(547),
    [aux_sym_type_qualifier_token1] = ACTIONS(500),
    [aux_sym_type_qualifier_token2] = ACTIONS(500),
    [aux_sym_type_qualifier_token3] = ACTIONS(550),
    [aux_sym_type_qualifier_token4] = ACTIONS(500),
    [aux_sym_type_qualifier_token5] = ACTIONS(553),
    [aux_sym_type_qualifier_token9] = ACTIONS(500),
    [aux_sym_type_qualifier_token10] = ACTIONS(556),
    [aux_sym_type_qualifier_token11] = ACTIONS(500),
    [aux_sym_type_qualifier_token12] = ACTIONS(500),
    [aux_sym_type_qualifier_token13] = ACTIONS(500),
    [aux_sym_type_qualifier_token14] = ACTIONS(500),
    [aux_sym_type_qualifier_token15] = ACTIONS(500),
    [aux_sym_equivalence_statement_token1] = ACTIONS(559),
    [aux_sym_stop_statement_token1] = ACTIONS(489),
    [aux_sym_stop_statement_token2] = ACTIONS(489),
    [aux_sym_subroutine_call_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token2] = ACTIONS(489),
    [aux_sym_keyword_statement_token3] = ACTIONS(489),
    [aux_sym_keyword_statement_token4] = ACTIONS(489),
    [aux_sym_keyword_statement_token6] = ACTIONS(489),
    [aux_sym_keyword_statement_token7] = ACTIONS(489),
    [aux_sym_include_statement_token1] = ACTIONS(562),
    [aux_sym_data_statement_token1] = ACTIONS(565),
    [aux_sym_do_loop_statement_token1] = ACTIONS(489),
    [aux_sym__inline_if_statement_token1] = ACTIONS(489),
    [aux_sym_end_if_statement_token1] = ACTIONS(489),
    [aux_sym_elseif_clause_token2] = ACTIONS(489),
    [aux_sym__inline_where_statement_token1] = ACTIONS(489),
    [aux_sym__forall_control_expression_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token3] = ACTIONS(489),
    [aux_sym_select_type_statement_token1] = ACTIONS(489),
    [aux_sym_select_rank_statement_token2] = ACTIONS(489),
    [aux_sym_block_construct_token1] = ACTIONS(489),
    [aux_sym_associate_statement_token1] = ACTIONS(489),
    [aux_sym_format_statement_token1] = ACTIONS(489),
    [aux_sym_print_statement_token1] = ACTIONS(489),
    [aux_sym_open_statement_token1] = ACTIONS(489),
    [aux_sym_close_statement_token1] = ACTIONS(489),
    [aux_sym_inquire_statement_token1] = ACTIONS(489),
    [aux_sym_enum_statement_token1] = ACTIONS(568),
    [aux_sym_file_position_statement_token1] = ACTIONS(489),
    [aux_sym_file_position_statement_token2] = ACTIONS(489),
    [aux_sym_file_position_statement_token3] = ACTIONS(489),
    [aux_sym_file_position_statement_token4] = ACTIONS(489),
    [aux_sym_logical_expression_token5] = ACTIONS(527),
    [anon_sym_PLUS] = ACTIONS(527),
    [anon_sym_DOT] = ACTIONS(489),
    [anon_sym_LPAREN_SLASH] = ACTIONS(527),
    [anon_sym_LBRACK] = ACTIONS(527),
    [sym_boolean_literal] = ACTIONS(527),
    [aux_sym_null_literal_token1] = ACTIONS(489),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(489),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
    [sym__float_literal] = ACTIONS(527),
    [sym__boz_literal] = ACTIONS(527),
    [sym_string_literal] = ACTIONS(527),
  },
  [58] = {
    [sym_interface] = STATE(352),
    [sym_interface_statement] = STATE(759),
    [sym__specification_part] = STATE(351),
    [sym_use_statement] = STATE(2612),
    [sym_implicit_statement] = STATE(2612),
    [sym_save_statement] = STATE(2612),
    [sym_private_statement] = STATE(2612),
    [sym_public_statement] = STATE(2612),
    [sym_namelist_statement] = STATE(2612),
    [sym_common_statement] = STATE(2612),
    [sym_import_statement] = STATE(2612),
    [sym_derived_type_definition] = STATE(352),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(940),
    [sym_variable_declaration] = STATE(2612),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2612),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2612),
    [sym_equivalence_statement] = STATE(2612),
    [sym_statement_label] = STATE(1945),
    [sym_include_statement] = STATE(2612),
    [sym_data_statement] = STATE(2612),
    [sym_enum] = STATE(352),
    [sym_enum_statement] = STATE(1849),
    [sym_preproc_file_line] = STATE(58),
    [aux_sym_program_repeat1] = STATE(58),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [anon_sym_LPAREN] = ACTIONS(489),
    [aux_sym_interface_statement_token1] = ACTIONS(491),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(489),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(489),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(494),
    [aux_sym_procedure_attributes_token1] = ACTIONS(497),
    [aux_sym_procedure_attributes_token3] = ACTIONS(500),
    [aux_sym_end_function_statement_token1] = ACTIONS(489),
    [aux_sym_contains_statement_token1] = ACTIONS(489),
    [aux_sym_use_statement_token1] = ACTIONS(503),
    [aux_sym_use_statement_token2] = ACTIONS(500),
    [aux_sym_implicit_statement_token1] = ACTIONS(506),
    [aux_sym_implicit_statement_token3] = ACTIONS(509),
    [aux_sym_implicit_statement_token4] = ACTIONS(500),
    [aux_sym_save_statement_token1] = ACTIONS(512),
    [aux_sym_private_statement_token1] = ACTIONS(515),
    [aux_sym_public_statement_token1] = ACTIONS(518),
    [aux_sym_namelist_statement_token1] = ACTIONS(521),
    [aux_sym_common_statement_token1] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(527),
    [aux_sym_import_statement_token1] = ACTIONS(529),
    [aux_sym_derived_type_definition_token1] = ACTIONS(532),
    [aux_sym_abstract_specifier_token1] = ACTIONS(535),
    [aux_sym_procedure_attribute_token6] = ACTIONS(500),
    [aux_sym_variable_attributes_token1] = ACTIONS(500),
    [aux_sym_variable_attributes_token2] = ACTIONS(500),
    [aux_sym_variable_attributes_token3] = ACTIONS(500),
    [aux_sym_variable_attributes_token4] = ACTIONS(500),
    [aux_sym_variable_attributes_token5] = ACTIONS(500),
    [aux_sym_intrinsic_type_token1] = ACTIONS(538),
    [aux_sym_intrinsic_type_token2] = ACTIONS(538),
    [aux_sym_intrinsic_type_token3] = ACTIONS(538),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(538),
    [aux_sym_intrinsic_type_token7] = ACTIONS(538),
    [aux_sym_intrinsic_type_token8] = ACTIONS(544),
    [aux_sym_intrinsic_type_token9] = ACTIONS(538),
    [aux_sym_intrinsic_type_token10] = ACTIONS(538),
    [aux_sym_derived_type_token1] = ACTIONS(547),
    [aux_sym_type_qualifier_token1] = ACTIONS(500),
    [aux_sym_type_qualifier_token2] = ACTIONS(500),
    [aux_sym_type_qualifier_token3] = ACTIONS(550),
    [aux_sym_type_qualifier_token4] = ACTIONS(500),
    [aux_sym_type_qualifier_token5] = ACTIONS(553),
    [aux_sym_type_qualifier_token9] = ACTIONS(500),
    [aux_sym_type_qualifier_token10] = ACTIONS(556),
    [aux_sym_type_qualifier_token11] = ACTIONS(500),
    [aux_sym_type_qualifier_token12] = ACTIONS(500),
    [aux_sym_type_qualifier_token13] = ACTIONS(500),
    [aux_sym_type_qualifier_token14] = ACTIONS(500),
    [aux_sym_type_qualifier_token15] = ACTIONS(500),
    [aux_sym_equivalence_statement_token1] = ACTIONS(559),
    [aux_sym_stop_statement_token1] = ACTIONS(489),
    [aux_sym_stop_statement_token2] = ACTIONS(489),
    [aux_sym_subroutine_call_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token2] = ACTIONS(489),
    [aux_sym_keyword_statement_token3] = ACTIONS(489),
    [aux_sym_keyword_statement_token4] = ACTIONS(489),
    [aux_sym_keyword_statement_token6] = ACTIONS(489),
    [aux_sym_keyword_statement_token7] = ACTIONS(489),
    [aux_sym_include_statement_token1] = ACTIONS(562),
    [aux_sym_data_statement_token1] = ACTIONS(565),
    [aux_sym_do_loop_statement_token1] = ACTIONS(489),
    [aux_sym__inline_if_statement_token1] = ACTIONS(489),
    [aux_sym_end_if_statement_token1] = ACTIONS(489),
    [aux_sym_elseif_clause_token2] = ACTIONS(489),
    [aux_sym__inline_where_statement_token1] = ACTIONS(489),
    [aux_sym__forall_control_expression_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token3] = ACTIONS(489),
    [aux_sym_select_type_statement_token1] = ACTIONS(489),
    [aux_sym_select_rank_statement_token2] = ACTIONS(489),
    [aux_sym_block_construct_token1] = ACTIONS(489),
    [aux_sym_associate_statement_token1] = ACTIONS(489),
    [aux_sym_format_statement_token1] = ACTIONS(489),
    [aux_sym_print_statement_token1] = ACTIONS(489),
    [aux_sym_open_statement_token1] = ACTIONS(489),
    [aux_sym_close_statement_token1] = ACTIONS(489),
    [aux_sym_inquire_statement_token1] = ACTIONS(489),
    [aux_sym_enum_statement_token1] = ACTIONS(568),
    [aux_sym_file_position_statement_token1] = ACTIONS(489),
    [aux_sym_file_position_statement_token2] = ACTIONS(489),
    [aux_sym_file_position_statement_token3] = ACTIONS(489),
    [aux_sym_file_position_statement_token4] = ACTIONS(489),
    [aux_sym_logical_expression_token5] = ACTIONS(527),
    [anon_sym_PLUS] = ACTIONS(527),
    [anon_sym_DOT] = ACTIONS(489),
    [anon_sym_LPAREN_SLASH] = ACTIONS(527),
    [anon_sym_LBRACK] = ACTIONS(527),
    [sym_boolean_literal] = ACTIONS(527),
    [aux_sym_null_literal_token1] = ACTIONS(489),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(489),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
    [sym__float_literal] = ACTIONS(527),
    [sym__boz_literal] = ACTIONS(527),
    [sym_string_literal] = ACTIONS(527),
  },
  [59] = {
    [sym_interface] = STATE(405),
    [sym_interface_statement] = STATE(770),
    [sym__specification_part] = STATE(415),
    [sym_use_statement] = STATE(2468),
    [sym_implicit_statement] = STATE(2468),
    [sym_save_statement] = STATE(2468),
    [sym_private_statement] = STATE(2468),
    [sym_public_statement] = STATE(2468),
    [sym_namelist_statement] = STATE(2468),
    [sym_common_statement] = STATE(2468),
    [sym_import_statement] = STATE(2468),
    [sym_derived_type_definition] = STATE(405),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(922),
    [sym_variable_declaration] = STATE(2468),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2468),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2468),
    [sym_equivalence_statement] = STATE(2468),
    [sym_statement_label] = STATE(1951),
    [sym_include_statement] = STATE(2468),
    [sym_data_statement] = STATE(2468),
    [sym_enum] = STATE(405),
    [sym_enum_statement] = STATE(1853),
    [sym_preproc_file_line] = STATE(59),
    [aux_sym_program_repeat1] = STATE(59),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [anon_sym_LPAREN] = ACTIONS(489),
    [aux_sym_interface_statement_token1] = ACTIONS(491),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(489),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(489),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(494),
    [aux_sym_procedure_attributes_token1] = ACTIONS(497),
    [aux_sym_procedure_attributes_token3] = ACTIONS(500),
    [aux_sym_use_statement_token1] = ACTIONS(503),
    [aux_sym_use_statement_token2] = ACTIONS(500),
    [aux_sym_implicit_statement_token1] = ACTIONS(506),
    [aux_sym_implicit_statement_token3] = ACTIONS(509),
    [aux_sym_implicit_statement_token4] = ACTIONS(500),
    [aux_sym_save_statement_token1] = ACTIONS(512),
    [aux_sym_private_statement_token1] = ACTIONS(515),
    [aux_sym_public_statement_token1] = ACTIONS(518),
    [aux_sym_namelist_statement_token1] = ACTIONS(521),
    [aux_sym_common_statement_token1] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(527),
    [aux_sym_import_statement_token1] = ACTIONS(529),
    [aux_sym_derived_type_definition_token1] = ACTIONS(532),
    [aux_sym_abstract_specifier_token1] = ACTIONS(535),
    [aux_sym_procedure_attribute_token6] = ACTIONS(500),
    [aux_sym_variable_attributes_token1] = ACTIONS(500),
    [aux_sym_variable_attributes_token2] = ACTIONS(500),
    [aux_sym_variable_attributes_token3] = ACTIONS(500),
    [aux_sym_variable_attributes_token4] = ACTIONS(500),
    [aux_sym_variable_attributes_token5] = ACTIONS(500),
    [aux_sym_intrinsic_type_token1] = ACTIONS(538),
    [aux_sym_intrinsic_type_token2] = ACTIONS(538),
    [aux_sym_intrinsic_type_token3] = ACTIONS(538),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(538),
    [aux_sym_intrinsic_type_token7] = ACTIONS(538),
    [aux_sym_intrinsic_type_token8] = ACTIONS(544),
    [aux_sym_intrinsic_type_token9] = ACTIONS(538),
    [aux_sym_intrinsic_type_token10] = ACTIONS(538),
    [aux_sym_derived_type_token1] = ACTIONS(547),
    [aux_sym_type_qualifier_token1] = ACTIONS(500),
    [aux_sym_type_qualifier_token2] = ACTIONS(500),
    [aux_sym_type_qualifier_token3] = ACTIONS(550),
    [aux_sym_type_qualifier_token4] = ACTIONS(500),
    [aux_sym_type_qualifier_token5] = ACTIONS(553),
    [aux_sym_type_qualifier_token9] = ACTIONS(500),
    [aux_sym_type_qualifier_token10] = ACTIONS(556),
    [aux_sym_type_qualifier_token11] = ACTIONS(500),
    [aux_sym_type_qualifier_token12] = ACTIONS(500),
    [aux_sym_type_qualifier_token13] = ACTIONS(500),
    [aux_sym_type_qualifier_token14] = ACTIONS(500),
    [aux_sym_type_qualifier_token15] = ACTIONS(500),
    [aux_sym_equivalence_statement_token1] = ACTIONS(559),
    [aux_sym_stop_statement_token1] = ACTIONS(489),
    [aux_sym_stop_statement_token2] = ACTIONS(489),
    [aux_sym_subroutine_call_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token1] = ACTIONS(489),
    [aux_sym_keyword_statement_token2] = ACTIONS(489),
    [aux_sym_keyword_statement_token3] = ACTIONS(489),
    [aux_sym_keyword_statement_token4] = ACTIONS(489),
    [aux_sym_keyword_statement_token6] = ACTIONS(489),
    [aux_sym_keyword_statement_token7] = ACTIONS(489),
    [aux_sym_include_statement_token1] = ACTIONS(562),
    [aux_sym_data_statement_token1] = ACTIONS(565),
    [aux_sym_do_loop_statement_token1] = ACTIONS(489),
    [aux_sym__inline_if_statement_token1] = ACTIONS(489),
    [aux_sym_end_if_statement_token1] = ACTIONS(489),
    [aux_sym_elseif_clause_token2] = ACTIONS(489),
    [aux_sym__inline_where_statement_token1] = ACTIONS(489),
    [aux_sym__forall_control_expression_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token1] = ACTIONS(489),
    [aux_sym_select_case_statement_token3] = ACTIONS(489),
    [aux_sym_select_type_statement_token1] = ACTIONS(489),
    [aux_sym_select_rank_statement_token2] = ACTIONS(489),
    [aux_sym_block_construct_token1] = ACTIONS(489),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(489),
    [aux_sym_associate_statement_token1] = ACTIONS(489),
    [aux_sym_format_statement_token1] = ACTIONS(489),
    [aux_sym_print_statement_token1] = ACTIONS(489),
    [aux_sym_open_statement_token1] = ACTIONS(489),
    [aux_sym_close_statement_token1] = ACTIONS(489),
    [aux_sym_inquire_statement_token1] = ACTIONS(489),
    [aux_sym_enum_statement_token1] = ACTIONS(568),
    [aux_sym_file_position_statement_token1] = ACTIONS(489),
    [aux_sym_file_position_statement_token2] = ACTIONS(489),
    [aux_sym_file_position_statement_token3] = ACTIONS(489),
    [aux_sym_file_position_statement_token4] = ACTIONS(489),
    [aux_sym_logical_expression_token5] = ACTIONS(527),
    [anon_sym_PLUS] = ACTIONS(527),
    [anon_sym_DOT] = ACTIONS(489),
    [anon_sym_LPAREN_SLASH] = ACTIONS(527),
    [anon_sym_LBRACK] = ACTIONS(527),
    [sym_boolean_literal] = ACTIONS(527),
    [aux_sym_null_literal_token1] = ACTIONS(489),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(489),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
    [sym__float_literal] = ACTIONS(527),
    [sym__boz_literal] = ACTIONS(527),
    [sym_string_literal] = ACTIONS(527),
  },
  [60] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(60),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(81),
    [aux_sym_end_program_statement_token1] = ACTIONS(574),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(577),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(577),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(577),
    [aux_sym_type_statement_token2] = ACTIONS(577),
    [aux_sym_type_statement_token3] = ACTIONS(577),
    [aux_sym__class_default_token1] = ACTIONS(577),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(581),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [61] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(61),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(584),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(587),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(587),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(587),
    [aux_sym_type_statement_token2] = ACTIONS(587),
    [aux_sym_type_statement_token3] = ACTIONS(587),
    [aux_sym__class_default_token1] = ACTIONS(587),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(589),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [62] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(62),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(592),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(595),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(595),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(595),
    [aux_sym_type_statement_token2] = ACTIONS(595),
    [aux_sym_type_statement_token3] = ACTIONS(595),
    [aux_sym__class_default_token1] = ACTIONS(595),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(597),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [63] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(63),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(74),
    [aux_sym_end_program_statement_token1] = ACTIONS(600),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(603),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(603),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(603),
    [aux_sym_type_statement_token2] = ACTIONS(603),
    [aux_sym_type_statement_token3] = ACTIONS(603),
    [aux_sym__class_default_token1] = ACTIONS(603),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(605),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [64] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(64),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(65),
    [aux_sym_end_program_statement_token1] = ACTIONS(608),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(611),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(611),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(611),
    [aux_sym_type_statement_token2] = ACTIONS(611),
    [aux_sym_type_statement_token3] = ACTIONS(611),
    [aux_sym__class_default_token1] = ACTIONS(611),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(613),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [65] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(65),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(616),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(619),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(619),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(619),
    [aux_sym_type_statement_token2] = ACTIONS(619),
    [aux_sym_type_statement_token3] = ACTIONS(619),
    [aux_sym__class_default_token1] = ACTIONS(619),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(621),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [66] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(66),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(624),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(627),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(627),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(627),
    [aux_sym_type_statement_token2] = ACTIONS(627),
    [aux_sym_type_statement_token3] = ACTIONS(627),
    [aux_sym__class_default_token1] = ACTIONS(627),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(629),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [67] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(67),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(632),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(635),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(635),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(635),
    [aux_sym_type_statement_token2] = ACTIONS(635),
    [aux_sym_type_statement_token3] = ACTIONS(635),
    [aux_sym__class_default_token1] = ACTIONS(635),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(637),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [68] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_end_where_statement] = STATE(2658),
    [sym_elsewhere_clause] = STATE(1910),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(68),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(80),
    [aux_sym__block_where_statement_repeat1] = STATE(1825),
    [aux_sym_end_program_statement_token1] = ACTIONS(640),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(642),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(644),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(646),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [69] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_end_where_statement] = STATE(2658),
    [sym_elsewhere_clause] = STATE(1910),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(69),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym__block_where_statement_repeat1] = STATE(1825),
    [aux_sym_end_program_statement_token1] = ACTIONS(640),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(642),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(644),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(646),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [70] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(70),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(66),
    [aux_sym_end_program_statement_token1] = ACTIONS(650),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(653),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(653),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(653),
    [aux_sym_type_statement_token2] = ACTIONS(653),
    [aux_sym_type_statement_token3] = ACTIONS(653),
    [aux_sym__class_default_token1] = ACTIONS(653),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(655),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [71] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(71),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(62),
    [aux_sym_end_program_statement_token1] = ACTIONS(616),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(619),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(619),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(619),
    [aux_sym_type_statement_token2] = ACTIONS(619),
    [aux_sym_type_statement_token3] = ACTIONS(619),
    [aux_sym__class_default_token1] = ACTIONS(619),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(621),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [72] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(72),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(658),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(661),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(661),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(661),
    [aux_sym_type_statement_token2] = ACTIONS(661),
    [aux_sym_type_statement_token3] = ACTIONS(661),
    [aux_sym__class_default_token1] = ACTIONS(661),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(663),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [73] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(73),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(83),
    [aux_sym_end_program_statement_token1] = ACTIONS(666),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(669),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(669),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(669),
    [aux_sym_type_statement_token2] = ACTIONS(669),
    [aux_sym_type_statement_token3] = ACTIONS(669),
    [aux_sym__class_default_token1] = ACTIONS(669),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(671),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [74] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(74),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(674),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(677),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(677),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(677),
    [aux_sym_type_statement_token2] = ACTIONS(677),
    [aux_sym_type_statement_token3] = ACTIONS(677),
    [aux_sym__class_default_token1] = ACTIONS(677),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(679),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [75] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_end_where_statement] = STATE(2492),
    [sym_elsewhere_clause] = STATE(1910),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(75),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(69),
    [aux_sym__block_where_statement_repeat1] = STATE(1829),
    [aux_sym_end_program_statement_token1] = ACTIONS(640),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(642),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(644),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(646),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [76] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(76),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(79),
    [aux_sym_end_program_statement_token1] = ACTIONS(674),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(677),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(677),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(677),
    [aux_sym_type_statement_token2] = ACTIONS(677),
    [aux_sym_type_statement_token3] = ACTIONS(677),
    [aux_sym__class_default_token1] = ACTIONS(677),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(679),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [77] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(77),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(72),
    [aux_sym_end_program_statement_token1] = ACTIONS(584),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(587),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(587),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(587),
    [aux_sym_type_statement_token2] = ACTIONS(587),
    [aux_sym_type_statement_token3] = ACTIONS(587),
    [aux_sym__class_default_token1] = ACTIONS(587),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(589),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [78] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(78),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(67),
    [aux_sym_end_program_statement_token1] = ACTIONS(624),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(627),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(627),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(627),
    [aux_sym_type_statement_token2] = ACTIONS(627),
    [aux_sym_type_statement_token3] = ACTIONS(627),
    [aux_sym__class_default_token1] = ACTIONS(627),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(629),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [79] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(79),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(682),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(685),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(685),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(685),
    [aux_sym_type_statement_token2] = ACTIONS(685),
    [aux_sym_type_statement_token3] = ACTIONS(685),
    [aux_sym__class_default_token1] = ACTIONS(685),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(687),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [80] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_end_where_statement] = STATE(2471),
    [sym_elsewhere_clause] = STATE(1910),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(80),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym__block_where_statement_repeat1] = STATE(1824),
    [aux_sym_end_program_statement_token1] = ACTIONS(640),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(642),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(644),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(646),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [81] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(81),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(690),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(693),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(693),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(693),
    [aux_sym_type_statement_token2] = ACTIONS(693),
    [aux_sym_type_statement_token3] = ACTIONS(693),
    [aux_sym__class_default_token1] = ACTIONS(693),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(695),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [82] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(82),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [aux_sym_implicit_statement_token3] = ACTIONS(710),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_derived_type_token1] = ACTIONS(710),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_end_select_statement_token1] = ACTIONS(710),
    [aux_sym_type_statement_token2] = ACTIONS(710),
    [aux_sym_type_statement_token3] = ACTIONS(710),
    [aux_sym__class_default_token1] = ACTIONS(710),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [83] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(83),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(82),
    [aux_sym_end_program_statement_token1] = ACTIONS(574),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(577),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(577),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(577),
    [aux_sym_type_statement_token2] = ACTIONS(577),
    [aux_sym_type_statement_token3] = ACTIONS(577),
    [aux_sym__class_default_token1] = ACTIONS(577),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(581),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [84] = {
    [sym__statements] = STATE(2765),
    [sym_statement_label] = STATE(269),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(84),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(61),
    [aux_sym_end_program_statement_token1] = ACTIONS(820),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(823),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_derived_type_token1] = ACTIONS(823),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(823),
    [aux_sym_type_statement_token2] = ACTIONS(823),
    [aux_sym_type_statement_token3] = ACTIONS(823),
    [aux_sym__class_default_token1] = ACTIONS(823),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(825),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [85] = {
    [sym_end_module_procedure_statement] = STATE(1640),
    [sym_internal_procedures] = STATE(2056),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(85),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(453),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(455),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [86] = {
    [sym_end_function_statement] = STATE(1594),
    [sym_internal_procedures] = STATE(1957),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(86),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(433),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(435),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [87] = {
    [sym_end_function_statement] = STATE(270),
    [sym_internal_procedures] = STATE(1963),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(87),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(407),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [88] = {
    [sym_end_module_procedure_statement] = STATE(1576),
    [sym_internal_procedures] = STATE(1928),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(88),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(417),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(419),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [89] = {
    [sym_end_module_procedure_statement] = STATE(1573),
    [sym_internal_procedures] = STATE(1929),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(89),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(417),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(419),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [90] = {
    [sym_end_subroutine_statement] = STATE(242),
    [sym_internal_procedures] = STATE(1998),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(90),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(477),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(479),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [91] = {
    [sym_end_function_statement] = STATE(250),
    [sym_internal_procedures] = STATE(2001),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(91),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(405),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(407),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [92] = {
    [sym_end_module_procedure_statement] = STATE(1673),
    [sym_internal_procedures] = STATE(2129),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(92),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(437),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(439),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [93] = {
    [sym_end_function_statement] = STATE(1196),
    [sym_internal_procedures] = STATE(2179),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(93),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(475),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [94] = {
    [sym_end_module_procedure_statement] = STATE(1662),
    [sym_internal_procedures] = STATE(2121),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(94),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(437),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(439),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [95] = {
    [sym_end_program_statement] = STATE(247),
    [sym_internal_procedures] = STATE(1949),
    [sym_contains_statement] = STATE(2802),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(268),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(95),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(178),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [96] = {
    [sym_end_module_procedure_statement] = STATE(1668),
    [sym_internal_procedures] = STATE(1967),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(96),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(413),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(415),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [97] = {
    [sym_end_module_procedure_statement] = STATE(1643),
    [sym_internal_procedures] = STATE(2061),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(97),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(453),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(455),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [98] = {
    [sym_end_subroutine_statement] = STATE(1650),
    [sym_internal_procedures] = STATE(2098),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(98),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(409),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(411),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [99] = {
    [sym_end_function_statement] = STATE(1632),
    [sym_internal_procedures] = STATE(2045),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(99),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(457),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(459),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [100] = {
    [sym_end_program_statement] = STATE(232),
    [sym_internal_procedures] = STATE(2004),
    [sym_contains_statement] = STATE(2802),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(268),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(100),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(178),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [101] = {
    [sym_end_function_statement] = STATE(1586),
    [sym_internal_procedures] = STATE(1952),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(101),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(481),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(483),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [102] = {
    [sym_end_subroutine_statement] = STATE(1589),
    [sym_internal_procedures] = STATE(1953),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(102),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(425),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(427),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [103] = {
    [sym_end_function_statement] = STATE(1652),
    [sym_internal_procedures] = STATE(2102),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(103),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(403),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [104] = {
    [sym_end_subroutine_statement] = STATE(1683),
    [sym_internal_procedures] = STATE(2164),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(104),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(423),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [105] = {
    [sym_end_subroutine_statement] = STATE(1657),
    [sym_internal_procedures] = STATE(2113),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(105),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(409),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(411),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [106] = {
    [sym_end_function_statement] = STATE(1660),
    [sym_internal_procedures] = STATE(2117),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(106),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(401),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(403),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [107] = {
    [sym_end_subroutine_statement] = STATE(1629),
    [sym_internal_procedures] = STATE(2041),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(107),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(465),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(467),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [108] = {
    [sym_end_function_statement] = STATE(1626),
    [sym_internal_procedures] = STATE(2030),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(108),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(457),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(459),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [109] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(226),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2662),
    [sym_elseif_clause] = STATE(1897),
    [sym_else_clause] = STATE(1880),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(109),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(134),
    [aux_sym__block_if_statement_repeat1] = STATE(1784),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token1] = ACTIONS(832),
    [aux_sym_elseif_clause_token2] = ACTIONS(834),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [110] = {
    [sym_end_module_procedure_statement] = STATE(1672),
    [sym_internal_procedures] = STATE(2128),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(110),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(383),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(385),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [111] = {
    [sym_end_module_procedure_statement] = STATE(1677),
    [sym_internal_procedures] = STATE(2133),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(111),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(383),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(385),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [112] = {
    [sym_end_function_statement] = STATE(1699),
    [sym_internal_procedures] = STATE(2221),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(112),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(449),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(451),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [113] = {
    [sym_end_subroutine_statement] = STATE(1704),
    [sym_internal_procedures] = STATE(2217),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(113),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(441),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(443),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [114] = {
    [sym_end_subroutine_statement] = STATE(1624),
    [sym_internal_procedures] = STATE(2026),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(114),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(465),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(467),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [115] = {
    [sym_end_subroutine_statement] = STATE(1590),
    [sym_internal_procedures] = STATE(1954),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(115),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(445),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(447),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [116] = {
    [sym_end_function_statement] = STATE(1707),
    [sym_internal_procedures] = STATE(2212),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(116),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(449),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(451),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [117] = {
    [sym_end_module_procedure_statement] = STATE(1577),
    [sym_internal_procedures] = STATE(2115),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(117),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(413),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(415),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [118] = {
    [sym_end_subroutine_statement] = STATE(1193),
    [sym_internal_procedures] = STATE(2185),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(118),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(469),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(471),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [119] = {
    [sym_end_program_statement] = STATE(243),
    [sym_internal_procedures] = STATE(2162),
    [sym_contains_statement] = STATE(2802),
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(268),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(119),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(178),
    [aux_sym_end_program_statement_token1] = ACTIONS(11),
    [aux_sym_end_program_statement_token2] = ACTIONS(13),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [120] = {
    [sym_end_function_statement] = STATE(1186),
    [sym_internal_procedures] = STATE(2116),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(120),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(473),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(475),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [121] = {
    [sym_end_subroutine_statement] = STATE(1184),
    [sym_internal_procedures] = STATE(2067),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(121),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(469),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(471),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [122] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(224),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2564),
    [sym_elseif_clause] = STATE(1897),
    [sym_else_clause] = STATE(1872),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(122),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(135),
    [aux_sym__block_if_statement_repeat1] = STATE(1782),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token1] = ACTIONS(832),
    [aux_sym_elseif_clause_token2] = ACTIONS(834),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [123] = {
    [sym_end_subroutine_statement] = STATE(1608),
    [sym_internal_procedures] = STATE(1978),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(123),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(445),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(447),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [124] = {
    [sym_end_module_procedure_statement] = STATE(1615),
    [sym_internal_procedures] = STATE(1988),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(124),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(463),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [125] = {
    [sym_end_function_statement] = STATE(1604),
    [sym_internal_procedures] = STATE(1976),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(125),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(481),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(483),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [126] = {
    [sym_end_module_procedure_statement] = STATE(1612),
    [sym_internal_procedures] = STATE(1983),
    [sym_contains_statement] = STATE(2611),
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(126),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(461),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(463),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [127] = {
    [sym_end_subroutine_statement] = STATE(1710),
    [sym_internal_procedures] = STATE(2208),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(127),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(441),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(443),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [128] = {
    [sym_end_subroutine_statement] = STATE(271),
    [sym_internal_procedures] = STATE(1933),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(128),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(477),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(479),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [129] = {
    [sym_end_function_statement] = STATE(1685),
    [sym_internal_procedures] = STATE(2168),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(129),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(429),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(431),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [130] = {
    [sym_end_function_statement] = STATE(1602),
    [sym_internal_procedures] = STATE(1972),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(130),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(433),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(435),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [131] = {
    [sym_end_subroutine_statement] = STATE(1688),
    [sym_internal_procedures] = STATE(2177),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(131),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(421),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(423),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [132] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(226),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2662),
    [sym_elseif_clause] = STATE(1897),
    [sym_else_clause] = STATE(1880),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(132),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym__block_if_statement_repeat1] = STATE(1784),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token1] = ACTIONS(832),
    [aux_sym_elseif_clause_token2] = ACTIONS(834),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [133] = {
    [sym_end_subroutine_statement] = STATE(1599),
    [sym_internal_procedures] = STATE(1968),
    [sym_contains_statement] = STATE(2467),
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(133),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(425),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(427),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [134] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(225),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2634),
    [sym_elseif_clause] = STATE(1897),
    [sym_else_clause] = STATE(1896),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(134),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym__block_if_statement_repeat1] = STATE(1755),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token1] = ACTIONS(832),
    [aux_sym_elseif_clause_token2] = ACTIONS(834),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [135] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(228),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2513),
    [sym_elseif_clause] = STATE(1897),
    [sym_else_clause] = STATE(1869),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(135),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym__block_if_statement_repeat1] = STATE(1770),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token1] = ACTIONS(832),
    [aux_sym_elseif_clause_token2] = ACTIONS(834),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [136] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(228),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2513),
    [sym_elseif_clause] = STATE(1897),
    [sym_else_clause] = STATE(1869),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(136),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(132),
    [aux_sym__block_if_statement_repeat1] = STATE(1770),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token1] = ACTIONS(832),
    [aux_sym_elseif_clause_token2] = ACTIONS(834),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [137] = {
    [sym_end_function_statement] = STATE(1692),
    [sym_internal_procedures] = STATE(2181),
    [sym_contains_statement] = STATE(2539),
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(137),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(429),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_end_function_statement_token1] = ACTIONS(431),
    [aux_sym_contains_statement_token1] = ACTIONS(37),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [138] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(138),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(143),
    [aux_sym_end_program_statement_token1] = ACTIONS(836),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(839),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(839),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(839),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [139] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(139),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(841),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(844),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(844),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(844),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [140] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(140),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(139),
    [aux_sym_end_program_statement_token1] = ACTIONS(846),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(849),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(849),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(849),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [141] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(141),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(145),
    [aux_sym_end_program_statement_token1] = ACTIONS(841),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(844),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(844),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(844),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [142] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(142),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(851),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(854),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(854),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(854),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [143] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(143),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(856),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(859),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(859),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(859),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [144] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(144),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(150),
    [aux_sym_end_program_statement_token1] = ACTIONS(856),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(859),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(859),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(859),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [145] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(145),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(861),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(864),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(864),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(864),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [146] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(146),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(148),
    [aux_sym_end_program_statement_token1] = ACTIONS(861),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(864),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(864),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(864),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [147] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(147),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token1] = ACTIONS(710),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym_end_where_statement_token1] = ACTIONS(710),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(710),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [148] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(148),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(866),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(869),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(869),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(869),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [149] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(149),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(142),
    [aux_sym_end_program_statement_token1] = ACTIONS(871),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(874),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(874),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(874),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [150] = {
    [sym__statements] = STATE(2683),
    [sym_statement_label] = STATE(260),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(150),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(147),
    [aux_sym_end_program_statement_token1] = ACTIONS(871),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token1] = ACTIONS(874),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym_end_where_statement_token1] = ACTIONS(874),
    [aux_sym_elsewhere_clause_token1] = ACTIONS(874),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [151] = {
    [sym__statements] = STATE(2542),
    [sym_statement_label] = STATE(274),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(151),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(151),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [aux_sym_end_function_statement_token1] = ACTIONS(710),
    [aux_sym_contains_statement_token1] = ACTIONS(710),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [152] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(152),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(153),
    [aux_sym_end_program_statement_token1] = ACTIONS(876),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(879),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(879),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(881),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [153] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(153),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(160),
    [aux_sym_end_program_statement_token1] = ACTIONS(884),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(887),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(887),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(889),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [154] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(154),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(174),
    [aux_sym_end_program_statement_token1] = ACTIONS(884),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(887),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(887),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(889),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [155] = {
    [sym__statements] = STATE(2763),
    [sym_statement_label] = STATE(259),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(155),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(155),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(710),
    [aux_sym_contains_statement_token1] = ACTIONS(710),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [156] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_end_associate_statement] = STATE(2414),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(156),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(181),
    [aux_sym_end_program_statement_token1] = ACTIONS(892),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_end_associate_statement_token1] = ACTIONS(894),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [157] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(157),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(159),
    [aux_sym_end_program_statement_token1] = ACTIONS(896),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(899),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(899),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(901),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [158] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(158),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(193),
    [aux_sym_end_program_statement_token1] = ACTIONS(896),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(899),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(899),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(901),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [159] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(159),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(159),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token2] = ACTIONS(710),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_end_select_statement_token1] = ACTIONS(710),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [160] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(160),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(160),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token1] = ACTIONS(710),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_end_select_statement_token1] = ACTIONS(710),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [161] = {
    [sym__statements] = STATE(2470),
    [sym_statement_label] = STATE(277),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(161),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(161),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(710),
    [aux_sym_contains_statement_token1] = ACTIONS(710),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [162] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(219),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2617),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(162),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(194),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [163] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(219),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2617),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(163),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(187),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [164] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(214),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2551),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(164),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(172),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [165] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_end_associate_statement] = STATE(2680),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(165),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(200),
    [aux_sym_end_program_statement_token1] = ACTIONS(892),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_end_associate_statement_token1] = ACTIONS(894),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [166] = {
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(281),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2505),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(166),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(206),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [167] = {
    [sym__statements] = STATE(2614),
    [sym_statement_label] = STATE(220),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2391),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(167),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(205),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(914),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [168] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(214),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2551),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(168),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(194),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [169] = {
    [sym__statements] = STATE(2614),
    [sym_statement_label] = STATE(216),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2912),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(169),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(167),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(914),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [170] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(170),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(186),
    [aux_sym_end_program_statement_token1] = ACTIONS(916),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(919),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(919),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(921),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [171] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(171),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(160),
    [aux_sym_end_program_statement_token1] = ACTIONS(924),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(927),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(927),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(929),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [172] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(209),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2616),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(172),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(194),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [173] = {
    [sym__statements] = STATE(2614),
    [sym_statement_label] = STATE(218),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2704),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(173),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(205),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(914),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [174] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(174),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(160),
    [aux_sym_end_program_statement_token1] = ACTIONS(932),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(935),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(935),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(937),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [175] = {
    [sym__statements] = STATE(2614),
    [sym_statement_label] = STATE(220),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2391),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(175),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(173),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(914),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [176] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(176),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(159),
    [aux_sym_end_program_statement_token1] = ACTIONS(940),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(943),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(943),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(945),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [177] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(177),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(157),
    [aux_sym_end_program_statement_token1] = ACTIONS(948),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(951),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(951),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(953),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [178] = {
    [sym__statements] = STATE(2792),
    [sym_statement_label] = STATE(268),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(178),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(178),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [aux_sym_end_program_statement_token2] = ACTIONS(710),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [aux_sym_contains_statement_token1] = ACTIONS(710),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [179] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(179),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(184),
    [aux_sym_end_program_statement_token1] = ACTIONS(956),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(959),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(959),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(961),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [180] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_end_associate_statement] = STATE(2580),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(180),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(165),
    [aux_sym_end_program_statement_token1] = ACTIONS(892),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_end_associate_statement_token1] = ACTIONS(894),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [181] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_end_associate_statement] = STATE(2868),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(181),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(200),
    [aux_sym_end_program_statement_token1] = ACTIONS(892),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_end_associate_statement_token1] = ACTIONS(894),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [182] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_end_associate_statement] = STATE(2580),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(182),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(200),
    [aux_sym_end_program_statement_token1] = ACTIONS(892),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_end_associate_statement_token1] = ACTIONS(894),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [183] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(183),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(171),
    [aux_sym_end_program_statement_token1] = ACTIONS(964),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(967),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(967),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(969),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [184] = {
    [sym__statements] = STATE(2773),
    [sym_statement_label] = STATE(252),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(184),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(160),
    [aux_sym_end_program_statement_token1] = ACTIONS(964),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token1] = ACTIONS(967),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(967),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(969),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [185] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(185),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(176),
    [aux_sym_end_program_statement_token1] = ACTIONS(972),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(975),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(975),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(977),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [186] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(186),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(159),
    [aux_sym_end_program_statement_token1] = ACTIONS(972),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(975),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(975),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(977),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [187] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(210),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2476),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(187),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(194),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [188] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(215),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2666),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(188),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(162),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [189] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_end_associate_statement] = STATE(2868),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(189),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(182),
    [aux_sym_end_program_statement_token1] = ACTIONS(892),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_end_associate_statement_token1] = ACTIONS(894),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [190] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(210),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2476),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(190),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(168),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(908),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [191] = {
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(281),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2707),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(191),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(206),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [192] = {
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(281),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_end_block_construct_statement] = STATE(2602),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(192),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(206),
    [aux_sym_end_program_statement_token1] = ACTIONS(485),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(487),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(648),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [193] = {
    [sym__statements] = STATE(2771),
    [sym_statement_label] = STATE(253),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(193),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(159),
    [aux_sym_end_program_statement_token1] = ACTIONS(980),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token2] = ACTIONS(983),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_end_select_statement_token1] = ACTIONS(983),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(985),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [194] = {
    [sym__statements] = STATE(2534),
    [sym_statement_label] = STATE(287),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(194),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(194),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(710),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [195] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(195),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(203),
    [aux_sym_end_program_statement_token1] = ACTIONS(988),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(991),
    [aux_sym_elseif_clause_token1] = ACTIONS(994),
    [aux_sym_elseif_clause_token2] = ACTIONS(991),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(996),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [196] = {
    [sym__statements] = STATE(2684),
    [sym_statement_label] = STATE(1497),
    [sym_statement_label_reference] = STATE(2685),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(196),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym__block_if_statement_token1] = ACTIONS(1001),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1003),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [197] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(197),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym_end_program_statement_token1] = ACTIONS(1005),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1008),
    [aux_sym_elseif_clause_token1] = ACTIONS(1011),
    [aux_sym_elseif_clause_token2] = ACTIONS(1008),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1013),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [198] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(198),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym_end_program_statement_token1] = ACTIONS(1016),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1019),
    [aux_sym_elseif_clause_token1] = ACTIONS(1022),
    [aux_sym_elseif_clause_token2] = ACTIONS(1019),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1024),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [199] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(199),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(207),
    [aux_sym_end_program_statement_token1] = ACTIONS(1016),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1019),
    [aux_sym_elseif_clause_token1] = ACTIONS(1022),
    [aux_sym_elseif_clause_token2] = ACTIONS(1019),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1024),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [200] = {
    [sym__statements] = STATE(2767),
    [sym_statement_label] = STATE(262),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(200),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(200),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_end_associate_statement_token1] = ACTIONS(710),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [201] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(201),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token1] = ACTIONS(710),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [202] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(202),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(197),
    [aux_sym_end_program_statement_token1] = ACTIONS(1027),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1030),
    [aux_sym_elseif_clause_token1] = ACTIONS(1033),
    [aux_sym_elseif_clause_token2] = ACTIONS(1030),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1035),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [203] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(203),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym_end_program_statement_token1] = ACTIONS(1027),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1030),
    [aux_sym_elseif_clause_token1] = ACTIONS(1033),
    [aux_sym_elseif_clause_token2] = ACTIONS(1030),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1035),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [204] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(204),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(198),
    [aux_sym_end_program_statement_token1] = ACTIONS(1038),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1041),
    [aux_sym_elseif_clause_token1] = ACTIONS(1044),
    [aux_sym_elseif_clause_token2] = ACTIONS(1041),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1046),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [205] = {
    [sym__statements] = STATE(2614),
    [sym_statement_label] = STATE(272),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(205),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(205),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_end_forall_statement_token1] = ACTIONS(710),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [206] = {
    [sym__statements] = STATE(2390),
    [sym_statement_label] = STATE(281),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(206),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(206),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(710),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [207] = {
    [sym__statements] = STATE(2736),
    [sym_statement_label] = STATE(255),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(207),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(201),
    [aux_sym_end_program_statement_token1] = ACTIONS(1049),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1052),
    [aux_sym_elseif_clause_token1] = ACTIONS(1055),
    [aux_sym_elseif_clause_token2] = ACTIONS(1052),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1057),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [208] = {
    [sym__statements] = STATE(2769),
    [sym_statement_label] = STATE(256),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(208),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(211),
    [aux_sym_end_program_statement_token1] = ACTIONS(1060),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1063),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1066),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [209] = {
    [sym__statements] = STATE(2503),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2572),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(209),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [210] = {
    [sym__statements] = STATE(2503),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2551),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(210),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [211] = {
    [sym__statements] = STATE(2769),
    [sym_statement_label] = STATE(256),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(211),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(212),
    [aux_sym_end_program_statement_token1] = ACTIONS(1069),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1072),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1075),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [212] = {
    [sym__statements] = STATE(2769),
    [sym_statement_label] = STATE(256),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(212),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(212),
    [aux_sym_end_program_statement_token1] = ACTIONS(698),
    [anon_sym_LPAREN] = ACTIONS(701),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(704),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(707),
    [anon_sym_DASH] = ACTIONS(712),
    [aux_sym_intrinsic_type_token1] = ACTIONS(715),
    [aux_sym_intrinsic_type_token3] = ACTIONS(715),
    [aux_sym_intrinsic_type_token4] = ACTIONS(718),
    [aux_sym_type_qualifier_token13] = ACTIONS(715),
    [aux_sym_type_qualifier_token14] = ACTIONS(715),
    [aux_sym_stop_statement_token1] = ACTIONS(721),
    [aux_sym_stop_statement_token2] = ACTIONS(724),
    [aux_sym_subroutine_call_token1] = ACTIONS(727),
    [aux_sym_keyword_statement_token1] = ACTIONS(730),
    [aux_sym_keyword_statement_token2] = ACTIONS(733),
    [aux_sym_keyword_statement_token3] = ACTIONS(733),
    [aux_sym_keyword_statement_token4] = ACTIONS(736),
    [aux_sym_keyword_statement_token6] = ACTIONS(739),
    [aux_sym_keyword_statement_token7] = ACTIONS(730),
    [aux_sym_include_statement_token1] = ACTIONS(742),
    [aux_sym_data_statement_token1] = ACTIONS(715),
    [aux_sym_do_loop_statement_token1] = ACTIONS(745),
    [aux_sym__inline_if_statement_token1] = ACTIONS(748),
    [aux_sym_end_if_statement_token1] = ACTIONS(715),
    [aux_sym_elseif_clause_token2] = ACTIONS(715),
    [aux_sym__inline_where_statement_token1] = ACTIONS(751),
    [aux_sym__forall_control_expression_token1] = ACTIONS(754),
    [aux_sym_select_case_statement_token1] = ACTIONS(757),
    [aux_sym_select_case_statement_token3] = ACTIONS(760),
    [aux_sym_select_type_statement_token1] = ACTIONS(763),
    [aux_sym_select_rank_statement_token2] = ACTIONS(766),
    [aux_sym_block_construct_token1] = ACTIONS(769),
    [aux_sym_associate_statement_token1] = ACTIONS(772),
    [aux_sym_format_statement_token1] = ACTIONS(775),
    [aux_sym_print_statement_token1] = ACTIONS(778),
    [aux_sym_open_statement_token1] = ACTIONS(781),
    [aux_sym_close_statement_token1] = ACTIONS(784),
    [aux_sym_inquire_statement_token1] = ACTIONS(787),
    [aux_sym_file_position_statement_token1] = ACTIONS(790),
    [aux_sym_file_position_statement_token2] = ACTIONS(790),
    [aux_sym_file_position_statement_token3] = ACTIONS(790),
    [aux_sym_file_position_statement_token4] = ACTIONS(793),
    [aux_sym_logical_expression_token5] = ACTIONS(796),
    [anon_sym_PLUS] = ACTIONS(712),
    [anon_sym_DOT] = ACTIONS(799),
    [anon_sym_LPAREN_SLASH] = ACTIONS(802),
    [anon_sym_LBRACK] = ACTIONS(805),
    [sym_boolean_literal] = ACTIONS(808),
    [aux_sym_null_literal_token1] = ACTIONS(811),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(715),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(814),
    [sym__float_literal] = ACTIONS(817),
    [sym__boz_literal] = ACTIONS(817),
    [sym_string_literal] = ACTIONS(808),
  },
  [213] = {
    [sym__statements] = STATE(2769),
    [sym_statement_label] = STATE(256),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(213),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(217),
    [aux_sym_end_program_statement_token1] = ACTIONS(1069),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1072),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1075),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [214] = {
    [sym__statements] = STATE(2503),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2616),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(214),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [215] = {
    [sym__statements] = STATE(2503),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2617),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(215),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [216] = {
    [sym__statements] = STATE(2626),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2391),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(216),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [217] = {
    [sym__statements] = STATE(2769),
    [sym_statement_label] = STATE(256),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(217),
    [sym_identifier] = STATE(1148),
    [aux_sym_program_repeat2] = STATE(212),
    [aux_sym_end_program_statement_token1] = ACTIONS(1078),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(1081),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1084),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [218] = {
    [sym__statements] = STATE(2626),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2508),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(218),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [219] = {
    [sym__statements] = STATE(2503),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_end_do_loop_statement] = STATE(2476),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(219),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(904),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(906),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [220] = {
    [sym__statements] = STATE(2626),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_end_forall_statement] = STATE(2704),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(220),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(910),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_end_forall_statement_token1] = ACTIONS(912),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [221] = {
    [sym__statements] = STATE(2555),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2484),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(221),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(1087),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [222] = {
    [sym__statements] = STATE(2384),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(222),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
    [sym__end_of_statement] = ACTIONS(1089),
  },
  [223] = {
    [sym__statements] = STATE(2399),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2501),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(223),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(1087),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [224] = {
    [sym__statements] = STATE(2741),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2513),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(224),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [225] = {
    [sym__statements] = STATE(2741),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2416),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(225),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [226] = {
    [sym__statements] = STATE(2741),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2634),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(226),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [227] = {
    [sym__statements] = STATE(2686),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(227),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
    [sym__end_of_statement] = ACTIONS(1091),
  },
  [228] = {
    [sym__statements] = STATE(2741),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_end_if_statement] = STATE(2662),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(228),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(830),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [229] = {
    [sym__statements] = STATE(2386),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2385),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(229),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(1087),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [230] = {
    [sym__statements] = STATE(2483),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2400),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(230),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(1087),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [231] = {
    [sym__statements] = STATE(2764),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2556),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(231),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [aux_sym_implicit_statement_token3] = ACTIONS(1087),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [232] = {
    [sym_preproc_file_line] = STATE(232),
    [ts_builtin_sym_end] = ACTIONS(1093),
    [aux_sym_program_statement_token1] = ACTIONS(1095),
    [aux_sym_end_program_statement_token1] = ACTIONS(1095),
    [aux_sym_end_program_statement_token2] = ACTIONS(1095),
    [aux_sym_module_statement_token1] = ACTIONS(1095),
    [aux_sym_submodule_statement_token1] = ACTIONS(1095),
    [anon_sym_LPAREN] = ACTIONS(1095),
    [aux_sym_interface_statement_token1] = ACTIONS(1095),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1095),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1095),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1095),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1095),
    [aux_sym_function_statement_token1] = ACTIONS(1095),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1095),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1095),
    [aux_sym_contains_statement_token1] = ACTIONS(1095),
    [aux_sym_use_statement_token1] = ACTIONS(1095),
    [aux_sym_use_statement_token2] = ACTIONS(1095),
    [aux_sym_implicit_statement_token1] = ACTIONS(1095),
    [aux_sym_implicit_statement_token3] = ACTIONS(1095),
    [aux_sym_implicit_statement_token4] = ACTIONS(1095),
    [aux_sym_save_statement_token1] = ACTIONS(1095),
    [aux_sym_private_statement_token1] = ACTIONS(1095),
    [aux_sym_public_statement_token1] = ACTIONS(1095),
    [aux_sym_namelist_statement_token1] = ACTIONS(1095),
    [aux_sym_common_statement_token1] = ACTIONS(1095),
    [anon_sym_DASH] = ACTIONS(1093),
    [aux_sym_import_statement_token1] = ACTIONS(1095),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1095),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1095),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1095),
    [aux_sym_variable_attributes_token1] = ACTIONS(1095),
    [aux_sym_variable_attributes_token2] = ACTIONS(1095),
    [aux_sym_variable_attributes_token3] = ACTIONS(1095),
    [aux_sym_variable_attributes_token4] = ACTIONS(1095),
    [aux_sym_variable_attributes_token5] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1095),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1095),
    [aux_sym_derived_type_token1] = ACTIONS(1095),
    [aux_sym_type_qualifier_token1] = ACTIONS(1095),
    [aux_sym_type_qualifier_token2] = ACTIONS(1095),
    [aux_sym_type_qualifier_token3] = ACTIONS(1095),
    [aux_sym_type_qualifier_token4] = ACTIONS(1095),
    [aux_sym_type_qualifier_token5] = ACTIONS(1095),
    [aux_sym_type_qualifier_token9] = ACTIONS(1095),
    [aux_sym_type_qualifier_token10] = ACTIONS(1095),
    [aux_sym_type_qualifier_token11] = ACTIONS(1095),
    [aux_sym_type_qualifier_token12] = ACTIONS(1095),
    [aux_sym_type_qualifier_token13] = ACTIONS(1095),
    [aux_sym_type_qualifier_token14] = ACTIONS(1095),
    [aux_sym_type_qualifier_token15] = ACTIONS(1095),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1095),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1095),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1095),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1095),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1095),
    [aux_sym_stop_statement_token1] = ACTIONS(1095),
    [aux_sym_stop_statement_token2] = ACTIONS(1095),
    [aux_sym_subroutine_call_token1] = ACTIONS(1095),
    [aux_sym_keyword_statement_token1] = ACTIONS(1095),
    [aux_sym_keyword_statement_token2] = ACTIONS(1095),
    [aux_sym_keyword_statement_token3] = ACTIONS(1095),
    [aux_sym_keyword_statement_token4] = ACTIONS(1095),
    [aux_sym_keyword_statement_token6] = ACTIONS(1095),
    [aux_sym_keyword_statement_token7] = ACTIONS(1095),
    [aux_sym_include_statement_token1] = ACTIONS(1095),
    [aux_sym_data_statement_token1] = ACTIONS(1095),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1095),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1095),
    [aux_sym_end_if_statement_token1] = ACTIONS(1095),
    [aux_sym_elseif_clause_token2] = ACTIONS(1095),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1095),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1095),
    [aux_sym_select_case_statement_token1] = ACTIONS(1095),
    [aux_sym_select_case_statement_token3] = ACTIONS(1095),
    [aux_sym_select_type_statement_token1] = ACTIONS(1095),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1095),
    [aux_sym_block_construct_token1] = ACTIONS(1095),
    [aux_sym_associate_statement_token1] = ACTIONS(1095),
    [aux_sym_format_statement_token1] = ACTIONS(1095),
    [aux_sym_print_statement_token1] = ACTIONS(1095),
    [aux_sym_open_statement_token1] = ACTIONS(1095),
    [aux_sym_close_statement_token1] = ACTIONS(1095),
    [aux_sym_inquire_statement_token1] = ACTIONS(1095),
    [aux_sym_enum_statement_token1] = ACTIONS(1095),
    [aux_sym_file_position_statement_token1] = ACTIONS(1095),
    [aux_sym_file_position_statement_token2] = ACTIONS(1095),
    [aux_sym_file_position_statement_token3] = ACTIONS(1095),
    [aux_sym_file_position_statement_token4] = ACTIONS(1095),
    [aux_sym_logical_expression_token5] = ACTIONS(1093),
    [anon_sym_PLUS] = ACTIONS(1093),
    [anon_sym_DOT] = ACTIONS(1095),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1093),
    [anon_sym_LBRACK] = ACTIONS(1093),
    [sym_boolean_literal] = ACTIONS(1093),
    [aux_sym_null_literal_token1] = ACTIONS(1095),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1095),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1093),
    [sym__float_literal] = ACTIONS(1093),
    [sym__boz_literal] = ACTIONS(1093),
    [sym_string_literal] = ACTIONS(1093),
  },
  [233] = {
    [sym_preproc_file_line] = STATE(233),
    [ts_builtin_sym_end] = ACTIONS(1097),
    [aux_sym_program_statement_token1] = ACTIONS(1099),
    [aux_sym_end_program_statement_token1] = ACTIONS(1099),
    [aux_sym_end_program_statement_token2] = ACTIONS(1099),
    [aux_sym_module_statement_token1] = ACTIONS(1099),
    [aux_sym_submodule_statement_token1] = ACTIONS(1099),
    [anon_sym_LPAREN] = ACTIONS(1099),
    [aux_sym_interface_statement_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1099),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1099),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_function_statement_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1099),
    [aux_sym_contains_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token2] = ACTIONS(1099),
    [aux_sym_implicit_statement_token1] = ACTIONS(1099),
    [aux_sym_implicit_statement_token3] = ACTIONS(1099),
    [aux_sym_implicit_statement_token4] = ACTIONS(1099),
    [aux_sym_save_statement_token1] = ACTIONS(1099),
    [aux_sym_private_statement_token1] = ACTIONS(1099),
    [aux_sym_public_statement_token1] = ACTIONS(1099),
    [aux_sym_namelist_statement_token1] = ACTIONS(1099),
    [aux_sym_common_statement_token1] = ACTIONS(1099),
    [anon_sym_DASH] = ACTIONS(1097),
    [aux_sym_import_statement_token1] = ACTIONS(1099),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1099),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1099),
    [aux_sym_variable_attributes_token1] = ACTIONS(1099),
    [aux_sym_variable_attributes_token2] = ACTIONS(1099),
    [aux_sym_variable_attributes_token3] = ACTIONS(1099),
    [aux_sym_variable_attributes_token4] = ACTIONS(1099),
    [aux_sym_variable_attributes_token5] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1099),
    [aux_sym_derived_type_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token2] = ACTIONS(1099),
    [aux_sym_type_qualifier_token3] = ACTIONS(1099),
    [aux_sym_type_qualifier_token4] = ACTIONS(1099),
    [aux_sym_type_qualifier_token5] = ACTIONS(1099),
    [aux_sym_type_qualifier_token9] = ACTIONS(1099),
    [aux_sym_type_qualifier_token10] = ACTIONS(1099),
    [aux_sym_type_qualifier_token11] = ACTIONS(1099),
    [aux_sym_type_qualifier_token12] = ACTIONS(1099),
    [aux_sym_type_qualifier_token13] = ACTIONS(1099),
    [aux_sym_type_qualifier_token14] = ACTIONS(1099),
    [aux_sym_type_qualifier_token15] = ACTIONS(1099),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1099),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1099),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1099),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token2] = ACTIONS(1099),
    [aux_sym_subroutine_call_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token2] = ACTIONS(1099),
    [aux_sym_keyword_statement_token3] = ACTIONS(1099),
    [aux_sym_keyword_statement_token4] = ACTIONS(1099),
    [aux_sym_keyword_statement_token6] = ACTIONS(1099),
    [aux_sym_keyword_statement_token7] = ACTIONS(1099),
    [aux_sym_include_statement_token1] = ACTIONS(1099),
    [aux_sym_data_statement_token1] = ACTIONS(1099),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1099),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1099),
    [aux_sym_end_if_statement_token1] = ACTIONS(1099),
    [aux_sym_elseif_clause_token2] = ACTIONS(1099),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1099),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token3] = ACTIONS(1099),
    [aux_sym_select_type_statement_token1] = ACTIONS(1099),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1099),
    [aux_sym_block_construct_token1] = ACTIONS(1099),
    [aux_sym_associate_statement_token1] = ACTIONS(1099),
    [aux_sym_format_statement_token1] = ACTIONS(1099),
    [aux_sym_print_statement_token1] = ACTIONS(1099),
    [aux_sym_open_statement_token1] = ACTIONS(1099),
    [aux_sym_close_statement_token1] = ACTIONS(1099),
    [aux_sym_inquire_statement_token1] = ACTIONS(1099),
    [aux_sym_enum_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token2] = ACTIONS(1099),
    [aux_sym_file_position_statement_token3] = ACTIONS(1099),
    [aux_sym_file_position_statement_token4] = ACTIONS(1099),
    [aux_sym_logical_expression_token5] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [sym_boolean_literal] = ACTIONS(1097),
    [aux_sym_null_literal_token1] = ACTIONS(1099),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1099),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1097),
    [sym__float_literal] = ACTIONS(1097),
    [sym__boz_literal] = ACTIONS(1097),
    [sym_string_literal] = ACTIONS(1097),
  },
  [234] = {
    [sym_preproc_file_line] = STATE(234),
    [ts_builtin_sym_end] = ACTIONS(1101),
    [aux_sym_program_statement_token1] = ACTIONS(1103),
    [aux_sym_end_program_statement_token1] = ACTIONS(1103),
    [aux_sym_end_program_statement_token2] = ACTIONS(1103),
    [aux_sym_module_statement_token1] = ACTIONS(1103),
    [aux_sym_submodule_statement_token1] = ACTIONS(1103),
    [anon_sym_LPAREN] = ACTIONS(1103),
    [aux_sym_interface_statement_token1] = ACTIONS(1103),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1103),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1103),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1103),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1103),
    [aux_sym_function_statement_token1] = ACTIONS(1103),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1103),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1103),
    [aux_sym_contains_statement_token1] = ACTIONS(1103),
    [aux_sym_use_statement_token1] = ACTIONS(1103),
    [aux_sym_use_statement_token2] = ACTIONS(1103),
    [aux_sym_implicit_statement_token1] = ACTIONS(1103),
    [aux_sym_implicit_statement_token3] = ACTIONS(1103),
    [aux_sym_implicit_statement_token4] = ACTIONS(1103),
    [aux_sym_save_statement_token1] = ACTIONS(1103),
    [aux_sym_private_statement_token1] = ACTIONS(1103),
    [aux_sym_public_statement_token1] = ACTIONS(1103),
    [aux_sym_namelist_statement_token1] = ACTIONS(1103),
    [aux_sym_common_statement_token1] = ACTIONS(1103),
    [anon_sym_DASH] = ACTIONS(1101),
    [aux_sym_import_statement_token1] = ACTIONS(1103),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1103),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1103),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1103),
    [aux_sym_variable_attributes_token1] = ACTIONS(1103),
    [aux_sym_variable_attributes_token2] = ACTIONS(1103),
    [aux_sym_variable_attributes_token3] = ACTIONS(1103),
    [aux_sym_variable_attributes_token4] = ACTIONS(1103),
    [aux_sym_variable_attributes_token5] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1103),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1103),
    [aux_sym_derived_type_token1] = ACTIONS(1103),
    [aux_sym_type_qualifier_token1] = ACTIONS(1103),
    [aux_sym_type_qualifier_token2] = ACTIONS(1103),
    [aux_sym_type_qualifier_token3] = ACTIONS(1103),
    [aux_sym_type_qualifier_token4] = ACTIONS(1103),
    [aux_sym_type_qualifier_token5] = ACTIONS(1103),
    [aux_sym_type_qualifier_token9] = ACTIONS(1103),
    [aux_sym_type_qualifier_token10] = ACTIONS(1103),
    [aux_sym_type_qualifier_token11] = ACTIONS(1103),
    [aux_sym_type_qualifier_token12] = ACTIONS(1103),
    [aux_sym_type_qualifier_token13] = ACTIONS(1103),
    [aux_sym_type_qualifier_token14] = ACTIONS(1103),
    [aux_sym_type_qualifier_token15] = ACTIONS(1103),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1103),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1103),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1103),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1103),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1103),
    [aux_sym_stop_statement_token1] = ACTIONS(1103),
    [aux_sym_stop_statement_token2] = ACTIONS(1103),
    [aux_sym_subroutine_call_token1] = ACTIONS(1103),
    [aux_sym_keyword_statement_token1] = ACTIONS(1103),
    [aux_sym_keyword_statement_token2] = ACTIONS(1103),
    [aux_sym_keyword_statement_token3] = ACTIONS(1103),
    [aux_sym_keyword_statement_token4] = ACTIONS(1103),
    [aux_sym_keyword_statement_token6] = ACTIONS(1103),
    [aux_sym_keyword_statement_token7] = ACTIONS(1103),
    [aux_sym_include_statement_token1] = ACTIONS(1103),
    [aux_sym_data_statement_token1] = ACTIONS(1103),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1103),
    [aux_sym_end_if_statement_token1] = ACTIONS(1103),
    [aux_sym_elseif_clause_token2] = ACTIONS(1103),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1103),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1103),
    [aux_sym_select_case_statement_token1] = ACTIONS(1103),
    [aux_sym_select_case_statement_token3] = ACTIONS(1103),
    [aux_sym_select_type_statement_token1] = ACTIONS(1103),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1103),
    [aux_sym_block_construct_token1] = ACTIONS(1103),
    [aux_sym_associate_statement_token1] = ACTIONS(1103),
    [aux_sym_format_statement_token1] = ACTIONS(1103),
    [aux_sym_print_statement_token1] = ACTIONS(1103),
    [aux_sym_open_statement_token1] = ACTIONS(1103),
    [aux_sym_close_statement_token1] = ACTIONS(1103),
    [aux_sym_inquire_statement_token1] = ACTIONS(1103),
    [aux_sym_enum_statement_token1] = ACTIONS(1103),
    [aux_sym_file_position_statement_token1] = ACTIONS(1103),
    [aux_sym_file_position_statement_token2] = ACTIONS(1103),
    [aux_sym_file_position_statement_token3] = ACTIONS(1103),
    [aux_sym_file_position_statement_token4] = ACTIONS(1103),
    [aux_sym_logical_expression_token5] = ACTIONS(1101),
    [anon_sym_PLUS] = ACTIONS(1101),
    [anon_sym_DOT] = ACTIONS(1103),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1101),
    [anon_sym_LBRACK] = ACTIONS(1101),
    [sym_boolean_literal] = ACTIONS(1101),
    [aux_sym_null_literal_token1] = ACTIONS(1103),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1103),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1101),
    [sym__float_literal] = ACTIONS(1101),
    [sym__boz_literal] = ACTIONS(1101),
    [sym_string_literal] = ACTIONS(1101),
  },
  [235] = {
    [sym_preproc_file_line] = STATE(235),
    [ts_builtin_sym_end] = ACTIONS(1105),
    [aux_sym_program_statement_token1] = ACTIONS(1107),
    [aux_sym_end_program_statement_token1] = ACTIONS(1107),
    [aux_sym_end_program_statement_token2] = ACTIONS(1107),
    [aux_sym_module_statement_token1] = ACTIONS(1107),
    [aux_sym_submodule_statement_token1] = ACTIONS(1107),
    [anon_sym_LPAREN] = ACTIONS(1107),
    [aux_sym_interface_statement_token1] = ACTIONS(1107),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1107),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1107),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1107),
    [aux_sym_function_statement_token1] = ACTIONS(1107),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1107),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1107),
    [aux_sym_contains_statement_token1] = ACTIONS(1107),
    [aux_sym_use_statement_token1] = ACTIONS(1107),
    [aux_sym_use_statement_token2] = ACTIONS(1107),
    [aux_sym_implicit_statement_token1] = ACTIONS(1107),
    [aux_sym_implicit_statement_token3] = ACTIONS(1107),
    [aux_sym_implicit_statement_token4] = ACTIONS(1107),
    [aux_sym_save_statement_token1] = ACTIONS(1107),
    [aux_sym_private_statement_token1] = ACTIONS(1107),
    [aux_sym_public_statement_token1] = ACTIONS(1107),
    [aux_sym_namelist_statement_token1] = ACTIONS(1107),
    [aux_sym_common_statement_token1] = ACTIONS(1107),
    [anon_sym_DASH] = ACTIONS(1105),
    [aux_sym_import_statement_token1] = ACTIONS(1107),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1107),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1107),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1107),
    [aux_sym_variable_attributes_token1] = ACTIONS(1107),
    [aux_sym_variable_attributes_token2] = ACTIONS(1107),
    [aux_sym_variable_attributes_token3] = ACTIONS(1107),
    [aux_sym_variable_attributes_token4] = ACTIONS(1107),
    [aux_sym_variable_attributes_token5] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1107),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1107),
    [aux_sym_derived_type_token1] = ACTIONS(1107),
    [aux_sym_type_qualifier_token1] = ACTIONS(1107),
    [aux_sym_type_qualifier_token2] = ACTIONS(1107),
    [aux_sym_type_qualifier_token3] = ACTIONS(1107),
    [aux_sym_type_qualifier_token4] = ACTIONS(1107),
    [aux_sym_type_qualifier_token5] = ACTIONS(1107),
    [aux_sym_type_qualifier_token9] = ACTIONS(1107),
    [aux_sym_type_qualifier_token10] = ACTIONS(1107),
    [aux_sym_type_qualifier_token11] = ACTIONS(1107),
    [aux_sym_type_qualifier_token12] = ACTIONS(1107),
    [aux_sym_type_qualifier_token13] = ACTIONS(1107),
    [aux_sym_type_qualifier_token14] = ACTIONS(1107),
    [aux_sym_type_qualifier_token15] = ACTIONS(1107),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1107),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1107),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1107),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1107),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1107),
    [aux_sym_stop_statement_token1] = ACTIONS(1107),
    [aux_sym_stop_statement_token2] = ACTIONS(1107),
    [aux_sym_subroutine_call_token1] = ACTIONS(1107),
    [aux_sym_keyword_statement_token1] = ACTIONS(1107),
    [aux_sym_keyword_statement_token2] = ACTIONS(1107),
    [aux_sym_keyword_statement_token3] = ACTIONS(1107),
    [aux_sym_keyword_statement_token4] = ACTIONS(1107),
    [aux_sym_keyword_statement_token6] = ACTIONS(1107),
    [aux_sym_keyword_statement_token7] = ACTIONS(1107),
    [aux_sym_include_statement_token1] = ACTIONS(1107),
    [aux_sym_data_statement_token1] = ACTIONS(1107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1107),
    [aux_sym_end_if_statement_token1] = ACTIONS(1107),
    [aux_sym_elseif_clause_token2] = ACTIONS(1107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1107),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1107),
    [aux_sym_select_case_statement_token3] = ACTIONS(1107),
    [aux_sym_select_type_statement_token1] = ACTIONS(1107),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1107),
    [aux_sym_block_construct_token1] = ACTIONS(1107),
    [aux_sym_associate_statement_token1] = ACTIONS(1107),
    [aux_sym_format_statement_token1] = ACTIONS(1107),
    [aux_sym_print_statement_token1] = ACTIONS(1107),
    [aux_sym_open_statement_token1] = ACTIONS(1107),
    [aux_sym_close_statement_token1] = ACTIONS(1107),
    [aux_sym_inquire_statement_token1] = ACTIONS(1107),
    [aux_sym_enum_statement_token1] = ACTIONS(1107),
    [aux_sym_file_position_statement_token1] = ACTIONS(1107),
    [aux_sym_file_position_statement_token2] = ACTIONS(1107),
    [aux_sym_file_position_statement_token3] = ACTIONS(1107),
    [aux_sym_file_position_statement_token4] = ACTIONS(1107),
    [aux_sym_logical_expression_token5] = ACTIONS(1105),
    [anon_sym_PLUS] = ACTIONS(1105),
    [anon_sym_DOT] = ACTIONS(1107),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1105),
    [anon_sym_LBRACK] = ACTIONS(1105),
    [sym_boolean_literal] = ACTIONS(1105),
    [aux_sym_null_literal_token1] = ACTIONS(1107),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1105),
    [sym__float_literal] = ACTIONS(1105),
    [sym__boz_literal] = ACTIONS(1105),
    [sym_string_literal] = ACTIONS(1105),
  },
  [236] = {
    [sym_preproc_file_line] = STATE(236),
    [ts_builtin_sym_end] = ACTIONS(1109),
    [aux_sym_program_statement_token1] = ACTIONS(1111),
    [aux_sym_end_program_statement_token1] = ACTIONS(1111),
    [aux_sym_end_program_statement_token2] = ACTIONS(1111),
    [aux_sym_module_statement_token1] = ACTIONS(1111),
    [aux_sym_submodule_statement_token1] = ACTIONS(1111),
    [anon_sym_LPAREN] = ACTIONS(1111),
    [aux_sym_interface_statement_token1] = ACTIONS(1111),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1111),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1111),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1111),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1111),
    [aux_sym_function_statement_token1] = ACTIONS(1111),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1111),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1111),
    [aux_sym_contains_statement_token1] = ACTIONS(1111),
    [aux_sym_use_statement_token1] = ACTIONS(1111),
    [aux_sym_use_statement_token2] = ACTIONS(1111),
    [aux_sym_implicit_statement_token1] = ACTIONS(1111),
    [aux_sym_implicit_statement_token3] = ACTIONS(1111),
    [aux_sym_implicit_statement_token4] = ACTIONS(1111),
    [aux_sym_save_statement_token1] = ACTIONS(1111),
    [aux_sym_private_statement_token1] = ACTIONS(1111),
    [aux_sym_public_statement_token1] = ACTIONS(1111),
    [aux_sym_namelist_statement_token1] = ACTIONS(1111),
    [aux_sym_common_statement_token1] = ACTIONS(1111),
    [anon_sym_DASH] = ACTIONS(1109),
    [aux_sym_import_statement_token1] = ACTIONS(1111),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1111),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1111),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1111),
    [aux_sym_variable_attributes_token1] = ACTIONS(1111),
    [aux_sym_variable_attributes_token2] = ACTIONS(1111),
    [aux_sym_variable_attributes_token3] = ACTIONS(1111),
    [aux_sym_variable_attributes_token4] = ACTIONS(1111),
    [aux_sym_variable_attributes_token5] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1111),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1111),
    [aux_sym_derived_type_token1] = ACTIONS(1111),
    [aux_sym_type_qualifier_token1] = ACTIONS(1111),
    [aux_sym_type_qualifier_token2] = ACTIONS(1111),
    [aux_sym_type_qualifier_token3] = ACTIONS(1111),
    [aux_sym_type_qualifier_token4] = ACTIONS(1111),
    [aux_sym_type_qualifier_token5] = ACTIONS(1111),
    [aux_sym_type_qualifier_token9] = ACTIONS(1111),
    [aux_sym_type_qualifier_token10] = ACTIONS(1111),
    [aux_sym_type_qualifier_token11] = ACTIONS(1111),
    [aux_sym_type_qualifier_token12] = ACTIONS(1111),
    [aux_sym_type_qualifier_token13] = ACTIONS(1111),
    [aux_sym_type_qualifier_token14] = ACTIONS(1111),
    [aux_sym_type_qualifier_token15] = ACTIONS(1111),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1111),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1111),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1111),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1111),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1111),
    [aux_sym_stop_statement_token1] = ACTIONS(1111),
    [aux_sym_stop_statement_token2] = ACTIONS(1111),
    [aux_sym_subroutine_call_token1] = ACTIONS(1111),
    [aux_sym_keyword_statement_token1] = ACTIONS(1111),
    [aux_sym_keyword_statement_token2] = ACTIONS(1111),
    [aux_sym_keyword_statement_token3] = ACTIONS(1111),
    [aux_sym_keyword_statement_token4] = ACTIONS(1111),
    [aux_sym_keyword_statement_token6] = ACTIONS(1111),
    [aux_sym_keyword_statement_token7] = ACTIONS(1111),
    [aux_sym_include_statement_token1] = ACTIONS(1111),
    [aux_sym_data_statement_token1] = ACTIONS(1111),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1111),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1111),
    [aux_sym_end_if_statement_token1] = ACTIONS(1111),
    [aux_sym_elseif_clause_token2] = ACTIONS(1111),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1111),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1111),
    [aux_sym_select_case_statement_token1] = ACTIONS(1111),
    [aux_sym_select_case_statement_token3] = ACTIONS(1111),
    [aux_sym_select_type_statement_token1] = ACTIONS(1111),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1111),
    [aux_sym_block_construct_token1] = ACTIONS(1111),
    [aux_sym_associate_statement_token1] = ACTIONS(1111),
    [aux_sym_format_statement_token1] = ACTIONS(1111),
    [aux_sym_print_statement_token1] = ACTIONS(1111),
    [aux_sym_open_statement_token1] = ACTIONS(1111),
    [aux_sym_close_statement_token1] = ACTIONS(1111),
    [aux_sym_inquire_statement_token1] = ACTIONS(1111),
    [aux_sym_enum_statement_token1] = ACTIONS(1111),
    [aux_sym_file_position_statement_token1] = ACTIONS(1111),
    [aux_sym_file_position_statement_token2] = ACTIONS(1111),
    [aux_sym_file_position_statement_token3] = ACTIONS(1111),
    [aux_sym_file_position_statement_token4] = ACTIONS(1111),
    [aux_sym_logical_expression_token5] = ACTIONS(1109),
    [anon_sym_PLUS] = ACTIONS(1109),
    [anon_sym_DOT] = ACTIONS(1111),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1109),
    [anon_sym_LBRACK] = ACTIONS(1109),
    [sym_boolean_literal] = ACTIONS(1109),
    [aux_sym_null_literal_token1] = ACTIONS(1111),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1111),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1109),
    [sym__float_literal] = ACTIONS(1109),
    [sym__boz_literal] = ACTIONS(1109),
    [sym_string_literal] = ACTIONS(1109),
  },
  [237] = {
    [sym_preproc_file_line] = STATE(237),
    [ts_builtin_sym_end] = ACTIONS(1113),
    [aux_sym_program_statement_token1] = ACTIONS(1115),
    [aux_sym_end_program_statement_token1] = ACTIONS(1115),
    [aux_sym_end_program_statement_token2] = ACTIONS(1115),
    [aux_sym_module_statement_token1] = ACTIONS(1115),
    [aux_sym_submodule_statement_token1] = ACTIONS(1115),
    [anon_sym_LPAREN] = ACTIONS(1115),
    [aux_sym_interface_statement_token1] = ACTIONS(1115),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1115),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1115),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1115),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1115),
    [aux_sym_function_statement_token1] = ACTIONS(1115),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1115),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1115),
    [aux_sym_contains_statement_token1] = ACTIONS(1115),
    [aux_sym_use_statement_token1] = ACTIONS(1115),
    [aux_sym_use_statement_token2] = ACTIONS(1115),
    [aux_sym_implicit_statement_token1] = ACTIONS(1115),
    [aux_sym_implicit_statement_token3] = ACTIONS(1115),
    [aux_sym_implicit_statement_token4] = ACTIONS(1115),
    [aux_sym_save_statement_token1] = ACTIONS(1115),
    [aux_sym_private_statement_token1] = ACTIONS(1115),
    [aux_sym_public_statement_token1] = ACTIONS(1115),
    [aux_sym_namelist_statement_token1] = ACTIONS(1115),
    [aux_sym_common_statement_token1] = ACTIONS(1115),
    [anon_sym_DASH] = ACTIONS(1113),
    [aux_sym_import_statement_token1] = ACTIONS(1115),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1115),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1115),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1115),
    [aux_sym_variable_attributes_token1] = ACTIONS(1115),
    [aux_sym_variable_attributes_token2] = ACTIONS(1115),
    [aux_sym_variable_attributes_token3] = ACTIONS(1115),
    [aux_sym_variable_attributes_token4] = ACTIONS(1115),
    [aux_sym_variable_attributes_token5] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1115),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1115),
    [aux_sym_derived_type_token1] = ACTIONS(1115),
    [aux_sym_type_qualifier_token1] = ACTIONS(1115),
    [aux_sym_type_qualifier_token2] = ACTIONS(1115),
    [aux_sym_type_qualifier_token3] = ACTIONS(1115),
    [aux_sym_type_qualifier_token4] = ACTIONS(1115),
    [aux_sym_type_qualifier_token5] = ACTIONS(1115),
    [aux_sym_type_qualifier_token9] = ACTIONS(1115),
    [aux_sym_type_qualifier_token10] = ACTIONS(1115),
    [aux_sym_type_qualifier_token11] = ACTIONS(1115),
    [aux_sym_type_qualifier_token12] = ACTIONS(1115),
    [aux_sym_type_qualifier_token13] = ACTIONS(1115),
    [aux_sym_type_qualifier_token14] = ACTIONS(1115),
    [aux_sym_type_qualifier_token15] = ACTIONS(1115),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1115),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1115),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1115),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1115),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1115),
    [aux_sym_stop_statement_token1] = ACTIONS(1115),
    [aux_sym_stop_statement_token2] = ACTIONS(1115),
    [aux_sym_subroutine_call_token1] = ACTIONS(1115),
    [aux_sym_keyword_statement_token1] = ACTIONS(1115),
    [aux_sym_keyword_statement_token2] = ACTIONS(1115),
    [aux_sym_keyword_statement_token3] = ACTIONS(1115),
    [aux_sym_keyword_statement_token4] = ACTIONS(1115),
    [aux_sym_keyword_statement_token6] = ACTIONS(1115),
    [aux_sym_keyword_statement_token7] = ACTIONS(1115),
    [aux_sym_include_statement_token1] = ACTIONS(1115),
    [aux_sym_data_statement_token1] = ACTIONS(1115),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1115),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1115),
    [aux_sym_end_if_statement_token1] = ACTIONS(1115),
    [aux_sym_elseif_clause_token2] = ACTIONS(1115),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1115),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1115),
    [aux_sym_select_case_statement_token1] = ACTIONS(1115),
    [aux_sym_select_case_statement_token3] = ACTIONS(1115),
    [aux_sym_select_type_statement_token1] = ACTIONS(1115),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1115),
    [aux_sym_block_construct_token1] = ACTIONS(1115),
    [aux_sym_associate_statement_token1] = ACTIONS(1115),
    [aux_sym_format_statement_token1] = ACTIONS(1115),
    [aux_sym_print_statement_token1] = ACTIONS(1115),
    [aux_sym_open_statement_token1] = ACTIONS(1115),
    [aux_sym_close_statement_token1] = ACTIONS(1115),
    [aux_sym_inquire_statement_token1] = ACTIONS(1115),
    [aux_sym_enum_statement_token1] = ACTIONS(1115),
    [aux_sym_file_position_statement_token1] = ACTIONS(1115),
    [aux_sym_file_position_statement_token2] = ACTIONS(1115),
    [aux_sym_file_position_statement_token3] = ACTIONS(1115),
    [aux_sym_file_position_statement_token4] = ACTIONS(1115),
    [aux_sym_logical_expression_token5] = ACTIONS(1113),
    [anon_sym_PLUS] = ACTIONS(1113),
    [anon_sym_DOT] = ACTIONS(1115),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1113),
    [anon_sym_LBRACK] = ACTIONS(1113),
    [sym_boolean_literal] = ACTIONS(1113),
    [aux_sym_null_literal_token1] = ACTIONS(1115),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1115),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1113),
    [sym__float_literal] = ACTIONS(1113),
    [sym__boz_literal] = ACTIONS(1113),
    [sym_string_literal] = ACTIONS(1113),
  },
  [238] = {
    [sym_preproc_file_line] = STATE(238),
    [ts_builtin_sym_end] = ACTIONS(1117),
    [aux_sym_program_statement_token1] = ACTIONS(1119),
    [aux_sym_end_program_statement_token1] = ACTIONS(1119),
    [aux_sym_end_program_statement_token2] = ACTIONS(1119),
    [aux_sym_module_statement_token1] = ACTIONS(1119),
    [aux_sym_submodule_statement_token1] = ACTIONS(1119),
    [anon_sym_LPAREN] = ACTIONS(1119),
    [aux_sym_interface_statement_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1119),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1119),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_function_statement_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1119),
    [aux_sym_contains_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token2] = ACTIONS(1119),
    [aux_sym_implicit_statement_token1] = ACTIONS(1119),
    [aux_sym_implicit_statement_token3] = ACTIONS(1119),
    [aux_sym_implicit_statement_token4] = ACTIONS(1119),
    [aux_sym_save_statement_token1] = ACTIONS(1119),
    [aux_sym_private_statement_token1] = ACTIONS(1119),
    [aux_sym_public_statement_token1] = ACTIONS(1119),
    [aux_sym_namelist_statement_token1] = ACTIONS(1119),
    [aux_sym_common_statement_token1] = ACTIONS(1119),
    [anon_sym_DASH] = ACTIONS(1117),
    [aux_sym_import_statement_token1] = ACTIONS(1119),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1119),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1119),
    [aux_sym_variable_attributes_token1] = ACTIONS(1119),
    [aux_sym_variable_attributes_token2] = ACTIONS(1119),
    [aux_sym_variable_attributes_token3] = ACTIONS(1119),
    [aux_sym_variable_attributes_token4] = ACTIONS(1119),
    [aux_sym_variable_attributes_token5] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1119),
    [aux_sym_derived_type_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token2] = ACTIONS(1119),
    [aux_sym_type_qualifier_token3] = ACTIONS(1119),
    [aux_sym_type_qualifier_token4] = ACTIONS(1119),
    [aux_sym_type_qualifier_token5] = ACTIONS(1119),
    [aux_sym_type_qualifier_token9] = ACTIONS(1119),
    [aux_sym_type_qualifier_token10] = ACTIONS(1119),
    [aux_sym_type_qualifier_token11] = ACTIONS(1119),
    [aux_sym_type_qualifier_token12] = ACTIONS(1119),
    [aux_sym_type_qualifier_token13] = ACTIONS(1119),
    [aux_sym_type_qualifier_token14] = ACTIONS(1119),
    [aux_sym_type_qualifier_token15] = ACTIONS(1119),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1119),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1119),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1119),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token2] = ACTIONS(1119),
    [aux_sym_subroutine_call_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token2] = ACTIONS(1119),
    [aux_sym_keyword_statement_token3] = ACTIONS(1119),
    [aux_sym_keyword_statement_token4] = ACTIONS(1119),
    [aux_sym_keyword_statement_token6] = ACTIONS(1119),
    [aux_sym_keyword_statement_token7] = ACTIONS(1119),
    [aux_sym_include_statement_token1] = ACTIONS(1119),
    [aux_sym_data_statement_token1] = ACTIONS(1119),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1119),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1119),
    [aux_sym_end_if_statement_token1] = ACTIONS(1119),
    [aux_sym_elseif_clause_token2] = ACTIONS(1119),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1119),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token3] = ACTIONS(1119),
    [aux_sym_select_type_statement_token1] = ACTIONS(1119),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1119),
    [aux_sym_block_construct_token1] = ACTIONS(1119),
    [aux_sym_associate_statement_token1] = ACTIONS(1119),
    [aux_sym_format_statement_token1] = ACTIONS(1119),
    [aux_sym_print_statement_token1] = ACTIONS(1119),
    [aux_sym_open_statement_token1] = ACTIONS(1119),
    [aux_sym_close_statement_token1] = ACTIONS(1119),
    [aux_sym_inquire_statement_token1] = ACTIONS(1119),
    [aux_sym_enum_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token2] = ACTIONS(1119),
    [aux_sym_file_position_statement_token3] = ACTIONS(1119),
    [aux_sym_file_position_statement_token4] = ACTIONS(1119),
    [aux_sym_logical_expression_token5] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [sym_boolean_literal] = ACTIONS(1117),
    [aux_sym_null_literal_token1] = ACTIONS(1119),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1119),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1117),
    [sym__float_literal] = ACTIONS(1117),
    [sym__boz_literal] = ACTIONS(1117),
    [sym_string_literal] = ACTIONS(1117),
  },
  [239] = {
    [sym_preproc_file_line] = STATE(239),
    [ts_builtin_sym_end] = ACTIONS(1121),
    [aux_sym_program_statement_token1] = ACTIONS(1123),
    [aux_sym_end_program_statement_token1] = ACTIONS(1123),
    [aux_sym_end_program_statement_token2] = ACTIONS(1123),
    [aux_sym_module_statement_token1] = ACTIONS(1123),
    [aux_sym_submodule_statement_token1] = ACTIONS(1123),
    [anon_sym_LPAREN] = ACTIONS(1123),
    [aux_sym_interface_statement_token1] = ACTIONS(1123),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1123),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1123),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1123),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1123),
    [aux_sym_function_statement_token1] = ACTIONS(1123),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1123),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1123),
    [aux_sym_contains_statement_token1] = ACTIONS(1123),
    [aux_sym_use_statement_token1] = ACTIONS(1123),
    [aux_sym_use_statement_token2] = ACTIONS(1123),
    [aux_sym_implicit_statement_token1] = ACTIONS(1123),
    [aux_sym_implicit_statement_token3] = ACTIONS(1123),
    [aux_sym_implicit_statement_token4] = ACTIONS(1123),
    [aux_sym_save_statement_token1] = ACTIONS(1123),
    [aux_sym_private_statement_token1] = ACTIONS(1123),
    [aux_sym_public_statement_token1] = ACTIONS(1123),
    [aux_sym_namelist_statement_token1] = ACTIONS(1123),
    [aux_sym_common_statement_token1] = ACTIONS(1123),
    [anon_sym_DASH] = ACTIONS(1121),
    [aux_sym_import_statement_token1] = ACTIONS(1123),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1123),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1123),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1123),
    [aux_sym_variable_attributes_token1] = ACTIONS(1123),
    [aux_sym_variable_attributes_token2] = ACTIONS(1123),
    [aux_sym_variable_attributes_token3] = ACTIONS(1123),
    [aux_sym_variable_attributes_token4] = ACTIONS(1123),
    [aux_sym_variable_attributes_token5] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1123),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1123),
    [aux_sym_derived_type_token1] = ACTIONS(1123),
    [aux_sym_type_qualifier_token1] = ACTIONS(1123),
    [aux_sym_type_qualifier_token2] = ACTIONS(1123),
    [aux_sym_type_qualifier_token3] = ACTIONS(1123),
    [aux_sym_type_qualifier_token4] = ACTIONS(1123),
    [aux_sym_type_qualifier_token5] = ACTIONS(1123),
    [aux_sym_type_qualifier_token9] = ACTIONS(1123),
    [aux_sym_type_qualifier_token10] = ACTIONS(1123),
    [aux_sym_type_qualifier_token11] = ACTIONS(1123),
    [aux_sym_type_qualifier_token12] = ACTIONS(1123),
    [aux_sym_type_qualifier_token13] = ACTIONS(1123),
    [aux_sym_type_qualifier_token14] = ACTIONS(1123),
    [aux_sym_type_qualifier_token15] = ACTIONS(1123),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1123),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1123),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1123),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1123),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1123),
    [aux_sym_stop_statement_token1] = ACTIONS(1123),
    [aux_sym_stop_statement_token2] = ACTIONS(1123),
    [aux_sym_subroutine_call_token1] = ACTIONS(1123),
    [aux_sym_keyword_statement_token1] = ACTIONS(1123),
    [aux_sym_keyword_statement_token2] = ACTIONS(1123),
    [aux_sym_keyword_statement_token3] = ACTIONS(1123),
    [aux_sym_keyword_statement_token4] = ACTIONS(1123),
    [aux_sym_keyword_statement_token6] = ACTIONS(1123),
    [aux_sym_keyword_statement_token7] = ACTIONS(1123),
    [aux_sym_include_statement_token1] = ACTIONS(1123),
    [aux_sym_data_statement_token1] = ACTIONS(1123),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1123),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1123),
    [aux_sym_end_if_statement_token1] = ACTIONS(1123),
    [aux_sym_elseif_clause_token2] = ACTIONS(1123),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1123),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1123),
    [aux_sym_select_case_statement_token1] = ACTIONS(1123),
    [aux_sym_select_case_statement_token3] = ACTIONS(1123),
    [aux_sym_select_type_statement_token1] = ACTIONS(1123),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1123),
    [aux_sym_block_construct_token1] = ACTIONS(1123),
    [aux_sym_associate_statement_token1] = ACTIONS(1123),
    [aux_sym_format_statement_token1] = ACTIONS(1123),
    [aux_sym_print_statement_token1] = ACTIONS(1123),
    [aux_sym_open_statement_token1] = ACTIONS(1123),
    [aux_sym_close_statement_token1] = ACTIONS(1123),
    [aux_sym_inquire_statement_token1] = ACTIONS(1123),
    [aux_sym_enum_statement_token1] = ACTIONS(1123),
    [aux_sym_file_position_statement_token1] = ACTIONS(1123),
    [aux_sym_file_position_statement_token2] = ACTIONS(1123),
    [aux_sym_file_position_statement_token3] = ACTIONS(1123),
    [aux_sym_file_position_statement_token4] = ACTIONS(1123),
    [aux_sym_logical_expression_token5] = ACTIONS(1121),
    [anon_sym_PLUS] = ACTIONS(1121),
    [anon_sym_DOT] = ACTIONS(1123),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1121),
    [anon_sym_LBRACK] = ACTIONS(1121),
    [sym_boolean_literal] = ACTIONS(1121),
    [aux_sym_null_literal_token1] = ACTIONS(1123),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1123),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1121),
    [sym__float_literal] = ACTIONS(1121),
    [sym__boz_literal] = ACTIONS(1121),
    [sym_string_literal] = ACTIONS(1121),
  },
  [240] = {
    [sym_preproc_file_line] = STATE(240),
    [ts_builtin_sym_end] = ACTIONS(1125),
    [aux_sym_program_statement_token1] = ACTIONS(1127),
    [aux_sym_end_program_statement_token1] = ACTIONS(1127),
    [aux_sym_end_program_statement_token2] = ACTIONS(1127),
    [aux_sym_module_statement_token1] = ACTIONS(1127),
    [aux_sym_submodule_statement_token1] = ACTIONS(1127),
    [anon_sym_LPAREN] = ACTIONS(1127),
    [aux_sym_interface_statement_token1] = ACTIONS(1127),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1127),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1127),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1127),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1127),
    [aux_sym_function_statement_token1] = ACTIONS(1127),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1127),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1127),
    [aux_sym_contains_statement_token1] = ACTIONS(1127),
    [aux_sym_use_statement_token1] = ACTIONS(1127),
    [aux_sym_use_statement_token2] = ACTIONS(1127),
    [aux_sym_implicit_statement_token1] = ACTIONS(1127),
    [aux_sym_implicit_statement_token3] = ACTIONS(1127),
    [aux_sym_implicit_statement_token4] = ACTIONS(1127),
    [aux_sym_save_statement_token1] = ACTIONS(1127),
    [aux_sym_private_statement_token1] = ACTIONS(1127),
    [aux_sym_public_statement_token1] = ACTIONS(1127),
    [aux_sym_namelist_statement_token1] = ACTIONS(1127),
    [aux_sym_common_statement_token1] = ACTIONS(1127),
    [anon_sym_DASH] = ACTIONS(1125),
    [aux_sym_import_statement_token1] = ACTIONS(1127),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1127),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1127),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1127),
    [aux_sym_variable_attributes_token1] = ACTIONS(1127),
    [aux_sym_variable_attributes_token2] = ACTIONS(1127),
    [aux_sym_variable_attributes_token3] = ACTIONS(1127),
    [aux_sym_variable_attributes_token4] = ACTIONS(1127),
    [aux_sym_variable_attributes_token5] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1127),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1127),
    [aux_sym_derived_type_token1] = ACTIONS(1127),
    [aux_sym_type_qualifier_token1] = ACTIONS(1127),
    [aux_sym_type_qualifier_token2] = ACTIONS(1127),
    [aux_sym_type_qualifier_token3] = ACTIONS(1127),
    [aux_sym_type_qualifier_token4] = ACTIONS(1127),
    [aux_sym_type_qualifier_token5] = ACTIONS(1127),
    [aux_sym_type_qualifier_token9] = ACTIONS(1127),
    [aux_sym_type_qualifier_token10] = ACTIONS(1127),
    [aux_sym_type_qualifier_token11] = ACTIONS(1127),
    [aux_sym_type_qualifier_token12] = ACTIONS(1127),
    [aux_sym_type_qualifier_token13] = ACTIONS(1127),
    [aux_sym_type_qualifier_token14] = ACTIONS(1127),
    [aux_sym_type_qualifier_token15] = ACTIONS(1127),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1127),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1127),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1127),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1127),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1127),
    [aux_sym_stop_statement_token1] = ACTIONS(1127),
    [aux_sym_stop_statement_token2] = ACTIONS(1127),
    [aux_sym_subroutine_call_token1] = ACTIONS(1127),
    [aux_sym_keyword_statement_token1] = ACTIONS(1127),
    [aux_sym_keyword_statement_token2] = ACTIONS(1127),
    [aux_sym_keyword_statement_token3] = ACTIONS(1127),
    [aux_sym_keyword_statement_token4] = ACTIONS(1127),
    [aux_sym_keyword_statement_token6] = ACTIONS(1127),
    [aux_sym_keyword_statement_token7] = ACTIONS(1127),
    [aux_sym_include_statement_token1] = ACTIONS(1127),
    [aux_sym_data_statement_token1] = ACTIONS(1127),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1127),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1127),
    [aux_sym_end_if_statement_token1] = ACTIONS(1127),
    [aux_sym_elseif_clause_token2] = ACTIONS(1127),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1127),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1127),
    [aux_sym_select_case_statement_token1] = ACTIONS(1127),
    [aux_sym_select_case_statement_token3] = ACTIONS(1127),
    [aux_sym_select_type_statement_token1] = ACTIONS(1127),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1127),
    [aux_sym_block_construct_token1] = ACTIONS(1127),
    [aux_sym_associate_statement_token1] = ACTIONS(1127),
    [aux_sym_format_statement_token1] = ACTIONS(1127),
    [aux_sym_print_statement_token1] = ACTIONS(1127),
    [aux_sym_open_statement_token1] = ACTIONS(1127),
    [aux_sym_close_statement_token1] = ACTIONS(1127),
    [aux_sym_inquire_statement_token1] = ACTIONS(1127),
    [aux_sym_enum_statement_token1] = ACTIONS(1127),
    [aux_sym_file_position_statement_token1] = ACTIONS(1127),
    [aux_sym_file_position_statement_token2] = ACTIONS(1127),
    [aux_sym_file_position_statement_token3] = ACTIONS(1127),
    [aux_sym_file_position_statement_token4] = ACTIONS(1127),
    [aux_sym_logical_expression_token5] = ACTIONS(1125),
    [anon_sym_PLUS] = ACTIONS(1125),
    [anon_sym_DOT] = ACTIONS(1127),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1125),
    [anon_sym_LBRACK] = ACTIONS(1125),
    [sym_boolean_literal] = ACTIONS(1125),
    [aux_sym_null_literal_token1] = ACTIONS(1127),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1127),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1125),
    [sym__float_literal] = ACTIONS(1125),
    [sym__boz_literal] = ACTIONS(1125),
    [sym_string_literal] = ACTIONS(1125),
  },
  [241] = {
    [sym_preproc_file_line] = STATE(241),
    [ts_builtin_sym_end] = ACTIONS(1129),
    [aux_sym_program_statement_token1] = ACTIONS(1131),
    [aux_sym_end_program_statement_token1] = ACTIONS(1131),
    [aux_sym_end_program_statement_token2] = ACTIONS(1131),
    [aux_sym_module_statement_token1] = ACTIONS(1131),
    [aux_sym_submodule_statement_token1] = ACTIONS(1131),
    [anon_sym_LPAREN] = ACTIONS(1131),
    [aux_sym_interface_statement_token1] = ACTIONS(1131),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1131),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1131),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1131),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1131),
    [aux_sym_function_statement_token1] = ACTIONS(1131),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1131),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1131),
    [aux_sym_contains_statement_token1] = ACTIONS(1131),
    [aux_sym_use_statement_token1] = ACTIONS(1131),
    [aux_sym_use_statement_token2] = ACTIONS(1131),
    [aux_sym_implicit_statement_token1] = ACTIONS(1131),
    [aux_sym_implicit_statement_token3] = ACTIONS(1131),
    [aux_sym_implicit_statement_token4] = ACTIONS(1131),
    [aux_sym_save_statement_token1] = ACTIONS(1131),
    [aux_sym_private_statement_token1] = ACTIONS(1131),
    [aux_sym_public_statement_token1] = ACTIONS(1131),
    [aux_sym_namelist_statement_token1] = ACTIONS(1131),
    [aux_sym_common_statement_token1] = ACTIONS(1131),
    [anon_sym_DASH] = ACTIONS(1129),
    [aux_sym_import_statement_token1] = ACTIONS(1131),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1131),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1131),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1131),
    [aux_sym_variable_attributes_token1] = ACTIONS(1131),
    [aux_sym_variable_attributes_token2] = ACTIONS(1131),
    [aux_sym_variable_attributes_token3] = ACTIONS(1131),
    [aux_sym_variable_attributes_token4] = ACTIONS(1131),
    [aux_sym_variable_attributes_token5] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1131),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1131),
    [aux_sym_derived_type_token1] = ACTIONS(1131),
    [aux_sym_type_qualifier_token1] = ACTIONS(1131),
    [aux_sym_type_qualifier_token2] = ACTIONS(1131),
    [aux_sym_type_qualifier_token3] = ACTIONS(1131),
    [aux_sym_type_qualifier_token4] = ACTIONS(1131),
    [aux_sym_type_qualifier_token5] = ACTIONS(1131),
    [aux_sym_type_qualifier_token9] = ACTIONS(1131),
    [aux_sym_type_qualifier_token10] = ACTIONS(1131),
    [aux_sym_type_qualifier_token11] = ACTIONS(1131),
    [aux_sym_type_qualifier_token12] = ACTIONS(1131),
    [aux_sym_type_qualifier_token13] = ACTIONS(1131),
    [aux_sym_type_qualifier_token14] = ACTIONS(1131),
    [aux_sym_type_qualifier_token15] = ACTIONS(1131),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1131),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1131),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1131),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1131),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1131),
    [aux_sym_stop_statement_token1] = ACTIONS(1131),
    [aux_sym_stop_statement_token2] = ACTIONS(1131),
    [aux_sym_subroutine_call_token1] = ACTIONS(1131),
    [aux_sym_keyword_statement_token1] = ACTIONS(1131),
    [aux_sym_keyword_statement_token2] = ACTIONS(1131),
    [aux_sym_keyword_statement_token3] = ACTIONS(1131),
    [aux_sym_keyword_statement_token4] = ACTIONS(1131),
    [aux_sym_keyword_statement_token6] = ACTIONS(1131),
    [aux_sym_keyword_statement_token7] = ACTIONS(1131),
    [aux_sym_include_statement_token1] = ACTIONS(1131),
    [aux_sym_data_statement_token1] = ACTIONS(1131),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1131),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1131),
    [aux_sym_end_if_statement_token1] = ACTIONS(1131),
    [aux_sym_elseif_clause_token2] = ACTIONS(1131),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1131),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1131),
    [aux_sym_select_case_statement_token1] = ACTIONS(1131),
    [aux_sym_select_case_statement_token3] = ACTIONS(1131),
    [aux_sym_select_type_statement_token1] = ACTIONS(1131),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1131),
    [aux_sym_block_construct_token1] = ACTIONS(1131),
    [aux_sym_associate_statement_token1] = ACTIONS(1131),
    [aux_sym_format_statement_token1] = ACTIONS(1131),
    [aux_sym_print_statement_token1] = ACTIONS(1131),
    [aux_sym_open_statement_token1] = ACTIONS(1131),
    [aux_sym_close_statement_token1] = ACTIONS(1131),
    [aux_sym_inquire_statement_token1] = ACTIONS(1131),
    [aux_sym_enum_statement_token1] = ACTIONS(1131),
    [aux_sym_file_position_statement_token1] = ACTIONS(1131),
    [aux_sym_file_position_statement_token2] = ACTIONS(1131),
    [aux_sym_file_position_statement_token3] = ACTIONS(1131),
    [aux_sym_file_position_statement_token4] = ACTIONS(1131),
    [aux_sym_logical_expression_token5] = ACTIONS(1129),
    [anon_sym_PLUS] = ACTIONS(1129),
    [anon_sym_DOT] = ACTIONS(1131),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1129),
    [anon_sym_LBRACK] = ACTIONS(1129),
    [sym_boolean_literal] = ACTIONS(1129),
    [aux_sym_null_literal_token1] = ACTIONS(1131),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1131),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1129),
    [sym__float_literal] = ACTIONS(1129),
    [sym__boz_literal] = ACTIONS(1129),
    [sym_string_literal] = ACTIONS(1129),
  },
  [242] = {
    [sym_preproc_file_line] = STATE(242),
    [ts_builtin_sym_end] = ACTIONS(1133),
    [aux_sym_program_statement_token1] = ACTIONS(1135),
    [aux_sym_end_program_statement_token1] = ACTIONS(1135),
    [aux_sym_end_program_statement_token2] = ACTIONS(1135),
    [aux_sym_module_statement_token1] = ACTIONS(1135),
    [aux_sym_submodule_statement_token1] = ACTIONS(1135),
    [anon_sym_LPAREN] = ACTIONS(1135),
    [aux_sym_interface_statement_token1] = ACTIONS(1135),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1135),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1135),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1135),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1135),
    [aux_sym_function_statement_token1] = ACTIONS(1135),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1135),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1135),
    [aux_sym_contains_statement_token1] = ACTIONS(1135),
    [aux_sym_use_statement_token1] = ACTIONS(1135),
    [aux_sym_use_statement_token2] = ACTIONS(1135),
    [aux_sym_implicit_statement_token1] = ACTIONS(1135),
    [aux_sym_implicit_statement_token3] = ACTIONS(1135),
    [aux_sym_implicit_statement_token4] = ACTIONS(1135),
    [aux_sym_save_statement_token1] = ACTIONS(1135),
    [aux_sym_private_statement_token1] = ACTIONS(1135),
    [aux_sym_public_statement_token1] = ACTIONS(1135),
    [aux_sym_namelist_statement_token1] = ACTIONS(1135),
    [aux_sym_common_statement_token1] = ACTIONS(1135),
    [anon_sym_DASH] = ACTIONS(1133),
    [aux_sym_import_statement_token1] = ACTIONS(1135),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1135),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1135),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1135),
    [aux_sym_variable_attributes_token1] = ACTIONS(1135),
    [aux_sym_variable_attributes_token2] = ACTIONS(1135),
    [aux_sym_variable_attributes_token3] = ACTIONS(1135),
    [aux_sym_variable_attributes_token4] = ACTIONS(1135),
    [aux_sym_variable_attributes_token5] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1135),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1135),
    [aux_sym_derived_type_token1] = ACTIONS(1135),
    [aux_sym_type_qualifier_token1] = ACTIONS(1135),
    [aux_sym_type_qualifier_token2] = ACTIONS(1135),
    [aux_sym_type_qualifier_token3] = ACTIONS(1135),
    [aux_sym_type_qualifier_token4] = ACTIONS(1135),
    [aux_sym_type_qualifier_token5] = ACTIONS(1135),
    [aux_sym_type_qualifier_token9] = ACTIONS(1135),
    [aux_sym_type_qualifier_token10] = ACTIONS(1135),
    [aux_sym_type_qualifier_token11] = ACTIONS(1135),
    [aux_sym_type_qualifier_token12] = ACTIONS(1135),
    [aux_sym_type_qualifier_token13] = ACTIONS(1135),
    [aux_sym_type_qualifier_token14] = ACTIONS(1135),
    [aux_sym_type_qualifier_token15] = ACTIONS(1135),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1135),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1135),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1135),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1135),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1135),
    [aux_sym_stop_statement_token1] = ACTIONS(1135),
    [aux_sym_stop_statement_token2] = ACTIONS(1135),
    [aux_sym_subroutine_call_token1] = ACTIONS(1135),
    [aux_sym_keyword_statement_token1] = ACTIONS(1135),
    [aux_sym_keyword_statement_token2] = ACTIONS(1135),
    [aux_sym_keyword_statement_token3] = ACTIONS(1135),
    [aux_sym_keyword_statement_token4] = ACTIONS(1135),
    [aux_sym_keyword_statement_token6] = ACTIONS(1135),
    [aux_sym_keyword_statement_token7] = ACTIONS(1135),
    [aux_sym_include_statement_token1] = ACTIONS(1135),
    [aux_sym_data_statement_token1] = ACTIONS(1135),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1135),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1135),
    [aux_sym_end_if_statement_token1] = ACTIONS(1135),
    [aux_sym_elseif_clause_token2] = ACTIONS(1135),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1135),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1135),
    [aux_sym_select_case_statement_token1] = ACTIONS(1135),
    [aux_sym_select_case_statement_token3] = ACTIONS(1135),
    [aux_sym_select_type_statement_token1] = ACTIONS(1135),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1135),
    [aux_sym_block_construct_token1] = ACTIONS(1135),
    [aux_sym_associate_statement_token1] = ACTIONS(1135),
    [aux_sym_format_statement_token1] = ACTIONS(1135),
    [aux_sym_print_statement_token1] = ACTIONS(1135),
    [aux_sym_open_statement_token1] = ACTIONS(1135),
    [aux_sym_close_statement_token1] = ACTIONS(1135),
    [aux_sym_inquire_statement_token1] = ACTIONS(1135),
    [aux_sym_enum_statement_token1] = ACTIONS(1135),
    [aux_sym_file_position_statement_token1] = ACTIONS(1135),
    [aux_sym_file_position_statement_token2] = ACTIONS(1135),
    [aux_sym_file_position_statement_token3] = ACTIONS(1135),
    [aux_sym_file_position_statement_token4] = ACTIONS(1135),
    [aux_sym_logical_expression_token5] = ACTIONS(1133),
    [anon_sym_PLUS] = ACTIONS(1133),
    [anon_sym_DOT] = ACTIONS(1135),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1133),
    [anon_sym_LBRACK] = ACTIONS(1133),
    [sym_boolean_literal] = ACTIONS(1133),
    [aux_sym_null_literal_token1] = ACTIONS(1135),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1135),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1133),
    [sym__float_literal] = ACTIONS(1133),
    [sym__boz_literal] = ACTIONS(1133),
    [sym_string_literal] = ACTIONS(1133),
  },
  [243] = {
    [sym_preproc_file_line] = STATE(243),
    [ts_builtin_sym_end] = ACTIONS(1137),
    [aux_sym_program_statement_token1] = ACTIONS(1139),
    [aux_sym_end_program_statement_token1] = ACTIONS(1139),
    [aux_sym_end_program_statement_token2] = ACTIONS(1139),
    [aux_sym_module_statement_token1] = ACTIONS(1139),
    [aux_sym_submodule_statement_token1] = ACTIONS(1139),
    [anon_sym_LPAREN] = ACTIONS(1139),
    [aux_sym_interface_statement_token1] = ACTIONS(1139),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1139),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1139),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1139),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1139),
    [aux_sym_function_statement_token1] = ACTIONS(1139),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1139),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1139),
    [aux_sym_contains_statement_token1] = ACTIONS(1139),
    [aux_sym_use_statement_token1] = ACTIONS(1139),
    [aux_sym_use_statement_token2] = ACTIONS(1139),
    [aux_sym_implicit_statement_token1] = ACTIONS(1139),
    [aux_sym_implicit_statement_token3] = ACTIONS(1139),
    [aux_sym_implicit_statement_token4] = ACTIONS(1139),
    [aux_sym_save_statement_token1] = ACTIONS(1139),
    [aux_sym_private_statement_token1] = ACTIONS(1139),
    [aux_sym_public_statement_token1] = ACTIONS(1139),
    [aux_sym_namelist_statement_token1] = ACTIONS(1139),
    [aux_sym_common_statement_token1] = ACTIONS(1139),
    [anon_sym_DASH] = ACTIONS(1137),
    [aux_sym_import_statement_token1] = ACTIONS(1139),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1139),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1139),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1139),
    [aux_sym_variable_attributes_token1] = ACTIONS(1139),
    [aux_sym_variable_attributes_token2] = ACTIONS(1139),
    [aux_sym_variable_attributes_token3] = ACTIONS(1139),
    [aux_sym_variable_attributes_token4] = ACTIONS(1139),
    [aux_sym_variable_attributes_token5] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1139),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1139),
    [aux_sym_derived_type_token1] = ACTIONS(1139),
    [aux_sym_type_qualifier_token1] = ACTIONS(1139),
    [aux_sym_type_qualifier_token2] = ACTIONS(1139),
    [aux_sym_type_qualifier_token3] = ACTIONS(1139),
    [aux_sym_type_qualifier_token4] = ACTIONS(1139),
    [aux_sym_type_qualifier_token5] = ACTIONS(1139),
    [aux_sym_type_qualifier_token9] = ACTIONS(1139),
    [aux_sym_type_qualifier_token10] = ACTIONS(1139),
    [aux_sym_type_qualifier_token11] = ACTIONS(1139),
    [aux_sym_type_qualifier_token12] = ACTIONS(1139),
    [aux_sym_type_qualifier_token13] = ACTIONS(1139),
    [aux_sym_type_qualifier_token14] = ACTIONS(1139),
    [aux_sym_type_qualifier_token15] = ACTIONS(1139),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1139),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1139),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1139),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1139),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1139),
    [aux_sym_stop_statement_token1] = ACTIONS(1139),
    [aux_sym_stop_statement_token2] = ACTIONS(1139),
    [aux_sym_subroutine_call_token1] = ACTIONS(1139),
    [aux_sym_keyword_statement_token1] = ACTIONS(1139),
    [aux_sym_keyword_statement_token2] = ACTIONS(1139),
    [aux_sym_keyword_statement_token3] = ACTIONS(1139),
    [aux_sym_keyword_statement_token4] = ACTIONS(1139),
    [aux_sym_keyword_statement_token6] = ACTIONS(1139),
    [aux_sym_keyword_statement_token7] = ACTIONS(1139),
    [aux_sym_include_statement_token1] = ACTIONS(1139),
    [aux_sym_data_statement_token1] = ACTIONS(1139),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1139),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1139),
    [aux_sym_end_if_statement_token1] = ACTIONS(1139),
    [aux_sym_elseif_clause_token2] = ACTIONS(1139),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1139),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1139),
    [aux_sym_select_case_statement_token1] = ACTIONS(1139),
    [aux_sym_select_case_statement_token3] = ACTIONS(1139),
    [aux_sym_select_type_statement_token1] = ACTIONS(1139),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1139),
    [aux_sym_block_construct_token1] = ACTIONS(1139),
    [aux_sym_associate_statement_token1] = ACTIONS(1139),
    [aux_sym_format_statement_token1] = ACTIONS(1139),
    [aux_sym_print_statement_token1] = ACTIONS(1139),
    [aux_sym_open_statement_token1] = ACTIONS(1139),
    [aux_sym_close_statement_token1] = ACTIONS(1139),
    [aux_sym_inquire_statement_token1] = ACTIONS(1139),
    [aux_sym_enum_statement_token1] = ACTIONS(1139),
    [aux_sym_file_position_statement_token1] = ACTIONS(1139),
    [aux_sym_file_position_statement_token2] = ACTIONS(1139),
    [aux_sym_file_position_statement_token3] = ACTIONS(1139),
    [aux_sym_file_position_statement_token4] = ACTIONS(1139),
    [aux_sym_logical_expression_token5] = ACTIONS(1137),
    [anon_sym_PLUS] = ACTIONS(1137),
    [anon_sym_DOT] = ACTIONS(1139),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1137),
    [anon_sym_LBRACK] = ACTIONS(1137),
    [sym_boolean_literal] = ACTIONS(1137),
    [aux_sym_null_literal_token1] = ACTIONS(1139),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1139),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1137),
    [sym__float_literal] = ACTIONS(1137),
    [sym__boz_literal] = ACTIONS(1137),
    [sym_string_literal] = ACTIONS(1137),
  },
  [244] = {
    [sym_preproc_file_line] = STATE(244),
    [ts_builtin_sym_end] = ACTIONS(1141),
    [aux_sym_program_statement_token1] = ACTIONS(1143),
    [aux_sym_end_program_statement_token1] = ACTIONS(1143),
    [aux_sym_end_program_statement_token2] = ACTIONS(1143),
    [aux_sym_module_statement_token1] = ACTIONS(1143),
    [aux_sym_submodule_statement_token1] = ACTIONS(1143),
    [anon_sym_LPAREN] = ACTIONS(1143),
    [aux_sym_interface_statement_token1] = ACTIONS(1143),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1143),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1143),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1143),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1143),
    [aux_sym_function_statement_token1] = ACTIONS(1143),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1143),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1143),
    [aux_sym_contains_statement_token1] = ACTIONS(1143),
    [aux_sym_use_statement_token1] = ACTIONS(1143),
    [aux_sym_use_statement_token2] = ACTIONS(1143),
    [aux_sym_implicit_statement_token1] = ACTIONS(1143),
    [aux_sym_implicit_statement_token3] = ACTIONS(1143),
    [aux_sym_implicit_statement_token4] = ACTIONS(1143),
    [aux_sym_save_statement_token1] = ACTIONS(1143),
    [aux_sym_private_statement_token1] = ACTIONS(1143),
    [aux_sym_public_statement_token1] = ACTIONS(1143),
    [aux_sym_namelist_statement_token1] = ACTIONS(1143),
    [aux_sym_common_statement_token1] = ACTIONS(1143),
    [anon_sym_DASH] = ACTIONS(1141),
    [aux_sym_import_statement_token1] = ACTIONS(1143),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1143),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1143),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1143),
    [aux_sym_variable_attributes_token1] = ACTIONS(1143),
    [aux_sym_variable_attributes_token2] = ACTIONS(1143),
    [aux_sym_variable_attributes_token3] = ACTIONS(1143),
    [aux_sym_variable_attributes_token4] = ACTIONS(1143),
    [aux_sym_variable_attributes_token5] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1143),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1143),
    [aux_sym_derived_type_token1] = ACTIONS(1143),
    [aux_sym_type_qualifier_token1] = ACTIONS(1143),
    [aux_sym_type_qualifier_token2] = ACTIONS(1143),
    [aux_sym_type_qualifier_token3] = ACTIONS(1143),
    [aux_sym_type_qualifier_token4] = ACTIONS(1143),
    [aux_sym_type_qualifier_token5] = ACTIONS(1143),
    [aux_sym_type_qualifier_token9] = ACTIONS(1143),
    [aux_sym_type_qualifier_token10] = ACTIONS(1143),
    [aux_sym_type_qualifier_token11] = ACTIONS(1143),
    [aux_sym_type_qualifier_token12] = ACTIONS(1143),
    [aux_sym_type_qualifier_token13] = ACTIONS(1143),
    [aux_sym_type_qualifier_token14] = ACTIONS(1143),
    [aux_sym_type_qualifier_token15] = ACTIONS(1143),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1143),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1143),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1143),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1143),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1143),
    [aux_sym_stop_statement_token1] = ACTIONS(1143),
    [aux_sym_stop_statement_token2] = ACTIONS(1143),
    [aux_sym_subroutine_call_token1] = ACTIONS(1143),
    [aux_sym_keyword_statement_token1] = ACTIONS(1143),
    [aux_sym_keyword_statement_token2] = ACTIONS(1143),
    [aux_sym_keyword_statement_token3] = ACTIONS(1143),
    [aux_sym_keyword_statement_token4] = ACTIONS(1143),
    [aux_sym_keyword_statement_token6] = ACTIONS(1143),
    [aux_sym_keyword_statement_token7] = ACTIONS(1143),
    [aux_sym_include_statement_token1] = ACTIONS(1143),
    [aux_sym_data_statement_token1] = ACTIONS(1143),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1143),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1143),
    [aux_sym_end_if_statement_token1] = ACTIONS(1143),
    [aux_sym_elseif_clause_token2] = ACTIONS(1143),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1143),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1143),
    [aux_sym_select_case_statement_token1] = ACTIONS(1143),
    [aux_sym_select_case_statement_token3] = ACTIONS(1143),
    [aux_sym_select_type_statement_token1] = ACTIONS(1143),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1143),
    [aux_sym_block_construct_token1] = ACTIONS(1143),
    [aux_sym_associate_statement_token1] = ACTIONS(1143),
    [aux_sym_format_statement_token1] = ACTIONS(1143),
    [aux_sym_print_statement_token1] = ACTIONS(1143),
    [aux_sym_open_statement_token1] = ACTIONS(1143),
    [aux_sym_close_statement_token1] = ACTIONS(1143),
    [aux_sym_inquire_statement_token1] = ACTIONS(1143),
    [aux_sym_enum_statement_token1] = ACTIONS(1143),
    [aux_sym_file_position_statement_token1] = ACTIONS(1143),
    [aux_sym_file_position_statement_token2] = ACTIONS(1143),
    [aux_sym_file_position_statement_token3] = ACTIONS(1143),
    [aux_sym_file_position_statement_token4] = ACTIONS(1143),
    [aux_sym_logical_expression_token5] = ACTIONS(1141),
    [anon_sym_PLUS] = ACTIONS(1141),
    [anon_sym_DOT] = ACTIONS(1143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1141),
    [anon_sym_LBRACK] = ACTIONS(1141),
    [sym_boolean_literal] = ACTIONS(1141),
    [aux_sym_null_literal_token1] = ACTIONS(1143),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1143),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1141),
    [sym__float_literal] = ACTIONS(1141),
    [sym__boz_literal] = ACTIONS(1141),
    [sym_string_literal] = ACTIONS(1141),
  },
  [245] = {
    [sym_preproc_file_line] = STATE(245),
    [ts_builtin_sym_end] = ACTIONS(1145),
    [aux_sym_program_statement_token1] = ACTIONS(1147),
    [aux_sym_end_program_statement_token1] = ACTIONS(1147),
    [aux_sym_end_program_statement_token2] = ACTIONS(1147),
    [aux_sym_module_statement_token1] = ACTIONS(1147),
    [aux_sym_submodule_statement_token1] = ACTIONS(1147),
    [anon_sym_LPAREN] = ACTIONS(1147),
    [aux_sym_interface_statement_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1147),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1147),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_function_statement_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1147),
    [aux_sym_contains_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token2] = ACTIONS(1147),
    [aux_sym_implicit_statement_token1] = ACTIONS(1147),
    [aux_sym_implicit_statement_token3] = ACTIONS(1147),
    [aux_sym_implicit_statement_token4] = ACTIONS(1147),
    [aux_sym_save_statement_token1] = ACTIONS(1147),
    [aux_sym_private_statement_token1] = ACTIONS(1147),
    [aux_sym_public_statement_token1] = ACTIONS(1147),
    [aux_sym_namelist_statement_token1] = ACTIONS(1147),
    [aux_sym_common_statement_token1] = ACTIONS(1147),
    [anon_sym_DASH] = ACTIONS(1145),
    [aux_sym_import_statement_token1] = ACTIONS(1147),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1147),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1147),
    [aux_sym_variable_attributes_token1] = ACTIONS(1147),
    [aux_sym_variable_attributes_token2] = ACTIONS(1147),
    [aux_sym_variable_attributes_token3] = ACTIONS(1147),
    [aux_sym_variable_attributes_token4] = ACTIONS(1147),
    [aux_sym_variable_attributes_token5] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1147),
    [aux_sym_derived_type_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token2] = ACTIONS(1147),
    [aux_sym_type_qualifier_token3] = ACTIONS(1147),
    [aux_sym_type_qualifier_token4] = ACTIONS(1147),
    [aux_sym_type_qualifier_token5] = ACTIONS(1147),
    [aux_sym_type_qualifier_token9] = ACTIONS(1147),
    [aux_sym_type_qualifier_token10] = ACTIONS(1147),
    [aux_sym_type_qualifier_token11] = ACTIONS(1147),
    [aux_sym_type_qualifier_token12] = ACTIONS(1147),
    [aux_sym_type_qualifier_token13] = ACTIONS(1147),
    [aux_sym_type_qualifier_token14] = ACTIONS(1147),
    [aux_sym_type_qualifier_token15] = ACTIONS(1147),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1147),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1147),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1147),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token2] = ACTIONS(1147),
    [aux_sym_subroutine_call_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token2] = ACTIONS(1147),
    [aux_sym_keyword_statement_token3] = ACTIONS(1147),
    [aux_sym_keyword_statement_token4] = ACTIONS(1147),
    [aux_sym_keyword_statement_token6] = ACTIONS(1147),
    [aux_sym_keyword_statement_token7] = ACTIONS(1147),
    [aux_sym_include_statement_token1] = ACTIONS(1147),
    [aux_sym_data_statement_token1] = ACTIONS(1147),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1147),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1147),
    [aux_sym_end_if_statement_token1] = ACTIONS(1147),
    [aux_sym_elseif_clause_token2] = ACTIONS(1147),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1147),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token3] = ACTIONS(1147),
    [aux_sym_select_type_statement_token1] = ACTIONS(1147),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1147),
    [aux_sym_block_construct_token1] = ACTIONS(1147),
    [aux_sym_associate_statement_token1] = ACTIONS(1147),
    [aux_sym_format_statement_token1] = ACTIONS(1147),
    [aux_sym_print_statement_token1] = ACTIONS(1147),
    [aux_sym_open_statement_token1] = ACTIONS(1147),
    [aux_sym_close_statement_token1] = ACTIONS(1147),
    [aux_sym_inquire_statement_token1] = ACTIONS(1147),
    [aux_sym_enum_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token2] = ACTIONS(1147),
    [aux_sym_file_position_statement_token3] = ACTIONS(1147),
    [aux_sym_file_position_statement_token4] = ACTIONS(1147),
    [aux_sym_logical_expression_token5] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1145),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [sym_boolean_literal] = ACTIONS(1145),
    [aux_sym_null_literal_token1] = ACTIONS(1147),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1147),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1145),
    [sym__float_literal] = ACTIONS(1145),
    [sym__boz_literal] = ACTIONS(1145),
    [sym_string_literal] = ACTIONS(1145),
  },
  [246] = {
    [sym_preproc_file_line] = STATE(246),
    [ts_builtin_sym_end] = ACTIONS(1149),
    [aux_sym_program_statement_token1] = ACTIONS(1151),
    [aux_sym_end_program_statement_token1] = ACTIONS(1151),
    [aux_sym_end_program_statement_token2] = ACTIONS(1151),
    [aux_sym_module_statement_token1] = ACTIONS(1151),
    [aux_sym_submodule_statement_token1] = ACTIONS(1151),
    [anon_sym_LPAREN] = ACTIONS(1151),
    [aux_sym_interface_statement_token1] = ACTIONS(1151),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1151),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1151),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1151),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1151),
    [aux_sym_function_statement_token1] = ACTIONS(1151),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1151),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1151),
    [aux_sym_contains_statement_token1] = ACTIONS(1151),
    [aux_sym_use_statement_token1] = ACTIONS(1151),
    [aux_sym_use_statement_token2] = ACTIONS(1151),
    [aux_sym_implicit_statement_token1] = ACTIONS(1151),
    [aux_sym_implicit_statement_token3] = ACTIONS(1151),
    [aux_sym_implicit_statement_token4] = ACTIONS(1151),
    [aux_sym_save_statement_token1] = ACTIONS(1151),
    [aux_sym_private_statement_token1] = ACTIONS(1151),
    [aux_sym_public_statement_token1] = ACTIONS(1151),
    [aux_sym_namelist_statement_token1] = ACTIONS(1151),
    [aux_sym_common_statement_token1] = ACTIONS(1151),
    [anon_sym_DASH] = ACTIONS(1149),
    [aux_sym_import_statement_token1] = ACTIONS(1151),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1151),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1151),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1151),
    [aux_sym_variable_attributes_token1] = ACTIONS(1151),
    [aux_sym_variable_attributes_token2] = ACTIONS(1151),
    [aux_sym_variable_attributes_token3] = ACTIONS(1151),
    [aux_sym_variable_attributes_token4] = ACTIONS(1151),
    [aux_sym_variable_attributes_token5] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1151),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1151),
    [aux_sym_derived_type_token1] = ACTIONS(1151),
    [aux_sym_type_qualifier_token1] = ACTIONS(1151),
    [aux_sym_type_qualifier_token2] = ACTIONS(1151),
    [aux_sym_type_qualifier_token3] = ACTIONS(1151),
    [aux_sym_type_qualifier_token4] = ACTIONS(1151),
    [aux_sym_type_qualifier_token5] = ACTIONS(1151),
    [aux_sym_type_qualifier_token9] = ACTIONS(1151),
    [aux_sym_type_qualifier_token10] = ACTIONS(1151),
    [aux_sym_type_qualifier_token11] = ACTIONS(1151),
    [aux_sym_type_qualifier_token12] = ACTIONS(1151),
    [aux_sym_type_qualifier_token13] = ACTIONS(1151),
    [aux_sym_type_qualifier_token14] = ACTIONS(1151),
    [aux_sym_type_qualifier_token15] = ACTIONS(1151),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1151),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1151),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1151),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1151),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1151),
    [aux_sym_stop_statement_token1] = ACTIONS(1151),
    [aux_sym_stop_statement_token2] = ACTIONS(1151),
    [aux_sym_subroutine_call_token1] = ACTIONS(1151),
    [aux_sym_keyword_statement_token1] = ACTIONS(1151),
    [aux_sym_keyword_statement_token2] = ACTIONS(1151),
    [aux_sym_keyword_statement_token3] = ACTIONS(1151),
    [aux_sym_keyword_statement_token4] = ACTIONS(1151),
    [aux_sym_keyword_statement_token6] = ACTIONS(1151),
    [aux_sym_keyword_statement_token7] = ACTIONS(1151),
    [aux_sym_include_statement_token1] = ACTIONS(1151),
    [aux_sym_data_statement_token1] = ACTIONS(1151),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1151),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1151),
    [aux_sym_end_if_statement_token1] = ACTIONS(1151),
    [aux_sym_elseif_clause_token2] = ACTIONS(1151),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1151),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1151),
    [aux_sym_select_case_statement_token1] = ACTIONS(1151),
    [aux_sym_select_case_statement_token3] = ACTIONS(1151),
    [aux_sym_select_type_statement_token1] = ACTIONS(1151),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1151),
    [aux_sym_block_construct_token1] = ACTIONS(1151),
    [aux_sym_associate_statement_token1] = ACTIONS(1151),
    [aux_sym_format_statement_token1] = ACTIONS(1151),
    [aux_sym_print_statement_token1] = ACTIONS(1151),
    [aux_sym_open_statement_token1] = ACTIONS(1151),
    [aux_sym_close_statement_token1] = ACTIONS(1151),
    [aux_sym_inquire_statement_token1] = ACTIONS(1151),
    [aux_sym_enum_statement_token1] = ACTIONS(1151),
    [aux_sym_file_position_statement_token1] = ACTIONS(1151),
    [aux_sym_file_position_statement_token2] = ACTIONS(1151),
    [aux_sym_file_position_statement_token3] = ACTIONS(1151),
    [aux_sym_file_position_statement_token4] = ACTIONS(1151),
    [aux_sym_logical_expression_token5] = ACTIONS(1149),
    [anon_sym_PLUS] = ACTIONS(1149),
    [anon_sym_DOT] = ACTIONS(1151),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1149),
    [anon_sym_LBRACK] = ACTIONS(1149),
    [sym_boolean_literal] = ACTIONS(1149),
    [aux_sym_null_literal_token1] = ACTIONS(1151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1151),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1149),
    [sym__float_literal] = ACTIONS(1149),
    [sym__boz_literal] = ACTIONS(1149),
    [sym_string_literal] = ACTIONS(1149),
  },
  [247] = {
    [sym_preproc_file_line] = STATE(247),
    [ts_builtin_sym_end] = ACTIONS(1153),
    [aux_sym_program_statement_token1] = ACTIONS(1155),
    [aux_sym_end_program_statement_token1] = ACTIONS(1155),
    [aux_sym_end_program_statement_token2] = ACTIONS(1155),
    [aux_sym_module_statement_token1] = ACTIONS(1155),
    [aux_sym_submodule_statement_token1] = ACTIONS(1155),
    [anon_sym_LPAREN] = ACTIONS(1155),
    [aux_sym_interface_statement_token1] = ACTIONS(1155),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1155),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1155),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1155),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1155),
    [aux_sym_function_statement_token1] = ACTIONS(1155),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1155),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1155),
    [aux_sym_contains_statement_token1] = ACTIONS(1155),
    [aux_sym_use_statement_token1] = ACTIONS(1155),
    [aux_sym_use_statement_token2] = ACTIONS(1155),
    [aux_sym_implicit_statement_token1] = ACTIONS(1155),
    [aux_sym_implicit_statement_token3] = ACTIONS(1155),
    [aux_sym_implicit_statement_token4] = ACTIONS(1155),
    [aux_sym_save_statement_token1] = ACTIONS(1155),
    [aux_sym_private_statement_token1] = ACTIONS(1155),
    [aux_sym_public_statement_token1] = ACTIONS(1155),
    [aux_sym_namelist_statement_token1] = ACTIONS(1155),
    [aux_sym_common_statement_token1] = ACTIONS(1155),
    [anon_sym_DASH] = ACTIONS(1153),
    [aux_sym_import_statement_token1] = ACTIONS(1155),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1155),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1155),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1155),
    [aux_sym_variable_attributes_token1] = ACTIONS(1155),
    [aux_sym_variable_attributes_token2] = ACTIONS(1155),
    [aux_sym_variable_attributes_token3] = ACTIONS(1155),
    [aux_sym_variable_attributes_token4] = ACTIONS(1155),
    [aux_sym_variable_attributes_token5] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1155),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1155),
    [aux_sym_derived_type_token1] = ACTIONS(1155),
    [aux_sym_type_qualifier_token1] = ACTIONS(1155),
    [aux_sym_type_qualifier_token2] = ACTIONS(1155),
    [aux_sym_type_qualifier_token3] = ACTIONS(1155),
    [aux_sym_type_qualifier_token4] = ACTIONS(1155),
    [aux_sym_type_qualifier_token5] = ACTIONS(1155),
    [aux_sym_type_qualifier_token9] = ACTIONS(1155),
    [aux_sym_type_qualifier_token10] = ACTIONS(1155),
    [aux_sym_type_qualifier_token11] = ACTIONS(1155),
    [aux_sym_type_qualifier_token12] = ACTIONS(1155),
    [aux_sym_type_qualifier_token13] = ACTIONS(1155),
    [aux_sym_type_qualifier_token14] = ACTIONS(1155),
    [aux_sym_type_qualifier_token15] = ACTIONS(1155),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1155),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1155),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1155),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1155),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1155),
    [aux_sym_stop_statement_token1] = ACTIONS(1155),
    [aux_sym_stop_statement_token2] = ACTIONS(1155),
    [aux_sym_subroutine_call_token1] = ACTIONS(1155),
    [aux_sym_keyword_statement_token1] = ACTIONS(1155),
    [aux_sym_keyword_statement_token2] = ACTIONS(1155),
    [aux_sym_keyword_statement_token3] = ACTIONS(1155),
    [aux_sym_keyword_statement_token4] = ACTIONS(1155),
    [aux_sym_keyword_statement_token6] = ACTIONS(1155),
    [aux_sym_keyword_statement_token7] = ACTIONS(1155),
    [aux_sym_include_statement_token1] = ACTIONS(1155),
    [aux_sym_data_statement_token1] = ACTIONS(1155),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1155),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1155),
    [aux_sym_end_if_statement_token1] = ACTIONS(1155),
    [aux_sym_elseif_clause_token2] = ACTIONS(1155),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1155),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1155),
    [aux_sym_select_case_statement_token1] = ACTIONS(1155),
    [aux_sym_select_case_statement_token3] = ACTIONS(1155),
    [aux_sym_select_type_statement_token1] = ACTIONS(1155),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1155),
    [aux_sym_block_construct_token1] = ACTIONS(1155),
    [aux_sym_associate_statement_token1] = ACTIONS(1155),
    [aux_sym_format_statement_token1] = ACTIONS(1155),
    [aux_sym_print_statement_token1] = ACTIONS(1155),
    [aux_sym_open_statement_token1] = ACTIONS(1155),
    [aux_sym_close_statement_token1] = ACTIONS(1155),
    [aux_sym_inquire_statement_token1] = ACTIONS(1155),
    [aux_sym_enum_statement_token1] = ACTIONS(1155),
    [aux_sym_file_position_statement_token1] = ACTIONS(1155),
    [aux_sym_file_position_statement_token2] = ACTIONS(1155),
    [aux_sym_file_position_statement_token3] = ACTIONS(1155),
    [aux_sym_file_position_statement_token4] = ACTIONS(1155),
    [aux_sym_logical_expression_token5] = ACTIONS(1153),
    [anon_sym_PLUS] = ACTIONS(1153),
    [anon_sym_DOT] = ACTIONS(1155),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1153),
    [anon_sym_LBRACK] = ACTIONS(1153),
    [sym_boolean_literal] = ACTIONS(1153),
    [aux_sym_null_literal_token1] = ACTIONS(1155),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1155),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1153),
    [sym__float_literal] = ACTIONS(1153),
    [sym__boz_literal] = ACTIONS(1153),
    [sym_string_literal] = ACTIONS(1153),
  },
  [248] = {
    [sym_preproc_file_line] = STATE(248),
    [ts_builtin_sym_end] = ACTIONS(1157),
    [aux_sym_program_statement_token1] = ACTIONS(1159),
    [aux_sym_end_program_statement_token1] = ACTIONS(1159),
    [aux_sym_end_program_statement_token2] = ACTIONS(1159),
    [aux_sym_module_statement_token1] = ACTIONS(1159),
    [aux_sym_submodule_statement_token1] = ACTIONS(1159),
    [anon_sym_LPAREN] = ACTIONS(1159),
    [aux_sym_interface_statement_token1] = ACTIONS(1159),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1159),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1159),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1159),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1159),
    [aux_sym_function_statement_token1] = ACTIONS(1159),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1159),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1159),
    [aux_sym_contains_statement_token1] = ACTIONS(1159),
    [aux_sym_use_statement_token1] = ACTIONS(1159),
    [aux_sym_use_statement_token2] = ACTIONS(1159),
    [aux_sym_implicit_statement_token1] = ACTIONS(1159),
    [aux_sym_implicit_statement_token3] = ACTIONS(1159),
    [aux_sym_implicit_statement_token4] = ACTIONS(1159),
    [aux_sym_save_statement_token1] = ACTIONS(1159),
    [aux_sym_private_statement_token1] = ACTIONS(1159),
    [aux_sym_public_statement_token1] = ACTIONS(1159),
    [aux_sym_namelist_statement_token1] = ACTIONS(1159),
    [aux_sym_common_statement_token1] = ACTIONS(1159),
    [anon_sym_DASH] = ACTIONS(1157),
    [aux_sym_import_statement_token1] = ACTIONS(1159),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1159),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1159),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1159),
    [aux_sym_variable_attributes_token1] = ACTIONS(1159),
    [aux_sym_variable_attributes_token2] = ACTIONS(1159),
    [aux_sym_variable_attributes_token3] = ACTIONS(1159),
    [aux_sym_variable_attributes_token4] = ACTIONS(1159),
    [aux_sym_variable_attributes_token5] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1159),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1159),
    [aux_sym_derived_type_token1] = ACTIONS(1159),
    [aux_sym_type_qualifier_token1] = ACTIONS(1159),
    [aux_sym_type_qualifier_token2] = ACTIONS(1159),
    [aux_sym_type_qualifier_token3] = ACTIONS(1159),
    [aux_sym_type_qualifier_token4] = ACTIONS(1159),
    [aux_sym_type_qualifier_token5] = ACTIONS(1159),
    [aux_sym_type_qualifier_token9] = ACTIONS(1159),
    [aux_sym_type_qualifier_token10] = ACTIONS(1159),
    [aux_sym_type_qualifier_token11] = ACTIONS(1159),
    [aux_sym_type_qualifier_token12] = ACTIONS(1159),
    [aux_sym_type_qualifier_token13] = ACTIONS(1159),
    [aux_sym_type_qualifier_token14] = ACTIONS(1159),
    [aux_sym_type_qualifier_token15] = ACTIONS(1159),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1159),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1159),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1159),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1159),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1159),
    [aux_sym_stop_statement_token1] = ACTIONS(1159),
    [aux_sym_stop_statement_token2] = ACTIONS(1159),
    [aux_sym_subroutine_call_token1] = ACTIONS(1159),
    [aux_sym_keyword_statement_token1] = ACTIONS(1159),
    [aux_sym_keyword_statement_token2] = ACTIONS(1159),
    [aux_sym_keyword_statement_token3] = ACTIONS(1159),
    [aux_sym_keyword_statement_token4] = ACTIONS(1159),
    [aux_sym_keyword_statement_token6] = ACTIONS(1159),
    [aux_sym_keyword_statement_token7] = ACTIONS(1159),
    [aux_sym_include_statement_token1] = ACTIONS(1159),
    [aux_sym_data_statement_token1] = ACTIONS(1159),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1159),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1159),
    [aux_sym_end_if_statement_token1] = ACTIONS(1159),
    [aux_sym_elseif_clause_token2] = ACTIONS(1159),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1159),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1159),
    [aux_sym_select_case_statement_token1] = ACTIONS(1159),
    [aux_sym_select_case_statement_token3] = ACTIONS(1159),
    [aux_sym_select_type_statement_token1] = ACTIONS(1159),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1159),
    [aux_sym_block_construct_token1] = ACTIONS(1159),
    [aux_sym_associate_statement_token1] = ACTIONS(1159),
    [aux_sym_format_statement_token1] = ACTIONS(1159),
    [aux_sym_print_statement_token1] = ACTIONS(1159),
    [aux_sym_open_statement_token1] = ACTIONS(1159),
    [aux_sym_close_statement_token1] = ACTIONS(1159),
    [aux_sym_inquire_statement_token1] = ACTIONS(1159),
    [aux_sym_enum_statement_token1] = ACTIONS(1159),
    [aux_sym_file_position_statement_token1] = ACTIONS(1159),
    [aux_sym_file_position_statement_token2] = ACTIONS(1159),
    [aux_sym_file_position_statement_token3] = ACTIONS(1159),
    [aux_sym_file_position_statement_token4] = ACTIONS(1159),
    [aux_sym_logical_expression_token5] = ACTIONS(1157),
    [anon_sym_PLUS] = ACTIONS(1157),
    [anon_sym_DOT] = ACTIONS(1159),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1157),
    [anon_sym_LBRACK] = ACTIONS(1157),
    [sym_boolean_literal] = ACTIONS(1157),
    [aux_sym_null_literal_token1] = ACTIONS(1159),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1159),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1157),
    [sym__float_literal] = ACTIONS(1157),
    [sym__boz_literal] = ACTIONS(1157),
    [sym_string_literal] = ACTIONS(1157),
  },
  [249] = {
    [sym_preproc_file_line] = STATE(249),
    [ts_builtin_sym_end] = ACTIONS(1161),
    [aux_sym_program_statement_token1] = ACTIONS(1163),
    [aux_sym_end_program_statement_token1] = ACTIONS(1163),
    [aux_sym_end_program_statement_token2] = ACTIONS(1163),
    [aux_sym_module_statement_token1] = ACTIONS(1163),
    [aux_sym_submodule_statement_token1] = ACTIONS(1163),
    [anon_sym_LPAREN] = ACTIONS(1163),
    [aux_sym_interface_statement_token1] = ACTIONS(1163),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1163),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1163),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1163),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1163),
    [aux_sym_function_statement_token1] = ACTIONS(1163),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1163),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1163),
    [aux_sym_contains_statement_token1] = ACTIONS(1163),
    [aux_sym_use_statement_token1] = ACTIONS(1163),
    [aux_sym_use_statement_token2] = ACTIONS(1163),
    [aux_sym_implicit_statement_token1] = ACTIONS(1163),
    [aux_sym_implicit_statement_token3] = ACTIONS(1163),
    [aux_sym_implicit_statement_token4] = ACTIONS(1163),
    [aux_sym_save_statement_token1] = ACTIONS(1163),
    [aux_sym_private_statement_token1] = ACTIONS(1163),
    [aux_sym_public_statement_token1] = ACTIONS(1163),
    [aux_sym_namelist_statement_token1] = ACTIONS(1163),
    [aux_sym_common_statement_token1] = ACTIONS(1163),
    [anon_sym_DASH] = ACTIONS(1161),
    [aux_sym_import_statement_token1] = ACTIONS(1163),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1163),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1163),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1163),
    [aux_sym_variable_attributes_token1] = ACTIONS(1163),
    [aux_sym_variable_attributes_token2] = ACTIONS(1163),
    [aux_sym_variable_attributes_token3] = ACTIONS(1163),
    [aux_sym_variable_attributes_token4] = ACTIONS(1163),
    [aux_sym_variable_attributes_token5] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1163),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1163),
    [aux_sym_derived_type_token1] = ACTIONS(1163),
    [aux_sym_type_qualifier_token1] = ACTIONS(1163),
    [aux_sym_type_qualifier_token2] = ACTIONS(1163),
    [aux_sym_type_qualifier_token3] = ACTIONS(1163),
    [aux_sym_type_qualifier_token4] = ACTIONS(1163),
    [aux_sym_type_qualifier_token5] = ACTIONS(1163),
    [aux_sym_type_qualifier_token9] = ACTIONS(1163),
    [aux_sym_type_qualifier_token10] = ACTIONS(1163),
    [aux_sym_type_qualifier_token11] = ACTIONS(1163),
    [aux_sym_type_qualifier_token12] = ACTIONS(1163),
    [aux_sym_type_qualifier_token13] = ACTIONS(1163),
    [aux_sym_type_qualifier_token14] = ACTIONS(1163),
    [aux_sym_type_qualifier_token15] = ACTIONS(1163),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1163),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1163),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1163),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1163),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1163),
    [aux_sym_stop_statement_token1] = ACTIONS(1163),
    [aux_sym_stop_statement_token2] = ACTIONS(1163),
    [aux_sym_subroutine_call_token1] = ACTIONS(1163),
    [aux_sym_keyword_statement_token1] = ACTIONS(1163),
    [aux_sym_keyword_statement_token2] = ACTIONS(1163),
    [aux_sym_keyword_statement_token3] = ACTIONS(1163),
    [aux_sym_keyword_statement_token4] = ACTIONS(1163),
    [aux_sym_keyword_statement_token6] = ACTIONS(1163),
    [aux_sym_keyword_statement_token7] = ACTIONS(1163),
    [aux_sym_include_statement_token1] = ACTIONS(1163),
    [aux_sym_data_statement_token1] = ACTIONS(1163),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1163),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1163),
    [aux_sym_end_if_statement_token1] = ACTIONS(1163),
    [aux_sym_elseif_clause_token2] = ACTIONS(1163),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1163),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1163),
    [aux_sym_select_case_statement_token1] = ACTIONS(1163),
    [aux_sym_select_case_statement_token3] = ACTIONS(1163),
    [aux_sym_select_type_statement_token1] = ACTIONS(1163),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1163),
    [aux_sym_block_construct_token1] = ACTIONS(1163),
    [aux_sym_associate_statement_token1] = ACTIONS(1163),
    [aux_sym_format_statement_token1] = ACTIONS(1163),
    [aux_sym_print_statement_token1] = ACTIONS(1163),
    [aux_sym_open_statement_token1] = ACTIONS(1163),
    [aux_sym_close_statement_token1] = ACTIONS(1163),
    [aux_sym_inquire_statement_token1] = ACTIONS(1163),
    [aux_sym_enum_statement_token1] = ACTIONS(1163),
    [aux_sym_file_position_statement_token1] = ACTIONS(1163),
    [aux_sym_file_position_statement_token2] = ACTIONS(1163),
    [aux_sym_file_position_statement_token3] = ACTIONS(1163),
    [aux_sym_file_position_statement_token4] = ACTIONS(1163),
    [aux_sym_logical_expression_token5] = ACTIONS(1161),
    [anon_sym_PLUS] = ACTIONS(1161),
    [anon_sym_DOT] = ACTIONS(1163),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1161),
    [anon_sym_LBRACK] = ACTIONS(1161),
    [sym_boolean_literal] = ACTIONS(1161),
    [aux_sym_null_literal_token1] = ACTIONS(1163),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1163),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1161),
    [sym__float_literal] = ACTIONS(1161),
    [sym__boz_literal] = ACTIONS(1161),
    [sym_string_literal] = ACTIONS(1161),
  },
  [250] = {
    [sym_preproc_file_line] = STATE(250),
    [ts_builtin_sym_end] = ACTIONS(1165),
    [aux_sym_program_statement_token1] = ACTIONS(1167),
    [aux_sym_end_program_statement_token1] = ACTIONS(1167),
    [aux_sym_end_program_statement_token2] = ACTIONS(1167),
    [aux_sym_module_statement_token1] = ACTIONS(1167),
    [aux_sym_submodule_statement_token1] = ACTIONS(1167),
    [anon_sym_LPAREN] = ACTIONS(1167),
    [aux_sym_interface_statement_token1] = ACTIONS(1167),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1167),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1167),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1167),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1167),
    [aux_sym_function_statement_token1] = ACTIONS(1167),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1167),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1167),
    [aux_sym_contains_statement_token1] = ACTIONS(1167),
    [aux_sym_use_statement_token1] = ACTIONS(1167),
    [aux_sym_use_statement_token2] = ACTIONS(1167),
    [aux_sym_implicit_statement_token1] = ACTIONS(1167),
    [aux_sym_implicit_statement_token3] = ACTIONS(1167),
    [aux_sym_implicit_statement_token4] = ACTIONS(1167),
    [aux_sym_save_statement_token1] = ACTIONS(1167),
    [aux_sym_private_statement_token1] = ACTIONS(1167),
    [aux_sym_public_statement_token1] = ACTIONS(1167),
    [aux_sym_namelist_statement_token1] = ACTIONS(1167),
    [aux_sym_common_statement_token1] = ACTIONS(1167),
    [anon_sym_DASH] = ACTIONS(1165),
    [aux_sym_import_statement_token1] = ACTIONS(1167),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1167),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1167),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1167),
    [aux_sym_variable_attributes_token1] = ACTIONS(1167),
    [aux_sym_variable_attributes_token2] = ACTIONS(1167),
    [aux_sym_variable_attributes_token3] = ACTIONS(1167),
    [aux_sym_variable_attributes_token4] = ACTIONS(1167),
    [aux_sym_variable_attributes_token5] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1167),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1167),
    [aux_sym_derived_type_token1] = ACTIONS(1167),
    [aux_sym_type_qualifier_token1] = ACTIONS(1167),
    [aux_sym_type_qualifier_token2] = ACTIONS(1167),
    [aux_sym_type_qualifier_token3] = ACTIONS(1167),
    [aux_sym_type_qualifier_token4] = ACTIONS(1167),
    [aux_sym_type_qualifier_token5] = ACTIONS(1167),
    [aux_sym_type_qualifier_token9] = ACTIONS(1167),
    [aux_sym_type_qualifier_token10] = ACTIONS(1167),
    [aux_sym_type_qualifier_token11] = ACTIONS(1167),
    [aux_sym_type_qualifier_token12] = ACTIONS(1167),
    [aux_sym_type_qualifier_token13] = ACTIONS(1167),
    [aux_sym_type_qualifier_token14] = ACTIONS(1167),
    [aux_sym_type_qualifier_token15] = ACTIONS(1167),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1167),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1167),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1167),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1167),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1167),
    [aux_sym_stop_statement_token1] = ACTIONS(1167),
    [aux_sym_stop_statement_token2] = ACTIONS(1167),
    [aux_sym_subroutine_call_token1] = ACTIONS(1167),
    [aux_sym_keyword_statement_token1] = ACTIONS(1167),
    [aux_sym_keyword_statement_token2] = ACTIONS(1167),
    [aux_sym_keyword_statement_token3] = ACTIONS(1167),
    [aux_sym_keyword_statement_token4] = ACTIONS(1167),
    [aux_sym_keyword_statement_token6] = ACTIONS(1167),
    [aux_sym_keyword_statement_token7] = ACTIONS(1167),
    [aux_sym_include_statement_token1] = ACTIONS(1167),
    [aux_sym_data_statement_token1] = ACTIONS(1167),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1167),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1167),
    [aux_sym_end_if_statement_token1] = ACTIONS(1167),
    [aux_sym_elseif_clause_token2] = ACTIONS(1167),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1167),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1167),
    [aux_sym_select_case_statement_token1] = ACTIONS(1167),
    [aux_sym_select_case_statement_token3] = ACTIONS(1167),
    [aux_sym_select_type_statement_token1] = ACTIONS(1167),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1167),
    [aux_sym_block_construct_token1] = ACTIONS(1167),
    [aux_sym_associate_statement_token1] = ACTIONS(1167),
    [aux_sym_format_statement_token1] = ACTIONS(1167),
    [aux_sym_print_statement_token1] = ACTIONS(1167),
    [aux_sym_open_statement_token1] = ACTIONS(1167),
    [aux_sym_close_statement_token1] = ACTIONS(1167),
    [aux_sym_inquire_statement_token1] = ACTIONS(1167),
    [aux_sym_enum_statement_token1] = ACTIONS(1167),
    [aux_sym_file_position_statement_token1] = ACTIONS(1167),
    [aux_sym_file_position_statement_token2] = ACTIONS(1167),
    [aux_sym_file_position_statement_token3] = ACTIONS(1167),
    [aux_sym_file_position_statement_token4] = ACTIONS(1167),
    [aux_sym_logical_expression_token5] = ACTIONS(1165),
    [anon_sym_PLUS] = ACTIONS(1165),
    [anon_sym_DOT] = ACTIONS(1167),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1165),
    [anon_sym_LBRACK] = ACTIONS(1165),
    [sym_boolean_literal] = ACTIONS(1165),
    [aux_sym_null_literal_token1] = ACTIONS(1167),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1167),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1165),
    [sym__float_literal] = ACTIONS(1165),
    [sym__boz_literal] = ACTIONS(1165),
    [sym_string_literal] = ACTIONS(1165),
  },
  [251] = {
    [sym_preproc_file_line] = STATE(251),
    [ts_builtin_sym_end] = ACTIONS(1169),
    [aux_sym_program_statement_token1] = ACTIONS(1171),
    [aux_sym_end_program_statement_token1] = ACTIONS(1171),
    [aux_sym_end_program_statement_token2] = ACTIONS(1171),
    [aux_sym_module_statement_token1] = ACTIONS(1171),
    [aux_sym_submodule_statement_token1] = ACTIONS(1171),
    [anon_sym_LPAREN] = ACTIONS(1171),
    [aux_sym_interface_statement_token1] = ACTIONS(1171),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1171),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1171),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1171),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1171),
    [aux_sym_function_statement_token1] = ACTIONS(1171),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1171),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1171),
    [aux_sym_contains_statement_token1] = ACTIONS(1171),
    [aux_sym_use_statement_token1] = ACTIONS(1171),
    [aux_sym_use_statement_token2] = ACTIONS(1171),
    [aux_sym_implicit_statement_token1] = ACTIONS(1171),
    [aux_sym_implicit_statement_token3] = ACTIONS(1171),
    [aux_sym_implicit_statement_token4] = ACTIONS(1171),
    [aux_sym_save_statement_token1] = ACTIONS(1171),
    [aux_sym_private_statement_token1] = ACTIONS(1171),
    [aux_sym_public_statement_token1] = ACTIONS(1171),
    [aux_sym_namelist_statement_token1] = ACTIONS(1171),
    [aux_sym_common_statement_token1] = ACTIONS(1171),
    [anon_sym_DASH] = ACTIONS(1169),
    [aux_sym_import_statement_token1] = ACTIONS(1171),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1171),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1171),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1171),
    [aux_sym_variable_attributes_token1] = ACTIONS(1171),
    [aux_sym_variable_attributes_token2] = ACTIONS(1171),
    [aux_sym_variable_attributes_token3] = ACTIONS(1171),
    [aux_sym_variable_attributes_token4] = ACTIONS(1171),
    [aux_sym_variable_attributes_token5] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1171),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1171),
    [aux_sym_derived_type_token1] = ACTIONS(1171),
    [aux_sym_type_qualifier_token1] = ACTIONS(1171),
    [aux_sym_type_qualifier_token2] = ACTIONS(1171),
    [aux_sym_type_qualifier_token3] = ACTIONS(1171),
    [aux_sym_type_qualifier_token4] = ACTIONS(1171),
    [aux_sym_type_qualifier_token5] = ACTIONS(1171),
    [aux_sym_type_qualifier_token9] = ACTIONS(1171),
    [aux_sym_type_qualifier_token10] = ACTIONS(1171),
    [aux_sym_type_qualifier_token11] = ACTIONS(1171),
    [aux_sym_type_qualifier_token12] = ACTIONS(1171),
    [aux_sym_type_qualifier_token13] = ACTIONS(1171),
    [aux_sym_type_qualifier_token14] = ACTIONS(1171),
    [aux_sym_type_qualifier_token15] = ACTIONS(1171),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1171),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1171),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1171),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1171),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1171),
    [aux_sym_stop_statement_token1] = ACTIONS(1171),
    [aux_sym_stop_statement_token2] = ACTIONS(1171),
    [aux_sym_subroutine_call_token1] = ACTIONS(1171),
    [aux_sym_keyword_statement_token1] = ACTIONS(1171),
    [aux_sym_keyword_statement_token2] = ACTIONS(1171),
    [aux_sym_keyword_statement_token3] = ACTIONS(1171),
    [aux_sym_keyword_statement_token4] = ACTIONS(1171),
    [aux_sym_keyword_statement_token6] = ACTIONS(1171),
    [aux_sym_keyword_statement_token7] = ACTIONS(1171),
    [aux_sym_include_statement_token1] = ACTIONS(1171),
    [aux_sym_data_statement_token1] = ACTIONS(1171),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1171),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1171),
    [aux_sym_end_if_statement_token1] = ACTIONS(1171),
    [aux_sym_elseif_clause_token2] = ACTIONS(1171),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1171),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1171),
    [aux_sym_select_case_statement_token1] = ACTIONS(1171),
    [aux_sym_select_case_statement_token3] = ACTIONS(1171),
    [aux_sym_select_type_statement_token1] = ACTIONS(1171),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1171),
    [aux_sym_block_construct_token1] = ACTIONS(1171),
    [aux_sym_associate_statement_token1] = ACTIONS(1171),
    [aux_sym_format_statement_token1] = ACTIONS(1171),
    [aux_sym_print_statement_token1] = ACTIONS(1171),
    [aux_sym_open_statement_token1] = ACTIONS(1171),
    [aux_sym_close_statement_token1] = ACTIONS(1171),
    [aux_sym_inquire_statement_token1] = ACTIONS(1171),
    [aux_sym_enum_statement_token1] = ACTIONS(1171),
    [aux_sym_file_position_statement_token1] = ACTIONS(1171),
    [aux_sym_file_position_statement_token2] = ACTIONS(1171),
    [aux_sym_file_position_statement_token3] = ACTIONS(1171),
    [aux_sym_file_position_statement_token4] = ACTIONS(1171),
    [aux_sym_logical_expression_token5] = ACTIONS(1169),
    [anon_sym_PLUS] = ACTIONS(1169),
    [anon_sym_DOT] = ACTIONS(1171),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1169),
    [anon_sym_LBRACK] = ACTIONS(1169),
    [sym_boolean_literal] = ACTIONS(1169),
    [aux_sym_null_literal_token1] = ACTIONS(1171),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1171),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1169),
    [sym__float_literal] = ACTIONS(1169),
    [sym__boz_literal] = ACTIONS(1169),
    [sym_string_literal] = ACTIONS(1169),
  },
  [252] = {
    [sym__statements] = STATE(2774),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(252),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [253] = {
    [sym__statements] = STATE(2772),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(253),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [254] = {
    [sym_preproc_file_line] = STATE(254),
    [ts_builtin_sym_end] = ACTIONS(1173),
    [aux_sym_program_statement_token1] = ACTIONS(1175),
    [aux_sym_end_program_statement_token1] = ACTIONS(1175),
    [aux_sym_end_program_statement_token2] = ACTIONS(1175),
    [aux_sym_module_statement_token1] = ACTIONS(1175),
    [aux_sym_submodule_statement_token1] = ACTIONS(1175),
    [anon_sym_LPAREN] = ACTIONS(1175),
    [aux_sym_interface_statement_token1] = ACTIONS(1175),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1175),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1175),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1175),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1175),
    [aux_sym_function_statement_token1] = ACTIONS(1175),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1175),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1175),
    [aux_sym_contains_statement_token1] = ACTIONS(1175),
    [aux_sym_use_statement_token1] = ACTIONS(1175),
    [aux_sym_use_statement_token2] = ACTIONS(1175),
    [aux_sym_implicit_statement_token1] = ACTIONS(1175),
    [aux_sym_implicit_statement_token3] = ACTIONS(1175),
    [aux_sym_implicit_statement_token4] = ACTIONS(1175),
    [aux_sym_save_statement_token1] = ACTIONS(1175),
    [aux_sym_private_statement_token1] = ACTIONS(1175),
    [aux_sym_public_statement_token1] = ACTIONS(1175),
    [aux_sym_namelist_statement_token1] = ACTIONS(1175),
    [aux_sym_common_statement_token1] = ACTIONS(1175),
    [anon_sym_DASH] = ACTIONS(1173),
    [aux_sym_import_statement_token1] = ACTIONS(1175),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1175),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1175),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1175),
    [aux_sym_variable_attributes_token1] = ACTIONS(1175),
    [aux_sym_variable_attributes_token2] = ACTIONS(1175),
    [aux_sym_variable_attributes_token3] = ACTIONS(1175),
    [aux_sym_variable_attributes_token4] = ACTIONS(1175),
    [aux_sym_variable_attributes_token5] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1175),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1175),
    [aux_sym_derived_type_token1] = ACTIONS(1175),
    [aux_sym_type_qualifier_token1] = ACTIONS(1175),
    [aux_sym_type_qualifier_token2] = ACTIONS(1175),
    [aux_sym_type_qualifier_token3] = ACTIONS(1175),
    [aux_sym_type_qualifier_token4] = ACTIONS(1175),
    [aux_sym_type_qualifier_token5] = ACTIONS(1175),
    [aux_sym_type_qualifier_token9] = ACTIONS(1175),
    [aux_sym_type_qualifier_token10] = ACTIONS(1175),
    [aux_sym_type_qualifier_token11] = ACTIONS(1175),
    [aux_sym_type_qualifier_token12] = ACTIONS(1175),
    [aux_sym_type_qualifier_token13] = ACTIONS(1175),
    [aux_sym_type_qualifier_token14] = ACTIONS(1175),
    [aux_sym_type_qualifier_token15] = ACTIONS(1175),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1175),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1175),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1175),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1175),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1175),
    [aux_sym_stop_statement_token1] = ACTIONS(1175),
    [aux_sym_stop_statement_token2] = ACTIONS(1175),
    [aux_sym_subroutine_call_token1] = ACTIONS(1175),
    [aux_sym_keyword_statement_token1] = ACTIONS(1175),
    [aux_sym_keyword_statement_token2] = ACTIONS(1175),
    [aux_sym_keyword_statement_token3] = ACTIONS(1175),
    [aux_sym_keyword_statement_token4] = ACTIONS(1175),
    [aux_sym_keyword_statement_token6] = ACTIONS(1175),
    [aux_sym_keyword_statement_token7] = ACTIONS(1175),
    [aux_sym_include_statement_token1] = ACTIONS(1175),
    [aux_sym_data_statement_token1] = ACTIONS(1175),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1175),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1175),
    [aux_sym_end_if_statement_token1] = ACTIONS(1175),
    [aux_sym_elseif_clause_token2] = ACTIONS(1175),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1175),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1175),
    [aux_sym_select_case_statement_token1] = ACTIONS(1175),
    [aux_sym_select_case_statement_token3] = ACTIONS(1175),
    [aux_sym_select_type_statement_token1] = ACTIONS(1175),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1175),
    [aux_sym_block_construct_token1] = ACTIONS(1175),
    [aux_sym_associate_statement_token1] = ACTIONS(1175),
    [aux_sym_format_statement_token1] = ACTIONS(1175),
    [aux_sym_print_statement_token1] = ACTIONS(1175),
    [aux_sym_open_statement_token1] = ACTIONS(1175),
    [aux_sym_close_statement_token1] = ACTIONS(1175),
    [aux_sym_inquire_statement_token1] = ACTIONS(1175),
    [aux_sym_enum_statement_token1] = ACTIONS(1175),
    [aux_sym_file_position_statement_token1] = ACTIONS(1175),
    [aux_sym_file_position_statement_token2] = ACTIONS(1175),
    [aux_sym_file_position_statement_token3] = ACTIONS(1175),
    [aux_sym_file_position_statement_token4] = ACTIONS(1175),
    [aux_sym_logical_expression_token5] = ACTIONS(1173),
    [anon_sym_PLUS] = ACTIONS(1173),
    [anon_sym_DOT] = ACTIONS(1175),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1173),
    [anon_sym_LBRACK] = ACTIONS(1173),
    [sym_boolean_literal] = ACTIONS(1173),
    [aux_sym_null_literal_token1] = ACTIONS(1175),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1175),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1173),
    [sym__float_literal] = ACTIONS(1173),
    [sym__boz_literal] = ACTIONS(1173),
    [sym_string_literal] = ACTIONS(1173),
  },
  [255] = {
    [sym__statements] = STATE(2741),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(255),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [256] = {
    [sym__statements] = STATE(2770),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(256),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [257] = {
    [sym_preproc_file_line] = STATE(257),
    [ts_builtin_sym_end] = ACTIONS(1177),
    [aux_sym_program_statement_token1] = ACTIONS(1179),
    [aux_sym_end_program_statement_token1] = ACTIONS(1179),
    [aux_sym_end_program_statement_token2] = ACTIONS(1179),
    [aux_sym_module_statement_token1] = ACTIONS(1179),
    [aux_sym_submodule_statement_token1] = ACTIONS(1179),
    [anon_sym_LPAREN] = ACTIONS(1179),
    [aux_sym_interface_statement_token1] = ACTIONS(1179),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1179),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1179),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1179),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1179),
    [aux_sym_function_statement_token1] = ACTIONS(1179),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1179),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1179),
    [aux_sym_contains_statement_token1] = ACTIONS(1179),
    [aux_sym_use_statement_token1] = ACTIONS(1179),
    [aux_sym_use_statement_token2] = ACTIONS(1179),
    [aux_sym_implicit_statement_token1] = ACTIONS(1179),
    [aux_sym_implicit_statement_token3] = ACTIONS(1179),
    [aux_sym_implicit_statement_token4] = ACTIONS(1179),
    [aux_sym_save_statement_token1] = ACTIONS(1179),
    [aux_sym_private_statement_token1] = ACTIONS(1179),
    [aux_sym_public_statement_token1] = ACTIONS(1179),
    [aux_sym_namelist_statement_token1] = ACTIONS(1179),
    [aux_sym_common_statement_token1] = ACTIONS(1179),
    [anon_sym_DASH] = ACTIONS(1177),
    [aux_sym_import_statement_token1] = ACTIONS(1179),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1179),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1179),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1179),
    [aux_sym_variable_attributes_token1] = ACTIONS(1179),
    [aux_sym_variable_attributes_token2] = ACTIONS(1179),
    [aux_sym_variable_attributes_token3] = ACTIONS(1179),
    [aux_sym_variable_attributes_token4] = ACTIONS(1179),
    [aux_sym_variable_attributes_token5] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1179),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1179),
    [aux_sym_derived_type_token1] = ACTIONS(1179),
    [aux_sym_type_qualifier_token1] = ACTIONS(1179),
    [aux_sym_type_qualifier_token2] = ACTIONS(1179),
    [aux_sym_type_qualifier_token3] = ACTIONS(1179),
    [aux_sym_type_qualifier_token4] = ACTIONS(1179),
    [aux_sym_type_qualifier_token5] = ACTIONS(1179),
    [aux_sym_type_qualifier_token9] = ACTIONS(1179),
    [aux_sym_type_qualifier_token10] = ACTIONS(1179),
    [aux_sym_type_qualifier_token11] = ACTIONS(1179),
    [aux_sym_type_qualifier_token12] = ACTIONS(1179),
    [aux_sym_type_qualifier_token13] = ACTIONS(1179),
    [aux_sym_type_qualifier_token14] = ACTIONS(1179),
    [aux_sym_type_qualifier_token15] = ACTIONS(1179),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1179),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1179),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1179),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1179),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1179),
    [aux_sym_stop_statement_token1] = ACTIONS(1179),
    [aux_sym_stop_statement_token2] = ACTIONS(1179),
    [aux_sym_subroutine_call_token1] = ACTIONS(1179),
    [aux_sym_keyword_statement_token1] = ACTIONS(1179),
    [aux_sym_keyword_statement_token2] = ACTIONS(1179),
    [aux_sym_keyword_statement_token3] = ACTIONS(1179),
    [aux_sym_keyword_statement_token4] = ACTIONS(1179),
    [aux_sym_keyword_statement_token6] = ACTIONS(1179),
    [aux_sym_keyword_statement_token7] = ACTIONS(1179),
    [aux_sym_include_statement_token1] = ACTIONS(1179),
    [aux_sym_data_statement_token1] = ACTIONS(1179),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1179),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1179),
    [aux_sym_end_if_statement_token1] = ACTIONS(1179),
    [aux_sym_elseif_clause_token2] = ACTIONS(1179),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1179),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1179),
    [aux_sym_select_case_statement_token1] = ACTIONS(1179),
    [aux_sym_select_case_statement_token3] = ACTIONS(1179),
    [aux_sym_select_type_statement_token1] = ACTIONS(1179),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1179),
    [aux_sym_block_construct_token1] = ACTIONS(1179),
    [aux_sym_associate_statement_token1] = ACTIONS(1179),
    [aux_sym_format_statement_token1] = ACTIONS(1179),
    [aux_sym_print_statement_token1] = ACTIONS(1179),
    [aux_sym_open_statement_token1] = ACTIONS(1179),
    [aux_sym_close_statement_token1] = ACTIONS(1179),
    [aux_sym_inquire_statement_token1] = ACTIONS(1179),
    [aux_sym_enum_statement_token1] = ACTIONS(1179),
    [aux_sym_file_position_statement_token1] = ACTIONS(1179),
    [aux_sym_file_position_statement_token2] = ACTIONS(1179),
    [aux_sym_file_position_statement_token3] = ACTIONS(1179),
    [aux_sym_file_position_statement_token4] = ACTIONS(1179),
    [aux_sym_logical_expression_token5] = ACTIONS(1177),
    [anon_sym_PLUS] = ACTIONS(1177),
    [anon_sym_DOT] = ACTIONS(1179),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1177),
    [anon_sym_LBRACK] = ACTIONS(1177),
    [sym_boolean_literal] = ACTIONS(1177),
    [aux_sym_null_literal_token1] = ACTIONS(1179),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1179),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1177),
    [sym__float_literal] = ACTIONS(1177),
    [sym__boz_literal] = ACTIONS(1177),
    [sym_string_literal] = ACTIONS(1177),
  },
  [258] = {
    [sym_preproc_file_line] = STATE(258),
    [ts_builtin_sym_end] = ACTIONS(1181),
    [aux_sym_program_statement_token1] = ACTIONS(1183),
    [aux_sym_end_program_statement_token1] = ACTIONS(1183),
    [aux_sym_end_program_statement_token2] = ACTIONS(1183),
    [aux_sym_module_statement_token1] = ACTIONS(1183),
    [aux_sym_submodule_statement_token1] = ACTIONS(1183),
    [anon_sym_LPAREN] = ACTIONS(1183),
    [aux_sym_interface_statement_token1] = ACTIONS(1183),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1183),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1183),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1183),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1183),
    [aux_sym_function_statement_token1] = ACTIONS(1183),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1183),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1183),
    [aux_sym_contains_statement_token1] = ACTIONS(1183),
    [aux_sym_use_statement_token1] = ACTIONS(1183),
    [aux_sym_use_statement_token2] = ACTIONS(1183),
    [aux_sym_implicit_statement_token1] = ACTIONS(1183),
    [aux_sym_implicit_statement_token3] = ACTIONS(1183),
    [aux_sym_implicit_statement_token4] = ACTIONS(1183),
    [aux_sym_save_statement_token1] = ACTIONS(1183),
    [aux_sym_private_statement_token1] = ACTIONS(1183),
    [aux_sym_public_statement_token1] = ACTIONS(1183),
    [aux_sym_namelist_statement_token1] = ACTIONS(1183),
    [aux_sym_common_statement_token1] = ACTIONS(1183),
    [anon_sym_DASH] = ACTIONS(1181),
    [aux_sym_import_statement_token1] = ACTIONS(1183),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1183),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1183),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1183),
    [aux_sym_variable_attributes_token1] = ACTIONS(1183),
    [aux_sym_variable_attributes_token2] = ACTIONS(1183),
    [aux_sym_variable_attributes_token3] = ACTIONS(1183),
    [aux_sym_variable_attributes_token4] = ACTIONS(1183),
    [aux_sym_variable_attributes_token5] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1183),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1183),
    [aux_sym_derived_type_token1] = ACTIONS(1183),
    [aux_sym_type_qualifier_token1] = ACTIONS(1183),
    [aux_sym_type_qualifier_token2] = ACTIONS(1183),
    [aux_sym_type_qualifier_token3] = ACTIONS(1183),
    [aux_sym_type_qualifier_token4] = ACTIONS(1183),
    [aux_sym_type_qualifier_token5] = ACTIONS(1183),
    [aux_sym_type_qualifier_token9] = ACTIONS(1183),
    [aux_sym_type_qualifier_token10] = ACTIONS(1183),
    [aux_sym_type_qualifier_token11] = ACTIONS(1183),
    [aux_sym_type_qualifier_token12] = ACTIONS(1183),
    [aux_sym_type_qualifier_token13] = ACTIONS(1183),
    [aux_sym_type_qualifier_token14] = ACTIONS(1183),
    [aux_sym_type_qualifier_token15] = ACTIONS(1183),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1183),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1183),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1183),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1183),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1183),
    [aux_sym_stop_statement_token1] = ACTIONS(1183),
    [aux_sym_stop_statement_token2] = ACTIONS(1183),
    [aux_sym_subroutine_call_token1] = ACTIONS(1183),
    [aux_sym_keyword_statement_token1] = ACTIONS(1183),
    [aux_sym_keyword_statement_token2] = ACTIONS(1183),
    [aux_sym_keyword_statement_token3] = ACTIONS(1183),
    [aux_sym_keyword_statement_token4] = ACTIONS(1183),
    [aux_sym_keyword_statement_token6] = ACTIONS(1183),
    [aux_sym_keyword_statement_token7] = ACTIONS(1183),
    [aux_sym_include_statement_token1] = ACTIONS(1183),
    [aux_sym_data_statement_token1] = ACTIONS(1183),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1183),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1183),
    [aux_sym_end_if_statement_token1] = ACTIONS(1183),
    [aux_sym_elseif_clause_token2] = ACTIONS(1183),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1183),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1183),
    [aux_sym_select_case_statement_token1] = ACTIONS(1183),
    [aux_sym_select_case_statement_token3] = ACTIONS(1183),
    [aux_sym_select_type_statement_token1] = ACTIONS(1183),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1183),
    [aux_sym_block_construct_token1] = ACTIONS(1183),
    [aux_sym_associate_statement_token1] = ACTIONS(1183),
    [aux_sym_format_statement_token1] = ACTIONS(1183),
    [aux_sym_print_statement_token1] = ACTIONS(1183),
    [aux_sym_open_statement_token1] = ACTIONS(1183),
    [aux_sym_close_statement_token1] = ACTIONS(1183),
    [aux_sym_inquire_statement_token1] = ACTIONS(1183),
    [aux_sym_enum_statement_token1] = ACTIONS(1183),
    [aux_sym_file_position_statement_token1] = ACTIONS(1183),
    [aux_sym_file_position_statement_token2] = ACTIONS(1183),
    [aux_sym_file_position_statement_token3] = ACTIONS(1183),
    [aux_sym_file_position_statement_token4] = ACTIONS(1183),
    [aux_sym_logical_expression_token5] = ACTIONS(1181),
    [anon_sym_PLUS] = ACTIONS(1181),
    [anon_sym_DOT] = ACTIONS(1183),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1181),
    [anon_sym_LBRACK] = ACTIONS(1181),
    [sym_boolean_literal] = ACTIONS(1181),
    [aux_sym_null_literal_token1] = ACTIONS(1183),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1183),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1181),
    [sym__float_literal] = ACTIONS(1181),
    [sym__boz_literal] = ACTIONS(1181),
    [sym_string_literal] = ACTIONS(1181),
  },
  [259] = {
    [sym__statements] = STATE(2764),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(259),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [260] = {
    [sym__statements] = STATE(2694),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(260),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [261] = {
    [sym_preproc_file_line] = STATE(261),
    [ts_builtin_sym_end] = ACTIONS(1185),
    [aux_sym_program_statement_token1] = ACTIONS(1187),
    [aux_sym_end_program_statement_token1] = ACTIONS(1187),
    [aux_sym_end_program_statement_token2] = ACTIONS(1187),
    [aux_sym_module_statement_token1] = ACTIONS(1187),
    [aux_sym_submodule_statement_token1] = ACTIONS(1187),
    [anon_sym_LPAREN] = ACTIONS(1187),
    [aux_sym_interface_statement_token1] = ACTIONS(1187),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1187),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1187),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1187),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1187),
    [aux_sym_function_statement_token1] = ACTIONS(1187),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1187),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1187),
    [aux_sym_contains_statement_token1] = ACTIONS(1187),
    [aux_sym_use_statement_token1] = ACTIONS(1187),
    [aux_sym_use_statement_token2] = ACTIONS(1187),
    [aux_sym_implicit_statement_token1] = ACTIONS(1187),
    [aux_sym_implicit_statement_token3] = ACTIONS(1187),
    [aux_sym_implicit_statement_token4] = ACTIONS(1187),
    [aux_sym_save_statement_token1] = ACTIONS(1187),
    [aux_sym_private_statement_token1] = ACTIONS(1187),
    [aux_sym_public_statement_token1] = ACTIONS(1187),
    [aux_sym_namelist_statement_token1] = ACTIONS(1187),
    [aux_sym_common_statement_token1] = ACTIONS(1187),
    [anon_sym_DASH] = ACTIONS(1185),
    [aux_sym_import_statement_token1] = ACTIONS(1187),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1187),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1187),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1187),
    [aux_sym_variable_attributes_token1] = ACTIONS(1187),
    [aux_sym_variable_attributes_token2] = ACTIONS(1187),
    [aux_sym_variable_attributes_token3] = ACTIONS(1187),
    [aux_sym_variable_attributes_token4] = ACTIONS(1187),
    [aux_sym_variable_attributes_token5] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1187),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1187),
    [aux_sym_derived_type_token1] = ACTIONS(1187),
    [aux_sym_type_qualifier_token1] = ACTIONS(1187),
    [aux_sym_type_qualifier_token2] = ACTIONS(1187),
    [aux_sym_type_qualifier_token3] = ACTIONS(1187),
    [aux_sym_type_qualifier_token4] = ACTIONS(1187),
    [aux_sym_type_qualifier_token5] = ACTIONS(1187),
    [aux_sym_type_qualifier_token9] = ACTIONS(1187),
    [aux_sym_type_qualifier_token10] = ACTIONS(1187),
    [aux_sym_type_qualifier_token11] = ACTIONS(1187),
    [aux_sym_type_qualifier_token12] = ACTIONS(1187),
    [aux_sym_type_qualifier_token13] = ACTIONS(1187),
    [aux_sym_type_qualifier_token14] = ACTIONS(1187),
    [aux_sym_type_qualifier_token15] = ACTIONS(1187),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1187),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1187),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1187),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1187),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1187),
    [aux_sym_stop_statement_token1] = ACTIONS(1187),
    [aux_sym_stop_statement_token2] = ACTIONS(1187),
    [aux_sym_subroutine_call_token1] = ACTIONS(1187),
    [aux_sym_keyword_statement_token1] = ACTIONS(1187),
    [aux_sym_keyword_statement_token2] = ACTIONS(1187),
    [aux_sym_keyword_statement_token3] = ACTIONS(1187),
    [aux_sym_keyword_statement_token4] = ACTIONS(1187),
    [aux_sym_keyword_statement_token6] = ACTIONS(1187),
    [aux_sym_keyword_statement_token7] = ACTIONS(1187),
    [aux_sym_include_statement_token1] = ACTIONS(1187),
    [aux_sym_data_statement_token1] = ACTIONS(1187),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1187),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1187),
    [aux_sym_end_if_statement_token1] = ACTIONS(1187),
    [aux_sym_elseif_clause_token2] = ACTIONS(1187),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1187),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1187),
    [aux_sym_select_case_statement_token1] = ACTIONS(1187),
    [aux_sym_select_case_statement_token3] = ACTIONS(1187),
    [aux_sym_select_type_statement_token1] = ACTIONS(1187),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1187),
    [aux_sym_block_construct_token1] = ACTIONS(1187),
    [aux_sym_associate_statement_token1] = ACTIONS(1187),
    [aux_sym_format_statement_token1] = ACTIONS(1187),
    [aux_sym_print_statement_token1] = ACTIONS(1187),
    [aux_sym_open_statement_token1] = ACTIONS(1187),
    [aux_sym_close_statement_token1] = ACTIONS(1187),
    [aux_sym_inquire_statement_token1] = ACTIONS(1187),
    [aux_sym_enum_statement_token1] = ACTIONS(1187),
    [aux_sym_file_position_statement_token1] = ACTIONS(1187),
    [aux_sym_file_position_statement_token2] = ACTIONS(1187),
    [aux_sym_file_position_statement_token3] = ACTIONS(1187),
    [aux_sym_file_position_statement_token4] = ACTIONS(1187),
    [aux_sym_logical_expression_token5] = ACTIONS(1185),
    [anon_sym_PLUS] = ACTIONS(1185),
    [anon_sym_DOT] = ACTIONS(1187),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1185),
    [anon_sym_LBRACK] = ACTIONS(1185),
    [sym_boolean_literal] = ACTIONS(1185),
    [aux_sym_null_literal_token1] = ACTIONS(1187),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1187),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1185),
    [sym__float_literal] = ACTIONS(1185),
    [sym__boz_literal] = ACTIONS(1185),
    [sym_string_literal] = ACTIONS(1185),
  },
  [262] = {
    [sym__statements] = STATE(2768),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(262),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [263] = {
    [sym_preproc_file_line] = STATE(263),
    [ts_builtin_sym_end] = ACTIONS(1189),
    [aux_sym_program_statement_token1] = ACTIONS(1191),
    [aux_sym_end_program_statement_token1] = ACTIONS(1191),
    [aux_sym_end_program_statement_token2] = ACTIONS(1191),
    [aux_sym_module_statement_token1] = ACTIONS(1191),
    [aux_sym_submodule_statement_token1] = ACTIONS(1191),
    [anon_sym_LPAREN] = ACTIONS(1191),
    [aux_sym_interface_statement_token1] = ACTIONS(1191),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1191),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1191),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1191),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1191),
    [aux_sym_function_statement_token1] = ACTIONS(1191),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1191),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1191),
    [aux_sym_contains_statement_token1] = ACTIONS(1191),
    [aux_sym_use_statement_token1] = ACTIONS(1191),
    [aux_sym_use_statement_token2] = ACTIONS(1191),
    [aux_sym_implicit_statement_token1] = ACTIONS(1191),
    [aux_sym_implicit_statement_token3] = ACTIONS(1191),
    [aux_sym_implicit_statement_token4] = ACTIONS(1191),
    [aux_sym_save_statement_token1] = ACTIONS(1191),
    [aux_sym_private_statement_token1] = ACTIONS(1191),
    [aux_sym_public_statement_token1] = ACTIONS(1191),
    [aux_sym_namelist_statement_token1] = ACTIONS(1191),
    [aux_sym_common_statement_token1] = ACTIONS(1191),
    [anon_sym_DASH] = ACTIONS(1189),
    [aux_sym_import_statement_token1] = ACTIONS(1191),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1191),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1191),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1191),
    [aux_sym_variable_attributes_token1] = ACTIONS(1191),
    [aux_sym_variable_attributes_token2] = ACTIONS(1191),
    [aux_sym_variable_attributes_token3] = ACTIONS(1191),
    [aux_sym_variable_attributes_token4] = ACTIONS(1191),
    [aux_sym_variable_attributes_token5] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1191),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1191),
    [aux_sym_derived_type_token1] = ACTIONS(1191),
    [aux_sym_type_qualifier_token1] = ACTIONS(1191),
    [aux_sym_type_qualifier_token2] = ACTIONS(1191),
    [aux_sym_type_qualifier_token3] = ACTIONS(1191),
    [aux_sym_type_qualifier_token4] = ACTIONS(1191),
    [aux_sym_type_qualifier_token5] = ACTIONS(1191),
    [aux_sym_type_qualifier_token9] = ACTIONS(1191),
    [aux_sym_type_qualifier_token10] = ACTIONS(1191),
    [aux_sym_type_qualifier_token11] = ACTIONS(1191),
    [aux_sym_type_qualifier_token12] = ACTIONS(1191),
    [aux_sym_type_qualifier_token13] = ACTIONS(1191),
    [aux_sym_type_qualifier_token14] = ACTIONS(1191),
    [aux_sym_type_qualifier_token15] = ACTIONS(1191),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1191),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1191),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1191),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1191),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1191),
    [aux_sym_stop_statement_token1] = ACTIONS(1191),
    [aux_sym_stop_statement_token2] = ACTIONS(1191),
    [aux_sym_subroutine_call_token1] = ACTIONS(1191),
    [aux_sym_keyword_statement_token1] = ACTIONS(1191),
    [aux_sym_keyword_statement_token2] = ACTIONS(1191),
    [aux_sym_keyword_statement_token3] = ACTIONS(1191),
    [aux_sym_keyword_statement_token4] = ACTIONS(1191),
    [aux_sym_keyword_statement_token6] = ACTIONS(1191),
    [aux_sym_keyword_statement_token7] = ACTIONS(1191),
    [aux_sym_include_statement_token1] = ACTIONS(1191),
    [aux_sym_data_statement_token1] = ACTIONS(1191),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1191),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1191),
    [aux_sym_end_if_statement_token1] = ACTIONS(1191),
    [aux_sym_elseif_clause_token2] = ACTIONS(1191),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1191),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1191),
    [aux_sym_select_case_statement_token1] = ACTIONS(1191),
    [aux_sym_select_case_statement_token3] = ACTIONS(1191),
    [aux_sym_select_type_statement_token1] = ACTIONS(1191),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1191),
    [aux_sym_block_construct_token1] = ACTIONS(1191),
    [aux_sym_associate_statement_token1] = ACTIONS(1191),
    [aux_sym_format_statement_token1] = ACTIONS(1191),
    [aux_sym_print_statement_token1] = ACTIONS(1191),
    [aux_sym_open_statement_token1] = ACTIONS(1191),
    [aux_sym_close_statement_token1] = ACTIONS(1191),
    [aux_sym_inquire_statement_token1] = ACTIONS(1191),
    [aux_sym_enum_statement_token1] = ACTIONS(1191),
    [aux_sym_file_position_statement_token1] = ACTIONS(1191),
    [aux_sym_file_position_statement_token2] = ACTIONS(1191),
    [aux_sym_file_position_statement_token3] = ACTIONS(1191),
    [aux_sym_file_position_statement_token4] = ACTIONS(1191),
    [aux_sym_logical_expression_token5] = ACTIONS(1189),
    [anon_sym_PLUS] = ACTIONS(1189),
    [anon_sym_DOT] = ACTIONS(1191),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1189),
    [anon_sym_LBRACK] = ACTIONS(1189),
    [sym_boolean_literal] = ACTIONS(1189),
    [aux_sym_null_literal_token1] = ACTIONS(1191),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1191),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1189),
    [sym__float_literal] = ACTIONS(1189),
    [sym__boz_literal] = ACTIONS(1189),
    [sym_string_literal] = ACTIONS(1189),
  },
  [264] = {
    [sym_preproc_file_line] = STATE(264),
    [ts_builtin_sym_end] = ACTIONS(1193),
    [aux_sym_program_statement_token1] = ACTIONS(1195),
    [aux_sym_end_program_statement_token1] = ACTIONS(1195),
    [aux_sym_end_program_statement_token2] = ACTIONS(1195),
    [aux_sym_module_statement_token1] = ACTIONS(1195),
    [aux_sym_submodule_statement_token1] = ACTIONS(1195),
    [anon_sym_LPAREN] = ACTIONS(1195),
    [aux_sym_interface_statement_token1] = ACTIONS(1195),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1195),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1195),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1195),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1195),
    [aux_sym_function_statement_token1] = ACTIONS(1195),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1195),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1195),
    [aux_sym_contains_statement_token1] = ACTIONS(1195),
    [aux_sym_use_statement_token1] = ACTIONS(1195),
    [aux_sym_use_statement_token2] = ACTIONS(1195),
    [aux_sym_implicit_statement_token1] = ACTIONS(1195),
    [aux_sym_implicit_statement_token3] = ACTIONS(1195),
    [aux_sym_implicit_statement_token4] = ACTIONS(1195),
    [aux_sym_save_statement_token1] = ACTIONS(1195),
    [aux_sym_private_statement_token1] = ACTIONS(1195),
    [aux_sym_public_statement_token1] = ACTIONS(1195),
    [aux_sym_namelist_statement_token1] = ACTIONS(1195),
    [aux_sym_common_statement_token1] = ACTIONS(1195),
    [anon_sym_DASH] = ACTIONS(1193),
    [aux_sym_import_statement_token1] = ACTIONS(1195),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1195),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1195),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1195),
    [aux_sym_variable_attributes_token1] = ACTIONS(1195),
    [aux_sym_variable_attributes_token2] = ACTIONS(1195),
    [aux_sym_variable_attributes_token3] = ACTIONS(1195),
    [aux_sym_variable_attributes_token4] = ACTIONS(1195),
    [aux_sym_variable_attributes_token5] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1195),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1195),
    [aux_sym_derived_type_token1] = ACTIONS(1195),
    [aux_sym_type_qualifier_token1] = ACTIONS(1195),
    [aux_sym_type_qualifier_token2] = ACTIONS(1195),
    [aux_sym_type_qualifier_token3] = ACTIONS(1195),
    [aux_sym_type_qualifier_token4] = ACTIONS(1195),
    [aux_sym_type_qualifier_token5] = ACTIONS(1195),
    [aux_sym_type_qualifier_token9] = ACTIONS(1195),
    [aux_sym_type_qualifier_token10] = ACTIONS(1195),
    [aux_sym_type_qualifier_token11] = ACTIONS(1195),
    [aux_sym_type_qualifier_token12] = ACTIONS(1195),
    [aux_sym_type_qualifier_token13] = ACTIONS(1195),
    [aux_sym_type_qualifier_token14] = ACTIONS(1195),
    [aux_sym_type_qualifier_token15] = ACTIONS(1195),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1195),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1195),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1195),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1195),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1195),
    [aux_sym_stop_statement_token1] = ACTIONS(1195),
    [aux_sym_stop_statement_token2] = ACTIONS(1195),
    [aux_sym_subroutine_call_token1] = ACTIONS(1195),
    [aux_sym_keyword_statement_token1] = ACTIONS(1195),
    [aux_sym_keyword_statement_token2] = ACTIONS(1195),
    [aux_sym_keyword_statement_token3] = ACTIONS(1195),
    [aux_sym_keyword_statement_token4] = ACTIONS(1195),
    [aux_sym_keyword_statement_token6] = ACTIONS(1195),
    [aux_sym_keyword_statement_token7] = ACTIONS(1195),
    [aux_sym_include_statement_token1] = ACTIONS(1195),
    [aux_sym_data_statement_token1] = ACTIONS(1195),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1195),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1195),
    [aux_sym_end_if_statement_token1] = ACTIONS(1195),
    [aux_sym_elseif_clause_token2] = ACTIONS(1195),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1195),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1195),
    [aux_sym_select_case_statement_token1] = ACTIONS(1195),
    [aux_sym_select_case_statement_token3] = ACTIONS(1195),
    [aux_sym_select_type_statement_token1] = ACTIONS(1195),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1195),
    [aux_sym_block_construct_token1] = ACTIONS(1195),
    [aux_sym_associate_statement_token1] = ACTIONS(1195),
    [aux_sym_format_statement_token1] = ACTIONS(1195),
    [aux_sym_print_statement_token1] = ACTIONS(1195),
    [aux_sym_open_statement_token1] = ACTIONS(1195),
    [aux_sym_close_statement_token1] = ACTIONS(1195),
    [aux_sym_inquire_statement_token1] = ACTIONS(1195),
    [aux_sym_enum_statement_token1] = ACTIONS(1195),
    [aux_sym_file_position_statement_token1] = ACTIONS(1195),
    [aux_sym_file_position_statement_token2] = ACTIONS(1195),
    [aux_sym_file_position_statement_token3] = ACTIONS(1195),
    [aux_sym_file_position_statement_token4] = ACTIONS(1195),
    [aux_sym_logical_expression_token5] = ACTIONS(1193),
    [anon_sym_PLUS] = ACTIONS(1193),
    [anon_sym_DOT] = ACTIONS(1195),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1193),
    [anon_sym_LBRACK] = ACTIONS(1193),
    [sym_boolean_literal] = ACTIONS(1193),
    [aux_sym_null_literal_token1] = ACTIONS(1195),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1195),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1193),
    [sym__float_literal] = ACTIONS(1193),
    [sym__boz_literal] = ACTIONS(1193),
    [sym_string_literal] = ACTIONS(1193),
  },
  [265] = {
    [sym_preproc_file_line] = STATE(265),
    [ts_builtin_sym_end] = ACTIONS(1197),
    [aux_sym_program_statement_token1] = ACTIONS(1199),
    [aux_sym_end_program_statement_token1] = ACTIONS(1199),
    [aux_sym_end_program_statement_token2] = ACTIONS(1199),
    [aux_sym_module_statement_token1] = ACTIONS(1199),
    [aux_sym_submodule_statement_token1] = ACTIONS(1199),
    [anon_sym_LPAREN] = ACTIONS(1199),
    [aux_sym_interface_statement_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1199),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1199),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_function_statement_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1199),
    [aux_sym_contains_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token2] = ACTIONS(1199),
    [aux_sym_implicit_statement_token1] = ACTIONS(1199),
    [aux_sym_implicit_statement_token3] = ACTIONS(1199),
    [aux_sym_implicit_statement_token4] = ACTIONS(1199),
    [aux_sym_save_statement_token1] = ACTIONS(1199),
    [aux_sym_private_statement_token1] = ACTIONS(1199),
    [aux_sym_public_statement_token1] = ACTIONS(1199),
    [aux_sym_namelist_statement_token1] = ACTIONS(1199),
    [aux_sym_common_statement_token1] = ACTIONS(1199),
    [anon_sym_DASH] = ACTIONS(1197),
    [aux_sym_import_statement_token1] = ACTIONS(1199),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1199),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1199),
    [aux_sym_variable_attributes_token1] = ACTIONS(1199),
    [aux_sym_variable_attributes_token2] = ACTIONS(1199),
    [aux_sym_variable_attributes_token3] = ACTIONS(1199),
    [aux_sym_variable_attributes_token4] = ACTIONS(1199),
    [aux_sym_variable_attributes_token5] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1199),
    [aux_sym_derived_type_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token2] = ACTIONS(1199),
    [aux_sym_type_qualifier_token3] = ACTIONS(1199),
    [aux_sym_type_qualifier_token4] = ACTIONS(1199),
    [aux_sym_type_qualifier_token5] = ACTIONS(1199),
    [aux_sym_type_qualifier_token9] = ACTIONS(1199),
    [aux_sym_type_qualifier_token10] = ACTIONS(1199),
    [aux_sym_type_qualifier_token11] = ACTIONS(1199),
    [aux_sym_type_qualifier_token12] = ACTIONS(1199),
    [aux_sym_type_qualifier_token13] = ACTIONS(1199),
    [aux_sym_type_qualifier_token14] = ACTIONS(1199),
    [aux_sym_type_qualifier_token15] = ACTIONS(1199),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1199),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1199),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1199),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token2] = ACTIONS(1199),
    [aux_sym_subroutine_call_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token2] = ACTIONS(1199),
    [aux_sym_keyword_statement_token3] = ACTIONS(1199),
    [aux_sym_keyword_statement_token4] = ACTIONS(1199),
    [aux_sym_keyword_statement_token6] = ACTIONS(1199),
    [aux_sym_keyword_statement_token7] = ACTIONS(1199),
    [aux_sym_include_statement_token1] = ACTIONS(1199),
    [aux_sym_data_statement_token1] = ACTIONS(1199),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1199),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1199),
    [aux_sym_end_if_statement_token1] = ACTIONS(1199),
    [aux_sym_elseif_clause_token2] = ACTIONS(1199),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1199),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token3] = ACTIONS(1199),
    [aux_sym_select_type_statement_token1] = ACTIONS(1199),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1199),
    [aux_sym_block_construct_token1] = ACTIONS(1199),
    [aux_sym_associate_statement_token1] = ACTIONS(1199),
    [aux_sym_format_statement_token1] = ACTIONS(1199),
    [aux_sym_print_statement_token1] = ACTIONS(1199),
    [aux_sym_open_statement_token1] = ACTIONS(1199),
    [aux_sym_close_statement_token1] = ACTIONS(1199),
    [aux_sym_inquire_statement_token1] = ACTIONS(1199),
    [aux_sym_enum_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token2] = ACTIONS(1199),
    [aux_sym_file_position_statement_token3] = ACTIONS(1199),
    [aux_sym_file_position_statement_token4] = ACTIONS(1199),
    [aux_sym_logical_expression_token5] = ACTIONS(1197),
    [anon_sym_PLUS] = ACTIONS(1197),
    [anon_sym_DOT] = ACTIONS(1199),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1197),
    [anon_sym_LBRACK] = ACTIONS(1197),
    [sym_boolean_literal] = ACTIONS(1197),
    [aux_sym_null_literal_token1] = ACTIONS(1199),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1199),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1197),
    [sym__float_literal] = ACTIONS(1197),
    [sym__boz_literal] = ACTIONS(1197),
    [sym_string_literal] = ACTIONS(1197),
  },
  [266] = {
    [sym_preproc_file_line] = STATE(266),
    [ts_builtin_sym_end] = ACTIONS(1201),
    [aux_sym_program_statement_token1] = ACTIONS(1203),
    [aux_sym_end_program_statement_token1] = ACTIONS(1203),
    [aux_sym_end_program_statement_token2] = ACTIONS(1203),
    [aux_sym_module_statement_token1] = ACTIONS(1203),
    [aux_sym_submodule_statement_token1] = ACTIONS(1203),
    [anon_sym_LPAREN] = ACTIONS(1203),
    [aux_sym_interface_statement_token1] = ACTIONS(1203),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1203),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1203),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1203),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1203),
    [aux_sym_function_statement_token1] = ACTIONS(1203),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1203),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1203),
    [aux_sym_contains_statement_token1] = ACTIONS(1203),
    [aux_sym_use_statement_token1] = ACTIONS(1203),
    [aux_sym_use_statement_token2] = ACTIONS(1203),
    [aux_sym_implicit_statement_token1] = ACTIONS(1203),
    [aux_sym_implicit_statement_token3] = ACTIONS(1203),
    [aux_sym_implicit_statement_token4] = ACTIONS(1203),
    [aux_sym_save_statement_token1] = ACTIONS(1203),
    [aux_sym_private_statement_token1] = ACTIONS(1203),
    [aux_sym_public_statement_token1] = ACTIONS(1203),
    [aux_sym_namelist_statement_token1] = ACTIONS(1203),
    [aux_sym_common_statement_token1] = ACTIONS(1203),
    [anon_sym_DASH] = ACTIONS(1201),
    [aux_sym_import_statement_token1] = ACTIONS(1203),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1203),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1203),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1203),
    [aux_sym_variable_attributes_token1] = ACTIONS(1203),
    [aux_sym_variable_attributes_token2] = ACTIONS(1203),
    [aux_sym_variable_attributes_token3] = ACTIONS(1203),
    [aux_sym_variable_attributes_token4] = ACTIONS(1203),
    [aux_sym_variable_attributes_token5] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1203),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1203),
    [aux_sym_derived_type_token1] = ACTIONS(1203),
    [aux_sym_type_qualifier_token1] = ACTIONS(1203),
    [aux_sym_type_qualifier_token2] = ACTIONS(1203),
    [aux_sym_type_qualifier_token3] = ACTIONS(1203),
    [aux_sym_type_qualifier_token4] = ACTIONS(1203),
    [aux_sym_type_qualifier_token5] = ACTIONS(1203),
    [aux_sym_type_qualifier_token9] = ACTIONS(1203),
    [aux_sym_type_qualifier_token10] = ACTIONS(1203),
    [aux_sym_type_qualifier_token11] = ACTIONS(1203),
    [aux_sym_type_qualifier_token12] = ACTIONS(1203),
    [aux_sym_type_qualifier_token13] = ACTIONS(1203),
    [aux_sym_type_qualifier_token14] = ACTIONS(1203),
    [aux_sym_type_qualifier_token15] = ACTIONS(1203),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1203),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1203),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1203),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1203),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1203),
    [aux_sym_stop_statement_token1] = ACTIONS(1203),
    [aux_sym_stop_statement_token2] = ACTIONS(1203),
    [aux_sym_subroutine_call_token1] = ACTIONS(1203),
    [aux_sym_keyword_statement_token1] = ACTIONS(1203),
    [aux_sym_keyword_statement_token2] = ACTIONS(1203),
    [aux_sym_keyword_statement_token3] = ACTIONS(1203),
    [aux_sym_keyword_statement_token4] = ACTIONS(1203),
    [aux_sym_keyword_statement_token6] = ACTIONS(1203),
    [aux_sym_keyword_statement_token7] = ACTIONS(1203),
    [aux_sym_include_statement_token1] = ACTIONS(1203),
    [aux_sym_data_statement_token1] = ACTIONS(1203),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1203),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1203),
    [aux_sym_end_if_statement_token1] = ACTIONS(1203),
    [aux_sym_elseif_clause_token2] = ACTIONS(1203),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1203),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1203),
    [aux_sym_select_case_statement_token1] = ACTIONS(1203),
    [aux_sym_select_case_statement_token3] = ACTIONS(1203),
    [aux_sym_select_type_statement_token1] = ACTIONS(1203),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1203),
    [aux_sym_block_construct_token1] = ACTIONS(1203),
    [aux_sym_associate_statement_token1] = ACTIONS(1203),
    [aux_sym_format_statement_token1] = ACTIONS(1203),
    [aux_sym_print_statement_token1] = ACTIONS(1203),
    [aux_sym_open_statement_token1] = ACTIONS(1203),
    [aux_sym_close_statement_token1] = ACTIONS(1203),
    [aux_sym_inquire_statement_token1] = ACTIONS(1203),
    [aux_sym_enum_statement_token1] = ACTIONS(1203),
    [aux_sym_file_position_statement_token1] = ACTIONS(1203),
    [aux_sym_file_position_statement_token2] = ACTIONS(1203),
    [aux_sym_file_position_statement_token3] = ACTIONS(1203),
    [aux_sym_file_position_statement_token4] = ACTIONS(1203),
    [aux_sym_logical_expression_token5] = ACTIONS(1201),
    [anon_sym_PLUS] = ACTIONS(1201),
    [anon_sym_DOT] = ACTIONS(1203),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1201),
    [anon_sym_LBRACK] = ACTIONS(1201),
    [sym_boolean_literal] = ACTIONS(1201),
    [aux_sym_null_literal_token1] = ACTIONS(1203),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1203),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1201),
    [sym__float_literal] = ACTIONS(1201),
    [sym__boz_literal] = ACTIONS(1201),
    [sym_string_literal] = ACTIONS(1201),
  },
  [267] = {
    [sym_preproc_file_line] = STATE(267),
    [ts_builtin_sym_end] = ACTIONS(1205),
    [aux_sym_program_statement_token1] = ACTIONS(1207),
    [aux_sym_end_program_statement_token1] = ACTIONS(1207),
    [aux_sym_end_program_statement_token2] = ACTIONS(1207),
    [aux_sym_module_statement_token1] = ACTIONS(1207),
    [aux_sym_submodule_statement_token1] = ACTIONS(1207),
    [anon_sym_LPAREN] = ACTIONS(1207),
    [aux_sym_interface_statement_token1] = ACTIONS(1207),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1207),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1207),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1207),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1207),
    [aux_sym_function_statement_token1] = ACTIONS(1207),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1207),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1207),
    [aux_sym_contains_statement_token1] = ACTIONS(1207),
    [aux_sym_use_statement_token1] = ACTIONS(1207),
    [aux_sym_use_statement_token2] = ACTIONS(1207),
    [aux_sym_implicit_statement_token1] = ACTIONS(1207),
    [aux_sym_implicit_statement_token3] = ACTIONS(1207),
    [aux_sym_implicit_statement_token4] = ACTIONS(1207),
    [aux_sym_save_statement_token1] = ACTIONS(1207),
    [aux_sym_private_statement_token1] = ACTIONS(1207),
    [aux_sym_public_statement_token1] = ACTIONS(1207),
    [aux_sym_namelist_statement_token1] = ACTIONS(1207),
    [aux_sym_common_statement_token1] = ACTIONS(1207),
    [anon_sym_DASH] = ACTIONS(1205),
    [aux_sym_import_statement_token1] = ACTIONS(1207),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1207),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1207),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1207),
    [aux_sym_variable_attributes_token1] = ACTIONS(1207),
    [aux_sym_variable_attributes_token2] = ACTIONS(1207),
    [aux_sym_variable_attributes_token3] = ACTIONS(1207),
    [aux_sym_variable_attributes_token4] = ACTIONS(1207),
    [aux_sym_variable_attributes_token5] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1207),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1207),
    [aux_sym_derived_type_token1] = ACTIONS(1207),
    [aux_sym_type_qualifier_token1] = ACTIONS(1207),
    [aux_sym_type_qualifier_token2] = ACTIONS(1207),
    [aux_sym_type_qualifier_token3] = ACTIONS(1207),
    [aux_sym_type_qualifier_token4] = ACTIONS(1207),
    [aux_sym_type_qualifier_token5] = ACTIONS(1207),
    [aux_sym_type_qualifier_token9] = ACTIONS(1207),
    [aux_sym_type_qualifier_token10] = ACTIONS(1207),
    [aux_sym_type_qualifier_token11] = ACTIONS(1207),
    [aux_sym_type_qualifier_token12] = ACTIONS(1207),
    [aux_sym_type_qualifier_token13] = ACTIONS(1207),
    [aux_sym_type_qualifier_token14] = ACTIONS(1207),
    [aux_sym_type_qualifier_token15] = ACTIONS(1207),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1207),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1207),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1207),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1207),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1207),
    [aux_sym_stop_statement_token1] = ACTIONS(1207),
    [aux_sym_stop_statement_token2] = ACTIONS(1207),
    [aux_sym_subroutine_call_token1] = ACTIONS(1207),
    [aux_sym_keyword_statement_token1] = ACTIONS(1207),
    [aux_sym_keyword_statement_token2] = ACTIONS(1207),
    [aux_sym_keyword_statement_token3] = ACTIONS(1207),
    [aux_sym_keyword_statement_token4] = ACTIONS(1207),
    [aux_sym_keyword_statement_token6] = ACTIONS(1207),
    [aux_sym_keyword_statement_token7] = ACTIONS(1207),
    [aux_sym_include_statement_token1] = ACTIONS(1207),
    [aux_sym_data_statement_token1] = ACTIONS(1207),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1207),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1207),
    [aux_sym_end_if_statement_token1] = ACTIONS(1207),
    [aux_sym_elseif_clause_token2] = ACTIONS(1207),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1207),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1207),
    [aux_sym_select_case_statement_token1] = ACTIONS(1207),
    [aux_sym_select_case_statement_token3] = ACTIONS(1207),
    [aux_sym_select_type_statement_token1] = ACTIONS(1207),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1207),
    [aux_sym_block_construct_token1] = ACTIONS(1207),
    [aux_sym_associate_statement_token1] = ACTIONS(1207),
    [aux_sym_format_statement_token1] = ACTIONS(1207),
    [aux_sym_print_statement_token1] = ACTIONS(1207),
    [aux_sym_open_statement_token1] = ACTIONS(1207),
    [aux_sym_close_statement_token1] = ACTIONS(1207),
    [aux_sym_inquire_statement_token1] = ACTIONS(1207),
    [aux_sym_enum_statement_token1] = ACTIONS(1207),
    [aux_sym_file_position_statement_token1] = ACTIONS(1207),
    [aux_sym_file_position_statement_token2] = ACTIONS(1207),
    [aux_sym_file_position_statement_token3] = ACTIONS(1207),
    [aux_sym_file_position_statement_token4] = ACTIONS(1207),
    [aux_sym_logical_expression_token5] = ACTIONS(1205),
    [anon_sym_PLUS] = ACTIONS(1205),
    [anon_sym_DOT] = ACTIONS(1207),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1205),
    [anon_sym_LBRACK] = ACTIONS(1205),
    [sym_boolean_literal] = ACTIONS(1205),
    [aux_sym_null_literal_token1] = ACTIONS(1207),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1207),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1205),
    [sym__float_literal] = ACTIONS(1205),
    [sym__boz_literal] = ACTIONS(1205),
    [sym_string_literal] = ACTIONS(1205),
  },
  [268] = {
    [sym__statements] = STATE(2386),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(268),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [269] = {
    [sym__statements] = STATE(2766),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(269),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [270] = {
    [sym_preproc_file_line] = STATE(270),
    [ts_builtin_sym_end] = ACTIONS(1209),
    [aux_sym_program_statement_token1] = ACTIONS(1211),
    [aux_sym_end_program_statement_token1] = ACTIONS(1211),
    [aux_sym_end_program_statement_token2] = ACTIONS(1211),
    [aux_sym_module_statement_token1] = ACTIONS(1211),
    [aux_sym_submodule_statement_token1] = ACTIONS(1211),
    [anon_sym_LPAREN] = ACTIONS(1211),
    [aux_sym_interface_statement_token1] = ACTIONS(1211),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1211),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1211),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1211),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1211),
    [aux_sym_function_statement_token1] = ACTIONS(1211),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1211),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1211),
    [aux_sym_contains_statement_token1] = ACTIONS(1211),
    [aux_sym_use_statement_token1] = ACTIONS(1211),
    [aux_sym_use_statement_token2] = ACTIONS(1211),
    [aux_sym_implicit_statement_token1] = ACTIONS(1211),
    [aux_sym_implicit_statement_token3] = ACTIONS(1211),
    [aux_sym_implicit_statement_token4] = ACTIONS(1211),
    [aux_sym_save_statement_token1] = ACTIONS(1211),
    [aux_sym_private_statement_token1] = ACTIONS(1211),
    [aux_sym_public_statement_token1] = ACTIONS(1211),
    [aux_sym_namelist_statement_token1] = ACTIONS(1211),
    [aux_sym_common_statement_token1] = ACTIONS(1211),
    [anon_sym_DASH] = ACTIONS(1209),
    [aux_sym_import_statement_token1] = ACTIONS(1211),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1211),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1211),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1211),
    [aux_sym_variable_attributes_token1] = ACTIONS(1211),
    [aux_sym_variable_attributes_token2] = ACTIONS(1211),
    [aux_sym_variable_attributes_token3] = ACTIONS(1211),
    [aux_sym_variable_attributes_token4] = ACTIONS(1211),
    [aux_sym_variable_attributes_token5] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1211),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1211),
    [aux_sym_derived_type_token1] = ACTIONS(1211),
    [aux_sym_type_qualifier_token1] = ACTIONS(1211),
    [aux_sym_type_qualifier_token2] = ACTIONS(1211),
    [aux_sym_type_qualifier_token3] = ACTIONS(1211),
    [aux_sym_type_qualifier_token4] = ACTIONS(1211),
    [aux_sym_type_qualifier_token5] = ACTIONS(1211),
    [aux_sym_type_qualifier_token9] = ACTIONS(1211),
    [aux_sym_type_qualifier_token10] = ACTIONS(1211),
    [aux_sym_type_qualifier_token11] = ACTIONS(1211),
    [aux_sym_type_qualifier_token12] = ACTIONS(1211),
    [aux_sym_type_qualifier_token13] = ACTIONS(1211),
    [aux_sym_type_qualifier_token14] = ACTIONS(1211),
    [aux_sym_type_qualifier_token15] = ACTIONS(1211),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1211),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1211),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1211),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1211),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1211),
    [aux_sym_stop_statement_token1] = ACTIONS(1211),
    [aux_sym_stop_statement_token2] = ACTIONS(1211),
    [aux_sym_subroutine_call_token1] = ACTIONS(1211),
    [aux_sym_keyword_statement_token1] = ACTIONS(1211),
    [aux_sym_keyword_statement_token2] = ACTIONS(1211),
    [aux_sym_keyword_statement_token3] = ACTIONS(1211),
    [aux_sym_keyword_statement_token4] = ACTIONS(1211),
    [aux_sym_keyword_statement_token6] = ACTIONS(1211),
    [aux_sym_keyword_statement_token7] = ACTIONS(1211),
    [aux_sym_include_statement_token1] = ACTIONS(1211),
    [aux_sym_data_statement_token1] = ACTIONS(1211),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1211),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1211),
    [aux_sym_end_if_statement_token1] = ACTIONS(1211),
    [aux_sym_elseif_clause_token2] = ACTIONS(1211),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1211),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1211),
    [aux_sym_select_case_statement_token1] = ACTIONS(1211),
    [aux_sym_select_case_statement_token3] = ACTIONS(1211),
    [aux_sym_select_type_statement_token1] = ACTIONS(1211),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1211),
    [aux_sym_block_construct_token1] = ACTIONS(1211),
    [aux_sym_associate_statement_token1] = ACTIONS(1211),
    [aux_sym_format_statement_token1] = ACTIONS(1211),
    [aux_sym_print_statement_token1] = ACTIONS(1211),
    [aux_sym_open_statement_token1] = ACTIONS(1211),
    [aux_sym_close_statement_token1] = ACTIONS(1211),
    [aux_sym_inquire_statement_token1] = ACTIONS(1211),
    [aux_sym_enum_statement_token1] = ACTIONS(1211),
    [aux_sym_file_position_statement_token1] = ACTIONS(1211),
    [aux_sym_file_position_statement_token2] = ACTIONS(1211),
    [aux_sym_file_position_statement_token3] = ACTIONS(1211),
    [aux_sym_file_position_statement_token4] = ACTIONS(1211),
    [aux_sym_logical_expression_token5] = ACTIONS(1209),
    [anon_sym_PLUS] = ACTIONS(1209),
    [anon_sym_DOT] = ACTIONS(1211),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1209),
    [anon_sym_LBRACK] = ACTIONS(1209),
    [sym_boolean_literal] = ACTIONS(1209),
    [aux_sym_null_literal_token1] = ACTIONS(1211),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1211),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1209),
    [sym__float_literal] = ACTIONS(1209),
    [sym__boz_literal] = ACTIONS(1209),
    [sym_string_literal] = ACTIONS(1209),
  },
  [271] = {
    [sym_preproc_file_line] = STATE(271),
    [ts_builtin_sym_end] = ACTIONS(1213),
    [aux_sym_program_statement_token1] = ACTIONS(1215),
    [aux_sym_end_program_statement_token1] = ACTIONS(1215),
    [aux_sym_end_program_statement_token2] = ACTIONS(1215),
    [aux_sym_module_statement_token1] = ACTIONS(1215),
    [aux_sym_submodule_statement_token1] = ACTIONS(1215),
    [anon_sym_LPAREN] = ACTIONS(1215),
    [aux_sym_interface_statement_token1] = ACTIONS(1215),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1215),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1215),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1215),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1215),
    [aux_sym_function_statement_token1] = ACTIONS(1215),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1215),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1215),
    [aux_sym_contains_statement_token1] = ACTIONS(1215),
    [aux_sym_use_statement_token1] = ACTIONS(1215),
    [aux_sym_use_statement_token2] = ACTIONS(1215),
    [aux_sym_implicit_statement_token1] = ACTIONS(1215),
    [aux_sym_implicit_statement_token3] = ACTIONS(1215),
    [aux_sym_implicit_statement_token4] = ACTIONS(1215),
    [aux_sym_save_statement_token1] = ACTIONS(1215),
    [aux_sym_private_statement_token1] = ACTIONS(1215),
    [aux_sym_public_statement_token1] = ACTIONS(1215),
    [aux_sym_namelist_statement_token1] = ACTIONS(1215),
    [aux_sym_common_statement_token1] = ACTIONS(1215),
    [anon_sym_DASH] = ACTIONS(1213),
    [aux_sym_import_statement_token1] = ACTIONS(1215),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1215),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1215),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1215),
    [aux_sym_variable_attributes_token1] = ACTIONS(1215),
    [aux_sym_variable_attributes_token2] = ACTIONS(1215),
    [aux_sym_variable_attributes_token3] = ACTIONS(1215),
    [aux_sym_variable_attributes_token4] = ACTIONS(1215),
    [aux_sym_variable_attributes_token5] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1215),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1215),
    [aux_sym_derived_type_token1] = ACTIONS(1215),
    [aux_sym_type_qualifier_token1] = ACTIONS(1215),
    [aux_sym_type_qualifier_token2] = ACTIONS(1215),
    [aux_sym_type_qualifier_token3] = ACTIONS(1215),
    [aux_sym_type_qualifier_token4] = ACTIONS(1215),
    [aux_sym_type_qualifier_token5] = ACTIONS(1215),
    [aux_sym_type_qualifier_token9] = ACTIONS(1215),
    [aux_sym_type_qualifier_token10] = ACTIONS(1215),
    [aux_sym_type_qualifier_token11] = ACTIONS(1215),
    [aux_sym_type_qualifier_token12] = ACTIONS(1215),
    [aux_sym_type_qualifier_token13] = ACTIONS(1215),
    [aux_sym_type_qualifier_token14] = ACTIONS(1215),
    [aux_sym_type_qualifier_token15] = ACTIONS(1215),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1215),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1215),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1215),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1215),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1215),
    [aux_sym_stop_statement_token1] = ACTIONS(1215),
    [aux_sym_stop_statement_token2] = ACTIONS(1215),
    [aux_sym_subroutine_call_token1] = ACTIONS(1215),
    [aux_sym_keyword_statement_token1] = ACTIONS(1215),
    [aux_sym_keyword_statement_token2] = ACTIONS(1215),
    [aux_sym_keyword_statement_token3] = ACTIONS(1215),
    [aux_sym_keyword_statement_token4] = ACTIONS(1215),
    [aux_sym_keyword_statement_token6] = ACTIONS(1215),
    [aux_sym_keyword_statement_token7] = ACTIONS(1215),
    [aux_sym_include_statement_token1] = ACTIONS(1215),
    [aux_sym_data_statement_token1] = ACTIONS(1215),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1215),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1215),
    [aux_sym_end_if_statement_token1] = ACTIONS(1215),
    [aux_sym_elseif_clause_token2] = ACTIONS(1215),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1215),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1215),
    [aux_sym_select_case_statement_token1] = ACTIONS(1215),
    [aux_sym_select_case_statement_token3] = ACTIONS(1215),
    [aux_sym_select_type_statement_token1] = ACTIONS(1215),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1215),
    [aux_sym_block_construct_token1] = ACTIONS(1215),
    [aux_sym_associate_statement_token1] = ACTIONS(1215),
    [aux_sym_format_statement_token1] = ACTIONS(1215),
    [aux_sym_print_statement_token1] = ACTIONS(1215),
    [aux_sym_open_statement_token1] = ACTIONS(1215),
    [aux_sym_close_statement_token1] = ACTIONS(1215),
    [aux_sym_inquire_statement_token1] = ACTIONS(1215),
    [aux_sym_enum_statement_token1] = ACTIONS(1215),
    [aux_sym_file_position_statement_token1] = ACTIONS(1215),
    [aux_sym_file_position_statement_token2] = ACTIONS(1215),
    [aux_sym_file_position_statement_token3] = ACTIONS(1215),
    [aux_sym_file_position_statement_token4] = ACTIONS(1215),
    [aux_sym_logical_expression_token5] = ACTIONS(1213),
    [anon_sym_PLUS] = ACTIONS(1213),
    [anon_sym_DOT] = ACTIONS(1215),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1213),
    [anon_sym_LBRACK] = ACTIONS(1213),
    [sym_boolean_literal] = ACTIONS(1213),
    [aux_sym_null_literal_token1] = ACTIONS(1215),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1215),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1213),
    [sym__float_literal] = ACTIONS(1213),
    [sym__boz_literal] = ACTIONS(1213),
    [sym_string_literal] = ACTIONS(1213),
  },
  [272] = {
    [sym__statements] = STATE(2626),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(272),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [273] = {
    [sym_preproc_file_line] = STATE(273),
    [ts_builtin_sym_end] = ACTIONS(1217),
    [aux_sym_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token2] = ACTIONS(1219),
    [aux_sym_module_statement_token1] = ACTIONS(1219),
    [aux_sym_submodule_statement_token1] = ACTIONS(1219),
    [anon_sym_LPAREN] = ACTIONS(1219),
    [aux_sym_interface_statement_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1219),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1219),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1219),
    [aux_sym_function_statement_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1219),
    [aux_sym_contains_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token2] = ACTIONS(1219),
    [aux_sym_implicit_statement_token1] = ACTIONS(1219),
    [aux_sym_implicit_statement_token3] = ACTIONS(1219),
    [aux_sym_implicit_statement_token4] = ACTIONS(1219),
    [aux_sym_save_statement_token1] = ACTIONS(1219),
    [aux_sym_private_statement_token1] = ACTIONS(1219),
    [aux_sym_public_statement_token1] = ACTIONS(1219),
    [aux_sym_namelist_statement_token1] = ACTIONS(1219),
    [aux_sym_common_statement_token1] = ACTIONS(1219),
    [anon_sym_DASH] = ACTIONS(1217),
    [aux_sym_import_statement_token1] = ACTIONS(1219),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1219),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1219),
    [aux_sym_variable_attributes_token1] = ACTIONS(1219),
    [aux_sym_variable_attributes_token2] = ACTIONS(1219),
    [aux_sym_variable_attributes_token3] = ACTIONS(1219),
    [aux_sym_variable_attributes_token4] = ACTIONS(1219),
    [aux_sym_variable_attributes_token5] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1219),
    [aux_sym_derived_type_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token2] = ACTIONS(1219),
    [aux_sym_type_qualifier_token3] = ACTIONS(1219),
    [aux_sym_type_qualifier_token4] = ACTIONS(1219),
    [aux_sym_type_qualifier_token5] = ACTIONS(1219),
    [aux_sym_type_qualifier_token9] = ACTIONS(1219),
    [aux_sym_type_qualifier_token10] = ACTIONS(1219),
    [aux_sym_type_qualifier_token11] = ACTIONS(1219),
    [aux_sym_type_qualifier_token12] = ACTIONS(1219),
    [aux_sym_type_qualifier_token13] = ACTIONS(1219),
    [aux_sym_type_qualifier_token14] = ACTIONS(1219),
    [aux_sym_type_qualifier_token15] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1219),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token2] = ACTIONS(1219),
    [aux_sym_subroutine_call_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token2] = ACTIONS(1219),
    [aux_sym_keyword_statement_token3] = ACTIONS(1219),
    [aux_sym_keyword_statement_token4] = ACTIONS(1219),
    [aux_sym_keyword_statement_token6] = ACTIONS(1219),
    [aux_sym_keyword_statement_token7] = ACTIONS(1219),
    [aux_sym_include_statement_token1] = ACTIONS(1219),
    [aux_sym_data_statement_token1] = ACTIONS(1219),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1219),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1219),
    [aux_sym_end_if_statement_token1] = ACTIONS(1219),
    [aux_sym_elseif_clause_token2] = ACTIONS(1219),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1219),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token3] = ACTIONS(1219),
    [aux_sym_select_type_statement_token1] = ACTIONS(1219),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1219),
    [aux_sym_block_construct_token1] = ACTIONS(1219),
    [aux_sym_associate_statement_token1] = ACTIONS(1219),
    [aux_sym_format_statement_token1] = ACTIONS(1219),
    [aux_sym_print_statement_token1] = ACTIONS(1219),
    [aux_sym_open_statement_token1] = ACTIONS(1219),
    [aux_sym_close_statement_token1] = ACTIONS(1219),
    [aux_sym_inquire_statement_token1] = ACTIONS(1219),
    [aux_sym_enum_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token2] = ACTIONS(1219),
    [aux_sym_file_position_statement_token3] = ACTIONS(1219),
    [aux_sym_file_position_statement_token4] = ACTIONS(1219),
    [aux_sym_logical_expression_token5] = ACTIONS(1217),
    [anon_sym_PLUS] = ACTIONS(1217),
    [anon_sym_DOT] = ACTIONS(1219),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1217),
    [anon_sym_LBRACK] = ACTIONS(1217),
    [sym_boolean_literal] = ACTIONS(1217),
    [aux_sym_null_literal_token1] = ACTIONS(1219),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1219),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1217),
    [sym__float_literal] = ACTIONS(1217),
    [sym__boz_literal] = ACTIONS(1217),
    [sym_string_literal] = ACTIONS(1217),
  },
  [274] = {
    [sym__statements] = STATE(2555),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(274),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [275] = {
    [sym_preproc_file_line] = STATE(275),
    [ts_builtin_sym_end] = ACTIONS(1221),
    [aux_sym_program_statement_token1] = ACTIONS(1223),
    [aux_sym_end_program_statement_token1] = ACTIONS(1223),
    [aux_sym_end_program_statement_token2] = ACTIONS(1223),
    [aux_sym_module_statement_token1] = ACTIONS(1223),
    [aux_sym_submodule_statement_token1] = ACTIONS(1223),
    [anon_sym_LPAREN] = ACTIONS(1223),
    [aux_sym_interface_statement_token1] = ACTIONS(1223),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1223),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1223),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1223),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1223),
    [aux_sym_function_statement_token1] = ACTIONS(1223),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1223),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1223),
    [aux_sym_contains_statement_token1] = ACTIONS(1223),
    [aux_sym_use_statement_token1] = ACTIONS(1223),
    [aux_sym_use_statement_token2] = ACTIONS(1223),
    [aux_sym_implicit_statement_token1] = ACTIONS(1223),
    [aux_sym_implicit_statement_token3] = ACTIONS(1223),
    [aux_sym_implicit_statement_token4] = ACTIONS(1223),
    [aux_sym_save_statement_token1] = ACTIONS(1223),
    [aux_sym_private_statement_token1] = ACTIONS(1223),
    [aux_sym_public_statement_token1] = ACTIONS(1223),
    [aux_sym_namelist_statement_token1] = ACTIONS(1223),
    [aux_sym_common_statement_token1] = ACTIONS(1223),
    [anon_sym_DASH] = ACTIONS(1221),
    [aux_sym_import_statement_token1] = ACTIONS(1223),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1223),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1223),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1223),
    [aux_sym_variable_attributes_token1] = ACTIONS(1223),
    [aux_sym_variable_attributes_token2] = ACTIONS(1223),
    [aux_sym_variable_attributes_token3] = ACTIONS(1223),
    [aux_sym_variable_attributes_token4] = ACTIONS(1223),
    [aux_sym_variable_attributes_token5] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1223),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1223),
    [aux_sym_derived_type_token1] = ACTIONS(1223),
    [aux_sym_type_qualifier_token1] = ACTIONS(1223),
    [aux_sym_type_qualifier_token2] = ACTIONS(1223),
    [aux_sym_type_qualifier_token3] = ACTIONS(1223),
    [aux_sym_type_qualifier_token4] = ACTIONS(1223),
    [aux_sym_type_qualifier_token5] = ACTIONS(1223),
    [aux_sym_type_qualifier_token9] = ACTIONS(1223),
    [aux_sym_type_qualifier_token10] = ACTIONS(1223),
    [aux_sym_type_qualifier_token11] = ACTIONS(1223),
    [aux_sym_type_qualifier_token12] = ACTIONS(1223),
    [aux_sym_type_qualifier_token13] = ACTIONS(1223),
    [aux_sym_type_qualifier_token14] = ACTIONS(1223),
    [aux_sym_type_qualifier_token15] = ACTIONS(1223),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1223),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1223),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1223),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1223),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1223),
    [aux_sym_stop_statement_token1] = ACTIONS(1223),
    [aux_sym_stop_statement_token2] = ACTIONS(1223),
    [aux_sym_subroutine_call_token1] = ACTIONS(1223),
    [aux_sym_keyword_statement_token1] = ACTIONS(1223),
    [aux_sym_keyword_statement_token2] = ACTIONS(1223),
    [aux_sym_keyword_statement_token3] = ACTIONS(1223),
    [aux_sym_keyword_statement_token4] = ACTIONS(1223),
    [aux_sym_keyword_statement_token6] = ACTIONS(1223),
    [aux_sym_keyword_statement_token7] = ACTIONS(1223),
    [aux_sym_include_statement_token1] = ACTIONS(1223),
    [aux_sym_data_statement_token1] = ACTIONS(1223),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1223),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1223),
    [aux_sym_end_if_statement_token1] = ACTIONS(1223),
    [aux_sym_elseif_clause_token2] = ACTIONS(1223),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1223),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1223),
    [aux_sym_select_case_statement_token1] = ACTIONS(1223),
    [aux_sym_select_case_statement_token3] = ACTIONS(1223),
    [aux_sym_select_type_statement_token1] = ACTIONS(1223),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1223),
    [aux_sym_block_construct_token1] = ACTIONS(1223),
    [aux_sym_associate_statement_token1] = ACTIONS(1223),
    [aux_sym_format_statement_token1] = ACTIONS(1223),
    [aux_sym_print_statement_token1] = ACTIONS(1223),
    [aux_sym_open_statement_token1] = ACTIONS(1223),
    [aux_sym_close_statement_token1] = ACTIONS(1223),
    [aux_sym_inquire_statement_token1] = ACTIONS(1223),
    [aux_sym_enum_statement_token1] = ACTIONS(1223),
    [aux_sym_file_position_statement_token1] = ACTIONS(1223),
    [aux_sym_file_position_statement_token2] = ACTIONS(1223),
    [aux_sym_file_position_statement_token3] = ACTIONS(1223),
    [aux_sym_file_position_statement_token4] = ACTIONS(1223),
    [aux_sym_logical_expression_token5] = ACTIONS(1221),
    [anon_sym_PLUS] = ACTIONS(1221),
    [anon_sym_DOT] = ACTIONS(1223),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1221),
    [anon_sym_LBRACK] = ACTIONS(1221),
    [sym_boolean_literal] = ACTIONS(1221),
    [aux_sym_null_literal_token1] = ACTIONS(1223),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1223),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1221),
    [sym__float_literal] = ACTIONS(1221),
    [sym__boz_literal] = ACTIONS(1221),
    [sym_string_literal] = ACTIONS(1221),
  },
  [276] = {
    [sym_preproc_file_line] = STATE(276),
    [ts_builtin_sym_end] = ACTIONS(1225),
    [aux_sym_program_statement_token1] = ACTIONS(1227),
    [aux_sym_end_program_statement_token1] = ACTIONS(1227),
    [aux_sym_end_program_statement_token2] = ACTIONS(1227),
    [aux_sym_module_statement_token1] = ACTIONS(1227),
    [aux_sym_submodule_statement_token1] = ACTIONS(1227),
    [anon_sym_LPAREN] = ACTIONS(1227),
    [aux_sym_interface_statement_token1] = ACTIONS(1227),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1227),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1227),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1227),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1227),
    [aux_sym_function_statement_token1] = ACTIONS(1227),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1227),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1227),
    [aux_sym_contains_statement_token1] = ACTIONS(1227),
    [aux_sym_use_statement_token1] = ACTIONS(1227),
    [aux_sym_use_statement_token2] = ACTIONS(1227),
    [aux_sym_implicit_statement_token1] = ACTIONS(1227),
    [aux_sym_implicit_statement_token3] = ACTIONS(1227),
    [aux_sym_implicit_statement_token4] = ACTIONS(1227),
    [aux_sym_save_statement_token1] = ACTIONS(1227),
    [aux_sym_private_statement_token1] = ACTIONS(1227),
    [aux_sym_public_statement_token1] = ACTIONS(1227),
    [aux_sym_namelist_statement_token1] = ACTIONS(1227),
    [aux_sym_common_statement_token1] = ACTIONS(1227),
    [anon_sym_DASH] = ACTIONS(1225),
    [aux_sym_import_statement_token1] = ACTIONS(1227),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1227),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1227),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1227),
    [aux_sym_variable_attributes_token1] = ACTIONS(1227),
    [aux_sym_variable_attributes_token2] = ACTIONS(1227),
    [aux_sym_variable_attributes_token3] = ACTIONS(1227),
    [aux_sym_variable_attributes_token4] = ACTIONS(1227),
    [aux_sym_variable_attributes_token5] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1227),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1227),
    [aux_sym_derived_type_token1] = ACTIONS(1227),
    [aux_sym_type_qualifier_token1] = ACTIONS(1227),
    [aux_sym_type_qualifier_token2] = ACTIONS(1227),
    [aux_sym_type_qualifier_token3] = ACTIONS(1227),
    [aux_sym_type_qualifier_token4] = ACTIONS(1227),
    [aux_sym_type_qualifier_token5] = ACTIONS(1227),
    [aux_sym_type_qualifier_token9] = ACTIONS(1227),
    [aux_sym_type_qualifier_token10] = ACTIONS(1227),
    [aux_sym_type_qualifier_token11] = ACTIONS(1227),
    [aux_sym_type_qualifier_token12] = ACTIONS(1227),
    [aux_sym_type_qualifier_token13] = ACTIONS(1227),
    [aux_sym_type_qualifier_token14] = ACTIONS(1227),
    [aux_sym_type_qualifier_token15] = ACTIONS(1227),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1227),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1227),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1227),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1227),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1227),
    [aux_sym_stop_statement_token1] = ACTIONS(1227),
    [aux_sym_stop_statement_token2] = ACTIONS(1227),
    [aux_sym_subroutine_call_token1] = ACTIONS(1227),
    [aux_sym_keyword_statement_token1] = ACTIONS(1227),
    [aux_sym_keyword_statement_token2] = ACTIONS(1227),
    [aux_sym_keyword_statement_token3] = ACTIONS(1227),
    [aux_sym_keyword_statement_token4] = ACTIONS(1227),
    [aux_sym_keyword_statement_token6] = ACTIONS(1227),
    [aux_sym_keyword_statement_token7] = ACTIONS(1227),
    [aux_sym_include_statement_token1] = ACTIONS(1227),
    [aux_sym_data_statement_token1] = ACTIONS(1227),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1227),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1227),
    [aux_sym_end_if_statement_token1] = ACTIONS(1227),
    [aux_sym_elseif_clause_token2] = ACTIONS(1227),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1227),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1227),
    [aux_sym_select_case_statement_token1] = ACTIONS(1227),
    [aux_sym_select_case_statement_token3] = ACTIONS(1227),
    [aux_sym_select_type_statement_token1] = ACTIONS(1227),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1227),
    [aux_sym_block_construct_token1] = ACTIONS(1227),
    [aux_sym_associate_statement_token1] = ACTIONS(1227),
    [aux_sym_format_statement_token1] = ACTIONS(1227),
    [aux_sym_print_statement_token1] = ACTIONS(1227),
    [aux_sym_open_statement_token1] = ACTIONS(1227),
    [aux_sym_close_statement_token1] = ACTIONS(1227),
    [aux_sym_inquire_statement_token1] = ACTIONS(1227),
    [aux_sym_enum_statement_token1] = ACTIONS(1227),
    [aux_sym_file_position_statement_token1] = ACTIONS(1227),
    [aux_sym_file_position_statement_token2] = ACTIONS(1227),
    [aux_sym_file_position_statement_token3] = ACTIONS(1227),
    [aux_sym_file_position_statement_token4] = ACTIONS(1227),
    [aux_sym_logical_expression_token5] = ACTIONS(1225),
    [anon_sym_PLUS] = ACTIONS(1225),
    [anon_sym_DOT] = ACTIONS(1227),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1225),
    [anon_sym_LBRACK] = ACTIONS(1225),
    [sym_boolean_literal] = ACTIONS(1225),
    [aux_sym_null_literal_token1] = ACTIONS(1227),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1227),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1225),
    [sym__float_literal] = ACTIONS(1225),
    [sym__boz_literal] = ACTIONS(1225),
    [sym_string_literal] = ACTIONS(1225),
  },
  [277] = {
    [sym__statements] = STATE(2483),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(277),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [278] = {
    [sym_preproc_file_line] = STATE(278),
    [ts_builtin_sym_end] = ACTIONS(1229),
    [aux_sym_program_statement_token1] = ACTIONS(1231),
    [aux_sym_end_program_statement_token1] = ACTIONS(1231),
    [aux_sym_end_program_statement_token2] = ACTIONS(1231),
    [aux_sym_module_statement_token1] = ACTIONS(1231),
    [aux_sym_submodule_statement_token1] = ACTIONS(1231),
    [anon_sym_LPAREN] = ACTIONS(1231),
    [aux_sym_interface_statement_token1] = ACTIONS(1231),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1231),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1231),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1231),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1231),
    [aux_sym_function_statement_token1] = ACTIONS(1231),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1231),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1231),
    [aux_sym_contains_statement_token1] = ACTIONS(1231),
    [aux_sym_use_statement_token1] = ACTIONS(1231),
    [aux_sym_use_statement_token2] = ACTIONS(1231),
    [aux_sym_implicit_statement_token1] = ACTIONS(1231),
    [aux_sym_implicit_statement_token3] = ACTIONS(1231),
    [aux_sym_implicit_statement_token4] = ACTIONS(1231),
    [aux_sym_save_statement_token1] = ACTIONS(1231),
    [aux_sym_private_statement_token1] = ACTIONS(1231),
    [aux_sym_public_statement_token1] = ACTIONS(1231),
    [aux_sym_namelist_statement_token1] = ACTIONS(1231),
    [aux_sym_common_statement_token1] = ACTIONS(1231),
    [anon_sym_DASH] = ACTIONS(1229),
    [aux_sym_import_statement_token1] = ACTIONS(1231),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1231),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1231),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1231),
    [aux_sym_variable_attributes_token1] = ACTIONS(1231),
    [aux_sym_variable_attributes_token2] = ACTIONS(1231),
    [aux_sym_variable_attributes_token3] = ACTIONS(1231),
    [aux_sym_variable_attributes_token4] = ACTIONS(1231),
    [aux_sym_variable_attributes_token5] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1231),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1231),
    [aux_sym_derived_type_token1] = ACTIONS(1231),
    [aux_sym_type_qualifier_token1] = ACTIONS(1231),
    [aux_sym_type_qualifier_token2] = ACTIONS(1231),
    [aux_sym_type_qualifier_token3] = ACTIONS(1231),
    [aux_sym_type_qualifier_token4] = ACTIONS(1231),
    [aux_sym_type_qualifier_token5] = ACTIONS(1231),
    [aux_sym_type_qualifier_token9] = ACTIONS(1231),
    [aux_sym_type_qualifier_token10] = ACTIONS(1231),
    [aux_sym_type_qualifier_token11] = ACTIONS(1231),
    [aux_sym_type_qualifier_token12] = ACTIONS(1231),
    [aux_sym_type_qualifier_token13] = ACTIONS(1231),
    [aux_sym_type_qualifier_token14] = ACTIONS(1231),
    [aux_sym_type_qualifier_token15] = ACTIONS(1231),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1231),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1231),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1231),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1231),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1231),
    [aux_sym_stop_statement_token1] = ACTIONS(1231),
    [aux_sym_stop_statement_token2] = ACTIONS(1231),
    [aux_sym_subroutine_call_token1] = ACTIONS(1231),
    [aux_sym_keyword_statement_token1] = ACTIONS(1231),
    [aux_sym_keyword_statement_token2] = ACTIONS(1231),
    [aux_sym_keyword_statement_token3] = ACTIONS(1231),
    [aux_sym_keyword_statement_token4] = ACTIONS(1231),
    [aux_sym_keyword_statement_token6] = ACTIONS(1231),
    [aux_sym_keyword_statement_token7] = ACTIONS(1231),
    [aux_sym_include_statement_token1] = ACTIONS(1231),
    [aux_sym_data_statement_token1] = ACTIONS(1231),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1231),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1231),
    [aux_sym_end_if_statement_token1] = ACTIONS(1231),
    [aux_sym_elseif_clause_token2] = ACTIONS(1231),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1231),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1231),
    [aux_sym_select_case_statement_token1] = ACTIONS(1231),
    [aux_sym_select_case_statement_token3] = ACTIONS(1231),
    [aux_sym_select_type_statement_token1] = ACTIONS(1231),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1231),
    [aux_sym_block_construct_token1] = ACTIONS(1231),
    [aux_sym_associate_statement_token1] = ACTIONS(1231),
    [aux_sym_format_statement_token1] = ACTIONS(1231),
    [aux_sym_print_statement_token1] = ACTIONS(1231),
    [aux_sym_open_statement_token1] = ACTIONS(1231),
    [aux_sym_close_statement_token1] = ACTIONS(1231),
    [aux_sym_inquire_statement_token1] = ACTIONS(1231),
    [aux_sym_enum_statement_token1] = ACTIONS(1231),
    [aux_sym_file_position_statement_token1] = ACTIONS(1231),
    [aux_sym_file_position_statement_token2] = ACTIONS(1231),
    [aux_sym_file_position_statement_token3] = ACTIONS(1231),
    [aux_sym_file_position_statement_token4] = ACTIONS(1231),
    [aux_sym_logical_expression_token5] = ACTIONS(1229),
    [anon_sym_PLUS] = ACTIONS(1229),
    [anon_sym_DOT] = ACTIONS(1231),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1229),
    [anon_sym_LBRACK] = ACTIONS(1229),
    [sym_boolean_literal] = ACTIONS(1229),
    [aux_sym_null_literal_token1] = ACTIONS(1231),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1231),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1229),
    [sym__float_literal] = ACTIONS(1229),
    [sym__boz_literal] = ACTIONS(1229),
    [sym_string_literal] = ACTIONS(1229),
  },
  [279] = {
    [sym_preproc_file_line] = STATE(279),
    [ts_builtin_sym_end] = ACTIONS(1233),
    [aux_sym_program_statement_token1] = ACTIONS(1235),
    [aux_sym_end_program_statement_token1] = ACTIONS(1235),
    [aux_sym_end_program_statement_token2] = ACTIONS(1235),
    [aux_sym_module_statement_token1] = ACTIONS(1235),
    [aux_sym_submodule_statement_token1] = ACTIONS(1235),
    [anon_sym_LPAREN] = ACTIONS(1235),
    [aux_sym_interface_statement_token1] = ACTIONS(1235),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1235),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1235),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1235),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1235),
    [aux_sym_function_statement_token1] = ACTIONS(1235),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1235),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1235),
    [aux_sym_contains_statement_token1] = ACTIONS(1235),
    [aux_sym_use_statement_token1] = ACTIONS(1235),
    [aux_sym_use_statement_token2] = ACTIONS(1235),
    [aux_sym_implicit_statement_token1] = ACTIONS(1235),
    [aux_sym_implicit_statement_token3] = ACTIONS(1235),
    [aux_sym_implicit_statement_token4] = ACTIONS(1235),
    [aux_sym_save_statement_token1] = ACTIONS(1235),
    [aux_sym_private_statement_token1] = ACTIONS(1235),
    [aux_sym_public_statement_token1] = ACTIONS(1235),
    [aux_sym_namelist_statement_token1] = ACTIONS(1235),
    [aux_sym_common_statement_token1] = ACTIONS(1235),
    [anon_sym_DASH] = ACTIONS(1233),
    [aux_sym_import_statement_token1] = ACTIONS(1235),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1235),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1235),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1235),
    [aux_sym_variable_attributes_token1] = ACTIONS(1235),
    [aux_sym_variable_attributes_token2] = ACTIONS(1235),
    [aux_sym_variable_attributes_token3] = ACTIONS(1235),
    [aux_sym_variable_attributes_token4] = ACTIONS(1235),
    [aux_sym_variable_attributes_token5] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1235),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1235),
    [aux_sym_derived_type_token1] = ACTIONS(1235),
    [aux_sym_type_qualifier_token1] = ACTIONS(1235),
    [aux_sym_type_qualifier_token2] = ACTIONS(1235),
    [aux_sym_type_qualifier_token3] = ACTIONS(1235),
    [aux_sym_type_qualifier_token4] = ACTIONS(1235),
    [aux_sym_type_qualifier_token5] = ACTIONS(1235),
    [aux_sym_type_qualifier_token9] = ACTIONS(1235),
    [aux_sym_type_qualifier_token10] = ACTIONS(1235),
    [aux_sym_type_qualifier_token11] = ACTIONS(1235),
    [aux_sym_type_qualifier_token12] = ACTIONS(1235),
    [aux_sym_type_qualifier_token13] = ACTIONS(1235),
    [aux_sym_type_qualifier_token14] = ACTIONS(1235),
    [aux_sym_type_qualifier_token15] = ACTIONS(1235),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1235),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1235),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1235),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1235),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1235),
    [aux_sym_stop_statement_token1] = ACTIONS(1235),
    [aux_sym_stop_statement_token2] = ACTIONS(1235),
    [aux_sym_subroutine_call_token1] = ACTIONS(1235),
    [aux_sym_keyword_statement_token1] = ACTIONS(1235),
    [aux_sym_keyword_statement_token2] = ACTIONS(1235),
    [aux_sym_keyword_statement_token3] = ACTIONS(1235),
    [aux_sym_keyword_statement_token4] = ACTIONS(1235),
    [aux_sym_keyword_statement_token6] = ACTIONS(1235),
    [aux_sym_keyword_statement_token7] = ACTIONS(1235),
    [aux_sym_include_statement_token1] = ACTIONS(1235),
    [aux_sym_data_statement_token1] = ACTIONS(1235),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1235),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1235),
    [aux_sym_end_if_statement_token1] = ACTIONS(1235),
    [aux_sym_elseif_clause_token2] = ACTIONS(1235),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1235),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1235),
    [aux_sym_select_case_statement_token1] = ACTIONS(1235),
    [aux_sym_select_case_statement_token3] = ACTIONS(1235),
    [aux_sym_select_type_statement_token1] = ACTIONS(1235),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1235),
    [aux_sym_block_construct_token1] = ACTIONS(1235),
    [aux_sym_associate_statement_token1] = ACTIONS(1235),
    [aux_sym_format_statement_token1] = ACTIONS(1235),
    [aux_sym_print_statement_token1] = ACTIONS(1235),
    [aux_sym_open_statement_token1] = ACTIONS(1235),
    [aux_sym_close_statement_token1] = ACTIONS(1235),
    [aux_sym_inquire_statement_token1] = ACTIONS(1235),
    [aux_sym_enum_statement_token1] = ACTIONS(1235),
    [aux_sym_file_position_statement_token1] = ACTIONS(1235),
    [aux_sym_file_position_statement_token2] = ACTIONS(1235),
    [aux_sym_file_position_statement_token3] = ACTIONS(1235),
    [aux_sym_file_position_statement_token4] = ACTIONS(1235),
    [aux_sym_logical_expression_token5] = ACTIONS(1233),
    [anon_sym_PLUS] = ACTIONS(1233),
    [anon_sym_DOT] = ACTIONS(1235),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1233),
    [anon_sym_LBRACK] = ACTIONS(1233),
    [sym_boolean_literal] = ACTIONS(1233),
    [aux_sym_null_literal_token1] = ACTIONS(1235),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1235),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1233),
    [sym__float_literal] = ACTIONS(1233),
    [sym__boz_literal] = ACTIONS(1233),
    [sym_string_literal] = ACTIONS(1233),
  },
  [280] = {
    [sym_preproc_file_line] = STATE(280),
    [ts_builtin_sym_end] = ACTIONS(1237),
    [aux_sym_program_statement_token1] = ACTIONS(1239),
    [aux_sym_end_program_statement_token1] = ACTIONS(1239),
    [aux_sym_end_program_statement_token2] = ACTIONS(1239),
    [aux_sym_module_statement_token1] = ACTIONS(1239),
    [aux_sym_submodule_statement_token1] = ACTIONS(1239),
    [anon_sym_LPAREN] = ACTIONS(1239),
    [aux_sym_interface_statement_token1] = ACTIONS(1239),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1239),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1239),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1239),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1239),
    [aux_sym_function_statement_token1] = ACTIONS(1239),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1239),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1239),
    [aux_sym_contains_statement_token1] = ACTIONS(1239),
    [aux_sym_use_statement_token1] = ACTIONS(1239),
    [aux_sym_use_statement_token2] = ACTIONS(1239),
    [aux_sym_implicit_statement_token1] = ACTIONS(1239),
    [aux_sym_implicit_statement_token3] = ACTIONS(1239),
    [aux_sym_implicit_statement_token4] = ACTIONS(1239),
    [aux_sym_save_statement_token1] = ACTIONS(1239),
    [aux_sym_private_statement_token1] = ACTIONS(1239),
    [aux_sym_public_statement_token1] = ACTIONS(1239),
    [aux_sym_namelist_statement_token1] = ACTIONS(1239),
    [aux_sym_common_statement_token1] = ACTIONS(1239),
    [anon_sym_DASH] = ACTIONS(1237),
    [aux_sym_import_statement_token1] = ACTIONS(1239),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1239),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1239),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1239),
    [aux_sym_variable_attributes_token1] = ACTIONS(1239),
    [aux_sym_variable_attributes_token2] = ACTIONS(1239),
    [aux_sym_variable_attributes_token3] = ACTIONS(1239),
    [aux_sym_variable_attributes_token4] = ACTIONS(1239),
    [aux_sym_variable_attributes_token5] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1239),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1239),
    [aux_sym_derived_type_token1] = ACTIONS(1239),
    [aux_sym_type_qualifier_token1] = ACTIONS(1239),
    [aux_sym_type_qualifier_token2] = ACTIONS(1239),
    [aux_sym_type_qualifier_token3] = ACTIONS(1239),
    [aux_sym_type_qualifier_token4] = ACTIONS(1239),
    [aux_sym_type_qualifier_token5] = ACTIONS(1239),
    [aux_sym_type_qualifier_token9] = ACTIONS(1239),
    [aux_sym_type_qualifier_token10] = ACTIONS(1239),
    [aux_sym_type_qualifier_token11] = ACTIONS(1239),
    [aux_sym_type_qualifier_token12] = ACTIONS(1239),
    [aux_sym_type_qualifier_token13] = ACTIONS(1239),
    [aux_sym_type_qualifier_token14] = ACTIONS(1239),
    [aux_sym_type_qualifier_token15] = ACTIONS(1239),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1239),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1239),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1239),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1239),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1239),
    [aux_sym_stop_statement_token1] = ACTIONS(1239),
    [aux_sym_stop_statement_token2] = ACTIONS(1239),
    [aux_sym_subroutine_call_token1] = ACTIONS(1239),
    [aux_sym_keyword_statement_token1] = ACTIONS(1239),
    [aux_sym_keyword_statement_token2] = ACTIONS(1239),
    [aux_sym_keyword_statement_token3] = ACTIONS(1239),
    [aux_sym_keyword_statement_token4] = ACTIONS(1239),
    [aux_sym_keyword_statement_token6] = ACTIONS(1239),
    [aux_sym_keyword_statement_token7] = ACTIONS(1239),
    [aux_sym_include_statement_token1] = ACTIONS(1239),
    [aux_sym_data_statement_token1] = ACTIONS(1239),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1239),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1239),
    [aux_sym_end_if_statement_token1] = ACTIONS(1239),
    [aux_sym_elseif_clause_token2] = ACTIONS(1239),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1239),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1239),
    [aux_sym_select_case_statement_token1] = ACTIONS(1239),
    [aux_sym_select_case_statement_token3] = ACTIONS(1239),
    [aux_sym_select_type_statement_token1] = ACTIONS(1239),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1239),
    [aux_sym_block_construct_token1] = ACTIONS(1239),
    [aux_sym_associate_statement_token1] = ACTIONS(1239),
    [aux_sym_format_statement_token1] = ACTIONS(1239),
    [aux_sym_print_statement_token1] = ACTIONS(1239),
    [aux_sym_open_statement_token1] = ACTIONS(1239),
    [aux_sym_close_statement_token1] = ACTIONS(1239),
    [aux_sym_inquire_statement_token1] = ACTIONS(1239),
    [aux_sym_enum_statement_token1] = ACTIONS(1239),
    [aux_sym_file_position_statement_token1] = ACTIONS(1239),
    [aux_sym_file_position_statement_token2] = ACTIONS(1239),
    [aux_sym_file_position_statement_token3] = ACTIONS(1239),
    [aux_sym_file_position_statement_token4] = ACTIONS(1239),
    [aux_sym_logical_expression_token5] = ACTIONS(1237),
    [anon_sym_PLUS] = ACTIONS(1237),
    [anon_sym_DOT] = ACTIONS(1239),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1237),
    [anon_sym_LBRACK] = ACTIONS(1237),
    [sym_boolean_literal] = ACTIONS(1237),
    [aux_sym_null_literal_token1] = ACTIONS(1239),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1239),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1237),
    [sym__float_literal] = ACTIONS(1237),
    [sym__boz_literal] = ACTIONS(1237),
    [sym_string_literal] = ACTIONS(1237),
  },
  [281] = {
    [sym__statements] = STATE(2399),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(281),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [282] = {
    [sym_preproc_file_line] = STATE(282),
    [ts_builtin_sym_end] = ACTIONS(1241),
    [aux_sym_program_statement_token1] = ACTIONS(1243),
    [aux_sym_end_program_statement_token1] = ACTIONS(1243),
    [aux_sym_end_program_statement_token2] = ACTIONS(1243),
    [aux_sym_module_statement_token1] = ACTIONS(1243),
    [aux_sym_submodule_statement_token1] = ACTIONS(1243),
    [anon_sym_LPAREN] = ACTIONS(1243),
    [aux_sym_interface_statement_token1] = ACTIONS(1243),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1243),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1243),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1243),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1243),
    [aux_sym_function_statement_token1] = ACTIONS(1243),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1243),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1243),
    [aux_sym_contains_statement_token1] = ACTIONS(1243),
    [aux_sym_use_statement_token1] = ACTIONS(1243),
    [aux_sym_use_statement_token2] = ACTIONS(1243),
    [aux_sym_implicit_statement_token1] = ACTIONS(1243),
    [aux_sym_implicit_statement_token3] = ACTIONS(1243),
    [aux_sym_implicit_statement_token4] = ACTIONS(1243),
    [aux_sym_save_statement_token1] = ACTIONS(1243),
    [aux_sym_private_statement_token1] = ACTIONS(1243),
    [aux_sym_public_statement_token1] = ACTIONS(1243),
    [aux_sym_namelist_statement_token1] = ACTIONS(1243),
    [aux_sym_common_statement_token1] = ACTIONS(1243),
    [anon_sym_DASH] = ACTIONS(1241),
    [aux_sym_import_statement_token1] = ACTIONS(1243),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1243),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1243),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1243),
    [aux_sym_variable_attributes_token1] = ACTIONS(1243),
    [aux_sym_variable_attributes_token2] = ACTIONS(1243),
    [aux_sym_variable_attributes_token3] = ACTIONS(1243),
    [aux_sym_variable_attributes_token4] = ACTIONS(1243),
    [aux_sym_variable_attributes_token5] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1243),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1243),
    [aux_sym_derived_type_token1] = ACTIONS(1243),
    [aux_sym_type_qualifier_token1] = ACTIONS(1243),
    [aux_sym_type_qualifier_token2] = ACTIONS(1243),
    [aux_sym_type_qualifier_token3] = ACTIONS(1243),
    [aux_sym_type_qualifier_token4] = ACTIONS(1243),
    [aux_sym_type_qualifier_token5] = ACTIONS(1243),
    [aux_sym_type_qualifier_token9] = ACTIONS(1243),
    [aux_sym_type_qualifier_token10] = ACTIONS(1243),
    [aux_sym_type_qualifier_token11] = ACTIONS(1243),
    [aux_sym_type_qualifier_token12] = ACTIONS(1243),
    [aux_sym_type_qualifier_token13] = ACTIONS(1243),
    [aux_sym_type_qualifier_token14] = ACTIONS(1243),
    [aux_sym_type_qualifier_token15] = ACTIONS(1243),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1243),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1243),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1243),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1243),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1243),
    [aux_sym_stop_statement_token1] = ACTIONS(1243),
    [aux_sym_stop_statement_token2] = ACTIONS(1243),
    [aux_sym_subroutine_call_token1] = ACTIONS(1243),
    [aux_sym_keyword_statement_token1] = ACTIONS(1243),
    [aux_sym_keyword_statement_token2] = ACTIONS(1243),
    [aux_sym_keyword_statement_token3] = ACTIONS(1243),
    [aux_sym_keyword_statement_token4] = ACTIONS(1243),
    [aux_sym_keyword_statement_token6] = ACTIONS(1243),
    [aux_sym_keyword_statement_token7] = ACTIONS(1243),
    [aux_sym_include_statement_token1] = ACTIONS(1243),
    [aux_sym_data_statement_token1] = ACTIONS(1243),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1243),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1243),
    [aux_sym_end_if_statement_token1] = ACTIONS(1243),
    [aux_sym_elseif_clause_token2] = ACTIONS(1243),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1243),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1243),
    [aux_sym_select_case_statement_token1] = ACTIONS(1243),
    [aux_sym_select_case_statement_token3] = ACTIONS(1243),
    [aux_sym_select_type_statement_token1] = ACTIONS(1243),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1243),
    [aux_sym_block_construct_token1] = ACTIONS(1243),
    [aux_sym_associate_statement_token1] = ACTIONS(1243),
    [aux_sym_format_statement_token1] = ACTIONS(1243),
    [aux_sym_print_statement_token1] = ACTIONS(1243),
    [aux_sym_open_statement_token1] = ACTIONS(1243),
    [aux_sym_close_statement_token1] = ACTIONS(1243),
    [aux_sym_inquire_statement_token1] = ACTIONS(1243),
    [aux_sym_enum_statement_token1] = ACTIONS(1243),
    [aux_sym_file_position_statement_token1] = ACTIONS(1243),
    [aux_sym_file_position_statement_token2] = ACTIONS(1243),
    [aux_sym_file_position_statement_token3] = ACTIONS(1243),
    [aux_sym_file_position_statement_token4] = ACTIONS(1243),
    [aux_sym_logical_expression_token5] = ACTIONS(1241),
    [anon_sym_PLUS] = ACTIONS(1241),
    [anon_sym_DOT] = ACTIONS(1243),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1241),
    [anon_sym_LBRACK] = ACTIONS(1241),
    [sym_boolean_literal] = ACTIONS(1241),
    [aux_sym_null_literal_token1] = ACTIONS(1243),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1243),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1241),
    [sym__float_literal] = ACTIONS(1241),
    [sym__boz_literal] = ACTIONS(1241),
    [sym_string_literal] = ACTIONS(1241),
  },
  [283] = {
    [sym_preproc_file_line] = STATE(283),
    [ts_builtin_sym_end] = ACTIONS(1217),
    [aux_sym_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token2] = ACTIONS(1219),
    [aux_sym_module_statement_token1] = ACTIONS(1219),
    [aux_sym_submodule_statement_token1] = ACTIONS(1219),
    [anon_sym_LPAREN] = ACTIONS(1219),
    [aux_sym_interface_statement_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1219),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1219),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1219),
    [aux_sym_function_statement_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1219),
    [aux_sym_contains_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token2] = ACTIONS(1219),
    [aux_sym_implicit_statement_token1] = ACTIONS(1219),
    [aux_sym_implicit_statement_token3] = ACTIONS(1219),
    [aux_sym_implicit_statement_token4] = ACTIONS(1219),
    [aux_sym_save_statement_token1] = ACTIONS(1219),
    [aux_sym_private_statement_token1] = ACTIONS(1219),
    [aux_sym_public_statement_token1] = ACTIONS(1219),
    [aux_sym_namelist_statement_token1] = ACTIONS(1219),
    [aux_sym_common_statement_token1] = ACTIONS(1219),
    [anon_sym_DASH] = ACTIONS(1217),
    [aux_sym_import_statement_token1] = ACTIONS(1219),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1219),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1219),
    [aux_sym_variable_attributes_token1] = ACTIONS(1219),
    [aux_sym_variable_attributes_token2] = ACTIONS(1219),
    [aux_sym_variable_attributes_token3] = ACTIONS(1219),
    [aux_sym_variable_attributes_token4] = ACTIONS(1219),
    [aux_sym_variable_attributes_token5] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1219),
    [aux_sym_derived_type_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token2] = ACTIONS(1219),
    [aux_sym_type_qualifier_token3] = ACTIONS(1219),
    [aux_sym_type_qualifier_token4] = ACTIONS(1219),
    [aux_sym_type_qualifier_token5] = ACTIONS(1219),
    [aux_sym_type_qualifier_token9] = ACTIONS(1219),
    [aux_sym_type_qualifier_token10] = ACTIONS(1219),
    [aux_sym_type_qualifier_token11] = ACTIONS(1219),
    [aux_sym_type_qualifier_token12] = ACTIONS(1219),
    [aux_sym_type_qualifier_token13] = ACTIONS(1219),
    [aux_sym_type_qualifier_token14] = ACTIONS(1219),
    [aux_sym_type_qualifier_token15] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1219),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token2] = ACTIONS(1219),
    [aux_sym_subroutine_call_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token2] = ACTIONS(1219),
    [aux_sym_keyword_statement_token3] = ACTIONS(1219),
    [aux_sym_keyword_statement_token4] = ACTIONS(1219),
    [aux_sym_keyword_statement_token6] = ACTIONS(1219),
    [aux_sym_keyword_statement_token7] = ACTIONS(1219),
    [aux_sym_include_statement_token1] = ACTIONS(1219),
    [aux_sym_data_statement_token1] = ACTIONS(1219),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1219),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1219),
    [aux_sym_end_if_statement_token1] = ACTIONS(1219),
    [aux_sym_elseif_clause_token2] = ACTIONS(1219),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1219),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token3] = ACTIONS(1219),
    [aux_sym_select_type_statement_token1] = ACTIONS(1219),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1219),
    [aux_sym_block_construct_token1] = ACTIONS(1219),
    [aux_sym_associate_statement_token1] = ACTIONS(1219),
    [aux_sym_format_statement_token1] = ACTIONS(1219),
    [aux_sym_print_statement_token1] = ACTIONS(1219),
    [aux_sym_open_statement_token1] = ACTIONS(1219),
    [aux_sym_close_statement_token1] = ACTIONS(1219),
    [aux_sym_inquire_statement_token1] = ACTIONS(1219),
    [aux_sym_enum_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token2] = ACTIONS(1219),
    [aux_sym_file_position_statement_token3] = ACTIONS(1219),
    [aux_sym_file_position_statement_token4] = ACTIONS(1219),
    [aux_sym_logical_expression_token5] = ACTIONS(1217),
    [anon_sym_PLUS] = ACTIONS(1217),
    [anon_sym_DOT] = ACTIONS(1219),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1217),
    [anon_sym_LBRACK] = ACTIONS(1217),
    [sym_boolean_literal] = ACTIONS(1217),
    [aux_sym_null_literal_token1] = ACTIONS(1219),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1219),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1217),
    [sym__float_literal] = ACTIONS(1217),
    [sym__boz_literal] = ACTIONS(1217),
    [sym_string_literal] = ACTIONS(1217),
  },
  [284] = {
    [sym_preproc_file_line] = STATE(284),
    [ts_builtin_sym_end] = ACTIONS(1245),
    [aux_sym_program_statement_token1] = ACTIONS(1247),
    [aux_sym_end_program_statement_token1] = ACTIONS(1247),
    [aux_sym_end_program_statement_token2] = ACTIONS(1247),
    [aux_sym_module_statement_token1] = ACTIONS(1247),
    [aux_sym_submodule_statement_token1] = ACTIONS(1247),
    [anon_sym_LPAREN] = ACTIONS(1247),
    [aux_sym_interface_statement_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1247),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1247),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_function_statement_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1247),
    [aux_sym_contains_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token2] = ACTIONS(1247),
    [aux_sym_implicit_statement_token1] = ACTIONS(1247),
    [aux_sym_implicit_statement_token3] = ACTIONS(1247),
    [aux_sym_implicit_statement_token4] = ACTIONS(1247),
    [aux_sym_save_statement_token1] = ACTIONS(1247),
    [aux_sym_private_statement_token1] = ACTIONS(1247),
    [aux_sym_public_statement_token1] = ACTIONS(1247),
    [aux_sym_namelist_statement_token1] = ACTIONS(1247),
    [aux_sym_common_statement_token1] = ACTIONS(1247),
    [anon_sym_DASH] = ACTIONS(1245),
    [aux_sym_import_statement_token1] = ACTIONS(1247),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1247),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1247),
    [aux_sym_variable_attributes_token1] = ACTIONS(1247),
    [aux_sym_variable_attributes_token2] = ACTIONS(1247),
    [aux_sym_variable_attributes_token3] = ACTIONS(1247),
    [aux_sym_variable_attributes_token4] = ACTIONS(1247),
    [aux_sym_variable_attributes_token5] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1247),
    [aux_sym_derived_type_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token2] = ACTIONS(1247),
    [aux_sym_type_qualifier_token3] = ACTIONS(1247),
    [aux_sym_type_qualifier_token4] = ACTIONS(1247),
    [aux_sym_type_qualifier_token5] = ACTIONS(1247),
    [aux_sym_type_qualifier_token9] = ACTIONS(1247),
    [aux_sym_type_qualifier_token10] = ACTIONS(1247),
    [aux_sym_type_qualifier_token11] = ACTIONS(1247),
    [aux_sym_type_qualifier_token12] = ACTIONS(1247),
    [aux_sym_type_qualifier_token13] = ACTIONS(1247),
    [aux_sym_type_qualifier_token14] = ACTIONS(1247),
    [aux_sym_type_qualifier_token15] = ACTIONS(1247),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1247),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1247),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1247),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token2] = ACTIONS(1247),
    [aux_sym_subroutine_call_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token2] = ACTIONS(1247),
    [aux_sym_keyword_statement_token3] = ACTIONS(1247),
    [aux_sym_keyword_statement_token4] = ACTIONS(1247),
    [aux_sym_keyword_statement_token6] = ACTIONS(1247),
    [aux_sym_keyword_statement_token7] = ACTIONS(1247),
    [aux_sym_include_statement_token1] = ACTIONS(1247),
    [aux_sym_data_statement_token1] = ACTIONS(1247),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1247),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1247),
    [aux_sym_end_if_statement_token1] = ACTIONS(1247),
    [aux_sym_elseif_clause_token2] = ACTIONS(1247),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1247),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token3] = ACTIONS(1247),
    [aux_sym_select_type_statement_token1] = ACTIONS(1247),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1247),
    [aux_sym_block_construct_token1] = ACTIONS(1247),
    [aux_sym_associate_statement_token1] = ACTIONS(1247),
    [aux_sym_format_statement_token1] = ACTIONS(1247),
    [aux_sym_print_statement_token1] = ACTIONS(1247),
    [aux_sym_open_statement_token1] = ACTIONS(1247),
    [aux_sym_close_statement_token1] = ACTIONS(1247),
    [aux_sym_inquire_statement_token1] = ACTIONS(1247),
    [aux_sym_enum_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token2] = ACTIONS(1247),
    [aux_sym_file_position_statement_token3] = ACTIONS(1247),
    [aux_sym_file_position_statement_token4] = ACTIONS(1247),
    [aux_sym_logical_expression_token5] = ACTIONS(1245),
    [anon_sym_PLUS] = ACTIONS(1245),
    [anon_sym_DOT] = ACTIONS(1247),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1245),
    [anon_sym_LBRACK] = ACTIONS(1245),
    [sym_boolean_literal] = ACTIONS(1245),
    [aux_sym_null_literal_token1] = ACTIONS(1247),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1247),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1245),
    [sym__float_literal] = ACTIONS(1245),
    [sym__boz_literal] = ACTIONS(1245),
    [sym_string_literal] = ACTIONS(1245),
  },
  [285] = {
    [sym_preproc_file_line] = STATE(285),
    [ts_builtin_sym_end] = ACTIONS(1249),
    [aux_sym_program_statement_token1] = ACTIONS(1251),
    [aux_sym_end_program_statement_token1] = ACTIONS(1251),
    [aux_sym_end_program_statement_token2] = ACTIONS(1251),
    [aux_sym_module_statement_token1] = ACTIONS(1251),
    [aux_sym_submodule_statement_token1] = ACTIONS(1251),
    [anon_sym_LPAREN] = ACTIONS(1251),
    [aux_sym_interface_statement_token1] = ACTIONS(1251),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1251),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1251),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1251),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1251),
    [aux_sym_function_statement_token1] = ACTIONS(1251),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1251),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1251),
    [aux_sym_contains_statement_token1] = ACTIONS(1251),
    [aux_sym_use_statement_token1] = ACTIONS(1251),
    [aux_sym_use_statement_token2] = ACTIONS(1251),
    [aux_sym_implicit_statement_token1] = ACTIONS(1251),
    [aux_sym_implicit_statement_token3] = ACTIONS(1251),
    [aux_sym_implicit_statement_token4] = ACTIONS(1251),
    [aux_sym_save_statement_token1] = ACTIONS(1251),
    [aux_sym_private_statement_token1] = ACTIONS(1251),
    [aux_sym_public_statement_token1] = ACTIONS(1251),
    [aux_sym_namelist_statement_token1] = ACTIONS(1251),
    [aux_sym_common_statement_token1] = ACTIONS(1251),
    [anon_sym_DASH] = ACTIONS(1249),
    [aux_sym_import_statement_token1] = ACTIONS(1251),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1251),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1251),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1251),
    [aux_sym_variable_attributes_token1] = ACTIONS(1251),
    [aux_sym_variable_attributes_token2] = ACTIONS(1251),
    [aux_sym_variable_attributes_token3] = ACTIONS(1251),
    [aux_sym_variable_attributes_token4] = ACTIONS(1251),
    [aux_sym_variable_attributes_token5] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1251),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1251),
    [aux_sym_derived_type_token1] = ACTIONS(1251),
    [aux_sym_type_qualifier_token1] = ACTIONS(1251),
    [aux_sym_type_qualifier_token2] = ACTIONS(1251),
    [aux_sym_type_qualifier_token3] = ACTIONS(1251),
    [aux_sym_type_qualifier_token4] = ACTIONS(1251),
    [aux_sym_type_qualifier_token5] = ACTIONS(1251),
    [aux_sym_type_qualifier_token9] = ACTIONS(1251),
    [aux_sym_type_qualifier_token10] = ACTIONS(1251),
    [aux_sym_type_qualifier_token11] = ACTIONS(1251),
    [aux_sym_type_qualifier_token12] = ACTIONS(1251),
    [aux_sym_type_qualifier_token13] = ACTIONS(1251),
    [aux_sym_type_qualifier_token14] = ACTIONS(1251),
    [aux_sym_type_qualifier_token15] = ACTIONS(1251),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1251),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1251),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1251),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1251),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1251),
    [aux_sym_stop_statement_token1] = ACTIONS(1251),
    [aux_sym_stop_statement_token2] = ACTIONS(1251),
    [aux_sym_subroutine_call_token1] = ACTIONS(1251),
    [aux_sym_keyword_statement_token1] = ACTIONS(1251),
    [aux_sym_keyword_statement_token2] = ACTIONS(1251),
    [aux_sym_keyword_statement_token3] = ACTIONS(1251),
    [aux_sym_keyword_statement_token4] = ACTIONS(1251),
    [aux_sym_keyword_statement_token6] = ACTIONS(1251),
    [aux_sym_keyword_statement_token7] = ACTIONS(1251),
    [aux_sym_include_statement_token1] = ACTIONS(1251),
    [aux_sym_data_statement_token1] = ACTIONS(1251),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1251),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1251),
    [aux_sym_end_if_statement_token1] = ACTIONS(1251),
    [aux_sym_elseif_clause_token2] = ACTIONS(1251),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1251),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1251),
    [aux_sym_select_case_statement_token1] = ACTIONS(1251),
    [aux_sym_select_case_statement_token3] = ACTIONS(1251),
    [aux_sym_select_type_statement_token1] = ACTIONS(1251),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1251),
    [aux_sym_block_construct_token1] = ACTIONS(1251),
    [aux_sym_associate_statement_token1] = ACTIONS(1251),
    [aux_sym_format_statement_token1] = ACTIONS(1251),
    [aux_sym_print_statement_token1] = ACTIONS(1251),
    [aux_sym_open_statement_token1] = ACTIONS(1251),
    [aux_sym_close_statement_token1] = ACTIONS(1251),
    [aux_sym_inquire_statement_token1] = ACTIONS(1251),
    [aux_sym_enum_statement_token1] = ACTIONS(1251),
    [aux_sym_file_position_statement_token1] = ACTIONS(1251),
    [aux_sym_file_position_statement_token2] = ACTIONS(1251),
    [aux_sym_file_position_statement_token3] = ACTIONS(1251),
    [aux_sym_file_position_statement_token4] = ACTIONS(1251),
    [aux_sym_logical_expression_token5] = ACTIONS(1249),
    [anon_sym_PLUS] = ACTIONS(1249),
    [anon_sym_DOT] = ACTIONS(1251),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1249),
    [anon_sym_LBRACK] = ACTIONS(1249),
    [sym_boolean_literal] = ACTIONS(1249),
    [aux_sym_null_literal_token1] = ACTIONS(1251),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1251),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1249),
    [sym__float_literal] = ACTIONS(1249),
    [sym__boz_literal] = ACTIONS(1249),
    [sym_string_literal] = ACTIONS(1249),
  },
  [286] = {
    [sym_preproc_file_line] = STATE(286),
    [ts_builtin_sym_end] = ACTIONS(1253),
    [aux_sym_program_statement_token1] = ACTIONS(1255),
    [aux_sym_end_program_statement_token1] = ACTIONS(1255),
    [aux_sym_end_program_statement_token2] = ACTIONS(1255),
    [aux_sym_module_statement_token1] = ACTIONS(1255),
    [aux_sym_submodule_statement_token1] = ACTIONS(1255),
    [anon_sym_LPAREN] = ACTIONS(1255),
    [aux_sym_interface_statement_token1] = ACTIONS(1255),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1255),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1255),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1255),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1255),
    [aux_sym_function_statement_token1] = ACTIONS(1255),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1255),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1255),
    [aux_sym_contains_statement_token1] = ACTIONS(1255),
    [aux_sym_use_statement_token1] = ACTIONS(1255),
    [aux_sym_use_statement_token2] = ACTIONS(1255),
    [aux_sym_implicit_statement_token1] = ACTIONS(1255),
    [aux_sym_implicit_statement_token3] = ACTIONS(1255),
    [aux_sym_implicit_statement_token4] = ACTIONS(1255),
    [aux_sym_save_statement_token1] = ACTIONS(1255),
    [aux_sym_private_statement_token1] = ACTIONS(1255),
    [aux_sym_public_statement_token1] = ACTIONS(1255),
    [aux_sym_namelist_statement_token1] = ACTIONS(1255),
    [aux_sym_common_statement_token1] = ACTIONS(1255),
    [anon_sym_DASH] = ACTIONS(1253),
    [aux_sym_import_statement_token1] = ACTIONS(1255),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1255),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1255),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1255),
    [aux_sym_variable_attributes_token1] = ACTIONS(1255),
    [aux_sym_variable_attributes_token2] = ACTIONS(1255),
    [aux_sym_variable_attributes_token3] = ACTIONS(1255),
    [aux_sym_variable_attributes_token4] = ACTIONS(1255),
    [aux_sym_variable_attributes_token5] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1255),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1255),
    [aux_sym_derived_type_token1] = ACTIONS(1255),
    [aux_sym_type_qualifier_token1] = ACTIONS(1255),
    [aux_sym_type_qualifier_token2] = ACTIONS(1255),
    [aux_sym_type_qualifier_token3] = ACTIONS(1255),
    [aux_sym_type_qualifier_token4] = ACTIONS(1255),
    [aux_sym_type_qualifier_token5] = ACTIONS(1255),
    [aux_sym_type_qualifier_token9] = ACTIONS(1255),
    [aux_sym_type_qualifier_token10] = ACTIONS(1255),
    [aux_sym_type_qualifier_token11] = ACTIONS(1255),
    [aux_sym_type_qualifier_token12] = ACTIONS(1255),
    [aux_sym_type_qualifier_token13] = ACTIONS(1255),
    [aux_sym_type_qualifier_token14] = ACTIONS(1255),
    [aux_sym_type_qualifier_token15] = ACTIONS(1255),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1255),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1255),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1255),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1255),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1255),
    [aux_sym_stop_statement_token1] = ACTIONS(1255),
    [aux_sym_stop_statement_token2] = ACTIONS(1255),
    [aux_sym_subroutine_call_token1] = ACTIONS(1255),
    [aux_sym_keyword_statement_token1] = ACTIONS(1255),
    [aux_sym_keyword_statement_token2] = ACTIONS(1255),
    [aux_sym_keyword_statement_token3] = ACTIONS(1255),
    [aux_sym_keyword_statement_token4] = ACTIONS(1255),
    [aux_sym_keyword_statement_token6] = ACTIONS(1255),
    [aux_sym_keyword_statement_token7] = ACTIONS(1255),
    [aux_sym_include_statement_token1] = ACTIONS(1255),
    [aux_sym_data_statement_token1] = ACTIONS(1255),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1255),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1255),
    [aux_sym_end_if_statement_token1] = ACTIONS(1255),
    [aux_sym_elseif_clause_token2] = ACTIONS(1255),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1255),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1255),
    [aux_sym_select_case_statement_token1] = ACTIONS(1255),
    [aux_sym_select_case_statement_token3] = ACTIONS(1255),
    [aux_sym_select_type_statement_token1] = ACTIONS(1255),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1255),
    [aux_sym_block_construct_token1] = ACTIONS(1255),
    [aux_sym_associate_statement_token1] = ACTIONS(1255),
    [aux_sym_format_statement_token1] = ACTIONS(1255),
    [aux_sym_print_statement_token1] = ACTIONS(1255),
    [aux_sym_open_statement_token1] = ACTIONS(1255),
    [aux_sym_close_statement_token1] = ACTIONS(1255),
    [aux_sym_inquire_statement_token1] = ACTIONS(1255),
    [aux_sym_enum_statement_token1] = ACTIONS(1255),
    [aux_sym_file_position_statement_token1] = ACTIONS(1255),
    [aux_sym_file_position_statement_token2] = ACTIONS(1255),
    [aux_sym_file_position_statement_token3] = ACTIONS(1255),
    [aux_sym_file_position_statement_token4] = ACTIONS(1255),
    [aux_sym_logical_expression_token5] = ACTIONS(1253),
    [anon_sym_PLUS] = ACTIONS(1253),
    [anon_sym_DOT] = ACTIONS(1255),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1253),
    [anon_sym_LBRACK] = ACTIONS(1253),
    [sym_boolean_literal] = ACTIONS(1253),
    [aux_sym_null_literal_token1] = ACTIONS(1255),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1255),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1253),
    [sym__float_literal] = ACTIONS(1253),
    [sym__boz_literal] = ACTIONS(1253),
    [sym_string_literal] = ACTIONS(1253),
  },
  [287] = {
    [sym__statements] = STATE(2503),
    [sym_stop_statement] = STATE(2787),
    [sym_assignment_statement] = STATE(2787),
    [sym_pointer_association_statement] = STATE(2787),
    [sym_subroutine_call] = STATE(2787),
    [sym_keyword_statement] = STATE(2787),
    [sym_include_statement] = STATE(2787),
    [sym_do_loop_statement] = STATE(2787),
    [sym_do_label_statement] = STATE(2787),
    [sym_if_statement] = STATE(2787),
    [sym__inline_if_statement] = STATE(2783),
    [sym_arithmetic_if_statement] = STATE(2787),
    [sym__block_if_statement] = STATE(2783),
    [sym_where_statement] = STATE(2787),
    [sym__inline_where_statement] = STATE(2782),
    [sym__block_where_statement] = STATE(2782),
    [sym_forall_statement] = STATE(2787),
    [sym__forall_control_expression] = STATE(222),
    [sym__inline_forall_statement] = STATE(2780),
    [sym__block_forall_statement] = STATE(2780),
    [sym_select_case_statement] = STATE(2787),
    [sym_select_type_statement] = STATE(2787),
    [sym_select_rank_statement] = STATE(2787),
    [sym_block_construct] = STATE(2787),
    [sym_associate_statement] = STATE(2787),
    [sym_format_statement] = STATE(2787),
    [sym_read_statement] = STATE(2787),
    [sym__simple_read_statement] = STATE(2777),
    [sym__parameterized_read_statement] = STATE(2777),
    [sym_print_statement] = STATE(2787),
    [sym_open_statement] = STATE(2787),
    [sym_close_statement] = STATE(2787),
    [sym_write_statement] = STATE(2787),
    [sym_inquire_statement] = STATE(2787),
    [sym_file_position_statement] = STATE(2787),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1149),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_block_label_start_expression] = STATE(1744),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(287),
    [sym_identifier] = STATE(1148),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(23),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(25),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(85),
    [aux_sym_stop_statement_token2] = ACTIONS(87),
    [aux_sym_subroutine_call_token1] = ACTIONS(89),
    [aux_sym_keyword_statement_token1] = ACTIONS(91),
    [aux_sym_keyword_statement_token2] = ACTIONS(93),
    [aux_sym_keyword_statement_token3] = ACTIONS(93),
    [aux_sym_keyword_statement_token4] = ACTIONS(95),
    [aux_sym_keyword_statement_token6] = ACTIONS(97),
    [aux_sym_keyword_statement_token7] = ACTIONS(91),
    [aux_sym_include_statement_token1] = ACTIONS(99),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym_do_loop_statement_token1] = ACTIONS(103),
    [aux_sym__inline_if_statement_token1] = ACTIONS(105),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym__inline_where_statement_token1] = ACTIONS(109),
    [aux_sym__forall_control_expression_token1] = ACTIONS(111),
    [aux_sym_select_case_statement_token1] = ACTIONS(113),
    [aux_sym_select_case_statement_token3] = ACTIONS(115),
    [aux_sym_select_type_statement_token1] = ACTIONS(117),
    [aux_sym_select_rank_statement_token2] = ACTIONS(119),
    [aux_sym_block_construct_token1] = ACTIONS(121),
    [aux_sym_associate_statement_token1] = ACTIONS(123),
    [aux_sym_format_statement_token1] = ACTIONS(125),
    [aux_sym_print_statement_token1] = ACTIONS(127),
    [aux_sym_open_statement_token1] = ACTIONS(129),
    [aux_sym_close_statement_token1] = ACTIONS(131),
    [aux_sym_inquire_statement_token1] = ACTIONS(133),
    [aux_sym_file_position_statement_token1] = ACTIONS(137),
    [aux_sym_file_position_statement_token2] = ACTIONS(137),
    [aux_sym_file_position_statement_token3] = ACTIONS(137),
    [aux_sym_file_position_statement_token4] = ACTIONS(139),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [288] = {
    [sym_preproc_file_line] = STATE(288),
    [ts_builtin_sym_end] = ACTIONS(1257),
    [aux_sym_program_statement_token1] = ACTIONS(1259),
    [aux_sym_end_program_statement_token1] = ACTIONS(1259),
    [aux_sym_end_program_statement_token2] = ACTIONS(1259),
    [aux_sym_module_statement_token1] = ACTIONS(1259),
    [aux_sym_submodule_statement_token1] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1259),
    [aux_sym_interface_statement_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1259),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1259),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_function_statement_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1259),
    [aux_sym_contains_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token2] = ACTIONS(1259),
    [aux_sym_implicit_statement_token1] = ACTIONS(1259),
    [aux_sym_implicit_statement_token3] = ACTIONS(1259),
    [aux_sym_implicit_statement_token4] = ACTIONS(1259),
    [aux_sym_save_statement_token1] = ACTIONS(1259),
    [aux_sym_private_statement_token1] = ACTIONS(1259),
    [aux_sym_public_statement_token1] = ACTIONS(1259),
    [aux_sym_namelist_statement_token1] = ACTIONS(1259),
    [aux_sym_common_statement_token1] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [aux_sym_import_statement_token1] = ACTIONS(1259),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1259),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1259),
    [aux_sym_variable_attributes_token1] = ACTIONS(1259),
    [aux_sym_variable_attributes_token2] = ACTIONS(1259),
    [aux_sym_variable_attributes_token3] = ACTIONS(1259),
    [aux_sym_variable_attributes_token4] = ACTIONS(1259),
    [aux_sym_variable_attributes_token5] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1259),
    [aux_sym_derived_type_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token2] = ACTIONS(1259),
    [aux_sym_type_qualifier_token3] = ACTIONS(1259),
    [aux_sym_type_qualifier_token4] = ACTIONS(1259),
    [aux_sym_type_qualifier_token5] = ACTIONS(1259),
    [aux_sym_type_qualifier_token9] = ACTIONS(1259),
    [aux_sym_type_qualifier_token10] = ACTIONS(1259),
    [aux_sym_type_qualifier_token11] = ACTIONS(1259),
    [aux_sym_type_qualifier_token12] = ACTIONS(1259),
    [aux_sym_type_qualifier_token13] = ACTIONS(1259),
    [aux_sym_type_qualifier_token14] = ACTIONS(1259),
    [aux_sym_type_qualifier_token15] = ACTIONS(1259),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1259),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1259),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1259),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token2] = ACTIONS(1259),
    [aux_sym_subroutine_call_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token2] = ACTIONS(1259),
    [aux_sym_keyword_statement_token3] = ACTIONS(1259),
    [aux_sym_keyword_statement_token4] = ACTIONS(1259),
    [aux_sym_keyword_statement_token6] = ACTIONS(1259),
    [aux_sym_keyword_statement_token7] = ACTIONS(1259),
    [aux_sym_include_statement_token1] = ACTIONS(1259),
    [aux_sym_data_statement_token1] = ACTIONS(1259),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1259),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1259),
    [aux_sym_end_if_statement_token1] = ACTIONS(1259),
    [aux_sym_elseif_clause_token2] = ACTIONS(1259),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1259),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token3] = ACTIONS(1259),
    [aux_sym_select_type_statement_token1] = ACTIONS(1259),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1259),
    [aux_sym_block_construct_token1] = ACTIONS(1259),
    [aux_sym_associate_statement_token1] = ACTIONS(1259),
    [aux_sym_format_statement_token1] = ACTIONS(1259),
    [aux_sym_print_statement_token1] = ACTIONS(1259),
    [aux_sym_open_statement_token1] = ACTIONS(1259),
    [aux_sym_close_statement_token1] = ACTIONS(1259),
    [aux_sym_inquire_statement_token1] = ACTIONS(1259),
    [aux_sym_enum_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token2] = ACTIONS(1259),
    [aux_sym_file_position_statement_token3] = ACTIONS(1259),
    [aux_sym_file_position_statement_token4] = ACTIONS(1259),
    [aux_sym_logical_expression_token5] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [sym_boolean_literal] = ACTIONS(1257),
    [aux_sym_null_literal_token1] = ACTIONS(1259),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1259),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1257),
    [sym__float_literal] = ACTIONS(1257),
    [sym__boz_literal] = ACTIONS(1257),
    [sym_string_literal] = ACTIONS(1257),
  },
  [289] = {
    [sym_preproc_file_line] = STATE(289),
    [ts_builtin_sym_end] = ACTIONS(1261),
    [aux_sym_program_statement_token1] = ACTIONS(1263),
    [aux_sym_end_program_statement_token1] = ACTIONS(1263),
    [aux_sym_end_program_statement_token2] = ACTIONS(1263),
    [aux_sym_module_statement_token1] = ACTIONS(1263),
    [aux_sym_submodule_statement_token1] = ACTIONS(1263),
    [anon_sym_LPAREN] = ACTIONS(1263),
    [aux_sym_interface_statement_token1] = ACTIONS(1263),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1263),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1263),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1263),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1263),
    [aux_sym_function_statement_token1] = ACTIONS(1263),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1263),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1263),
    [aux_sym_contains_statement_token1] = ACTIONS(1263),
    [aux_sym_use_statement_token1] = ACTIONS(1263),
    [aux_sym_use_statement_token2] = ACTIONS(1263),
    [aux_sym_implicit_statement_token1] = ACTIONS(1263),
    [aux_sym_implicit_statement_token3] = ACTIONS(1263),
    [aux_sym_implicit_statement_token4] = ACTIONS(1263),
    [aux_sym_save_statement_token1] = ACTIONS(1263),
    [aux_sym_private_statement_token1] = ACTIONS(1263),
    [aux_sym_public_statement_token1] = ACTIONS(1263),
    [aux_sym_namelist_statement_token1] = ACTIONS(1263),
    [aux_sym_common_statement_token1] = ACTIONS(1263),
    [anon_sym_DASH] = ACTIONS(1261),
    [aux_sym_import_statement_token1] = ACTIONS(1263),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1263),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1263),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1263),
    [aux_sym_variable_attributes_token1] = ACTIONS(1263),
    [aux_sym_variable_attributes_token2] = ACTIONS(1263),
    [aux_sym_variable_attributes_token3] = ACTIONS(1263),
    [aux_sym_variable_attributes_token4] = ACTIONS(1263),
    [aux_sym_variable_attributes_token5] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1263),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1263),
    [aux_sym_derived_type_token1] = ACTIONS(1263),
    [aux_sym_type_qualifier_token1] = ACTIONS(1263),
    [aux_sym_type_qualifier_token2] = ACTIONS(1263),
    [aux_sym_type_qualifier_token3] = ACTIONS(1263),
    [aux_sym_type_qualifier_token4] = ACTIONS(1263),
    [aux_sym_type_qualifier_token5] = ACTIONS(1263),
    [aux_sym_type_qualifier_token9] = ACTIONS(1263),
    [aux_sym_type_qualifier_token10] = ACTIONS(1263),
    [aux_sym_type_qualifier_token11] = ACTIONS(1263),
    [aux_sym_type_qualifier_token12] = ACTIONS(1263),
    [aux_sym_type_qualifier_token13] = ACTIONS(1263),
    [aux_sym_type_qualifier_token14] = ACTIONS(1263),
    [aux_sym_type_qualifier_token15] = ACTIONS(1263),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1263),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1263),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1263),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1263),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1263),
    [aux_sym_stop_statement_token1] = ACTIONS(1263),
    [aux_sym_stop_statement_token2] = ACTIONS(1263),
    [aux_sym_subroutine_call_token1] = ACTIONS(1263),
    [aux_sym_keyword_statement_token1] = ACTIONS(1263),
    [aux_sym_keyword_statement_token2] = ACTIONS(1263),
    [aux_sym_keyword_statement_token3] = ACTIONS(1263),
    [aux_sym_keyword_statement_token4] = ACTIONS(1263),
    [aux_sym_keyword_statement_token6] = ACTIONS(1263),
    [aux_sym_keyword_statement_token7] = ACTIONS(1263),
    [aux_sym_include_statement_token1] = ACTIONS(1263),
    [aux_sym_data_statement_token1] = ACTIONS(1263),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1263),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1263),
    [aux_sym_end_if_statement_token1] = ACTIONS(1263),
    [aux_sym_elseif_clause_token2] = ACTIONS(1263),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1263),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1263),
    [aux_sym_select_case_statement_token1] = ACTIONS(1263),
    [aux_sym_select_case_statement_token3] = ACTIONS(1263),
    [aux_sym_select_type_statement_token1] = ACTIONS(1263),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1263),
    [aux_sym_block_construct_token1] = ACTIONS(1263),
    [aux_sym_associate_statement_token1] = ACTIONS(1263),
    [aux_sym_format_statement_token1] = ACTIONS(1263),
    [aux_sym_print_statement_token1] = ACTIONS(1263),
    [aux_sym_open_statement_token1] = ACTIONS(1263),
    [aux_sym_close_statement_token1] = ACTIONS(1263),
    [aux_sym_inquire_statement_token1] = ACTIONS(1263),
    [aux_sym_enum_statement_token1] = ACTIONS(1263),
    [aux_sym_file_position_statement_token1] = ACTIONS(1263),
    [aux_sym_file_position_statement_token2] = ACTIONS(1263),
    [aux_sym_file_position_statement_token3] = ACTIONS(1263),
    [aux_sym_file_position_statement_token4] = ACTIONS(1263),
    [aux_sym_logical_expression_token5] = ACTIONS(1261),
    [anon_sym_PLUS] = ACTIONS(1261),
    [anon_sym_DOT] = ACTIONS(1263),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1261),
    [anon_sym_LBRACK] = ACTIONS(1261),
    [sym_boolean_literal] = ACTIONS(1261),
    [aux_sym_null_literal_token1] = ACTIONS(1263),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1263),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1261),
    [sym__float_literal] = ACTIONS(1261),
    [sym__boz_literal] = ACTIONS(1261),
    [sym_string_literal] = ACTIONS(1261),
  },
  [290] = {
    [sym_preproc_file_line] = STATE(290),
    [ts_builtin_sym_end] = ACTIONS(1265),
    [aux_sym_program_statement_token1] = ACTIONS(1267),
    [aux_sym_end_program_statement_token1] = ACTIONS(1267),
    [aux_sym_end_program_statement_token2] = ACTIONS(1267),
    [aux_sym_module_statement_token1] = ACTIONS(1267),
    [aux_sym_submodule_statement_token1] = ACTIONS(1267),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [aux_sym_interface_statement_token1] = ACTIONS(1267),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1267),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1267),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1267),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1267),
    [aux_sym_function_statement_token1] = ACTIONS(1267),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1267),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1267),
    [aux_sym_contains_statement_token1] = ACTIONS(1267),
    [aux_sym_use_statement_token1] = ACTIONS(1267),
    [aux_sym_use_statement_token2] = ACTIONS(1267),
    [aux_sym_implicit_statement_token1] = ACTIONS(1267),
    [aux_sym_implicit_statement_token3] = ACTIONS(1267),
    [aux_sym_implicit_statement_token4] = ACTIONS(1267),
    [aux_sym_save_statement_token1] = ACTIONS(1267),
    [aux_sym_private_statement_token1] = ACTIONS(1267),
    [aux_sym_public_statement_token1] = ACTIONS(1267),
    [aux_sym_namelist_statement_token1] = ACTIONS(1267),
    [aux_sym_common_statement_token1] = ACTIONS(1267),
    [anon_sym_DASH] = ACTIONS(1265),
    [aux_sym_import_statement_token1] = ACTIONS(1267),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1267),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1267),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1267),
    [aux_sym_variable_attributes_token1] = ACTIONS(1267),
    [aux_sym_variable_attributes_token2] = ACTIONS(1267),
    [aux_sym_variable_attributes_token3] = ACTIONS(1267),
    [aux_sym_variable_attributes_token4] = ACTIONS(1267),
    [aux_sym_variable_attributes_token5] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1267),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1267),
    [aux_sym_derived_type_token1] = ACTIONS(1267),
    [aux_sym_type_qualifier_token1] = ACTIONS(1267),
    [aux_sym_type_qualifier_token2] = ACTIONS(1267),
    [aux_sym_type_qualifier_token3] = ACTIONS(1267),
    [aux_sym_type_qualifier_token4] = ACTIONS(1267),
    [aux_sym_type_qualifier_token5] = ACTIONS(1267),
    [aux_sym_type_qualifier_token9] = ACTIONS(1267),
    [aux_sym_type_qualifier_token10] = ACTIONS(1267),
    [aux_sym_type_qualifier_token11] = ACTIONS(1267),
    [aux_sym_type_qualifier_token12] = ACTIONS(1267),
    [aux_sym_type_qualifier_token13] = ACTIONS(1267),
    [aux_sym_type_qualifier_token14] = ACTIONS(1267),
    [aux_sym_type_qualifier_token15] = ACTIONS(1267),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1267),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1267),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1267),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1267),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1267),
    [aux_sym_stop_statement_token1] = ACTIONS(1267),
    [aux_sym_stop_statement_token2] = ACTIONS(1267),
    [aux_sym_subroutine_call_token1] = ACTIONS(1267),
    [aux_sym_keyword_statement_token1] = ACTIONS(1267),
    [aux_sym_keyword_statement_token2] = ACTIONS(1267),
    [aux_sym_keyword_statement_token3] = ACTIONS(1267),
    [aux_sym_keyword_statement_token4] = ACTIONS(1267),
    [aux_sym_keyword_statement_token6] = ACTIONS(1267),
    [aux_sym_keyword_statement_token7] = ACTIONS(1267),
    [aux_sym_include_statement_token1] = ACTIONS(1267),
    [aux_sym_data_statement_token1] = ACTIONS(1267),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1267),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1267),
    [aux_sym_end_if_statement_token1] = ACTIONS(1267),
    [aux_sym_elseif_clause_token2] = ACTIONS(1267),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1267),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1267),
    [aux_sym_select_case_statement_token1] = ACTIONS(1267),
    [aux_sym_select_case_statement_token3] = ACTIONS(1267),
    [aux_sym_select_type_statement_token1] = ACTIONS(1267),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1267),
    [aux_sym_block_construct_token1] = ACTIONS(1267),
    [aux_sym_associate_statement_token1] = ACTIONS(1267),
    [aux_sym_format_statement_token1] = ACTIONS(1267),
    [aux_sym_print_statement_token1] = ACTIONS(1267),
    [aux_sym_open_statement_token1] = ACTIONS(1267),
    [aux_sym_close_statement_token1] = ACTIONS(1267),
    [aux_sym_inquire_statement_token1] = ACTIONS(1267),
    [aux_sym_enum_statement_token1] = ACTIONS(1267),
    [aux_sym_file_position_statement_token1] = ACTIONS(1267),
    [aux_sym_file_position_statement_token2] = ACTIONS(1267),
    [aux_sym_file_position_statement_token3] = ACTIONS(1267),
    [aux_sym_file_position_statement_token4] = ACTIONS(1267),
    [aux_sym_logical_expression_token5] = ACTIONS(1265),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_DOT] = ACTIONS(1267),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1265),
    [sym_boolean_literal] = ACTIONS(1265),
    [aux_sym_null_literal_token1] = ACTIONS(1267),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1267),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1265),
    [sym__float_literal] = ACTIONS(1265),
    [sym__boz_literal] = ACTIONS(1265),
    [sym_string_literal] = ACTIONS(1265),
  },
  [291] = {
    [sym_preproc_file_line] = STATE(291),
    [ts_builtin_sym_end] = ACTIONS(157),
    [aux_sym_program_statement_token1] = ACTIONS(1269),
    [aux_sym_end_program_statement_token1] = ACTIONS(1269),
    [aux_sym_end_program_statement_token2] = ACTIONS(1269),
    [aux_sym_module_statement_token1] = ACTIONS(1269),
    [aux_sym_submodule_statement_token1] = ACTIONS(1269),
    [anon_sym_LPAREN] = ACTIONS(1269),
    [aux_sym_interface_statement_token1] = ACTIONS(1269),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1269),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1269),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1269),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1269),
    [aux_sym_function_statement_token1] = ACTIONS(1269),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1269),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1269),
    [aux_sym_contains_statement_token1] = ACTIONS(1269),
    [aux_sym_use_statement_token1] = ACTIONS(1269),
    [aux_sym_use_statement_token2] = ACTIONS(1269),
    [aux_sym_implicit_statement_token1] = ACTIONS(1269),
    [aux_sym_implicit_statement_token3] = ACTIONS(1269),
    [aux_sym_implicit_statement_token4] = ACTIONS(1269),
    [aux_sym_save_statement_token1] = ACTIONS(1269),
    [aux_sym_private_statement_token1] = ACTIONS(1269),
    [aux_sym_public_statement_token1] = ACTIONS(1269),
    [aux_sym_namelist_statement_token1] = ACTIONS(1269),
    [aux_sym_common_statement_token1] = ACTIONS(1269),
    [anon_sym_DASH] = ACTIONS(157),
    [aux_sym_import_statement_token1] = ACTIONS(1269),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1269),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1269),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1269),
    [aux_sym_variable_attributes_token1] = ACTIONS(1269),
    [aux_sym_variable_attributes_token2] = ACTIONS(1269),
    [aux_sym_variable_attributes_token3] = ACTIONS(1269),
    [aux_sym_variable_attributes_token4] = ACTIONS(1269),
    [aux_sym_variable_attributes_token5] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1269),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1269),
    [aux_sym_derived_type_token1] = ACTIONS(1269),
    [aux_sym_type_qualifier_token1] = ACTIONS(1269),
    [aux_sym_type_qualifier_token2] = ACTIONS(1269),
    [aux_sym_type_qualifier_token3] = ACTIONS(1269),
    [aux_sym_type_qualifier_token4] = ACTIONS(1269),
    [aux_sym_type_qualifier_token5] = ACTIONS(1269),
    [aux_sym_type_qualifier_token9] = ACTIONS(1269),
    [aux_sym_type_qualifier_token10] = ACTIONS(1269),
    [aux_sym_type_qualifier_token11] = ACTIONS(1269),
    [aux_sym_type_qualifier_token12] = ACTIONS(1269),
    [aux_sym_type_qualifier_token13] = ACTIONS(1269),
    [aux_sym_type_qualifier_token14] = ACTIONS(1269),
    [aux_sym_type_qualifier_token15] = ACTIONS(1269),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1269),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1269),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1269),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1269),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1269),
    [aux_sym_stop_statement_token1] = ACTIONS(1269),
    [aux_sym_stop_statement_token2] = ACTIONS(1269),
    [aux_sym_subroutine_call_token1] = ACTIONS(1269),
    [aux_sym_keyword_statement_token1] = ACTIONS(1269),
    [aux_sym_keyword_statement_token2] = ACTIONS(1269),
    [aux_sym_keyword_statement_token3] = ACTIONS(1269),
    [aux_sym_keyword_statement_token4] = ACTIONS(1269),
    [aux_sym_keyword_statement_token6] = ACTIONS(1269),
    [aux_sym_keyword_statement_token7] = ACTIONS(1269),
    [aux_sym_include_statement_token1] = ACTIONS(1269),
    [aux_sym_data_statement_token1] = ACTIONS(1269),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1269),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1269),
    [aux_sym_end_if_statement_token1] = ACTIONS(1269),
    [aux_sym_elseif_clause_token2] = ACTIONS(1269),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1269),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1269),
    [aux_sym_select_case_statement_token1] = ACTIONS(1269),
    [aux_sym_select_case_statement_token3] = ACTIONS(1269),
    [aux_sym_select_type_statement_token1] = ACTIONS(1269),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1269),
    [aux_sym_block_construct_token1] = ACTIONS(1269),
    [aux_sym_associate_statement_token1] = ACTIONS(1269),
    [aux_sym_format_statement_token1] = ACTIONS(1269),
    [aux_sym_print_statement_token1] = ACTIONS(1269),
    [aux_sym_open_statement_token1] = ACTIONS(1269),
    [aux_sym_close_statement_token1] = ACTIONS(1269),
    [aux_sym_inquire_statement_token1] = ACTIONS(1269),
    [aux_sym_enum_statement_token1] = ACTIONS(1269),
    [aux_sym_file_position_statement_token1] = ACTIONS(1269),
    [aux_sym_file_position_statement_token2] = ACTIONS(1269),
    [aux_sym_file_position_statement_token3] = ACTIONS(1269),
    [aux_sym_file_position_statement_token4] = ACTIONS(1269),
    [aux_sym_logical_expression_token5] = ACTIONS(157),
    [anon_sym_PLUS] = ACTIONS(157),
    [anon_sym_DOT] = ACTIONS(1269),
    [anon_sym_LPAREN_SLASH] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(157),
    [sym_boolean_literal] = ACTIONS(157),
    [aux_sym_null_literal_token1] = ACTIONS(1269),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1269),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(157),
    [sym__float_literal] = ACTIONS(157),
    [sym__boz_literal] = ACTIONS(157),
    [sym_string_literal] = ACTIONS(157),
  },
  [292] = {
    [sym_preproc_file_line] = STATE(292),
    [ts_builtin_sym_end] = ACTIONS(1217),
    [aux_sym_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token2] = ACTIONS(1219),
    [aux_sym_module_statement_token1] = ACTIONS(1219),
    [aux_sym_submodule_statement_token1] = ACTIONS(1219),
    [anon_sym_LPAREN] = ACTIONS(1219),
    [aux_sym_interface_statement_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1219),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1219),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1219),
    [aux_sym_function_statement_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1219),
    [aux_sym_contains_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token2] = ACTIONS(1219),
    [aux_sym_implicit_statement_token1] = ACTIONS(1219),
    [aux_sym_implicit_statement_token3] = ACTIONS(1219),
    [aux_sym_implicit_statement_token4] = ACTIONS(1219),
    [aux_sym_save_statement_token1] = ACTIONS(1219),
    [aux_sym_private_statement_token1] = ACTIONS(1219),
    [aux_sym_public_statement_token1] = ACTIONS(1219),
    [aux_sym_namelist_statement_token1] = ACTIONS(1219),
    [aux_sym_common_statement_token1] = ACTIONS(1219),
    [anon_sym_DASH] = ACTIONS(1217),
    [aux_sym_import_statement_token1] = ACTIONS(1219),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1219),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1219),
    [aux_sym_variable_attributes_token1] = ACTIONS(1219),
    [aux_sym_variable_attributes_token2] = ACTIONS(1219),
    [aux_sym_variable_attributes_token3] = ACTIONS(1219),
    [aux_sym_variable_attributes_token4] = ACTIONS(1219),
    [aux_sym_variable_attributes_token5] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1219),
    [aux_sym_derived_type_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token2] = ACTIONS(1219),
    [aux_sym_type_qualifier_token3] = ACTIONS(1219),
    [aux_sym_type_qualifier_token4] = ACTIONS(1219),
    [aux_sym_type_qualifier_token5] = ACTIONS(1219),
    [aux_sym_type_qualifier_token9] = ACTIONS(1219),
    [aux_sym_type_qualifier_token10] = ACTIONS(1219),
    [aux_sym_type_qualifier_token11] = ACTIONS(1219),
    [aux_sym_type_qualifier_token12] = ACTIONS(1219),
    [aux_sym_type_qualifier_token13] = ACTIONS(1219),
    [aux_sym_type_qualifier_token14] = ACTIONS(1219),
    [aux_sym_type_qualifier_token15] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1219),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token2] = ACTIONS(1219),
    [aux_sym_subroutine_call_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token2] = ACTIONS(1219),
    [aux_sym_keyword_statement_token3] = ACTIONS(1219),
    [aux_sym_keyword_statement_token4] = ACTIONS(1219),
    [aux_sym_keyword_statement_token6] = ACTIONS(1219),
    [aux_sym_keyword_statement_token7] = ACTIONS(1219),
    [aux_sym_include_statement_token1] = ACTIONS(1219),
    [aux_sym_data_statement_token1] = ACTIONS(1219),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1219),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1219),
    [aux_sym_end_if_statement_token1] = ACTIONS(1219),
    [aux_sym_elseif_clause_token2] = ACTIONS(1219),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1219),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token3] = ACTIONS(1219),
    [aux_sym_select_type_statement_token1] = ACTIONS(1219),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1219),
    [aux_sym_block_construct_token1] = ACTIONS(1219),
    [aux_sym_associate_statement_token1] = ACTIONS(1219),
    [aux_sym_format_statement_token1] = ACTIONS(1219),
    [aux_sym_print_statement_token1] = ACTIONS(1219),
    [aux_sym_open_statement_token1] = ACTIONS(1219),
    [aux_sym_close_statement_token1] = ACTIONS(1219),
    [aux_sym_inquire_statement_token1] = ACTIONS(1219),
    [aux_sym_enum_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token2] = ACTIONS(1219),
    [aux_sym_file_position_statement_token3] = ACTIONS(1219),
    [aux_sym_file_position_statement_token4] = ACTIONS(1219),
    [aux_sym_logical_expression_token5] = ACTIONS(1217),
    [anon_sym_PLUS] = ACTIONS(1217),
    [anon_sym_DOT] = ACTIONS(1219),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1217),
    [anon_sym_LBRACK] = ACTIONS(1217),
    [sym_boolean_literal] = ACTIONS(1217),
    [aux_sym_null_literal_token1] = ACTIONS(1219),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1219),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1217),
    [sym__float_literal] = ACTIONS(1217),
    [sym__boz_literal] = ACTIONS(1217),
    [sym_string_literal] = ACTIONS(1217),
  },
  [293] = {
    [sym_preproc_file_line] = STATE(293),
    [ts_builtin_sym_end] = ACTIONS(1271),
    [aux_sym_program_statement_token1] = ACTIONS(1273),
    [aux_sym_end_program_statement_token1] = ACTIONS(1273),
    [aux_sym_end_program_statement_token2] = ACTIONS(1273),
    [aux_sym_module_statement_token1] = ACTIONS(1273),
    [aux_sym_submodule_statement_token1] = ACTIONS(1273),
    [anon_sym_LPAREN] = ACTIONS(1273),
    [aux_sym_interface_statement_token1] = ACTIONS(1273),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1273),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1273),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1273),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1273),
    [aux_sym_function_statement_token1] = ACTIONS(1273),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1273),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1273),
    [aux_sym_contains_statement_token1] = ACTIONS(1273),
    [aux_sym_use_statement_token1] = ACTIONS(1273),
    [aux_sym_use_statement_token2] = ACTIONS(1273),
    [aux_sym_implicit_statement_token1] = ACTIONS(1273),
    [aux_sym_implicit_statement_token3] = ACTIONS(1273),
    [aux_sym_implicit_statement_token4] = ACTIONS(1273),
    [aux_sym_save_statement_token1] = ACTIONS(1273),
    [aux_sym_private_statement_token1] = ACTIONS(1273),
    [aux_sym_public_statement_token1] = ACTIONS(1273),
    [aux_sym_namelist_statement_token1] = ACTIONS(1273),
    [aux_sym_common_statement_token1] = ACTIONS(1273),
    [anon_sym_DASH] = ACTIONS(1271),
    [aux_sym_import_statement_token1] = ACTIONS(1273),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1273),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1273),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1273),
    [aux_sym_variable_attributes_token1] = ACTIONS(1273),
    [aux_sym_variable_attributes_token2] = ACTIONS(1273),
    [aux_sym_variable_attributes_token3] = ACTIONS(1273),
    [aux_sym_variable_attributes_token4] = ACTIONS(1273),
    [aux_sym_variable_attributes_token5] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1273),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1273),
    [aux_sym_derived_type_token1] = ACTIONS(1273),
    [aux_sym_type_qualifier_token1] = ACTIONS(1273),
    [aux_sym_type_qualifier_token2] = ACTIONS(1273),
    [aux_sym_type_qualifier_token3] = ACTIONS(1273),
    [aux_sym_type_qualifier_token4] = ACTIONS(1273),
    [aux_sym_type_qualifier_token5] = ACTIONS(1273),
    [aux_sym_type_qualifier_token9] = ACTIONS(1273),
    [aux_sym_type_qualifier_token10] = ACTIONS(1273),
    [aux_sym_type_qualifier_token11] = ACTIONS(1273),
    [aux_sym_type_qualifier_token12] = ACTIONS(1273),
    [aux_sym_type_qualifier_token13] = ACTIONS(1273),
    [aux_sym_type_qualifier_token14] = ACTIONS(1273),
    [aux_sym_type_qualifier_token15] = ACTIONS(1273),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1273),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1273),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1273),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1273),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1273),
    [aux_sym_stop_statement_token1] = ACTIONS(1273),
    [aux_sym_stop_statement_token2] = ACTIONS(1273),
    [aux_sym_subroutine_call_token1] = ACTIONS(1273),
    [aux_sym_keyword_statement_token1] = ACTIONS(1273),
    [aux_sym_keyword_statement_token2] = ACTIONS(1273),
    [aux_sym_keyword_statement_token3] = ACTIONS(1273),
    [aux_sym_keyword_statement_token4] = ACTIONS(1273),
    [aux_sym_keyword_statement_token6] = ACTIONS(1273),
    [aux_sym_keyword_statement_token7] = ACTIONS(1273),
    [aux_sym_include_statement_token1] = ACTIONS(1273),
    [aux_sym_data_statement_token1] = ACTIONS(1273),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1273),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1273),
    [aux_sym_end_if_statement_token1] = ACTIONS(1273),
    [aux_sym_elseif_clause_token2] = ACTIONS(1273),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1273),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1273),
    [aux_sym_select_case_statement_token1] = ACTIONS(1273),
    [aux_sym_select_case_statement_token3] = ACTIONS(1273),
    [aux_sym_select_type_statement_token1] = ACTIONS(1273),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1273),
    [aux_sym_block_construct_token1] = ACTIONS(1273),
    [aux_sym_associate_statement_token1] = ACTIONS(1273),
    [aux_sym_format_statement_token1] = ACTIONS(1273),
    [aux_sym_print_statement_token1] = ACTIONS(1273),
    [aux_sym_open_statement_token1] = ACTIONS(1273),
    [aux_sym_close_statement_token1] = ACTIONS(1273),
    [aux_sym_inquire_statement_token1] = ACTIONS(1273),
    [aux_sym_enum_statement_token1] = ACTIONS(1273),
    [aux_sym_file_position_statement_token1] = ACTIONS(1273),
    [aux_sym_file_position_statement_token2] = ACTIONS(1273),
    [aux_sym_file_position_statement_token3] = ACTIONS(1273),
    [aux_sym_file_position_statement_token4] = ACTIONS(1273),
    [aux_sym_logical_expression_token5] = ACTIONS(1271),
    [anon_sym_PLUS] = ACTIONS(1271),
    [anon_sym_DOT] = ACTIONS(1273),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1271),
    [anon_sym_LBRACK] = ACTIONS(1271),
    [sym_boolean_literal] = ACTIONS(1271),
    [aux_sym_null_literal_token1] = ACTIONS(1273),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1273),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1271),
    [sym__float_literal] = ACTIONS(1271),
    [sym__boz_literal] = ACTIONS(1271),
    [sym_string_literal] = ACTIONS(1271),
  },
  [294] = {
    [sym_preproc_file_line] = STATE(294),
    [ts_builtin_sym_end] = ACTIONS(1217),
    [aux_sym_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token1] = ACTIONS(1219),
    [aux_sym_end_program_statement_token2] = ACTIONS(1219),
    [aux_sym_module_statement_token1] = ACTIONS(1219),
    [aux_sym_submodule_statement_token1] = ACTIONS(1219),
    [anon_sym_LPAREN] = ACTIONS(1219),
    [aux_sym_interface_statement_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1219),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1219),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1219),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1219),
    [aux_sym_function_statement_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1219),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1219),
    [aux_sym_contains_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token1] = ACTIONS(1219),
    [aux_sym_use_statement_token2] = ACTIONS(1219),
    [aux_sym_implicit_statement_token1] = ACTIONS(1219),
    [aux_sym_implicit_statement_token3] = ACTIONS(1219),
    [aux_sym_implicit_statement_token4] = ACTIONS(1219),
    [aux_sym_save_statement_token1] = ACTIONS(1219),
    [aux_sym_private_statement_token1] = ACTIONS(1219),
    [aux_sym_public_statement_token1] = ACTIONS(1219),
    [aux_sym_namelist_statement_token1] = ACTIONS(1219),
    [aux_sym_common_statement_token1] = ACTIONS(1219),
    [anon_sym_DASH] = ACTIONS(1217),
    [aux_sym_import_statement_token1] = ACTIONS(1219),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1219),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1219),
    [aux_sym_variable_attributes_token1] = ACTIONS(1219),
    [aux_sym_variable_attributes_token2] = ACTIONS(1219),
    [aux_sym_variable_attributes_token3] = ACTIONS(1219),
    [aux_sym_variable_attributes_token4] = ACTIONS(1219),
    [aux_sym_variable_attributes_token5] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1219),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1219),
    [aux_sym_derived_type_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token1] = ACTIONS(1219),
    [aux_sym_type_qualifier_token2] = ACTIONS(1219),
    [aux_sym_type_qualifier_token3] = ACTIONS(1219),
    [aux_sym_type_qualifier_token4] = ACTIONS(1219),
    [aux_sym_type_qualifier_token5] = ACTIONS(1219),
    [aux_sym_type_qualifier_token9] = ACTIONS(1219),
    [aux_sym_type_qualifier_token10] = ACTIONS(1219),
    [aux_sym_type_qualifier_token11] = ACTIONS(1219),
    [aux_sym_type_qualifier_token12] = ACTIONS(1219),
    [aux_sym_type_qualifier_token13] = ACTIONS(1219),
    [aux_sym_type_qualifier_token14] = ACTIONS(1219),
    [aux_sym_type_qualifier_token15] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1219),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1219),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token1] = ACTIONS(1219),
    [aux_sym_stop_statement_token2] = ACTIONS(1219),
    [aux_sym_subroutine_call_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token1] = ACTIONS(1219),
    [aux_sym_keyword_statement_token2] = ACTIONS(1219),
    [aux_sym_keyword_statement_token3] = ACTIONS(1219),
    [aux_sym_keyword_statement_token4] = ACTIONS(1219),
    [aux_sym_keyword_statement_token6] = ACTIONS(1219),
    [aux_sym_keyword_statement_token7] = ACTIONS(1219),
    [aux_sym_include_statement_token1] = ACTIONS(1219),
    [aux_sym_data_statement_token1] = ACTIONS(1219),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1219),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1219),
    [aux_sym_end_if_statement_token1] = ACTIONS(1219),
    [aux_sym_elseif_clause_token2] = ACTIONS(1219),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1219),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token1] = ACTIONS(1219),
    [aux_sym_select_case_statement_token3] = ACTIONS(1219),
    [aux_sym_select_type_statement_token1] = ACTIONS(1219),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1219),
    [aux_sym_block_construct_token1] = ACTIONS(1219),
    [aux_sym_associate_statement_token1] = ACTIONS(1219),
    [aux_sym_format_statement_token1] = ACTIONS(1219),
    [aux_sym_print_statement_token1] = ACTIONS(1219),
    [aux_sym_open_statement_token1] = ACTIONS(1219),
    [aux_sym_close_statement_token1] = ACTIONS(1219),
    [aux_sym_inquire_statement_token1] = ACTIONS(1219),
    [aux_sym_enum_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token1] = ACTIONS(1219),
    [aux_sym_file_position_statement_token2] = ACTIONS(1219),
    [aux_sym_file_position_statement_token3] = ACTIONS(1219),
    [aux_sym_file_position_statement_token4] = ACTIONS(1219),
    [aux_sym_logical_expression_token5] = ACTIONS(1217),
    [anon_sym_PLUS] = ACTIONS(1217),
    [anon_sym_DOT] = ACTIONS(1219),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1217),
    [anon_sym_LBRACK] = ACTIONS(1217),
    [sym_boolean_literal] = ACTIONS(1217),
    [aux_sym_null_literal_token1] = ACTIONS(1219),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1219),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1217),
    [sym__float_literal] = ACTIONS(1217),
    [sym__boz_literal] = ACTIONS(1217),
    [sym_string_literal] = ACTIONS(1217),
  },
  [295] = {
    [sym_preproc_file_line] = STATE(295),
    [aux_sym_end_program_statement_token1] = ACTIONS(1275),
    [anon_sym_LPAREN] = ACTIONS(1275),
    [aux_sym_interface_statement_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1275),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1275),
    [aux_sym_end_function_statement_token1] = ACTIONS(1275),
    [aux_sym_contains_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token2] = ACTIONS(1275),
    [aux_sym_implicit_statement_token1] = ACTIONS(1275),
    [aux_sym_implicit_statement_token3] = ACTIONS(1275),
    [aux_sym_implicit_statement_token4] = ACTIONS(1275),
    [aux_sym_save_statement_token1] = ACTIONS(1275),
    [aux_sym_private_statement_token1] = ACTIONS(1275),
    [aux_sym_public_statement_token1] = ACTIONS(1275),
    [aux_sym_namelist_statement_token1] = ACTIONS(1275),
    [aux_sym_common_statement_token1] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1277),
    [aux_sym_import_statement_token1] = ACTIONS(1275),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1275),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1275),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1275),
    [aux_sym_variable_attributes_token1] = ACTIONS(1275),
    [aux_sym_variable_attributes_token2] = ACTIONS(1275),
    [aux_sym_variable_attributes_token3] = ACTIONS(1275),
    [aux_sym_variable_attributes_token4] = ACTIONS(1275),
    [aux_sym_variable_attributes_token5] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1275),
    [aux_sym_derived_type_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token2] = ACTIONS(1275),
    [aux_sym_type_qualifier_token3] = ACTIONS(1275),
    [aux_sym_type_qualifier_token4] = ACTIONS(1275),
    [aux_sym_type_qualifier_token5] = ACTIONS(1275),
    [aux_sym_type_qualifier_token9] = ACTIONS(1275),
    [aux_sym_type_qualifier_token10] = ACTIONS(1275),
    [aux_sym_type_qualifier_token11] = ACTIONS(1275),
    [aux_sym_type_qualifier_token12] = ACTIONS(1275),
    [aux_sym_type_qualifier_token13] = ACTIONS(1275),
    [aux_sym_type_qualifier_token14] = ACTIONS(1275),
    [aux_sym_type_qualifier_token15] = ACTIONS(1275),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token2] = ACTIONS(1275),
    [aux_sym_subroutine_call_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token2] = ACTIONS(1275),
    [aux_sym_keyword_statement_token3] = ACTIONS(1275),
    [aux_sym_keyword_statement_token4] = ACTIONS(1275),
    [aux_sym_keyword_statement_token6] = ACTIONS(1275),
    [aux_sym_keyword_statement_token7] = ACTIONS(1275),
    [aux_sym_include_statement_token1] = ACTIONS(1275),
    [aux_sym_data_statement_token1] = ACTIONS(1275),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1275),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1275),
    [aux_sym_end_if_statement_token1] = ACTIONS(1275),
    [aux_sym_elseif_clause_token2] = ACTIONS(1275),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1275),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token3] = ACTIONS(1275),
    [aux_sym_select_type_statement_token1] = ACTIONS(1275),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1275),
    [aux_sym_block_construct_token1] = ACTIONS(1275),
    [aux_sym_associate_statement_token1] = ACTIONS(1275),
    [aux_sym_format_statement_token1] = ACTIONS(1275),
    [aux_sym_print_statement_token1] = ACTIONS(1275),
    [aux_sym_open_statement_token1] = ACTIONS(1275),
    [aux_sym_close_statement_token1] = ACTIONS(1275),
    [aux_sym_inquire_statement_token1] = ACTIONS(1275),
    [aux_sym_enum_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token2] = ACTIONS(1275),
    [aux_sym_file_position_statement_token3] = ACTIONS(1275),
    [aux_sym_file_position_statement_token4] = ACTIONS(1275),
    [aux_sym_logical_expression_token5] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1277),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [sym_boolean_literal] = ACTIONS(1277),
    [aux_sym_null_literal_token1] = ACTIONS(1275),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1275),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1277),
    [sym__float_literal] = ACTIONS(1277),
    [sym__boz_literal] = ACTIONS(1277),
    [sym_string_literal] = ACTIONS(1277),
  },
  [296] = {
    [sym_preproc_file_line] = STATE(296),
    [aux_sym_end_program_statement_token1] = ACTIONS(1199),
    [aux_sym_end_program_statement_token2] = ACTIONS(1199),
    [anon_sym_LPAREN] = ACTIONS(1199),
    [aux_sym_interface_statement_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1199),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1199),
    [aux_sym_contains_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token2] = ACTIONS(1199),
    [aux_sym_implicit_statement_token1] = ACTIONS(1199),
    [aux_sym_implicit_statement_token3] = ACTIONS(1199),
    [aux_sym_implicit_statement_token4] = ACTIONS(1199),
    [aux_sym_save_statement_token1] = ACTIONS(1199),
    [aux_sym_private_statement_token1] = ACTIONS(1199),
    [aux_sym_public_statement_token1] = ACTIONS(1199),
    [aux_sym_namelist_statement_token1] = ACTIONS(1199),
    [aux_sym_common_statement_token1] = ACTIONS(1199),
    [anon_sym_DASH] = ACTIONS(1197),
    [aux_sym_import_statement_token1] = ACTIONS(1199),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1199),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1199),
    [aux_sym_variable_attributes_token1] = ACTIONS(1199),
    [aux_sym_variable_attributes_token2] = ACTIONS(1199),
    [aux_sym_variable_attributes_token3] = ACTIONS(1199),
    [aux_sym_variable_attributes_token4] = ACTIONS(1199),
    [aux_sym_variable_attributes_token5] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1199),
    [aux_sym_derived_type_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token2] = ACTIONS(1199),
    [aux_sym_type_qualifier_token3] = ACTIONS(1199),
    [aux_sym_type_qualifier_token4] = ACTIONS(1199),
    [aux_sym_type_qualifier_token5] = ACTIONS(1199),
    [aux_sym_type_qualifier_token9] = ACTIONS(1199),
    [aux_sym_type_qualifier_token10] = ACTIONS(1199),
    [aux_sym_type_qualifier_token11] = ACTIONS(1199),
    [aux_sym_type_qualifier_token12] = ACTIONS(1199),
    [aux_sym_type_qualifier_token13] = ACTIONS(1199),
    [aux_sym_type_qualifier_token14] = ACTIONS(1199),
    [aux_sym_type_qualifier_token15] = ACTIONS(1199),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token2] = ACTIONS(1199),
    [aux_sym_subroutine_call_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token2] = ACTIONS(1199),
    [aux_sym_keyword_statement_token3] = ACTIONS(1199),
    [aux_sym_keyword_statement_token4] = ACTIONS(1199),
    [aux_sym_keyword_statement_token6] = ACTIONS(1199),
    [aux_sym_keyword_statement_token7] = ACTIONS(1199),
    [aux_sym_include_statement_token1] = ACTIONS(1199),
    [aux_sym_data_statement_token1] = ACTIONS(1199),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1199),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1199),
    [aux_sym_end_if_statement_token1] = ACTIONS(1199),
    [aux_sym_elseif_clause_token2] = ACTIONS(1199),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1199),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token3] = ACTIONS(1199),
    [aux_sym_select_type_statement_token1] = ACTIONS(1199),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1199),
    [aux_sym_block_construct_token1] = ACTIONS(1199),
    [aux_sym_associate_statement_token1] = ACTIONS(1199),
    [aux_sym_format_statement_token1] = ACTIONS(1199),
    [aux_sym_print_statement_token1] = ACTIONS(1199),
    [aux_sym_open_statement_token1] = ACTIONS(1199),
    [aux_sym_close_statement_token1] = ACTIONS(1199),
    [aux_sym_inquire_statement_token1] = ACTIONS(1199),
    [aux_sym_enum_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token2] = ACTIONS(1199),
    [aux_sym_file_position_statement_token3] = ACTIONS(1199),
    [aux_sym_file_position_statement_token4] = ACTIONS(1199),
    [aux_sym_logical_expression_token5] = ACTIONS(1197),
    [anon_sym_PLUS] = ACTIONS(1197),
    [anon_sym_DOT] = ACTIONS(1199),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1197),
    [anon_sym_LBRACK] = ACTIONS(1197),
    [sym_boolean_literal] = ACTIONS(1197),
    [aux_sym_null_literal_token1] = ACTIONS(1199),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1199),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1197),
    [sym__float_literal] = ACTIONS(1197),
    [sym__boz_literal] = ACTIONS(1197),
    [sym_string_literal] = ACTIONS(1197),
  },
  [297] = {
    [sym_preproc_file_line] = STATE(297),
    [aux_sym_end_program_statement_token1] = ACTIONS(1279),
    [aux_sym_end_program_statement_token2] = ACTIONS(1279),
    [anon_sym_LPAREN] = ACTIONS(1279),
    [aux_sym_interface_statement_token1] = ACTIONS(1279),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1279),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1279),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1279),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1279),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1279),
    [aux_sym_contains_statement_token1] = ACTIONS(1279),
    [aux_sym_use_statement_token1] = ACTIONS(1279),
    [aux_sym_use_statement_token2] = ACTIONS(1279),
    [aux_sym_implicit_statement_token1] = ACTIONS(1279),
    [aux_sym_implicit_statement_token3] = ACTIONS(1279),
    [aux_sym_implicit_statement_token4] = ACTIONS(1279),
    [aux_sym_save_statement_token1] = ACTIONS(1279),
    [aux_sym_private_statement_token1] = ACTIONS(1279),
    [aux_sym_public_statement_token1] = ACTIONS(1279),
    [aux_sym_namelist_statement_token1] = ACTIONS(1279),
    [aux_sym_common_statement_token1] = ACTIONS(1279),
    [anon_sym_DASH] = ACTIONS(1281),
    [aux_sym_import_statement_token1] = ACTIONS(1279),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1279),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1279),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1279),
    [aux_sym_variable_attributes_token1] = ACTIONS(1279),
    [aux_sym_variable_attributes_token2] = ACTIONS(1279),
    [aux_sym_variable_attributes_token3] = ACTIONS(1279),
    [aux_sym_variable_attributes_token4] = ACTIONS(1279),
    [aux_sym_variable_attributes_token5] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1279),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1279),
    [aux_sym_derived_type_token1] = ACTIONS(1279),
    [aux_sym_type_qualifier_token1] = ACTIONS(1279),
    [aux_sym_type_qualifier_token2] = ACTIONS(1279),
    [aux_sym_type_qualifier_token3] = ACTIONS(1279),
    [aux_sym_type_qualifier_token4] = ACTIONS(1279),
    [aux_sym_type_qualifier_token5] = ACTIONS(1279),
    [aux_sym_type_qualifier_token9] = ACTIONS(1279),
    [aux_sym_type_qualifier_token10] = ACTIONS(1279),
    [aux_sym_type_qualifier_token11] = ACTIONS(1279),
    [aux_sym_type_qualifier_token12] = ACTIONS(1279),
    [aux_sym_type_qualifier_token13] = ACTIONS(1279),
    [aux_sym_type_qualifier_token14] = ACTIONS(1279),
    [aux_sym_type_qualifier_token15] = ACTIONS(1279),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1279),
    [aux_sym_stop_statement_token1] = ACTIONS(1279),
    [aux_sym_stop_statement_token2] = ACTIONS(1279),
    [aux_sym_subroutine_call_token1] = ACTIONS(1279),
    [aux_sym_keyword_statement_token1] = ACTIONS(1279),
    [aux_sym_keyword_statement_token2] = ACTIONS(1279),
    [aux_sym_keyword_statement_token3] = ACTIONS(1279),
    [aux_sym_keyword_statement_token4] = ACTIONS(1279),
    [aux_sym_keyword_statement_token6] = ACTIONS(1279),
    [aux_sym_keyword_statement_token7] = ACTIONS(1279),
    [aux_sym_include_statement_token1] = ACTIONS(1279),
    [aux_sym_data_statement_token1] = ACTIONS(1279),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1279),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1279),
    [aux_sym_end_if_statement_token1] = ACTIONS(1279),
    [aux_sym_elseif_clause_token2] = ACTIONS(1279),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1279),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1279),
    [aux_sym_select_case_statement_token1] = ACTIONS(1279),
    [aux_sym_select_case_statement_token3] = ACTIONS(1279),
    [aux_sym_select_type_statement_token1] = ACTIONS(1279),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1279),
    [aux_sym_block_construct_token1] = ACTIONS(1279),
    [aux_sym_associate_statement_token1] = ACTIONS(1279),
    [aux_sym_format_statement_token1] = ACTIONS(1279),
    [aux_sym_print_statement_token1] = ACTIONS(1279),
    [aux_sym_open_statement_token1] = ACTIONS(1279),
    [aux_sym_close_statement_token1] = ACTIONS(1279),
    [aux_sym_inquire_statement_token1] = ACTIONS(1279),
    [aux_sym_enum_statement_token1] = ACTIONS(1279),
    [aux_sym_file_position_statement_token1] = ACTIONS(1279),
    [aux_sym_file_position_statement_token2] = ACTIONS(1279),
    [aux_sym_file_position_statement_token3] = ACTIONS(1279),
    [aux_sym_file_position_statement_token4] = ACTIONS(1279),
    [aux_sym_logical_expression_token5] = ACTIONS(1281),
    [anon_sym_PLUS] = ACTIONS(1281),
    [anon_sym_DOT] = ACTIONS(1279),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1281),
    [anon_sym_LBRACK] = ACTIONS(1281),
    [sym_boolean_literal] = ACTIONS(1281),
    [aux_sym_null_literal_token1] = ACTIONS(1279),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1279),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1281),
    [sym__float_literal] = ACTIONS(1281),
    [sym__boz_literal] = ACTIONS(1281),
    [sym_string_literal] = ACTIONS(1281),
  },
  [298] = {
    [sym_preproc_file_line] = STATE(298),
    [aux_sym_end_program_statement_token1] = ACTIONS(1283),
    [aux_sym_end_program_statement_token2] = ACTIONS(1283),
    [anon_sym_LPAREN] = ACTIONS(1283),
    [aux_sym_interface_statement_token1] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1283),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1283),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1283),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1283),
    [aux_sym_contains_statement_token1] = ACTIONS(1283),
    [aux_sym_use_statement_token1] = ACTIONS(1283),
    [aux_sym_use_statement_token2] = ACTIONS(1283),
    [aux_sym_implicit_statement_token1] = ACTIONS(1283),
    [aux_sym_implicit_statement_token3] = ACTIONS(1283),
    [aux_sym_implicit_statement_token4] = ACTIONS(1283),
    [aux_sym_save_statement_token1] = ACTIONS(1283),
    [aux_sym_private_statement_token1] = ACTIONS(1283),
    [aux_sym_public_statement_token1] = ACTIONS(1283),
    [aux_sym_namelist_statement_token1] = ACTIONS(1283),
    [aux_sym_common_statement_token1] = ACTIONS(1283),
    [anon_sym_DASH] = ACTIONS(1285),
    [aux_sym_import_statement_token1] = ACTIONS(1283),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1283),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1283),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1283),
    [aux_sym_variable_attributes_token1] = ACTIONS(1283),
    [aux_sym_variable_attributes_token2] = ACTIONS(1283),
    [aux_sym_variable_attributes_token3] = ACTIONS(1283),
    [aux_sym_variable_attributes_token4] = ACTIONS(1283),
    [aux_sym_variable_attributes_token5] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1283),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1283),
    [aux_sym_derived_type_token1] = ACTIONS(1283),
    [aux_sym_type_qualifier_token1] = ACTIONS(1283),
    [aux_sym_type_qualifier_token2] = ACTIONS(1283),
    [aux_sym_type_qualifier_token3] = ACTIONS(1283),
    [aux_sym_type_qualifier_token4] = ACTIONS(1283),
    [aux_sym_type_qualifier_token5] = ACTIONS(1283),
    [aux_sym_type_qualifier_token9] = ACTIONS(1283),
    [aux_sym_type_qualifier_token10] = ACTIONS(1283),
    [aux_sym_type_qualifier_token11] = ACTIONS(1283),
    [aux_sym_type_qualifier_token12] = ACTIONS(1283),
    [aux_sym_type_qualifier_token13] = ACTIONS(1283),
    [aux_sym_type_qualifier_token14] = ACTIONS(1283),
    [aux_sym_type_qualifier_token15] = ACTIONS(1283),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1283),
    [aux_sym_stop_statement_token1] = ACTIONS(1283),
    [aux_sym_stop_statement_token2] = ACTIONS(1283),
    [aux_sym_subroutine_call_token1] = ACTIONS(1283),
    [aux_sym_keyword_statement_token1] = ACTIONS(1283),
    [aux_sym_keyword_statement_token2] = ACTIONS(1283),
    [aux_sym_keyword_statement_token3] = ACTIONS(1283),
    [aux_sym_keyword_statement_token4] = ACTIONS(1283),
    [aux_sym_keyword_statement_token6] = ACTIONS(1283),
    [aux_sym_keyword_statement_token7] = ACTIONS(1283),
    [aux_sym_include_statement_token1] = ACTIONS(1283),
    [aux_sym_data_statement_token1] = ACTIONS(1283),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1283),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1283),
    [aux_sym_end_if_statement_token1] = ACTIONS(1283),
    [aux_sym_elseif_clause_token2] = ACTIONS(1283),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1283),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1283),
    [aux_sym_select_case_statement_token1] = ACTIONS(1283),
    [aux_sym_select_case_statement_token3] = ACTIONS(1283),
    [aux_sym_select_type_statement_token1] = ACTIONS(1283),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1283),
    [aux_sym_block_construct_token1] = ACTIONS(1283),
    [aux_sym_associate_statement_token1] = ACTIONS(1283),
    [aux_sym_format_statement_token1] = ACTIONS(1283),
    [aux_sym_print_statement_token1] = ACTIONS(1283),
    [aux_sym_open_statement_token1] = ACTIONS(1283),
    [aux_sym_close_statement_token1] = ACTIONS(1283),
    [aux_sym_inquire_statement_token1] = ACTIONS(1283),
    [aux_sym_enum_statement_token1] = ACTIONS(1283),
    [aux_sym_file_position_statement_token1] = ACTIONS(1283),
    [aux_sym_file_position_statement_token2] = ACTIONS(1283),
    [aux_sym_file_position_statement_token3] = ACTIONS(1283),
    [aux_sym_file_position_statement_token4] = ACTIONS(1283),
    [aux_sym_logical_expression_token5] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(1283),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1285),
    [anon_sym_LBRACK] = ACTIONS(1285),
    [sym_boolean_literal] = ACTIONS(1285),
    [aux_sym_null_literal_token1] = ACTIONS(1283),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1283),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1285),
    [sym__float_literal] = ACTIONS(1285),
    [sym__boz_literal] = ACTIONS(1285),
    [sym_string_literal] = ACTIONS(1285),
  },
  [299] = {
    [sym_preproc_file_line] = STATE(299),
    [aux_sym_end_program_statement_token1] = ACTIONS(1099),
    [anon_sym_LPAREN] = ACTIONS(1099),
    [aux_sym_interface_statement_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1099),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1099),
    [aux_sym_end_function_statement_token1] = ACTIONS(1099),
    [aux_sym_contains_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token2] = ACTIONS(1099),
    [aux_sym_implicit_statement_token1] = ACTIONS(1099),
    [aux_sym_implicit_statement_token3] = ACTIONS(1099),
    [aux_sym_implicit_statement_token4] = ACTIONS(1099),
    [aux_sym_save_statement_token1] = ACTIONS(1099),
    [aux_sym_private_statement_token1] = ACTIONS(1099),
    [aux_sym_public_statement_token1] = ACTIONS(1099),
    [aux_sym_namelist_statement_token1] = ACTIONS(1099),
    [aux_sym_common_statement_token1] = ACTIONS(1099),
    [anon_sym_DASH] = ACTIONS(1097),
    [aux_sym_import_statement_token1] = ACTIONS(1099),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1099),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1099),
    [aux_sym_variable_attributes_token1] = ACTIONS(1099),
    [aux_sym_variable_attributes_token2] = ACTIONS(1099),
    [aux_sym_variable_attributes_token3] = ACTIONS(1099),
    [aux_sym_variable_attributes_token4] = ACTIONS(1099),
    [aux_sym_variable_attributes_token5] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1099),
    [aux_sym_derived_type_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token2] = ACTIONS(1099),
    [aux_sym_type_qualifier_token3] = ACTIONS(1099),
    [aux_sym_type_qualifier_token4] = ACTIONS(1099),
    [aux_sym_type_qualifier_token5] = ACTIONS(1099),
    [aux_sym_type_qualifier_token9] = ACTIONS(1099),
    [aux_sym_type_qualifier_token10] = ACTIONS(1099),
    [aux_sym_type_qualifier_token11] = ACTIONS(1099),
    [aux_sym_type_qualifier_token12] = ACTIONS(1099),
    [aux_sym_type_qualifier_token13] = ACTIONS(1099),
    [aux_sym_type_qualifier_token14] = ACTIONS(1099),
    [aux_sym_type_qualifier_token15] = ACTIONS(1099),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token2] = ACTIONS(1099),
    [aux_sym_subroutine_call_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token2] = ACTIONS(1099),
    [aux_sym_keyword_statement_token3] = ACTIONS(1099),
    [aux_sym_keyword_statement_token4] = ACTIONS(1099),
    [aux_sym_keyword_statement_token6] = ACTIONS(1099),
    [aux_sym_keyword_statement_token7] = ACTIONS(1099),
    [aux_sym_include_statement_token1] = ACTIONS(1099),
    [aux_sym_data_statement_token1] = ACTIONS(1099),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1099),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1099),
    [aux_sym_end_if_statement_token1] = ACTIONS(1099),
    [aux_sym_elseif_clause_token2] = ACTIONS(1099),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1099),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token3] = ACTIONS(1099),
    [aux_sym_select_type_statement_token1] = ACTIONS(1099),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1099),
    [aux_sym_block_construct_token1] = ACTIONS(1099),
    [aux_sym_associate_statement_token1] = ACTIONS(1099),
    [aux_sym_format_statement_token1] = ACTIONS(1099),
    [aux_sym_print_statement_token1] = ACTIONS(1099),
    [aux_sym_open_statement_token1] = ACTIONS(1099),
    [aux_sym_close_statement_token1] = ACTIONS(1099),
    [aux_sym_inquire_statement_token1] = ACTIONS(1099),
    [aux_sym_enum_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token2] = ACTIONS(1099),
    [aux_sym_file_position_statement_token3] = ACTIONS(1099),
    [aux_sym_file_position_statement_token4] = ACTIONS(1099),
    [aux_sym_logical_expression_token5] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [sym_boolean_literal] = ACTIONS(1097),
    [aux_sym_null_literal_token1] = ACTIONS(1099),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1099),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1097),
    [sym__float_literal] = ACTIONS(1097),
    [sym__boz_literal] = ACTIONS(1097),
    [sym_string_literal] = ACTIONS(1097),
  },
  [300] = {
    [sym_preproc_file_line] = STATE(300),
    [aux_sym_end_program_statement_token1] = ACTIONS(1287),
    [aux_sym_end_program_statement_token2] = ACTIONS(1287),
    [anon_sym_LPAREN] = ACTIONS(1287),
    [aux_sym_interface_statement_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1287),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1287),
    [aux_sym_contains_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token2] = ACTIONS(1287),
    [aux_sym_implicit_statement_token1] = ACTIONS(1287),
    [aux_sym_implicit_statement_token3] = ACTIONS(1287),
    [aux_sym_implicit_statement_token4] = ACTIONS(1287),
    [aux_sym_save_statement_token1] = ACTIONS(1287),
    [aux_sym_private_statement_token1] = ACTIONS(1287),
    [aux_sym_public_statement_token1] = ACTIONS(1287),
    [aux_sym_namelist_statement_token1] = ACTIONS(1287),
    [aux_sym_common_statement_token1] = ACTIONS(1287),
    [anon_sym_DASH] = ACTIONS(1289),
    [aux_sym_import_statement_token1] = ACTIONS(1287),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1287),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1287),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1287),
    [aux_sym_variable_attributes_token1] = ACTIONS(1287),
    [aux_sym_variable_attributes_token2] = ACTIONS(1287),
    [aux_sym_variable_attributes_token3] = ACTIONS(1287),
    [aux_sym_variable_attributes_token4] = ACTIONS(1287),
    [aux_sym_variable_attributes_token5] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1287),
    [aux_sym_derived_type_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token2] = ACTIONS(1287),
    [aux_sym_type_qualifier_token3] = ACTIONS(1287),
    [aux_sym_type_qualifier_token4] = ACTIONS(1287),
    [aux_sym_type_qualifier_token5] = ACTIONS(1287),
    [aux_sym_type_qualifier_token9] = ACTIONS(1287),
    [aux_sym_type_qualifier_token10] = ACTIONS(1287),
    [aux_sym_type_qualifier_token11] = ACTIONS(1287),
    [aux_sym_type_qualifier_token12] = ACTIONS(1287),
    [aux_sym_type_qualifier_token13] = ACTIONS(1287),
    [aux_sym_type_qualifier_token14] = ACTIONS(1287),
    [aux_sym_type_qualifier_token15] = ACTIONS(1287),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token2] = ACTIONS(1287),
    [aux_sym_subroutine_call_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token2] = ACTIONS(1287),
    [aux_sym_keyword_statement_token3] = ACTIONS(1287),
    [aux_sym_keyword_statement_token4] = ACTIONS(1287),
    [aux_sym_keyword_statement_token6] = ACTIONS(1287),
    [aux_sym_keyword_statement_token7] = ACTIONS(1287),
    [aux_sym_include_statement_token1] = ACTIONS(1287),
    [aux_sym_data_statement_token1] = ACTIONS(1287),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1287),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1287),
    [aux_sym_end_if_statement_token1] = ACTIONS(1287),
    [aux_sym_elseif_clause_token2] = ACTIONS(1287),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1287),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token3] = ACTIONS(1287),
    [aux_sym_select_type_statement_token1] = ACTIONS(1287),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1287),
    [aux_sym_block_construct_token1] = ACTIONS(1287),
    [aux_sym_associate_statement_token1] = ACTIONS(1287),
    [aux_sym_format_statement_token1] = ACTIONS(1287),
    [aux_sym_print_statement_token1] = ACTIONS(1287),
    [aux_sym_open_statement_token1] = ACTIONS(1287),
    [aux_sym_close_statement_token1] = ACTIONS(1287),
    [aux_sym_inquire_statement_token1] = ACTIONS(1287),
    [aux_sym_enum_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token2] = ACTIONS(1287),
    [aux_sym_file_position_statement_token3] = ACTIONS(1287),
    [aux_sym_file_position_statement_token4] = ACTIONS(1287),
    [aux_sym_logical_expression_token5] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1289),
    [anon_sym_DOT] = ACTIONS(1287),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1289),
    [anon_sym_LBRACK] = ACTIONS(1289),
    [sym_boolean_literal] = ACTIONS(1289),
    [aux_sym_null_literal_token1] = ACTIONS(1287),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1287),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1289),
    [sym__float_literal] = ACTIONS(1289),
    [sym__boz_literal] = ACTIONS(1289),
    [sym_string_literal] = ACTIONS(1289),
  },
  [301] = {
    [sym_preproc_file_line] = STATE(301),
    [aux_sym_end_program_statement_token1] = ACTIONS(1291),
    [aux_sym_end_program_statement_token2] = ACTIONS(1291),
    [anon_sym_LPAREN] = ACTIONS(1291),
    [aux_sym_interface_statement_token1] = ACTIONS(1291),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1291),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1291),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1291),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1291),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1291),
    [aux_sym_contains_statement_token1] = ACTIONS(1291),
    [aux_sym_use_statement_token1] = ACTIONS(1291),
    [aux_sym_use_statement_token2] = ACTIONS(1291),
    [aux_sym_implicit_statement_token1] = ACTIONS(1291),
    [aux_sym_implicit_statement_token3] = ACTIONS(1291),
    [aux_sym_implicit_statement_token4] = ACTIONS(1291),
    [aux_sym_save_statement_token1] = ACTIONS(1291),
    [aux_sym_private_statement_token1] = ACTIONS(1291),
    [aux_sym_public_statement_token1] = ACTIONS(1291),
    [aux_sym_namelist_statement_token1] = ACTIONS(1291),
    [aux_sym_common_statement_token1] = ACTIONS(1291),
    [anon_sym_DASH] = ACTIONS(1293),
    [aux_sym_import_statement_token1] = ACTIONS(1291),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1291),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1291),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1291),
    [aux_sym_variable_attributes_token1] = ACTIONS(1291),
    [aux_sym_variable_attributes_token2] = ACTIONS(1291),
    [aux_sym_variable_attributes_token3] = ACTIONS(1291),
    [aux_sym_variable_attributes_token4] = ACTIONS(1291),
    [aux_sym_variable_attributes_token5] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1291),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1291),
    [aux_sym_derived_type_token1] = ACTIONS(1291),
    [aux_sym_type_qualifier_token1] = ACTIONS(1291),
    [aux_sym_type_qualifier_token2] = ACTIONS(1291),
    [aux_sym_type_qualifier_token3] = ACTIONS(1291),
    [aux_sym_type_qualifier_token4] = ACTIONS(1291),
    [aux_sym_type_qualifier_token5] = ACTIONS(1291),
    [aux_sym_type_qualifier_token9] = ACTIONS(1291),
    [aux_sym_type_qualifier_token10] = ACTIONS(1291),
    [aux_sym_type_qualifier_token11] = ACTIONS(1291),
    [aux_sym_type_qualifier_token12] = ACTIONS(1291),
    [aux_sym_type_qualifier_token13] = ACTIONS(1291),
    [aux_sym_type_qualifier_token14] = ACTIONS(1291),
    [aux_sym_type_qualifier_token15] = ACTIONS(1291),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1291),
    [aux_sym_stop_statement_token1] = ACTIONS(1291),
    [aux_sym_stop_statement_token2] = ACTIONS(1291),
    [aux_sym_subroutine_call_token1] = ACTIONS(1291),
    [aux_sym_keyword_statement_token1] = ACTIONS(1291),
    [aux_sym_keyword_statement_token2] = ACTIONS(1291),
    [aux_sym_keyword_statement_token3] = ACTIONS(1291),
    [aux_sym_keyword_statement_token4] = ACTIONS(1291),
    [aux_sym_keyword_statement_token6] = ACTIONS(1291),
    [aux_sym_keyword_statement_token7] = ACTIONS(1291),
    [aux_sym_include_statement_token1] = ACTIONS(1291),
    [aux_sym_data_statement_token1] = ACTIONS(1291),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1291),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1291),
    [aux_sym_end_if_statement_token1] = ACTIONS(1291),
    [aux_sym_elseif_clause_token2] = ACTIONS(1291),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1291),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1291),
    [aux_sym_select_case_statement_token1] = ACTIONS(1291),
    [aux_sym_select_case_statement_token3] = ACTIONS(1291),
    [aux_sym_select_type_statement_token1] = ACTIONS(1291),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1291),
    [aux_sym_block_construct_token1] = ACTIONS(1291),
    [aux_sym_associate_statement_token1] = ACTIONS(1291),
    [aux_sym_format_statement_token1] = ACTIONS(1291),
    [aux_sym_print_statement_token1] = ACTIONS(1291),
    [aux_sym_open_statement_token1] = ACTIONS(1291),
    [aux_sym_close_statement_token1] = ACTIONS(1291),
    [aux_sym_inquire_statement_token1] = ACTIONS(1291),
    [aux_sym_enum_statement_token1] = ACTIONS(1291),
    [aux_sym_file_position_statement_token1] = ACTIONS(1291),
    [aux_sym_file_position_statement_token2] = ACTIONS(1291),
    [aux_sym_file_position_statement_token3] = ACTIONS(1291),
    [aux_sym_file_position_statement_token4] = ACTIONS(1291),
    [aux_sym_logical_expression_token5] = ACTIONS(1293),
    [anon_sym_PLUS] = ACTIONS(1293),
    [anon_sym_DOT] = ACTIONS(1291),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1293),
    [anon_sym_LBRACK] = ACTIONS(1293),
    [sym_boolean_literal] = ACTIONS(1293),
    [aux_sym_null_literal_token1] = ACTIONS(1291),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1291),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1293),
    [sym__float_literal] = ACTIONS(1293),
    [sym__boz_literal] = ACTIONS(1293),
    [sym_string_literal] = ACTIONS(1293),
  },
  [302] = {
    [sym_preproc_file_line] = STATE(302),
    [aux_sym_end_program_statement_token1] = ACTIONS(1247),
    [anon_sym_LPAREN] = ACTIONS(1247),
    [aux_sym_interface_statement_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1247),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1247),
    [aux_sym_end_function_statement_token1] = ACTIONS(1247),
    [aux_sym_contains_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token2] = ACTIONS(1247),
    [aux_sym_implicit_statement_token1] = ACTIONS(1247),
    [aux_sym_implicit_statement_token3] = ACTIONS(1247),
    [aux_sym_implicit_statement_token4] = ACTIONS(1247),
    [aux_sym_save_statement_token1] = ACTIONS(1247),
    [aux_sym_private_statement_token1] = ACTIONS(1247),
    [aux_sym_public_statement_token1] = ACTIONS(1247),
    [aux_sym_namelist_statement_token1] = ACTIONS(1247),
    [aux_sym_common_statement_token1] = ACTIONS(1247),
    [anon_sym_DASH] = ACTIONS(1245),
    [aux_sym_import_statement_token1] = ACTIONS(1247),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1247),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1247),
    [aux_sym_variable_attributes_token1] = ACTIONS(1247),
    [aux_sym_variable_attributes_token2] = ACTIONS(1247),
    [aux_sym_variable_attributes_token3] = ACTIONS(1247),
    [aux_sym_variable_attributes_token4] = ACTIONS(1247),
    [aux_sym_variable_attributes_token5] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1247),
    [aux_sym_derived_type_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token2] = ACTIONS(1247),
    [aux_sym_type_qualifier_token3] = ACTIONS(1247),
    [aux_sym_type_qualifier_token4] = ACTIONS(1247),
    [aux_sym_type_qualifier_token5] = ACTIONS(1247),
    [aux_sym_type_qualifier_token9] = ACTIONS(1247),
    [aux_sym_type_qualifier_token10] = ACTIONS(1247),
    [aux_sym_type_qualifier_token11] = ACTIONS(1247),
    [aux_sym_type_qualifier_token12] = ACTIONS(1247),
    [aux_sym_type_qualifier_token13] = ACTIONS(1247),
    [aux_sym_type_qualifier_token14] = ACTIONS(1247),
    [aux_sym_type_qualifier_token15] = ACTIONS(1247),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token2] = ACTIONS(1247),
    [aux_sym_subroutine_call_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token2] = ACTIONS(1247),
    [aux_sym_keyword_statement_token3] = ACTIONS(1247),
    [aux_sym_keyword_statement_token4] = ACTIONS(1247),
    [aux_sym_keyword_statement_token6] = ACTIONS(1247),
    [aux_sym_keyword_statement_token7] = ACTIONS(1247),
    [aux_sym_include_statement_token1] = ACTIONS(1247),
    [aux_sym_data_statement_token1] = ACTIONS(1247),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1247),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1247),
    [aux_sym_end_if_statement_token1] = ACTIONS(1247),
    [aux_sym_elseif_clause_token2] = ACTIONS(1247),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1247),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token3] = ACTIONS(1247),
    [aux_sym_select_type_statement_token1] = ACTIONS(1247),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1247),
    [aux_sym_block_construct_token1] = ACTIONS(1247),
    [aux_sym_associate_statement_token1] = ACTIONS(1247),
    [aux_sym_format_statement_token1] = ACTIONS(1247),
    [aux_sym_print_statement_token1] = ACTIONS(1247),
    [aux_sym_open_statement_token1] = ACTIONS(1247),
    [aux_sym_close_statement_token1] = ACTIONS(1247),
    [aux_sym_inquire_statement_token1] = ACTIONS(1247),
    [aux_sym_enum_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token2] = ACTIONS(1247),
    [aux_sym_file_position_statement_token3] = ACTIONS(1247),
    [aux_sym_file_position_statement_token4] = ACTIONS(1247),
    [aux_sym_logical_expression_token5] = ACTIONS(1245),
    [anon_sym_PLUS] = ACTIONS(1245),
    [anon_sym_DOT] = ACTIONS(1247),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1245),
    [anon_sym_LBRACK] = ACTIONS(1245),
    [sym_boolean_literal] = ACTIONS(1245),
    [aux_sym_null_literal_token1] = ACTIONS(1247),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1247),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1245),
    [sym__float_literal] = ACTIONS(1245),
    [sym__boz_literal] = ACTIONS(1245),
    [sym_string_literal] = ACTIONS(1245),
  },
  [303] = {
    [sym_preproc_file_line] = STATE(303),
    [aux_sym_end_program_statement_token1] = ACTIONS(1295),
    [anon_sym_LPAREN] = ACTIONS(1295),
    [aux_sym_interface_statement_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1295),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1295),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1295),
    [aux_sym_contains_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token2] = ACTIONS(1295),
    [aux_sym_implicit_statement_token1] = ACTIONS(1295),
    [aux_sym_implicit_statement_token3] = ACTIONS(1295),
    [aux_sym_implicit_statement_token4] = ACTIONS(1295),
    [aux_sym_save_statement_token1] = ACTIONS(1295),
    [aux_sym_private_statement_token1] = ACTIONS(1295),
    [aux_sym_public_statement_token1] = ACTIONS(1295),
    [aux_sym_namelist_statement_token1] = ACTIONS(1295),
    [aux_sym_common_statement_token1] = ACTIONS(1295),
    [anon_sym_DASH] = ACTIONS(1297),
    [aux_sym_import_statement_token1] = ACTIONS(1295),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1295),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1295),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1295),
    [aux_sym_variable_attributes_token1] = ACTIONS(1295),
    [aux_sym_variable_attributes_token2] = ACTIONS(1295),
    [aux_sym_variable_attributes_token3] = ACTIONS(1295),
    [aux_sym_variable_attributes_token4] = ACTIONS(1295),
    [aux_sym_variable_attributes_token5] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1295),
    [aux_sym_derived_type_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token2] = ACTIONS(1295),
    [aux_sym_type_qualifier_token3] = ACTIONS(1295),
    [aux_sym_type_qualifier_token4] = ACTIONS(1295),
    [aux_sym_type_qualifier_token5] = ACTIONS(1295),
    [aux_sym_type_qualifier_token9] = ACTIONS(1295),
    [aux_sym_type_qualifier_token10] = ACTIONS(1295),
    [aux_sym_type_qualifier_token11] = ACTIONS(1295),
    [aux_sym_type_qualifier_token12] = ACTIONS(1295),
    [aux_sym_type_qualifier_token13] = ACTIONS(1295),
    [aux_sym_type_qualifier_token14] = ACTIONS(1295),
    [aux_sym_type_qualifier_token15] = ACTIONS(1295),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token2] = ACTIONS(1295),
    [aux_sym_subroutine_call_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token2] = ACTIONS(1295),
    [aux_sym_keyword_statement_token3] = ACTIONS(1295),
    [aux_sym_keyword_statement_token4] = ACTIONS(1295),
    [aux_sym_keyword_statement_token6] = ACTIONS(1295),
    [aux_sym_keyword_statement_token7] = ACTIONS(1295),
    [aux_sym_include_statement_token1] = ACTIONS(1295),
    [aux_sym_data_statement_token1] = ACTIONS(1295),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1295),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1295),
    [aux_sym_end_if_statement_token1] = ACTIONS(1295),
    [aux_sym_elseif_clause_token2] = ACTIONS(1295),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1295),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token3] = ACTIONS(1295),
    [aux_sym_select_type_statement_token1] = ACTIONS(1295),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1295),
    [aux_sym_block_construct_token1] = ACTIONS(1295),
    [aux_sym_associate_statement_token1] = ACTIONS(1295),
    [aux_sym_format_statement_token1] = ACTIONS(1295),
    [aux_sym_print_statement_token1] = ACTIONS(1295),
    [aux_sym_open_statement_token1] = ACTIONS(1295),
    [aux_sym_close_statement_token1] = ACTIONS(1295),
    [aux_sym_inquire_statement_token1] = ACTIONS(1295),
    [aux_sym_enum_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token2] = ACTIONS(1295),
    [aux_sym_file_position_statement_token3] = ACTIONS(1295),
    [aux_sym_file_position_statement_token4] = ACTIONS(1295),
    [aux_sym_logical_expression_token5] = ACTIONS(1297),
    [anon_sym_PLUS] = ACTIONS(1297),
    [anon_sym_DOT] = ACTIONS(1295),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1297),
    [anon_sym_LBRACK] = ACTIONS(1297),
    [sym_boolean_literal] = ACTIONS(1297),
    [aux_sym_null_literal_token1] = ACTIONS(1295),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1295),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1297),
    [sym__float_literal] = ACTIONS(1297),
    [sym__boz_literal] = ACTIONS(1297),
    [sym_string_literal] = ACTIONS(1297),
  },
  [304] = {
    [sym_preproc_file_line] = STATE(304),
    [aux_sym_end_program_statement_token1] = ACTIONS(1299),
    [anon_sym_LPAREN] = ACTIONS(1299),
    [aux_sym_interface_statement_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1299),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1299),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1299),
    [aux_sym_contains_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token2] = ACTIONS(1299),
    [aux_sym_implicit_statement_token1] = ACTIONS(1299),
    [aux_sym_implicit_statement_token3] = ACTIONS(1299),
    [aux_sym_implicit_statement_token4] = ACTIONS(1299),
    [aux_sym_save_statement_token1] = ACTIONS(1299),
    [aux_sym_private_statement_token1] = ACTIONS(1299),
    [aux_sym_public_statement_token1] = ACTIONS(1299),
    [aux_sym_namelist_statement_token1] = ACTIONS(1299),
    [aux_sym_common_statement_token1] = ACTIONS(1299),
    [anon_sym_DASH] = ACTIONS(1301),
    [aux_sym_import_statement_token1] = ACTIONS(1299),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1299),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1299),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1299),
    [aux_sym_variable_attributes_token1] = ACTIONS(1299),
    [aux_sym_variable_attributes_token2] = ACTIONS(1299),
    [aux_sym_variable_attributes_token3] = ACTIONS(1299),
    [aux_sym_variable_attributes_token4] = ACTIONS(1299),
    [aux_sym_variable_attributes_token5] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1299),
    [aux_sym_derived_type_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token2] = ACTIONS(1299),
    [aux_sym_type_qualifier_token3] = ACTIONS(1299),
    [aux_sym_type_qualifier_token4] = ACTIONS(1299),
    [aux_sym_type_qualifier_token5] = ACTIONS(1299),
    [aux_sym_type_qualifier_token9] = ACTIONS(1299),
    [aux_sym_type_qualifier_token10] = ACTIONS(1299),
    [aux_sym_type_qualifier_token11] = ACTIONS(1299),
    [aux_sym_type_qualifier_token12] = ACTIONS(1299),
    [aux_sym_type_qualifier_token13] = ACTIONS(1299),
    [aux_sym_type_qualifier_token14] = ACTIONS(1299),
    [aux_sym_type_qualifier_token15] = ACTIONS(1299),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token2] = ACTIONS(1299),
    [aux_sym_subroutine_call_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token2] = ACTIONS(1299),
    [aux_sym_keyword_statement_token3] = ACTIONS(1299),
    [aux_sym_keyword_statement_token4] = ACTIONS(1299),
    [aux_sym_keyword_statement_token6] = ACTIONS(1299),
    [aux_sym_keyword_statement_token7] = ACTIONS(1299),
    [aux_sym_include_statement_token1] = ACTIONS(1299),
    [aux_sym_data_statement_token1] = ACTIONS(1299),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1299),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1299),
    [aux_sym_end_if_statement_token1] = ACTIONS(1299),
    [aux_sym_elseif_clause_token2] = ACTIONS(1299),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1299),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token3] = ACTIONS(1299),
    [aux_sym_select_type_statement_token1] = ACTIONS(1299),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1299),
    [aux_sym_block_construct_token1] = ACTIONS(1299),
    [aux_sym_associate_statement_token1] = ACTIONS(1299),
    [aux_sym_format_statement_token1] = ACTIONS(1299),
    [aux_sym_print_statement_token1] = ACTIONS(1299),
    [aux_sym_open_statement_token1] = ACTIONS(1299),
    [aux_sym_close_statement_token1] = ACTIONS(1299),
    [aux_sym_inquire_statement_token1] = ACTIONS(1299),
    [aux_sym_enum_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token2] = ACTIONS(1299),
    [aux_sym_file_position_statement_token3] = ACTIONS(1299),
    [aux_sym_file_position_statement_token4] = ACTIONS(1299),
    [aux_sym_logical_expression_token5] = ACTIONS(1301),
    [anon_sym_PLUS] = ACTIONS(1301),
    [anon_sym_DOT] = ACTIONS(1299),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1301),
    [anon_sym_LBRACK] = ACTIONS(1301),
    [sym_boolean_literal] = ACTIONS(1301),
    [aux_sym_null_literal_token1] = ACTIONS(1299),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1299),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1301),
    [sym__float_literal] = ACTIONS(1301),
    [sym__boz_literal] = ACTIONS(1301),
    [sym_string_literal] = ACTIONS(1301),
  },
  [305] = {
    [sym_preproc_file_line] = STATE(305),
    [aux_sym_end_program_statement_token1] = ACTIONS(1303),
    [anon_sym_LPAREN] = ACTIONS(1303),
    [aux_sym_interface_statement_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1303),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1303),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1303),
    [aux_sym_contains_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token2] = ACTIONS(1303),
    [aux_sym_implicit_statement_token1] = ACTIONS(1303),
    [aux_sym_implicit_statement_token3] = ACTIONS(1303),
    [aux_sym_implicit_statement_token4] = ACTIONS(1303),
    [aux_sym_save_statement_token1] = ACTIONS(1303),
    [aux_sym_private_statement_token1] = ACTIONS(1303),
    [aux_sym_public_statement_token1] = ACTIONS(1303),
    [aux_sym_namelist_statement_token1] = ACTIONS(1303),
    [aux_sym_common_statement_token1] = ACTIONS(1303),
    [anon_sym_DASH] = ACTIONS(1305),
    [aux_sym_import_statement_token1] = ACTIONS(1303),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1303),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1303),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1303),
    [aux_sym_variable_attributes_token1] = ACTIONS(1303),
    [aux_sym_variable_attributes_token2] = ACTIONS(1303),
    [aux_sym_variable_attributes_token3] = ACTIONS(1303),
    [aux_sym_variable_attributes_token4] = ACTIONS(1303),
    [aux_sym_variable_attributes_token5] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1303),
    [aux_sym_derived_type_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token2] = ACTIONS(1303),
    [aux_sym_type_qualifier_token3] = ACTIONS(1303),
    [aux_sym_type_qualifier_token4] = ACTIONS(1303),
    [aux_sym_type_qualifier_token5] = ACTIONS(1303),
    [aux_sym_type_qualifier_token9] = ACTIONS(1303),
    [aux_sym_type_qualifier_token10] = ACTIONS(1303),
    [aux_sym_type_qualifier_token11] = ACTIONS(1303),
    [aux_sym_type_qualifier_token12] = ACTIONS(1303),
    [aux_sym_type_qualifier_token13] = ACTIONS(1303),
    [aux_sym_type_qualifier_token14] = ACTIONS(1303),
    [aux_sym_type_qualifier_token15] = ACTIONS(1303),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token2] = ACTIONS(1303),
    [aux_sym_subroutine_call_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token2] = ACTIONS(1303),
    [aux_sym_keyword_statement_token3] = ACTIONS(1303),
    [aux_sym_keyword_statement_token4] = ACTIONS(1303),
    [aux_sym_keyword_statement_token6] = ACTIONS(1303),
    [aux_sym_keyword_statement_token7] = ACTIONS(1303),
    [aux_sym_include_statement_token1] = ACTIONS(1303),
    [aux_sym_data_statement_token1] = ACTIONS(1303),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1303),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1303),
    [aux_sym_end_if_statement_token1] = ACTIONS(1303),
    [aux_sym_elseif_clause_token2] = ACTIONS(1303),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1303),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token3] = ACTIONS(1303),
    [aux_sym_select_type_statement_token1] = ACTIONS(1303),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1303),
    [aux_sym_block_construct_token1] = ACTIONS(1303),
    [aux_sym_associate_statement_token1] = ACTIONS(1303),
    [aux_sym_format_statement_token1] = ACTIONS(1303),
    [aux_sym_print_statement_token1] = ACTIONS(1303),
    [aux_sym_open_statement_token1] = ACTIONS(1303),
    [aux_sym_close_statement_token1] = ACTIONS(1303),
    [aux_sym_inquire_statement_token1] = ACTIONS(1303),
    [aux_sym_enum_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token2] = ACTIONS(1303),
    [aux_sym_file_position_statement_token3] = ACTIONS(1303),
    [aux_sym_file_position_statement_token4] = ACTIONS(1303),
    [aux_sym_logical_expression_token5] = ACTIONS(1305),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_DOT] = ACTIONS(1303),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1305),
    [anon_sym_LBRACK] = ACTIONS(1305),
    [sym_boolean_literal] = ACTIONS(1305),
    [aux_sym_null_literal_token1] = ACTIONS(1303),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1303),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1305),
    [sym__float_literal] = ACTIONS(1305),
    [sym__boz_literal] = ACTIONS(1305),
    [sym_string_literal] = ACTIONS(1305),
  },
  [306] = {
    [sym_preproc_file_line] = STATE(306),
    [aux_sym_end_program_statement_token1] = ACTIONS(1275),
    [anon_sym_LPAREN] = ACTIONS(1275),
    [aux_sym_interface_statement_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1275),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1275),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1275),
    [aux_sym_contains_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token2] = ACTIONS(1275),
    [aux_sym_implicit_statement_token1] = ACTIONS(1275),
    [aux_sym_implicit_statement_token3] = ACTIONS(1275),
    [aux_sym_implicit_statement_token4] = ACTIONS(1275),
    [aux_sym_save_statement_token1] = ACTIONS(1275),
    [aux_sym_private_statement_token1] = ACTIONS(1275),
    [aux_sym_public_statement_token1] = ACTIONS(1275),
    [aux_sym_namelist_statement_token1] = ACTIONS(1275),
    [aux_sym_common_statement_token1] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1277),
    [aux_sym_import_statement_token1] = ACTIONS(1275),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1275),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1275),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1275),
    [aux_sym_variable_attributes_token1] = ACTIONS(1275),
    [aux_sym_variable_attributes_token2] = ACTIONS(1275),
    [aux_sym_variable_attributes_token3] = ACTIONS(1275),
    [aux_sym_variable_attributes_token4] = ACTIONS(1275),
    [aux_sym_variable_attributes_token5] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1275),
    [aux_sym_derived_type_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token2] = ACTIONS(1275),
    [aux_sym_type_qualifier_token3] = ACTIONS(1275),
    [aux_sym_type_qualifier_token4] = ACTIONS(1275),
    [aux_sym_type_qualifier_token5] = ACTIONS(1275),
    [aux_sym_type_qualifier_token9] = ACTIONS(1275),
    [aux_sym_type_qualifier_token10] = ACTIONS(1275),
    [aux_sym_type_qualifier_token11] = ACTIONS(1275),
    [aux_sym_type_qualifier_token12] = ACTIONS(1275),
    [aux_sym_type_qualifier_token13] = ACTIONS(1275),
    [aux_sym_type_qualifier_token14] = ACTIONS(1275),
    [aux_sym_type_qualifier_token15] = ACTIONS(1275),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token2] = ACTIONS(1275),
    [aux_sym_subroutine_call_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token2] = ACTIONS(1275),
    [aux_sym_keyword_statement_token3] = ACTIONS(1275),
    [aux_sym_keyword_statement_token4] = ACTIONS(1275),
    [aux_sym_keyword_statement_token6] = ACTIONS(1275),
    [aux_sym_keyword_statement_token7] = ACTIONS(1275),
    [aux_sym_include_statement_token1] = ACTIONS(1275),
    [aux_sym_data_statement_token1] = ACTIONS(1275),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1275),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1275),
    [aux_sym_end_if_statement_token1] = ACTIONS(1275),
    [aux_sym_elseif_clause_token2] = ACTIONS(1275),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1275),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token3] = ACTIONS(1275),
    [aux_sym_select_type_statement_token1] = ACTIONS(1275),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1275),
    [aux_sym_block_construct_token1] = ACTIONS(1275),
    [aux_sym_associate_statement_token1] = ACTIONS(1275),
    [aux_sym_format_statement_token1] = ACTIONS(1275),
    [aux_sym_print_statement_token1] = ACTIONS(1275),
    [aux_sym_open_statement_token1] = ACTIONS(1275),
    [aux_sym_close_statement_token1] = ACTIONS(1275),
    [aux_sym_inquire_statement_token1] = ACTIONS(1275),
    [aux_sym_enum_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token2] = ACTIONS(1275),
    [aux_sym_file_position_statement_token3] = ACTIONS(1275),
    [aux_sym_file_position_statement_token4] = ACTIONS(1275),
    [aux_sym_logical_expression_token5] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1277),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [sym_boolean_literal] = ACTIONS(1277),
    [aux_sym_null_literal_token1] = ACTIONS(1275),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1275),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1277),
    [sym__float_literal] = ACTIONS(1277),
    [sym__boz_literal] = ACTIONS(1277),
    [sym_string_literal] = ACTIONS(1277),
  },
  [307] = {
    [sym_preproc_file_line] = STATE(307),
    [aux_sym_end_program_statement_token1] = ACTIONS(1307),
    [anon_sym_LPAREN] = ACTIONS(1307),
    [aux_sym_interface_statement_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1307),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1307),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1307),
    [aux_sym_contains_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token2] = ACTIONS(1307),
    [aux_sym_implicit_statement_token1] = ACTIONS(1307),
    [aux_sym_implicit_statement_token3] = ACTIONS(1307),
    [aux_sym_implicit_statement_token4] = ACTIONS(1307),
    [aux_sym_save_statement_token1] = ACTIONS(1307),
    [aux_sym_private_statement_token1] = ACTIONS(1307),
    [aux_sym_public_statement_token1] = ACTIONS(1307),
    [aux_sym_namelist_statement_token1] = ACTIONS(1307),
    [aux_sym_common_statement_token1] = ACTIONS(1307),
    [anon_sym_DASH] = ACTIONS(1309),
    [aux_sym_import_statement_token1] = ACTIONS(1307),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1307),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1307),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1307),
    [aux_sym_variable_attributes_token1] = ACTIONS(1307),
    [aux_sym_variable_attributes_token2] = ACTIONS(1307),
    [aux_sym_variable_attributes_token3] = ACTIONS(1307),
    [aux_sym_variable_attributes_token4] = ACTIONS(1307),
    [aux_sym_variable_attributes_token5] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1307),
    [aux_sym_derived_type_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token2] = ACTIONS(1307),
    [aux_sym_type_qualifier_token3] = ACTIONS(1307),
    [aux_sym_type_qualifier_token4] = ACTIONS(1307),
    [aux_sym_type_qualifier_token5] = ACTIONS(1307),
    [aux_sym_type_qualifier_token9] = ACTIONS(1307),
    [aux_sym_type_qualifier_token10] = ACTIONS(1307),
    [aux_sym_type_qualifier_token11] = ACTIONS(1307),
    [aux_sym_type_qualifier_token12] = ACTIONS(1307),
    [aux_sym_type_qualifier_token13] = ACTIONS(1307),
    [aux_sym_type_qualifier_token14] = ACTIONS(1307),
    [aux_sym_type_qualifier_token15] = ACTIONS(1307),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token2] = ACTIONS(1307),
    [aux_sym_subroutine_call_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token2] = ACTIONS(1307),
    [aux_sym_keyword_statement_token3] = ACTIONS(1307),
    [aux_sym_keyword_statement_token4] = ACTIONS(1307),
    [aux_sym_keyword_statement_token6] = ACTIONS(1307),
    [aux_sym_keyword_statement_token7] = ACTIONS(1307),
    [aux_sym_include_statement_token1] = ACTIONS(1307),
    [aux_sym_data_statement_token1] = ACTIONS(1307),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1307),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1307),
    [aux_sym_end_if_statement_token1] = ACTIONS(1307),
    [aux_sym_elseif_clause_token2] = ACTIONS(1307),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1307),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token3] = ACTIONS(1307),
    [aux_sym_select_type_statement_token1] = ACTIONS(1307),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1307),
    [aux_sym_block_construct_token1] = ACTIONS(1307),
    [aux_sym_associate_statement_token1] = ACTIONS(1307),
    [aux_sym_format_statement_token1] = ACTIONS(1307),
    [aux_sym_print_statement_token1] = ACTIONS(1307),
    [aux_sym_open_statement_token1] = ACTIONS(1307),
    [aux_sym_close_statement_token1] = ACTIONS(1307),
    [aux_sym_inquire_statement_token1] = ACTIONS(1307),
    [aux_sym_enum_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token2] = ACTIONS(1307),
    [aux_sym_file_position_statement_token3] = ACTIONS(1307),
    [aux_sym_file_position_statement_token4] = ACTIONS(1307),
    [aux_sym_logical_expression_token5] = ACTIONS(1309),
    [anon_sym_PLUS] = ACTIONS(1309),
    [anon_sym_DOT] = ACTIONS(1307),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1309),
    [anon_sym_LBRACK] = ACTIONS(1309),
    [sym_boolean_literal] = ACTIONS(1309),
    [aux_sym_null_literal_token1] = ACTIONS(1307),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1307),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1309),
    [sym__float_literal] = ACTIONS(1309),
    [sym__boz_literal] = ACTIONS(1309),
    [sym_string_literal] = ACTIONS(1309),
  },
  [308] = {
    [sym_preproc_file_line] = STATE(308),
    [aux_sym_end_program_statement_token1] = ACTIONS(1311),
    [anon_sym_LPAREN] = ACTIONS(1311),
    [aux_sym_interface_statement_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1311),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1311),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1311),
    [aux_sym_contains_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token2] = ACTIONS(1311),
    [aux_sym_implicit_statement_token1] = ACTIONS(1311),
    [aux_sym_implicit_statement_token3] = ACTIONS(1311),
    [aux_sym_implicit_statement_token4] = ACTIONS(1311),
    [aux_sym_save_statement_token1] = ACTIONS(1311),
    [aux_sym_private_statement_token1] = ACTIONS(1311),
    [aux_sym_public_statement_token1] = ACTIONS(1311),
    [aux_sym_namelist_statement_token1] = ACTIONS(1311),
    [aux_sym_common_statement_token1] = ACTIONS(1311),
    [anon_sym_DASH] = ACTIONS(1313),
    [aux_sym_import_statement_token1] = ACTIONS(1311),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1311),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1311),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1311),
    [aux_sym_variable_attributes_token1] = ACTIONS(1311),
    [aux_sym_variable_attributes_token2] = ACTIONS(1311),
    [aux_sym_variable_attributes_token3] = ACTIONS(1311),
    [aux_sym_variable_attributes_token4] = ACTIONS(1311),
    [aux_sym_variable_attributes_token5] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1311),
    [aux_sym_derived_type_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token2] = ACTIONS(1311),
    [aux_sym_type_qualifier_token3] = ACTIONS(1311),
    [aux_sym_type_qualifier_token4] = ACTIONS(1311),
    [aux_sym_type_qualifier_token5] = ACTIONS(1311),
    [aux_sym_type_qualifier_token9] = ACTIONS(1311),
    [aux_sym_type_qualifier_token10] = ACTIONS(1311),
    [aux_sym_type_qualifier_token11] = ACTIONS(1311),
    [aux_sym_type_qualifier_token12] = ACTIONS(1311),
    [aux_sym_type_qualifier_token13] = ACTIONS(1311),
    [aux_sym_type_qualifier_token14] = ACTIONS(1311),
    [aux_sym_type_qualifier_token15] = ACTIONS(1311),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token2] = ACTIONS(1311),
    [aux_sym_subroutine_call_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token2] = ACTIONS(1311),
    [aux_sym_keyword_statement_token3] = ACTIONS(1311),
    [aux_sym_keyword_statement_token4] = ACTIONS(1311),
    [aux_sym_keyword_statement_token6] = ACTIONS(1311),
    [aux_sym_keyword_statement_token7] = ACTIONS(1311),
    [aux_sym_include_statement_token1] = ACTIONS(1311),
    [aux_sym_data_statement_token1] = ACTIONS(1311),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1311),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1311),
    [aux_sym_end_if_statement_token1] = ACTIONS(1311),
    [aux_sym_elseif_clause_token2] = ACTIONS(1311),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1311),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token3] = ACTIONS(1311),
    [aux_sym_select_type_statement_token1] = ACTIONS(1311),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1311),
    [aux_sym_block_construct_token1] = ACTIONS(1311),
    [aux_sym_associate_statement_token1] = ACTIONS(1311),
    [aux_sym_format_statement_token1] = ACTIONS(1311),
    [aux_sym_print_statement_token1] = ACTIONS(1311),
    [aux_sym_open_statement_token1] = ACTIONS(1311),
    [aux_sym_close_statement_token1] = ACTIONS(1311),
    [aux_sym_inquire_statement_token1] = ACTIONS(1311),
    [aux_sym_enum_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token2] = ACTIONS(1311),
    [aux_sym_file_position_statement_token3] = ACTIONS(1311),
    [aux_sym_file_position_statement_token4] = ACTIONS(1311),
    [aux_sym_logical_expression_token5] = ACTIONS(1313),
    [anon_sym_PLUS] = ACTIONS(1313),
    [anon_sym_DOT] = ACTIONS(1311),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1313),
    [anon_sym_LBRACK] = ACTIONS(1313),
    [sym_boolean_literal] = ACTIONS(1313),
    [aux_sym_null_literal_token1] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1311),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1313),
    [sym__float_literal] = ACTIONS(1313),
    [sym__boz_literal] = ACTIONS(1313),
    [sym_string_literal] = ACTIONS(1313),
  },
  [309] = {
    [sym_preproc_file_line] = STATE(309),
    [aux_sym_end_program_statement_token1] = ACTIONS(1147),
    [anon_sym_LPAREN] = ACTIONS(1147),
    [aux_sym_interface_statement_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1147),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1147),
    [aux_sym_end_function_statement_token1] = ACTIONS(1147),
    [aux_sym_contains_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token2] = ACTIONS(1147),
    [aux_sym_implicit_statement_token1] = ACTIONS(1147),
    [aux_sym_implicit_statement_token3] = ACTIONS(1147),
    [aux_sym_implicit_statement_token4] = ACTIONS(1147),
    [aux_sym_save_statement_token1] = ACTIONS(1147),
    [aux_sym_private_statement_token1] = ACTIONS(1147),
    [aux_sym_public_statement_token1] = ACTIONS(1147),
    [aux_sym_namelist_statement_token1] = ACTIONS(1147),
    [aux_sym_common_statement_token1] = ACTIONS(1147),
    [anon_sym_DASH] = ACTIONS(1145),
    [aux_sym_import_statement_token1] = ACTIONS(1147),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1147),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1147),
    [aux_sym_variable_attributes_token1] = ACTIONS(1147),
    [aux_sym_variable_attributes_token2] = ACTIONS(1147),
    [aux_sym_variable_attributes_token3] = ACTIONS(1147),
    [aux_sym_variable_attributes_token4] = ACTIONS(1147),
    [aux_sym_variable_attributes_token5] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1147),
    [aux_sym_derived_type_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token2] = ACTIONS(1147),
    [aux_sym_type_qualifier_token3] = ACTIONS(1147),
    [aux_sym_type_qualifier_token4] = ACTIONS(1147),
    [aux_sym_type_qualifier_token5] = ACTIONS(1147),
    [aux_sym_type_qualifier_token9] = ACTIONS(1147),
    [aux_sym_type_qualifier_token10] = ACTIONS(1147),
    [aux_sym_type_qualifier_token11] = ACTIONS(1147),
    [aux_sym_type_qualifier_token12] = ACTIONS(1147),
    [aux_sym_type_qualifier_token13] = ACTIONS(1147),
    [aux_sym_type_qualifier_token14] = ACTIONS(1147),
    [aux_sym_type_qualifier_token15] = ACTIONS(1147),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token2] = ACTIONS(1147),
    [aux_sym_subroutine_call_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token2] = ACTIONS(1147),
    [aux_sym_keyword_statement_token3] = ACTIONS(1147),
    [aux_sym_keyword_statement_token4] = ACTIONS(1147),
    [aux_sym_keyword_statement_token6] = ACTIONS(1147),
    [aux_sym_keyword_statement_token7] = ACTIONS(1147),
    [aux_sym_include_statement_token1] = ACTIONS(1147),
    [aux_sym_data_statement_token1] = ACTIONS(1147),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1147),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1147),
    [aux_sym_end_if_statement_token1] = ACTIONS(1147),
    [aux_sym_elseif_clause_token2] = ACTIONS(1147),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1147),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token3] = ACTIONS(1147),
    [aux_sym_select_type_statement_token1] = ACTIONS(1147),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1147),
    [aux_sym_block_construct_token1] = ACTIONS(1147),
    [aux_sym_associate_statement_token1] = ACTIONS(1147),
    [aux_sym_format_statement_token1] = ACTIONS(1147),
    [aux_sym_print_statement_token1] = ACTIONS(1147),
    [aux_sym_open_statement_token1] = ACTIONS(1147),
    [aux_sym_close_statement_token1] = ACTIONS(1147),
    [aux_sym_inquire_statement_token1] = ACTIONS(1147),
    [aux_sym_enum_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token2] = ACTIONS(1147),
    [aux_sym_file_position_statement_token3] = ACTIONS(1147),
    [aux_sym_file_position_statement_token4] = ACTIONS(1147),
    [aux_sym_logical_expression_token5] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1145),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [sym_boolean_literal] = ACTIONS(1145),
    [aux_sym_null_literal_token1] = ACTIONS(1147),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1147),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1145),
    [sym__float_literal] = ACTIONS(1145),
    [sym__boz_literal] = ACTIONS(1145),
    [sym_string_literal] = ACTIONS(1145),
  },
  [310] = {
    [sym_preproc_file_line] = STATE(310),
    [aux_sym_end_program_statement_token1] = ACTIONS(1315),
    [aux_sym_end_program_statement_token2] = ACTIONS(1315),
    [anon_sym_LPAREN] = ACTIONS(1315),
    [aux_sym_interface_statement_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1315),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1315),
    [aux_sym_contains_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token2] = ACTIONS(1315),
    [aux_sym_implicit_statement_token1] = ACTIONS(1315),
    [aux_sym_implicit_statement_token3] = ACTIONS(1315),
    [aux_sym_implicit_statement_token4] = ACTIONS(1315),
    [aux_sym_save_statement_token1] = ACTIONS(1315),
    [aux_sym_private_statement_token1] = ACTIONS(1315),
    [aux_sym_public_statement_token1] = ACTIONS(1315),
    [aux_sym_namelist_statement_token1] = ACTIONS(1315),
    [aux_sym_common_statement_token1] = ACTIONS(1315),
    [anon_sym_DASH] = ACTIONS(1317),
    [aux_sym_import_statement_token1] = ACTIONS(1315),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1315),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1315),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1315),
    [aux_sym_variable_attributes_token1] = ACTIONS(1315),
    [aux_sym_variable_attributes_token2] = ACTIONS(1315),
    [aux_sym_variable_attributes_token3] = ACTIONS(1315),
    [aux_sym_variable_attributes_token4] = ACTIONS(1315),
    [aux_sym_variable_attributes_token5] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1315),
    [aux_sym_derived_type_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token2] = ACTIONS(1315),
    [aux_sym_type_qualifier_token3] = ACTIONS(1315),
    [aux_sym_type_qualifier_token4] = ACTIONS(1315),
    [aux_sym_type_qualifier_token5] = ACTIONS(1315),
    [aux_sym_type_qualifier_token9] = ACTIONS(1315),
    [aux_sym_type_qualifier_token10] = ACTIONS(1315),
    [aux_sym_type_qualifier_token11] = ACTIONS(1315),
    [aux_sym_type_qualifier_token12] = ACTIONS(1315),
    [aux_sym_type_qualifier_token13] = ACTIONS(1315),
    [aux_sym_type_qualifier_token14] = ACTIONS(1315),
    [aux_sym_type_qualifier_token15] = ACTIONS(1315),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token2] = ACTIONS(1315),
    [aux_sym_subroutine_call_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token2] = ACTIONS(1315),
    [aux_sym_keyword_statement_token3] = ACTIONS(1315),
    [aux_sym_keyword_statement_token4] = ACTIONS(1315),
    [aux_sym_keyword_statement_token6] = ACTIONS(1315),
    [aux_sym_keyword_statement_token7] = ACTIONS(1315),
    [aux_sym_include_statement_token1] = ACTIONS(1315),
    [aux_sym_data_statement_token1] = ACTIONS(1315),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1315),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1315),
    [aux_sym_end_if_statement_token1] = ACTIONS(1315),
    [aux_sym_elseif_clause_token2] = ACTIONS(1315),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1315),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token3] = ACTIONS(1315),
    [aux_sym_select_type_statement_token1] = ACTIONS(1315),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1315),
    [aux_sym_block_construct_token1] = ACTIONS(1315),
    [aux_sym_associate_statement_token1] = ACTIONS(1315),
    [aux_sym_format_statement_token1] = ACTIONS(1315),
    [aux_sym_print_statement_token1] = ACTIONS(1315),
    [aux_sym_open_statement_token1] = ACTIONS(1315),
    [aux_sym_close_statement_token1] = ACTIONS(1315),
    [aux_sym_inquire_statement_token1] = ACTIONS(1315),
    [aux_sym_enum_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token2] = ACTIONS(1315),
    [aux_sym_file_position_statement_token3] = ACTIONS(1315),
    [aux_sym_file_position_statement_token4] = ACTIONS(1315),
    [aux_sym_logical_expression_token5] = ACTIONS(1317),
    [anon_sym_PLUS] = ACTIONS(1317),
    [anon_sym_DOT] = ACTIONS(1315),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1317),
    [anon_sym_LBRACK] = ACTIONS(1317),
    [sym_boolean_literal] = ACTIONS(1317),
    [aux_sym_null_literal_token1] = ACTIONS(1315),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1315),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1317),
    [sym__float_literal] = ACTIONS(1317),
    [sym__boz_literal] = ACTIONS(1317),
    [sym_string_literal] = ACTIONS(1317),
  },
  [311] = {
    [sym_preproc_file_line] = STATE(311),
    [aux_sym_end_program_statement_token1] = ACTIONS(1319),
    [aux_sym_end_program_statement_token2] = ACTIONS(1319),
    [anon_sym_LPAREN] = ACTIONS(1319),
    [aux_sym_interface_statement_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1319),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1319),
    [aux_sym_contains_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token2] = ACTIONS(1319),
    [aux_sym_implicit_statement_token1] = ACTIONS(1319),
    [aux_sym_implicit_statement_token3] = ACTIONS(1319),
    [aux_sym_implicit_statement_token4] = ACTIONS(1319),
    [aux_sym_save_statement_token1] = ACTIONS(1319),
    [aux_sym_private_statement_token1] = ACTIONS(1319),
    [aux_sym_public_statement_token1] = ACTIONS(1319),
    [aux_sym_namelist_statement_token1] = ACTIONS(1319),
    [aux_sym_common_statement_token1] = ACTIONS(1319),
    [anon_sym_DASH] = ACTIONS(1321),
    [aux_sym_import_statement_token1] = ACTIONS(1319),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1319),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1319),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1319),
    [aux_sym_variable_attributes_token1] = ACTIONS(1319),
    [aux_sym_variable_attributes_token2] = ACTIONS(1319),
    [aux_sym_variable_attributes_token3] = ACTIONS(1319),
    [aux_sym_variable_attributes_token4] = ACTIONS(1319),
    [aux_sym_variable_attributes_token5] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1319),
    [aux_sym_derived_type_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token2] = ACTIONS(1319),
    [aux_sym_type_qualifier_token3] = ACTIONS(1319),
    [aux_sym_type_qualifier_token4] = ACTIONS(1319),
    [aux_sym_type_qualifier_token5] = ACTIONS(1319),
    [aux_sym_type_qualifier_token9] = ACTIONS(1319),
    [aux_sym_type_qualifier_token10] = ACTIONS(1319),
    [aux_sym_type_qualifier_token11] = ACTIONS(1319),
    [aux_sym_type_qualifier_token12] = ACTIONS(1319),
    [aux_sym_type_qualifier_token13] = ACTIONS(1319),
    [aux_sym_type_qualifier_token14] = ACTIONS(1319),
    [aux_sym_type_qualifier_token15] = ACTIONS(1319),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token2] = ACTIONS(1319),
    [aux_sym_subroutine_call_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token2] = ACTIONS(1319),
    [aux_sym_keyword_statement_token3] = ACTIONS(1319),
    [aux_sym_keyword_statement_token4] = ACTIONS(1319),
    [aux_sym_keyword_statement_token6] = ACTIONS(1319),
    [aux_sym_keyword_statement_token7] = ACTIONS(1319),
    [aux_sym_include_statement_token1] = ACTIONS(1319),
    [aux_sym_data_statement_token1] = ACTIONS(1319),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1319),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1319),
    [aux_sym_end_if_statement_token1] = ACTIONS(1319),
    [aux_sym_elseif_clause_token2] = ACTIONS(1319),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1319),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token3] = ACTIONS(1319),
    [aux_sym_select_type_statement_token1] = ACTIONS(1319),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1319),
    [aux_sym_block_construct_token1] = ACTIONS(1319),
    [aux_sym_associate_statement_token1] = ACTIONS(1319),
    [aux_sym_format_statement_token1] = ACTIONS(1319),
    [aux_sym_print_statement_token1] = ACTIONS(1319),
    [aux_sym_open_statement_token1] = ACTIONS(1319),
    [aux_sym_close_statement_token1] = ACTIONS(1319),
    [aux_sym_inquire_statement_token1] = ACTIONS(1319),
    [aux_sym_enum_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token2] = ACTIONS(1319),
    [aux_sym_file_position_statement_token3] = ACTIONS(1319),
    [aux_sym_file_position_statement_token4] = ACTIONS(1319),
    [aux_sym_logical_expression_token5] = ACTIONS(1321),
    [anon_sym_PLUS] = ACTIONS(1321),
    [anon_sym_DOT] = ACTIONS(1319),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1321),
    [anon_sym_LBRACK] = ACTIONS(1321),
    [sym_boolean_literal] = ACTIONS(1321),
    [aux_sym_null_literal_token1] = ACTIONS(1319),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1319),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1321),
    [sym__float_literal] = ACTIONS(1321),
    [sym__boz_literal] = ACTIONS(1321),
    [sym_string_literal] = ACTIONS(1321),
  },
  [312] = {
    [sym_preproc_file_line] = STATE(312),
    [aux_sym_end_program_statement_token1] = ACTIONS(1311),
    [aux_sym_end_program_statement_token2] = ACTIONS(1311),
    [anon_sym_LPAREN] = ACTIONS(1311),
    [aux_sym_interface_statement_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1311),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1311),
    [aux_sym_contains_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token2] = ACTIONS(1311),
    [aux_sym_implicit_statement_token1] = ACTIONS(1311),
    [aux_sym_implicit_statement_token3] = ACTIONS(1311),
    [aux_sym_implicit_statement_token4] = ACTIONS(1311),
    [aux_sym_save_statement_token1] = ACTIONS(1311),
    [aux_sym_private_statement_token1] = ACTIONS(1311),
    [aux_sym_public_statement_token1] = ACTIONS(1311),
    [aux_sym_namelist_statement_token1] = ACTIONS(1311),
    [aux_sym_common_statement_token1] = ACTIONS(1311),
    [anon_sym_DASH] = ACTIONS(1313),
    [aux_sym_import_statement_token1] = ACTIONS(1311),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1311),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1311),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1311),
    [aux_sym_variable_attributes_token1] = ACTIONS(1311),
    [aux_sym_variable_attributes_token2] = ACTIONS(1311),
    [aux_sym_variable_attributes_token3] = ACTIONS(1311),
    [aux_sym_variable_attributes_token4] = ACTIONS(1311),
    [aux_sym_variable_attributes_token5] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1311),
    [aux_sym_derived_type_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token2] = ACTIONS(1311),
    [aux_sym_type_qualifier_token3] = ACTIONS(1311),
    [aux_sym_type_qualifier_token4] = ACTIONS(1311),
    [aux_sym_type_qualifier_token5] = ACTIONS(1311),
    [aux_sym_type_qualifier_token9] = ACTIONS(1311),
    [aux_sym_type_qualifier_token10] = ACTIONS(1311),
    [aux_sym_type_qualifier_token11] = ACTIONS(1311),
    [aux_sym_type_qualifier_token12] = ACTIONS(1311),
    [aux_sym_type_qualifier_token13] = ACTIONS(1311),
    [aux_sym_type_qualifier_token14] = ACTIONS(1311),
    [aux_sym_type_qualifier_token15] = ACTIONS(1311),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token2] = ACTIONS(1311),
    [aux_sym_subroutine_call_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token2] = ACTIONS(1311),
    [aux_sym_keyword_statement_token3] = ACTIONS(1311),
    [aux_sym_keyword_statement_token4] = ACTIONS(1311),
    [aux_sym_keyword_statement_token6] = ACTIONS(1311),
    [aux_sym_keyword_statement_token7] = ACTIONS(1311),
    [aux_sym_include_statement_token1] = ACTIONS(1311),
    [aux_sym_data_statement_token1] = ACTIONS(1311),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1311),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1311),
    [aux_sym_end_if_statement_token1] = ACTIONS(1311),
    [aux_sym_elseif_clause_token2] = ACTIONS(1311),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1311),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token3] = ACTIONS(1311),
    [aux_sym_select_type_statement_token1] = ACTIONS(1311),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1311),
    [aux_sym_block_construct_token1] = ACTIONS(1311),
    [aux_sym_associate_statement_token1] = ACTIONS(1311),
    [aux_sym_format_statement_token1] = ACTIONS(1311),
    [aux_sym_print_statement_token1] = ACTIONS(1311),
    [aux_sym_open_statement_token1] = ACTIONS(1311),
    [aux_sym_close_statement_token1] = ACTIONS(1311),
    [aux_sym_inquire_statement_token1] = ACTIONS(1311),
    [aux_sym_enum_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token2] = ACTIONS(1311),
    [aux_sym_file_position_statement_token3] = ACTIONS(1311),
    [aux_sym_file_position_statement_token4] = ACTIONS(1311),
    [aux_sym_logical_expression_token5] = ACTIONS(1313),
    [anon_sym_PLUS] = ACTIONS(1313),
    [anon_sym_DOT] = ACTIONS(1311),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1313),
    [anon_sym_LBRACK] = ACTIONS(1313),
    [sym_boolean_literal] = ACTIONS(1313),
    [aux_sym_null_literal_token1] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1311),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1313),
    [sym__float_literal] = ACTIONS(1313),
    [sym__boz_literal] = ACTIONS(1313),
    [sym_string_literal] = ACTIONS(1313),
  },
  [313] = {
    [sym_preproc_file_line] = STATE(313),
    [aux_sym_end_program_statement_token1] = ACTIONS(1307),
    [aux_sym_end_program_statement_token2] = ACTIONS(1307),
    [anon_sym_LPAREN] = ACTIONS(1307),
    [aux_sym_interface_statement_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1307),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1307),
    [aux_sym_contains_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token2] = ACTIONS(1307),
    [aux_sym_implicit_statement_token1] = ACTIONS(1307),
    [aux_sym_implicit_statement_token3] = ACTIONS(1307),
    [aux_sym_implicit_statement_token4] = ACTIONS(1307),
    [aux_sym_save_statement_token1] = ACTIONS(1307),
    [aux_sym_private_statement_token1] = ACTIONS(1307),
    [aux_sym_public_statement_token1] = ACTIONS(1307),
    [aux_sym_namelist_statement_token1] = ACTIONS(1307),
    [aux_sym_common_statement_token1] = ACTIONS(1307),
    [anon_sym_DASH] = ACTIONS(1309),
    [aux_sym_import_statement_token1] = ACTIONS(1307),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1307),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1307),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1307),
    [aux_sym_variable_attributes_token1] = ACTIONS(1307),
    [aux_sym_variable_attributes_token2] = ACTIONS(1307),
    [aux_sym_variable_attributes_token3] = ACTIONS(1307),
    [aux_sym_variable_attributes_token4] = ACTIONS(1307),
    [aux_sym_variable_attributes_token5] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1307),
    [aux_sym_derived_type_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token2] = ACTIONS(1307),
    [aux_sym_type_qualifier_token3] = ACTIONS(1307),
    [aux_sym_type_qualifier_token4] = ACTIONS(1307),
    [aux_sym_type_qualifier_token5] = ACTIONS(1307),
    [aux_sym_type_qualifier_token9] = ACTIONS(1307),
    [aux_sym_type_qualifier_token10] = ACTIONS(1307),
    [aux_sym_type_qualifier_token11] = ACTIONS(1307),
    [aux_sym_type_qualifier_token12] = ACTIONS(1307),
    [aux_sym_type_qualifier_token13] = ACTIONS(1307),
    [aux_sym_type_qualifier_token14] = ACTIONS(1307),
    [aux_sym_type_qualifier_token15] = ACTIONS(1307),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token2] = ACTIONS(1307),
    [aux_sym_subroutine_call_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token2] = ACTIONS(1307),
    [aux_sym_keyword_statement_token3] = ACTIONS(1307),
    [aux_sym_keyword_statement_token4] = ACTIONS(1307),
    [aux_sym_keyword_statement_token6] = ACTIONS(1307),
    [aux_sym_keyword_statement_token7] = ACTIONS(1307),
    [aux_sym_include_statement_token1] = ACTIONS(1307),
    [aux_sym_data_statement_token1] = ACTIONS(1307),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1307),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1307),
    [aux_sym_end_if_statement_token1] = ACTIONS(1307),
    [aux_sym_elseif_clause_token2] = ACTIONS(1307),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1307),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token3] = ACTIONS(1307),
    [aux_sym_select_type_statement_token1] = ACTIONS(1307),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1307),
    [aux_sym_block_construct_token1] = ACTIONS(1307),
    [aux_sym_associate_statement_token1] = ACTIONS(1307),
    [aux_sym_format_statement_token1] = ACTIONS(1307),
    [aux_sym_print_statement_token1] = ACTIONS(1307),
    [aux_sym_open_statement_token1] = ACTIONS(1307),
    [aux_sym_close_statement_token1] = ACTIONS(1307),
    [aux_sym_inquire_statement_token1] = ACTIONS(1307),
    [aux_sym_enum_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token2] = ACTIONS(1307),
    [aux_sym_file_position_statement_token3] = ACTIONS(1307),
    [aux_sym_file_position_statement_token4] = ACTIONS(1307),
    [aux_sym_logical_expression_token5] = ACTIONS(1309),
    [anon_sym_PLUS] = ACTIONS(1309),
    [anon_sym_DOT] = ACTIONS(1307),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1309),
    [anon_sym_LBRACK] = ACTIONS(1309),
    [sym_boolean_literal] = ACTIONS(1309),
    [aux_sym_null_literal_token1] = ACTIONS(1307),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1307),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1309),
    [sym__float_literal] = ACTIONS(1309),
    [sym__boz_literal] = ACTIONS(1309),
    [sym_string_literal] = ACTIONS(1309),
  },
  [314] = {
    [sym_preproc_file_line] = STATE(314),
    [aux_sym_end_program_statement_token1] = ACTIONS(1323),
    [anon_sym_LPAREN] = ACTIONS(1323),
    [aux_sym_interface_statement_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1323),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1323),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1323),
    [aux_sym_contains_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token2] = ACTIONS(1323),
    [aux_sym_implicit_statement_token1] = ACTIONS(1323),
    [aux_sym_implicit_statement_token3] = ACTIONS(1323),
    [aux_sym_implicit_statement_token4] = ACTIONS(1323),
    [aux_sym_save_statement_token1] = ACTIONS(1323),
    [aux_sym_private_statement_token1] = ACTIONS(1323),
    [aux_sym_public_statement_token1] = ACTIONS(1323),
    [aux_sym_namelist_statement_token1] = ACTIONS(1323),
    [aux_sym_common_statement_token1] = ACTIONS(1323),
    [anon_sym_DASH] = ACTIONS(1325),
    [aux_sym_import_statement_token1] = ACTIONS(1323),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1323),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1323),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1323),
    [aux_sym_variable_attributes_token1] = ACTIONS(1323),
    [aux_sym_variable_attributes_token2] = ACTIONS(1323),
    [aux_sym_variable_attributes_token3] = ACTIONS(1323),
    [aux_sym_variable_attributes_token4] = ACTIONS(1323),
    [aux_sym_variable_attributes_token5] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1323),
    [aux_sym_derived_type_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token2] = ACTIONS(1323),
    [aux_sym_type_qualifier_token3] = ACTIONS(1323),
    [aux_sym_type_qualifier_token4] = ACTIONS(1323),
    [aux_sym_type_qualifier_token5] = ACTIONS(1323),
    [aux_sym_type_qualifier_token9] = ACTIONS(1323),
    [aux_sym_type_qualifier_token10] = ACTIONS(1323),
    [aux_sym_type_qualifier_token11] = ACTIONS(1323),
    [aux_sym_type_qualifier_token12] = ACTIONS(1323),
    [aux_sym_type_qualifier_token13] = ACTIONS(1323),
    [aux_sym_type_qualifier_token14] = ACTIONS(1323),
    [aux_sym_type_qualifier_token15] = ACTIONS(1323),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token2] = ACTIONS(1323),
    [aux_sym_subroutine_call_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token2] = ACTIONS(1323),
    [aux_sym_keyword_statement_token3] = ACTIONS(1323),
    [aux_sym_keyword_statement_token4] = ACTIONS(1323),
    [aux_sym_keyword_statement_token6] = ACTIONS(1323),
    [aux_sym_keyword_statement_token7] = ACTIONS(1323),
    [aux_sym_include_statement_token1] = ACTIONS(1323),
    [aux_sym_data_statement_token1] = ACTIONS(1323),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1323),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1323),
    [aux_sym_end_if_statement_token1] = ACTIONS(1323),
    [aux_sym_elseif_clause_token2] = ACTIONS(1323),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1323),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token3] = ACTIONS(1323),
    [aux_sym_select_type_statement_token1] = ACTIONS(1323),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1323),
    [aux_sym_block_construct_token1] = ACTIONS(1323),
    [aux_sym_associate_statement_token1] = ACTIONS(1323),
    [aux_sym_format_statement_token1] = ACTIONS(1323),
    [aux_sym_print_statement_token1] = ACTIONS(1323),
    [aux_sym_open_statement_token1] = ACTIONS(1323),
    [aux_sym_close_statement_token1] = ACTIONS(1323),
    [aux_sym_inquire_statement_token1] = ACTIONS(1323),
    [aux_sym_enum_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token2] = ACTIONS(1323),
    [aux_sym_file_position_statement_token3] = ACTIONS(1323),
    [aux_sym_file_position_statement_token4] = ACTIONS(1323),
    [aux_sym_logical_expression_token5] = ACTIONS(1325),
    [anon_sym_PLUS] = ACTIONS(1325),
    [anon_sym_DOT] = ACTIONS(1323),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1325),
    [anon_sym_LBRACK] = ACTIONS(1325),
    [sym_boolean_literal] = ACTIONS(1325),
    [aux_sym_null_literal_token1] = ACTIONS(1323),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1323),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1325),
    [sym__float_literal] = ACTIONS(1325),
    [sym__boz_literal] = ACTIONS(1325),
    [sym_string_literal] = ACTIONS(1325),
  },
  [315] = {
    [sym_preproc_file_line] = STATE(315),
    [aux_sym_end_program_statement_token1] = ACTIONS(1287),
    [anon_sym_LPAREN] = ACTIONS(1287),
    [aux_sym_interface_statement_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1287),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1287),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1287),
    [aux_sym_contains_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token2] = ACTIONS(1287),
    [aux_sym_implicit_statement_token1] = ACTIONS(1287),
    [aux_sym_implicit_statement_token3] = ACTIONS(1287),
    [aux_sym_implicit_statement_token4] = ACTIONS(1287),
    [aux_sym_save_statement_token1] = ACTIONS(1287),
    [aux_sym_private_statement_token1] = ACTIONS(1287),
    [aux_sym_public_statement_token1] = ACTIONS(1287),
    [aux_sym_namelist_statement_token1] = ACTIONS(1287),
    [aux_sym_common_statement_token1] = ACTIONS(1287),
    [anon_sym_DASH] = ACTIONS(1289),
    [aux_sym_import_statement_token1] = ACTIONS(1287),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1287),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1287),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1287),
    [aux_sym_variable_attributes_token1] = ACTIONS(1287),
    [aux_sym_variable_attributes_token2] = ACTIONS(1287),
    [aux_sym_variable_attributes_token3] = ACTIONS(1287),
    [aux_sym_variable_attributes_token4] = ACTIONS(1287),
    [aux_sym_variable_attributes_token5] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1287),
    [aux_sym_derived_type_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token2] = ACTIONS(1287),
    [aux_sym_type_qualifier_token3] = ACTIONS(1287),
    [aux_sym_type_qualifier_token4] = ACTIONS(1287),
    [aux_sym_type_qualifier_token5] = ACTIONS(1287),
    [aux_sym_type_qualifier_token9] = ACTIONS(1287),
    [aux_sym_type_qualifier_token10] = ACTIONS(1287),
    [aux_sym_type_qualifier_token11] = ACTIONS(1287),
    [aux_sym_type_qualifier_token12] = ACTIONS(1287),
    [aux_sym_type_qualifier_token13] = ACTIONS(1287),
    [aux_sym_type_qualifier_token14] = ACTIONS(1287),
    [aux_sym_type_qualifier_token15] = ACTIONS(1287),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token2] = ACTIONS(1287),
    [aux_sym_subroutine_call_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token2] = ACTIONS(1287),
    [aux_sym_keyword_statement_token3] = ACTIONS(1287),
    [aux_sym_keyword_statement_token4] = ACTIONS(1287),
    [aux_sym_keyword_statement_token6] = ACTIONS(1287),
    [aux_sym_keyword_statement_token7] = ACTIONS(1287),
    [aux_sym_include_statement_token1] = ACTIONS(1287),
    [aux_sym_data_statement_token1] = ACTIONS(1287),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1287),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1287),
    [aux_sym_end_if_statement_token1] = ACTIONS(1287),
    [aux_sym_elseif_clause_token2] = ACTIONS(1287),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1287),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token3] = ACTIONS(1287),
    [aux_sym_select_type_statement_token1] = ACTIONS(1287),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1287),
    [aux_sym_block_construct_token1] = ACTIONS(1287),
    [aux_sym_associate_statement_token1] = ACTIONS(1287),
    [aux_sym_format_statement_token1] = ACTIONS(1287),
    [aux_sym_print_statement_token1] = ACTIONS(1287),
    [aux_sym_open_statement_token1] = ACTIONS(1287),
    [aux_sym_close_statement_token1] = ACTIONS(1287),
    [aux_sym_inquire_statement_token1] = ACTIONS(1287),
    [aux_sym_enum_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token2] = ACTIONS(1287),
    [aux_sym_file_position_statement_token3] = ACTIONS(1287),
    [aux_sym_file_position_statement_token4] = ACTIONS(1287),
    [aux_sym_logical_expression_token5] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1289),
    [anon_sym_DOT] = ACTIONS(1287),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1289),
    [anon_sym_LBRACK] = ACTIONS(1289),
    [sym_boolean_literal] = ACTIONS(1289),
    [aux_sym_null_literal_token1] = ACTIONS(1287),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1287),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1289),
    [sym__float_literal] = ACTIONS(1289),
    [sym__boz_literal] = ACTIONS(1289),
    [sym_string_literal] = ACTIONS(1289),
  },
  [316] = {
    [sym_preproc_file_line] = STATE(316),
    [aux_sym_end_program_statement_token1] = ACTIONS(1327),
    [aux_sym_end_program_statement_token2] = ACTIONS(1327),
    [anon_sym_LPAREN] = ACTIONS(1327),
    [aux_sym_interface_statement_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1327),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1327),
    [aux_sym_contains_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token2] = ACTIONS(1327),
    [aux_sym_implicit_statement_token1] = ACTIONS(1327),
    [aux_sym_implicit_statement_token3] = ACTIONS(1327),
    [aux_sym_implicit_statement_token4] = ACTIONS(1327),
    [aux_sym_save_statement_token1] = ACTIONS(1327),
    [aux_sym_private_statement_token1] = ACTIONS(1327),
    [aux_sym_public_statement_token1] = ACTIONS(1327),
    [aux_sym_namelist_statement_token1] = ACTIONS(1327),
    [aux_sym_common_statement_token1] = ACTIONS(1327),
    [anon_sym_DASH] = ACTIONS(1329),
    [aux_sym_import_statement_token1] = ACTIONS(1327),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1327),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1327),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1327),
    [aux_sym_variable_attributes_token1] = ACTIONS(1327),
    [aux_sym_variable_attributes_token2] = ACTIONS(1327),
    [aux_sym_variable_attributes_token3] = ACTIONS(1327),
    [aux_sym_variable_attributes_token4] = ACTIONS(1327),
    [aux_sym_variable_attributes_token5] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1327),
    [aux_sym_derived_type_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token2] = ACTIONS(1327),
    [aux_sym_type_qualifier_token3] = ACTIONS(1327),
    [aux_sym_type_qualifier_token4] = ACTIONS(1327),
    [aux_sym_type_qualifier_token5] = ACTIONS(1327),
    [aux_sym_type_qualifier_token9] = ACTIONS(1327),
    [aux_sym_type_qualifier_token10] = ACTIONS(1327),
    [aux_sym_type_qualifier_token11] = ACTIONS(1327),
    [aux_sym_type_qualifier_token12] = ACTIONS(1327),
    [aux_sym_type_qualifier_token13] = ACTIONS(1327),
    [aux_sym_type_qualifier_token14] = ACTIONS(1327),
    [aux_sym_type_qualifier_token15] = ACTIONS(1327),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token2] = ACTIONS(1327),
    [aux_sym_subroutine_call_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token2] = ACTIONS(1327),
    [aux_sym_keyword_statement_token3] = ACTIONS(1327),
    [aux_sym_keyword_statement_token4] = ACTIONS(1327),
    [aux_sym_keyword_statement_token6] = ACTIONS(1327),
    [aux_sym_keyword_statement_token7] = ACTIONS(1327),
    [aux_sym_include_statement_token1] = ACTIONS(1327),
    [aux_sym_data_statement_token1] = ACTIONS(1327),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1327),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1327),
    [aux_sym_end_if_statement_token1] = ACTIONS(1327),
    [aux_sym_elseif_clause_token2] = ACTIONS(1327),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1327),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token3] = ACTIONS(1327),
    [aux_sym_select_type_statement_token1] = ACTIONS(1327),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1327),
    [aux_sym_block_construct_token1] = ACTIONS(1327),
    [aux_sym_associate_statement_token1] = ACTIONS(1327),
    [aux_sym_format_statement_token1] = ACTIONS(1327),
    [aux_sym_print_statement_token1] = ACTIONS(1327),
    [aux_sym_open_statement_token1] = ACTIONS(1327),
    [aux_sym_close_statement_token1] = ACTIONS(1327),
    [aux_sym_inquire_statement_token1] = ACTIONS(1327),
    [aux_sym_enum_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token2] = ACTIONS(1327),
    [aux_sym_file_position_statement_token3] = ACTIONS(1327),
    [aux_sym_file_position_statement_token4] = ACTIONS(1327),
    [aux_sym_logical_expression_token5] = ACTIONS(1329),
    [anon_sym_PLUS] = ACTIONS(1329),
    [anon_sym_DOT] = ACTIONS(1327),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1329),
    [anon_sym_LBRACK] = ACTIONS(1329),
    [sym_boolean_literal] = ACTIONS(1329),
    [aux_sym_null_literal_token1] = ACTIONS(1327),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1327),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1329),
    [sym__float_literal] = ACTIONS(1329),
    [sym__boz_literal] = ACTIONS(1329),
    [sym_string_literal] = ACTIONS(1329),
  },
  [317] = {
    [sym_preproc_file_line] = STATE(317),
    [aux_sym_end_program_statement_token1] = ACTIONS(1331),
    [aux_sym_end_program_statement_token2] = ACTIONS(1331),
    [anon_sym_LPAREN] = ACTIONS(1331),
    [aux_sym_interface_statement_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1331),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1331),
    [aux_sym_contains_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token2] = ACTIONS(1331),
    [aux_sym_implicit_statement_token1] = ACTIONS(1331),
    [aux_sym_implicit_statement_token3] = ACTIONS(1331),
    [aux_sym_implicit_statement_token4] = ACTIONS(1331),
    [aux_sym_save_statement_token1] = ACTIONS(1331),
    [aux_sym_private_statement_token1] = ACTIONS(1331),
    [aux_sym_public_statement_token1] = ACTIONS(1331),
    [aux_sym_namelist_statement_token1] = ACTIONS(1331),
    [aux_sym_common_statement_token1] = ACTIONS(1331),
    [anon_sym_DASH] = ACTIONS(1333),
    [aux_sym_import_statement_token1] = ACTIONS(1331),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1331),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1331),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1331),
    [aux_sym_variable_attributes_token1] = ACTIONS(1331),
    [aux_sym_variable_attributes_token2] = ACTIONS(1331),
    [aux_sym_variable_attributes_token3] = ACTIONS(1331),
    [aux_sym_variable_attributes_token4] = ACTIONS(1331),
    [aux_sym_variable_attributes_token5] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1331),
    [aux_sym_derived_type_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token2] = ACTIONS(1331),
    [aux_sym_type_qualifier_token3] = ACTIONS(1331),
    [aux_sym_type_qualifier_token4] = ACTIONS(1331),
    [aux_sym_type_qualifier_token5] = ACTIONS(1331),
    [aux_sym_type_qualifier_token9] = ACTIONS(1331),
    [aux_sym_type_qualifier_token10] = ACTIONS(1331),
    [aux_sym_type_qualifier_token11] = ACTIONS(1331),
    [aux_sym_type_qualifier_token12] = ACTIONS(1331),
    [aux_sym_type_qualifier_token13] = ACTIONS(1331),
    [aux_sym_type_qualifier_token14] = ACTIONS(1331),
    [aux_sym_type_qualifier_token15] = ACTIONS(1331),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token2] = ACTIONS(1331),
    [aux_sym_subroutine_call_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token2] = ACTIONS(1331),
    [aux_sym_keyword_statement_token3] = ACTIONS(1331),
    [aux_sym_keyword_statement_token4] = ACTIONS(1331),
    [aux_sym_keyword_statement_token6] = ACTIONS(1331),
    [aux_sym_keyword_statement_token7] = ACTIONS(1331),
    [aux_sym_include_statement_token1] = ACTIONS(1331),
    [aux_sym_data_statement_token1] = ACTIONS(1331),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1331),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1331),
    [aux_sym_end_if_statement_token1] = ACTIONS(1331),
    [aux_sym_elseif_clause_token2] = ACTIONS(1331),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1331),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token3] = ACTIONS(1331),
    [aux_sym_select_type_statement_token1] = ACTIONS(1331),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1331),
    [aux_sym_block_construct_token1] = ACTIONS(1331),
    [aux_sym_associate_statement_token1] = ACTIONS(1331),
    [aux_sym_format_statement_token1] = ACTIONS(1331),
    [aux_sym_print_statement_token1] = ACTIONS(1331),
    [aux_sym_open_statement_token1] = ACTIONS(1331),
    [aux_sym_close_statement_token1] = ACTIONS(1331),
    [aux_sym_inquire_statement_token1] = ACTIONS(1331),
    [aux_sym_enum_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token2] = ACTIONS(1331),
    [aux_sym_file_position_statement_token3] = ACTIONS(1331),
    [aux_sym_file_position_statement_token4] = ACTIONS(1331),
    [aux_sym_logical_expression_token5] = ACTIONS(1333),
    [anon_sym_PLUS] = ACTIONS(1333),
    [anon_sym_DOT] = ACTIONS(1331),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1333),
    [anon_sym_LBRACK] = ACTIONS(1333),
    [sym_boolean_literal] = ACTIONS(1333),
    [aux_sym_null_literal_token1] = ACTIONS(1331),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1331),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1333),
    [sym__float_literal] = ACTIONS(1333),
    [sym__boz_literal] = ACTIONS(1333),
    [sym_string_literal] = ACTIONS(1333),
  },
  [318] = {
    [sym_preproc_file_line] = STATE(318),
    [aux_sym_end_program_statement_token1] = ACTIONS(1119),
    [anon_sym_LPAREN] = ACTIONS(1119),
    [aux_sym_interface_statement_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1119),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1119),
    [aux_sym_end_function_statement_token1] = ACTIONS(1119),
    [aux_sym_contains_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token2] = ACTIONS(1119),
    [aux_sym_implicit_statement_token1] = ACTIONS(1119),
    [aux_sym_implicit_statement_token3] = ACTIONS(1119),
    [aux_sym_implicit_statement_token4] = ACTIONS(1119),
    [aux_sym_save_statement_token1] = ACTIONS(1119),
    [aux_sym_private_statement_token1] = ACTIONS(1119),
    [aux_sym_public_statement_token1] = ACTIONS(1119),
    [aux_sym_namelist_statement_token1] = ACTIONS(1119),
    [aux_sym_common_statement_token1] = ACTIONS(1119),
    [anon_sym_DASH] = ACTIONS(1117),
    [aux_sym_import_statement_token1] = ACTIONS(1119),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1119),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1119),
    [aux_sym_variable_attributes_token1] = ACTIONS(1119),
    [aux_sym_variable_attributes_token2] = ACTIONS(1119),
    [aux_sym_variable_attributes_token3] = ACTIONS(1119),
    [aux_sym_variable_attributes_token4] = ACTIONS(1119),
    [aux_sym_variable_attributes_token5] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1119),
    [aux_sym_derived_type_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token2] = ACTIONS(1119),
    [aux_sym_type_qualifier_token3] = ACTIONS(1119),
    [aux_sym_type_qualifier_token4] = ACTIONS(1119),
    [aux_sym_type_qualifier_token5] = ACTIONS(1119),
    [aux_sym_type_qualifier_token9] = ACTIONS(1119),
    [aux_sym_type_qualifier_token10] = ACTIONS(1119),
    [aux_sym_type_qualifier_token11] = ACTIONS(1119),
    [aux_sym_type_qualifier_token12] = ACTIONS(1119),
    [aux_sym_type_qualifier_token13] = ACTIONS(1119),
    [aux_sym_type_qualifier_token14] = ACTIONS(1119),
    [aux_sym_type_qualifier_token15] = ACTIONS(1119),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token2] = ACTIONS(1119),
    [aux_sym_subroutine_call_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token2] = ACTIONS(1119),
    [aux_sym_keyword_statement_token3] = ACTIONS(1119),
    [aux_sym_keyword_statement_token4] = ACTIONS(1119),
    [aux_sym_keyword_statement_token6] = ACTIONS(1119),
    [aux_sym_keyword_statement_token7] = ACTIONS(1119),
    [aux_sym_include_statement_token1] = ACTIONS(1119),
    [aux_sym_data_statement_token1] = ACTIONS(1119),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1119),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1119),
    [aux_sym_end_if_statement_token1] = ACTIONS(1119),
    [aux_sym_elseif_clause_token2] = ACTIONS(1119),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1119),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token3] = ACTIONS(1119),
    [aux_sym_select_type_statement_token1] = ACTIONS(1119),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1119),
    [aux_sym_block_construct_token1] = ACTIONS(1119),
    [aux_sym_associate_statement_token1] = ACTIONS(1119),
    [aux_sym_format_statement_token1] = ACTIONS(1119),
    [aux_sym_print_statement_token1] = ACTIONS(1119),
    [aux_sym_open_statement_token1] = ACTIONS(1119),
    [aux_sym_close_statement_token1] = ACTIONS(1119),
    [aux_sym_inquire_statement_token1] = ACTIONS(1119),
    [aux_sym_enum_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token2] = ACTIONS(1119),
    [aux_sym_file_position_statement_token3] = ACTIONS(1119),
    [aux_sym_file_position_statement_token4] = ACTIONS(1119),
    [aux_sym_logical_expression_token5] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [sym_boolean_literal] = ACTIONS(1117),
    [aux_sym_null_literal_token1] = ACTIONS(1119),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1119),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1117),
    [sym__float_literal] = ACTIONS(1117),
    [sym__boz_literal] = ACTIONS(1117),
    [sym_string_literal] = ACTIONS(1117),
  },
  [319] = {
    [sym_preproc_file_line] = STATE(319),
    [aux_sym_end_program_statement_token1] = ACTIONS(1335),
    [anon_sym_LPAREN] = ACTIONS(1335),
    [aux_sym_interface_statement_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1335),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1335),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1335),
    [aux_sym_contains_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token2] = ACTIONS(1335),
    [aux_sym_implicit_statement_token1] = ACTIONS(1335),
    [aux_sym_implicit_statement_token3] = ACTIONS(1335),
    [aux_sym_implicit_statement_token4] = ACTIONS(1335),
    [aux_sym_save_statement_token1] = ACTIONS(1335),
    [aux_sym_private_statement_token1] = ACTIONS(1335),
    [aux_sym_public_statement_token1] = ACTIONS(1335),
    [aux_sym_namelist_statement_token1] = ACTIONS(1335),
    [aux_sym_common_statement_token1] = ACTIONS(1335),
    [anon_sym_DASH] = ACTIONS(1337),
    [aux_sym_import_statement_token1] = ACTIONS(1335),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1335),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1335),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1335),
    [aux_sym_variable_attributes_token1] = ACTIONS(1335),
    [aux_sym_variable_attributes_token2] = ACTIONS(1335),
    [aux_sym_variable_attributes_token3] = ACTIONS(1335),
    [aux_sym_variable_attributes_token4] = ACTIONS(1335),
    [aux_sym_variable_attributes_token5] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1335),
    [aux_sym_derived_type_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token2] = ACTIONS(1335),
    [aux_sym_type_qualifier_token3] = ACTIONS(1335),
    [aux_sym_type_qualifier_token4] = ACTIONS(1335),
    [aux_sym_type_qualifier_token5] = ACTIONS(1335),
    [aux_sym_type_qualifier_token9] = ACTIONS(1335),
    [aux_sym_type_qualifier_token10] = ACTIONS(1335),
    [aux_sym_type_qualifier_token11] = ACTIONS(1335),
    [aux_sym_type_qualifier_token12] = ACTIONS(1335),
    [aux_sym_type_qualifier_token13] = ACTIONS(1335),
    [aux_sym_type_qualifier_token14] = ACTIONS(1335),
    [aux_sym_type_qualifier_token15] = ACTIONS(1335),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token2] = ACTIONS(1335),
    [aux_sym_subroutine_call_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token2] = ACTIONS(1335),
    [aux_sym_keyword_statement_token3] = ACTIONS(1335),
    [aux_sym_keyword_statement_token4] = ACTIONS(1335),
    [aux_sym_keyword_statement_token6] = ACTIONS(1335),
    [aux_sym_keyword_statement_token7] = ACTIONS(1335),
    [aux_sym_include_statement_token1] = ACTIONS(1335),
    [aux_sym_data_statement_token1] = ACTIONS(1335),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1335),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1335),
    [aux_sym_end_if_statement_token1] = ACTIONS(1335),
    [aux_sym_elseif_clause_token2] = ACTIONS(1335),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1335),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token3] = ACTIONS(1335),
    [aux_sym_select_type_statement_token1] = ACTIONS(1335),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1335),
    [aux_sym_block_construct_token1] = ACTIONS(1335),
    [aux_sym_associate_statement_token1] = ACTIONS(1335),
    [aux_sym_format_statement_token1] = ACTIONS(1335),
    [aux_sym_print_statement_token1] = ACTIONS(1335),
    [aux_sym_open_statement_token1] = ACTIONS(1335),
    [aux_sym_close_statement_token1] = ACTIONS(1335),
    [aux_sym_inquire_statement_token1] = ACTIONS(1335),
    [aux_sym_enum_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token2] = ACTIONS(1335),
    [aux_sym_file_position_statement_token3] = ACTIONS(1335),
    [aux_sym_file_position_statement_token4] = ACTIONS(1335),
    [aux_sym_logical_expression_token5] = ACTIONS(1337),
    [anon_sym_PLUS] = ACTIONS(1337),
    [anon_sym_DOT] = ACTIONS(1335),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1337),
    [anon_sym_LBRACK] = ACTIONS(1337),
    [sym_boolean_literal] = ACTIONS(1337),
    [aux_sym_null_literal_token1] = ACTIONS(1335),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1335),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1337),
    [sym__float_literal] = ACTIONS(1337),
    [sym__boz_literal] = ACTIONS(1337),
    [sym_string_literal] = ACTIONS(1337),
  },
  [320] = {
    [sym_preproc_file_line] = STATE(320),
    [aux_sym_end_program_statement_token1] = ACTIONS(1295),
    [anon_sym_LPAREN] = ACTIONS(1295),
    [aux_sym_interface_statement_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1295),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1295),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1295),
    [aux_sym_contains_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token2] = ACTIONS(1295),
    [aux_sym_implicit_statement_token1] = ACTIONS(1295),
    [aux_sym_implicit_statement_token3] = ACTIONS(1295),
    [aux_sym_implicit_statement_token4] = ACTIONS(1295),
    [aux_sym_save_statement_token1] = ACTIONS(1295),
    [aux_sym_private_statement_token1] = ACTIONS(1295),
    [aux_sym_public_statement_token1] = ACTIONS(1295),
    [aux_sym_namelist_statement_token1] = ACTIONS(1295),
    [aux_sym_common_statement_token1] = ACTIONS(1295),
    [anon_sym_DASH] = ACTIONS(1297),
    [aux_sym_import_statement_token1] = ACTIONS(1295),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1295),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1295),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1295),
    [aux_sym_variable_attributes_token1] = ACTIONS(1295),
    [aux_sym_variable_attributes_token2] = ACTIONS(1295),
    [aux_sym_variable_attributes_token3] = ACTIONS(1295),
    [aux_sym_variable_attributes_token4] = ACTIONS(1295),
    [aux_sym_variable_attributes_token5] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1295),
    [aux_sym_derived_type_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token2] = ACTIONS(1295),
    [aux_sym_type_qualifier_token3] = ACTIONS(1295),
    [aux_sym_type_qualifier_token4] = ACTIONS(1295),
    [aux_sym_type_qualifier_token5] = ACTIONS(1295),
    [aux_sym_type_qualifier_token9] = ACTIONS(1295),
    [aux_sym_type_qualifier_token10] = ACTIONS(1295),
    [aux_sym_type_qualifier_token11] = ACTIONS(1295),
    [aux_sym_type_qualifier_token12] = ACTIONS(1295),
    [aux_sym_type_qualifier_token13] = ACTIONS(1295),
    [aux_sym_type_qualifier_token14] = ACTIONS(1295),
    [aux_sym_type_qualifier_token15] = ACTIONS(1295),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token2] = ACTIONS(1295),
    [aux_sym_subroutine_call_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token2] = ACTIONS(1295),
    [aux_sym_keyword_statement_token3] = ACTIONS(1295),
    [aux_sym_keyword_statement_token4] = ACTIONS(1295),
    [aux_sym_keyword_statement_token6] = ACTIONS(1295),
    [aux_sym_keyword_statement_token7] = ACTIONS(1295),
    [aux_sym_include_statement_token1] = ACTIONS(1295),
    [aux_sym_data_statement_token1] = ACTIONS(1295),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1295),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1295),
    [aux_sym_end_if_statement_token1] = ACTIONS(1295),
    [aux_sym_elseif_clause_token2] = ACTIONS(1295),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1295),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token3] = ACTIONS(1295),
    [aux_sym_select_type_statement_token1] = ACTIONS(1295),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1295),
    [aux_sym_block_construct_token1] = ACTIONS(1295),
    [aux_sym_associate_statement_token1] = ACTIONS(1295),
    [aux_sym_format_statement_token1] = ACTIONS(1295),
    [aux_sym_print_statement_token1] = ACTIONS(1295),
    [aux_sym_open_statement_token1] = ACTIONS(1295),
    [aux_sym_close_statement_token1] = ACTIONS(1295),
    [aux_sym_inquire_statement_token1] = ACTIONS(1295),
    [aux_sym_enum_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token2] = ACTIONS(1295),
    [aux_sym_file_position_statement_token3] = ACTIONS(1295),
    [aux_sym_file_position_statement_token4] = ACTIONS(1295),
    [aux_sym_logical_expression_token5] = ACTIONS(1297),
    [anon_sym_PLUS] = ACTIONS(1297),
    [anon_sym_DOT] = ACTIONS(1295),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1297),
    [anon_sym_LBRACK] = ACTIONS(1297),
    [sym_boolean_literal] = ACTIONS(1297),
    [aux_sym_null_literal_token1] = ACTIONS(1295),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1295),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1297),
    [sym__float_literal] = ACTIONS(1297),
    [sym__boz_literal] = ACTIONS(1297),
    [sym_string_literal] = ACTIONS(1297),
  },
  [321] = {
    [sym_preproc_file_line] = STATE(321),
    [aux_sym_end_program_statement_token1] = ACTIONS(1299),
    [anon_sym_LPAREN] = ACTIONS(1299),
    [aux_sym_interface_statement_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1299),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1299),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1299),
    [aux_sym_contains_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token2] = ACTIONS(1299),
    [aux_sym_implicit_statement_token1] = ACTIONS(1299),
    [aux_sym_implicit_statement_token3] = ACTIONS(1299),
    [aux_sym_implicit_statement_token4] = ACTIONS(1299),
    [aux_sym_save_statement_token1] = ACTIONS(1299),
    [aux_sym_private_statement_token1] = ACTIONS(1299),
    [aux_sym_public_statement_token1] = ACTIONS(1299),
    [aux_sym_namelist_statement_token1] = ACTIONS(1299),
    [aux_sym_common_statement_token1] = ACTIONS(1299),
    [anon_sym_DASH] = ACTIONS(1301),
    [aux_sym_import_statement_token1] = ACTIONS(1299),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1299),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1299),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1299),
    [aux_sym_variable_attributes_token1] = ACTIONS(1299),
    [aux_sym_variable_attributes_token2] = ACTIONS(1299),
    [aux_sym_variable_attributes_token3] = ACTIONS(1299),
    [aux_sym_variable_attributes_token4] = ACTIONS(1299),
    [aux_sym_variable_attributes_token5] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1299),
    [aux_sym_derived_type_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token2] = ACTIONS(1299),
    [aux_sym_type_qualifier_token3] = ACTIONS(1299),
    [aux_sym_type_qualifier_token4] = ACTIONS(1299),
    [aux_sym_type_qualifier_token5] = ACTIONS(1299),
    [aux_sym_type_qualifier_token9] = ACTIONS(1299),
    [aux_sym_type_qualifier_token10] = ACTIONS(1299),
    [aux_sym_type_qualifier_token11] = ACTIONS(1299),
    [aux_sym_type_qualifier_token12] = ACTIONS(1299),
    [aux_sym_type_qualifier_token13] = ACTIONS(1299),
    [aux_sym_type_qualifier_token14] = ACTIONS(1299),
    [aux_sym_type_qualifier_token15] = ACTIONS(1299),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token2] = ACTIONS(1299),
    [aux_sym_subroutine_call_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token2] = ACTIONS(1299),
    [aux_sym_keyword_statement_token3] = ACTIONS(1299),
    [aux_sym_keyword_statement_token4] = ACTIONS(1299),
    [aux_sym_keyword_statement_token6] = ACTIONS(1299),
    [aux_sym_keyword_statement_token7] = ACTIONS(1299),
    [aux_sym_include_statement_token1] = ACTIONS(1299),
    [aux_sym_data_statement_token1] = ACTIONS(1299),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1299),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1299),
    [aux_sym_end_if_statement_token1] = ACTIONS(1299),
    [aux_sym_elseif_clause_token2] = ACTIONS(1299),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1299),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token3] = ACTIONS(1299),
    [aux_sym_select_type_statement_token1] = ACTIONS(1299),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1299),
    [aux_sym_block_construct_token1] = ACTIONS(1299),
    [aux_sym_associate_statement_token1] = ACTIONS(1299),
    [aux_sym_format_statement_token1] = ACTIONS(1299),
    [aux_sym_print_statement_token1] = ACTIONS(1299),
    [aux_sym_open_statement_token1] = ACTIONS(1299),
    [aux_sym_close_statement_token1] = ACTIONS(1299),
    [aux_sym_inquire_statement_token1] = ACTIONS(1299),
    [aux_sym_enum_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token2] = ACTIONS(1299),
    [aux_sym_file_position_statement_token3] = ACTIONS(1299),
    [aux_sym_file_position_statement_token4] = ACTIONS(1299),
    [aux_sym_logical_expression_token5] = ACTIONS(1301),
    [anon_sym_PLUS] = ACTIONS(1301),
    [anon_sym_DOT] = ACTIONS(1299),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1301),
    [anon_sym_LBRACK] = ACTIONS(1301),
    [sym_boolean_literal] = ACTIONS(1301),
    [aux_sym_null_literal_token1] = ACTIONS(1299),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1299),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1301),
    [sym__float_literal] = ACTIONS(1301),
    [sym__boz_literal] = ACTIONS(1301),
    [sym_string_literal] = ACTIONS(1301),
  },
  [322] = {
    [sym_preproc_file_line] = STATE(322),
    [aux_sym_end_program_statement_token1] = ACTIONS(1303),
    [anon_sym_LPAREN] = ACTIONS(1303),
    [aux_sym_interface_statement_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1303),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1303),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1303),
    [aux_sym_contains_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token2] = ACTIONS(1303),
    [aux_sym_implicit_statement_token1] = ACTIONS(1303),
    [aux_sym_implicit_statement_token3] = ACTIONS(1303),
    [aux_sym_implicit_statement_token4] = ACTIONS(1303),
    [aux_sym_save_statement_token1] = ACTIONS(1303),
    [aux_sym_private_statement_token1] = ACTIONS(1303),
    [aux_sym_public_statement_token1] = ACTIONS(1303),
    [aux_sym_namelist_statement_token1] = ACTIONS(1303),
    [aux_sym_common_statement_token1] = ACTIONS(1303),
    [anon_sym_DASH] = ACTIONS(1305),
    [aux_sym_import_statement_token1] = ACTIONS(1303),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1303),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1303),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1303),
    [aux_sym_variable_attributes_token1] = ACTIONS(1303),
    [aux_sym_variable_attributes_token2] = ACTIONS(1303),
    [aux_sym_variable_attributes_token3] = ACTIONS(1303),
    [aux_sym_variable_attributes_token4] = ACTIONS(1303),
    [aux_sym_variable_attributes_token5] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1303),
    [aux_sym_derived_type_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token2] = ACTIONS(1303),
    [aux_sym_type_qualifier_token3] = ACTIONS(1303),
    [aux_sym_type_qualifier_token4] = ACTIONS(1303),
    [aux_sym_type_qualifier_token5] = ACTIONS(1303),
    [aux_sym_type_qualifier_token9] = ACTIONS(1303),
    [aux_sym_type_qualifier_token10] = ACTIONS(1303),
    [aux_sym_type_qualifier_token11] = ACTIONS(1303),
    [aux_sym_type_qualifier_token12] = ACTIONS(1303),
    [aux_sym_type_qualifier_token13] = ACTIONS(1303),
    [aux_sym_type_qualifier_token14] = ACTIONS(1303),
    [aux_sym_type_qualifier_token15] = ACTIONS(1303),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token2] = ACTIONS(1303),
    [aux_sym_subroutine_call_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token2] = ACTIONS(1303),
    [aux_sym_keyword_statement_token3] = ACTIONS(1303),
    [aux_sym_keyword_statement_token4] = ACTIONS(1303),
    [aux_sym_keyword_statement_token6] = ACTIONS(1303),
    [aux_sym_keyword_statement_token7] = ACTIONS(1303),
    [aux_sym_include_statement_token1] = ACTIONS(1303),
    [aux_sym_data_statement_token1] = ACTIONS(1303),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1303),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1303),
    [aux_sym_end_if_statement_token1] = ACTIONS(1303),
    [aux_sym_elseif_clause_token2] = ACTIONS(1303),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1303),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token3] = ACTIONS(1303),
    [aux_sym_select_type_statement_token1] = ACTIONS(1303),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1303),
    [aux_sym_block_construct_token1] = ACTIONS(1303),
    [aux_sym_associate_statement_token1] = ACTIONS(1303),
    [aux_sym_format_statement_token1] = ACTIONS(1303),
    [aux_sym_print_statement_token1] = ACTIONS(1303),
    [aux_sym_open_statement_token1] = ACTIONS(1303),
    [aux_sym_close_statement_token1] = ACTIONS(1303),
    [aux_sym_inquire_statement_token1] = ACTIONS(1303),
    [aux_sym_enum_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token2] = ACTIONS(1303),
    [aux_sym_file_position_statement_token3] = ACTIONS(1303),
    [aux_sym_file_position_statement_token4] = ACTIONS(1303),
    [aux_sym_logical_expression_token5] = ACTIONS(1305),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_DOT] = ACTIONS(1303),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1305),
    [anon_sym_LBRACK] = ACTIONS(1305),
    [sym_boolean_literal] = ACTIONS(1305),
    [aux_sym_null_literal_token1] = ACTIONS(1303),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1303),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1305),
    [sym__float_literal] = ACTIONS(1305),
    [sym__boz_literal] = ACTIONS(1305),
    [sym_string_literal] = ACTIONS(1305),
  },
  [323] = {
    [sym_preproc_file_line] = STATE(323),
    [aux_sym_end_program_statement_token1] = ACTIONS(1275),
    [anon_sym_LPAREN] = ACTIONS(1275),
    [aux_sym_interface_statement_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1275),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1275),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1275),
    [aux_sym_contains_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token2] = ACTIONS(1275),
    [aux_sym_implicit_statement_token1] = ACTIONS(1275),
    [aux_sym_implicit_statement_token3] = ACTIONS(1275),
    [aux_sym_implicit_statement_token4] = ACTIONS(1275),
    [aux_sym_save_statement_token1] = ACTIONS(1275),
    [aux_sym_private_statement_token1] = ACTIONS(1275),
    [aux_sym_public_statement_token1] = ACTIONS(1275),
    [aux_sym_namelist_statement_token1] = ACTIONS(1275),
    [aux_sym_common_statement_token1] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1277),
    [aux_sym_import_statement_token1] = ACTIONS(1275),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1275),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1275),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1275),
    [aux_sym_variable_attributes_token1] = ACTIONS(1275),
    [aux_sym_variable_attributes_token2] = ACTIONS(1275),
    [aux_sym_variable_attributes_token3] = ACTIONS(1275),
    [aux_sym_variable_attributes_token4] = ACTIONS(1275),
    [aux_sym_variable_attributes_token5] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1275),
    [aux_sym_derived_type_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token2] = ACTIONS(1275),
    [aux_sym_type_qualifier_token3] = ACTIONS(1275),
    [aux_sym_type_qualifier_token4] = ACTIONS(1275),
    [aux_sym_type_qualifier_token5] = ACTIONS(1275),
    [aux_sym_type_qualifier_token9] = ACTIONS(1275),
    [aux_sym_type_qualifier_token10] = ACTIONS(1275),
    [aux_sym_type_qualifier_token11] = ACTIONS(1275),
    [aux_sym_type_qualifier_token12] = ACTIONS(1275),
    [aux_sym_type_qualifier_token13] = ACTIONS(1275),
    [aux_sym_type_qualifier_token14] = ACTIONS(1275),
    [aux_sym_type_qualifier_token15] = ACTIONS(1275),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token2] = ACTIONS(1275),
    [aux_sym_subroutine_call_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token2] = ACTIONS(1275),
    [aux_sym_keyword_statement_token3] = ACTIONS(1275),
    [aux_sym_keyword_statement_token4] = ACTIONS(1275),
    [aux_sym_keyword_statement_token6] = ACTIONS(1275),
    [aux_sym_keyword_statement_token7] = ACTIONS(1275),
    [aux_sym_include_statement_token1] = ACTIONS(1275),
    [aux_sym_data_statement_token1] = ACTIONS(1275),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1275),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1275),
    [aux_sym_end_if_statement_token1] = ACTIONS(1275),
    [aux_sym_elseif_clause_token2] = ACTIONS(1275),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1275),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token3] = ACTIONS(1275),
    [aux_sym_select_type_statement_token1] = ACTIONS(1275),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1275),
    [aux_sym_block_construct_token1] = ACTIONS(1275),
    [aux_sym_associate_statement_token1] = ACTIONS(1275),
    [aux_sym_format_statement_token1] = ACTIONS(1275),
    [aux_sym_print_statement_token1] = ACTIONS(1275),
    [aux_sym_open_statement_token1] = ACTIONS(1275),
    [aux_sym_close_statement_token1] = ACTIONS(1275),
    [aux_sym_inquire_statement_token1] = ACTIONS(1275),
    [aux_sym_enum_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token2] = ACTIONS(1275),
    [aux_sym_file_position_statement_token3] = ACTIONS(1275),
    [aux_sym_file_position_statement_token4] = ACTIONS(1275),
    [aux_sym_logical_expression_token5] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1277),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [sym_boolean_literal] = ACTIONS(1277),
    [aux_sym_null_literal_token1] = ACTIONS(1275),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1275),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1277),
    [sym__float_literal] = ACTIONS(1277),
    [sym__boz_literal] = ACTIONS(1277),
    [sym_string_literal] = ACTIONS(1277),
  },
  [324] = {
    [sym_preproc_file_line] = STATE(324),
    [aux_sym_end_program_statement_token1] = ACTIONS(1339),
    [anon_sym_LPAREN] = ACTIONS(1339),
    [aux_sym_interface_statement_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1339),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1339),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1339),
    [aux_sym_contains_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token2] = ACTIONS(1339),
    [aux_sym_implicit_statement_token1] = ACTIONS(1339),
    [aux_sym_implicit_statement_token3] = ACTIONS(1339),
    [aux_sym_implicit_statement_token4] = ACTIONS(1339),
    [aux_sym_save_statement_token1] = ACTIONS(1339),
    [aux_sym_private_statement_token1] = ACTIONS(1339),
    [aux_sym_public_statement_token1] = ACTIONS(1339),
    [aux_sym_namelist_statement_token1] = ACTIONS(1339),
    [aux_sym_common_statement_token1] = ACTIONS(1339),
    [anon_sym_DASH] = ACTIONS(1341),
    [aux_sym_import_statement_token1] = ACTIONS(1339),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1339),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1339),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1339),
    [aux_sym_variable_attributes_token1] = ACTIONS(1339),
    [aux_sym_variable_attributes_token2] = ACTIONS(1339),
    [aux_sym_variable_attributes_token3] = ACTIONS(1339),
    [aux_sym_variable_attributes_token4] = ACTIONS(1339),
    [aux_sym_variable_attributes_token5] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1339),
    [aux_sym_derived_type_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token2] = ACTIONS(1339),
    [aux_sym_type_qualifier_token3] = ACTIONS(1339),
    [aux_sym_type_qualifier_token4] = ACTIONS(1339),
    [aux_sym_type_qualifier_token5] = ACTIONS(1339),
    [aux_sym_type_qualifier_token9] = ACTIONS(1339),
    [aux_sym_type_qualifier_token10] = ACTIONS(1339),
    [aux_sym_type_qualifier_token11] = ACTIONS(1339),
    [aux_sym_type_qualifier_token12] = ACTIONS(1339),
    [aux_sym_type_qualifier_token13] = ACTIONS(1339),
    [aux_sym_type_qualifier_token14] = ACTIONS(1339),
    [aux_sym_type_qualifier_token15] = ACTIONS(1339),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token2] = ACTIONS(1339),
    [aux_sym_subroutine_call_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token2] = ACTIONS(1339),
    [aux_sym_keyword_statement_token3] = ACTIONS(1339),
    [aux_sym_keyword_statement_token4] = ACTIONS(1339),
    [aux_sym_keyword_statement_token6] = ACTIONS(1339),
    [aux_sym_keyword_statement_token7] = ACTIONS(1339),
    [aux_sym_include_statement_token1] = ACTIONS(1339),
    [aux_sym_data_statement_token1] = ACTIONS(1339),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1339),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1339),
    [aux_sym_end_if_statement_token1] = ACTIONS(1339),
    [aux_sym_elseif_clause_token2] = ACTIONS(1339),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1339),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token3] = ACTIONS(1339),
    [aux_sym_select_type_statement_token1] = ACTIONS(1339),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1339),
    [aux_sym_block_construct_token1] = ACTIONS(1339),
    [aux_sym_associate_statement_token1] = ACTIONS(1339),
    [aux_sym_format_statement_token1] = ACTIONS(1339),
    [aux_sym_print_statement_token1] = ACTIONS(1339),
    [aux_sym_open_statement_token1] = ACTIONS(1339),
    [aux_sym_close_statement_token1] = ACTIONS(1339),
    [aux_sym_inquire_statement_token1] = ACTIONS(1339),
    [aux_sym_enum_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token2] = ACTIONS(1339),
    [aux_sym_file_position_statement_token3] = ACTIONS(1339),
    [aux_sym_file_position_statement_token4] = ACTIONS(1339),
    [aux_sym_logical_expression_token5] = ACTIONS(1341),
    [anon_sym_PLUS] = ACTIONS(1341),
    [anon_sym_DOT] = ACTIONS(1339),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1341),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [sym_boolean_literal] = ACTIONS(1341),
    [aux_sym_null_literal_token1] = ACTIONS(1339),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1339),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1341),
    [sym__float_literal] = ACTIONS(1341),
    [sym__boz_literal] = ACTIONS(1341),
    [sym_string_literal] = ACTIONS(1341),
  },
  [325] = {
    [sym_preproc_file_line] = STATE(325),
    [aux_sym_end_program_statement_token1] = ACTIONS(1343),
    [anon_sym_LPAREN] = ACTIONS(1343),
    [aux_sym_interface_statement_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1343),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1343),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1343),
    [aux_sym_contains_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token2] = ACTIONS(1343),
    [aux_sym_implicit_statement_token1] = ACTIONS(1343),
    [aux_sym_implicit_statement_token3] = ACTIONS(1343),
    [aux_sym_implicit_statement_token4] = ACTIONS(1343),
    [aux_sym_save_statement_token1] = ACTIONS(1343),
    [aux_sym_private_statement_token1] = ACTIONS(1343),
    [aux_sym_public_statement_token1] = ACTIONS(1343),
    [aux_sym_namelist_statement_token1] = ACTIONS(1343),
    [aux_sym_common_statement_token1] = ACTIONS(1343),
    [anon_sym_DASH] = ACTIONS(1345),
    [aux_sym_import_statement_token1] = ACTIONS(1343),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1343),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1343),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1343),
    [aux_sym_variable_attributes_token1] = ACTIONS(1343),
    [aux_sym_variable_attributes_token2] = ACTIONS(1343),
    [aux_sym_variable_attributes_token3] = ACTIONS(1343),
    [aux_sym_variable_attributes_token4] = ACTIONS(1343),
    [aux_sym_variable_attributes_token5] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1343),
    [aux_sym_derived_type_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token2] = ACTIONS(1343),
    [aux_sym_type_qualifier_token3] = ACTIONS(1343),
    [aux_sym_type_qualifier_token4] = ACTIONS(1343),
    [aux_sym_type_qualifier_token5] = ACTIONS(1343),
    [aux_sym_type_qualifier_token9] = ACTIONS(1343),
    [aux_sym_type_qualifier_token10] = ACTIONS(1343),
    [aux_sym_type_qualifier_token11] = ACTIONS(1343),
    [aux_sym_type_qualifier_token12] = ACTIONS(1343),
    [aux_sym_type_qualifier_token13] = ACTIONS(1343),
    [aux_sym_type_qualifier_token14] = ACTIONS(1343),
    [aux_sym_type_qualifier_token15] = ACTIONS(1343),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token2] = ACTIONS(1343),
    [aux_sym_subroutine_call_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token2] = ACTIONS(1343),
    [aux_sym_keyword_statement_token3] = ACTIONS(1343),
    [aux_sym_keyword_statement_token4] = ACTIONS(1343),
    [aux_sym_keyword_statement_token6] = ACTIONS(1343),
    [aux_sym_keyword_statement_token7] = ACTIONS(1343),
    [aux_sym_include_statement_token1] = ACTIONS(1343),
    [aux_sym_data_statement_token1] = ACTIONS(1343),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1343),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1343),
    [aux_sym_end_if_statement_token1] = ACTIONS(1343),
    [aux_sym_elseif_clause_token2] = ACTIONS(1343),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1343),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token3] = ACTIONS(1343),
    [aux_sym_select_type_statement_token1] = ACTIONS(1343),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1343),
    [aux_sym_block_construct_token1] = ACTIONS(1343),
    [aux_sym_associate_statement_token1] = ACTIONS(1343),
    [aux_sym_format_statement_token1] = ACTIONS(1343),
    [aux_sym_print_statement_token1] = ACTIONS(1343),
    [aux_sym_open_statement_token1] = ACTIONS(1343),
    [aux_sym_close_statement_token1] = ACTIONS(1343),
    [aux_sym_inquire_statement_token1] = ACTIONS(1343),
    [aux_sym_enum_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token2] = ACTIONS(1343),
    [aux_sym_file_position_statement_token3] = ACTIONS(1343),
    [aux_sym_file_position_statement_token4] = ACTIONS(1343),
    [aux_sym_logical_expression_token5] = ACTIONS(1345),
    [anon_sym_PLUS] = ACTIONS(1345),
    [anon_sym_DOT] = ACTIONS(1343),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1345),
    [anon_sym_LBRACK] = ACTIONS(1345),
    [sym_boolean_literal] = ACTIONS(1345),
    [aux_sym_null_literal_token1] = ACTIONS(1343),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1343),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1345),
    [sym__float_literal] = ACTIONS(1345),
    [sym__boz_literal] = ACTIONS(1345),
    [sym_string_literal] = ACTIONS(1345),
  },
  [326] = {
    [sym_preproc_file_line] = STATE(326),
    [aux_sym_end_program_statement_token1] = ACTIONS(1275),
    [aux_sym_end_program_statement_token2] = ACTIONS(1275),
    [anon_sym_LPAREN] = ACTIONS(1275),
    [aux_sym_interface_statement_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1275),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1275),
    [aux_sym_contains_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token2] = ACTIONS(1275),
    [aux_sym_implicit_statement_token1] = ACTIONS(1275),
    [aux_sym_implicit_statement_token3] = ACTIONS(1275),
    [aux_sym_implicit_statement_token4] = ACTIONS(1275),
    [aux_sym_save_statement_token1] = ACTIONS(1275),
    [aux_sym_private_statement_token1] = ACTIONS(1275),
    [aux_sym_public_statement_token1] = ACTIONS(1275),
    [aux_sym_namelist_statement_token1] = ACTIONS(1275),
    [aux_sym_common_statement_token1] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1277),
    [aux_sym_import_statement_token1] = ACTIONS(1275),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1275),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1275),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1275),
    [aux_sym_variable_attributes_token1] = ACTIONS(1275),
    [aux_sym_variable_attributes_token2] = ACTIONS(1275),
    [aux_sym_variable_attributes_token3] = ACTIONS(1275),
    [aux_sym_variable_attributes_token4] = ACTIONS(1275),
    [aux_sym_variable_attributes_token5] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1275),
    [aux_sym_derived_type_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token2] = ACTIONS(1275),
    [aux_sym_type_qualifier_token3] = ACTIONS(1275),
    [aux_sym_type_qualifier_token4] = ACTIONS(1275),
    [aux_sym_type_qualifier_token5] = ACTIONS(1275),
    [aux_sym_type_qualifier_token9] = ACTIONS(1275),
    [aux_sym_type_qualifier_token10] = ACTIONS(1275),
    [aux_sym_type_qualifier_token11] = ACTIONS(1275),
    [aux_sym_type_qualifier_token12] = ACTIONS(1275),
    [aux_sym_type_qualifier_token13] = ACTIONS(1275),
    [aux_sym_type_qualifier_token14] = ACTIONS(1275),
    [aux_sym_type_qualifier_token15] = ACTIONS(1275),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token2] = ACTIONS(1275),
    [aux_sym_subroutine_call_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token2] = ACTIONS(1275),
    [aux_sym_keyword_statement_token3] = ACTIONS(1275),
    [aux_sym_keyword_statement_token4] = ACTIONS(1275),
    [aux_sym_keyword_statement_token6] = ACTIONS(1275),
    [aux_sym_keyword_statement_token7] = ACTIONS(1275),
    [aux_sym_include_statement_token1] = ACTIONS(1275),
    [aux_sym_data_statement_token1] = ACTIONS(1275),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1275),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1275),
    [aux_sym_end_if_statement_token1] = ACTIONS(1275),
    [aux_sym_elseif_clause_token2] = ACTIONS(1275),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1275),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token3] = ACTIONS(1275),
    [aux_sym_select_type_statement_token1] = ACTIONS(1275),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1275),
    [aux_sym_block_construct_token1] = ACTIONS(1275),
    [aux_sym_associate_statement_token1] = ACTIONS(1275),
    [aux_sym_format_statement_token1] = ACTIONS(1275),
    [aux_sym_print_statement_token1] = ACTIONS(1275),
    [aux_sym_open_statement_token1] = ACTIONS(1275),
    [aux_sym_close_statement_token1] = ACTIONS(1275),
    [aux_sym_inquire_statement_token1] = ACTIONS(1275),
    [aux_sym_enum_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token2] = ACTIONS(1275),
    [aux_sym_file_position_statement_token3] = ACTIONS(1275),
    [aux_sym_file_position_statement_token4] = ACTIONS(1275),
    [aux_sym_logical_expression_token5] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1277),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [sym_boolean_literal] = ACTIONS(1277),
    [aux_sym_null_literal_token1] = ACTIONS(1275),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1275),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1277),
    [sym__float_literal] = ACTIONS(1277),
    [sym__boz_literal] = ACTIONS(1277),
    [sym_string_literal] = ACTIONS(1277),
  },
  [327] = {
    [sym_preproc_file_line] = STATE(327),
    [aux_sym_end_program_statement_token1] = ACTIONS(1335),
    [anon_sym_LPAREN] = ACTIONS(1335),
    [aux_sym_interface_statement_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1335),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1335),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1335),
    [aux_sym_contains_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token2] = ACTIONS(1335),
    [aux_sym_implicit_statement_token1] = ACTIONS(1335),
    [aux_sym_implicit_statement_token3] = ACTIONS(1335),
    [aux_sym_implicit_statement_token4] = ACTIONS(1335),
    [aux_sym_save_statement_token1] = ACTIONS(1335),
    [aux_sym_private_statement_token1] = ACTIONS(1335),
    [aux_sym_public_statement_token1] = ACTIONS(1335),
    [aux_sym_namelist_statement_token1] = ACTIONS(1335),
    [aux_sym_common_statement_token1] = ACTIONS(1335),
    [anon_sym_DASH] = ACTIONS(1337),
    [aux_sym_import_statement_token1] = ACTIONS(1335),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1335),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1335),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1335),
    [aux_sym_variable_attributes_token1] = ACTIONS(1335),
    [aux_sym_variable_attributes_token2] = ACTIONS(1335),
    [aux_sym_variable_attributes_token3] = ACTIONS(1335),
    [aux_sym_variable_attributes_token4] = ACTIONS(1335),
    [aux_sym_variable_attributes_token5] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1335),
    [aux_sym_derived_type_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token2] = ACTIONS(1335),
    [aux_sym_type_qualifier_token3] = ACTIONS(1335),
    [aux_sym_type_qualifier_token4] = ACTIONS(1335),
    [aux_sym_type_qualifier_token5] = ACTIONS(1335),
    [aux_sym_type_qualifier_token9] = ACTIONS(1335),
    [aux_sym_type_qualifier_token10] = ACTIONS(1335),
    [aux_sym_type_qualifier_token11] = ACTIONS(1335),
    [aux_sym_type_qualifier_token12] = ACTIONS(1335),
    [aux_sym_type_qualifier_token13] = ACTIONS(1335),
    [aux_sym_type_qualifier_token14] = ACTIONS(1335),
    [aux_sym_type_qualifier_token15] = ACTIONS(1335),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token2] = ACTIONS(1335),
    [aux_sym_subroutine_call_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token2] = ACTIONS(1335),
    [aux_sym_keyword_statement_token3] = ACTIONS(1335),
    [aux_sym_keyword_statement_token4] = ACTIONS(1335),
    [aux_sym_keyword_statement_token6] = ACTIONS(1335),
    [aux_sym_keyword_statement_token7] = ACTIONS(1335),
    [aux_sym_include_statement_token1] = ACTIONS(1335),
    [aux_sym_data_statement_token1] = ACTIONS(1335),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1335),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1335),
    [aux_sym_end_if_statement_token1] = ACTIONS(1335),
    [aux_sym_elseif_clause_token2] = ACTIONS(1335),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1335),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token3] = ACTIONS(1335),
    [aux_sym_select_type_statement_token1] = ACTIONS(1335),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1335),
    [aux_sym_block_construct_token1] = ACTIONS(1335),
    [aux_sym_associate_statement_token1] = ACTIONS(1335),
    [aux_sym_format_statement_token1] = ACTIONS(1335),
    [aux_sym_print_statement_token1] = ACTIONS(1335),
    [aux_sym_open_statement_token1] = ACTIONS(1335),
    [aux_sym_close_statement_token1] = ACTIONS(1335),
    [aux_sym_inquire_statement_token1] = ACTIONS(1335),
    [aux_sym_enum_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token2] = ACTIONS(1335),
    [aux_sym_file_position_statement_token3] = ACTIONS(1335),
    [aux_sym_file_position_statement_token4] = ACTIONS(1335),
    [aux_sym_logical_expression_token5] = ACTIONS(1337),
    [anon_sym_PLUS] = ACTIONS(1337),
    [anon_sym_DOT] = ACTIONS(1335),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1337),
    [anon_sym_LBRACK] = ACTIONS(1337),
    [sym_boolean_literal] = ACTIONS(1337),
    [aux_sym_null_literal_token1] = ACTIONS(1335),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1335),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1337),
    [sym__float_literal] = ACTIONS(1337),
    [sym__boz_literal] = ACTIONS(1337),
    [sym_string_literal] = ACTIONS(1337),
  },
  [328] = {
    [sym_preproc_file_line] = STATE(328),
    [aux_sym_end_program_statement_token1] = ACTIONS(1347),
    [aux_sym_end_program_statement_token2] = ACTIONS(1347),
    [anon_sym_LPAREN] = ACTIONS(1347),
    [aux_sym_interface_statement_token1] = ACTIONS(1347),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1347),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1347),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1347),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1347),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1347),
    [aux_sym_contains_statement_token1] = ACTIONS(1347),
    [aux_sym_use_statement_token1] = ACTIONS(1347),
    [aux_sym_use_statement_token2] = ACTIONS(1347),
    [aux_sym_implicit_statement_token1] = ACTIONS(1347),
    [aux_sym_implicit_statement_token3] = ACTIONS(1347),
    [aux_sym_implicit_statement_token4] = ACTIONS(1347),
    [aux_sym_save_statement_token1] = ACTIONS(1347),
    [aux_sym_private_statement_token1] = ACTIONS(1347),
    [aux_sym_public_statement_token1] = ACTIONS(1347),
    [aux_sym_namelist_statement_token1] = ACTIONS(1347),
    [aux_sym_common_statement_token1] = ACTIONS(1347),
    [anon_sym_DASH] = ACTIONS(1349),
    [aux_sym_import_statement_token1] = ACTIONS(1347),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1347),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1347),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1347),
    [aux_sym_variable_attributes_token1] = ACTIONS(1347),
    [aux_sym_variable_attributes_token2] = ACTIONS(1347),
    [aux_sym_variable_attributes_token3] = ACTIONS(1347),
    [aux_sym_variable_attributes_token4] = ACTIONS(1347),
    [aux_sym_variable_attributes_token5] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1347),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1347),
    [aux_sym_derived_type_token1] = ACTIONS(1347),
    [aux_sym_type_qualifier_token1] = ACTIONS(1347),
    [aux_sym_type_qualifier_token2] = ACTIONS(1347),
    [aux_sym_type_qualifier_token3] = ACTIONS(1347),
    [aux_sym_type_qualifier_token4] = ACTIONS(1347),
    [aux_sym_type_qualifier_token5] = ACTIONS(1347),
    [aux_sym_type_qualifier_token9] = ACTIONS(1347),
    [aux_sym_type_qualifier_token10] = ACTIONS(1347),
    [aux_sym_type_qualifier_token11] = ACTIONS(1347),
    [aux_sym_type_qualifier_token12] = ACTIONS(1347),
    [aux_sym_type_qualifier_token13] = ACTIONS(1347),
    [aux_sym_type_qualifier_token14] = ACTIONS(1347),
    [aux_sym_type_qualifier_token15] = ACTIONS(1347),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1347),
    [aux_sym_stop_statement_token1] = ACTIONS(1347),
    [aux_sym_stop_statement_token2] = ACTIONS(1347),
    [aux_sym_subroutine_call_token1] = ACTIONS(1347),
    [aux_sym_keyword_statement_token1] = ACTIONS(1347),
    [aux_sym_keyword_statement_token2] = ACTIONS(1347),
    [aux_sym_keyword_statement_token3] = ACTIONS(1347),
    [aux_sym_keyword_statement_token4] = ACTIONS(1347),
    [aux_sym_keyword_statement_token6] = ACTIONS(1347),
    [aux_sym_keyword_statement_token7] = ACTIONS(1347),
    [aux_sym_include_statement_token1] = ACTIONS(1347),
    [aux_sym_data_statement_token1] = ACTIONS(1347),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1347),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1347),
    [aux_sym_end_if_statement_token1] = ACTIONS(1347),
    [aux_sym_elseif_clause_token2] = ACTIONS(1347),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1347),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1347),
    [aux_sym_select_case_statement_token1] = ACTIONS(1347),
    [aux_sym_select_case_statement_token3] = ACTIONS(1347),
    [aux_sym_select_type_statement_token1] = ACTIONS(1347),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1347),
    [aux_sym_block_construct_token1] = ACTIONS(1347),
    [aux_sym_associate_statement_token1] = ACTIONS(1347),
    [aux_sym_format_statement_token1] = ACTIONS(1347),
    [aux_sym_print_statement_token1] = ACTIONS(1347),
    [aux_sym_open_statement_token1] = ACTIONS(1347),
    [aux_sym_close_statement_token1] = ACTIONS(1347),
    [aux_sym_inquire_statement_token1] = ACTIONS(1347),
    [aux_sym_enum_statement_token1] = ACTIONS(1347),
    [aux_sym_file_position_statement_token1] = ACTIONS(1347),
    [aux_sym_file_position_statement_token2] = ACTIONS(1347),
    [aux_sym_file_position_statement_token3] = ACTIONS(1347),
    [aux_sym_file_position_statement_token4] = ACTIONS(1347),
    [aux_sym_logical_expression_token5] = ACTIONS(1349),
    [anon_sym_PLUS] = ACTIONS(1349),
    [anon_sym_DOT] = ACTIONS(1347),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1349),
    [anon_sym_LBRACK] = ACTIONS(1349),
    [sym_boolean_literal] = ACTIONS(1349),
    [aux_sym_null_literal_token1] = ACTIONS(1347),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1347),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1349),
    [sym__float_literal] = ACTIONS(1349),
    [sym__boz_literal] = ACTIONS(1349),
    [sym_string_literal] = ACTIONS(1349),
  },
  [329] = {
    [sym_preproc_file_line] = STATE(329),
    [aux_sym_end_program_statement_token1] = ACTIONS(1315),
    [anon_sym_LPAREN] = ACTIONS(1315),
    [aux_sym_interface_statement_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1315),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1315),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1315),
    [aux_sym_contains_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token2] = ACTIONS(1315),
    [aux_sym_implicit_statement_token1] = ACTIONS(1315),
    [aux_sym_implicit_statement_token3] = ACTIONS(1315),
    [aux_sym_implicit_statement_token4] = ACTIONS(1315),
    [aux_sym_save_statement_token1] = ACTIONS(1315),
    [aux_sym_private_statement_token1] = ACTIONS(1315),
    [aux_sym_public_statement_token1] = ACTIONS(1315),
    [aux_sym_namelist_statement_token1] = ACTIONS(1315),
    [aux_sym_common_statement_token1] = ACTIONS(1315),
    [anon_sym_DASH] = ACTIONS(1317),
    [aux_sym_import_statement_token1] = ACTIONS(1315),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1315),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1315),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1315),
    [aux_sym_variable_attributes_token1] = ACTIONS(1315),
    [aux_sym_variable_attributes_token2] = ACTIONS(1315),
    [aux_sym_variable_attributes_token3] = ACTIONS(1315),
    [aux_sym_variable_attributes_token4] = ACTIONS(1315),
    [aux_sym_variable_attributes_token5] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1315),
    [aux_sym_derived_type_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token2] = ACTIONS(1315),
    [aux_sym_type_qualifier_token3] = ACTIONS(1315),
    [aux_sym_type_qualifier_token4] = ACTIONS(1315),
    [aux_sym_type_qualifier_token5] = ACTIONS(1315),
    [aux_sym_type_qualifier_token9] = ACTIONS(1315),
    [aux_sym_type_qualifier_token10] = ACTIONS(1315),
    [aux_sym_type_qualifier_token11] = ACTIONS(1315),
    [aux_sym_type_qualifier_token12] = ACTIONS(1315),
    [aux_sym_type_qualifier_token13] = ACTIONS(1315),
    [aux_sym_type_qualifier_token14] = ACTIONS(1315),
    [aux_sym_type_qualifier_token15] = ACTIONS(1315),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token2] = ACTIONS(1315),
    [aux_sym_subroutine_call_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token2] = ACTIONS(1315),
    [aux_sym_keyword_statement_token3] = ACTIONS(1315),
    [aux_sym_keyword_statement_token4] = ACTIONS(1315),
    [aux_sym_keyword_statement_token6] = ACTIONS(1315),
    [aux_sym_keyword_statement_token7] = ACTIONS(1315),
    [aux_sym_include_statement_token1] = ACTIONS(1315),
    [aux_sym_data_statement_token1] = ACTIONS(1315),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1315),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1315),
    [aux_sym_end_if_statement_token1] = ACTIONS(1315),
    [aux_sym_elseif_clause_token2] = ACTIONS(1315),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1315),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token3] = ACTIONS(1315),
    [aux_sym_select_type_statement_token1] = ACTIONS(1315),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1315),
    [aux_sym_block_construct_token1] = ACTIONS(1315),
    [aux_sym_associate_statement_token1] = ACTIONS(1315),
    [aux_sym_format_statement_token1] = ACTIONS(1315),
    [aux_sym_print_statement_token1] = ACTIONS(1315),
    [aux_sym_open_statement_token1] = ACTIONS(1315),
    [aux_sym_close_statement_token1] = ACTIONS(1315),
    [aux_sym_inquire_statement_token1] = ACTIONS(1315),
    [aux_sym_enum_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token2] = ACTIONS(1315),
    [aux_sym_file_position_statement_token3] = ACTIONS(1315),
    [aux_sym_file_position_statement_token4] = ACTIONS(1315),
    [aux_sym_logical_expression_token5] = ACTIONS(1317),
    [anon_sym_PLUS] = ACTIONS(1317),
    [anon_sym_DOT] = ACTIONS(1315),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1317),
    [anon_sym_LBRACK] = ACTIONS(1317),
    [sym_boolean_literal] = ACTIONS(1317),
    [aux_sym_null_literal_token1] = ACTIONS(1315),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1315),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1317),
    [sym__float_literal] = ACTIONS(1317),
    [sym__boz_literal] = ACTIONS(1317),
    [sym_string_literal] = ACTIONS(1317),
  },
  [330] = {
    [sym_preproc_file_line] = STATE(330),
    [aux_sym_end_program_statement_token1] = ACTIONS(1339),
    [anon_sym_LPAREN] = ACTIONS(1339),
    [aux_sym_interface_statement_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1339),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1339),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1339),
    [aux_sym_contains_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token2] = ACTIONS(1339),
    [aux_sym_implicit_statement_token1] = ACTIONS(1339),
    [aux_sym_implicit_statement_token3] = ACTIONS(1339),
    [aux_sym_implicit_statement_token4] = ACTIONS(1339),
    [aux_sym_save_statement_token1] = ACTIONS(1339),
    [aux_sym_private_statement_token1] = ACTIONS(1339),
    [aux_sym_public_statement_token1] = ACTIONS(1339),
    [aux_sym_namelist_statement_token1] = ACTIONS(1339),
    [aux_sym_common_statement_token1] = ACTIONS(1339),
    [anon_sym_DASH] = ACTIONS(1341),
    [aux_sym_import_statement_token1] = ACTIONS(1339),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1339),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1339),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1339),
    [aux_sym_variable_attributes_token1] = ACTIONS(1339),
    [aux_sym_variable_attributes_token2] = ACTIONS(1339),
    [aux_sym_variable_attributes_token3] = ACTIONS(1339),
    [aux_sym_variable_attributes_token4] = ACTIONS(1339),
    [aux_sym_variable_attributes_token5] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1339),
    [aux_sym_derived_type_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token2] = ACTIONS(1339),
    [aux_sym_type_qualifier_token3] = ACTIONS(1339),
    [aux_sym_type_qualifier_token4] = ACTIONS(1339),
    [aux_sym_type_qualifier_token5] = ACTIONS(1339),
    [aux_sym_type_qualifier_token9] = ACTIONS(1339),
    [aux_sym_type_qualifier_token10] = ACTIONS(1339),
    [aux_sym_type_qualifier_token11] = ACTIONS(1339),
    [aux_sym_type_qualifier_token12] = ACTIONS(1339),
    [aux_sym_type_qualifier_token13] = ACTIONS(1339),
    [aux_sym_type_qualifier_token14] = ACTIONS(1339),
    [aux_sym_type_qualifier_token15] = ACTIONS(1339),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token2] = ACTIONS(1339),
    [aux_sym_subroutine_call_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token2] = ACTIONS(1339),
    [aux_sym_keyword_statement_token3] = ACTIONS(1339),
    [aux_sym_keyword_statement_token4] = ACTIONS(1339),
    [aux_sym_keyword_statement_token6] = ACTIONS(1339),
    [aux_sym_keyword_statement_token7] = ACTIONS(1339),
    [aux_sym_include_statement_token1] = ACTIONS(1339),
    [aux_sym_data_statement_token1] = ACTIONS(1339),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1339),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1339),
    [aux_sym_end_if_statement_token1] = ACTIONS(1339),
    [aux_sym_elseif_clause_token2] = ACTIONS(1339),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1339),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token3] = ACTIONS(1339),
    [aux_sym_select_type_statement_token1] = ACTIONS(1339),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1339),
    [aux_sym_block_construct_token1] = ACTIONS(1339),
    [aux_sym_associate_statement_token1] = ACTIONS(1339),
    [aux_sym_format_statement_token1] = ACTIONS(1339),
    [aux_sym_print_statement_token1] = ACTIONS(1339),
    [aux_sym_open_statement_token1] = ACTIONS(1339),
    [aux_sym_close_statement_token1] = ACTIONS(1339),
    [aux_sym_inquire_statement_token1] = ACTIONS(1339),
    [aux_sym_enum_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token2] = ACTIONS(1339),
    [aux_sym_file_position_statement_token3] = ACTIONS(1339),
    [aux_sym_file_position_statement_token4] = ACTIONS(1339),
    [aux_sym_logical_expression_token5] = ACTIONS(1341),
    [anon_sym_PLUS] = ACTIONS(1341),
    [anon_sym_DOT] = ACTIONS(1339),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1341),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [sym_boolean_literal] = ACTIONS(1341),
    [aux_sym_null_literal_token1] = ACTIONS(1339),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1339),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1341),
    [sym__float_literal] = ACTIONS(1341),
    [sym__boz_literal] = ACTIONS(1341),
    [sym_string_literal] = ACTIONS(1341),
  },
  [331] = {
    [sym_preproc_file_line] = STATE(331),
    [aux_sym_end_program_statement_token1] = ACTIONS(1323),
    [aux_sym_end_program_statement_token2] = ACTIONS(1323),
    [anon_sym_LPAREN] = ACTIONS(1323),
    [aux_sym_interface_statement_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1323),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1323),
    [aux_sym_contains_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token2] = ACTIONS(1323),
    [aux_sym_implicit_statement_token1] = ACTIONS(1323),
    [aux_sym_implicit_statement_token3] = ACTIONS(1323),
    [aux_sym_implicit_statement_token4] = ACTIONS(1323),
    [aux_sym_save_statement_token1] = ACTIONS(1323),
    [aux_sym_private_statement_token1] = ACTIONS(1323),
    [aux_sym_public_statement_token1] = ACTIONS(1323),
    [aux_sym_namelist_statement_token1] = ACTIONS(1323),
    [aux_sym_common_statement_token1] = ACTIONS(1323),
    [anon_sym_DASH] = ACTIONS(1325),
    [aux_sym_import_statement_token1] = ACTIONS(1323),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1323),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1323),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1323),
    [aux_sym_variable_attributes_token1] = ACTIONS(1323),
    [aux_sym_variable_attributes_token2] = ACTIONS(1323),
    [aux_sym_variable_attributes_token3] = ACTIONS(1323),
    [aux_sym_variable_attributes_token4] = ACTIONS(1323),
    [aux_sym_variable_attributes_token5] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1323),
    [aux_sym_derived_type_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token2] = ACTIONS(1323),
    [aux_sym_type_qualifier_token3] = ACTIONS(1323),
    [aux_sym_type_qualifier_token4] = ACTIONS(1323),
    [aux_sym_type_qualifier_token5] = ACTIONS(1323),
    [aux_sym_type_qualifier_token9] = ACTIONS(1323),
    [aux_sym_type_qualifier_token10] = ACTIONS(1323),
    [aux_sym_type_qualifier_token11] = ACTIONS(1323),
    [aux_sym_type_qualifier_token12] = ACTIONS(1323),
    [aux_sym_type_qualifier_token13] = ACTIONS(1323),
    [aux_sym_type_qualifier_token14] = ACTIONS(1323),
    [aux_sym_type_qualifier_token15] = ACTIONS(1323),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token2] = ACTIONS(1323),
    [aux_sym_subroutine_call_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token2] = ACTIONS(1323),
    [aux_sym_keyword_statement_token3] = ACTIONS(1323),
    [aux_sym_keyword_statement_token4] = ACTIONS(1323),
    [aux_sym_keyword_statement_token6] = ACTIONS(1323),
    [aux_sym_keyword_statement_token7] = ACTIONS(1323),
    [aux_sym_include_statement_token1] = ACTIONS(1323),
    [aux_sym_data_statement_token1] = ACTIONS(1323),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1323),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1323),
    [aux_sym_end_if_statement_token1] = ACTIONS(1323),
    [aux_sym_elseif_clause_token2] = ACTIONS(1323),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1323),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token3] = ACTIONS(1323),
    [aux_sym_select_type_statement_token1] = ACTIONS(1323),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1323),
    [aux_sym_block_construct_token1] = ACTIONS(1323),
    [aux_sym_associate_statement_token1] = ACTIONS(1323),
    [aux_sym_format_statement_token1] = ACTIONS(1323),
    [aux_sym_print_statement_token1] = ACTIONS(1323),
    [aux_sym_open_statement_token1] = ACTIONS(1323),
    [aux_sym_close_statement_token1] = ACTIONS(1323),
    [aux_sym_inquire_statement_token1] = ACTIONS(1323),
    [aux_sym_enum_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token2] = ACTIONS(1323),
    [aux_sym_file_position_statement_token3] = ACTIONS(1323),
    [aux_sym_file_position_statement_token4] = ACTIONS(1323),
    [aux_sym_logical_expression_token5] = ACTIONS(1325),
    [anon_sym_PLUS] = ACTIONS(1325),
    [anon_sym_DOT] = ACTIONS(1323),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1325),
    [anon_sym_LBRACK] = ACTIONS(1325),
    [sym_boolean_literal] = ACTIONS(1325),
    [aux_sym_null_literal_token1] = ACTIONS(1323),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1323),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1325),
    [sym__float_literal] = ACTIONS(1325),
    [sym__boz_literal] = ACTIONS(1325),
    [sym_string_literal] = ACTIONS(1325),
  },
  [332] = {
    [sym_preproc_file_line] = STATE(332),
    [aux_sym_end_program_statement_token1] = ACTIONS(1351),
    [aux_sym_end_program_statement_token2] = ACTIONS(1351),
    [anon_sym_LPAREN] = ACTIONS(1351),
    [aux_sym_interface_statement_token1] = ACTIONS(1351),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1351),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1351),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1351),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1351),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1351),
    [aux_sym_contains_statement_token1] = ACTIONS(1351),
    [aux_sym_use_statement_token1] = ACTIONS(1351),
    [aux_sym_use_statement_token2] = ACTIONS(1351),
    [aux_sym_implicit_statement_token1] = ACTIONS(1351),
    [aux_sym_implicit_statement_token3] = ACTIONS(1351),
    [aux_sym_implicit_statement_token4] = ACTIONS(1351),
    [aux_sym_save_statement_token1] = ACTIONS(1351),
    [aux_sym_private_statement_token1] = ACTIONS(1351),
    [aux_sym_public_statement_token1] = ACTIONS(1351),
    [aux_sym_namelist_statement_token1] = ACTIONS(1351),
    [aux_sym_common_statement_token1] = ACTIONS(1351),
    [anon_sym_DASH] = ACTIONS(1353),
    [aux_sym_import_statement_token1] = ACTIONS(1351),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1351),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1351),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1351),
    [aux_sym_variable_attributes_token1] = ACTIONS(1351),
    [aux_sym_variable_attributes_token2] = ACTIONS(1351),
    [aux_sym_variable_attributes_token3] = ACTIONS(1351),
    [aux_sym_variable_attributes_token4] = ACTIONS(1351),
    [aux_sym_variable_attributes_token5] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1351),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1351),
    [aux_sym_derived_type_token1] = ACTIONS(1351),
    [aux_sym_type_qualifier_token1] = ACTIONS(1351),
    [aux_sym_type_qualifier_token2] = ACTIONS(1351),
    [aux_sym_type_qualifier_token3] = ACTIONS(1351),
    [aux_sym_type_qualifier_token4] = ACTIONS(1351),
    [aux_sym_type_qualifier_token5] = ACTIONS(1351),
    [aux_sym_type_qualifier_token9] = ACTIONS(1351),
    [aux_sym_type_qualifier_token10] = ACTIONS(1351),
    [aux_sym_type_qualifier_token11] = ACTIONS(1351),
    [aux_sym_type_qualifier_token12] = ACTIONS(1351),
    [aux_sym_type_qualifier_token13] = ACTIONS(1351),
    [aux_sym_type_qualifier_token14] = ACTIONS(1351),
    [aux_sym_type_qualifier_token15] = ACTIONS(1351),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1351),
    [aux_sym_stop_statement_token1] = ACTIONS(1351),
    [aux_sym_stop_statement_token2] = ACTIONS(1351),
    [aux_sym_subroutine_call_token1] = ACTIONS(1351),
    [aux_sym_keyword_statement_token1] = ACTIONS(1351),
    [aux_sym_keyword_statement_token2] = ACTIONS(1351),
    [aux_sym_keyword_statement_token3] = ACTIONS(1351),
    [aux_sym_keyword_statement_token4] = ACTIONS(1351),
    [aux_sym_keyword_statement_token6] = ACTIONS(1351),
    [aux_sym_keyword_statement_token7] = ACTIONS(1351),
    [aux_sym_include_statement_token1] = ACTIONS(1351),
    [aux_sym_data_statement_token1] = ACTIONS(1351),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1351),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1351),
    [aux_sym_end_if_statement_token1] = ACTIONS(1351),
    [aux_sym_elseif_clause_token2] = ACTIONS(1351),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1351),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1351),
    [aux_sym_select_case_statement_token1] = ACTIONS(1351),
    [aux_sym_select_case_statement_token3] = ACTIONS(1351),
    [aux_sym_select_type_statement_token1] = ACTIONS(1351),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1351),
    [aux_sym_block_construct_token1] = ACTIONS(1351),
    [aux_sym_associate_statement_token1] = ACTIONS(1351),
    [aux_sym_format_statement_token1] = ACTIONS(1351),
    [aux_sym_print_statement_token1] = ACTIONS(1351),
    [aux_sym_open_statement_token1] = ACTIONS(1351),
    [aux_sym_close_statement_token1] = ACTIONS(1351),
    [aux_sym_inquire_statement_token1] = ACTIONS(1351),
    [aux_sym_enum_statement_token1] = ACTIONS(1351),
    [aux_sym_file_position_statement_token1] = ACTIONS(1351),
    [aux_sym_file_position_statement_token2] = ACTIONS(1351),
    [aux_sym_file_position_statement_token3] = ACTIONS(1351),
    [aux_sym_file_position_statement_token4] = ACTIONS(1351),
    [aux_sym_logical_expression_token5] = ACTIONS(1353),
    [anon_sym_PLUS] = ACTIONS(1353),
    [anon_sym_DOT] = ACTIONS(1351),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1353),
    [anon_sym_LBRACK] = ACTIONS(1353),
    [sym_boolean_literal] = ACTIONS(1353),
    [aux_sym_null_literal_token1] = ACTIONS(1351),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1351),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1353),
    [sym__float_literal] = ACTIONS(1353),
    [sym__boz_literal] = ACTIONS(1353),
    [sym_string_literal] = ACTIONS(1353),
  },
  [333] = {
    [sym_preproc_file_line] = STATE(333),
    [aux_sym_end_program_statement_token1] = ACTIONS(1343),
    [anon_sym_LPAREN] = ACTIONS(1343),
    [aux_sym_interface_statement_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1343),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1343),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1343),
    [aux_sym_contains_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token2] = ACTIONS(1343),
    [aux_sym_implicit_statement_token1] = ACTIONS(1343),
    [aux_sym_implicit_statement_token3] = ACTIONS(1343),
    [aux_sym_implicit_statement_token4] = ACTIONS(1343),
    [aux_sym_save_statement_token1] = ACTIONS(1343),
    [aux_sym_private_statement_token1] = ACTIONS(1343),
    [aux_sym_public_statement_token1] = ACTIONS(1343),
    [aux_sym_namelist_statement_token1] = ACTIONS(1343),
    [aux_sym_common_statement_token1] = ACTIONS(1343),
    [anon_sym_DASH] = ACTIONS(1345),
    [aux_sym_import_statement_token1] = ACTIONS(1343),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1343),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1343),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1343),
    [aux_sym_variable_attributes_token1] = ACTIONS(1343),
    [aux_sym_variable_attributes_token2] = ACTIONS(1343),
    [aux_sym_variable_attributes_token3] = ACTIONS(1343),
    [aux_sym_variable_attributes_token4] = ACTIONS(1343),
    [aux_sym_variable_attributes_token5] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1343),
    [aux_sym_derived_type_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token2] = ACTIONS(1343),
    [aux_sym_type_qualifier_token3] = ACTIONS(1343),
    [aux_sym_type_qualifier_token4] = ACTIONS(1343),
    [aux_sym_type_qualifier_token5] = ACTIONS(1343),
    [aux_sym_type_qualifier_token9] = ACTIONS(1343),
    [aux_sym_type_qualifier_token10] = ACTIONS(1343),
    [aux_sym_type_qualifier_token11] = ACTIONS(1343),
    [aux_sym_type_qualifier_token12] = ACTIONS(1343),
    [aux_sym_type_qualifier_token13] = ACTIONS(1343),
    [aux_sym_type_qualifier_token14] = ACTIONS(1343),
    [aux_sym_type_qualifier_token15] = ACTIONS(1343),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token2] = ACTIONS(1343),
    [aux_sym_subroutine_call_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token2] = ACTIONS(1343),
    [aux_sym_keyword_statement_token3] = ACTIONS(1343),
    [aux_sym_keyword_statement_token4] = ACTIONS(1343),
    [aux_sym_keyword_statement_token6] = ACTIONS(1343),
    [aux_sym_keyword_statement_token7] = ACTIONS(1343),
    [aux_sym_include_statement_token1] = ACTIONS(1343),
    [aux_sym_data_statement_token1] = ACTIONS(1343),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1343),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1343),
    [aux_sym_end_if_statement_token1] = ACTIONS(1343),
    [aux_sym_elseif_clause_token2] = ACTIONS(1343),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1343),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token3] = ACTIONS(1343),
    [aux_sym_select_type_statement_token1] = ACTIONS(1343),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1343),
    [aux_sym_block_construct_token1] = ACTIONS(1343),
    [aux_sym_associate_statement_token1] = ACTIONS(1343),
    [aux_sym_format_statement_token1] = ACTIONS(1343),
    [aux_sym_print_statement_token1] = ACTIONS(1343),
    [aux_sym_open_statement_token1] = ACTIONS(1343),
    [aux_sym_close_statement_token1] = ACTIONS(1343),
    [aux_sym_inquire_statement_token1] = ACTIONS(1343),
    [aux_sym_enum_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token2] = ACTIONS(1343),
    [aux_sym_file_position_statement_token3] = ACTIONS(1343),
    [aux_sym_file_position_statement_token4] = ACTIONS(1343),
    [aux_sym_logical_expression_token5] = ACTIONS(1345),
    [anon_sym_PLUS] = ACTIONS(1345),
    [anon_sym_DOT] = ACTIONS(1343),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1345),
    [anon_sym_LBRACK] = ACTIONS(1345),
    [sym_boolean_literal] = ACTIONS(1345),
    [aux_sym_null_literal_token1] = ACTIONS(1343),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1343),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1345),
    [sym__float_literal] = ACTIONS(1345),
    [sym__boz_literal] = ACTIONS(1345),
    [sym_string_literal] = ACTIONS(1345),
  },
  [334] = {
    [sym_preproc_file_line] = STATE(334),
    [aux_sym_end_program_statement_token1] = ACTIONS(1315),
    [anon_sym_LPAREN] = ACTIONS(1315),
    [aux_sym_interface_statement_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1315),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1315),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1315),
    [aux_sym_contains_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token2] = ACTIONS(1315),
    [aux_sym_implicit_statement_token1] = ACTIONS(1315),
    [aux_sym_implicit_statement_token3] = ACTIONS(1315),
    [aux_sym_implicit_statement_token4] = ACTIONS(1315),
    [aux_sym_save_statement_token1] = ACTIONS(1315),
    [aux_sym_private_statement_token1] = ACTIONS(1315),
    [aux_sym_public_statement_token1] = ACTIONS(1315),
    [aux_sym_namelist_statement_token1] = ACTIONS(1315),
    [aux_sym_common_statement_token1] = ACTIONS(1315),
    [anon_sym_DASH] = ACTIONS(1317),
    [aux_sym_import_statement_token1] = ACTIONS(1315),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1315),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1315),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1315),
    [aux_sym_variable_attributes_token1] = ACTIONS(1315),
    [aux_sym_variable_attributes_token2] = ACTIONS(1315),
    [aux_sym_variable_attributes_token3] = ACTIONS(1315),
    [aux_sym_variable_attributes_token4] = ACTIONS(1315),
    [aux_sym_variable_attributes_token5] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1315),
    [aux_sym_derived_type_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token2] = ACTIONS(1315),
    [aux_sym_type_qualifier_token3] = ACTIONS(1315),
    [aux_sym_type_qualifier_token4] = ACTIONS(1315),
    [aux_sym_type_qualifier_token5] = ACTIONS(1315),
    [aux_sym_type_qualifier_token9] = ACTIONS(1315),
    [aux_sym_type_qualifier_token10] = ACTIONS(1315),
    [aux_sym_type_qualifier_token11] = ACTIONS(1315),
    [aux_sym_type_qualifier_token12] = ACTIONS(1315),
    [aux_sym_type_qualifier_token13] = ACTIONS(1315),
    [aux_sym_type_qualifier_token14] = ACTIONS(1315),
    [aux_sym_type_qualifier_token15] = ACTIONS(1315),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token2] = ACTIONS(1315),
    [aux_sym_subroutine_call_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token2] = ACTIONS(1315),
    [aux_sym_keyword_statement_token3] = ACTIONS(1315),
    [aux_sym_keyword_statement_token4] = ACTIONS(1315),
    [aux_sym_keyword_statement_token6] = ACTIONS(1315),
    [aux_sym_keyword_statement_token7] = ACTIONS(1315),
    [aux_sym_include_statement_token1] = ACTIONS(1315),
    [aux_sym_data_statement_token1] = ACTIONS(1315),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1315),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1315),
    [aux_sym_end_if_statement_token1] = ACTIONS(1315),
    [aux_sym_elseif_clause_token2] = ACTIONS(1315),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1315),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token3] = ACTIONS(1315),
    [aux_sym_select_type_statement_token1] = ACTIONS(1315),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1315),
    [aux_sym_block_construct_token1] = ACTIONS(1315),
    [aux_sym_associate_statement_token1] = ACTIONS(1315),
    [aux_sym_format_statement_token1] = ACTIONS(1315),
    [aux_sym_print_statement_token1] = ACTIONS(1315),
    [aux_sym_open_statement_token1] = ACTIONS(1315),
    [aux_sym_close_statement_token1] = ACTIONS(1315),
    [aux_sym_inquire_statement_token1] = ACTIONS(1315),
    [aux_sym_enum_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token2] = ACTIONS(1315),
    [aux_sym_file_position_statement_token3] = ACTIONS(1315),
    [aux_sym_file_position_statement_token4] = ACTIONS(1315),
    [aux_sym_logical_expression_token5] = ACTIONS(1317),
    [anon_sym_PLUS] = ACTIONS(1317),
    [anon_sym_DOT] = ACTIONS(1315),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1317),
    [anon_sym_LBRACK] = ACTIONS(1317),
    [sym_boolean_literal] = ACTIONS(1317),
    [aux_sym_null_literal_token1] = ACTIONS(1315),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1315),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1317),
    [sym__float_literal] = ACTIONS(1317),
    [sym__boz_literal] = ACTIONS(1317),
    [sym_string_literal] = ACTIONS(1317),
  },
  [335] = {
    [sym_preproc_file_line] = STATE(335),
    [aux_sym_end_program_statement_token1] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1259),
    [aux_sym_interface_statement_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1259),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1259),
    [aux_sym_end_function_statement_token1] = ACTIONS(1259),
    [aux_sym_contains_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token2] = ACTIONS(1259),
    [aux_sym_implicit_statement_token1] = ACTIONS(1259),
    [aux_sym_implicit_statement_token3] = ACTIONS(1259),
    [aux_sym_implicit_statement_token4] = ACTIONS(1259),
    [aux_sym_save_statement_token1] = ACTIONS(1259),
    [aux_sym_private_statement_token1] = ACTIONS(1259),
    [aux_sym_public_statement_token1] = ACTIONS(1259),
    [aux_sym_namelist_statement_token1] = ACTIONS(1259),
    [aux_sym_common_statement_token1] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [aux_sym_import_statement_token1] = ACTIONS(1259),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1259),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1259),
    [aux_sym_variable_attributes_token1] = ACTIONS(1259),
    [aux_sym_variable_attributes_token2] = ACTIONS(1259),
    [aux_sym_variable_attributes_token3] = ACTIONS(1259),
    [aux_sym_variable_attributes_token4] = ACTIONS(1259),
    [aux_sym_variable_attributes_token5] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1259),
    [aux_sym_derived_type_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token2] = ACTIONS(1259),
    [aux_sym_type_qualifier_token3] = ACTIONS(1259),
    [aux_sym_type_qualifier_token4] = ACTIONS(1259),
    [aux_sym_type_qualifier_token5] = ACTIONS(1259),
    [aux_sym_type_qualifier_token9] = ACTIONS(1259),
    [aux_sym_type_qualifier_token10] = ACTIONS(1259),
    [aux_sym_type_qualifier_token11] = ACTIONS(1259),
    [aux_sym_type_qualifier_token12] = ACTIONS(1259),
    [aux_sym_type_qualifier_token13] = ACTIONS(1259),
    [aux_sym_type_qualifier_token14] = ACTIONS(1259),
    [aux_sym_type_qualifier_token15] = ACTIONS(1259),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token2] = ACTIONS(1259),
    [aux_sym_subroutine_call_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token2] = ACTIONS(1259),
    [aux_sym_keyword_statement_token3] = ACTIONS(1259),
    [aux_sym_keyword_statement_token4] = ACTIONS(1259),
    [aux_sym_keyword_statement_token6] = ACTIONS(1259),
    [aux_sym_keyword_statement_token7] = ACTIONS(1259),
    [aux_sym_include_statement_token1] = ACTIONS(1259),
    [aux_sym_data_statement_token1] = ACTIONS(1259),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1259),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1259),
    [aux_sym_end_if_statement_token1] = ACTIONS(1259),
    [aux_sym_elseif_clause_token2] = ACTIONS(1259),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1259),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token3] = ACTIONS(1259),
    [aux_sym_select_type_statement_token1] = ACTIONS(1259),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1259),
    [aux_sym_block_construct_token1] = ACTIONS(1259),
    [aux_sym_associate_statement_token1] = ACTIONS(1259),
    [aux_sym_format_statement_token1] = ACTIONS(1259),
    [aux_sym_print_statement_token1] = ACTIONS(1259),
    [aux_sym_open_statement_token1] = ACTIONS(1259),
    [aux_sym_close_statement_token1] = ACTIONS(1259),
    [aux_sym_inquire_statement_token1] = ACTIONS(1259),
    [aux_sym_enum_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token2] = ACTIONS(1259),
    [aux_sym_file_position_statement_token3] = ACTIONS(1259),
    [aux_sym_file_position_statement_token4] = ACTIONS(1259),
    [aux_sym_logical_expression_token5] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [sym_boolean_literal] = ACTIONS(1257),
    [aux_sym_null_literal_token1] = ACTIONS(1259),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1259),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1257),
    [sym__float_literal] = ACTIONS(1257),
    [sym__boz_literal] = ACTIONS(1257),
    [sym_string_literal] = ACTIONS(1257),
  },
  [336] = {
    [sym_preproc_file_line] = STATE(336),
    [aux_sym_end_program_statement_token1] = ACTIONS(1339),
    [aux_sym_end_program_statement_token2] = ACTIONS(1339),
    [anon_sym_LPAREN] = ACTIONS(1339),
    [aux_sym_interface_statement_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1339),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1339),
    [aux_sym_contains_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token2] = ACTIONS(1339),
    [aux_sym_implicit_statement_token1] = ACTIONS(1339),
    [aux_sym_implicit_statement_token3] = ACTIONS(1339),
    [aux_sym_implicit_statement_token4] = ACTIONS(1339),
    [aux_sym_save_statement_token1] = ACTIONS(1339),
    [aux_sym_private_statement_token1] = ACTIONS(1339),
    [aux_sym_public_statement_token1] = ACTIONS(1339),
    [aux_sym_namelist_statement_token1] = ACTIONS(1339),
    [aux_sym_common_statement_token1] = ACTIONS(1339),
    [anon_sym_DASH] = ACTIONS(1341),
    [aux_sym_import_statement_token1] = ACTIONS(1339),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1339),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1339),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1339),
    [aux_sym_variable_attributes_token1] = ACTIONS(1339),
    [aux_sym_variable_attributes_token2] = ACTIONS(1339),
    [aux_sym_variable_attributes_token3] = ACTIONS(1339),
    [aux_sym_variable_attributes_token4] = ACTIONS(1339),
    [aux_sym_variable_attributes_token5] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1339),
    [aux_sym_derived_type_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token2] = ACTIONS(1339),
    [aux_sym_type_qualifier_token3] = ACTIONS(1339),
    [aux_sym_type_qualifier_token4] = ACTIONS(1339),
    [aux_sym_type_qualifier_token5] = ACTIONS(1339),
    [aux_sym_type_qualifier_token9] = ACTIONS(1339),
    [aux_sym_type_qualifier_token10] = ACTIONS(1339),
    [aux_sym_type_qualifier_token11] = ACTIONS(1339),
    [aux_sym_type_qualifier_token12] = ACTIONS(1339),
    [aux_sym_type_qualifier_token13] = ACTIONS(1339),
    [aux_sym_type_qualifier_token14] = ACTIONS(1339),
    [aux_sym_type_qualifier_token15] = ACTIONS(1339),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token2] = ACTIONS(1339),
    [aux_sym_subroutine_call_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token2] = ACTIONS(1339),
    [aux_sym_keyword_statement_token3] = ACTIONS(1339),
    [aux_sym_keyword_statement_token4] = ACTIONS(1339),
    [aux_sym_keyword_statement_token6] = ACTIONS(1339),
    [aux_sym_keyword_statement_token7] = ACTIONS(1339),
    [aux_sym_include_statement_token1] = ACTIONS(1339),
    [aux_sym_data_statement_token1] = ACTIONS(1339),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1339),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1339),
    [aux_sym_end_if_statement_token1] = ACTIONS(1339),
    [aux_sym_elseif_clause_token2] = ACTIONS(1339),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1339),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token3] = ACTIONS(1339),
    [aux_sym_select_type_statement_token1] = ACTIONS(1339),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1339),
    [aux_sym_block_construct_token1] = ACTIONS(1339),
    [aux_sym_associate_statement_token1] = ACTIONS(1339),
    [aux_sym_format_statement_token1] = ACTIONS(1339),
    [aux_sym_print_statement_token1] = ACTIONS(1339),
    [aux_sym_open_statement_token1] = ACTIONS(1339),
    [aux_sym_close_statement_token1] = ACTIONS(1339),
    [aux_sym_inquire_statement_token1] = ACTIONS(1339),
    [aux_sym_enum_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token2] = ACTIONS(1339),
    [aux_sym_file_position_statement_token3] = ACTIONS(1339),
    [aux_sym_file_position_statement_token4] = ACTIONS(1339),
    [aux_sym_logical_expression_token5] = ACTIONS(1341),
    [anon_sym_PLUS] = ACTIONS(1341),
    [anon_sym_DOT] = ACTIONS(1339),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1341),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [sym_boolean_literal] = ACTIONS(1341),
    [aux_sym_null_literal_token1] = ACTIONS(1339),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1339),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1341),
    [sym__float_literal] = ACTIONS(1341),
    [sym__boz_literal] = ACTIONS(1341),
    [sym_string_literal] = ACTIONS(1341),
  },
  [337] = {
    [sym_preproc_file_line] = STATE(337),
    [aux_sym_end_program_statement_token1] = ACTIONS(1335),
    [aux_sym_end_program_statement_token2] = ACTIONS(1335),
    [anon_sym_LPAREN] = ACTIONS(1335),
    [aux_sym_interface_statement_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1335),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1335),
    [aux_sym_contains_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token2] = ACTIONS(1335),
    [aux_sym_implicit_statement_token1] = ACTIONS(1335),
    [aux_sym_implicit_statement_token3] = ACTIONS(1335),
    [aux_sym_implicit_statement_token4] = ACTIONS(1335),
    [aux_sym_save_statement_token1] = ACTIONS(1335),
    [aux_sym_private_statement_token1] = ACTIONS(1335),
    [aux_sym_public_statement_token1] = ACTIONS(1335),
    [aux_sym_namelist_statement_token1] = ACTIONS(1335),
    [aux_sym_common_statement_token1] = ACTIONS(1335),
    [anon_sym_DASH] = ACTIONS(1337),
    [aux_sym_import_statement_token1] = ACTIONS(1335),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1335),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1335),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1335),
    [aux_sym_variable_attributes_token1] = ACTIONS(1335),
    [aux_sym_variable_attributes_token2] = ACTIONS(1335),
    [aux_sym_variable_attributes_token3] = ACTIONS(1335),
    [aux_sym_variable_attributes_token4] = ACTIONS(1335),
    [aux_sym_variable_attributes_token5] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1335),
    [aux_sym_derived_type_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token2] = ACTIONS(1335),
    [aux_sym_type_qualifier_token3] = ACTIONS(1335),
    [aux_sym_type_qualifier_token4] = ACTIONS(1335),
    [aux_sym_type_qualifier_token5] = ACTIONS(1335),
    [aux_sym_type_qualifier_token9] = ACTIONS(1335),
    [aux_sym_type_qualifier_token10] = ACTIONS(1335),
    [aux_sym_type_qualifier_token11] = ACTIONS(1335),
    [aux_sym_type_qualifier_token12] = ACTIONS(1335),
    [aux_sym_type_qualifier_token13] = ACTIONS(1335),
    [aux_sym_type_qualifier_token14] = ACTIONS(1335),
    [aux_sym_type_qualifier_token15] = ACTIONS(1335),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token2] = ACTIONS(1335),
    [aux_sym_subroutine_call_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token2] = ACTIONS(1335),
    [aux_sym_keyword_statement_token3] = ACTIONS(1335),
    [aux_sym_keyword_statement_token4] = ACTIONS(1335),
    [aux_sym_keyword_statement_token6] = ACTIONS(1335),
    [aux_sym_keyword_statement_token7] = ACTIONS(1335),
    [aux_sym_include_statement_token1] = ACTIONS(1335),
    [aux_sym_data_statement_token1] = ACTIONS(1335),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1335),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1335),
    [aux_sym_end_if_statement_token1] = ACTIONS(1335),
    [aux_sym_elseif_clause_token2] = ACTIONS(1335),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1335),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token3] = ACTIONS(1335),
    [aux_sym_select_type_statement_token1] = ACTIONS(1335),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1335),
    [aux_sym_block_construct_token1] = ACTIONS(1335),
    [aux_sym_associate_statement_token1] = ACTIONS(1335),
    [aux_sym_format_statement_token1] = ACTIONS(1335),
    [aux_sym_print_statement_token1] = ACTIONS(1335),
    [aux_sym_open_statement_token1] = ACTIONS(1335),
    [aux_sym_close_statement_token1] = ACTIONS(1335),
    [aux_sym_inquire_statement_token1] = ACTIONS(1335),
    [aux_sym_enum_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token2] = ACTIONS(1335),
    [aux_sym_file_position_statement_token3] = ACTIONS(1335),
    [aux_sym_file_position_statement_token4] = ACTIONS(1335),
    [aux_sym_logical_expression_token5] = ACTIONS(1337),
    [anon_sym_PLUS] = ACTIONS(1337),
    [anon_sym_DOT] = ACTIONS(1335),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1337),
    [anon_sym_LBRACK] = ACTIONS(1337),
    [sym_boolean_literal] = ACTIONS(1337),
    [aux_sym_null_literal_token1] = ACTIONS(1335),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1335),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1337),
    [sym__float_literal] = ACTIONS(1337),
    [sym__boz_literal] = ACTIONS(1337),
    [sym_string_literal] = ACTIONS(1337),
  },
  [338] = {
    [sym_preproc_file_line] = STATE(338),
    [aux_sym_end_program_statement_token1] = ACTIONS(1343),
    [aux_sym_end_program_statement_token2] = ACTIONS(1343),
    [anon_sym_LPAREN] = ACTIONS(1343),
    [aux_sym_interface_statement_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1343),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1343),
    [aux_sym_contains_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token2] = ACTIONS(1343),
    [aux_sym_implicit_statement_token1] = ACTIONS(1343),
    [aux_sym_implicit_statement_token3] = ACTIONS(1343),
    [aux_sym_implicit_statement_token4] = ACTIONS(1343),
    [aux_sym_save_statement_token1] = ACTIONS(1343),
    [aux_sym_private_statement_token1] = ACTIONS(1343),
    [aux_sym_public_statement_token1] = ACTIONS(1343),
    [aux_sym_namelist_statement_token1] = ACTIONS(1343),
    [aux_sym_common_statement_token1] = ACTIONS(1343),
    [anon_sym_DASH] = ACTIONS(1345),
    [aux_sym_import_statement_token1] = ACTIONS(1343),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1343),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1343),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1343),
    [aux_sym_variable_attributes_token1] = ACTIONS(1343),
    [aux_sym_variable_attributes_token2] = ACTIONS(1343),
    [aux_sym_variable_attributes_token3] = ACTIONS(1343),
    [aux_sym_variable_attributes_token4] = ACTIONS(1343),
    [aux_sym_variable_attributes_token5] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1343),
    [aux_sym_derived_type_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token2] = ACTIONS(1343),
    [aux_sym_type_qualifier_token3] = ACTIONS(1343),
    [aux_sym_type_qualifier_token4] = ACTIONS(1343),
    [aux_sym_type_qualifier_token5] = ACTIONS(1343),
    [aux_sym_type_qualifier_token9] = ACTIONS(1343),
    [aux_sym_type_qualifier_token10] = ACTIONS(1343),
    [aux_sym_type_qualifier_token11] = ACTIONS(1343),
    [aux_sym_type_qualifier_token12] = ACTIONS(1343),
    [aux_sym_type_qualifier_token13] = ACTIONS(1343),
    [aux_sym_type_qualifier_token14] = ACTIONS(1343),
    [aux_sym_type_qualifier_token15] = ACTIONS(1343),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token2] = ACTIONS(1343),
    [aux_sym_subroutine_call_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token2] = ACTIONS(1343),
    [aux_sym_keyword_statement_token3] = ACTIONS(1343),
    [aux_sym_keyword_statement_token4] = ACTIONS(1343),
    [aux_sym_keyword_statement_token6] = ACTIONS(1343),
    [aux_sym_keyword_statement_token7] = ACTIONS(1343),
    [aux_sym_include_statement_token1] = ACTIONS(1343),
    [aux_sym_data_statement_token1] = ACTIONS(1343),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1343),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1343),
    [aux_sym_end_if_statement_token1] = ACTIONS(1343),
    [aux_sym_elseif_clause_token2] = ACTIONS(1343),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1343),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token3] = ACTIONS(1343),
    [aux_sym_select_type_statement_token1] = ACTIONS(1343),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1343),
    [aux_sym_block_construct_token1] = ACTIONS(1343),
    [aux_sym_associate_statement_token1] = ACTIONS(1343),
    [aux_sym_format_statement_token1] = ACTIONS(1343),
    [aux_sym_print_statement_token1] = ACTIONS(1343),
    [aux_sym_open_statement_token1] = ACTIONS(1343),
    [aux_sym_close_statement_token1] = ACTIONS(1343),
    [aux_sym_inquire_statement_token1] = ACTIONS(1343),
    [aux_sym_enum_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token2] = ACTIONS(1343),
    [aux_sym_file_position_statement_token3] = ACTIONS(1343),
    [aux_sym_file_position_statement_token4] = ACTIONS(1343),
    [aux_sym_logical_expression_token5] = ACTIONS(1345),
    [anon_sym_PLUS] = ACTIONS(1345),
    [anon_sym_DOT] = ACTIONS(1343),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1345),
    [anon_sym_LBRACK] = ACTIONS(1345),
    [sym_boolean_literal] = ACTIONS(1345),
    [aux_sym_null_literal_token1] = ACTIONS(1343),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1343),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1345),
    [sym__float_literal] = ACTIONS(1345),
    [sym__boz_literal] = ACTIONS(1345),
    [sym_string_literal] = ACTIONS(1345),
  },
  [339] = {
    [sym_preproc_file_line] = STATE(339),
    [aux_sym_end_program_statement_token1] = ACTIONS(1099),
    [aux_sym_end_program_statement_token2] = ACTIONS(1099),
    [anon_sym_LPAREN] = ACTIONS(1099),
    [aux_sym_interface_statement_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1099),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1099),
    [aux_sym_contains_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token2] = ACTIONS(1099),
    [aux_sym_implicit_statement_token1] = ACTIONS(1099),
    [aux_sym_implicit_statement_token3] = ACTIONS(1099),
    [aux_sym_implicit_statement_token4] = ACTIONS(1099),
    [aux_sym_save_statement_token1] = ACTIONS(1099),
    [aux_sym_private_statement_token1] = ACTIONS(1099),
    [aux_sym_public_statement_token1] = ACTIONS(1099),
    [aux_sym_namelist_statement_token1] = ACTIONS(1099),
    [aux_sym_common_statement_token1] = ACTIONS(1099),
    [anon_sym_DASH] = ACTIONS(1097),
    [aux_sym_import_statement_token1] = ACTIONS(1099),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1099),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1099),
    [aux_sym_variable_attributes_token1] = ACTIONS(1099),
    [aux_sym_variable_attributes_token2] = ACTIONS(1099),
    [aux_sym_variable_attributes_token3] = ACTIONS(1099),
    [aux_sym_variable_attributes_token4] = ACTIONS(1099),
    [aux_sym_variable_attributes_token5] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1099),
    [aux_sym_derived_type_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token2] = ACTIONS(1099),
    [aux_sym_type_qualifier_token3] = ACTIONS(1099),
    [aux_sym_type_qualifier_token4] = ACTIONS(1099),
    [aux_sym_type_qualifier_token5] = ACTIONS(1099),
    [aux_sym_type_qualifier_token9] = ACTIONS(1099),
    [aux_sym_type_qualifier_token10] = ACTIONS(1099),
    [aux_sym_type_qualifier_token11] = ACTIONS(1099),
    [aux_sym_type_qualifier_token12] = ACTIONS(1099),
    [aux_sym_type_qualifier_token13] = ACTIONS(1099),
    [aux_sym_type_qualifier_token14] = ACTIONS(1099),
    [aux_sym_type_qualifier_token15] = ACTIONS(1099),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token2] = ACTIONS(1099),
    [aux_sym_subroutine_call_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token2] = ACTIONS(1099),
    [aux_sym_keyword_statement_token3] = ACTIONS(1099),
    [aux_sym_keyword_statement_token4] = ACTIONS(1099),
    [aux_sym_keyword_statement_token6] = ACTIONS(1099),
    [aux_sym_keyword_statement_token7] = ACTIONS(1099),
    [aux_sym_include_statement_token1] = ACTIONS(1099),
    [aux_sym_data_statement_token1] = ACTIONS(1099),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1099),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1099),
    [aux_sym_end_if_statement_token1] = ACTIONS(1099),
    [aux_sym_elseif_clause_token2] = ACTIONS(1099),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1099),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token3] = ACTIONS(1099),
    [aux_sym_select_type_statement_token1] = ACTIONS(1099),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1099),
    [aux_sym_block_construct_token1] = ACTIONS(1099),
    [aux_sym_associate_statement_token1] = ACTIONS(1099),
    [aux_sym_format_statement_token1] = ACTIONS(1099),
    [aux_sym_print_statement_token1] = ACTIONS(1099),
    [aux_sym_open_statement_token1] = ACTIONS(1099),
    [aux_sym_close_statement_token1] = ACTIONS(1099),
    [aux_sym_inquire_statement_token1] = ACTIONS(1099),
    [aux_sym_enum_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token2] = ACTIONS(1099),
    [aux_sym_file_position_statement_token3] = ACTIONS(1099),
    [aux_sym_file_position_statement_token4] = ACTIONS(1099),
    [aux_sym_logical_expression_token5] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [sym_boolean_literal] = ACTIONS(1097),
    [aux_sym_null_literal_token1] = ACTIONS(1099),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1099),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1097),
    [sym__float_literal] = ACTIONS(1097),
    [sym__boz_literal] = ACTIONS(1097),
    [sym_string_literal] = ACTIONS(1097),
  },
  [340] = {
    [sym_preproc_file_line] = STATE(340),
    [aux_sym_end_program_statement_token1] = ACTIONS(1355),
    [anon_sym_LPAREN] = ACTIONS(1355),
    [aux_sym_interface_statement_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1355),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1355),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1355),
    [aux_sym_contains_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token2] = ACTIONS(1355),
    [aux_sym_implicit_statement_token1] = ACTIONS(1355),
    [aux_sym_implicit_statement_token3] = ACTIONS(1355),
    [aux_sym_implicit_statement_token4] = ACTIONS(1355),
    [aux_sym_save_statement_token1] = ACTIONS(1355),
    [aux_sym_private_statement_token1] = ACTIONS(1355),
    [aux_sym_public_statement_token1] = ACTIONS(1355),
    [aux_sym_namelist_statement_token1] = ACTIONS(1355),
    [aux_sym_common_statement_token1] = ACTIONS(1355),
    [anon_sym_DASH] = ACTIONS(1357),
    [aux_sym_import_statement_token1] = ACTIONS(1355),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1355),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1355),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1355),
    [aux_sym_variable_attributes_token1] = ACTIONS(1355),
    [aux_sym_variable_attributes_token2] = ACTIONS(1355),
    [aux_sym_variable_attributes_token3] = ACTIONS(1355),
    [aux_sym_variable_attributes_token4] = ACTIONS(1355),
    [aux_sym_variable_attributes_token5] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1355),
    [aux_sym_derived_type_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token2] = ACTIONS(1355),
    [aux_sym_type_qualifier_token3] = ACTIONS(1355),
    [aux_sym_type_qualifier_token4] = ACTIONS(1355),
    [aux_sym_type_qualifier_token5] = ACTIONS(1355),
    [aux_sym_type_qualifier_token9] = ACTIONS(1355),
    [aux_sym_type_qualifier_token10] = ACTIONS(1355),
    [aux_sym_type_qualifier_token11] = ACTIONS(1355),
    [aux_sym_type_qualifier_token12] = ACTIONS(1355),
    [aux_sym_type_qualifier_token13] = ACTIONS(1355),
    [aux_sym_type_qualifier_token14] = ACTIONS(1355),
    [aux_sym_type_qualifier_token15] = ACTIONS(1355),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token2] = ACTIONS(1355),
    [aux_sym_subroutine_call_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token2] = ACTIONS(1355),
    [aux_sym_keyword_statement_token3] = ACTIONS(1355),
    [aux_sym_keyword_statement_token4] = ACTIONS(1355),
    [aux_sym_keyword_statement_token6] = ACTIONS(1355),
    [aux_sym_keyword_statement_token7] = ACTIONS(1355),
    [aux_sym_include_statement_token1] = ACTIONS(1355),
    [aux_sym_data_statement_token1] = ACTIONS(1355),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1355),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1355),
    [aux_sym_end_if_statement_token1] = ACTIONS(1355),
    [aux_sym_elseif_clause_token2] = ACTIONS(1355),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1355),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token3] = ACTIONS(1355),
    [aux_sym_select_type_statement_token1] = ACTIONS(1355),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1355),
    [aux_sym_block_construct_token1] = ACTIONS(1355),
    [aux_sym_associate_statement_token1] = ACTIONS(1355),
    [aux_sym_format_statement_token1] = ACTIONS(1355),
    [aux_sym_print_statement_token1] = ACTIONS(1355),
    [aux_sym_open_statement_token1] = ACTIONS(1355),
    [aux_sym_close_statement_token1] = ACTIONS(1355),
    [aux_sym_inquire_statement_token1] = ACTIONS(1355),
    [aux_sym_enum_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token2] = ACTIONS(1355),
    [aux_sym_file_position_statement_token3] = ACTIONS(1355),
    [aux_sym_file_position_statement_token4] = ACTIONS(1355),
    [aux_sym_logical_expression_token5] = ACTIONS(1357),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_DOT] = ACTIONS(1355),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1357),
    [anon_sym_LBRACK] = ACTIONS(1357),
    [sym_boolean_literal] = ACTIONS(1357),
    [aux_sym_null_literal_token1] = ACTIONS(1355),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1355),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1357),
    [sym__float_literal] = ACTIONS(1357),
    [sym__boz_literal] = ACTIONS(1357),
    [sym_string_literal] = ACTIONS(1357),
  },
  [341] = {
    [sym_preproc_file_line] = STATE(341),
    [aux_sym_end_program_statement_token1] = ACTIONS(1355),
    [anon_sym_LPAREN] = ACTIONS(1355),
    [aux_sym_interface_statement_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1355),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1355),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1355),
    [aux_sym_contains_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token2] = ACTIONS(1355),
    [aux_sym_implicit_statement_token1] = ACTIONS(1355),
    [aux_sym_implicit_statement_token3] = ACTIONS(1355),
    [aux_sym_implicit_statement_token4] = ACTIONS(1355),
    [aux_sym_save_statement_token1] = ACTIONS(1355),
    [aux_sym_private_statement_token1] = ACTIONS(1355),
    [aux_sym_public_statement_token1] = ACTIONS(1355),
    [aux_sym_namelist_statement_token1] = ACTIONS(1355),
    [aux_sym_common_statement_token1] = ACTIONS(1355),
    [anon_sym_DASH] = ACTIONS(1357),
    [aux_sym_import_statement_token1] = ACTIONS(1355),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1355),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1355),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1355),
    [aux_sym_variable_attributes_token1] = ACTIONS(1355),
    [aux_sym_variable_attributes_token2] = ACTIONS(1355),
    [aux_sym_variable_attributes_token3] = ACTIONS(1355),
    [aux_sym_variable_attributes_token4] = ACTIONS(1355),
    [aux_sym_variable_attributes_token5] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1355),
    [aux_sym_derived_type_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token2] = ACTIONS(1355),
    [aux_sym_type_qualifier_token3] = ACTIONS(1355),
    [aux_sym_type_qualifier_token4] = ACTIONS(1355),
    [aux_sym_type_qualifier_token5] = ACTIONS(1355),
    [aux_sym_type_qualifier_token9] = ACTIONS(1355),
    [aux_sym_type_qualifier_token10] = ACTIONS(1355),
    [aux_sym_type_qualifier_token11] = ACTIONS(1355),
    [aux_sym_type_qualifier_token12] = ACTIONS(1355),
    [aux_sym_type_qualifier_token13] = ACTIONS(1355),
    [aux_sym_type_qualifier_token14] = ACTIONS(1355),
    [aux_sym_type_qualifier_token15] = ACTIONS(1355),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token2] = ACTIONS(1355),
    [aux_sym_subroutine_call_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token2] = ACTIONS(1355),
    [aux_sym_keyword_statement_token3] = ACTIONS(1355),
    [aux_sym_keyword_statement_token4] = ACTIONS(1355),
    [aux_sym_keyword_statement_token6] = ACTIONS(1355),
    [aux_sym_keyword_statement_token7] = ACTIONS(1355),
    [aux_sym_include_statement_token1] = ACTIONS(1355),
    [aux_sym_data_statement_token1] = ACTIONS(1355),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1355),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1355),
    [aux_sym_end_if_statement_token1] = ACTIONS(1355),
    [aux_sym_elseif_clause_token2] = ACTIONS(1355),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1355),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token3] = ACTIONS(1355),
    [aux_sym_select_type_statement_token1] = ACTIONS(1355),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1355),
    [aux_sym_block_construct_token1] = ACTIONS(1355),
    [aux_sym_associate_statement_token1] = ACTIONS(1355),
    [aux_sym_format_statement_token1] = ACTIONS(1355),
    [aux_sym_print_statement_token1] = ACTIONS(1355),
    [aux_sym_open_statement_token1] = ACTIONS(1355),
    [aux_sym_close_statement_token1] = ACTIONS(1355),
    [aux_sym_inquire_statement_token1] = ACTIONS(1355),
    [aux_sym_enum_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token2] = ACTIONS(1355),
    [aux_sym_file_position_statement_token3] = ACTIONS(1355),
    [aux_sym_file_position_statement_token4] = ACTIONS(1355),
    [aux_sym_logical_expression_token5] = ACTIONS(1357),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_DOT] = ACTIONS(1355),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1357),
    [anon_sym_LBRACK] = ACTIONS(1357),
    [sym_boolean_literal] = ACTIONS(1357),
    [aux_sym_null_literal_token1] = ACTIONS(1355),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1355),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1357),
    [sym__float_literal] = ACTIONS(1357),
    [sym__boz_literal] = ACTIONS(1357),
    [sym_string_literal] = ACTIONS(1357),
  },
  [342] = {
    [sym_preproc_file_line] = STATE(342),
    [aux_sym_end_program_statement_token1] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [aux_sym_interface_statement_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1359),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1359),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1359),
    [aux_sym_contains_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token2] = ACTIONS(1359),
    [aux_sym_implicit_statement_token1] = ACTIONS(1359),
    [aux_sym_implicit_statement_token3] = ACTIONS(1359),
    [aux_sym_implicit_statement_token4] = ACTIONS(1359),
    [aux_sym_save_statement_token1] = ACTIONS(1359),
    [aux_sym_private_statement_token1] = ACTIONS(1359),
    [aux_sym_public_statement_token1] = ACTIONS(1359),
    [aux_sym_namelist_statement_token1] = ACTIONS(1359),
    [aux_sym_common_statement_token1] = ACTIONS(1359),
    [anon_sym_DASH] = ACTIONS(1361),
    [aux_sym_import_statement_token1] = ACTIONS(1359),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1359),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1359),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1359),
    [aux_sym_variable_attributes_token1] = ACTIONS(1359),
    [aux_sym_variable_attributes_token2] = ACTIONS(1359),
    [aux_sym_variable_attributes_token3] = ACTIONS(1359),
    [aux_sym_variable_attributes_token4] = ACTIONS(1359),
    [aux_sym_variable_attributes_token5] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1359),
    [aux_sym_derived_type_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token2] = ACTIONS(1359),
    [aux_sym_type_qualifier_token3] = ACTIONS(1359),
    [aux_sym_type_qualifier_token4] = ACTIONS(1359),
    [aux_sym_type_qualifier_token5] = ACTIONS(1359),
    [aux_sym_type_qualifier_token9] = ACTIONS(1359),
    [aux_sym_type_qualifier_token10] = ACTIONS(1359),
    [aux_sym_type_qualifier_token11] = ACTIONS(1359),
    [aux_sym_type_qualifier_token12] = ACTIONS(1359),
    [aux_sym_type_qualifier_token13] = ACTIONS(1359),
    [aux_sym_type_qualifier_token14] = ACTIONS(1359),
    [aux_sym_type_qualifier_token15] = ACTIONS(1359),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token2] = ACTIONS(1359),
    [aux_sym_subroutine_call_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token2] = ACTIONS(1359),
    [aux_sym_keyword_statement_token3] = ACTIONS(1359),
    [aux_sym_keyword_statement_token4] = ACTIONS(1359),
    [aux_sym_keyword_statement_token6] = ACTIONS(1359),
    [aux_sym_keyword_statement_token7] = ACTIONS(1359),
    [aux_sym_include_statement_token1] = ACTIONS(1359),
    [aux_sym_data_statement_token1] = ACTIONS(1359),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1359),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1359),
    [aux_sym_end_if_statement_token1] = ACTIONS(1359),
    [aux_sym_elseif_clause_token2] = ACTIONS(1359),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1359),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token3] = ACTIONS(1359),
    [aux_sym_select_type_statement_token1] = ACTIONS(1359),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1359),
    [aux_sym_block_construct_token1] = ACTIONS(1359),
    [aux_sym_associate_statement_token1] = ACTIONS(1359),
    [aux_sym_format_statement_token1] = ACTIONS(1359),
    [aux_sym_print_statement_token1] = ACTIONS(1359),
    [aux_sym_open_statement_token1] = ACTIONS(1359),
    [aux_sym_close_statement_token1] = ACTIONS(1359),
    [aux_sym_inquire_statement_token1] = ACTIONS(1359),
    [aux_sym_enum_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token2] = ACTIONS(1359),
    [aux_sym_file_position_statement_token3] = ACTIONS(1359),
    [aux_sym_file_position_statement_token4] = ACTIONS(1359),
    [aux_sym_logical_expression_token5] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(1359),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [sym_boolean_literal] = ACTIONS(1361),
    [aux_sym_null_literal_token1] = ACTIONS(1359),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1359),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1361),
    [sym__float_literal] = ACTIONS(1361),
    [sym__boz_literal] = ACTIONS(1361),
    [sym_string_literal] = ACTIONS(1361),
  },
  [343] = {
    [sym_preproc_file_line] = STATE(343),
    [aux_sym_end_program_statement_token1] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [aux_sym_interface_statement_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1359),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1359),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1359),
    [aux_sym_contains_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token2] = ACTIONS(1359),
    [aux_sym_implicit_statement_token1] = ACTIONS(1359),
    [aux_sym_implicit_statement_token3] = ACTIONS(1359),
    [aux_sym_implicit_statement_token4] = ACTIONS(1359),
    [aux_sym_save_statement_token1] = ACTIONS(1359),
    [aux_sym_private_statement_token1] = ACTIONS(1359),
    [aux_sym_public_statement_token1] = ACTIONS(1359),
    [aux_sym_namelist_statement_token1] = ACTIONS(1359),
    [aux_sym_common_statement_token1] = ACTIONS(1359),
    [anon_sym_DASH] = ACTIONS(1361),
    [aux_sym_import_statement_token1] = ACTIONS(1359),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1359),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1359),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1359),
    [aux_sym_variable_attributes_token1] = ACTIONS(1359),
    [aux_sym_variable_attributes_token2] = ACTIONS(1359),
    [aux_sym_variable_attributes_token3] = ACTIONS(1359),
    [aux_sym_variable_attributes_token4] = ACTIONS(1359),
    [aux_sym_variable_attributes_token5] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1359),
    [aux_sym_derived_type_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token2] = ACTIONS(1359),
    [aux_sym_type_qualifier_token3] = ACTIONS(1359),
    [aux_sym_type_qualifier_token4] = ACTIONS(1359),
    [aux_sym_type_qualifier_token5] = ACTIONS(1359),
    [aux_sym_type_qualifier_token9] = ACTIONS(1359),
    [aux_sym_type_qualifier_token10] = ACTIONS(1359),
    [aux_sym_type_qualifier_token11] = ACTIONS(1359),
    [aux_sym_type_qualifier_token12] = ACTIONS(1359),
    [aux_sym_type_qualifier_token13] = ACTIONS(1359),
    [aux_sym_type_qualifier_token14] = ACTIONS(1359),
    [aux_sym_type_qualifier_token15] = ACTIONS(1359),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token2] = ACTIONS(1359),
    [aux_sym_subroutine_call_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token2] = ACTIONS(1359),
    [aux_sym_keyword_statement_token3] = ACTIONS(1359),
    [aux_sym_keyword_statement_token4] = ACTIONS(1359),
    [aux_sym_keyword_statement_token6] = ACTIONS(1359),
    [aux_sym_keyword_statement_token7] = ACTIONS(1359),
    [aux_sym_include_statement_token1] = ACTIONS(1359),
    [aux_sym_data_statement_token1] = ACTIONS(1359),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1359),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1359),
    [aux_sym_end_if_statement_token1] = ACTIONS(1359),
    [aux_sym_elseif_clause_token2] = ACTIONS(1359),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1359),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token3] = ACTIONS(1359),
    [aux_sym_select_type_statement_token1] = ACTIONS(1359),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1359),
    [aux_sym_block_construct_token1] = ACTIONS(1359),
    [aux_sym_associate_statement_token1] = ACTIONS(1359),
    [aux_sym_format_statement_token1] = ACTIONS(1359),
    [aux_sym_print_statement_token1] = ACTIONS(1359),
    [aux_sym_open_statement_token1] = ACTIONS(1359),
    [aux_sym_close_statement_token1] = ACTIONS(1359),
    [aux_sym_inquire_statement_token1] = ACTIONS(1359),
    [aux_sym_enum_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token2] = ACTIONS(1359),
    [aux_sym_file_position_statement_token3] = ACTIONS(1359),
    [aux_sym_file_position_statement_token4] = ACTIONS(1359),
    [aux_sym_logical_expression_token5] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(1359),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [sym_boolean_literal] = ACTIONS(1361),
    [aux_sym_null_literal_token1] = ACTIONS(1359),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1359),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1361),
    [sym__float_literal] = ACTIONS(1361),
    [sym__boz_literal] = ACTIONS(1361),
    [sym_string_literal] = ACTIONS(1361),
  },
  [344] = {
    [sym_preproc_file_line] = STATE(344),
    [aux_sym_end_program_statement_token1] = ACTIONS(1259),
    [aux_sym_end_program_statement_token2] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1259),
    [aux_sym_interface_statement_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1259),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1259),
    [aux_sym_contains_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token2] = ACTIONS(1259),
    [aux_sym_implicit_statement_token1] = ACTIONS(1259),
    [aux_sym_implicit_statement_token3] = ACTIONS(1259),
    [aux_sym_implicit_statement_token4] = ACTIONS(1259),
    [aux_sym_save_statement_token1] = ACTIONS(1259),
    [aux_sym_private_statement_token1] = ACTIONS(1259),
    [aux_sym_public_statement_token1] = ACTIONS(1259),
    [aux_sym_namelist_statement_token1] = ACTIONS(1259),
    [aux_sym_common_statement_token1] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [aux_sym_import_statement_token1] = ACTIONS(1259),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1259),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1259),
    [aux_sym_variable_attributes_token1] = ACTIONS(1259),
    [aux_sym_variable_attributes_token2] = ACTIONS(1259),
    [aux_sym_variable_attributes_token3] = ACTIONS(1259),
    [aux_sym_variable_attributes_token4] = ACTIONS(1259),
    [aux_sym_variable_attributes_token5] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1259),
    [aux_sym_derived_type_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token2] = ACTIONS(1259),
    [aux_sym_type_qualifier_token3] = ACTIONS(1259),
    [aux_sym_type_qualifier_token4] = ACTIONS(1259),
    [aux_sym_type_qualifier_token5] = ACTIONS(1259),
    [aux_sym_type_qualifier_token9] = ACTIONS(1259),
    [aux_sym_type_qualifier_token10] = ACTIONS(1259),
    [aux_sym_type_qualifier_token11] = ACTIONS(1259),
    [aux_sym_type_qualifier_token12] = ACTIONS(1259),
    [aux_sym_type_qualifier_token13] = ACTIONS(1259),
    [aux_sym_type_qualifier_token14] = ACTIONS(1259),
    [aux_sym_type_qualifier_token15] = ACTIONS(1259),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token2] = ACTIONS(1259),
    [aux_sym_subroutine_call_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token2] = ACTIONS(1259),
    [aux_sym_keyword_statement_token3] = ACTIONS(1259),
    [aux_sym_keyword_statement_token4] = ACTIONS(1259),
    [aux_sym_keyword_statement_token6] = ACTIONS(1259),
    [aux_sym_keyword_statement_token7] = ACTIONS(1259),
    [aux_sym_include_statement_token1] = ACTIONS(1259),
    [aux_sym_data_statement_token1] = ACTIONS(1259),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1259),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1259),
    [aux_sym_end_if_statement_token1] = ACTIONS(1259),
    [aux_sym_elseif_clause_token2] = ACTIONS(1259),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1259),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token3] = ACTIONS(1259),
    [aux_sym_select_type_statement_token1] = ACTIONS(1259),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1259),
    [aux_sym_block_construct_token1] = ACTIONS(1259),
    [aux_sym_associate_statement_token1] = ACTIONS(1259),
    [aux_sym_format_statement_token1] = ACTIONS(1259),
    [aux_sym_print_statement_token1] = ACTIONS(1259),
    [aux_sym_open_statement_token1] = ACTIONS(1259),
    [aux_sym_close_statement_token1] = ACTIONS(1259),
    [aux_sym_inquire_statement_token1] = ACTIONS(1259),
    [aux_sym_enum_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token2] = ACTIONS(1259),
    [aux_sym_file_position_statement_token3] = ACTIONS(1259),
    [aux_sym_file_position_statement_token4] = ACTIONS(1259),
    [aux_sym_logical_expression_token5] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [sym_boolean_literal] = ACTIONS(1257),
    [aux_sym_null_literal_token1] = ACTIONS(1259),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1259),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1257),
    [sym__float_literal] = ACTIONS(1257),
    [sym__boz_literal] = ACTIONS(1257),
    [sym_string_literal] = ACTIONS(1257),
  },
  [345] = {
    [sym_preproc_file_line] = STATE(345),
    [aux_sym_end_program_statement_token1] = ACTIONS(1331),
    [anon_sym_LPAREN] = ACTIONS(1331),
    [aux_sym_interface_statement_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1331),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1331),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1331),
    [aux_sym_contains_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token2] = ACTIONS(1331),
    [aux_sym_implicit_statement_token1] = ACTIONS(1331),
    [aux_sym_implicit_statement_token3] = ACTIONS(1331),
    [aux_sym_implicit_statement_token4] = ACTIONS(1331),
    [aux_sym_save_statement_token1] = ACTIONS(1331),
    [aux_sym_private_statement_token1] = ACTIONS(1331),
    [aux_sym_public_statement_token1] = ACTIONS(1331),
    [aux_sym_namelist_statement_token1] = ACTIONS(1331),
    [aux_sym_common_statement_token1] = ACTIONS(1331),
    [anon_sym_DASH] = ACTIONS(1333),
    [aux_sym_import_statement_token1] = ACTIONS(1331),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1331),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1331),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1331),
    [aux_sym_variable_attributes_token1] = ACTIONS(1331),
    [aux_sym_variable_attributes_token2] = ACTIONS(1331),
    [aux_sym_variable_attributes_token3] = ACTIONS(1331),
    [aux_sym_variable_attributes_token4] = ACTIONS(1331),
    [aux_sym_variable_attributes_token5] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1331),
    [aux_sym_derived_type_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token2] = ACTIONS(1331),
    [aux_sym_type_qualifier_token3] = ACTIONS(1331),
    [aux_sym_type_qualifier_token4] = ACTIONS(1331),
    [aux_sym_type_qualifier_token5] = ACTIONS(1331),
    [aux_sym_type_qualifier_token9] = ACTIONS(1331),
    [aux_sym_type_qualifier_token10] = ACTIONS(1331),
    [aux_sym_type_qualifier_token11] = ACTIONS(1331),
    [aux_sym_type_qualifier_token12] = ACTIONS(1331),
    [aux_sym_type_qualifier_token13] = ACTIONS(1331),
    [aux_sym_type_qualifier_token14] = ACTIONS(1331),
    [aux_sym_type_qualifier_token15] = ACTIONS(1331),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token2] = ACTIONS(1331),
    [aux_sym_subroutine_call_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token2] = ACTIONS(1331),
    [aux_sym_keyword_statement_token3] = ACTIONS(1331),
    [aux_sym_keyword_statement_token4] = ACTIONS(1331),
    [aux_sym_keyword_statement_token6] = ACTIONS(1331),
    [aux_sym_keyword_statement_token7] = ACTIONS(1331),
    [aux_sym_include_statement_token1] = ACTIONS(1331),
    [aux_sym_data_statement_token1] = ACTIONS(1331),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1331),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1331),
    [aux_sym_end_if_statement_token1] = ACTIONS(1331),
    [aux_sym_elseif_clause_token2] = ACTIONS(1331),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1331),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token3] = ACTIONS(1331),
    [aux_sym_select_type_statement_token1] = ACTIONS(1331),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1331),
    [aux_sym_block_construct_token1] = ACTIONS(1331),
    [aux_sym_associate_statement_token1] = ACTIONS(1331),
    [aux_sym_format_statement_token1] = ACTIONS(1331),
    [aux_sym_print_statement_token1] = ACTIONS(1331),
    [aux_sym_open_statement_token1] = ACTIONS(1331),
    [aux_sym_close_statement_token1] = ACTIONS(1331),
    [aux_sym_inquire_statement_token1] = ACTIONS(1331),
    [aux_sym_enum_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token2] = ACTIONS(1331),
    [aux_sym_file_position_statement_token3] = ACTIONS(1331),
    [aux_sym_file_position_statement_token4] = ACTIONS(1331),
    [aux_sym_logical_expression_token5] = ACTIONS(1333),
    [anon_sym_PLUS] = ACTIONS(1333),
    [anon_sym_DOT] = ACTIONS(1331),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1333),
    [anon_sym_LBRACK] = ACTIONS(1333),
    [sym_boolean_literal] = ACTIONS(1333),
    [aux_sym_null_literal_token1] = ACTIONS(1331),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1331),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1333),
    [sym__float_literal] = ACTIONS(1333),
    [sym__boz_literal] = ACTIONS(1333),
    [sym_string_literal] = ACTIONS(1333),
  },
  [346] = {
    [sym_preproc_file_line] = STATE(346),
    [aux_sym_end_program_statement_token1] = ACTIONS(1327),
    [anon_sym_LPAREN] = ACTIONS(1327),
    [aux_sym_interface_statement_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1327),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1327),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1327),
    [aux_sym_contains_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token2] = ACTIONS(1327),
    [aux_sym_implicit_statement_token1] = ACTIONS(1327),
    [aux_sym_implicit_statement_token3] = ACTIONS(1327),
    [aux_sym_implicit_statement_token4] = ACTIONS(1327),
    [aux_sym_save_statement_token1] = ACTIONS(1327),
    [aux_sym_private_statement_token1] = ACTIONS(1327),
    [aux_sym_public_statement_token1] = ACTIONS(1327),
    [aux_sym_namelist_statement_token1] = ACTIONS(1327),
    [aux_sym_common_statement_token1] = ACTIONS(1327),
    [anon_sym_DASH] = ACTIONS(1329),
    [aux_sym_import_statement_token1] = ACTIONS(1327),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1327),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1327),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1327),
    [aux_sym_variable_attributes_token1] = ACTIONS(1327),
    [aux_sym_variable_attributes_token2] = ACTIONS(1327),
    [aux_sym_variable_attributes_token3] = ACTIONS(1327),
    [aux_sym_variable_attributes_token4] = ACTIONS(1327),
    [aux_sym_variable_attributes_token5] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1327),
    [aux_sym_derived_type_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token2] = ACTIONS(1327),
    [aux_sym_type_qualifier_token3] = ACTIONS(1327),
    [aux_sym_type_qualifier_token4] = ACTIONS(1327),
    [aux_sym_type_qualifier_token5] = ACTIONS(1327),
    [aux_sym_type_qualifier_token9] = ACTIONS(1327),
    [aux_sym_type_qualifier_token10] = ACTIONS(1327),
    [aux_sym_type_qualifier_token11] = ACTIONS(1327),
    [aux_sym_type_qualifier_token12] = ACTIONS(1327),
    [aux_sym_type_qualifier_token13] = ACTIONS(1327),
    [aux_sym_type_qualifier_token14] = ACTIONS(1327),
    [aux_sym_type_qualifier_token15] = ACTIONS(1327),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token2] = ACTIONS(1327),
    [aux_sym_subroutine_call_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token2] = ACTIONS(1327),
    [aux_sym_keyword_statement_token3] = ACTIONS(1327),
    [aux_sym_keyword_statement_token4] = ACTIONS(1327),
    [aux_sym_keyword_statement_token6] = ACTIONS(1327),
    [aux_sym_keyword_statement_token7] = ACTIONS(1327),
    [aux_sym_include_statement_token1] = ACTIONS(1327),
    [aux_sym_data_statement_token1] = ACTIONS(1327),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1327),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1327),
    [aux_sym_end_if_statement_token1] = ACTIONS(1327),
    [aux_sym_elseif_clause_token2] = ACTIONS(1327),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1327),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token3] = ACTIONS(1327),
    [aux_sym_select_type_statement_token1] = ACTIONS(1327),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1327),
    [aux_sym_block_construct_token1] = ACTIONS(1327),
    [aux_sym_associate_statement_token1] = ACTIONS(1327),
    [aux_sym_format_statement_token1] = ACTIONS(1327),
    [aux_sym_print_statement_token1] = ACTIONS(1327),
    [aux_sym_open_statement_token1] = ACTIONS(1327),
    [aux_sym_close_statement_token1] = ACTIONS(1327),
    [aux_sym_inquire_statement_token1] = ACTIONS(1327),
    [aux_sym_enum_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token2] = ACTIONS(1327),
    [aux_sym_file_position_statement_token3] = ACTIONS(1327),
    [aux_sym_file_position_statement_token4] = ACTIONS(1327),
    [aux_sym_logical_expression_token5] = ACTIONS(1329),
    [anon_sym_PLUS] = ACTIONS(1329),
    [anon_sym_DOT] = ACTIONS(1327),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1329),
    [anon_sym_LBRACK] = ACTIONS(1329),
    [sym_boolean_literal] = ACTIONS(1329),
    [aux_sym_null_literal_token1] = ACTIONS(1327),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1327),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1329),
    [sym__float_literal] = ACTIONS(1329),
    [sym__boz_literal] = ACTIONS(1329),
    [sym_string_literal] = ACTIONS(1329),
  },
  [347] = {
    [sym_preproc_file_line] = STATE(347),
    [aux_sym_end_program_statement_token1] = ACTIONS(1331),
    [anon_sym_LPAREN] = ACTIONS(1331),
    [aux_sym_interface_statement_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1331),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1331),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1331),
    [aux_sym_contains_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token2] = ACTIONS(1331),
    [aux_sym_implicit_statement_token1] = ACTIONS(1331),
    [aux_sym_implicit_statement_token3] = ACTIONS(1331),
    [aux_sym_implicit_statement_token4] = ACTIONS(1331),
    [aux_sym_save_statement_token1] = ACTIONS(1331),
    [aux_sym_private_statement_token1] = ACTIONS(1331),
    [aux_sym_public_statement_token1] = ACTIONS(1331),
    [aux_sym_namelist_statement_token1] = ACTIONS(1331),
    [aux_sym_common_statement_token1] = ACTIONS(1331),
    [anon_sym_DASH] = ACTIONS(1333),
    [aux_sym_import_statement_token1] = ACTIONS(1331),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1331),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1331),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1331),
    [aux_sym_variable_attributes_token1] = ACTIONS(1331),
    [aux_sym_variable_attributes_token2] = ACTIONS(1331),
    [aux_sym_variable_attributes_token3] = ACTIONS(1331),
    [aux_sym_variable_attributes_token4] = ACTIONS(1331),
    [aux_sym_variable_attributes_token5] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1331),
    [aux_sym_derived_type_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token2] = ACTIONS(1331),
    [aux_sym_type_qualifier_token3] = ACTIONS(1331),
    [aux_sym_type_qualifier_token4] = ACTIONS(1331),
    [aux_sym_type_qualifier_token5] = ACTIONS(1331),
    [aux_sym_type_qualifier_token9] = ACTIONS(1331),
    [aux_sym_type_qualifier_token10] = ACTIONS(1331),
    [aux_sym_type_qualifier_token11] = ACTIONS(1331),
    [aux_sym_type_qualifier_token12] = ACTIONS(1331),
    [aux_sym_type_qualifier_token13] = ACTIONS(1331),
    [aux_sym_type_qualifier_token14] = ACTIONS(1331),
    [aux_sym_type_qualifier_token15] = ACTIONS(1331),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token2] = ACTIONS(1331),
    [aux_sym_subroutine_call_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token2] = ACTIONS(1331),
    [aux_sym_keyword_statement_token3] = ACTIONS(1331),
    [aux_sym_keyword_statement_token4] = ACTIONS(1331),
    [aux_sym_keyword_statement_token6] = ACTIONS(1331),
    [aux_sym_keyword_statement_token7] = ACTIONS(1331),
    [aux_sym_include_statement_token1] = ACTIONS(1331),
    [aux_sym_data_statement_token1] = ACTIONS(1331),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1331),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1331),
    [aux_sym_end_if_statement_token1] = ACTIONS(1331),
    [aux_sym_elseif_clause_token2] = ACTIONS(1331),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1331),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token3] = ACTIONS(1331),
    [aux_sym_select_type_statement_token1] = ACTIONS(1331),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1331),
    [aux_sym_block_construct_token1] = ACTIONS(1331),
    [aux_sym_associate_statement_token1] = ACTIONS(1331),
    [aux_sym_format_statement_token1] = ACTIONS(1331),
    [aux_sym_print_statement_token1] = ACTIONS(1331),
    [aux_sym_open_statement_token1] = ACTIONS(1331),
    [aux_sym_close_statement_token1] = ACTIONS(1331),
    [aux_sym_inquire_statement_token1] = ACTIONS(1331),
    [aux_sym_enum_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token2] = ACTIONS(1331),
    [aux_sym_file_position_statement_token3] = ACTIONS(1331),
    [aux_sym_file_position_statement_token4] = ACTIONS(1331),
    [aux_sym_logical_expression_token5] = ACTIONS(1333),
    [anon_sym_PLUS] = ACTIONS(1333),
    [anon_sym_DOT] = ACTIONS(1331),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1333),
    [anon_sym_LBRACK] = ACTIONS(1333),
    [sym_boolean_literal] = ACTIONS(1333),
    [aux_sym_null_literal_token1] = ACTIONS(1331),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1331),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1333),
    [sym__float_literal] = ACTIONS(1333),
    [sym__boz_literal] = ACTIONS(1333),
    [sym_string_literal] = ACTIONS(1333),
  },
  [348] = {
    [sym_preproc_file_line] = STATE(348),
    [aux_sym_end_program_statement_token1] = ACTIONS(1327),
    [anon_sym_LPAREN] = ACTIONS(1327),
    [aux_sym_interface_statement_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1327),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1327),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1327),
    [aux_sym_contains_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token2] = ACTIONS(1327),
    [aux_sym_implicit_statement_token1] = ACTIONS(1327),
    [aux_sym_implicit_statement_token3] = ACTIONS(1327),
    [aux_sym_implicit_statement_token4] = ACTIONS(1327),
    [aux_sym_save_statement_token1] = ACTIONS(1327),
    [aux_sym_private_statement_token1] = ACTIONS(1327),
    [aux_sym_public_statement_token1] = ACTIONS(1327),
    [aux_sym_namelist_statement_token1] = ACTIONS(1327),
    [aux_sym_common_statement_token1] = ACTIONS(1327),
    [anon_sym_DASH] = ACTIONS(1329),
    [aux_sym_import_statement_token1] = ACTIONS(1327),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1327),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1327),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1327),
    [aux_sym_variable_attributes_token1] = ACTIONS(1327),
    [aux_sym_variable_attributes_token2] = ACTIONS(1327),
    [aux_sym_variable_attributes_token3] = ACTIONS(1327),
    [aux_sym_variable_attributes_token4] = ACTIONS(1327),
    [aux_sym_variable_attributes_token5] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1327),
    [aux_sym_derived_type_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token2] = ACTIONS(1327),
    [aux_sym_type_qualifier_token3] = ACTIONS(1327),
    [aux_sym_type_qualifier_token4] = ACTIONS(1327),
    [aux_sym_type_qualifier_token5] = ACTIONS(1327),
    [aux_sym_type_qualifier_token9] = ACTIONS(1327),
    [aux_sym_type_qualifier_token10] = ACTIONS(1327),
    [aux_sym_type_qualifier_token11] = ACTIONS(1327),
    [aux_sym_type_qualifier_token12] = ACTIONS(1327),
    [aux_sym_type_qualifier_token13] = ACTIONS(1327),
    [aux_sym_type_qualifier_token14] = ACTIONS(1327),
    [aux_sym_type_qualifier_token15] = ACTIONS(1327),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token2] = ACTIONS(1327),
    [aux_sym_subroutine_call_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token2] = ACTIONS(1327),
    [aux_sym_keyword_statement_token3] = ACTIONS(1327),
    [aux_sym_keyword_statement_token4] = ACTIONS(1327),
    [aux_sym_keyword_statement_token6] = ACTIONS(1327),
    [aux_sym_keyword_statement_token7] = ACTIONS(1327),
    [aux_sym_include_statement_token1] = ACTIONS(1327),
    [aux_sym_data_statement_token1] = ACTIONS(1327),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1327),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1327),
    [aux_sym_end_if_statement_token1] = ACTIONS(1327),
    [aux_sym_elseif_clause_token2] = ACTIONS(1327),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1327),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token3] = ACTIONS(1327),
    [aux_sym_select_type_statement_token1] = ACTIONS(1327),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1327),
    [aux_sym_block_construct_token1] = ACTIONS(1327),
    [aux_sym_associate_statement_token1] = ACTIONS(1327),
    [aux_sym_format_statement_token1] = ACTIONS(1327),
    [aux_sym_print_statement_token1] = ACTIONS(1327),
    [aux_sym_open_statement_token1] = ACTIONS(1327),
    [aux_sym_close_statement_token1] = ACTIONS(1327),
    [aux_sym_inquire_statement_token1] = ACTIONS(1327),
    [aux_sym_enum_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token2] = ACTIONS(1327),
    [aux_sym_file_position_statement_token3] = ACTIONS(1327),
    [aux_sym_file_position_statement_token4] = ACTIONS(1327),
    [aux_sym_logical_expression_token5] = ACTIONS(1329),
    [anon_sym_PLUS] = ACTIONS(1329),
    [anon_sym_DOT] = ACTIONS(1327),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1329),
    [anon_sym_LBRACK] = ACTIONS(1329),
    [sym_boolean_literal] = ACTIONS(1329),
    [aux_sym_null_literal_token1] = ACTIONS(1327),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1327),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1329),
    [sym__float_literal] = ACTIONS(1329),
    [sym__boz_literal] = ACTIONS(1329),
    [sym_string_literal] = ACTIONS(1329),
  },
  [349] = {
    [sym_preproc_file_line] = STATE(349),
    [aux_sym_end_program_statement_token1] = ACTIONS(1319),
    [anon_sym_LPAREN] = ACTIONS(1319),
    [aux_sym_interface_statement_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1319),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1319),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1319),
    [aux_sym_contains_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token2] = ACTIONS(1319),
    [aux_sym_implicit_statement_token1] = ACTIONS(1319),
    [aux_sym_implicit_statement_token3] = ACTIONS(1319),
    [aux_sym_implicit_statement_token4] = ACTIONS(1319),
    [aux_sym_save_statement_token1] = ACTIONS(1319),
    [aux_sym_private_statement_token1] = ACTIONS(1319),
    [aux_sym_public_statement_token1] = ACTIONS(1319),
    [aux_sym_namelist_statement_token1] = ACTIONS(1319),
    [aux_sym_common_statement_token1] = ACTIONS(1319),
    [anon_sym_DASH] = ACTIONS(1321),
    [aux_sym_import_statement_token1] = ACTIONS(1319),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1319),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1319),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1319),
    [aux_sym_variable_attributes_token1] = ACTIONS(1319),
    [aux_sym_variable_attributes_token2] = ACTIONS(1319),
    [aux_sym_variable_attributes_token3] = ACTIONS(1319),
    [aux_sym_variable_attributes_token4] = ACTIONS(1319),
    [aux_sym_variable_attributes_token5] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1319),
    [aux_sym_derived_type_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token2] = ACTIONS(1319),
    [aux_sym_type_qualifier_token3] = ACTIONS(1319),
    [aux_sym_type_qualifier_token4] = ACTIONS(1319),
    [aux_sym_type_qualifier_token5] = ACTIONS(1319),
    [aux_sym_type_qualifier_token9] = ACTIONS(1319),
    [aux_sym_type_qualifier_token10] = ACTIONS(1319),
    [aux_sym_type_qualifier_token11] = ACTIONS(1319),
    [aux_sym_type_qualifier_token12] = ACTIONS(1319),
    [aux_sym_type_qualifier_token13] = ACTIONS(1319),
    [aux_sym_type_qualifier_token14] = ACTIONS(1319),
    [aux_sym_type_qualifier_token15] = ACTIONS(1319),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token2] = ACTIONS(1319),
    [aux_sym_subroutine_call_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token2] = ACTIONS(1319),
    [aux_sym_keyword_statement_token3] = ACTIONS(1319),
    [aux_sym_keyword_statement_token4] = ACTIONS(1319),
    [aux_sym_keyword_statement_token6] = ACTIONS(1319),
    [aux_sym_keyword_statement_token7] = ACTIONS(1319),
    [aux_sym_include_statement_token1] = ACTIONS(1319),
    [aux_sym_data_statement_token1] = ACTIONS(1319),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1319),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1319),
    [aux_sym_end_if_statement_token1] = ACTIONS(1319),
    [aux_sym_elseif_clause_token2] = ACTIONS(1319),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1319),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token3] = ACTIONS(1319),
    [aux_sym_select_type_statement_token1] = ACTIONS(1319),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1319),
    [aux_sym_block_construct_token1] = ACTIONS(1319),
    [aux_sym_associate_statement_token1] = ACTIONS(1319),
    [aux_sym_format_statement_token1] = ACTIONS(1319),
    [aux_sym_print_statement_token1] = ACTIONS(1319),
    [aux_sym_open_statement_token1] = ACTIONS(1319),
    [aux_sym_close_statement_token1] = ACTIONS(1319),
    [aux_sym_inquire_statement_token1] = ACTIONS(1319),
    [aux_sym_enum_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token2] = ACTIONS(1319),
    [aux_sym_file_position_statement_token3] = ACTIONS(1319),
    [aux_sym_file_position_statement_token4] = ACTIONS(1319),
    [aux_sym_logical_expression_token5] = ACTIONS(1321),
    [anon_sym_PLUS] = ACTIONS(1321),
    [anon_sym_DOT] = ACTIONS(1319),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1321),
    [anon_sym_LBRACK] = ACTIONS(1321),
    [sym_boolean_literal] = ACTIONS(1321),
    [aux_sym_null_literal_token1] = ACTIONS(1319),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1319),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1321),
    [sym__float_literal] = ACTIONS(1321),
    [sym__boz_literal] = ACTIONS(1321),
    [sym_string_literal] = ACTIONS(1321),
  },
  [350] = {
    [sym_preproc_file_line] = STATE(350),
    [aux_sym_end_program_statement_token1] = ACTIONS(1295),
    [aux_sym_end_program_statement_token2] = ACTIONS(1295),
    [anon_sym_LPAREN] = ACTIONS(1295),
    [aux_sym_interface_statement_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1295),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1295),
    [aux_sym_contains_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token2] = ACTIONS(1295),
    [aux_sym_implicit_statement_token1] = ACTIONS(1295),
    [aux_sym_implicit_statement_token3] = ACTIONS(1295),
    [aux_sym_implicit_statement_token4] = ACTIONS(1295),
    [aux_sym_save_statement_token1] = ACTIONS(1295),
    [aux_sym_private_statement_token1] = ACTIONS(1295),
    [aux_sym_public_statement_token1] = ACTIONS(1295),
    [aux_sym_namelist_statement_token1] = ACTIONS(1295),
    [aux_sym_common_statement_token1] = ACTIONS(1295),
    [anon_sym_DASH] = ACTIONS(1297),
    [aux_sym_import_statement_token1] = ACTIONS(1295),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1295),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1295),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1295),
    [aux_sym_variable_attributes_token1] = ACTIONS(1295),
    [aux_sym_variable_attributes_token2] = ACTIONS(1295),
    [aux_sym_variable_attributes_token3] = ACTIONS(1295),
    [aux_sym_variable_attributes_token4] = ACTIONS(1295),
    [aux_sym_variable_attributes_token5] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1295),
    [aux_sym_derived_type_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token2] = ACTIONS(1295),
    [aux_sym_type_qualifier_token3] = ACTIONS(1295),
    [aux_sym_type_qualifier_token4] = ACTIONS(1295),
    [aux_sym_type_qualifier_token5] = ACTIONS(1295),
    [aux_sym_type_qualifier_token9] = ACTIONS(1295),
    [aux_sym_type_qualifier_token10] = ACTIONS(1295),
    [aux_sym_type_qualifier_token11] = ACTIONS(1295),
    [aux_sym_type_qualifier_token12] = ACTIONS(1295),
    [aux_sym_type_qualifier_token13] = ACTIONS(1295),
    [aux_sym_type_qualifier_token14] = ACTIONS(1295),
    [aux_sym_type_qualifier_token15] = ACTIONS(1295),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token2] = ACTIONS(1295),
    [aux_sym_subroutine_call_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token2] = ACTIONS(1295),
    [aux_sym_keyword_statement_token3] = ACTIONS(1295),
    [aux_sym_keyword_statement_token4] = ACTIONS(1295),
    [aux_sym_keyword_statement_token6] = ACTIONS(1295),
    [aux_sym_keyword_statement_token7] = ACTIONS(1295),
    [aux_sym_include_statement_token1] = ACTIONS(1295),
    [aux_sym_data_statement_token1] = ACTIONS(1295),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1295),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1295),
    [aux_sym_end_if_statement_token1] = ACTIONS(1295),
    [aux_sym_elseif_clause_token2] = ACTIONS(1295),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1295),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token3] = ACTIONS(1295),
    [aux_sym_select_type_statement_token1] = ACTIONS(1295),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1295),
    [aux_sym_block_construct_token1] = ACTIONS(1295),
    [aux_sym_associate_statement_token1] = ACTIONS(1295),
    [aux_sym_format_statement_token1] = ACTIONS(1295),
    [aux_sym_print_statement_token1] = ACTIONS(1295),
    [aux_sym_open_statement_token1] = ACTIONS(1295),
    [aux_sym_close_statement_token1] = ACTIONS(1295),
    [aux_sym_inquire_statement_token1] = ACTIONS(1295),
    [aux_sym_enum_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token2] = ACTIONS(1295),
    [aux_sym_file_position_statement_token3] = ACTIONS(1295),
    [aux_sym_file_position_statement_token4] = ACTIONS(1295),
    [aux_sym_logical_expression_token5] = ACTIONS(1297),
    [anon_sym_PLUS] = ACTIONS(1297),
    [anon_sym_DOT] = ACTIONS(1295),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1297),
    [anon_sym_LBRACK] = ACTIONS(1297),
    [sym_boolean_literal] = ACTIONS(1297),
    [aux_sym_null_literal_token1] = ACTIONS(1295),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1295),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1297),
    [sym__float_literal] = ACTIONS(1297),
    [sym__boz_literal] = ACTIONS(1297),
    [sym_string_literal] = ACTIONS(1297),
  },
  [351] = {
    [sym_preproc_file_line] = STATE(351),
    [aux_sym_end_program_statement_token1] = ACTIONS(1307),
    [anon_sym_LPAREN] = ACTIONS(1307),
    [aux_sym_interface_statement_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1307),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1307),
    [aux_sym_end_function_statement_token1] = ACTIONS(1307),
    [aux_sym_contains_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token2] = ACTIONS(1307),
    [aux_sym_implicit_statement_token1] = ACTIONS(1307),
    [aux_sym_implicit_statement_token3] = ACTIONS(1307),
    [aux_sym_implicit_statement_token4] = ACTIONS(1307),
    [aux_sym_save_statement_token1] = ACTIONS(1307),
    [aux_sym_private_statement_token1] = ACTIONS(1307),
    [aux_sym_public_statement_token1] = ACTIONS(1307),
    [aux_sym_namelist_statement_token1] = ACTIONS(1307),
    [aux_sym_common_statement_token1] = ACTIONS(1307),
    [anon_sym_DASH] = ACTIONS(1309),
    [aux_sym_import_statement_token1] = ACTIONS(1307),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1307),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1307),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1307),
    [aux_sym_variable_attributes_token1] = ACTIONS(1307),
    [aux_sym_variable_attributes_token2] = ACTIONS(1307),
    [aux_sym_variable_attributes_token3] = ACTIONS(1307),
    [aux_sym_variable_attributes_token4] = ACTIONS(1307),
    [aux_sym_variable_attributes_token5] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1307),
    [aux_sym_derived_type_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token2] = ACTIONS(1307),
    [aux_sym_type_qualifier_token3] = ACTIONS(1307),
    [aux_sym_type_qualifier_token4] = ACTIONS(1307),
    [aux_sym_type_qualifier_token5] = ACTIONS(1307),
    [aux_sym_type_qualifier_token9] = ACTIONS(1307),
    [aux_sym_type_qualifier_token10] = ACTIONS(1307),
    [aux_sym_type_qualifier_token11] = ACTIONS(1307),
    [aux_sym_type_qualifier_token12] = ACTIONS(1307),
    [aux_sym_type_qualifier_token13] = ACTIONS(1307),
    [aux_sym_type_qualifier_token14] = ACTIONS(1307),
    [aux_sym_type_qualifier_token15] = ACTIONS(1307),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token2] = ACTIONS(1307),
    [aux_sym_subroutine_call_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token2] = ACTIONS(1307),
    [aux_sym_keyword_statement_token3] = ACTIONS(1307),
    [aux_sym_keyword_statement_token4] = ACTIONS(1307),
    [aux_sym_keyword_statement_token6] = ACTIONS(1307),
    [aux_sym_keyword_statement_token7] = ACTIONS(1307),
    [aux_sym_include_statement_token1] = ACTIONS(1307),
    [aux_sym_data_statement_token1] = ACTIONS(1307),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1307),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1307),
    [aux_sym_end_if_statement_token1] = ACTIONS(1307),
    [aux_sym_elseif_clause_token2] = ACTIONS(1307),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1307),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token3] = ACTIONS(1307),
    [aux_sym_select_type_statement_token1] = ACTIONS(1307),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1307),
    [aux_sym_block_construct_token1] = ACTIONS(1307),
    [aux_sym_associate_statement_token1] = ACTIONS(1307),
    [aux_sym_format_statement_token1] = ACTIONS(1307),
    [aux_sym_print_statement_token1] = ACTIONS(1307),
    [aux_sym_open_statement_token1] = ACTIONS(1307),
    [aux_sym_close_statement_token1] = ACTIONS(1307),
    [aux_sym_inquire_statement_token1] = ACTIONS(1307),
    [aux_sym_enum_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token2] = ACTIONS(1307),
    [aux_sym_file_position_statement_token3] = ACTIONS(1307),
    [aux_sym_file_position_statement_token4] = ACTIONS(1307),
    [aux_sym_logical_expression_token5] = ACTIONS(1309),
    [anon_sym_PLUS] = ACTIONS(1309),
    [anon_sym_DOT] = ACTIONS(1307),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1309),
    [anon_sym_LBRACK] = ACTIONS(1309),
    [sym_boolean_literal] = ACTIONS(1309),
    [aux_sym_null_literal_token1] = ACTIONS(1307),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1307),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1309),
    [sym__float_literal] = ACTIONS(1309),
    [sym__boz_literal] = ACTIONS(1309),
    [sym_string_literal] = ACTIONS(1309),
  },
  [352] = {
    [sym_preproc_file_line] = STATE(352),
    [aux_sym_end_program_statement_token1] = ACTIONS(1311),
    [anon_sym_LPAREN] = ACTIONS(1311),
    [aux_sym_interface_statement_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1311),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1311),
    [aux_sym_end_function_statement_token1] = ACTIONS(1311),
    [aux_sym_contains_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token2] = ACTIONS(1311),
    [aux_sym_implicit_statement_token1] = ACTIONS(1311),
    [aux_sym_implicit_statement_token3] = ACTIONS(1311),
    [aux_sym_implicit_statement_token4] = ACTIONS(1311),
    [aux_sym_save_statement_token1] = ACTIONS(1311),
    [aux_sym_private_statement_token1] = ACTIONS(1311),
    [aux_sym_public_statement_token1] = ACTIONS(1311),
    [aux_sym_namelist_statement_token1] = ACTIONS(1311),
    [aux_sym_common_statement_token1] = ACTIONS(1311),
    [anon_sym_DASH] = ACTIONS(1313),
    [aux_sym_import_statement_token1] = ACTIONS(1311),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1311),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1311),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1311),
    [aux_sym_variable_attributes_token1] = ACTIONS(1311),
    [aux_sym_variable_attributes_token2] = ACTIONS(1311),
    [aux_sym_variable_attributes_token3] = ACTIONS(1311),
    [aux_sym_variable_attributes_token4] = ACTIONS(1311),
    [aux_sym_variable_attributes_token5] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1311),
    [aux_sym_derived_type_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token2] = ACTIONS(1311),
    [aux_sym_type_qualifier_token3] = ACTIONS(1311),
    [aux_sym_type_qualifier_token4] = ACTIONS(1311),
    [aux_sym_type_qualifier_token5] = ACTIONS(1311),
    [aux_sym_type_qualifier_token9] = ACTIONS(1311),
    [aux_sym_type_qualifier_token10] = ACTIONS(1311),
    [aux_sym_type_qualifier_token11] = ACTIONS(1311),
    [aux_sym_type_qualifier_token12] = ACTIONS(1311),
    [aux_sym_type_qualifier_token13] = ACTIONS(1311),
    [aux_sym_type_qualifier_token14] = ACTIONS(1311),
    [aux_sym_type_qualifier_token15] = ACTIONS(1311),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token2] = ACTIONS(1311),
    [aux_sym_subroutine_call_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token2] = ACTIONS(1311),
    [aux_sym_keyword_statement_token3] = ACTIONS(1311),
    [aux_sym_keyword_statement_token4] = ACTIONS(1311),
    [aux_sym_keyword_statement_token6] = ACTIONS(1311),
    [aux_sym_keyword_statement_token7] = ACTIONS(1311),
    [aux_sym_include_statement_token1] = ACTIONS(1311),
    [aux_sym_data_statement_token1] = ACTIONS(1311),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1311),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1311),
    [aux_sym_end_if_statement_token1] = ACTIONS(1311),
    [aux_sym_elseif_clause_token2] = ACTIONS(1311),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1311),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token3] = ACTIONS(1311),
    [aux_sym_select_type_statement_token1] = ACTIONS(1311),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1311),
    [aux_sym_block_construct_token1] = ACTIONS(1311),
    [aux_sym_associate_statement_token1] = ACTIONS(1311),
    [aux_sym_format_statement_token1] = ACTIONS(1311),
    [aux_sym_print_statement_token1] = ACTIONS(1311),
    [aux_sym_open_statement_token1] = ACTIONS(1311),
    [aux_sym_close_statement_token1] = ACTIONS(1311),
    [aux_sym_inquire_statement_token1] = ACTIONS(1311),
    [aux_sym_enum_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token2] = ACTIONS(1311),
    [aux_sym_file_position_statement_token3] = ACTIONS(1311),
    [aux_sym_file_position_statement_token4] = ACTIONS(1311),
    [aux_sym_logical_expression_token5] = ACTIONS(1313),
    [anon_sym_PLUS] = ACTIONS(1313),
    [anon_sym_DOT] = ACTIONS(1311),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1313),
    [anon_sym_LBRACK] = ACTIONS(1313),
    [sym_boolean_literal] = ACTIONS(1313),
    [aux_sym_null_literal_token1] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1311),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1313),
    [sym__float_literal] = ACTIONS(1313),
    [sym__boz_literal] = ACTIONS(1313),
    [sym_string_literal] = ACTIONS(1313),
  },
  [353] = {
    [sym_preproc_file_line] = STATE(353),
    [aux_sym_end_program_statement_token1] = ACTIONS(1199),
    [anon_sym_LPAREN] = ACTIONS(1199),
    [aux_sym_interface_statement_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1199),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1199),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1199),
    [aux_sym_contains_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token2] = ACTIONS(1199),
    [aux_sym_implicit_statement_token1] = ACTIONS(1199),
    [aux_sym_implicit_statement_token3] = ACTIONS(1199),
    [aux_sym_implicit_statement_token4] = ACTIONS(1199),
    [aux_sym_save_statement_token1] = ACTIONS(1199),
    [aux_sym_private_statement_token1] = ACTIONS(1199),
    [aux_sym_public_statement_token1] = ACTIONS(1199),
    [aux_sym_namelist_statement_token1] = ACTIONS(1199),
    [aux_sym_common_statement_token1] = ACTIONS(1199),
    [anon_sym_DASH] = ACTIONS(1197),
    [aux_sym_import_statement_token1] = ACTIONS(1199),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1199),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1199),
    [aux_sym_variable_attributes_token1] = ACTIONS(1199),
    [aux_sym_variable_attributes_token2] = ACTIONS(1199),
    [aux_sym_variable_attributes_token3] = ACTIONS(1199),
    [aux_sym_variable_attributes_token4] = ACTIONS(1199),
    [aux_sym_variable_attributes_token5] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1199),
    [aux_sym_derived_type_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token2] = ACTIONS(1199),
    [aux_sym_type_qualifier_token3] = ACTIONS(1199),
    [aux_sym_type_qualifier_token4] = ACTIONS(1199),
    [aux_sym_type_qualifier_token5] = ACTIONS(1199),
    [aux_sym_type_qualifier_token9] = ACTIONS(1199),
    [aux_sym_type_qualifier_token10] = ACTIONS(1199),
    [aux_sym_type_qualifier_token11] = ACTIONS(1199),
    [aux_sym_type_qualifier_token12] = ACTIONS(1199),
    [aux_sym_type_qualifier_token13] = ACTIONS(1199),
    [aux_sym_type_qualifier_token14] = ACTIONS(1199),
    [aux_sym_type_qualifier_token15] = ACTIONS(1199),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token2] = ACTIONS(1199),
    [aux_sym_subroutine_call_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token2] = ACTIONS(1199),
    [aux_sym_keyword_statement_token3] = ACTIONS(1199),
    [aux_sym_keyword_statement_token4] = ACTIONS(1199),
    [aux_sym_keyword_statement_token6] = ACTIONS(1199),
    [aux_sym_keyword_statement_token7] = ACTIONS(1199),
    [aux_sym_include_statement_token1] = ACTIONS(1199),
    [aux_sym_data_statement_token1] = ACTIONS(1199),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1199),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1199),
    [aux_sym_end_if_statement_token1] = ACTIONS(1199),
    [aux_sym_elseif_clause_token2] = ACTIONS(1199),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1199),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token3] = ACTIONS(1199),
    [aux_sym_select_type_statement_token1] = ACTIONS(1199),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1199),
    [aux_sym_block_construct_token1] = ACTIONS(1199),
    [aux_sym_associate_statement_token1] = ACTIONS(1199),
    [aux_sym_format_statement_token1] = ACTIONS(1199),
    [aux_sym_print_statement_token1] = ACTIONS(1199),
    [aux_sym_open_statement_token1] = ACTIONS(1199),
    [aux_sym_close_statement_token1] = ACTIONS(1199),
    [aux_sym_inquire_statement_token1] = ACTIONS(1199),
    [aux_sym_enum_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token2] = ACTIONS(1199),
    [aux_sym_file_position_statement_token3] = ACTIONS(1199),
    [aux_sym_file_position_statement_token4] = ACTIONS(1199),
    [aux_sym_logical_expression_token5] = ACTIONS(1197),
    [anon_sym_PLUS] = ACTIONS(1197),
    [anon_sym_DOT] = ACTIONS(1199),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1197),
    [anon_sym_LBRACK] = ACTIONS(1197),
    [sym_boolean_literal] = ACTIONS(1197),
    [aux_sym_null_literal_token1] = ACTIONS(1199),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1199),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1197),
    [sym__float_literal] = ACTIONS(1197),
    [sym__boz_literal] = ACTIONS(1197),
    [sym_string_literal] = ACTIONS(1197),
  },
  [354] = {
    [sym_preproc_file_line] = STATE(354),
    [aux_sym_end_program_statement_token1] = ACTIONS(1323),
    [anon_sym_LPAREN] = ACTIONS(1323),
    [aux_sym_interface_statement_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1323),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1323),
    [aux_sym_end_function_statement_token1] = ACTIONS(1323),
    [aux_sym_contains_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token2] = ACTIONS(1323),
    [aux_sym_implicit_statement_token1] = ACTIONS(1323),
    [aux_sym_implicit_statement_token3] = ACTIONS(1323),
    [aux_sym_implicit_statement_token4] = ACTIONS(1323),
    [aux_sym_save_statement_token1] = ACTIONS(1323),
    [aux_sym_private_statement_token1] = ACTIONS(1323),
    [aux_sym_public_statement_token1] = ACTIONS(1323),
    [aux_sym_namelist_statement_token1] = ACTIONS(1323),
    [aux_sym_common_statement_token1] = ACTIONS(1323),
    [anon_sym_DASH] = ACTIONS(1325),
    [aux_sym_import_statement_token1] = ACTIONS(1323),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1323),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1323),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1323),
    [aux_sym_variable_attributes_token1] = ACTIONS(1323),
    [aux_sym_variable_attributes_token2] = ACTIONS(1323),
    [aux_sym_variable_attributes_token3] = ACTIONS(1323),
    [aux_sym_variable_attributes_token4] = ACTIONS(1323),
    [aux_sym_variable_attributes_token5] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1323),
    [aux_sym_derived_type_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token2] = ACTIONS(1323),
    [aux_sym_type_qualifier_token3] = ACTIONS(1323),
    [aux_sym_type_qualifier_token4] = ACTIONS(1323),
    [aux_sym_type_qualifier_token5] = ACTIONS(1323),
    [aux_sym_type_qualifier_token9] = ACTIONS(1323),
    [aux_sym_type_qualifier_token10] = ACTIONS(1323),
    [aux_sym_type_qualifier_token11] = ACTIONS(1323),
    [aux_sym_type_qualifier_token12] = ACTIONS(1323),
    [aux_sym_type_qualifier_token13] = ACTIONS(1323),
    [aux_sym_type_qualifier_token14] = ACTIONS(1323),
    [aux_sym_type_qualifier_token15] = ACTIONS(1323),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token2] = ACTIONS(1323),
    [aux_sym_subroutine_call_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token2] = ACTIONS(1323),
    [aux_sym_keyword_statement_token3] = ACTIONS(1323),
    [aux_sym_keyword_statement_token4] = ACTIONS(1323),
    [aux_sym_keyword_statement_token6] = ACTIONS(1323),
    [aux_sym_keyword_statement_token7] = ACTIONS(1323),
    [aux_sym_include_statement_token1] = ACTIONS(1323),
    [aux_sym_data_statement_token1] = ACTIONS(1323),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1323),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1323),
    [aux_sym_end_if_statement_token1] = ACTIONS(1323),
    [aux_sym_elseif_clause_token2] = ACTIONS(1323),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1323),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token3] = ACTIONS(1323),
    [aux_sym_select_type_statement_token1] = ACTIONS(1323),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1323),
    [aux_sym_block_construct_token1] = ACTIONS(1323),
    [aux_sym_associate_statement_token1] = ACTIONS(1323),
    [aux_sym_format_statement_token1] = ACTIONS(1323),
    [aux_sym_print_statement_token1] = ACTIONS(1323),
    [aux_sym_open_statement_token1] = ACTIONS(1323),
    [aux_sym_close_statement_token1] = ACTIONS(1323),
    [aux_sym_inquire_statement_token1] = ACTIONS(1323),
    [aux_sym_enum_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token2] = ACTIONS(1323),
    [aux_sym_file_position_statement_token3] = ACTIONS(1323),
    [aux_sym_file_position_statement_token4] = ACTIONS(1323),
    [aux_sym_logical_expression_token5] = ACTIONS(1325),
    [anon_sym_PLUS] = ACTIONS(1325),
    [anon_sym_DOT] = ACTIONS(1323),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1325),
    [anon_sym_LBRACK] = ACTIONS(1325),
    [sym_boolean_literal] = ACTIONS(1325),
    [aux_sym_null_literal_token1] = ACTIONS(1323),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1323),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1325),
    [sym__float_literal] = ACTIONS(1325),
    [sym__boz_literal] = ACTIONS(1325),
    [sym_string_literal] = ACTIONS(1325),
  },
  [355] = {
    [sym_preproc_file_line] = STATE(355),
    [aux_sym_end_program_statement_token1] = ACTIONS(1287),
    [anon_sym_LPAREN] = ACTIONS(1287),
    [aux_sym_interface_statement_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1287),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1287),
    [aux_sym_end_function_statement_token1] = ACTIONS(1287),
    [aux_sym_contains_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token2] = ACTIONS(1287),
    [aux_sym_implicit_statement_token1] = ACTIONS(1287),
    [aux_sym_implicit_statement_token3] = ACTIONS(1287),
    [aux_sym_implicit_statement_token4] = ACTIONS(1287),
    [aux_sym_save_statement_token1] = ACTIONS(1287),
    [aux_sym_private_statement_token1] = ACTIONS(1287),
    [aux_sym_public_statement_token1] = ACTIONS(1287),
    [aux_sym_namelist_statement_token1] = ACTIONS(1287),
    [aux_sym_common_statement_token1] = ACTIONS(1287),
    [anon_sym_DASH] = ACTIONS(1289),
    [aux_sym_import_statement_token1] = ACTIONS(1287),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1287),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1287),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1287),
    [aux_sym_variable_attributes_token1] = ACTIONS(1287),
    [aux_sym_variable_attributes_token2] = ACTIONS(1287),
    [aux_sym_variable_attributes_token3] = ACTIONS(1287),
    [aux_sym_variable_attributes_token4] = ACTIONS(1287),
    [aux_sym_variable_attributes_token5] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1287),
    [aux_sym_derived_type_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token2] = ACTIONS(1287),
    [aux_sym_type_qualifier_token3] = ACTIONS(1287),
    [aux_sym_type_qualifier_token4] = ACTIONS(1287),
    [aux_sym_type_qualifier_token5] = ACTIONS(1287),
    [aux_sym_type_qualifier_token9] = ACTIONS(1287),
    [aux_sym_type_qualifier_token10] = ACTIONS(1287),
    [aux_sym_type_qualifier_token11] = ACTIONS(1287),
    [aux_sym_type_qualifier_token12] = ACTIONS(1287),
    [aux_sym_type_qualifier_token13] = ACTIONS(1287),
    [aux_sym_type_qualifier_token14] = ACTIONS(1287),
    [aux_sym_type_qualifier_token15] = ACTIONS(1287),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token2] = ACTIONS(1287),
    [aux_sym_subroutine_call_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token2] = ACTIONS(1287),
    [aux_sym_keyword_statement_token3] = ACTIONS(1287),
    [aux_sym_keyword_statement_token4] = ACTIONS(1287),
    [aux_sym_keyword_statement_token6] = ACTIONS(1287),
    [aux_sym_keyword_statement_token7] = ACTIONS(1287),
    [aux_sym_include_statement_token1] = ACTIONS(1287),
    [aux_sym_data_statement_token1] = ACTIONS(1287),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1287),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1287),
    [aux_sym_end_if_statement_token1] = ACTIONS(1287),
    [aux_sym_elseif_clause_token2] = ACTIONS(1287),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1287),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token3] = ACTIONS(1287),
    [aux_sym_select_type_statement_token1] = ACTIONS(1287),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1287),
    [aux_sym_block_construct_token1] = ACTIONS(1287),
    [aux_sym_associate_statement_token1] = ACTIONS(1287),
    [aux_sym_format_statement_token1] = ACTIONS(1287),
    [aux_sym_print_statement_token1] = ACTIONS(1287),
    [aux_sym_open_statement_token1] = ACTIONS(1287),
    [aux_sym_close_statement_token1] = ACTIONS(1287),
    [aux_sym_inquire_statement_token1] = ACTIONS(1287),
    [aux_sym_enum_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token2] = ACTIONS(1287),
    [aux_sym_file_position_statement_token3] = ACTIONS(1287),
    [aux_sym_file_position_statement_token4] = ACTIONS(1287),
    [aux_sym_logical_expression_token5] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1289),
    [anon_sym_DOT] = ACTIONS(1287),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1289),
    [anon_sym_LBRACK] = ACTIONS(1289),
    [sym_boolean_literal] = ACTIONS(1289),
    [aux_sym_null_literal_token1] = ACTIONS(1287),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1287),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1289),
    [sym__float_literal] = ACTIONS(1289),
    [sym__boz_literal] = ACTIONS(1289),
    [sym_string_literal] = ACTIONS(1289),
  },
  [356] = {
    [sym_preproc_file_line] = STATE(356),
    [aux_sym_end_program_statement_token1] = ACTIONS(1319),
    [anon_sym_LPAREN] = ACTIONS(1319),
    [aux_sym_interface_statement_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1319),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1319),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1319),
    [aux_sym_contains_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token2] = ACTIONS(1319),
    [aux_sym_implicit_statement_token1] = ACTIONS(1319),
    [aux_sym_implicit_statement_token3] = ACTIONS(1319),
    [aux_sym_implicit_statement_token4] = ACTIONS(1319),
    [aux_sym_save_statement_token1] = ACTIONS(1319),
    [aux_sym_private_statement_token1] = ACTIONS(1319),
    [aux_sym_public_statement_token1] = ACTIONS(1319),
    [aux_sym_namelist_statement_token1] = ACTIONS(1319),
    [aux_sym_common_statement_token1] = ACTIONS(1319),
    [anon_sym_DASH] = ACTIONS(1321),
    [aux_sym_import_statement_token1] = ACTIONS(1319),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1319),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1319),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1319),
    [aux_sym_variable_attributes_token1] = ACTIONS(1319),
    [aux_sym_variable_attributes_token2] = ACTIONS(1319),
    [aux_sym_variable_attributes_token3] = ACTIONS(1319),
    [aux_sym_variable_attributes_token4] = ACTIONS(1319),
    [aux_sym_variable_attributes_token5] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1319),
    [aux_sym_derived_type_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token2] = ACTIONS(1319),
    [aux_sym_type_qualifier_token3] = ACTIONS(1319),
    [aux_sym_type_qualifier_token4] = ACTIONS(1319),
    [aux_sym_type_qualifier_token5] = ACTIONS(1319),
    [aux_sym_type_qualifier_token9] = ACTIONS(1319),
    [aux_sym_type_qualifier_token10] = ACTIONS(1319),
    [aux_sym_type_qualifier_token11] = ACTIONS(1319),
    [aux_sym_type_qualifier_token12] = ACTIONS(1319),
    [aux_sym_type_qualifier_token13] = ACTIONS(1319),
    [aux_sym_type_qualifier_token14] = ACTIONS(1319),
    [aux_sym_type_qualifier_token15] = ACTIONS(1319),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token2] = ACTIONS(1319),
    [aux_sym_subroutine_call_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token2] = ACTIONS(1319),
    [aux_sym_keyword_statement_token3] = ACTIONS(1319),
    [aux_sym_keyword_statement_token4] = ACTIONS(1319),
    [aux_sym_keyword_statement_token6] = ACTIONS(1319),
    [aux_sym_keyword_statement_token7] = ACTIONS(1319),
    [aux_sym_include_statement_token1] = ACTIONS(1319),
    [aux_sym_data_statement_token1] = ACTIONS(1319),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1319),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1319),
    [aux_sym_end_if_statement_token1] = ACTIONS(1319),
    [aux_sym_elseif_clause_token2] = ACTIONS(1319),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1319),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token3] = ACTIONS(1319),
    [aux_sym_select_type_statement_token1] = ACTIONS(1319),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1319),
    [aux_sym_block_construct_token1] = ACTIONS(1319),
    [aux_sym_associate_statement_token1] = ACTIONS(1319),
    [aux_sym_format_statement_token1] = ACTIONS(1319),
    [aux_sym_print_statement_token1] = ACTIONS(1319),
    [aux_sym_open_statement_token1] = ACTIONS(1319),
    [aux_sym_close_statement_token1] = ACTIONS(1319),
    [aux_sym_inquire_statement_token1] = ACTIONS(1319),
    [aux_sym_enum_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token2] = ACTIONS(1319),
    [aux_sym_file_position_statement_token3] = ACTIONS(1319),
    [aux_sym_file_position_statement_token4] = ACTIONS(1319),
    [aux_sym_logical_expression_token5] = ACTIONS(1321),
    [anon_sym_PLUS] = ACTIONS(1321),
    [anon_sym_DOT] = ACTIONS(1319),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1321),
    [anon_sym_LBRACK] = ACTIONS(1321),
    [sym_boolean_literal] = ACTIONS(1321),
    [aux_sym_null_literal_token1] = ACTIONS(1319),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1319),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1321),
    [sym__float_literal] = ACTIONS(1321),
    [sym__boz_literal] = ACTIONS(1321),
    [sym_string_literal] = ACTIONS(1321),
  },
  [357] = {
    [sym_preproc_file_line] = STATE(357),
    [aux_sym_end_program_statement_token1] = ACTIONS(1363),
    [aux_sym_end_program_statement_token2] = ACTIONS(1363),
    [anon_sym_LPAREN] = ACTIONS(1363),
    [aux_sym_interface_statement_token1] = ACTIONS(1363),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1363),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1363),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1363),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1363),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1363),
    [aux_sym_contains_statement_token1] = ACTIONS(1363),
    [aux_sym_use_statement_token1] = ACTIONS(1363),
    [aux_sym_use_statement_token2] = ACTIONS(1363),
    [aux_sym_implicit_statement_token1] = ACTIONS(1363),
    [aux_sym_implicit_statement_token3] = ACTIONS(1363),
    [aux_sym_implicit_statement_token4] = ACTIONS(1363),
    [aux_sym_save_statement_token1] = ACTIONS(1363),
    [aux_sym_private_statement_token1] = ACTIONS(1363),
    [aux_sym_public_statement_token1] = ACTIONS(1363),
    [aux_sym_namelist_statement_token1] = ACTIONS(1363),
    [aux_sym_common_statement_token1] = ACTIONS(1363),
    [anon_sym_DASH] = ACTIONS(1365),
    [aux_sym_import_statement_token1] = ACTIONS(1363),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1363),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1363),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1363),
    [aux_sym_variable_attributes_token1] = ACTIONS(1363),
    [aux_sym_variable_attributes_token2] = ACTIONS(1363),
    [aux_sym_variable_attributes_token3] = ACTIONS(1363),
    [aux_sym_variable_attributes_token4] = ACTIONS(1363),
    [aux_sym_variable_attributes_token5] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1363),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1363),
    [aux_sym_derived_type_token1] = ACTIONS(1363),
    [aux_sym_type_qualifier_token1] = ACTIONS(1363),
    [aux_sym_type_qualifier_token2] = ACTIONS(1363),
    [aux_sym_type_qualifier_token3] = ACTIONS(1363),
    [aux_sym_type_qualifier_token4] = ACTIONS(1363),
    [aux_sym_type_qualifier_token5] = ACTIONS(1363),
    [aux_sym_type_qualifier_token9] = ACTIONS(1363),
    [aux_sym_type_qualifier_token10] = ACTIONS(1363),
    [aux_sym_type_qualifier_token11] = ACTIONS(1363),
    [aux_sym_type_qualifier_token12] = ACTIONS(1363),
    [aux_sym_type_qualifier_token13] = ACTIONS(1363),
    [aux_sym_type_qualifier_token14] = ACTIONS(1363),
    [aux_sym_type_qualifier_token15] = ACTIONS(1363),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1363),
    [aux_sym_stop_statement_token1] = ACTIONS(1363),
    [aux_sym_stop_statement_token2] = ACTIONS(1363),
    [aux_sym_subroutine_call_token1] = ACTIONS(1363),
    [aux_sym_keyword_statement_token1] = ACTIONS(1363),
    [aux_sym_keyword_statement_token2] = ACTIONS(1363),
    [aux_sym_keyword_statement_token3] = ACTIONS(1363),
    [aux_sym_keyword_statement_token4] = ACTIONS(1363),
    [aux_sym_keyword_statement_token6] = ACTIONS(1363),
    [aux_sym_keyword_statement_token7] = ACTIONS(1363),
    [aux_sym_include_statement_token1] = ACTIONS(1363),
    [aux_sym_data_statement_token1] = ACTIONS(1363),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1363),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1363),
    [aux_sym_end_if_statement_token1] = ACTIONS(1363),
    [aux_sym_elseif_clause_token2] = ACTIONS(1363),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1363),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1363),
    [aux_sym_select_case_statement_token1] = ACTIONS(1363),
    [aux_sym_select_case_statement_token3] = ACTIONS(1363),
    [aux_sym_select_type_statement_token1] = ACTIONS(1363),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1363),
    [aux_sym_block_construct_token1] = ACTIONS(1363),
    [aux_sym_associate_statement_token1] = ACTIONS(1363),
    [aux_sym_format_statement_token1] = ACTIONS(1363),
    [aux_sym_print_statement_token1] = ACTIONS(1363),
    [aux_sym_open_statement_token1] = ACTIONS(1363),
    [aux_sym_close_statement_token1] = ACTIONS(1363),
    [aux_sym_inquire_statement_token1] = ACTIONS(1363),
    [aux_sym_enum_statement_token1] = ACTIONS(1363),
    [aux_sym_file_position_statement_token1] = ACTIONS(1363),
    [aux_sym_file_position_statement_token2] = ACTIONS(1363),
    [aux_sym_file_position_statement_token3] = ACTIONS(1363),
    [aux_sym_file_position_statement_token4] = ACTIONS(1363),
    [aux_sym_logical_expression_token5] = ACTIONS(1365),
    [anon_sym_PLUS] = ACTIONS(1365),
    [anon_sym_DOT] = ACTIONS(1363),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1365),
    [anon_sym_LBRACK] = ACTIONS(1365),
    [sym_boolean_literal] = ACTIONS(1365),
    [aux_sym_null_literal_token1] = ACTIONS(1363),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1363),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1365),
    [sym__float_literal] = ACTIONS(1365),
    [sym__boz_literal] = ACTIONS(1365),
    [sym_string_literal] = ACTIONS(1365),
  },
  [358] = {
    [sym_preproc_file_line] = STATE(358),
    [aux_sym_end_program_statement_token1] = ACTIONS(1099),
    [anon_sym_LPAREN] = ACTIONS(1099),
    [aux_sym_interface_statement_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1099),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1099),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1099),
    [aux_sym_contains_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token2] = ACTIONS(1099),
    [aux_sym_implicit_statement_token1] = ACTIONS(1099),
    [aux_sym_implicit_statement_token3] = ACTIONS(1099),
    [aux_sym_implicit_statement_token4] = ACTIONS(1099),
    [aux_sym_save_statement_token1] = ACTIONS(1099),
    [aux_sym_private_statement_token1] = ACTIONS(1099),
    [aux_sym_public_statement_token1] = ACTIONS(1099),
    [aux_sym_namelist_statement_token1] = ACTIONS(1099),
    [aux_sym_common_statement_token1] = ACTIONS(1099),
    [anon_sym_DASH] = ACTIONS(1097),
    [aux_sym_import_statement_token1] = ACTIONS(1099),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1099),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1099),
    [aux_sym_variable_attributes_token1] = ACTIONS(1099),
    [aux_sym_variable_attributes_token2] = ACTIONS(1099),
    [aux_sym_variable_attributes_token3] = ACTIONS(1099),
    [aux_sym_variable_attributes_token4] = ACTIONS(1099),
    [aux_sym_variable_attributes_token5] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1099),
    [aux_sym_derived_type_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token2] = ACTIONS(1099),
    [aux_sym_type_qualifier_token3] = ACTIONS(1099),
    [aux_sym_type_qualifier_token4] = ACTIONS(1099),
    [aux_sym_type_qualifier_token5] = ACTIONS(1099),
    [aux_sym_type_qualifier_token9] = ACTIONS(1099),
    [aux_sym_type_qualifier_token10] = ACTIONS(1099),
    [aux_sym_type_qualifier_token11] = ACTIONS(1099),
    [aux_sym_type_qualifier_token12] = ACTIONS(1099),
    [aux_sym_type_qualifier_token13] = ACTIONS(1099),
    [aux_sym_type_qualifier_token14] = ACTIONS(1099),
    [aux_sym_type_qualifier_token15] = ACTIONS(1099),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token2] = ACTIONS(1099),
    [aux_sym_subroutine_call_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token2] = ACTIONS(1099),
    [aux_sym_keyword_statement_token3] = ACTIONS(1099),
    [aux_sym_keyword_statement_token4] = ACTIONS(1099),
    [aux_sym_keyword_statement_token6] = ACTIONS(1099),
    [aux_sym_keyword_statement_token7] = ACTIONS(1099),
    [aux_sym_include_statement_token1] = ACTIONS(1099),
    [aux_sym_data_statement_token1] = ACTIONS(1099),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1099),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1099),
    [aux_sym_end_if_statement_token1] = ACTIONS(1099),
    [aux_sym_elseif_clause_token2] = ACTIONS(1099),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1099),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token3] = ACTIONS(1099),
    [aux_sym_select_type_statement_token1] = ACTIONS(1099),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1099),
    [aux_sym_block_construct_token1] = ACTIONS(1099),
    [aux_sym_associate_statement_token1] = ACTIONS(1099),
    [aux_sym_format_statement_token1] = ACTIONS(1099),
    [aux_sym_print_statement_token1] = ACTIONS(1099),
    [aux_sym_open_statement_token1] = ACTIONS(1099),
    [aux_sym_close_statement_token1] = ACTIONS(1099),
    [aux_sym_inquire_statement_token1] = ACTIONS(1099),
    [aux_sym_enum_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token2] = ACTIONS(1099),
    [aux_sym_file_position_statement_token3] = ACTIONS(1099),
    [aux_sym_file_position_statement_token4] = ACTIONS(1099),
    [aux_sym_logical_expression_token5] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [sym_boolean_literal] = ACTIONS(1097),
    [aux_sym_null_literal_token1] = ACTIONS(1099),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1099),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1097),
    [sym__float_literal] = ACTIONS(1097),
    [sym__boz_literal] = ACTIONS(1097),
    [sym_string_literal] = ACTIONS(1097),
  },
  [359] = {
    [sym_preproc_file_line] = STATE(359),
    [aux_sym_end_program_statement_token1] = ACTIONS(1247),
    [anon_sym_LPAREN] = ACTIONS(1247),
    [aux_sym_interface_statement_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1247),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1247),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1247),
    [aux_sym_contains_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token2] = ACTIONS(1247),
    [aux_sym_implicit_statement_token1] = ACTIONS(1247),
    [aux_sym_implicit_statement_token3] = ACTIONS(1247),
    [aux_sym_implicit_statement_token4] = ACTIONS(1247),
    [aux_sym_save_statement_token1] = ACTIONS(1247),
    [aux_sym_private_statement_token1] = ACTIONS(1247),
    [aux_sym_public_statement_token1] = ACTIONS(1247),
    [aux_sym_namelist_statement_token1] = ACTIONS(1247),
    [aux_sym_common_statement_token1] = ACTIONS(1247),
    [anon_sym_DASH] = ACTIONS(1245),
    [aux_sym_import_statement_token1] = ACTIONS(1247),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1247),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1247),
    [aux_sym_variable_attributes_token1] = ACTIONS(1247),
    [aux_sym_variable_attributes_token2] = ACTIONS(1247),
    [aux_sym_variable_attributes_token3] = ACTIONS(1247),
    [aux_sym_variable_attributes_token4] = ACTIONS(1247),
    [aux_sym_variable_attributes_token5] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1247),
    [aux_sym_derived_type_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token2] = ACTIONS(1247),
    [aux_sym_type_qualifier_token3] = ACTIONS(1247),
    [aux_sym_type_qualifier_token4] = ACTIONS(1247),
    [aux_sym_type_qualifier_token5] = ACTIONS(1247),
    [aux_sym_type_qualifier_token9] = ACTIONS(1247),
    [aux_sym_type_qualifier_token10] = ACTIONS(1247),
    [aux_sym_type_qualifier_token11] = ACTIONS(1247),
    [aux_sym_type_qualifier_token12] = ACTIONS(1247),
    [aux_sym_type_qualifier_token13] = ACTIONS(1247),
    [aux_sym_type_qualifier_token14] = ACTIONS(1247),
    [aux_sym_type_qualifier_token15] = ACTIONS(1247),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token2] = ACTIONS(1247),
    [aux_sym_subroutine_call_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token2] = ACTIONS(1247),
    [aux_sym_keyword_statement_token3] = ACTIONS(1247),
    [aux_sym_keyword_statement_token4] = ACTIONS(1247),
    [aux_sym_keyword_statement_token6] = ACTIONS(1247),
    [aux_sym_keyword_statement_token7] = ACTIONS(1247),
    [aux_sym_include_statement_token1] = ACTIONS(1247),
    [aux_sym_data_statement_token1] = ACTIONS(1247),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1247),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1247),
    [aux_sym_end_if_statement_token1] = ACTIONS(1247),
    [aux_sym_elseif_clause_token2] = ACTIONS(1247),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1247),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token3] = ACTIONS(1247),
    [aux_sym_select_type_statement_token1] = ACTIONS(1247),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1247),
    [aux_sym_block_construct_token1] = ACTIONS(1247),
    [aux_sym_associate_statement_token1] = ACTIONS(1247),
    [aux_sym_format_statement_token1] = ACTIONS(1247),
    [aux_sym_print_statement_token1] = ACTIONS(1247),
    [aux_sym_open_statement_token1] = ACTIONS(1247),
    [aux_sym_close_statement_token1] = ACTIONS(1247),
    [aux_sym_inquire_statement_token1] = ACTIONS(1247),
    [aux_sym_enum_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token2] = ACTIONS(1247),
    [aux_sym_file_position_statement_token3] = ACTIONS(1247),
    [aux_sym_file_position_statement_token4] = ACTIONS(1247),
    [aux_sym_logical_expression_token5] = ACTIONS(1245),
    [anon_sym_PLUS] = ACTIONS(1245),
    [anon_sym_DOT] = ACTIONS(1247),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1245),
    [anon_sym_LBRACK] = ACTIONS(1245),
    [sym_boolean_literal] = ACTIONS(1245),
    [aux_sym_null_literal_token1] = ACTIONS(1247),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1247),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1245),
    [sym__float_literal] = ACTIONS(1245),
    [sym__boz_literal] = ACTIONS(1245),
    [sym_string_literal] = ACTIONS(1245),
  },
  [360] = {
    [sym_preproc_file_line] = STATE(360),
    [aux_sym_end_program_statement_token1] = ACTIONS(1359),
    [aux_sym_end_program_statement_token2] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [aux_sym_interface_statement_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1359),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1359),
    [aux_sym_contains_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token2] = ACTIONS(1359),
    [aux_sym_implicit_statement_token1] = ACTIONS(1359),
    [aux_sym_implicit_statement_token3] = ACTIONS(1359),
    [aux_sym_implicit_statement_token4] = ACTIONS(1359),
    [aux_sym_save_statement_token1] = ACTIONS(1359),
    [aux_sym_private_statement_token1] = ACTIONS(1359),
    [aux_sym_public_statement_token1] = ACTIONS(1359),
    [aux_sym_namelist_statement_token1] = ACTIONS(1359),
    [aux_sym_common_statement_token1] = ACTIONS(1359),
    [anon_sym_DASH] = ACTIONS(1361),
    [aux_sym_import_statement_token1] = ACTIONS(1359),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1359),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1359),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1359),
    [aux_sym_variable_attributes_token1] = ACTIONS(1359),
    [aux_sym_variable_attributes_token2] = ACTIONS(1359),
    [aux_sym_variable_attributes_token3] = ACTIONS(1359),
    [aux_sym_variable_attributes_token4] = ACTIONS(1359),
    [aux_sym_variable_attributes_token5] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1359),
    [aux_sym_derived_type_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token2] = ACTIONS(1359),
    [aux_sym_type_qualifier_token3] = ACTIONS(1359),
    [aux_sym_type_qualifier_token4] = ACTIONS(1359),
    [aux_sym_type_qualifier_token5] = ACTIONS(1359),
    [aux_sym_type_qualifier_token9] = ACTIONS(1359),
    [aux_sym_type_qualifier_token10] = ACTIONS(1359),
    [aux_sym_type_qualifier_token11] = ACTIONS(1359),
    [aux_sym_type_qualifier_token12] = ACTIONS(1359),
    [aux_sym_type_qualifier_token13] = ACTIONS(1359),
    [aux_sym_type_qualifier_token14] = ACTIONS(1359),
    [aux_sym_type_qualifier_token15] = ACTIONS(1359),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token2] = ACTIONS(1359),
    [aux_sym_subroutine_call_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token2] = ACTIONS(1359),
    [aux_sym_keyword_statement_token3] = ACTIONS(1359),
    [aux_sym_keyword_statement_token4] = ACTIONS(1359),
    [aux_sym_keyword_statement_token6] = ACTIONS(1359),
    [aux_sym_keyword_statement_token7] = ACTIONS(1359),
    [aux_sym_include_statement_token1] = ACTIONS(1359),
    [aux_sym_data_statement_token1] = ACTIONS(1359),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1359),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1359),
    [aux_sym_end_if_statement_token1] = ACTIONS(1359),
    [aux_sym_elseif_clause_token2] = ACTIONS(1359),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1359),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token3] = ACTIONS(1359),
    [aux_sym_select_type_statement_token1] = ACTIONS(1359),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1359),
    [aux_sym_block_construct_token1] = ACTIONS(1359),
    [aux_sym_associate_statement_token1] = ACTIONS(1359),
    [aux_sym_format_statement_token1] = ACTIONS(1359),
    [aux_sym_print_statement_token1] = ACTIONS(1359),
    [aux_sym_open_statement_token1] = ACTIONS(1359),
    [aux_sym_close_statement_token1] = ACTIONS(1359),
    [aux_sym_inquire_statement_token1] = ACTIONS(1359),
    [aux_sym_enum_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token2] = ACTIONS(1359),
    [aux_sym_file_position_statement_token3] = ACTIONS(1359),
    [aux_sym_file_position_statement_token4] = ACTIONS(1359),
    [aux_sym_logical_expression_token5] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(1359),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [sym_boolean_literal] = ACTIONS(1361),
    [aux_sym_null_literal_token1] = ACTIONS(1359),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1359),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1361),
    [sym__float_literal] = ACTIONS(1361),
    [sym__boz_literal] = ACTIONS(1361),
    [sym_string_literal] = ACTIONS(1361),
  },
  [361] = {
    [sym_preproc_file_line] = STATE(361),
    [aux_sym_end_program_statement_token1] = ACTIONS(1355),
    [aux_sym_end_program_statement_token2] = ACTIONS(1355),
    [anon_sym_LPAREN] = ACTIONS(1355),
    [aux_sym_interface_statement_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1355),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1355),
    [aux_sym_contains_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token2] = ACTIONS(1355),
    [aux_sym_implicit_statement_token1] = ACTIONS(1355),
    [aux_sym_implicit_statement_token3] = ACTIONS(1355),
    [aux_sym_implicit_statement_token4] = ACTIONS(1355),
    [aux_sym_save_statement_token1] = ACTIONS(1355),
    [aux_sym_private_statement_token1] = ACTIONS(1355),
    [aux_sym_public_statement_token1] = ACTIONS(1355),
    [aux_sym_namelist_statement_token1] = ACTIONS(1355),
    [aux_sym_common_statement_token1] = ACTIONS(1355),
    [anon_sym_DASH] = ACTIONS(1357),
    [aux_sym_import_statement_token1] = ACTIONS(1355),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1355),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1355),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1355),
    [aux_sym_variable_attributes_token1] = ACTIONS(1355),
    [aux_sym_variable_attributes_token2] = ACTIONS(1355),
    [aux_sym_variable_attributes_token3] = ACTIONS(1355),
    [aux_sym_variable_attributes_token4] = ACTIONS(1355),
    [aux_sym_variable_attributes_token5] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1355),
    [aux_sym_derived_type_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token2] = ACTIONS(1355),
    [aux_sym_type_qualifier_token3] = ACTIONS(1355),
    [aux_sym_type_qualifier_token4] = ACTIONS(1355),
    [aux_sym_type_qualifier_token5] = ACTIONS(1355),
    [aux_sym_type_qualifier_token9] = ACTIONS(1355),
    [aux_sym_type_qualifier_token10] = ACTIONS(1355),
    [aux_sym_type_qualifier_token11] = ACTIONS(1355),
    [aux_sym_type_qualifier_token12] = ACTIONS(1355),
    [aux_sym_type_qualifier_token13] = ACTIONS(1355),
    [aux_sym_type_qualifier_token14] = ACTIONS(1355),
    [aux_sym_type_qualifier_token15] = ACTIONS(1355),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token2] = ACTIONS(1355),
    [aux_sym_subroutine_call_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token2] = ACTIONS(1355),
    [aux_sym_keyword_statement_token3] = ACTIONS(1355),
    [aux_sym_keyword_statement_token4] = ACTIONS(1355),
    [aux_sym_keyword_statement_token6] = ACTIONS(1355),
    [aux_sym_keyword_statement_token7] = ACTIONS(1355),
    [aux_sym_include_statement_token1] = ACTIONS(1355),
    [aux_sym_data_statement_token1] = ACTIONS(1355),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1355),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1355),
    [aux_sym_end_if_statement_token1] = ACTIONS(1355),
    [aux_sym_elseif_clause_token2] = ACTIONS(1355),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1355),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token3] = ACTIONS(1355),
    [aux_sym_select_type_statement_token1] = ACTIONS(1355),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1355),
    [aux_sym_block_construct_token1] = ACTIONS(1355),
    [aux_sym_associate_statement_token1] = ACTIONS(1355),
    [aux_sym_format_statement_token1] = ACTIONS(1355),
    [aux_sym_print_statement_token1] = ACTIONS(1355),
    [aux_sym_open_statement_token1] = ACTIONS(1355),
    [aux_sym_close_statement_token1] = ACTIONS(1355),
    [aux_sym_inquire_statement_token1] = ACTIONS(1355),
    [aux_sym_enum_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token2] = ACTIONS(1355),
    [aux_sym_file_position_statement_token3] = ACTIONS(1355),
    [aux_sym_file_position_statement_token4] = ACTIONS(1355),
    [aux_sym_logical_expression_token5] = ACTIONS(1357),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_DOT] = ACTIONS(1355),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1357),
    [anon_sym_LBRACK] = ACTIONS(1357),
    [sym_boolean_literal] = ACTIONS(1357),
    [aux_sym_null_literal_token1] = ACTIONS(1355),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1355),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1357),
    [sym__float_literal] = ACTIONS(1357),
    [sym__boz_literal] = ACTIONS(1357),
    [sym_string_literal] = ACTIONS(1357),
  },
  [362] = {
    [sym_preproc_file_line] = STATE(362),
    [aux_sym_end_program_statement_token1] = ACTIONS(1299),
    [aux_sym_end_program_statement_token2] = ACTIONS(1299),
    [anon_sym_LPAREN] = ACTIONS(1299),
    [aux_sym_interface_statement_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1299),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1299),
    [aux_sym_contains_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token2] = ACTIONS(1299),
    [aux_sym_implicit_statement_token1] = ACTIONS(1299),
    [aux_sym_implicit_statement_token3] = ACTIONS(1299),
    [aux_sym_implicit_statement_token4] = ACTIONS(1299),
    [aux_sym_save_statement_token1] = ACTIONS(1299),
    [aux_sym_private_statement_token1] = ACTIONS(1299),
    [aux_sym_public_statement_token1] = ACTIONS(1299),
    [aux_sym_namelist_statement_token1] = ACTIONS(1299),
    [aux_sym_common_statement_token1] = ACTIONS(1299),
    [anon_sym_DASH] = ACTIONS(1301),
    [aux_sym_import_statement_token1] = ACTIONS(1299),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1299),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1299),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1299),
    [aux_sym_variable_attributes_token1] = ACTIONS(1299),
    [aux_sym_variable_attributes_token2] = ACTIONS(1299),
    [aux_sym_variable_attributes_token3] = ACTIONS(1299),
    [aux_sym_variable_attributes_token4] = ACTIONS(1299),
    [aux_sym_variable_attributes_token5] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1299),
    [aux_sym_derived_type_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token2] = ACTIONS(1299),
    [aux_sym_type_qualifier_token3] = ACTIONS(1299),
    [aux_sym_type_qualifier_token4] = ACTIONS(1299),
    [aux_sym_type_qualifier_token5] = ACTIONS(1299),
    [aux_sym_type_qualifier_token9] = ACTIONS(1299),
    [aux_sym_type_qualifier_token10] = ACTIONS(1299),
    [aux_sym_type_qualifier_token11] = ACTIONS(1299),
    [aux_sym_type_qualifier_token12] = ACTIONS(1299),
    [aux_sym_type_qualifier_token13] = ACTIONS(1299),
    [aux_sym_type_qualifier_token14] = ACTIONS(1299),
    [aux_sym_type_qualifier_token15] = ACTIONS(1299),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token2] = ACTIONS(1299),
    [aux_sym_subroutine_call_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token2] = ACTIONS(1299),
    [aux_sym_keyword_statement_token3] = ACTIONS(1299),
    [aux_sym_keyword_statement_token4] = ACTIONS(1299),
    [aux_sym_keyword_statement_token6] = ACTIONS(1299),
    [aux_sym_keyword_statement_token7] = ACTIONS(1299),
    [aux_sym_include_statement_token1] = ACTIONS(1299),
    [aux_sym_data_statement_token1] = ACTIONS(1299),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1299),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1299),
    [aux_sym_end_if_statement_token1] = ACTIONS(1299),
    [aux_sym_elseif_clause_token2] = ACTIONS(1299),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1299),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token3] = ACTIONS(1299),
    [aux_sym_select_type_statement_token1] = ACTIONS(1299),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1299),
    [aux_sym_block_construct_token1] = ACTIONS(1299),
    [aux_sym_associate_statement_token1] = ACTIONS(1299),
    [aux_sym_format_statement_token1] = ACTIONS(1299),
    [aux_sym_print_statement_token1] = ACTIONS(1299),
    [aux_sym_open_statement_token1] = ACTIONS(1299),
    [aux_sym_close_statement_token1] = ACTIONS(1299),
    [aux_sym_inquire_statement_token1] = ACTIONS(1299),
    [aux_sym_enum_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token2] = ACTIONS(1299),
    [aux_sym_file_position_statement_token3] = ACTIONS(1299),
    [aux_sym_file_position_statement_token4] = ACTIONS(1299),
    [aux_sym_logical_expression_token5] = ACTIONS(1301),
    [anon_sym_PLUS] = ACTIONS(1301),
    [anon_sym_DOT] = ACTIONS(1299),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1301),
    [anon_sym_LBRACK] = ACTIONS(1301),
    [sym_boolean_literal] = ACTIONS(1301),
    [aux_sym_null_literal_token1] = ACTIONS(1299),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1299),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1301),
    [sym__float_literal] = ACTIONS(1301),
    [sym__boz_literal] = ACTIONS(1301),
    [sym_string_literal] = ACTIONS(1301),
  },
  [363] = {
    [sym_preproc_file_line] = STATE(363),
    [aux_sym_end_program_statement_token1] = ACTIONS(1295),
    [anon_sym_LPAREN] = ACTIONS(1295),
    [aux_sym_interface_statement_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1295),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1295),
    [aux_sym_end_function_statement_token1] = ACTIONS(1295),
    [aux_sym_contains_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token2] = ACTIONS(1295),
    [aux_sym_implicit_statement_token1] = ACTIONS(1295),
    [aux_sym_implicit_statement_token3] = ACTIONS(1295),
    [aux_sym_implicit_statement_token4] = ACTIONS(1295),
    [aux_sym_save_statement_token1] = ACTIONS(1295),
    [aux_sym_private_statement_token1] = ACTIONS(1295),
    [aux_sym_public_statement_token1] = ACTIONS(1295),
    [aux_sym_namelist_statement_token1] = ACTIONS(1295),
    [aux_sym_common_statement_token1] = ACTIONS(1295),
    [anon_sym_DASH] = ACTIONS(1297),
    [aux_sym_import_statement_token1] = ACTIONS(1295),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1295),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1295),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1295),
    [aux_sym_variable_attributes_token1] = ACTIONS(1295),
    [aux_sym_variable_attributes_token2] = ACTIONS(1295),
    [aux_sym_variable_attributes_token3] = ACTIONS(1295),
    [aux_sym_variable_attributes_token4] = ACTIONS(1295),
    [aux_sym_variable_attributes_token5] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1295),
    [aux_sym_derived_type_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token2] = ACTIONS(1295),
    [aux_sym_type_qualifier_token3] = ACTIONS(1295),
    [aux_sym_type_qualifier_token4] = ACTIONS(1295),
    [aux_sym_type_qualifier_token5] = ACTIONS(1295),
    [aux_sym_type_qualifier_token9] = ACTIONS(1295),
    [aux_sym_type_qualifier_token10] = ACTIONS(1295),
    [aux_sym_type_qualifier_token11] = ACTIONS(1295),
    [aux_sym_type_qualifier_token12] = ACTIONS(1295),
    [aux_sym_type_qualifier_token13] = ACTIONS(1295),
    [aux_sym_type_qualifier_token14] = ACTIONS(1295),
    [aux_sym_type_qualifier_token15] = ACTIONS(1295),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token2] = ACTIONS(1295),
    [aux_sym_subroutine_call_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token2] = ACTIONS(1295),
    [aux_sym_keyword_statement_token3] = ACTIONS(1295),
    [aux_sym_keyword_statement_token4] = ACTIONS(1295),
    [aux_sym_keyword_statement_token6] = ACTIONS(1295),
    [aux_sym_keyword_statement_token7] = ACTIONS(1295),
    [aux_sym_include_statement_token1] = ACTIONS(1295),
    [aux_sym_data_statement_token1] = ACTIONS(1295),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1295),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1295),
    [aux_sym_end_if_statement_token1] = ACTIONS(1295),
    [aux_sym_elseif_clause_token2] = ACTIONS(1295),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1295),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token3] = ACTIONS(1295),
    [aux_sym_select_type_statement_token1] = ACTIONS(1295),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1295),
    [aux_sym_block_construct_token1] = ACTIONS(1295),
    [aux_sym_associate_statement_token1] = ACTIONS(1295),
    [aux_sym_format_statement_token1] = ACTIONS(1295),
    [aux_sym_print_statement_token1] = ACTIONS(1295),
    [aux_sym_open_statement_token1] = ACTIONS(1295),
    [aux_sym_close_statement_token1] = ACTIONS(1295),
    [aux_sym_inquire_statement_token1] = ACTIONS(1295),
    [aux_sym_enum_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token2] = ACTIONS(1295),
    [aux_sym_file_position_statement_token3] = ACTIONS(1295),
    [aux_sym_file_position_statement_token4] = ACTIONS(1295),
    [aux_sym_logical_expression_token5] = ACTIONS(1297),
    [anon_sym_PLUS] = ACTIONS(1297),
    [anon_sym_DOT] = ACTIONS(1295),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1297),
    [anon_sym_LBRACK] = ACTIONS(1297),
    [sym_boolean_literal] = ACTIONS(1297),
    [aux_sym_null_literal_token1] = ACTIONS(1295),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1295),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1297),
    [sym__float_literal] = ACTIONS(1297),
    [sym__boz_literal] = ACTIONS(1297),
    [sym_string_literal] = ACTIONS(1297),
  },
  [364] = {
    [sym_preproc_file_line] = STATE(364),
    [aux_sym_end_program_statement_token1] = ACTIONS(1299),
    [anon_sym_LPAREN] = ACTIONS(1299),
    [aux_sym_interface_statement_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1299),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1299),
    [aux_sym_end_function_statement_token1] = ACTIONS(1299),
    [aux_sym_contains_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token2] = ACTIONS(1299),
    [aux_sym_implicit_statement_token1] = ACTIONS(1299),
    [aux_sym_implicit_statement_token3] = ACTIONS(1299),
    [aux_sym_implicit_statement_token4] = ACTIONS(1299),
    [aux_sym_save_statement_token1] = ACTIONS(1299),
    [aux_sym_private_statement_token1] = ACTIONS(1299),
    [aux_sym_public_statement_token1] = ACTIONS(1299),
    [aux_sym_namelist_statement_token1] = ACTIONS(1299),
    [aux_sym_common_statement_token1] = ACTIONS(1299),
    [anon_sym_DASH] = ACTIONS(1301),
    [aux_sym_import_statement_token1] = ACTIONS(1299),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1299),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1299),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1299),
    [aux_sym_variable_attributes_token1] = ACTIONS(1299),
    [aux_sym_variable_attributes_token2] = ACTIONS(1299),
    [aux_sym_variable_attributes_token3] = ACTIONS(1299),
    [aux_sym_variable_attributes_token4] = ACTIONS(1299),
    [aux_sym_variable_attributes_token5] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1299),
    [aux_sym_derived_type_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token2] = ACTIONS(1299),
    [aux_sym_type_qualifier_token3] = ACTIONS(1299),
    [aux_sym_type_qualifier_token4] = ACTIONS(1299),
    [aux_sym_type_qualifier_token5] = ACTIONS(1299),
    [aux_sym_type_qualifier_token9] = ACTIONS(1299),
    [aux_sym_type_qualifier_token10] = ACTIONS(1299),
    [aux_sym_type_qualifier_token11] = ACTIONS(1299),
    [aux_sym_type_qualifier_token12] = ACTIONS(1299),
    [aux_sym_type_qualifier_token13] = ACTIONS(1299),
    [aux_sym_type_qualifier_token14] = ACTIONS(1299),
    [aux_sym_type_qualifier_token15] = ACTIONS(1299),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token2] = ACTIONS(1299),
    [aux_sym_subroutine_call_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token2] = ACTIONS(1299),
    [aux_sym_keyword_statement_token3] = ACTIONS(1299),
    [aux_sym_keyword_statement_token4] = ACTIONS(1299),
    [aux_sym_keyword_statement_token6] = ACTIONS(1299),
    [aux_sym_keyword_statement_token7] = ACTIONS(1299),
    [aux_sym_include_statement_token1] = ACTIONS(1299),
    [aux_sym_data_statement_token1] = ACTIONS(1299),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1299),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1299),
    [aux_sym_end_if_statement_token1] = ACTIONS(1299),
    [aux_sym_elseif_clause_token2] = ACTIONS(1299),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1299),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token3] = ACTIONS(1299),
    [aux_sym_select_type_statement_token1] = ACTIONS(1299),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1299),
    [aux_sym_block_construct_token1] = ACTIONS(1299),
    [aux_sym_associate_statement_token1] = ACTIONS(1299),
    [aux_sym_format_statement_token1] = ACTIONS(1299),
    [aux_sym_print_statement_token1] = ACTIONS(1299),
    [aux_sym_open_statement_token1] = ACTIONS(1299),
    [aux_sym_close_statement_token1] = ACTIONS(1299),
    [aux_sym_inquire_statement_token1] = ACTIONS(1299),
    [aux_sym_enum_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token2] = ACTIONS(1299),
    [aux_sym_file_position_statement_token3] = ACTIONS(1299),
    [aux_sym_file_position_statement_token4] = ACTIONS(1299),
    [aux_sym_logical_expression_token5] = ACTIONS(1301),
    [anon_sym_PLUS] = ACTIONS(1301),
    [anon_sym_DOT] = ACTIONS(1299),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1301),
    [anon_sym_LBRACK] = ACTIONS(1301),
    [sym_boolean_literal] = ACTIONS(1301),
    [aux_sym_null_literal_token1] = ACTIONS(1299),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1299),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1301),
    [sym__float_literal] = ACTIONS(1301),
    [sym__boz_literal] = ACTIONS(1301),
    [sym_string_literal] = ACTIONS(1301),
  },
  [365] = {
    [sym_preproc_file_line] = STATE(365),
    [aux_sym_end_program_statement_token1] = ACTIONS(1199),
    [anon_sym_LPAREN] = ACTIONS(1199),
    [aux_sym_interface_statement_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1199),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1199),
    [aux_sym_contains_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token2] = ACTIONS(1199),
    [aux_sym_implicit_statement_token1] = ACTIONS(1199),
    [aux_sym_implicit_statement_token3] = ACTIONS(1199),
    [aux_sym_implicit_statement_token4] = ACTIONS(1199),
    [aux_sym_save_statement_token1] = ACTIONS(1199),
    [aux_sym_private_statement_token1] = ACTIONS(1199),
    [aux_sym_public_statement_token1] = ACTIONS(1199),
    [aux_sym_namelist_statement_token1] = ACTIONS(1199),
    [aux_sym_common_statement_token1] = ACTIONS(1199),
    [anon_sym_DASH] = ACTIONS(1197),
    [aux_sym_import_statement_token1] = ACTIONS(1199),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1199),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1199),
    [aux_sym_variable_attributes_token1] = ACTIONS(1199),
    [aux_sym_variable_attributes_token2] = ACTIONS(1199),
    [aux_sym_variable_attributes_token3] = ACTIONS(1199),
    [aux_sym_variable_attributes_token4] = ACTIONS(1199),
    [aux_sym_variable_attributes_token5] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1199),
    [aux_sym_derived_type_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token2] = ACTIONS(1199),
    [aux_sym_type_qualifier_token3] = ACTIONS(1199),
    [aux_sym_type_qualifier_token4] = ACTIONS(1199),
    [aux_sym_type_qualifier_token5] = ACTIONS(1199),
    [aux_sym_type_qualifier_token9] = ACTIONS(1199),
    [aux_sym_type_qualifier_token10] = ACTIONS(1199),
    [aux_sym_type_qualifier_token11] = ACTIONS(1199),
    [aux_sym_type_qualifier_token12] = ACTIONS(1199),
    [aux_sym_type_qualifier_token13] = ACTIONS(1199),
    [aux_sym_type_qualifier_token14] = ACTIONS(1199),
    [aux_sym_type_qualifier_token15] = ACTIONS(1199),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token2] = ACTIONS(1199),
    [aux_sym_subroutine_call_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token2] = ACTIONS(1199),
    [aux_sym_keyword_statement_token3] = ACTIONS(1199),
    [aux_sym_keyword_statement_token4] = ACTIONS(1199),
    [aux_sym_keyword_statement_token6] = ACTIONS(1199),
    [aux_sym_keyword_statement_token7] = ACTIONS(1199),
    [aux_sym_include_statement_token1] = ACTIONS(1199),
    [aux_sym_data_statement_token1] = ACTIONS(1199),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1199),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1199),
    [aux_sym_end_if_statement_token1] = ACTIONS(1199),
    [aux_sym_elseif_clause_token2] = ACTIONS(1199),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1199),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token3] = ACTIONS(1199),
    [aux_sym_select_type_statement_token1] = ACTIONS(1199),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1199),
    [aux_sym_block_construct_token1] = ACTIONS(1199),
    [aux_sym_associate_statement_token1] = ACTIONS(1199),
    [aux_sym_format_statement_token1] = ACTIONS(1199),
    [aux_sym_print_statement_token1] = ACTIONS(1199),
    [aux_sym_open_statement_token1] = ACTIONS(1199),
    [aux_sym_close_statement_token1] = ACTIONS(1199),
    [aux_sym_inquire_statement_token1] = ACTIONS(1199),
    [aux_sym_enum_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token2] = ACTIONS(1199),
    [aux_sym_file_position_statement_token3] = ACTIONS(1199),
    [aux_sym_file_position_statement_token4] = ACTIONS(1199),
    [aux_sym_logical_expression_token5] = ACTIONS(1197),
    [anon_sym_PLUS] = ACTIONS(1197),
    [anon_sym_DOT] = ACTIONS(1199),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1197),
    [anon_sym_LBRACK] = ACTIONS(1197),
    [sym_boolean_literal] = ACTIONS(1197),
    [aux_sym_null_literal_token1] = ACTIONS(1199),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1199),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1197),
    [sym__float_literal] = ACTIONS(1197),
    [sym__boz_literal] = ACTIONS(1197),
    [sym_string_literal] = ACTIONS(1197),
  },
  [366] = {
    [sym_preproc_file_line] = STATE(366),
    [aux_sym_end_program_statement_token1] = ACTIONS(1303),
    [anon_sym_LPAREN] = ACTIONS(1303),
    [aux_sym_interface_statement_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1303),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1303),
    [aux_sym_end_function_statement_token1] = ACTIONS(1303),
    [aux_sym_contains_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token2] = ACTIONS(1303),
    [aux_sym_implicit_statement_token1] = ACTIONS(1303),
    [aux_sym_implicit_statement_token3] = ACTIONS(1303),
    [aux_sym_implicit_statement_token4] = ACTIONS(1303),
    [aux_sym_save_statement_token1] = ACTIONS(1303),
    [aux_sym_private_statement_token1] = ACTIONS(1303),
    [aux_sym_public_statement_token1] = ACTIONS(1303),
    [aux_sym_namelist_statement_token1] = ACTIONS(1303),
    [aux_sym_common_statement_token1] = ACTIONS(1303),
    [anon_sym_DASH] = ACTIONS(1305),
    [aux_sym_import_statement_token1] = ACTIONS(1303),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1303),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1303),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1303),
    [aux_sym_variable_attributes_token1] = ACTIONS(1303),
    [aux_sym_variable_attributes_token2] = ACTIONS(1303),
    [aux_sym_variable_attributes_token3] = ACTIONS(1303),
    [aux_sym_variable_attributes_token4] = ACTIONS(1303),
    [aux_sym_variable_attributes_token5] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1303),
    [aux_sym_derived_type_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token2] = ACTIONS(1303),
    [aux_sym_type_qualifier_token3] = ACTIONS(1303),
    [aux_sym_type_qualifier_token4] = ACTIONS(1303),
    [aux_sym_type_qualifier_token5] = ACTIONS(1303),
    [aux_sym_type_qualifier_token9] = ACTIONS(1303),
    [aux_sym_type_qualifier_token10] = ACTIONS(1303),
    [aux_sym_type_qualifier_token11] = ACTIONS(1303),
    [aux_sym_type_qualifier_token12] = ACTIONS(1303),
    [aux_sym_type_qualifier_token13] = ACTIONS(1303),
    [aux_sym_type_qualifier_token14] = ACTIONS(1303),
    [aux_sym_type_qualifier_token15] = ACTIONS(1303),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token2] = ACTIONS(1303),
    [aux_sym_subroutine_call_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token2] = ACTIONS(1303),
    [aux_sym_keyword_statement_token3] = ACTIONS(1303),
    [aux_sym_keyword_statement_token4] = ACTIONS(1303),
    [aux_sym_keyword_statement_token6] = ACTIONS(1303),
    [aux_sym_keyword_statement_token7] = ACTIONS(1303),
    [aux_sym_include_statement_token1] = ACTIONS(1303),
    [aux_sym_data_statement_token1] = ACTIONS(1303),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1303),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1303),
    [aux_sym_end_if_statement_token1] = ACTIONS(1303),
    [aux_sym_elseif_clause_token2] = ACTIONS(1303),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1303),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token3] = ACTIONS(1303),
    [aux_sym_select_type_statement_token1] = ACTIONS(1303),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1303),
    [aux_sym_block_construct_token1] = ACTIONS(1303),
    [aux_sym_associate_statement_token1] = ACTIONS(1303),
    [aux_sym_format_statement_token1] = ACTIONS(1303),
    [aux_sym_print_statement_token1] = ACTIONS(1303),
    [aux_sym_open_statement_token1] = ACTIONS(1303),
    [aux_sym_close_statement_token1] = ACTIONS(1303),
    [aux_sym_inquire_statement_token1] = ACTIONS(1303),
    [aux_sym_enum_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token2] = ACTIONS(1303),
    [aux_sym_file_position_statement_token3] = ACTIONS(1303),
    [aux_sym_file_position_statement_token4] = ACTIONS(1303),
    [aux_sym_logical_expression_token5] = ACTIONS(1305),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_DOT] = ACTIONS(1303),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1305),
    [anon_sym_LBRACK] = ACTIONS(1305),
    [sym_boolean_literal] = ACTIONS(1305),
    [aux_sym_null_literal_token1] = ACTIONS(1303),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1303),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1305),
    [sym__float_literal] = ACTIONS(1305),
    [sym__boz_literal] = ACTIONS(1305),
    [sym_string_literal] = ACTIONS(1305),
  },
  [367] = {
    [sym_preproc_file_line] = STATE(367),
    [aux_sym_end_program_statement_token1] = ACTIONS(1247),
    [aux_sym_end_program_statement_token2] = ACTIONS(1247),
    [anon_sym_LPAREN] = ACTIONS(1247),
    [aux_sym_interface_statement_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1247),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1247),
    [aux_sym_contains_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token2] = ACTIONS(1247),
    [aux_sym_implicit_statement_token1] = ACTIONS(1247),
    [aux_sym_implicit_statement_token3] = ACTIONS(1247),
    [aux_sym_implicit_statement_token4] = ACTIONS(1247),
    [aux_sym_save_statement_token1] = ACTIONS(1247),
    [aux_sym_private_statement_token1] = ACTIONS(1247),
    [aux_sym_public_statement_token1] = ACTIONS(1247),
    [aux_sym_namelist_statement_token1] = ACTIONS(1247),
    [aux_sym_common_statement_token1] = ACTIONS(1247),
    [anon_sym_DASH] = ACTIONS(1245),
    [aux_sym_import_statement_token1] = ACTIONS(1247),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1247),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1247),
    [aux_sym_variable_attributes_token1] = ACTIONS(1247),
    [aux_sym_variable_attributes_token2] = ACTIONS(1247),
    [aux_sym_variable_attributes_token3] = ACTIONS(1247),
    [aux_sym_variable_attributes_token4] = ACTIONS(1247),
    [aux_sym_variable_attributes_token5] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1247),
    [aux_sym_derived_type_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token2] = ACTIONS(1247),
    [aux_sym_type_qualifier_token3] = ACTIONS(1247),
    [aux_sym_type_qualifier_token4] = ACTIONS(1247),
    [aux_sym_type_qualifier_token5] = ACTIONS(1247),
    [aux_sym_type_qualifier_token9] = ACTIONS(1247),
    [aux_sym_type_qualifier_token10] = ACTIONS(1247),
    [aux_sym_type_qualifier_token11] = ACTIONS(1247),
    [aux_sym_type_qualifier_token12] = ACTIONS(1247),
    [aux_sym_type_qualifier_token13] = ACTIONS(1247),
    [aux_sym_type_qualifier_token14] = ACTIONS(1247),
    [aux_sym_type_qualifier_token15] = ACTIONS(1247),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token2] = ACTIONS(1247),
    [aux_sym_subroutine_call_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token2] = ACTIONS(1247),
    [aux_sym_keyword_statement_token3] = ACTIONS(1247),
    [aux_sym_keyword_statement_token4] = ACTIONS(1247),
    [aux_sym_keyword_statement_token6] = ACTIONS(1247),
    [aux_sym_keyword_statement_token7] = ACTIONS(1247),
    [aux_sym_include_statement_token1] = ACTIONS(1247),
    [aux_sym_data_statement_token1] = ACTIONS(1247),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1247),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1247),
    [aux_sym_end_if_statement_token1] = ACTIONS(1247),
    [aux_sym_elseif_clause_token2] = ACTIONS(1247),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1247),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token3] = ACTIONS(1247),
    [aux_sym_select_type_statement_token1] = ACTIONS(1247),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1247),
    [aux_sym_block_construct_token1] = ACTIONS(1247),
    [aux_sym_associate_statement_token1] = ACTIONS(1247),
    [aux_sym_format_statement_token1] = ACTIONS(1247),
    [aux_sym_print_statement_token1] = ACTIONS(1247),
    [aux_sym_open_statement_token1] = ACTIONS(1247),
    [aux_sym_close_statement_token1] = ACTIONS(1247),
    [aux_sym_inquire_statement_token1] = ACTIONS(1247),
    [aux_sym_enum_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token2] = ACTIONS(1247),
    [aux_sym_file_position_statement_token3] = ACTIONS(1247),
    [aux_sym_file_position_statement_token4] = ACTIONS(1247),
    [aux_sym_logical_expression_token5] = ACTIONS(1245),
    [anon_sym_PLUS] = ACTIONS(1245),
    [anon_sym_DOT] = ACTIONS(1247),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1245),
    [anon_sym_LBRACK] = ACTIONS(1245),
    [sym_boolean_literal] = ACTIONS(1245),
    [aux_sym_null_literal_token1] = ACTIONS(1247),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1247),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1245),
    [sym__float_literal] = ACTIONS(1245),
    [sym__boz_literal] = ACTIONS(1245),
    [sym_string_literal] = ACTIONS(1245),
  },
  [368] = {
    [sym_preproc_file_line] = STATE(368),
    [aux_sym_end_program_statement_token1] = ACTIONS(1147),
    [anon_sym_LPAREN] = ACTIONS(1147),
    [aux_sym_interface_statement_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1147),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1147),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1147),
    [aux_sym_contains_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token2] = ACTIONS(1147),
    [aux_sym_implicit_statement_token1] = ACTIONS(1147),
    [aux_sym_implicit_statement_token3] = ACTIONS(1147),
    [aux_sym_implicit_statement_token4] = ACTIONS(1147),
    [aux_sym_save_statement_token1] = ACTIONS(1147),
    [aux_sym_private_statement_token1] = ACTIONS(1147),
    [aux_sym_public_statement_token1] = ACTIONS(1147),
    [aux_sym_namelist_statement_token1] = ACTIONS(1147),
    [aux_sym_common_statement_token1] = ACTIONS(1147),
    [anon_sym_DASH] = ACTIONS(1145),
    [aux_sym_import_statement_token1] = ACTIONS(1147),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1147),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1147),
    [aux_sym_variable_attributes_token1] = ACTIONS(1147),
    [aux_sym_variable_attributes_token2] = ACTIONS(1147),
    [aux_sym_variable_attributes_token3] = ACTIONS(1147),
    [aux_sym_variable_attributes_token4] = ACTIONS(1147),
    [aux_sym_variable_attributes_token5] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1147),
    [aux_sym_derived_type_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token2] = ACTIONS(1147),
    [aux_sym_type_qualifier_token3] = ACTIONS(1147),
    [aux_sym_type_qualifier_token4] = ACTIONS(1147),
    [aux_sym_type_qualifier_token5] = ACTIONS(1147),
    [aux_sym_type_qualifier_token9] = ACTIONS(1147),
    [aux_sym_type_qualifier_token10] = ACTIONS(1147),
    [aux_sym_type_qualifier_token11] = ACTIONS(1147),
    [aux_sym_type_qualifier_token12] = ACTIONS(1147),
    [aux_sym_type_qualifier_token13] = ACTIONS(1147),
    [aux_sym_type_qualifier_token14] = ACTIONS(1147),
    [aux_sym_type_qualifier_token15] = ACTIONS(1147),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token2] = ACTIONS(1147),
    [aux_sym_subroutine_call_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token2] = ACTIONS(1147),
    [aux_sym_keyword_statement_token3] = ACTIONS(1147),
    [aux_sym_keyword_statement_token4] = ACTIONS(1147),
    [aux_sym_keyword_statement_token6] = ACTIONS(1147),
    [aux_sym_keyword_statement_token7] = ACTIONS(1147),
    [aux_sym_include_statement_token1] = ACTIONS(1147),
    [aux_sym_data_statement_token1] = ACTIONS(1147),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1147),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1147),
    [aux_sym_end_if_statement_token1] = ACTIONS(1147),
    [aux_sym_elseif_clause_token2] = ACTIONS(1147),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1147),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token3] = ACTIONS(1147),
    [aux_sym_select_type_statement_token1] = ACTIONS(1147),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1147),
    [aux_sym_block_construct_token1] = ACTIONS(1147),
    [aux_sym_associate_statement_token1] = ACTIONS(1147),
    [aux_sym_format_statement_token1] = ACTIONS(1147),
    [aux_sym_print_statement_token1] = ACTIONS(1147),
    [aux_sym_open_statement_token1] = ACTIONS(1147),
    [aux_sym_close_statement_token1] = ACTIONS(1147),
    [aux_sym_inquire_statement_token1] = ACTIONS(1147),
    [aux_sym_enum_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token2] = ACTIONS(1147),
    [aux_sym_file_position_statement_token3] = ACTIONS(1147),
    [aux_sym_file_position_statement_token4] = ACTIONS(1147),
    [aux_sym_logical_expression_token5] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1145),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [sym_boolean_literal] = ACTIONS(1145),
    [aux_sym_null_literal_token1] = ACTIONS(1147),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1147),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1145),
    [sym__float_literal] = ACTIONS(1145),
    [sym__boz_literal] = ACTIONS(1145),
    [sym_string_literal] = ACTIONS(1145),
  },
  [369] = {
    [sym_preproc_file_line] = STATE(369),
    [aux_sym_end_program_statement_token1] = ACTIONS(1287),
    [anon_sym_LPAREN] = ACTIONS(1287),
    [aux_sym_interface_statement_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1287),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1287),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1287),
    [aux_sym_contains_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token2] = ACTIONS(1287),
    [aux_sym_implicit_statement_token1] = ACTIONS(1287),
    [aux_sym_implicit_statement_token3] = ACTIONS(1287),
    [aux_sym_implicit_statement_token4] = ACTIONS(1287),
    [aux_sym_save_statement_token1] = ACTIONS(1287),
    [aux_sym_private_statement_token1] = ACTIONS(1287),
    [aux_sym_public_statement_token1] = ACTIONS(1287),
    [aux_sym_namelist_statement_token1] = ACTIONS(1287),
    [aux_sym_common_statement_token1] = ACTIONS(1287),
    [anon_sym_DASH] = ACTIONS(1289),
    [aux_sym_import_statement_token1] = ACTIONS(1287),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1287),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1287),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1287),
    [aux_sym_variable_attributes_token1] = ACTIONS(1287),
    [aux_sym_variable_attributes_token2] = ACTIONS(1287),
    [aux_sym_variable_attributes_token3] = ACTIONS(1287),
    [aux_sym_variable_attributes_token4] = ACTIONS(1287),
    [aux_sym_variable_attributes_token5] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1287),
    [aux_sym_derived_type_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token2] = ACTIONS(1287),
    [aux_sym_type_qualifier_token3] = ACTIONS(1287),
    [aux_sym_type_qualifier_token4] = ACTIONS(1287),
    [aux_sym_type_qualifier_token5] = ACTIONS(1287),
    [aux_sym_type_qualifier_token9] = ACTIONS(1287),
    [aux_sym_type_qualifier_token10] = ACTIONS(1287),
    [aux_sym_type_qualifier_token11] = ACTIONS(1287),
    [aux_sym_type_qualifier_token12] = ACTIONS(1287),
    [aux_sym_type_qualifier_token13] = ACTIONS(1287),
    [aux_sym_type_qualifier_token14] = ACTIONS(1287),
    [aux_sym_type_qualifier_token15] = ACTIONS(1287),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token2] = ACTIONS(1287),
    [aux_sym_subroutine_call_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token2] = ACTIONS(1287),
    [aux_sym_keyword_statement_token3] = ACTIONS(1287),
    [aux_sym_keyword_statement_token4] = ACTIONS(1287),
    [aux_sym_keyword_statement_token6] = ACTIONS(1287),
    [aux_sym_keyword_statement_token7] = ACTIONS(1287),
    [aux_sym_include_statement_token1] = ACTIONS(1287),
    [aux_sym_data_statement_token1] = ACTIONS(1287),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1287),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1287),
    [aux_sym_end_if_statement_token1] = ACTIONS(1287),
    [aux_sym_elseif_clause_token2] = ACTIONS(1287),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1287),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token3] = ACTIONS(1287),
    [aux_sym_select_type_statement_token1] = ACTIONS(1287),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1287),
    [aux_sym_block_construct_token1] = ACTIONS(1287),
    [aux_sym_associate_statement_token1] = ACTIONS(1287),
    [aux_sym_format_statement_token1] = ACTIONS(1287),
    [aux_sym_print_statement_token1] = ACTIONS(1287),
    [aux_sym_open_statement_token1] = ACTIONS(1287),
    [aux_sym_close_statement_token1] = ACTIONS(1287),
    [aux_sym_inquire_statement_token1] = ACTIONS(1287),
    [aux_sym_enum_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token2] = ACTIONS(1287),
    [aux_sym_file_position_statement_token3] = ACTIONS(1287),
    [aux_sym_file_position_statement_token4] = ACTIONS(1287),
    [aux_sym_logical_expression_token5] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1289),
    [anon_sym_DOT] = ACTIONS(1287),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1289),
    [anon_sym_LBRACK] = ACTIONS(1289),
    [sym_boolean_literal] = ACTIONS(1289),
    [aux_sym_null_literal_token1] = ACTIONS(1287),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1287),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1289),
    [sym__float_literal] = ACTIONS(1289),
    [sym__boz_literal] = ACTIONS(1289),
    [sym_string_literal] = ACTIONS(1289),
  },
  [370] = {
    [sym_preproc_file_line] = STATE(370),
    [aux_sym_end_program_statement_token1] = ACTIONS(1119),
    [anon_sym_LPAREN] = ACTIONS(1119),
    [aux_sym_interface_statement_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1119),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1119),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1119),
    [aux_sym_contains_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token2] = ACTIONS(1119),
    [aux_sym_implicit_statement_token1] = ACTIONS(1119),
    [aux_sym_implicit_statement_token3] = ACTIONS(1119),
    [aux_sym_implicit_statement_token4] = ACTIONS(1119),
    [aux_sym_save_statement_token1] = ACTIONS(1119),
    [aux_sym_private_statement_token1] = ACTIONS(1119),
    [aux_sym_public_statement_token1] = ACTIONS(1119),
    [aux_sym_namelist_statement_token1] = ACTIONS(1119),
    [aux_sym_common_statement_token1] = ACTIONS(1119),
    [anon_sym_DASH] = ACTIONS(1117),
    [aux_sym_import_statement_token1] = ACTIONS(1119),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1119),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1119),
    [aux_sym_variable_attributes_token1] = ACTIONS(1119),
    [aux_sym_variable_attributes_token2] = ACTIONS(1119),
    [aux_sym_variable_attributes_token3] = ACTIONS(1119),
    [aux_sym_variable_attributes_token4] = ACTIONS(1119),
    [aux_sym_variable_attributes_token5] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1119),
    [aux_sym_derived_type_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token2] = ACTIONS(1119),
    [aux_sym_type_qualifier_token3] = ACTIONS(1119),
    [aux_sym_type_qualifier_token4] = ACTIONS(1119),
    [aux_sym_type_qualifier_token5] = ACTIONS(1119),
    [aux_sym_type_qualifier_token9] = ACTIONS(1119),
    [aux_sym_type_qualifier_token10] = ACTIONS(1119),
    [aux_sym_type_qualifier_token11] = ACTIONS(1119),
    [aux_sym_type_qualifier_token12] = ACTIONS(1119),
    [aux_sym_type_qualifier_token13] = ACTIONS(1119),
    [aux_sym_type_qualifier_token14] = ACTIONS(1119),
    [aux_sym_type_qualifier_token15] = ACTIONS(1119),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token2] = ACTIONS(1119),
    [aux_sym_subroutine_call_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token2] = ACTIONS(1119),
    [aux_sym_keyword_statement_token3] = ACTIONS(1119),
    [aux_sym_keyword_statement_token4] = ACTIONS(1119),
    [aux_sym_keyword_statement_token6] = ACTIONS(1119),
    [aux_sym_keyword_statement_token7] = ACTIONS(1119),
    [aux_sym_include_statement_token1] = ACTIONS(1119),
    [aux_sym_data_statement_token1] = ACTIONS(1119),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1119),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1119),
    [aux_sym_end_if_statement_token1] = ACTIONS(1119),
    [aux_sym_elseif_clause_token2] = ACTIONS(1119),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1119),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token3] = ACTIONS(1119),
    [aux_sym_select_type_statement_token1] = ACTIONS(1119),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1119),
    [aux_sym_block_construct_token1] = ACTIONS(1119),
    [aux_sym_associate_statement_token1] = ACTIONS(1119),
    [aux_sym_format_statement_token1] = ACTIONS(1119),
    [aux_sym_print_statement_token1] = ACTIONS(1119),
    [aux_sym_open_statement_token1] = ACTIONS(1119),
    [aux_sym_close_statement_token1] = ACTIONS(1119),
    [aux_sym_inquire_statement_token1] = ACTIONS(1119),
    [aux_sym_enum_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token2] = ACTIONS(1119),
    [aux_sym_file_position_statement_token3] = ACTIONS(1119),
    [aux_sym_file_position_statement_token4] = ACTIONS(1119),
    [aux_sym_logical_expression_token5] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [sym_boolean_literal] = ACTIONS(1117),
    [aux_sym_null_literal_token1] = ACTIONS(1119),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1119),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1117),
    [sym__float_literal] = ACTIONS(1117),
    [sym__boz_literal] = ACTIONS(1117),
    [sym_string_literal] = ACTIONS(1117),
  },
  [371] = {
    [sym_preproc_file_line] = STATE(371),
    [aux_sym_end_program_statement_token1] = ACTIONS(1335),
    [anon_sym_LPAREN] = ACTIONS(1335),
    [aux_sym_interface_statement_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1335),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1335),
    [aux_sym_end_function_statement_token1] = ACTIONS(1335),
    [aux_sym_contains_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token2] = ACTIONS(1335),
    [aux_sym_implicit_statement_token1] = ACTIONS(1335),
    [aux_sym_implicit_statement_token3] = ACTIONS(1335),
    [aux_sym_implicit_statement_token4] = ACTIONS(1335),
    [aux_sym_save_statement_token1] = ACTIONS(1335),
    [aux_sym_private_statement_token1] = ACTIONS(1335),
    [aux_sym_public_statement_token1] = ACTIONS(1335),
    [aux_sym_namelist_statement_token1] = ACTIONS(1335),
    [aux_sym_common_statement_token1] = ACTIONS(1335),
    [anon_sym_DASH] = ACTIONS(1337),
    [aux_sym_import_statement_token1] = ACTIONS(1335),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1335),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1335),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1335),
    [aux_sym_variable_attributes_token1] = ACTIONS(1335),
    [aux_sym_variable_attributes_token2] = ACTIONS(1335),
    [aux_sym_variable_attributes_token3] = ACTIONS(1335),
    [aux_sym_variable_attributes_token4] = ACTIONS(1335),
    [aux_sym_variable_attributes_token5] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1335),
    [aux_sym_derived_type_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token2] = ACTIONS(1335),
    [aux_sym_type_qualifier_token3] = ACTIONS(1335),
    [aux_sym_type_qualifier_token4] = ACTIONS(1335),
    [aux_sym_type_qualifier_token5] = ACTIONS(1335),
    [aux_sym_type_qualifier_token9] = ACTIONS(1335),
    [aux_sym_type_qualifier_token10] = ACTIONS(1335),
    [aux_sym_type_qualifier_token11] = ACTIONS(1335),
    [aux_sym_type_qualifier_token12] = ACTIONS(1335),
    [aux_sym_type_qualifier_token13] = ACTIONS(1335),
    [aux_sym_type_qualifier_token14] = ACTIONS(1335),
    [aux_sym_type_qualifier_token15] = ACTIONS(1335),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token2] = ACTIONS(1335),
    [aux_sym_subroutine_call_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token2] = ACTIONS(1335),
    [aux_sym_keyword_statement_token3] = ACTIONS(1335),
    [aux_sym_keyword_statement_token4] = ACTIONS(1335),
    [aux_sym_keyword_statement_token6] = ACTIONS(1335),
    [aux_sym_keyword_statement_token7] = ACTIONS(1335),
    [aux_sym_include_statement_token1] = ACTIONS(1335),
    [aux_sym_data_statement_token1] = ACTIONS(1335),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1335),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1335),
    [aux_sym_end_if_statement_token1] = ACTIONS(1335),
    [aux_sym_elseif_clause_token2] = ACTIONS(1335),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1335),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token3] = ACTIONS(1335),
    [aux_sym_select_type_statement_token1] = ACTIONS(1335),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1335),
    [aux_sym_block_construct_token1] = ACTIONS(1335),
    [aux_sym_associate_statement_token1] = ACTIONS(1335),
    [aux_sym_format_statement_token1] = ACTIONS(1335),
    [aux_sym_print_statement_token1] = ACTIONS(1335),
    [aux_sym_open_statement_token1] = ACTIONS(1335),
    [aux_sym_close_statement_token1] = ACTIONS(1335),
    [aux_sym_inquire_statement_token1] = ACTIONS(1335),
    [aux_sym_enum_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token2] = ACTIONS(1335),
    [aux_sym_file_position_statement_token3] = ACTIONS(1335),
    [aux_sym_file_position_statement_token4] = ACTIONS(1335),
    [aux_sym_logical_expression_token5] = ACTIONS(1337),
    [anon_sym_PLUS] = ACTIONS(1337),
    [anon_sym_DOT] = ACTIONS(1335),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1337),
    [anon_sym_LBRACK] = ACTIONS(1337),
    [sym_boolean_literal] = ACTIONS(1337),
    [aux_sym_null_literal_token1] = ACTIONS(1335),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1335),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1337),
    [sym__float_literal] = ACTIONS(1337),
    [sym__boz_literal] = ACTIONS(1337),
    [sym_string_literal] = ACTIONS(1337),
  },
  [372] = {
    [sym_preproc_file_line] = STATE(372),
    [aux_sym_end_program_statement_token1] = ACTIONS(1339),
    [anon_sym_LPAREN] = ACTIONS(1339),
    [aux_sym_interface_statement_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1339),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1339),
    [aux_sym_end_function_statement_token1] = ACTIONS(1339),
    [aux_sym_contains_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token2] = ACTIONS(1339),
    [aux_sym_implicit_statement_token1] = ACTIONS(1339),
    [aux_sym_implicit_statement_token3] = ACTIONS(1339),
    [aux_sym_implicit_statement_token4] = ACTIONS(1339),
    [aux_sym_save_statement_token1] = ACTIONS(1339),
    [aux_sym_private_statement_token1] = ACTIONS(1339),
    [aux_sym_public_statement_token1] = ACTIONS(1339),
    [aux_sym_namelist_statement_token1] = ACTIONS(1339),
    [aux_sym_common_statement_token1] = ACTIONS(1339),
    [anon_sym_DASH] = ACTIONS(1341),
    [aux_sym_import_statement_token1] = ACTIONS(1339),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1339),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1339),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1339),
    [aux_sym_variable_attributes_token1] = ACTIONS(1339),
    [aux_sym_variable_attributes_token2] = ACTIONS(1339),
    [aux_sym_variable_attributes_token3] = ACTIONS(1339),
    [aux_sym_variable_attributes_token4] = ACTIONS(1339),
    [aux_sym_variable_attributes_token5] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1339),
    [aux_sym_derived_type_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token2] = ACTIONS(1339),
    [aux_sym_type_qualifier_token3] = ACTIONS(1339),
    [aux_sym_type_qualifier_token4] = ACTIONS(1339),
    [aux_sym_type_qualifier_token5] = ACTIONS(1339),
    [aux_sym_type_qualifier_token9] = ACTIONS(1339),
    [aux_sym_type_qualifier_token10] = ACTIONS(1339),
    [aux_sym_type_qualifier_token11] = ACTIONS(1339),
    [aux_sym_type_qualifier_token12] = ACTIONS(1339),
    [aux_sym_type_qualifier_token13] = ACTIONS(1339),
    [aux_sym_type_qualifier_token14] = ACTIONS(1339),
    [aux_sym_type_qualifier_token15] = ACTIONS(1339),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token2] = ACTIONS(1339),
    [aux_sym_subroutine_call_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token2] = ACTIONS(1339),
    [aux_sym_keyword_statement_token3] = ACTIONS(1339),
    [aux_sym_keyword_statement_token4] = ACTIONS(1339),
    [aux_sym_keyword_statement_token6] = ACTIONS(1339),
    [aux_sym_keyword_statement_token7] = ACTIONS(1339),
    [aux_sym_include_statement_token1] = ACTIONS(1339),
    [aux_sym_data_statement_token1] = ACTIONS(1339),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1339),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1339),
    [aux_sym_end_if_statement_token1] = ACTIONS(1339),
    [aux_sym_elseif_clause_token2] = ACTIONS(1339),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1339),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token3] = ACTIONS(1339),
    [aux_sym_select_type_statement_token1] = ACTIONS(1339),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1339),
    [aux_sym_block_construct_token1] = ACTIONS(1339),
    [aux_sym_associate_statement_token1] = ACTIONS(1339),
    [aux_sym_format_statement_token1] = ACTIONS(1339),
    [aux_sym_print_statement_token1] = ACTIONS(1339),
    [aux_sym_open_statement_token1] = ACTIONS(1339),
    [aux_sym_close_statement_token1] = ACTIONS(1339),
    [aux_sym_inquire_statement_token1] = ACTIONS(1339),
    [aux_sym_enum_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token2] = ACTIONS(1339),
    [aux_sym_file_position_statement_token3] = ACTIONS(1339),
    [aux_sym_file_position_statement_token4] = ACTIONS(1339),
    [aux_sym_logical_expression_token5] = ACTIONS(1341),
    [anon_sym_PLUS] = ACTIONS(1341),
    [anon_sym_DOT] = ACTIONS(1339),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1341),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [sym_boolean_literal] = ACTIONS(1341),
    [aux_sym_null_literal_token1] = ACTIONS(1339),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1339),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1341),
    [sym__float_literal] = ACTIONS(1341),
    [sym__boz_literal] = ACTIONS(1341),
    [sym_string_literal] = ACTIONS(1341),
  },
  [373] = {
    [sym_preproc_file_line] = STATE(373),
    [aux_sym_end_program_statement_token1] = ACTIONS(1147),
    [aux_sym_end_program_statement_token2] = ACTIONS(1147),
    [anon_sym_LPAREN] = ACTIONS(1147),
    [aux_sym_interface_statement_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1147),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1147),
    [aux_sym_contains_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token2] = ACTIONS(1147),
    [aux_sym_implicit_statement_token1] = ACTIONS(1147),
    [aux_sym_implicit_statement_token3] = ACTIONS(1147),
    [aux_sym_implicit_statement_token4] = ACTIONS(1147),
    [aux_sym_save_statement_token1] = ACTIONS(1147),
    [aux_sym_private_statement_token1] = ACTIONS(1147),
    [aux_sym_public_statement_token1] = ACTIONS(1147),
    [aux_sym_namelist_statement_token1] = ACTIONS(1147),
    [aux_sym_common_statement_token1] = ACTIONS(1147),
    [anon_sym_DASH] = ACTIONS(1145),
    [aux_sym_import_statement_token1] = ACTIONS(1147),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1147),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1147),
    [aux_sym_variable_attributes_token1] = ACTIONS(1147),
    [aux_sym_variable_attributes_token2] = ACTIONS(1147),
    [aux_sym_variable_attributes_token3] = ACTIONS(1147),
    [aux_sym_variable_attributes_token4] = ACTIONS(1147),
    [aux_sym_variable_attributes_token5] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1147),
    [aux_sym_derived_type_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token2] = ACTIONS(1147),
    [aux_sym_type_qualifier_token3] = ACTIONS(1147),
    [aux_sym_type_qualifier_token4] = ACTIONS(1147),
    [aux_sym_type_qualifier_token5] = ACTIONS(1147),
    [aux_sym_type_qualifier_token9] = ACTIONS(1147),
    [aux_sym_type_qualifier_token10] = ACTIONS(1147),
    [aux_sym_type_qualifier_token11] = ACTIONS(1147),
    [aux_sym_type_qualifier_token12] = ACTIONS(1147),
    [aux_sym_type_qualifier_token13] = ACTIONS(1147),
    [aux_sym_type_qualifier_token14] = ACTIONS(1147),
    [aux_sym_type_qualifier_token15] = ACTIONS(1147),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token2] = ACTIONS(1147),
    [aux_sym_subroutine_call_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token2] = ACTIONS(1147),
    [aux_sym_keyword_statement_token3] = ACTIONS(1147),
    [aux_sym_keyword_statement_token4] = ACTIONS(1147),
    [aux_sym_keyword_statement_token6] = ACTIONS(1147),
    [aux_sym_keyword_statement_token7] = ACTIONS(1147),
    [aux_sym_include_statement_token1] = ACTIONS(1147),
    [aux_sym_data_statement_token1] = ACTIONS(1147),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1147),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1147),
    [aux_sym_end_if_statement_token1] = ACTIONS(1147),
    [aux_sym_elseif_clause_token2] = ACTIONS(1147),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1147),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token3] = ACTIONS(1147),
    [aux_sym_select_type_statement_token1] = ACTIONS(1147),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1147),
    [aux_sym_block_construct_token1] = ACTIONS(1147),
    [aux_sym_associate_statement_token1] = ACTIONS(1147),
    [aux_sym_format_statement_token1] = ACTIONS(1147),
    [aux_sym_print_statement_token1] = ACTIONS(1147),
    [aux_sym_open_statement_token1] = ACTIONS(1147),
    [aux_sym_close_statement_token1] = ACTIONS(1147),
    [aux_sym_inquire_statement_token1] = ACTIONS(1147),
    [aux_sym_enum_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token2] = ACTIONS(1147),
    [aux_sym_file_position_statement_token3] = ACTIONS(1147),
    [aux_sym_file_position_statement_token4] = ACTIONS(1147),
    [aux_sym_logical_expression_token5] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1145),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [sym_boolean_literal] = ACTIONS(1145),
    [aux_sym_null_literal_token1] = ACTIONS(1147),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1147),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1145),
    [sym__float_literal] = ACTIONS(1145),
    [sym__boz_literal] = ACTIONS(1145),
    [sym_string_literal] = ACTIONS(1145),
  },
  [374] = {
    [sym_preproc_file_line] = STATE(374),
    [aux_sym_end_program_statement_token1] = ACTIONS(1343),
    [anon_sym_LPAREN] = ACTIONS(1343),
    [aux_sym_interface_statement_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1343),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1343),
    [aux_sym_end_function_statement_token1] = ACTIONS(1343),
    [aux_sym_contains_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token2] = ACTIONS(1343),
    [aux_sym_implicit_statement_token1] = ACTIONS(1343),
    [aux_sym_implicit_statement_token3] = ACTIONS(1343),
    [aux_sym_implicit_statement_token4] = ACTIONS(1343),
    [aux_sym_save_statement_token1] = ACTIONS(1343),
    [aux_sym_private_statement_token1] = ACTIONS(1343),
    [aux_sym_public_statement_token1] = ACTIONS(1343),
    [aux_sym_namelist_statement_token1] = ACTIONS(1343),
    [aux_sym_common_statement_token1] = ACTIONS(1343),
    [anon_sym_DASH] = ACTIONS(1345),
    [aux_sym_import_statement_token1] = ACTIONS(1343),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1343),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1343),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1343),
    [aux_sym_variable_attributes_token1] = ACTIONS(1343),
    [aux_sym_variable_attributes_token2] = ACTIONS(1343),
    [aux_sym_variable_attributes_token3] = ACTIONS(1343),
    [aux_sym_variable_attributes_token4] = ACTIONS(1343),
    [aux_sym_variable_attributes_token5] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1343),
    [aux_sym_derived_type_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token2] = ACTIONS(1343),
    [aux_sym_type_qualifier_token3] = ACTIONS(1343),
    [aux_sym_type_qualifier_token4] = ACTIONS(1343),
    [aux_sym_type_qualifier_token5] = ACTIONS(1343),
    [aux_sym_type_qualifier_token9] = ACTIONS(1343),
    [aux_sym_type_qualifier_token10] = ACTIONS(1343),
    [aux_sym_type_qualifier_token11] = ACTIONS(1343),
    [aux_sym_type_qualifier_token12] = ACTIONS(1343),
    [aux_sym_type_qualifier_token13] = ACTIONS(1343),
    [aux_sym_type_qualifier_token14] = ACTIONS(1343),
    [aux_sym_type_qualifier_token15] = ACTIONS(1343),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token2] = ACTIONS(1343),
    [aux_sym_subroutine_call_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token2] = ACTIONS(1343),
    [aux_sym_keyword_statement_token3] = ACTIONS(1343),
    [aux_sym_keyword_statement_token4] = ACTIONS(1343),
    [aux_sym_keyword_statement_token6] = ACTIONS(1343),
    [aux_sym_keyword_statement_token7] = ACTIONS(1343),
    [aux_sym_include_statement_token1] = ACTIONS(1343),
    [aux_sym_data_statement_token1] = ACTIONS(1343),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1343),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1343),
    [aux_sym_end_if_statement_token1] = ACTIONS(1343),
    [aux_sym_elseif_clause_token2] = ACTIONS(1343),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1343),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token3] = ACTIONS(1343),
    [aux_sym_select_type_statement_token1] = ACTIONS(1343),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1343),
    [aux_sym_block_construct_token1] = ACTIONS(1343),
    [aux_sym_associate_statement_token1] = ACTIONS(1343),
    [aux_sym_format_statement_token1] = ACTIONS(1343),
    [aux_sym_print_statement_token1] = ACTIONS(1343),
    [aux_sym_open_statement_token1] = ACTIONS(1343),
    [aux_sym_close_statement_token1] = ACTIONS(1343),
    [aux_sym_inquire_statement_token1] = ACTIONS(1343),
    [aux_sym_enum_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token2] = ACTIONS(1343),
    [aux_sym_file_position_statement_token3] = ACTIONS(1343),
    [aux_sym_file_position_statement_token4] = ACTIONS(1343),
    [aux_sym_logical_expression_token5] = ACTIONS(1345),
    [anon_sym_PLUS] = ACTIONS(1345),
    [anon_sym_DOT] = ACTIONS(1343),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1345),
    [anon_sym_LBRACK] = ACTIONS(1345),
    [sym_boolean_literal] = ACTIONS(1345),
    [aux_sym_null_literal_token1] = ACTIONS(1343),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1343),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1345),
    [sym__float_literal] = ACTIONS(1345),
    [sym__boz_literal] = ACTIONS(1345),
    [sym_string_literal] = ACTIONS(1345),
  },
  [375] = {
    [sym_preproc_file_line] = STATE(375),
    [aux_sym_end_program_statement_token1] = ACTIONS(1315),
    [anon_sym_LPAREN] = ACTIONS(1315),
    [aux_sym_interface_statement_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1315),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1315),
    [aux_sym_end_function_statement_token1] = ACTIONS(1315),
    [aux_sym_contains_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token2] = ACTIONS(1315),
    [aux_sym_implicit_statement_token1] = ACTIONS(1315),
    [aux_sym_implicit_statement_token3] = ACTIONS(1315),
    [aux_sym_implicit_statement_token4] = ACTIONS(1315),
    [aux_sym_save_statement_token1] = ACTIONS(1315),
    [aux_sym_private_statement_token1] = ACTIONS(1315),
    [aux_sym_public_statement_token1] = ACTIONS(1315),
    [aux_sym_namelist_statement_token1] = ACTIONS(1315),
    [aux_sym_common_statement_token1] = ACTIONS(1315),
    [anon_sym_DASH] = ACTIONS(1317),
    [aux_sym_import_statement_token1] = ACTIONS(1315),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1315),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1315),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1315),
    [aux_sym_variable_attributes_token1] = ACTIONS(1315),
    [aux_sym_variable_attributes_token2] = ACTIONS(1315),
    [aux_sym_variable_attributes_token3] = ACTIONS(1315),
    [aux_sym_variable_attributes_token4] = ACTIONS(1315),
    [aux_sym_variable_attributes_token5] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1315),
    [aux_sym_derived_type_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token2] = ACTIONS(1315),
    [aux_sym_type_qualifier_token3] = ACTIONS(1315),
    [aux_sym_type_qualifier_token4] = ACTIONS(1315),
    [aux_sym_type_qualifier_token5] = ACTIONS(1315),
    [aux_sym_type_qualifier_token9] = ACTIONS(1315),
    [aux_sym_type_qualifier_token10] = ACTIONS(1315),
    [aux_sym_type_qualifier_token11] = ACTIONS(1315),
    [aux_sym_type_qualifier_token12] = ACTIONS(1315),
    [aux_sym_type_qualifier_token13] = ACTIONS(1315),
    [aux_sym_type_qualifier_token14] = ACTIONS(1315),
    [aux_sym_type_qualifier_token15] = ACTIONS(1315),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token2] = ACTIONS(1315),
    [aux_sym_subroutine_call_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token2] = ACTIONS(1315),
    [aux_sym_keyword_statement_token3] = ACTIONS(1315),
    [aux_sym_keyword_statement_token4] = ACTIONS(1315),
    [aux_sym_keyword_statement_token6] = ACTIONS(1315),
    [aux_sym_keyword_statement_token7] = ACTIONS(1315),
    [aux_sym_include_statement_token1] = ACTIONS(1315),
    [aux_sym_data_statement_token1] = ACTIONS(1315),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1315),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1315),
    [aux_sym_end_if_statement_token1] = ACTIONS(1315),
    [aux_sym_elseif_clause_token2] = ACTIONS(1315),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1315),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token3] = ACTIONS(1315),
    [aux_sym_select_type_statement_token1] = ACTIONS(1315),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1315),
    [aux_sym_block_construct_token1] = ACTIONS(1315),
    [aux_sym_associate_statement_token1] = ACTIONS(1315),
    [aux_sym_format_statement_token1] = ACTIONS(1315),
    [aux_sym_print_statement_token1] = ACTIONS(1315),
    [aux_sym_open_statement_token1] = ACTIONS(1315),
    [aux_sym_close_statement_token1] = ACTIONS(1315),
    [aux_sym_inquire_statement_token1] = ACTIONS(1315),
    [aux_sym_enum_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token2] = ACTIONS(1315),
    [aux_sym_file_position_statement_token3] = ACTIONS(1315),
    [aux_sym_file_position_statement_token4] = ACTIONS(1315),
    [aux_sym_logical_expression_token5] = ACTIONS(1317),
    [anon_sym_PLUS] = ACTIONS(1317),
    [anon_sym_DOT] = ACTIONS(1315),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1317),
    [anon_sym_LBRACK] = ACTIONS(1317),
    [sym_boolean_literal] = ACTIONS(1317),
    [aux_sym_null_literal_token1] = ACTIONS(1315),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1315),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1317),
    [sym__float_literal] = ACTIONS(1317),
    [sym__boz_literal] = ACTIONS(1317),
    [sym_string_literal] = ACTIONS(1317),
  },
  [376] = {
    [sym_preproc_file_line] = STATE(376),
    [aux_sym_end_program_statement_token1] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1259),
    [aux_sym_interface_statement_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1259),
    [aux_sym_end_subroutine_statement_token1] = ACTIONS(1259),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1259),
    [aux_sym_contains_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token2] = ACTIONS(1259),
    [aux_sym_implicit_statement_token1] = ACTIONS(1259),
    [aux_sym_implicit_statement_token3] = ACTIONS(1259),
    [aux_sym_implicit_statement_token4] = ACTIONS(1259),
    [aux_sym_save_statement_token1] = ACTIONS(1259),
    [aux_sym_private_statement_token1] = ACTIONS(1259),
    [aux_sym_public_statement_token1] = ACTIONS(1259),
    [aux_sym_namelist_statement_token1] = ACTIONS(1259),
    [aux_sym_common_statement_token1] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [aux_sym_import_statement_token1] = ACTIONS(1259),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1259),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1259),
    [aux_sym_variable_attributes_token1] = ACTIONS(1259),
    [aux_sym_variable_attributes_token2] = ACTIONS(1259),
    [aux_sym_variable_attributes_token3] = ACTIONS(1259),
    [aux_sym_variable_attributes_token4] = ACTIONS(1259),
    [aux_sym_variable_attributes_token5] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1259),
    [aux_sym_derived_type_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token2] = ACTIONS(1259),
    [aux_sym_type_qualifier_token3] = ACTIONS(1259),
    [aux_sym_type_qualifier_token4] = ACTIONS(1259),
    [aux_sym_type_qualifier_token5] = ACTIONS(1259),
    [aux_sym_type_qualifier_token9] = ACTIONS(1259),
    [aux_sym_type_qualifier_token10] = ACTIONS(1259),
    [aux_sym_type_qualifier_token11] = ACTIONS(1259),
    [aux_sym_type_qualifier_token12] = ACTIONS(1259),
    [aux_sym_type_qualifier_token13] = ACTIONS(1259),
    [aux_sym_type_qualifier_token14] = ACTIONS(1259),
    [aux_sym_type_qualifier_token15] = ACTIONS(1259),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token2] = ACTIONS(1259),
    [aux_sym_subroutine_call_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token2] = ACTIONS(1259),
    [aux_sym_keyword_statement_token3] = ACTIONS(1259),
    [aux_sym_keyword_statement_token4] = ACTIONS(1259),
    [aux_sym_keyword_statement_token6] = ACTIONS(1259),
    [aux_sym_keyword_statement_token7] = ACTIONS(1259),
    [aux_sym_include_statement_token1] = ACTIONS(1259),
    [aux_sym_data_statement_token1] = ACTIONS(1259),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1259),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1259),
    [aux_sym_end_if_statement_token1] = ACTIONS(1259),
    [aux_sym_elseif_clause_token2] = ACTIONS(1259),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1259),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token3] = ACTIONS(1259),
    [aux_sym_select_type_statement_token1] = ACTIONS(1259),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1259),
    [aux_sym_block_construct_token1] = ACTIONS(1259),
    [aux_sym_associate_statement_token1] = ACTIONS(1259),
    [aux_sym_format_statement_token1] = ACTIONS(1259),
    [aux_sym_print_statement_token1] = ACTIONS(1259),
    [aux_sym_open_statement_token1] = ACTIONS(1259),
    [aux_sym_close_statement_token1] = ACTIONS(1259),
    [aux_sym_inquire_statement_token1] = ACTIONS(1259),
    [aux_sym_enum_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token2] = ACTIONS(1259),
    [aux_sym_file_position_statement_token3] = ACTIONS(1259),
    [aux_sym_file_position_statement_token4] = ACTIONS(1259),
    [aux_sym_logical_expression_token5] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [sym_boolean_literal] = ACTIONS(1257),
    [aux_sym_null_literal_token1] = ACTIONS(1259),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1259),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1257),
    [sym__float_literal] = ACTIONS(1257),
    [sym__boz_literal] = ACTIONS(1257),
    [sym_string_literal] = ACTIONS(1257),
  },
  [377] = {
    [sym_preproc_file_line] = STATE(377),
    [aux_sym_end_program_statement_token1] = ACTIONS(1355),
    [anon_sym_LPAREN] = ACTIONS(1355),
    [aux_sym_interface_statement_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1355),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1355),
    [aux_sym_end_function_statement_token1] = ACTIONS(1355),
    [aux_sym_contains_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token2] = ACTIONS(1355),
    [aux_sym_implicit_statement_token1] = ACTIONS(1355),
    [aux_sym_implicit_statement_token3] = ACTIONS(1355),
    [aux_sym_implicit_statement_token4] = ACTIONS(1355),
    [aux_sym_save_statement_token1] = ACTIONS(1355),
    [aux_sym_private_statement_token1] = ACTIONS(1355),
    [aux_sym_public_statement_token1] = ACTIONS(1355),
    [aux_sym_namelist_statement_token1] = ACTIONS(1355),
    [aux_sym_common_statement_token1] = ACTIONS(1355),
    [anon_sym_DASH] = ACTIONS(1357),
    [aux_sym_import_statement_token1] = ACTIONS(1355),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1355),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1355),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1355),
    [aux_sym_variable_attributes_token1] = ACTIONS(1355),
    [aux_sym_variable_attributes_token2] = ACTIONS(1355),
    [aux_sym_variable_attributes_token3] = ACTIONS(1355),
    [aux_sym_variable_attributes_token4] = ACTIONS(1355),
    [aux_sym_variable_attributes_token5] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1355),
    [aux_sym_derived_type_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token2] = ACTIONS(1355),
    [aux_sym_type_qualifier_token3] = ACTIONS(1355),
    [aux_sym_type_qualifier_token4] = ACTIONS(1355),
    [aux_sym_type_qualifier_token5] = ACTIONS(1355),
    [aux_sym_type_qualifier_token9] = ACTIONS(1355),
    [aux_sym_type_qualifier_token10] = ACTIONS(1355),
    [aux_sym_type_qualifier_token11] = ACTIONS(1355),
    [aux_sym_type_qualifier_token12] = ACTIONS(1355),
    [aux_sym_type_qualifier_token13] = ACTIONS(1355),
    [aux_sym_type_qualifier_token14] = ACTIONS(1355),
    [aux_sym_type_qualifier_token15] = ACTIONS(1355),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token2] = ACTIONS(1355),
    [aux_sym_subroutine_call_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token2] = ACTIONS(1355),
    [aux_sym_keyword_statement_token3] = ACTIONS(1355),
    [aux_sym_keyword_statement_token4] = ACTIONS(1355),
    [aux_sym_keyword_statement_token6] = ACTIONS(1355),
    [aux_sym_keyword_statement_token7] = ACTIONS(1355),
    [aux_sym_include_statement_token1] = ACTIONS(1355),
    [aux_sym_data_statement_token1] = ACTIONS(1355),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1355),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1355),
    [aux_sym_end_if_statement_token1] = ACTIONS(1355),
    [aux_sym_elseif_clause_token2] = ACTIONS(1355),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1355),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token3] = ACTIONS(1355),
    [aux_sym_select_type_statement_token1] = ACTIONS(1355),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1355),
    [aux_sym_block_construct_token1] = ACTIONS(1355),
    [aux_sym_associate_statement_token1] = ACTIONS(1355),
    [aux_sym_format_statement_token1] = ACTIONS(1355),
    [aux_sym_print_statement_token1] = ACTIONS(1355),
    [aux_sym_open_statement_token1] = ACTIONS(1355),
    [aux_sym_close_statement_token1] = ACTIONS(1355),
    [aux_sym_inquire_statement_token1] = ACTIONS(1355),
    [aux_sym_enum_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token2] = ACTIONS(1355),
    [aux_sym_file_position_statement_token3] = ACTIONS(1355),
    [aux_sym_file_position_statement_token4] = ACTIONS(1355),
    [aux_sym_logical_expression_token5] = ACTIONS(1357),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_DOT] = ACTIONS(1355),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1357),
    [anon_sym_LBRACK] = ACTIONS(1357),
    [sym_boolean_literal] = ACTIONS(1357),
    [aux_sym_null_literal_token1] = ACTIONS(1355),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1355),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1357),
    [sym__float_literal] = ACTIONS(1357),
    [sym__boz_literal] = ACTIONS(1357),
    [sym_string_literal] = ACTIONS(1357),
  },
  [378] = {
    [sym_preproc_file_line] = STATE(378),
    [aux_sym_end_program_statement_token1] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [aux_sym_interface_statement_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1359),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1359),
    [aux_sym_end_function_statement_token1] = ACTIONS(1359),
    [aux_sym_contains_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token2] = ACTIONS(1359),
    [aux_sym_implicit_statement_token1] = ACTIONS(1359),
    [aux_sym_implicit_statement_token3] = ACTIONS(1359),
    [aux_sym_implicit_statement_token4] = ACTIONS(1359),
    [aux_sym_save_statement_token1] = ACTIONS(1359),
    [aux_sym_private_statement_token1] = ACTIONS(1359),
    [aux_sym_public_statement_token1] = ACTIONS(1359),
    [aux_sym_namelist_statement_token1] = ACTIONS(1359),
    [aux_sym_common_statement_token1] = ACTIONS(1359),
    [anon_sym_DASH] = ACTIONS(1361),
    [aux_sym_import_statement_token1] = ACTIONS(1359),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1359),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1359),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1359),
    [aux_sym_variable_attributes_token1] = ACTIONS(1359),
    [aux_sym_variable_attributes_token2] = ACTIONS(1359),
    [aux_sym_variable_attributes_token3] = ACTIONS(1359),
    [aux_sym_variable_attributes_token4] = ACTIONS(1359),
    [aux_sym_variable_attributes_token5] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1359),
    [aux_sym_derived_type_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token2] = ACTIONS(1359),
    [aux_sym_type_qualifier_token3] = ACTIONS(1359),
    [aux_sym_type_qualifier_token4] = ACTIONS(1359),
    [aux_sym_type_qualifier_token5] = ACTIONS(1359),
    [aux_sym_type_qualifier_token9] = ACTIONS(1359),
    [aux_sym_type_qualifier_token10] = ACTIONS(1359),
    [aux_sym_type_qualifier_token11] = ACTIONS(1359),
    [aux_sym_type_qualifier_token12] = ACTIONS(1359),
    [aux_sym_type_qualifier_token13] = ACTIONS(1359),
    [aux_sym_type_qualifier_token14] = ACTIONS(1359),
    [aux_sym_type_qualifier_token15] = ACTIONS(1359),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token2] = ACTIONS(1359),
    [aux_sym_subroutine_call_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token2] = ACTIONS(1359),
    [aux_sym_keyword_statement_token3] = ACTIONS(1359),
    [aux_sym_keyword_statement_token4] = ACTIONS(1359),
    [aux_sym_keyword_statement_token6] = ACTIONS(1359),
    [aux_sym_keyword_statement_token7] = ACTIONS(1359),
    [aux_sym_include_statement_token1] = ACTIONS(1359),
    [aux_sym_data_statement_token1] = ACTIONS(1359),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1359),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1359),
    [aux_sym_end_if_statement_token1] = ACTIONS(1359),
    [aux_sym_elseif_clause_token2] = ACTIONS(1359),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1359),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token3] = ACTIONS(1359),
    [aux_sym_select_type_statement_token1] = ACTIONS(1359),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1359),
    [aux_sym_block_construct_token1] = ACTIONS(1359),
    [aux_sym_associate_statement_token1] = ACTIONS(1359),
    [aux_sym_format_statement_token1] = ACTIONS(1359),
    [aux_sym_print_statement_token1] = ACTIONS(1359),
    [aux_sym_open_statement_token1] = ACTIONS(1359),
    [aux_sym_close_statement_token1] = ACTIONS(1359),
    [aux_sym_inquire_statement_token1] = ACTIONS(1359),
    [aux_sym_enum_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token2] = ACTIONS(1359),
    [aux_sym_file_position_statement_token3] = ACTIONS(1359),
    [aux_sym_file_position_statement_token4] = ACTIONS(1359),
    [aux_sym_logical_expression_token5] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(1359),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [sym_boolean_literal] = ACTIONS(1361),
    [aux_sym_null_literal_token1] = ACTIONS(1359),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1359),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1361),
    [sym__float_literal] = ACTIONS(1361),
    [sym__boz_literal] = ACTIONS(1361),
    [sym_string_literal] = ACTIONS(1361),
  },
  [379] = {
    [sym_preproc_file_line] = STATE(379),
    [aux_sym_end_program_statement_token1] = ACTIONS(1099),
    [anon_sym_LPAREN] = ACTIONS(1099),
    [aux_sym_interface_statement_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1099),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1099),
    [aux_sym_contains_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token2] = ACTIONS(1099),
    [aux_sym_implicit_statement_token1] = ACTIONS(1099),
    [aux_sym_implicit_statement_token3] = ACTIONS(1099),
    [aux_sym_implicit_statement_token4] = ACTIONS(1099),
    [aux_sym_save_statement_token1] = ACTIONS(1099),
    [aux_sym_private_statement_token1] = ACTIONS(1099),
    [aux_sym_public_statement_token1] = ACTIONS(1099),
    [aux_sym_namelist_statement_token1] = ACTIONS(1099),
    [aux_sym_common_statement_token1] = ACTIONS(1099),
    [anon_sym_DASH] = ACTIONS(1097),
    [aux_sym_import_statement_token1] = ACTIONS(1099),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1099),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1099),
    [aux_sym_variable_attributes_token1] = ACTIONS(1099),
    [aux_sym_variable_attributes_token2] = ACTIONS(1099),
    [aux_sym_variable_attributes_token3] = ACTIONS(1099),
    [aux_sym_variable_attributes_token4] = ACTIONS(1099),
    [aux_sym_variable_attributes_token5] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1099),
    [aux_sym_derived_type_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token2] = ACTIONS(1099),
    [aux_sym_type_qualifier_token3] = ACTIONS(1099),
    [aux_sym_type_qualifier_token4] = ACTIONS(1099),
    [aux_sym_type_qualifier_token5] = ACTIONS(1099),
    [aux_sym_type_qualifier_token9] = ACTIONS(1099),
    [aux_sym_type_qualifier_token10] = ACTIONS(1099),
    [aux_sym_type_qualifier_token11] = ACTIONS(1099),
    [aux_sym_type_qualifier_token12] = ACTIONS(1099),
    [aux_sym_type_qualifier_token13] = ACTIONS(1099),
    [aux_sym_type_qualifier_token14] = ACTIONS(1099),
    [aux_sym_type_qualifier_token15] = ACTIONS(1099),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token2] = ACTIONS(1099),
    [aux_sym_subroutine_call_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token2] = ACTIONS(1099),
    [aux_sym_keyword_statement_token3] = ACTIONS(1099),
    [aux_sym_keyword_statement_token4] = ACTIONS(1099),
    [aux_sym_keyword_statement_token6] = ACTIONS(1099),
    [aux_sym_keyword_statement_token7] = ACTIONS(1099),
    [aux_sym_include_statement_token1] = ACTIONS(1099),
    [aux_sym_data_statement_token1] = ACTIONS(1099),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1099),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1099),
    [aux_sym_end_if_statement_token1] = ACTIONS(1099),
    [aux_sym_elseif_clause_token2] = ACTIONS(1099),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1099),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token3] = ACTIONS(1099),
    [aux_sym_select_type_statement_token1] = ACTIONS(1099),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1099),
    [aux_sym_block_construct_token1] = ACTIONS(1099),
    [aux_sym_associate_statement_token1] = ACTIONS(1099),
    [aux_sym_format_statement_token1] = ACTIONS(1099),
    [aux_sym_print_statement_token1] = ACTIONS(1099),
    [aux_sym_open_statement_token1] = ACTIONS(1099),
    [aux_sym_close_statement_token1] = ACTIONS(1099),
    [aux_sym_inquire_statement_token1] = ACTIONS(1099),
    [aux_sym_enum_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token2] = ACTIONS(1099),
    [aux_sym_file_position_statement_token3] = ACTIONS(1099),
    [aux_sym_file_position_statement_token4] = ACTIONS(1099),
    [aux_sym_logical_expression_token5] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [sym_boolean_literal] = ACTIONS(1097),
    [aux_sym_null_literal_token1] = ACTIONS(1099),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1099),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1097),
    [sym__float_literal] = ACTIONS(1097),
    [sym__boz_literal] = ACTIONS(1097),
    [sym_string_literal] = ACTIONS(1097),
  },
  [380] = {
    [sym_preproc_file_line] = STATE(380),
    [aux_sym_end_program_statement_token1] = ACTIONS(1331),
    [anon_sym_LPAREN] = ACTIONS(1331),
    [aux_sym_interface_statement_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1331),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1331),
    [aux_sym_end_function_statement_token1] = ACTIONS(1331),
    [aux_sym_contains_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token2] = ACTIONS(1331),
    [aux_sym_implicit_statement_token1] = ACTIONS(1331),
    [aux_sym_implicit_statement_token3] = ACTIONS(1331),
    [aux_sym_implicit_statement_token4] = ACTIONS(1331),
    [aux_sym_save_statement_token1] = ACTIONS(1331),
    [aux_sym_private_statement_token1] = ACTIONS(1331),
    [aux_sym_public_statement_token1] = ACTIONS(1331),
    [aux_sym_namelist_statement_token1] = ACTIONS(1331),
    [aux_sym_common_statement_token1] = ACTIONS(1331),
    [anon_sym_DASH] = ACTIONS(1333),
    [aux_sym_import_statement_token1] = ACTIONS(1331),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1331),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1331),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1331),
    [aux_sym_variable_attributes_token1] = ACTIONS(1331),
    [aux_sym_variable_attributes_token2] = ACTIONS(1331),
    [aux_sym_variable_attributes_token3] = ACTIONS(1331),
    [aux_sym_variable_attributes_token4] = ACTIONS(1331),
    [aux_sym_variable_attributes_token5] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1331),
    [aux_sym_derived_type_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token2] = ACTIONS(1331),
    [aux_sym_type_qualifier_token3] = ACTIONS(1331),
    [aux_sym_type_qualifier_token4] = ACTIONS(1331),
    [aux_sym_type_qualifier_token5] = ACTIONS(1331),
    [aux_sym_type_qualifier_token9] = ACTIONS(1331),
    [aux_sym_type_qualifier_token10] = ACTIONS(1331),
    [aux_sym_type_qualifier_token11] = ACTIONS(1331),
    [aux_sym_type_qualifier_token12] = ACTIONS(1331),
    [aux_sym_type_qualifier_token13] = ACTIONS(1331),
    [aux_sym_type_qualifier_token14] = ACTIONS(1331),
    [aux_sym_type_qualifier_token15] = ACTIONS(1331),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token2] = ACTIONS(1331),
    [aux_sym_subroutine_call_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token2] = ACTIONS(1331),
    [aux_sym_keyword_statement_token3] = ACTIONS(1331),
    [aux_sym_keyword_statement_token4] = ACTIONS(1331),
    [aux_sym_keyword_statement_token6] = ACTIONS(1331),
    [aux_sym_keyword_statement_token7] = ACTIONS(1331),
    [aux_sym_include_statement_token1] = ACTIONS(1331),
    [aux_sym_data_statement_token1] = ACTIONS(1331),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1331),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1331),
    [aux_sym_end_if_statement_token1] = ACTIONS(1331),
    [aux_sym_elseif_clause_token2] = ACTIONS(1331),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1331),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token3] = ACTIONS(1331),
    [aux_sym_select_type_statement_token1] = ACTIONS(1331),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1331),
    [aux_sym_block_construct_token1] = ACTIONS(1331),
    [aux_sym_associate_statement_token1] = ACTIONS(1331),
    [aux_sym_format_statement_token1] = ACTIONS(1331),
    [aux_sym_print_statement_token1] = ACTIONS(1331),
    [aux_sym_open_statement_token1] = ACTIONS(1331),
    [aux_sym_close_statement_token1] = ACTIONS(1331),
    [aux_sym_inquire_statement_token1] = ACTIONS(1331),
    [aux_sym_enum_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token2] = ACTIONS(1331),
    [aux_sym_file_position_statement_token3] = ACTIONS(1331),
    [aux_sym_file_position_statement_token4] = ACTIONS(1331),
    [aux_sym_logical_expression_token5] = ACTIONS(1333),
    [anon_sym_PLUS] = ACTIONS(1333),
    [anon_sym_DOT] = ACTIONS(1331),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1333),
    [anon_sym_LBRACK] = ACTIONS(1333),
    [sym_boolean_literal] = ACTIONS(1333),
    [aux_sym_null_literal_token1] = ACTIONS(1331),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1331),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1333),
    [sym__float_literal] = ACTIONS(1333),
    [sym__boz_literal] = ACTIONS(1333),
    [sym_string_literal] = ACTIONS(1333),
  },
  [381] = {
    [sym_preproc_file_line] = STATE(381),
    [aux_sym_end_program_statement_token1] = ACTIONS(1327),
    [anon_sym_LPAREN] = ACTIONS(1327),
    [aux_sym_interface_statement_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1327),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1327),
    [aux_sym_end_function_statement_token1] = ACTIONS(1327),
    [aux_sym_contains_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token2] = ACTIONS(1327),
    [aux_sym_implicit_statement_token1] = ACTIONS(1327),
    [aux_sym_implicit_statement_token3] = ACTIONS(1327),
    [aux_sym_implicit_statement_token4] = ACTIONS(1327),
    [aux_sym_save_statement_token1] = ACTIONS(1327),
    [aux_sym_private_statement_token1] = ACTIONS(1327),
    [aux_sym_public_statement_token1] = ACTIONS(1327),
    [aux_sym_namelist_statement_token1] = ACTIONS(1327),
    [aux_sym_common_statement_token1] = ACTIONS(1327),
    [anon_sym_DASH] = ACTIONS(1329),
    [aux_sym_import_statement_token1] = ACTIONS(1327),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1327),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1327),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1327),
    [aux_sym_variable_attributes_token1] = ACTIONS(1327),
    [aux_sym_variable_attributes_token2] = ACTIONS(1327),
    [aux_sym_variable_attributes_token3] = ACTIONS(1327),
    [aux_sym_variable_attributes_token4] = ACTIONS(1327),
    [aux_sym_variable_attributes_token5] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1327),
    [aux_sym_derived_type_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token2] = ACTIONS(1327),
    [aux_sym_type_qualifier_token3] = ACTIONS(1327),
    [aux_sym_type_qualifier_token4] = ACTIONS(1327),
    [aux_sym_type_qualifier_token5] = ACTIONS(1327),
    [aux_sym_type_qualifier_token9] = ACTIONS(1327),
    [aux_sym_type_qualifier_token10] = ACTIONS(1327),
    [aux_sym_type_qualifier_token11] = ACTIONS(1327),
    [aux_sym_type_qualifier_token12] = ACTIONS(1327),
    [aux_sym_type_qualifier_token13] = ACTIONS(1327),
    [aux_sym_type_qualifier_token14] = ACTIONS(1327),
    [aux_sym_type_qualifier_token15] = ACTIONS(1327),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token2] = ACTIONS(1327),
    [aux_sym_subroutine_call_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token2] = ACTIONS(1327),
    [aux_sym_keyword_statement_token3] = ACTIONS(1327),
    [aux_sym_keyword_statement_token4] = ACTIONS(1327),
    [aux_sym_keyword_statement_token6] = ACTIONS(1327),
    [aux_sym_keyword_statement_token7] = ACTIONS(1327),
    [aux_sym_include_statement_token1] = ACTIONS(1327),
    [aux_sym_data_statement_token1] = ACTIONS(1327),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1327),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1327),
    [aux_sym_end_if_statement_token1] = ACTIONS(1327),
    [aux_sym_elseif_clause_token2] = ACTIONS(1327),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1327),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token3] = ACTIONS(1327),
    [aux_sym_select_type_statement_token1] = ACTIONS(1327),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1327),
    [aux_sym_block_construct_token1] = ACTIONS(1327),
    [aux_sym_associate_statement_token1] = ACTIONS(1327),
    [aux_sym_format_statement_token1] = ACTIONS(1327),
    [aux_sym_print_statement_token1] = ACTIONS(1327),
    [aux_sym_open_statement_token1] = ACTIONS(1327),
    [aux_sym_close_statement_token1] = ACTIONS(1327),
    [aux_sym_inquire_statement_token1] = ACTIONS(1327),
    [aux_sym_enum_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token2] = ACTIONS(1327),
    [aux_sym_file_position_statement_token3] = ACTIONS(1327),
    [aux_sym_file_position_statement_token4] = ACTIONS(1327),
    [aux_sym_logical_expression_token5] = ACTIONS(1329),
    [anon_sym_PLUS] = ACTIONS(1329),
    [anon_sym_DOT] = ACTIONS(1327),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1329),
    [anon_sym_LBRACK] = ACTIONS(1329),
    [sym_boolean_literal] = ACTIONS(1329),
    [aux_sym_null_literal_token1] = ACTIONS(1327),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1327),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1329),
    [sym__float_literal] = ACTIONS(1329),
    [sym__boz_literal] = ACTIONS(1329),
    [sym_string_literal] = ACTIONS(1329),
  },
  [382] = {
    [sym_preproc_file_line] = STATE(382),
    [aux_sym_end_program_statement_token1] = ACTIONS(1303),
    [aux_sym_end_program_statement_token2] = ACTIONS(1303),
    [anon_sym_LPAREN] = ACTIONS(1303),
    [aux_sym_interface_statement_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1303),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1303),
    [aux_sym_contains_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token2] = ACTIONS(1303),
    [aux_sym_implicit_statement_token1] = ACTIONS(1303),
    [aux_sym_implicit_statement_token3] = ACTIONS(1303),
    [aux_sym_implicit_statement_token4] = ACTIONS(1303),
    [aux_sym_save_statement_token1] = ACTIONS(1303),
    [aux_sym_private_statement_token1] = ACTIONS(1303),
    [aux_sym_public_statement_token1] = ACTIONS(1303),
    [aux_sym_namelist_statement_token1] = ACTIONS(1303),
    [aux_sym_common_statement_token1] = ACTIONS(1303),
    [anon_sym_DASH] = ACTIONS(1305),
    [aux_sym_import_statement_token1] = ACTIONS(1303),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1303),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1303),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1303),
    [aux_sym_variable_attributes_token1] = ACTIONS(1303),
    [aux_sym_variable_attributes_token2] = ACTIONS(1303),
    [aux_sym_variable_attributes_token3] = ACTIONS(1303),
    [aux_sym_variable_attributes_token4] = ACTIONS(1303),
    [aux_sym_variable_attributes_token5] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1303),
    [aux_sym_derived_type_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token2] = ACTIONS(1303),
    [aux_sym_type_qualifier_token3] = ACTIONS(1303),
    [aux_sym_type_qualifier_token4] = ACTIONS(1303),
    [aux_sym_type_qualifier_token5] = ACTIONS(1303),
    [aux_sym_type_qualifier_token9] = ACTIONS(1303),
    [aux_sym_type_qualifier_token10] = ACTIONS(1303),
    [aux_sym_type_qualifier_token11] = ACTIONS(1303),
    [aux_sym_type_qualifier_token12] = ACTIONS(1303),
    [aux_sym_type_qualifier_token13] = ACTIONS(1303),
    [aux_sym_type_qualifier_token14] = ACTIONS(1303),
    [aux_sym_type_qualifier_token15] = ACTIONS(1303),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token2] = ACTIONS(1303),
    [aux_sym_subroutine_call_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token2] = ACTIONS(1303),
    [aux_sym_keyword_statement_token3] = ACTIONS(1303),
    [aux_sym_keyword_statement_token4] = ACTIONS(1303),
    [aux_sym_keyword_statement_token6] = ACTIONS(1303),
    [aux_sym_keyword_statement_token7] = ACTIONS(1303),
    [aux_sym_include_statement_token1] = ACTIONS(1303),
    [aux_sym_data_statement_token1] = ACTIONS(1303),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1303),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1303),
    [aux_sym_end_if_statement_token1] = ACTIONS(1303),
    [aux_sym_elseif_clause_token2] = ACTIONS(1303),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1303),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token3] = ACTIONS(1303),
    [aux_sym_select_type_statement_token1] = ACTIONS(1303),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1303),
    [aux_sym_block_construct_token1] = ACTIONS(1303),
    [aux_sym_associate_statement_token1] = ACTIONS(1303),
    [aux_sym_format_statement_token1] = ACTIONS(1303),
    [aux_sym_print_statement_token1] = ACTIONS(1303),
    [aux_sym_open_statement_token1] = ACTIONS(1303),
    [aux_sym_close_statement_token1] = ACTIONS(1303),
    [aux_sym_inquire_statement_token1] = ACTIONS(1303),
    [aux_sym_enum_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token2] = ACTIONS(1303),
    [aux_sym_file_position_statement_token3] = ACTIONS(1303),
    [aux_sym_file_position_statement_token4] = ACTIONS(1303),
    [aux_sym_logical_expression_token5] = ACTIONS(1305),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_DOT] = ACTIONS(1303),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1305),
    [anon_sym_LBRACK] = ACTIONS(1305),
    [sym_boolean_literal] = ACTIONS(1305),
    [aux_sym_null_literal_token1] = ACTIONS(1303),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1303),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1305),
    [sym__float_literal] = ACTIONS(1305),
    [sym__boz_literal] = ACTIONS(1305),
    [sym_string_literal] = ACTIONS(1305),
  },
  [383] = {
    [sym_preproc_file_line] = STATE(383),
    [aux_sym_end_program_statement_token1] = ACTIONS(1319),
    [anon_sym_LPAREN] = ACTIONS(1319),
    [aux_sym_interface_statement_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1319),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1319),
    [aux_sym_end_function_statement_token1] = ACTIONS(1319),
    [aux_sym_contains_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token2] = ACTIONS(1319),
    [aux_sym_implicit_statement_token1] = ACTIONS(1319),
    [aux_sym_implicit_statement_token3] = ACTIONS(1319),
    [aux_sym_implicit_statement_token4] = ACTIONS(1319),
    [aux_sym_save_statement_token1] = ACTIONS(1319),
    [aux_sym_private_statement_token1] = ACTIONS(1319),
    [aux_sym_public_statement_token1] = ACTIONS(1319),
    [aux_sym_namelist_statement_token1] = ACTIONS(1319),
    [aux_sym_common_statement_token1] = ACTIONS(1319),
    [anon_sym_DASH] = ACTIONS(1321),
    [aux_sym_import_statement_token1] = ACTIONS(1319),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1319),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1319),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1319),
    [aux_sym_variable_attributes_token1] = ACTIONS(1319),
    [aux_sym_variable_attributes_token2] = ACTIONS(1319),
    [aux_sym_variable_attributes_token3] = ACTIONS(1319),
    [aux_sym_variable_attributes_token4] = ACTIONS(1319),
    [aux_sym_variable_attributes_token5] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1319),
    [aux_sym_derived_type_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token2] = ACTIONS(1319),
    [aux_sym_type_qualifier_token3] = ACTIONS(1319),
    [aux_sym_type_qualifier_token4] = ACTIONS(1319),
    [aux_sym_type_qualifier_token5] = ACTIONS(1319),
    [aux_sym_type_qualifier_token9] = ACTIONS(1319),
    [aux_sym_type_qualifier_token10] = ACTIONS(1319),
    [aux_sym_type_qualifier_token11] = ACTIONS(1319),
    [aux_sym_type_qualifier_token12] = ACTIONS(1319),
    [aux_sym_type_qualifier_token13] = ACTIONS(1319),
    [aux_sym_type_qualifier_token14] = ACTIONS(1319),
    [aux_sym_type_qualifier_token15] = ACTIONS(1319),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token2] = ACTIONS(1319),
    [aux_sym_subroutine_call_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token2] = ACTIONS(1319),
    [aux_sym_keyword_statement_token3] = ACTIONS(1319),
    [aux_sym_keyword_statement_token4] = ACTIONS(1319),
    [aux_sym_keyword_statement_token6] = ACTIONS(1319),
    [aux_sym_keyword_statement_token7] = ACTIONS(1319),
    [aux_sym_include_statement_token1] = ACTIONS(1319),
    [aux_sym_data_statement_token1] = ACTIONS(1319),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1319),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1319),
    [aux_sym_end_if_statement_token1] = ACTIONS(1319),
    [aux_sym_elseif_clause_token2] = ACTIONS(1319),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1319),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token3] = ACTIONS(1319),
    [aux_sym_select_type_statement_token1] = ACTIONS(1319),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1319),
    [aux_sym_block_construct_token1] = ACTIONS(1319),
    [aux_sym_associate_statement_token1] = ACTIONS(1319),
    [aux_sym_format_statement_token1] = ACTIONS(1319),
    [aux_sym_print_statement_token1] = ACTIONS(1319),
    [aux_sym_open_statement_token1] = ACTIONS(1319),
    [aux_sym_close_statement_token1] = ACTIONS(1319),
    [aux_sym_inquire_statement_token1] = ACTIONS(1319),
    [aux_sym_enum_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token2] = ACTIONS(1319),
    [aux_sym_file_position_statement_token3] = ACTIONS(1319),
    [aux_sym_file_position_statement_token4] = ACTIONS(1319),
    [aux_sym_logical_expression_token5] = ACTIONS(1321),
    [anon_sym_PLUS] = ACTIONS(1321),
    [anon_sym_DOT] = ACTIONS(1319),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1321),
    [anon_sym_LBRACK] = ACTIONS(1321),
    [sym_boolean_literal] = ACTIONS(1321),
    [aux_sym_null_literal_token1] = ACTIONS(1319),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1319),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1321),
    [sym__float_literal] = ACTIONS(1321),
    [sym__boz_literal] = ACTIONS(1321),
    [sym_string_literal] = ACTIONS(1321),
  },
  [384] = {
    [sym_preproc_file_line] = STATE(384),
    [aux_sym_end_program_statement_token1] = ACTIONS(1307),
    [anon_sym_LPAREN] = ACTIONS(1307),
    [aux_sym_interface_statement_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1307),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1307),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1307),
    [aux_sym_contains_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token2] = ACTIONS(1307),
    [aux_sym_implicit_statement_token1] = ACTIONS(1307),
    [aux_sym_implicit_statement_token3] = ACTIONS(1307),
    [aux_sym_implicit_statement_token4] = ACTIONS(1307),
    [aux_sym_save_statement_token1] = ACTIONS(1307),
    [aux_sym_private_statement_token1] = ACTIONS(1307),
    [aux_sym_public_statement_token1] = ACTIONS(1307),
    [aux_sym_namelist_statement_token1] = ACTIONS(1307),
    [aux_sym_common_statement_token1] = ACTIONS(1307),
    [anon_sym_DASH] = ACTIONS(1309),
    [aux_sym_import_statement_token1] = ACTIONS(1307),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1307),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1307),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1307),
    [aux_sym_variable_attributes_token1] = ACTIONS(1307),
    [aux_sym_variable_attributes_token2] = ACTIONS(1307),
    [aux_sym_variable_attributes_token3] = ACTIONS(1307),
    [aux_sym_variable_attributes_token4] = ACTIONS(1307),
    [aux_sym_variable_attributes_token5] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1307),
    [aux_sym_derived_type_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token2] = ACTIONS(1307),
    [aux_sym_type_qualifier_token3] = ACTIONS(1307),
    [aux_sym_type_qualifier_token4] = ACTIONS(1307),
    [aux_sym_type_qualifier_token5] = ACTIONS(1307),
    [aux_sym_type_qualifier_token9] = ACTIONS(1307),
    [aux_sym_type_qualifier_token10] = ACTIONS(1307),
    [aux_sym_type_qualifier_token11] = ACTIONS(1307),
    [aux_sym_type_qualifier_token12] = ACTIONS(1307),
    [aux_sym_type_qualifier_token13] = ACTIONS(1307),
    [aux_sym_type_qualifier_token14] = ACTIONS(1307),
    [aux_sym_type_qualifier_token15] = ACTIONS(1307),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token2] = ACTIONS(1307),
    [aux_sym_subroutine_call_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token2] = ACTIONS(1307),
    [aux_sym_keyword_statement_token3] = ACTIONS(1307),
    [aux_sym_keyword_statement_token4] = ACTIONS(1307),
    [aux_sym_keyword_statement_token6] = ACTIONS(1307),
    [aux_sym_keyword_statement_token7] = ACTIONS(1307),
    [aux_sym_include_statement_token1] = ACTIONS(1307),
    [aux_sym_data_statement_token1] = ACTIONS(1307),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1307),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1307),
    [aux_sym_end_if_statement_token1] = ACTIONS(1307),
    [aux_sym_elseif_clause_token2] = ACTIONS(1307),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1307),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token3] = ACTIONS(1307),
    [aux_sym_select_type_statement_token1] = ACTIONS(1307),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1307),
    [aux_sym_block_construct_token1] = ACTIONS(1307),
    [aux_sym_associate_statement_token1] = ACTIONS(1307),
    [aux_sym_format_statement_token1] = ACTIONS(1307),
    [aux_sym_print_statement_token1] = ACTIONS(1307),
    [aux_sym_open_statement_token1] = ACTIONS(1307),
    [aux_sym_close_statement_token1] = ACTIONS(1307),
    [aux_sym_inquire_statement_token1] = ACTIONS(1307),
    [aux_sym_enum_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token2] = ACTIONS(1307),
    [aux_sym_file_position_statement_token3] = ACTIONS(1307),
    [aux_sym_file_position_statement_token4] = ACTIONS(1307),
    [aux_sym_logical_expression_token5] = ACTIONS(1309),
    [anon_sym_PLUS] = ACTIONS(1309),
    [anon_sym_DOT] = ACTIONS(1307),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1309),
    [anon_sym_LBRACK] = ACTIONS(1309),
    [sym_boolean_literal] = ACTIONS(1309),
    [aux_sym_null_literal_token1] = ACTIONS(1307),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1307),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1309),
    [sym__float_literal] = ACTIONS(1309),
    [sym__boz_literal] = ACTIONS(1309),
    [sym_string_literal] = ACTIONS(1309),
  },
  [385] = {
    [sym_preproc_file_line] = STATE(385),
    [aux_sym_end_program_statement_token1] = ACTIONS(1119),
    [aux_sym_end_program_statement_token2] = ACTIONS(1119),
    [anon_sym_LPAREN] = ACTIONS(1119),
    [aux_sym_interface_statement_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1119),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1119),
    [aux_sym_contains_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token2] = ACTIONS(1119),
    [aux_sym_implicit_statement_token1] = ACTIONS(1119),
    [aux_sym_implicit_statement_token3] = ACTIONS(1119),
    [aux_sym_implicit_statement_token4] = ACTIONS(1119),
    [aux_sym_save_statement_token1] = ACTIONS(1119),
    [aux_sym_private_statement_token1] = ACTIONS(1119),
    [aux_sym_public_statement_token1] = ACTIONS(1119),
    [aux_sym_namelist_statement_token1] = ACTIONS(1119),
    [aux_sym_common_statement_token1] = ACTIONS(1119),
    [anon_sym_DASH] = ACTIONS(1117),
    [aux_sym_import_statement_token1] = ACTIONS(1119),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1119),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1119),
    [aux_sym_variable_attributes_token1] = ACTIONS(1119),
    [aux_sym_variable_attributes_token2] = ACTIONS(1119),
    [aux_sym_variable_attributes_token3] = ACTIONS(1119),
    [aux_sym_variable_attributes_token4] = ACTIONS(1119),
    [aux_sym_variable_attributes_token5] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1119),
    [aux_sym_derived_type_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token2] = ACTIONS(1119),
    [aux_sym_type_qualifier_token3] = ACTIONS(1119),
    [aux_sym_type_qualifier_token4] = ACTIONS(1119),
    [aux_sym_type_qualifier_token5] = ACTIONS(1119),
    [aux_sym_type_qualifier_token9] = ACTIONS(1119),
    [aux_sym_type_qualifier_token10] = ACTIONS(1119),
    [aux_sym_type_qualifier_token11] = ACTIONS(1119),
    [aux_sym_type_qualifier_token12] = ACTIONS(1119),
    [aux_sym_type_qualifier_token13] = ACTIONS(1119),
    [aux_sym_type_qualifier_token14] = ACTIONS(1119),
    [aux_sym_type_qualifier_token15] = ACTIONS(1119),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token2] = ACTIONS(1119),
    [aux_sym_subroutine_call_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token2] = ACTIONS(1119),
    [aux_sym_keyword_statement_token3] = ACTIONS(1119),
    [aux_sym_keyword_statement_token4] = ACTIONS(1119),
    [aux_sym_keyword_statement_token6] = ACTIONS(1119),
    [aux_sym_keyword_statement_token7] = ACTIONS(1119),
    [aux_sym_include_statement_token1] = ACTIONS(1119),
    [aux_sym_data_statement_token1] = ACTIONS(1119),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1119),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1119),
    [aux_sym_end_if_statement_token1] = ACTIONS(1119),
    [aux_sym_elseif_clause_token2] = ACTIONS(1119),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1119),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token3] = ACTIONS(1119),
    [aux_sym_select_type_statement_token1] = ACTIONS(1119),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1119),
    [aux_sym_block_construct_token1] = ACTIONS(1119),
    [aux_sym_associate_statement_token1] = ACTIONS(1119),
    [aux_sym_format_statement_token1] = ACTIONS(1119),
    [aux_sym_print_statement_token1] = ACTIONS(1119),
    [aux_sym_open_statement_token1] = ACTIONS(1119),
    [aux_sym_close_statement_token1] = ACTIONS(1119),
    [aux_sym_inquire_statement_token1] = ACTIONS(1119),
    [aux_sym_enum_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token2] = ACTIONS(1119),
    [aux_sym_file_position_statement_token3] = ACTIONS(1119),
    [aux_sym_file_position_statement_token4] = ACTIONS(1119),
    [aux_sym_logical_expression_token5] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [sym_boolean_literal] = ACTIONS(1117),
    [aux_sym_null_literal_token1] = ACTIONS(1119),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1119),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1117),
    [sym__float_literal] = ACTIONS(1117),
    [sym__boz_literal] = ACTIONS(1117),
    [sym_string_literal] = ACTIONS(1117),
  },
  [386] = {
    [sym_preproc_file_line] = STATE(386),
    [aux_sym_end_program_statement_token1] = ACTIONS(1311),
    [anon_sym_LPAREN] = ACTIONS(1311),
    [aux_sym_interface_statement_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1311),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1311),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1311),
    [aux_sym_contains_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token2] = ACTIONS(1311),
    [aux_sym_implicit_statement_token1] = ACTIONS(1311),
    [aux_sym_implicit_statement_token3] = ACTIONS(1311),
    [aux_sym_implicit_statement_token4] = ACTIONS(1311),
    [aux_sym_save_statement_token1] = ACTIONS(1311),
    [aux_sym_private_statement_token1] = ACTIONS(1311),
    [aux_sym_public_statement_token1] = ACTIONS(1311),
    [aux_sym_namelist_statement_token1] = ACTIONS(1311),
    [aux_sym_common_statement_token1] = ACTIONS(1311),
    [anon_sym_DASH] = ACTIONS(1313),
    [aux_sym_import_statement_token1] = ACTIONS(1311),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1311),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1311),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1311),
    [aux_sym_variable_attributes_token1] = ACTIONS(1311),
    [aux_sym_variable_attributes_token2] = ACTIONS(1311),
    [aux_sym_variable_attributes_token3] = ACTIONS(1311),
    [aux_sym_variable_attributes_token4] = ACTIONS(1311),
    [aux_sym_variable_attributes_token5] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1311),
    [aux_sym_derived_type_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token2] = ACTIONS(1311),
    [aux_sym_type_qualifier_token3] = ACTIONS(1311),
    [aux_sym_type_qualifier_token4] = ACTIONS(1311),
    [aux_sym_type_qualifier_token5] = ACTIONS(1311),
    [aux_sym_type_qualifier_token9] = ACTIONS(1311),
    [aux_sym_type_qualifier_token10] = ACTIONS(1311),
    [aux_sym_type_qualifier_token11] = ACTIONS(1311),
    [aux_sym_type_qualifier_token12] = ACTIONS(1311),
    [aux_sym_type_qualifier_token13] = ACTIONS(1311),
    [aux_sym_type_qualifier_token14] = ACTIONS(1311),
    [aux_sym_type_qualifier_token15] = ACTIONS(1311),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token2] = ACTIONS(1311),
    [aux_sym_subroutine_call_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token2] = ACTIONS(1311),
    [aux_sym_keyword_statement_token3] = ACTIONS(1311),
    [aux_sym_keyword_statement_token4] = ACTIONS(1311),
    [aux_sym_keyword_statement_token6] = ACTIONS(1311),
    [aux_sym_keyword_statement_token7] = ACTIONS(1311),
    [aux_sym_include_statement_token1] = ACTIONS(1311),
    [aux_sym_data_statement_token1] = ACTIONS(1311),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1311),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1311),
    [aux_sym_end_if_statement_token1] = ACTIONS(1311),
    [aux_sym_elseif_clause_token2] = ACTIONS(1311),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1311),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token3] = ACTIONS(1311),
    [aux_sym_select_type_statement_token1] = ACTIONS(1311),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1311),
    [aux_sym_block_construct_token1] = ACTIONS(1311),
    [aux_sym_associate_statement_token1] = ACTIONS(1311),
    [aux_sym_format_statement_token1] = ACTIONS(1311),
    [aux_sym_print_statement_token1] = ACTIONS(1311),
    [aux_sym_open_statement_token1] = ACTIONS(1311),
    [aux_sym_close_statement_token1] = ACTIONS(1311),
    [aux_sym_inquire_statement_token1] = ACTIONS(1311),
    [aux_sym_enum_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token2] = ACTIONS(1311),
    [aux_sym_file_position_statement_token3] = ACTIONS(1311),
    [aux_sym_file_position_statement_token4] = ACTIONS(1311),
    [aux_sym_logical_expression_token5] = ACTIONS(1313),
    [anon_sym_PLUS] = ACTIONS(1313),
    [anon_sym_DOT] = ACTIONS(1311),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1313),
    [anon_sym_LBRACK] = ACTIONS(1313),
    [sym_boolean_literal] = ACTIONS(1313),
    [aux_sym_null_literal_token1] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1311),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1313),
    [sym__float_literal] = ACTIONS(1313),
    [sym__boz_literal] = ACTIONS(1313),
    [sym_string_literal] = ACTIONS(1313),
  },
  [387] = {
    [sym_preproc_file_line] = STATE(387),
    [aux_sym_end_program_statement_token1] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1259),
    [aux_sym_interface_statement_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1259),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1259),
    [aux_sym_contains_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token2] = ACTIONS(1259),
    [aux_sym_implicit_statement_token1] = ACTIONS(1259),
    [aux_sym_implicit_statement_token3] = ACTIONS(1259),
    [aux_sym_implicit_statement_token4] = ACTIONS(1259),
    [aux_sym_save_statement_token1] = ACTIONS(1259),
    [aux_sym_private_statement_token1] = ACTIONS(1259),
    [aux_sym_public_statement_token1] = ACTIONS(1259),
    [aux_sym_namelist_statement_token1] = ACTIONS(1259),
    [aux_sym_common_statement_token1] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [aux_sym_import_statement_token1] = ACTIONS(1259),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1259),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1259),
    [aux_sym_variable_attributes_token1] = ACTIONS(1259),
    [aux_sym_variable_attributes_token2] = ACTIONS(1259),
    [aux_sym_variable_attributes_token3] = ACTIONS(1259),
    [aux_sym_variable_attributes_token4] = ACTIONS(1259),
    [aux_sym_variable_attributes_token5] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1259),
    [aux_sym_derived_type_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token2] = ACTIONS(1259),
    [aux_sym_type_qualifier_token3] = ACTIONS(1259),
    [aux_sym_type_qualifier_token4] = ACTIONS(1259),
    [aux_sym_type_qualifier_token5] = ACTIONS(1259),
    [aux_sym_type_qualifier_token9] = ACTIONS(1259),
    [aux_sym_type_qualifier_token10] = ACTIONS(1259),
    [aux_sym_type_qualifier_token11] = ACTIONS(1259),
    [aux_sym_type_qualifier_token12] = ACTIONS(1259),
    [aux_sym_type_qualifier_token13] = ACTIONS(1259),
    [aux_sym_type_qualifier_token14] = ACTIONS(1259),
    [aux_sym_type_qualifier_token15] = ACTIONS(1259),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token2] = ACTIONS(1259),
    [aux_sym_subroutine_call_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token2] = ACTIONS(1259),
    [aux_sym_keyword_statement_token3] = ACTIONS(1259),
    [aux_sym_keyword_statement_token4] = ACTIONS(1259),
    [aux_sym_keyword_statement_token6] = ACTIONS(1259),
    [aux_sym_keyword_statement_token7] = ACTIONS(1259),
    [aux_sym_include_statement_token1] = ACTIONS(1259),
    [aux_sym_data_statement_token1] = ACTIONS(1259),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1259),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1259),
    [aux_sym_end_if_statement_token1] = ACTIONS(1259),
    [aux_sym_elseif_clause_token2] = ACTIONS(1259),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1259),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token3] = ACTIONS(1259),
    [aux_sym_select_type_statement_token1] = ACTIONS(1259),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1259),
    [aux_sym_block_construct_token1] = ACTIONS(1259),
    [aux_sym_associate_statement_token1] = ACTIONS(1259),
    [aux_sym_format_statement_token1] = ACTIONS(1259),
    [aux_sym_print_statement_token1] = ACTIONS(1259),
    [aux_sym_open_statement_token1] = ACTIONS(1259),
    [aux_sym_close_statement_token1] = ACTIONS(1259),
    [aux_sym_inquire_statement_token1] = ACTIONS(1259),
    [aux_sym_enum_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token2] = ACTIONS(1259),
    [aux_sym_file_position_statement_token3] = ACTIONS(1259),
    [aux_sym_file_position_statement_token4] = ACTIONS(1259),
    [aux_sym_logical_expression_token5] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [sym_boolean_literal] = ACTIONS(1257),
    [aux_sym_null_literal_token1] = ACTIONS(1259),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1259),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1257),
    [sym__float_literal] = ACTIONS(1257),
    [sym__boz_literal] = ACTIONS(1257),
    [sym_string_literal] = ACTIONS(1257),
  },
  [388] = {
    [sym_preproc_file_line] = STATE(388),
    [aux_sym_end_program_statement_token1] = ACTIONS(1119),
    [anon_sym_LPAREN] = ACTIONS(1119),
    [aux_sym_interface_statement_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1119),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1119),
    [aux_sym_contains_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token2] = ACTIONS(1119),
    [aux_sym_implicit_statement_token1] = ACTIONS(1119),
    [aux_sym_implicit_statement_token3] = ACTIONS(1119),
    [aux_sym_implicit_statement_token4] = ACTIONS(1119),
    [aux_sym_save_statement_token1] = ACTIONS(1119),
    [aux_sym_private_statement_token1] = ACTIONS(1119),
    [aux_sym_public_statement_token1] = ACTIONS(1119),
    [aux_sym_namelist_statement_token1] = ACTIONS(1119),
    [aux_sym_common_statement_token1] = ACTIONS(1119),
    [anon_sym_DASH] = ACTIONS(1117),
    [aux_sym_import_statement_token1] = ACTIONS(1119),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1119),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1119),
    [aux_sym_variable_attributes_token1] = ACTIONS(1119),
    [aux_sym_variable_attributes_token2] = ACTIONS(1119),
    [aux_sym_variable_attributes_token3] = ACTIONS(1119),
    [aux_sym_variable_attributes_token4] = ACTIONS(1119),
    [aux_sym_variable_attributes_token5] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1119),
    [aux_sym_derived_type_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token2] = ACTIONS(1119),
    [aux_sym_type_qualifier_token3] = ACTIONS(1119),
    [aux_sym_type_qualifier_token4] = ACTIONS(1119),
    [aux_sym_type_qualifier_token5] = ACTIONS(1119),
    [aux_sym_type_qualifier_token9] = ACTIONS(1119),
    [aux_sym_type_qualifier_token10] = ACTIONS(1119),
    [aux_sym_type_qualifier_token11] = ACTIONS(1119),
    [aux_sym_type_qualifier_token12] = ACTIONS(1119),
    [aux_sym_type_qualifier_token13] = ACTIONS(1119),
    [aux_sym_type_qualifier_token14] = ACTIONS(1119),
    [aux_sym_type_qualifier_token15] = ACTIONS(1119),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token2] = ACTIONS(1119),
    [aux_sym_subroutine_call_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token2] = ACTIONS(1119),
    [aux_sym_keyword_statement_token3] = ACTIONS(1119),
    [aux_sym_keyword_statement_token4] = ACTIONS(1119),
    [aux_sym_keyword_statement_token6] = ACTIONS(1119),
    [aux_sym_keyword_statement_token7] = ACTIONS(1119),
    [aux_sym_include_statement_token1] = ACTIONS(1119),
    [aux_sym_data_statement_token1] = ACTIONS(1119),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1119),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1119),
    [aux_sym_end_if_statement_token1] = ACTIONS(1119),
    [aux_sym_elseif_clause_token2] = ACTIONS(1119),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1119),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token3] = ACTIONS(1119),
    [aux_sym_select_type_statement_token1] = ACTIONS(1119),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1119),
    [aux_sym_block_construct_token1] = ACTIONS(1119),
    [aux_sym_associate_statement_token1] = ACTIONS(1119),
    [aux_sym_format_statement_token1] = ACTIONS(1119),
    [aux_sym_print_statement_token1] = ACTIONS(1119),
    [aux_sym_open_statement_token1] = ACTIONS(1119),
    [aux_sym_close_statement_token1] = ACTIONS(1119),
    [aux_sym_inquire_statement_token1] = ACTIONS(1119),
    [aux_sym_enum_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token2] = ACTIONS(1119),
    [aux_sym_file_position_statement_token3] = ACTIONS(1119),
    [aux_sym_file_position_statement_token4] = ACTIONS(1119),
    [aux_sym_logical_expression_token5] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [sym_boolean_literal] = ACTIONS(1117),
    [aux_sym_null_literal_token1] = ACTIONS(1119),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1119),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1117),
    [sym__float_literal] = ACTIONS(1117),
    [sym__boz_literal] = ACTIONS(1117),
    [sym_string_literal] = ACTIONS(1117),
  },
  [389] = {
    [sym_preproc_file_line] = STATE(389),
    [aux_sym_end_program_statement_token1] = ACTIONS(1199),
    [anon_sym_LPAREN] = ACTIONS(1199),
    [aux_sym_interface_statement_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1199),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1199),
    [aux_sym_end_function_statement_token1] = ACTIONS(1199),
    [aux_sym_contains_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token2] = ACTIONS(1199),
    [aux_sym_implicit_statement_token1] = ACTIONS(1199),
    [aux_sym_implicit_statement_token3] = ACTIONS(1199),
    [aux_sym_implicit_statement_token4] = ACTIONS(1199),
    [aux_sym_save_statement_token1] = ACTIONS(1199),
    [aux_sym_private_statement_token1] = ACTIONS(1199),
    [aux_sym_public_statement_token1] = ACTIONS(1199),
    [aux_sym_namelist_statement_token1] = ACTIONS(1199),
    [aux_sym_common_statement_token1] = ACTIONS(1199),
    [anon_sym_DASH] = ACTIONS(1197),
    [aux_sym_import_statement_token1] = ACTIONS(1199),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1199),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1199),
    [aux_sym_variable_attributes_token1] = ACTIONS(1199),
    [aux_sym_variable_attributes_token2] = ACTIONS(1199),
    [aux_sym_variable_attributes_token3] = ACTIONS(1199),
    [aux_sym_variable_attributes_token4] = ACTIONS(1199),
    [aux_sym_variable_attributes_token5] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1199),
    [aux_sym_derived_type_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token2] = ACTIONS(1199),
    [aux_sym_type_qualifier_token3] = ACTIONS(1199),
    [aux_sym_type_qualifier_token4] = ACTIONS(1199),
    [aux_sym_type_qualifier_token5] = ACTIONS(1199),
    [aux_sym_type_qualifier_token9] = ACTIONS(1199),
    [aux_sym_type_qualifier_token10] = ACTIONS(1199),
    [aux_sym_type_qualifier_token11] = ACTIONS(1199),
    [aux_sym_type_qualifier_token12] = ACTIONS(1199),
    [aux_sym_type_qualifier_token13] = ACTIONS(1199),
    [aux_sym_type_qualifier_token14] = ACTIONS(1199),
    [aux_sym_type_qualifier_token15] = ACTIONS(1199),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token2] = ACTIONS(1199),
    [aux_sym_subroutine_call_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token2] = ACTIONS(1199),
    [aux_sym_keyword_statement_token3] = ACTIONS(1199),
    [aux_sym_keyword_statement_token4] = ACTIONS(1199),
    [aux_sym_keyword_statement_token6] = ACTIONS(1199),
    [aux_sym_keyword_statement_token7] = ACTIONS(1199),
    [aux_sym_include_statement_token1] = ACTIONS(1199),
    [aux_sym_data_statement_token1] = ACTIONS(1199),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1199),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1199),
    [aux_sym_end_if_statement_token1] = ACTIONS(1199),
    [aux_sym_elseif_clause_token2] = ACTIONS(1199),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1199),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token3] = ACTIONS(1199),
    [aux_sym_select_type_statement_token1] = ACTIONS(1199),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1199),
    [aux_sym_block_construct_token1] = ACTIONS(1199),
    [aux_sym_associate_statement_token1] = ACTIONS(1199),
    [aux_sym_format_statement_token1] = ACTIONS(1199),
    [aux_sym_print_statement_token1] = ACTIONS(1199),
    [aux_sym_open_statement_token1] = ACTIONS(1199),
    [aux_sym_close_statement_token1] = ACTIONS(1199),
    [aux_sym_inquire_statement_token1] = ACTIONS(1199),
    [aux_sym_enum_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token2] = ACTIONS(1199),
    [aux_sym_file_position_statement_token3] = ACTIONS(1199),
    [aux_sym_file_position_statement_token4] = ACTIONS(1199),
    [aux_sym_logical_expression_token5] = ACTIONS(1197),
    [anon_sym_PLUS] = ACTIONS(1197),
    [anon_sym_DOT] = ACTIONS(1199),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1197),
    [anon_sym_LBRACK] = ACTIONS(1197),
    [sym_boolean_literal] = ACTIONS(1197),
    [aux_sym_null_literal_token1] = ACTIONS(1199),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1199),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1197),
    [sym__float_literal] = ACTIONS(1197),
    [sym__boz_literal] = ACTIONS(1197),
    [sym_string_literal] = ACTIONS(1197),
  },
  [390] = {
    [sym_preproc_file_line] = STATE(390),
    [aux_sym_end_program_statement_token1] = ACTIONS(1147),
    [anon_sym_LPAREN] = ACTIONS(1147),
    [aux_sym_interface_statement_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1147),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1147),
    [aux_sym_contains_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token2] = ACTIONS(1147),
    [aux_sym_implicit_statement_token1] = ACTIONS(1147),
    [aux_sym_implicit_statement_token3] = ACTIONS(1147),
    [aux_sym_implicit_statement_token4] = ACTIONS(1147),
    [aux_sym_save_statement_token1] = ACTIONS(1147),
    [aux_sym_private_statement_token1] = ACTIONS(1147),
    [aux_sym_public_statement_token1] = ACTIONS(1147),
    [aux_sym_namelist_statement_token1] = ACTIONS(1147),
    [aux_sym_common_statement_token1] = ACTIONS(1147),
    [anon_sym_DASH] = ACTIONS(1145),
    [aux_sym_import_statement_token1] = ACTIONS(1147),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1147),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1147),
    [aux_sym_variable_attributes_token1] = ACTIONS(1147),
    [aux_sym_variable_attributes_token2] = ACTIONS(1147),
    [aux_sym_variable_attributes_token3] = ACTIONS(1147),
    [aux_sym_variable_attributes_token4] = ACTIONS(1147),
    [aux_sym_variable_attributes_token5] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1147),
    [aux_sym_derived_type_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token2] = ACTIONS(1147),
    [aux_sym_type_qualifier_token3] = ACTIONS(1147),
    [aux_sym_type_qualifier_token4] = ACTIONS(1147),
    [aux_sym_type_qualifier_token5] = ACTIONS(1147),
    [aux_sym_type_qualifier_token9] = ACTIONS(1147),
    [aux_sym_type_qualifier_token10] = ACTIONS(1147),
    [aux_sym_type_qualifier_token11] = ACTIONS(1147),
    [aux_sym_type_qualifier_token12] = ACTIONS(1147),
    [aux_sym_type_qualifier_token13] = ACTIONS(1147),
    [aux_sym_type_qualifier_token14] = ACTIONS(1147),
    [aux_sym_type_qualifier_token15] = ACTIONS(1147),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token2] = ACTIONS(1147),
    [aux_sym_subroutine_call_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token2] = ACTIONS(1147),
    [aux_sym_keyword_statement_token3] = ACTIONS(1147),
    [aux_sym_keyword_statement_token4] = ACTIONS(1147),
    [aux_sym_keyword_statement_token6] = ACTIONS(1147),
    [aux_sym_keyword_statement_token7] = ACTIONS(1147),
    [aux_sym_include_statement_token1] = ACTIONS(1147),
    [aux_sym_data_statement_token1] = ACTIONS(1147),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1147),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1147),
    [aux_sym_end_if_statement_token1] = ACTIONS(1147),
    [aux_sym_elseif_clause_token2] = ACTIONS(1147),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1147),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token3] = ACTIONS(1147),
    [aux_sym_select_type_statement_token1] = ACTIONS(1147),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1147),
    [aux_sym_block_construct_token1] = ACTIONS(1147),
    [aux_sym_associate_statement_token1] = ACTIONS(1147),
    [aux_sym_format_statement_token1] = ACTIONS(1147),
    [aux_sym_print_statement_token1] = ACTIONS(1147),
    [aux_sym_open_statement_token1] = ACTIONS(1147),
    [aux_sym_close_statement_token1] = ACTIONS(1147),
    [aux_sym_inquire_statement_token1] = ACTIONS(1147),
    [aux_sym_enum_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token2] = ACTIONS(1147),
    [aux_sym_file_position_statement_token3] = ACTIONS(1147),
    [aux_sym_file_position_statement_token4] = ACTIONS(1147),
    [aux_sym_logical_expression_token5] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1145),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [sym_boolean_literal] = ACTIONS(1145),
    [aux_sym_null_literal_token1] = ACTIONS(1147),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1147),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1145),
    [sym__float_literal] = ACTIONS(1145),
    [sym__boz_literal] = ACTIONS(1145),
    [sym_string_literal] = ACTIONS(1145),
  },
  [391] = {
    [sym_preproc_file_line] = STATE(391),
    [aux_sym_end_program_statement_token1] = ACTIONS(1323),
    [anon_sym_LPAREN] = ACTIONS(1323),
    [aux_sym_interface_statement_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1323),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1323),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1323),
    [aux_sym_contains_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token2] = ACTIONS(1323),
    [aux_sym_implicit_statement_token1] = ACTIONS(1323),
    [aux_sym_implicit_statement_token3] = ACTIONS(1323),
    [aux_sym_implicit_statement_token4] = ACTIONS(1323),
    [aux_sym_save_statement_token1] = ACTIONS(1323),
    [aux_sym_private_statement_token1] = ACTIONS(1323),
    [aux_sym_public_statement_token1] = ACTIONS(1323),
    [aux_sym_namelist_statement_token1] = ACTIONS(1323),
    [aux_sym_common_statement_token1] = ACTIONS(1323),
    [anon_sym_DASH] = ACTIONS(1325),
    [aux_sym_import_statement_token1] = ACTIONS(1323),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1323),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1323),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1323),
    [aux_sym_variable_attributes_token1] = ACTIONS(1323),
    [aux_sym_variable_attributes_token2] = ACTIONS(1323),
    [aux_sym_variable_attributes_token3] = ACTIONS(1323),
    [aux_sym_variable_attributes_token4] = ACTIONS(1323),
    [aux_sym_variable_attributes_token5] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1323),
    [aux_sym_derived_type_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token2] = ACTIONS(1323),
    [aux_sym_type_qualifier_token3] = ACTIONS(1323),
    [aux_sym_type_qualifier_token4] = ACTIONS(1323),
    [aux_sym_type_qualifier_token5] = ACTIONS(1323),
    [aux_sym_type_qualifier_token9] = ACTIONS(1323),
    [aux_sym_type_qualifier_token10] = ACTIONS(1323),
    [aux_sym_type_qualifier_token11] = ACTIONS(1323),
    [aux_sym_type_qualifier_token12] = ACTIONS(1323),
    [aux_sym_type_qualifier_token13] = ACTIONS(1323),
    [aux_sym_type_qualifier_token14] = ACTIONS(1323),
    [aux_sym_type_qualifier_token15] = ACTIONS(1323),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token2] = ACTIONS(1323),
    [aux_sym_subroutine_call_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token2] = ACTIONS(1323),
    [aux_sym_keyword_statement_token3] = ACTIONS(1323),
    [aux_sym_keyword_statement_token4] = ACTIONS(1323),
    [aux_sym_keyword_statement_token6] = ACTIONS(1323),
    [aux_sym_keyword_statement_token7] = ACTIONS(1323),
    [aux_sym_include_statement_token1] = ACTIONS(1323),
    [aux_sym_data_statement_token1] = ACTIONS(1323),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1323),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1323),
    [aux_sym_end_if_statement_token1] = ACTIONS(1323),
    [aux_sym_elseif_clause_token2] = ACTIONS(1323),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1323),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token3] = ACTIONS(1323),
    [aux_sym_select_type_statement_token1] = ACTIONS(1323),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1323),
    [aux_sym_block_construct_token1] = ACTIONS(1323),
    [aux_sym_associate_statement_token1] = ACTIONS(1323),
    [aux_sym_format_statement_token1] = ACTIONS(1323),
    [aux_sym_print_statement_token1] = ACTIONS(1323),
    [aux_sym_open_statement_token1] = ACTIONS(1323),
    [aux_sym_close_statement_token1] = ACTIONS(1323),
    [aux_sym_inquire_statement_token1] = ACTIONS(1323),
    [aux_sym_enum_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token2] = ACTIONS(1323),
    [aux_sym_file_position_statement_token3] = ACTIONS(1323),
    [aux_sym_file_position_statement_token4] = ACTIONS(1323),
    [aux_sym_logical_expression_token5] = ACTIONS(1325),
    [anon_sym_PLUS] = ACTIONS(1325),
    [anon_sym_DOT] = ACTIONS(1323),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1325),
    [anon_sym_LBRACK] = ACTIONS(1325),
    [sym_boolean_literal] = ACTIONS(1325),
    [aux_sym_null_literal_token1] = ACTIONS(1323),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1323),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1325),
    [sym__float_literal] = ACTIONS(1325),
    [sym__boz_literal] = ACTIONS(1325),
    [sym_string_literal] = ACTIONS(1325),
  },
  [392] = {
    [sym_preproc_file_line] = STATE(392),
    [aux_sym_end_program_statement_token1] = ACTIONS(1247),
    [anon_sym_LPAREN] = ACTIONS(1247),
    [aux_sym_interface_statement_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1247),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_end_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1247),
    [aux_sym_contains_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token2] = ACTIONS(1247),
    [aux_sym_implicit_statement_token1] = ACTIONS(1247),
    [aux_sym_implicit_statement_token3] = ACTIONS(1247),
    [aux_sym_implicit_statement_token4] = ACTIONS(1247),
    [aux_sym_save_statement_token1] = ACTIONS(1247),
    [aux_sym_private_statement_token1] = ACTIONS(1247),
    [aux_sym_public_statement_token1] = ACTIONS(1247),
    [aux_sym_namelist_statement_token1] = ACTIONS(1247),
    [aux_sym_common_statement_token1] = ACTIONS(1247),
    [anon_sym_DASH] = ACTIONS(1245),
    [aux_sym_import_statement_token1] = ACTIONS(1247),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1247),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1247),
    [aux_sym_variable_attributes_token1] = ACTIONS(1247),
    [aux_sym_variable_attributes_token2] = ACTIONS(1247),
    [aux_sym_variable_attributes_token3] = ACTIONS(1247),
    [aux_sym_variable_attributes_token4] = ACTIONS(1247),
    [aux_sym_variable_attributes_token5] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1247),
    [aux_sym_derived_type_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token2] = ACTIONS(1247),
    [aux_sym_type_qualifier_token3] = ACTIONS(1247),
    [aux_sym_type_qualifier_token4] = ACTIONS(1247),
    [aux_sym_type_qualifier_token5] = ACTIONS(1247),
    [aux_sym_type_qualifier_token9] = ACTIONS(1247),
    [aux_sym_type_qualifier_token10] = ACTIONS(1247),
    [aux_sym_type_qualifier_token11] = ACTIONS(1247),
    [aux_sym_type_qualifier_token12] = ACTIONS(1247),
    [aux_sym_type_qualifier_token13] = ACTIONS(1247),
    [aux_sym_type_qualifier_token14] = ACTIONS(1247),
    [aux_sym_type_qualifier_token15] = ACTIONS(1247),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token2] = ACTIONS(1247),
    [aux_sym_subroutine_call_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token2] = ACTIONS(1247),
    [aux_sym_keyword_statement_token3] = ACTIONS(1247),
    [aux_sym_keyword_statement_token4] = ACTIONS(1247),
    [aux_sym_keyword_statement_token6] = ACTIONS(1247),
    [aux_sym_keyword_statement_token7] = ACTIONS(1247),
    [aux_sym_include_statement_token1] = ACTIONS(1247),
    [aux_sym_data_statement_token1] = ACTIONS(1247),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1247),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1247),
    [aux_sym_end_if_statement_token1] = ACTIONS(1247),
    [aux_sym_elseif_clause_token2] = ACTIONS(1247),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1247),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token3] = ACTIONS(1247),
    [aux_sym_select_type_statement_token1] = ACTIONS(1247),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1247),
    [aux_sym_block_construct_token1] = ACTIONS(1247),
    [aux_sym_associate_statement_token1] = ACTIONS(1247),
    [aux_sym_format_statement_token1] = ACTIONS(1247),
    [aux_sym_print_statement_token1] = ACTIONS(1247),
    [aux_sym_open_statement_token1] = ACTIONS(1247),
    [aux_sym_close_statement_token1] = ACTIONS(1247),
    [aux_sym_inquire_statement_token1] = ACTIONS(1247),
    [aux_sym_enum_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token2] = ACTIONS(1247),
    [aux_sym_file_position_statement_token3] = ACTIONS(1247),
    [aux_sym_file_position_statement_token4] = ACTIONS(1247),
    [aux_sym_logical_expression_token5] = ACTIONS(1245),
    [anon_sym_PLUS] = ACTIONS(1245),
    [anon_sym_DOT] = ACTIONS(1247),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1245),
    [anon_sym_LBRACK] = ACTIONS(1245),
    [sym_boolean_literal] = ACTIONS(1245),
    [aux_sym_null_literal_token1] = ACTIONS(1247),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1247),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1245),
    [sym__float_literal] = ACTIONS(1245),
    [sym__boz_literal] = ACTIONS(1245),
    [sym_string_literal] = ACTIONS(1245),
  },
  [393] = {
    [sym_preproc_file_line] = STATE(393),
    [aux_sym_end_program_statement_token1] = ACTIONS(1339),
    [anon_sym_LPAREN] = ACTIONS(1339),
    [aux_sym_interface_statement_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1339),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1339),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1339),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1339),
    [aux_sym_use_statement_token1] = ACTIONS(1339),
    [aux_sym_use_statement_token2] = ACTIONS(1339),
    [aux_sym_implicit_statement_token1] = ACTIONS(1339),
    [aux_sym_implicit_statement_token3] = ACTIONS(1339),
    [aux_sym_implicit_statement_token4] = ACTIONS(1339),
    [aux_sym_save_statement_token1] = ACTIONS(1339),
    [aux_sym_private_statement_token1] = ACTIONS(1339),
    [aux_sym_public_statement_token1] = ACTIONS(1339),
    [aux_sym_namelist_statement_token1] = ACTIONS(1339),
    [aux_sym_common_statement_token1] = ACTIONS(1339),
    [anon_sym_DASH] = ACTIONS(1341),
    [aux_sym_import_statement_token1] = ACTIONS(1339),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1339),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1339),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1339),
    [aux_sym_variable_attributes_token1] = ACTIONS(1339),
    [aux_sym_variable_attributes_token2] = ACTIONS(1339),
    [aux_sym_variable_attributes_token3] = ACTIONS(1339),
    [aux_sym_variable_attributes_token4] = ACTIONS(1339),
    [aux_sym_variable_attributes_token5] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1339),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1339),
    [aux_sym_derived_type_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token1] = ACTIONS(1339),
    [aux_sym_type_qualifier_token2] = ACTIONS(1339),
    [aux_sym_type_qualifier_token3] = ACTIONS(1339),
    [aux_sym_type_qualifier_token4] = ACTIONS(1339),
    [aux_sym_type_qualifier_token5] = ACTIONS(1339),
    [aux_sym_type_qualifier_token9] = ACTIONS(1339),
    [aux_sym_type_qualifier_token10] = ACTIONS(1339),
    [aux_sym_type_qualifier_token11] = ACTIONS(1339),
    [aux_sym_type_qualifier_token12] = ACTIONS(1339),
    [aux_sym_type_qualifier_token13] = ACTIONS(1339),
    [aux_sym_type_qualifier_token14] = ACTIONS(1339),
    [aux_sym_type_qualifier_token15] = ACTIONS(1339),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token1] = ACTIONS(1339),
    [aux_sym_stop_statement_token2] = ACTIONS(1339),
    [aux_sym_subroutine_call_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token1] = ACTIONS(1339),
    [aux_sym_keyword_statement_token2] = ACTIONS(1339),
    [aux_sym_keyword_statement_token3] = ACTIONS(1339),
    [aux_sym_keyword_statement_token4] = ACTIONS(1339),
    [aux_sym_keyword_statement_token6] = ACTIONS(1339),
    [aux_sym_keyword_statement_token7] = ACTIONS(1339),
    [aux_sym_include_statement_token1] = ACTIONS(1339),
    [aux_sym_data_statement_token1] = ACTIONS(1339),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1339),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1339),
    [aux_sym_end_if_statement_token1] = ACTIONS(1339),
    [aux_sym_elseif_clause_token2] = ACTIONS(1339),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1339),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token1] = ACTIONS(1339),
    [aux_sym_select_case_statement_token3] = ACTIONS(1339),
    [aux_sym_select_type_statement_token1] = ACTIONS(1339),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1339),
    [aux_sym_block_construct_token1] = ACTIONS(1339),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1339),
    [aux_sym_associate_statement_token1] = ACTIONS(1339),
    [aux_sym_format_statement_token1] = ACTIONS(1339),
    [aux_sym_print_statement_token1] = ACTIONS(1339),
    [aux_sym_open_statement_token1] = ACTIONS(1339),
    [aux_sym_close_statement_token1] = ACTIONS(1339),
    [aux_sym_inquire_statement_token1] = ACTIONS(1339),
    [aux_sym_enum_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token1] = ACTIONS(1339),
    [aux_sym_file_position_statement_token2] = ACTIONS(1339),
    [aux_sym_file_position_statement_token3] = ACTIONS(1339),
    [aux_sym_file_position_statement_token4] = ACTIONS(1339),
    [aux_sym_logical_expression_token5] = ACTIONS(1341),
    [anon_sym_PLUS] = ACTIONS(1341),
    [anon_sym_DOT] = ACTIONS(1339),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1341),
    [anon_sym_LBRACK] = ACTIONS(1341),
    [sym_boolean_literal] = ACTIONS(1341),
    [aux_sym_null_literal_token1] = ACTIONS(1339),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1339),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1341),
    [sym__float_literal] = ACTIONS(1341),
    [sym__boz_literal] = ACTIONS(1341),
    [sym_string_literal] = ACTIONS(1341),
  },
  [394] = {
    [sym_preproc_file_line] = STATE(394),
    [aux_sym_end_program_statement_token1] = ACTIONS(1327),
    [anon_sym_LPAREN] = ACTIONS(1327),
    [aux_sym_interface_statement_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1327),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1327),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1327),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1327),
    [aux_sym_use_statement_token1] = ACTIONS(1327),
    [aux_sym_use_statement_token2] = ACTIONS(1327),
    [aux_sym_implicit_statement_token1] = ACTIONS(1327),
    [aux_sym_implicit_statement_token3] = ACTIONS(1327),
    [aux_sym_implicit_statement_token4] = ACTIONS(1327),
    [aux_sym_save_statement_token1] = ACTIONS(1327),
    [aux_sym_private_statement_token1] = ACTIONS(1327),
    [aux_sym_public_statement_token1] = ACTIONS(1327),
    [aux_sym_namelist_statement_token1] = ACTIONS(1327),
    [aux_sym_common_statement_token1] = ACTIONS(1327),
    [anon_sym_DASH] = ACTIONS(1329),
    [aux_sym_import_statement_token1] = ACTIONS(1327),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1327),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1327),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1327),
    [aux_sym_variable_attributes_token1] = ACTIONS(1327),
    [aux_sym_variable_attributes_token2] = ACTIONS(1327),
    [aux_sym_variable_attributes_token3] = ACTIONS(1327),
    [aux_sym_variable_attributes_token4] = ACTIONS(1327),
    [aux_sym_variable_attributes_token5] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1327),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1327),
    [aux_sym_derived_type_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token1] = ACTIONS(1327),
    [aux_sym_type_qualifier_token2] = ACTIONS(1327),
    [aux_sym_type_qualifier_token3] = ACTIONS(1327),
    [aux_sym_type_qualifier_token4] = ACTIONS(1327),
    [aux_sym_type_qualifier_token5] = ACTIONS(1327),
    [aux_sym_type_qualifier_token9] = ACTIONS(1327),
    [aux_sym_type_qualifier_token10] = ACTIONS(1327),
    [aux_sym_type_qualifier_token11] = ACTIONS(1327),
    [aux_sym_type_qualifier_token12] = ACTIONS(1327),
    [aux_sym_type_qualifier_token13] = ACTIONS(1327),
    [aux_sym_type_qualifier_token14] = ACTIONS(1327),
    [aux_sym_type_qualifier_token15] = ACTIONS(1327),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token1] = ACTIONS(1327),
    [aux_sym_stop_statement_token2] = ACTIONS(1327),
    [aux_sym_subroutine_call_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token1] = ACTIONS(1327),
    [aux_sym_keyword_statement_token2] = ACTIONS(1327),
    [aux_sym_keyword_statement_token3] = ACTIONS(1327),
    [aux_sym_keyword_statement_token4] = ACTIONS(1327),
    [aux_sym_keyword_statement_token6] = ACTIONS(1327),
    [aux_sym_keyword_statement_token7] = ACTIONS(1327),
    [aux_sym_include_statement_token1] = ACTIONS(1327),
    [aux_sym_data_statement_token1] = ACTIONS(1327),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1327),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1327),
    [aux_sym_end_if_statement_token1] = ACTIONS(1327),
    [aux_sym_elseif_clause_token2] = ACTIONS(1327),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1327),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token1] = ACTIONS(1327),
    [aux_sym_select_case_statement_token3] = ACTIONS(1327),
    [aux_sym_select_type_statement_token1] = ACTIONS(1327),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1327),
    [aux_sym_block_construct_token1] = ACTIONS(1327),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1327),
    [aux_sym_associate_statement_token1] = ACTIONS(1327),
    [aux_sym_format_statement_token1] = ACTIONS(1327),
    [aux_sym_print_statement_token1] = ACTIONS(1327),
    [aux_sym_open_statement_token1] = ACTIONS(1327),
    [aux_sym_close_statement_token1] = ACTIONS(1327),
    [aux_sym_inquire_statement_token1] = ACTIONS(1327),
    [aux_sym_enum_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token1] = ACTIONS(1327),
    [aux_sym_file_position_statement_token2] = ACTIONS(1327),
    [aux_sym_file_position_statement_token3] = ACTIONS(1327),
    [aux_sym_file_position_statement_token4] = ACTIONS(1327),
    [aux_sym_logical_expression_token5] = ACTIONS(1329),
    [anon_sym_PLUS] = ACTIONS(1329),
    [anon_sym_DOT] = ACTIONS(1327),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1329),
    [anon_sym_LBRACK] = ACTIONS(1329),
    [sym_boolean_literal] = ACTIONS(1329),
    [aux_sym_null_literal_token1] = ACTIONS(1327),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1327),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1329),
    [sym__float_literal] = ACTIONS(1329),
    [sym__boz_literal] = ACTIONS(1329),
    [sym_string_literal] = ACTIONS(1329),
  },
  [395] = {
    [sym_preproc_file_line] = STATE(395),
    [aux_sym_end_program_statement_token1] = ACTIONS(1323),
    [anon_sym_LPAREN] = ACTIONS(1323),
    [aux_sym_interface_statement_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1323),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1323),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1323),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1323),
    [aux_sym_use_statement_token1] = ACTIONS(1323),
    [aux_sym_use_statement_token2] = ACTIONS(1323),
    [aux_sym_implicit_statement_token1] = ACTIONS(1323),
    [aux_sym_implicit_statement_token3] = ACTIONS(1323),
    [aux_sym_implicit_statement_token4] = ACTIONS(1323),
    [aux_sym_save_statement_token1] = ACTIONS(1323),
    [aux_sym_private_statement_token1] = ACTIONS(1323),
    [aux_sym_public_statement_token1] = ACTIONS(1323),
    [aux_sym_namelist_statement_token1] = ACTIONS(1323),
    [aux_sym_common_statement_token1] = ACTIONS(1323),
    [anon_sym_DASH] = ACTIONS(1325),
    [aux_sym_import_statement_token1] = ACTIONS(1323),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1323),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1323),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1323),
    [aux_sym_variable_attributes_token1] = ACTIONS(1323),
    [aux_sym_variable_attributes_token2] = ACTIONS(1323),
    [aux_sym_variable_attributes_token3] = ACTIONS(1323),
    [aux_sym_variable_attributes_token4] = ACTIONS(1323),
    [aux_sym_variable_attributes_token5] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1323),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1323),
    [aux_sym_derived_type_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token1] = ACTIONS(1323),
    [aux_sym_type_qualifier_token2] = ACTIONS(1323),
    [aux_sym_type_qualifier_token3] = ACTIONS(1323),
    [aux_sym_type_qualifier_token4] = ACTIONS(1323),
    [aux_sym_type_qualifier_token5] = ACTIONS(1323),
    [aux_sym_type_qualifier_token9] = ACTIONS(1323),
    [aux_sym_type_qualifier_token10] = ACTIONS(1323),
    [aux_sym_type_qualifier_token11] = ACTIONS(1323),
    [aux_sym_type_qualifier_token12] = ACTIONS(1323),
    [aux_sym_type_qualifier_token13] = ACTIONS(1323),
    [aux_sym_type_qualifier_token14] = ACTIONS(1323),
    [aux_sym_type_qualifier_token15] = ACTIONS(1323),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token1] = ACTIONS(1323),
    [aux_sym_stop_statement_token2] = ACTIONS(1323),
    [aux_sym_subroutine_call_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token1] = ACTIONS(1323),
    [aux_sym_keyword_statement_token2] = ACTIONS(1323),
    [aux_sym_keyword_statement_token3] = ACTIONS(1323),
    [aux_sym_keyword_statement_token4] = ACTIONS(1323),
    [aux_sym_keyword_statement_token6] = ACTIONS(1323),
    [aux_sym_keyword_statement_token7] = ACTIONS(1323),
    [aux_sym_include_statement_token1] = ACTIONS(1323),
    [aux_sym_data_statement_token1] = ACTIONS(1323),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1323),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1323),
    [aux_sym_end_if_statement_token1] = ACTIONS(1323),
    [aux_sym_elseif_clause_token2] = ACTIONS(1323),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1323),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token1] = ACTIONS(1323),
    [aux_sym_select_case_statement_token3] = ACTIONS(1323),
    [aux_sym_select_type_statement_token1] = ACTIONS(1323),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1323),
    [aux_sym_block_construct_token1] = ACTIONS(1323),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1323),
    [aux_sym_associate_statement_token1] = ACTIONS(1323),
    [aux_sym_format_statement_token1] = ACTIONS(1323),
    [aux_sym_print_statement_token1] = ACTIONS(1323),
    [aux_sym_open_statement_token1] = ACTIONS(1323),
    [aux_sym_close_statement_token1] = ACTIONS(1323),
    [aux_sym_inquire_statement_token1] = ACTIONS(1323),
    [aux_sym_enum_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token1] = ACTIONS(1323),
    [aux_sym_file_position_statement_token2] = ACTIONS(1323),
    [aux_sym_file_position_statement_token3] = ACTIONS(1323),
    [aux_sym_file_position_statement_token4] = ACTIONS(1323),
    [aux_sym_logical_expression_token5] = ACTIONS(1325),
    [anon_sym_PLUS] = ACTIONS(1325),
    [anon_sym_DOT] = ACTIONS(1323),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1325),
    [anon_sym_LBRACK] = ACTIONS(1325),
    [sym_boolean_literal] = ACTIONS(1325),
    [aux_sym_null_literal_token1] = ACTIONS(1323),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1323),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1325),
    [sym__float_literal] = ACTIONS(1325),
    [sym__boz_literal] = ACTIONS(1325),
    [sym_string_literal] = ACTIONS(1325),
  },
  [396] = {
    [sym_preproc_file_line] = STATE(396),
    [aux_sym_end_program_statement_token1] = ACTIONS(1287),
    [anon_sym_LPAREN] = ACTIONS(1287),
    [aux_sym_interface_statement_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1287),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1287),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1287),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1287),
    [aux_sym_use_statement_token1] = ACTIONS(1287),
    [aux_sym_use_statement_token2] = ACTIONS(1287),
    [aux_sym_implicit_statement_token1] = ACTIONS(1287),
    [aux_sym_implicit_statement_token3] = ACTIONS(1287),
    [aux_sym_implicit_statement_token4] = ACTIONS(1287),
    [aux_sym_save_statement_token1] = ACTIONS(1287),
    [aux_sym_private_statement_token1] = ACTIONS(1287),
    [aux_sym_public_statement_token1] = ACTIONS(1287),
    [aux_sym_namelist_statement_token1] = ACTIONS(1287),
    [aux_sym_common_statement_token1] = ACTIONS(1287),
    [anon_sym_DASH] = ACTIONS(1289),
    [aux_sym_import_statement_token1] = ACTIONS(1287),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1287),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1287),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1287),
    [aux_sym_variable_attributes_token1] = ACTIONS(1287),
    [aux_sym_variable_attributes_token2] = ACTIONS(1287),
    [aux_sym_variable_attributes_token3] = ACTIONS(1287),
    [aux_sym_variable_attributes_token4] = ACTIONS(1287),
    [aux_sym_variable_attributes_token5] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1287),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1287),
    [aux_sym_derived_type_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token1] = ACTIONS(1287),
    [aux_sym_type_qualifier_token2] = ACTIONS(1287),
    [aux_sym_type_qualifier_token3] = ACTIONS(1287),
    [aux_sym_type_qualifier_token4] = ACTIONS(1287),
    [aux_sym_type_qualifier_token5] = ACTIONS(1287),
    [aux_sym_type_qualifier_token9] = ACTIONS(1287),
    [aux_sym_type_qualifier_token10] = ACTIONS(1287),
    [aux_sym_type_qualifier_token11] = ACTIONS(1287),
    [aux_sym_type_qualifier_token12] = ACTIONS(1287),
    [aux_sym_type_qualifier_token13] = ACTIONS(1287),
    [aux_sym_type_qualifier_token14] = ACTIONS(1287),
    [aux_sym_type_qualifier_token15] = ACTIONS(1287),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token1] = ACTIONS(1287),
    [aux_sym_stop_statement_token2] = ACTIONS(1287),
    [aux_sym_subroutine_call_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token1] = ACTIONS(1287),
    [aux_sym_keyword_statement_token2] = ACTIONS(1287),
    [aux_sym_keyword_statement_token3] = ACTIONS(1287),
    [aux_sym_keyword_statement_token4] = ACTIONS(1287),
    [aux_sym_keyword_statement_token6] = ACTIONS(1287),
    [aux_sym_keyword_statement_token7] = ACTIONS(1287),
    [aux_sym_include_statement_token1] = ACTIONS(1287),
    [aux_sym_data_statement_token1] = ACTIONS(1287),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1287),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1287),
    [aux_sym_end_if_statement_token1] = ACTIONS(1287),
    [aux_sym_elseif_clause_token2] = ACTIONS(1287),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1287),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token1] = ACTIONS(1287),
    [aux_sym_select_case_statement_token3] = ACTIONS(1287),
    [aux_sym_select_type_statement_token1] = ACTIONS(1287),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1287),
    [aux_sym_block_construct_token1] = ACTIONS(1287),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1287),
    [aux_sym_associate_statement_token1] = ACTIONS(1287),
    [aux_sym_format_statement_token1] = ACTIONS(1287),
    [aux_sym_print_statement_token1] = ACTIONS(1287),
    [aux_sym_open_statement_token1] = ACTIONS(1287),
    [aux_sym_close_statement_token1] = ACTIONS(1287),
    [aux_sym_inquire_statement_token1] = ACTIONS(1287),
    [aux_sym_enum_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token1] = ACTIONS(1287),
    [aux_sym_file_position_statement_token2] = ACTIONS(1287),
    [aux_sym_file_position_statement_token3] = ACTIONS(1287),
    [aux_sym_file_position_statement_token4] = ACTIONS(1287),
    [aux_sym_logical_expression_token5] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1289),
    [anon_sym_DOT] = ACTIONS(1287),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1289),
    [anon_sym_LBRACK] = ACTIONS(1289),
    [sym_boolean_literal] = ACTIONS(1289),
    [aux_sym_null_literal_token1] = ACTIONS(1287),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1287),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1289),
    [sym__float_literal] = ACTIONS(1289),
    [sym__boz_literal] = ACTIONS(1289),
    [sym_string_literal] = ACTIONS(1289),
  },
  [397] = {
    [sym_preproc_file_line] = STATE(397),
    [aux_sym_end_program_statement_token1] = ACTIONS(1295),
    [anon_sym_LPAREN] = ACTIONS(1295),
    [aux_sym_interface_statement_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1295),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1295),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1295),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1295),
    [aux_sym_use_statement_token1] = ACTIONS(1295),
    [aux_sym_use_statement_token2] = ACTIONS(1295),
    [aux_sym_implicit_statement_token1] = ACTIONS(1295),
    [aux_sym_implicit_statement_token3] = ACTIONS(1295),
    [aux_sym_implicit_statement_token4] = ACTIONS(1295),
    [aux_sym_save_statement_token1] = ACTIONS(1295),
    [aux_sym_private_statement_token1] = ACTIONS(1295),
    [aux_sym_public_statement_token1] = ACTIONS(1295),
    [aux_sym_namelist_statement_token1] = ACTIONS(1295),
    [aux_sym_common_statement_token1] = ACTIONS(1295),
    [anon_sym_DASH] = ACTIONS(1297),
    [aux_sym_import_statement_token1] = ACTIONS(1295),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1295),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1295),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1295),
    [aux_sym_variable_attributes_token1] = ACTIONS(1295),
    [aux_sym_variable_attributes_token2] = ACTIONS(1295),
    [aux_sym_variable_attributes_token3] = ACTIONS(1295),
    [aux_sym_variable_attributes_token4] = ACTIONS(1295),
    [aux_sym_variable_attributes_token5] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1295),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1295),
    [aux_sym_derived_type_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token1] = ACTIONS(1295),
    [aux_sym_type_qualifier_token2] = ACTIONS(1295),
    [aux_sym_type_qualifier_token3] = ACTIONS(1295),
    [aux_sym_type_qualifier_token4] = ACTIONS(1295),
    [aux_sym_type_qualifier_token5] = ACTIONS(1295),
    [aux_sym_type_qualifier_token9] = ACTIONS(1295),
    [aux_sym_type_qualifier_token10] = ACTIONS(1295),
    [aux_sym_type_qualifier_token11] = ACTIONS(1295),
    [aux_sym_type_qualifier_token12] = ACTIONS(1295),
    [aux_sym_type_qualifier_token13] = ACTIONS(1295),
    [aux_sym_type_qualifier_token14] = ACTIONS(1295),
    [aux_sym_type_qualifier_token15] = ACTIONS(1295),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token1] = ACTIONS(1295),
    [aux_sym_stop_statement_token2] = ACTIONS(1295),
    [aux_sym_subroutine_call_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token1] = ACTIONS(1295),
    [aux_sym_keyword_statement_token2] = ACTIONS(1295),
    [aux_sym_keyword_statement_token3] = ACTIONS(1295),
    [aux_sym_keyword_statement_token4] = ACTIONS(1295),
    [aux_sym_keyword_statement_token6] = ACTIONS(1295),
    [aux_sym_keyword_statement_token7] = ACTIONS(1295),
    [aux_sym_include_statement_token1] = ACTIONS(1295),
    [aux_sym_data_statement_token1] = ACTIONS(1295),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1295),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1295),
    [aux_sym_end_if_statement_token1] = ACTIONS(1295),
    [aux_sym_elseif_clause_token2] = ACTIONS(1295),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1295),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token1] = ACTIONS(1295),
    [aux_sym_select_case_statement_token3] = ACTIONS(1295),
    [aux_sym_select_type_statement_token1] = ACTIONS(1295),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1295),
    [aux_sym_block_construct_token1] = ACTIONS(1295),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1295),
    [aux_sym_associate_statement_token1] = ACTIONS(1295),
    [aux_sym_format_statement_token1] = ACTIONS(1295),
    [aux_sym_print_statement_token1] = ACTIONS(1295),
    [aux_sym_open_statement_token1] = ACTIONS(1295),
    [aux_sym_close_statement_token1] = ACTIONS(1295),
    [aux_sym_inquire_statement_token1] = ACTIONS(1295),
    [aux_sym_enum_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token1] = ACTIONS(1295),
    [aux_sym_file_position_statement_token2] = ACTIONS(1295),
    [aux_sym_file_position_statement_token3] = ACTIONS(1295),
    [aux_sym_file_position_statement_token4] = ACTIONS(1295),
    [aux_sym_logical_expression_token5] = ACTIONS(1297),
    [anon_sym_PLUS] = ACTIONS(1297),
    [anon_sym_DOT] = ACTIONS(1295),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1297),
    [anon_sym_LBRACK] = ACTIONS(1297),
    [sym_boolean_literal] = ACTIONS(1297),
    [aux_sym_null_literal_token1] = ACTIONS(1295),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1295),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1297),
    [sym__float_literal] = ACTIONS(1297),
    [sym__boz_literal] = ACTIONS(1297),
    [sym_string_literal] = ACTIONS(1297),
  },
  [398] = {
    [sym_preproc_file_line] = STATE(398),
    [aux_sym_end_program_statement_token1] = ACTIONS(1299),
    [anon_sym_LPAREN] = ACTIONS(1299),
    [aux_sym_interface_statement_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1299),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1299),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1299),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1299),
    [aux_sym_use_statement_token1] = ACTIONS(1299),
    [aux_sym_use_statement_token2] = ACTIONS(1299),
    [aux_sym_implicit_statement_token1] = ACTIONS(1299),
    [aux_sym_implicit_statement_token3] = ACTIONS(1299),
    [aux_sym_implicit_statement_token4] = ACTIONS(1299),
    [aux_sym_save_statement_token1] = ACTIONS(1299),
    [aux_sym_private_statement_token1] = ACTIONS(1299),
    [aux_sym_public_statement_token1] = ACTIONS(1299),
    [aux_sym_namelist_statement_token1] = ACTIONS(1299),
    [aux_sym_common_statement_token1] = ACTIONS(1299),
    [anon_sym_DASH] = ACTIONS(1301),
    [aux_sym_import_statement_token1] = ACTIONS(1299),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1299),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1299),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1299),
    [aux_sym_variable_attributes_token1] = ACTIONS(1299),
    [aux_sym_variable_attributes_token2] = ACTIONS(1299),
    [aux_sym_variable_attributes_token3] = ACTIONS(1299),
    [aux_sym_variable_attributes_token4] = ACTIONS(1299),
    [aux_sym_variable_attributes_token5] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1299),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1299),
    [aux_sym_derived_type_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token1] = ACTIONS(1299),
    [aux_sym_type_qualifier_token2] = ACTIONS(1299),
    [aux_sym_type_qualifier_token3] = ACTIONS(1299),
    [aux_sym_type_qualifier_token4] = ACTIONS(1299),
    [aux_sym_type_qualifier_token5] = ACTIONS(1299),
    [aux_sym_type_qualifier_token9] = ACTIONS(1299),
    [aux_sym_type_qualifier_token10] = ACTIONS(1299),
    [aux_sym_type_qualifier_token11] = ACTIONS(1299),
    [aux_sym_type_qualifier_token12] = ACTIONS(1299),
    [aux_sym_type_qualifier_token13] = ACTIONS(1299),
    [aux_sym_type_qualifier_token14] = ACTIONS(1299),
    [aux_sym_type_qualifier_token15] = ACTIONS(1299),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token1] = ACTIONS(1299),
    [aux_sym_stop_statement_token2] = ACTIONS(1299),
    [aux_sym_subroutine_call_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token1] = ACTIONS(1299),
    [aux_sym_keyword_statement_token2] = ACTIONS(1299),
    [aux_sym_keyword_statement_token3] = ACTIONS(1299),
    [aux_sym_keyword_statement_token4] = ACTIONS(1299),
    [aux_sym_keyword_statement_token6] = ACTIONS(1299),
    [aux_sym_keyword_statement_token7] = ACTIONS(1299),
    [aux_sym_include_statement_token1] = ACTIONS(1299),
    [aux_sym_data_statement_token1] = ACTIONS(1299),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1299),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1299),
    [aux_sym_end_if_statement_token1] = ACTIONS(1299),
    [aux_sym_elseif_clause_token2] = ACTIONS(1299),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1299),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token1] = ACTIONS(1299),
    [aux_sym_select_case_statement_token3] = ACTIONS(1299),
    [aux_sym_select_type_statement_token1] = ACTIONS(1299),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1299),
    [aux_sym_block_construct_token1] = ACTIONS(1299),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1299),
    [aux_sym_associate_statement_token1] = ACTIONS(1299),
    [aux_sym_format_statement_token1] = ACTIONS(1299),
    [aux_sym_print_statement_token1] = ACTIONS(1299),
    [aux_sym_open_statement_token1] = ACTIONS(1299),
    [aux_sym_close_statement_token1] = ACTIONS(1299),
    [aux_sym_inquire_statement_token1] = ACTIONS(1299),
    [aux_sym_enum_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token1] = ACTIONS(1299),
    [aux_sym_file_position_statement_token2] = ACTIONS(1299),
    [aux_sym_file_position_statement_token3] = ACTIONS(1299),
    [aux_sym_file_position_statement_token4] = ACTIONS(1299),
    [aux_sym_logical_expression_token5] = ACTIONS(1301),
    [anon_sym_PLUS] = ACTIONS(1301),
    [anon_sym_DOT] = ACTIONS(1299),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1301),
    [anon_sym_LBRACK] = ACTIONS(1301),
    [sym_boolean_literal] = ACTIONS(1301),
    [aux_sym_null_literal_token1] = ACTIONS(1299),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1299),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1301),
    [sym__float_literal] = ACTIONS(1301),
    [sym__boz_literal] = ACTIONS(1301),
    [sym_string_literal] = ACTIONS(1301),
  },
  [399] = {
    [sym_preproc_file_line] = STATE(399),
    [aux_sym_end_program_statement_token1] = ACTIONS(1303),
    [anon_sym_LPAREN] = ACTIONS(1303),
    [aux_sym_interface_statement_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1303),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1303),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1303),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1303),
    [aux_sym_use_statement_token1] = ACTIONS(1303),
    [aux_sym_use_statement_token2] = ACTIONS(1303),
    [aux_sym_implicit_statement_token1] = ACTIONS(1303),
    [aux_sym_implicit_statement_token3] = ACTIONS(1303),
    [aux_sym_implicit_statement_token4] = ACTIONS(1303),
    [aux_sym_save_statement_token1] = ACTIONS(1303),
    [aux_sym_private_statement_token1] = ACTIONS(1303),
    [aux_sym_public_statement_token1] = ACTIONS(1303),
    [aux_sym_namelist_statement_token1] = ACTIONS(1303),
    [aux_sym_common_statement_token1] = ACTIONS(1303),
    [anon_sym_DASH] = ACTIONS(1305),
    [aux_sym_import_statement_token1] = ACTIONS(1303),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1303),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1303),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1303),
    [aux_sym_variable_attributes_token1] = ACTIONS(1303),
    [aux_sym_variable_attributes_token2] = ACTIONS(1303),
    [aux_sym_variable_attributes_token3] = ACTIONS(1303),
    [aux_sym_variable_attributes_token4] = ACTIONS(1303),
    [aux_sym_variable_attributes_token5] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1303),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1303),
    [aux_sym_derived_type_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token1] = ACTIONS(1303),
    [aux_sym_type_qualifier_token2] = ACTIONS(1303),
    [aux_sym_type_qualifier_token3] = ACTIONS(1303),
    [aux_sym_type_qualifier_token4] = ACTIONS(1303),
    [aux_sym_type_qualifier_token5] = ACTIONS(1303),
    [aux_sym_type_qualifier_token9] = ACTIONS(1303),
    [aux_sym_type_qualifier_token10] = ACTIONS(1303),
    [aux_sym_type_qualifier_token11] = ACTIONS(1303),
    [aux_sym_type_qualifier_token12] = ACTIONS(1303),
    [aux_sym_type_qualifier_token13] = ACTIONS(1303),
    [aux_sym_type_qualifier_token14] = ACTIONS(1303),
    [aux_sym_type_qualifier_token15] = ACTIONS(1303),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token1] = ACTIONS(1303),
    [aux_sym_stop_statement_token2] = ACTIONS(1303),
    [aux_sym_subroutine_call_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token1] = ACTIONS(1303),
    [aux_sym_keyword_statement_token2] = ACTIONS(1303),
    [aux_sym_keyword_statement_token3] = ACTIONS(1303),
    [aux_sym_keyword_statement_token4] = ACTIONS(1303),
    [aux_sym_keyword_statement_token6] = ACTIONS(1303),
    [aux_sym_keyword_statement_token7] = ACTIONS(1303),
    [aux_sym_include_statement_token1] = ACTIONS(1303),
    [aux_sym_data_statement_token1] = ACTIONS(1303),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1303),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1303),
    [aux_sym_end_if_statement_token1] = ACTIONS(1303),
    [aux_sym_elseif_clause_token2] = ACTIONS(1303),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1303),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token1] = ACTIONS(1303),
    [aux_sym_select_case_statement_token3] = ACTIONS(1303),
    [aux_sym_select_type_statement_token1] = ACTIONS(1303),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1303),
    [aux_sym_block_construct_token1] = ACTIONS(1303),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1303),
    [aux_sym_associate_statement_token1] = ACTIONS(1303),
    [aux_sym_format_statement_token1] = ACTIONS(1303),
    [aux_sym_print_statement_token1] = ACTIONS(1303),
    [aux_sym_open_statement_token1] = ACTIONS(1303),
    [aux_sym_close_statement_token1] = ACTIONS(1303),
    [aux_sym_inquire_statement_token1] = ACTIONS(1303),
    [aux_sym_enum_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token1] = ACTIONS(1303),
    [aux_sym_file_position_statement_token2] = ACTIONS(1303),
    [aux_sym_file_position_statement_token3] = ACTIONS(1303),
    [aux_sym_file_position_statement_token4] = ACTIONS(1303),
    [aux_sym_logical_expression_token5] = ACTIONS(1305),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_DOT] = ACTIONS(1303),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1305),
    [anon_sym_LBRACK] = ACTIONS(1305),
    [sym_boolean_literal] = ACTIONS(1305),
    [aux_sym_null_literal_token1] = ACTIONS(1303),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1303),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1305),
    [sym__float_literal] = ACTIONS(1305),
    [sym__boz_literal] = ACTIONS(1305),
    [sym_string_literal] = ACTIONS(1305),
  },
  [400] = {
    [sym_preproc_file_line] = STATE(400),
    [aux_sym_end_program_statement_token1] = ACTIONS(1275),
    [anon_sym_LPAREN] = ACTIONS(1275),
    [aux_sym_interface_statement_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1275),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1275),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1275),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1275),
    [aux_sym_use_statement_token1] = ACTIONS(1275),
    [aux_sym_use_statement_token2] = ACTIONS(1275),
    [aux_sym_implicit_statement_token1] = ACTIONS(1275),
    [aux_sym_implicit_statement_token3] = ACTIONS(1275),
    [aux_sym_implicit_statement_token4] = ACTIONS(1275),
    [aux_sym_save_statement_token1] = ACTIONS(1275),
    [aux_sym_private_statement_token1] = ACTIONS(1275),
    [aux_sym_public_statement_token1] = ACTIONS(1275),
    [aux_sym_namelist_statement_token1] = ACTIONS(1275),
    [aux_sym_common_statement_token1] = ACTIONS(1275),
    [anon_sym_DASH] = ACTIONS(1277),
    [aux_sym_import_statement_token1] = ACTIONS(1275),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1275),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1275),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1275),
    [aux_sym_variable_attributes_token1] = ACTIONS(1275),
    [aux_sym_variable_attributes_token2] = ACTIONS(1275),
    [aux_sym_variable_attributes_token3] = ACTIONS(1275),
    [aux_sym_variable_attributes_token4] = ACTIONS(1275),
    [aux_sym_variable_attributes_token5] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1275),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1275),
    [aux_sym_derived_type_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token1] = ACTIONS(1275),
    [aux_sym_type_qualifier_token2] = ACTIONS(1275),
    [aux_sym_type_qualifier_token3] = ACTIONS(1275),
    [aux_sym_type_qualifier_token4] = ACTIONS(1275),
    [aux_sym_type_qualifier_token5] = ACTIONS(1275),
    [aux_sym_type_qualifier_token9] = ACTIONS(1275),
    [aux_sym_type_qualifier_token10] = ACTIONS(1275),
    [aux_sym_type_qualifier_token11] = ACTIONS(1275),
    [aux_sym_type_qualifier_token12] = ACTIONS(1275),
    [aux_sym_type_qualifier_token13] = ACTIONS(1275),
    [aux_sym_type_qualifier_token14] = ACTIONS(1275),
    [aux_sym_type_qualifier_token15] = ACTIONS(1275),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token1] = ACTIONS(1275),
    [aux_sym_stop_statement_token2] = ACTIONS(1275),
    [aux_sym_subroutine_call_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token1] = ACTIONS(1275),
    [aux_sym_keyword_statement_token2] = ACTIONS(1275),
    [aux_sym_keyword_statement_token3] = ACTIONS(1275),
    [aux_sym_keyword_statement_token4] = ACTIONS(1275),
    [aux_sym_keyword_statement_token6] = ACTIONS(1275),
    [aux_sym_keyword_statement_token7] = ACTIONS(1275),
    [aux_sym_include_statement_token1] = ACTIONS(1275),
    [aux_sym_data_statement_token1] = ACTIONS(1275),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1275),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1275),
    [aux_sym_end_if_statement_token1] = ACTIONS(1275),
    [aux_sym_elseif_clause_token2] = ACTIONS(1275),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1275),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token1] = ACTIONS(1275),
    [aux_sym_select_case_statement_token3] = ACTIONS(1275),
    [aux_sym_select_type_statement_token1] = ACTIONS(1275),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1275),
    [aux_sym_block_construct_token1] = ACTIONS(1275),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1275),
    [aux_sym_associate_statement_token1] = ACTIONS(1275),
    [aux_sym_format_statement_token1] = ACTIONS(1275),
    [aux_sym_print_statement_token1] = ACTIONS(1275),
    [aux_sym_open_statement_token1] = ACTIONS(1275),
    [aux_sym_close_statement_token1] = ACTIONS(1275),
    [aux_sym_inquire_statement_token1] = ACTIONS(1275),
    [aux_sym_enum_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token1] = ACTIONS(1275),
    [aux_sym_file_position_statement_token2] = ACTIONS(1275),
    [aux_sym_file_position_statement_token3] = ACTIONS(1275),
    [aux_sym_file_position_statement_token4] = ACTIONS(1275),
    [aux_sym_logical_expression_token5] = ACTIONS(1277),
    [anon_sym_PLUS] = ACTIONS(1277),
    [anon_sym_DOT] = ACTIONS(1275),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1277),
    [anon_sym_LBRACK] = ACTIONS(1277),
    [sym_boolean_literal] = ACTIONS(1277),
    [aux_sym_null_literal_token1] = ACTIONS(1275),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1275),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1277),
    [sym__float_literal] = ACTIONS(1277),
    [sym__boz_literal] = ACTIONS(1277),
    [sym_string_literal] = ACTIONS(1277),
  },
  [401] = {
    [sym_preproc_file_line] = STATE(401),
    [aux_sym_end_program_statement_token1] = ACTIONS(1335),
    [anon_sym_LPAREN] = ACTIONS(1335),
    [aux_sym_interface_statement_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1335),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1335),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1335),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1335),
    [aux_sym_use_statement_token1] = ACTIONS(1335),
    [aux_sym_use_statement_token2] = ACTIONS(1335),
    [aux_sym_implicit_statement_token1] = ACTIONS(1335),
    [aux_sym_implicit_statement_token3] = ACTIONS(1335),
    [aux_sym_implicit_statement_token4] = ACTIONS(1335),
    [aux_sym_save_statement_token1] = ACTIONS(1335),
    [aux_sym_private_statement_token1] = ACTIONS(1335),
    [aux_sym_public_statement_token1] = ACTIONS(1335),
    [aux_sym_namelist_statement_token1] = ACTIONS(1335),
    [aux_sym_common_statement_token1] = ACTIONS(1335),
    [anon_sym_DASH] = ACTIONS(1337),
    [aux_sym_import_statement_token1] = ACTIONS(1335),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1335),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1335),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1335),
    [aux_sym_variable_attributes_token1] = ACTIONS(1335),
    [aux_sym_variable_attributes_token2] = ACTIONS(1335),
    [aux_sym_variable_attributes_token3] = ACTIONS(1335),
    [aux_sym_variable_attributes_token4] = ACTIONS(1335),
    [aux_sym_variable_attributes_token5] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1335),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1335),
    [aux_sym_derived_type_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token1] = ACTIONS(1335),
    [aux_sym_type_qualifier_token2] = ACTIONS(1335),
    [aux_sym_type_qualifier_token3] = ACTIONS(1335),
    [aux_sym_type_qualifier_token4] = ACTIONS(1335),
    [aux_sym_type_qualifier_token5] = ACTIONS(1335),
    [aux_sym_type_qualifier_token9] = ACTIONS(1335),
    [aux_sym_type_qualifier_token10] = ACTIONS(1335),
    [aux_sym_type_qualifier_token11] = ACTIONS(1335),
    [aux_sym_type_qualifier_token12] = ACTIONS(1335),
    [aux_sym_type_qualifier_token13] = ACTIONS(1335),
    [aux_sym_type_qualifier_token14] = ACTIONS(1335),
    [aux_sym_type_qualifier_token15] = ACTIONS(1335),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token1] = ACTIONS(1335),
    [aux_sym_stop_statement_token2] = ACTIONS(1335),
    [aux_sym_subroutine_call_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token1] = ACTIONS(1335),
    [aux_sym_keyword_statement_token2] = ACTIONS(1335),
    [aux_sym_keyword_statement_token3] = ACTIONS(1335),
    [aux_sym_keyword_statement_token4] = ACTIONS(1335),
    [aux_sym_keyword_statement_token6] = ACTIONS(1335),
    [aux_sym_keyword_statement_token7] = ACTIONS(1335),
    [aux_sym_include_statement_token1] = ACTIONS(1335),
    [aux_sym_data_statement_token1] = ACTIONS(1335),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1335),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1335),
    [aux_sym_end_if_statement_token1] = ACTIONS(1335),
    [aux_sym_elseif_clause_token2] = ACTIONS(1335),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1335),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token1] = ACTIONS(1335),
    [aux_sym_select_case_statement_token3] = ACTIONS(1335),
    [aux_sym_select_type_statement_token1] = ACTIONS(1335),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1335),
    [aux_sym_block_construct_token1] = ACTIONS(1335),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1335),
    [aux_sym_associate_statement_token1] = ACTIONS(1335),
    [aux_sym_format_statement_token1] = ACTIONS(1335),
    [aux_sym_print_statement_token1] = ACTIONS(1335),
    [aux_sym_open_statement_token1] = ACTIONS(1335),
    [aux_sym_close_statement_token1] = ACTIONS(1335),
    [aux_sym_inquire_statement_token1] = ACTIONS(1335),
    [aux_sym_enum_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token1] = ACTIONS(1335),
    [aux_sym_file_position_statement_token2] = ACTIONS(1335),
    [aux_sym_file_position_statement_token3] = ACTIONS(1335),
    [aux_sym_file_position_statement_token4] = ACTIONS(1335),
    [aux_sym_logical_expression_token5] = ACTIONS(1337),
    [anon_sym_PLUS] = ACTIONS(1337),
    [anon_sym_DOT] = ACTIONS(1335),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1337),
    [anon_sym_LBRACK] = ACTIONS(1337),
    [sym_boolean_literal] = ACTIONS(1337),
    [aux_sym_null_literal_token1] = ACTIONS(1335),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1335),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1337),
    [sym__float_literal] = ACTIONS(1337),
    [sym__boz_literal] = ACTIONS(1337),
    [sym_string_literal] = ACTIONS(1337),
  },
  [402] = {
    [sym_preproc_file_line] = STATE(402),
    [aux_sym_end_program_statement_token1] = ACTIONS(1259),
    [anon_sym_LPAREN] = ACTIONS(1259),
    [aux_sym_interface_statement_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1259),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1259),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1259),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1259),
    [aux_sym_use_statement_token1] = ACTIONS(1259),
    [aux_sym_use_statement_token2] = ACTIONS(1259),
    [aux_sym_implicit_statement_token1] = ACTIONS(1259),
    [aux_sym_implicit_statement_token3] = ACTIONS(1259),
    [aux_sym_implicit_statement_token4] = ACTIONS(1259),
    [aux_sym_save_statement_token1] = ACTIONS(1259),
    [aux_sym_private_statement_token1] = ACTIONS(1259),
    [aux_sym_public_statement_token1] = ACTIONS(1259),
    [aux_sym_namelist_statement_token1] = ACTIONS(1259),
    [aux_sym_common_statement_token1] = ACTIONS(1259),
    [anon_sym_DASH] = ACTIONS(1257),
    [aux_sym_import_statement_token1] = ACTIONS(1259),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1259),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1259),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1259),
    [aux_sym_variable_attributes_token1] = ACTIONS(1259),
    [aux_sym_variable_attributes_token2] = ACTIONS(1259),
    [aux_sym_variable_attributes_token3] = ACTIONS(1259),
    [aux_sym_variable_attributes_token4] = ACTIONS(1259),
    [aux_sym_variable_attributes_token5] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1259),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1259),
    [aux_sym_derived_type_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token1] = ACTIONS(1259),
    [aux_sym_type_qualifier_token2] = ACTIONS(1259),
    [aux_sym_type_qualifier_token3] = ACTIONS(1259),
    [aux_sym_type_qualifier_token4] = ACTIONS(1259),
    [aux_sym_type_qualifier_token5] = ACTIONS(1259),
    [aux_sym_type_qualifier_token9] = ACTIONS(1259),
    [aux_sym_type_qualifier_token10] = ACTIONS(1259),
    [aux_sym_type_qualifier_token11] = ACTIONS(1259),
    [aux_sym_type_qualifier_token12] = ACTIONS(1259),
    [aux_sym_type_qualifier_token13] = ACTIONS(1259),
    [aux_sym_type_qualifier_token14] = ACTIONS(1259),
    [aux_sym_type_qualifier_token15] = ACTIONS(1259),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token1] = ACTIONS(1259),
    [aux_sym_stop_statement_token2] = ACTIONS(1259),
    [aux_sym_subroutine_call_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token1] = ACTIONS(1259),
    [aux_sym_keyword_statement_token2] = ACTIONS(1259),
    [aux_sym_keyword_statement_token3] = ACTIONS(1259),
    [aux_sym_keyword_statement_token4] = ACTIONS(1259),
    [aux_sym_keyword_statement_token6] = ACTIONS(1259),
    [aux_sym_keyword_statement_token7] = ACTIONS(1259),
    [aux_sym_include_statement_token1] = ACTIONS(1259),
    [aux_sym_data_statement_token1] = ACTIONS(1259),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1259),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1259),
    [aux_sym_end_if_statement_token1] = ACTIONS(1259),
    [aux_sym_elseif_clause_token2] = ACTIONS(1259),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1259),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token1] = ACTIONS(1259),
    [aux_sym_select_case_statement_token3] = ACTIONS(1259),
    [aux_sym_select_type_statement_token1] = ACTIONS(1259),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1259),
    [aux_sym_block_construct_token1] = ACTIONS(1259),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1259),
    [aux_sym_associate_statement_token1] = ACTIONS(1259),
    [aux_sym_format_statement_token1] = ACTIONS(1259),
    [aux_sym_print_statement_token1] = ACTIONS(1259),
    [aux_sym_open_statement_token1] = ACTIONS(1259),
    [aux_sym_close_statement_token1] = ACTIONS(1259),
    [aux_sym_inquire_statement_token1] = ACTIONS(1259),
    [aux_sym_enum_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token1] = ACTIONS(1259),
    [aux_sym_file_position_statement_token2] = ACTIONS(1259),
    [aux_sym_file_position_statement_token3] = ACTIONS(1259),
    [aux_sym_file_position_statement_token4] = ACTIONS(1259),
    [aux_sym_logical_expression_token5] = ACTIONS(1257),
    [anon_sym_PLUS] = ACTIONS(1257),
    [anon_sym_DOT] = ACTIONS(1259),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1257),
    [anon_sym_LBRACK] = ACTIONS(1257),
    [sym_boolean_literal] = ACTIONS(1257),
    [aux_sym_null_literal_token1] = ACTIONS(1259),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1259),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1257),
    [sym__float_literal] = ACTIONS(1257),
    [sym__boz_literal] = ACTIONS(1257),
    [sym_string_literal] = ACTIONS(1257),
  },
  [403] = {
    [sym_preproc_file_line] = STATE(403),
    [aux_sym_end_program_statement_token1] = ACTIONS(1343),
    [anon_sym_LPAREN] = ACTIONS(1343),
    [aux_sym_interface_statement_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1343),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1343),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1343),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1343),
    [aux_sym_use_statement_token1] = ACTIONS(1343),
    [aux_sym_use_statement_token2] = ACTIONS(1343),
    [aux_sym_implicit_statement_token1] = ACTIONS(1343),
    [aux_sym_implicit_statement_token3] = ACTIONS(1343),
    [aux_sym_implicit_statement_token4] = ACTIONS(1343),
    [aux_sym_save_statement_token1] = ACTIONS(1343),
    [aux_sym_private_statement_token1] = ACTIONS(1343),
    [aux_sym_public_statement_token1] = ACTIONS(1343),
    [aux_sym_namelist_statement_token1] = ACTIONS(1343),
    [aux_sym_common_statement_token1] = ACTIONS(1343),
    [anon_sym_DASH] = ACTIONS(1345),
    [aux_sym_import_statement_token1] = ACTIONS(1343),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1343),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1343),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1343),
    [aux_sym_variable_attributes_token1] = ACTIONS(1343),
    [aux_sym_variable_attributes_token2] = ACTIONS(1343),
    [aux_sym_variable_attributes_token3] = ACTIONS(1343),
    [aux_sym_variable_attributes_token4] = ACTIONS(1343),
    [aux_sym_variable_attributes_token5] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1343),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1343),
    [aux_sym_derived_type_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token1] = ACTIONS(1343),
    [aux_sym_type_qualifier_token2] = ACTIONS(1343),
    [aux_sym_type_qualifier_token3] = ACTIONS(1343),
    [aux_sym_type_qualifier_token4] = ACTIONS(1343),
    [aux_sym_type_qualifier_token5] = ACTIONS(1343),
    [aux_sym_type_qualifier_token9] = ACTIONS(1343),
    [aux_sym_type_qualifier_token10] = ACTIONS(1343),
    [aux_sym_type_qualifier_token11] = ACTIONS(1343),
    [aux_sym_type_qualifier_token12] = ACTIONS(1343),
    [aux_sym_type_qualifier_token13] = ACTIONS(1343),
    [aux_sym_type_qualifier_token14] = ACTIONS(1343),
    [aux_sym_type_qualifier_token15] = ACTIONS(1343),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token1] = ACTIONS(1343),
    [aux_sym_stop_statement_token2] = ACTIONS(1343),
    [aux_sym_subroutine_call_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token1] = ACTIONS(1343),
    [aux_sym_keyword_statement_token2] = ACTIONS(1343),
    [aux_sym_keyword_statement_token3] = ACTIONS(1343),
    [aux_sym_keyword_statement_token4] = ACTIONS(1343),
    [aux_sym_keyword_statement_token6] = ACTIONS(1343),
    [aux_sym_keyword_statement_token7] = ACTIONS(1343),
    [aux_sym_include_statement_token1] = ACTIONS(1343),
    [aux_sym_data_statement_token1] = ACTIONS(1343),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1343),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1343),
    [aux_sym_end_if_statement_token1] = ACTIONS(1343),
    [aux_sym_elseif_clause_token2] = ACTIONS(1343),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1343),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token1] = ACTIONS(1343),
    [aux_sym_select_case_statement_token3] = ACTIONS(1343),
    [aux_sym_select_type_statement_token1] = ACTIONS(1343),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1343),
    [aux_sym_block_construct_token1] = ACTIONS(1343),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1343),
    [aux_sym_associate_statement_token1] = ACTIONS(1343),
    [aux_sym_format_statement_token1] = ACTIONS(1343),
    [aux_sym_print_statement_token1] = ACTIONS(1343),
    [aux_sym_open_statement_token1] = ACTIONS(1343),
    [aux_sym_close_statement_token1] = ACTIONS(1343),
    [aux_sym_inquire_statement_token1] = ACTIONS(1343),
    [aux_sym_enum_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token1] = ACTIONS(1343),
    [aux_sym_file_position_statement_token2] = ACTIONS(1343),
    [aux_sym_file_position_statement_token3] = ACTIONS(1343),
    [aux_sym_file_position_statement_token4] = ACTIONS(1343),
    [aux_sym_logical_expression_token5] = ACTIONS(1345),
    [anon_sym_PLUS] = ACTIONS(1345),
    [anon_sym_DOT] = ACTIONS(1343),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1345),
    [anon_sym_LBRACK] = ACTIONS(1345),
    [sym_boolean_literal] = ACTIONS(1345),
    [aux_sym_null_literal_token1] = ACTIONS(1343),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1343),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1345),
    [sym__float_literal] = ACTIONS(1345),
    [sym__boz_literal] = ACTIONS(1345),
    [sym_string_literal] = ACTIONS(1345),
  },
  [404] = {
    [sym_preproc_file_line] = STATE(404),
    [aux_sym_end_program_statement_token1] = ACTIONS(1147),
    [anon_sym_LPAREN] = ACTIONS(1147),
    [aux_sym_interface_statement_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1147),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1147),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1147),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1147),
    [aux_sym_use_statement_token1] = ACTIONS(1147),
    [aux_sym_use_statement_token2] = ACTIONS(1147),
    [aux_sym_implicit_statement_token1] = ACTIONS(1147),
    [aux_sym_implicit_statement_token3] = ACTIONS(1147),
    [aux_sym_implicit_statement_token4] = ACTIONS(1147),
    [aux_sym_save_statement_token1] = ACTIONS(1147),
    [aux_sym_private_statement_token1] = ACTIONS(1147),
    [aux_sym_public_statement_token1] = ACTIONS(1147),
    [aux_sym_namelist_statement_token1] = ACTIONS(1147),
    [aux_sym_common_statement_token1] = ACTIONS(1147),
    [anon_sym_DASH] = ACTIONS(1145),
    [aux_sym_import_statement_token1] = ACTIONS(1147),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1147),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1147),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1147),
    [aux_sym_variable_attributes_token1] = ACTIONS(1147),
    [aux_sym_variable_attributes_token2] = ACTIONS(1147),
    [aux_sym_variable_attributes_token3] = ACTIONS(1147),
    [aux_sym_variable_attributes_token4] = ACTIONS(1147),
    [aux_sym_variable_attributes_token5] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1147),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1147),
    [aux_sym_derived_type_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token1] = ACTIONS(1147),
    [aux_sym_type_qualifier_token2] = ACTIONS(1147),
    [aux_sym_type_qualifier_token3] = ACTIONS(1147),
    [aux_sym_type_qualifier_token4] = ACTIONS(1147),
    [aux_sym_type_qualifier_token5] = ACTIONS(1147),
    [aux_sym_type_qualifier_token9] = ACTIONS(1147),
    [aux_sym_type_qualifier_token10] = ACTIONS(1147),
    [aux_sym_type_qualifier_token11] = ACTIONS(1147),
    [aux_sym_type_qualifier_token12] = ACTIONS(1147),
    [aux_sym_type_qualifier_token13] = ACTIONS(1147),
    [aux_sym_type_qualifier_token14] = ACTIONS(1147),
    [aux_sym_type_qualifier_token15] = ACTIONS(1147),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token1] = ACTIONS(1147),
    [aux_sym_stop_statement_token2] = ACTIONS(1147),
    [aux_sym_subroutine_call_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token1] = ACTIONS(1147),
    [aux_sym_keyword_statement_token2] = ACTIONS(1147),
    [aux_sym_keyword_statement_token3] = ACTIONS(1147),
    [aux_sym_keyword_statement_token4] = ACTIONS(1147),
    [aux_sym_keyword_statement_token6] = ACTIONS(1147),
    [aux_sym_keyword_statement_token7] = ACTIONS(1147),
    [aux_sym_include_statement_token1] = ACTIONS(1147),
    [aux_sym_data_statement_token1] = ACTIONS(1147),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1147),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1147),
    [aux_sym_end_if_statement_token1] = ACTIONS(1147),
    [aux_sym_elseif_clause_token2] = ACTIONS(1147),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1147),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token1] = ACTIONS(1147),
    [aux_sym_select_case_statement_token3] = ACTIONS(1147),
    [aux_sym_select_type_statement_token1] = ACTIONS(1147),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1147),
    [aux_sym_block_construct_token1] = ACTIONS(1147),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1147),
    [aux_sym_associate_statement_token1] = ACTIONS(1147),
    [aux_sym_format_statement_token1] = ACTIONS(1147),
    [aux_sym_print_statement_token1] = ACTIONS(1147),
    [aux_sym_open_statement_token1] = ACTIONS(1147),
    [aux_sym_close_statement_token1] = ACTIONS(1147),
    [aux_sym_inquire_statement_token1] = ACTIONS(1147),
    [aux_sym_enum_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token1] = ACTIONS(1147),
    [aux_sym_file_position_statement_token2] = ACTIONS(1147),
    [aux_sym_file_position_statement_token3] = ACTIONS(1147),
    [aux_sym_file_position_statement_token4] = ACTIONS(1147),
    [aux_sym_logical_expression_token5] = ACTIONS(1145),
    [anon_sym_PLUS] = ACTIONS(1145),
    [anon_sym_DOT] = ACTIONS(1147),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1145),
    [anon_sym_LBRACK] = ACTIONS(1145),
    [sym_boolean_literal] = ACTIONS(1145),
    [aux_sym_null_literal_token1] = ACTIONS(1147),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1147),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1145),
    [sym__float_literal] = ACTIONS(1145),
    [sym__boz_literal] = ACTIONS(1145),
    [sym_string_literal] = ACTIONS(1145),
  },
  [405] = {
    [sym_preproc_file_line] = STATE(405),
    [aux_sym_end_program_statement_token1] = ACTIONS(1311),
    [anon_sym_LPAREN] = ACTIONS(1311),
    [aux_sym_interface_statement_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1311),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1311),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1311),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1311),
    [aux_sym_use_statement_token1] = ACTIONS(1311),
    [aux_sym_use_statement_token2] = ACTIONS(1311),
    [aux_sym_implicit_statement_token1] = ACTIONS(1311),
    [aux_sym_implicit_statement_token3] = ACTIONS(1311),
    [aux_sym_implicit_statement_token4] = ACTIONS(1311),
    [aux_sym_save_statement_token1] = ACTIONS(1311),
    [aux_sym_private_statement_token1] = ACTIONS(1311),
    [aux_sym_public_statement_token1] = ACTIONS(1311),
    [aux_sym_namelist_statement_token1] = ACTIONS(1311),
    [aux_sym_common_statement_token1] = ACTIONS(1311),
    [anon_sym_DASH] = ACTIONS(1313),
    [aux_sym_import_statement_token1] = ACTIONS(1311),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1311),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1311),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1311),
    [aux_sym_variable_attributes_token1] = ACTIONS(1311),
    [aux_sym_variable_attributes_token2] = ACTIONS(1311),
    [aux_sym_variable_attributes_token3] = ACTIONS(1311),
    [aux_sym_variable_attributes_token4] = ACTIONS(1311),
    [aux_sym_variable_attributes_token5] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1311),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1311),
    [aux_sym_derived_type_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token1] = ACTIONS(1311),
    [aux_sym_type_qualifier_token2] = ACTIONS(1311),
    [aux_sym_type_qualifier_token3] = ACTIONS(1311),
    [aux_sym_type_qualifier_token4] = ACTIONS(1311),
    [aux_sym_type_qualifier_token5] = ACTIONS(1311),
    [aux_sym_type_qualifier_token9] = ACTIONS(1311),
    [aux_sym_type_qualifier_token10] = ACTIONS(1311),
    [aux_sym_type_qualifier_token11] = ACTIONS(1311),
    [aux_sym_type_qualifier_token12] = ACTIONS(1311),
    [aux_sym_type_qualifier_token13] = ACTIONS(1311),
    [aux_sym_type_qualifier_token14] = ACTIONS(1311),
    [aux_sym_type_qualifier_token15] = ACTIONS(1311),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token1] = ACTIONS(1311),
    [aux_sym_stop_statement_token2] = ACTIONS(1311),
    [aux_sym_subroutine_call_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token1] = ACTIONS(1311),
    [aux_sym_keyword_statement_token2] = ACTIONS(1311),
    [aux_sym_keyword_statement_token3] = ACTIONS(1311),
    [aux_sym_keyword_statement_token4] = ACTIONS(1311),
    [aux_sym_keyword_statement_token6] = ACTIONS(1311),
    [aux_sym_keyword_statement_token7] = ACTIONS(1311),
    [aux_sym_include_statement_token1] = ACTIONS(1311),
    [aux_sym_data_statement_token1] = ACTIONS(1311),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1311),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1311),
    [aux_sym_end_if_statement_token1] = ACTIONS(1311),
    [aux_sym_elseif_clause_token2] = ACTIONS(1311),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1311),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token1] = ACTIONS(1311),
    [aux_sym_select_case_statement_token3] = ACTIONS(1311),
    [aux_sym_select_type_statement_token1] = ACTIONS(1311),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1311),
    [aux_sym_block_construct_token1] = ACTIONS(1311),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1311),
    [aux_sym_associate_statement_token1] = ACTIONS(1311),
    [aux_sym_format_statement_token1] = ACTIONS(1311),
    [aux_sym_print_statement_token1] = ACTIONS(1311),
    [aux_sym_open_statement_token1] = ACTIONS(1311),
    [aux_sym_close_statement_token1] = ACTIONS(1311),
    [aux_sym_inquire_statement_token1] = ACTIONS(1311),
    [aux_sym_enum_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token1] = ACTIONS(1311),
    [aux_sym_file_position_statement_token2] = ACTIONS(1311),
    [aux_sym_file_position_statement_token3] = ACTIONS(1311),
    [aux_sym_file_position_statement_token4] = ACTIONS(1311),
    [aux_sym_logical_expression_token5] = ACTIONS(1313),
    [anon_sym_PLUS] = ACTIONS(1313),
    [anon_sym_DOT] = ACTIONS(1311),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1313),
    [anon_sym_LBRACK] = ACTIONS(1313),
    [sym_boolean_literal] = ACTIONS(1313),
    [aux_sym_null_literal_token1] = ACTIONS(1311),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1311),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1313),
    [sym__float_literal] = ACTIONS(1313),
    [sym__boz_literal] = ACTIONS(1313),
    [sym_string_literal] = ACTIONS(1313),
  },
  [406] = {
    [sym_preproc_file_line] = STATE(406),
    [aux_sym_end_program_statement_token1] = ACTIONS(1247),
    [anon_sym_LPAREN] = ACTIONS(1247),
    [aux_sym_interface_statement_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1247),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1247),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1247),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1247),
    [aux_sym_use_statement_token1] = ACTIONS(1247),
    [aux_sym_use_statement_token2] = ACTIONS(1247),
    [aux_sym_implicit_statement_token1] = ACTIONS(1247),
    [aux_sym_implicit_statement_token3] = ACTIONS(1247),
    [aux_sym_implicit_statement_token4] = ACTIONS(1247),
    [aux_sym_save_statement_token1] = ACTIONS(1247),
    [aux_sym_private_statement_token1] = ACTIONS(1247),
    [aux_sym_public_statement_token1] = ACTIONS(1247),
    [aux_sym_namelist_statement_token1] = ACTIONS(1247),
    [aux_sym_common_statement_token1] = ACTIONS(1247),
    [anon_sym_DASH] = ACTIONS(1245),
    [aux_sym_import_statement_token1] = ACTIONS(1247),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1247),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1247),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1247),
    [aux_sym_variable_attributes_token1] = ACTIONS(1247),
    [aux_sym_variable_attributes_token2] = ACTIONS(1247),
    [aux_sym_variable_attributes_token3] = ACTIONS(1247),
    [aux_sym_variable_attributes_token4] = ACTIONS(1247),
    [aux_sym_variable_attributes_token5] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1247),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1247),
    [aux_sym_derived_type_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token1] = ACTIONS(1247),
    [aux_sym_type_qualifier_token2] = ACTIONS(1247),
    [aux_sym_type_qualifier_token3] = ACTIONS(1247),
    [aux_sym_type_qualifier_token4] = ACTIONS(1247),
    [aux_sym_type_qualifier_token5] = ACTIONS(1247),
    [aux_sym_type_qualifier_token9] = ACTIONS(1247),
    [aux_sym_type_qualifier_token10] = ACTIONS(1247),
    [aux_sym_type_qualifier_token11] = ACTIONS(1247),
    [aux_sym_type_qualifier_token12] = ACTIONS(1247),
    [aux_sym_type_qualifier_token13] = ACTIONS(1247),
    [aux_sym_type_qualifier_token14] = ACTIONS(1247),
    [aux_sym_type_qualifier_token15] = ACTIONS(1247),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token1] = ACTIONS(1247),
    [aux_sym_stop_statement_token2] = ACTIONS(1247),
    [aux_sym_subroutine_call_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token1] = ACTIONS(1247),
    [aux_sym_keyword_statement_token2] = ACTIONS(1247),
    [aux_sym_keyword_statement_token3] = ACTIONS(1247),
    [aux_sym_keyword_statement_token4] = ACTIONS(1247),
    [aux_sym_keyword_statement_token6] = ACTIONS(1247),
    [aux_sym_keyword_statement_token7] = ACTIONS(1247),
    [aux_sym_include_statement_token1] = ACTIONS(1247),
    [aux_sym_data_statement_token1] = ACTIONS(1247),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1247),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1247),
    [aux_sym_end_if_statement_token1] = ACTIONS(1247),
    [aux_sym_elseif_clause_token2] = ACTIONS(1247),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1247),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token1] = ACTIONS(1247),
    [aux_sym_select_case_statement_token3] = ACTIONS(1247),
    [aux_sym_select_type_statement_token1] = ACTIONS(1247),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1247),
    [aux_sym_block_construct_token1] = ACTIONS(1247),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1247),
    [aux_sym_associate_statement_token1] = ACTIONS(1247),
    [aux_sym_format_statement_token1] = ACTIONS(1247),
    [aux_sym_print_statement_token1] = ACTIONS(1247),
    [aux_sym_open_statement_token1] = ACTIONS(1247),
    [aux_sym_close_statement_token1] = ACTIONS(1247),
    [aux_sym_inquire_statement_token1] = ACTIONS(1247),
    [aux_sym_enum_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token1] = ACTIONS(1247),
    [aux_sym_file_position_statement_token2] = ACTIONS(1247),
    [aux_sym_file_position_statement_token3] = ACTIONS(1247),
    [aux_sym_file_position_statement_token4] = ACTIONS(1247),
    [aux_sym_logical_expression_token5] = ACTIONS(1245),
    [anon_sym_PLUS] = ACTIONS(1245),
    [anon_sym_DOT] = ACTIONS(1247),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1245),
    [anon_sym_LBRACK] = ACTIONS(1245),
    [sym_boolean_literal] = ACTIONS(1245),
    [aux_sym_null_literal_token1] = ACTIONS(1247),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1247),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1245),
    [sym__float_literal] = ACTIONS(1245),
    [sym__boz_literal] = ACTIONS(1245),
    [sym_string_literal] = ACTIONS(1245),
  },
  [407] = {
    [sym_preproc_file_line] = STATE(407),
    [aux_sym_end_program_statement_token1] = ACTIONS(1099),
    [anon_sym_LPAREN] = ACTIONS(1099),
    [aux_sym_interface_statement_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1099),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1099),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1099),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1099),
    [aux_sym_use_statement_token1] = ACTIONS(1099),
    [aux_sym_use_statement_token2] = ACTIONS(1099),
    [aux_sym_implicit_statement_token1] = ACTIONS(1099),
    [aux_sym_implicit_statement_token3] = ACTIONS(1099),
    [aux_sym_implicit_statement_token4] = ACTIONS(1099),
    [aux_sym_save_statement_token1] = ACTIONS(1099),
    [aux_sym_private_statement_token1] = ACTIONS(1099),
    [aux_sym_public_statement_token1] = ACTIONS(1099),
    [aux_sym_namelist_statement_token1] = ACTIONS(1099),
    [aux_sym_common_statement_token1] = ACTIONS(1099),
    [anon_sym_DASH] = ACTIONS(1097),
    [aux_sym_import_statement_token1] = ACTIONS(1099),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1099),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1099),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1099),
    [aux_sym_variable_attributes_token1] = ACTIONS(1099),
    [aux_sym_variable_attributes_token2] = ACTIONS(1099),
    [aux_sym_variable_attributes_token3] = ACTIONS(1099),
    [aux_sym_variable_attributes_token4] = ACTIONS(1099),
    [aux_sym_variable_attributes_token5] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1099),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1099),
    [aux_sym_derived_type_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token1] = ACTIONS(1099),
    [aux_sym_type_qualifier_token2] = ACTIONS(1099),
    [aux_sym_type_qualifier_token3] = ACTIONS(1099),
    [aux_sym_type_qualifier_token4] = ACTIONS(1099),
    [aux_sym_type_qualifier_token5] = ACTIONS(1099),
    [aux_sym_type_qualifier_token9] = ACTIONS(1099),
    [aux_sym_type_qualifier_token10] = ACTIONS(1099),
    [aux_sym_type_qualifier_token11] = ACTIONS(1099),
    [aux_sym_type_qualifier_token12] = ACTIONS(1099),
    [aux_sym_type_qualifier_token13] = ACTIONS(1099),
    [aux_sym_type_qualifier_token14] = ACTIONS(1099),
    [aux_sym_type_qualifier_token15] = ACTIONS(1099),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token1] = ACTIONS(1099),
    [aux_sym_stop_statement_token2] = ACTIONS(1099),
    [aux_sym_subroutine_call_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token1] = ACTIONS(1099),
    [aux_sym_keyword_statement_token2] = ACTIONS(1099),
    [aux_sym_keyword_statement_token3] = ACTIONS(1099),
    [aux_sym_keyword_statement_token4] = ACTIONS(1099),
    [aux_sym_keyword_statement_token6] = ACTIONS(1099),
    [aux_sym_keyword_statement_token7] = ACTIONS(1099),
    [aux_sym_include_statement_token1] = ACTIONS(1099),
    [aux_sym_data_statement_token1] = ACTIONS(1099),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1099),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1099),
    [aux_sym_end_if_statement_token1] = ACTIONS(1099),
    [aux_sym_elseif_clause_token2] = ACTIONS(1099),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1099),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token1] = ACTIONS(1099),
    [aux_sym_select_case_statement_token3] = ACTIONS(1099),
    [aux_sym_select_type_statement_token1] = ACTIONS(1099),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1099),
    [aux_sym_block_construct_token1] = ACTIONS(1099),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1099),
    [aux_sym_associate_statement_token1] = ACTIONS(1099),
    [aux_sym_format_statement_token1] = ACTIONS(1099),
    [aux_sym_print_statement_token1] = ACTIONS(1099),
    [aux_sym_open_statement_token1] = ACTIONS(1099),
    [aux_sym_close_statement_token1] = ACTIONS(1099),
    [aux_sym_inquire_statement_token1] = ACTIONS(1099),
    [aux_sym_enum_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token1] = ACTIONS(1099),
    [aux_sym_file_position_statement_token2] = ACTIONS(1099),
    [aux_sym_file_position_statement_token3] = ACTIONS(1099),
    [aux_sym_file_position_statement_token4] = ACTIONS(1099),
    [aux_sym_logical_expression_token5] = ACTIONS(1097),
    [anon_sym_PLUS] = ACTIONS(1097),
    [anon_sym_DOT] = ACTIONS(1099),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1097),
    [anon_sym_LBRACK] = ACTIONS(1097),
    [sym_boolean_literal] = ACTIONS(1097),
    [aux_sym_null_literal_token1] = ACTIONS(1099),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1099),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1097),
    [sym__float_literal] = ACTIONS(1097),
    [sym__boz_literal] = ACTIONS(1097),
    [sym_string_literal] = ACTIONS(1097),
  },
  [408] = {
    [sym_preproc_file_line] = STATE(408),
    [aux_sym_end_program_statement_token1] = ACTIONS(1199),
    [anon_sym_LPAREN] = ACTIONS(1199),
    [aux_sym_interface_statement_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1199),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1199),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1199),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1199),
    [aux_sym_use_statement_token1] = ACTIONS(1199),
    [aux_sym_use_statement_token2] = ACTIONS(1199),
    [aux_sym_implicit_statement_token1] = ACTIONS(1199),
    [aux_sym_implicit_statement_token3] = ACTIONS(1199),
    [aux_sym_implicit_statement_token4] = ACTIONS(1199),
    [aux_sym_save_statement_token1] = ACTIONS(1199),
    [aux_sym_private_statement_token1] = ACTIONS(1199),
    [aux_sym_public_statement_token1] = ACTIONS(1199),
    [aux_sym_namelist_statement_token1] = ACTIONS(1199),
    [aux_sym_common_statement_token1] = ACTIONS(1199),
    [anon_sym_DASH] = ACTIONS(1197),
    [aux_sym_import_statement_token1] = ACTIONS(1199),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1199),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1199),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1199),
    [aux_sym_variable_attributes_token1] = ACTIONS(1199),
    [aux_sym_variable_attributes_token2] = ACTIONS(1199),
    [aux_sym_variable_attributes_token3] = ACTIONS(1199),
    [aux_sym_variable_attributes_token4] = ACTIONS(1199),
    [aux_sym_variable_attributes_token5] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1199),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1199),
    [aux_sym_derived_type_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token1] = ACTIONS(1199),
    [aux_sym_type_qualifier_token2] = ACTIONS(1199),
    [aux_sym_type_qualifier_token3] = ACTIONS(1199),
    [aux_sym_type_qualifier_token4] = ACTIONS(1199),
    [aux_sym_type_qualifier_token5] = ACTIONS(1199),
    [aux_sym_type_qualifier_token9] = ACTIONS(1199),
    [aux_sym_type_qualifier_token10] = ACTIONS(1199),
    [aux_sym_type_qualifier_token11] = ACTIONS(1199),
    [aux_sym_type_qualifier_token12] = ACTIONS(1199),
    [aux_sym_type_qualifier_token13] = ACTIONS(1199),
    [aux_sym_type_qualifier_token14] = ACTIONS(1199),
    [aux_sym_type_qualifier_token15] = ACTIONS(1199),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token1] = ACTIONS(1199),
    [aux_sym_stop_statement_token2] = ACTIONS(1199),
    [aux_sym_subroutine_call_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token1] = ACTIONS(1199),
    [aux_sym_keyword_statement_token2] = ACTIONS(1199),
    [aux_sym_keyword_statement_token3] = ACTIONS(1199),
    [aux_sym_keyword_statement_token4] = ACTIONS(1199),
    [aux_sym_keyword_statement_token6] = ACTIONS(1199),
    [aux_sym_keyword_statement_token7] = ACTIONS(1199),
    [aux_sym_include_statement_token1] = ACTIONS(1199),
    [aux_sym_data_statement_token1] = ACTIONS(1199),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1199),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1199),
    [aux_sym_end_if_statement_token1] = ACTIONS(1199),
    [aux_sym_elseif_clause_token2] = ACTIONS(1199),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1199),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token1] = ACTIONS(1199),
    [aux_sym_select_case_statement_token3] = ACTIONS(1199),
    [aux_sym_select_type_statement_token1] = ACTIONS(1199),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1199),
    [aux_sym_block_construct_token1] = ACTIONS(1199),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1199),
    [aux_sym_associate_statement_token1] = ACTIONS(1199),
    [aux_sym_format_statement_token1] = ACTIONS(1199),
    [aux_sym_print_statement_token1] = ACTIONS(1199),
    [aux_sym_open_statement_token1] = ACTIONS(1199),
    [aux_sym_close_statement_token1] = ACTIONS(1199),
    [aux_sym_inquire_statement_token1] = ACTIONS(1199),
    [aux_sym_enum_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token1] = ACTIONS(1199),
    [aux_sym_file_position_statement_token2] = ACTIONS(1199),
    [aux_sym_file_position_statement_token3] = ACTIONS(1199),
    [aux_sym_file_position_statement_token4] = ACTIONS(1199),
    [aux_sym_logical_expression_token5] = ACTIONS(1197),
    [anon_sym_PLUS] = ACTIONS(1197),
    [anon_sym_DOT] = ACTIONS(1199),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1197),
    [anon_sym_LBRACK] = ACTIONS(1197),
    [sym_boolean_literal] = ACTIONS(1197),
    [aux_sym_null_literal_token1] = ACTIONS(1199),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1199),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1197),
    [sym__float_literal] = ACTIONS(1197),
    [sym__boz_literal] = ACTIONS(1197),
    [sym_string_literal] = ACTIONS(1197),
  },
  [409] = {
    [sym_preproc_file_line] = STATE(409),
    [aux_sym_end_program_statement_token1] = ACTIONS(1319),
    [anon_sym_LPAREN] = ACTIONS(1319),
    [aux_sym_interface_statement_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1319),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1319),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1319),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1319),
    [aux_sym_use_statement_token1] = ACTIONS(1319),
    [aux_sym_use_statement_token2] = ACTIONS(1319),
    [aux_sym_implicit_statement_token1] = ACTIONS(1319),
    [aux_sym_implicit_statement_token3] = ACTIONS(1319),
    [aux_sym_implicit_statement_token4] = ACTIONS(1319),
    [aux_sym_save_statement_token1] = ACTIONS(1319),
    [aux_sym_private_statement_token1] = ACTIONS(1319),
    [aux_sym_public_statement_token1] = ACTIONS(1319),
    [aux_sym_namelist_statement_token1] = ACTIONS(1319),
    [aux_sym_common_statement_token1] = ACTIONS(1319),
    [anon_sym_DASH] = ACTIONS(1321),
    [aux_sym_import_statement_token1] = ACTIONS(1319),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1319),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1319),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1319),
    [aux_sym_variable_attributes_token1] = ACTIONS(1319),
    [aux_sym_variable_attributes_token2] = ACTIONS(1319),
    [aux_sym_variable_attributes_token3] = ACTIONS(1319),
    [aux_sym_variable_attributes_token4] = ACTIONS(1319),
    [aux_sym_variable_attributes_token5] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1319),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1319),
    [aux_sym_derived_type_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token1] = ACTIONS(1319),
    [aux_sym_type_qualifier_token2] = ACTIONS(1319),
    [aux_sym_type_qualifier_token3] = ACTIONS(1319),
    [aux_sym_type_qualifier_token4] = ACTIONS(1319),
    [aux_sym_type_qualifier_token5] = ACTIONS(1319),
    [aux_sym_type_qualifier_token9] = ACTIONS(1319),
    [aux_sym_type_qualifier_token10] = ACTIONS(1319),
    [aux_sym_type_qualifier_token11] = ACTIONS(1319),
    [aux_sym_type_qualifier_token12] = ACTIONS(1319),
    [aux_sym_type_qualifier_token13] = ACTIONS(1319),
    [aux_sym_type_qualifier_token14] = ACTIONS(1319),
    [aux_sym_type_qualifier_token15] = ACTIONS(1319),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token1] = ACTIONS(1319),
    [aux_sym_stop_statement_token2] = ACTIONS(1319),
    [aux_sym_subroutine_call_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token1] = ACTIONS(1319),
    [aux_sym_keyword_statement_token2] = ACTIONS(1319),
    [aux_sym_keyword_statement_token3] = ACTIONS(1319),
    [aux_sym_keyword_statement_token4] = ACTIONS(1319),
    [aux_sym_keyword_statement_token6] = ACTIONS(1319),
    [aux_sym_keyword_statement_token7] = ACTIONS(1319),
    [aux_sym_include_statement_token1] = ACTIONS(1319),
    [aux_sym_data_statement_token1] = ACTIONS(1319),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1319),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1319),
    [aux_sym_end_if_statement_token1] = ACTIONS(1319),
    [aux_sym_elseif_clause_token2] = ACTIONS(1319),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1319),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token1] = ACTIONS(1319),
    [aux_sym_select_case_statement_token3] = ACTIONS(1319),
    [aux_sym_select_type_statement_token1] = ACTIONS(1319),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1319),
    [aux_sym_block_construct_token1] = ACTIONS(1319),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1319),
    [aux_sym_associate_statement_token1] = ACTIONS(1319),
    [aux_sym_format_statement_token1] = ACTIONS(1319),
    [aux_sym_print_statement_token1] = ACTIONS(1319),
    [aux_sym_open_statement_token1] = ACTIONS(1319),
    [aux_sym_close_statement_token1] = ACTIONS(1319),
    [aux_sym_inquire_statement_token1] = ACTIONS(1319),
    [aux_sym_enum_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token1] = ACTIONS(1319),
    [aux_sym_file_position_statement_token2] = ACTIONS(1319),
    [aux_sym_file_position_statement_token3] = ACTIONS(1319),
    [aux_sym_file_position_statement_token4] = ACTIONS(1319),
    [aux_sym_logical_expression_token5] = ACTIONS(1321),
    [anon_sym_PLUS] = ACTIONS(1321),
    [anon_sym_DOT] = ACTIONS(1319),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1321),
    [anon_sym_LBRACK] = ACTIONS(1321),
    [sym_boolean_literal] = ACTIONS(1321),
    [aux_sym_null_literal_token1] = ACTIONS(1319),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1319),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1321),
    [sym__float_literal] = ACTIONS(1321),
    [sym__boz_literal] = ACTIONS(1321),
    [sym_string_literal] = ACTIONS(1321),
  },
  [410] = {
    [sym_preproc_file_line] = STATE(410),
    [aux_sym_end_program_statement_token1] = ACTIONS(1119),
    [anon_sym_LPAREN] = ACTIONS(1119),
    [aux_sym_interface_statement_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1119),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1119),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1119),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1119),
    [aux_sym_use_statement_token1] = ACTIONS(1119),
    [aux_sym_use_statement_token2] = ACTIONS(1119),
    [aux_sym_implicit_statement_token1] = ACTIONS(1119),
    [aux_sym_implicit_statement_token3] = ACTIONS(1119),
    [aux_sym_implicit_statement_token4] = ACTIONS(1119),
    [aux_sym_save_statement_token1] = ACTIONS(1119),
    [aux_sym_private_statement_token1] = ACTIONS(1119),
    [aux_sym_public_statement_token1] = ACTIONS(1119),
    [aux_sym_namelist_statement_token1] = ACTIONS(1119),
    [aux_sym_common_statement_token1] = ACTIONS(1119),
    [anon_sym_DASH] = ACTIONS(1117),
    [aux_sym_import_statement_token1] = ACTIONS(1119),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1119),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1119),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1119),
    [aux_sym_variable_attributes_token1] = ACTIONS(1119),
    [aux_sym_variable_attributes_token2] = ACTIONS(1119),
    [aux_sym_variable_attributes_token3] = ACTIONS(1119),
    [aux_sym_variable_attributes_token4] = ACTIONS(1119),
    [aux_sym_variable_attributes_token5] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1119),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1119),
    [aux_sym_derived_type_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token1] = ACTIONS(1119),
    [aux_sym_type_qualifier_token2] = ACTIONS(1119),
    [aux_sym_type_qualifier_token3] = ACTIONS(1119),
    [aux_sym_type_qualifier_token4] = ACTIONS(1119),
    [aux_sym_type_qualifier_token5] = ACTIONS(1119),
    [aux_sym_type_qualifier_token9] = ACTIONS(1119),
    [aux_sym_type_qualifier_token10] = ACTIONS(1119),
    [aux_sym_type_qualifier_token11] = ACTIONS(1119),
    [aux_sym_type_qualifier_token12] = ACTIONS(1119),
    [aux_sym_type_qualifier_token13] = ACTIONS(1119),
    [aux_sym_type_qualifier_token14] = ACTIONS(1119),
    [aux_sym_type_qualifier_token15] = ACTIONS(1119),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token1] = ACTIONS(1119),
    [aux_sym_stop_statement_token2] = ACTIONS(1119),
    [aux_sym_subroutine_call_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token1] = ACTIONS(1119),
    [aux_sym_keyword_statement_token2] = ACTIONS(1119),
    [aux_sym_keyword_statement_token3] = ACTIONS(1119),
    [aux_sym_keyword_statement_token4] = ACTIONS(1119),
    [aux_sym_keyword_statement_token6] = ACTIONS(1119),
    [aux_sym_keyword_statement_token7] = ACTIONS(1119),
    [aux_sym_include_statement_token1] = ACTIONS(1119),
    [aux_sym_data_statement_token1] = ACTIONS(1119),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1119),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1119),
    [aux_sym_end_if_statement_token1] = ACTIONS(1119),
    [aux_sym_elseif_clause_token2] = ACTIONS(1119),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1119),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token1] = ACTIONS(1119),
    [aux_sym_select_case_statement_token3] = ACTIONS(1119),
    [aux_sym_select_type_statement_token1] = ACTIONS(1119),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1119),
    [aux_sym_block_construct_token1] = ACTIONS(1119),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1119),
    [aux_sym_associate_statement_token1] = ACTIONS(1119),
    [aux_sym_format_statement_token1] = ACTIONS(1119),
    [aux_sym_print_statement_token1] = ACTIONS(1119),
    [aux_sym_open_statement_token1] = ACTIONS(1119),
    [aux_sym_close_statement_token1] = ACTIONS(1119),
    [aux_sym_inquire_statement_token1] = ACTIONS(1119),
    [aux_sym_enum_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token1] = ACTIONS(1119),
    [aux_sym_file_position_statement_token2] = ACTIONS(1119),
    [aux_sym_file_position_statement_token3] = ACTIONS(1119),
    [aux_sym_file_position_statement_token4] = ACTIONS(1119),
    [aux_sym_logical_expression_token5] = ACTIONS(1117),
    [anon_sym_PLUS] = ACTIONS(1117),
    [anon_sym_DOT] = ACTIONS(1119),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1117),
    [anon_sym_LBRACK] = ACTIONS(1117),
    [sym_boolean_literal] = ACTIONS(1117),
    [aux_sym_null_literal_token1] = ACTIONS(1119),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1119),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1117),
    [sym__float_literal] = ACTIONS(1117),
    [sym__boz_literal] = ACTIONS(1117),
    [sym_string_literal] = ACTIONS(1117),
  },
  [411] = {
    [sym_preproc_file_line] = STATE(411),
    [aux_sym_end_program_statement_token1] = ACTIONS(1331),
    [anon_sym_LPAREN] = ACTIONS(1331),
    [aux_sym_interface_statement_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1331),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1331),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1331),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1331),
    [aux_sym_use_statement_token1] = ACTIONS(1331),
    [aux_sym_use_statement_token2] = ACTIONS(1331),
    [aux_sym_implicit_statement_token1] = ACTIONS(1331),
    [aux_sym_implicit_statement_token3] = ACTIONS(1331),
    [aux_sym_implicit_statement_token4] = ACTIONS(1331),
    [aux_sym_save_statement_token1] = ACTIONS(1331),
    [aux_sym_private_statement_token1] = ACTIONS(1331),
    [aux_sym_public_statement_token1] = ACTIONS(1331),
    [aux_sym_namelist_statement_token1] = ACTIONS(1331),
    [aux_sym_common_statement_token1] = ACTIONS(1331),
    [anon_sym_DASH] = ACTIONS(1333),
    [aux_sym_import_statement_token1] = ACTIONS(1331),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1331),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1331),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1331),
    [aux_sym_variable_attributes_token1] = ACTIONS(1331),
    [aux_sym_variable_attributes_token2] = ACTIONS(1331),
    [aux_sym_variable_attributes_token3] = ACTIONS(1331),
    [aux_sym_variable_attributes_token4] = ACTIONS(1331),
    [aux_sym_variable_attributes_token5] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1331),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1331),
    [aux_sym_derived_type_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token1] = ACTIONS(1331),
    [aux_sym_type_qualifier_token2] = ACTIONS(1331),
    [aux_sym_type_qualifier_token3] = ACTIONS(1331),
    [aux_sym_type_qualifier_token4] = ACTIONS(1331),
    [aux_sym_type_qualifier_token5] = ACTIONS(1331),
    [aux_sym_type_qualifier_token9] = ACTIONS(1331),
    [aux_sym_type_qualifier_token10] = ACTIONS(1331),
    [aux_sym_type_qualifier_token11] = ACTIONS(1331),
    [aux_sym_type_qualifier_token12] = ACTIONS(1331),
    [aux_sym_type_qualifier_token13] = ACTIONS(1331),
    [aux_sym_type_qualifier_token14] = ACTIONS(1331),
    [aux_sym_type_qualifier_token15] = ACTIONS(1331),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token1] = ACTIONS(1331),
    [aux_sym_stop_statement_token2] = ACTIONS(1331),
    [aux_sym_subroutine_call_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token1] = ACTIONS(1331),
    [aux_sym_keyword_statement_token2] = ACTIONS(1331),
    [aux_sym_keyword_statement_token3] = ACTIONS(1331),
    [aux_sym_keyword_statement_token4] = ACTIONS(1331),
    [aux_sym_keyword_statement_token6] = ACTIONS(1331),
    [aux_sym_keyword_statement_token7] = ACTIONS(1331),
    [aux_sym_include_statement_token1] = ACTIONS(1331),
    [aux_sym_data_statement_token1] = ACTIONS(1331),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1331),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1331),
    [aux_sym_end_if_statement_token1] = ACTIONS(1331),
    [aux_sym_elseif_clause_token2] = ACTIONS(1331),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1331),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token1] = ACTIONS(1331),
    [aux_sym_select_case_statement_token3] = ACTIONS(1331),
    [aux_sym_select_type_statement_token1] = ACTIONS(1331),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1331),
    [aux_sym_block_construct_token1] = ACTIONS(1331),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1331),
    [aux_sym_associate_statement_token1] = ACTIONS(1331),
    [aux_sym_format_statement_token1] = ACTIONS(1331),
    [aux_sym_print_statement_token1] = ACTIONS(1331),
    [aux_sym_open_statement_token1] = ACTIONS(1331),
    [aux_sym_close_statement_token1] = ACTIONS(1331),
    [aux_sym_inquire_statement_token1] = ACTIONS(1331),
    [aux_sym_enum_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token1] = ACTIONS(1331),
    [aux_sym_file_position_statement_token2] = ACTIONS(1331),
    [aux_sym_file_position_statement_token3] = ACTIONS(1331),
    [aux_sym_file_position_statement_token4] = ACTIONS(1331),
    [aux_sym_logical_expression_token5] = ACTIONS(1333),
    [anon_sym_PLUS] = ACTIONS(1333),
    [anon_sym_DOT] = ACTIONS(1331),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1333),
    [anon_sym_LBRACK] = ACTIONS(1333),
    [sym_boolean_literal] = ACTIONS(1333),
    [aux_sym_null_literal_token1] = ACTIONS(1331),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1331),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1333),
    [sym__float_literal] = ACTIONS(1333),
    [sym__boz_literal] = ACTIONS(1333),
    [sym_string_literal] = ACTIONS(1333),
  },
  [412] = {
    [sym_preproc_file_line] = STATE(412),
    [aux_sym_end_program_statement_token1] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(1359),
    [aux_sym_interface_statement_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1359),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1359),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1359),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1359),
    [aux_sym_use_statement_token1] = ACTIONS(1359),
    [aux_sym_use_statement_token2] = ACTIONS(1359),
    [aux_sym_implicit_statement_token1] = ACTIONS(1359),
    [aux_sym_implicit_statement_token3] = ACTIONS(1359),
    [aux_sym_implicit_statement_token4] = ACTIONS(1359),
    [aux_sym_save_statement_token1] = ACTIONS(1359),
    [aux_sym_private_statement_token1] = ACTIONS(1359),
    [aux_sym_public_statement_token1] = ACTIONS(1359),
    [aux_sym_namelist_statement_token1] = ACTIONS(1359),
    [aux_sym_common_statement_token1] = ACTIONS(1359),
    [anon_sym_DASH] = ACTIONS(1361),
    [aux_sym_import_statement_token1] = ACTIONS(1359),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1359),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1359),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1359),
    [aux_sym_variable_attributes_token1] = ACTIONS(1359),
    [aux_sym_variable_attributes_token2] = ACTIONS(1359),
    [aux_sym_variable_attributes_token3] = ACTIONS(1359),
    [aux_sym_variable_attributes_token4] = ACTIONS(1359),
    [aux_sym_variable_attributes_token5] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1359),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1359),
    [aux_sym_derived_type_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token1] = ACTIONS(1359),
    [aux_sym_type_qualifier_token2] = ACTIONS(1359),
    [aux_sym_type_qualifier_token3] = ACTIONS(1359),
    [aux_sym_type_qualifier_token4] = ACTIONS(1359),
    [aux_sym_type_qualifier_token5] = ACTIONS(1359),
    [aux_sym_type_qualifier_token9] = ACTIONS(1359),
    [aux_sym_type_qualifier_token10] = ACTIONS(1359),
    [aux_sym_type_qualifier_token11] = ACTIONS(1359),
    [aux_sym_type_qualifier_token12] = ACTIONS(1359),
    [aux_sym_type_qualifier_token13] = ACTIONS(1359),
    [aux_sym_type_qualifier_token14] = ACTIONS(1359),
    [aux_sym_type_qualifier_token15] = ACTIONS(1359),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token1] = ACTIONS(1359),
    [aux_sym_stop_statement_token2] = ACTIONS(1359),
    [aux_sym_subroutine_call_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token1] = ACTIONS(1359),
    [aux_sym_keyword_statement_token2] = ACTIONS(1359),
    [aux_sym_keyword_statement_token3] = ACTIONS(1359),
    [aux_sym_keyword_statement_token4] = ACTIONS(1359),
    [aux_sym_keyword_statement_token6] = ACTIONS(1359),
    [aux_sym_keyword_statement_token7] = ACTIONS(1359),
    [aux_sym_include_statement_token1] = ACTIONS(1359),
    [aux_sym_data_statement_token1] = ACTIONS(1359),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1359),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1359),
    [aux_sym_end_if_statement_token1] = ACTIONS(1359),
    [aux_sym_elseif_clause_token2] = ACTIONS(1359),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1359),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token1] = ACTIONS(1359),
    [aux_sym_select_case_statement_token3] = ACTIONS(1359),
    [aux_sym_select_type_statement_token1] = ACTIONS(1359),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1359),
    [aux_sym_block_construct_token1] = ACTIONS(1359),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1359),
    [aux_sym_associate_statement_token1] = ACTIONS(1359),
    [aux_sym_format_statement_token1] = ACTIONS(1359),
    [aux_sym_print_statement_token1] = ACTIONS(1359),
    [aux_sym_open_statement_token1] = ACTIONS(1359),
    [aux_sym_close_statement_token1] = ACTIONS(1359),
    [aux_sym_inquire_statement_token1] = ACTIONS(1359),
    [aux_sym_enum_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token1] = ACTIONS(1359),
    [aux_sym_file_position_statement_token2] = ACTIONS(1359),
    [aux_sym_file_position_statement_token3] = ACTIONS(1359),
    [aux_sym_file_position_statement_token4] = ACTIONS(1359),
    [aux_sym_logical_expression_token5] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(1359),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1361),
    [anon_sym_LBRACK] = ACTIONS(1361),
    [sym_boolean_literal] = ACTIONS(1361),
    [aux_sym_null_literal_token1] = ACTIONS(1359),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1359),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1361),
    [sym__float_literal] = ACTIONS(1361),
    [sym__boz_literal] = ACTIONS(1361),
    [sym_string_literal] = ACTIONS(1361),
  },
  [413] = {
    [sym_preproc_file_line] = STATE(413),
    [aux_sym_end_program_statement_token1] = ACTIONS(1355),
    [anon_sym_LPAREN] = ACTIONS(1355),
    [aux_sym_interface_statement_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1355),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1355),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1355),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1355),
    [aux_sym_use_statement_token1] = ACTIONS(1355),
    [aux_sym_use_statement_token2] = ACTIONS(1355),
    [aux_sym_implicit_statement_token1] = ACTIONS(1355),
    [aux_sym_implicit_statement_token3] = ACTIONS(1355),
    [aux_sym_implicit_statement_token4] = ACTIONS(1355),
    [aux_sym_save_statement_token1] = ACTIONS(1355),
    [aux_sym_private_statement_token1] = ACTIONS(1355),
    [aux_sym_public_statement_token1] = ACTIONS(1355),
    [aux_sym_namelist_statement_token1] = ACTIONS(1355),
    [aux_sym_common_statement_token1] = ACTIONS(1355),
    [anon_sym_DASH] = ACTIONS(1357),
    [aux_sym_import_statement_token1] = ACTIONS(1355),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1355),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1355),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1355),
    [aux_sym_variable_attributes_token1] = ACTIONS(1355),
    [aux_sym_variable_attributes_token2] = ACTIONS(1355),
    [aux_sym_variable_attributes_token3] = ACTIONS(1355),
    [aux_sym_variable_attributes_token4] = ACTIONS(1355),
    [aux_sym_variable_attributes_token5] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1355),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1355),
    [aux_sym_derived_type_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token1] = ACTIONS(1355),
    [aux_sym_type_qualifier_token2] = ACTIONS(1355),
    [aux_sym_type_qualifier_token3] = ACTIONS(1355),
    [aux_sym_type_qualifier_token4] = ACTIONS(1355),
    [aux_sym_type_qualifier_token5] = ACTIONS(1355),
    [aux_sym_type_qualifier_token9] = ACTIONS(1355),
    [aux_sym_type_qualifier_token10] = ACTIONS(1355),
    [aux_sym_type_qualifier_token11] = ACTIONS(1355),
    [aux_sym_type_qualifier_token12] = ACTIONS(1355),
    [aux_sym_type_qualifier_token13] = ACTIONS(1355),
    [aux_sym_type_qualifier_token14] = ACTIONS(1355),
    [aux_sym_type_qualifier_token15] = ACTIONS(1355),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token1] = ACTIONS(1355),
    [aux_sym_stop_statement_token2] = ACTIONS(1355),
    [aux_sym_subroutine_call_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token1] = ACTIONS(1355),
    [aux_sym_keyword_statement_token2] = ACTIONS(1355),
    [aux_sym_keyword_statement_token3] = ACTIONS(1355),
    [aux_sym_keyword_statement_token4] = ACTIONS(1355),
    [aux_sym_keyword_statement_token6] = ACTIONS(1355),
    [aux_sym_keyword_statement_token7] = ACTIONS(1355),
    [aux_sym_include_statement_token1] = ACTIONS(1355),
    [aux_sym_data_statement_token1] = ACTIONS(1355),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1355),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1355),
    [aux_sym_end_if_statement_token1] = ACTIONS(1355),
    [aux_sym_elseif_clause_token2] = ACTIONS(1355),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1355),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token1] = ACTIONS(1355),
    [aux_sym_select_case_statement_token3] = ACTIONS(1355),
    [aux_sym_select_type_statement_token1] = ACTIONS(1355),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1355),
    [aux_sym_block_construct_token1] = ACTIONS(1355),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1355),
    [aux_sym_associate_statement_token1] = ACTIONS(1355),
    [aux_sym_format_statement_token1] = ACTIONS(1355),
    [aux_sym_print_statement_token1] = ACTIONS(1355),
    [aux_sym_open_statement_token1] = ACTIONS(1355),
    [aux_sym_close_statement_token1] = ACTIONS(1355),
    [aux_sym_inquire_statement_token1] = ACTIONS(1355),
    [aux_sym_enum_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token1] = ACTIONS(1355),
    [aux_sym_file_position_statement_token2] = ACTIONS(1355),
    [aux_sym_file_position_statement_token3] = ACTIONS(1355),
    [aux_sym_file_position_statement_token4] = ACTIONS(1355),
    [aux_sym_logical_expression_token5] = ACTIONS(1357),
    [anon_sym_PLUS] = ACTIONS(1357),
    [anon_sym_DOT] = ACTIONS(1355),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1357),
    [anon_sym_LBRACK] = ACTIONS(1357),
    [sym_boolean_literal] = ACTIONS(1357),
    [aux_sym_null_literal_token1] = ACTIONS(1355),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1355),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1357),
    [sym__float_literal] = ACTIONS(1357),
    [sym__boz_literal] = ACTIONS(1357),
    [sym_string_literal] = ACTIONS(1357),
  },
  [414] = {
    [sym_preproc_file_line] = STATE(414),
    [aux_sym_end_program_statement_token1] = ACTIONS(1315),
    [anon_sym_LPAREN] = ACTIONS(1315),
    [aux_sym_interface_statement_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1315),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1315),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1315),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1315),
    [aux_sym_use_statement_token1] = ACTIONS(1315),
    [aux_sym_use_statement_token2] = ACTIONS(1315),
    [aux_sym_implicit_statement_token1] = ACTIONS(1315),
    [aux_sym_implicit_statement_token3] = ACTIONS(1315),
    [aux_sym_implicit_statement_token4] = ACTIONS(1315),
    [aux_sym_save_statement_token1] = ACTIONS(1315),
    [aux_sym_private_statement_token1] = ACTIONS(1315),
    [aux_sym_public_statement_token1] = ACTIONS(1315),
    [aux_sym_namelist_statement_token1] = ACTIONS(1315),
    [aux_sym_common_statement_token1] = ACTIONS(1315),
    [anon_sym_DASH] = ACTIONS(1317),
    [aux_sym_import_statement_token1] = ACTIONS(1315),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1315),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1315),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1315),
    [aux_sym_variable_attributes_token1] = ACTIONS(1315),
    [aux_sym_variable_attributes_token2] = ACTIONS(1315),
    [aux_sym_variable_attributes_token3] = ACTIONS(1315),
    [aux_sym_variable_attributes_token4] = ACTIONS(1315),
    [aux_sym_variable_attributes_token5] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1315),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1315),
    [aux_sym_derived_type_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token1] = ACTIONS(1315),
    [aux_sym_type_qualifier_token2] = ACTIONS(1315),
    [aux_sym_type_qualifier_token3] = ACTIONS(1315),
    [aux_sym_type_qualifier_token4] = ACTIONS(1315),
    [aux_sym_type_qualifier_token5] = ACTIONS(1315),
    [aux_sym_type_qualifier_token9] = ACTIONS(1315),
    [aux_sym_type_qualifier_token10] = ACTIONS(1315),
    [aux_sym_type_qualifier_token11] = ACTIONS(1315),
    [aux_sym_type_qualifier_token12] = ACTIONS(1315),
    [aux_sym_type_qualifier_token13] = ACTIONS(1315),
    [aux_sym_type_qualifier_token14] = ACTIONS(1315),
    [aux_sym_type_qualifier_token15] = ACTIONS(1315),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token1] = ACTIONS(1315),
    [aux_sym_stop_statement_token2] = ACTIONS(1315),
    [aux_sym_subroutine_call_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token1] = ACTIONS(1315),
    [aux_sym_keyword_statement_token2] = ACTIONS(1315),
    [aux_sym_keyword_statement_token3] = ACTIONS(1315),
    [aux_sym_keyword_statement_token4] = ACTIONS(1315),
    [aux_sym_keyword_statement_token6] = ACTIONS(1315),
    [aux_sym_keyword_statement_token7] = ACTIONS(1315),
    [aux_sym_include_statement_token1] = ACTIONS(1315),
    [aux_sym_data_statement_token1] = ACTIONS(1315),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1315),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1315),
    [aux_sym_end_if_statement_token1] = ACTIONS(1315),
    [aux_sym_elseif_clause_token2] = ACTIONS(1315),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1315),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token1] = ACTIONS(1315),
    [aux_sym_select_case_statement_token3] = ACTIONS(1315),
    [aux_sym_select_type_statement_token1] = ACTIONS(1315),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1315),
    [aux_sym_block_construct_token1] = ACTIONS(1315),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1315),
    [aux_sym_associate_statement_token1] = ACTIONS(1315),
    [aux_sym_format_statement_token1] = ACTIONS(1315),
    [aux_sym_print_statement_token1] = ACTIONS(1315),
    [aux_sym_open_statement_token1] = ACTIONS(1315),
    [aux_sym_close_statement_token1] = ACTIONS(1315),
    [aux_sym_inquire_statement_token1] = ACTIONS(1315),
    [aux_sym_enum_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token1] = ACTIONS(1315),
    [aux_sym_file_position_statement_token2] = ACTIONS(1315),
    [aux_sym_file_position_statement_token3] = ACTIONS(1315),
    [aux_sym_file_position_statement_token4] = ACTIONS(1315),
    [aux_sym_logical_expression_token5] = ACTIONS(1317),
    [anon_sym_PLUS] = ACTIONS(1317),
    [anon_sym_DOT] = ACTIONS(1315),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1317),
    [anon_sym_LBRACK] = ACTIONS(1317),
    [sym_boolean_literal] = ACTIONS(1317),
    [aux_sym_null_literal_token1] = ACTIONS(1315),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1315),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1317),
    [sym__float_literal] = ACTIONS(1317),
    [sym__boz_literal] = ACTIONS(1317),
    [sym_string_literal] = ACTIONS(1317),
  },
  [415] = {
    [sym_preproc_file_line] = STATE(415),
    [aux_sym_end_program_statement_token1] = ACTIONS(1307),
    [anon_sym_LPAREN] = ACTIONS(1307),
    [aux_sym_interface_statement_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1307),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1307),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1307),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1307),
    [aux_sym_use_statement_token1] = ACTIONS(1307),
    [aux_sym_use_statement_token2] = ACTIONS(1307),
    [aux_sym_implicit_statement_token1] = ACTIONS(1307),
    [aux_sym_implicit_statement_token3] = ACTIONS(1307),
    [aux_sym_implicit_statement_token4] = ACTIONS(1307),
    [aux_sym_save_statement_token1] = ACTIONS(1307),
    [aux_sym_private_statement_token1] = ACTIONS(1307),
    [aux_sym_public_statement_token1] = ACTIONS(1307),
    [aux_sym_namelist_statement_token1] = ACTIONS(1307),
    [aux_sym_common_statement_token1] = ACTIONS(1307),
    [anon_sym_DASH] = ACTIONS(1309),
    [aux_sym_import_statement_token1] = ACTIONS(1307),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1307),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1307),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1307),
    [aux_sym_variable_attributes_token1] = ACTIONS(1307),
    [aux_sym_variable_attributes_token2] = ACTIONS(1307),
    [aux_sym_variable_attributes_token3] = ACTIONS(1307),
    [aux_sym_variable_attributes_token4] = ACTIONS(1307),
    [aux_sym_variable_attributes_token5] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1307),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1307),
    [aux_sym_derived_type_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token1] = ACTIONS(1307),
    [aux_sym_type_qualifier_token2] = ACTIONS(1307),
    [aux_sym_type_qualifier_token3] = ACTIONS(1307),
    [aux_sym_type_qualifier_token4] = ACTIONS(1307),
    [aux_sym_type_qualifier_token5] = ACTIONS(1307),
    [aux_sym_type_qualifier_token9] = ACTIONS(1307),
    [aux_sym_type_qualifier_token10] = ACTIONS(1307),
    [aux_sym_type_qualifier_token11] = ACTIONS(1307),
    [aux_sym_type_qualifier_token12] = ACTIONS(1307),
    [aux_sym_type_qualifier_token13] = ACTIONS(1307),
    [aux_sym_type_qualifier_token14] = ACTIONS(1307),
    [aux_sym_type_qualifier_token15] = ACTIONS(1307),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token1] = ACTIONS(1307),
    [aux_sym_stop_statement_token2] = ACTIONS(1307),
    [aux_sym_subroutine_call_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token1] = ACTIONS(1307),
    [aux_sym_keyword_statement_token2] = ACTIONS(1307),
    [aux_sym_keyword_statement_token3] = ACTIONS(1307),
    [aux_sym_keyword_statement_token4] = ACTIONS(1307),
    [aux_sym_keyword_statement_token6] = ACTIONS(1307),
    [aux_sym_keyword_statement_token7] = ACTIONS(1307),
    [aux_sym_include_statement_token1] = ACTIONS(1307),
    [aux_sym_data_statement_token1] = ACTIONS(1307),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1307),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1307),
    [aux_sym_end_if_statement_token1] = ACTIONS(1307),
    [aux_sym_elseif_clause_token2] = ACTIONS(1307),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1307),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token1] = ACTIONS(1307),
    [aux_sym_select_case_statement_token3] = ACTIONS(1307),
    [aux_sym_select_type_statement_token1] = ACTIONS(1307),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1307),
    [aux_sym_block_construct_token1] = ACTIONS(1307),
    [aux_sym_end_block_construct_statement_token1] = ACTIONS(1307),
    [aux_sym_associate_statement_token1] = ACTIONS(1307),
    [aux_sym_format_statement_token1] = ACTIONS(1307),
    [aux_sym_print_statement_token1] = ACTIONS(1307),
    [aux_sym_open_statement_token1] = ACTIONS(1307),
    [aux_sym_close_statement_token1] = ACTIONS(1307),
    [aux_sym_inquire_statement_token1] = ACTIONS(1307),
    [aux_sym_enum_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token1] = ACTIONS(1307),
    [aux_sym_file_position_statement_token2] = ACTIONS(1307),
    [aux_sym_file_position_statement_token3] = ACTIONS(1307),
    [aux_sym_file_position_statement_token4] = ACTIONS(1307),
    [aux_sym_logical_expression_token5] = ACTIONS(1309),
    [anon_sym_PLUS] = ACTIONS(1309),
    [anon_sym_DOT] = ACTIONS(1307),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1309),
    [anon_sym_LBRACK] = ACTIONS(1309),
    [sym_boolean_literal] = ACTIONS(1309),
    [aux_sym_null_literal_token1] = ACTIONS(1307),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1307),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1309),
    [sym__float_literal] = ACTIONS(1309),
    [sym__boz_literal] = ACTIONS(1309),
    [sym_string_literal] = ACTIONS(1309),
  },
  [416] = {
    [sym_end_submodule_statement] = STATE(282),
    [sym_interface] = STATE(631),
    [sym_interface_statement] = STATE(758),
    [sym_internal_procedures] = STATE(1996),
    [sym_contains_statement] = STATE(2387),
    [sym__specification_part] = STATE(645),
    [sym_use_statement] = STATE(2388),
    [sym_implicit_statement] = STATE(2388),
    [sym_save_statement] = STATE(2388),
    [sym_private_statement] = STATE(2388),
    [sym_public_statement] = STATE(2388),
    [sym_namelist_statement] = STATE(2388),
    [sym_common_statement] = STATE(2388),
    [sym_import_statement] = STATE(2388),
    [sym_derived_type_definition] = STATE(631),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(994),
    [sym_variable_declaration] = STATE(2388),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2388),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2388),
    [sym_equivalence_statement] = STATE(2388),
    [sym_statement_label] = STATE(1994),
    [sym_include_statement] = STATE(2388),
    [sym_data_statement] = STATE(2388),
    [sym_enum] = STATE(631),
    [sym_enum_statement] = STATE(1856),
    [sym_preproc_file_line] = STATE(416),
    [aux_sym_program_repeat1] = STATE(421),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(1367),
    [aux_sym_end_submodule_statement_token1] = ACTIONS(1369),
    [aux_sym_interface_statement_token1] = ACTIONS(1371),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1373),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1375),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1377),
    [aux_sym_contains_statement_token1] = ACTIONS(1379),
    [aux_sym_use_statement_token1] = ACTIONS(1381),
    [aux_sym_use_statement_token2] = ACTIONS(1377),
    [aux_sym_implicit_statement_token1] = ACTIONS(1383),
    [aux_sym_implicit_statement_token3] = ACTIONS(1385),
    [aux_sym_implicit_statement_token4] = ACTIONS(1377),
    [aux_sym_save_statement_token1] = ACTIONS(1387),
    [aux_sym_private_statement_token1] = ACTIONS(1389),
    [aux_sym_public_statement_token1] = ACTIONS(1391),
    [aux_sym_namelist_statement_token1] = ACTIONS(1393),
    [aux_sym_common_statement_token1] = ACTIONS(1395),
    [aux_sym_import_statement_token1] = ACTIONS(1397),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1399),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1401),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1377),
    [aux_sym_variable_attributes_token1] = ACTIONS(1377),
    [aux_sym_variable_attributes_token2] = ACTIONS(1377),
    [aux_sym_variable_attributes_token3] = ACTIONS(1377),
    [aux_sym_variable_attributes_token4] = ACTIONS(1377),
    [aux_sym_variable_attributes_token5] = ACTIONS(1377),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1405),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1407),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1403),
    [aux_sym_derived_type_token1] = ACTIONS(1409),
    [aux_sym_type_qualifier_token1] = ACTIONS(1377),
    [aux_sym_type_qualifier_token2] = ACTIONS(1377),
    [aux_sym_type_qualifier_token3] = ACTIONS(1411),
    [aux_sym_type_qualifier_token4] = ACTIONS(1377),
    [aux_sym_type_qualifier_token5] = ACTIONS(1413),
    [aux_sym_type_qualifier_token9] = ACTIONS(1377),
    [aux_sym_type_qualifier_token10] = ACTIONS(1415),
    [aux_sym_type_qualifier_token11] = ACTIONS(1377),
    [aux_sym_type_qualifier_token12] = ACTIONS(1377),
    [aux_sym_type_qualifier_token13] = ACTIONS(1377),
    [aux_sym_type_qualifier_token14] = ACTIONS(1377),
    [aux_sym_type_qualifier_token15] = ACTIONS(1377),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1417),
    [aux_sym_include_statement_token1] = ACTIONS(1419),
    [aux_sym_data_statement_token1] = ACTIONS(1421),
    [aux_sym_enum_statement_token1] = ACTIONS(1423),
    [anon_sym_POUND] = ACTIONS(3),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1425),
  },
  [417] = {
    [sym_end_module_statement] = STATE(251),
    [sym_interface] = STATE(581),
    [sym_interface_statement] = STATE(763),
    [sym_internal_procedures] = STATE(2134),
    [sym_contains_statement] = STATE(2538),
    [sym__specification_part] = STATE(580),
    [sym_use_statement] = STATE(2537),
    [sym_implicit_statement] = STATE(2537),
    [sym_save_statement] = STATE(2537),
    [sym_private_statement] = STATE(2537),
    [sym_public_statement] = STATE(2537),
    [sym_namelist_statement] = STATE(2537),
    [sym_common_statement] = STATE(2537),
    [sym_import_statement] = STATE(2537),
    [sym_derived_type_definition] = STATE(581),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(954),
    [sym_variable_declaration] = STATE(2537),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2537),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2537),
    [sym_equivalence_statement] = STATE(2537),
    [sym_statement_label] = STATE(2131),
    [sym_include_statement] = STATE(2537),
    [sym_data_statement] = STATE(2537),
    [sym_enum] = STATE(581),
    [sym_enum_statement] = STATE(1858),
    [sym_preproc_file_line] = STATE(417),
    [aux_sym_program_repeat1] = STATE(419),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(1427),
    [aux_sym_end_module_statement_token1] = ACTIONS(1429),
    [aux_sym_interface_statement_token1] = ACTIONS(1371),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1373),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1375),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1377),
    [aux_sym_contains_statement_token1] = ACTIONS(1379),
    [aux_sym_use_statement_token1] = ACTIONS(1381),
    [aux_sym_use_statement_token2] = ACTIONS(1377),
    [aux_sym_implicit_statement_token1] = ACTIONS(1383),
    [aux_sym_implicit_statement_token3] = ACTIONS(1385),
    [aux_sym_implicit_statement_token4] = ACTIONS(1377),
    [aux_sym_save_statement_token1] = ACTIONS(1387),
    [aux_sym_private_statement_token1] = ACTIONS(1389),
    [aux_sym_public_statement_token1] = ACTIONS(1391),
    [aux_sym_namelist_statement_token1] = ACTIONS(1393),
    [aux_sym_common_statement_token1] = ACTIONS(1395),
    [aux_sym_import_statement_token1] = ACTIONS(1397),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1399),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1401),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1377),
    [aux_sym_variable_attributes_token1] = ACTIONS(1377),
    [aux_sym_variable_attributes_token2] = ACTIONS(1377),
    [aux_sym_variable_attributes_token3] = ACTIONS(1377),
    [aux_sym_variable_attributes_token4] = ACTIONS(1377),
    [aux_sym_variable_attributes_token5] = ACTIONS(1377),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1405),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1407),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1403),
    [aux_sym_derived_type_token1] = ACTIONS(1409),
    [aux_sym_type_qualifier_token1] = ACTIONS(1377),
    [aux_sym_type_qualifier_token2] = ACTIONS(1377),
    [aux_sym_type_qualifier_token3] = ACTIONS(1411),
    [aux_sym_type_qualifier_token4] = ACTIONS(1377),
    [aux_sym_type_qualifier_token5] = ACTIONS(1413),
    [aux_sym_type_qualifier_token9] = ACTIONS(1377),
    [aux_sym_type_qualifier_token10] = ACTIONS(1415),
    [aux_sym_type_qualifier_token11] = ACTIONS(1377),
    [aux_sym_type_qualifier_token12] = ACTIONS(1377),
    [aux_sym_type_qualifier_token13] = ACTIONS(1377),
    [aux_sym_type_qualifier_token14] = ACTIONS(1377),
    [aux_sym_type_qualifier_token15] = ACTIONS(1377),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1417),
    [aux_sym_include_statement_token1] = ACTIONS(1419),
    [aux_sym_data_statement_token1] = ACTIONS(1421),
    [aux_sym_enum_statement_token1] = ACTIONS(1423),
    [anon_sym_POUND] = ACTIONS(3),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1425),
  },
  [418] = {
    [sym_end_submodule_statement] = STATE(257),
    [sym_interface] = STATE(631),
    [sym_interface_statement] = STATE(758),
    [sym_internal_procedures] = STATE(1977),
    [sym_contains_statement] = STATE(2387),
    [sym__specification_part] = STATE(645),
    [sym_use_statement] = STATE(2388),
    [sym_implicit_statement] = STATE(2388),
    [sym_save_statement] = STATE(2388),
    [sym_private_statement] = STATE(2388),
    [sym_public_statement] = STATE(2388),
    [sym_namelist_statement] = STATE(2388),
    [sym_common_statement] = STATE(2388),
    [sym_import_statement] = STATE(2388),
    [sym_derived_type_definition] = STATE(631),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(994),
    [sym_variable_declaration] = STATE(2388),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2388),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2388),
    [sym_equivalence_statement] = STATE(2388),
    [sym_statement_label] = STATE(1994),
    [sym_include_statement] = STATE(2388),
    [sym_data_statement] = STATE(2388),
    [sym_enum] = STATE(631),
    [sym_enum_statement] = STATE(1856),
    [sym_preproc_file_line] = STATE(418),
    [aux_sym_program_repeat1] = STATE(416),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(1367),
    [aux_sym_end_submodule_statement_token1] = ACTIONS(1369),
    [aux_sym_interface_statement_token1] = ACTIONS(1371),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1373),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1375),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1377),
    [aux_sym_contains_statement_token1] = ACTIONS(1379),
    [aux_sym_use_statement_token1] = ACTIONS(1381),
    [aux_sym_use_statement_token2] = ACTIONS(1377),
    [aux_sym_implicit_statement_token1] = ACTIONS(1383),
    [aux_sym_implicit_statement_token3] = ACTIONS(1385),
    [aux_sym_implicit_statement_token4] = ACTIONS(1377),
    [aux_sym_save_statement_token1] = ACTIONS(1387),
    [aux_sym_private_statement_token1] = ACTIONS(1389),
    [aux_sym_public_statement_token1] = ACTIONS(1391),
    [aux_sym_namelist_statement_token1] = ACTIONS(1393),
    [aux_sym_common_statement_token1] = ACTIONS(1395),
    [aux_sym_import_statement_token1] = ACTIONS(1397),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1399),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1401),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1377),
    [aux_sym_variable_attributes_token1] = ACTIONS(1377),
    [aux_sym_variable_attributes_token2] = ACTIONS(1377),
    [aux_sym_variable_attributes_token3] = ACTIONS(1377),
    [aux_sym_variable_attributes_token4] = ACTIONS(1377),
    [aux_sym_variable_attributes_token5] = ACTIONS(1377),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1405),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1407),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1403),
    [aux_sym_derived_type_token1] = ACTIONS(1409),
    [aux_sym_type_qualifier_token1] = ACTIONS(1377),
    [aux_sym_type_qualifier_token2] = ACTIONS(1377),
    [aux_sym_type_qualifier_token3] = ACTIONS(1411),
    [aux_sym_type_qualifier_token4] = ACTIONS(1377),
    [aux_sym_type_qualifier_token5] = ACTIONS(1413),
    [aux_sym_type_qualifier_token9] = ACTIONS(1377),
    [aux_sym_type_qualifier_token10] = ACTIONS(1415),
    [aux_sym_type_qualifier_token11] = ACTIONS(1377),
    [aux_sym_type_qualifier_token12] = ACTIONS(1377),
    [aux_sym_type_qualifier_token13] = ACTIONS(1377),
    [aux_sym_type_qualifier_token14] = ACTIONS(1377),
    [aux_sym_type_qualifier_token15] = ACTIONS(1377),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1417),
    [aux_sym_include_statement_token1] = ACTIONS(1419),
    [aux_sym_data_statement_token1] = ACTIONS(1421),
    [aux_sym_enum_statement_token1] = ACTIONS(1423),
    [anon_sym_POUND] = ACTIONS(3),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1425),
  },
  [419] = {
    [sym_end_module_statement] = STATE(286),
    [sym_interface] = STATE(581),
    [sym_interface_statement] = STATE(763),
    [sym_internal_procedures] = STATE(1999),
    [sym_contains_statement] = STATE(2538),
    [sym__specification_part] = STATE(580),
    [sym_use_statement] = STATE(2537),
    [sym_implicit_statement] = STATE(2537),
    [sym_save_statement] = STATE(2537),
    [sym_private_statement] = STATE(2537),
    [sym_public_statement] = STATE(2537),
    [sym_namelist_statement] = STATE(2537),
    [sym_common_statement] = STATE(2537),
    [sym_import_statement] = STATE(2537),
    [sym_derived_type_definition] = STATE(581),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(954),
    [sym_variable_declaration] = STATE(2537),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2537),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2537),
    [sym_equivalence_statement] = STATE(2537),
    [sym_statement_label] = STATE(2131),
    [sym_include_statement] = STATE(2537),
    [sym_data_statement] = STATE(2537),
    [sym_enum] = STATE(581),
    [sym_enum_statement] = STATE(1858),
    [sym_preproc_file_line] = STATE(419),
    [aux_sym_program_repeat1] = STATE(422),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(1427),
    [aux_sym_end_module_statement_token1] = ACTIONS(1429),
    [aux_sym_interface_statement_token1] = ACTIONS(1371),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1373),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1375),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1377),
    [aux_sym_contains_statement_token1] = ACTIONS(1379),
    [aux_sym_use_statement_token1] = ACTIONS(1381),
    [aux_sym_use_statement_token2] = ACTIONS(1377),
    [aux_sym_implicit_statement_token1] = ACTIONS(1383),
    [aux_sym_implicit_statement_token3] = ACTIONS(1385),
    [aux_sym_implicit_statement_token4] = ACTIONS(1377),
    [aux_sym_save_statement_token1] = ACTIONS(1387),
    [aux_sym_private_statement_token1] = ACTIONS(1389),
    [aux_sym_public_statement_token1] = ACTIONS(1391),
    [aux_sym_namelist_statement_token1] = ACTIONS(1393),
    [aux_sym_common_statement_token1] = ACTIONS(1395),
    [aux_sym_import_statement_token1] = ACTIONS(1397),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1399),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1401),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1377),
    [aux_sym_variable_attributes_token1] = ACTIONS(1377),
    [aux_sym_variable_attributes_token2] = ACTIONS(1377),
    [aux_sym_variable_attributes_token3] = ACTIONS(1377),
    [aux_sym_variable_attributes_token4] = ACTIONS(1377),
    [aux_sym_variable_attributes_token5] = ACTIONS(1377),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1405),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1407),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1403),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1403),
    [aux_sym_derived_type_token1] = ACTIONS(1409),
    [aux_sym_type_qualifier_token1] = ACTIONS(1377),
    [aux_sym_type_qualifier_token2] = ACTIONS(1377),
    [aux_sym_type_qualifier_token3] = ACTIONS(1411),
    [aux_sym_type_qualifier_token4] = ACTIONS(1377),
    [aux_sym_type_qualifier_token5] = ACTIONS(1413),
    [aux_sym_type_qualifier_token9] = ACTIONS(1377),
    [aux_sym_type_qualifier_token10] = ACTIONS(1415),
    [aux_sym_type_qualifier_token11] = ACTIONS(1377),
    [aux_sym_type_qualifier_token12] = ACTIONS(1377),
    [aux_sym_type_qualifier_token13] = ACTIONS(1377),
    [aux_sym_type_qualifier_token14] = ACTIONS(1377),
    [aux_sym_type_qualifier_token15] = ACTIONS(1377),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1417),
    [aux_sym_include_statement_token1] = ACTIONS(1419),
    [aux_sym_data_statement_token1] = ACTIONS(1421),
    [aux_sym_enum_statement_token1] = ACTIONS(1423),
    [anon_sym_POUND] = ACTIONS(3),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1425),
  },
  [420] = {
    [sym_preproc_file_line] = STATE(420),
    [aux_sym_end_program_statement_token1] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_EQ] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1431),
    [anon_sym_COMMA] = ACTIONS(1285),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1436),
    [anon_sym_EQ_GT] = ACTIONS(1285),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [anon_sym_SLASH] = ACTIONS(1283),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1438),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1431),
    [anon_sym_STAR] = ACTIONS(1283),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1431),
    [aux_sym_type_qualifier_token14] = ACTIONS(1431),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1431),
    [aux_sym_stop_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token3] = ACTIONS(1431),
    [aux_sym_data_statement_token1] = ACTIONS(1431),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1431),
    [aux_sym_end_if_statement_token1] = ACTIONS(1431),
    [aux_sym_elseif_clause_token2] = ACTIONS(1431),
    [aux_sym_select_case_statement_token1] = ACTIONS(1431),
    [aux_sym_block_construct_token1] = ACTIONS(1431),
    [aux_sym_format_statement_token1] = ACTIONS(1431),
    [aux_sym_inquire_statement_token1] = ACTIONS(1431),
    [anon_sym_PERCENT] = ACTIONS(1285),
    [aux_sym_logical_expression_token1] = ACTIONS(1285),
    [aux_sym_logical_expression_token2] = ACTIONS(1285),
    [aux_sym_logical_expression_token3] = ACTIONS(1285),
    [aux_sym_logical_expression_token4] = ACTIONS(1285),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_LT] = ACTIONS(1283),
    [aux_sym_relational_expression_token1] = ACTIONS(1285),
    [anon_sym_GT] = ACTIONS(1283),
    [aux_sym_relational_expression_token2] = ACTIONS(1285),
    [anon_sym_LT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token3] = ACTIONS(1285),
    [anon_sym_GT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token4] = ACTIONS(1285),
    [anon_sym_EQ_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token5] = ACTIONS(1285),
    [anon_sym_SLASH_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token6] = ACTIONS(1285),
    [anon_sym_SLASH_SLASH] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(1438),
    [anon_sym_STAR_STAR] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(1433),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1431),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1431),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
    [sym__end_of_statement] = ACTIONS(1285),
  },
  [421] = {
    [sym_interface] = STATE(631),
    [sym_interface_statement] = STATE(758),
    [sym__specification_part] = STATE(645),
    [sym_use_statement] = STATE(2388),
    [sym_implicit_statement] = STATE(2388),
    [sym_save_statement] = STATE(2388),
    [sym_private_statement] = STATE(2388),
    [sym_public_statement] = STATE(2388),
    [sym_namelist_statement] = STATE(2388),
    [sym_common_statement] = STATE(2388),
    [sym_import_statement] = STATE(2388),
    [sym_derived_type_definition] = STATE(631),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(994),
    [sym_variable_declaration] = STATE(2388),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2388),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2388),
    [sym_equivalence_statement] = STATE(2388),
    [sym_statement_label] = STATE(1994),
    [sym_include_statement] = STATE(2388),
    [sym_data_statement] = STATE(2388),
    [sym_enum] = STATE(631),
    [sym_enum_statement] = STATE(1856),
    [sym_preproc_file_line] = STATE(421),
    [aux_sym_program_repeat1] = STATE(421),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [aux_sym_end_submodule_statement_token1] = ACTIONS(527),
    [aux_sym_interface_statement_token1] = ACTIONS(1441),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1444),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1447),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1450),
    [aux_sym_contains_statement_token1] = ACTIONS(527),
    [aux_sym_use_statement_token1] = ACTIONS(1453),
    [aux_sym_use_statement_token2] = ACTIONS(1450),
    [aux_sym_implicit_statement_token1] = ACTIONS(1456),
    [aux_sym_implicit_statement_token3] = ACTIONS(1459),
    [aux_sym_implicit_statement_token4] = ACTIONS(1450),
    [aux_sym_save_statement_token1] = ACTIONS(1462),
    [aux_sym_private_statement_token1] = ACTIONS(1465),
    [aux_sym_public_statement_token1] = ACTIONS(1468),
    [aux_sym_namelist_statement_token1] = ACTIONS(1471),
    [aux_sym_common_statement_token1] = ACTIONS(1474),
    [aux_sym_import_statement_token1] = ACTIONS(1477),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1480),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1483),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1450),
    [aux_sym_variable_attributes_token1] = ACTIONS(1450),
    [aux_sym_variable_attributes_token2] = ACTIONS(1450),
    [aux_sym_variable_attributes_token3] = ACTIONS(1450),
    [aux_sym_variable_attributes_token4] = ACTIONS(1450),
    [aux_sym_variable_attributes_token5] = ACTIONS(1450),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1489),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1486),
    [aux_sym_derived_type_token1] = ACTIONS(1492),
    [aux_sym_type_qualifier_token1] = ACTIONS(1450),
    [aux_sym_type_qualifier_token2] = ACTIONS(1450),
    [aux_sym_type_qualifier_token3] = ACTIONS(1495),
    [aux_sym_type_qualifier_token4] = ACTIONS(1450),
    [aux_sym_type_qualifier_token5] = ACTIONS(1498),
    [aux_sym_type_qualifier_token9] = ACTIONS(1450),
    [aux_sym_type_qualifier_token10] = ACTIONS(1501),
    [aux_sym_type_qualifier_token11] = ACTIONS(1450),
    [aux_sym_type_qualifier_token12] = ACTIONS(1450),
    [aux_sym_type_qualifier_token13] = ACTIONS(1450),
    [aux_sym_type_qualifier_token14] = ACTIONS(1450),
    [aux_sym_type_qualifier_token15] = ACTIONS(1450),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1504),
    [aux_sym_include_statement_token1] = ACTIONS(1507),
    [aux_sym_data_statement_token1] = ACTIONS(1510),
    [aux_sym_enum_statement_token1] = ACTIONS(1513),
    [anon_sym_POUND] = ACTIONS(3),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
  },
  [422] = {
    [sym_interface] = STATE(581),
    [sym_interface_statement] = STATE(763),
    [sym__specification_part] = STATE(580),
    [sym_use_statement] = STATE(2537),
    [sym_implicit_statement] = STATE(2537),
    [sym_save_statement] = STATE(2537),
    [sym_private_statement] = STATE(2537),
    [sym_public_statement] = STATE(2537),
    [sym_namelist_statement] = STATE(2537),
    [sym_common_statement] = STATE(2537),
    [sym_import_statement] = STATE(2537),
    [sym_derived_type_definition] = STATE(581),
    [sym_abstract_specifier] = STATE(2748),
    [sym_derived_type_statement] = STATE(954),
    [sym_variable_declaration] = STATE(2537),
    [sym_procedure_declaration] = STATE(482),
    [sym_variable_modification] = STATE(2537),
    [sym_variable_attributes] = STATE(457),
    [sym__intrinsic_type] = STATE(482),
    [sym_intrinsic_type] = STATE(773),
    [sym_derived_type] = STATE(482),
    [sym_type_qualifier] = STATE(457),
    [sym_parameter_statement] = STATE(2537),
    [sym_equivalence_statement] = STATE(2537),
    [sym_statement_label] = STATE(2131),
    [sym_include_statement] = STATE(2537),
    [sym_data_statement] = STATE(2537),
    [sym_enum] = STATE(581),
    [sym_enum_statement] = STATE(1858),
    [sym_preproc_file_line] = STATE(422),
    [aux_sym_program_repeat1] = STATE(422),
    [aux_sym_variable_modification_repeat1] = STATE(424),
    [aux_sym_end_program_statement_token1] = ACTIONS(489),
    [aux_sym_end_module_statement_token1] = ACTIONS(527),
    [aux_sym_interface_statement_token1] = ACTIONS(1441),
    [aux_sym_module_procedure_statement_token1] = ACTIONS(1444),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1447),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1450),
    [aux_sym_contains_statement_token1] = ACTIONS(527),
    [aux_sym_use_statement_token1] = ACTIONS(1453),
    [aux_sym_use_statement_token2] = ACTIONS(1450),
    [aux_sym_implicit_statement_token1] = ACTIONS(1456),
    [aux_sym_implicit_statement_token3] = ACTIONS(1459),
    [aux_sym_implicit_statement_token4] = ACTIONS(1450),
    [aux_sym_save_statement_token1] = ACTIONS(1462),
    [aux_sym_private_statement_token1] = ACTIONS(1465),
    [aux_sym_public_statement_token1] = ACTIONS(1468),
    [aux_sym_namelist_statement_token1] = ACTIONS(1471),
    [aux_sym_common_statement_token1] = ACTIONS(1474),
    [aux_sym_import_statement_token1] = ACTIONS(1477),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1480),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1483),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1450),
    [aux_sym_variable_attributes_token1] = ACTIONS(1450),
    [aux_sym_variable_attributes_token2] = ACTIONS(1450),
    [aux_sym_variable_attributes_token3] = ACTIONS(1450),
    [aux_sym_variable_attributes_token4] = ACTIONS(1450),
    [aux_sym_variable_attributes_token5] = ACTIONS(1450),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token4] = ACTIONS(541),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1489),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1486),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1486),
    [aux_sym_derived_type_token1] = ACTIONS(1492),
    [aux_sym_type_qualifier_token1] = ACTIONS(1450),
    [aux_sym_type_qualifier_token2] = ACTIONS(1450),
    [aux_sym_type_qualifier_token3] = ACTIONS(1495),
    [aux_sym_type_qualifier_token4] = ACTIONS(1450),
    [aux_sym_type_qualifier_token5] = ACTIONS(1498),
    [aux_sym_type_qualifier_token9] = ACTIONS(1450),
    [aux_sym_type_qualifier_token10] = ACTIONS(1501),
    [aux_sym_type_qualifier_token11] = ACTIONS(1450),
    [aux_sym_type_qualifier_token12] = ACTIONS(1450),
    [aux_sym_type_qualifier_token13] = ACTIONS(1450),
    [aux_sym_type_qualifier_token14] = ACTIONS(1450),
    [aux_sym_type_qualifier_token15] = ACTIONS(1450),
    [aux_sym_equivalence_statement_token1] = ACTIONS(1504),
    [aux_sym_include_statement_token1] = ACTIONS(1507),
    [aux_sym_data_statement_token1] = ACTIONS(1510),
    [aux_sym_enum_statement_token1] = ACTIONS(1513),
    [anon_sym_POUND] = ACTIONS(3),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(571),
  },
  [423] = {
    [sym_preproc_file_line] = STATE(423),
    [aux_sym_end_program_statement_token1] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_COLON] = ACTIONS(1283),
    [anon_sym_EQ] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1436),
    [anon_sym_EQ_GT] = ACTIONS(1285),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [anon_sym_SLASH] = ACTIONS(1283),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1438),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1431),
    [anon_sym_STAR] = ACTIONS(1283),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1431),
    [aux_sym_type_qualifier_token14] = ACTIONS(1431),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1431),
    [aux_sym_stop_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token3] = ACTIONS(1431),
    [aux_sym_data_statement_token1] = ACTIONS(1431),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1431),
    [aux_sym_end_if_statement_token1] = ACTIONS(1431),
    [aux_sym_elseif_clause_token2] = ACTIONS(1431),
    [aux_sym_select_case_statement_token1] = ACTIONS(1431),
    [aux_sym_block_construct_token1] = ACTIONS(1431),
    [aux_sym_format_statement_token1] = ACTIONS(1431),
    [aux_sym_inquire_statement_token1] = ACTIONS(1431),
    [anon_sym_PERCENT] = ACTIONS(1285),
    [aux_sym_logical_expression_token1] = ACTIONS(1285),
    [aux_sym_logical_expression_token2] = ACTIONS(1285),
    [aux_sym_logical_expression_token3] = ACTIONS(1285),
    [aux_sym_logical_expression_token4] = ACTIONS(1285),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_LT] = ACTIONS(1283),
    [aux_sym_relational_expression_token1] = ACTIONS(1285),
    [anon_sym_GT] = ACTIONS(1283),
    [aux_sym_relational_expression_token2] = ACTIONS(1285),
    [anon_sym_LT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token3] = ACTIONS(1285),
    [anon_sym_GT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token4] = ACTIONS(1285),
    [anon_sym_EQ_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token5] = ACTIONS(1285),
    [anon_sym_SLASH_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token6] = ACTIONS(1285),
    [anon_sym_SLASH_SLASH] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(1438),
    [anon_sym_STAR_STAR] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(1433),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1431),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1431),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
  },
  [424] = {
    [sym_variable_attributes] = STATE(457),
    [sym__declaration_targets] = STATE(2561),
    [sym_type_qualifier] = STATE(457),
    [sym_assignment_statement] = STATE(1956),
    [sym_pointer_association_statement] = STATE(1956),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1023),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(424),
    [sym_identifier] = STATE(995),
    [aux_sym_variable_modification_repeat1] = STATE(439),
    [aux_sym_end_program_statement_token1] = ACTIONS(1516),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1518),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1518),
    [aux_sym_procedure_attributes_token1] = ACTIONS(387),
    [aux_sym_procedure_attributes_token3] = ACTIONS(35),
    [aux_sym_use_statement_token2] = ACTIONS(35),
    [anon_sym_COLON_COLON] = ACTIONS(1520),
    [aux_sym_implicit_statement_token4] = ACTIONS(35),
    [aux_sym_save_statement_token1] = ACTIONS(35),
    [aux_sym_private_statement_token1] = ACTIONS(35),
    [aux_sym_public_statement_token1] = ACTIONS(35),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_derived_type_definition_token1] = ACTIONS(59),
    [aux_sym_abstract_specifier_token1] = ACTIONS(35),
    [aux_sym_procedure_attribute_token6] = ACTIONS(35),
    [aux_sym_variable_attributes_token1] = ACTIONS(35),
    [aux_sym_variable_attributes_token2] = ACTIONS(35),
    [aux_sym_variable_attributes_token3] = ACTIONS(35),
    [aux_sym_variable_attributes_token4] = ACTIONS(35),
    [aux_sym_variable_attributes_token5] = ACTIONS(35),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1522),
    [aux_sym_type_qualifier_token1] = ACTIONS(35),
    [aux_sym_type_qualifier_token2] = ACTIONS(35),
    [aux_sym_type_qualifier_token3] = ACTIONS(73),
    [aux_sym_type_qualifier_token4] = ACTIONS(35),
    [aux_sym_type_qualifier_token5] = ACTIONS(75),
    [aux_sym_type_qualifier_token9] = ACTIONS(35),
    [aux_sym_type_qualifier_token10] = ACTIONS(35),
    [aux_sym_type_qualifier_token11] = ACTIONS(35),
    [aux_sym_type_qualifier_token12] = ACTIONS(35),
    [aux_sym_type_qualifier_token13] = ACTIONS(1524),
    [aux_sym_type_qualifier_token14] = ACTIONS(1524),
    [aux_sym_type_qualifier_token15] = ACTIONS(35),
    [aux_sym_stop_statement_token1] = ACTIONS(1518),
    [aux_sym_stop_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token3] = ACTIONS(1518),
    [aux_sym_data_statement_token1] = ACTIONS(1518),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1518),
    [aux_sym_end_if_statement_token1] = ACTIONS(1518),
    [aux_sym_elseif_clause_token2] = ACTIONS(1518),
    [aux_sym_select_case_statement_token1] = ACTIONS(1526),
    [aux_sym_block_construct_token1] = ACTIONS(1518),
    [aux_sym_format_statement_token1] = ACTIONS(1518),
    [aux_sym_inquire_statement_token1] = ACTIONS(1518),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(1528),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1518),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [425] = {
    [sym_statement_label] = STATE(2318),
    [sym_statement_label_reference] = STATE(2316),
    [sym__io_arguments] = STATE(559),
    [sym_format_identifier] = STATE(2315),
    [sym__io_expressions] = STATE(2316),
    [sym__expression] = STATE(1135),
    [sym_parenthesized_expression] = STATE(1180),
    [sym_derived_type_member_expression] = STATE(1180),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(1180),
    [sym_math_expression] = STATE(1180),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1180),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(425),
    [sym_identifier] = STATE(1180),
    [aux_sym_end_program_statement_token1] = ACTIONS(1516),
    [anon_sym_LPAREN] = ACTIONS(1530),
    [anon_sym_COLON] = ACTIONS(1285),
    [anon_sym_EQ] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1518),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1518),
    [anon_sym_EQ_GT] = ACTIONS(1285),
    [anon_sym_SLASH] = ACTIONS(1283),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1522),
    [anon_sym_STAR] = ACTIONS(1532),
    [aux_sym_type_qualifier_token13] = ACTIONS(1518),
    [aux_sym_type_qualifier_token14] = ACTIONS(1518),
    [aux_sym_stop_statement_token1] = ACTIONS(1518),
    [aux_sym_stop_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token3] = ACTIONS(1518),
    [aux_sym_data_statement_token1] = ACTIONS(1518),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1518),
    [aux_sym_end_if_statement_token1] = ACTIONS(1518),
    [aux_sym_elseif_clause_token2] = ACTIONS(1518),
    [aux_sym_select_case_statement_token1] = ACTIONS(1526),
    [aux_sym_block_construct_token1] = ACTIONS(1518),
    [aux_sym_format_statement_token1] = ACTIONS(1518),
    [aux_sym_inquire_statement_token1] = ACTIONS(1518),
    [anon_sym_PERCENT] = ACTIONS(1285),
    [aux_sym_logical_expression_token1] = ACTIONS(1285),
    [aux_sym_logical_expression_token2] = ACTIONS(1285),
    [aux_sym_logical_expression_token3] = ACTIONS(1285),
    [aux_sym_logical_expression_token4] = ACTIONS(1285),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_LT] = ACTIONS(1283),
    [aux_sym_relational_expression_token1] = ACTIONS(1285),
    [anon_sym_GT] = ACTIONS(1283),
    [aux_sym_relational_expression_token2] = ACTIONS(1285),
    [anon_sym_LT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token3] = ACTIONS(1285),
    [anon_sym_GT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token4] = ACTIONS(1285),
    [anon_sym_EQ_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token5] = ACTIONS(1285),
    [anon_sym_SLASH_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token6] = ACTIONS(1285),
    [anon_sym_SLASH_SLASH] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_STAR_STAR] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(1528),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1518),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1534),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(1536),
  },
  [426] = {
    [sym__expression] = STATE(1021),
    [sym_parenthesized_expression] = STATE(1155),
    [sym_derived_type_member_expression] = STATE(1155),
    [sym_logical_expression] = STATE(1155),
    [sym_relational_expression] = STATE(1155),
    [sym_concatenation_expression] = STATE(1155),
    [sym_math_expression] = STATE(1155),
    [sym_unary_expression] = STATE(1155),
    [sym_user_defined_operator] = STATE(689),
    [sym_call_expression] = STATE(1155),
    [sym_implied_do_loop_expression] = STATE(1155),
    [sym_array_literal] = STATE(1155),
    [sym__array_constructor_legacy] = STATE(1156),
    [sym__array_constructor_f2003] = STATE(1157),
    [sym_complex_literal] = STATE(1155),
    [sym_number_literal] = STATE(1155),
    [sym_null_literal] = STATE(1155),
    [sym_preproc_file_line] = STATE(426),
    [sym_identifier] = STATE(1155),
    [aux_sym_end_program_statement_token1] = ACTIONS(1516),
    [anon_sym_LPAREN] = ACTIONS(1538),
    [anon_sym_COLON] = ACTIONS(1285),
    [anon_sym_EQ] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1518),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1518),
    [anon_sym_COMMA] = ACTIONS(1541),
    [anon_sym_EQ_GT] = ACTIONS(1285),
    [anon_sym_SLASH] = ACTIONS(1283),
    [anon_sym_DASH] = ACTIONS(1543),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1522),
    [anon_sym_STAR] = ACTIONS(1283),
    [aux_sym_type_qualifier_token13] = ACTIONS(1518),
    [aux_sym_type_qualifier_token14] = ACTIONS(1518),
    [aux_sym_stop_statement_token1] = ACTIONS(1518),
    [aux_sym_stop_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token3] = ACTIONS(1518),
    [aux_sym_data_statement_token1] = ACTIONS(1518),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1518),
    [aux_sym_end_if_statement_token1] = ACTIONS(1518),
    [aux_sym_elseif_clause_token2] = ACTIONS(1518),
    [aux_sym_select_case_statement_token1] = ACTIONS(1526),
    [aux_sym_block_construct_token1] = ACTIONS(1518),
    [aux_sym_format_statement_token1] = ACTIONS(1518),
    [aux_sym_inquire_statement_token1] = ACTIONS(1518),
    [anon_sym_PERCENT] = ACTIONS(1285),
    [aux_sym_logical_expression_token1] = ACTIONS(1285),
    [aux_sym_logical_expression_token2] = ACTIONS(1285),
    [aux_sym_logical_expression_token3] = ACTIONS(1285),
    [aux_sym_logical_expression_token4] = ACTIONS(1285),
    [aux_sym_logical_expression_token5] = ACTIONS(1546),
    [anon_sym_LT] = ACTIONS(1283),
    [aux_sym_relational_expression_token1] = ACTIONS(1285),
    [anon_sym_GT] = ACTIONS(1283),
    [aux_sym_relational_expression_token2] = ACTIONS(1285),
    [anon_sym_LT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token3] = ACTIONS(1285),
    [anon_sym_GT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token4] = ACTIONS(1285),
    [anon_sym_EQ_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token5] = ACTIONS(1285),
    [anon_sym_SLASH_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token6] = ACTIONS(1285),
    [anon_sym_SLASH_SLASH] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(1543),
    [anon_sym_STAR_STAR] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(1548),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1551),
    [anon_sym_LBRACK] = ACTIONS(1553),
    [sym_boolean_literal] = ACTIONS(1555),
    [aux_sym_null_literal_token1] = ACTIONS(1557),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1518),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1559),
    [sym__float_literal] = ACTIONS(1559),
    [sym__boz_literal] = ACTIONS(1559),
    [sym_string_literal] = ACTIONS(1555),
    [sym__end_of_statement] = ACTIONS(1561),
  },
  [427] = {
    [sym_preproc_file_line] = STATE(427),
    [aux_sym_end_program_statement_token1] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1563),
    [anon_sym_EQ] = ACTIONS(1565),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1563),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1563),
    [anon_sym_EQ_GT] = ACTIONS(1567),
    [aux_sym_implicit_statement_token3] = ACTIONS(1563),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_DASH] = ACTIONS(1569),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1563),
    [anon_sym_STAR] = ACTIONS(1565),
    [aux_sym_type_qualifier_token13] = ACTIONS(1563),
    [aux_sym_type_qualifier_token14] = ACTIONS(1563),
    [aux_sym_stop_statement_token1] = ACTIONS(1563),
    [aux_sym_stop_statement_token2] = ACTIONS(1563),
    [aux_sym_subroutine_call_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token2] = ACTIONS(1563),
    [aux_sym_keyword_statement_token3] = ACTIONS(1563),
    [aux_sym_keyword_statement_token4] = ACTIONS(1563),
    [aux_sym_keyword_statement_token6] = ACTIONS(1563),
    [aux_sym_keyword_statement_token7] = ACTIONS(1563),
    [aux_sym_include_statement_token1] = ACTIONS(1563),
    [aux_sym_data_statement_token1] = ACTIONS(1563),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1563),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1563),
    [aux_sym_end_if_statement_token1] = ACTIONS(1563),
    [aux_sym_elseif_clause_token2] = ACTIONS(1563),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1563),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token3] = ACTIONS(1563),
    [aux_sym_select_type_statement_token1] = ACTIONS(1563),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1563),
    [aux_sym_block_construct_token1] = ACTIONS(1563),
    [aux_sym_associate_statement_token1] = ACTIONS(1563),
    [aux_sym_format_statement_token1] = ACTIONS(1563),
    [aux_sym_print_statement_token1] = ACTIONS(1563),
    [aux_sym_open_statement_token1] = ACTIONS(1563),
    [aux_sym_close_statement_token1] = ACTIONS(1563),
    [aux_sym_inquire_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token2] = ACTIONS(1563),
    [aux_sym_file_position_statement_token3] = ACTIONS(1563),
    [aux_sym_file_position_statement_token4] = ACTIONS(1563),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [aux_sym_logical_expression_token1] = ACTIONS(1567),
    [aux_sym_logical_expression_token2] = ACTIONS(1567),
    [aux_sym_logical_expression_token3] = ACTIONS(1567),
    [aux_sym_logical_expression_token4] = ACTIONS(1567),
    [aux_sym_logical_expression_token5] = ACTIONS(1569),
    [anon_sym_LT] = ACTIONS(1565),
    [aux_sym_relational_expression_token1] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [aux_sym_relational_expression_token2] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token3] = ACTIONS(1567),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token4] = ACTIONS(1567),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token5] = ACTIONS(1567),
    [anon_sym_SLASH_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token6] = ACTIONS(1567),
    [anon_sym_SLASH_SLASH] = ACTIONS(1567),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR_STAR] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1563),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1569),
    [anon_sym_LBRACK] = ACTIONS(1569),
    [sym_boolean_literal] = ACTIONS(1569),
    [aux_sym_null_literal_token1] = ACTIONS(1563),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1563),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1569),
    [sym__float_literal] = ACTIONS(1569),
    [sym__boz_literal] = ACTIONS(1569),
    [sym_string_literal] = ACTIONS(1569),
  },
  [428] = {
    [sym_preproc_file_line] = STATE(428),
    [aux_sym_end_program_statement_token1] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1563),
    [anon_sym_EQ] = ACTIONS(1565),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1563),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1563),
    [anon_sym_EQ_GT] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_DASH] = ACTIONS(1569),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1563),
    [anon_sym_STAR] = ACTIONS(1565),
    [aux_sym_type_qualifier_token13] = ACTIONS(1563),
    [aux_sym_type_qualifier_token14] = ACTIONS(1563),
    [aux_sym_stop_statement_token1] = ACTIONS(1563),
    [aux_sym_stop_statement_token2] = ACTIONS(1563),
    [aux_sym_subroutine_call_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token2] = ACTIONS(1563),
    [aux_sym_keyword_statement_token3] = ACTIONS(1563),
    [aux_sym_keyword_statement_token4] = ACTIONS(1563),
    [aux_sym_keyword_statement_token6] = ACTIONS(1563),
    [aux_sym_keyword_statement_token7] = ACTIONS(1563),
    [aux_sym_include_statement_token1] = ACTIONS(1563),
    [aux_sym_data_statement_token1] = ACTIONS(1563),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1563),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1563),
    [aux_sym_end_if_statement_token1] = ACTIONS(1563),
    [aux_sym_elseif_clause_token2] = ACTIONS(1563),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1563),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1563),
    [aux_sym_end_forall_statement_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token3] = ACTIONS(1563),
    [aux_sym_select_type_statement_token1] = ACTIONS(1563),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1563),
    [aux_sym_block_construct_token1] = ACTIONS(1563),
    [aux_sym_associate_statement_token1] = ACTIONS(1563),
    [aux_sym_format_statement_token1] = ACTIONS(1563),
    [aux_sym_print_statement_token1] = ACTIONS(1563),
    [aux_sym_open_statement_token1] = ACTIONS(1563),
    [aux_sym_close_statement_token1] = ACTIONS(1563),
    [aux_sym_inquire_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token2] = ACTIONS(1563),
    [aux_sym_file_position_statement_token3] = ACTIONS(1563),
    [aux_sym_file_position_statement_token4] = ACTIONS(1563),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [aux_sym_logical_expression_token1] = ACTIONS(1567),
    [aux_sym_logical_expression_token2] = ACTIONS(1567),
    [aux_sym_logical_expression_token3] = ACTIONS(1567),
    [aux_sym_logical_expression_token4] = ACTIONS(1567),
    [aux_sym_logical_expression_token5] = ACTIONS(1569),
    [anon_sym_LT] = ACTIONS(1565),
    [aux_sym_relational_expression_token1] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [aux_sym_relational_expression_token2] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token3] = ACTIONS(1567),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token4] = ACTIONS(1567),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token5] = ACTIONS(1567),
    [anon_sym_SLASH_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token6] = ACTIONS(1567),
    [anon_sym_SLASH_SLASH] = ACTIONS(1567),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR_STAR] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1563),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1569),
    [anon_sym_LBRACK] = ACTIONS(1569),
    [sym_boolean_literal] = ACTIONS(1569),
    [aux_sym_null_literal_token1] = ACTIONS(1563),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1563),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1569),
    [sym__float_literal] = ACTIONS(1569),
    [sym__boz_literal] = ACTIONS(1569),
    [sym_string_literal] = ACTIONS(1569),
  },
  [429] = {
    [sym_preproc_file_line] = STATE(429),
    [aux_sym_end_program_statement_token1] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1563),
    [anon_sym_EQ] = ACTIONS(1565),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1563),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1563),
    [anon_sym_EQ_GT] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_DASH] = ACTIONS(1569),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1563),
    [anon_sym_STAR] = ACTIONS(1565),
    [aux_sym_type_qualifier_token13] = ACTIONS(1563),
    [aux_sym_type_qualifier_token14] = ACTIONS(1563),
    [aux_sym_stop_statement_token1] = ACTIONS(1563),
    [aux_sym_stop_statement_token2] = ACTIONS(1563),
    [aux_sym_subroutine_call_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token2] = ACTIONS(1563),
    [aux_sym_keyword_statement_token3] = ACTIONS(1563),
    [aux_sym_keyword_statement_token4] = ACTIONS(1563),
    [aux_sym_keyword_statement_token6] = ACTIONS(1563),
    [aux_sym_keyword_statement_token7] = ACTIONS(1563),
    [aux_sym_include_statement_token1] = ACTIONS(1563),
    [aux_sym_data_statement_token1] = ACTIONS(1563),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1563),
    [aux_sym_end_do_loop_statement_token1] = ACTIONS(1563),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1563),
    [aux_sym_end_if_statement_token1] = ACTIONS(1563),
    [aux_sym_elseif_clause_token2] = ACTIONS(1563),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1563),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token3] = ACTIONS(1563),
    [aux_sym_select_type_statement_token1] = ACTIONS(1563),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1563),
    [aux_sym_block_construct_token1] = ACTIONS(1563),
    [aux_sym_associate_statement_token1] = ACTIONS(1563),
    [aux_sym_format_statement_token1] = ACTIONS(1563),
    [aux_sym_print_statement_token1] = ACTIONS(1563),
    [aux_sym_open_statement_token1] = ACTIONS(1563),
    [aux_sym_close_statement_token1] = ACTIONS(1563),
    [aux_sym_inquire_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token2] = ACTIONS(1563),
    [aux_sym_file_position_statement_token3] = ACTIONS(1563),
    [aux_sym_file_position_statement_token4] = ACTIONS(1563),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [aux_sym_logical_expression_token1] = ACTIONS(1567),
    [aux_sym_logical_expression_token2] = ACTIONS(1567),
    [aux_sym_logical_expression_token3] = ACTIONS(1567),
    [aux_sym_logical_expression_token4] = ACTIONS(1567),
    [aux_sym_logical_expression_token5] = ACTIONS(1569),
    [anon_sym_LT] = ACTIONS(1565),
    [aux_sym_relational_expression_token1] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [aux_sym_relational_expression_token2] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token3] = ACTIONS(1567),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token4] = ACTIONS(1567),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token5] = ACTIONS(1567),
    [anon_sym_SLASH_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token6] = ACTIONS(1567),
    [anon_sym_SLASH_SLASH] = ACTIONS(1567),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR_STAR] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1563),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1569),
    [anon_sym_LBRACK] = ACTIONS(1569),
    [sym_boolean_literal] = ACTIONS(1569),
    [aux_sym_null_literal_token1] = ACTIONS(1563),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1563),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1569),
    [sym__float_literal] = ACTIONS(1569),
    [sym__boz_literal] = ACTIONS(1569),
    [sym_string_literal] = ACTIONS(1569),
  },
  [430] = {
    [sym_data_set] = STATE(2202),
    [sym__expression] = STATE(1132),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(1158),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1158),
    [sym_implied_do_loop_expression] = STATE(1158),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(430),
    [sym_identifier] = STATE(1158),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [anon_sym_COLON] = ACTIONS(1285),
    [anon_sym_EQ] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [anon_sym_EQ_GT] = ACTIONS(1285),
    [anon_sym_SLASH] = ACTIONS(1283),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [anon_sym_STAR] = ACTIONS(1283),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [anon_sym_PERCENT] = ACTIONS(1285),
    [aux_sym_logical_expression_token1] = ACTIONS(1285),
    [aux_sym_logical_expression_token2] = ACTIONS(1285),
    [aux_sym_logical_expression_token3] = ACTIONS(1285),
    [aux_sym_logical_expression_token4] = ACTIONS(1285),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_LT] = ACTIONS(1283),
    [aux_sym_relational_expression_token1] = ACTIONS(1285),
    [anon_sym_GT] = ACTIONS(1283),
    [aux_sym_relational_expression_token2] = ACTIONS(1285),
    [anon_sym_LT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token3] = ACTIONS(1285),
    [anon_sym_GT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token4] = ACTIONS(1285),
    [anon_sym_EQ_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token5] = ACTIONS(1285),
    [anon_sym_SLASH_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token6] = ACTIONS(1285),
    [anon_sym_SLASH_SLASH] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_STAR_STAR] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [431] = {
    [sym_preproc_file_line] = STATE(431),
    [aux_sym_end_program_statement_token1] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1563),
    [anon_sym_EQ] = ACTIONS(1565),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1563),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1563),
    [anon_sym_EQ_GT] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_DASH] = ACTIONS(1569),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1563),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1563),
    [anon_sym_STAR] = ACTIONS(1565),
    [aux_sym_type_qualifier_token13] = ACTIONS(1563),
    [aux_sym_type_qualifier_token14] = ACTIONS(1563),
    [aux_sym_stop_statement_token1] = ACTIONS(1563),
    [aux_sym_stop_statement_token2] = ACTIONS(1563),
    [aux_sym_subroutine_call_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token1] = ACTIONS(1563),
    [aux_sym_keyword_statement_token2] = ACTIONS(1563),
    [aux_sym_keyword_statement_token3] = ACTIONS(1563),
    [aux_sym_keyword_statement_token4] = ACTIONS(1563),
    [aux_sym_keyword_statement_token6] = ACTIONS(1563),
    [aux_sym_keyword_statement_token7] = ACTIONS(1563),
    [aux_sym_include_statement_token1] = ACTIONS(1563),
    [aux_sym_data_statement_token1] = ACTIONS(1563),
    [aux_sym_do_loop_statement_token1] = ACTIONS(1563),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1563),
    [aux_sym_end_if_statement_token1] = ACTIONS(1563),
    [aux_sym_elseif_clause_token2] = ACTIONS(1563),
    [aux_sym__inline_where_statement_token1] = ACTIONS(1563),
    [aux_sym__forall_control_expression_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token1] = ACTIONS(1563),
    [aux_sym_select_case_statement_token3] = ACTIONS(1563),
    [aux_sym_select_type_statement_token1] = ACTIONS(1563),
    [aux_sym_select_rank_statement_token2] = ACTIONS(1563),
    [aux_sym_block_construct_token1] = ACTIONS(1563),
    [aux_sym_associate_statement_token1] = ACTIONS(1563),
    [aux_sym_format_statement_token1] = ACTIONS(1563),
    [aux_sym_print_statement_token1] = ACTIONS(1563),
    [aux_sym_open_statement_token1] = ACTIONS(1563),
    [aux_sym_close_statement_token1] = ACTIONS(1563),
    [aux_sym_inquire_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token1] = ACTIONS(1563),
    [aux_sym_file_position_statement_token2] = ACTIONS(1563),
    [aux_sym_file_position_statement_token3] = ACTIONS(1563),
    [aux_sym_file_position_statement_token4] = ACTIONS(1563),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [aux_sym_logical_expression_token1] = ACTIONS(1567),
    [aux_sym_logical_expression_token2] = ACTIONS(1567),
    [aux_sym_logical_expression_token3] = ACTIONS(1567),
    [aux_sym_logical_expression_token4] = ACTIONS(1567),
    [aux_sym_logical_expression_token5] = ACTIONS(1569),
    [anon_sym_LT] = ACTIONS(1565),
    [aux_sym_relational_expression_token1] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [aux_sym_relational_expression_token2] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token3] = ACTIONS(1567),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token4] = ACTIONS(1567),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token5] = ACTIONS(1567),
    [anon_sym_SLASH_EQ] = ACTIONS(1567),
    [aux_sym_relational_expression_token6] = ACTIONS(1567),
    [anon_sym_SLASH_SLASH] = ACTIONS(1567),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR_STAR] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1563),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1569),
    [anon_sym_LBRACK] = ACTIONS(1569),
    [sym_boolean_literal] = ACTIONS(1569),
    [aux_sym_null_literal_token1] = ACTIONS(1563),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1563),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1569),
    [sym__float_literal] = ACTIONS(1569),
    [sym__boz_literal] = ACTIONS(1569),
    [sym_string_literal] = ACTIONS(1569),
  },
  [432] = {
    [sym_preproc_file_line] = STATE(432),
    [aux_sym_end_program_statement_token1] = ACTIONS(1573),
    [aux_sym_module_statement_token1] = ACTIONS(1573),
    [anon_sym_LPAREN] = ACTIONS(1575),
    [anon_sym_COLON] = ACTIONS(1283),
    [anon_sym_EQ] = ACTIONS(1283),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1573),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1573),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1573),
    [aux_sym_function_statement_token1] = ACTIONS(1573),
    [anon_sym_COMMA] = ACTIONS(1578),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1573),
    [anon_sym_COLON_COLON] = ACTIONS(1578),
    [anon_sym_EQ_GT] = ACTIONS(1285),
    [aux_sym_implicit_statement_token3] = ACTIONS(1573),
    [anon_sym_SLASH] = ACTIONS(1283),
    [anon_sym_DASH] = ACTIONS(1580),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1573),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1573),
    [aux_sym_derived_type_token1] = ACTIONS(1573),
    [anon_sym_STAR] = ACTIONS(1575),
    [aux_sym_type_qualifier_token13] = ACTIONS(1573),
    [aux_sym_type_qualifier_token14] = ACTIONS(1573),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1573),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1573),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1573),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1573),
    [aux_sym_stop_statement_token1] = ACTIONS(1573),
    [aux_sym_stop_statement_token2] = ACTIONS(1573),
    [aux_sym_keyword_statement_token2] = ACTIONS(1573),
    [aux_sym_keyword_statement_token3] = ACTIONS(1573),
    [aux_sym_data_statement_token1] = ACTIONS(1573),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1573),
    [aux_sym_end_if_statement_token1] = ACTIONS(1573),
    [aux_sym_elseif_clause_token2] = ACTIONS(1573),
    [aux_sym_select_case_statement_token1] = ACTIONS(1573),
    [aux_sym_block_construct_token1] = ACTIONS(1573),
    [aux_sym_format_statement_token1] = ACTIONS(1573),
    [aux_sym_inquire_statement_token1] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1285),
    [aux_sym_logical_expression_token1] = ACTIONS(1285),
    [aux_sym_logical_expression_token2] = ACTIONS(1285),
    [aux_sym_logical_expression_token3] = ACTIONS(1285),
    [aux_sym_logical_expression_token4] = ACTIONS(1285),
    [aux_sym_logical_expression_token5] = ACTIONS(1578),
    [anon_sym_LT] = ACTIONS(1283),
    [aux_sym_relational_expression_token1] = ACTIONS(1285),
    [anon_sym_GT] = ACTIONS(1283),
    [aux_sym_relational_expression_token2] = ACTIONS(1285),
    [anon_sym_LT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token3] = ACTIONS(1285),
    [anon_sym_GT_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token4] = ACTIONS(1285),
    [anon_sym_EQ_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token5] = ACTIONS(1285),
    [anon_sym_SLASH_EQ] = ACTIONS(1285),
    [aux_sym_relational_expression_token6] = ACTIONS(1285),
    [anon_sym_SLASH_SLASH] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(1580),
    [anon_sym_STAR_STAR] = ACTIONS(1285),
    [anon_sym_DOT] = ACTIONS(1575),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1578),
    [anon_sym_LBRACK] = ACTIONS(1578),
    [sym_boolean_literal] = ACTIONS(1578),
    [aux_sym_null_literal_token1] = ACTIONS(1573),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1573),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1578),
    [sym__float_literal] = ACTIONS(1578),
    [sym__boz_literal] = ACTIONS(1578),
    [sym_string_literal] = ACTIONS(1578),
  },
  [433] = {
    [sym_preproc_file_line] = STATE(433),
    [aux_sym_end_program_statement_token1] = ACTIONS(1583),
    [aux_sym_module_statement_token1] = ACTIONS(1585),
    [anon_sym_LPAREN] = ACTIONS(1583),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1583),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1583),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1585),
    [aux_sym_function_statement_token1] = ACTIONS(1585),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1585),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1583),
    [aux_sym_use_statement_token2] = ACTIONS(1583),
    [anon_sym_COLON_COLON] = ACTIONS(1587),
    [aux_sym_implicit_statement_token3] = ACTIONS(1585),
    [aux_sym_implicit_statement_token4] = ACTIONS(1583),
    [aux_sym_save_statement_token1] = ACTIONS(1583),
    [aux_sym_private_statement_token1] = ACTIONS(1583),
    [aux_sym_public_statement_token1] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1587),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1583),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1583),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1583),
    [aux_sym_variable_attributes_token1] = ACTIONS(1583),
    [aux_sym_variable_attributes_token2] = ACTIONS(1583),
    [aux_sym_variable_attributes_token3] = ACTIONS(1583),
    [aux_sym_variable_attributes_token4] = ACTIONS(1583),
    [aux_sym_variable_attributes_token5] = ACTIONS(1583),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1585),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1585),
    [aux_sym_derived_type_token1] = ACTIONS(1585),
    [aux_sym_type_qualifier_token1] = ACTIONS(1583),
    [aux_sym_type_qualifier_token2] = ACTIONS(1583),
    [aux_sym_type_qualifier_token3] = ACTIONS(1583),
    [aux_sym_type_qualifier_token4] = ACTIONS(1583),
    [aux_sym_type_qualifier_token5] = ACTIONS(1583),
    [aux_sym_type_qualifier_token9] = ACTIONS(1583),
    [aux_sym_type_qualifier_token10] = ACTIONS(1583),
    [aux_sym_type_qualifier_token11] = ACTIONS(1583),
    [aux_sym_type_qualifier_token12] = ACTIONS(1583),
    [aux_sym_type_qualifier_token13] = ACTIONS(1583),
    [aux_sym_type_qualifier_token14] = ACTIONS(1583),
    [aux_sym_type_qualifier_token15] = ACTIONS(1583),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1585),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1585),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1585),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1585),
    [aux_sym_stop_statement_token1] = ACTIONS(1583),
    [aux_sym_stop_statement_token2] = ACTIONS(1583),
    [aux_sym_keyword_statement_token2] = ACTIONS(1583),
    [aux_sym_keyword_statement_token3] = ACTIONS(1583),
    [aux_sym_data_statement_token1] = ACTIONS(1583),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1583),
    [aux_sym_end_if_statement_token1] = ACTIONS(1583),
    [aux_sym_elseif_clause_token2] = ACTIONS(1583),
    [aux_sym_select_case_statement_token1] = ACTIONS(1583),
    [aux_sym_block_construct_token1] = ACTIONS(1583),
    [aux_sym_format_statement_token1] = ACTIONS(1583),
    [aux_sym_inquire_statement_token1] = ACTIONS(1583),
    [aux_sym_logical_expression_token5] = ACTIONS(1587),
    [anon_sym_PLUS] = ACTIONS(1587),
    [anon_sym_DOT] = ACTIONS(1583),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1587),
    [anon_sym_LBRACK] = ACTIONS(1587),
    [sym_boolean_literal] = ACTIONS(1587),
    [aux_sym_null_literal_token1] = ACTIONS(1583),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1583),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1587),
    [sym__float_literal] = ACTIONS(1587),
    [sym__boz_literal] = ACTIONS(1587),
    [sym_string_literal] = ACTIONS(1587),
  },
  [434] = {
    [sym__declaration_targets] = STATE(2393),
    [sym_assignment_statement] = STATE(1956),
    [sym_pointer_association_statement] = STATE(1956),
    [sym__expression] = STATE(1025),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(1023),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(434),
    [sym_identifier] = STATE(995),
    [aux_sym_end_program_statement_token1] = ACTIONS(1516),
    [aux_sym_module_statement_token1] = ACTIONS(1589),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1518),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1518),
    [aux_sym_subroutine_statement_token1] = ACTIONS(1589),
    [aux_sym_function_statement_token1] = ACTIONS(1589),
    [anon_sym_COMMA] = ACTIONS(1591),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1589),
    [anon_sym_COLON_COLON] = ACTIONS(1593),
    [aux_sym_implicit_statement_token3] = ACTIONS(1589),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1589),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1589),
    [aux_sym_derived_type_token1] = ACTIONS(1589),
    [aux_sym_type_qualifier_token13] = ACTIONS(1518),
    [aux_sym_type_qualifier_token14] = ACTIONS(1518),
    [aux_sym_procedure_qualifier_token1] = ACTIONS(1589),
    [aux_sym_procedure_qualifier_token2] = ACTIONS(1589),
    [aux_sym_procedure_qualifier_token3] = ACTIONS(1589),
    [aux_sym_procedure_qualifier_token4] = ACTIONS(1589),
    [aux_sym_stop_statement_token1] = ACTIONS(1518),
    [aux_sym_stop_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token3] = ACTIONS(1518),
    [aux_sym_data_statement_token1] = ACTIONS(1518),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1518),
    [aux_sym_end_if_statement_token1] = ACTIONS(1518),
    [aux_sym_elseif_clause_token2] = ACTIONS(1518),
    [aux_sym_select_case_statement_token1] = ACTIONS(1526),
    [aux_sym_block_construct_token1] = ACTIONS(1518),
    [aux_sym_format_statement_token1] = ACTIONS(1518),
    [aux_sym_inquire_statement_token1] = ACTIONS(1518),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(1528),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1518),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [435] = {
    [sym__intrinsic_type] = STATE(2499),
    [sym_intrinsic_type] = STATE(1338),
    [sym_derived_type] = STATE(2499),
    [sym__expression] = STATE(893),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(808),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym__type_spec] = STATE(539),
    [sym__ac_value_list] = STATE(2496),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(435),
    [sym_identifier] = STATE(808),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [aux_sym_implicit_statement_token3] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1601),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1603),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1599),
    [aux_sym_derived_type_token1] = ACTIONS(1605),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [436] = {
    [sym__intrinsic_type] = STATE(2499),
    [sym_intrinsic_type] = STATE(1338),
    [sym_derived_type] = STATE(2499),
    [sym__expression] = STATE(893),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(808),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym__type_spec] = STATE(539),
    [sym__ac_value_list] = STATE(2528),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(436),
    [sym_identifier] = STATE(808),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [aux_sym_implicit_statement_token3] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1601),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1603),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1599),
    [aux_sym_derived_type_token1] = ACTIONS(1605),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [437] = {
    [sym__intrinsic_type] = STATE(2499),
    [sym_intrinsic_type] = STATE(1338),
    [sym_derived_type] = STATE(2499),
    [sym__expression] = STATE(893),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(808),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym__type_spec] = STATE(539),
    [sym__ac_value_list] = STATE(2527),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(437),
    [sym_identifier] = STATE(808),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [aux_sym_implicit_statement_token3] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1601),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1603),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1599),
    [aux_sym_derived_type_token1] = ACTIONS(1605),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [438] = {
    [sym__intrinsic_type] = STATE(2499),
    [sym_intrinsic_type] = STATE(1338),
    [sym_derived_type] = STATE(2499),
    [sym__expression] = STATE(893),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(725),
    [sym_call_expression] = STATE(808),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym__type_spec] = STATE(539),
    [sym__ac_value_list] = STATE(2495),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(438),
    [sym_identifier] = STATE(808),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [aux_sym_implicit_statement_token3] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(55),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token2] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1597),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1601),
    [aux_sym_intrinsic_type_token6] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token7] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token8] = ACTIONS(1603),
    [aux_sym_intrinsic_type_token9] = ACTIONS(1599),
    [aux_sym_intrinsic_type_token10] = ACTIONS(1599),
    [aux_sym_derived_type_token1] = ACTIONS(1605),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [aux_sym_logical_expression_token5] = ACTIONS(141),
    [anon_sym_PLUS] = ACTIONS(55),
    [anon_sym_DOT] = ACTIONS(143),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [439] = {
    [sym_variable_attributes] = STATE(457),
    [sym_type_qualifier] = STATE(457),
    [sym_preproc_file_line] = STATE(439),
    [aux_sym_variable_modification_repeat1] = STATE(439),
    [aux_sym_end_program_statement_token1] = ACTIONS(1607),
    [anon_sym_LPAREN] = ACTIONS(1607),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1607),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1607),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1609),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1612),
    [aux_sym_use_statement_token2] = ACTIONS(1612),
    [anon_sym_COLON_COLON] = ACTIONS(1615),
    [aux_sym_implicit_statement_token4] = ACTIONS(1612),
    [aux_sym_save_statement_token1] = ACTIONS(1612),
    [aux_sym_private_statement_token1] = ACTIONS(1612),
    [aux_sym_public_statement_token1] = ACTIONS(1612),
    [anon_sym_DASH] = ACTIONS(1615),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1617),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1612),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1612),
    [aux_sym_variable_attributes_token1] = ACTIONS(1612),
    [aux_sym_variable_attributes_token2] = ACTIONS(1612),
    [aux_sym_variable_attributes_token3] = ACTIONS(1612),
    [aux_sym_variable_attributes_token4] = ACTIONS(1612),
    [aux_sym_variable_attributes_token5] = ACTIONS(1612),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1607),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1607),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1607),
    [aux_sym_type_qualifier_token1] = ACTIONS(1612),
    [aux_sym_type_qualifier_token2] = ACTIONS(1612),
    [aux_sym_type_qualifier_token3] = ACTIONS(1620),
    [aux_sym_type_qualifier_token4] = ACTIONS(1612),
    [aux_sym_type_qualifier_token5] = ACTIONS(1623),
    [aux_sym_type_qualifier_token9] = ACTIONS(1612),
    [aux_sym_type_qualifier_token10] = ACTIONS(1612),
    [aux_sym_type_qualifier_token11] = ACTIONS(1612),
    [aux_sym_type_qualifier_token12] = ACTIONS(1612),
    [aux_sym_type_qualifier_token13] = ACTIONS(1612),
    [aux_sym_type_qualifier_token14] = ACTIONS(1612),
    [aux_sym_type_qualifier_token15] = ACTIONS(1612),
    [aux_sym_stop_statement_token1] = ACTIONS(1607),
    [aux_sym_stop_statement_token2] = ACTIONS(1607),
    [aux_sym_keyword_statement_token2] = ACTIONS(1607),
    [aux_sym_keyword_statement_token3] = ACTIONS(1607),
    [aux_sym_data_statement_token1] = ACTIONS(1607),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1607),
    [aux_sym_end_if_statement_token1] = ACTIONS(1607),
    [aux_sym_elseif_clause_token2] = ACTIONS(1607),
    [aux_sym_select_case_statement_token1] = ACTIONS(1607),
    [aux_sym_block_construct_token1] = ACTIONS(1607),
    [aux_sym_format_statement_token1] = ACTIONS(1607),
    [aux_sym_inquire_statement_token1] = ACTIONS(1607),
    [aux_sym_logical_expression_token5] = ACTIONS(1615),
    [anon_sym_PLUS] = ACTIONS(1615),
    [anon_sym_DOT] = ACTIONS(1607),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1615),
    [anon_sym_LBRACK] = ACTIONS(1615),
    [sym_boolean_literal] = ACTIONS(1615),
    [aux_sym_null_literal_token1] = ACTIONS(1607),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1607),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1615),
    [sym__float_literal] = ACTIONS(1615),
    [sym__boz_literal] = ACTIONS(1615),
    [sym_string_literal] = ACTIONS(1615),
  },
  [440] = {
    [sym_preproc_file_line] = STATE(440),
    [sym_identifier] = STATE(2092),
    [aux_sym_end_program_statement_token1] = ACTIONS(1516),
    [anon_sym_LPAREN] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1518),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1518),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1626),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [anon_sym_SLASH] = ACTIONS(1628),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1436),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1518),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1522),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1518),
    [aux_sym_type_qualifier_token14] = ACTIONS(1518),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1518),
    [aux_sym_stop_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token2] = ACTIONS(1518),
    [aux_sym_keyword_statement_token3] = ACTIONS(1518),
    [aux_sym_data_statement_token1] = ACTIONS(1518),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1518),
    [aux_sym_end_if_statement_token1] = ACTIONS(1518),
    [aux_sym_elseif_clause_token2] = ACTIONS(1518),
    [aux_sym_select_case_statement_token1] = ACTIONS(1526),
    [aux_sym_block_construct_token1] = ACTIONS(1518),
    [aux_sym_format_statement_token1] = ACTIONS(1518),
    [aux_sym_inquire_statement_token1] = ACTIONS(1518),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_PLUS] = ACTIONS(1436),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1518),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1518),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
    [sym__end_of_statement] = ACTIONS(1630),
  },
  [441] = {
    [sym_implicit_range] = STATE(2043),
    [sym__expression] = STATE(892),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(660),
    [sym_call_expression] = STATE(808),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_keyword_argument] = STATE(2153),
    [sym_extent_specifier] = STATE(2153),
    [sym_assumed_size] = STATE(2153),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(441),
    [sym_identifier] = STATE(1126),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [anon_sym_COLON] = ACTIONS(1632),
    [anon_sym_RPAREN] = ACTIONS(1634),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [aux_sym_implicit_range_token1] = ACTIONS(1636),
    [anon_sym_DASH] = ACTIONS(1638),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [anon_sym_STAR] = ACTIONS(1640),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [aux_sym_logical_expression_token5] = ACTIONS(1642),
    [anon_sym_PLUS] = ACTIONS(1638),
    [anon_sym_DOT] = ACTIONS(143),
    [sym_assumed_rank] = ACTIONS(1644),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [442] = {
    [sym_implicit_range] = STATE(1990),
    [sym__expression] = STATE(892),
    [sym_parenthesized_expression] = STATE(808),
    [sym_derived_type_member_expression] = STATE(808),
    [sym_logical_expression] = STATE(808),
    [sym_relational_expression] = STATE(808),
    [sym_concatenation_expression] = STATE(808),
    [sym_math_expression] = STATE(808),
    [sym_unary_expression] = STATE(808),
    [sym_user_defined_operator] = STATE(660),
    [sym_call_expression] = STATE(808),
    [sym_implied_do_loop_expression] = STATE(808),
    [sym_keyword_argument] = STATE(2153),
    [sym_extent_specifier] = STATE(2153),
    [sym_assumed_size] = STATE(2153),
    [sym_array_literal] = STATE(808),
    [sym__array_constructor_legacy] = STATE(803),
    [sym__array_constructor_f2003] = STATE(804),
    [sym_complex_literal] = STATE(808),
    [sym_number_literal] = STATE(808),
    [sym_null_literal] = STATE(808),
    [sym_preproc_file_line] = STATE(442),
    [sym_identifier] = STATE(1126),
    [aux_sym_end_program_statement_token1] = ACTIONS(999),
    [anon_sym_LPAREN] = ACTIONS(19),
    [anon_sym_COLON] = ACTIONS(1632),
    [anon_sym_RPAREN] = ACTIONS(1634),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(107),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(107),
    [aux_sym_implicit_range_token1] = ACTIONS(1636),
    [anon_sym_DASH] = ACTIONS(1638),
    [aux_sym_intrinsic_type_token1] = ACTIONS(107),
    [aux_sym_intrinsic_type_token3] = ACTIONS(107),
    [aux_sym_intrinsic_type_token4] = ACTIONS(579),
    [anon_sym_STAR] = ACTIONS(1640),
    [aux_sym_type_qualifier_token13] = ACTIONS(107),
    [aux_sym_type_qualifier_token14] = ACTIONS(107),
    [aux_sym_stop_statement_token1] = ACTIONS(107),
    [aux_sym_stop_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token2] = ACTIONS(107),
    [aux_sym_keyword_statement_token3] = ACTIONS(107),
    [aux_sym_data_statement_token1] = ACTIONS(107),
    [aux_sym__inline_if_statement_token1] = ACTIONS(107),
    [aux_sym_end_if_statement_token1] = ACTIONS(107),
    [aux_sym_elseif_clause_token2] = ACTIONS(107),
    [aux_sym_select_case_statement_token1] = ACTIONS(1571),
    [aux_sym_block_construct_token1] = ACTIONS(107),
    [aux_sym_format_statement_token1] = ACTIONS(107),
    [aux_sym_inquire_statement_token1] = ACTIONS(107),
    [aux_sym_logical_expression_token5] = ACTIONS(1642),
    [anon_sym_PLUS] = ACTIONS(1638),
    [anon_sym_DOT] = ACTIONS(143),
    [sym_assumed_rank] = ACTIONS(1644),
    [anon_sym_LPAREN_SLASH] = ACTIONS(145),
    [anon_sym_LBRACK] = ACTIONS(147),
    [sym_boolean_literal] = ACTIONS(149),
    [aux_sym_null_literal_token1] = ACTIONS(151),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(107),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(155),
    [sym__float_literal] = ACTIONS(155),
    [sym__boz_literal] = ACTIONS(155),
    [sym_string_literal] = ACTIONS(149),
  },
  [443] = {
    [sym_argument_list] = STATE(455),
    [sym_preproc_file_line] = STATE(443),
    [aux_sym_end_program_statement_token1] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1646),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1436),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1436),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1431),
    [aux_sym_type_qualifier_token14] = ACTIONS(1431),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1431),
    [aux_sym_stop_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token3] = ACTIONS(1431),
    [aux_sym_data_statement_token1] = ACTIONS(1431),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1431),
    [aux_sym_end_if_statement_token1] = ACTIONS(1431),
    [aux_sym_elseif_clause_token2] = ACTIONS(1431),
    [aux_sym_select_case_statement_token1] = ACTIONS(1431),
    [aux_sym_block_construct_token1] = ACTIONS(1431),
    [aux_sym_format_statement_token1] = ACTIONS(1431),
    [aux_sym_inquire_statement_token1] = ACTIONS(1431),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_PLUS] = ACTIONS(1436),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1431),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1431),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
  },
  [444] = {
    [sym_preproc_file_line] = STATE(444),
    [aux_sym_end_program_statement_token1] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1648),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1436),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1431),
    [aux_sym_type_qualifier_token14] = ACTIONS(1431),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1431),
    [aux_sym_stop_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token3] = ACTIONS(1431),
    [aux_sym_data_statement_token1] = ACTIONS(1431),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1431),
    [aux_sym_end_if_statement_token1] = ACTIONS(1431),
    [aux_sym_elseif_clause_token2] = ACTIONS(1431),
    [aux_sym_select_case_statement_token1] = ACTIONS(1431),
    [aux_sym_block_construct_token1] = ACTIONS(1431),
    [aux_sym_format_statement_token1] = ACTIONS(1431),
    [aux_sym_inquire_statement_token1] = ACTIONS(1431),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_PLUS] = ACTIONS(1436),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1431),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1431),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
    [sym__end_of_statement] = ACTIONS(1650),
  },
  [445] = {
    [sym_preproc_file_line] = STATE(445),
    [aux_sym_end_program_statement_token1] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1652),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1436),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1431),
    [aux_sym_type_qualifier_token14] = ACTIONS(1431),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1431),
    [aux_sym_stop_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token3] = ACTIONS(1431),
    [aux_sym_data_statement_token1] = ACTIONS(1431),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1431),
    [aux_sym_end_if_statement_token1] = ACTIONS(1431),
    [aux_sym_elseif_clause_token2] = ACTIONS(1431),
    [aux_sym_select_case_statement_token1] = ACTIONS(1431),
    [aux_sym_block_construct_token1] = ACTIONS(1431),
    [aux_sym_format_statement_token1] = ACTIONS(1431),
    [aux_sym_inquire_statement_token1] = ACTIONS(1431),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_PLUS] = ACTIONS(1436),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1431),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1431),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
    [sym__end_of_statement] = ACTIONS(1654),
  },
  [446] = {
    [sym_preproc_file_line] = STATE(446),
    [aux_sym_end_program_statement_token1] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1431),
    [aux_sym_interface_statement_token1] = ACTIONS(1656),
    [aux_sym_defined_io_procedure_token1] = ACTIONS(1431),
    [aux_sym_defined_io_procedure_token2] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token1] = ACTIONS(1431),
    [aux_sym_procedure_attributes_token3] = ACTIONS(1431),
    [aux_sym_use_statement_token2] = ACTIONS(1431),
    [anon_sym_COLON_COLON] = ACTIONS(1436),
    [aux_sym_implicit_statement_token4] = ACTIONS(1431),
    [aux_sym_save_statement_token1] = ACTIONS(1431),
    [aux_sym_private_statement_token1] = ACTIONS(1431),
    [aux_sym_public_statement_token1] = ACTIONS(1431),
    [anon_sym_DASH] = ACTIONS(1436),
    [aux_sym_derived_type_definition_token1] = ACTIONS(1431),
    [aux_sym_abstract_specifier_token1] = ACTIONS(1431),
    [aux_sym_procedure_attribute_token6] = ACTIONS(1431),
    [aux_sym_variable_attributes_token1] = ACTIONS(1431),
    [aux_sym_variable_attributes_token2] = ACTIONS(1431),
    [aux_sym_variable_attributes_token3] = ACTIONS(1431),
    [aux_sym_variable_attributes_token4] = ACTIONS(1431),
    [aux_sym_variable_attributes_token5] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token1] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token3] = ACTIONS(1431),
    [aux_sym_intrinsic_type_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token1] = ACTIONS(1431),
    [aux_sym_type_qualifier_token2] = ACTIONS(1431),
    [aux_sym_type_qualifier_token3] = ACTIONS(1431),
    [aux_sym_type_qualifier_token4] = ACTIONS(1431),
    [aux_sym_type_qualifier_token5] = ACTIONS(1431),
    [aux_sym_type_qualifier_token9] = ACTIONS(1431),
    [aux_sym_type_qualifier_token10] = ACTIONS(1431),
    [aux_sym_type_qualifier_token11] = ACTIONS(1431),
    [aux_sym_type_qualifier_token12] = ACTIONS(1431),
    [aux_sym_type_qualifier_token13] = ACTIONS(1431),
    [aux_sym_type_qualifier_token14] = ACTIONS(1431),
    [aux_sym_type_qualifier_token15] = ACTIONS(1431),
    [aux_sym_stop_statement_token1] = ACTIONS(1431),
    [aux_sym_stop_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token2] = ACTIONS(1431),
    [aux_sym_keyword_statement_token3] = ACTIONS(1431),
    [aux_sym_data_statement_token1] = ACTIONS(1431),
    [aux_sym__inline_if_statement_token1] = ACTIONS(1431),
    [aux_sym_end_if_statement_token1] = ACTIONS(1431),
    [aux_sym_elseif_clause_token2] = ACTIONS(1431),
    [aux_sym_select_case_statement_token1] = ACTIONS(1431),
    [aux_sym_block_construct_token1] = ACTIONS(1431),
    [aux_sym_format_statement_token1] = ACTIONS(1431),
    [aux_sym_inquire_statement_token1] = ACTIONS(1431),
    [aux_sym_logical_expression_token5] = ACTIONS(1436),
    [anon_sym_PLUS] = ACTIONS(1436),
    [anon_sym_DOT] = ACTIONS(1431),
    [anon_sym_LPAREN_SLASH] = ACTIONS(1436),
    [anon_sym_LBRACK] = ACTIONS(1436),
    [sym_boolean_literal] = ACTIONS(1436),
    [aux_sym_null_literal_token1] = ACTIONS(1431),
    [anon_sym_POUND] = ACTIONS(3),
    [aux_sym_identifier_token1] = ACTIONS(1431),
    [sym_comment] = ACTIONS(5),
    [sym__line_continuation] = ACTIONS(5),
    [sym__integer_literal] = ACTIONS(1436),
    [sym__float_literal] = ACTIONS(1436),
    [sym__boz_literal] = ACTIONS(1436),
    [sym_string_literal] = ACTIONS(1436),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(447), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1353), 52,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [75] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(448), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1587), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1583), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [150] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(449), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1660), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1658), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [225] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(450), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1580), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1575), 3,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(1283), 5,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1578), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      aux_sym_logical_expression_token5,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1285), 18,
      anon_sym_EQ_GT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_STAR_STAR,
    ACTIONS(1573), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [308] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(451), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1664), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1662), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [383] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(452), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1668), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1666), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [458] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(453), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1672), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1670), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [533] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1674), 1,
      anon_sym_LPAREN,
    STATE(454), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1436), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1431), 48,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [610] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(455), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1678), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1676), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [685] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(456), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1436), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1431), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [760] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(457), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1682), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1680), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [835] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(458), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 52,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [910] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(459), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1686), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1684), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [985] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(460), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1690), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1688), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [1060] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(461), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 50,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_end_select_statement_token1,
      aux_sym_type_statement_token2,
      aux_sym_type_statement_token3,
      aux_sym__class_default_token1,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [1135] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(462), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 50,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_end_select_statement_token1,
      aux_sym_type_statement_token2,
      aux_sym_type_statement_token3,
      aux_sym__class_default_token1,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [1210] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(463), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 52,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [1285] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(464), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1281), 52,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [1360] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(465), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1700), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1698), 49,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [1435] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1702), 1,
      anon_sym_RPAREN,
    ACTIONS(1704), 1,
      sym_assumed_rank,
    STATE(466), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(887), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2229), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [1553] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1706), 1,
      anon_sym_RPAREN,
    ACTIONS(1708), 1,
      sym_assumed_rank,
    STATE(467), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(896), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2135), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [1671] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1702), 1,
      anon_sym_RPAREN,
    ACTIONS(1704), 1,
      sym_assumed_rank,
    STATE(468), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(913), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2229), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [1789] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1710), 1,
      anon_sym_RPAREN,
    ACTIONS(1712), 1,
      sym_assumed_rank,
    STATE(469), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(916), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2140), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [1907] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1714), 1,
      anon_sym_RPAREN,
    ACTIONS(1716), 1,
      sym_assumed_rank,
    STATE(470), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(895), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1971), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2025] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1706), 1,
      anon_sym_RPAREN,
    ACTIONS(1708), 1,
      sym_assumed_rank,
    STATE(471), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(889), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2135), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2143] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1634), 1,
      anon_sym_RPAREN,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1644), 1,
      sym_assumed_rank,
    STATE(472), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(892), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2153), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2261] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1718), 1,
      anon_sym_RPAREN,
    ACTIONS(1720), 1,
      sym_assumed_rank,
    STATE(473), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(890), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2143), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2379] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1634), 1,
      anon_sym_RPAREN,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1644), 1,
      sym_assumed_rank,
    STATE(474), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(910), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2153), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2497] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1722), 1,
      sym_assumed_rank,
    STATE(475), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(949), 1,
      sym__expression,
    STATE(1126), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2346), 3,
      sym_keyword_argument,
      sym_extent_specifier,
      sym_assumed_size,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2612] = 30,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1003), 1,
      sym__integer_literal,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1724), 1,
      anon_sym_STAR,
    ACTIONS(1726), 1,
      sym_string_literal,
    STATE(476), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1129), 1,
      sym_identifier,
    STATE(1132), 1,
      sym__expression,
    STATE(1497), 1,
      sym_statement_label,
    STATE(2136), 1,
      sym_keyword_argument,
    STATE(2282), 1,
      sym_format_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 2,
      sym__float_literal,
      sym__boz_literal,
    STATE(2342), 2,
      sym_statement_label_reference,
      sym__io_expressions,
    STATE(1200), 5,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
    STATE(808), 8,
      sym_logical_expression,
      sym_relational_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2735] = 28,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1728), 1,
      anon_sym_COMMA,
    ACTIONS(1730), 1,
      anon_sym_COLON_COLON,
    STATE(477), 1,
      sym_preproc_file_line,
    STATE(478), 1,
      aux_sym_variable_declaration_repeat1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2671), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2854] = 28,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1728), 1,
      anon_sym_COMMA,
    ACTIONS(1732), 1,
      anon_sym_COLON_COLON,
    STATE(478), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(778), 1,
      aux_sym_variable_declaration_repeat1,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2720), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [2973] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1734), 1,
      anon_sym_COLON,
    STATE(479), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(951), 1,
      sym__expression,
    STATE(1946), 1,
      sym_assumed_size,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1736), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [3085] = 28,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1534), 1,
      sym__integer_literal,
    ACTIONS(1536), 1,
      sym_string_literal,
    ACTIONS(1738), 1,
      anon_sym_LPAREN,
    ACTIONS(1740), 1,
      anon_sym_STAR,
    STATE(480), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1135), 1,
      sym__expression,
    STATE(2250), 1,
      sym_format_identifier,
    STATE(2318), 1,
      sym_statement_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 2,
      sym__float_literal,
      sym__boz_literal,
    STATE(2316), 2,
      sym_statement_label_reference,
      sym__io_expressions,
    STATE(1180), 6,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
      sym_identifier,
    STATE(808), 8,
      sym_logical_expression,
      sym_relational_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [3203] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(481), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 47,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym_end_where_statement_token1,
      aux_sym_elsewhere_clause_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3275] = 27,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1591), 1,
      anon_sym_COMMA,
    ACTIONS(1593), 1,
      anon_sym_COLON_COLON,
    STATE(482), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2393), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [3391] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1742), 1,
      anon_sym_COLON,
    STATE(483), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(938), 1,
      sym__expression,
    STATE(2097), 1,
      sym_assumed_size,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1744), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [3503] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(484), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 47,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym_end_where_statement_token1,
      aux_sym_elsewhere_clause_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3575] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(485), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_end_select_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3646] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(486), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_end_function_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3717] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(487), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token2,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_end_select_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3788] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(488), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token2,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_end_select_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3859] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(489), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [3930] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(490), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_end_select_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [4001] = 30,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1724), 1,
      anon_sym_STAR,
    ACTIONS(1726), 1,
      sym_string_literal,
    STATE(491), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(953), 1,
      sym__expression,
    STATE(1130), 1,
      sym_identifier,
    STATE(1174), 1,
      sym_unary_expression,
    STATE(1213), 1,
      sym_number_literal,
    STATE(2224), 1,
      sym_keyword_argument,
    STATE(2256), 1,
      sym_unit_identifier,
    STATE(2327), 1,
      sym__io_expressions,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1200), 5,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
    STATE(808), 6,
      sym_logical_expression,
      sym_relational_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [4122] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1640), 1,
      anon_sym_STAR,
    ACTIONS(1746), 1,
      anon_sym_COLON,
    STATE(492), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1035), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(2296), 2,
      sym_assumed_size,
      sym_assumed_shape,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [4231] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(493), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_end_function_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [4302] = 29,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1724), 1,
      anon_sym_STAR,
    ACTIONS(1726), 1,
      sym_string_literal,
    STATE(494), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1129), 1,
      sym_identifier,
    STATE(1132), 1,
      sym__expression,
    STATE(1168), 1,
      sym_number_literal,
    STATE(2224), 1,
      sym_keyword_argument,
    STATE(2256), 1,
      sym_unit_identifier,
    STATE(2327), 1,
      sym__io_expressions,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1200), 5,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
    STATE(808), 7,
      sym_logical_expression,
      sym_relational_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [4421] = 29,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1724), 1,
      anon_sym_STAR,
    ACTIONS(1726), 1,
      sym_string_literal,
    STATE(495), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1129), 1,
      sym_identifier,
    STATE(1132), 1,
      sym__expression,
    STATE(1168), 1,
      sym_number_literal,
    STATE(2014), 1,
      sym_keyword_argument,
    STATE(2319), 1,
      sym_unit_identifier,
    STATE(2327), 1,
      sym__io_expressions,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1200), 5,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
    STATE(808), 7,
      sym_logical_expression,
      sym_relational_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [4540] = 28,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1536), 1,
      sym_string_literal,
    ACTIONS(1740), 1,
      anon_sym_STAR,
    ACTIONS(1748), 1,
      anon_sym_LPAREN,
    STATE(496), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1135), 1,
      sym__expression,
    STATE(1285), 1,
      sym_number_literal,
    STATE(2516), 1,
      sym__io_expressions,
    STATE(2523), 1,
      sym__file_position_spec,
    STATE(2526), 1,
      sym_unit_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1180), 6,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
      sym_identifier,
    STATE(808), 7,
      sym_logical_expression,
      sym_relational_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [4657] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(497), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 46,
      aux_sym_end_program_statement_token1,
      aux_sym_end_program_statement_token2,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [4728] = 30,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(149), 1,
      sym_boolean_literal,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1724), 1,
      anon_sym_STAR,
    ACTIONS(1750), 1,
      sym_string_literal,
    STATE(498), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(953), 1,
      sym__expression,
    STATE(1151), 1,
      sym_identifier,
    STATE(1169), 1,
      sym_number_literal,
    STATE(1174), 1,
      sym_unary_expression,
    STATE(2007), 1,
      sym_keyword_argument,
    STATE(2327), 1,
      sym__io_expressions,
    STATE(2746), 1,
      sym_unit_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1202), 5,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
    STATE(808), 6,
      sym_logical_expression,
      sym_relational_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [4849] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(499), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [4920] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(500), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 46,
      aux_sym_end_program_statement_token1,
      aux_sym_end_program_statement_token2,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [4991] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(501), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5062] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(502), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 46,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5133] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(503), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_end_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5203] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    STATE(504), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2906), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [5313] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1754), 1,
      anon_sym_LPAREN,
    ACTIONS(1758), 1,
      anon_sym_STAR,
    STATE(505), 1,
      sym_preproc_file_line,
    STATE(695), 1,
      sym_size,
    STATE(700), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1756), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1752), 39,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5391] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    STATE(506), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2822), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [5501] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(507), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_end_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5571] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(508), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_end_block_construct_statement_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5641] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(509), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5711] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(510), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym_end_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5781] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(511), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1762), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1760), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5851] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(512), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym_end_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5921] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(513), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1766), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1764), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [5991] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(514), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1698), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1700), 47,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [6061] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(515), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1688), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1690), 47,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [6131] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(516), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1658), 8,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1660), 47,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      anon_sym_DASH,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [6201] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(517), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1770), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1768), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [6271] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(518), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(909), 1,
      sym__expression,
    STATE(2048), 1,
      sym_extent_specifier,
    STATE(2447), 1,
      sym_case_value_range_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [6379] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(519), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_end_forall_statement_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [6449] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(520), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1774), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1772), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [6519] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1778), 1,
      anon_sym_COMMA,
    ACTIONS(1782), 1,
      sym__end_of_statement,
    STATE(521), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(978), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2548), 1,
      sym_output_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [6627] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(522), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1786), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1784), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [6697] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(523), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_end_forall_statement_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [6767] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(524), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(909), 1,
      sym__expression,
    STATE(2048), 1,
      sym_extent_specifier,
    STATE(2353), 1,
      sym_case_value_range_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [6875] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    STATE(525), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2720), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [6985] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(526), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_end_block_construct_statement_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [7055] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(527), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [7125] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    STATE(528), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(995), 1,
      sym_identifier,
    STATE(1023), 1,
      sym_call_expression,
    STATE(1025), 1,
      sym__expression,
    STATE(2359), 1,
      sym__declaration_targets,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1956), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [7235] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(529), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1770), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1768), 45,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym__block_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [7305] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1632), 1,
      anon_sym_COLON,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(530), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(964), 1,
      sym__expression,
    STATE(2277), 1,
      sym_extent_specifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [7410] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(531), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1694), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1692), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [7479] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1788), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1790), 1,
      sym__end_of_statement,
    STATE(532), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2356), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [7564] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1792), 1,
      sym__end_of_statement,
    STATE(533), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(978), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2856), 1,
      sym_output_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [7669] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1528), 1,
      aux_sym_null_literal_token1,
    STATE(534), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1010), 1,
      sym_identifier,
    STATE(1025), 1,
      sym__expression,
    STATE(1079), 1,
      sym_call_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(2298), 2,
      sym_assignment_statement,
      sym_pointer_association_statement,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [7776] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1794), 1,
      sym__end_of_statement,
    STATE(535), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(978), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2739), 1,
      sym_output_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [7881] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1796), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1798), 1,
      sym__end_of_statement,
    STATE(536), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2532), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [7966] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1800), 1,
      sym__end_of_statement,
    STATE(537), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(978), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2737), 1,
      sym_output_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [8071] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1802), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1804), 1,
      sym__end_of_statement,
    STATE(538), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2514), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8156] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(539), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(915), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1806), 2,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [8259] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1808), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1810), 1,
      sym__end_of_statement,
    STATE(540), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2511), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8344] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1812), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1814), 1,
      sym__end_of_statement,
    STATE(541), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2781), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8429] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1816), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1818), 1,
      sym__end_of_statement,
    STATE(542), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2370), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8514] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1820), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1822), 1,
      sym__end_of_statement,
    STATE(543), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2437), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8599] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1824), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1826), 1,
      sym__end_of_statement,
    STATE(544), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2440), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8684] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1828), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1830), 1,
      sym__end_of_statement,
    STATE(545), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2374), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8769] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(546), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1696), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(710), 44,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_subroutine_call_token1,
      aux_sym_keyword_statement_token1,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_keyword_statement_token4,
      aux_sym_keyword_statement_token6,
      aux_sym_keyword_statement_token7,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
      aux_sym_format_statement_token1,
      aux_sym_print_statement_token1,
      aux_sym_open_statement_token1,
      aux_sym_close_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_file_position_statement_token1,
      aux_sym_file_position_statement_token2,
      aux_sym_file_position_statement_token3,
      aux_sym_file_position_statement_token4,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [8838] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1832), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1834), 1,
      sym__end_of_statement,
    STATE(547), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2844), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [8923] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1836), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1838), 1,
      sym__end_of_statement,
    STATE(548), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2583), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9008] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1840), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1842), 1,
      sym__end_of_statement,
    STATE(549), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2586), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9093] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1844), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1846), 1,
      sym__end_of_statement,
    STATE(550), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2604), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9178] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1848), 1,
      anon_sym_COMMA,
    ACTIONS(1850), 1,
      sym__end_of_statement,
    STATE(551), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(1000), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [9283] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1852), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1854), 1,
      sym__end_of_statement,
    STATE(552), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2636), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9368] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1856), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1858), 1,
      sym__end_of_statement,
    STATE(553), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2459), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9453] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1860), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1862), 1,
      sym__end_of_statement,
    STATE(554), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2653), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9538] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1864), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1866), 1,
      sym__end_of_statement,
    STATE(555), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2656), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9623] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1868), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1870), 1,
      sym__end_of_statement,
    STATE(556), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2674), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9708] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1872), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1874), 1,
      sym__end_of_statement,
    STATE(557), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2718), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9793] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1876), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1878), 1,
      sym__end_of_statement,
    STATE(558), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2721), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [9878] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1880), 1,
      sym__end_of_statement,
    STATE(559), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(972), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2541), 1,
      sym_input_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [9983] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1882), 1,
      aux_sym_program_statement_token1,
    ACTIONS(1884), 1,
      sym__end_of_statement,
    STATE(560), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2749), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [10068] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1886), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1888), 1,
      sym__end_of_statement,
    STATE(561), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2756), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [10153] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1890), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1892), 1,
      sym__end_of_statement,
    STATE(562), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2759), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [10238] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(563), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(2202), 1,
      sym_data_set,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1158), 4,
      sym_derived_type_member_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_identifier,
    STATE(808), 10,
      sym_parenthesized_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [10342] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(564), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1285), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [10410] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(565), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1099), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1097), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [10478] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(566), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1119), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1117), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [10546] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(567), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(897), 1,
      sym__expression,
    STATE(1204), 1,
      sym_identifier,
    STATE(2817), 1,
      sym_loop_control_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [10650] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(568), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1086), 1,
      sym__expression,
    STATE(1182), 1,
      sym_identifier,
    STATE(2246), 1,
      sym_concurrent_control,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [10754] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(569), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1147), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1145), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [10822] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(570), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1894), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1896), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [10890] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(571), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(897), 1,
      sym__expression,
    STATE(1204), 1,
      sym_identifier,
    STATE(2436), 1,
      sym_loop_control_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [10994] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(572), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(978), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2733), 1,
      sym_output_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [11096] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(573), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(897), 1,
      sym__expression,
    STATE(1204), 1,
      sym_identifier,
    STATE(2361), 1,
      sym_loop_control_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [11200] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(574), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(1286), 1,
      sym_identifier,
    STATE(2307), 1,
      sym_triplet_spec,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1275), 2,
      sym_logical_expression,
      sym_relational_expression,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [11306] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(575), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1199), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1197), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [11374] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(576), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(1286), 1,
      sym_identifier,
    STATE(2307), 1,
      sym_triplet_spec,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1267), 2,
      sym_logical_expression,
      sym_relational_expression,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [11480] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(577), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1900), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1898), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [11548] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(578), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1904), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1902), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [11616] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(579), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1104), 1,
      sym__expression,
    STATE(1182), 1,
      sym_identifier,
    STATE(2246), 1,
      sym_concurrent_control,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [11720] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(580), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1307), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1309), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [11788] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(581), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1311), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1313), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [11856] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(582), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1906), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1908), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [11924] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(583), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1247), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1245), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [11992] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(584), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1323), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1325), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12060] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(585), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1287), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1289), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12128] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(586), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1910), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1912), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12196] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1914), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(1928), 1,
      aux_sym_intrinsic_type_token4,
    STATE(587), 1,
      sym_preproc_file_line,
    STATE(1024), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    ACTIONS(1925), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(1177), 2,
      sym__generic_procedure,
      sym__name,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 15,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1917), 23,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [12284] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(588), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1295), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1297), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12352] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(589), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1299), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1301), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12420] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(590), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1285), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12488] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(1931), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1939), 1,
      aux_sym_intrinsic_type_token4,
    STATE(591), 1,
      sym_preproc_file_line,
    STATE(1024), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    ACTIONS(1936), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(1206), 2,
      sym__generic_procedure,
      sym__name,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 15,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1934), 23,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [12576] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(592), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1303), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1305), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12644] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(593), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1319), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1321), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12712] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(594), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1275), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1277), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12780] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(595), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1327), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1329), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12848] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(596), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1331), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1333), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [12916] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(597), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(1286), 1,
      sym_identifier,
    STATE(2307), 1,
      sym_triplet_spec,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1317), 2,
      sym_logical_expression,
      sym_relational_expression,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [13022] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1942), 1,
      anon_sym_COMMA,
    STATE(598), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(1039), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [13124] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(599), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1359), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1361), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [13192] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(600), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1355), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1357), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [13260] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(601), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1012), 1,
      sym__expression,
    STATE(2709), 1,
      sym_pointer_association_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [13362] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(602), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(972), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    STATE(2751), 1,
      sym_input_item_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [13464] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(603), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1946), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1944), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [13532] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(604), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1578), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1573), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [13600] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(605), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1047), 1,
      sym__expression,
    STATE(1182), 1,
      sym_identifier,
    STATE(2246), 1,
      sym_concurrent_control,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [13704] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(606), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1335), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1337), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [13772] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(607), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1339), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1341), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [13840] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(608), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1343), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1345), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [13908] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(609), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1315), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1317), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [13976] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(610), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1347), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1349), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14044] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(611), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1355), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1357), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14112] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(612), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1359), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1361), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14180] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1948), 1,
      anon_sym_COMMA,
    STATE(613), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(1066), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [14282] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(614), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1124), 1,
      sym__expression,
    STATE(1182), 1,
      sym_identifier,
    STATE(2246), 1,
      sym_concurrent_control,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [14386] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(615), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1259), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1257), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14454] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1950), 1,
      sym__end_of_statement,
    STATE(616), 1,
      sym_preproc_file_line,
    STATE(2529), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1285), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [14536] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(617), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1315), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1317), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14604] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(618), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1343), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1345), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14672] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(619), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1339), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1341), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14740] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(620), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1335), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1337), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14808] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(621), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1331), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1333), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [14876] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(622), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(1286), 1,
      sym_identifier,
    STATE(2307), 1,
      sym_triplet_spec,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1226), 2,
      sym_logical_expression,
      sym_relational_expression,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [14982] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(623), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1119), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1117), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15050] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(624), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1327), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1329), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15118] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(625), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1147), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1145), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15186] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    ACTIONS(1952), 1,
      anon_sym_COMMA,
    STATE(626), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(1123), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [15288] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(627), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1281), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15356] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(628), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1319), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1321), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15424] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(629), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1353), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15492] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(630), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1293), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15560] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(631), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1311), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1313), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15628] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(632), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1299), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1301), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15696] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(633), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1295), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1297), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15764] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(634), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(897), 1,
      sym__expression,
    STATE(1204), 1,
      sym_identifier,
    STATE(2607), 1,
      sym_loop_control_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 13,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [15868] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(635), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1347), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1349), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [15936] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(636), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1281), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16004] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(637), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1247), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1245), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16072] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(638), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1353), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16140] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(639), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1099), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1097), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16208] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(640), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1287), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1289), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16276] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(641), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1259), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1257), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16344] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(642), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1323), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1325), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16412] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(643), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1293), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16480] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(644), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1199), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1197), 51,
      sym__integer_literal,
      aux_sym_end_module_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16548] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(645), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1307), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1309), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16616] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(646), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1275), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1277), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16684] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(647), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1303), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1305), 51,
      sym__integer_literal,
      aux_sym_end_submodule_statement_token1,
      aux_sym_interface_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_procedure_attributes_token3,
      aux_sym_contains_statement_token1,
      aux_sym_use_statement_token1,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_namelist_statement_token1,
      aux_sym_common_statement_token1,
      aux_sym_import_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token3,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token5,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
      aux_sym_equivalence_statement_token1,
      aux_sym_include_statement_token1,
      aux_sym_data_statement_token1,
      aux_sym_enum_statement_token1,
  [16752] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(648), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(2276), 1,
      sym_data_set,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1158), 4,
      sym_derived_type_member_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_identifier,
    STATE(808), 10,
      sym_parenthesized_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [16856] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(649), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(953), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1174), 3,
      sym_unary_expression,
      sym_number_literal,
      sym_identifier,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [16957] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(650), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(845), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17056] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(651), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1097), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17155] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(652), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1032), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17254] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(653), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1957), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1954), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [17321] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(654), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1030), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17420] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(655), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(982), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17519] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(656), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(977), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17618] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(657), 1,
      sym_preproc_file_line,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(976), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17717] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(658), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(944), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1163), 3,
      sym_unary_expression,
      sym_number_literal,
      sym_identifier,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17818] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(659), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1090), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [17917] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(992), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    STATE(660), 2,
      sym_user_defined_operator,
      sym_preproc_file_line,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18014] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(661), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(975), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18113] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(662), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(841), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18212] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(663), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(842), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18311] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(664), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(840), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18410] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(665), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1076), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18509] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(666), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1088), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18608] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(667), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(838), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18707] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(668), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(837), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18806] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(669), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(843), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [18905] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(670), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(836), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19004] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(671), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(844), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19103] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(672), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1026), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19202] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(673), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(898), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19301] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(674), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(900), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19400] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(675), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(901), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19499] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(676), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(902), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19598] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(677), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(903), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19697] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(678), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(904), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19796] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(679), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(905), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19895] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(680), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(906), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [19994] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(681), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(907), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20093] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(682), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1212), 2,
      sym_call_expression,
      sym_identifier,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20194] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(683), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(846), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20293] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(684), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1974), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1972), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [20360] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(685), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1978), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1976), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [20427] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(686), 1,
      sym_preproc_file_line,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(1101), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20526] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(687), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(985), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20625] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(688), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(931), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20724] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(950), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    STATE(689), 2,
      sym_user_defined_operator,
      sym_preproc_file_line,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20821] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(690), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(959), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1174), 3,
      sym_unary_expression,
      sym_number_literal,
      sym_identifier,
    STATE(808), 11,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [20922] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(691), 1,
      sym_preproc_file_line,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(918), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21021] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(692), 1,
      sym_preproc_file_line,
    STATE(1038), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21120] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(693), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(897), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21219] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(694), 1,
      sym_preproc_file_line,
    STATE(952), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21318] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(695), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1982), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1980), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [21385] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(696), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1986), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1984), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [21452] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(697), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1990), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1988), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [21519] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(698), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1027), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21618] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(699), 1,
      sym_preproc_file_line,
    STATE(1123), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21717] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(700), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1994), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1992), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [21784] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(919), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    STATE(701), 2,
      sym_user_defined_operator,
      sym_preproc_file_line,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21881] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(702), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1996), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1296), 6,
      sym_unary_expression,
      sym_call_expression,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    STATE(808), 8,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [21982] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(703), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1281), 4,
      sym_derived_type_member_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_identifier,
    STATE(808), 10,
      sym_parenthesized_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22083] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(704), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1326), 2,
      sym_call_expression,
      sym_identifier,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22184] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(705), 1,
      sym_preproc_file_line,
    STATE(1105), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22283] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(706), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2000), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1998), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [22350] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(707), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1002), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22449] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(708), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2004), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2002), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [22516] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(709), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1073), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22615] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(710), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(987), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22714] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(711), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1015), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22813] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(712), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(1131), 2,
      sym_call_expression,
      sym_identifier,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 12,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [22914] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(713), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1082), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23013] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(714), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1102), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23112] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(715), 1,
      sym_preproc_file_line,
    STATE(920), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23211] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(716), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(947), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23310] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(717), 1,
      sym_preproc_file_line,
    STATE(1077), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23409] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(718), 1,
      sym_preproc_file_line,
    STATE(960), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23508] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(719), 1,
      sym_preproc_file_line,
    STATE(962), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23607] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(720), 1,
      sym_preproc_file_line,
    STATE(1004), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23706] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(721), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(983), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23805] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(722), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(961), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [23904] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(723), 1,
      sym_preproc_file_line,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1031), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24003] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(724), 1,
      sym_preproc_file_line,
    STATE(967), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24102] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(839), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    STATE(725), 2,
      sym_user_defined_operator,
      sym_preproc_file_line,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24199] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(726), 1,
      sym_preproc_file_line,
    STATE(968), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24298] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(727), 1,
      sym_preproc_file_line,
    STATE(970), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24397] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(728), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1003), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24496] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(729), 1,
      sym_preproc_file_line,
    STATE(971), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24595] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(730), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1690), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1688), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [24662] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(731), 1,
      sym_preproc_file_line,
    STATE(973), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24761] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(732), 1,
      sym_preproc_file_line,
    STATE(974), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24860] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(733), 1,
      sym_preproc_file_line,
    STATE(1016), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [24959] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(734), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1119), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25058] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(735), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1054), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25157] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1970), 1,
      aux_sym_logical_expression_token5,
    STATE(701), 1,
      sym_user_defined_operator,
    STATE(736), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(891), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1964), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1962), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25256] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(737), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1700), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1698), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [25323] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(738), 1,
      sym_preproc_file_line,
    STATE(1007), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25422] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(739), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(1189), 1,
      sym_number_literal,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2006), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1190), 5,
      sym_unary_expression,
      sym_call_expression,
      sym_complex_literal,
      sym_null_literal,
      sym_identifier,
    STATE(808), 8,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25525] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(740), 1,
      sym_preproc_file_line,
    STATE(1122), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25624] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(741), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(926), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25723] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(742), 1,
      sym_preproc_file_line,
    STATE(1017), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [25822] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2008), 1,
      sym__end_of_statement,
    STATE(743), 1,
      sym_preproc_file_line,
    STATE(2605), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(1285), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [25901] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(744), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1094), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26000] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(745), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1660), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1658), 40,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [26067] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(746), 1,
      sym_preproc_file_line,
    STATE(945), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26166] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(747), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(924), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26265] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1546), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(1551), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(1553), 1,
      anon_sym_LBRACK,
    ACTIONS(1557), 1,
      aux_sym_null_literal_token1,
    ACTIONS(1776), 1,
      anon_sym_LPAREN,
    STATE(689), 1,
      sym_user_defined_operator,
    STATE(748), 1,
      sym_preproc_file_line,
    STATE(1039), 1,
      sym__expression,
    STATE(1156), 1,
      sym__array_constructor_legacy,
    STATE(1157), 1,
      sym__array_constructor_f2003,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1555), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1780), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1559), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1155), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26364] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(749), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1107), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26463] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(750), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    STATE(1244), 1,
      sym_number_literal,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2010), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1240), 5,
      sym_unary_expression,
      sym_call_expression,
      sym_complex_literal,
      sym_null_literal,
      sym_identifier,
    STATE(808), 8,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26566] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(141), 1,
      aux_sym_logical_expression_token5,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(725), 1,
      sym_user_defined_operator,
    STATE(751), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(1132), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(55), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2012), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(1175), 6,
      sym_unary_expression,
      sym_call_expression,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    STATE(808), 8,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26667] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(19), 1,
      anon_sym_LPAREN,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(145), 1,
      anon_sym_LPAREN_SLASH,
    ACTIONS(147), 1,
      anon_sym_LBRACK,
    ACTIONS(151), 1,
      aux_sym_null_literal_token1,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1642), 1,
      aux_sym_logical_expression_token5,
    STATE(660), 1,
      sym_user_defined_operator,
    STATE(752), 1,
      sym_preproc_file_line,
    STATE(803), 1,
      sym__array_constructor_legacy,
    STATE(804), 1,
      sym__array_constructor_f2003,
    STATE(928), 1,
      sym__expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(149), 2,
      sym_string_literal,
      sym_boolean_literal,
    ACTIONS(1638), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(808), 14,
      sym_parenthesized_expression,
      sym_derived_type_member_expression,
      sym_logical_expression,
      sym_relational_expression,
      sym_concatenation_expression,
      sym_math_expression,
      sym_unary_expression,
      sym_call_expression,
      sym_implied_do_loop_expression,
      sym_array_literal,
      sym_complex_literal,
      sym_number_literal,
      sym_null_literal,
      sym_identifier,
    ACTIONS(107), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_identifier_token1,
  [26766] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(753), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1770), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
    ACTIONS(1768), 27,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      anon_sym_SLASH,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      anon_sym_STAR,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [26831] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2016), 1,
      anon_sym_COMMA,
    ACTIONS(2018), 1,
      anon_sym_COLON_COLON,
    STATE(754), 1,
      sym_preproc_file_line,
    STATE(1058), 1,
      sym_identifier,
    STATE(1167), 1,
      sym__import_names,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
    ACTIONS(2014), 27,
      aux_sym_end_program_statement_token1,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [26905] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2020), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2024), 1,
      aux_sym_end_interface_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    STATE(583), 1,
      sym_end_interface_statement,
    STATE(755), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27004] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2048), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2050), 1,
      aux_sym_end_interface_statement_token1,
    STATE(365), 1,
      sym_end_interface_statement,
    STATE(756), 1,
      sym_preproc_file_line,
    STATE(765), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27103] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2052), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2054), 1,
      aux_sym_end_interface_statement_token1,
    STATE(302), 1,
      sym_end_interface_statement,
    STATE(757), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27202] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2020), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2024), 1,
      aux_sym_end_interface_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    STATE(575), 1,
      sym_end_interface_statement,
    STATE(755), 1,
      aux_sym_interface_repeat1,
    STATE(758), 1,
      sym_preproc_file_line,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27301] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2052), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2054), 1,
      aux_sym_end_interface_statement_token1,
    STATE(389), 1,
      sym_end_interface_statement,
    STATE(757), 1,
      aux_sym_interface_repeat1,
    STATE(759), 1,
      sym_preproc_file_line,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27400] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2056), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2058), 1,
      aux_sym_end_interface_statement_token1,
    STATE(359), 1,
      sym_end_interface_statement,
    STATE(760), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27499] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2060), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2062), 1,
      aux_sym_end_interface_statement_token1,
    STATE(296), 1,
      sym_end_interface_statement,
    STATE(761), 1,
      sym_preproc_file_line,
    STATE(762), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27598] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2060), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2062), 1,
      aux_sym_end_interface_statement_token1,
    STATE(367), 1,
      sym_end_interface_statement,
    STATE(762), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27697] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2064), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2066), 1,
      aux_sym_end_interface_statement_token1,
    STATE(644), 1,
      sym_end_interface_statement,
    STATE(763), 1,
      sym_preproc_file_line,
    STATE(767), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27796] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2068), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2070), 1,
      aux_sym_end_interface_statement_token1,
    STATE(265), 1,
      sym_end_interface_statement,
    STATE(764), 1,
      sym_preproc_file_line,
    STATE(768), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27895] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2048), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2050), 1,
      aux_sym_end_interface_statement_token1,
    STATE(392), 1,
      sym_end_interface_statement,
    STATE(765), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [27994] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2072), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2074), 1,
      aux_sym_end_interface_statement_token1,
    STATE(406), 1,
      sym_end_interface_statement,
    STATE(766), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [28093] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2064), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2066), 1,
      aux_sym_end_interface_statement_token1,
    STATE(637), 1,
      sym_end_interface_statement,
    STATE(767), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [28192] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2068), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2070), 1,
      aux_sym_end_interface_statement_token1,
    STATE(284), 1,
      sym_end_interface_statement,
    STATE(768), 1,
      sym_preproc_file_line,
    STATE(771), 1,
      aux_sym_interface_repeat1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [28291] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2056), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2058), 1,
      aux_sym_end_interface_statement_token1,
    STATE(353), 1,
      sym_end_interface_statement,
    STATE(760), 1,
      aux_sym_interface_repeat1,
    STATE(769), 1,
      sym_preproc_file_line,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [28390] = 26,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2022), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2036), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2072), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2074), 1,
      aux_sym_end_interface_statement_token1,
    STATE(408), 1,
      sym_end_interface_statement,
    STATE(766), 1,
      aux_sym_interface_repeat1,
    STATE(770), 1,
      sym_preproc_file_line,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [28489] = 24,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2076), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2078), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2081), 1,
      aux_sym_end_interface_statement_token1,
    ACTIONS(2083), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2089), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2092), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2095), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2098), 1,
      aux_sym_import_statement_token1,
    ACTIONS(2104), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2107), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2110), 1,
      aux_sym_derived_type_token1,
    STATE(855), 1,
      sym__procedure_kind,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1376), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2778), 1,
      sym_subroutine_statement,
    STATE(2779), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(771), 2,
      sym_preproc_file_line,
      aux_sym_interface_repeat1,
    ACTIONS(2113), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1160), 4,
      sym_subroutine,
      sym_function,
      sym_import_statement,
      sym_procedure_statement,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2086), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
    ACTIONS(2101), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [28583] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2116), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2125), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2128), 1,
      aux_sym_select_case_statement_token1,
    STATE(586), 1,
      sym__name,
    STATE(635), 1,
      sym_identifier,
    STATE(772), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2122), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    ACTIONS(2118), 16,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    ACTIONS(2120), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28650] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2130), 1,
      anon_sym_LPAREN,
    ACTIONS(2132), 1,
      anon_sym_STAR,
    STATE(773), 1,
      sym_preproc_file_line,
    STATE(832), 1,
      sym_size,
    STATE(834), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1756), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1752), 23,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28712] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2136), 1,
      anon_sym_COMMA,
    STATE(774), 1,
      sym_preproc_file_line,
    STATE(789), 1,
      aux_sym_procedure_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2139), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2134), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28768] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(775), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1946), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1944), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28820] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2143), 1,
      anon_sym_COMMA,
    STATE(776), 1,
      sym_preproc_file_line,
    STATE(781), 1,
      aux_sym_procedure_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2146), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2141), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28876] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2150), 1,
      anon_sym_COMMA,
    STATE(777), 1,
      sym_preproc_file_line,
    STATE(782), 1,
      aux_sym_procedure_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2153), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2148), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28932] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2157), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(778), 2,
      sym_preproc_file_line,
      aux_sym_variable_declaration_repeat1,
    ACTIONS(2160), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2155), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [28986] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(779), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1768), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1770), 30,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym__block_if_statement_token1,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [29038] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2164), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(780), 2,
      sym_preproc_file_line,
      aux_sym_procedure_statement_repeat1,
    ACTIONS(2167), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2162), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29092] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2171), 1,
      anon_sym_COMMA,
    STATE(780), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(781), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2174), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2169), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29148] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2178), 1,
      anon_sym_COMMA,
    STATE(780), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(782), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2181), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2176), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29204] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2185), 1,
      anon_sym_LPAREN,
    STATE(792), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(783), 2,
      sym_preproc_file_line,
      aux_sym_call_expression_repeat1,
    ACTIONS(2183), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2188), 27,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [29260] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(783), 1,
      aux_sym_call_expression_repeat1,
    STATE(784), 1,
      sym_preproc_file_line,
    STATE(792), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2190), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2192), 28,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [29316] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2130), 1,
      anon_sym_LPAREN,
    STATE(785), 1,
      sym_preproc_file_line,
    STATE(811), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1436), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1431), 23,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29372] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(786), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1900), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1898), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29424] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(787), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1904), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1902), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29476] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(788), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1578), 13,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      anon_sym_STAR,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1573), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29528] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2196), 1,
      anon_sym_COMMA,
    STATE(780), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(789), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2199), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2194), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29584] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2201), 1,
      anon_sym_LPAREN,
    STATE(790), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 28,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [29637] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(791), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2205), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2203), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29688] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(792), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2207), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2209), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [29739] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(793), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2213), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2211), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29790] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(794), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1690), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1688), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29841] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(795), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1686), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1684), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29892] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(796), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1565), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1567), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [29943] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(797), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1974), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1972), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [29994] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(798), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2217), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2215), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30045] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2219), 1,
      anon_sym_COMMA,
    STATE(799), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2153), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2148), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30098] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(800), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2224), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2222), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30149] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(801), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2160), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2155), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30200] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(802), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2226), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2228), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30251] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(803), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2230), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2232), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30302] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(804), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2230), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2232), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30353] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(805), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1990), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1988), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30404] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(806), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2234), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2236), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30455] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(807), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2238), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2240), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30506] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(808), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30557] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(809), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2246), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2248), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30608] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(810), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2252), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2250), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30659] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(811), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1678), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1676), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30710] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(812), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2254), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2256), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30761] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(813), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1986), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1984), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30812] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(814), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2167), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2162), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30863] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(815), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2258), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2260), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [30914] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(816), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2264), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2262), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [30965] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(817), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1978), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1976), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31016] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(818), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1660), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1658), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31067] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2268), 1,
      anon_sym_LPAREN,
    ACTIONS(2271), 1,
      anon_sym_COMMA,
    STATE(819), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2274), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2266), 23,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31122] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(820), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2004), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2002), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31173] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(821), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2276), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2278), 29,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [31224] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(822), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2282), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_SLASH_RPAREN,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      sym_boolean_literal,
    ACTIONS(2280), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31275] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(823), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2000), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1998), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31326] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(824), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1668), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1666), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31377] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(825), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2286), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2284), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31428] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(826), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1436), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1431), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31479] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2290), 1,
      anon_sym_COMMA,
    STATE(827), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2293), 11,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2288), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31532] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(828), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2297), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2295), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31583] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(829), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2286), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2284), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31634] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(830), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1957), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1954), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31685] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(831), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1664), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1662), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31736] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(832), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1982), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1980), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31787] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(833), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1672), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1670), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31838] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(834), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1994), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1992), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31889] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(835), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1700), 12,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(1698), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [31940] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(836), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 23,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32002] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(837), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2311), 4,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2309), 20,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32070] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(838), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2321), 4,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2319), 20,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32138] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(839), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2325), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2323), 25,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32196] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(840), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 24,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32256] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(841), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2301), 6,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [32334] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(842), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2339), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
    ACTIONS(2337), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32406] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(843), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2343), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2341), 8,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [32482] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(844), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2343), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
    ACTIONS(2341), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32554] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(845), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2303), 4,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 23,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [32618] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(846), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2343), 2,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2341), 9,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_GT_GT_GT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [32692] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2347), 1,
      anon_sym_COLON_COLON,
    STATE(847), 1,
      sym_preproc_file_line,
    STATE(877), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(1037), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2576), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [32767] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2353), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(848), 1,
      sym_preproc_file_line,
    STATE(1187), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(1783), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2382), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [32842] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2361), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2365), 1,
      aux_sym_end_function_statement_token1,
    STATE(849), 1,
      sym_preproc_file_line,
    STATE(873), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2861), 1,
      sym_module_procedure_statement,
    STATE(2876), 1,
      sym_subroutine_statement,
    STATE(2877), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1620), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [32927] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2367), 1,
      anon_sym_COLON_COLON,
    STATE(850), 1,
      sym_preproc_file_line,
    STATE(1083), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(1187), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(2394), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33002] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2371), 1,
      anon_sym_STAR,
    STATE(851), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2624), 1,
      sym__intrinsic_type,
    STATE(2860), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2623), 2,
      sym__type_name,
      sym_unlimited_polymorphic,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33071] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2361), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2365), 1,
      aux_sym_end_submodule_statement_token1,
    STATE(852), 1,
      sym_preproc_file_line,
    STATE(869), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2837), 1,
      sym_module_procedure_statement,
    STATE(2853), 1,
      sym_subroutine_statement,
    STATE(2854), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1621), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [33156] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(853), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2375), 10,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
      sym_string_literal,
      anon_sym_DASH,
      aux_sym_logical_expression_token5,
      anon_sym_PLUS,
      anon_sym_LPAREN_SLASH,
      anon_sym_LBRACK,
      sym_boolean_literal,
    ACTIONS(2373), 24,
      aux_sym_end_program_statement_token1,
      anon_sym_LPAREN,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      anon_sym_DOT,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33205] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2377), 1,
      anon_sym_COLON_COLON,
    STATE(850), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(854), 1,
      sym_preproc_file_line,
    STATE(1060), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2813), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33280] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2379), 1,
      anon_sym_LPAREN,
    ACTIONS(2381), 1,
      anon_sym_COMMA,
    ACTIONS(2383), 1,
      anon_sym_COLON_COLON,
    STATE(855), 1,
      sym_preproc_file_line,
    STATE(1115), 1,
      sym_identifier,
    STATE(1116), 1,
      sym_method_name,
    STATE(2826), 1,
      sym__generic_procedure,
    STATE(2827), 1,
      sym_binding_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33355] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2385), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2387), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2390), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2393), 1,
      aux_sym_end_subroutine_statement_token1,
    ACTIONS(2395), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2398), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2401), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2407), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2410), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2413), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2849), 1,
      sym_module_procedure_statement,
    STATE(2865), 1,
      sym_subroutine_statement,
    STATE(2866), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(856), 2,
      sym_preproc_file_line,
      aux_sym_internal_procedures_repeat1,
    STATE(1568), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2416), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2404), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [33438] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2419), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2421), 1,
      aux_sym_end_function_statement_token1,
    STATE(849), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(857), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2861), 1,
      sym_module_procedure_statement,
    STATE(2876), 1,
      sym_subroutine_statement,
    STATE(2877), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1620), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [33523] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2385), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2387), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2390), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2393), 1,
      aux_sym_end_module_statement_token1,
    ACTIONS(2395), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2398), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2401), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2407), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2410), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2413), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2800), 1,
      sym_module_procedure_statement,
    STATE(2841), 1,
      sym_subroutine_statement,
    STATE(2842), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(858), 2,
      sym_preproc_file_line,
      aux_sym_internal_procedures_repeat1,
    STATE(1639), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2416), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2404), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [33606] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2371), 1,
      anon_sym_STAR,
    STATE(859), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2404), 1,
      sym__intrinsic_type,
    STATE(2860), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2403), 2,
      sym__type_name,
      sym_unlimited_polymorphic,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33675] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2385), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2387), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2390), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2393), 1,
      aux_sym_end_module_procedure_statement_token1,
    ACTIONS(2395), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2398), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2401), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2407), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2410), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2413), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2872), 1,
      sym_module_procedure_statement,
    STATE(2886), 1,
      sym_subroutine_statement,
    STATE(2887), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(860), 2,
      sym_preproc_file_line,
      aux_sym_internal_procedures_repeat1,
    STATE(1648), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2416), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2404), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [33758] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2371), 1,
      anon_sym_STAR,
    STATE(861), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2592), 1,
      sym__intrinsic_type,
    STATE(2860), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2590), 2,
      sym__type_name,
      sym_unlimited_polymorphic,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33827] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2423), 1,
      anon_sym_COMMA,
    ACTIONS(2425), 1,
      aux_sym_while_statement_token1,
    ACTIONS(2427), 1,
      aux_sym_concurrent_header_token1,
    ACTIONS(2429), 1,
      sym__integer_literal,
    ACTIONS(2431), 1,
      sym__end_of_statement,
    STATE(862), 1,
      sym_preproc_file_line,
    STATE(1359), 1,
      sym_statement_label_reference,
    STATE(1497), 1,
      sym_statement_label,
    STATE(1832), 1,
      sym_concurrent_header,
    STATE(2563), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2567), 3,
      sym_while_statement,
      sym_concurrent_statement,
      sym_loop_control_expression,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33900] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2371), 1,
      anon_sym_STAR,
    STATE(863), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2406), 1,
      sym__intrinsic_type,
    STATE(2860), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2405), 2,
      sym__type_name,
      sym_unlimited_polymorphic,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [33969] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2433), 1,
      anon_sym_LPAREN,
    ACTIONS(2435), 1,
      anon_sym_COMMA,
    ACTIONS(2437), 1,
      anon_sym_COLON_COLON,
    STATE(864), 1,
      sym_preproc_file_line,
    STATE(1758), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2461), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [34044] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2419), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2421), 1,
      aux_sym_end_module_statement_token1,
    STATE(865), 1,
      sym_preproc_file_line,
    STATE(881), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2800), 1,
      sym_module_procedure_statement,
    STATE(2841), 1,
      sym_subroutine_statement,
    STATE(2842), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1639), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34129] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2361), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2365), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(856), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(866), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2849), 1,
      sym_module_procedure_statement,
    STATE(2865), 1,
      sym_subroutine_statement,
    STATE(2866), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1568), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34214] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2439), 1,
      anon_sym_COLON_COLON,
    STATE(848), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(867), 1,
      sym_preproc_file_line,
    STATE(1773), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2381), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [34289] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2419), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2421), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(866), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(868), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2849), 1,
      sym_module_procedure_statement,
    STATE(2865), 1,
      sym_subroutine_statement,
    STATE(2866), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1568), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34374] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2385), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2387), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2390), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2393), 1,
      aux_sym_end_submodule_statement_token1,
    ACTIONS(2395), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2398), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2401), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2407), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2410), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2413), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2837), 1,
      sym_module_procedure_statement,
    STATE(2853), 1,
      sym_subroutine_statement,
    STATE(2854), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(869), 2,
      sym_preproc_file_line,
      aux_sym_internal_procedures_repeat1,
    STATE(1621), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2416), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2404), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34457] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2419), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2421), 1,
      aux_sym_end_submodule_statement_token1,
    STATE(852), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(870), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2837), 1,
      sym_module_procedure_statement,
    STATE(2853), 1,
      sym_subroutine_statement,
    STATE(2854), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1621), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34542] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2441), 1,
      anon_sym_COLON_COLON,
    STATE(871), 1,
      sym_preproc_file_line,
    STATE(1187), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(1769), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2373), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [34617] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2419), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2421), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(872), 1,
      sym_preproc_file_line,
    STATE(882), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2872), 1,
      sym_module_procedure_statement,
    STATE(2886), 1,
      sym_subroutine_statement,
    STATE(2887), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1648), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34702] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2385), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2387), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2390), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2393), 1,
      aux_sym_end_function_statement_token1,
    ACTIONS(2395), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2398), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2401), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2407), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2410), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2413), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2861), 1,
      sym_module_procedure_statement,
    STATE(2876), 1,
      sym_subroutine_statement,
    STATE(2877), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(873), 2,
      sym_preproc_file_line,
      aux_sym_internal_procedures_repeat1,
    STATE(1620), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2416), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2404), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [34785] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2443), 1,
      anon_sym_COLON_COLON,
    STATE(871), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(874), 1,
      sym_preproc_file_line,
    STATE(1767), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2364), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [34860] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2371), 1,
      anon_sym_STAR,
    STATE(875), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2493), 1,
      sym__intrinsic_type,
    STATE(2860), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2497), 2,
      sym__type_name,
      sym_unlimited_polymorphic,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [34929] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2371), 1,
      anon_sym_STAR,
    STATE(876), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2481), 1,
      sym__intrinsic_type,
    STATE(2860), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2482), 2,
      sym__type_name,
      sym_unlimited_polymorphic,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [34998] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2345), 1,
      anon_sym_COMMA,
    ACTIONS(2445), 1,
      anon_sym_COLON_COLON,
    STATE(877), 1,
      sym_preproc_file_line,
    STATE(1075), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(1187), 1,
      aux_sym_procedure_statement_repeat1,
    STATE(2677), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [35073] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2385), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2387), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2390), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2393), 1,
      aux_sym_end_program_statement_token2,
    ACTIONS(2395), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2398), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2401), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2407), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2410), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2413), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2823), 1,
      sym_subroutine_statement,
    STATE(2824), 1,
      sym_function_statement,
    STATE(2857), 1,
      sym_module_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(878), 2,
      sym_preproc_file_line,
      aux_sym_internal_procedures_repeat1,
    STATE(1567), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2416), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2404), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [35156] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2361), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2365), 1,
      aux_sym_end_program_statement_token2,
    STATE(878), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(879), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2823), 1,
      sym_subroutine_statement,
    STATE(2824), 1,
      sym_function_statement,
    STATE(2857), 1,
      sym_module_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1567), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [35241] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2419), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2421), 1,
      aux_sym_end_program_statement_token2,
    STATE(879), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(880), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2823), 1,
      sym_subroutine_statement,
    STATE(2824), 1,
      sym_function_statement,
    STATE(2857), 1,
      sym_module_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1567), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [35326] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2361), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2365), 1,
      aux_sym_end_module_statement_token1,
    STATE(858), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(881), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2800), 1,
      sym_module_procedure_statement,
    STATE(2841), 1,
      sym_subroutine_statement,
    STATE(2842), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1639), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [35411] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2026), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(2030), 1,
      aux_sym_function_statement_token1,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(2361), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2363), 1,
      aux_sym_module_statement_token1,
    ACTIONS(2365), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(860), 1,
      aux_sym_internal_procedures_repeat1,
    STATE(882), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1368), 1,
      aux_sym__callable_interface_qualifers,
    STATE(2872), 1,
      sym_module_procedure_statement,
    STATE(2886), 1,
      sym_subroutine_statement,
    STATE(2887), 1,
      sym_function_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1648), 3,
      sym_subroutine,
      sym_module_procedure,
      sym_function,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [35496] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2447), 1,
      anon_sym_COMMA,
    ACTIONS(2449), 1,
      anon_sym_COLON_COLON,
    STATE(883), 1,
      sym_preproc_file_line,
    STATE(1786), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2372), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [35568] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2451), 1,
      anon_sym_COMMA,
    ACTIONS(2453), 1,
      anon_sym_COLON_COLON,
    STATE(884), 1,
      sym_preproc_file_line,
    STATE(1080), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2395), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [35640] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(885), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1281), 26,
      sym__end_of_statement,
      anon_sym_LPAREN,
      aux_sym_language_binding_token1,
      anon_sym_COMMA,
      aux_sym_function_result_token1,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [35687] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2457), 1,
      anon_sym_DOT,
    STATE(749), 1,
      sym_user_defined_operator,
    STATE(886), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2455), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2810), 3,
      sym_unary_expression,
      sym_number_literal,
      sym_identifier,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [35748] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2463), 1,
      anon_sym_RPAREN,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(887), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2205), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [35829] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(888), 1,
      sym_preproc_file_line,
    STATE(894), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2190), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2192), 24,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [35880] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2485), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(889), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(1984), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [35961] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2487), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(890), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2099), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36042] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2325), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2495), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2497), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(891), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2499), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2501), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2323), 3,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
    ACTIONS(2503), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36119] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2509), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(892), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2199), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36200] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2511), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(893), 1,
      sym_preproc_file_line,
    STATE(1901), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2513), 2,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36279] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2515), 1,
      anon_sym_LPAREN,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(894), 2,
      sym_preproc_file_line,
      aux_sym_call_expression_repeat1,
    ACTIONS(2183), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2188), 23,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [36330] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2518), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(895), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2028), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36411] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2520), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(896), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(1984), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36492] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(897), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2522), 4,
      anon_sym_COMMA,
      anon_sym_GT_GT_GT,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36567] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2495), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2497), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(898), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 2,
      aux_sym_end_program_statement_token1,
      anon_sym_DOT,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2499), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2501), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2301), 3,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
    ACTIONS(2503), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36642] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2457), 1,
      anon_sym_DOT,
    STATE(749), 1,
      sym_user_defined_operator,
    STATE(899), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2455), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
    STATE(2434), 3,
      sym_unary_expression,
      sym_number_literal,
      sym_identifier,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [36703] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(900), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 6,
      aux_sym_end_program_statement_token1,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 21,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [36760] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(901), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2321), 4,
      aux_sym_end_program_statement_token1,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2319), 17,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36825] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(902), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2311), 4,
      aux_sym_end_program_statement_token1,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2309), 17,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36890] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2495), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2497), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(903), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2343), 2,
      aux_sym_end_program_statement_token1,
      anon_sym_DOT,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2501), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 5,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2503), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [36963] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(904), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2343), 2,
      aux_sym_end_program_statement_token1,
      anon_sym_DOT,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2501), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 7,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2503), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37032] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2497), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(905), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2343), 2,
      aux_sym_end_program_statement_token1,
      anon_sym_DOT,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2501), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 6,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2503), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37103] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(906), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2303), 4,
      aux_sym_end_program_statement_token1,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 20,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
  [37164] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(907), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 6,
      aux_sym_end_program_statement_token1,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 20,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
  [37223] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(908), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 26,
      sym__end_of_statement,
      anon_sym_LPAREN,
      aux_sym_language_binding_token1,
      anon_sym_COMMA,
      aux_sym_function_result_token1,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [37270] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2524), 1,
      anon_sym_RPAREN,
    ACTIONS(2526), 1,
      anon_sym_COMMA,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(909), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(1989), 1,
      aux_sym_case_value_range_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37351] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2528), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(910), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2199), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37432] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2530), 1,
      aux_sym_intrinsic_type_token5,
    ACTIONS(2532), 1,
      aux_sym_intrinsic_type_token7,
    STATE(911), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1353), 24,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [37483] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    STATE(912), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(2233), 1,
      sym_concurrent_control,
    STATE(2407), 1,
      sym_identifier,
    STATE(2554), 1,
      sym__intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [37548] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2534), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(913), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2205), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37629] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(914), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1353), 26,
      sym__end_of_statement,
      anon_sym_LPAREN,
      aux_sym_language_binding_token1,
      anon_sym_COMMA,
      aux_sym_function_result_token1,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [37676] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2511), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(915), 1,
      sym_preproc_file_line,
    STATE(1903), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2536), 2,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37755] = 22,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2538), 1,
      anon_sym_RPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(916), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(2105), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37836] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(917), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 26,
      sym__end_of_statement,
      anon_sym_LPAREN,
      aux_sym_language_binding_token1,
      anon_sym_COMMA,
      aux_sym_function_result_token1,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [37883] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    ACTIONS(2505), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2507), 1,
      anon_sym_STAR_STAR,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(918), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2339), 2,
      aux_sym_end_program_statement_token1,
      anon_sym_DOT,
    ACTIONS(2489), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2491), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2501), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2337), 7,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2503), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [37952] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2493), 1,
      anon_sym_PERCENT,
    STATE(673), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(919), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2325), 6,
      aux_sym_end_program_statement_token1,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2323), 22,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [38007] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(920), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 20,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
  [38065] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2552), 1,
      sym__end_of_statement,
    STATE(921), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2562), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38129] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2564), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(396), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(922), 1,
      sym_preproc_file_line,
    STATE(1065), 1,
      sym_public_statement,
    STATE(1207), 1,
      sym_private_statement,
    STATE(1300), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2146), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [38215] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(923), 1,
      sym_preproc_file_line,
    STATE(1083), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2394), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38281] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(924), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2321), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2319), 17,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [38345] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2568), 1,
      sym__end_of_statement,
    STATE(925), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2490), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38409] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(926), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 21,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [38465] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2570), 1,
      sym__end_of_statement,
    STATE(927), 1,
      sym_preproc_file_line,
    STATE(1918), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2112), 2,
      sym__generic_procedure,
      sym_use_alias,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38529] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2572), 1,
      anon_sym_COLON,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(928), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2574), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [38605] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2576), 1,
      sym__end_of_statement,
    STATE(929), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2743), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38669] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(930), 1,
      sym_preproc_file_line,
    STATE(1061), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2661), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38735] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2303), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(931), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2301), 3,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [38809] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2425), 1,
      aux_sym_while_statement_token1,
    ACTIONS(2427), 1,
      aux_sym_concurrent_header_token1,
    ACTIONS(2578), 1,
      anon_sym_COMMA,
    ACTIONS(2580), 1,
      sym__end_of_statement,
    STATE(932), 1,
      sym_preproc_file_line,
    STATE(1832), 1,
      sym_concurrent_header,
    STATE(2563), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2667), 3,
      sym_while_statement,
      sym_concurrent_statement,
      sym_loop_control_expression,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38873] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2582), 1,
      sym__end_of_statement,
    STATE(933), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2716), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [38937] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2584), 1,
      sym__end_of_statement,
    STATE(934), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2698), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39001] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2588), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(369), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(935), 1,
      sym_preproc_file_line,
    STATE(1108), 1,
      sym_public_statement,
    STATE(1203), 1,
      sym_private_statement,
    STATE(1298), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2055), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [39087] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    STATE(936), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2427), 2,
      sym__intrinsic_type,
      sym_identifier,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39147] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2592), 1,
      sym__end_of_statement,
    STATE(937), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2651), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39211] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1734), 1,
      anon_sym_COLON,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(938), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1736), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [39287] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(939), 1,
      sym_preproc_file_line,
    STATE(1115), 1,
      sym_identifier,
    STATE(1150), 1,
      sym_method_name,
    STATE(2812), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39353] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2596), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(355), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(940), 1,
      sym_preproc_file_line,
    STATE(1081), 1,
      sym_public_statement,
    STATE(1223), 1,
      sym_private_statement,
    STATE(1263), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2142), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [39439] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    STATE(941), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2428), 2,
      sym__intrinsic_type,
      sym_identifier,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39499] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2600), 1,
      sym__end_of_statement,
    STATE(942), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2413), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39563] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2602), 1,
      sym__end_of_statement,
    STATE(943), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2509), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39627] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2604), 1,
      anon_sym_RPAREN,
    ACTIONS(2606), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(944), 1,
      sym_preproc_file_line,
    STATE(2352), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [39705] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2612), 1,
      anon_sym_LT_LT_LT,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2626), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(945), 1,
      sym_preproc_file_line,
    STATE(1210), 1,
      sym_argument_list,
    STATE(2106), 1,
      sym_cuda_kernel_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [39783] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2628), 1,
      sym__end_of_statement,
    STATE(946), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2435), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [39847] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2339), 1,
      anon_sym_DOT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(947), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2337), 7,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [39915] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2630), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2632), 1,
      aux_sym_select_case_statement_token2,
    ACTIONS(2634), 1,
      aux_sym_select_rank_statement_token1,
    STATE(948), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1281), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [39967] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(949), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2636), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40043] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(950), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2325), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2323), 22,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [40097] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2638), 1,
      anon_sym_COLON,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(951), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2640), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40173] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2339), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(952), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2337), 7,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40241] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2604), 1,
      anon_sym_RPAREN,
    ACTIONS(2642), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(953), 1,
      sym_preproc_file_line,
    STATE(2265), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40319] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2646), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(585), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(954), 1,
      sym_preproc_file_line,
    STATE(1103), 1,
      sym_public_statement,
    STATE(1215), 1,
      sym_private_statement,
    STATE(1295), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2189), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [40405] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(955), 1,
      sym_preproc_file_line,
    STATE(1111), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2826), 1,
      sym__generic_procedure,
    STATE(2907), 1,
      sym_binding_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [40471] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(956), 1,
      sym_preproc_file_line,
    STATE(1075), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2677), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [40537] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(957), 1,
      sym_preproc_file_line,
    STATE(1114), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2826), 1,
      sym__generic_procedure,
    STATE(2904), 1,
      sym_binding_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [40603] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(958), 1,
      sym_preproc_file_line,
    STATE(1752), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2409), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [40669] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2642), 1,
      anon_sym_COMMA,
    ACTIONS(2650), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(959), 1,
      sym_preproc_file_line,
    STATE(2265), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40747] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(960), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2303), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 20,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
  [40807] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(961), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2311), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2309), 17,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40871] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2343), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(962), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 6,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [40941] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2654), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(300), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(963), 1,
      sym_preproc_file_line,
    STATE(1098), 1,
      sym_public_statement,
    STATE(1218), 1,
      sym_private_statement,
    STATE(1259), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1973), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [41027] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2461), 1,
      anon_sym_COLON,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(964), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2658), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41103] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    STATE(965), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2840), 2,
      sym__intrinsic_type,
      sym_identifier,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [41163] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1601), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1603), 1,
      aux_sym_intrinsic_type_token8,
    STATE(966), 1,
      sym_preproc_file_line,
    STATE(1338), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2369), 2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
    STATE(2836), 2,
      sym__intrinsic_type,
      sym_identifier,
    ACTIONS(1599), 5,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
    ACTIONS(107), 17,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [41223] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2343), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(967), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 7,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41291] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2343), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(968), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 5,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41363] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2660), 1,
      sym__end_of_statement,
    STATE(969), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2632), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [41427] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(970), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2311), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2309), 17,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41491] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(971), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2321), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2319), 17,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41555] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2662), 1,
      anon_sym_COMMA,
    ACTIONS(2664), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(972), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    STATE(2005), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41633] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(973), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 21,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [41689] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2303), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(974), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2301), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_LT_LT_LT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41763] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(975), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2303), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 20,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
  [41821] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(976), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2303), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2301), 20,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
  [41881] = 17,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2343), 1,
      anon_sym_DOT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(977), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 6,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [41951] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2662), 1,
      anon_sym_COMMA,
    ACTIONS(2666), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(978), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    STATE(2012), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [42029] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2668), 1,
      sym__end_of_statement,
    STATE(979), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2754), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42093] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2672), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(315), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(980), 1,
      sym_preproc_file_line,
    STATE(1040), 1,
      sym_public_statement,
    STATE(1176), 1,
      sym_private_statement,
    STATE(1245), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2234), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [42179] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(981), 1,
      sym_preproc_file_line,
    STATE(1789), 1,
      sym_identifier,
    STATE(1794), 1,
      sym_method_name,
    STATE(2365), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42245] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2343), 1,
      anon_sym_DOT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(982), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 7,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [42313] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2343), 1,
      anon_sym_DOT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(983), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2341), 5,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [42385] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(984), 1,
      sym_preproc_file_line,
    STATE(1769), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2373), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42451] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2676), 1,
      anon_sym_COLON,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(985), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2678), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [42527] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(986), 1,
      sym_preproc_file_line,
    STATE(1759), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2378), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42593] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2511), 1,
      anon_sym_COMMA,
    ACTIONS(2680), 1,
      anon_sym_GT_GT_GT,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(987), 1,
      sym_preproc_file_line,
    STATE(2011), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [42671] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(988), 1,
      sym_preproc_file_line,
    STATE(1763), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2379), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42737] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(989), 1,
      sym_preproc_file_line,
    STATE(1783), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2382), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42803] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(990), 1,
      sym_preproc_file_line,
    STATE(1761), 1,
      sym_method_name,
    STATE(1789), 1,
      sym_identifier,
    STATE(2383), 1,
      sym_binding_name,
    STATE(2826), 1,
      sym__generic_procedure,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2351), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42869] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2682), 1,
      sym__end_of_statement,
    STATE(991), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2791), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [42933] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(992), 1,
      sym_preproc_file_line,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2325), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2323), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [42987] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2684), 1,
      sym__end_of_statement,
    STATE(993), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2905), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [43051] = 25,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2688), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(640), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(994), 1,
      sym_preproc_file_line,
    STATE(1078), 1,
      sym_public_statement,
    STATE(1217), 1,
      sym_private_statement,
    STATE(1255), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1962), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [43137] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    ACTIONS(2694), 1,
      anon_sym_STAR,
    ACTIONS(2696), 1,
      sym__end_of_statement,
    STATE(995), 1,
      sym_preproc_file_line,
    STATE(1934), 1,
      sym_character_length,
    STATE(1935), 1,
      aux_sym__declaration_targets_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [43193] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2698), 1,
      sym__end_of_statement,
    STATE(996), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2464), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [43257] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2700), 1,
      sym__end_of_statement,
    STATE(997), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2581), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [43321] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1919), 1,
      aux_sym_assignment_token1,
    ACTIONS(1921), 1,
      aux_sym_operator_token1,
    STATE(998), 1,
      sym_preproc_file_line,
    STATE(1043), 1,
      sym_method_name,
    STATE(1115), 1,
      sym_identifier,
    STATE(2826), 1,
      sym__generic_procedure,
    STATE(2882), 1,
      sym_binding_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1923), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1145), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(107), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [43387] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    ACTIONS(2702), 1,
      sym__end_of_statement,
    STATE(999), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2412), 2,
      sym__generic_procedure,
      sym__name,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [43451] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2704), 1,
      anon_sym_COMMA,
    ACTIONS(2706), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1000), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [43526] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1001), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1688), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1690), 24,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [43571] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1002), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2708), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [43644] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2710), 1,
      anon_sym_RPAREN,
    ACTIONS(2712), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1003), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [43719] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2710), 1,
      sym__end_of_statement,
    ACTIONS(2714), 1,
      anon_sym_COMMA,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1004), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [43794] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2716), 1,
      aux_sym_intrinsic_type_token5,
    ACTIONS(2718), 1,
      aux_sym_intrinsic_type_token7,
    STATE(1005), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1353), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [43843] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2724), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2720), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    STATE(1006), 2,
      sym_preproc_file_line,
      aux_sym_parameters_repeat1,
    ACTIONS(2722), 26,
      aux_sym_module_statement_token1,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [43890] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1007), 1,
      sym_preproc_file_line,
    STATE(1009), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2522), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [43963] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2727), 1,
      aux_sym_intrinsic_type_token5,
    ACTIONS(2729), 1,
      aux_sym_intrinsic_type_token7,
    STATE(1008), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1353), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44012] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1009), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2207), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2209), 24,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44057] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2694), 1,
      anon_sym_STAR,
    STATE(1010), 1,
      sym_preproc_file_line,
    STATE(2311), 1,
      sym_character_length,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2731), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44108] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1011), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1658), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1660), 24,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44153] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2733), 1,
      anon_sym_RPAREN,
    ACTIONS(2735), 1,
      anon_sym_EQ_GT,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1012), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [44228] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2425), 1,
      aux_sym_while_statement_token1,
    ACTIONS(2427), 1,
      aux_sym_concurrent_header_token1,
    ACTIONS(2580), 1,
      sym__end_of_statement,
    STATE(1013), 1,
      sym_preproc_file_line,
    STATE(1832), 1,
      sym_concurrent_header,
    STATE(2563), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2667), 3,
      sym_while_statement,
      sym_concurrent_statement,
      sym_loop_control_expression,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [44289] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2425), 1,
      aux_sym_while_statement_token1,
    ACTIONS(2427), 1,
      aux_sym_concurrent_header_token1,
    ACTIONS(2737), 1,
      sym__end_of_statement,
    STATE(1014), 1,
      sym_preproc_file_line,
    STATE(1832), 1,
      sym_concurrent_header,
    STATE(2563), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2462), 3,
      sym_while_statement,
      sym_concurrent_statement,
      sym_loop_control_expression,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [44350] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1015), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2739), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [44423] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1016), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2741), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [44496] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1017), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2743), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [44569] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1018), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1353), 23,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44614] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1019), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1698), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1700), 24,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44659] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2745), 1,
      anon_sym_LPAREN,
    STATE(792), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1020), 2,
      sym_preproc_file_line,
      aux_sym_call_expression_repeat1,
    ACTIONS(2183), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2188), 22,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44708] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(1848), 1,
      anon_sym_COMMA,
    ACTIONS(1850), 1,
      sym__end_of_statement,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1021), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [44783] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    STATE(1022), 1,
      sym_preproc_file_line,
    STATE(2085), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2299), 2,
      sym__generic_procedure,
      sym_use_alias,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [44844] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    ACTIONS(2696), 1,
      sym__end_of_statement,
    STATE(1023), 1,
      sym_preproc_file_line,
    STATE(1935), 1,
      aux_sym__declaration_targets_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [44895] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1024), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1347), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1349), 28,
      aux_sym_module_statement_token1,
      anon_sym_COLON,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      anon_sym_SLASH,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [44940] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2748), 1,
      anon_sym_EQ,
    ACTIONS(2750), 1,
      anon_sym_EQ_GT,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1025), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45015] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1026), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2752), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45088] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1027), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2640), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45161] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(792), 1,
      sym_argument_list,
    STATE(1020), 1,
      aux_sym_call_expression_repeat1,
    STATE(1028), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2190), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2192), 23,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [45210] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1029), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 23,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [45255] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1030), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2754), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45328] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1031), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2756), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45401] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1032), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2758), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45474] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2760), 1,
      anon_sym_LPAREN,
    STATE(1033), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1285), 27,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [45521] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1034), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 23,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [45566] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1035), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2762), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45639] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1036), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 7,
      aux_sym_end_program_statement_token1,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1281), 23,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [45684] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1037), 1,
      sym_preproc_file_line,
    STATE(1074), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2764), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2766), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [45732] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2770), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1038), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45804] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2772), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1039), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [45876] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2774), 1,
      aux_sym_derived_type_definition_token1,
    STATE(321), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1040), 1,
      sym_preproc_file_line,
    STATE(1162), 1,
      sym_private_statement,
    STATE(1227), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2214), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [45956] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2776), 1,
      anon_sym_LPAREN,
    STATE(521), 1,
      sym__io_arguments,
    STATE(1041), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 21,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46004] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2201), 1,
      anon_sym_LPAREN,
    STATE(1042), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 22,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46050] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1043), 1,
      sym_preproc_file_line,
    STATE(1063), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2778), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2780), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46098] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1044), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2778), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2780), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46146] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1575), 1,
      anon_sym_STAR,
    ACTIONS(1578), 1,
      anon_sym_COLON_COLON,
    ACTIONS(1580), 1,
      anon_sym_LPAREN,
    STATE(1045), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 4,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 22,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [46196] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2782), 1,
      anon_sym_LPAREN,
    STATE(1046), 1,
      sym_preproc_file_line,
    STATE(2515), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 21,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46244] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2785), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1047), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [46316] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1048), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2787), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2789), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46364] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1048), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1049), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2791), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2793), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46412] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1050), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2791), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2793), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46460] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2795), 1,
      aux_sym_stop_statement_token2,
    STATE(1051), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46506] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2797), 1,
      anon_sym_LPAREN,
    STATE(196), 1,
      sym_parenthesized_expression,
    STATE(1052), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 21,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46554] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2799), 1,
      sym__end_of_statement,
    STATE(1053), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46600] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2801), 1,
      anon_sym_COLON,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(1054), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [46672] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1055), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2803), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2805), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46720] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1050), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1056), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2803), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2805), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46768] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2776), 1,
      anon_sym_LPAREN,
    STATE(535), 1,
      sym__io_arguments,
    STATE(1057), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 21,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [46816] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    STATE(1058), 1,
      sym_preproc_file_line,
    STATE(1069), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2807), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2809), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46864] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1059), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2813), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2815), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46912] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1060), 1,
      sym_preproc_file_line,
    STATE(1089), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2817), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2819), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [46960] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1055), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1061), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2821), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2823), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47008] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1062), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2821), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2823), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47056] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1063), 1,
      sym_preproc_file_line,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2817), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2819), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47104] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2825), 1,
      aux_sym__inline_if_statement_token1,
    STATE(1064), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [47150] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2827), 1,
      aux_sym_derived_type_definition_token1,
    STATE(398), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1065), 1,
      sym_preproc_file_line,
    STATE(1198), 1,
      sym_private_statement,
    STATE(1292), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2190), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [47230] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2829), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1066), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [47302] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2835), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2831), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    STATE(1067), 2,
      sym_preproc_file_line,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(2833), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47348] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    STATE(1068), 1,
      sym_preproc_file_line,
    STATE(1087), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2838), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2840), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47396] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    STATE(1006), 1,
      aux_sym_parameters_repeat1,
    STATE(1069), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2838), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2840), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47444] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1059), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1070), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2842), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2844), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47492] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2846), 1,
      aux_sym_associate_statement_token1,
    STATE(1071), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [47538] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1072), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2842), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2844), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47586] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2604), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1073), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [47658] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1074), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2848), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2850), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47706] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1062), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1075), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2848), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2850), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [47754] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2650), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1076), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [47826] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2852), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1077), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [47898] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2854), 1,
      aux_sym_derived_type_definition_token1,
    STATE(632), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1078), 1,
      sym_preproc_file_line,
    STATE(1219), 1,
      sym_private_statement,
    STATE(1242), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2016), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [47978] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1079), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2731), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(2242), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [48024] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1080), 1,
      sym_preproc_file_line,
    STATE(1113), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2856), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2858), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48072] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2860), 1,
      aux_sym_derived_type_definition_token1,
    STATE(364), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1081), 1,
      sym_preproc_file_line,
    STATE(1211), 1,
      sym_private_statement,
    STATE(1249), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2108), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [48152] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2741), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1082), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [48224] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1083), 1,
      sym_preproc_file_line,
    STATE(1112), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2862), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2864), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48272] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2866), 1,
      aux_sym__inline_where_statement_token1,
    STATE(1084), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [48318] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1085), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2720), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2722), 27,
      aux_sym_module_statement_token1,
      anon_sym_RPAREN,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48362] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2868), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1086), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [48434] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    STATE(1006), 1,
      aux_sym_parameters_repeat1,
    STATE(1087), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2870), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2872), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48482] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2874), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1088), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [48554] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1089), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2862), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2864), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48602] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2876), 1,
      anon_sym_COMMA,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1090), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [48674] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1091), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2878), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2880), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48722] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1072), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1092), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2878), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2880), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48770] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1093), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2764), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2766), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48818] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2459), 1,
      anon_sym_LPAREN,
    ACTIONS(2471), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2473), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2481), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2483), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2882), 1,
      anon_sym_COLON,
    STATE(688), 1,
      sym_user_defined_operator,
    STATE(792), 1,
      sym_argument_list,
    STATE(1028), 1,
      aux_sym_call_expression_repeat1,
    STATE(1094), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2467), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2469), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2475), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2477), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2479), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [48890] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    STATE(1095), 1,
      sym_preproc_file_line,
    STATE(1100), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2884), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2886), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [48938] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2888), 1,
      aux_sym_assignment_token1,
    ACTIONS(2890), 1,
      aux_sym_operator_token1,
    STATE(1096), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2892), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1459), 2,
      sym__generic_procedure,
      sym_identifier,
    STATE(1491), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [48996] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2894), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1097), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [49068] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2896), 1,
      aux_sym_derived_type_definition_token1,
    STATE(362), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1098), 1,
      sym_preproc_file_line,
    STATE(1205), 1,
      sym_private_statement,
    STATE(1312), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1941), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [49148] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    STATE(1099), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2159), 2,
      sym__generic_procedure,
      sym_identifier,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [49206] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    STATE(1006), 1,
      aux_sym_parameters_repeat1,
    STATE(1100), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2898), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2900), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [49254] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2902), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1101), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [49326] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2902), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1102), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [49398] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2904), 1,
      aux_sym_derived_type_definition_token1,
    STATE(589), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1103), 1,
      sym_preproc_file_line,
    STATE(1221), 1,
      sym_private_statement,
    STATE(1287), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2034), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [49478] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2906), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1104), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [49550] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2908), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1105), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [49622] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    STATE(1106), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2254), 2,
      sym__generic_procedure,
      sym_identifier,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [49680] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2323), 1,
      anon_sym_RPAREN,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1107), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [49752] = 23,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2910), 1,
      aux_sym_derived_type_definition_token1,
    STATE(304), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1108), 1,
      sym_preproc_file_line,
    STATE(1188), 1,
      sym_private_statement,
    STATE(1290), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2038), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [49832] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2888), 1,
      aux_sym_assignment_token1,
    ACTIONS(2890), 1,
      aux_sym_operator_token1,
    STATE(1109), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2892), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1386), 2,
      sym__generic_procedure,
      sym_identifier,
    STATE(1491), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [49890] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2912), 1,
      aux_sym__forall_control_expression_token1,
    STATE(1110), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [49936] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1091), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1111), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2914), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2916), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [49984] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1112), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2914), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2916), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [50032] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1067), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1113), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2918), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2920), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [50080] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1093), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1114), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2918), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2920), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [50128] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2926), 1,
      anon_sym_EQ_GT,
    STATE(1115), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2922), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2924), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [50174] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2768), 1,
      anon_sym_COMMA,
    STATE(1044), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1116), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2928), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2930), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [50222] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1117), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1569), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(1565), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1567), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50268] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2932), 1,
      aux_sym_block_construct_token1,
    STATE(1118), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50314] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2934), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1119), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [50386] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2888), 1,
      aux_sym_assignment_token1,
    ACTIONS(2890), 1,
      aux_sym_operator_token1,
    STATE(1120), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2892), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(1512), 2,
      sym__generic_procedure,
      sym_identifier,
    STATE(1491), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(2355), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [50444] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2936), 1,
      aux_sym_do_loop_statement_token1,
    STATE(1121), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1293), 22,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50490] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2938), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1122), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [50562] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2614), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2616), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2940), 1,
      sym__end_of_statement,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1123), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2618), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2620), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2622), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [50634] = 19,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2942), 1,
      anon_sym_RPAREN,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1124), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [50706] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2546), 1,
      aux_sym_assignment_token1,
    ACTIONS(2548), 1,
      aux_sym_operator_token1,
    STATE(1125), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2550), 2,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
    STATE(2160), 2,
      sym__generic_procedure,
      sym_identifier,
    STATE(2348), 3,
      sym_assignment,
      sym_operator,
      sym_defined_io_procedure,
    ACTIONS(1518), 17,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [50764] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2944), 1,
      anon_sym_EQ,
    STATE(1126), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 23,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50810] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1127), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1688), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1690), 23,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50853] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1128), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1658), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1660), 23,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50896] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2944), 1,
      anon_sym_EQ,
    STATE(1129), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2946), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50943] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2944), 1,
      anon_sym_EQ,
    STATE(1130), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2946), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [50990] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2948), 1,
      anon_sym_RPAREN,
    ACTIONS(2950), 1,
      anon_sym_COMMA,
    STATE(1131), 1,
      sym_preproc_file_line,
    STATE(2215), 1,
      aux_sym_equivalence_set_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51039] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(2305), 1,
      anon_sym_PERCENT,
    ACTIONS(2307), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2317), 1,
      anon_sym_SLASH_SLASH,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    STATE(662), 1,
      sym_user_defined_operator,
    STATE(784), 1,
      aux_sym_call_expression_repeat1,
    STATE(792), 1,
      sym_argument_list,
    STATE(1132), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2313), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2315), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [51108] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1133), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1698), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1700), 23,
      anon_sym_LPAREN,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51151] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1134), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2952), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2954), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [51194] = 18,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(143), 1,
      anon_sym_DOT,
    ACTIONS(2327), 1,
      aux_sym_logical_expression_token1,
    ACTIONS(2329), 1,
      aux_sym_logical_expression_token2,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(2542), 1,
      anon_sym_PERCENT,
    ACTIONS(2544), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2624), 1,
      anon_sym_SLASH_SLASH,
    STATE(732), 1,
      sym_user_defined_operator,
    STATE(888), 1,
      aux_sym_call_expression_repeat1,
    STATE(1009), 1,
      sym_argument_list,
    STATE(1135), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2331), 2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
    ACTIONS(2333), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(2608), 2,
      anon_sym_SLASH,
      anon_sym_STAR,
    ACTIONS(2610), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2335), 10,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
  [51263] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1136), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2956), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2958), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [51306] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1137), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2960), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2962), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [51349] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1138), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2964), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2966), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [51392] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1139), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2234), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2236), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51435] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1140), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2226), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2228), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51478] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2968), 1,
      anon_sym_LPAREN,
    STATE(1141), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 22,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51523] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1142), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2238), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2240), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51566] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1143), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2276), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2278), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51609] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1144), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2246), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2248), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51652] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1145), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2970), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2972), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [51695] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1146), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2254), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2256), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51738] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1147), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2258), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2260), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51781] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2974), 1,
      anon_sym_COLON,
    STATE(1148), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51826] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2976), 1,
      sym__end_of_statement,
    STATE(1149), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51871] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1150), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2831), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2833), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [51914] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2944), 1,
      anon_sym_EQ,
    ACTIONS(2946), 1,
      anon_sym_COMMA,
    STATE(1151), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [51961] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1152), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2922), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2924), 26,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      anon_sym_COMMA,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [52004] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2978), 1,
      anon_sym_LPAREN,
    STATE(1153), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 6,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1285), 21,
      anon_sym_COLON,
      anon_sym_EQ_GT,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52049] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1154), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1565), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1567), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52092] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1155), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52135] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1156), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2230), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2232), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52178] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1157), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2230), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2232), 23,
      sym__end_of_statement,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52221] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(2982), 1,
      anon_sym_SLASH,
    STATE(1158), 1,
      sym_preproc_file_line,
    STATE(1904), 1,
      aux_sym_data_set_repeat1,
    STATE(2297), 1,
      sym_data_value,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52272] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1159), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [52314] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1160), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2984), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2986), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [52356] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2988), 1,
      anon_sym_LPAREN,
    ACTIONS(2990), 1,
      anon_sym_COMMA,
    ACTIONS(2992), 1,
      anon_sym_COLON_COLON,
    STATE(1161), 1,
      sym_preproc_file_line,
    STATE(2668), 1,
      sym_identifier,
    STATE(2669), 1,
      sym__type_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [52412] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2994), 1,
      aux_sym_derived_type_definition_token1,
    STATE(333), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1162), 1,
      sym_preproc_file_line,
    STATE(1247), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2200), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [52486] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2996), 1,
      anon_sym_COMMA,
    STATE(1163), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52530] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1164), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1569), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(1565), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(1567), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52574] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(2990), 1,
      anon_sym_COMMA,
    ACTIONS(2992), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2999), 1,
      anon_sym_LPAREN,
    STATE(1165), 1,
      sym_preproc_file_line,
    STATE(2668), 1,
      sym_identifier,
    STATE(2669), 1,
      sym__type_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [52630] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3001), 1,
      anon_sym_COMMA,
    ACTIONS(3003), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3005), 1,
      sym__end_of_statement,
    STATE(1166), 1,
      sym_preproc_file_line,
    STATE(2151), 1,
      sym_identifier,
    STATE(2618), 1,
      sym__import_names,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [52686] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1167), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3007), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3009), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [52728] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1168), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3011), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52772] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3011), 1,
      anon_sym_COMMA,
    STATE(1169), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [52816] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3013), 1,
      anon_sym_LPAREN,
    STATE(1170), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2286), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
    ACTIONS(2284), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [52860] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1171), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3015), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3017), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [52902] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1172), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2838), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2840), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [52944] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3021), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(3023), 1,
      aux_sym_type_qualifier_token3,
    ACTIONS(3025), 1,
      aux_sym_type_qualifier_token5,
    STATE(801), 1,
      sym_type_qualifier,
    STATE(1173), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3019), 23,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
  [52992] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3027), 1,
      anon_sym_COMMA,
    STATE(1174), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [53036] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3030), 1,
      anon_sym_COMMA,
    ACTIONS(3032), 1,
      anon_sym_SLASH,
    STATE(1175), 1,
      sym_preproc_file_line,
    STATE(2156), 1,
      aux_sym_data_value_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [53084] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2774), 1,
      aux_sym_derived_type_definition_token1,
    STATE(321), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1176), 1,
      sym_preproc_file_line,
    STATE(1227), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2214), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [53158] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1177), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1934), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3035), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53200] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1178), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53242] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1179), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53284] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1180), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2946), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [53328] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3037), 1,
      anon_sym_LPAREN,
    ACTIONS(3039), 1,
      sym__end_of_statement,
    STATE(1181), 1,
      sym_preproc_file_line,
    STATE(1373), 1,
      sym_parenthesized_expression,
    STATE(2568), 1,
      sym_identifier,
    STATE(2655), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [53384] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3041), 1,
      anon_sym_EQ,
    STATE(1182), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [53428] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1183), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53470] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1184), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53512] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1185), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53554] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1186), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53596] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2167), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3043), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1187), 2,
      sym_preproc_file_line,
      aux_sym_procedure_statement_repeat1,
    ACTIONS(2162), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [53640] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(3046), 1,
      aux_sym_derived_type_definition_token1,
    STATE(325), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1188), 1,
      sym_preproc_file_line,
    STATE(1264), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2025), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [53714] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3030), 1,
      anon_sym_COMMA,
    ACTIONS(3048), 1,
      anon_sym_SLASH,
    ACTIONS(3051), 1,
      anon_sym_STAR,
    STATE(1189), 1,
      sym_preproc_file_line,
    STATE(2114), 1,
      aux_sym_data_value_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [53764] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3030), 1,
      anon_sym_COMMA,
    ACTIONS(3048), 1,
      anon_sym_SLASH,
    STATE(1190), 1,
      sym_preproc_file_line,
    STATE(2054), 1,
      aux_sym_data_value_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [53812] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1191), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53854] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1192), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53896] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1193), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53938] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1194), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [53980] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1195), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [54022] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1196), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [54064] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1197), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [54106] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(3053), 1,
      aux_sym_derived_type_definition_token1,
    STATE(403), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1198), 1,
      sym_preproc_file_line,
    STATE(1284), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2203), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [54180] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1199), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [54222] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1200), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2946), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [54266] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1201), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [54308] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2946), 1,
      anon_sym_COMMA,
    STATE(1202), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [54352] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2910), 1,
      aux_sym_derived_type_definition_token1,
    STATE(304), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1203), 1,
      sym_preproc_file_line,
    STATE(1290), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2038), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [54426] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3055), 1,
      anon_sym_EQ,
    STATE(1204), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 21,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [54470] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(3057), 1,
      aux_sym_derived_type_definition_token1,
    STATE(338), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1205), 1,
      sym_preproc_file_line,
    STATE(1237), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2051), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [54544] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1206), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3059), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3061), 25,
      aux_sym_module_statement_token1,
      aux_sym_end_interface_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_import_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [54586] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2827), 1,
      aux_sym_derived_type_definition_token1,
    STATE(398), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1207), 1,
      sym_preproc_file_line,
    STATE(1292), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2190), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [54660] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1208), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3065), 3,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
    ACTIONS(3063), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [54702] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3067), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3073), 1,
      anon_sym_SLASH,
    ACTIONS(3076), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3079), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3082), 1,
      sym__end_of_statement,
    STATE(1283), 1,
      sym_identifier,
    STATE(1455), 1,
      sym_variable_group,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1209), 2,
      sym_preproc_file_line,
      aux_sym_common_statement_repeat1,
    ACTIONS(3070), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [54756] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3084), 1,
      sym__end_of_statement,
    STATE(1210), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2207), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2209), 21,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_LT_LT_LT,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [54800] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(3086), 1,
      aux_sym_derived_type_definition_token1,
    STATE(374), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1211), 1,
      sym_preproc_file_line,
    STATE(1330), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2088), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [54874] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1212), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3088), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [54918] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1213), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3011), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [54962] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3037), 1,
      anon_sym_LPAREN,
    ACTIONS(3090), 1,
      sym__end_of_statement,
    STATE(1214), 1,
      sym_preproc_file_line,
    STATE(1351), 1,
      sym_parenthesized_expression,
    STATE(2477), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [55018] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2904), 1,
      aux_sym_derived_type_definition_token1,
    STATE(589), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1215), 1,
      sym_preproc_file_line,
    STATE(1287), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2034), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55092] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3021), 1,
      aux_sym_derived_type_definition_token1,
    ACTIONS(3023), 1,
      aux_sym_type_qualifier_token3,
    ACTIONS(3025), 1,
      aux_sym_type_qualifier_token5,
    STATE(477), 1,
      sym_type_qualifier,
    STATE(1216), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3019), 23,
      aux_sym_procedure_attributes_token3,
      aux_sym_use_statement_token2,
      aux_sym_implicit_statement_token4,
      aux_sym_save_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_abstract_specifier_token1,
      aux_sym_procedure_attribute_token6,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
      aux_sym_type_qualifier_token1,
      aux_sym_type_qualifier_token2,
      aux_sym_type_qualifier_token4,
      aux_sym_type_qualifier_token9,
      aux_sym_type_qualifier_token10,
      aux_sym_type_qualifier_token11,
      aux_sym_type_qualifier_token12,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_type_qualifier_token15,
  [55140] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2854), 1,
      aux_sym_derived_type_definition_token1,
    STATE(632), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1217), 1,
      sym_preproc_file_line,
    STATE(1242), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2016), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55214] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2896), 1,
      aux_sym_derived_type_definition_token1,
    STATE(362), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1218), 1,
      sym_preproc_file_line,
    STATE(1312), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1941), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55288] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(3092), 1,
      aux_sym_derived_type_definition_token1,
    STATE(618), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1219), 1,
      sym_preproc_file_line,
    STATE(1234), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2044), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55362] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3094), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3098), 1,
      anon_sym_SLASH,
    ACTIONS(3100), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3102), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3104), 1,
      sym__end_of_statement,
    STATE(1209), 1,
      aux_sym_common_statement_repeat1,
    STATE(1220), 1,
      sym_preproc_file_line,
    STATE(1283), 1,
      sym_identifier,
    STATE(1455), 1,
      sym_variable_group,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3096), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [55418] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(3106), 1,
      aux_sym_derived_type_definition_token1,
    STATE(608), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1221), 1,
      sym_preproc_file_line,
    STATE(1278), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1958), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55492] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1222), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3110), 3,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
    ACTIONS(3108), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [55534] = 21,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    ACTIONS(2860), 1,
      aux_sym_derived_type_definition_token1,
    STATE(364), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1223), 1,
      sym_preproc_file_line,
    STATE(1249), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2108), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55608] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(317), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1224), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1950), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55679] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1860), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1862), 1,
      sym__end_of_statement,
    STATE(1225), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2653), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [55732] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3112), 1,
      anon_sym_RPAREN,
    STATE(1226), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [55775] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(333), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1227), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2200), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55846] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1812), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1814), 1,
      sym__end_of_statement,
    STATE(1228), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2781), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [55899] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(596), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1229), 1,
      sym_preproc_file_line,
    STATE(1309), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2101), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [55970] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(596), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1230), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2101), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56041] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(333), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1231), 1,
      sym_preproc_file_line,
    STATE(1247), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2200), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56112] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3094), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3098), 1,
      anon_sym_SLASH,
    ACTIONS(3100), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3102), 1,
      aux_sym_select_case_statement_token1,
    STATE(1220), 1,
      aux_sym_common_statement_repeat1,
    STATE(1232), 1,
      sym_preproc_file_line,
    STATE(1283), 1,
      sym_identifier,
    STATE(1455), 1,
      sym_variable_group,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3096), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [56165] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(599), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1230), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1233), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2084), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56236] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(599), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1234), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2084), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56307] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1832), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1834), 1,
      sym__end_of_statement,
    STATE(1235), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2844), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [56360] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(343), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1236), 1,
      sym_preproc_file_line,
    STATE(1280), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2182), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56431] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(360), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1237), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2022), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56502] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1238), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2286), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
    ACTIONS(2284), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [56543] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1882), 1,
      aux_sym_program_statement_token1,
    ACTIONS(1884), 1,
      sym__end_of_statement,
    STATE(1239), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2749), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [56596] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3114), 1,
      anon_sym_COMMA,
    ACTIONS(3116), 1,
      anon_sym_SLASH,
    STATE(1240), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [56641] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3119), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3121), 1,
      sym__end_of_statement,
    STATE(1241), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2878), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [56694] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(618), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1242), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2044), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56765] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(618), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1234), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1243), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2044), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56836] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3114), 1,
      anon_sym_COMMA,
    ACTIONS(3116), 1,
      anon_sym_SLASH,
    ACTIONS(3123), 1,
      anon_sym_STAR,
    STATE(1244), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 3,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [56883] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(321), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1245), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2214), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [56954] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(360), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1224), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1246), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2022), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57025] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(343), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1247), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2182), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57096] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3125), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3127), 1,
      sym__end_of_statement,
    STATE(1248), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2574), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [57149] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(374), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1249), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2088), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57220] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(346), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1250), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2017), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57291] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(348), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1251), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2161), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57362] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(345), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1250), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1252), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2019), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57433] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(345), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1253), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2019), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57504] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(342), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1253), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1254), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2020), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57575] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(632), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1255), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2016), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57646] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3129), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3131), 1,
      sym__end_of_statement,
    STATE(1256), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2644), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [57699] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1852), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1854), 1,
      sym__end_of_statement,
    STATE(1257), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2636), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [57752] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3133), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3135), 1,
      sym__end_of_statement,
    STATE(1258), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2424), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [57805] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(362), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1259), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1941), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57876] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(394), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1260), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2236), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [57947] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3137), 1,
      anon_sym_COMMA,
    ACTIONS(3139), 1,
      anon_sym_COLON_COLON,
    STATE(1261), 1,
      sym_preproc_file_line,
    STATE(2530), 1,
      sym__type_name,
    STATE(2668), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [58000] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1788), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1790), 1,
      sym__end_of_statement,
    STATE(1262), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2356), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [58053] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(364), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1263), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2108), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58124] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(342), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1264), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2020), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58195] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(624), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1265), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1937), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58266] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1266), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2167), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
    ACTIONS(2162), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [58307] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3141), 1,
      anon_sym_RPAREN,
    STATE(1267), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [58350] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3145), 1,
      anon_sym_COMMA,
    STATE(1268), 1,
      sym_preproc_file_line,
    STATE(1297), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3147), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    ACTIONS(3143), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [58395] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(621), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1265), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1269), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1969), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58466] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(317), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1270), 1,
      sym_preproc_file_line,
    STATE(1279), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1950), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58537] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(621), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1271), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1969), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58608] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1816), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1818), 1,
      sym__end_of_statement,
    STATE(1272), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2370), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [58661] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(411), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1260), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1273), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2226), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58732] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(411), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1274), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2226), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58803] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3149), 1,
      anon_sym_RPAREN,
    STATE(1275), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [58846] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3145), 1,
      anon_sym_COMMA,
    STATE(1268), 1,
      aux_sym_parameters_repeat1,
    STATE(1276), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3153), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    ACTIONS(3151), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [58891] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(612), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1271), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1277), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1965), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [58962] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(612), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1278), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1965), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59033] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(316), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1279), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1966), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59104] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(347), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1280), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2167), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59175] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3155), 1,
      anon_sym_COMMA,
    ACTIONS(3157), 1,
      anon_sym_SLASH,
    STATE(1281), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [59220] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(412), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1274), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1282), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2213), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59291] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3145), 1,
      anon_sym_COMMA,
    STATE(1283), 1,
      sym_preproc_file_line,
    STATE(1322), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3162), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    ACTIONS(3160), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [59336] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(412), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1284), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2213), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59407] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3011), 1,
      sym__end_of_statement,
    STATE(1285), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [59450] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3164), 1,
      anon_sym_EQ,
    STATE(1286), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [59493] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(608), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1287), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1958), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59564] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(608), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1278), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1288), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(1958), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59635] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1289), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2297), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
    ACTIONS(2295), 24,
      aux_sym_end_program_statement_token1,
      aux_sym_assignment_token1,
      aux_sym_operator_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [59676] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(325), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1290), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2025), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59747] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(325), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1264), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1291), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2025), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59818] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(403), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1292), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2203), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59889] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(403), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1284), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1293), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2203), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [59960] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1828), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1830), 1,
      sym__end_of_statement,
    STATE(1294), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2374), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60013] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(589), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1295), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2034), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [60084] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3166), 1,
      anon_sym_COMMA,
    ACTIONS(3168), 1,
      anon_sym_SLASH,
    STATE(1296), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 4,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [60129] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3171), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2722), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    STATE(1297), 2,
      sym_preproc_file_line,
      aux_sym_parameters_repeat1,
    ACTIONS(2720), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60172] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(304), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1298), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2038), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [60243] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3174), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3176), 1,
      sym__end_of_statement,
    STATE(1299), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2446), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60296] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(398), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1300), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2190), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [60367] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1820), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1822), 1,
      sym__end_of_statement,
    STATE(1301), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2437), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60420] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1824), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1826), 1,
      sym__end_of_statement,
    STATE(1302), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2440), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60473] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3178), 1,
      aux_sym_submodule_statement_token1,
    ACTIONS(3180), 1,
      sym__end_of_statement,
    STATE(1303), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2784), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60526] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(374), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1304), 1,
      sym_preproc_file_line,
    STATE(1330), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2088), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [60597] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1856), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1858), 1,
      sym__end_of_statement,
    STATE(1305), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2459), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60650] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3182), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3184), 1,
      sym__end_of_statement,
    STATE(1306), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2710), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60703] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1890), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1892), 1,
      sym__end_of_statement,
    STATE(1307), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2759), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60756] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1886), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1888), 1,
      sym__end_of_statement,
    STATE(1308), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2756), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [60809] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(595), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1309), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2120), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [60880] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(381), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1310), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2071), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [60951] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(347), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1251), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1311), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2167), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [61022] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(338), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1312), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2051), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [61093] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(338), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1237), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1313), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2051), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [61164] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1808), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1810), 1,
      sym__end_of_statement,
    STATE(1314), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2511), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61217] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1802), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1804), 1,
      sym__end_of_statement,
    STATE(1315), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2514), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61270] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1876), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1878), 1,
      sym__end_of_statement,
    STATE(1316), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2721), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61323] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3186), 1,
      anon_sym_RPAREN,
    STATE(1317), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [61366] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(380), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1310), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1318), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2074), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [61437] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1872), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1874), 1,
      sym__end_of_statement,
    STATE(1319), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2718), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61490] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(380), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1320), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2074), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [61561] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1796), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1798), 1,
      sym__end_of_statement,
    STATE(1321), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2532), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61614] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3145), 1,
      anon_sym_COMMA,
    STATE(1297), 1,
      aux_sym_parameters_repeat1,
    STATE(1322), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3082), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    ACTIONS(3188), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61659] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(378), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1320), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1323), 1,
      sym_preproc_file_line,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2079), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [61730] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1844), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1846), 1,
      sym__end_of_statement,
    STATE(1324), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2604), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61783] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1840), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1842), 1,
      sym__end_of_statement,
    STATE(1325), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2586), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61836] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3190), 1,
      anon_sym_COMMA,
    STATE(1326), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2242), 5,
      anon_sym_SLASH,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_GT,
      anon_sym_DOT,
    ACTIONS(2244), 20,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PERCENT,
      aux_sym_logical_expression_token1,
      aux_sym_logical_expression_token2,
      aux_sym_logical_expression_token3,
      aux_sym_logical_expression_token4,
      aux_sym_relational_expression_token1,
      aux_sym_relational_expression_token2,
      anon_sym_LT_EQ,
      aux_sym_relational_expression_token3,
      anon_sym_GT_EQ,
      aux_sym_relational_expression_token4,
      anon_sym_EQ_EQ,
      aux_sym_relational_expression_token5,
      anon_sym_SLASH_EQ,
      aux_sym_relational_expression_token6,
      anon_sym_SLASH_SLASH,
      anon_sym_PLUS,
      anon_sym_STAR_STAR,
  [61879] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1868), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1870), 1,
      sym__end_of_statement,
    STATE(1327), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2674), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61932] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1836), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(1838), 1,
      sym__end_of_statement,
    STATE(1328), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2583), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [61985] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3192), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3194), 1,
      sym__end_of_statement,
    STATE(1329), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2502), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62038] = 20,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1373), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(1405), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1407), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(1409), 1,
      aux_sym_derived_type_token1,
    ACTIONS(1419), 1,
      aux_sym_include_statement_token1,
    ACTIONS(2556), 1,
      aux_sym_contains_statement_token1,
    ACTIONS(2558), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(378), 1,
      sym_end_type_statement,
    STATE(773), 1,
      sym_intrinsic_type,
    STATE(1330), 1,
      sym_preproc_file_line,
    STATE(1532), 1,
      aux_sym_derived_type_definition_repeat1,
    STATE(1734), 1,
      sym_contains_statement,
    STATE(2079), 1,
      sym_derived_type_procedures,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(1403), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [62109] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3196), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3198), 1,
      sym__end_of_statement,
    STATE(1331), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2752), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62162] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1864), 1,
      aux_sym_function_statement_token1,
    ACTIONS(1866), 1,
      sym__end_of_statement,
    STATE(1332), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2656), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62215] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1333), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1279), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1281), 23,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [62255] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3200), 1,
      sym__end_of_statement,
    STATE(1334), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2518), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62305] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3202), 1,
      sym__end_of_statement,
    STATE(1335), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2663), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62355] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1336), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1285), 23,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [62395] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1884), 1,
      sym__end_of_statement,
    STATE(1337), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2749), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62445] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1752), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2299), 1,
      anon_sym_LPAREN,
    ACTIONS(3204), 1,
      anon_sym_STAR,
    STATE(1338), 1,
      sym_preproc_file_line,
    STATE(1562), 1,
      sym_argument_list,
    STATE(1596), 1,
      sym_size,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1756), 20,
      aux_sym_module_statement_token1,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [62493] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3206), 1,
      sym__end_of_statement,
    STATE(1339), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2664), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62543] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3208), 1,
      anon_sym_COMMA,
    ACTIONS(3210), 1,
      anon_sym_COLON_COLON,
    STATE(1340), 1,
      sym_preproc_file_line,
    STATE(2065), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62593] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3212), 1,
      sym__end_of_statement,
    STATE(1341), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2463), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62643] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3214), 1,
      sym__end_of_statement,
    STATE(1342), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2570), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62693] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3216), 1,
      sym__end_of_statement,
    STATE(1343), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2635), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62743] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3218), 1,
      sym__end_of_statement,
    STATE(1344), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2805), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62793] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3220), 1,
      sym__end_of_statement,
    STATE(1345), 1,
      sym_preproc_file_line,
    STATE(2369), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62843] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1346), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1281), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_SLASH,
    ACTIONS(1279), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62883] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1347), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1353), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_SLASH,
    ACTIONS(1351), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62923] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1348), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1293), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_SLASH,
    ACTIONS(1291), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [62963] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1024), 1,
      sym_identifier,
    STATE(1349), 1,
      sym_preproc_file_line,
    STATE(2266), 1,
      sym__name,
    STATE(2267), 1,
      sym_module_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63013] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3222), 1,
      sym__end_of_statement,
    STATE(1350), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2703), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63063] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3224), 1,
      sym__end_of_statement,
    STATE(1351), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2717), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63113] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1858), 1,
      sym__end_of_statement,
    STATE(1352), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2459), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63163] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3226), 1,
      sym__end_of_statement,
    STATE(1353), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2902), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63213] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1870), 1,
      sym__end_of_statement,
    STATE(1354), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2674), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63263] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3228), 1,
      sym__end_of_statement,
    STATE(1355), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2609), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63313] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3230), 1,
      sym__end_of_statement,
    STATE(1356), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2757), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63363] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3232), 1,
      sym__end_of_statement,
    STATE(1357), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2589), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63413] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1842), 1,
      sym__end_of_statement,
    STATE(1358), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2586), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63463] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3234), 1,
      anon_sym_COMMA,
    STATE(1359), 1,
      sym_preproc_file_line,
    STATE(2563), 1,
      sym_identifier,
    STATE(2672), 1,
      sym_loop_control_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63513] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3236), 1,
      sym__end_of_statement,
    STATE(1360), 1,
      sym_preproc_file_line,
    STATE(2419), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63563] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1814), 1,
      sym__end_of_statement,
    STATE(1361), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2781), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63613] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1834), 1,
      sym__end_of_statement,
    STATE(1362), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2844), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63663] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3238), 1,
      sym__end_of_statement,
    STATE(1363), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2599), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63713] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3240), 1,
      sym__end_of_statement,
    STATE(1364), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2597), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63763] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1838), 1,
      sym__end_of_statement,
    STATE(1365), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2583), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63813] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3242), 1,
      sym__end_of_statement,
    STATE(1366), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2591), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [63863] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3249), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(3252), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3258), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3261), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(3264), 1,
      aux_sym_derived_type_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3247), 2,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
    STATE(1367), 2,
      aux_sym__callable_interface_qualifers,
      sym_preproc_file_line,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(3244), 5,
      aux_sym_module_statement_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    ACTIONS(3255), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [63919] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3267), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3269), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(3271), 1,
      aux_sym_function_statement_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1367), 1,
      aux_sym__callable_interface_qualifers,
    STATE(1368), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2046), 4,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [63981] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3273), 1,
      sym__end_of_statement,
    STATE(1369), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2584), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64031] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3194), 1,
      sym__end_of_statement,
    STATE(1370), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2502), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64081] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3275), 1,
      sym__end_of_statement,
    STATE(1371), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2700), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64131] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3198), 1,
      sym__end_of_statement,
    STATE(1372), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2752), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64181] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3277), 1,
      sym__end_of_statement,
    STATE(1373), 1,
      sym_preproc_file_line,
    STATE(2475), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64231] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1374), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1285), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_SLASH,
    ACTIONS(1283), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64271] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3279), 1,
      sym__end_of_statement,
    STATE(1375), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2449), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64321] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2032), 1,
      aux_sym_procedure_attributes_token1,
    ACTIONS(2034), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(2044), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3269), 1,
      aux_sym_subroutine_statement_token1,
    ACTIONS(3271), 1,
      aux_sym_function_statement_token1,
    STATE(1338), 1,
      sym_intrinsic_type,
    STATE(1367), 1,
      aux_sym__callable_interface_qualifers,
    STATE(1376), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1729), 4,
      sym_procedure_attributes,
      sym__intrinsic_type,
      sym_derived_type,
      sym_procedure_qualifier,
    ACTIONS(2046), 5,
      aux_sym_module_statement_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [64381] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3281), 1,
      sym__end_of_statement,
    STATE(1377), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2678), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64431] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3283), 1,
      sym__end_of_statement,
    STATE(1378), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2799), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64481] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3285), 1,
      sym__end_of_statement,
    STATE(1379), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2833), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64531] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3287), 1,
      sym__end_of_statement,
    STATE(1380), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2448), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64581] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1381), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1291), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1293), 23,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [64621] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3289), 1,
      sym__end_of_statement,
    STATE(1382), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2608), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64671] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1383), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1351), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1353), 23,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      anon_sym_EQ_GT,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [64711] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3184), 1,
      sym__end_of_statement,
    STATE(1384), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2710), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64761] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3291), 1,
      sym__end_of_statement,
    STATE(1385), 1,
      sym_preproc_file_line,
    STATE(2397), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64811] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3297), 1,
      anon_sym_COMMA,
    STATE(1386), 1,
      sym_preproc_file_line,
    STATE(1416), 1,
      aux_sym_private_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3293), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3295), 21,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [64855] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3299), 1,
      sym__end_of_statement,
    STATE(1387), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2443), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64905] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3301), 1,
      sym__end_of_statement,
    STATE(1388), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2899), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [64955] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3303), 1,
      sym__end_of_statement,
    STATE(1389), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2762), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65005] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3305), 1,
      sym__end_of_statement,
    STATE(1390), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2761), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65055] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3307), 1,
      aux_sym_included_items_token1,
    STATE(1391), 1,
      sym_preproc_file_line,
    STATE(2036), 1,
      sym_use_alias,
    STATE(2714), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65105] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3309), 1,
      sym__end_of_statement,
    STATE(1392), 1,
      sym_preproc_file_line,
    STATE(2439), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65155] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3311), 1,
      sym__end_of_statement,
    STATE(1393), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2818), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65205] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3313), 1,
      sym__end_of_statement,
    STATE(1394), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2517), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65255] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1024), 1,
      sym_identifier,
    STATE(1395), 1,
      sym_preproc_file_line,
    STATE(2266), 1,
      sym__name,
    STATE(2603), 1,
      sym_module_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65305] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1892), 1,
      sym__end_of_statement,
    STATE(1396), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2759), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65355] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3315), 1,
      sym__end_of_statement,
    STATE(1397), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2659), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65405] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3317), 1,
      sym__end_of_statement,
    STATE(1398), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2594), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65455] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3127), 1,
      sym__end_of_statement,
    STATE(1399), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2574), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65505] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1888), 1,
      sym__end_of_statement,
    STATE(1400), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2756), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65555] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3307), 1,
      aux_sym_included_items_token1,
    STATE(1401), 1,
      sym_preproc_file_line,
    STATE(1981), 1,
      sym_use_alias,
    STATE(2714), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65605] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1826), 1,
      sym__end_of_statement,
    STATE(1402), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2440), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65655] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1818), 1,
      sym__end_of_statement,
    STATE(1403), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2370), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65705] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3121), 1,
      sym__end_of_statement,
    STATE(1404), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2878), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65755] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1810), 1,
      sym__end_of_statement,
    STATE(1405), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2511), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65805] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1804), 1,
      sym__end_of_statement,
    STATE(1406), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2514), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65855] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1822), 1,
      sym__end_of_statement,
    STATE(1407), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2437), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65905] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3319), 1,
      anon_sym_LPAREN,
    STATE(1408), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1283), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1285), 22,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [65947] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3321), 1,
      sym__end_of_statement,
    STATE(1409), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2867), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [65997] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3307), 1,
      aux_sym_included_items_token1,
    STATE(1410), 1,
      sym_preproc_file_line,
    STATE(2240), 1,
      sym_use_alias,
    STATE(2714), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66047] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3323), 1,
      sym__end_of_statement,
    STATE(1411), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2705), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66097] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1412), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2722), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_SLASH,
    ACTIONS(2720), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66137] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3176), 1,
      sym__end_of_statement,
    STATE(1413), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2446), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66187] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3325), 1,
      sym__end_of_statement,
    STATE(1414), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2647), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66237] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3327), 1,
      sym__end_of_statement,
    STATE(1415), 1,
      sym_preproc_file_line,
    STATE(2441), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66287] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3297), 1,
      anon_sym_COMMA,
    STATE(1416), 1,
      sym_preproc_file_line,
    STATE(1436), 1,
      aux_sym_private_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3329), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3331), 21,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [66331] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3135), 1,
      sym__end_of_statement,
    STATE(1417), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2424), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66381] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3333), 1,
      aux_sym__inline_if_statement_token1,
    ACTIONS(3335), 1,
      sym__end_of_statement,
    STATE(1418), 1,
      sym_preproc_file_line,
    STATE(2519), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 18,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66433] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1874), 1,
      sym__end_of_statement,
    STATE(1419), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2718), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66483] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3180), 1,
      sym__end_of_statement,
    STATE(1420), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2784), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66533] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3131), 1,
      sym__end_of_statement,
    STATE(1421), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2644), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66583] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1846), 1,
      sym__end_of_statement,
    STATE(1422), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2604), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66633] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3337), 1,
      sym__end_of_statement,
    STATE(1423), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2536), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66683] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1878), 1,
      sym__end_of_statement,
    STATE(1424), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2721), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66733] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3339), 1,
      sym__end_of_statement,
    STATE(1425), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2380), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66783] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3341), 1,
      sym__end_of_statement,
    STATE(1426), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2687), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66833] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3343), 1,
      sym__end_of_statement,
    STATE(1427), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2728), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66883] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3345), 1,
      sym__end_of_statement,
    STATE(1428), 1,
      sym_preproc_file_line,
    STATE(2371), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66933] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3347), 1,
      sym__end_of_statement,
    STATE(1429), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2910), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [66983] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3349), 1,
      sym__end_of_statement,
    STATE(1430), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2593), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67033] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3351), 1,
      sym__end_of_statement,
    STATE(1431), 1,
      sym_preproc_file_line,
    STATE(2479), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67083] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3353), 1,
      sym__end_of_statement,
    STATE(1432), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2727), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67133] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1866), 1,
      sym__end_of_statement,
    STATE(1433), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2656), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67183] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1950), 1,
      sym__end_of_statement,
    STATE(1434), 1,
      sym_preproc_file_line,
    STATE(2529), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67233] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1830), 1,
      sym__end_of_statement,
    STATE(1435), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2374), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67283] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3359), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3355), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    STATE(1436), 2,
      sym_preproc_file_line,
      aux_sym_private_statement_repeat1,
    ACTIONS(3357), 21,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [67325] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3362), 1,
      sym__end_of_statement,
    STATE(1437), 1,
      sym_preproc_file_line,
    STATE(2469), 1,
      sym__block_label,
    STATE(2568), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67375] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3364), 1,
      sym__end_of_statement,
    STATE(1438), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2723), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67425] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3366), 1,
      sym__end_of_statement,
    STATE(1439), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2367), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67475] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1854), 1,
      sym__end_of_statement,
    STATE(1440), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2636), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67525] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1790), 1,
      sym__end_of_statement,
    STATE(1441), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2356), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67575] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3368), 1,
      sym__end_of_statement,
    STATE(1442), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2358), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67625] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3370), 1,
      sym__end_of_statement,
    STATE(1443), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2657), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67675] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3372), 1,
      sym__end_of_statement,
    STATE(1444), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2521), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67725] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3374), 1,
      sym__end_of_statement,
    STATE(1445), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2522), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67775] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3376), 1,
      sym__end_of_statement,
    STATE(1446), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2871), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67825] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3378), 1,
      sym__end_of_statement,
    STATE(1447), 1,
      sym_preproc_file_line,
    STATE(2568), 1,
      sym_identifier,
    STATE(2621), 1,
      sym__block_label,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67875] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1862), 1,
      sym__end_of_statement,
    STATE(1448), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2653), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67925] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3380), 1,
      sym__end_of_statement,
    STATE(1449), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2366), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [67975] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(1798), 1,
      sym__end_of_statement,
    STATE(1450), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2532), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68025] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1024), 1,
      sym_identifier,
    STATE(1451), 1,
      sym_preproc_file_line,
    STATE(2797), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68072] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1452), 1,
      sym_preproc_file_line,
    STATE(2533), 1,
      sym_keyword_argument,
    STATE(2641), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68119] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1453), 1,
      sym_preproc_file_line,
    STATE(2220), 1,
      sym_keyword_argument,
    STATE(2641), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68166] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1454), 1,
      sym_preproc_file_line,
    STATE(1866), 1,
      sym__name,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68213] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1455), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3162), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    ACTIONS(3160), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68252] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3382), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3386), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3388), 1,
      aux_sym_select_case_statement_token1,
    STATE(328), 1,
      sym_identifier,
    STATE(357), 1,
      sym__name,
    STATE(1456), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3384), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68299] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1457), 1,
      sym_preproc_file_line,
    STATE(2246), 1,
      sym_concurrent_control,
    STATE(2407), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68346] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1458), 1,
      sym_preproc_file_line,
    STATE(1763), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68393] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1459), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3355), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3357), 22,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [68432] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1460), 1,
      sym_preproc_file_line,
    STATE(1791), 1,
      sym_identifier,
    STATE(1792), 1,
      sym_method_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68479] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1075), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1461), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68526] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1462), 1,
      sym_preproc_file_line,
    STATE(1807), 1,
      sym__name,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68573] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3390), 1,
      anon_sym_COLON_COLON,
    STATE(1463), 1,
      sym_preproc_file_line,
    STATE(1847), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1962), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68620] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1464), 1,
      sym_preproc_file_line,
    STATE(2307), 1,
      sym_triplet_spec,
    STATE(2688), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68667] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1070), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1465), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68714] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1466), 1,
      sym_preproc_file_line,
    STATE(1773), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68761] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1467), 1,
      sym_preproc_file_line,
    STATE(2408), 1,
      sym__type_name,
    STATE(2668), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68808] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1468), 1,
      sym_preproc_file_line,
    STATE(1777), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68855] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1469), 1,
      sym_preproc_file_line,
    STATE(2337), 1,
      sym_keyword_argument,
    STATE(2641), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68902] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1470), 1,
      sym_preproc_file_line,
    STATE(2349), 1,
      sym_parameter_assignment,
    STATE(2633), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68949] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1134), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1471), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [68996] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3392), 1,
      anon_sym_RPAREN,
    STATE(1472), 1,
      sym_preproc_file_line,
    STATE(2552), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69043] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1111), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1473), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69090] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1474), 1,
      sym_preproc_file_line,
    STATE(1783), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69137] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1475), 1,
      sym_preproc_file_line,
    STATE(1788), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69184] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1476), 1,
      sym_preproc_file_line,
    STATE(1767), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69231] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3394), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3398), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3400), 1,
      aux_sym_select_case_statement_token1,
    STATE(582), 1,
      sym__name,
    STATE(610), 1,
      sym_identifier,
    STATE(1477), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3396), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69278] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1478), 1,
      sym_preproc_file_line,
    STATE(2668), 1,
      sym_identifier,
    STATE(2825), 1,
      sym__type_name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69325] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1479), 1,
      sym_preproc_file_line,
    STATE(1761), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69372] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1060), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1480), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69419] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1481), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2363), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69466] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1482), 1,
      sym_preproc_file_line,
    STATE(2421), 1,
      sym_loop_control_expression,
    STATE(2563), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69513] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1483), 1,
      sym_preproc_file_line,
    STATE(2137), 1,
      sym_parameter_assignment,
    STATE(2633), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69560] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1484), 1,
      sym_preproc_file_line,
    STATE(1768), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69607] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3297), 1,
      anon_sym_COMMA,
    STATE(1436), 1,
      aux_sym_private_statement_repeat1,
    STATE(1485), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3402), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3404), 20,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [69650] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1486), 1,
      sym_preproc_file_line,
    STATE(1772), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69697] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1487), 1,
      sym_preproc_file_line,
    STATE(1939), 1,
      sym_keyword_argument,
    STATE(2641), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69744] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1488), 1,
      sym_preproc_file_line,
    STATE(1861), 1,
      sym__name,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69791] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2349), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2357), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2359), 1,
      aux_sym_select_case_statement_token1,
    STATE(1489), 1,
      sym_preproc_file_line,
    STATE(1769), 1,
      sym_method_name,
    STATE(1791), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2355), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69838] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1490), 1,
      sym_preproc_file_line,
    STATE(1803), 1,
      sym__name,
    STATE(1905), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69885] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1491), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2970), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2972), 22,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [69924] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1024), 1,
      sym_identifier,
    STATE(1492), 1,
      sym_preproc_file_line,
    STATE(2615), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [69971] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1493), 1,
      sym_preproc_file_line,
    STATE(2291), 1,
      sym_use_alias,
    STATE(2714), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70018] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1494), 1,
      sym_preproc_file_line,
    STATE(2003), 1,
      sym_association,
    STATE(2729), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70065] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1495), 1,
      sym_preproc_file_line,
    STATE(2628), 1,
      sym__type_name,
    STATE(2668), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70112] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1496), 1,
      sym_preproc_file_line,
    STATE(2377), 1,
      sym_keyword_argument,
    STATE(2641), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70159] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1497), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3408), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(3406), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70198] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1049), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1498), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70245] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1499), 1,
      sym_preproc_file_line,
    STATE(2596), 1,
      sym__type_name,
    STATE(2668), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70292] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1500), 1,
      sym_preproc_file_line,
    STATE(2237), 1,
      sym_concurrent_control,
    STATE(2407), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70339] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3410), 1,
      anon_sym_SLASH,
    STATE(1501), 1,
      sym_preproc_file_line,
    STATE(2305), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70386] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3394), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3398), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3400), 1,
      aux_sym_select_case_statement_token1,
    STATE(570), 1,
      sym__name,
    STATE(610), 1,
      sym_identifier,
    STATE(1502), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3396), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70433] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3412), 1,
      anon_sym_SLASH,
    STATE(1503), 1,
      sym_preproc_file_line,
    STATE(2172), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70480] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1504), 1,
      sym_preproc_file_line,
    STATE(2186), 1,
      sym_association,
    STATE(2729), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70527] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1061), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1505), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70574] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1506), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2964), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2966), 22,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [70613] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3416), 1,
      anon_sym_COLON_COLON,
    STATE(1507), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3414), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1654), 21,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [70654] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1508), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2960), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2962), 22,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [70693] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1509), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2956), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2958), 22,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [70732] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1510), 1,
      sym_preproc_file_line,
    STATE(2124), 1,
      sym_triplet_spec,
    STATE(2688), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70779] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3418), 1,
      anon_sym_RPAREN,
    STATE(1511), 1,
      sym_preproc_file_line,
    STATE(2018), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70826] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3297), 1,
      anon_sym_COMMA,
    STATE(1485), 1,
      aux_sym_private_statement_repeat1,
    STATE(1512), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3420), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3422), 20,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [70869] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1092), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1513), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70916] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1514), 1,
      sym_preproc_file_line,
    STATE(2530), 1,
      sym__type_name,
    STATE(2668), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [70963] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1515), 1,
      sym_preproc_file_line,
    STATE(2066), 1,
      sym_triplet_spec,
    STATE(2688), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71010] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1037), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1516), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71057] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1517), 1,
      sym_preproc_file_line,
    STATE(2351), 1,
      sym_association,
    STATE(2729), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71104] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1518), 1,
      sym_preproc_file_line,
    STATE(1905), 1,
      sym_identifier,
    STATE(2745), 1,
      sym__name,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71151] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1519), 1,
      sym_preproc_file_line,
    STATE(1938), 1,
      sym_keyword_argument,
    STATE(2641), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71198] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1520), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1569), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(1563), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71237] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1056), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1521), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71284] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1522), 1,
      sym_preproc_file_line,
    STATE(2478), 1,
      sym__type_name,
    STATE(2668), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71331] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1083), 1,
      sym_method_name,
    STATE(1152), 1,
      sym_identifier,
    STATE(1523), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71378] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1524), 1,
      sym_preproc_file_line,
    STATE(2195), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71422] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1068), 1,
      sym_identifier,
    STATE(1525), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71466] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(821), 1,
      sym_identifier,
    STATE(1526), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71510] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1085), 1,
      sym_identifier,
    STATE(1527), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71554] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3426), 1,
      sym__end_of_statement,
    STATE(1528), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3424), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71592] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    STATE(1529), 1,
      sym_preproc_file_line,
    STATE(1837), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1962), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71636] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3430), 1,
      anon_sym_COLON_COLON,
    STATE(1530), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3428), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1650), 20,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [71676] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1531), 1,
      sym_preproc_file_line,
    STATE(2228), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71720] = 15,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3432), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3434), 1,
      aux_sym_module_procedure_statement_token1,
    ACTIONS(3439), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3445), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3448), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(3451), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3454), 1,
      aux_sym_include_statement_token1,
    STATE(773), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3437), 2,
      aux_sym_contains_statement_token1,
      aux_sym_end_type_statement_token1,
    STATE(1532), 2,
      sym_preproc_file_line,
      aux_sym_derived_type_definition_repeat1,
    STATE(2402), 2,
      sym_variable_declaration,
      sym_include_statement,
    STATE(482), 3,
      sym_procedure_declaration,
      sym__intrinsic_type,
      sym_derived_type,
    ACTIONS(3442), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [71778] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1533), 1,
      sym_preproc_file_line,
    STATE(2701), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71822] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1534), 1,
      sym_preproc_file_line,
    STATE(2411), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71866] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1902), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1535), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1904), 22,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      anon_sym_STAR,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [71904] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1898), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1536), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1900), 22,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      anon_sym_STAR,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [71942] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1537), 1,
      sym_preproc_file_line,
    STATE(2344), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [71986] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3094), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3100), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3102), 1,
      aux_sym_select_case_statement_token1,
    STATE(1276), 1,
      sym_identifier,
    STATE(1538), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3096), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72030] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1539), 1,
      sym_preproc_file_line,
    STATE(2322), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72074] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1540), 1,
      sym_preproc_file_line,
    STATE(2362), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72118] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1541), 1,
      sym_preproc_file_line,
    STATE(2898), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72162] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1542), 1,
      sym_preproc_file_line,
    STATE(1960), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72206] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1143), 1,
      sym_identifier,
    STATE(1543), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72250] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    STATE(821), 1,
      sym_identifier,
    STATE(1544), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1962), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72294] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1545), 1,
      sym_preproc_file_line,
    STATE(2652), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72338] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3094), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3100), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3102), 1,
      aux_sym_select_case_statement_token1,
    STATE(1412), 1,
      sym_identifier,
    STATE(1546), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3096), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72382] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3459), 1,
      sym__end_of_statement,
    STATE(1547), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3457), 22,
      aux_sym_end_program_statement_token1,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token4,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_select_case_statement_token1,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72420] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1412), 1,
      sym_identifier,
    STATE(1548), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72464] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1960), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1966), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1968), 1,
      aux_sym_select_case_statement_token1,
    STATE(1549), 1,
      sym_preproc_file_line,
    STATE(1893), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1962), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72508] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1550), 1,
      sym_preproc_file_line,
    STATE(2535), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72552] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1551), 1,
      sym_preproc_file_line,
    STATE(2547), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72596] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1552), 1,
      sym_preproc_file_line,
    STATE(2289), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72640] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1553), 1,
      sym_preproc_file_line,
    STATE(1908), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72684] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1573), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1554), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1578), 22,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      anon_sym_STAR,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [72722] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1944), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1555), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1946), 22,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      anon_sym_STAR,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [72760] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1556), 1,
      sym_preproc_file_line,
    STATE(2031), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72804] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1557), 1,
      sym_preproc_file_line,
    STATE(2606), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72848] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(579), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(999), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1571), 1,
      aux_sym_select_case_statement_token1,
    STATE(1095), 1,
      sym_identifier,
    STATE(1558), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(107), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72892] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1516), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1522), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1526), 1,
      aux_sym_select_case_statement_token1,
    STATE(1559), 1,
      sym_preproc_file_line,
    STATE(2148), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1518), 19,
      aux_sym_defined_io_procedure_token1,
      aux_sym_defined_io_procedure_token2,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token3,
      aux_sym_type_qualifier_token13,
      aux_sym_type_qualifier_token14,
      aux_sym_stop_statement_token1,
      aux_sym_stop_statement_token2,
      aux_sym_keyword_statement_token2,
      aux_sym_keyword_statement_token3,
      aux_sym_data_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
      aux_sym_block_construct_token1,
      aux_sym_format_statement_token1,
      aux_sym_inquire_statement_token1,
      aux_sym_null_literal_token1,
      aux_sym_identifier_token1,
  [72936] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2002), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1560), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2004), 21,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [72973] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1998), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1561), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2000), 21,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73010] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1992), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1562), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1994), 21,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73047] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1954), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1563), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1957), 21,
      aux_sym_module_statement_token1,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73084] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1564), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73120] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1565), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73156] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1566), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3461), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3463), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73192] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1567), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3465), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3467), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73228] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1568), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3465), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3467), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73264] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1569), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3469), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3471), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73300] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1570), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3473), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3475), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73336] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1571), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3477), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3479), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73372] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1572), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73408] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1573), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3481), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3483), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73444] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1574), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3477), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3479), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73480] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1575), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3461), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3463), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73516] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1576), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3485), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3487), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73552] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1577), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3481), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3483), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73588] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1578), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3489), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3491), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73624] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1579), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73660] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1580), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73696] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1581), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73732] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1582), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73768] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1583), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73804] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1584), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73840] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1585), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3493), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3495), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73876] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1586), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73912] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1587), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73948] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1588), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [73984] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1589), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74020] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1590), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74056] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1591), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74092] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1592), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74128] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1593), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74164] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1594), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74200] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1595), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3493), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3495), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74236] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1980), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1596), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1982), 20,
      aux_sym_module_statement_token1,
      anon_sym_RPAREN,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74272] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1597), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74308] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1598), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74344] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1599), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74380] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1600), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74416] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1601), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74452] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1602), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74488] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1603), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3493), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3495), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74524] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1604), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74560] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1605), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74596] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1606), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74632] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1607), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74668] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1608), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74704] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1609), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74740] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1610), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74776] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1611), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3489), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3491), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74812] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1612), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3485), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3487), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74848] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1613), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3461), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3463), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74884] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1614), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3477), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3479), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74920] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1615), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3481), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3483), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74956] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1616), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3473), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3475), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [74992] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1617), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3469), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3471), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75028] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1618), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75064] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1619), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75100] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1620), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3465), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3467), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75136] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1621), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3465), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3467), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75172] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1622), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75208] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1623), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75244] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1624), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75280] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1625), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75316] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1626), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75352] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1627), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75388] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1628), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75424] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1629), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75460] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1630), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75496] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1631), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75532] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1632), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75568] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1633), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3493), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3495), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75604] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1634), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75640] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1635), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75676] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1636), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75712] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1637), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_submodule_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75748] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1638), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3489), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3491), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75784] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1639), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3465), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3467), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75820] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1640), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3485), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3487), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75856] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1641), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3461), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3463), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75892] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1642), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3477), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3479), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75928] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1643), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3481), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3483), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [75964] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1644), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3473), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3475), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76000] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1645), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3469), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3471), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76036] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1646), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76072] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1647), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76108] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1648), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3465), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3467), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76144] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1649), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76180] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1650), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76216] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1651), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76252] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1652), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76288] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1653), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76324] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1654), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76360] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1655), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3469), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3471), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76396] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1656), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3473), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3475), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76432] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1657), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76468] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1658), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76504] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1659), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76540] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1660), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76576] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1661), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3493), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3495), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76612] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1662), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3481), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3483), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76648] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1663), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3477), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3479), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76684] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1664), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3461), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3463), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76720] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1665), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76756] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1666), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76792] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1667), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3489), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3491), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76828] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1668), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3485), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3487), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76864] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1669), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76900] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1670), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76936] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1671), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3489), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3491), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [76972] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1672), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3485), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3487), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77008] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1673), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3485), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3487), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77044] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1674), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3489), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3491), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77080] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1675), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3461), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3463), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77116] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1676), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3477), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3479), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77152] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1677), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3481), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3483), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77188] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1678), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3473), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3475), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77224] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1679), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3469), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3471), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77260] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1680), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77296] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1681), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77332] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1682), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77368] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1683), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77404] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1684), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77440] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1685), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77476] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1686), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77512] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1687), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77548] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1688), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77584] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1689), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3493), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3495), 19,
      aux_sym_module_statement_token1,
      aux_sym_end_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77620] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1690), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77656] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1691), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77692] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1692), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77728] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1693), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77764] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1694), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77800] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1695), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77836] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1696), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77872] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1697), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1223), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1221), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77908] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1698), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1227), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1225), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77944] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1699), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1167), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1165), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [77980] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1700), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1163), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1161), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78016] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1701), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1143), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1141), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78052] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1702), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3469), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3471), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78088] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1703), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3473), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3475), 19,
      aux_sym_end_program_statement_token2,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78124] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1704), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1135), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1133), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78160] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1705), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1131), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1129), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78196] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1706), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1127), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1125), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78232] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1707), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1211), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1209), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78268] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1708), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1235), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1233), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78304] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1709), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1187), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1185), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78340] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1710), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1215), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1213), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78376] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1711), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1107), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1105), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78412] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1712), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1207), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1205), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78448] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1713), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1263), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1261), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_end_function_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78484] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1714), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1239), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(1237), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_end_module_procedure_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78520] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1972), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1715), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1974), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78555] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2118), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3499), 1,
      aux_sym_module_procedure_statement_token1,
    STATE(1716), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3497), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78592] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1988), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1717), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1990), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78627] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1984), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1718), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1986), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78662] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1976), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1719), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1978), 19,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      anon_sym_COLON_COLON,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78697] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2118), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3501), 1,
      aux_sym_module_procedure_statement_token1,
    STATE(1720), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3497), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78734] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2118), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3503), 1,
      aux_sym_module_procedure_statement_token1,
    STATE(1721), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3497), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78771] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2118), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1722), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3497), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [78805] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1723), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3505), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3507), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [78839] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1724), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3509), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3511), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [78873] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1725), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3513), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3515), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [78907] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1726), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3517), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3519), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [78941] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1727), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3521), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3523), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [78975] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3527), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1728), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3525), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [79009] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1589), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1729), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3529), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [79043] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1585), 1,
      aux_sym_intrinsic_type_token4,
    STATE(1730), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3531), 18,
      aux_sym_module_statement_token1,
      aux_sym_subroutine_statement_token1,
      aux_sym_function_statement_token1,
      aux_sym_procedure_attributes_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_procedure_qualifier_token1,
      aux_sym_procedure_qualifier_token2,
      aux_sym_procedure_qualifier_token3,
      aux_sym_procedure_qualifier_token4,
  [79077] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1731), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3533), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3535), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [79111] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1732), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3537), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3539), 17,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_derived_type_definition_token1,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [79145] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1733), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3432), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(3437), 14,
      aux_sym_module_procedure_statement_token1,
      aux_sym_contains_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_end_type_statement_token1,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token8,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
      aux_sym_derived_type_token1,
      aux_sym_include_statement_token1,
  [79176] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(2562), 1,
      aux_sym_public_statement_token1,
    ACTIONS(3541), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3545), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1734), 1,
      sym_preproc_file_line,
    STATE(1736), 1,
      sym_public_statement,
    STATE(1740), 1,
      sym_private_statement,
    STATE(1748), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79224] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1735), 1,
      sym_preproc_file_line,
    STATE(1745), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1821), 1,
      sym_type_statement,
    STATE(2125), 1,
      sym_statement_label,
    STATE(2480), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [79274] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2560), 1,
      aux_sym_private_statement_token1,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3561), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3563), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1736), 1,
      sym_preproc_file_line,
    STATE(1741), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1746), 1,
      sym_private_statement,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79316] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1737), 1,
      sym_preproc_file_line,
    STATE(1745), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1821), 1,
      sym_type_statement,
    STATE(2021), 1,
      sym_statement_label,
    STATE(2725), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [79366] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1738), 1,
      sym_preproc_file_line,
    STATE(1745), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1821), 1,
      sym_type_statement,
    STATE(1932), 1,
      sym_statement_label,
    STATE(2843), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [79416] = 16,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1739), 1,
      sym_preproc_file_line,
    STATE(1745), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1821), 1,
      sym_type_statement,
    STATE(2091), 1,
      sym_statement_label,
    STATE(2457), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [79466] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3561), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3563), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1740), 1,
      sym_preproc_file_line,
    STATE(1741), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79502] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3565), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3567), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1741), 1,
      sym_preproc_file_line,
    STATE(1747), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79538] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1283), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2760), 1,
      anon_sym_LPAREN,
    STATE(1742), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1285), 9,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79566] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    ACTIONS(3569), 1,
      aux_sym_implicit_statement_token2,
    STATE(1743), 1,
      sym_preproc_file_line,
    STATE(1907), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [79598] = 14,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2632), 1,
      aux_sym_select_case_statement_token3,
    ACTIONS(3571), 1,
      aux_sym_do_loop_statement_token1,
    ACTIONS(3573), 1,
      aux_sym__inline_if_statement_token1,
    ACTIONS(3575), 1,
      aux_sym__inline_where_statement_token1,
    ACTIONS(3577), 1,
      aux_sym__forall_control_expression_token1,
    ACTIONS(3579), 1,
      aux_sym_select_case_statement_token1,
    ACTIONS(3581), 1,
      aux_sym_select_type_statement_token1,
    ACTIONS(3583), 1,
      aux_sym_select_rank_statement_token2,
    ACTIONS(3585), 1,
      aux_sym_block_construct_token1,
    ACTIONS(3587), 1,
      aux_sym_associate_statement_token1,
    STATE(1744), 1,
      sym_preproc_file_line,
    STATE(2458), 1,
      sym__forall_control_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [79642] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3589), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3591), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3594), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3599), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3602), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3605), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1821), 1,
      sym_type_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3597), 2,
      sym__integer_literal,
      aux_sym_end_select_statement_token1,
    STATE(1745), 2,
      sym_preproc_file_line,
      aux_sym_select_type_statement_repeat1,
  [79682] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3565), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3567), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1746), 1,
      sym_preproc_file_line,
    STATE(1749), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79718] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3608), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3610), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3616), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1747), 2,
      sym_preproc_file_line,
      aux_sym_derived_type_procedures_repeat1,
    ACTIONS(3613), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79752] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3561), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3563), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1747), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1748), 1,
      sym_preproc_file_line,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79788] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3543), 1,
      aux_sym_module_statement_token1,
    ACTIONS(3618), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3620), 1,
      aux_sym_end_type_statement_token1,
    STATE(864), 1,
      sym__procedure_kind,
    STATE(1747), 1,
      aux_sym_derived_type_procedures_repeat1,
    STATE(1749), 1,
      sym_preproc_file_line,
    STATE(1813), 1,
      sym_procedure_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2028), 5,
      aux_sym_module_procedure_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79824] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3622), 1,
      aux_sym_end_program_statement_token1,
    STATE(1750), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3624), 9,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79849] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2803), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1751), 1,
      sym_preproc_file_line,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2805), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79878] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2778), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1752), 1,
      sym_preproc_file_line,
    STATE(1775), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2780), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79907] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2862), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1753), 1,
      sym_preproc_file_line,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2864), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [79936] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3628), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3632), 1,
      aux_sym_abstract_specifier_token1,
    ACTIONS(3634), 1,
      aux_sym_base_type_specifier_token1,
    STATE(1754), 1,
      sym_preproc_file_line,
    STATE(2188), 1,
      sym__derived_type_qualifier,
    STATE(2272), 1,
      sym_access_specifier,
    STATE(2273), 1,
      sym_base_type_specifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3630), 2,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
    STATE(2271), 2,
      sym_language_binding,
      sym_abstract_specifier,
  [79973] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(832), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3640), 1,
      aux_sym_elseif_clause_token2,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1755), 1,
      sym_preproc_file_line,
    STATE(1830), 1,
      aux_sym__block_if_statement_repeat1,
    STATE(1876), 1,
      sym_else_clause,
    STATE(1897), 1,
      sym_elseif_clause,
    STATE(2126), 1,
      sym_statement_label,
    STATE(2416), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [80014] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3646), 1,
      aux_sym_select_case_statement_token1,
    STATE(1756), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3644), 9,
      aux_sym_do_loop_statement_token1,
      aux_sym__inline_if_statement_token1,
      aux_sym__inline_where_statement_token1,
      aux_sym__forall_control_expression_token1,
      aux_sym_select_case_statement_token3,
      aux_sym_select_type_statement_token1,
      aux_sym_select_rank_statement_token2,
      aux_sym_block_construct_token1,
      aux_sym_associate_statement_token1,
  [80039] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3628), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3632), 1,
      aux_sym_abstract_specifier_token1,
    ACTIONS(3634), 1,
      aux_sym_base_type_specifier_token1,
    STATE(1757), 1,
      sym_preproc_file_line,
    STATE(2272), 1,
      sym_access_specifier,
    STATE(2273), 1,
      sym_base_type_specifier,
    STATE(2290), 1,
      sym__derived_type_qualifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3630), 2,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
    STATE(2271), 2,
      sym_language_binding,
      sym_abstract_specifier,
  [80076] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2928), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1758), 1,
      sym_preproc_file_line,
    STATE(1765), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2930), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80105] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2918), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1759), 1,
      sym_preproc_file_line,
    STATE(1776), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2920), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80134] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2918), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1760), 1,
      sym_preproc_file_line,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2920), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80163] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2821), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1751), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1761), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2823), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80192] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2914), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1762), 1,
      sym_preproc_file_line,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2916), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80221] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2914), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1763), 1,
      sym_preproc_file_line,
    STATE(1778), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2916), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80250] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2813), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1764), 1,
      sym_preproc_file_line,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2815), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80279] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2778), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1765), 1,
      sym_preproc_file_line,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2780), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80308] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2831), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3648), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1766), 2,
      sym_preproc_file_line,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(2833), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80335] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2817), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1753), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1767), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2819), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80364] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2803), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1768), 1,
      sym_preproc_file_line,
    STATE(1771), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2805), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80393] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2862), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1762), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1769), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2864), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80422] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(832), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3640), 1,
      aux_sym_elseif_clause_token2,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1770), 1,
      sym_preproc_file_line,
    STATE(1830), 1,
      aux_sym__block_if_statement_repeat1,
    STATE(1880), 1,
      sym_else_clause,
    STATE(1897), 1,
      sym_elseif_clause,
    STATE(2081), 1,
      sym_statement_label,
    STATE(2662), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [80463] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2791), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1771), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2793), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80492] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2791), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1772), 1,
      sym_preproc_file_line,
    STATE(1779), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2793), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80521] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1773), 1,
      sym_preproc_file_line,
    STATE(1785), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2766), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80550] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3628), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3632), 1,
      aux_sym_abstract_specifier_token1,
    ACTIONS(3634), 1,
      aux_sym_base_type_specifier_token1,
    STATE(1774), 1,
      sym_preproc_file_line,
    STATE(2183), 1,
      sym__derived_type_qualifier,
    STATE(2272), 1,
      sym_access_specifier,
    STATE(2273), 1,
      sym_base_type_specifier,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3630), 2,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
    STATE(2271), 2,
      sym_language_binding,
      sym_abstract_specifier,
  [80587] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2817), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1775), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2819), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80616] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1776), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2766), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80645] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2878), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1777), 1,
      sym_preproc_file_line,
    STATE(1787), 1,
      aux_sym_procedure_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2880), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80674] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2878), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1778), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2880), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80703] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2787), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1779), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2789), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80732] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2821), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1780), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2823), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80761] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2040), 1,
      aux_sym_intrinsic_type_token4,
    ACTIONS(2042), 1,
      aux_sym_intrinsic_type_token8,
    STATE(1781), 1,
      sym_preproc_file_line,
    STATE(1920), 1,
      sym_intrinsic_type,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2038), 7,
      aux_sym_intrinsic_type_token1,
      aux_sym_intrinsic_type_token2,
      aux_sym_intrinsic_type_token3,
      aux_sym_intrinsic_type_token6,
      aux_sym_intrinsic_type_token7,
      aux_sym_intrinsic_type_token9,
      aux_sym_intrinsic_type_token10,
  [80790] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(832), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3640), 1,
      aux_sym_elseif_clause_token2,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1782), 1,
      sym_preproc_file_line,
    STATE(1830), 1,
      aux_sym__block_if_statement_repeat1,
    STATE(1869), 1,
      sym_else_clause,
    STATE(1897), 1,
      sym_elseif_clause,
    STATE(2197), 1,
      sym_statement_label,
    STATE(2513), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [80831] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2848), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1780), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1783), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2850), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80860] = 13,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(832), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3640), 1,
      aux_sym_elseif_clause_token2,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1784), 1,
      sym_preproc_file_line,
    STATE(1830), 1,
      aux_sym__block_if_statement_repeat1,
    STATE(1896), 1,
      sym_else_clause,
    STATE(1897), 1,
      sym_elseif_clause,
    STATE(2132), 1,
      sym_statement_label,
    STATE(2634), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [80901] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2848), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1785), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2850), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80930] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2856), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1760), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1786), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2858), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80959] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2842), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1766), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1787), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2844), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [80988] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2842), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3626), 1,
      anon_sym_COMMA,
    STATE(1764), 1,
      aux_sym_procedure_statement_repeat2,
    STATE(1788), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2844), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [81017] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2922), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2926), 1,
      anon_sym_EQ_GT,
    STATE(1789), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2924), 8,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [81044] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3653), 1,
      aux_sym_procedure_attribute_token2,
    STATE(1266), 1,
      sym_procedure_attribute,
    STATE(1790), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3651), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81070] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2922), 1,
      aux_sym_end_program_statement_token1,
    STATE(1791), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2924), 8,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [81094] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2952), 1,
      aux_sym_end_program_statement_token1,
    STATE(1792), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2954), 8,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [81118] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3653), 1,
      aux_sym_procedure_attribute_token2,
    STATE(847), 1,
      sym_procedure_attribute,
    STATE(1793), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3651), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81144] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2831), 1,
      aux_sym_end_program_statement_token1,
    STATE(1794), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2833), 8,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      anon_sym_COMMA,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [81168] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3657), 1,
      aux_sym_procedure_attribute_token2,
    STATE(814), 1,
      sym_procedure_attribute,
    STATE(1795), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3655), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81194] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3659), 1,
      anon_sym_LPAREN,
    ACTIONS(3661), 1,
      anon_sym_RPAREN,
    ACTIONS(3663), 1,
      anon_sym_COMMA,
    ACTIONS(3665), 1,
      sym_edit_descriptor,
    ACTIONS(3667), 1,
      aux_sym_hollerith_constant_token1,
    ACTIONS(3669), 1,
      sym_string_literal,
    STATE(1796), 1,
      sym_preproc_file_line,
    STATE(1809), 1,
      aux_sym__transfer_items_repeat1,
    STATE(1842), 1,
      sym_hollerith_constant,
    STATE(1862), 1,
      sym__transfer_item,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81232] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3671), 1,
      anon_sym_LPAREN,
    ACTIONS(3674), 1,
      anon_sym_RPAREN,
    ACTIONS(3676), 1,
      anon_sym_COMMA,
    ACTIONS(3679), 1,
      sym_edit_descriptor,
    ACTIONS(3682), 1,
      aux_sym_hollerith_constant_token1,
    ACTIONS(3685), 1,
      sym_string_literal,
    STATE(1842), 1,
      sym_hollerith_constant,
    STATE(1862), 1,
      sym__transfer_item,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1797), 2,
      sym_preproc_file_line,
      aux_sym__transfer_items_repeat1,
  [81268] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3690), 1,
      aux_sym_procedure_attributes_token3,
    STATE(1798), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3688), 3,
      aux_sym_procedure_attributes_token2,
      aux_sym_procedure_attributes_token4,
      aux_sym_procedure_attributes_token5,
    ACTIONS(3692), 5,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
  [81294] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3653), 1,
      aux_sym_procedure_attribute_token2,
    STATE(874), 1,
      sym_procedure_attribute,
    STATE(1799), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3651), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81320] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3653), 1,
      aux_sym_procedure_attribute_token2,
    STATE(867), 1,
      sym_procedure_attribute,
    STATE(1800), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3651), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81346] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3657), 1,
      aux_sym_procedure_attribute_token2,
    STATE(776), 1,
      sym_procedure_attribute,
    STATE(1801), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3655), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81372] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2457), 1,
      anon_sym_DOT,
    STATE(736), 1,
      sym_user_defined_operator,
    STATE(1802), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3694), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    STATE(1868), 2,
      sym_unary_expression,
      sym_number_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
  [81402] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3696), 1,
      anon_sym_LPAREN,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3702), 1,
      sym__end_of_statement,
    STATE(1803), 1,
      sym_preproc_file_line,
    STATE(1863), 1,
      sym__parameters,
    STATE(1865), 1,
      aux_sym_function_statement_repeat1,
    STATE(2238), 1,
      sym_parameters,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [81438] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2457), 1,
      anon_sym_DOT,
    STATE(736), 1,
      sym_user_defined_operator,
    STATE(1804), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3694), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    STATE(1887), 2,
      sym_unary_expression,
      sym_number_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
  [81468] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3653), 1,
      aux_sym_procedure_attribute_token2,
    STATE(854), 1,
      sym_procedure_attribute,
    STATE(1805), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3651), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81494] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3657), 1,
      aux_sym_procedure_attribute_token2,
    STATE(774), 1,
      sym_procedure_attribute,
    STATE(1806), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3655), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81520] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3696), 1,
      anon_sym_LPAREN,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3704), 1,
      sym__end_of_statement,
    STATE(1807), 1,
      sym_preproc_file_line,
    STATE(1835), 1,
      aux_sym_function_statement_repeat1,
    STATE(1836), 1,
      sym__parameters,
    STATE(2238), 1,
      sym_parameters,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [81556] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3657), 1,
      aux_sym_procedure_attribute_token2,
    STATE(777), 1,
      sym_procedure_attribute,
    STATE(1808), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3655), 7,
      aux_sym_private_statement_token1,
      aux_sym_public_statement_token1,
      aux_sym_procedure_attribute_token1,
      aux_sym_procedure_attribute_token3,
      aux_sym_procedure_attribute_token4,
      aux_sym_procedure_attribute_token5,
      aux_sym_procedure_attribute_token6,
  [81582] = 12,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3659), 1,
      anon_sym_LPAREN,
    ACTIONS(3663), 1,
      anon_sym_COMMA,
    ACTIONS(3665), 1,
      sym_edit_descriptor,
    ACTIONS(3667), 1,
      aux_sym_hollerith_constant_token1,
    ACTIONS(3669), 1,
      sym_string_literal,
    ACTIONS(3706), 1,
      anon_sym_RPAREN,
    STATE(1797), 1,
      aux_sym__transfer_items_repeat1,
    STATE(1809), 1,
      sym_preproc_file_line,
    STATE(1842), 1,
      sym_hollerith_constant,
    STATE(1862), 1,
      sym__transfer_item,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81620] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2457), 1,
      anon_sym_DOT,
    STATE(736), 1,
      sym_user_defined_operator,
    STATE(1810), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3694), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    STATE(1922), 2,
      sym_unary_expression,
      sym_number_literal,
    ACTIONS(155), 3,
      sym__integer_literal,
      sym__float_literal,
      sym__boz_literal,
  [81650] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1811), 1,
      sym_preproc_file_line,
    STATE(1867), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2154), 1,
      sym_statement_label,
    STATE(2504), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81685] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3710), 1,
      aux_sym_select_case_statement_token2,
    STATE(1812), 1,
      sym_preproc_file_line,
    STATE(1833), 1,
      aux_sym_select_case_statement_repeat1,
    STATE(1915), 1,
      sym_case_statement,
    STATE(2086), 1,
      sym_statement_label,
    STATE(2454), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81720] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3712), 1,
      aux_sym_end_program_statement_token1,
    STATE(1813), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3714), 7,
      aux_sym_module_statement_token1,
      aux_sym_module_procedure_statement_token1,
      aux_sym_end_type_statement_token1,
      aux_sym__procedure_kind_token1,
      aux_sym__procedure_kind_token2,
      aux_sym__procedure_kind_token3,
      aux_sym__procedure_kind_token4,
  [81743] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1814), 1,
      sym_preproc_file_line,
    STATE(1867), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2068), 1,
      sym_statement_label,
    STATE(2450), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81778] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1815), 1,
      sym_preproc_file_line,
    STATE(1867), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(1982), 1,
      sym_statement_label,
    STATE(2814), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81813] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1738), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1816), 1,
      sym_preproc_file_line,
    STATE(1821), 1,
      sym_type_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81848] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3710), 1,
      aux_sym_select_case_statement_token2,
    STATE(1817), 1,
      sym_preproc_file_line,
    STATE(1833), 1,
      aux_sym_select_case_statement_repeat1,
    STATE(1915), 1,
      sym_case_statement,
    STATE(2000), 1,
      sym_statement_label,
    STATE(2758), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81883] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3710), 1,
      aux_sym_select_case_statement_token2,
    STATE(1818), 1,
      sym_preproc_file_line,
    STATE(1833), 1,
      aux_sym_select_case_statement_repeat1,
    STATE(1915), 1,
      sym_case_statement,
    STATE(2027), 1,
      sym_statement_label,
    STATE(2712), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81918] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1739), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1819), 1,
      sym_preproc_file_line,
    STATE(1821), 1,
      sym_type_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81953] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1737), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1820), 1,
      sym_preproc_file_line,
    STATE(1821), 1,
      sym_type_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [81988] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1821), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3716), 3,
      aux_sym_end_program_statement_token1,
      aux_sym_implicit_statement_token3,
      aux_sym_derived_type_token1,
    ACTIONS(3718), 5,
      sym__integer_literal,
      aux_sym_end_select_statement_token1,
      aux_sym_type_statement_token2,
      aux_sym_type_statement_token3,
      aux_sym__class_default_token1,
  [82011] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3549), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(3551), 1,
      aux_sym_derived_type_token1,
    ACTIONS(3555), 1,
      aux_sym_type_statement_token2,
    ACTIONS(3557), 1,
      aux_sym_type_statement_token3,
    ACTIONS(3559), 1,
      aux_sym__class_default_token1,
    STATE(1393), 1,
      sym__class_default,
    STATE(1735), 1,
      aux_sym_select_type_statement_repeat1,
    STATE(1821), 1,
      sym_type_statement,
    STATE(1822), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82046] = 11,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1425), 1,
      sym__integer_literal,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1823), 1,
      sym_preproc_file_line,
    STATE(1867), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2046), 1,
      sym_statement_label,
    STATE(2738), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82081] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(642), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3720), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3722), 1,
      aux_sym_end_where_statement_token1,
    ACTIONS(3724), 1,
      aux_sym_elsewhere_clause_token1,
    STATE(1824), 1,
      sym_preproc_file_line,
    STATE(1840), 1,
      aux_sym__block_where_statement_repeat1,
    STATE(1910), 1,
      sym_elsewhere_clause,
    STATE(2734), 1,
      sym_end_where_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82113] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(642), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3720), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3722), 1,
      aux_sym_end_where_statement_token1,
    ACTIONS(3724), 1,
      aux_sym_elsewhere_clause_token1,
    STATE(1825), 1,
      sym_preproc_file_line,
    STATE(1840), 1,
      aux_sym__block_where_statement_repeat1,
    STATE(1910), 1,
      sym_elsewhere_clause,
    STATE(2471), 1,
      sym_end_where_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82145] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3659), 1,
      anon_sym_LPAREN,
    ACTIONS(3665), 1,
      sym_edit_descriptor,
    ACTIONS(3667), 1,
      aux_sym_hollerith_constant_token1,
    ACTIONS(3669), 1,
      sym_string_literal,
    STATE(1796), 1,
      sym__transfer_item,
    STATE(1826), 1,
      sym_preproc_file_line,
    STATE(1842), 1,
      sym_hollerith_constant,
    STATE(2735), 1,
      sym__transfer_items,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82177] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3728), 1,
      aux_sym_concurrent_locality_token1,
    ACTIONS(3730), 1,
      aux_sym_concurrent_locality_token3,
    ACTIONS(3732), 1,
      sym__end_of_statement,
    STATE(1827), 1,
      sym_preproc_file_line,
    STATE(1831), 1,
      aux_sym_concurrent_statement_repeat1,
    STATE(1877), 1,
      sym_concurrent_locality,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3726), 2,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
  [82207] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3659), 1,
      anon_sym_LPAREN,
    ACTIONS(3665), 1,
      sym_edit_descriptor,
    ACTIONS(3667), 1,
      aux_sym_hollerith_constant_token1,
    ACTIONS(3669), 1,
      sym_string_literal,
    STATE(1796), 1,
      sym__transfer_item,
    STATE(1828), 1,
      sym_preproc_file_line,
    STATE(1842), 1,
      sym_hollerith_constant,
    STATE(2730), 1,
      sym__transfer_items,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82239] = 10,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(642), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3720), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3722), 1,
      aux_sym_end_where_statement_token1,
    ACTIONS(3724), 1,
      aux_sym_elsewhere_clause_token1,
    STATE(1829), 1,
      sym_preproc_file_line,
    STATE(1840), 1,
      aux_sym__block_where_statement_repeat1,
    STATE(1910), 1,
      sym_elsewhere_clause,
    STATE(2658), 1,
      sym_end_where_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82271] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3734), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3738), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3741), 1,
      aux_sym_elseif_clause_token2,
    STATE(1897), 1,
      sym_elseif_clause,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3736), 2,
      sym__integer_literal,
      aux_sym_end_if_statement_token1,
    STATE(1830), 2,
      sym_preproc_file_line,
      aux_sym__block_if_statement_repeat1,
  [82299] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3747), 1,
      aux_sym_concurrent_locality_token1,
    ACTIONS(3750), 1,
      aux_sym_concurrent_locality_token3,
    ACTIONS(3753), 1,
      sym__end_of_statement,
    STATE(1877), 1,
      sym_concurrent_locality,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3744), 2,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
    STATE(1831), 2,
      sym_preproc_file_line,
      aux_sym_concurrent_statement_repeat1,
  [82327] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3728), 1,
      aux_sym_concurrent_locality_token1,
    ACTIONS(3730), 1,
      aux_sym_concurrent_locality_token3,
    ACTIONS(3755), 1,
      sym__end_of_statement,
    STATE(1827), 1,
      aux_sym_concurrent_statement_repeat1,
    STATE(1832), 1,
      sym_preproc_file_line,
    STATE(1877), 1,
      sym_concurrent_locality,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3726), 2,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
  [82357] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3757), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3759), 1,
      aux_sym_select_case_statement_token2,
    STATE(1915), 1,
      sym_case_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3762), 2,
      sym__integer_literal,
      aux_sym_end_select_statement_token1,
    STATE(1833), 2,
      sym_preproc_file_line,
      aux_sym_select_case_statement_repeat1,
  [82382] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1834), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2507), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82411] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3770), 1,
      sym__end_of_statement,
    STATE(1835), 1,
      sym_preproc_file_line,
    STATE(1845), 1,
      aux_sym_function_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [82438] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3772), 1,
      sym__end_of_statement,
    STATE(1836), 1,
      sym_preproc_file_line,
    STATE(1846), 1,
      aux_sym_function_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [82465] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3774), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3776), 1,
      anon_sym_EQ,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    STATE(1837), 1,
      sym_preproc_file_line,
    STATE(1888), 1,
      aux_sym_enumerator_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3780), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [82492] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3784), 1,
      sym_edit_descriptor,
    STATE(1838), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3782), 5,
      sym_string_literal,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_hollerith_constant_token1,
  [82513] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3786), 1,
      sym_edit_descriptor,
    STATE(1839), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3674), 5,
      sym_string_literal,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_hollerith_constant_token1,
  [82534] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3788), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3790), 1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3793), 1,
      aux_sym_end_where_statement_token1,
    ACTIONS(3795), 1,
      aux_sym_elsewhere_clause_token1,
    STATE(1910), 1,
      sym_elsewhere_clause,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1840), 2,
      sym_preproc_file_line,
      aux_sym__block_where_statement_repeat1,
  [82561] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1841), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2788), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82590] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3800), 1,
      sym_edit_descriptor,
    STATE(1842), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3798), 5,
      sym_string_literal,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_hollerith_constant_token1,
  [82611] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1843), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2715), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82640] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1843), 1,
      aux_sym_enum_repeat1,
    STATE(1844), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2695), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82669] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3802), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3805), 1,
      aux_sym_function_result_token1,
    ACTIONS(3808), 1,
      sym__end_of_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1845), 2,
      sym_preproc_file_line,
      aux_sym_function_statement_repeat1,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [82694] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3810), 1,
      sym__end_of_statement,
    STATE(1845), 1,
      aux_sym_function_statement_repeat1,
    STATE(1846), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [82721] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    ACTIONS(3812), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3814), 1,
      anon_sym_EQ,
    STATE(1847), 1,
      sym_preproc_file_line,
    STATE(1892), 1,
      aux_sym_enumerator_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3816), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [82748] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1848), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2649), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82777] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1848), 1,
      aux_sym_enum_repeat1,
    STATE(1849), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2627), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82806] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3820), 1,
      sym_edit_descriptor,
    STATE(1850), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3818), 5,
      sym_string_literal,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_hollerith_constant_token1,
  [82827] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1851), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2579), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82856] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1851), 1,
      aux_sym_enum_repeat1,
    STATE(1852), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2557), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82885] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1834), 1,
      aux_sym_enum_repeat1,
    STATE(1853), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2485), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82914] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1854), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3692), 6,
      aux_sym_procedure_attributes_token3,
      aux_sym_variable_attributes_token1,
      aux_sym_variable_attributes_token2,
      aux_sym_variable_attributes_token3,
      aux_sym_variable_attributes_token4,
      aux_sym_variable_attributes_token5,
  [82933] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1855), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2430), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82962] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1855), 1,
      aux_sym_enum_repeat1,
    STATE(1856), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2401), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [82991] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1857), 1,
      sym_preproc_file_line,
    STATE(1890), 1,
      aux_sym_enum_repeat1,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2442), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83020] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1857), 1,
      aux_sym_enum_repeat1,
    STATE(1858), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2498), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83049] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3764), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3766), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3768), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1841), 1,
      aux_sym_enum_repeat1,
    STATE(1859), 1,
      sym_preproc_file_line,
    STATE(1927), 1,
      sym_enumerator_statement,
    STATE(2360), 1,
      sym_end_enum_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83078] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3822), 1,
      sym__end_of_statement,
    STATE(1845), 1,
      aux_sym_function_statement_repeat1,
    STATE(1860), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [83105] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3696), 1,
      anon_sym_LPAREN,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3824), 1,
      sym__end_of_statement,
    STATE(1861), 1,
      sym_preproc_file_line,
    STATE(2008), 1,
      sym__parameters,
    STATE(2238), 1,
      sym_parameters,
    STATE(2760), 1,
      sym_language_binding,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83134] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3828), 1,
      sym_edit_descriptor,
    STATE(1862), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3826), 5,
      sym_string_literal,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      aux_sym_hollerith_constant_token1,
  [83155] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3830), 1,
      sym__end_of_statement,
    STATE(1860), 1,
      aux_sym_function_statement_repeat1,
    STATE(1863), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [83182] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3659), 1,
      anon_sym_LPAREN,
    ACTIONS(3665), 1,
      sym_edit_descriptor,
    ACTIONS(3667), 1,
      aux_sym_hollerith_constant_token1,
    ACTIONS(3669), 1,
      sym_string_literal,
    STATE(1839), 1,
      sym__transfer_item,
    STATE(1842), 1,
      sym_hollerith_constant,
    STATE(1864), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83211] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3700), 1,
      aux_sym_function_result_token1,
    ACTIONS(3832), 1,
      sym__end_of_statement,
    STATE(1845), 1,
      aux_sym_function_statement_repeat1,
    STATE(1865), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2235), 2,
      sym_language_binding,
      sym_function_result,
  [83238] = 9,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3696), 1,
      anon_sym_LPAREN,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(3834), 1,
      sym__end_of_statement,
    STATE(1866), 1,
      sym_preproc_file_line,
    STATE(2238), 1,
      sym_parameters,
    STATE(2239), 1,
      sym__parameters,
    STATE(2550), 1,
      sym_language_binding,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83267] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3836), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3838), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1914), 1,
      sym_rank_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3841), 2,
      sym__integer_literal,
      aux_sym_end_select_statement_token1,
    STATE(1867), 2,
      sym_preproc_file_line,
      aux_sym_select_rank_statement_repeat1,
  [83292] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    ACTIONS(3843), 1,
      aux_sym_end_program_statement_token1,
    STATE(1868), 1,
      sym_preproc_file_line,
    STATE(1875), 1,
      aux_sym_enumerator_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3845), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83316] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1869), 1,
      sym_preproc_file_line,
    STATE(2081), 1,
      sym_statement_label,
    STATE(2662), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83342] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    ACTIONS(3847), 1,
      aux_sym_end_program_statement_token1,
    STATE(1870), 1,
      sym_preproc_file_line,
    STATE(1889), 1,
      aux_sym_enumerator_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3849), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83366] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3851), 1,
      aux_sym_end_program_statement_token1,
    STATE(1871), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3853), 4,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83386] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1872), 1,
      sym_preproc_file_line,
    STATE(2197), 1,
      sym_statement_label,
    STATE(2513), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83412] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3857), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1873), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3855), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83432] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1285), 1,
      anon_sym_EQ,
    ACTIONS(1580), 1,
      anon_sym_RPAREN,
    STATE(1874), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1578), 3,
      anon_sym_LPAREN,
      anon_sym_COLON_COLON,
      anon_sym_STAR,
  [83454] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    ACTIONS(3859), 1,
      aux_sym_end_program_statement_token1,
    STATE(1875), 1,
      sym_preproc_file_line,
    STATE(1889), 1,
      aux_sym_enumerator_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3861), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83478] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1876), 1,
      sym_preproc_file_line,
    STATE(1986), 1,
      sym_statement_label,
    STATE(2693), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83504] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3865), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1877), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3863), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83524] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3869), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1878), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3867), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83544] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3873), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1879), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3871), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83564] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1880), 1,
      sym_preproc_file_line,
    STATE(2132), 1,
      sym_statement_label,
    STATE(2634), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83590] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3877), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1881), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3875), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83610] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3881), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1882), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3879), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83630] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3885), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1883), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3883), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83650] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3887), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1884), 2,
      sym_preproc_file_line,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(2522), 3,
      anon_sym_GT_GT_GT,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [83670] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3892), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1885), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3890), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83690] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3896), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1886), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3894), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83710] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    ACTIONS(3859), 1,
      aux_sym_end_program_statement_token1,
    STATE(1870), 1,
      aux_sym_enumerator_statement_repeat1,
    STATE(1887), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3861), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83734] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    ACTIONS(3843), 1,
      aux_sym_end_program_statement_token1,
    STATE(1888), 1,
      sym_preproc_file_line,
    STATE(1889), 1,
      aux_sym_enumerator_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3845), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83758] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3898), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3900), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3903), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
    STATE(1889), 2,
      sym_preproc_file_line,
      aux_sym_enumerator_statement_repeat1,
  [83780] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3905), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3907), 1,
      aux_sym_enumerator_statement_token1,
    ACTIONS(3910), 1,
      aux_sym_end_enum_statement_token1,
    STATE(1927), 1,
      sym_enumerator_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1890), 2,
      sym_preproc_file_line,
      aux_sym_enum_repeat1,
  [83804] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3914), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1891), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3912), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83824] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3774), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3778), 1,
      anon_sym_COMMA,
    STATE(1889), 1,
      aux_sym_enumerator_statement_repeat1,
    STATE(1892), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3780), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83848] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3898), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3916), 1,
      anon_sym_EQ,
    STATE(1893), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3903), 3,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83870] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3918), 1,
      aux_sym_end_program_statement_token1,
    STATE(1894), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3920), 4,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [83890] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3924), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1895), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3922), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83910] = 8,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    ACTIONS(3642), 1,
      sym__integer_literal,
    STATE(1896), 1,
      sym_preproc_file_line,
    STATE(2126), 1,
      sym_statement_label,
    STATE(2416), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83936] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1897), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3926), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3928), 3,
      sym__integer_literal,
      aux_sym_end_if_statement_token1,
      aux_sym_elseif_clause_token2,
  [83956] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3932), 1,
      aux_sym_concurrent_locality_token1,
    STATE(1898), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3930), 4,
      sym__end_of_statement,
      aux_sym_variable_attributes_token3,
      aux_sym_concurrent_locality_token2,
      aux_sym_concurrent_locality_token3,
  [83976] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3934), 1,
      anon_sym_LPAREN,
    ACTIONS(3936), 1,
      aux_sym_size_token1,
    STATE(706), 1,
      sym_parenthesized_expression,
    STATE(708), 1,
      sym_argument_list,
    STATE(1899), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [83999] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3938), 1,
      anon_sym_LPAREN,
    ACTIONS(3940), 1,
      aux_sym_size_token1,
    STATE(1560), 1,
      sym_argument_list,
    STATE(1561), 1,
      sym_parenthesized_expression,
    STATE(1900), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84022] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2511), 1,
      anon_sym_COMMA,
    STATE(1884), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    STATE(1901), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3942), 2,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [84043] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3944), 1,
      anon_sym_SLASH,
    ACTIONS(3947), 1,
      sym__end_of_statement,
    STATE(2292), 1,
      sym_variable_group,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1902), 2,
      sym_preproc_file_line,
      aux_sym_namelist_statement_repeat1,
  [84064] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2511), 1,
      anon_sym_COMMA,
    STATE(1884), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    STATE(1903), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3949), 2,
      anon_sym_SLASH_RPAREN,
      anon_sym_RBRACK,
  [84085] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3951), 1,
      anon_sym_SLASH,
    STATE(1904), 1,
      sym_preproc_file_line,
    STATE(2211), 1,
      aux_sym_data_set_repeat1,
    STATE(2262), 1,
      sym_data_value,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84108] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1905), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1349), 4,
      sym__end_of_statement,
      anon_sym_LPAREN,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [84125] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1906), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3688), 4,
      aux_sym_procedure_attributes_token2,
      aux_sym_procedure_attributes_token3,
      aux_sym_procedure_attributes_token4,
      aux_sym_procedure_attributes_token5,
  [84142] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3204), 1,
      anon_sym_STAR,
    ACTIONS(3953), 1,
      anon_sym_LPAREN,
    STATE(1562), 1,
      sym_argument_list,
    STATE(1907), 1,
      sym_preproc_file_line,
    STATE(2585), 1,
      sym_size,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84165] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(1908), 1,
      sym_preproc_file_line,
    STATE(1916), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3153), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
  [84186] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1909), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3957), 4,
      aux_sym_procedure_attributes_token2,
      aux_sym_procedure_attributes_token3,
      aux_sym_procedure_attributes_token4,
      aux_sym_procedure_attributes_token5,
  [84203] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(1910), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3959), 2,
      aux_sym_end_program_statement_token1,
      aux_sym_elseif_clause_token1,
    ACTIONS(3961), 2,
      aux_sym_end_where_statement_token1,
      aux_sym_elsewhere_clause_token1,
  [84222] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3963), 1,
      anon_sym_LPAREN,
    ACTIONS(3965), 1,
      sym__integer_literal,
    STATE(1911), 1,
      sym_preproc_file_line,
    STATE(2318), 1,
      sym_statement_label,
    STATE(2605), 1,
      sym_statement_label_reference,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84245] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1563), 1,
      aux_sym_end_program_statement_token1,
    STATE(1912), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1569), 3,
      aux_sym_implicit_statement_token3,
      aux_sym_end_select_statement_token1,
      aux_sym_format_statement_token1,
  [84264] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3967), 1,
      anon_sym_LPAREN,
    STATE(1913), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1285), 3,
      sym__end_of_statement,
      anon_sym_COMMA,
      anon_sym_EQ_GT,
  [84283] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3969), 1,
      aux_sym_end_program_statement_token1,
    STATE(1914), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3971), 3,
      sym__integer_literal,
      aux_sym_select_rank_statement_token1,
      aux_sym_end_select_statement_token1,
  [84302] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3973), 1,
      aux_sym_end_program_statement_token1,
    STATE(1915), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3975), 3,
      sym__integer_literal,
      aux_sym_select_case_statement_token2,
      aux_sym_end_select_statement_token1,
  [84321] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(1916), 1,
      sym_preproc_file_line,
    STATE(1917), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3147), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
  [84342] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3977), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2722), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
    STATE(1917), 2,
      sym_preproc_file_line,
      aux_sym_parameters_repeat1,
  [84361] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3980), 1,
      anon_sym_COMMA,
    ACTIONS(3982), 1,
      anon_sym_EQ_GT,
    ACTIONS(3984), 1,
      sym__end_of_statement,
    STATE(1918), 1,
      sym_preproc_file_line,
    STATE(2242), 1,
      aux_sym_included_items_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84384] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3986), 1,
      anon_sym_SLASH,
    ACTIONS(3988), 1,
      sym__end_of_statement,
    STATE(1902), 1,
      aux_sym_namelist_statement_repeat1,
    STATE(1919), 1,
      sym_preproc_file_line,
    STATE(2292), 1,
      sym_variable_group,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84407] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3204), 1,
      anon_sym_STAR,
    ACTIONS(3990), 1,
      anon_sym_LPAREN,
    STATE(1562), 1,
      sym_argument_list,
    STATE(1920), 1,
      sym_preproc_file_line,
    STATE(2648), 1,
      sym_size,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84430] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3992), 1,
      anon_sym_LPAREN,
    ACTIONS(3994), 1,
      aux_sym_size_token1,
    STATE(820), 1,
      sym_argument_list,
    STATE(823), 1,
      sym_parenthesized_expression,
    STATE(1921), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84453] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3996), 1,
      aux_sym_end_program_statement_token1,
    STATE(1922), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3998), 3,
      anon_sym_COMMA,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [84472] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4000), 1,
      anon_sym_LPAREN,
    ACTIONS(4004), 1,
      aux_sym_size_token1,
    STATE(1923), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4002), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [84493] = 7,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3965), 1,
      sym__integer_literal,
    ACTIONS(4006), 1,
      anon_sym_LPAREN,
    STATE(1924), 1,
      sym_preproc_file_line,
    STATE(2318), 1,
      sym_statement_label,
    STATE(2654), 1,
      sym_statement_label_reference,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84516] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4010), 1,
      sym__end_of_statement,
    STATE(1925), 1,
      sym_preproc_file_line,
    STATE(2072), 1,
      aux_sym_save_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84536] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4012), 1,
      anon_sym_RPAREN,
    ACTIONS(4014), 1,
      anon_sym_COMMA,
    STATE(1926), 1,
      sym_preproc_file_line,
    STATE(1975), 1,
      aux_sym_concurrent_header_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84556] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4016), 1,
      aux_sym_end_program_statement_token1,
    STATE(1927), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4018), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [84574] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4020), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4022), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1573), 1,
      sym_end_module_procedure_statement,
    STATE(1928), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84594] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4020), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4022), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1569), 1,
      sym_end_module_procedure_statement,
    STATE(1929), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84614] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3710), 1,
      aux_sym_select_case_statement_token2,
    STATE(1812), 1,
      aux_sym_select_case_statement_repeat1,
    STATE(1915), 1,
      sym_case_statement,
    STATE(1930), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84634] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4024), 1,
      anon_sym_RPAREN,
    ACTIONS(4026), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1931), 2,
      sym_preproc_file_line,
      aux_sym_associate_statement_repeat1,
  [84652] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(1932), 1,
      sym_preproc_file_line,
    STATE(2423), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84672] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4029), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4031), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(242), 1,
      sym_end_subroutine_statement,
    STATE(1933), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84692] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    ACTIONS(4033), 1,
      sym__end_of_statement,
    STATE(1934), 1,
      sym_preproc_file_line,
    STATE(2241), 1,
      aux_sym__declaration_targets_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84712] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    ACTIONS(4033), 1,
      sym__end_of_statement,
    STATE(1935), 1,
      sym_preproc_file_line,
    STATE(2218), 1,
      aux_sym__declaration_targets_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84732] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3986), 1,
      anon_sym_SLASH,
    STATE(1919), 1,
      aux_sym_namelist_statement_repeat1,
    STATE(1936), 1,
      sym_preproc_file_line,
    STATE(2292), 1,
      sym_variable_group,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84752] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(628), 1,
      sym_end_type_statement,
    STATE(1937), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84772] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4035), 1,
      anon_sym_RPAREN,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    STATE(1938), 1,
      sym_preproc_file_line,
    STATE(1942), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84792] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4039), 1,
      anon_sym_RPAREN,
    STATE(1939), 1,
      sym_preproc_file_line,
    STATE(1940), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84812] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4041), 1,
      anon_sym_RPAREN,
    STATE(1940), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84832] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(338), 1,
      sym_end_type_statement,
    STATE(1941), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84852] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4043), 1,
      anon_sym_RPAREN,
    STATE(1942), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84872] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(1943), 1,
      sym_preproc_file_line,
    STATE(2696), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84892] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4049), 1,
      anon_sym_RPAREN,
    ACTIONS(4051), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1944), 2,
      sym_preproc_file_line,
      aux_sym_parameter_statement_repeat1,
  [84910] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(1945), 1,
      sym_preproc_file_line,
    STATE(2630), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84930] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2638), 1,
      anon_sym_COLON,
    STATE(1946), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2640), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [84948] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2429), 1,
      sym__integer_literal,
    STATE(1497), 1,
      sym_statement_label,
    STATE(1947), 1,
      sym_preproc_file_line,
    STATE(2582), 1,
      sym_statement_label_reference,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84968] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(1948), 1,
      sym_preproc_file_line,
    STATE(2560), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [84988] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4054), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4056), 1,
      aux_sym_end_program_statement_token2,
    STATE(239), 1,
      sym_end_program_statement,
    STATE(1949), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85008] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(316), 1,
      sym_end_type_statement,
    STATE(1950), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85028] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(1951), 1,
      sym_preproc_file_line,
    STATE(2488), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85048] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4058), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4060), 1,
      aux_sym_end_function_statement_token1,
    STATE(1579), 1,
      sym_end_function_statement,
    STATE(1952), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85068] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4062), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4064), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1599), 1,
      sym_end_subroutine_statement,
    STATE(1953), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85088] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4066), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4068), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1581), 1,
      sym_end_subroutine_statement,
    STATE(1954), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85108] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4070), 1,
      anon_sym_RPAREN,
    ACTIONS(4072), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1955), 2,
      sym_preproc_file_line,
      aux_sym_procedure_attributes_repeat1,
  [85126] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    ACTIONS(2696), 1,
      sym__end_of_statement,
    STATE(1935), 1,
      aux_sym__declaration_targets_repeat1,
    STATE(1956), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85146] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4075), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4077), 1,
      aux_sym_end_function_statement_token1,
    STATE(1602), 1,
      sym_end_function_statement,
    STATE(1957), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85166] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(612), 1,
      sym_end_type_statement,
    STATE(1958), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85186] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4079), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4081), 1,
      aux_sym_end_function_statement_token1,
    STATE(270), 1,
      sym_end_function_statement,
    STATE(1959), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85206] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4083), 1,
      anon_sym_COMMA,
    ACTIONS(4085), 1,
      sym__end_of_statement,
    STATE(1960), 1,
      sym_preproc_file_line,
    STATE(2643), 1,
      sym_included_items,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85226] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4029), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4031), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(271), 1,
      sym_end_subroutine_statement,
    STATE(1961), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85246] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(632), 1,
      sym_end_type_statement,
    STATE(1962), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85266] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4079), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4081), 1,
      aux_sym_end_function_statement_token1,
    STATE(250), 1,
      sym_end_function_statement,
    STATE(1963), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85286] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(1964), 1,
      sym_preproc_file_line,
    STATE(2410), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85306] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(621), 1,
      sym_end_type_statement,
    STATE(1965), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85326] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(311), 1,
      sym_end_type_statement,
    STATE(1966), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85346] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4087), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4089), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1577), 1,
      sym_end_module_procedure_statement,
    STATE(1967), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85366] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4062), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4064), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1607), 1,
      sym_end_subroutine_statement,
    STATE(1968), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85386] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(624), 1,
      sym_end_type_statement,
    STATE(1969), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85406] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4091), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4093), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1673), 1,
      sym_end_module_procedure_statement,
    STATE(1970), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85426] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2518), 1,
      anon_sym_RPAREN,
    STATE(1971), 1,
      sym_preproc_file_line,
    STATE(2028), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85446] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4075), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4077), 1,
      aux_sym_end_function_statement_token1,
    STATE(1610), 1,
      sym_end_function_statement,
    STATE(1972), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85466] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(362), 1,
      sym_end_type_statement,
    STATE(1973), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85486] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4095), 1,
      anon_sym_RPAREN,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    STATE(1974), 1,
      sym_preproc_file_line,
    STATE(2165), 1,
      aux_sym_implicit_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85506] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4099), 1,
      anon_sym_RPAREN,
    ACTIONS(4101), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(1975), 2,
      sym_preproc_file_line,
      aux_sym_concurrent_header_repeat1,
  [85524] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4058), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4060), 1,
      aux_sym_end_function_statement_token1,
    STATE(1586), 1,
      sym_end_function_statement,
    STATE(1976), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85544] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1367), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1369), 1,
      aux_sym_end_submodule_statement_token1,
    STATE(282), 1,
      sym_end_submodule_statement,
    STATE(1977), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85564] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4066), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4068), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1590), 1,
      sym_end_subroutine_statement,
    STATE(1978), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85584] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4104), 1,
      anon_sym_RPAREN,
    ACTIONS(4106), 1,
      anon_sym_COMMA,
    STATE(1979), 1,
      sym_preproc_file_line,
    STATE(2032), 1,
      aux_sym__forall_control_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85604] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4066), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4068), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1608), 1,
      sym_end_subroutine_statement,
    STATE(1980), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85624] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4108), 1,
      anon_sym_COMMA,
    ACTIONS(4110), 1,
      sym__end_of_statement,
    STATE(1981), 1,
      sym_preproc_file_line,
    STATE(2110), 1,
      aux_sym_use_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85644] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(1982), 1,
      sym_preproc_file_line,
    STATE(2417), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85664] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4112), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4114), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1615), 1,
      sym_end_module_procedure_statement,
    STATE(1983), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85684] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(4116), 1,
      anon_sym_RPAREN,
    STATE(1984), 1,
      sym_preproc_file_line,
    STATE(2138), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85704] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4058), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4060), 1,
      aux_sym_end_function_statement_token1,
    STATE(1604), 1,
      sym_end_function_statement,
    STATE(1985), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85724] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    STATE(1986), 1,
      sym_preproc_file_line,
    STATE(2848), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85744] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4054), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4056), 1,
      aux_sym_end_program_statement_token2,
    STATE(243), 1,
      sym_end_program_statement,
    STATE(1987), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85764] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4112), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4114), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1617), 1,
      sym_end_module_procedure_statement,
    STATE(1988), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85784] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2526), 1,
      anon_sym_COMMA,
    ACTIONS(4118), 1,
      anon_sym_RPAREN,
    STATE(1989), 1,
      sym_preproc_file_line,
    STATE(2175), 1,
      aux_sym_case_value_range_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85804] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4120), 1,
      anon_sym_RPAREN,
    STATE(1990), 1,
      sym_preproc_file_line,
    STATE(2058), 1,
      aux_sym_implicit_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85824] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4122), 1,
      anon_sym_RPAREN,
    STATE(1991), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85844] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4124), 1,
      anon_sym_RPAREN,
    STATE(1992), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85864] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4108), 1,
      anon_sym_COMMA,
    ACTIONS(4126), 1,
      sym__end_of_statement,
    STATE(1993), 1,
      sym_preproc_file_line,
    STATE(2130), 1,
      aux_sym_use_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85884] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(1994), 1,
      sym_preproc_file_line,
    STATE(2466), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85904] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4128), 1,
      aux_sym_type_qualifier_token6,
    ACTIONS(4130), 1,
      aux_sym_type_qualifier_token7,
    ACTIONS(4132), 1,
      aux_sym_type_qualifier_token8,
    STATE(1995), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85924] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1367), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1369), 1,
      aux_sym_end_submodule_statement_token1,
    STATE(246), 1,
      sym_end_submodule_statement,
    STATE(1996), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85944] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4134), 1,
      anon_sym_RPAREN,
    STATE(1997), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85964] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4029), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4031), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(276), 1,
      sym_end_subroutine_statement,
    STATE(1998), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [85984] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1427), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1429), 1,
      aux_sym_end_module_statement_token1,
    STATE(263), 1,
      sym_end_module_statement,
    STATE(1999), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86004] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2000), 1,
      sym_preproc_file_line,
    STATE(2438), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86024] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4079), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4081), 1,
      aux_sym_end_function_statement_token1,
    STATE(289), 1,
      sym_end_function_statement,
    STATE(2001), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86044] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4136), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4138), 1,
      aux_sym_end_function_statement_token1,
    STATE(1707), 1,
      sym_end_function_statement,
    STATE(2002), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86064] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4140), 1,
      anon_sym_RPAREN,
    ACTIONS(4142), 1,
      anon_sym_COMMA,
    STATE(2003), 1,
      sym_preproc_file_line,
    STATE(2024), 1,
      aux_sym_associate_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86084] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4054), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4056), 1,
      aux_sym_end_program_statement_token2,
    STATE(247), 1,
      sym_end_program_statement,
    STATE(2004), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86104] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2662), 1,
      anon_sym_COMMA,
    ACTIONS(4144), 1,
      sym__end_of_statement,
    STATE(2005), 1,
      sym_preproc_file_line,
    STATE(2180), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86124] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4146), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4148), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1710), 1,
      sym_end_subroutine_statement,
    STATE(2006), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86144] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4150), 1,
      anon_sym_RPAREN,
    STATE(1991), 1,
      aux_sym__io_arguments_repeat1,
    STATE(2007), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86164] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(4152), 1,
      sym__end_of_statement,
    STATE(2008), 1,
      sym_preproc_file_line,
    STATE(2750), 1,
      sym_language_binding,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86184] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4154), 1,
      aux_sym_end_program_statement_token1,
    STATE(2009), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4156), 2,
      aux_sym_enumerator_statement_token1,
      aux_sym_end_enum_statement_token1,
  [86202] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4087), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4089), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1668), 1,
      sym_end_module_procedure_statement,
    STATE(2010), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86222] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2511), 1,
      anon_sym_COMMA,
    ACTIONS(4158), 1,
      anon_sym_GT_GT_GT,
    STATE(1884), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    STATE(2011), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86242] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2662), 1,
      anon_sym_COMMA,
    ACTIONS(4160), 1,
      sym__end_of_statement,
    STATE(2012), 1,
      sym_preproc_file_line,
    STATE(2180), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86262] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2013), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4162), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [86278] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4164), 1,
      anon_sym_RPAREN,
    STATE(1997), 1,
      aux_sym__io_arguments_repeat1,
    STATE(2014), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86298] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4166), 1,
      anon_sym_RPAREN,
    ACTIONS(4168), 1,
      anon_sym_COMMA,
    STATE(2015), 1,
      sym_preproc_file_line,
    STATE(2029), 1,
      aux_sym_keyword_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86318] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(618), 1,
      sym_end_type_statement,
    STATE(2016), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86338] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(356), 1,
      sym_end_type_statement,
    STATE(2017), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86358] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    ACTIONS(4170), 1,
      anon_sym_RPAREN,
    STATE(2018), 1,
      sym_preproc_file_line,
    STATE(2059), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86378] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(346), 1,
      sym_end_type_statement,
    STATE(2019), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86398] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(345), 1,
      sym_end_type_statement,
    STATE(2020), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86418] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2021), 1,
      sym_preproc_file_line,
    STATE(2456), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86438] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(317), 1,
      sym_end_type_statement,
    STATE(2022), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86458] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4172), 1,
      anon_sym_RPAREN,
    ACTIONS(4174), 1,
      anon_sym_COMMA,
    STATE(1955), 1,
      aux_sym_procedure_attributes_repeat1,
    STATE(2023), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86478] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4142), 1,
      anon_sym_COMMA,
    ACTIONS(4176), 1,
      anon_sym_RPAREN,
    STATE(1931), 1,
      aux_sym_associate_statement_repeat1,
    STATE(2024), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86498] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(342), 1,
      sym_end_type_statement,
    STATE(2025), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86518] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4178), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4180), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1629), 1,
      sym_end_subroutine_statement,
    STATE(2026), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86538] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2027), 1,
      sym_preproc_file_line,
    STATE(2451), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86558] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(4182), 1,
      anon_sym_RPAREN,
    STATE(2028), 1,
      sym_preproc_file_line,
    STATE(2138), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86578] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4184), 1,
      anon_sym_RPAREN,
    ACTIONS(4186), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2029), 2,
      sym_preproc_file_line,
      aux_sym_keyword_statement_repeat1,
  [86596] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4189), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4191), 1,
      aux_sym_end_function_statement_token1,
    STATE(1632), 1,
      sym_end_function_statement,
    STATE(2030), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86616] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2886), 1,
      sym__end_of_statement,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(2031), 1,
      sym_preproc_file_line,
    STATE(2204), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86636] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4193), 1,
      anon_sym_RPAREN,
    ACTIONS(4195), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2032), 2,
      sym_preproc_file_line,
      aux_sym__forall_control_expression_repeat1,
  [86654] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4198), 1,
      anon_sym_RPAREN,
    ACTIONS(4200), 1,
      anon_sym_COMMA,
    STATE(2033), 1,
      sym_preproc_file_line,
    STATE(2145), 1,
      aux_sym_implicit_statement_repeat3,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86674] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(608), 1,
      sym_end_type_statement,
    STATE(2034), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86694] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1815), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2035), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86714] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4108), 1,
      anon_sym_COMMA,
    ACTIONS(4202), 1,
      sym__end_of_statement,
    STATE(1993), 1,
      aux_sym_use_statement_repeat1,
    STATE(2036), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86734] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3710), 1,
      aux_sym_select_case_statement_token2,
    STATE(1817), 1,
      aux_sym_select_case_statement_repeat1,
    STATE(1915), 1,
      sym_case_statement,
    STATE(2037), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86754] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(325), 1,
      sym_end_type_statement,
    STATE(2038), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86774] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4200), 1,
      anon_sym_COMMA,
    ACTIONS(4204), 1,
      anon_sym_RPAREN,
    STATE(2033), 1,
      aux_sym_implicit_statement_repeat3,
    STATE(2039), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86794] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4206), 1,
      anon_sym_COMMA,
    ACTIONS(4208), 1,
      anon_sym_COLON_COLON,
    STATE(2040), 1,
      sym_preproc_file_line,
    STATE(2166), 1,
      aux_sym_derived_type_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86814] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4178), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4180), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1635), 1,
      sym_end_subroutine_statement,
    STATE(2041), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86834] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4212), 1,
      anon_sym_DASH,
    STATE(2042), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4210), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [86852] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4214), 1,
      anon_sym_RPAREN,
    STATE(2043), 1,
      sym_preproc_file_line,
    STATE(2089), 1,
      aux_sym_implicit_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86872] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(599), 1,
      sym_end_type_statement,
    STATE(2044), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86892] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4189), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4191), 1,
      aux_sym_end_function_statement_token1,
    STATE(1637), 1,
      sym_end_function_statement,
    STATE(2045), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86912] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2046), 1,
      sym_preproc_file_line,
    STATE(2445), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86932] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4216), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4218), 1,
      aux_sym_end_function_statement_token1,
    STATE(1685), 1,
      sym_end_function_statement,
    STATE(2047), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86952] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2524), 1,
      anon_sym_RPAREN,
    ACTIONS(2526), 1,
      anon_sym_COMMA,
    STATE(1989), 1,
      aux_sym_case_value_range_list_repeat1,
    STATE(2048), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86972] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4220), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4222), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1683), 1,
      sym_end_subroutine_statement,
    STATE(2049), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [86992] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4206), 1,
      anon_sym_COMMA,
    ACTIONS(4224), 1,
      anon_sym_COLON_COLON,
    STATE(2050), 1,
      sym_preproc_file_line,
    STATE(2166), 1,
      aux_sym_derived_type_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87012] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2652), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2656), 1,
      aux_sym_end_type_statement_token1,
    STATE(360), 1,
      sym_end_type_statement,
    STATE(2051), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87032] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4226), 1,
      sym__end_of_statement,
    STATE(2052), 1,
      sym_preproc_file_line,
    STATE(2072), 1,
      aux_sym_save_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87052] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4226), 1,
      sym__end_of_statement,
    STATE(1925), 1,
      aux_sym_save_statement_repeat1,
    STATE(2053), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87072] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3030), 1,
      anon_sym_COMMA,
    ACTIONS(4228), 1,
      anon_sym_SLASH,
    STATE(2054), 1,
      sym_preproc_file_line,
    STATE(2230), 1,
      aux_sym_data_value_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87092] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2586), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2590), 1,
      aux_sym_end_type_statement_token1,
    STATE(304), 1,
      sym_end_type_statement,
    STATE(2055), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87112] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4230), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4232), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1643), 1,
      sym_end_module_procedure_statement,
    STATE(2056), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87132] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4234), 1,
      anon_sym_RPAREN,
    STATE(1974), 1,
      aux_sym_implicit_statement_repeat1,
    STATE(2057), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87152] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4234), 1,
      anon_sym_RPAREN,
    STATE(2058), 1,
      sym_preproc_file_line,
    STATE(2165), 1,
      aux_sym_implicit_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87172] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    ACTIONS(4236), 1,
      anon_sym_RPAREN,
    STATE(1006), 1,
      aux_sym_parameters_repeat1,
    STATE(2059), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87192] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2060), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4238), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [87208] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4230), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4232), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1645), 1,
      sym_end_module_procedure_statement,
    STATE(2061), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87228] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4240), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4242), 1,
      aux_sym_select_case_statement_token2,
    ACTIONS(4244), 1,
      aux_sym_select_rank_statement_token1,
    STATE(2062), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87248] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4246), 1,
      anon_sym_RPAREN,
    STATE(2063), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87268] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4248), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4250), 1,
      aux_sym_end_function_statement_token1,
    STATE(1186), 1,
      sym_end_function_statement,
    STATE(2064), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87288] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4252), 1,
      anon_sym_COMMA,
    ACTIONS(4254), 1,
      sym__end_of_statement,
    STATE(2065), 1,
      sym_preproc_file_line,
    STATE(2565), 1,
      sym_included_items,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87308] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4256), 1,
      anon_sym_RPAREN,
    ACTIONS(4258), 1,
      anon_sym_COMMA,
    STATE(2066), 1,
      sym_preproc_file_line,
    STATE(2178), 1,
      aux_sym__forall_control_expression_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87328] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4260), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4262), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1193), 1,
      sym_end_subroutine_statement,
    STATE(2067), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87348] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2068), 1,
      sym_preproc_file_line,
    STATE(2697), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87368] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4020), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4022), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1576), 1,
      sym_end_module_procedure_statement,
    STATE(2069), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87388] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4264), 1,
      anon_sym_COMMA,
    ACTIONS(4266), 1,
      sym__end_of_statement,
    STATE(2070), 1,
      sym_preproc_file_line,
    STATE(2149), 1,
      aux_sym_implicit_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87408] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(383), 1,
      sym_end_type_statement,
    STATE(2071), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87428] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4268), 1,
      anon_sym_COMMA,
    ACTIONS(4271), 1,
      sym__end_of_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2072), 2,
      sym_preproc_file_line,
      aux_sym_save_statement_repeat1,
  [87446] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4260), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4262), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1184), 1,
      sym_end_subroutine_statement,
    STATE(2073), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87466] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(381), 1,
      sym_end_type_statement,
    STATE(2074), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87486] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2075), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3853), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [87502] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3404), 1,
      sym__end_of_statement,
    ACTIONS(4273), 1,
      anon_sym_COMMA,
    STATE(2076), 1,
      sym_preproc_file_line,
    STATE(2198), 1,
      aux_sym_private_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87522] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3710), 1,
      aux_sym_select_case_statement_token2,
    STATE(1818), 1,
      aux_sym_select_case_statement_repeat1,
    STATE(1915), 1,
      sym_case_statement,
    STATE(2077), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87542] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1823), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2078), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87562] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(380), 1,
      sym_end_type_statement,
    STATE(2079), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87582] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3331), 1,
      sym__end_of_statement,
    ACTIONS(4273), 1,
      anon_sym_COMMA,
    STATE(2080), 1,
      sym_preproc_file_line,
    STATE(2198), 1,
      aux_sym_private_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87602] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    STATE(2081), 1,
      sym_preproc_file_line,
    STATE(2634), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87622] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4275), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4277), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1672), 1,
      sym_end_module_procedure_statement,
    STATE(2082), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87642] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4108), 1,
      anon_sym_COMMA,
    ACTIONS(4279), 1,
      sym__end_of_statement,
    STATE(2083), 1,
      sym_preproc_file_line,
    STATE(2130), 1,
      aux_sym_use_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87662] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(596), 1,
      sym_end_type_statement,
    STATE(2084), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87682] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3982), 1,
      anon_sym_EQ_GT,
    STATE(2085), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4281), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [87700] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2086), 1,
      sym_preproc_file_line,
    STATE(2776), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87720] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4142), 1,
      anon_sym_COMMA,
    ACTIONS(4283), 1,
      anon_sym_RPAREN,
    STATE(1931), 1,
      aux_sym_associate_statement_repeat1,
    STATE(2087), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87740] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(378), 1,
      sym_end_type_statement,
    STATE(2088), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87760] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4285), 1,
      anon_sym_RPAREN,
    STATE(2089), 1,
      sym_preproc_file_line,
    STATE(2165), 1,
      aux_sym_implicit_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87780] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4281), 1,
      sym__end_of_statement,
    ACTIONS(4287), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2090), 2,
      sym_preproc_file_line,
      aux_sym_included_items_repeat1,
  [87798] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2091), 1,
      sym_preproc_file_line,
    STATE(2726), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87818] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4290), 1,
      sym__end_of_statement,
    STATE(2092), 1,
      sym_preproc_file_line,
    STATE(2169), 1,
      aux_sym_save_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87838] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4292), 1,
      anon_sym_COMMA,
    ACTIONS(4294), 1,
      sym__end_of_statement,
    STATE(2093), 1,
      sym_preproc_file_line,
    STATE(2201), 1,
      aux_sym_data_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87858] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4168), 1,
      anon_sym_COMMA,
    ACTIONS(4296), 1,
      anon_sym_RPAREN,
    STATE(2094), 1,
      sym_preproc_file_line,
    STATE(2176), 1,
      aux_sym_keyword_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87878] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2872), 1,
      sym__end_of_statement,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(1917), 1,
      aux_sym_parameters_repeat1,
    STATE(2095), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87898] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4285), 1,
      anon_sym_RPAREN,
    STATE(2096), 1,
      sym_preproc_file_line,
    STATE(2171), 1,
      aux_sym_implicit_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87918] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1734), 1,
      anon_sym_COLON,
    STATE(2097), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(1736), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [87936] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4298), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4300), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1657), 1,
      sym_end_subroutine_statement,
    STATE(2098), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87956] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(4302), 1,
      anon_sym_RPAREN,
    STATE(2099), 1,
      sym_preproc_file_line,
    STATE(2138), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87976] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4304), 1,
      anon_sym_RPAREN,
    STATE(2100), 1,
      sym_preproc_file_line,
    STATE(2209), 1,
      aux_sym__io_arguments_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [87996] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(595), 1,
      sym_end_type_statement,
    STATE(2101), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88016] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4306), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4308), 1,
      aux_sym_end_function_statement_token1,
    STATE(1660), 1,
      sym_end_function_statement,
    STATE(2102), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88036] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2103), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4310), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [88052] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2429), 1,
      sym__integer_literal,
    STATE(1497), 1,
      sym_statement_label,
    STATE(2104), 1,
      sym_preproc_file_line,
    STATE(2157), 1,
      sym_statement_label_reference,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88072] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(4312), 1,
      anon_sym_RPAREN,
    STATE(2105), 1,
      sym_preproc_file_line,
    STATE(2138), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88092] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2540), 1,
      anon_sym_LPAREN,
    ACTIONS(3084), 1,
      sym__end_of_statement,
    STATE(2106), 1,
      sym_preproc_file_line,
    STATE(2702), 1,
      sym_argument_list,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88112] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2107), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3920), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [88128] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(374), 1,
      sym_end_type_statement,
    STATE(2108), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88148] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2109), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4314), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [88164] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4108), 1,
      anon_sym_COMMA,
    ACTIONS(4316), 1,
      sym__end_of_statement,
    STATE(2110), 1,
      sym_preproc_file_line,
    STATE(2130), 1,
      aux_sym_use_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88184] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4264), 1,
      anon_sym_COMMA,
    ACTIONS(4318), 1,
      sym__end_of_statement,
    STATE(2111), 1,
      sym_preproc_file_line,
    STATE(2206), 1,
      aux_sym_implicit_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88204] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3980), 1,
      anon_sym_COMMA,
    ACTIONS(3984), 1,
      sym__end_of_statement,
    STATE(2112), 1,
      sym_preproc_file_line,
    STATE(2242), 1,
      aux_sym_included_items_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88224] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4298), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4300), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1666), 1,
      sym_end_subroutine_statement,
    STATE(2113), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88244] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3030), 1,
      anon_sym_COMMA,
    ACTIONS(4320), 1,
      anon_sym_SLASH,
    STATE(2114), 1,
      sym_preproc_file_line,
    STATE(2230), 1,
      aux_sym_data_value_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88264] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4087), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4089), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1702), 1,
      sym_end_module_procedure_statement,
    STATE(2115), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88284] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4248), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4250), 1,
      aux_sym_end_function_statement_token1,
    STATE(1196), 1,
      sym_end_function_statement,
    STATE(2116), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88304] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4306), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4308), 1,
      aux_sym_end_function_statement_token1,
    STATE(1670), 1,
      sym_end_function_statement,
    STATE(2117), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88324] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4322), 1,
      anon_sym_RPAREN,
    ACTIONS(4324), 1,
      anon_sym_COMMA,
    STATE(1944), 1,
      aux_sym_parameter_statement_repeat1,
    STATE(2118), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88344] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4326), 1,
      anon_sym_COMMA,
    ACTIONS(4328), 1,
      sym__end_of_statement,
    STATE(2119), 1,
      sym_preproc_file_line,
    STATE(2123), 1,
      aux_sym_equivalence_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88364] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2686), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2690), 1,
      aux_sym_end_type_statement_token1,
    STATE(593), 1,
      sym_end_type_statement,
    STATE(2120), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88384] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4091), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4093), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1655), 1,
      sym_end_module_procedure_statement,
    STATE(2121), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88404] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4306), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4308), 1,
      aux_sym_end_function_statement_token1,
    STATE(1652), 1,
      sym_end_function_statement,
    STATE(2122), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88424] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4330), 1,
      anon_sym_COMMA,
    ACTIONS(4333), 1,
      sym__end_of_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2123), 2,
      sym_preproc_file_line,
      aux_sym_equivalence_statement_repeat1,
  [88442] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4335), 1,
      anon_sym_RPAREN,
    ACTIONS(4337), 1,
      anon_sym_COMMA,
    STATE(1979), 1,
      aux_sym__forall_control_expression_repeat1,
    STATE(2124), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88462] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2125), 1,
      sym_preproc_file_line,
    STATE(2660), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88482] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    STATE(2126), 1,
      sym_preproc_file_line,
    STATE(2693), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88502] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4298), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4300), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1650), 1,
      sym_end_subroutine_statement,
    STATE(2127), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88522] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4275), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4277), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1677), 1,
      sym_end_module_procedure_statement,
    STATE(2128), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88542] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4091), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4093), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1662), 1,
      sym_end_module_procedure_statement,
    STATE(2129), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88562] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4339), 1,
      anon_sym_COMMA,
    ACTIONS(4342), 1,
      sym__end_of_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2130), 2,
      sym_preproc_file_line,
      aux_sym_use_statement_repeat1,
  [88580] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4045), 1,
      aux_sym_implicit_statement_token3,
    ACTIONS(4047), 1,
      aux_sym_format_statement_token1,
    STATE(2131), 1,
      sym_preproc_file_line,
    STATE(2775), 1,
      sym_format_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88600] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    STATE(2132), 1,
      sym_preproc_file_line,
    STATE(2416), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88620] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4275), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4277), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1679), 1,
      sym_end_module_procedure_statement,
    STATE(2133), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88640] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1427), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1429), 1,
      aux_sym_end_module_statement_token1,
    STATE(286), 1,
      sym_end_module_statement,
    STATE(2134), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88660] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2520), 1,
      anon_sym_RPAREN,
    STATE(1984), 1,
      aux_sym_argument_list_repeat1,
    STATE(2135), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88680] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4344), 1,
      anon_sym_RPAREN,
    STATE(2100), 1,
      aux_sym__io_arguments_repeat1,
    STATE(2136), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88700] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4324), 1,
      anon_sym_COMMA,
    ACTIONS(4346), 1,
      anon_sym_RPAREN,
    STATE(2118), 1,
      aux_sym_parameter_statement_repeat1,
    STATE(2137), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88720] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2636), 1,
      anon_sym_RPAREN,
    ACTIONS(4348), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2138), 2,
      sym_preproc_file_line,
      aux_sym_argument_list_repeat1,
  [88738] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4351), 1,
      aux_sym_included_items_token1,
    ACTIONS(4353), 1,
      aux_sym_implicit_statement_token2,
    ACTIONS(4355), 1,
      aux_sym__import_names_token1,
    STATE(2139), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88758] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2538), 1,
      anon_sym_RPAREN,
    STATE(2105), 1,
      aux_sym_argument_list_repeat1,
    STATE(2140), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88778] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3965), 1,
      sym__integer_literal,
    STATE(2141), 1,
      sym_preproc_file_line,
    STATE(2318), 1,
      sym_statement_label,
    STATE(2682), 1,
      sym_statement_label_reference,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88798] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2594), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2598), 1,
      aux_sym_end_type_statement_token1,
    STATE(364), 1,
      sym_end_type_statement,
    STATE(2142), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88818] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2487), 1,
      anon_sym_RPAREN,
    STATE(2099), 1,
      aux_sym_argument_list_repeat1,
    STATE(2143), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88838] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2144), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4357), 3,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [88854] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4359), 1,
      anon_sym_RPAREN,
    ACTIONS(4361), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2145), 2,
      sym_preproc_file_line,
      aux_sym_implicit_statement_repeat3,
  [88872] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(398), 1,
      sym_end_type_statement,
    STATE(2146), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88892] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2840), 1,
      sym__end_of_statement,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(1917), 1,
      aux_sym_parameters_repeat1,
    STATE(2147), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88912] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2840), 1,
      sym__end_of_statement,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(2095), 1,
      aux_sym_parameters_repeat1,
    STATE(2148), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88932] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4264), 1,
      anon_sym_COMMA,
    ACTIONS(4364), 1,
      sym__end_of_statement,
    STATE(2149), 1,
      sym_preproc_file_line,
    STATE(2206), 1,
      aux_sym_implicit_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88952] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4366), 1,
      anon_sym_RPAREN,
    ACTIONS(4368), 1,
      anon_sym_COMMA,
    STATE(1975), 1,
      aux_sym_concurrent_header_repeat1,
    STATE(2150), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88972] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2809), 1,
      sym__end_of_statement,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(2147), 1,
      aux_sym_parameters_repeat1,
    STATE(2151), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [88992] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4370), 1,
      aux_sym_type_qualifier_token6,
    ACTIONS(4372), 1,
      aux_sym_type_qualifier_token7,
    ACTIONS(4374), 1,
      aux_sym_type_qualifier_token8,
    STATE(2152), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89012] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2509), 1,
      anon_sym_RPAREN,
    STATE(2153), 1,
      sym_preproc_file_line,
    STATE(2199), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89032] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3547), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3553), 1,
      aux_sym_end_select_statement_token1,
    STATE(2154), 1,
      sym_preproc_file_line,
    STATE(2690), 1,
      sym_end_select_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89052] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4264), 1,
      anon_sym_COMMA,
    ACTIONS(4364), 1,
      sym__end_of_statement,
    STATE(2155), 1,
      sym_preproc_file_line,
    STATE(2194), 1,
      aux_sym_implicit_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89072] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3030), 1,
      anon_sym_COMMA,
    ACTIONS(4376), 1,
      anon_sym_SLASH,
    STATE(2156), 1,
      sym_preproc_file_line,
    STATE(2230), 1,
      aux_sym_data_value_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89092] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4168), 1,
      anon_sym_COMMA,
    ACTIONS(4378), 1,
      anon_sym_RPAREN,
    STATE(2015), 1,
      aux_sym_keyword_statement_repeat1,
    STATE(2157), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89112] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4326), 1,
      anon_sym_COMMA,
    ACTIONS(4380), 1,
      sym__end_of_statement,
    STATE(2119), 1,
      aux_sym_equivalence_statement_repeat1,
    STATE(2158), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89132] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3295), 1,
      sym__end_of_statement,
    ACTIONS(4273), 1,
      anon_sym_COMMA,
    STATE(2080), 1,
      aux_sym_private_statement_repeat1,
    STATE(2159), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89152] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3422), 1,
      sym__end_of_statement,
    ACTIONS(4273), 1,
      anon_sym_COMMA,
    STATE(2076), 1,
      aux_sym_private_statement_repeat1,
    STATE(2160), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89172] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(349), 1,
      sym_end_type_statement,
    STATE(2161), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89192] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4054), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4056), 1,
      aux_sym_end_program_statement_token2,
    STATE(232), 1,
      sym_end_program_statement,
    STATE(2162), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89212] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2429), 1,
      sym__integer_literal,
    STATE(1497), 1,
      sym_statement_label,
    STATE(2163), 1,
      sym_preproc_file_line,
    STATE(2317), 1,
      sym_statement_label_reference,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89232] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4220), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4222), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1688), 1,
      sym_end_subroutine_statement,
    STATE(2164), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89252] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4382), 1,
      anon_sym_RPAREN,
    ACTIONS(4384), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2165), 2,
      sym_preproc_file_line,
      aux_sym_implicit_statement_repeat1,
  [89270] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4387), 1,
      anon_sym_COMMA,
    ACTIONS(4390), 1,
      anon_sym_COLON_COLON,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2166), 2,
      sym_preproc_file_line,
      aux_sym_derived_type_statement_repeat1,
  [89288] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(348), 1,
      sym_end_type_statement,
    STATE(2167), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89308] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4216), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4218), 1,
      aux_sym_end_function_statement_token1,
    STATE(1692), 1,
      sym_end_function_statement,
    STATE(2168), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89328] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4392), 1,
      sym__end_of_statement,
    STATE(2072), 1,
      aux_sym_save_statement_repeat1,
    STATE(2169), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89348] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3088), 1,
      anon_sym_RPAREN,
    ACTIONS(4394), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2170), 2,
      sym_preproc_file_line,
      aux_sym_equivalence_set_repeat1,
  [89366] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4097), 1,
      anon_sym_COMMA,
    ACTIONS(4397), 1,
      anon_sym_RPAREN,
    STATE(2165), 1,
      aux_sym_implicit_statement_repeat1,
    STATE(2171), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89386] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4392), 1,
      sym__end_of_statement,
    STATE(2052), 1,
      aux_sym_save_statement_repeat1,
    STATE(2172), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89406] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4062), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4064), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1589), 1,
      sym_end_subroutine_statement,
    STATE(2173), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89426] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4075), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4077), 1,
      aux_sym_end_function_statement_token1,
    STATE(1594), 1,
      sym_end_function_statement,
    STATE(2174), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89446] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2658), 1,
      anon_sym_RPAREN,
    ACTIONS(4399), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2175), 2,
      sym_preproc_file_line,
      aux_sym_case_value_range_list_repeat1,
  [89464] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4168), 1,
      anon_sym_COMMA,
    ACTIONS(4378), 1,
      anon_sym_RPAREN,
    STATE(2029), 1,
      aux_sym_keyword_statement_repeat1,
    STATE(2176), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89484] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4220), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4222), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1698), 1,
      sym_end_subroutine_statement,
    STATE(2177), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89504] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4402), 1,
      anon_sym_RPAREN,
    ACTIONS(4404), 1,
      anon_sym_COMMA,
    STATE(2032), 1,
      aux_sym__forall_control_expression_repeat1,
    STATE(2178), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89524] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4248), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4250), 1,
      aux_sym_end_function_statement_token1,
    STATE(1201), 1,
      sym_end_function_statement,
    STATE(2179), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89544] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2522), 1,
      sym__end_of_statement,
    ACTIONS(4406), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2180), 2,
      sym_preproc_file_line,
      aux_sym_cuda_kernel_argument_list_repeat1,
  [89562] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4216), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4218), 1,
      aux_sym_end_function_statement_token1,
    STATE(1713), 1,
      sym_end_function_statement,
    STATE(2181), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89582] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(347), 1,
      sym_end_type_statement,
    STATE(2182), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89602] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4206), 1,
      anon_sym_COMMA,
    ACTIONS(4409), 1,
      anon_sym_COLON_COLON,
    STATE(2040), 1,
      aux_sym_derived_type_statement_repeat1,
    STATE(2183), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89622] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4230), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4232), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1640), 1,
      sym_end_module_procedure_statement,
    STATE(2184), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89642] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4260), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4262), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1199), 1,
      sym_end_subroutine_statement,
    STATE(2185), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89662] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4142), 1,
      anon_sym_COMMA,
    ACTIONS(4176), 1,
      anon_sym_RPAREN,
    STATE(2087), 1,
      aux_sym_associate_statement_repeat1,
    STATE(2186), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89682] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2429), 1,
      sym__integer_literal,
    STATE(1497), 1,
      sym_statement_label,
    STATE(2094), 1,
      sym_statement_label_reference,
    STATE(2187), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89702] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4206), 1,
      anon_sym_COMMA,
    ACTIONS(4411), 1,
      anon_sym_COLON_COLON,
    STATE(2050), 1,
      aux_sym_derived_type_statement_repeat1,
    STATE(2188), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89722] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2644), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2648), 1,
      aux_sym_end_type_statement_token1,
    STATE(589), 1,
      sym_end_type_statement,
    STATE(2189), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89742] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(403), 1,
      sym_end_type_statement,
    STATE(2190), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89762] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4010), 1,
      sym__end_of_statement,
    STATE(2191), 1,
      sym_preproc_file_line,
    STATE(2196), 1,
      aux_sym_save_statement_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89782] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1811), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2192), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89802] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4264), 1,
      anon_sym_COMMA,
    ACTIONS(4413), 1,
      sym__end_of_statement,
    STATE(2111), 1,
      aux_sym_implicit_statement_repeat2,
    STATE(2193), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89822] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4264), 1,
      anon_sym_COMMA,
    ACTIONS(4413), 1,
      sym__end_of_statement,
    STATE(2194), 1,
      sym_preproc_file_line,
    STATE(2206), 1,
      aux_sym_implicit_statement_repeat2,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89842] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    ACTIONS(4415), 1,
      anon_sym_RPAREN,
    STATE(2195), 1,
      sym_preproc_file_line,
    STATE(2222), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89862] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4008), 1,
      anon_sym_COMMA,
    ACTIONS(4417), 1,
      sym__end_of_statement,
    STATE(2072), 1,
      aux_sym_save_statement_repeat1,
    STATE(2196), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89882] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3636), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(3638), 1,
      aux_sym_end_if_statement_token1,
    STATE(2197), 1,
      sym_preproc_file_line,
    STATE(2662), 1,
      sym_end_if_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89902] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3357), 1,
      sym__end_of_statement,
    ACTIONS(4419), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2198), 2,
      sym_preproc_file_line,
      aux_sym_private_statement_repeat1,
  [89920] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(4422), 1,
      anon_sym_RPAREN,
    STATE(2138), 1,
      aux_sym_argument_list_repeat1,
    STATE(2199), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89940] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(343), 1,
      sym_end_type_statement,
    STATE(2200), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89960] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4424), 1,
      anon_sym_COMMA,
    ACTIONS(4427), 1,
      sym__end_of_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2201), 2,
      sym_preproc_file_line,
      aux_sym_data_statement_repeat1,
  [89978] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4292), 1,
      anon_sym_COMMA,
    ACTIONS(4429), 1,
      sym__end_of_statement,
    STATE(2093), 1,
      aux_sym_data_statement_repeat1,
    STATE(2202), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [89998] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(412), 1,
      sym_end_type_statement,
    STATE(2203), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90018] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2900), 1,
      sym__end_of_statement,
    ACTIONS(3955), 1,
      anon_sym_COMMA,
    STATE(1917), 1,
      aux_sym_parameters_repeat1,
    STATE(2204), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90038] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(4431), 1,
      anon_sym_RPAREN,
    STATE(2138), 1,
      aux_sym_argument_list_repeat1,
    STATE(2205), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90058] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4433), 1,
      anon_sym_COMMA,
    ACTIONS(4436), 1,
      sym__end_of_statement,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2206), 2,
      sym_preproc_file_line,
      aux_sym_implicit_statement_repeat2,
  [90076] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1285), 1,
      sym__end_of_statement,
    ACTIONS(4438), 1,
      anon_sym_LPAREN,
    STATE(2207), 1,
      sym_preproc_file_line,
    STATE(2600), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90096] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4146), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4148), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1704), 1,
      sym_end_subroutine_statement,
    STATE(2208), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90116] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4440), 1,
      anon_sym_RPAREN,
    ACTIONS(4442), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2209), 2,
      sym_preproc_file_line,
      aux_sym__io_arguments_repeat1,
  [90134] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4112), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4114), 1,
      aux_sym_end_module_procedure_statement_token1,
    STATE(1612), 1,
      sym_end_module_procedure_statement,
    STATE(2210), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90154] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(4445), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2211), 2,
      sym_preproc_file_line,
      aux_sym_data_set_repeat1,
  [90172] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4136), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4138), 1,
      aux_sym_end_function_statement_token1,
    STATE(1699), 1,
      sym_end_function_statement,
    STATE(2212), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90192] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(411), 1,
      sym_end_type_statement,
    STATE(2213), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90212] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(333), 1,
      sym_end_type_statement,
    STATE(2214), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90232] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2950), 1,
      anon_sym_COMMA,
    ACTIONS(4448), 1,
      anon_sym_RPAREN,
    STATE(2170), 1,
      aux_sym_equivalence_set_repeat1,
    STATE(2215), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90252] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4450), 1,
      anon_sym_LPAREN,
    STATE(2216), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4452), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [90270] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4146), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4148), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1696), 1,
      sym_end_subroutine_statement,
    STATE(2217), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90290] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2731), 1,
      sym__end_of_statement,
    ACTIONS(4454), 1,
      anon_sym_COMMA,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2218), 2,
      sym_preproc_file_line,
      aux_sym__declaration_targets_repeat1,
  [90308] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4457), 1,
      aux_sym_included_items_token1,
    ACTIONS(4459), 1,
      aux_sym_implicit_statement_token2,
    ACTIONS(4461), 1,
      aux_sym__import_names_token1,
    STATE(2219), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90328] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4463), 1,
      anon_sym_RPAREN,
    STATE(2063), 1,
      aux_sym__io_arguments_repeat1,
    STATE(2220), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90348] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4136), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4138), 1,
      aux_sym_end_function_statement_token1,
    STATE(1694), 1,
      sym_end_function_statement,
    STATE(2221), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90368] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2811), 1,
      anon_sym_COMMA,
    ACTIONS(4465), 1,
      anon_sym_RPAREN,
    STATE(1006), 1,
      aux_sym_parameters_repeat1,
    STATE(2222), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90388] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4189), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4191), 1,
      aux_sym_end_function_statement_token1,
    STATE(1626), 1,
      sym_end_function_statement,
    STATE(2223), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90408] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4037), 1,
      anon_sym_COMMA,
    ACTIONS(4467), 1,
      anon_sym_RPAREN,
    STATE(1992), 1,
      aux_sym__io_arguments_repeat1,
    STATE(2224), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90428] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4178), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(4180), 1,
      aux_sym_end_subroutine_statement_token1,
    STATE(1624), 1,
      sym_end_subroutine_statement,
    STATE(2225), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90448] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(394), 1,
      sym_end_type_statement,
    STATE(2226), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90468] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3708), 1,
      aux_sym_select_rank_statement_token1,
    STATE(1814), 1,
      aux_sym_select_rank_statement_repeat1,
    STATE(1914), 1,
      sym_rank_statement,
    STATE(2227), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90488] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4469), 1,
      anon_sym_COMMA,
    ACTIONS(4471), 1,
      sym__end_of_statement,
    STATE(2228), 1,
      sym_preproc_file_line,
    STATE(2724), 1,
      sym_included_items,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90508] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2465), 1,
      anon_sym_COMMA,
    ACTIONS(2534), 1,
      anon_sym_RPAREN,
    STATE(2205), 1,
      aux_sym_argument_list_repeat1,
    STATE(2229), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90528] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4473), 1,
      anon_sym_COMMA,
    ACTIONS(4476), 1,
      anon_sym_SLASH,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    STATE(2230), 2,
      sym_preproc_file_line,
      aux_sym_data_value_repeat1,
  [90546] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4174), 1,
      anon_sym_COMMA,
    ACTIONS(4478), 1,
      anon_sym_RPAREN,
    STATE(2023), 1,
      aux_sym_procedure_attributes_repeat1,
    STATE(2231), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90566] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4174), 1,
      anon_sym_COMMA,
    ACTIONS(4480), 1,
      anon_sym_RPAREN,
    STATE(2023), 1,
      aux_sym_procedure_attributes_repeat1,
    STATE(2232), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90586] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4482), 1,
      anon_sym_RPAREN,
    ACTIONS(4484), 1,
      anon_sym_COMMA,
    STATE(2150), 1,
      aux_sym_concurrent_header_repeat1,
    STATE(2233), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90606] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2670), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2674), 1,
      aux_sym_end_type_statement_token1,
    STATE(321), 1,
      sym_end_type_statement,
    STATE(2234), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90626] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2235), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4486), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [90642] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2554), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(2566), 1,
      aux_sym_end_type_statement_token1,
    STATE(409), 1,
      sym_end_type_statement,
    STATE(2236), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90662] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4488), 1,
      anon_sym_RPAREN,
    ACTIONS(4490), 1,
      anon_sym_COMMA,
    STATE(1926), 1,
      aux_sym_concurrent_header_repeat1,
    STATE(2237), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90682] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2238), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4492), 3,
      sym__end_of_statement,
      aux_sym_language_binding_token1,
      aux_sym_function_result_token1,
  [90698] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3698), 1,
      aux_sym_language_binding_token1,
    ACTIONS(4494), 1,
      sym__end_of_statement,
    STATE(2239), 1,
      sym_preproc_file_line,
    STATE(2732), 1,
      sym_language_binding,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90718] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4108), 1,
      anon_sym_COMMA,
    ACTIONS(4496), 1,
      sym__end_of_statement,
    STATE(2083), 1,
      aux_sym_use_statement_repeat1,
    STATE(2240), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90738] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2692), 1,
      anon_sym_COMMA,
    ACTIONS(4498), 1,
      sym__end_of_statement,
    STATE(2218), 1,
      aux_sym__declaration_targets_repeat1,
    STATE(2241), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90758] = 6,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3980), 1,
      anon_sym_COMMA,
    ACTIONS(4500), 1,
      sym__end_of_statement,
    STATE(2090), 1,
      aux_sym_included_items_repeat1,
    STATE(2242), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90778] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2243), 1,
      sym_preproc_file_line,
    STATE(2706), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90795] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2244), 1,
      sym_preproc_file_line,
    STATE(2692), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90812] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2245), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4504), 2,
      sym__end_of_statement,
      anon_sym_LPAREN,
  [90827] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2246), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4099), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [90842] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2247), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4506), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [90857] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2248), 1,
      sym_preproc_file_line,
    STATE(2689), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90874] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2249), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4506), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [90889] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4508), 1,
      anon_sym_COMMA,
    ACTIONS(4510), 1,
      sym__end_of_statement,
    STATE(2250), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90906] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4512), 1,
      anon_sym_LPAREN,
    ACTIONS(4514), 1,
      aux_sym_concurrent_locality_token3,
    STATE(2251), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90923] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2252), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2966), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [90938] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2716), 1,
      aux_sym_intrinsic_type_token5,
    ACTIONS(2718), 1,
      aux_sym_intrinsic_type_token7,
    STATE(2253), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [90955] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2254), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3357), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [90970] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2255), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2962), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [90985] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4467), 1,
      anon_sym_RPAREN,
    ACTIONS(4516), 1,
      anon_sym_COMMA,
    STATE(2256), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91002] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2257), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2958), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91017] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2258), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4518), 2,
      aux_sym_defined_io_procedure_token3,
      aux_sym_defined_io_procedure_token4,
  [91032] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3037), 1,
      anon_sym_LPAREN,
    STATE(2259), 1,
      sym_preproc_file_line,
    STATE(2670), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91049] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2260), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4520), 2,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
  [91064] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3628), 1,
      aux_sym_language_binding_token1,
    STATE(2009), 1,
      sym_language_binding,
    STATE(2261), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91081] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2262), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4522), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91096] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2263), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4524), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91111] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2264), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4333), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91126] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4526), 1,
      anon_sym_COMMA,
    STATE(1884), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    STATE(2265), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91143] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2266), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4528), 2,
      anon_sym_COLON,
      anon_sym_RPAREN,
  [91158] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4530), 1,
      anon_sym_COLON,
    ACTIONS(4532), 1,
      anon_sym_RPAREN,
    STATE(2267), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91175] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2268), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4534), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91190] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2269), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4536), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91205] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2270), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4538), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91220] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2271), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4540), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91235] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2272), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4542), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91250] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2273), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4544), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91265] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4546), 1,
      aux_sym_concurrent_locality_token3,
    ACTIONS(4548), 1,
      aux_sym_type_statement_token1,
    STATE(2274), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91282] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2275), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4550), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91297] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2276), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4427), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91312] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2277), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2658), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91327] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4438), 1,
      anon_sym_LPAREN,
    STATE(2278), 1,
      sym_preproc_file_line,
    STATE(2515), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91344] = 5,
    ACTIONS(4552), 1,
      aux_sym_size_token1,
    ACTIONS(4554), 1,
      anon_sym_POUND,
    ACTIONS(4558), 1,
      sym__newline,
    STATE(2279), 1,
      sym_preproc_file_line,
    ACTIONS(4556), 2,
      sym_comment,
      sym__line_continuation,
  [91361] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2280), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4560), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91376] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2281), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4562), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91391] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4344), 1,
      anon_sym_RPAREN,
    ACTIONS(4564), 1,
      anon_sym_COMMA,
    STATE(2282), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91408] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2283), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4382), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91423] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2284), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4566), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91438] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4568), 1,
      anon_sym_RPAREN,
    ACTIONS(4570), 1,
      aux_sym_type_qualifier_token7,
    STATE(2285), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91455] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4438), 1,
      anon_sym_LPAREN,
    STATE(2286), 1,
      sym_preproc_file_line,
    STATE(2600), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91472] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2287), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4359), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91487] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4572), 1,
      anon_sym_RPAREN,
    ACTIONS(4574), 1,
      aux_sym_type_qualifier_token7,
    STATE(2288), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91504] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2289), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4576), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91519] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2290), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4578), 2,
      anon_sym_COMMA,
      anon_sym_COLON_COLON,
  [91534] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2291), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4342), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91549] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2292), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4580), 2,
      sym__end_of_statement,
      anon_sym_SLASH,
  [91564] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4582), 1,
      anon_sym_LPAREN,
    STATE(2264), 1,
      sym_equivalence_set,
    STATE(2293), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91581] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2294), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4584), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91596] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2295), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4586), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91611] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2296), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2762), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91626] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2297), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4588), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91641] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2298), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2731), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91656] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2299), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4281), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91671] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4590), 1,
      aux_sym_implicit_range_token1,
    STATE(2283), 1,
      sym_implicit_range,
    STATE(2300), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91688] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2301), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4592), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91703] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2530), 1,
      aux_sym_intrinsic_type_token5,
    ACTIONS(2532), 1,
      aux_sym_intrinsic_type_token7,
    STATE(2302), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91720] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2303), 1,
      sym_preproc_file_line,
    STATE(2828), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91737] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4594), 1,
      anon_sym_LPAREN,
    ACTIONS(4596), 1,
      sym__end_of_statement,
    STATE(2304), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91754] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2305), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4271), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91769] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2306), 1,
      sym_preproc_file_line,
    STATE(2830), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91786] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2307), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4193), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91801] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2308), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4598), 2,
      aux_sym_use_statement_token2,
      aux_sym_use_statement_token3,
  [91816] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4600), 1,
      anon_sym_LPAREN,
    ACTIONS(4602), 1,
      aux_sym_concurrent_locality_token3,
    STATE(2309), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91833] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4590), 1,
      aux_sym_implicit_range_token1,
    STATE(2096), 1,
      sym_implicit_range,
    STATE(2310), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91850] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2311), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4604), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91865] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2312), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4606), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91880] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2313), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4608), 2,
      aux_sym_implicit_statement_token3,
      aux_sym_implicit_statement_token4,
  [91895] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3037), 1,
      anon_sym_LPAREN,
    STATE(2314), 1,
      sym_preproc_file_line,
    STATE(2845), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91912] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4610), 1,
      anon_sym_COMMA,
    ACTIONS(4612), 1,
      sym__end_of_statement,
    STATE(2315), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91929] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2316), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4614), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91944] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2317), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4184), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [91959] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2318), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3408), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [91974] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4164), 1,
      anon_sym_RPAREN,
    ACTIONS(4616), 1,
      anon_sym_COMMA,
    STATE(2319), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [91991] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2320), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2946), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [92006] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2321), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4618), 2,
      aux_sym_defined_io_procedure_token3,
      aux_sym_defined_io_procedure_token4,
  [92021] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4620), 1,
      anon_sym_RPAREN,
    ACTIONS(4622), 1,
      anon_sym_COMMA,
    STATE(2322), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92038] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4590), 1,
      aux_sym_implicit_range_token1,
    STATE(2057), 1,
      sym_implicit_range,
    STATE(2323), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92055] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4624), 1,
      sym_string_literal,
    ACTIONS(4626), 1,
      sym__end_of_statement,
    STATE(2324), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92072] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2776), 1,
      anon_sym_LPAREN,
    STATE(537), 1,
      sym__io_arguments,
    STATE(2325), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92089] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2326), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4628), 2,
      sym__end_of_statement,
      anon_sym_LPAREN,
  [92104] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2327), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(3011), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92119] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2328), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4630), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [92134] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2329), 1,
      sym_preproc_file_line,
    STATE(2543), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92151] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2330), 1,
      sym_preproc_file_line,
    STATE(2545), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92168] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2331), 1,
      sym_preproc_file_line,
    STATE(2546), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92185] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2332), 1,
      sym_preproc_file_line,
    STATE(2577), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92202] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4438), 1,
      anon_sym_LPAREN,
    STATE(2333), 1,
      sym_preproc_file_line,
    STATE(2855), 1,
      sym_parenthesized_expression,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92219] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2334), 1,
      sym_preproc_file_line,
    STATE(2575), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92236] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2335), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4632), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [92251] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2336), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4634), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [92266] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2337), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4440), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92281] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2338), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4636), 2,
      aux_sym_defined_io_procedure_token3,
      aux_sym_defined_io_procedure_token4,
  [92296] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4502), 1,
      anon_sym_LPAREN,
    STATE(2339), 1,
      sym_preproc_file_line,
    STATE(2573), 1,
      sym_selector,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92313] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4638), 1,
      anon_sym_LPAREN,
    STATE(227), 1,
      sym_parenthesized_expression,
    STATE(2340), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92330] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2341), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4640), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [92345] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2342), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4614), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92360] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2343), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4070), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92375] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4642), 1,
      anon_sym_RPAREN,
    ACTIONS(4644), 1,
      anon_sym_COMMA,
    STATE(2344), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92392] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2345), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2946), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92407] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2346), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2636), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92422] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4582), 1,
      anon_sym_LPAREN,
    STATE(2158), 1,
      sym_equivalence_set,
    STATE(2347), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92439] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2348), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(2972), 2,
      sym__end_of_statement,
      anon_sym_COMMA,
  [92454] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2349), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4049), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92469] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1563), 1,
      aux_sym_end_program_statement_token1,
    ACTIONS(1569), 1,
      aux_sym_end_if_statement_token1,
    STATE(2350), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92486] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    STATE(2351), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
    ACTIONS(4024), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [92501] = 5,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4646), 1,
      anon_sym_COMMA,
    STATE(1884), 1,
      aux_sym_cuda_kernel_argument_list_repeat1,
    STATE(2352), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92518] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4648), 1,
      anon_sym_RPAREN,
    STATE(2353), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92532] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4650), 1,
      aux_sym_stop_statement_token3,
    STATE(2354), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92546] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4652), 1,
      anon_sym_RPAREN,
    STATE(2355), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92560] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4654), 1,
      sym__end_of_statement,
    STATE(2356), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92574] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1766), 1,
      sym__end_of_statement,
    STATE(2357), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92588] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4656), 1,
      sym__end_of_statement,
    STATE(2358), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92602] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4658), 1,
      sym__end_of_statement,
    STATE(2359), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92616] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4660), 1,
      sym__end_of_statement,
    STATE(2360), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92630] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4662), 1,
      anon_sym_RPAREN,
    STATE(2361), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92644] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4664), 1,
      anon_sym_RPAREN,
    STATE(2362), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92658] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4666), 1,
      sym__end_of_statement,
    STATE(2363), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92672] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4668), 1,
      anon_sym_EQ_GT,
    STATE(2364), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92686] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4670), 1,
      anon_sym_EQ_GT,
    STATE(2365), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92700] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4672), 1,
      sym__end_of_statement,
    STATE(2366), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92714] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4674), 1,
      sym__end_of_statement,
    STATE(2367), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92728] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4676), 1,
      sym__end_of_statement,
    STATE(2368), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92742] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3341), 1,
      sym__end_of_statement,
    STATE(2369), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92756] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4678), 1,
      sym__end_of_statement,
    STATE(2370), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92770] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4680), 1,
      sym__end_of_statement,
    STATE(2371), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92784] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4682), 1,
      anon_sym_EQ_GT,
    STATE(2372), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92798] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4684), 1,
      anon_sym_EQ_GT,
    STATE(2373), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92812] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4686), 1,
      sym__end_of_statement,
    STATE(2374), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92826] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4688), 1,
      aux_sym_enum_statement_token1,
    STATE(2375), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92840] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4690), 1,
      sym__end_of_statement,
    STATE(2376), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92854] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4692), 1,
      anon_sym_RPAREN,
    STATE(2377), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92868] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4694), 1,
      anon_sym_EQ_GT,
    STATE(2378), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92882] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4696), 1,
      anon_sym_EQ_GT,
    STATE(2379), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92896] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4698), 1,
      sym__end_of_statement,
    STATE(2380), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92910] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4700), 1,
      anon_sym_EQ_GT,
    STATE(2381), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92924] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4702), 1,
      anon_sym_EQ_GT,
    STATE(2382), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92938] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4704), 1,
      anon_sym_EQ_GT,
    STATE(2383), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92952] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4706), 1,
      sym__end_of_statement,
    STATE(2384), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92966] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4708), 1,
      sym__end_of_statement,
    STATE(2385), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92980] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4710), 1,
      sym__end_of_statement,
    STATE(2386), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [92994] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4712), 1,
      sym__end_of_statement,
    STATE(2387), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93008] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4714), 1,
      sym__end_of_statement,
    STATE(2388), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93022] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4716), 1,
      anon_sym_RPAREN,
    STATE(2389), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93036] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4718), 1,
      sym__end_of_statement,
    STATE(2390), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93050] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4720), 1,
      sym__end_of_statement,
    STATE(2391), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93064] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4722), 1,
      sym__end_of_statement,
    STATE(2392), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93078] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4724), 1,
      sym__end_of_statement,
    STATE(2393), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93092] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4726), 1,
      anon_sym_EQ_GT,
    STATE(2394), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93106] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4728), 1,
      anon_sym_EQ_GT,
    STATE(2395), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93120] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4730), 1,
      anon_sym_EQ,
    STATE(2396), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93134] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4732), 1,
      sym__end_of_statement,
    STATE(2397), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93148] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4734), 1,
      sym__end_of_statement,
    STATE(2398), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93162] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4736), 1,
      sym__end_of_statement,
    STATE(2399), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93176] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4738), 1,
      sym__end_of_statement,
    STATE(2400), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93190] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4740), 1,
      sym__end_of_statement,
    STATE(2401), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93204] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4742), 1,
      sym__end_of_statement,
    STATE(2402), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93218] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4744), 1,
      anon_sym_RPAREN,
    STATE(2403), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93232] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4746), 1,
      anon_sym_RPAREN,
    STATE(2404), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93246] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4748), 1,
      anon_sym_RPAREN,
    STATE(2405), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93260] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4750), 1,
      anon_sym_RPAREN,
    STATE(2406), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93274] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4752), 1,
      anon_sym_EQ,
    STATE(2407), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93288] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4754), 1,
      sym__end_of_statement,
    STATE(2408), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93302] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4756), 1,
      anon_sym_EQ_GT,
    STATE(2409), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93316] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4708), 1,
      sym__end_of_statement,
    STATE(2410), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93330] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4758), 1,
      anon_sym_RPAREN,
    STATE(2411), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93344] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4760), 1,
      sym__end_of_statement,
    STATE(2412), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93358] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4762), 1,
      sym__end_of_statement,
    STATE(2413), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93372] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4764), 1,
      sym__end_of_statement,
    STATE(2414), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93386] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4766), 1,
      aux_sym_implicit_statement_token2,
    STATE(2415), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93400] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4768), 1,
      sym__end_of_statement,
    STATE(2416), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93414] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4770), 1,
      sym__end_of_statement,
    STATE(2417), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93428] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4772), 1,
      sym__end_of_statement,
    STATE(2418), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93442] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4774), 1,
      sym__end_of_statement,
    STATE(2419), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93456] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4776), 1,
      anon_sym_RPAREN,
    STATE(2420), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93470] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4778), 1,
      sym__end_of_statement,
    STATE(2421), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93484] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4266), 1,
      sym__end_of_statement,
    STATE(2422), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93498] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4780), 1,
      sym__end_of_statement,
    STATE(2423), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93512] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4782), 1,
      sym__end_of_statement,
    STATE(2424), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93526] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4784), 1,
      anon_sym_RPAREN,
    STATE(2425), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93540] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4786), 1,
      anon_sym_RPAREN,
    STATE(2426), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93554] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4788), 1,
      anon_sym_RPAREN,
    STATE(2427), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93568] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4790), 1,
      anon_sym_RPAREN,
    STATE(2428), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93582] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4792), 1,
      anon_sym_RPAREN,
    STATE(2429), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93596] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4794), 1,
      sym__end_of_statement,
    STATE(2430), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93610] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4796), 1,
      anon_sym_RPAREN,
    STATE(2431), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93624] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4798), 1,
      anon_sym_RPAREN,
    STATE(2432), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93638] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4800), 1,
      anon_sym_RPAREN,
    STATE(2433), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93652] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4802), 1,
      anon_sym_RPAREN,
    STATE(2434), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93666] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4804), 1,
      sym__end_of_statement,
    STATE(2435), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93680] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4806), 1,
      anon_sym_RPAREN,
    STATE(2436), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93694] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4808), 1,
      sym__end_of_statement,
    STATE(2437), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93708] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4810), 1,
      sym__end_of_statement,
    STATE(2438), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93722] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4812), 1,
      sym__end_of_statement,
    STATE(2439), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93736] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4814), 1,
      sym__end_of_statement,
    STATE(2440), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93750] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4816), 1,
      sym__end_of_statement,
    STATE(2441), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93764] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4818), 1,
      sym__end_of_statement,
    STATE(2442), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93778] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4820), 1,
      sym__end_of_statement,
    STATE(2443), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93792] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4822), 1,
      anon_sym_EQ,
    STATE(2444), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93806] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4824), 1,
      sym__end_of_statement,
    STATE(2445), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93820] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4826), 1,
      sym__end_of_statement,
    STATE(2446), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93834] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4828), 1,
      anon_sym_RPAREN,
    STATE(2447), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93848] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4830), 1,
      sym__end_of_statement,
    STATE(2448), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93862] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4832), 1,
      sym__end_of_statement,
    STATE(2449), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93876] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4834), 1,
      sym__end_of_statement,
    STATE(2450), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93890] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4836), 1,
      sym__end_of_statement,
    STATE(2451), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93904] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4838), 1,
      sym__end_of_statement,
    STATE(2452), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93918] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4840), 1,
      anon_sym_LPAREN,
    STATE(2453), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93932] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4842), 1,
      sym__end_of_statement,
    STATE(2454), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93946] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4844), 1,
      anon_sym_LPAREN,
    STATE(2455), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93960] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4846), 1,
      sym__end_of_statement,
    STATE(2456), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93974] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4848), 1,
      sym__end_of_statement,
    STATE(2457), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [93988] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4850), 1,
      sym__end_of_statement,
    STATE(2458), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94002] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4852), 1,
      sym__end_of_statement,
    STATE(2459), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94016] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4854), 1,
      aux_sym__inline_where_statement_token1,
    STATE(2460), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94030] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4856), 1,
      anon_sym_EQ_GT,
    STATE(2461), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94044] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4858), 1,
      sym__end_of_statement,
    STATE(2462), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94058] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4860), 1,
      sym__end_of_statement,
    STATE(2463), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94072] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4862), 1,
      sym__end_of_statement,
    STATE(2464), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94086] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4864), 1,
      aux_sym_interface_statement_token1,
    STATE(2465), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94100] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4866), 1,
      sym__end_of_statement,
    STATE(2466), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94114] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4868), 1,
      sym__end_of_statement,
    STATE(2467), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94128] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym__end_of_statement,
    STATE(2468), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94142] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4872), 1,
      sym__end_of_statement,
    STATE(2469), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94156] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4874), 1,
      sym__end_of_statement,
    STATE(2470), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94170] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4876), 1,
      sym__end_of_statement,
    STATE(2471), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94184] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4878), 1,
      sym__end_of_statement,
    STATE(2472), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94198] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4880), 1,
      anon_sym_RPAREN,
    STATE(2473), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94212] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4572), 1,
      anon_sym_RPAREN,
    STATE(2474), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94226] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4882), 1,
      sym__end_of_statement,
    STATE(2475), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94240] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4884), 1,
      sym__end_of_statement,
    STATE(2476), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94254] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3224), 1,
      sym__end_of_statement,
    STATE(2477), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94268] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4886), 1,
      sym__end_of_statement,
    STATE(2478), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94282] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4888), 1,
      sym__end_of_statement,
    STATE(2479), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94296] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4890), 1,
      sym__end_of_statement,
    STATE(2480), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94310] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4892), 1,
      anon_sym_RPAREN,
    STATE(2481), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94324] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4894), 1,
      anon_sym_RPAREN,
    STATE(2482), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94338] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4896), 1,
      sym__end_of_statement,
    STATE(2483), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94352] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4898), 1,
      sym__end_of_statement,
    STATE(2484), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94366] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4900), 1,
      sym__end_of_statement,
    STATE(2485), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94380] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4902), 1,
      anon_sym_COLON,
    STATE(2486), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94394] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4904), 1,
      sym__end_of_statement,
    STATE(2487), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94408] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4906), 1,
      sym__end_of_statement,
    STATE(2488), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94422] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4908), 1,
      sym__end_of_statement,
    STATE(2489), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94436] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4910), 1,
      sym__end_of_statement,
    STATE(2490), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94450] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4912), 1,
      anon_sym_LPAREN,
    STATE(2491), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94464] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4914), 1,
      sym__end_of_statement,
    STATE(2492), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94478] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4916), 1,
      anon_sym_RPAREN,
    STATE(2493), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94492] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4918), 1,
      anon_sym_RPAREN,
    STATE(2494), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94506] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4920), 1,
      anon_sym_RBRACK,
    STATE(2495), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94520] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4922), 1,
      anon_sym_SLASH_RPAREN,
    STATE(2496), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94534] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4924), 1,
      anon_sym_RPAREN,
    STATE(2497), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94548] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4926), 1,
      sym__end_of_statement,
    STATE(2498), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94562] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4928), 1,
      anon_sym_COLON_COLON,
    STATE(2499), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94576] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4930), 1,
      anon_sym_LPAREN,
    STATE(2500), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94590] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4906), 1,
      sym__end_of_statement,
    STATE(2501), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94604] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4932), 1,
      sym__end_of_statement,
    STATE(2502), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94618] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4934), 1,
      sym__end_of_statement,
    STATE(2503), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94632] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4936), 1,
      sym__end_of_statement,
    STATE(2504), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94646] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4938), 1,
      sym__end_of_statement,
    STATE(2505), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94660] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4940), 1,
      sym__end_of_statement,
    STATE(2506), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94674] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4942), 1,
      sym__end_of_statement,
    STATE(2507), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94688] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4944), 1,
      sym__end_of_statement,
    STATE(2508), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94702] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4946), 1,
      sym__end_of_statement,
    STATE(2509), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94716] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2825), 1,
      aux_sym__inline_if_statement_token1,
    STATE(2510), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94730] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4948), 1,
      sym__end_of_statement,
    STATE(2511), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94744] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4950), 1,
      anon_sym_DOT,
    STATE(2512), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94758] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4952), 1,
      sym__end_of_statement,
    STATE(2513), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94772] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4954), 1,
      sym__end_of_statement,
    STATE(2514), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94786] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4956), 1,
      aux_sym__block_if_statement_token1,
    STATE(2515), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94800] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3011), 1,
      sym__end_of_statement,
    STATE(2516), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94814] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4958), 1,
      sym__end_of_statement,
    STATE(2517), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94828] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4960), 1,
      sym__end_of_statement,
    STATE(2518), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94842] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4962), 1,
      sym__end_of_statement,
    STATE(2519), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94856] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4870), 1,
      sym__end_of_statement,
    STATE(2520), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94870] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4964), 1,
      sym__end_of_statement,
    STATE(2521), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94884] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4966), 1,
      sym__end_of_statement,
    STATE(2522), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94898] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4968), 1,
      sym__end_of_statement,
    STATE(2523), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94912] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4970), 1,
      anon_sym_RPAREN,
    STATE(2524), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94926] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4972), 1,
      anon_sym_RPAREN,
    STATE(2525), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94940] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4974), 1,
      sym__end_of_statement,
    STATE(2526), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94954] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4976), 1,
      anon_sym_RBRACK,
    STATE(2527), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94968] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4978), 1,
      anon_sym_SLASH_RPAREN,
    STATE(2528), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94982] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4980), 1,
      sym__end_of_statement,
    STATE(2529), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [94996] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4982), 1,
      sym__end_of_statement,
    STATE(2530), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95010] = 4,
    ACTIONS(4554), 1,
      anon_sym_POUND,
    ACTIONS(4984), 1,
      aux_sym_operator_token2,
    STATE(2531), 1,
      sym_preproc_file_line,
    ACTIONS(4556), 2,
      sym_comment,
      sym__line_continuation,
  [95024] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4986), 1,
      sym__end_of_statement,
    STATE(2532), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95038] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4988), 1,
      anon_sym_RPAREN,
    STATE(2533), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95052] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4990), 1,
      sym__end_of_statement,
    STATE(2534), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95066] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4992), 1,
      anon_sym_SLASH,
    STATE(2535), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95080] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4994), 1,
      sym__end_of_statement,
    STATE(2536), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95094] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4996), 1,
      sym__end_of_statement,
    STATE(2537), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95108] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4998), 1,
      sym__end_of_statement,
    STATE(2538), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95122] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5000), 1,
      sym__end_of_statement,
    STATE(2539), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95136] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4722), 1,
      sym__end_of_statement,
    STATE(2540), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95150] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5002), 1,
      sym__end_of_statement,
    STATE(2541), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95164] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5004), 1,
      sym__end_of_statement,
    STATE(2542), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95178] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5006), 1,
      sym__end_of_statement,
    STATE(2543), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95192] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5008), 1,
      sym__end_of_statement,
    STATE(2544), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95206] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5010), 1,
      sym__end_of_statement,
    STATE(2545), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95220] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5012), 1,
      sym__end_of_statement,
    STATE(2546), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95234] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5014), 1,
      anon_sym_RPAREN,
    STATE(2547), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95248] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1792), 1,
      sym__end_of_statement,
    STATE(2548), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95262] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5016), 1,
      anon_sym_LPAREN,
    STATE(2549), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95276] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5018), 1,
      sym__end_of_statement,
    STATE(2550), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95290] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5020), 1,
      sym__end_of_statement,
    STATE(2551), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95304] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5022), 1,
      anon_sym_RPAREN,
    STATE(2552), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95318] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5024), 1,
      anon_sym_LPAREN,
    STATE(2553), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95332] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5026), 1,
      anon_sym_COLON_COLON,
    STATE(2554), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95346] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5028), 1,
      sym__end_of_statement,
    STATE(2555), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95360] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5030), 1,
      sym__end_of_statement,
    STATE(2556), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95374] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5032), 1,
      sym__end_of_statement,
    STATE(2557), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95388] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5034), 1,
      anon_sym_RPAREN,
    STATE(2558), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95402] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5036), 1,
      anon_sym_COLON_COLON,
    STATE(2559), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95416] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4738), 1,
      sym__end_of_statement,
    STATE(2560), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95430] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5038), 1,
      sym__end_of_statement,
    STATE(2561), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95444] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5040), 1,
      sym__end_of_statement,
    STATE(2562), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95458] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5042), 1,
      anon_sym_EQ,
    STATE(2563), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95472] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5044), 1,
      sym__end_of_statement,
    STATE(2564), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95486] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5046), 1,
      sym__end_of_statement,
    STATE(2565), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95500] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5048), 1,
      sym__end_of_statement,
    STATE(2566), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95514] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2580), 1,
      sym__end_of_statement,
    STATE(2567), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95528] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5050), 1,
      sym__end_of_statement,
    STATE(2568), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95542] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2866), 1,
      aux_sym__inline_where_statement_token1,
    STATE(2569), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95556] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5052), 1,
      sym__end_of_statement,
    STATE(2570), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95570] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5054), 1,
      anon_sym_LPAREN,
    STATE(2571), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95584] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5056), 1,
      sym__end_of_statement,
    STATE(2572), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95598] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5058), 1,
      sym__end_of_statement,
    STATE(2573), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95612] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5060), 1,
      sym__end_of_statement,
    STATE(2574), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95626] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5062), 1,
      sym__end_of_statement,
    STATE(2575), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95640] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5064), 1,
      anon_sym_EQ_GT,
    STATE(2576), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95654] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5066), 1,
      sym__end_of_statement,
    STATE(2577), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95668] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1786), 1,
      sym__end_of_statement,
    STATE(2578), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95682] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5068), 1,
      sym__end_of_statement,
    STATE(2579), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95696] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5070), 1,
      sym__end_of_statement,
    STATE(2580), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95710] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5072), 1,
      sym__end_of_statement,
    STATE(2581), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95724] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5074), 1,
      anon_sym_COMMA,
    STATE(2582), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95738] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5076), 1,
      sym__end_of_statement,
    STATE(2583), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95752] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5078), 1,
      sym__end_of_statement,
    STATE(2584), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95766] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5080), 1,
      anon_sym_LPAREN,
    STATE(2585), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95780] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5082), 1,
      sym__end_of_statement,
    STATE(2586), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95794] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5084), 1,
      anon_sym_RPAREN,
    STATE(2587), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95808] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5086), 1,
      sym__end_of_statement,
    STATE(2588), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95822] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5088), 1,
      sym__end_of_statement,
    STATE(2589), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95836] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5090), 1,
      anon_sym_RPAREN,
    STATE(2590), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95850] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5092), 1,
      sym__end_of_statement,
    STATE(2591), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95864] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5094), 1,
      anon_sym_RPAREN,
    STATE(2592), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95878] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5096), 1,
      sym__end_of_statement,
    STATE(2593), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95892] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5098), 1,
      sym__end_of_statement,
    STATE(2594), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95906] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5100), 1,
      anon_sym_LPAREN,
    STATE(2595), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95920] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5102), 1,
      sym__end_of_statement,
    STATE(2596), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95934] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5104), 1,
      sym__end_of_statement,
    STATE(2597), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95948] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1569), 1,
      sym__end_of_statement,
    STATE(2598), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95962] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5106), 1,
      sym__end_of_statement,
    STATE(2599), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95976] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5108), 1,
      aux_sym__block_if_statement_token1,
    STATE(2600), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [95990] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5110), 1,
      anon_sym_STAR,
    STATE(2601), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96004] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5112), 1,
      sym__end_of_statement,
    STATE(2602), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96018] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5114), 1,
      anon_sym_RPAREN,
    STATE(2603), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96032] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5116), 1,
      sym__end_of_statement,
    STATE(2604), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96046] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5118), 1,
      sym__end_of_statement,
    STATE(2605), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96060] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5120), 1,
      anon_sym_SLASH,
    STATE(2606), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96074] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5122), 1,
      anon_sym_RPAREN,
    STATE(2607), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96088] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5124), 1,
      sym__end_of_statement,
    STATE(2608), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96102] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5126), 1,
      sym__end_of_statement,
    STATE(2609), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96116] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5128), 1,
      aux_sym_stop_statement_token3,
    STATE(2610), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96130] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5130), 1,
      sym__end_of_statement,
    STATE(2611), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96144] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4878), 1,
      sym__end_of_statement,
    STATE(2612), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96158] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1774), 1,
      sym__end_of_statement,
    STATE(2613), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96172] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5132), 1,
      sym__end_of_statement,
    STATE(2614), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96186] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5134), 1,
      anon_sym_SLASH,
    STATE(2615), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96200] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5136), 1,
      sym__end_of_statement,
    STATE(2616), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96214] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5138), 1,
      sym__end_of_statement,
    STATE(2617), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96228] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3009), 1,
      sym__end_of_statement,
    STATE(2618), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96242] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5140), 1,
      anon_sym_COLON,
    STATE(2619), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96256] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3017), 1,
      sym__end_of_statement,
    STATE(2620), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96270] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5142), 1,
      sym__end_of_statement,
    STATE(2621), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96284] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2840), 1,
      sym__end_of_statement,
    STATE(2622), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96298] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5144), 1,
      anon_sym_RPAREN,
    STATE(2623), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96312] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5146), 1,
      anon_sym_RPAREN,
    STATE(2624), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96326] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4364), 1,
      sym__end_of_statement,
    STATE(2625), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96340] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5148), 1,
      sym__end_of_statement,
    STATE(2626), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96354] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5150), 1,
      sym__end_of_statement,
    STATE(2627), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96368] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5152), 1,
      sym__end_of_statement,
    STATE(2628), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96382] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4568), 1,
      anon_sym_RPAREN,
    STATE(2629), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96396] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4898), 1,
      sym__end_of_statement,
    STATE(2630), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96410] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5154), 1,
      anon_sym_RPAREN,
    STATE(2631), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96424] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5156), 1,
      sym__end_of_statement,
    STATE(2632), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96438] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5158), 1,
      anon_sym_EQ,
    STATE(2633), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96452] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5160), 1,
      sym__end_of_statement,
    STATE(2634), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96466] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5162), 1,
      sym__end_of_statement,
    STATE(2635), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96480] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5164), 1,
      sym__end_of_statement,
    STATE(2636), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96494] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5166), 1,
      aux_sym_type_statement_token1,
    STATE(2637), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96508] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5168), 1,
      anon_sym_EQ,
    STATE(2638), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96522] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5170), 1,
      anon_sym_LPAREN,
    STATE(2639), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96536] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(1762), 1,
      sym__end_of_statement,
    STATE(2640), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96550] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5172), 1,
      anon_sym_EQ,
    STATE(2641), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96564] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5174), 1,
      sym__end_of_statement,
    STATE(2642), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96578] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5176), 1,
      sym__end_of_statement,
    STATE(2643), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96592] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5178), 1,
      sym__end_of_statement,
    STATE(2644), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96606] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5180), 1,
      anon_sym_RPAREN,
    STATE(2645), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96620] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5182), 1,
      aux_sym_stop_statement_token3,
    STATE(2646), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96634] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5184), 1,
      sym__end_of_statement,
    STATE(2647), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96648] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5186), 1,
      anon_sym_LPAREN,
    STATE(2648), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96662] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5188), 1,
      sym__end_of_statement,
    STATE(2649), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96676] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5190), 1,
      anon_sym_EQ,
    STATE(2650), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96690] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5192), 1,
      sym__end_of_statement,
    STATE(2651), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96704] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5194), 1,
      anon_sym_RPAREN,
    STATE(2652), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96718] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5196), 1,
      sym__end_of_statement,
    STATE(2653), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96732] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5198), 1,
      sym__end_of_statement,
    STATE(2654), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96746] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3277), 1,
      sym__end_of_statement,
    STATE(2655), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96760] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5200), 1,
      sym__end_of_statement,
    STATE(2656), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96774] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5202), 1,
      sym__end_of_statement,
    STATE(2657), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96788] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5204), 1,
      sym__end_of_statement,
    STATE(2658), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96802] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5206), 1,
      sym__end_of_statement,
    STATE(2659), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96816] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5208), 1,
      sym__end_of_statement,
    STATE(2660), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96830] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5210), 1,
      anon_sym_EQ_GT,
    STATE(2661), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96844] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5212), 1,
      sym__end_of_statement,
    STATE(2662), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96858] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5214), 1,
      sym__end_of_statement,
    STATE(2663), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96872] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5216), 1,
      sym__end_of_statement,
    STATE(2664), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96886] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5218), 1,
      anon_sym_LPAREN,
    STATE(2665), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96900] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5220), 1,
      sym__end_of_statement,
    STATE(2666), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96914] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2737), 1,
      sym__end_of_statement,
    STATE(2667), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96928] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5222), 1,
      sym__end_of_statement,
    STATE(2668), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96942] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5224), 1,
      sym__end_of_statement,
    STATE(2669), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96956] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5226), 1,
      sym__end_of_statement,
    STATE(2670), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96970] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5228), 1,
      sym__end_of_statement,
    STATE(2671), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96984] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5230), 1,
      sym__end_of_statement,
    STATE(2672), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [96998] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5232), 1,
      anon_sym_LPAREN,
    STATE(2673), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97012] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5234), 1,
      sym__end_of_statement,
    STATE(2674), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97026] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5236), 1,
      sym__end_of_statement,
    STATE(2675), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97040] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5238), 1,
      anon_sym_LPAREN,
    STATE(2676), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97054] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5240), 1,
      anon_sym_EQ_GT,
    STATE(2677), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97068] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5242), 1,
      sym__end_of_statement,
    STATE(2678), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97082] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5244), 1,
      aux_sym__inline_if_statement_token1,
    STATE(2679), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97096] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5246), 1,
      sym__end_of_statement,
    STATE(2680), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97110] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5008), 1,
      sym__end_of_statement,
    STATE(2681), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97124] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5248), 1,
      sym__end_of_statement,
    STATE(2682), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97138] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5250), 1,
      sym__end_of_statement,
    STATE(2683), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97152] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5252), 1,
      sym__end_of_statement,
    STATE(2684), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97166] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5254), 1,
      anon_sym_COMMA,
    STATE(2685), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97180] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5256), 1,
      sym__end_of_statement,
    STATE(2686), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97194] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5258), 1,
      sym__end_of_statement,
    STATE(2687), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97208] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5260), 1,
      anon_sym_EQ,
    STATE(2688), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97222] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5262), 1,
      sym__end_of_statement,
    STATE(2689), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97236] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5264), 1,
      sym__end_of_statement,
    STATE(2690), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97250] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5266), 1,
      anon_sym_LPAREN,
    STATE(2691), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97264] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5268), 1,
      sym__end_of_statement,
    STATE(2692), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97278] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5270), 1,
      sym__end_of_statement,
    STATE(2693), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97292] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5272), 1,
      sym__end_of_statement,
    STATE(2694), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97306] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5274), 1,
      sym__end_of_statement,
    STATE(2695), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97320] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5030), 1,
      sym__end_of_statement,
    STATE(2696), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97334] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5276), 1,
      sym__end_of_statement,
    STATE(2697), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97348] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5278), 1,
      sym__end_of_statement,
    STATE(2698), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97362] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5280), 1,
      aux_sym_implicit_range_token1,
    STATE(2699), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97376] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5282), 1,
      sym__end_of_statement,
    STATE(2700), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97390] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5284), 1,
      anon_sym_SLASH,
    STATE(2701), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97404] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5286), 1,
      sym__end_of_statement,
    STATE(2702), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97418] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5288), 1,
      sym__end_of_statement,
    STATE(2703), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97432] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5290), 1,
      sym__end_of_statement,
    STATE(2704), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97446] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5292), 1,
      sym__end_of_statement,
    STATE(2705), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97460] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5294), 1,
      sym__end_of_statement,
    STATE(2706), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97474] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5296), 1,
      sym__end_of_statement,
    STATE(2707), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97488] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5298), 1,
      sym__end_of_statement,
    STATE(2708), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97502] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2733), 1,
      anon_sym_RPAREN,
    STATE(2709), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97516] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5300), 1,
      sym__end_of_statement,
    STATE(2710), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97530] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5302), 1,
      anon_sym_STAR,
    STATE(2711), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97544] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5304), 1,
      sym__end_of_statement,
    STATE(2712), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97558] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5306), 1,
      anon_sym_RPAREN,
    STATE(2713), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97572] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3982), 1,
      anon_sym_EQ_GT,
    STATE(2714), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97586] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5308), 1,
      sym__end_of_statement,
    STATE(2715), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97600] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5310), 1,
      sym__end_of_statement,
    STATE(2716), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97614] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5312), 1,
      sym__end_of_statement,
    STATE(2717), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97628] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5314), 1,
      sym__end_of_statement,
    STATE(2718), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97642] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5316), 1,
      sym__end_of_statement,
    STATE(2719), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97656] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5318), 1,
      sym__end_of_statement,
    STATE(2720), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97670] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5320), 1,
      sym__end_of_statement,
    STATE(2721), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97684] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5322), 1,
      anon_sym_COLON,
    STATE(2722), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97698] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5324), 1,
      sym__end_of_statement,
    STATE(2723), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97712] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5326), 1,
      sym__end_of_statement,
    STATE(2724), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97726] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5328), 1,
      sym__end_of_statement,
    STATE(2725), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97740] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5330), 1,
      sym__end_of_statement,
    STATE(2726), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97754] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5332), 1,
      sym__end_of_statement,
    STATE(2727), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97768] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5334), 1,
      sym__end_of_statement,
    STATE(2728), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97782] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5336), 1,
      anon_sym_EQ_GT,
    STATE(2729), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97796] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5338), 1,
      anon_sym_RPAREN,
    STATE(2730), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97810] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5340), 1,
      sym__end_of_statement,
    STATE(2731), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97824] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5342), 1,
      sym__end_of_statement,
    STATE(2732), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97838] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5344), 1,
      sym__end_of_statement,
    STATE(2733), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97852] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5346), 1,
      sym__end_of_statement,
    STATE(2734), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97866] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5348), 1,
      anon_sym_RPAREN,
    STATE(2735), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97880] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5350), 1,
      sym__end_of_statement,
    STATE(2736), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97894] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5352), 1,
      sym__end_of_statement,
    STATE(2737), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97908] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5354), 1,
      sym__end_of_statement,
    STATE(2738), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97922] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5356), 1,
      sym__end_of_statement,
    STATE(2739), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97936] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5358), 1,
      aux_sym_select_case_statement_token1,
    STATE(2740), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97950] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5360), 1,
      sym__end_of_statement,
    STATE(2741), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97964] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5362), 1,
      anon_sym_LPAREN,
    STATE(2742), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97978] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5364), 1,
      sym__end_of_statement,
    STATE(2743), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [97992] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5366), 1,
      anon_sym_LPAREN,
    STATE(2744), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98006] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5368), 1,
      sym__end_of_statement,
    STATE(2745), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98020] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5370), 1,
      anon_sym_COMMA,
    STATE(2746), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98034] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5372), 1,
      sym__end_of_statement,
    STATE(2747), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98048] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5374), 1,
      aux_sym_interface_statement_token1,
    STATE(2748), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98062] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3200), 1,
      sym__end_of_statement,
    STATE(2749), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98076] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5376), 1,
      sym__end_of_statement,
    STATE(2750), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98090] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5378), 1,
      sym__end_of_statement,
    STATE(2751), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98104] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5380), 1,
      sym__end_of_statement,
    STATE(2752), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98118] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5382), 1,
      aux_sym_preproc_file_line_token1,
    STATE(2753), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98132] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5384), 1,
      sym__end_of_statement,
    STATE(2754), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98146] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2978), 1,
      anon_sym_LPAREN,
    STATE(2755), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98160] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5386), 1,
      sym__end_of_statement,
    STATE(2756), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98174] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5388), 1,
      sym__end_of_statement,
    STATE(2757), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98188] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5390), 1,
      sym__end_of_statement,
    STATE(2758), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98202] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5392), 1,
      sym__end_of_statement,
    STATE(2759), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98216] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5394), 1,
      sym__end_of_statement,
    STATE(2760), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98230] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5396), 1,
      sym__end_of_statement,
    STATE(2761), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98244] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5398), 1,
      sym__end_of_statement,
    STATE(2762), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98258] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5400), 1,
      sym__end_of_statement,
    STATE(2763), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98272] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5402), 1,
      sym__end_of_statement,
    STATE(2764), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98286] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5404), 1,
      sym__end_of_statement,
    STATE(2765), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98300] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5406), 1,
      sym__end_of_statement,
    STATE(2766), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98314] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5408), 1,
      sym__end_of_statement,
    STATE(2767), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98328] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5410), 1,
      sym__end_of_statement,
    STATE(2768), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98342] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5412), 1,
      sym__end_of_statement,
    STATE(2769), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98356] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5414), 1,
      sym__end_of_statement,
    STATE(2770), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98370] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5416), 1,
      sym__end_of_statement,
    STATE(2771), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98384] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5418), 1,
      sym__end_of_statement,
    STATE(2772), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98398] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5420), 1,
      sym__end_of_statement,
    STATE(2773), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98412] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5422), 1,
      sym__end_of_statement,
    STATE(2774), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98426] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5424), 1,
      sym__end_of_statement,
    STATE(2775), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98440] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5426), 1,
      sym__end_of_statement,
    STATE(2776), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98454] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5428), 1,
      sym__end_of_statement,
    STATE(2777), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98468] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5430), 1,
      sym__end_of_statement,
    STATE(2778), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98482] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5432), 1,
      sym__end_of_statement,
    STATE(2779), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98496] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5434), 1,
      sym__end_of_statement,
    STATE(2780), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98510] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5436), 1,
      sym__end_of_statement,
    STATE(2781), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98524] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5438), 1,
      sym__end_of_statement,
    STATE(2782), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98538] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5440), 1,
      sym__end_of_statement,
    STATE(2783), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98552] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5442), 1,
      sym__end_of_statement,
    STATE(2784), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98566] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5444), 1,
      sym__end_of_statement,
    STATE(2785), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98580] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5446), 1,
      sym__end_of_statement,
    STATE(2786), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98594] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2976), 1,
      sym__end_of_statement,
    STATE(2787), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98608] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5448), 1,
      sym__end_of_statement,
    STATE(2788), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98622] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5450), 1,
      aux_sym_interface_statement_token1,
    STATE(2789), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98636] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5452), 1,
      sym__end_of_statement,
    STATE(2790), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98650] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5454), 1,
      sym__end_of_statement,
    STATE(2791), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98664] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5456), 1,
      sym__end_of_statement,
    STATE(2792), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98678] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5458), 1,
      aux_sym_size_token1,
    STATE(2793), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98692] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5460), 1,
      anon_sym_EQ,
    STATE(2794), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98706] = 4,
    ACTIONS(4554), 1,
      anon_sym_POUND,
    ACTIONS(5462), 1,
      aux_sym_operator_token2,
    STATE(2795), 1,
      sym_preproc_file_line,
    ACTIONS(4556), 2,
      sym_comment,
      sym__line_continuation,
  [98720] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5464), 1,
      anon_sym_RPAREN,
    STATE(2796), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98734] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5466), 1,
      anon_sym_SLASH,
    STATE(2797), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98748] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(4904), 1,
      sym__end_of_statement,
    STATE(2798), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98762] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5468), 1,
      sym__end_of_statement,
    STATE(2799), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98776] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5470), 1,
      sym__end_of_statement,
    STATE(2800), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98790] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5472), 1,
      sym__end_of_statement,
    STATE(2801), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98804] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5474), 1,
      sym__end_of_statement,
    STATE(2802), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98818] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5476), 1,
      anon_sym_RPAREN,
    STATE(2803), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98832] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5478), 1,
      anon_sym_RPAREN,
    STATE(2804), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98846] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5480), 1,
      sym__end_of_statement,
    STATE(2805), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98860] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5482), 1,
      sym__end_of_statement,
    STATE(2806), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98874] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5484), 1,
      sym__end_of_statement,
    STATE(2807), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98888] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5486), 1,
      ts_builtin_sym_end,
    STATE(2808), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98902] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5488), 1,
      aux_sym_user_defined_operator_token1,
    STATE(2809), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98916] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5490), 1,
      anon_sym_RPAREN,
    STATE(2810), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98930] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5492), 1,
      anon_sym_COMMA,
    STATE(2811), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98944] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5494), 1,
      anon_sym_EQ_GT,
    STATE(2812), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98958] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5496), 1,
      anon_sym_EQ_GT,
    STATE(2813), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98972] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5498), 1,
      sym__end_of_statement,
    STATE(2814), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [98986] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5500), 1,
      sym__end_of_statement,
    STATE(2815), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99000] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5502), 1,
      anon_sym_LPAREN,
    STATE(2816), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99014] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5504), 1,
      anon_sym_RPAREN,
    STATE(2817), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99028] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5506), 1,
      sym__end_of_statement,
    STATE(2818), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99042] = 4,
    ACTIONS(4554), 1,
      anon_sym_POUND,
    ACTIONS(5508), 1,
      sym__newline,
    STATE(2819), 1,
      sym_preproc_file_line,
    ACTIONS(4556), 2,
      sym_comment,
      sym__line_continuation,
  [99056] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5510), 1,
      anon_sym_LPAREN,
    STATE(2820), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99070] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5512), 1,
      anon_sym_LPAREN,
    STATE(2821), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99084] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5514), 1,
      sym__end_of_statement,
    STATE(2822), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99098] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5516), 1,
      sym__end_of_statement,
    STATE(2823), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99112] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5518), 1,
      sym__end_of_statement,
    STATE(2824), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99126] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5520), 1,
      sym__end_of_statement,
    STATE(2825), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99140] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2926), 1,
      anon_sym_EQ_GT,
    STATE(2826), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99154] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5522), 1,
      anon_sym_EQ_GT,
    STATE(2827), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99168] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5524), 1,
      sym__end_of_statement,
    STATE(2828), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99182] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5526), 1,
      anon_sym_LPAREN,
    STATE(2829), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99196] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5528), 1,
      sym__end_of_statement,
    STATE(2830), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99210] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5530), 1,
      aux_sym_interface_statement_token1,
    STATE(2831), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99224] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5532), 1,
      sym__end_of_statement,
    STATE(2832), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99238] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5534), 1,
      sym__end_of_statement,
    STATE(2833), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99252] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5536), 1,
      anon_sym_EQ,
    STATE(2834), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99266] = 4,
    ACTIONS(4554), 1,
      anon_sym_POUND,
    ACTIONS(5538), 1,
      aux_sym_operator_token2,
    STATE(2835), 1,
      sym_preproc_file_line,
    ACTIONS(4556), 2,
      sym_comment,
      sym__line_continuation,
  [99280] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5540), 1,
      anon_sym_RPAREN,
    STATE(2836), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99294] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5542), 1,
      sym__end_of_statement,
    STATE(2837), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99308] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5544), 1,
      sym__end_of_statement,
    STATE(2838), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99322] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5546), 1,
      sym__end_of_statement,
    STATE(2839), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99336] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5548), 1,
      anon_sym_RPAREN,
    STATE(2840), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99350] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5550), 1,
      sym__end_of_statement,
    STATE(2841), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99364] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5552), 1,
      sym__end_of_statement,
    STATE(2842), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99378] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5554), 1,
      sym__end_of_statement,
    STATE(2843), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99392] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5556), 1,
      sym__end_of_statement,
    STATE(2844), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99406] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5558), 1,
      sym__end_of_statement,
    STATE(2845), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99420] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5560), 1,
      aux_sym_interface_statement_token1,
    STATE(2846), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99434] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5562), 1,
      sym__end_of_statement,
    STATE(2847), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99448] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5564), 1,
      sym__end_of_statement,
    STATE(2848), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99462] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5566), 1,
      sym__end_of_statement,
    STATE(2849), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99476] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5568), 1,
      sym__end_of_statement,
    STATE(2850), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99490] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5570), 1,
      sym__end_of_statement,
    STATE(2851), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99504] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5572), 1,
      sym_string_literal,
    STATE(2852), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99518] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5574), 1,
      sym__end_of_statement,
    STATE(2853), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99532] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5576), 1,
      sym__end_of_statement,
    STATE(2854), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99546] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5578), 1,
      aux_sym__block_if_statement_token1,
    STATE(2855), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99560] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5580), 1,
      sym__end_of_statement,
    STATE(2856), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99574] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5582), 1,
      sym__end_of_statement,
    STATE(2857), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99588] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5584), 1,
      aux_sym_interface_statement_token1,
    STATE(2858), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99602] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5586), 1,
      sym__end_of_statement,
    STATE(2859), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99616] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5222), 1,
      anon_sym_RPAREN,
    STATE(2860), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99630] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5588), 1,
      sym__end_of_statement,
    STATE(2861), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99644] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5590), 1,
      sym__end_of_statement,
    STATE(2862), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99658] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5592), 1,
      sym__end_of_statement,
    STATE(2863), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99672] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5594), 1,
      aux_sym_keyword_statement_token5,
    STATE(2864), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99686] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5596), 1,
      sym__end_of_statement,
    STATE(2865), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99700] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5598), 1,
      sym__end_of_statement,
    STATE(2866), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99714] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5600), 1,
      sym__end_of_statement,
    STATE(2867), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99728] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5602), 1,
      sym__end_of_statement,
    STATE(2868), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99742] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5604), 1,
      aux_sym_interface_statement_token1,
    STATE(2869), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99756] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5606), 1,
      sym__end_of_statement,
    STATE(2870), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99770] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5608), 1,
      sym__end_of_statement,
    STATE(2871), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99784] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5610), 1,
      sym__end_of_statement,
    STATE(2872), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99798] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5612), 1,
      sym__end_of_statement,
    STATE(2873), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99812] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5614), 1,
      sym__end_of_statement,
    STATE(2874), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99826] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2008), 1,
      sym__end_of_statement,
    STATE(2875), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99840] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5616), 1,
      sym__end_of_statement,
    STATE(2876), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99854] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5618), 1,
      sym__end_of_statement,
    STATE(2877), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99868] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5620), 1,
      sym__end_of_statement,
    STATE(2878), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99882] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5622), 1,
      sym__end_of_statement,
    STATE(2879), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99896] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5624), 1,
      aux_sym_interface_statement_token1,
    STATE(2880), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99910] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5626), 1,
      sym__end_of_statement,
    STATE(2881), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99924] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5628), 1,
      anon_sym_EQ_GT,
    STATE(2882), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99938] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5630), 1,
      sym__end_of_statement,
    STATE(2883), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99952] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5632), 1,
      sym__end_of_statement,
    STATE(2884), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99966] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5634), 1,
      sym__end_of_statement,
    STATE(2885), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99980] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5636), 1,
      sym__end_of_statement,
    STATE(2886), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [99994] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5638), 1,
      sym__end_of_statement,
    STATE(2887), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100008] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5640), 1,
      aux_sym_interface_statement_token1,
    STATE(2888), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100022] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3501), 1,
      aux_sym_module_procedure_statement_token1,
    STATE(2889), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100036] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5642), 1,
      sym__end_of_statement,
    STATE(2890), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100050] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5644), 1,
      anon_sym_LPAREN,
    STATE(2891), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100064] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5646), 1,
      anon_sym_LPAREN,
    STATE(2892), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100078] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5648), 1,
      sym__end_of_statement,
    STATE(2893), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100092] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5650), 1,
      sym__end_of_statement,
    STATE(2894), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100106] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5652), 1,
      anon_sym_LPAREN,
    STATE(2895), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100120] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5654), 1,
      anon_sym_LPAREN,
    STATE(2896), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100134] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5656), 1,
      anon_sym_LPAREN,
    STATE(2897), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100148] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5658), 1,
      anon_sym_RPAREN,
    STATE(2898), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100162] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5660), 1,
      sym__end_of_statement,
    STATE(2899), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100176] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5662), 1,
      anon_sym_LPAREN,
    STATE(2900), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100190] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5664), 1,
      anon_sym_LPAREN,
    STATE(2901), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100204] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5666), 1,
      sym__end_of_statement,
    STATE(2902), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100218] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(2999), 1,
      anon_sym_LPAREN,
    STATE(2903), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100232] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5668), 1,
      anon_sym_EQ_GT,
    STATE(2904), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100246] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5670), 1,
      sym__end_of_statement,
    STATE(2905), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100260] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5672), 1,
      sym__end_of_statement,
    STATE(2906), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100274] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5674), 1,
      anon_sym_EQ_GT,
    STATE(2907), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100288] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(3624), 1,
      sym__end_of_statement,
    STATE(2908), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100302] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5676), 1,
      anon_sym_LPAREN,
    STATE(2909), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100316] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5678), 1,
      sym__end_of_statement,
    STATE(2910), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100330] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5680), 1,
      sym__end_of_statement,
    STATE(2911), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100344] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5682), 1,
      sym__end_of_statement,
    STATE(2912), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100358] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5684), 1,
      sym__end_of_statement,
    STATE(2913), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100372] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5686), 1,
      anon_sym_LPAREN,
    STATE(2914), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100386] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5688), 1,
      anon_sym_LPAREN,
    STATE(2915), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100400] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5690), 1,
      anon_sym_LPAREN,
    STATE(2916), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100414] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5692), 1,
      anon_sym_LPAREN,
    STATE(2917), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100428] = 4,
    ACTIONS(3), 1,
      anon_sym_POUND,
    ACTIONS(5694), 1,
      anon_sym_LPAREN,
    STATE(2918), 1,
      sym_preproc_file_line,
    ACTIONS(5), 2,
      sym_comment,
      sym__line_continuation,
  [100442] = 1,
    ACTIONS(5696), 1,
      ts_builtin_sym_end,
  [100446] = 1,
    ACTIONS(5698), 1,
      ts_builtin_sym_end,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(447)] = 0,
  [SMALL_STATE(448)] = 75,
  [SMALL_STATE(449)] = 150,
  [SMALL_STATE(450)] = 225,
  [SMALL_STATE(451)] = 308,
  [SMALL_STATE(452)] = 383,
  [SMALL_STATE(453)] = 458,
  [SMALL_STATE(454)] = 533,
  [SMALL_STATE(455)] = 610,
  [SMALL_STATE(456)] = 685,
  [SMALL_STATE(457)] = 760,
  [SMALL_STATE(458)] = 835,
  [SMALL_STATE(459)] = 910,
  [SMALL_STATE(460)] = 985,
  [SMALL_STATE(461)] = 1060,
  [SMALL_STATE(462)] = 1135,
  [SMALL_STATE(463)] = 1210,
  [SMALL_STATE(464)] = 1285,
  [SMALL_STATE(465)] = 1360,
  [SMALL_STATE(466)] = 1435,
  [SMALL_STATE(467)] = 1553,
  [SMALL_STATE(468)] = 1671,
  [SMALL_STATE(469)] = 1789,
  [SMALL_STATE(470)] = 1907,
  [SMALL_STATE(471)] = 2025,
  [SMALL_STATE(472)] = 2143,
  [SMALL_STATE(473)] = 2261,
  [SMALL_STATE(474)] = 2379,
  [SMALL_STATE(475)] = 2497,
  [SMALL_STATE(476)] = 2612,
  [SMALL_STATE(477)] = 2735,
  [SMALL_STATE(478)] = 2854,
  [SMALL_STATE(479)] = 2973,
  [SMALL_STATE(480)] = 3085,
  [SMALL_STATE(481)] = 3203,
  [SMALL_STATE(482)] = 3275,
  [SMALL_STATE(483)] = 3391,
  [SMALL_STATE(484)] = 3503,
  [SMALL_STATE(485)] = 3575,
  [SMALL_STATE(486)] = 3646,
  [SMALL_STATE(487)] = 3717,
  [SMALL_STATE(488)] = 3788,
  [SMALL_STATE(489)] = 3859,
  [SMALL_STATE(490)] = 3930,
  [SMALL_STATE(491)] = 4001,
  [SMALL_STATE(492)] = 4122,
  [SMALL_STATE(493)] = 4231,
  [SMALL_STATE(494)] = 4302,
  [SMALL_STATE(495)] = 4421,
  [SMALL_STATE(496)] = 4540,
  [SMALL_STATE(497)] = 4657,
  [SMALL_STATE(498)] = 4728,
  [SMALL_STATE(499)] = 4849,
  [SMALL_STATE(500)] = 4920,
  [SMALL_STATE(501)] = 4991,
  [SMALL_STATE(502)] = 5062,
  [SMALL_STATE(503)] = 5133,
  [SMALL_STATE(504)] = 5203,
  [SMALL_STATE(505)] = 5313,
  [SMALL_STATE(506)] = 5391,
  [SMALL_STATE(507)] = 5501,
  [SMALL_STATE(508)] = 5571,
  [SMALL_STATE(509)] = 5641,
  [SMALL_STATE(510)] = 5711,
  [SMALL_STATE(511)] = 5781,
  [SMALL_STATE(512)] = 5851,
  [SMALL_STATE(513)] = 5921,
  [SMALL_STATE(514)] = 5991,
  [SMALL_STATE(515)] = 6061,
  [SMALL_STATE(516)] = 6131,
  [SMALL_STATE(517)] = 6201,
  [SMALL_STATE(518)] = 6271,
  [SMALL_STATE(519)] = 6379,
  [SMALL_STATE(520)] = 6449,
  [SMALL_STATE(521)] = 6519,
  [SMALL_STATE(522)] = 6627,
  [SMALL_STATE(523)] = 6697,
  [SMALL_STATE(524)] = 6767,
  [SMALL_STATE(525)] = 6875,
  [SMALL_STATE(526)] = 6985,
  [SMALL_STATE(527)] = 7055,
  [SMALL_STATE(528)] = 7125,
  [SMALL_STATE(529)] = 7235,
  [SMALL_STATE(530)] = 7305,
  [SMALL_STATE(531)] = 7410,
  [SMALL_STATE(532)] = 7479,
  [SMALL_STATE(533)] = 7564,
  [SMALL_STATE(534)] = 7669,
  [SMALL_STATE(535)] = 7776,
  [SMALL_STATE(536)] = 7881,
  [SMALL_STATE(537)] = 7966,
  [SMALL_STATE(538)] = 8071,
  [SMALL_STATE(539)] = 8156,
  [SMALL_STATE(540)] = 8259,
  [SMALL_STATE(541)] = 8344,
  [SMALL_STATE(542)] = 8429,
  [SMALL_STATE(543)] = 8514,
  [SMALL_STATE(544)] = 8599,
  [SMALL_STATE(545)] = 8684,
  [SMALL_STATE(546)] = 8769,
  [SMALL_STATE(547)] = 8838,
  [SMALL_STATE(548)] = 8923,
  [SMALL_STATE(549)] = 9008,
  [SMALL_STATE(550)] = 9093,
  [SMALL_STATE(551)] = 9178,
  [SMALL_STATE(552)] = 9283,
  [SMALL_STATE(553)] = 9368,
  [SMALL_STATE(554)] = 9453,
  [SMALL_STATE(555)] = 9538,
  [SMALL_STATE(556)] = 9623,
  [SMALL_STATE(557)] = 9708,
  [SMALL_STATE(558)] = 9793,
  [SMALL_STATE(559)] = 9878,
  [SMALL_STATE(560)] = 9983,
  [SMALL_STATE(561)] = 10068,
  [SMALL_STATE(562)] = 10153,
  [SMALL_STATE(563)] = 10238,
  [SMALL_STATE(564)] = 10342,
  [SMALL_STATE(565)] = 10410,
  [SMALL_STATE(566)] = 10478,
  [SMALL_STATE(567)] = 10546,
  [SMALL_STATE(568)] = 10650,
  [SMALL_STATE(569)] = 10754,
  [SMALL_STATE(570)] = 10822,
  [SMALL_STATE(571)] = 10890,
  [SMALL_STATE(572)] = 10994,
  [SMALL_STATE(573)] = 11096,
  [SMALL_STATE(574)] = 11200,
  [SMALL_STATE(575)] = 11306,
  [SMALL_STATE(576)] = 11374,
  [SMALL_STATE(577)] = 11480,
  [SMALL_STATE(578)] = 11548,
  [SMALL_STATE(579)] = 11616,
  [SMALL_STATE(580)] = 11720,
  [SMALL_STATE(581)] = 11788,
  [SMALL_STATE(582)] = 11856,
  [SMALL_STATE(583)] = 11924,
  [SMALL_STATE(584)] = 11992,
  [SMALL_STATE(585)] = 12060,
  [SMALL_STATE(586)] = 12128,
  [SMALL_STATE(587)] = 12196,
  [SMALL_STATE(588)] = 12284,
  [SMALL_STATE(589)] = 12352,
  [SMALL_STATE(590)] = 12420,
  [SMALL_STATE(591)] = 12488,
  [SMALL_STATE(592)] = 12576,
  [SMALL_STATE(593)] = 12644,
  [SMALL_STATE(594)] = 12712,
  [SMALL_STATE(595)] = 12780,
  [SMALL_STATE(596)] = 12848,
  [SMALL_STATE(597)] = 12916,
  [SMALL_STATE(598)] = 13022,
  [SMALL_STATE(599)] = 13124,
  [SMALL_STATE(600)] = 13192,
  [SMALL_STATE(601)] = 13260,
  [SMALL_STATE(602)] = 13362,
  [SMALL_STATE(603)] = 13464,
  [SMALL_STATE(604)] = 13532,
  [SMALL_STATE(605)] = 13600,
  [SMALL_STATE(606)] = 13704,
  [SMALL_STATE(607)] = 13772,
  [SMALL_STATE(608)] = 13840,
  [SMALL_STATE(609)] = 13908,
  [SMALL_STATE(610)] = 13976,
  [SMALL_STATE(611)] = 14044,
  [SMALL_STATE(612)] = 14112,
  [SMALL_STATE(613)] = 14180,
  [SMALL_STATE(614)] = 14282,
  [SMALL_STATE(615)] = 14386,
  [SMALL_STATE(616)] = 14454,
  [SMALL_STATE(617)] = 14536,
  [SMALL_STATE(618)] = 14604,
  [SMALL_STATE(619)] = 14672,
  [SMALL_STATE(620)] = 14740,
  [SMALL_STATE(621)] = 14808,
  [SMALL_STATE(622)] = 14876,
  [SMALL_STATE(623)] = 14982,
  [SMALL_STATE(624)] = 15050,
  [SMALL_STATE(625)] = 15118,
  [SMALL_STATE(626)] = 15186,
  [SMALL_STATE(627)] = 15288,
  [SMALL_STATE(628)] = 15356,
  [SMALL_STATE(629)] = 15424,
  [SMALL_STATE(630)] = 15492,
  [SMALL_STATE(631)] = 15560,
  [SMALL_STATE(632)] = 15628,
  [SMALL_STATE(633)] = 15696,
  [SMALL_STATE(634)] = 15764,
  [SMALL_STATE(635)] = 15868,
  [SMALL_STATE(636)] = 15936,
  [SMALL_STATE(637)] = 16004,
  [SMALL_STATE(638)] = 16072,
  [SMALL_STATE(639)] = 16140,
  [SMALL_STATE(640)] = 16208,
  [SMALL_STATE(641)] = 16276,
  [SMALL_STATE(642)] = 16344,
  [SMALL_STATE(643)] = 16412,
  [SMALL_STATE(644)] = 16480,
  [SMALL_STATE(645)] = 16548,
  [SMALL_STATE(646)] = 16616,
  [SMALL_STATE(647)] = 16684,
  [SMALL_STATE(648)] = 16752,
  [SMALL_STATE(649)] = 16856,
  [SMALL_STATE(650)] = 16957,
  [SMALL_STATE(651)] = 17056,
  [SMALL_STATE(652)] = 17155,
  [SMALL_STATE(653)] = 17254,
  [SMALL_STATE(654)] = 17321,
  [SMALL_STATE(655)] = 17420,
  [SMALL_STATE(656)] = 17519,
  [SMALL_STATE(657)] = 17618,
  [SMALL_STATE(658)] = 17717,
  [SMALL_STATE(659)] = 17818,
  [SMALL_STATE(660)] = 17917,
  [SMALL_STATE(661)] = 18014,
  [SMALL_STATE(662)] = 18113,
  [SMALL_STATE(663)] = 18212,
  [SMALL_STATE(664)] = 18311,
  [SMALL_STATE(665)] = 18410,
  [SMALL_STATE(666)] = 18509,
  [SMALL_STATE(667)] = 18608,
  [SMALL_STATE(668)] = 18707,
  [SMALL_STATE(669)] = 18806,
  [SMALL_STATE(670)] = 18905,
  [SMALL_STATE(671)] = 19004,
  [SMALL_STATE(672)] = 19103,
  [SMALL_STATE(673)] = 19202,
  [SMALL_STATE(674)] = 19301,
  [SMALL_STATE(675)] = 19400,
  [SMALL_STATE(676)] = 19499,
  [SMALL_STATE(677)] = 19598,
  [SMALL_STATE(678)] = 19697,
  [SMALL_STATE(679)] = 19796,
  [SMALL_STATE(680)] = 19895,
  [SMALL_STATE(681)] = 19994,
  [SMALL_STATE(682)] = 20093,
  [SMALL_STATE(683)] = 20194,
  [SMALL_STATE(684)] = 20293,
  [SMALL_STATE(685)] = 20360,
  [SMALL_STATE(686)] = 20427,
  [SMALL_STATE(687)] = 20526,
  [SMALL_STATE(688)] = 20625,
  [SMALL_STATE(689)] = 20724,
  [SMALL_STATE(690)] = 20821,
  [SMALL_STATE(691)] = 20922,
  [SMALL_STATE(692)] = 21021,
  [SMALL_STATE(693)] = 21120,
  [SMALL_STATE(694)] = 21219,
  [SMALL_STATE(695)] = 21318,
  [SMALL_STATE(696)] = 21385,
  [SMALL_STATE(697)] = 21452,
  [SMALL_STATE(698)] = 21519,
  [SMALL_STATE(699)] = 21618,
  [SMALL_STATE(700)] = 21717,
  [SMALL_STATE(701)] = 21784,
  [SMALL_STATE(702)] = 21881,
  [SMALL_STATE(703)] = 21982,
  [SMALL_STATE(704)] = 22083,
  [SMALL_STATE(705)] = 22184,
  [SMALL_STATE(706)] = 22283,
  [SMALL_STATE(707)] = 22350,
  [SMALL_STATE(708)] = 22449,
  [SMALL_STATE(709)] = 22516,
  [SMALL_STATE(710)] = 22615,
  [SMALL_STATE(711)] = 22714,
  [SMALL_STATE(712)] = 22813,
  [SMALL_STATE(713)] = 22914,
  [SMALL_STATE(714)] = 23013,
  [SMALL_STATE(715)] = 23112,
  [SMALL_STATE(716)] = 23211,
  [SMALL_STATE(717)] = 23310,
  [SMALL_STATE(718)] = 23409,
  [SMALL_STATE(719)] = 23508,
  [SMALL_STATE(720)] = 23607,
  [SMALL_STATE(721)] = 23706,
  [SMALL_STATE(722)] = 23805,
  [SMALL_STATE(723)] = 23904,
  [SMALL_STATE(724)] = 24003,
  [SMALL_STATE(725)] = 24102,
  [SMALL_STATE(726)] = 24199,
  [SMALL_STATE(727)] = 24298,
  [SMALL_STATE(728)] = 24397,
  [SMALL_STATE(729)] = 24496,
  [SMALL_STATE(730)] = 24595,
  [SMALL_STATE(731)] = 24662,
  [SMALL_STATE(732)] = 24761,
  [SMALL_STATE(733)] = 24860,
  [SMALL_STATE(734)] = 24959,
  [SMALL_STATE(735)] = 25058,
  [SMALL_STATE(736)] = 25157,
  [SMALL_STATE(737)] = 25256,
  [SMALL_STATE(738)] = 25323,
  [SMALL_STATE(739)] = 25422,
  [SMALL_STATE(740)] = 25525,
  [SMALL_STATE(741)] = 25624,
  [SMALL_STATE(742)] = 25723,
  [SMALL_STATE(743)] = 25822,
  [SMALL_STATE(744)] = 25901,
  [SMALL_STATE(745)] = 26000,
  [SMALL_STATE(746)] = 26067,
  [SMALL_STATE(747)] = 26166,
  [SMALL_STATE(748)] = 26265,
  [SMALL_STATE(749)] = 26364,
  [SMALL_STATE(750)] = 26463,
  [SMALL_STATE(751)] = 26566,
  [SMALL_STATE(752)] = 26667,
  [SMALL_STATE(753)] = 26766,
  [SMALL_STATE(754)] = 26831,
  [SMALL_STATE(755)] = 26905,
  [SMALL_STATE(756)] = 27004,
  [SMALL_STATE(757)] = 27103,
  [SMALL_STATE(758)] = 27202,
  [SMALL_STATE(759)] = 27301,
  [SMALL_STATE(760)] = 27400,
  [SMALL_STATE(761)] = 27499,
  [SMALL_STATE(762)] = 27598,
  [SMALL_STATE(763)] = 27697,
  [SMALL_STATE(764)] = 27796,
  [SMALL_STATE(765)] = 27895,
  [SMALL_STATE(766)] = 27994,
  [SMALL_STATE(767)] = 28093,
  [SMALL_STATE(768)] = 28192,
  [SMALL_STATE(769)] = 28291,
  [SMALL_STATE(770)] = 28390,
  [SMALL_STATE(771)] = 28489,
  [SMALL_STATE(772)] = 28583,
  [SMALL_STATE(773)] = 28650,
  [SMALL_STATE(774)] = 28712,
  [SMALL_STATE(775)] = 28768,
  [SMALL_STATE(776)] = 28820,
  [SMALL_STATE(777)] = 28876,
  [SMALL_STATE(778)] = 28932,
  [SMALL_STATE(779)] = 28986,
  [SMALL_STATE(780)] = 29038,
  [SMALL_STATE(781)] = 29092,
  [SMALL_STATE(782)] = 29148,
  [SMALL_STATE(783)] = 29204,
  [SMALL_STATE(784)] = 29260,
  [SMALL_STATE(785)] = 29316,
  [SMALL_STATE(786)] = 29372,
  [SMALL_STATE(787)] = 29424,
  [SMALL_STATE(788)] = 29476,
  [SMALL_STATE(789)] = 29528,
  [SMALL_STATE(790)] = 29584,
  [SMALL_STATE(791)] = 29637,
  [SMALL_STATE(792)] = 29688,
  [SMALL_STATE(793)] = 29739,
  [SMALL_STATE(794)] = 29790,
  [SMALL_STATE(795)] = 29841,
  [SMALL_STATE(796)] = 29892,
  [SMALL_STATE(797)] = 29943,
  [SMALL_STATE(798)] = 29994,
  [SMALL_STATE(799)] = 30045,
  [SMALL_STATE(800)] = 30098,
  [SMALL_STATE(801)] = 30149,
  [SMALL_STATE(802)] = 30200,
  [SMALL_STATE(803)] = 30251,
  [SMALL_STATE(804)] = 30302,
  [SMALL_STATE(805)] = 30353,
  [SMALL_STATE(806)] = 30404,
  [SMALL_STATE(807)] = 30455,
  [SMALL_STATE(808)] = 30506,
  [SMALL_STATE(809)] = 30557,
  [SMALL_STATE(810)] = 30608,
  [SMALL_STATE(811)] = 30659,
  [SMALL_STATE(812)] = 30710,
  [SMALL_STATE(813)] = 30761,
  [SMALL_STATE(814)] = 30812,
  [SMALL_STATE(815)] = 30863,
  [SMALL_STATE(816)] = 30914,
  [SMALL_STATE(817)] = 30965,
  [SMALL_STATE(818)] = 31016,
  [SMALL_STATE(819)] = 31067,
  [SMALL_STATE(820)] = 31122,
  [SMALL_STATE(821)] = 31173,
  [SMALL_STATE(822)] = 31224,
  [SMALL_STATE(823)] = 31275,
  [SMALL_STATE(824)] = 31326,
  [SMALL_STATE(825)] = 31377,
  [SMALL_STATE(826)] = 31428,
  [SMALL_STATE(827)] = 31479,
  [SMALL_STATE(828)] = 31532,
  [SMALL_STATE(829)] = 31583,
  [SMALL_STATE(830)] = 31634,
  [SMALL_STATE(831)] = 31685,
  [SMALL_STATE(832)] = 31736,
  [SMALL_STATE(833)] = 31787,
  [SMALL_STATE(834)] = 31838,
  [SMALL_STATE(835)] = 31889,
  [SMALL_STATE(836)] = 31940,
  [SMALL_STATE(837)] = 32002,
  [SMALL_STATE(838)] = 32070,
  [SMALL_STATE(839)] = 32138,
  [SMALL_STATE(840)] = 32196,
  [SMALL_STATE(841)] = 32256,
  [SMALL_STATE(842)] = 32334,
  [SMALL_STATE(843)] = 32406,
  [SMALL_STATE(844)] = 32482,
  [SMALL_STATE(845)] = 32554,
  [SMALL_STATE(846)] = 32618,
  [SMALL_STATE(847)] = 32692,
  [SMALL_STATE(848)] = 32767,
  [SMALL_STATE(849)] = 32842,
  [SMALL_STATE(850)] = 32927,
  [SMALL_STATE(851)] = 33002,
  [SMALL_STATE(852)] = 33071,
  [SMALL_STATE(853)] = 33156,
  [SMALL_STATE(854)] = 33205,
  [SMALL_STATE(855)] = 33280,
  [SMALL_STATE(856)] = 33355,
  [SMALL_STATE(857)] = 33438,
  [SMALL_STATE(858)] = 33523,
  [SMALL_STATE(859)] = 33606,
  [SMALL_STATE(860)] = 33675,
  [SMALL_STATE(861)] = 33758,
  [SMALL_STATE(862)] = 33827,
  [SMALL_STATE(863)] = 33900,
  [SMALL_STATE(864)] = 33969,
  [SMALL_STATE(865)] = 34044,
  [SMALL_STATE(866)] = 34129,
  [SMALL_STATE(867)] = 34214,
  [SMALL_STATE(868)] = 34289,
  [SMALL_STATE(869)] = 34374,
  [SMALL_STATE(870)] = 34457,
  [SMALL_STATE(871)] = 34542,
  [SMALL_STATE(872)] = 34617,
  [SMALL_STATE(873)] = 34702,
  [SMALL_STATE(874)] = 34785,
  [SMALL_STATE(875)] = 34860,
  [SMALL_STATE(876)] = 34929,
  [SMALL_STATE(877)] = 34998,
  [SMALL_STATE(878)] = 35073,
  [SMALL_STATE(879)] = 35156,
  [SMALL_STATE(880)] = 35241,
  [SMALL_STATE(881)] = 35326,
  [SMALL_STATE(882)] = 35411,
  [SMALL_STATE(883)] = 35496,
  [SMALL_STATE(884)] = 35568,
  [SMALL_STATE(885)] = 35640,
  [SMALL_STATE(886)] = 35687,
  [SMALL_STATE(887)] = 35748,
  [SMALL_STATE(888)] = 35829,
  [SMALL_STATE(889)] = 35880,
  [SMALL_STATE(890)] = 35961,
  [SMALL_STATE(891)] = 36042,
  [SMALL_STATE(892)] = 36119,
  [SMALL_STATE(893)] = 36200,
  [SMALL_STATE(894)] = 36279,
  [SMALL_STATE(895)] = 36330,
  [SMALL_STATE(896)] = 36411,
  [SMALL_STATE(897)] = 36492,
  [SMALL_STATE(898)] = 36567,
  [SMALL_STATE(899)] = 36642,
  [SMALL_STATE(900)] = 36703,
  [SMALL_STATE(901)] = 36760,
  [SMALL_STATE(902)] = 36825,
  [SMALL_STATE(903)] = 36890,
  [SMALL_STATE(904)] = 36963,
  [SMALL_STATE(905)] = 37032,
  [SMALL_STATE(906)] = 37103,
  [SMALL_STATE(907)] = 37164,
  [SMALL_STATE(908)] = 37223,
  [SMALL_STATE(909)] = 37270,
  [SMALL_STATE(910)] = 37351,
  [SMALL_STATE(911)] = 37432,
  [SMALL_STATE(912)] = 37483,
  [SMALL_STATE(913)] = 37548,
  [SMALL_STATE(914)] = 37629,
  [SMALL_STATE(915)] = 37676,
  [SMALL_STATE(916)] = 37755,
  [SMALL_STATE(917)] = 37836,
  [SMALL_STATE(918)] = 37883,
  [SMALL_STATE(919)] = 37952,
  [SMALL_STATE(920)] = 38007,
  [SMALL_STATE(921)] = 38065,
  [SMALL_STATE(922)] = 38129,
  [SMALL_STATE(923)] = 38215,
  [SMALL_STATE(924)] = 38281,
  [SMALL_STATE(925)] = 38345,
  [SMALL_STATE(926)] = 38409,
  [SMALL_STATE(927)] = 38465,
  [SMALL_STATE(928)] = 38529,
  [SMALL_STATE(929)] = 38605,
  [SMALL_STATE(930)] = 38669,
  [SMALL_STATE(931)] = 38735,
  [SMALL_STATE(932)] = 38809,
  [SMALL_STATE(933)] = 38873,
  [SMALL_STATE(934)] = 38937,
  [SMALL_STATE(935)] = 39001,
  [SMALL_STATE(936)] = 39087,
  [SMALL_STATE(937)] = 39147,
  [SMALL_STATE(938)] = 39211,
  [SMALL_STATE(939)] = 39287,
  [SMALL_STATE(940)] = 39353,
  [SMALL_STATE(941)] = 39439,
  [SMALL_STATE(942)] = 39499,
  [SMALL_STATE(943)] = 39563,
  [SMALL_STATE(944)] = 39627,
  [SMALL_STATE(945)] = 39705,
  [SMALL_STATE(946)] = 39783,
  [SMALL_STATE(947)] = 39847,
  [SMALL_STATE(948)] = 39915,
  [SMALL_STATE(949)] = 39967,
  [SMALL_STATE(950)] = 40043,
  [SMALL_STATE(951)] = 40097,
  [SMALL_STATE(952)] = 40173,
  [SMALL_STATE(953)] = 40241,
  [SMALL_STATE(954)] = 40319,
  [SMALL_STATE(955)] = 40405,
  [SMALL_STATE(956)] = 40471,
  [SMALL_STATE(957)] = 40537,
  [SMALL_STATE(958)] = 40603,
  [SMALL_STATE(959)] = 40669,
  [SMALL_STATE(960)] = 40747,
  [SMALL_STATE(961)] = 40807,
  [SMALL_STATE(962)] = 40871,
  [SMALL_STATE(963)] = 40941,
  [SMALL_STATE(964)] = 41027,
  [SMALL_STATE(965)] = 41103,
  [SMALL_STATE(966)] = 41163,
  [SMALL_STATE(967)] = 41223,
  [SMALL_STATE(968)] = 41291,
  [SMALL_STATE(969)] = 41363,
  [SMALL_STATE(970)] = 41427,
  [SMALL_STATE(971)] = 41491,
  [SMALL_STATE(972)] = 41555,
  [SMALL_STATE(973)] = 41633,
  [SMALL_STATE(974)] = 41689,
  [SMALL_STATE(975)] = 41763,
  [SMALL_STATE(976)] = 41821,
  [SMALL_STATE(977)] = 41881,
  [SMALL_STATE(978)] = 41951,
  [SMALL_STATE(979)] = 42029,
  [SMALL_STATE(980)] = 42093,
  [SMALL_STATE(981)] = 42179,
  [SMALL_STATE(982)] = 42245,
  [SMALL_STATE(983)] = 42313,
  [SMALL_STATE(984)] = 42385,
  [SMALL_STATE(985)] = 42451,
  [SMALL_STATE(986)] = 42527,
  [SMALL_STATE(987)] = 42593,
  [SMALL_STATE(988)] = 42671,
  [SMALL_STATE(989)] = 42737,
  [SMALL_STATE(990)] = 42803,
  [SMALL_STATE(991)] = 42869,
  [SMALL_STATE(992)] = 42933,
  [SMALL_STATE(993)] = 42987,
  [SMALL_STATE(994)] = 43051,
  [SMALL_STATE(995)] = 43137,
  [SMALL_STATE(996)] = 43193,
  [SMALL_STATE(997)] = 43257,
  [SMALL_STATE(998)] = 43321,
  [SMALL_STATE(999)] = 43387,
  [SMALL_STATE(1000)] = 43451,
  [SMALL_STATE(1001)] = 43526,
  [SMALL_STATE(1002)] = 43571,
  [SMALL_STATE(1003)] = 43644,
  [SMALL_STATE(1004)] = 43719,
  [SMALL_STATE(1005)] = 43794,
  [SMALL_STATE(1006)] = 43843,
  [SMALL_STATE(1007)] = 43890,
  [SMALL_STATE(1008)] = 43963,
  [SMALL_STATE(1009)] = 44012,
  [SMALL_STATE(1010)] = 44057,
  [SMALL_STATE(1011)] = 44108,
  [SMALL_STATE(1012)] = 44153,
  [SMALL_STATE(1013)] = 44228,
  [SMALL_STATE(1014)] = 44289,
  [SMALL_STATE(1015)] = 44350,
  [SMALL_STATE(1016)] = 44423,
  [SMALL_STATE(1017)] = 44496,
  [SMALL_STATE(1018)] = 44569,
  [SMALL_STATE(1019)] = 44614,
  [SMALL_STATE(1020)] = 44659,
  [SMALL_STATE(1021)] = 44708,
  [SMALL_STATE(1022)] = 44783,
  [SMALL_STATE(1023)] = 44844,
  [SMALL_STATE(1024)] = 44895,
  [SMALL_STATE(1025)] = 44940,
  [SMALL_STATE(1026)] = 45015,
  [SMALL_STATE(1027)] = 45088,
  [SMALL_STATE(1028)] = 45161,
  [SMALL_STATE(1029)] = 45210,
  [SMALL_STATE(1030)] = 45255,
  [SMALL_STATE(1031)] = 45328,
  [SMALL_STATE(1032)] = 45401,
  [SMALL_STATE(1033)] = 45474,
  [SMALL_STATE(1034)] = 45521,
  [SMALL_STATE(1035)] = 45566,
  [SMALL_STATE(1036)] = 45639,
  [SMALL_STATE(1037)] = 45684,
  [SMALL_STATE(1038)] = 45732,
  [SMALL_STATE(1039)] = 45804,
  [SMALL_STATE(1040)] = 45876,
  [SMALL_STATE(1041)] = 45956,
  [SMALL_STATE(1042)] = 46004,
  [SMALL_STATE(1043)] = 46050,
  [SMALL_STATE(1044)] = 46098,
  [SMALL_STATE(1045)] = 46146,
  [SMALL_STATE(1046)] = 46196,
  [SMALL_STATE(1047)] = 46244,
  [SMALL_STATE(1048)] = 46316,
  [SMALL_STATE(1049)] = 46364,
  [SMALL_STATE(1050)] = 46412,
  [SMALL_STATE(1051)] = 46460,
  [SMALL_STATE(1052)] = 46506,
  [SMALL_STATE(1053)] = 46554,
  [SMALL_STATE(1054)] = 46600,
  [SMALL_STATE(1055)] = 46672,
  [SMALL_STATE(1056)] = 46720,
  [SMALL_STATE(1057)] = 46768,
  [SMALL_STATE(1058)] = 46816,
  [SMALL_STATE(1059)] = 46864,
  [SMALL_STATE(1060)] = 46912,
  [SMALL_STATE(1061)] = 46960,
  [SMALL_STATE(1062)] = 47008,
  [SMALL_STATE(1063)] = 47056,
  [SMALL_STATE(1064)] = 47104,
  [SMALL_STATE(1065)] = 47150,
  [SMALL_STATE(1066)] = 47230,
  [SMALL_STATE(1067)] = 47302,
  [SMALL_STATE(1068)] = 47348,
  [SMALL_STATE(1069)] = 47396,
  [SMALL_STATE(1070)] = 47444,
  [SMALL_STATE(1071)] = 47492,
  [SMALL_STATE(1072)] = 47538,
  [SMALL_STATE(1073)] = 47586,
  [SMALL_STATE(1074)] = 47658,
  [SMALL_STATE(1075)] = 47706,
  [SMALL_STATE(1076)] = 47754,
  [SMALL_STATE(1077)] = 47826,
  [SMALL_STATE(1078)] = 47898,
  [SMALL_STATE(1079)] = 47978,
  [SMALL_STATE(1080)] = 48024,
  [SMALL_STATE(1081)] = 48072,
  [SMALL_STATE(1082)] = 48152,
  [SMALL_STATE(1083)] = 48224,
  [SMALL_STATE(1084)] = 48272,
  [SMALL_STATE(1085)] = 48318,
  [SMALL_STATE(1086)] = 48362,
  [SMALL_STATE(1087)] = 48434,
  [SMALL_STATE(1088)] = 48482,
  [SMALL_STATE(1089)] = 48554,
  [SMALL_STATE(1090)] = 48602,
  [SMALL_STATE(1091)] = 48674,
  [SMALL_STATE(1092)] = 48722,
  [SMALL_STATE(1093)] = 48770,
  [SMALL_STATE(1094)] = 48818,
  [SMALL_STATE(1095)] = 48890,
  [SMALL_STATE(1096)] = 48938,
  [SMALL_STATE(1097)] = 48996,
  [SMALL_STATE(1098)] = 49068,
  [SMALL_STATE(1099)] = 49148,
  [SMALL_STATE(1100)] = 49206,
  [SMALL_STATE(1101)] = 49254,
  [SMALL_STATE(1102)] = 49326,
  [SMALL_STATE(1103)] = 49398,
  [SMALL_STATE(1104)] = 49478,
  [SMALL_STATE(1105)] = 49550,
  [SMALL_STATE(1106)] = 49622,
  [SMALL_STATE(1107)] = 49680,
  [SMALL_STATE(1108)] = 49752,
  [SMALL_STATE(1109)] = 49832,
  [SMALL_STATE(1110)] = 49890,
  [SMALL_STATE(1111)] = 49936,
  [SMALL_STATE(1112)] = 49984,
  [SMALL_STATE(1113)] = 50032,
  [SMALL_STATE(1114)] = 50080,
  [SMALL_STATE(1115)] = 50128,
  [SMALL_STATE(1116)] = 50174,
  [SMALL_STATE(1117)] = 50222,
  [SMALL_STATE(1118)] = 50268,
  [SMALL_STATE(1119)] = 50314,
  [SMALL_STATE(1120)] = 50386,
  [SMALL_STATE(1121)] = 50444,
  [SMALL_STATE(1122)] = 50490,
  [SMALL_STATE(1123)] = 50562,
  [SMALL_STATE(1124)] = 50634,
  [SMALL_STATE(1125)] = 50706,
  [SMALL_STATE(1126)] = 50764,
  [SMALL_STATE(1127)] = 50810,
  [SMALL_STATE(1128)] = 50853,
  [SMALL_STATE(1129)] = 50896,
  [SMALL_STATE(1130)] = 50943,
  [SMALL_STATE(1131)] = 50990,
  [SMALL_STATE(1132)] = 51039,
  [SMALL_STATE(1133)] = 51108,
  [SMALL_STATE(1134)] = 51151,
  [SMALL_STATE(1135)] = 51194,
  [SMALL_STATE(1136)] = 51263,
  [SMALL_STATE(1137)] = 51306,
  [SMALL_STATE(1138)] = 51349,
  [SMALL_STATE(1139)] = 51392,
  [SMALL_STATE(1140)] = 51435,
  [SMALL_STATE(1141)] = 51478,
  [SMALL_STATE(1142)] = 51523,
  [SMALL_STATE(1143)] = 51566,
  [SMALL_STATE(1144)] = 51609,
  [SMALL_STATE(1145)] = 51652,
  [SMALL_STATE(1146)] = 51695,
  [SMALL_STATE(1147)] = 51738,
  [SMALL_STATE(1148)] = 51781,
  [SMALL_STATE(1149)] = 51826,
  [SMALL_STATE(1150)] = 51871,
  [SMALL_STATE(1151)] = 51914,
  [SMALL_STATE(1152)] = 51961,
  [SMALL_STATE(1153)] = 52004,
  [SMALL_STATE(1154)] = 52049,
  [SMALL_STATE(1155)] = 52092,
  [SMALL_STATE(1156)] = 52135,
  [SMALL_STATE(1157)] = 52178,
  [SMALL_STATE(1158)] = 52221,
  [SMALL_STATE(1159)] = 52272,
  [SMALL_STATE(1160)] = 52314,
  [SMALL_STATE(1161)] = 52356,
  [SMALL_STATE(1162)] = 52412,
  [SMALL_STATE(1163)] = 52486,
  [SMALL_STATE(1164)] = 52530,
  [SMALL_STATE(1165)] = 52574,
  [SMALL_STATE(1166)] = 52630,
  [SMALL_STATE(1167)] = 52686,
  [SMALL_STATE(1168)] = 52728,
  [SMALL_STATE(1169)] = 52772,
  [SMALL_STATE(1170)] = 52816,
  [SMALL_STATE(1171)] = 52860,
  [SMALL_STATE(1172)] = 52902,
  [SMALL_STATE(1173)] = 52944,
  [SMALL_STATE(1174)] = 52992,
  [SMALL_STATE(1175)] = 53036,
  [SMALL_STATE(1176)] = 53084,
  [SMALL_STATE(1177)] = 53158,
  [SMALL_STATE(1178)] = 53200,
  [SMALL_STATE(1179)] = 53242,
  [SMALL_STATE(1180)] = 53284,
  [SMALL_STATE(1181)] = 53328,
  [SMALL_STATE(1182)] = 53384,
  [SMALL_STATE(1183)] = 53428,
  [SMALL_STATE(1184)] = 53470,
  [SMALL_STATE(1185)] = 53512,
  [SMALL_STATE(1186)] = 53554,
  [SMALL_STATE(1187)] = 53596,
  [SMALL_STATE(1188)] = 53640,
  [SMALL_STATE(1189)] = 53714,
  [SMALL_STATE(1190)] = 53764,
  [SMALL_STATE(1191)] = 53812,
  [SMALL_STATE(1192)] = 53854,
  [SMALL_STATE(1193)] = 53896,
  [SMALL_STATE(1194)] = 53938,
  [SMALL_STATE(1195)] = 53980,
  [SMALL_STATE(1196)] = 54022,
  [SMALL_STATE(1197)] = 54064,
  [SMALL_STATE(1198)] = 54106,
  [SMALL_STATE(1199)] = 54180,
  [SMALL_STATE(1200)] = 54222,
  [SMALL_STATE(1201)] = 54266,
  [SMALL_STATE(1202)] = 54308,
  [SMALL_STATE(1203)] = 54352,
  [SMALL_STATE(1204)] = 54426,
  [SMALL_STATE(1205)] = 54470,
  [SMALL_STATE(1206)] = 54544,
  [SMALL_STATE(1207)] = 54586,
  [SMALL_STATE(1208)] = 54660,
  [SMALL_STATE(1209)] = 54702,
  [SMALL_STATE(1210)] = 54756,
  [SMALL_STATE(1211)] = 54800,
  [SMALL_STATE(1212)] = 54874,
  [SMALL_STATE(1213)] = 54918,
  [SMALL_STATE(1214)] = 54962,
  [SMALL_STATE(1215)] = 55018,
  [SMALL_STATE(1216)] = 55092,
  [SMALL_STATE(1217)] = 55140,
  [SMALL_STATE(1218)] = 55214,
  [SMALL_STATE(1219)] = 55288,
  [SMALL_STATE(1220)] = 55362,
  [SMALL_STATE(1221)] = 55418,
  [SMALL_STATE(1222)] = 55492,
  [SMALL_STATE(1223)] = 55534,
  [SMALL_STATE(1224)] = 55608,
  [SMALL_STATE(1225)] = 55679,
  [SMALL_STATE(1226)] = 55732,
  [SMALL_STATE(1227)] = 55775,
  [SMALL_STATE(1228)] = 55846,
  [SMALL_STATE(1229)] = 55899,
  [SMALL_STATE(1230)] = 55970,
  [SMALL_STATE(1231)] = 56041,
  [SMALL_STATE(1232)] = 56112,
  [SMALL_STATE(1233)] = 56165,
  [SMALL_STATE(1234)] = 56236,
  [SMALL_STATE(1235)] = 56307,
  [SMALL_STATE(1236)] = 56360,
  [SMALL_STATE(1237)] = 56431,
  [SMALL_STATE(1238)] = 56502,
  [SMALL_STATE(1239)] = 56543,
  [SMALL_STATE(1240)] = 56596,
  [SMALL_STATE(1241)] = 56641,
  [SMALL_STATE(1242)] = 56694,
  [SMALL_STATE(1243)] = 56765,
  [SMALL_STATE(1244)] = 56836,
  [SMALL_STATE(1245)] = 56883,
  [SMALL_STATE(1246)] = 56954,
  [SMALL_STATE(1247)] = 57025,
  [SMALL_STATE(1248)] = 57096,
  [SMALL_STATE(1249)] = 57149,
  [SMALL_STATE(1250)] = 57220,
  [SMALL_STATE(1251)] = 57291,
  [SMALL_STATE(1252)] = 57362,
  [SMALL_STATE(1253)] = 57433,
  [SMALL_STATE(1254)] = 57504,
  [SMALL_STATE(1255)] = 57575,
  [SMALL_STATE(1256)] = 57646,
  [SMALL_STATE(1257)] = 57699,
  [SMALL_STATE(1258)] = 57752,
  [SMALL_STATE(1259)] = 57805,
  [SMALL_STATE(1260)] = 57876,
  [SMALL_STATE(1261)] = 57947,
  [SMALL_STATE(1262)] = 58000,
  [SMALL_STATE(1263)] = 58053,
  [SMALL_STATE(1264)] = 58124,
  [SMALL_STATE(1265)] = 58195,
  [SMALL_STATE(1266)] = 58266,
  [SMALL_STATE(1267)] = 58307,
  [SMALL_STATE(1268)] = 58350,
  [SMALL_STATE(1269)] = 58395,
  [SMALL_STATE(1270)] = 58466,
  [SMALL_STATE(1271)] = 58537,
  [SMALL_STATE(1272)] = 58608,
  [SMALL_STATE(1273)] = 58661,
  [SMALL_STATE(1274)] = 58732,
  [SMALL_STATE(1275)] = 58803,
  [SMALL_STATE(1276)] = 58846,
  [SMALL_STATE(1277)] = 58891,
  [SMALL_STATE(1278)] = 58962,
  [SMALL_STATE(1279)] = 59033,
  [SMALL_STATE(1280)] = 59104,
  [SMALL_STATE(1281)] = 59175,
  [SMALL_STATE(1282)] = 59220,
  [SMALL_STATE(1283)] = 59291,
  [SMALL_STATE(1284)] = 59336,
  [SMALL_STATE(1285)] = 59407,
  [SMALL_STATE(1286)] = 59450,
  [SMALL_STATE(1287)] = 59493,
  [SMALL_STATE(1288)] = 59564,
  [SMALL_STATE(1289)] = 59635,
  [SMALL_STATE(1290)] = 59676,
  [SMALL_STATE(1291)] = 59747,
  [SMALL_STATE(1292)] = 59818,
  [SMALL_STATE(1293)] = 59889,
  [SMALL_STATE(1294)] = 59960,
  [SMALL_STATE(1295)] = 60013,
  [SMALL_STATE(1296)] = 60084,
  [SMALL_STATE(1297)] = 60129,
  [SMALL_STATE(1298)] = 60172,
  [SMALL_STATE(1299)] = 60243,
  [SMALL_STATE(1300)] = 60296,
  [SMALL_STATE(1301)] = 60367,
  [SMALL_STATE(1302)] = 60420,
  [SMALL_STATE(1303)] = 60473,
  [SMALL_STATE(1304)] = 60526,
  [SMALL_STATE(1305)] = 60597,
  [SMALL_STATE(1306)] = 60650,
  [SMALL_STATE(1307)] = 60703,
  [SMALL_STATE(1308)] = 60756,
  [SMALL_STATE(1309)] = 60809,
  [SMALL_STATE(1310)] = 60880,
  [SMALL_STATE(1311)] = 60951,
  [SMALL_STATE(1312)] = 61022,
  [SMALL_STATE(1313)] = 61093,
  [SMALL_STATE(1314)] = 61164,
  [SMALL_STATE(1315)] = 61217,
  [SMALL_STATE(1316)] = 61270,
  [SMALL_STATE(1317)] = 61323,
  [SMALL_STATE(1318)] = 61366,
  [SMALL_STATE(1319)] = 61437,
  [SMALL_STATE(1320)] = 61490,
  [SMALL_STATE(1321)] = 61561,
  [SMALL_STATE(1322)] = 61614,
  [SMALL_STATE(1323)] = 61659,
  [SMALL_STATE(1324)] = 61730,
  [SMALL_STATE(1325)] = 61783,
  [SMALL_STATE(1326)] = 61836,
  [SMALL_STATE(1327)] = 61879,
  [SMALL_STATE(1328)] = 61932,
  [SMALL_STATE(1329)] = 61985,
  [SMALL_STATE(1330)] = 62038,
  [SMALL_STATE(1331)] = 62109,
  [SMALL_STATE(1332)] = 62162,
  [SMALL_STATE(1333)] = 62215,
  [SMALL_STATE(1334)] = 62255,
  [SMALL_STATE(1335)] = 62305,
  [SMALL_STATE(1336)] = 62355,
  [SMALL_STATE(1337)] = 62395,
  [SMALL_STATE(1338)] = 62445,
  [SMALL_STATE(1339)] = 62493,
  [SMALL_STATE(1340)] = 62543,
  [SMALL_STATE(1341)] = 62593,
  [SMALL_STATE(1342)] = 62643,
  [SMALL_STATE(1343)] = 62693,
  [SMALL_STATE(1344)] = 62743,
  [SMALL_STATE(1345)] = 62793,
  [SMALL_STATE(1346)] = 62843,
  [SMALL_STATE(1347)] = 62883,
  [SMALL_STATE(1348)] = 62923,
  [SMALL_STATE(1349)] = 62963,
  [SMALL_STATE(1350)] = 63013,
  [SMALL_STATE(1351)] = 63063,
  [SMALL_STATE(1352)] = 63113,
  [SMALL_STATE(1353)] = 63163,
  [SMALL_STATE(1354)] = 63213,
  [SMALL_STATE(1355)] = 63263,
  [SMALL_STATE(1356)] = 63313,
  [SMALL_STATE(1357)] = 63363,
  [SMALL_STATE(1358)] = 63413,
  [SMALL_STATE(1359)] = 63463,
  [SMALL_STATE(1360)] = 63513,
  [SMALL_STATE(1361)] = 63563,
  [SMALL_STATE(1362)] = 63613,
  [SMALL_STATE(1363)] = 63663,
  [SMALL_STATE(1364)] = 63713,
  [SMALL_STATE(1365)] = 63763,
  [SMALL_STATE(1366)] = 63813,
  [SMALL_STATE(1367)] = 63863,
  [SMALL_STATE(1368)] = 63919,
  [SMALL_STATE(1369)] = 63981,
  [SMALL_STATE(1370)] = 64031,
  [SMALL_STATE(1371)] = 64081,
  [SMALL_STATE(1372)] = 64131,
  [SMALL_STATE(1373)] = 64181,
  [SMALL_STATE(1374)] = 64231,
  [SMALL_STATE(1375)] = 64271,
  [SMALL_STATE(1376)] = 64321,
  [SMALL_STATE(1377)] = 64381,
  [SMALL_STATE(1378)] = 64431,
  [SMALL_STATE(1379)] = 64481,
  [SMALL_STATE(1380)] = 64531,
  [SMALL_STATE(1381)] = 64581,
  [SMALL_STATE(1382)] = 64621,
  [SMALL_STATE(1383)] = 64671,
  [SMALL_STATE(1384)] = 64711,
  [SMALL_STATE(1385)] = 64761,
  [SMALL_STATE(1386)] = 64811,
  [SMALL_STATE(1387)] = 64855,
  [SMALL_STATE(1388)] = 64905,
  [SMALL_STATE(1389)] = 64955,
  [SMALL_STATE(1390)] = 65005,
  [SMALL_STATE(1391)] = 65055,
  [SMALL_STATE(1392)] = 65105,
  [SMALL_STATE(1393)] = 65155,
  [SMALL_STATE(1394)] = 65205,
  [SMALL_STATE(1395)] = 65255,
  [SMALL_STATE(1396)] = 65305,
  [SMALL_STATE(1397)] = 65355,
  [SMALL_STATE(1398)] = 65405,
  [SMALL_STATE(1399)] = 65455,
  [SMALL_STATE(1400)] = 65505,
  [SMALL_STATE(1401)] = 65555,
  [SMALL_STATE(1402)] = 65605,
  [SMALL_STATE(1403)] = 65655,
  [SMALL_STATE(1404)] = 65705,
  [SMALL_STATE(1405)] = 65755,
  [SMALL_STATE(1406)] = 65805,
  [SMALL_STATE(1407)] = 65855,
  [SMALL_STATE(1408)] = 65905,
  [SMALL_STATE(1409)] = 65947,
  [SMALL_STATE(1410)] = 65997,
  [SMALL_STATE(1411)] = 66047,
  [SMALL_STATE(1412)] = 66097,
  [SMALL_STATE(1413)] = 66137,
  [SMALL_STATE(1414)] = 66187,
  [SMALL_STATE(1415)] = 66237,
  [SMALL_STATE(1416)] = 66287,
  [SMALL_STATE(1417)] = 66331,
  [SMALL_STATE(1418)] = 66381,
  [SMALL_STATE(1419)] = 66433,
  [SMALL_STATE(1420)] = 66483,
  [SMALL_STATE(1421)] = 66533,
  [SMALL_STATE(1422)] = 66583,
  [SMALL_STATE(1423)] = 66633,
  [SMALL_STATE(1424)] = 66683,
  [SMALL_STATE(1425)] = 66733,
  [SMALL_STATE(1426)] = 66783,
  [SMALL_STATE(1427)] = 66833,
  [SMALL_STATE(1428)] = 66883,
  [SMALL_STATE(1429)] = 66933,
  [SMALL_STATE(1430)] = 66983,
  [SMALL_STATE(1431)] = 67033,
  [SMALL_STATE(1432)] = 67083,
  [SMALL_STATE(1433)] = 67133,
  [SMALL_STATE(1434)] = 67183,
  [SMALL_STATE(1435)] = 67233,
  [SMALL_STATE(1436)] = 67283,
  [SMALL_STATE(1437)] = 67325,
  [SMALL_STATE(1438)] = 67375,
  [SMALL_STATE(1439)] = 67425,
  [SMALL_STATE(1440)] = 67475,
  [SMALL_STATE(1441)] = 67525,
  [SMALL_STATE(1442)] = 67575,
  [SMALL_STATE(1443)] = 67625,
  [SMALL_STATE(1444)] = 67675,
  [SMALL_STATE(1445)] = 67725,
  [SMALL_STATE(1446)] = 67775,
  [SMALL_STATE(1447)] = 67825,
  [SMALL_STATE(1448)] = 67875,
  [SMALL_STATE(1449)] = 67925,
  [SMALL_STATE(1450)] = 67975,
  [SMALL_STATE(1451)] = 68025,
  [SMALL_STATE(1452)] = 68072,
  [SMALL_STATE(1453)] = 68119,
  [SMALL_STATE(1454)] = 68166,
  [SMALL_STATE(1455)] = 68213,
  [SMALL_STATE(1456)] = 68252,
  [SMALL_STATE(1457)] = 68299,
  [SMALL_STATE(1458)] = 68346,
  [SMALL_STATE(1459)] = 68393,
  [SMALL_STATE(1460)] = 68432,
  [SMALL_STATE(1461)] = 68479,
  [SMALL_STATE(1462)] = 68526,
  [SMALL_STATE(1463)] = 68573,
  [SMALL_STATE(1464)] = 68620,
  [SMALL_STATE(1465)] = 68667,
  [SMALL_STATE(1466)] = 68714,
  [SMALL_STATE(1467)] = 68761,
  [SMALL_STATE(1468)] = 68808,
  [SMALL_STATE(1469)] = 68855,
  [SMALL_STATE(1470)] = 68902,
  [SMALL_STATE(1471)] = 68949,
  [SMALL_STATE(1472)] = 68996,
  [SMALL_STATE(1473)] = 69043,
  [SMALL_STATE(1474)] = 69090,
  [SMALL_STATE(1475)] = 69137,
  [SMALL_STATE(1476)] = 69184,
  [SMALL_STATE(1477)] = 69231,
  [SMALL_STATE(1478)] = 69278,
  [SMALL_STATE(1479)] = 69325,
  [SMALL_STATE(1480)] = 69372,
  [SMALL_STATE(1481)] = 69419,
  [SMALL_STATE(1482)] = 69466,
  [SMALL_STATE(1483)] = 69513,
  [SMALL_STATE(1484)] = 69560,
  [SMALL_STATE(1485)] = 69607,
  [SMALL_STATE(1486)] = 69650,
  [SMALL_STATE(1487)] = 69697,
  [SMALL_STATE(1488)] = 69744,
  [SMALL_STATE(1489)] = 69791,
  [SMALL_STATE(1490)] = 69838,
  [SMALL_STATE(1491)] = 69885,
  [SMALL_STATE(1492)] = 69924,
  [SMALL_STATE(1493)] = 69971,
  [SMALL_STATE(1494)] = 70018,
  [SMALL_STATE(1495)] = 70065,
  [SMALL_STATE(1496)] = 70112,
  [SMALL_STATE(1497)] = 70159,
  [SMALL_STATE(1498)] = 70198,
  [SMALL_STATE(1499)] = 70245,
  [SMALL_STATE(1500)] = 70292,
  [SMALL_STATE(1501)] = 70339,
  [SMALL_STATE(1502)] = 70386,
  [SMALL_STATE(1503)] = 70433,
  [SMALL_STATE(1504)] = 70480,
  [SMALL_STATE(1505)] = 70527,
  [SMALL_STATE(1506)] = 70574,
  [SMALL_STATE(1507)] = 70613,
  [SMALL_STATE(1508)] = 70654,
  [SMALL_STATE(1509)] = 70693,
  [SMALL_STATE(1510)] = 70732,
  [SMALL_STATE(1511)] = 70779,
  [SMALL_STATE(1512)] = 70826,
  [SMALL_STATE(1513)] = 70869,
  [SMALL_STATE(1514)] = 70916,
  [SMALL_STATE(1515)] = 70963,
  [SMALL_STATE(1516)] = 71010,
  [SMALL_STATE(1517)] = 71057,
  [SMALL_STATE(1518)] = 71104,
  [SMALL_STATE(1519)] = 71151,
  [SMALL_STATE(1520)] = 71198,
  [SMALL_STATE(1521)] = 71237,
  [SMALL_STATE(1522)] = 71284,
  [SMALL_STATE(1523)] = 71331,
  [SMALL_STATE(1524)] = 71378,
  [SMALL_STATE(1525)] = 71422,
  [SMALL_STATE(1526)] = 71466,
  [SMALL_STATE(1527)] = 71510,
  [SMALL_STATE(1528)] = 71554,
  [SMALL_STATE(1529)] = 71592,
  [SMALL_STATE(1530)] = 71636,
  [SMALL_STATE(1531)] = 71676,
  [SMALL_STATE(1532)] = 71720,
  [SMALL_STATE(1533)] = 71778,
  [SMALL_STATE(1534)] = 71822,
  [SMALL_STATE(1535)] = 71866,
  [SMALL_STATE(1536)] = 71904,
  [SMALL_STATE(1537)] = 71942,
  [SMALL_STATE(1538)] = 71986,
  [SMALL_STATE(1539)] = 72030,
  [SMALL_STATE(1540)] = 72074,
  [SMALL_STATE(1541)] = 72118,
  [SMALL_STATE(1542)] = 72162,
  [SMALL_STATE(1543)] = 72206,
  [SMALL_STATE(1544)] = 72250,
  [SMALL_STATE(1545)] = 72294,
  [SMALL_STATE(1546)] = 72338,
  [SMALL_STATE(1547)] = 72382,
  [SMALL_STATE(1548)] = 72420,
  [SMALL_STATE(1549)] = 72464,
  [SMALL_STATE(1550)] = 72508,
  [SMALL_STATE(1551)] = 72552,
  [SMALL_STATE(1552)] = 72596,
  [SMALL_STATE(1553)] = 72640,
  [SMALL_STATE(1554)] = 72684,
  [SMALL_STATE(1555)] = 72722,
  [SMALL_STATE(1556)] = 72760,
  [SMALL_STATE(1557)] = 72804,
  [SMALL_STATE(1558)] = 72848,
  [SMALL_STATE(1559)] = 72892,
  [SMALL_STATE(1560)] = 72936,
  [SMALL_STATE(1561)] = 72973,
  [SMALL_STATE(1562)] = 73010,
  [SMALL_STATE(1563)] = 73047,
  [SMALL_STATE(1564)] = 73084,
  [SMALL_STATE(1565)] = 73120,
  [SMALL_STATE(1566)] = 73156,
  [SMALL_STATE(1567)] = 73192,
  [SMALL_STATE(1568)] = 73228,
  [SMALL_STATE(1569)] = 73264,
  [SMALL_STATE(1570)] = 73300,
  [SMALL_STATE(1571)] = 73336,
  [SMALL_STATE(1572)] = 73372,
  [SMALL_STATE(1573)] = 73408,
  [SMALL_STATE(1574)] = 73444,
  [SMALL_STATE(1575)] = 73480,
  [SMALL_STATE(1576)] = 73516,
  [SMALL_STATE(1577)] = 73552,
  [SMALL_STATE(1578)] = 73588,
  [SMALL_STATE(1579)] = 73624,
  [SMALL_STATE(1580)] = 73660,
  [SMALL_STATE(1581)] = 73696,
  [SMALL_STATE(1582)] = 73732,
  [SMALL_STATE(1583)] = 73768,
  [SMALL_STATE(1584)] = 73804,
  [SMALL_STATE(1585)] = 73840,
  [SMALL_STATE(1586)] = 73876,
  [SMALL_STATE(1587)] = 73912,
  [SMALL_STATE(1588)] = 73948,
  [SMALL_STATE(1589)] = 73984,
  [SMALL_STATE(1590)] = 74020,
  [SMALL_STATE(1591)] = 74056,
  [SMALL_STATE(1592)] = 74092,
  [SMALL_STATE(1593)] = 74128,
  [SMALL_STATE(1594)] = 74164,
  [SMALL_STATE(1595)] = 74200,
  [SMALL_STATE(1596)] = 74236,
  [SMALL_STATE(1597)] = 74272,
  [SMALL_STATE(1598)] = 74308,
  [SMALL_STATE(1599)] = 74344,
  [SMALL_STATE(1600)] = 74380,
  [SMALL_STATE(1601)] = 74416,
  [SMALL_STATE(1602)] = 74452,
  [SMALL_STATE(1603)] = 74488,
  [SMALL_STATE(1604)] = 74524,
  [SMALL_STATE(1605)] = 74560,
  [SMALL_STATE(1606)] = 74596,
  [SMALL_STATE(1607)] = 74632,
  [SMALL_STATE(1608)] = 74668,
  [SMALL_STATE(1609)] = 74704,
  [SMALL_STATE(1610)] = 74740,
  [SMALL_STATE(1611)] = 74776,
  [SMALL_STATE(1612)] = 74812,
  [SMALL_STATE(1613)] = 74848,
  [SMALL_STATE(1614)] = 74884,
  [SMALL_STATE(1615)] = 74920,
  [SMALL_STATE(1616)] = 74956,
  [SMALL_STATE(1617)] = 74992,
  [SMALL_STATE(1618)] = 75028,
  [SMALL_STATE(1619)] = 75064,
  [SMALL_STATE(1620)] = 75100,
  [SMALL_STATE(1621)] = 75136,
  [SMALL_STATE(1622)] = 75172,
  [SMALL_STATE(1623)] = 75208,
  [SMALL_STATE(1624)] = 75244,
  [SMALL_STATE(1625)] = 75280,
  [SMALL_STATE(1626)] = 75316,
  [SMALL_STATE(1627)] = 75352,
  [SMALL_STATE(1628)] = 75388,
  [SMALL_STATE(1629)] = 75424,
  [SMALL_STATE(1630)] = 75460,
  [SMALL_STATE(1631)] = 75496,
  [SMALL_STATE(1632)] = 75532,
  [SMALL_STATE(1633)] = 75568,
  [SMALL_STATE(1634)] = 75604,
  [SMALL_STATE(1635)] = 75640,
  [SMALL_STATE(1636)] = 75676,
  [SMALL_STATE(1637)] = 75712,
  [SMALL_STATE(1638)] = 75748,
  [SMALL_STATE(1639)] = 75784,
  [SMALL_STATE(1640)] = 75820,
  [SMALL_STATE(1641)] = 75856,
  [SMALL_STATE(1642)] = 75892,
  [SMALL_STATE(1643)] = 75928,
  [SMALL_STATE(1644)] = 75964,
  [SMALL_STATE(1645)] = 76000,
  [SMALL_STATE(1646)] = 76036,
  [SMALL_STATE(1647)] = 76072,
  [SMALL_STATE(1648)] = 76108,
  [SMALL_STATE(1649)] = 76144,
  [SMALL_STATE(1650)] = 76180,
  [SMALL_STATE(1651)] = 76216,
  [SMALL_STATE(1652)] = 76252,
  [SMALL_STATE(1653)] = 76288,
  [SMALL_STATE(1654)] = 76324,
  [SMALL_STATE(1655)] = 76360,
  [SMALL_STATE(1656)] = 76396,
  [SMALL_STATE(1657)] = 76432,
  [SMALL_STATE(1658)] = 76468,
  [SMALL_STATE(1659)] = 76504,
  [SMALL_STATE(1660)] = 76540,
  [SMALL_STATE(1661)] = 76576,
  [SMALL_STATE(1662)] = 76612,
  [SMALL_STATE(1663)] = 76648,
  [SMALL_STATE(1664)] = 76684,
  [SMALL_STATE(1665)] = 76720,
  [SMALL_STATE(1666)] = 76756,
  [SMALL_STATE(1667)] = 76792,
  [SMALL_STATE(1668)] = 76828,
  [SMALL_STATE(1669)] = 76864,
  [SMALL_STATE(1670)] = 76900,
  [SMALL_STATE(1671)] = 76936,
  [SMALL_STATE(1672)] = 76972,
  [SMALL_STATE(1673)] = 77008,
  [SMALL_STATE(1674)] = 77044,
  [SMALL_STATE(1675)] = 77080,
  [SMALL_STATE(1676)] = 77116,
  [SMALL_STATE(1677)] = 77152,
  [SMALL_STATE(1678)] = 77188,
  [SMALL_STATE(1679)] = 77224,
  [SMALL_STATE(1680)] = 77260,
  [SMALL_STATE(1681)] = 77296,
  [SMALL_STATE(1682)] = 77332,
  [SMALL_STATE(1683)] = 77368,
  [SMALL_STATE(1684)] = 77404,
  [SMALL_STATE(1685)] = 77440,
  [SMALL_STATE(1686)] = 77476,
  [SMALL_STATE(1687)] = 77512,
  [SMALL_STATE(1688)] = 77548,
  [SMALL_STATE(1689)] = 77584,
  [SMALL_STATE(1690)] = 77620,
  [SMALL_STATE(1691)] = 77656,
  [SMALL_STATE(1692)] = 77692,
  [SMALL_STATE(1693)] = 77728,
  [SMALL_STATE(1694)] = 77764,
  [SMALL_STATE(1695)] = 77800,
  [SMALL_STATE(1696)] = 77836,
  [SMALL_STATE(1697)] = 77872,
  [SMALL_STATE(1698)] = 77908,
  [SMALL_STATE(1699)] = 77944,
  [SMALL_STATE(1700)] = 77980,
  [SMALL_STATE(1701)] = 78016,
  [SMALL_STATE(1702)] = 78052,
  [SMALL_STATE(1703)] = 78088,
  [SMALL_STATE(1704)] = 78124,
  [SMALL_STATE(1705)] = 78160,
  [SMALL_STATE(1706)] = 78196,
  [SMALL_STATE(1707)] = 78232,
  [SMALL_STATE(1708)] = 78268,
  [SMALL_STATE(1709)] = 78304,
  [SMALL_STATE(1710)] = 78340,
  [SMALL_STATE(1711)] = 78376,
  [SMALL_STATE(1712)] = 78412,
  [SMALL_STATE(1713)] = 78448,
  [SMALL_STATE(1714)] = 78484,
  [SMALL_STATE(1715)] = 78520,
  [SMALL_STATE(1716)] = 78555,
  [SMALL_STATE(1717)] = 78592,
  [SMALL_STATE(1718)] = 78627,
  [SMALL_STATE(1719)] = 78662,
  [SMALL_STATE(1720)] = 78697,
  [SMALL_STATE(1721)] = 78734,
  [SMALL_STATE(1722)] = 78771,
  [SMALL_STATE(1723)] = 78805,
  [SMALL_STATE(1724)] = 78839,
  [SMALL_STATE(1725)] = 78873,
  [SMALL_STATE(1726)] = 78907,
  [SMALL_STATE(1727)] = 78941,
  [SMALL_STATE(1728)] = 78975,
  [SMALL_STATE(1729)] = 79009,
  [SMALL_STATE(1730)] = 79043,
  [SMALL_STATE(1731)] = 79077,
  [SMALL_STATE(1732)] = 79111,
  [SMALL_STATE(1733)] = 79145,
  [SMALL_STATE(1734)] = 79176,
  [SMALL_STATE(1735)] = 79224,
  [SMALL_STATE(1736)] = 79274,
  [SMALL_STATE(1737)] = 79316,
  [SMALL_STATE(1738)] = 79366,
  [SMALL_STATE(1739)] = 79416,
  [SMALL_STATE(1740)] = 79466,
  [SMALL_STATE(1741)] = 79502,
  [SMALL_STATE(1742)] = 79538,
  [SMALL_STATE(1743)] = 79566,
  [SMALL_STATE(1744)] = 79598,
  [SMALL_STATE(1745)] = 79642,
  [SMALL_STATE(1746)] = 79682,
  [SMALL_STATE(1747)] = 79718,
  [SMALL_STATE(1748)] = 79752,
  [SMALL_STATE(1749)] = 79788,
  [SMALL_STATE(1750)] = 79824,
  [SMALL_STATE(1751)] = 79849,
  [SMALL_STATE(1752)] = 79878,
  [SMALL_STATE(1753)] = 79907,
  [SMALL_STATE(1754)] = 79936,
  [SMALL_STATE(1755)] = 79973,
  [SMALL_STATE(1756)] = 80014,
  [SMALL_STATE(1757)] = 80039,
  [SMALL_STATE(1758)] = 80076,
  [SMALL_STATE(1759)] = 80105,
  [SMALL_STATE(1760)] = 80134,
  [SMALL_STATE(1761)] = 80163,
  [SMALL_STATE(1762)] = 80192,
  [SMALL_STATE(1763)] = 80221,
  [SMALL_STATE(1764)] = 80250,
  [SMALL_STATE(1765)] = 80279,
  [SMALL_STATE(1766)] = 80308,
  [SMALL_STATE(1767)] = 80335,
  [SMALL_STATE(1768)] = 80364,
  [SMALL_STATE(1769)] = 80393,
  [SMALL_STATE(1770)] = 80422,
  [SMALL_STATE(1771)] = 80463,
  [SMALL_STATE(1772)] = 80492,
  [SMALL_STATE(1773)] = 80521,
  [SMALL_STATE(1774)] = 80550,
  [SMALL_STATE(1775)] = 80587,
  [SMALL_STATE(1776)] = 80616,
  [SMALL_STATE(1777)] = 80645,
  [SMALL_STATE(1778)] = 80674,
  [SMALL_STATE(1779)] = 80703,
  [SMALL_STATE(1780)] = 80732,
  [SMALL_STATE(1781)] = 80761,
  [SMALL_STATE(1782)] = 80790,
  [SMALL_STATE(1783)] = 80831,
  [SMALL_STATE(1784)] = 80860,
  [SMALL_STATE(1785)] = 80901,
  [SMALL_STATE(1786)] = 80930,
  [SMALL_STATE(1787)] = 80959,
  [SMALL_STATE(1788)] = 80988,
  [SMALL_STATE(1789)] = 81017,
  [SMALL_STATE(1790)] = 81044,
  [SMALL_STATE(1791)] = 81070,
  [SMALL_STATE(1792)] = 81094,
  [SMALL_STATE(1793)] = 81118,
  [SMALL_STATE(1794)] = 81144,
  [SMALL_STATE(1795)] = 81168,
  [SMALL_STATE(1796)] = 81194,
  [SMALL_STATE(1797)] = 81232,
  [SMALL_STATE(1798)] = 81268,
  [SMALL_STATE(1799)] = 81294,
  [SMALL_STATE(1800)] = 81320,
  [SMALL_STATE(1801)] = 81346,
  [SMALL_STATE(1802)] = 81372,
  [SMALL_STATE(1803)] = 81402,
  [SMALL_STATE(1804)] = 81438,
  [SMALL_STATE(1805)] = 81468,
  [SMALL_STATE(1806)] = 81494,
  [SMALL_STATE(1807)] = 81520,
  [SMALL_STATE(1808)] = 81556,
  [SMALL_STATE(1809)] = 81582,
  [SMALL_STATE(1810)] = 81620,
  [SMALL_STATE(1811)] = 81650,
  [SMALL_STATE(1812)] = 81685,
  [SMALL_STATE(1813)] = 81720,
  [SMALL_STATE(1814)] = 81743,
  [SMALL_STATE(1815)] = 81778,
  [SMALL_STATE(1816)] = 81813,
  [SMALL_STATE(1817)] = 81848,
  [SMALL_STATE(1818)] = 81883,
  [SMALL_STATE(1819)] = 81918,
  [SMALL_STATE(1820)] = 81953,
  [SMALL_STATE(1821)] = 81988,
  [SMALL_STATE(1822)] = 82011,
  [SMALL_STATE(1823)] = 82046,
  [SMALL_STATE(1824)] = 82081,
  [SMALL_STATE(1825)] = 82113,
  [SMALL_STATE(1826)] = 82145,
  [SMALL_STATE(1827)] = 82177,
  [SMALL_STATE(1828)] = 82207,
  [SMALL_STATE(1829)] = 82239,
  [SMALL_STATE(1830)] = 82271,
  [SMALL_STATE(1831)] = 82299,
  [SMALL_STATE(1832)] = 82327,
  [SMALL_STATE(1833)] = 82357,
  [SMALL_STATE(1834)] = 82382,
  [SMALL_STATE(1835)] = 82411,
  [SMALL_STATE(1836)] = 82438,
  [SMALL_STATE(1837)] = 82465,
  [SMALL_STATE(1838)] = 82492,
  [SMALL_STATE(1839)] = 82513,
  [SMALL_STATE(1840)] = 82534,
  [SMALL_STATE(1841)] = 82561,
  [SMALL_STATE(1842)] = 82590,
  [SMALL_STATE(1843)] = 82611,
  [SMALL_STATE(1844)] = 82640,
  [SMALL_STATE(1845)] = 82669,
  [SMALL_STATE(1846)] = 82694,
  [SMALL_STATE(1847)] = 82721,
  [SMALL_STATE(1848)] = 82748,
  [SMALL_STATE(1849)] = 82777,
  [SMALL_STATE(1850)] = 82806,
  [SMALL_STATE(1851)] = 82827,
  [SMALL_STATE(1852)] = 82856,
  [SMALL_STATE(1853)] = 82885,
  [SMALL_STATE(1854)] = 82914,
  [SMALL_STATE(1855)] = 82933,
  [SMALL_STATE(1856)] = 82962,
  [SMALL_STATE(1857)] = 82991,
  [SMALL_STATE(1858)] = 83020,
  [SMALL_STATE(1859)] = 83049,
  [SMALL_STATE(1860)] = 83078,
  [SMALL_STATE(1861)] = 83105,
  [SMALL_STATE(1862)] = 83134,
  [SMALL_STATE(1863)] = 83155,
  [SMALL_STATE(1864)] = 83182,
  [SMALL_STATE(1865)] = 83211,
  [SMALL_STATE(1866)] = 83238,
  [SMALL_STATE(1867)] = 83267,
  [SMALL_STATE(1868)] = 83292,
  [SMALL_STATE(1869)] = 83316,
  [SMALL_STATE(1870)] = 83342,
  [SMALL_STATE(1871)] = 83366,
  [SMALL_STATE(1872)] = 83386,
  [SMALL_STATE(1873)] = 83412,
  [SMALL_STATE(1874)] = 83432,
  [SMALL_STATE(1875)] = 83454,
  [SMALL_STATE(1876)] = 83478,
  [SMALL_STATE(1877)] = 83504,
  [SMALL_STATE(1878)] = 83524,
  [SMALL_STATE(1879)] = 83544,
  [SMALL_STATE(1880)] = 83564,
  [SMALL_STATE(1881)] = 83590,
  [SMALL_STATE(1882)] = 83610,
  [SMALL_STATE(1883)] = 83630,
  [SMALL_STATE(1884)] = 83650,
  [SMALL_STATE(1885)] = 83670,
  [SMALL_STATE(1886)] = 83690,
  [SMALL_STATE(1887)] = 83710,
  [SMALL_STATE(1888)] = 83734,
  [SMALL_STATE(1889)] = 83758,
  [SMALL_STATE(1890)] = 83780,
  [SMALL_STATE(1891)] = 83804,
  [SMALL_STATE(1892)] = 83824,
  [SMALL_STATE(1893)] = 83848,
  [SMALL_STATE(1894)] = 83870,
  [SMALL_STATE(1895)] = 83890,
  [SMALL_STATE(1896)] = 83910,
  [SMALL_STATE(1897)] = 83936,
  [SMALL_STATE(1898)] = 83956,
  [SMALL_STATE(1899)] = 83976,
  [SMALL_STATE(1900)] = 83999,
  [SMALL_STATE(1901)] = 84022,
  [SMALL_STATE(1902)] = 84043,
  [SMALL_STATE(1903)] = 84064,
  [SMALL_STATE(1904)] = 84085,
  [SMALL_STATE(1905)] = 84108,
  [SMALL_STATE(1906)] = 84125,
  [SMALL_STATE(1907)] = 84142,
  [SMALL_STATE(1908)] = 84165,
  [SMALL_STATE(1909)] = 84186,
  [SMALL_STATE(1910)] = 84203,
  [SMALL_STATE(1911)] = 84222,
  [SMALL_STATE(1912)] = 84245,
  [SMALL_STATE(1913)] = 84264,
  [SMALL_STATE(1914)] = 84283,
  [SMALL_STATE(1915)] = 84302,
  [SMALL_STATE(1916)] = 84321,
  [SMALL_STATE(1917)] = 84342,
  [SMALL_STATE(1918)] = 84361,
  [SMALL_STATE(1919)] = 84384,
  [SMALL_STATE(1920)] = 84407,
  [SMALL_STATE(1921)] = 84430,
  [SMALL_STATE(1922)] = 84453,
  [SMALL_STATE(1923)] = 84472,
  [SMALL_STATE(1924)] = 84493,
  [SMALL_STATE(1925)] = 84516,
  [SMALL_STATE(1926)] = 84536,
  [SMALL_STATE(1927)] = 84556,
  [SMALL_STATE(1928)] = 84574,
  [SMALL_STATE(1929)] = 84594,
  [SMALL_STATE(1930)] = 84614,
  [SMALL_STATE(1931)] = 84634,
  [SMALL_STATE(1932)] = 84652,
  [SMALL_STATE(1933)] = 84672,
  [SMALL_STATE(1934)] = 84692,
  [SMALL_STATE(1935)] = 84712,
  [SMALL_STATE(1936)] = 84732,
  [SMALL_STATE(1937)] = 84752,
  [SMALL_STATE(1938)] = 84772,
  [SMALL_STATE(1939)] = 84792,
  [SMALL_STATE(1940)] = 84812,
  [SMALL_STATE(1941)] = 84832,
  [SMALL_STATE(1942)] = 84852,
  [SMALL_STATE(1943)] = 84872,
  [SMALL_STATE(1944)] = 84892,
  [SMALL_STATE(1945)] = 84910,
  [SMALL_STATE(1946)] = 84930,
  [SMALL_STATE(1947)] = 84948,
  [SMALL_STATE(1948)] = 84968,
  [SMALL_STATE(1949)] = 84988,
  [SMALL_STATE(1950)] = 85008,
  [SMALL_STATE(1951)] = 85028,
  [SMALL_STATE(1952)] = 85048,
  [SMALL_STATE(1953)] = 85068,
  [SMALL_STATE(1954)] = 85088,
  [SMALL_STATE(1955)] = 85108,
  [SMALL_STATE(1956)] = 85126,
  [SMALL_STATE(1957)] = 85146,
  [SMALL_STATE(1958)] = 85166,
  [SMALL_STATE(1959)] = 85186,
  [SMALL_STATE(1960)] = 85206,
  [SMALL_STATE(1961)] = 85226,
  [SMALL_STATE(1962)] = 85246,
  [SMALL_STATE(1963)] = 85266,
  [SMALL_STATE(1964)] = 85286,
  [SMALL_STATE(1965)] = 85306,
  [SMALL_STATE(1966)] = 85326,
  [SMALL_STATE(1967)] = 85346,
  [SMALL_STATE(1968)] = 85366,
  [SMALL_STATE(1969)] = 85386,
  [SMALL_STATE(1970)] = 85406,
  [SMALL_STATE(1971)] = 85426,
  [SMALL_STATE(1972)] = 85446,
  [SMALL_STATE(1973)] = 85466,
  [SMALL_STATE(1974)] = 85486,
  [SMALL_STATE(1975)] = 85506,
  [SMALL_STATE(1976)] = 85524,
  [SMALL_STATE(1977)] = 85544,
  [SMALL_STATE(1978)] = 85564,
  [SMALL_STATE(1979)] = 85584,
  [SMALL_STATE(1980)] = 85604,
  [SMALL_STATE(1981)] = 85624,
  [SMALL_STATE(1982)] = 85644,
  [SMALL_STATE(1983)] = 85664,
  [SMALL_STATE(1984)] = 85684,
  [SMALL_STATE(1985)] = 85704,
  [SMALL_STATE(1986)] = 85724,
  [SMALL_STATE(1987)] = 85744,
  [SMALL_STATE(1988)] = 85764,
  [SMALL_STATE(1989)] = 85784,
  [SMALL_STATE(1990)] = 85804,
  [SMALL_STATE(1991)] = 85824,
  [SMALL_STATE(1992)] = 85844,
  [SMALL_STATE(1993)] = 85864,
  [SMALL_STATE(1994)] = 85884,
  [SMALL_STATE(1995)] = 85904,
  [SMALL_STATE(1996)] = 85924,
  [SMALL_STATE(1997)] = 85944,
  [SMALL_STATE(1998)] = 85964,
  [SMALL_STATE(1999)] = 85984,
  [SMALL_STATE(2000)] = 86004,
  [SMALL_STATE(2001)] = 86024,
  [SMALL_STATE(2002)] = 86044,
  [SMALL_STATE(2003)] = 86064,
  [SMALL_STATE(2004)] = 86084,
  [SMALL_STATE(2005)] = 86104,
  [SMALL_STATE(2006)] = 86124,
  [SMALL_STATE(2007)] = 86144,
  [SMALL_STATE(2008)] = 86164,
  [SMALL_STATE(2009)] = 86184,
  [SMALL_STATE(2010)] = 86202,
  [SMALL_STATE(2011)] = 86222,
  [SMALL_STATE(2012)] = 86242,
  [SMALL_STATE(2013)] = 86262,
  [SMALL_STATE(2014)] = 86278,
  [SMALL_STATE(2015)] = 86298,
  [SMALL_STATE(2016)] = 86318,
  [SMALL_STATE(2017)] = 86338,
  [SMALL_STATE(2018)] = 86358,
  [SMALL_STATE(2019)] = 86378,
  [SMALL_STATE(2020)] = 86398,
  [SMALL_STATE(2021)] = 86418,
  [SMALL_STATE(2022)] = 86438,
  [SMALL_STATE(2023)] = 86458,
  [SMALL_STATE(2024)] = 86478,
  [SMALL_STATE(2025)] = 86498,
  [SMALL_STATE(2026)] = 86518,
  [SMALL_STATE(2027)] = 86538,
  [SMALL_STATE(2028)] = 86558,
  [SMALL_STATE(2029)] = 86578,
  [SMALL_STATE(2030)] = 86596,
  [SMALL_STATE(2031)] = 86616,
  [SMALL_STATE(2032)] = 86636,
  [SMALL_STATE(2033)] = 86654,
  [SMALL_STATE(2034)] = 86674,
  [SMALL_STATE(2035)] = 86694,
  [SMALL_STATE(2036)] = 86714,
  [SMALL_STATE(2037)] = 86734,
  [SMALL_STATE(2038)] = 86754,
  [SMALL_STATE(2039)] = 86774,
  [SMALL_STATE(2040)] = 86794,
  [SMALL_STATE(2041)] = 86814,
  [SMALL_STATE(2042)] = 86834,
  [SMALL_STATE(2043)] = 86852,
  [SMALL_STATE(2044)] = 86872,
  [SMALL_STATE(2045)] = 86892,
  [SMALL_STATE(2046)] = 86912,
  [SMALL_STATE(2047)] = 86932,
  [SMALL_STATE(2048)] = 86952,
  [SMALL_STATE(2049)] = 86972,
  [SMALL_STATE(2050)] = 86992,
  [SMALL_STATE(2051)] = 87012,
  [SMALL_STATE(2052)] = 87032,
  [SMALL_STATE(2053)] = 87052,
  [SMALL_STATE(2054)] = 87072,
  [SMALL_STATE(2055)] = 87092,
  [SMALL_STATE(2056)] = 87112,
  [SMALL_STATE(2057)] = 87132,
  [SMALL_STATE(2058)] = 87152,
  [SMALL_STATE(2059)] = 87172,
  [SMALL_STATE(2060)] = 87192,
  [SMALL_STATE(2061)] = 87208,
  [SMALL_STATE(2062)] = 87228,
  [SMALL_STATE(2063)] = 87248,
  [SMALL_STATE(2064)] = 87268,
  [SMALL_STATE(2065)] = 87288,
  [SMALL_STATE(2066)] = 87308,
  [SMALL_STATE(2067)] = 87328,
  [SMALL_STATE(2068)] = 87348,
  [SMALL_STATE(2069)] = 87368,
  [SMALL_STATE(2070)] = 87388,
  [SMALL_STATE(2071)] = 87408,
  [SMALL_STATE(2072)] = 87428,
  [SMALL_STATE(2073)] = 87446,
  [SMALL_STATE(2074)] = 87466,
  [SMALL_STATE(2075)] = 87486,
  [SMALL_STATE(2076)] = 87502,
  [SMALL_STATE(2077)] = 87522,
  [SMALL_STATE(2078)] = 87542,
  [SMALL_STATE(2079)] = 87562,
  [SMALL_STATE(2080)] = 87582,
  [SMALL_STATE(2081)] = 87602,
  [SMALL_STATE(2082)] = 87622,
  [SMALL_STATE(2083)] = 87642,
  [SMALL_STATE(2084)] = 87662,
  [SMALL_STATE(2085)] = 87682,
  [SMALL_STATE(2086)] = 87700,
  [SMALL_STATE(2087)] = 87720,
  [SMALL_STATE(2088)] = 87740,
  [SMALL_STATE(2089)] = 87760,
  [SMALL_STATE(2090)] = 87780,
  [SMALL_STATE(2091)] = 87798,
  [SMALL_STATE(2092)] = 87818,
  [SMALL_STATE(2093)] = 87838,
  [SMALL_STATE(2094)] = 87858,
  [SMALL_STATE(2095)] = 87878,
  [SMALL_STATE(2096)] = 87898,
  [SMALL_STATE(2097)] = 87918,
  [SMALL_STATE(2098)] = 87936,
  [SMALL_STATE(2099)] = 87956,
  [SMALL_STATE(2100)] = 87976,
  [SMALL_STATE(2101)] = 87996,
  [SMALL_STATE(2102)] = 88016,
  [SMALL_STATE(2103)] = 88036,
  [SMALL_STATE(2104)] = 88052,
  [SMALL_STATE(2105)] = 88072,
  [SMALL_STATE(2106)] = 88092,
  [SMALL_STATE(2107)] = 88112,
  [SMALL_STATE(2108)] = 88128,
  [SMALL_STATE(2109)] = 88148,
  [SMALL_STATE(2110)] = 88164,
  [SMALL_STATE(2111)] = 88184,
  [SMALL_STATE(2112)] = 88204,
  [SMALL_STATE(2113)] = 88224,
  [SMALL_STATE(2114)] = 88244,
  [SMALL_STATE(2115)] = 88264,
  [SMALL_STATE(2116)] = 88284,
  [SMALL_STATE(2117)] = 88304,
  [SMALL_STATE(2118)] = 88324,
  [SMALL_STATE(2119)] = 88344,
  [SMALL_STATE(2120)] = 88364,
  [SMALL_STATE(2121)] = 88384,
  [SMALL_STATE(2122)] = 88404,
  [SMALL_STATE(2123)] = 88424,
  [SMALL_STATE(2124)] = 88442,
  [SMALL_STATE(2125)] = 88462,
  [SMALL_STATE(2126)] = 88482,
  [SMALL_STATE(2127)] = 88502,
  [SMALL_STATE(2128)] = 88522,
  [SMALL_STATE(2129)] = 88542,
  [SMALL_STATE(2130)] = 88562,
  [SMALL_STATE(2131)] = 88580,
  [SMALL_STATE(2132)] = 88600,
  [SMALL_STATE(2133)] = 88620,
  [SMALL_STATE(2134)] = 88640,
  [SMALL_STATE(2135)] = 88660,
  [SMALL_STATE(2136)] = 88680,
  [SMALL_STATE(2137)] = 88700,
  [SMALL_STATE(2138)] = 88720,
  [SMALL_STATE(2139)] = 88738,
  [SMALL_STATE(2140)] = 88758,
  [SMALL_STATE(2141)] = 88778,
  [SMALL_STATE(2142)] = 88798,
  [SMALL_STATE(2143)] = 88818,
  [SMALL_STATE(2144)] = 88838,
  [SMALL_STATE(2145)] = 88854,
  [SMALL_STATE(2146)] = 88872,
  [SMALL_STATE(2147)] = 88892,
  [SMALL_STATE(2148)] = 88912,
  [SMALL_STATE(2149)] = 88932,
  [SMALL_STATE(2150)] = 88952,
  [SMALL_STATE(2151)] = 88972,
  [SMALL_STATE(2152)] = 88992,
  [SMALL_STATE(2153)] = 89012,
  [SMALL_STATE(2154)] = 89032,
  [SMALL_STATE(2155)] = 89052,
  [SMALL_STATE(2156)] = 89072,
  [SMALL_STATE(2157)] = 89092,
  [SMALL_STATE(2158)] = 89112,
  [SMALL_STATE(2159)] = 89132,
  [SMALL_STATE(2160)] = 89152,
  [SMALL_STATE(2161)] = 89172,
  [SMALL_STATE(2162)] = 89192,
  [SMALL_STATE(2163)] = 89212,
  [SMALL_STATE(2164)] = 89232,
  [SMALL_STATE(2165)] = 89252,
  [SMALL_STATE(2166)] = 89270,
  [SMALL_STATE(2167)] = 89288,
  [SMALL_STATE(2168)] = 89308,
  [SMALL_STATE(2169)] = 89328,
  [SMALL_STATE(2170)] = 89348,
  [SMALL_STATE(2171)] = 89366,
  [SMALL_STATE(2172)] = 89386,
  [SMALL_STATE(2173)] = 89406,
  [SMALL_STATE(2174)] = 89426,
  [SMALL_STATE(2175)] = 89446,
  [SMALL_STATE(2176)] = 89464,
  [SMALL_STATE(2177)] = 89484,
  [SMALL_STATE(2178)] = 89504,
  [SMALL_STATE(2179)] = 89524,
  [SMALL_STATE(2180)] = 89544,
  [SMALL_STATE(2181)] = 89562,
  [SMALL_STATE(2182)] = 89582,
  [SMALL_STATE(2183)] = 89602,
  [SMALL_STATE(2184)] = 89622,
  [SMALL_STATE(2185)] = 89642,
  [SMALL_STATE(2186)] = 89662,
  [SMALL_STATE(2187)] = 89682,
  [SMALL_STATE(2188)] = 89702,
  [SMALL_STATE(2189)] = 89722,
  [SMALL_STATE(2190)] = 89742,
  [SMALL_STATE(2191)] = 89762,
  [SMALL_STATE(2192)] = 89782,
  [SMALL_STATE(2193)] = 89802,
  [SMALL_STATE(2194)] = 89822,
  [SMALL_STATE(2195)] = 89842,
  [SMALL_STATE(2196)] = 89862,
  [SMALL_STATE(2197)] = 89882,
  [SMALL_STATE(2198)] = 89902,
  [SMALL_STATE(2199)] = 89920,
  [SMALL_STATE(2200)] = 89940,
  [SMALL_STATE(2201)] = 89960,
  [SMALL_STATE(2202)] = 89978,
  [SMALL_STATE(2203)] = 89998,
  [SMALL_STATE(2204)] = 90018,
  [SMALL_STATE(2205)] = 90038,
  [SMALL_STATE(2206)] = 90058,
  [SMALL_STATE(2207)] = 90076,
  [SMALL_STATE(2208)] = 90096,
  [SMALL_STATE(2209)] = 90116,
  [SMALL_STATE(2210)] = 90134,
  [SMALL_STATE(2211)] = 90154,
  [SMALL_STATE(2212)] = 90172,
  [SMALL_STATE(2213)] = 90192,
  [SMALL_STATE(2214)] = 90212,
  [SMALL_STATE(2215)] = 90232,
  [SMALL_STATE(2216)] = 90252,
  [SMALL_STATE(2217)] = 90270,
  [SMALL_STATE(2218)] = 90290,
  [SMALL_STATE(2219)] = 90308,
  [SMALL_STATE(2220)] = 90328,
  [SMALL_STATE(2221)] = 90348,
  [SMALL_STATE(2222)] = 90368,
  [SMALL_STATE(2223)] = 90388,
  [SMALL_STATE(2224)] = 90408,
  [SMALL_STATE(2225)] = 90428,
  [SMALL_STATE(2226)] = 90448,
  [SMALL_STATE(2227)] = 90468,
  [SMALL_STATE(2228)] = 90488,
  [SMALL_STATE(2229)] = 90508,
  [SMALL_STATE(2230)] = 90528,
  [SMALL_STATE(2231)] = 90546,
  [SMALL_STATE(2232)] = 90566,
  [SMALL_STATE(2233)] = 90586,
  [SMALL_STATE(2234)] = 90606,
  [SMALL_STATE(2235)] = 90626,
  [SMALL_STATE(2236)] = 90642,
  [SMALL_STATE(2237)] = 90662,
  [SMALL_STATE(2238)] = 90682,
  [SMALL_STATE(2239)] = 90698,
  [SMALL_STATE(2240)] = 90718,
  [SMALL_STATE(2241)] = 90738,
  [SMALL_STATE(2242)] = 90758,
  [SMALL_STATE(2243)] = 90778,
  [SMALL_STATE(2244)] = 90795,
  [SMALL_STATE(2245)] = 90812,
  [SMALL_STATE(2246)] = 90827,
  [SMALL_STATE(2247)] = 90842,
  [SMALL_STATE(2248)] = 90857,
  [SMALL_STATE(2249)] = 90874,
  [SMALL_STATE(2250)] = 90889,
  [SMALL_STATE(2251)] = 90906,
  [SMALL_STATE(2252)] = 90923,
  [SMALL_STATE(2253)] = 90938,
  [SMALL_STATE(2254)] = 90955,
  [SMALL_STATE(2255)] = 90970,
  [SMALL_STATE(2256)] = 90985,
  [SMALL_STATE(2257)] = 91002,
  [SMALL_STATE(2258)] = 91017,
  [SMALL_STATE(2259)] = 91032,
  [SMALL_STATE(2260)] = 91049,
  [SMALL_STATE(2261)] = 91064,
  [SMALL_STATE(2262)] = 91081,
  [SMALL_STATE(2263)] = 91096,
  [SMALL_STATE(2264)] = 91111,
  [SMALL_STATE(2265)] = 91126,
  [SMALL_STATE(2266)] = 91143,
  [SMALL_STATE(2267)] = 91158,
  [SMALL_STATE(2268)] = 91175,
  [SMALL_STATE(2269)] = 91190,
  [SMALL_STATE(2270)] = 91205,
  [SMALL_STATE(2271)] = 91220,
  [SMALL_STATE(2272)] = 91235,
  [SMALL_STATE(2273)] = 91250,
  [SMALL_STATE(2274)] = 91265,
  [SMALL_STATE(2275)] = 91282,
  [SMALL_STATE(2276)] = 91297,
  [SMALL_STATE(2277)] = 91312,
  [SMALL_STATE(2278)] = 91327,
  [SMALL_STATE(2279)] = 91344,
  [SMALL_STATE(2280)] = 91361,
  [SMALL_STATE(2281)] = 91376,
  [SMALL_STATE(2282)] = 91391,
  [SMALL_STATE(2283)] = 91408,
  [SMALL_STATE(2284)] = 91423,
  [SMALL_STATE(2285)] = 91438,
  [SMALL_STATE(2286)] = 91455,
  [SMALL_STATE(2287)] = 91472,
  [SMALL_STATE(2288)] = 91487,
  [SMALL_STATE(2289)] = 91504,
  [SMALL_STATE(2290)] = 91519,
  [SMALL_STATE(2291)] = 91534,
  [SMALL_STATE(2292)] = 91549,
  [SMALL_STATE(2293)] = 91564,
  [SMALL_STATE(2294)] = 91581,
  [SMALL_STATE(2295)] = 91596,
  [SMALL_STATE(2296)] = 91611,
  [SMALL_STATE(2297)] = 91626,
  [SMALL_STATE(2298)] = 91641,
  [SMALL_STATE(2299)] = 91656,
  [SMALL_STATE(2300)] = 91671,
  [SMALL_STATE(2301)] = 91688,
  [SMALL_STATE(2302)] = 91703,
  [SMALL_STATE(2303)] = 91720,
  [SMALL_STATE(2304)] = 91737,
  [SMALL_STATE(2305)] = 91754,
  [SMALL_STATE(2306)] = 91769,
  [SMALL_STATE(2307)] = 91786,
  [SMALL_STATE(2308)] = 91801,
  [SMALL_STATE(2309)] = 91816,
  [SMALL_STATE(2310)] = 91833,
  [SMALL_STATE(2311)] = 91850,
  [SMALL_STATE(2312)] = 91865,
  [SMALL_STATE(2313)] = 91880,
  [SMALL_STATE(2314)] = 91895,
  [SMALL_STATE(2315)] = 91912,
  [SMALL_STATE(2316)] = 91929,
  [SMALL_STATE(2317)] = 91944,
  [SMALL_STATE(2318)] = 91959,
  [SMALL_STATE(2319)] = 91974,
  [SMALL_STATE(2320)] = 91991,
  [SMALL_STATE(2321)] = 92006,
  [SMALL_STATE(2322)] = 92021,
  [SMALL_STATE(2323)] = 92038,
  [SMALL_STATE(2324)] = 92055,
  [SMALL_STATE(2325)] = 92072,
  [SMALL_STATE(2326)] = 92089,
  [SMALL_STATE(2327)] = 92104,
  [SMALL_STATE(2328)] = 92119,
  [SMALL_STATE(2329)] = 92134,
  [SMALL_STATE(2330)] = 92151,
  [SMALL_STATE(2331)] = 92168,
  [SMALL_STATE(2332)] = 92185,
  [SMALL_STATE(2333)] = 92202,
  [SMALL_STATE(2334)] = 92219,
  [SMALL_STATE(2335)] = 92236,
  [SMALL_STATE(2336)] = 92251,
  [SMALL_STATE(2337)] = 92266,
  [SMALL_STATE(2338)] = 92281,
  [SMALL_STATE(2339)] = 92296,
  [SMALL_STATE(2340)] = 92313,
  [SMALL_STATE(2341)] = 92330,
  [SMALL_STATE(2342)] = 92345,
  [SMALL_STATE(2343)] = 92360,
  [SMALL_STATE(2344)] = 92375,
  [SMALL_STATE(2345)] = 92392,
  [SMALL_STATE(2346)] = 92407,
  [SMALL_STATE(2347)] = 92422,
  [SMALL_STATE(2348)] = 92439,
  [SMALL_STATE(2349)] = 92454,
  [SMALL_STATE(2350)] = 92469,
  [SMALL_STATE(2351)] = 92486,
  [SMALL_STATE(2352)] = 92501,
  [SMALL_STATE(2353)] = 92518,
  [SMALL_STATE(2354)] = 92532,
  [SMALL_STATE(2355)] = 92546,
  [SMALL_STATE(2356)] = 92560,
  [SMALL_STATE(2357)] = 92574,
  [SMALL_STATE(2358)] = 92588,
  [SMALL_STATE(2359)] = 92602,
  [SMALL_STATE(2360)] = 92616,
  [SMALL_STATE(2361)] = 92630,
  [SMALL_STATE(2362)] = 92644,
  [SMALL_STATE(2363)] = 92658,
  [SMALL_STATE(2364)] = 92672,
  [SMALL_STATE(2365)] = 92686,
  [SMALL_STATE(2366)] = 92700,
  [SMALL_STATE(2367)] = 92714,
  [SMALL_STATE(2368)] = 92728,
  [SMALL_STATE(2369)] = 92742,
  [SMALL_STATE(2370)] = 92756,
  [SMALL_STATE(2371)] = 92770,
  [SMALL_STATE(2372)] = 92784,
  [SMALL_STATE(2373)] = 92798,
  [SMALL_STATE(2374)] = 92812,
  [SMALL_STATE(2375)] = 92826,
  [SMALL_STATE(2376)] = 92840,
  [SMALL_STATE(2377)] = 92854,
  [SMALL_STATE(2378)] = 92868,
  [SMALL_STATE(2379)] = 92882,
  [SMALL_STATE(2380)] = 92896,
  [SMALL_STATE(2381)] = 92910,
  [SMALL_STATE(2382)] = 92924,
  [SMALL_STATE(2383)] = 92938,
  [SMALL_STATE(2384)] = 92952,
  [SMALL_STATE(2385)] = 92966,
  [SMALL_STATE(2386)] = 92980,
  [SMALL_STATE(2387)] = 92994,
  [SMALL_STATE(2388)] = 93008,
  [SMALL_STATE(2389)] = 93022,
  [SMALL_STATE(2390)] = 93036,
  [SMALL_STATE(2391)] = 93050,
  [SMALL_STATE(2392)] = 93064,
  [SMALL_STATE(2393)] = 93078,
  [SMALL_STATE(2394)] = 93092,
  [SMALL_STATE(2395)] = 93106,
  [SMALL_STATE(2396)] = 93120,
  [SMALL_STATE(2397)] = 93134,
  [SMALL_STATE(2398)] = 93148,
  [SMALL_STATE(2399)] = 93162,
  [SMALL_STATE(2400)] = 93176,
  [SMALL_STATE(2401)] = 93190,
  [SMALL_STATE(2402)] = 93204,
  [SMALL_STATE(2403)] = 93218,
  [SMALL_STATE(2404)] = 93232,
  [SMALL_STATE(2405)] = 93246,
  [SMALL_STATE(2406)] = 93260,
  [SMALL_STATE(2407)] = 93274,
  [SMALL_STATE(2408)] = 93288,
  [SMALL_STATE(2409)] = 93302,
  [SMALL_STATE(2410)] = 93316,
  [SMALL_STATE(2411)] = 93330,
  [SMALL_STATE(2412)] = 93344,
  [SMALL_STATE(2413)] = 93358,
  [SMALL_STATE(2414)] = 93372,
  [SMALL_STATE(2415)] = 93386,
  [SMALL_STATE(2416)] = 93400,
  [SMALL_STATE(2417)] = 93414,
  [SMALL_STATE(2418)] = 93428,
  [SMALL_STATE(2419)] = 93442,
  [SMALL_STATE(2420)] = 93456,
  [SMALL_STATE(2421)] = 93470,
  [SMALL_STATE(2422)] = 93484,
  [SMALL_STATE(2423)] = 93498,
  [SMALL_STATE(2424)] = 93512,
  [SMALL_STATE(2425)] = 93526,
  [SMALL_STATE(2426)] = 93540,
  [SMALL_STATE(2427)] = 93554,
  [SMALL_STATE(2428)] = 93568,
  [SMALL_STATE(2429)] = 93582,
  [SMALL_STATE(2430)] = 93596,
  [SMALL_STATE(2431)] = 93610,
  [SMALL_STATE(2432)] = 93624,
  [SMALL_STATE(2433)] = 93638,
  [SMALL_STATE(2434)] = 93652,
  [SMALL_STATE(2435)] = 93666,
  [SMALL_STATE(2436)] = 93680,
  [SMALL_STATE(2437)] = 93694,
  [SMALL_STATE(2438)] = 93708,
  [SMALL_STATE(2439)] = 93722,
  [SMALL_STATE(2440)] = 93736,
  [SMALL_STATE(2441)] = 93750,
  [SMALL_STATE(2442)] = 93764,
  [SMALL_STATE(2443)] = 93778,
  [SMALL_STATE(2444)] = 93792,
  [SMALL_STATE(2445)] = 93806,
  [SMALL_STATE(2446)] = 93820,
  [SMALL_STATE(2447)] = 93834,
  [SMALL_STATE(2448)] = 93848,
  [SMALL_STATE(2449)] = 93862,
  [SMALL_STATE(2450)] = 93876,
  [SMALL_STATE(2451)] = 93890,
  [SMALL_STATE(2452)] = 93904,
  [SMALL_STATE(2453)] = 93918,
  [SMALL_STATE(2454)] = 93932,
  [SMALL_STATE(2455)] = 93946,
  [SMALL_STATE(2456)] = 93960,
  [SMALL_STATE(2457)] = 93974,
  [SMALL_STATE(2458)] = 93988,
  [SMALL_STATE(2459)] = 94002,
  [SMALL_STATE(2460)] = 94016,
  [SMALL_STATE(2461)] = 94030,
  [SMALL_STATE(2462)] = 94044,
  [SMALL_STATE(2463)] = 94058,
  [SMALL_STATE(2464)] = 94072,
  [SMALL_STATE(2465)] = 94086,
  [SMALL_STATE(2466)] = 94100,
  [SMALL_STATE(2467)] = 94114,
  [SMALL_STATE(2468)] = 94128,
  [SMALL_STATE(2469)] = 94142,
  [SMALL_STATE(2470)] = 94156,
  [SMALL_STATE(2471)] = 94170,
  [SMALL_STATE(2472)] = 94184,
  [SMALL_STATE(2473)] = 94198,
  [SMALL_STATE(2474)] = 94212,
  [SMALL_STATE(2475)] = 94226,
  [SMALL_STATE(2476)] = 94240,
  [SMALL_STATE(2477)] = 94254,
  [SMALL_STATE(2478)] = 94268,
  [SMALL_STATE(2479)] = 94282,
  [SMALL_STATE(2480)] = 94296,
  [SMALL_STATE(2481)] = 94310,
  [SMALL_STATE(2482)] = 94324,
  [SMALL_STATE(2483)] = 94338,
  [SMALL_STATE(2484)] = 94352,
  [SMALL_STATE(2485)] = 94366,
  [SMALL_STATE(2486)] = 94380,
  [SMALL_STATE(2487)] = 94394,
  [SMALL_STATE(2488)] = 94408,
  [SMALL_STATE(2489)] = 94422,
  [SMALL_STATE(2490)] = 94436,
  [SMALL_STATE(2491)] = 94450,
  [SMALL_STATE(2492)] = 94464,
  [SMALL_STATE(2493)] = 94478,
  [SMALL_STATE(2494)] = 94492,
  [SMALL_STATE(2495)] = 94506,
  [SMALL_STATE(2496)] = 94520,
  [SMALL_STATE(2497)] = 94534,
  [SMALL_STATE(2498)] = 94548,
  [SMALL_STATE(2499)] = 94562,
  [SMALL_STATE(2500)] = 94576,
  [SMALL_STATE(2501)] = 94590,
  [SMALL_STATE(2502)] = 94604,
  [SMALL_STATE(2503)] = 94618,
  [SMALL_STATE(2504)] = 94632,
  [SMALL_STATE(2505)] = 94646,
  [SMALL_STATE(2506)] = 94660,
  [SMALL_STATE(2507)] = 94674,
  [SMALL_STATE(2508)] = 94688,
  [SMALL_STATE(2509)] = 94702,
  [SMALL_STATE(2510)] = 94716,
  [SMALL_STATE(2511)] = 94730,
  [SMALL_STATE(2512)] = 94744,
  [SMALL_STATE(2513)] = 94758,
  [SMALL_STATE(2514)] = 94772,
  [SMALL_STATE(2515)] = 94786,
  [SMALL_STATE(2516)] = 94800,
  [SMALL_STATE(2517)] = 94814,
  [SMALL_STATE(2518)] = 94828,
  [SMALL_STATE(2519)] = 94842,
  [SMALL_STATE(2520)] = 94856,
  [SMALL_STATE(2521)] = 94870,
  [SMALL_STATE(2522)] = 94884,
  [SMALL_STATE(2523)] = 94898,
  [SMALL_STATE(2524)] = 94912,
  [SMALL_STATE(2525)] = 94926,
  [SMALL_STATE(2526)] = 94940,
  [SMALL_STATE(2527)] = 94954,
  [SMALL_STATE(2528)] = 94968,
  [SMALL_STATE(2529)] = 94982,
  [SMALL_STATE(2530)] = 94996,
  [SMALL_STATE(2531)] = 95010,
  [SMALL_STATE(2532)] = 95024,
  [SMALL_STATE(2533)] = 95038,
  [SMALL_STATE(2534)] = 95052,
  [SMALL_STATE(2535)] = 95066,
  [SMALL_STATE(2536)] = 95080,
  [SMALL_STATE(2537)] = 95094,
  [SMALL_STATE(2538)] = 95108,
  [SMALL_STATE(2539)] = 95122,
  [SMALL_STATE(2540)] = 95136,
  [SMALL_STATE(2541)] = 95150,
  [SMALL_STATE(2542)] = 95164,
  [SMALL_STATE(2543)] = 95178,
  [SMALL_STATE(2544)] = 95192,
  [SMALL_STATE(2545)] = 95206,
  [SMALL_STATE(2546)] = 95220,
  [SMALL_STATE(2547)] = 95234,
  [SMALL_STATE(2548)] = 95248,
  [SMALL_STATE(2549)] = 95262,
  [SMALL_STATE(2550)] = 95276,
  [SMALL_STATE(2551)] = 95290,
  [SMALL_STATE(2552)] = 95304,
  [SMALL_STATE(2553)] = 95318,
  [SMALL_STATE(2554)] = 95332,
  [SMALL_STATE(2555)] = 95346,
  [SMALL_STATE(2556)] = 95360,
  [SMALL_STATE(2557)] = 95374,
  [SMALL_STATE(2558)] = 95388,
  [SMALL_STATE(2559)] = 95402,
  [SMALL_STATE(2560)] = 95416,
  [SMALL_STATE(2561)] = 95430,
  [SMALL_STATE(2562)] = 95444,
  [SMALL_STATE(2563)] = 95458,
  [SMALL_STATE(2564)] = 95472,
  [SMALL_STATE(2565)] = 95486,
  [SMALL_STATE(2566)] = 95500,
  [SMALL_STATE(2567)] = 95514,
  [SMALL_STATE(2568)] = 95528,
  [SMALL_STATE(2569)] = 95542,
  [SMALL_STATE(2570)] = 95556,
  [SMALL_STATE(2571)] = 95570,
  [SMALL_STATE(2572)] = 95584,
  [SMALL_STATE(2573)] = 95598,
  [SMALL_STATE(2574)] = 95612,
  [SMALL_STATE(2575)] = 95626,
  [SMALL_STATE(2576)] = 95640,
  [SMALL_STATE(2577)] = 95654,
  [SMALL_STATE(2578)] = 95668,
  [SMALL_STATE(2579)] = 95682,
  [SMALL_STATE(2580)] = 95696,
  [SMALL_STATE(2581)] = 95710,
  [SMALL_STATE(2582)] = 95724,
  [SMALL_STATE(2583)] = 95738,
  [SMALL_STATE(2584)] = 95752,
  [SMALL_STATE(2585)] = 95766,
  [SMALL_STATE(2586)] = 95780,
  [SMALL_STATE(2587)] = 95794,
  [SMALL_STATE(2588)] = 95808,
  [SMALL_STATE(2589)] = 95822,
  [SMALL_STATE(2590)] = 95836,
  [SMALL_STATE(2591)] = 95850,
  [SMALL_STATE(2592)] = 95864,
  [SMALL_STATE(2593)] = 95878,
  [SMALL_STATE(2594)] = 95892,
  [SMALL_STATE(2595)] = 95906,
  [SMALL_STATE(2596)] = 95920,
  [SMALL_STATE(2597)] = 95934,
  [SMALL_STATE(2598)] = 95948,
  [SMALL_STATE(2599)] = 95962,
  [SMALL_STATE(2600)] = 95976,
  [SMALL_STATE(2601)] = 95990,
  [SMALL_STATE(2602)] = 96004,
  [SMALL_STATE(2603)] = 96018,
  [SMALL_STATE(2604)] = 96032,
  [SMALL_STATE(2605)] = 96046,
  [SMALL_STATE(2606)] = 96060,
  [SMALL_STATE(2607)] = 96074,
  [SMALL_STATE(2608)] = 96088,
  [SMALL_STATE(2609)] = 96102,
  [SMALL_STATE(2610)] = 96116,
  [SMALL_STATE(2611)] = 96130,
  [SMALL_STATE(2612)] = 96144,
  [SMALL_STATE(2613)] = 96158,
  [SMALL_STATE(2614)] = 96172,
  [SMALL_STATE(2615)] = 96186,
  [SMALL_STATE(2616)] = 96200,
  [SMALL_STATE(2617)] = 96214,
  [SMALL_STATE(2618)] = 96228,
  [SMALL_STATE(2619)] = 96242,
  [SMALL_STATE(2620)] = 96256,
  [SMALL_STATE(2621)] = 96270,
  [SMALL_STATE(2622)] = 96284,
  [SMALL_STATE(2623)] = 96298,
  [SMALL_STATE(2624)] = 96312,
  [SMALL_STATE(2625)] = 96326,
  [SMALL_STATE(2626)] = 96340,
  [SMALL_STATE(2627)] = 96354,
  [SMALL_STATE(2628)] = 96368,
  [SMALL_STATE(2629)] = 96382,
  [SMALL_STATE(2630)] = 96396,
  [SMALL_STATE(2631)] = 96410,
  [SMALL_STATE(2632)] = 96424,
  [SMALL_STATE(2633)] = 96438,
  [SMALL_STATE(2634)] = 96452,
  [SMALL_STATE(2635)] = 96466,
  [SMALL_STATE(2636)] = 96480,
  [SMALL_STATE(2637)] = 96494,
  [SMALL_STATE(2638)] = 96508,
  [SMALL_STATE(2639)] = 96522,
  [SMALL_STATE(2640)] = 96536,
  [SMALL_STATE(2641)] = 96550,
  [SMALL_STATE(2642)] = 96564,
  [SMALL_STATE(2643)] = 96578,
  [SMALL_STATE(2644)] = 96592,
  [SMALL_STATE(2645)] = 96606,
  [SMALL_STATE(2646)] = 96620,
  [SMALL_STATE(2647)] = 96634,
  [SMALL_STATE(2648)] = 96648,
  [SMALL_STATE(2649)] = 96662,
  [SMALL_STATE(2650)] = 96676,
  [SMALL_STATE(2651)] = 96690,
  [SMALL_STATE(2652)] = 96704,
  [SMALL_STATE(2653)] = 96718,
  [SMALL_STATE(2654)] = 96732,
  [SMALL_STATE(2655)] = 96746,
  [SMALL_STATE(2656)] = 96760,
  [SMALL_STATE(2657)] = 96774,
  [SMALL_STATE(2658)] = 96788,
  [SMALL_STATE(2659)] = 96802,
  [SMALL_STATE(2660)] = 96816,
  [SMALL_STATE(2661)] = 96830,
  [SMALL_STATE(2662)] = 96844,
  [SMALL_STATE(2663)] = 96858,
  [SMALL_STATE(2664)] = 96872,
  [SMALL_STATE(2665)] = 96886,
  [SMALL_STATE(2666)] = 96900,
  [SMALL_STATE(2667)] = 96914,
  [SMALL_STATE(2668)] = 96928,
  [SMALL_STATE(2669)] = 96942,
  [SMALL_STATE(2670)] = 96956,
  [SMALL_STATE(2671)] = 96970,
  [SMALL_STATE(2672)] = 96984,
  [SMALL_STATE(2673)] = 96998,
  [SMALL_STATE(2674)] = 97012,
  [SMALL_STATE(2675)] = 97026,
  [SMALL_STATE(2676)] = 97040,
  [SMALL_STATE(2677)] = 97054,
  [SMALL_STATE(2678)] = 97068,
  [SMALL_STATE(2679)] = 97082,
  [SMALL_STATE(2680)] = 97096,
  [SMALL_STATE(2681)] = 97110,
  [SMALL_STATE(2682)] = 97124,
  [SMALL_STATE(2683)] = 97138,
  [SMALL_STATE(2684)] = 97152,
  [SMALL_STATE(2685)] = 97166,
  [SMALL_STATE(2686)] = 97180,
  [SMALL_STATE(2687)] = 97194,
  [SMALL_STATE(2688)] = 97208,
  [SMALL_STATE(2689)] = 97222,
  [SMALL_STATE(2690)] = 97236,
  [SMALL_STATE(2691)] = 97250,
  [SMALL_STATE(2692)] = 97264,
  [SMALL_STATE(2693)] = 97278,
  [SMALL_STATE(2694)] = 97292,
  [SMALL_STATE(2695)] = 97306,
  [SMALL_STATE(2696)] = 97320,
  [SMALL_STATE(2697)] = 97334,
  [SMALL_STATE(2698)] = 97348,
  [SMALL_STATE(2699)] = 97362,
  [SMALL_STATE(2700)] = 97376,
  [SMALL_STATE(2701)] = 97390,
  [SMALL_STATE(2702)] = 97404,
  [SMALL_STATE(2703)] = 97418,
  [SMALL_STATE(2704)] = 97432,
  [SMALL_STATE(2705)] = 97446,
  [SMALL_STATE(2706)] = 97460,
  [SMALL_STATE(2707)] = 97474,
  [SMALL_STATE(2708)] = 97488,
  [SMALL_STATE(2709)] = 97502,
  [SMALL_STATE(2710)] = 97516,
  [SMALL_STATE(2711)] = 97530,
  [SMALL_STATE(2712)] = 97544,
  [SMALL_STATE(2713)] = 97558,
  [SMALL_STATE(2714)] = 97572,
  [SMALL_STATE(2715)] = 97586,
  [SMALL_STATE(2716)] = 97600,
  [SMALL_STATE(2717)] = 97614,
  [SMALL_STATE(2718)] = 97628,
  [SMALL_STATE(2719)] = 97642,
  [SMALL_STATE(2720)] = 97656,
  [SMALL_STATE(2721)] = 97670,
  [SMALL_STATE(2722)] = 97684,
  [SMALL_STATE(2723)] = 97698,
  [SMALL_STATE(2724)] = 97712,
  [SMALL_STATE(2725)] = 97726,
  [SMALL_STATE(2726)] = 97740,
  [SMALL_STATE(2727)] = 97754,
  [SMALL_STATE(2728)] = 97768,
  [SMALL_STATE(2729)] = 97782,
  [SMALL_STATE(2730)] = 97796,
  [SMALL_STATE(2731)] = 97810,
  [SMALL_STATE(2732)] = 97824,
  [SMALL_STATE(2733)] = 97838,
  [SMALL_STATE(2734)] = 97852,
  [SMALL_STATE(2735)] = 97866,
  [SMALL_STATE(2736)] = 97880,
  [SMALL_STATE(2737)] = 97894,
  [SMALL_STATE(2738)] = 97908,
  [SMALL_STATE(2739)] = 97922,
  [SMALL_STATE(2740)] = 97936,
  [SMALL_STATE(2741)] = 97950,
  [SMALL_STATE(2742)] = 97964,
  [SMALL_STATE(2743)] = 97978,
  [SMALL_STATE(2744)] = 97992,
  [SMALL_STATE(2745)] = 98006,
  [SMALL_STATE(2746)] = 98020,
  [SMALL_STATE(2747)] = 98034,
  [SMALL_STATE(2748)] = 98048,
  [SMALL_STATE(2749)] = 98062,
  [SMALL_STATE(2750)] = 98076,
  [SMALL_STATE(2751)] = 98090,
  [SMALL_STATE(2752)] = 98104,
  [SMALL_STATE(2753)] = 98118,
  [SMALL_STATE(2754)] = 98132,
  [SMALL_STATE(2755)] = 98146,
  [SMALL_STATE(2756)] = 98160,
  [SMALL_STATE(2757)] = 98174,
  [SMALL_STATE(2758)] = 98188,
  [SMALL_STATE(2759)] = 98202,
  [SMALL_STATE(2760)] = 98216,
  [SMALL_STATE(2761)] = 98230,
  [SMALL_STATE(2762)] = 98244,
  [SMALL_STATE(2763)] = 98258,
  [SMALL_STATE(2764)] = 98272,
  [SMALL_STATE(2765)] = 98286,
  [SMALL_STATE(2766)] = 98300,
  [SMALL_STATE(2767)] = 98314,
  [SMALL_STATE(2768)] = 98328,
  [SMALL_STATE(2769)] = 98342,
  [SMALL_STATE(2770)] = 98356,
  [SMALL_STATE(2771)] = 98370,
  [SMALL_STATE(2772)] = 98384,
  [SMALL_STATE(2773)] = 98398,
  [SMALL_STATE(2774)] = 98412,
  [SMALL_STATE(2775)] = 98426,
  [SMALL_STATE(2776)] = 98440,
  [SMALL_STATE(2777)] = 98454,
  [SMALL_STATE(2778)] = 98468,
  [SMALL_STATE(2779)] = 98482,
  [SMALL_STATE(2780)] = 98496,
  [SMALL_STATE(2781)] = 98510,
  [SMALL_STATE(2782)] = 98524,
  [SMALL_STATE(2783)] = 98538,
  [SMALL_STATE(2784)] = 98552,
  [SMALL_STATE(2785)] = 98566,
  [SMALL_STATE(2786)] = 98580,
  [SMALL_STATE(2787)] = 98594,
  [SMALL_STATE(2788)] = 98608,
  [SMALL_STATE(2789)] = 98622,
  [SMALL_STATE(2790)] = 98636,
  [SMALL_STATE(2791)] = 98650,
  [SMALL_STATE(2792)] = 98664,
  [SMALL_STATE(2793)] = 98678,
  [SMALL_STATE(2794)] = 98692,
  [SMALL_STATE(2795)] = 98706,
  [SMALL_STATE(2796)] = 98720,
  [SMALL_STATE(2797)] = 98734,
  [SMALL_STATE(2798)] = 98748,
  [SMALL_STATE(2799)] = 98762,
  [SMALL_STATE(2800)] = 98776,
  [SMALL_STATE(2801)] = 98790,
  [SMALL_STATE(2802)] = 98804,
  [SMALL_STATE(2803)] = 98818,
  [SMALL_STATE(2804)] = 98832,
  [SMALL_STATE(2805)] = 98846,
  [SMALL_STATE(2806)] = 98860,
  [SMALL_STATE(2807)] = 98874,
  [SMALL_STATE(2808)] = 98888,
  [SMALL_STATE(2809)] = 98902,
  [SMALL_STATE(2810)] = 98916,
  [SMALL_STATE(2811)] = 98930,
  [SMALL_STATE(2812)] = 98944,
  [SMALL_STATE(2813)] = 98958,
  [SMALL_STATE(2814)] = 98972,
  [SMALL_STATE(2815)] = 98986,
  [SMALL_STATE(2816)] = 99000,
  [SMALL_STATE(2817)] = 99014,
  [SMALL_STATE(2818)] = 99028,
  [SMALL_STATE(2819)] = 99042,
  [SMALL_STATE(2820)] = 99056,
  [SMALL_STATE(2821)] = 99070,
  [SMALL_STATE(2822)] = 99084,
  [SMALL_STATE(2823)] = 99098,
  [SMALL_STATE(2824)] = 99112,
  [SMALL_STATE(2825)] = 99126,
  [SMALL_STATE(2826)] = 99140,
  [SMALL_STATE(2827)] = 99154,
  [SMALL_STATE(2828)] = 99168,
  [SMALL_STATE(2829)] = 99182,
  [SMALL_STATE(2830)] = 99196,
  [SMALL_STATE(2831)] = 99210,
  [SMALL_STATE(2832)] = 99224,
  [SMALL_STATE(2833)] = 99238,
  [SMALL_STATE(2834)] = 99252,
  [SMALL_STATE(2835)] = 99266,
  [SMALL_STATE(2836)] = 99280,
  [SMALL_STATE(2837)] = 99294,
  [SMALL_STATE(2838)] = 99308,
  [SMALL_STATE(2839)] = 99322,
  [SMALL_STATE(2840)] = 99336,
  [SMALL_STATE(2841)] = 99350,
  [SMALL_STATE(2842)] = 99364,
  [SMALL_STATE(2843)] = 99378,
  [SMALL_STATE(2844)] = 99392,
  [SMALL_STATE(2845)] = 99406,
  [SMALL_STATE(2846)] = 99420,
  [SMALL_STATE(2847)] = 99434,
  [SMALL_STATE(2848)] = 99448,
  [SMALL_STATE(2849)] = 99462,
  [SMALL_STATE(2850)] = 99476,
  [SMALL_STATE(2851)] = 99490,
  [SMALL_STATE(2852)] = 99504,
  [SMALL_STATE(2853)] = 99518,
  [SMALL_STATE(2854)] = 99532,
  [SMALL_STATE(2855)] = 99546,
  [SMALL_STATE(2856)] = 99560,
  [SMALL_STATE(2857)] = 99574,
  [SMALL_STATE(2858)] = 99588,
  [SMALL_STATE(2859)] = 99602,
  [SMALL_STATE(2860)] = 99616,
  [SMALL_STATE(2861)] = 99630,
  [SMALL_STATE(2862)] = 99644,
  [SMALL_STATE(2863)] = 99658,
  [SMALL_STATE(2864)] = 99672,
  [SMALL_STATE(2865)] = 99686,
  [SMALL_STATE(2866)] = 99700,
  [SMALL_STATE(2867)] = 99714,
  [SMALL_STATE(2868)] = 99728,
  [SMALL_STATE(2869)] = 99742,
  [SMALL_STATE(2870)] = 99756,
  [SMALL_STATE(2871)] = 99770,
  [SMALL_STATE(2872)] = 99784,
  [SMALL_STATE(2873)] = 99798,
  [SMALL_STATE(2874)] = 99812,
  [SMALL_STATE(2875)] = 99826,
  [SMALL_STATE(2876)] = 99840,
  [SMALL_STATE(2877)] = 99854,
  [SMALL_STATE(2878)] = 99868,
  [SMALL_STATE(2879)] = 99882,
  [SMALL_STATE(2880)] = 99896,
  [SMALL_STATE(2881)] = 99910,
  [SMALL_STATE(2882)] = 99924,
  [SMALL_STATE(2883)] = 99938,
  [SMALL_STATE(2884)] = 99952,
  [SMALL_STATE(2885)] = 99966,
  [SMALL_STATE(2886)] = 99980,
  [SMALL_STATE(2887)] = 99994,
  [SMALL_STATE(2888)] = 100008,
  [SMALL_STATE(2889)] = 100022,
  [SMALL_STATE(2890)] = 100036,
  [SMALL_STATE(2891)] = 100050,
  [SMALL_STATE(2892)] = 100064,
  [SMALL_STATE(2893)] = 100078,
  [SMALL_STATE(2894)] = 100092,
  [SMALL_STATE(2895)] = 100106,
  [SMALL_STATE(2896)] = 100120,
  [SMALL_STATE(2897)] = 100134,
  [SMALL_STATE(2898)] = 100148,
  [SMALL_STATE(2899)] = 100162,
  [SMALL_STATE(2900)] = 100176,
  [SMALL_STATE(2901)] = 100190,
  [SMALL_STATE(2902)] = 100204,
  [SMALL_STATE(2903)] = 100218,
  [SMALL_STATE(2904)] = 100232,
  [SMALL_STATE(2905)] = 100246,
  [SMALL_STATE(2906)] = 100260,
  [SMALL_STATE(2907)] = 100274,
  [SMALL_STATE(2908)] = 100288,
  [SMALL_STATE(2909)] = 100302,
  [SMALL_STATE(2910)] = 100316,
  [SMALL_STATE(2911)] = 100330,
  [SMALL_STATE(2912)] = 100344,
  [SMALL_STATE(2913)] = 100358,
  [SMALL_STATE(2914)] = 100372,
  [SMALL_STATE(2915)] = 100386,
  [SMALL_STATE(2916)] = 100400,
  [SMALL_STATE(2917)] = 100414,
  [SMALL_STATE(2918)] = 100428,
  [SMALL_STATE(2919)] = 100442,
  [SMALL_STATE(2920)] = 100446,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2793),
  [5] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [7] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_translation_unit, 0),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1456),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(560),
  [13] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1337),
  [15] = {.entry = {.count = 1, .reusable = false}}, SHIFT(772),
  [17] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2915),
  [19] = {.entry = {.count = 1, .reusable = false}}, SHIFT(690),
  [21] = {.entry = {.count = 1, .reusable = false}}, SHIFT(587),
  [23] = {.entry = {.count = 1, .reusable = false}}, SHIFT(425),
  [25] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1041),
  [27] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1454),
  [29] = {.entry = {.count = 1, .reusable = false}}, SHIFT(819),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1462),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2909),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(456),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2908),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1340),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1743),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1161),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(440),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(444),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(445),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1936),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1232),
  [55] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1166),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(452),
  [61] = {.entry = {.count = 1, .reusable = false}}, SHIFT(446),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(432),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(604),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1008),
  [69] = {.entry = {.count = 1, .reusable = false}}, SHIFT(603),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2892),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(443),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2891),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(454),
  [79] = {.entry = {.count = 1, .reusable = false}}, SHIFT(423),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1722),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2347),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1051),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(426),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(746),
  [91] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2875),
  [93] = {.entry = {.count = 1, .reusable = false}}, SHIFT(743),
  [95] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2864),
  [97] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1911),
  [99] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2852),
  [101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(430),
  [103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(862),
  [105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1052),
  [107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(458),
  [109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2340),
  [111] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2829),
  [113] = {.entry = {.count = 1, .reusable = false}}, SHIFT(948),
  [115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2331),
  [117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2330),
  [119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2329),
  [121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1053),
  [123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2821),
  [125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1153),
  [127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(480),
  [129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2325),
  [131] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2816),
  [133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1057),
  [135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2811),
  [137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(496),
  [139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2324),
  [141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(663),
  [143] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2809),
  [145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(790),
  [153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(427),
  [155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [157] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2),
  [159] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1456),
  [162] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(560),
  [165] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1337),
  [168] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(772),
  [171] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2915),
  [174] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(690),
  [177] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(587),
  [180] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(425),
  [183] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1041),
  [186] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1454),
  [189] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(819),
  [192] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1462),
  [195] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2909),
  [198] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(456),
  [201] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2908),
  [204] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1340),
  [207] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1743),
  [210] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1161),
  [213] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(440),
  [216] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(444),
  [219] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(445),
  [222] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1936),
  [225] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1232),
  [228] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(725),
  [231] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1166),
  [234] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(452),
  [237] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(446),
  [240] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(432),
  [243] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(604),
  [246] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1008),
  [249] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(603),
  [252] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2892),
  [255] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(443),
  [258] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2891),
  [261] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(454),
  [264] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(423),
  [267] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1722),
  [270] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2347),
  [273] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1051),
  [276] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(426),
  [279] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(746),
  [282] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2875),
  [285] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(743),
  [288] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2864),
  [291] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1911),
  [294] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2852),
  [297] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(430),
  [300] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(862),
  [303] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1052),
  [306] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(458),
  [309] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2340),
  [312] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2829),
  [315] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(948),
  [318] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2331),
  [321] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2330),
  [324] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2329),
  [327] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1053),
  [330] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2821),
  [333] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1153),
  [336] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(480),
  [339] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2325),
  [342] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2816),
  [345] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(1057),
  [348] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2811),
  [351] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(496),
  [354] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2324),
  [357] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(663),
  [360] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(2809),
  [363] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(435),
  [366] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(438),
  [369] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(808),
  [372] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(790),
  [375] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(427),
  [378] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2), SHIFT_REPEAT(796),
  [381] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_translation_unit, 1),
  [383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(556),
  [385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1354),
  [387] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2491),
  [389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1165),
  [391] = {.entry = {.count = 1, .reusable = false}}, SHIFT(450),
  [393] = {.entry = {.count = 1, .reusable = false}}, SHIFT(788),
  [395] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1005),
  [397] = {.entry = {.count = 1, .reusable = false}}, SHIFT(775),
  [399] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2916),
  [401] = {.entry = {.count = 1, .reusable = false}}, SHIFT(555),
  [403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1433),
  [405] = {.entry = {.count = 1, .reusable = false}}, SHIFT(547),
  [407] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1362),
  [409] = {.entry = {.count = 1, .reusable = false}}, SHIFT(554),
  [411] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1448),
  [413] = {.entry = {.count = 1, .reusable = false}}, SHIFT(542),
  [415] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1403),
  [417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(553),
  [419] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1352),
  [421] = {.entry = {.count = 1, .reusable = false}}, SHIFT(557),
  [423] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1419),
  [425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(540),
  [427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1405),
  [429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(558),
  [431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1424),
  [433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(538),
  [435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1406),
  [437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(545),
  [439] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1435),
  [441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(561),
  [443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1400),
  [445] = {.entry = {.count = 1, .reusable = false}}, SHIFT(543),
  [447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1407),
  [449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(562),
  [451] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1396),
  [453] = {.entry = {.count = 1, .reusable = false}}, SHIFT(550),
  [455] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1422),
  [457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(549),
  [459] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1358),
  [461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(536),
  [463] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1450),
  [465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(548),
  [467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1365),
  [469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(552),
  [471] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1440),
  [473] = {.entry = {.count = 1, .reusable = false}}, SHIFT(532),
  [475] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1441),
  [477] = {.entry = {.count = 1, .reusable = false}}, SHIFT(541),
  [479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1361),
  [481] = {.entry = {.count = 1, .reusable = false}}, SHIFT(544),
  [483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1402),
  [485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1118),
  [487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1350),
  [489] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2),
  [491] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(587),
  [494] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(819),
  [497] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2491),
  [500] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(456),
  [503] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1340),
  [506] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1743),
  [509] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1165),
  [512] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(440),
  [515] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(444),
  [518] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(445),
  [521] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1936),
  [524] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1232),
  [527] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2),
  [529] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1166),
  [532] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(452),
  [535] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(446),
  [538] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(788),
  [541] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2253),
  [544] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(775),
  [547] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2916),
  [550] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(443),
  [553] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2891),
  [556] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(454),
  [559] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2347),
  [562] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2852),
  [565] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(563),
  [568] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2811),
  [571] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1912),
  [574] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 110), SHIFT(463),
  [577] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 110),
  [579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(447),
  [581] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 7, .production_id = 110), SHIFT(431),
  [584] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 3), SHIFT(463),
  [587] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 3),
  [589] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 3), SHIFT(431),
  [592] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 105), SHIFT(463),
  [595] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 105),
  [597] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 7, .production_id = 105), SHIFT(431),
  [600] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 111), SHIFT(463),
  [603] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 111),
  [605] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 6, .production_id = 111), SHIFT(431),
  [608] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 5, .production_id = 105), SHIFT(463),
  [611] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 5, .production_id = 105),
  [613] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 5, .production_id = 105), SHIFT(431),
  [616] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 105), SHIFT(463),
  [619] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 105),
  [621] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 6, .production_id = 105), SHIFT(431),
  [624] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 106), SHIFT(463),
  [627] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 106),
  [629] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 6, .production_id = 106), SHIFT(431),
  [632] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 106), SHIFT(463),
  [635] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 106),
  [637] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 7, .production_id = 106), SHIFT(431),
  [640] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1084),
  [642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2460),
  [644] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1414),
  [646] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1181),
  [648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [650] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 5, .production_id = 106), SHIFT(463),
  [653] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 5, .production_id = 106),
  [655] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 5, .production_id = 106), SHIFT(431),
  [658] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 4), SHIFT(463),
  [661] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 4),
  [663] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 4), SHIFT(431),
  [666] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 110), SHIFT(463),
  [669] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 6, .production_id = 110),
  [671] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 6, .production_id = 110), SHIFT(431),
  [674] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 111), SHIFT(463),
  [677] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 7, .production_id = 111),
  [679] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 7, .production_id = 111), SHIFT(431),
  [682] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 8, .production_id = 111), SHIFT(463),
  [685] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 8, .production_id = 111),
  [687] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 8, .production_id = 111), SHIFT(431),
  [690] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 8, .production_id = 110), SHIFT(463),
  [693] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 8, .production_id = 110),
  [695] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 8, .production_id = 110), SHIFT(431),
  [698] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(463),
  [701] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(690),
  [704] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(425),
  [707] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1041),
  [710] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2),
  [712] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(725),
  [715] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(458),
  [718] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(447),
  [721] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1051),
  [724] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(426),
  [727] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(746),
  [730] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2875),
  [733] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(743),
  [736] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2864),
  [739] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1911),
  [742] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2852),
  [745] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(862),
  [748] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1052),
  [751] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2340),
  [754] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2829),
  [757] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(948),
  [760] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2331),
  [763] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2330),
  [766] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2329),
  [769] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1053),
  [772] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2821),
  [775] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1153),
  [778] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(480),
  [781] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2325),
  [784] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2816),
  [787] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(1057),
  [790] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(496),
  [793] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2324),
  [796] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(663),
  [799] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(2809),
  [802] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(435),
  [805] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(438),
  [808] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(808),
  [811] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(790),
  [814] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(431),
  [817] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2), SHIFT_REPEAT(796),
  [820] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_statement, 2), SHIFT(463),
  [823] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_statement, 2),
  [825] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_statement, 2), SHIFT(431),
  [828] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1064),
  [830] = {.entry = {.count = 1, .reusable = false}}, SHIFT(616),
  [832] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1418),
  [834] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1046),
  [836] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 3, .production_id = 79), SHIFT(463),
  [839] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 3, .production_id = 79),
  [841] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 3), SHIFT(463),
  [844] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 3),
  [846] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 2), SHIFT(463),
  [849] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 2),
  [851] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 6, .production_id = 79), SHIFT(463),
  [854] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 6, .production_id = 79),
  [856] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 4, .production_id = 79), SHIFT(463),
  [859] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 4, .production_id = 79),
  [861] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 4), SHIFT(463),
  [864] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 4),
  [866] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 5), SHIFT(463),
  [869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 5),
  [871] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elsewhere_clause, 5, .production_id = 79), SHIFT(463),
  [874] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsewhere_clause, 5, .production_id = 79),
  [876] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_rank_statement, 5), SHIFT(463),
  [879] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rank_statement, 5),
  [881] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_rank_statement, 5), SHIFT(431),
  [884] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_rank_statement, 6), SHIFT(463),
  [887] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rank_statement, 6),
  [889] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_rank_statement, 6), SHIFT(431),
  [892] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1071),
  [894] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1409),
  [896] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_case_statement, 6), SHIFT(463),
  [899] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 6),
  [901] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_case_statement, 6), SHIFT(431),
  [904] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1121),
  [906] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1342),
  [908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(429),
  [910] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1110),
  [912] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1385),
  [914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [916] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_case_statement, 3, .production_id = 83), SHIFT(463),
  [919] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 3, .production_id = 83),
  [921] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_case_statement, 3, .production_id = 83), SHIFT(431),
  [924] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_rank_statement, 5, .production_id = 83), SHIFT(463),
  [927] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rank_statement, 5, .production_id = 83),
  [929] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_rank_statement, 5, .production_id = 83), SHIFT(431),
  [932] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_rank_statement, 7), SHIFT(463),
  [935] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rank_statement, 7),
  [937] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_rank_statement, 7), SHIFT(431),
  [940] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_case_statement, 5, .production_id = 83), SHIFT(463),
  [943] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 5, .production_id = 83),
  [945] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_case_statement, 5, .production_id = 83), SHIFT(431),
  [948] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_case_statement, 5), SHIFT(463),
  [951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 5),
  [953] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_case_statement, 5), SHIFT(431),
  [956] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_rank_statement, 3, .production_id = 83), SHIFT(463),
  [959] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rank_statement, 3, .production_id = 83),
  [961] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_rank_statement, 3, .production_id = 83), SHIFT(431),
  [964] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_rank_statement, 4, .production_id = 83), SHIFT(463),
  [967] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rank_statement, 4, .production_id = 83),
  [969] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_rank_statement, 4, .production_id = 83), SHIFT(431),
  [972] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_case_statement, 4, .production_id = 83), SHIFT(463),
  [975] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 4, .production_id = 83),
  [977] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_case_statement, 4, .production_id = 83), SHIFT(431),
  [980] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_case_statement, 7), SHIFT(463),
  [983] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 7),
  [985] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_case_statement, 7), SHIFT(431),
  [988] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 4, .production_id = 104), SHIFT(463),
  [991] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 4, .production_id = 104), SHIFT(458),
  [994] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elseif_clause, 4, .production_id = 104),
  [996] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_elseif_clause, 4, .production_id = 104), SHIFT(431),
  [999] = {.entry = {.count = 1, .reusable = false}}, SHIFT(463),
  [1001] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1345),
  [1003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [1005] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 6, .production_id = 104), SHIFT(463),
  [1008] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 6, .production_id = 104), SHIFT(458),
  [1011] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elseif_clause, 6, .production_id = 104),
  [1013] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_elseif_clause, 6, .production_id = 104), SHIFT(431),
  [1016] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 6, .production_id = 109), SHIFT(463),
  [1019] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 6, .production_id = 109), SHIFT(458),
  [1022] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elseif_clause, 6, .production_id = 109),
  [1024] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_elseif_clause, 6, .production_id = 109), SHIFT(431),
  [1027] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 5, .production_id = 104), SHIFT(463),
  [1030] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 5, .production_id = 104), SHIFT(458),
  [1033] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elseif_clause, 5, .production_id = 104),
  [1035] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_elseif_clause, 5, .production_id = 104), SHIFT(431),
  [1038] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 5, .production_id = 109), SHIFT(463),
  [1041] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 5, .production_id = 109), SHIFT(458),
  [1044] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elseif_clause, 5, .production_id = 109),
  [1046] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_elseif_clause, 5, .production_id = 109), SHIFT(431),
  [1049] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 7, .production_id = 109), SHIFT(463),
  [1052] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_elseif_clause, 7, .production_id = 109), SHIFT(458),
  [1055] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elseif_clause, 7, .production_id = 109),
  [1057] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_elseif_clause, 7, .production_id = 109), SHIFT(431),
  [1060] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_else_clause, 2, .production_id = 78), SHIFT(463),
  [1063] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_else_clause, 2, .production_id = 78), SHIFT(458),
  [1066] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_else_clause, 2, .production_id = 78), SHIFT(431),
  [1069] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_else_clause, 3, .production_id = 78), SHIFT(463),
  [1072] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_else_clause, 3, .production_id = 78), SHIFT(458),
  [1075] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_else_clause, 3, .production_id = 78), SHIFT(431),
  [1078] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_else_clause, 4, .production_id = 78), SHIFT(463),
  [1081] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_else_clause, 4, .production_id = 78), SHIFT(458),
  [1084] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_else_clause, 4, .production_id = 78), SHIFT(431),
  [1087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1261),
  [1089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(169),
  [1091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [1093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 3),
  [1095] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 3),
  [1097] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_interface_statement, 2, .production_id = 31),
  [1099] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_interface_statement, 2, .production_id = 31),
  [1101] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_statement, 3, .production_id = 50),
  [1103] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_statement, 3, .production_id = 50),
  [1105] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_subroutine_statement, 2, .production_id = 53),
  [1107] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_subroutine_statement, 2, .production_id = 53),
  [1109] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_submodule_statement, 4, .production_id = 51),
  [1111] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_submodule_statement, 4, .production_id = 51),
  [1113] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_statement, 4, .production_id = 50),
  [1115] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_statement, 4, .production_id = 50),
  [1117] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_interface_statement, 3, .production_id = 31),
  [1119] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_interface_statement, 3, .production_id = 31),
  [1121] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 5),
  [1123] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 5),
  [1125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_subroutine_statement, 3, .production_id = 70),
  [1127] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_subroutine_statement, 3, .production_id = 70),
  [1129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_subroutine_statement, 3, .production_id = 53),
  [1131] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_subroutine_statement, 3, .production_id = 53),
  [1133] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine, 5),
  [1135] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subroutine, 5),
  [1137] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 2),
  [1139] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 2),
  [1141] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_function_statement, 3, .production_id = 71),
  [1143] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_function_statement, 3, .production_id = 71),
  [1145] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_interface_statement, 3, .production_id = 52),
  [1147] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_interface_statement, 3, .production_id = 52),
  [1149] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_submodule, 4),
  [1151] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_submodule, 4),
  [1153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 4),
  [1155] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 4),
  [1157] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_submodule_statement, 3, .production_id = 30),
  [1159] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_submodule_statement, 3, .production_id = 30),
  [1161] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_function_statement, 3, .production_id = 54),
  [1163] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_function_statement, 3, .production_id = 54),
  [1165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function, 5),
  [1167] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function, 5),
  [1169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 2),
  [1171] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 2),
  [1173] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_submodule_statement, 3, .production_id = 51),
  [1175] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_submodule_statement, 3, .production_id = 51),
  [1177] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_submodule, 2),
  [1179] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_submodule, 2),
  [1181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_program_statement, 3),
  [1183] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_program_statement, 3),
  [1185] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_function_statement, 2, .production_id = 54),
  [1187] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_function_statement, 2, .production_id = 54),
  [1189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 4),
  [1191] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 4),
  [1193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_program_statement, 4),
  [1195] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_program_statement, 4),
  [1197] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface, 2),
  [1199] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface, 2),
  [1201] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_statement, 3, .production_id = 29),
  [1203] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_statement, 3, .production_id = 29),
  [1205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function, 3),
  [1207] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function, 3),
  [1209] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function, 4),
  [1211] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function, 4),
  [1213] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine, 4),
  [1215] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subroutine, 4),
  [1217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 1),
  [1219] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 1),
  [1221] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_subroutine_statement, 4, .production_id = 70),
  [1223] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_subroutine_statement, 4, .production_id = 70),
  [1225] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine, 6),
  [1227] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subroutine, 6),
  [1229] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_program_statement, 2),
  [1231] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_program_statement, 2),
  [1233] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_function_statement, 4, .production_id = 71),
  [1235] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_function_statement, 4, .production_id = 71),
  [1237] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine, 3),
  [1239] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subroutine, 3),
  [1241] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_submodule, 3),
  [1243] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_submodule, 3),
  [1245] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface, 3),
  [1247] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface, 3),
  [1249] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_submodule_statement, 2, .production_id = 30),
  [1251] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_submodule_statement, 2, .production_id = 30),
  [1253] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 3),
  [1255] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 3),
  [1257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_interface_statement, 4, .production_id = 52),
  [1259] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_interface_statement, 4, .production_id = 52),
  [1261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function, 6),
  [1263] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function, 6),
  [1265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_statement, 2, .production_id = 29),
  [1267] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_statement, 2, .production_id = 29),
  [1269] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2),
  [1271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 1),
  [1273] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 1),
  [1275] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum, 3),
  [1277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum, 3),
  [1279] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_identifier, 1, .production_id = 6),
  [1281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_identifier, 1, .production_id = 6),
  [1283] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_identifier, 1),
  [1285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_identifier, 1),
  [1287] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 2),
  [1289] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 2),
  [1291] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_identifier, 1, .production_id = 2),
  [1293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_identifier, 1, .production_id = 2),
  [1295] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_type_statement, 2, .production_id = 34),
  [1297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_type_statement, 2, .production_id = 34),
  [1299] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 3),
  [1301] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 3),
  [1303] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__specification_part, 3),
  [1305] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__specification_part, 3),
  [1307] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_program_repeat1, 1),
  [1309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 1),
  [1311] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__specification_part, 1),
  [1313] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__specification_part, 1),
  [1315] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum, 4),
  [1317] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum, 4),
  [1319] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 8),
  [1321] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 8),
  [1323] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__specification_part, 2),
  [1325] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__specification_part, 2),
  [1327] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 7),
  [1329] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 7),
  [1331] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 6),
  [1333] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 6),
  [1335] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_type_statement, 3, .production_id = 56),
  [1337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_type_statement, 3, .production_id = 56),
  [1339] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_type_statement, 3, .production_id = 34),
  [1341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_type_statement, 3, .production_id = 34),
  [1343] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 4),
  [1345] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 4),
  [1347] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__name, 1, .production_id = 9),
  [1349] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__name, 1, .production_id = 9),
  [1351] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_identifier, 1, .production_id = 4),
  [1353] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_identifier, 1, .production_id = 4),
  [1355] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_type_statement, 4, .production_id = 56),
  [1357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_type_statement, 4, .production_id = 56),
  [1359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_definition, 5),
  [1361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_definition, 5),
  [1363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program_statement, 2, .production_id = 8),
  [1365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program_statement, 2, .production_id = 8),
  [1367] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1303),
  [1369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1420),
  [1371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [1373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(819),
  [1375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2491),
  [1377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [1379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2908),
  [1381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1340),
  [1383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1743),
  [1385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1165),
  [1387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [1389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [1391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [1393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1936),
  [1395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1232),
  [1397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1166),
  [1399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(452),
  [1401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [1403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [1405] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2253),
  [1407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [1409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2916),
  [1411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [1413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2891),
  [1415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [1417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2347),
  [1419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2852),
  [1421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [1423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2811),
  [1425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1912),
  [1427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1331),
  [1429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [1431] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 1),
  [1433] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_qualifier, 1), REDUCE(sym_identifier, 1),
  [1436] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 1),
  [1438] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_type_qualifier, 1), REDUCE(sym_identifier, 1),
  [1441] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(587),
  [1444] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(819),
  [1447] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2491),
  [1450] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(456),
  [1453] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1340),
  [1456] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1743),
  [1459] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1165),
  [1462] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(440),
  [1465] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(444),
  [1468] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(445),
  [1471] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1936),
  [1474] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1232),
  [1477] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1166),
  [1480] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(452),
  [1483] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(446),
  [1486] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(788),
  [1489] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(775),
  [1492] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2916),
  [1495] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(443),
  [1498] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2891),
  [1501] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(454),
  [1504] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2347),
  [1507] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2852),
  [1510] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(563),
  [1513] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(2811),
  [1516] = {.entry = {.count = 1, .reusable = false}}, SHIFT(917),
  [1518] = {.entry = {.count = 1, .reusable = false}}, SHIFT(908),
  [1520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(506),
  [1522] = {.entry = {.count = 1, .reusable = false}}, SHIFT(914),
  [1524] = {.entry = {.count = 1, .reusable = false}}, SHIFT(420),
  [1526] = {.entry = {.count = 1, .reusable = false}}, SHIFT(885),
  [1528] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1042),
  [1530] = {.entry = {.count = 1, .reusable = false}}, SHIFT(491),
  [1532] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2320),
  [1534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1164),
  [1536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1180),
  [1538] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_identifier, 1), SHIFT(658),
  [1541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2610),
  [1543] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_identifier, 1), SHIFT(689),
  [1546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [1548] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_identifier, 1), SHIFT(2809),
  [1551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [1553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [1555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1155),
  [1557] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1141),
  [1559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1154),
  [1561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stop_statement, 1),
  [1563] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_statement_label, 1, .production_id = 1),
  [1565] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_number_literal, 1),
  [1567] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_number_literal, 1),
  [1569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_statement_label, 1, .production_id = 1),
  [1571] = {.entry = {.count = 1, .reusable = false}}, SHIFT(464),
  [1573] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_intrinsic_type, 1),
  [1575] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_intrinsic_type, 1), REDUCE(sym_identifier, 1),
  [1578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_intrinsic_type, 1),
  [1580] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_intrinsic_type, 1), REDUCE(sym_identifier, 1),
  [1583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_variable_attributes, 4),
  [1585] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_attributes, 4),
  [1587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_attributes, 4),
  [1589] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__callable_interface_qualifers, 1),
  [1591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1216),
  [1593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [1595] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2500),
  [1597] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1045),
  [1599] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1554),
  [1601] = {.entry = {.count = 1, .reusable = false}}, SHIFT(911),
  [1603] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1555),
  [1605] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2895),
  [1607] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 2),
  [1609] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 2), SHIFT_REPEAT(2491),
  [1612] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 2), SHIFT_REPEAT(456),
  [1615] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_variable_modification_repeat1, 2),
  [1617] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 2), SHIFT_REPEAT(452),
  [1620] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 2), SHIFT_REPEAT(443),
  [1623] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 2), SHIFT_REPEAT(2891),
  [1626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1503),
  [1628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1557),
  [1630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_save_statement, 1),
  [1632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [1634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [1636] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2042),
  [1638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(660),
  [1640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2144),
  [1642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [1644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2153),
  [1646] = {.entry = {.count = 1, .reusable = false}}, SHIFT(473),
  [1648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1125),
  [1650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_private_statement, 1),
  [1652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [1654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_public_statement, 1),
  [1656] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abstract_specifier, 1),
  [1658] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3, .dynamic_precedence = 1),
  [1660] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3, .dynamic_precedence = 1),
  [1662] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 4),
  [1664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 4),
  [1666] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 1, .production_id = 3),
  [1668] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 1, .production_id = 3),
  [1670] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 4, .production_id = 44),
  [1672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 4, .production_id = 44),
  [1674] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1483),
  [1676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 2),
  [1678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 2),
  [1680] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_variable_modification_repeat1, 1),
  [1682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_variable_modification_repeat1, 1),
  [1684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 5, .production_id = 63),
  [1686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 5, .production_id = 63),
  [1688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 4, .dynamic_precedence = 1),
  [1690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 4, .dynamic_precedence = 1),
  [1692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_program_repeat2, 3),
  [1694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 3),
  [1696] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_program_repeat2, 2),
  [1698] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 2, .dynamic_precedence = 1),
  [1700] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 2, .dynamic_precedence = 1),
  [1702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [1704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2229),
  [1706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [1708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2135),
  [1710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1133),
  [1712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2140),
  [1714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1019),
  [1716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1971),
  [1718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [1720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2143),
  [1722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2346),
  [1724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2345),
  [1726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1200),
  [1728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1173),
  [1730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [1732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [1734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [1736] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extent_specifier, 2),
  [1738] = {.entry = {.count = 1, .reusable = false}}, SHIFT(649),
  [1740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2320),
  [1742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(698),
  [1744] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extent_specifier, 1),
  [1746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2294),
  [1748] = {.entry = {.count = 1, .reusable = false}}, SHIFT(498),
  [1750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1202),
  [1752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__intrinsic_type, 1),
  [1754] = {.entry = {.count = 1, .reusable = false}}, SHIFT(467),
  [1756] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__intrinsic_type, 1),
  [1758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1899),
  [1760] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__forall_control_expression, 7),
  [1762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__forall_control_expression, 7),
  [1764] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__forall_control_expression, 4),
  [1766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__forall_control_expression, 4),
  [1768] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3),
  [1770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3),
  [1772] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__forall_control_expression, 6),
  [1774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__forall_control_expression, 6),
  [1776] = {.entry = {.count = 1, .reusable = false}}, SHIFT(658),
  [1778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [1780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(689),
  [1782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_write_statement, 2),
  [1784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__forall_control_expression, 5),
  [1786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__forall_control_expression, 5),
  [1788] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1439),
  [1790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1185),
  [1792] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_write_statement, 3),
  [1794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_inquire_statement, 2),
  [1796] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1423),
  [1798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [1800] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_open_statement, 2),
  [1802] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1445),
  [1804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1593),
  [1806] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ac_value_list, 1, .production_id = 18),
  [1808] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1444),
  [1810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1584),
  [1812] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1344),
  [1814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [1816] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1429),
  [1818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1667),
  [1820] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1380),
  [1822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1609),
  [1824] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1375),
  [1826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1606),
  [1828] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1425),
  [1830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1674),
  [1832] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1356),
  [1834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [1836] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1430),
  [1838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1565),
  [1840] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1398),
  [1842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1625),
  [1844] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1382),
  [1846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [1848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2646),
  [1850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stop_statement, 2),
  [1852] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1449),
  [1854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1183),
  [1856] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1341),
  [1858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1578),
  [1860] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1335),
  [1862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1649),
  [1864] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1339),
  [1866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1651),
  [1868] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1377),
  [1870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1671),
  [1872] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1432),
  [1874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1682),
  [1876] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1427),
  [1878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1684),
  [1880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameterized_read_statement, 2),
  [1882] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1334),
  [1884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [1886] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1390),
  [1888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1711),
  [1890] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1389),
  [1892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1709),
  [1894] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_submodule_statement, 5, .production_id = 58),
  [1896] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_submodule_statement, 5, .production_id = 58),
  [1898] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_intrinsic_type, 2),
  [1900] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_intrinsic_type, 2),
  [1902] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_intrinsic_type, 2, .production_id = 15),
  [1904] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_intrinsic_type, 2, .production_id = 15),
  [1906] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_submodule_statement, 7, .production_id = 90),
  [1908] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_submodule_statement, 7, .production_id = 90),
  [1910] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_statement, 2),
  [1912] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_statement, 2),
  [1914] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_interface_statement, 1), SHIFT(463),
  [1917] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_statement, 1),
  [1919] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2744),
  [1921] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2742),
  [1923] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1033),
  [1925] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_interface_statement, 1), SHIFT(458),
  [1928] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_interface_statement, 1), SHIFT(447),
  [1931] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_interface_statement, 2), SHIFT(463),
  [1934] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_statement, 2),
  [1936] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_interface_statement, 2), SHIFT(458),
  [1939] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_interface_statement, 2), SHIFT(447),
  [1942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(699),
  [1944] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_intrinsic_type, 1, .production_id = 5),
  [1946] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_intrinsic_type, 1, .production_id = 5),
  [1948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(748),
  [1950] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_if_statement, 1, .production_id = 64),
  [1952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [1954] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3), REDUCE(sym_argument_list, 3, .dynamic_precedence = 1),
  [1957] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3), REDUCE(sym_argument_list, 3, .dynamic_precedence = 1),
  [1960] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1029),
  [1962] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1034),
  [1964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [1966] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1018),
  [1968] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1036),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(691),
  [1972] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type, 4),
  [1974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type, 4),
  [1976] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type, 4, .dynamic_precedence = 1),
  [1978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type, 4, .dynamic_precedence = 1),
  [1980] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__intrinsic_type, 2),
  [1982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__intrinsic_type, 2),
  [1984] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type, 4, .production_id = 43),
  [1986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type, 4, .production_id = 43),
  [1988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type, 4, .dynamic_precedence = 1, .production_id = 43),
  [1990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type, 4, .dynamic_precedence = 1, .production_id = 43),
  [1992] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_size, 1),
  [1994] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_size, 1),
  [1996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1296),
  [1998] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_size, 2),
  [2000] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_size, 2),
  [2002] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_size, 2, .production_id = 35),
  [2004] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_size, 2, .production_id = 35),
  [2006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1190),
  [2008] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 1),
  [2010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [2012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [2014] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_statement, 1),
  [2016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2219),
  [2018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1525),
  [2020] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2846),
  [2022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1720),
  [2024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(925),
  [2026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1454),
  [2028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1222),
  [2030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [2032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2453),
  [2034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2500),
  [2036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [2038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1554),
  [2040] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2302),
  [2042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1555),
  [2044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2895),
  [2046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1722),
  [2048] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2888),
  [2050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(929),
  [2052] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2880),
  [2054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(934),
  [2056] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2869),
  [2058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [2060] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2789),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(996),
  [2064] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2831),
  [2066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(999),
  [2068] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2465),
  [2070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(991),
  [2072] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2858),
  [2074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [2076] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_interface_repeat1, 2),
  [2078] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1720),
  [2081] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2),
  [2083] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1454),
  [2086] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1222),
  [2089] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1462),
  [2092] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(2453),
  [2095] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(2500),
  [2098] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(754),
  [2101] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1554),
  [2104] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(2302),
  [2107] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1555),
  [2110] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(2895),
  [2113] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 2), SHIFT_REPEAT(1722),
  [2116] = {.entry = {.count = 1, .reusable = false}}, SHIFT(643),
  [2118] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_qualifier, 1),
  [2120] = {.entry = {.count = 1, .reusable = false}}, SHIFT(590),
  [2122] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_procedure_qualifier, 1), SHIFT(590),
  [2125] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_procedure_qualifier, 1), SHIFT(638),
  [2128] = {.entry = {.count = 1, .reusable = false}}, SHIFT(636),
  [2130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(468),
  [2132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1921),
  [2134] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 6, .production_id = 42),
  [2136] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 6, .production_id = 42), SHIFT(1795),
  [2139] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 6, .production_id = 42),
  [2141] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 5),
  [2143] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 5), SHIFT(1795),
  [2146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 5),
  [2148] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 3),
  [2150] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 3), SHIFT(1795),
  [2153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 3),
  [2155] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_variable_declaration_repeat1, 2),
  [2157] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_variable_declaration_repeat1, 2), SHIFT_REPEAT(1173),
  [2160] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_variable_declaration_repeat1, 2),
  [2162] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_procedure_statement_repeat1, 2),
  [2164] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat1, 2), SHIFT_REPEAT(1795),
  [2167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat1, 2),
  [2169] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 6),
  [2171] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 6), SHIFT(1795),
  [2174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 6),
  [2176] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 4),
  [2178] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 4), SHIFT(1795),
  [2181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 4),
  [2183] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_call_expression_repeat1, 2),
  [2185] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_call_expression_repeat1, 2), SHIFT_REPEAT(472),
  [2188] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_call_expression_repeat1, 2),
  [2190] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2),
  [2192] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2),
  [2194] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 7, .production_id = 42),
  [2196] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 7, .production_id = 42), SHIFT(1795),
  [2199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 7, .production_id = 42),
  [2201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2494),
  [2203] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__io_arguments, 5),
  [2205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_arguments, 5),
  [2207] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_call_expression_repeat1, 1),
  [2209] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_call_expression_repeat1, 1),
  [2211] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__io_arguments, 7),
  [2213] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_arguments, 7),
  [2215] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__io_arguments, 8),
  [2217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_arguments, 8),
  [2219] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 3), SHIFT(1801),
  [2222] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__io_arguments, 3),
  [2224] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_arguments, 3),
  [2226] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_complex_literal, 5),
  [2228] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_complex_literal, 5),
  [2230] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_literal, 1, .production_id = 7),
  [2232] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_literal, 1, .production_id = 7),
  [2234] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_implied_do_loop_expression, 6),
  [2236] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implied_do_loop_expression, 6),
  [2238] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_implied_do_loop_expression, 5),
  [2240] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implied_do_loop_expression, 5),
  [2242] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1),
  [2244] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1),
  [2246] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_null_literal, 3),
  [2248] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_null_literal, 3),
  [2250] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__io_arguments, 4),
  [2252] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_arguments, 4),
  [2254] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__array_constructor_f2003, 3, .production_id = 28),
  [2256] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_constructor_f2003, 3, .production_id = 28),
  [2258] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__array_constructor_legacy, 3, .production_id = 28),
  [2260] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_constructor_legacy, 3, .production_id = 28),
  [2262] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__io_arguments, 6),
  [2264] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_arguments, 6),
  [2266] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 1),
  [2268] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_procedure_declaration, 1), SHIFT(1472),
  [2271] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 1), SHIFT(1808),
  [2274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 1),
  [2276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_member_expression, 3, .production_id = 40),
  [2278] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_member_expression, 3, .production_id = 40),
  [2280] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_spec, 2),
  [2282] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_spec, 2),
  [2284] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_attribute, 1),
  [2286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_attribute, 1),
  [2288] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_declaration, 4, .production_id = 42),
  [2290] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_procedure_declaration, 4, .production_id = 42), SHIFT(1806),
  [2293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_declaration, 4, .production_id = 42),
  [2295] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_attribute, 4),
  [2297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_attribute, 4),
  [2299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [2301] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_math_expression, 3, .production_id = 39),
  [2303] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_math_expression, 3, .production_id = 39),
  [2305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1526),
  [2307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(664),
  [2309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_relational_expression, 3, .production_id = 39),
  [2311] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_relational_expression, 3, .production_id = 39),
  [2313] = {.entry = {.count = 1, .reusable = false}}, SHIFT(670),
  [2315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [2317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [2319] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concatenation_expression, 3, .production_id = 39),
  [2321] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concatenation_expression, 3, .production_id = 39),
  [2323] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_expression, 2, .production_id = 14),
  [2325] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_expression, 2, .production_id = 14),
  [2327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(683),
  [2329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [2331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(669),
  [2333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(668),
  [2335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(668),
  [2337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_logical_expression, 2),
  [2339] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_logical_expression, 2),
  [2341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_logical_expression, 3, .production_id = 39),
  [2343] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_logical_expression, 3, .production_id = 39),
  [2345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1790),
  [2347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [2349] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1381),
  [2351] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1742),
  [2353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(990),
  [2355] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1336),
  [2357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1383),
  [2359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1333),
  [2361] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_internal_procedures, 3),
  [2363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1721),
  [2365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_internal_procedures, 3),
  [2367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(955),
  [2369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1874),
  [2371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2587),
  [2373] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_user_defined_operator, 3),
  [2375] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_user_defined_operator, 3),
  [2377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(923),
  [2379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1551),
  [2381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1805),
  [2383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(998),
  [2385] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_internal_procedures_repeat1, 2),
  [2387] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(1721),
  [2390] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(1454),
  [2393] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2),
  [2395] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(1462),
  [2398] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(2453),
  [2401] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(2500),
  [2404] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(1554),
  [2407] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(2302),
  [2410] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(1555),
  [2413] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(2895),
  [2416] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 2), SHIFT_REPEAT(1722),
  [2419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_internal_procedures, 2),
  [2421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_internal_procedures, 2),
  [2423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1013),
  [2425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2259),
  [2427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2571),
  [2429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [2431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [2433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1534),
  [2435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1799),
  [2437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(958),
  [2439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [2441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [2443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(984),
  [2445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(930),
  [2447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1800),
  [2449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(986),
  [2451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1793),
  [2453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [2455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [2457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2809),
  [2459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(469),
  [2461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [2463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [2465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [2467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(661),
  [2469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(657),
  [2471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(656),
  [2473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(655),
  [2475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(721),
  [2477] = {.entry = {.count = 1, .reusable = false}}, SHIFT(722),
  [2479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [2481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [2483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [2485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(653),
  [2487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [2489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(681),
  [2491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [2493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1544),
  [2495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [2497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [2499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [2501] = {.entry = {.count = 1, .reusable = false}}, SHIFT(676),
  [2503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(676),
  [2505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [2507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(674),
  [2509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [2511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [2513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ac_value_list, 1),
  [2515] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_call_expression_repeat1, 2), SHIFT_REPEAT(470),
  [2518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1011),
  [2520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [2522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_cuda_kernel_argument_list_repeat1, 2),
  [2524] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_value_range_list, 1),
  [2526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [2528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1563),
  [2530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1536),
  [2532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1535),
  [2534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [2536] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ac_value_list, 2, .production_id = 18),
  [2538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1128),
  [2540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(470),
  [2542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1543),
  [2544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [2546] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2900),
  [2548] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2901),
  [2550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1913),
  [2552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(407),
  [2554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1329),
  [2556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1750),
  [2558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2903),
  [2560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1530),
  [2562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1507),
  [2564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2847),
  [2566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1370),
  [2568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [2570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_included_items, 3),
  [2572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [2574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_triplet_spec, 5),
  [2576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [2578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [2580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [2582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(309),
  [2584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [2586] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1306),
  [2588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2881),
  [2590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1384),
  [2592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [2594] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1256),
  [2596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2870),
  [2598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1421),
  [2600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [2602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(569),
  [2604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [2606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [2608] = {.entry = {.count = 1, .reusable = false}}, SHIFT(715),
  [2610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [2612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(710),
  [2614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [2616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [2618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [2620] = {.entry = {.count = 1, .reusable = false}}, SHIFT(727),
  [2622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(727),
  [2624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(729),
  [2626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_call, 2, .production_id = 16),
  [2628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(625),
  [2630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2248),
  [2632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2244),
  [2634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2243),
  [2636] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2),
  [2638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(672),
  [2640] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extent_specifier, 3),
  [2642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [2644] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1299),
  [2646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2790),
  [2648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1413),
  [2650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [2652] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1241),
  [2654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2418),
  [2656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1404),
  [2658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_value_range_list_repeat1, 2),
  [2660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(358),
  [2662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [2664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_input_item_list, 1),
  [2666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_output_item_list, 1),
  [2668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(390),
  [2670] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1248),
  [2672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2859),
  [2674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1399),
  [2676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [2678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_control, 5, .production_id = 103),
  [2680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2326),
  [2682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [2684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [2686] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1258),
  [2688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2832),
  [2690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1417),
  [2692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [2694] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1923),
  [2696] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_targets, 1),
  [2698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [2700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [2702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [2704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2354),
  [2706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stop_statement, 3),
  [2708] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extent_specifier, 4),
  [2710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_control_expression, 5),
  [2712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(714),
  [2714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [2716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(786),
  [2718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [2720] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_parameters_repeat1, 2),
  [2722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2),
  [2724] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2), SHIFT_REPEAT(1527),
  [2727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [2729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(578),
  [2731] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__declaration_targets_repeat1, 2),
  [2733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2566),
  [2735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [2737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(190),
  [2739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_assignment, 3),
  [2741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_association_statement, 3),
  [2743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_statement, 3, .production_id = 38),
  [2745] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_call_expression_repeat1, 2), SHIFT_REPEAT(469),
  [2748] = {.entry = {.count = 1, .reusable = false}}, SHIFT(742),
  [2750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [2752] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_extent_specifier, 5),
  [2754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_control, 7, .production_id = 113),
  [2756] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_association, 3, .production_id = 69),
  [2758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_triplet_spec, 7),
  [2760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2258),
  [2762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_argument, 3, .production_id = 59),
  [2764] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 7, .production_id = 42),
  [2766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 7, .production_id = 42),
  [2768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(939),
  [2770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stop_statement, 7),
  [2772] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 6),
  [2774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2862),
  [2776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [2778] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 3),
  [2780] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 3),
  [2782] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_identifier, 1), SHIFT(665),
  [2785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1881),
  [2787] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 12, .production_id = 42),
  [2789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 12, .production_id = 42),
  [2791] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 11, .production_id = 42),
  [2793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 11, .production_id = 42),
  [2795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [2797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [2799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(53),
  [2801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [2803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 10, .production_id = 42),
  [2805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 10, .production_id = 42),
  [2807] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__import_names, 1),
  [2809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_names, 1),
  [2811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1527),
  [2813] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 9),
  [2815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 9),
  [2817] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 4),
  [2819] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 4),
  [2821] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 9, .production_id = 42),
  [2823] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 9, .production_id = 42),
  [2825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1443),
  [2827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2850),
  [2829] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 5),
  [2831] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_procedure_statement_repeat2, 2),
  [2833] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat2, 2),
  [2835] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat2, 2), SHIFT_REPEAT(939),
  [2838] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__import_names, 2),
  [2840] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_names, 2),
  [2842] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 8),
  [2844] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 8),
  [2846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1369),
  [2848] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 8, .production_id = 42),
  [2850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 8, .production_id = 42),
  [2852] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stop_statement, 5),
  [2854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2838),
  [2856] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 5, .production_id = 42),
  [2858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 5, .production_id = 42),
  [2860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2873),
  [2862] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 5),
  [2864] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 5),
  [2866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1431),
  [2868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1891),
  [2870] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__import_names, 3),
  [2872] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_names, 3),
  [2874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [2876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(728),
  [2878] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 7),
  [2880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 7),
  [2882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [2884] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__import_names, 4),
  [2886] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_names, 4),
  [2888] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2917),
  [2890] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2918),
  [2892] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1408),
  [2894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [2896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2890),
  [2898] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__import_names, 5),
  [2900] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_names, 5),
  [2902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_loop_control_expression, 7),
  [2904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2801),
  [2906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1898),
  [2908] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 8),
  [2910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2883),
  [2912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1411),
  [2914] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 6),
  [2916] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 6),
  [2918] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 6, .production_id = 42),
  [2920] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 6, .production_id = 42),
  [2922] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method_name, 1, .production_id = 32),
  [2924] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method_name, 1, .production_id = 32),
  [2926] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binding_name, 1),
  [2928] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_statement, 2),
  [2930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_statement, 2),
  [2932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1378),
  [2934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [2936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1437),
  [2938] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_stop_statement, 6),
  [2940] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 7),
  [2942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1873),
  [2944] = {.entry = {.count = 1, .reusable = false}}, SHIFT(492),
  [2946] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__io_expressions, 1),
  [2948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2263),
  [2950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(682),
  [2952] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_procedure_statement_repeat2, 4),
  [2954] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat2, 4),
  [2956] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_defined_io_procedure, 4),
  [2958] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_defined_io_procedure, 4),
  [2960] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_operator, 4),
  [2962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_operator, 4),
  [2964] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment, 4),
  [2966] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment, 4),
  [2968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2525),
  [2970] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__generic_procedure, 1),
  [2972] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__generic_procedure, 1),
  [2974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1756),
  [2976] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statements, 1),
  [2978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1826),
  [2980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(703),
  [2982] = {.entry = {.count = 1, .reusable = false}}, SHIFT(739),
  [2984] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_interface_repeat1, 1),
  [2986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_interface_repeat1, 1),
  [2988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [2990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1754),
  [2992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1514),
  [2994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2863),
  [2996] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__expression, 1), SHIFT(899),
  [2999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [3001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2139),
  [3003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1559),
  [3005] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_statement, 1),
  [3007] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_statement, 2),
  [3009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_statement, 2),
  [3011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unit_identifier, 1),
  [3013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1540),
  [3015] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__import_names, 2, .production_id = 25),
  [3017] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_names, 2, .production_id = 25),
  [3019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [3021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [3023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [3025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2896),
  [3027] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__expression, 1), SHIFT(886),
  [3030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(750),
  [3032] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__expression, 1), SHIFT(2281),
  [3035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface_statement, 2),
  [3037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [3039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [3041] = {.entry = {.count = 1, .reusable = false}}, SHIFT(735),
  [3043] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat1, 2), SHIFT_REPEAT(1790),
  [3046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2884),
  [3048] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__expression, 1), SHIFT(2312),
  [3051] = {.entry = {.count = 1, .reusable = false}}, SHIFT(751),
  [3053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2851),
  [3055] = {.entry = {.count = 1, .reusable = false}}, SHIFT(659),
  [3057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2675),
  [3059] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interface_statement, 3),
  [3061] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interface_statement, 3),
  [3063] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__procedure_kind, 2),
  [3065] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__procedure_kind, 2),
  [3067] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_common_statement_repeat1, 2), SHIFT_REPEAT(1348),
  [3070] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_common_statement_repeat1, 2), SHIFT_REPEAT(1374),
  [3073] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_common_statement_repeat1, 2), SHIFT_REPEAT(1451),
  [3076] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_common_statement_repeat1, 2), SHIFT_REPEAT(1347),
  [3079] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_common_statement_repeat1, 2), SHIFT_REPEAT(1346),
  [3082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_common_statement_repeat1, 2),
  [3084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_call, 3, .production_id = 16),
  [3086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2874),
  [3088] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_equivalence_set_repeat1, 2),
  [3090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [3092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2839),
  [3094] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1348),
  [3096] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1374),
  [3098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [3100] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1347),
  [3102] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1346),
  [3104] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_common_statement, 2),
  [3106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2815),
  [3108] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__procedure_kind, 1),
  [3110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__procedure_kind, 1),
  [3112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(511),
  [3114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_data_value_repeat1, 2),
  [3116] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__expression, 1), REDUCE(aux_sym_data_value_repeat1, 2),
  [3119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1371),
  [3121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [3123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(702),
  [3125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1357),
  [3127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [3129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1397),
  [3131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(363),
  [3133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1387),
  [3135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(633),
  [3137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1774),
  [3139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1522),
  [3141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2640),
  [3143] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_variable_group, 5),
  [3145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1546),
  [3147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_group, 5),
  [3149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2613),
  [3151] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_variable_group, 4),
  [3153] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_group, 4),
  [3155] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_data_set_repeat1, 2),
  [3157] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__expression, 1), REDUCE(aux_sym_data_set_repeat1, 2),
  [3160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_common_statement_repeat1, 1),
  [3162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_common_statement_repeat1, 1),
  [3164] = {.entry = {.count = 1, .reusable = false}}, SHIFT(744),
  [3166] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_data_value_repeat1, 4, .production_id = 92),
  [3168] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__expression, 1), REDUCE(aux_sym_data_value_repeat1, 4, .production_id = 92),
  [3171] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2), SHIFT_REPEAT(1546),
  [3174] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1442),
  [3176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(588),
  [3178] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1353),
  [3180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [3182] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1438),
  [3184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [3186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [3188] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_common_statement_repeat1, 2),
  [3190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(712),
  [3192] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1394),
  [3194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(397),
  [3196] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1388),
  [3198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(290),
  [3200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [3202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1653),
  [3204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1900),
  [3206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1658),
  [3208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2308),
  [3210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [3212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1575),
  [3214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_do_loop_statement, 1, .production_id = 26),
  [3216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(195),
  [3218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [3220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [3222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_block_construct_statement, 1, .production_id = 27),
  [3224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [3226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [3228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(177),
  [3230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [3232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(327),
  [3234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1482),
  [3236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [3238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [3240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [3242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(152),
  [3244] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(1722),
  [3247] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2),
  [3249] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(2453),
  [3252] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(2500),
  [3255] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(1554),
  [3258] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(2302),
  [3261] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(1555),
  [3264] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 2), SHIFT_REPEAT(2895),
  [3267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1716),
  [3269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1488),
  [3271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1490),
  [3273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_associate_statement, 2, .production_id = 95),
  [3275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(337),
  [3277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [3279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1588),
  [3281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1675),
  [3283] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_block_construct_statement, 2, .production_id = 48),
  [3285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_select_statement, 2, .production_id = 84),
  [3287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1592),
  [3289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1641),
  [3291] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_forall_statement, 1, .production_id = 36),
  [3293] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_public_statement, 3),
  [3295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_public_statement, 3),
  [3297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1096),
  [3299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [3301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [3303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1701),
  [3305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1706),
  [3307] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2722),
  [3309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_select_statement, 1, .production_id = 66),
  [3311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [3313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
  [3315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [3317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1630),
  [3319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2338),
  [3321] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_associate_statement, 1, .production_id = 85),
  [3323] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_forall_statement, 2, .production_id = 57),
  [3325] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_where_statement, 1, .production_id = 47),
  [3327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(170),
  [3329] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_public_statement, 4),
  [3331] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_public_statement, 4),
  [3333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2207),
  [3335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [3337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1613),
  [3339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1664),
  [3341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [3343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1690),
  [3345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [3347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1566),
  [3349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1627),
  [3351] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_where_statement, 2, .production_id = 65),
  [3353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1686),
  [3355] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_private_statement_repeat1, 2),
  [3357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_private_statement_repeat1, 2),
  [3359] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_private_statement_repeat1, 2), SHIFT_REPEAT(1096),
  [3362] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_do_loop_statement, 2, .production_id = 45),
  [3364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [3366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1194),
  [3368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [3370] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_if_statement, 2, .production_id = 77),
  [3372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1597),
  [3374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1600),
  [3376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [3378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
  [3380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [3382] = {.entry = {.count = 1, .reusable = false}}, SHIFT(301),
  [3384] = {.entry = {.count = 1, .reusable = false}}, SHIFT(298),
  [3386] = {.entry = {.count = 1, .reusable = false}}, SHIFT(332),
  [3388] = {.entry = {.count = 1, .reusable = false}}, SHIFT(297),
  [3390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [3392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [3394] = {.entry = {.count = 1, .reusable = false}}, SHIFT(630),
  [3396] = {.entry = {.count = 1, .reusable = false}}, SHIFT(564),
  [3398] = {.entry = {.count = 1, .reusable = false}}, SHIFT(629),
  [3400] = {.entry = {.count = 1, .reusable = false}}, SHIFT(627),
  [3402] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_private_statement, 4),
  [3404] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_private_statement, 4),
  [3406] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_statement_label_reference, 1, .production_id = 10),
  [3408] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_statement_label_reference, 1, .production_id = 10),
  [3410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1550),
  [3412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [3414] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_public_statement, 1),
  [3416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1109),
  [3418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2013),
  [3420] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_private_statement, 3),
  [3422] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_private_statement, 3),
  [3424] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__class_default, 2),
  [3426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__class_default, 2),
  [3428] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_private_statement, 1),
  [3430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1120),
  [3432] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2),
  [3434] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(819),
  [3437] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2),
  [3439] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(2903),
  [3442] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(788),
  [3445] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(2253),
  [3448] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(775),
  [3451] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(2916),
  [3454] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_definition_repeat1, 2), SHIFT_REPEAT(2852),
  [3457] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__class_default, 1),
  [3459] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__class_default, 1),
  [3461] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_procedure_statement, 3, .production_id = 96),
  [3463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_procedure_statement, 3, .production_id = 96),
  [3465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_internal_procedures_repeat1, 1),
  [3467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_internal_procedures_repeat1, 1),
  [3469] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_procedure, 6),
  [3471] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_procedure, 6),
  [3473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_procedure_statement, 4, .production_id = 96),
  [3475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_procedure_statement, 4, .production_id = 96),
  [3477] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_procedure_statement, 3, .production_id = 86),
  [3479] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_procedure_statement, 3, .production_id = 86),
  [3481] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_procedure, 5),
  [3483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_procedure, 5),
  [3485] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_procedure, 4),
  [3487] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_procedure, 4),
  [3489] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_end_module_procedure_statement, 2, .production_id = 86),
  [3491] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_module_procedure_statement, 2, .production_id = 86),
  [3493] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_procedure, 3),
  [3495] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_procedure, 3),
  [3497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_qualifier, 1),
  [3499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [3501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1208),
  [3503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1518),
  [3505] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 8, .production_id = 107),
  [3507] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 8, .production_id = 107),
  [3509] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 3),
  [3511] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 3),
  [3513] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 5),
  [3515] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 5),
  [3517] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 6, .production_id = 73),
  [3519] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 6, .production_id = 73),
  [3521] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 4),
  [3523] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 4),
  [3525] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_attributes, 5),
  [3527] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_procedure_attributes, 5),
  [3529] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__callable_interface_qualifers, 1),
  [3531] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_procedure_attributes, 4),
  [3533] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 7, .production_id = 91),
  [3535] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 7, .production_id = 91),
  [3537] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_statement, 7, .production_id = 97),
  [3539] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_statement, 7, .production_id = 97),
  [3541] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_procedures, 1),
  [3543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2889),
  [3545] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_procedures, 1),
  [3547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2740),
  [3549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2637),
  [3551] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2274),
  [3553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1392),
  [3555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2691),
  [3557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2639),
  [3559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1547),
  [3561] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_procedures, 2),
  [3563] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_procedures, 2),
  [3565] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_procedures, 3),
  [3567] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_procedures, 3),
  [3569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2304),
  [3571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(932),
  [3573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2333),
  [3575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2314),
  [3577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2897),
  [3579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2062),
  [3581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2306),
  [3583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2303),
  [3585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2452),
  [3587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2455),
  [3589] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_select_type_statement_repeat1, 2),
  [3591] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_select_type_statement_repeat1, 2), SHIFT_REPEAT(2637),
  [3594] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_select_type_statement_repeat1, 2), SHIFT_REPEAT(2274),
  [3597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_type_statement_repeat1, 2),
  [3599] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_type_statement_repeat1, 2), SHIFT_REPEAT(2691),
  [3602] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_type_statement_repeat1, 2), SHIFT_REPEAT(2639),
  [3605] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_type_statement_repeat1, 2), SHIFT_REPEAT(1547),
  [3608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_derived_type_procedures_repeat1, 2),
  [3610] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_procedures_repeat1, 2), SHIFT_REPEAT(2889),
  [3613] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_procedures_repeat1, 2), SHIFT_REPEAT(1222),
  [3616] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_derived_type_procedures_repeat1, 2),
  [3618] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_derived_type_procedures, 4),
  [3620] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_derived_type_procedures, 4),
  [3622] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_contains_statement, 1),
  [3624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_contains_statement, 1),
  [3626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(981),
  [3628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2914),
  [3630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2269),
  [3632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2270),
  [3634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2595),
  [3636] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2510),
  [3638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1434),
  [3640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2278),
  [3642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2350),
  [3644] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_label_start_expression, 2, .production_id = 20),
  [3646] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_label_start_expression, 2, .production_id = 20),
  [3648] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_procedure_statement_repeat2, 2), SHIFT_REPEAT(981),
  [3651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [3653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1170),
  [3655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(825),
  [3657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [3659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1828),
  [3661] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__transfer_items, 1),
  [3663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1864),
  [3665] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1842),
  [3667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1850),
  [3669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1842),
  [3671] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__transfer_items_repeat1, 2), SHIFT_REPEAT(1828),
  [3674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__transfer_items_repeat1, 2),
  [3676] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__transfer_items_repeat1, 2), SHIFT_REPEAT(1864),
  [3679] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__transfer_items_repeat1, 2), SHIFT_REPEAT(1842),
  [3682] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__transfer_items_repeat1, 2), SHIFT_REPEAT(1850),
  [3685] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__transfer_items_repeat1, 2), SHIFT_REPEAT(1842),
  [3688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2232),
  [3690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2231),
  [3692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2558),
  [3694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [3696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1511),
  [3698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2549),
  [3700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2553),
  [3702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 3, .production_id = 33),
  [3704] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 2, .production_id = 11),
  [3706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__transfer_items, 2),
  [3708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2309),
  [3710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2251),
  [3712] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_derived_type_procedures_repeat1, 1),
  [3714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_derived_type_procedures_repeat1, 1),
  [3716] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_select_type_statement_repeat1, 1),
  [3718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_type_statement_repeat1, 1),
  [3720] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2569),
  [3722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1414),
  [3724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1181),
  [3726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2673),
  [3728] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2673),
  [3730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2676),
  [3732] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_statement, 2),
  [3734] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__block_if_statement_repeat1, 2),
  [3736] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__block_if_statement_repeat1, 2),
  [3738] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__block_if_statement_repeat1, 2), SHIFT_REPEAT(2679),
  [3741] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__block_if_statement_repeat1, 2), SHIFT_REPEAT(2278),
  [3744] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concurrent_statement_repeat1, 2), SHIFT_REPEAT(2673),
  [3747] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_concurrent_statement_repeat1, 2), SHIFT_REPEAT(2673),
  [3750] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concurrent_statement_repeat1, 2), SHIFT_REPEAT(2676),
  [3753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_concurrent_statement_repeat1, 2),
  [3755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_statement, 1),
  [3757] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_select_case_statement_repeat1, 2),
  [3759] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_case_statement_repeat1, 2), SHIFT_REPEAT(2251),
  [3762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_case_statement_repeat1, 2),
  [3764] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2375),
  [3766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1463),
  [3768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2368),
  [3770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 3, .production_id = 11),
  [3772] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 3, .production_id = 21),
  [3774] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_statement, 3),
  [3776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1804),
  [3778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1549),
  [3780] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_statement, 3),
  [3782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__transfer_item, 3),
  [3784] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__transfer_item, 3),
  [3786] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__transfer_items_repeat1, 2),
  [3788] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__block_where_statement_repeat1, 2),
  [3790] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__block_where_statement_repeat1, 2), SHIFT_REPEAT(2460),
  [3793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__block_where_statement_repeat1, 2),
  [3795] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__block_where_statement_repeat1, 2), SHIFT_REPEAT(1181),
  [3798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__transfer_item, 1),
  [3800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__transfer_item, 1),
  [3802] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_function_statement_repeat1, 2), SHIFT_REPEAT(2549),
  [3805] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_function_statement_repeat1, 2), SHIFT_REPEAT(2553),
  [3808] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_statement_repeat1, 2),
  [3810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 4, .production_id = 21),
  [3812] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_statement, 2),
  [3814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1802),
  [3816] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_statement, 2),
  [3818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hollerith_constant, 1),
  [3820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hollerith_constant, 1),
  [3822] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 5, .production_id = 55),
  [3824] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 3, .production_id = 33),
  [3826] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__transfer_items_repeat1, 1),
  [3828] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__transfer_items_repeat1, 1),
  [3830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 4, .production_id = 55),
  [3832] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_statement, 4, .production_id = 33),
  [3834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 2, .production_id = 11),
  [3836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_select_rank_statement_repeat1, 2),
  [3838] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_select_rank_statement_repeat1, 2), SHIFT_REPEAT(2309),
  [3841] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_rank_statement_repeat1, 2),
  [3843] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_statement, 4),
  [3845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_statement, 4),
  [3847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_statement, 6),
  [3849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_statement, 6),
  [3851] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_language_binding, 6),
  [3853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_language_binding, 6),
  [3855] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 7, .production_id = 102),
  [3857] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 7, .production_id = 102),
  [3859] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_statement, 5),
  [3861] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_statement, 5),
  [3863] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_concurrent_statement_repeat1, 1),
  [3865] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_concurrent_statement_repeat1, 1),
  [3867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 6, .production_id = 93),
  [3869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 6, .production_id = 93),
  [3871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 4),
  [3873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 4),
  [3875] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 6, .production_id = 94),
  [3877] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 6, .production_id = 94),
  [3879] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 5),
  [3881] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 5),
  [3883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_locality, 4),
  [3885] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_locality, 4),
  [3887] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_cuda_kernel_argument_list_repeat1, 2), SHIFT_REPEAT(693),
  [3890] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 7, .production_id = 93),
  [3892] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 7, .production_id = 93),
  [3894] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_locality, 5),
  [3896] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_locality, 5),
  [3898] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enumerator_statement_repeat1, 2),
  [3900] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enumerator_statement_repeat1, 2), SHIFT_REPEAT(1549),
  [3903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enumerator_statement_repeat1, 2),
  [3905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enum_repeat1, 2),
  [3907] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enum_repeat1, 2), SHIFT_REPEAT(1463),
  [3910] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_repeat1, 2),
  [3912] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 9, .production_id = 112),
  [3914] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 9, .production_id = 112),
  [3916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1810),
  [3918] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_language_binding, 4),
  [3920] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_language_binding, 4),
  [3922] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_locality, 4, .production_id = 76),
  [3924] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_locality, 4, .production_id = 76),
  [3926] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__block_if_statement_repeat1, 1),
  [3928] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__block_if_statement_repeat1, 1),
  [3930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concurrent_header, 8, .production_id = 108),
  [3932] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concurrent_header, 8, .production_id = 108),
  [3934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [3936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [3938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [3940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1561),
  [3942] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ac_value_list, 2),
  [3944] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_namelist_statement_repeat1, 2), SHIFT_REPEAT(1492),
  [3947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_namelist_statement_repeat1, 2),
  [3949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__ac_value_list, 3, .production_id = 18),
  [3951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [3953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [3955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
  [3957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2343),
  [3959] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__block_where_statement_repeat1, 1),
  [3961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__block_where_statement_repeat1, 1),
  [3963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2187),
  [3965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2598),
  [3967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2321),
  [3969] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_select_rank_statement_repeat1, 1),
  [3971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_rank_statement_repeat1, 1),
  [3973] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_select_case_statement_repeat1, 1),
  [3975] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_select_case_statement_repeat1, 1),
  [3977] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2), SHIFT_REPEAT(1548),
  [3980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1022),
  [3982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1552),
  [3984] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_included_items, 4),
  [3986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1492),
  [3988] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_namelist_statement, 2),
  [3990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [3992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [3994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(823),
  [3996] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enumerator_statement_repeat1, 4),
  [3998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enumerator_statement_repeat1, 4),
  [4000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2601),
  [4002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_character_length, 1),
  [4004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2216),
  [4006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2104),
  [4008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1501),
  [4010] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_save_statement, 5),
  [4012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1885),
  [4014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [4016] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enum_repeat1, 1),
  [4018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enum_repeat1, 1),
  [4020] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1305),
  [4022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [4024] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_associate_statement_repeat1, 2),
  [4026] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_associate_statement_repeat1, 2), SHIFT_REPEAT(1517),
  [4029] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1228),
  [4031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1361),
  [4033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_targets, 2),
  [4035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2489),
  [4037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [4039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2398),
  [4041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2893),
  [4043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2879),
  [4045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1261),
  [4047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2755),
  [4049] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameter_statement_repeat1, 2),
  [4051] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameter_statement_repeat1, 2), SHIFT_REPEAT(1470),
  [4054] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1239),
  [4056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1337),
  [4058] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1302),
  [4060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1402),
  [4062] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1314),
  [4064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1405),
  [4066] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1301),
  [4068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1407),
  [4070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_procedure_attributes_repeat1, 2),
  [4072] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_procedure_attributes_repeat1, 2), SHIFT_REPEAT(1909),
  [4075] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1315),
  [4077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1406),
  [4079] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1235),
  [4081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1362),
  [4083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1410),
  [4085] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 5, .production_id = 60),
  [4087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1272),
  [4089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1403),
  [4091] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1294),
  [4093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1435),
  [4095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2341),
  [4097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2300),
  [4099] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_concurrent_header_repeat1, 2),
  [4101] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concurrent_header_repeat1, 2), SHIFT_REPEAT(1457),
  [4104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2578),
  [4106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [4108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [4110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 5, .production_id = 22),
  [4112] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1321),
  [4114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1450),
  [4116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(730),
  [4118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_value_range_list, 2),
  [4120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2301),
  [4122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2885),
  [4124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [4126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 5, .production_id = 12),
  [4128] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2288),
  [4130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2474),
  [4132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2473),
  [4134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2894),
  [4136] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1307),
  [4138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [4140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2708),
  [4142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1517),
  [4144] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_input_item_list, 2),
  [4146] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1308),
  [4148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1400),
  [4150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2786),
  [4152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 4, .production_id = 55),
  [4154] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_statement, 3),
  [4156] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_statement, 3),
  [4158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2245),
  [4160] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_output_item_list, 2),
  [4162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameters, 2),
  [4164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2747),
  [4166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [4168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2163),
  [4170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2060),
  [4172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1728),
  [4174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1909),
  [4176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2911),
  [4178] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1328),
  [4180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1365),
  [4182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1001),
  [4184] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_keyword_statement_repeat1, 2),
  [4186] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_keyword_statement_repeat1, 2), SHIFT_REPEAT(2163),
  [4189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1325),
  [4191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1358),
  [4193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__forall_control_expression_repeat1, 2),
  [4195] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__forall_control_expression_repeat1, 2), SHIFT_REPEAT(1464),
  [4198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2625),
  [4200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2313),
  [4202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 4, .production_id = 12),
  [4204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2422),
  [4206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1757),
  [4208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [4210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_range, 1),
  [4212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2699),
  [4214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2070),
  [4216] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1316),
  [4218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1424),
  [4220] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1319),
  [4222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1419),
  [4224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1499),
  [4226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_save_statement, 4),
  [4228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2249),
  [4230] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1324),
  [4232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1422),
  [4234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2328),
  [4236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2103),
  [4238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3),
  [4240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2339),
  [4242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2334),
  [4244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2332),
  [4246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(798),
  [4248] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1262),
  [4250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1441),
  [4252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1391),
  [4254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 2, .production_id = 12),
  [4256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [4258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [4260] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1257),
  [4262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1440),
  [4264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [4266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_statement, 5),
  [4268] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_save_statement_repeat1, 2), SHIFT_REPEAT(1501),
  [4271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_save_statement_repeat1, 2),
  [4273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1106),
  [4275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1327),
  [4277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1354),
  [4279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 8, .production_id = 60),
  [4281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_included_items_repeat1, 2),
  [4283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2506),
  [4285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2155),
  [4287] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_included_items_repeat1, 2), SHIFT_REPEAT(1022),
  [4290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_save_statement, 2),
  [4292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [4294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_statement, 3),
  [4296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [4298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1225),
  [4300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1448),
  [4302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(460),
  [4304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [4306] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1332),
  [4308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1433),
  [4310] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 4),
  [4312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1127),
  [4314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_result, 4),
  [4316] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 6, .production_id = 22),
  [4318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_statement, 8),
  [4320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2247),
  [4322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2376),
  [4324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1470),
  [4326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2293),
  [4328] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_equivalence_statement, 3),
  [4330] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_equivalence_statement_repeat1, 2), SHIFT_REPEAT(2293),
  [4333] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_equivalence_statement_repeat1, 2),
  [4335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2357),
  [4337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(574),
  [4339] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_use_statement_repeat1, 2), SHIFT_REPEAT(1493),
  [4342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_use_statement_repeat1, 2),
  [4344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [4346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2642),
  [4348] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2), SHIFT_REPEAT(475),
  [4351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2619),
  [4353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2620),
  [4355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2622),
  [4357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assumed_size, 1),
  [4359] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat3, 2),
  [4361] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat3, 2), SHIFT_REPEAT(2313),
  [4364] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_statement, 6),
  [4366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1882),
  [4368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [4370] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2285),
  [4372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2629),
  [4374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2631),
  [4376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2295),
  [4378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(598),
  [4380] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_equivalence_statement, 2),
  [4382] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat1, 2),
  [4384] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat1, 2), SHIFT_REPEAT(2300),
  [4387] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_derived_type_statement_repeat1, 2, .production_id = 62), SHIFT_REPEAT(1757),
  [4390] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_derived_type_statement_repeat1, 2, .production_id = 62),
  [4392] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_save_statement, 3),
  [4394] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_equivalence_set_repeat1, 2), SHIFT_REPEAT(682),
  [4397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2193),
  [4399] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_value_range_list_repeat1, 2), SHIFT_REPEAT(530),
  [4402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [4404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(622),
  [4406] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_cuda_kernel_argument_list_repeat1, 2), SHIFT_REPEAT(738),
  [4409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [4411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1495),
  [4413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_statement, 7),
  [4415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1883),
  [4417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_save_statement, 6),
  [4419] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_private_statement_repeat1, 2), SHIFT_REPEAT(1106),
  [4422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [4424] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_data_statement_repeat1, 2), SHIFT_REPEAT(648),
  [4427] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_data_statement_repeat1, 2),
  [4429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_statement, 2),
  [4431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(794),
  [4433] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat2, 2), SHIFT_REPEAT(1781),
  [4436] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat2, 2),
  [4438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [4440] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__io_arguments_repeat1, 2),
  [4442] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__io_arguments_repeat1, 2), SHIFT_REPEAT(1469),
  [4445] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_data_set_repeat1, 2), SHIFT_REPEAT(703),
  [4448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2280),
  [4450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2711),
  [4452] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_character_length, 2),
  [4454] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__declaration_targets_repeat1, 2), SHIFT_REPEAT(534),
  [4457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2486),
  [4459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1171),
  [4461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1172),
  [4463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(793),
  [4465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1886),
  [4467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [4469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1401),
  [4471] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 3, .production_id = 22),
  [4473] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_data_value_repeat1, 2, .production_id = 75), SHIFT_REPEAT(750),
  [4476] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_data_value_repeat1, 2, .production_id = 75),
  [4478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(433),
  [4480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1730),
  [4482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1879),
  [4484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(605),
  [4486] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_statement_repeat1, 1),
  [4488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1878),
  [4490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [4492] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameters, 1),
  [4494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 3, .production_id = 21),
  [4496] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 7, .production_id = 60),
  [4498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_targets, 3),
  [4500] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_included_items, 5),
  [4502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [4504] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cuda_kernel_argument_list, 4),
  [4506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_value, 4, .production_id = 74),
  [4508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(572),
  [4510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 2),
  [4512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [4514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1415),
  [4516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [4518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2796),
  [4520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2039),
  [4522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_set, 3),
  [4524] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_equivalence_set, 5),
  [4526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [4528] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_name, 1),
  [4530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1395),
  [4532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1502),
  [4534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_save_statement_repeat1, 4),
  [4536] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_access_specifier, 1),
  [4538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_specifier, 1),
  [4540] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__derived_type_qualifier, 1),
  [4542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__derived_type_qualifier, 1, .production_id = 23),
  [4544] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__derived_type_qualifier, 1, .production_id = 24),
  [4546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1528),
  [4548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2820),
  [4550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_base_type_specifier, 4),
  [4552] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2819),
  [4554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2793),
  [4556] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [4558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2919),
  [4560] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_equivalence_set, 6),
  [4562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_value, 5, .production_id = 92),
  [4564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1453),
  [4566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_range, 3),
  [4568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [4570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2645),
  [4572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [4574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2420),
  [4576] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_alias, 3, .production_id = 72),
  [4578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_derived_type_statement_repeat1, 2, .production_id = 61),
  [4580] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_namelist_statement_repeat1, 1),
  [4582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [4584] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assumed_shape, 1),
  [4586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_value, 6, .production_id = 101),
  [4588] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_set, 2),
  [4590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2042),
  [4592] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat2, 5),
  [4594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2260),
  [4596] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_implicit_statement, 2),
  [4598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2559),
  [4600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [4602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1360),
  [4604] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__declaration_targets_repeat1, 3),
  [4606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_value, 3),
  [4608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2287),
  [4610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [4612] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_read_statement, 2),
  [4614] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_format_identifier, 1),
  [4616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1519),
  [4618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2425),
  [4620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2107),
  [4622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [4624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2523),
  [4626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_file_position_statement, 1),
  [4628] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cuda_kernel_argument_list, 3),
  [4630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat2, 6),
  [4632] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_character_length, 5),
  [4634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_character_length, 4),
  [4636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2433),
  [4638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [4640] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_implicit_statement_repeat2, 7),
  [4642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1894),
  [4644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1496),
  [4646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [4648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1366),
  [4650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2396),
  [4652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2335),
  [4654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1195),
  [4656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(611),
  [4658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_declaration, 6),
  [4660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [4662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1139),
  [4664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1289),
  [4666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_procedure_statement, 4, .production_id = 87),
  [4668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [4670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1460),
  [4672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1197),
  [4674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1159),
  [4676] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_enum_statement, 1, .production_id = 19),
  [4678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1571),
  [4680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [4682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1466),
  [4684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1468),
  [4686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1663),
  [4688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2913),
  [4690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_statement, 5),
  [4692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1871),
  [4694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1474),
  [4696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1475),
  [4698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1656),
  [4700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1479),
  [4702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [4704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1486),
  [4706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__inline_forall_statement, 2),
  [4708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(382),
  [4710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [4712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [4714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [4716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [4718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [4720] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_forall_statement, 4),
  [4722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [4724] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_declaration, 2),
  [4726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1513),
  [4728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1516),
  [4730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [4732] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_forall_statement, 2, .production_id = 36),
  [4734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__file_position_spec, 5),
  [4736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [4738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [4740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [4742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [4744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [4746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [4748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [4750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [4752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [4754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1732),
  [4756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [4758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [4760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(623),
  [4762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(344),
  [4764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associate_statement, 6),
  [4766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2524),
  [4768] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 9),
  [4770] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 6, .production_id = 68),
  [4772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1313),
  [4774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(183),
  [4776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [4778] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_label_statement, 4),
  [4780] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 6, .production_id = 67),
  [4782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [4784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2257),
  [4786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2255),
  [4788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [4790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [4792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2252),
  [4794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(617),
  [4796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1506),
  [4798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1508),
  [4800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [4802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1140),
  [4804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(615),
  [4806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1142),
  [4808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1591),
  [4810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_case_statement, 6),
  [4812] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_select_statement, 2, .production_id = 66),
  [4814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1587),
  [4816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [4818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [4820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(600),
  [4822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2804),
  [4824] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 8, .production_id = 100),
  [4826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [4828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [4830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1582),
  [4832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1580),
  [4834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 6, .production_id = 82),
  [4836] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_case_statement, 8, .production_id = 99),
  [4838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [4840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1906),
  [4842] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_case_statement, 6, .production_id = 81),
  [4844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1504),
  [4846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 8, .production_id = 98),
  [4848] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 6, .production_id = 80),
  [4850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(175),
  [4852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1574),
  [4854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1214),
  [4856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [4858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(164),
  [4860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1570),
  [4862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(385),
  [4864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [4866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [4868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [4870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [4872] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_do_loop_statement, 3, .production_id = 45),
  [4874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [4876] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_where_statement, 6),
  [4878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [4880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [4882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [4884] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_loop_statement, 5),
  [4886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1725),
  [4888] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_where_statement, 3, .production_id = 65),
  [4890] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 6, .production_id = 88),
  [4892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1717),
  [4894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1718),
  [4896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [4898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [4900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [4902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1558),
  [4904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(331),
  [4906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [4908] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_close_statement, 6),
  [4910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [4912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1854),
  [4914] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_where_statement, 4),
  [4916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1719),
  [4918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [4920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [4922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(815),
  [4924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1715),
  [4926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [4928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [4930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(875),
  [4932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [4934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [4936] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 6, .production_id = 89),
  [4938] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_construct, 6),
  [4940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [4942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(414),
  [4944] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_forall_statement, 6),
  [4946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [4948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1598),
  [4950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(853),
  [4952] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 6),
  [4954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1601),
  [4956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1343),
  [4958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(413),
  [4960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [4962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [4964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1605),
  [4966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1564),
  [4968] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_file_position_statement, 2),
  [4970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1895),
  [4972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1144),
  [4974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__file_position_spec, 1),
  [4976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [4978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1147),
  [4980] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_if_statement, 2, .production_id = 64),
  [4982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1727),
  [4984] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2803),
  [4986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1614),
  [4988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2075),
  [4990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [4992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2268),
  [4994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1616),
  [4996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(584),
  [4998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [5000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [5002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameterized_read_statement, 3),
  [5004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [5006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2035),
  [5008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(391),
  [5010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1816),
  [5012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2037),
  [5014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [5016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1539),
  [5018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 3, .production_id = 11),
  [5020] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_loop_statement, 6),
  [5022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(827),
  [5024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1541),
  [5026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1500),
  [5028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [5030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [5032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(323),
  [5034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [5036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1542),
  [5038] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_modification, 2),
  [5040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(410),
  [5042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(666),
  [5044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 5),
  [5046] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 3, .production_id = 12),
  [5048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_selector, 3),
  [5050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_label, 1, .production_id = 46),
  [5052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_do_loop_statement, 2, .production_id = 26),
  [5054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(912),
  [5056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_loop_statement, 8),
  [5058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1820),
  [5060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(330),
  [5062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2077),
  [5064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1505),
  [5066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2078),
  [5068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [5070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associate_statement, 8),
  [5072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [5074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2141),
  [5076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1628),
  [5078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_associate_statement, 3, .production_id = 95),
  [5080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2310),
  [5082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1631),
  [5084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unlimited_polymorphic, 1),
  [5086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_include_statement, 2, .production_id = 17),
  [5088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [5090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [5092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [5094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(685),
  [5096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [5098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1636),
  [5100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [5102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1731),
  [5104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [5106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [5108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1428),
  [5110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2713),
  [5112] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_construct, 5),
  [5114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1477),
  [5116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1642),
  [5118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 2),
  [5120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2053),
  [5122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [5124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1644),
  [5126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [5128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2650),
  [5130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [5132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [5134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1553),
  [5136] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_loop_statement, 7),
  [5138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_loop_statement, 4),
  [5140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1556),
  [5142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [5144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [5146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [5148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [5150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [5152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1726),
  [5154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [5156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(370),
  [5158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [5160] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 8),
  [5162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [5164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1192),
  [5166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2665),
  [5168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [5170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [5172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [5174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_statement, 4),
  [5176] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 6, .production_id = 60),
  [5178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(372),
  [5180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [5182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2638),
  [5184] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_where_statement, 2, .production_id = 47),
  [5186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2323),
  [5188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(375),
  [5190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [5192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(376),
  [5194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2275),
  [5196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1654),
  [5198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_statement, 3),
  [5200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1659),
  [5202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_if_statement, 3, .production_id = 77),
  [5204] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_where_statement, 5),
  [5206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [5208] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 7, .production_id = 88),
  [5210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1498),
  [5212] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 7),
  [5214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1665),
  [5216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1669),
  [5218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(966),
  [5220] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_loop_statement, 3),
  [5222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_name, 1, .production_id = 13),
  [5224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1724),
  [5226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 2),
  [5228] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_declaration, 4),
  [5230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_label_statement, 3),
  [5232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1524),
  [5234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1676),
  [5236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1270),
  [5238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2415),
  [5240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1521),
  [5242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1678),
  [5244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2286),
  [5246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associate_statement, 9),
  [5248] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_arithmetic_if_statement, 7),
  [5250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(481),
  [5252] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__inline_if_statement, 3),
  [5254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1947),
  [5256] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__inline_where_statement, 3),
  [5258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [5260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(744),
  [5262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1819),
  [5264] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 7, .production_id = 89),
  [5266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [5268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1930),
  [5270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 10),
  [5272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [5274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [5276] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 7, .production_id = 82),
  [5278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [5280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2284),
  [5282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [5284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2191),
  [5286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_call, 4, .production_id = 16),
  [5288] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_block_construct_statement, 2, .production_id = 27),
  [5290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_forall_statement, 5),
  [5292] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_forall_statement, 3, .production_id = 57),
  [5294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2227),
  [5296] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_construct, 4),
  [5298] = {.entry = {.count = 1, .reusable = true}}, SHIFT(156),
  [5300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(324),
  [5302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2355),
  [5304] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_case_statement, 7, .production_id = 99),
  [5306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2336),
  [5308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [5310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [5312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [5314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1687),
  [5316] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_construct, 3),
  [5318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_declaration, 5),
  [5320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1691),
  [5322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(927),
  [5324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [5326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_use_statement, 4, .production_id = 22),
  [5328] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 7, .production_id = 98),
  [5330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 7, .production_id = 80),
  [5332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1693),
  [5334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1708),
  [5336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [5338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1838),
  [5340] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_format_statement, 4, .production_id = 49),
  [5342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 4, .production_id = 21),
  [5344] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 4),
  [5346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_where_statement, 7),
  [5348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2731),
  [5350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [5352] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_open_statement, 3),
  [5354] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 7, .production_id = 100),
  [5356] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_inquire_statement, 3),
  [5358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1379),
  [5360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [5362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2531),
  [5364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(388),
  [5366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2444),
  [5368] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_procedure_statement, 3, .production_id = 33),
  [5370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [5372] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_close_statement, 4),
  [5374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [5376] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 5, .production_id = 55),
  [5378] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_read_statement, 4),
  [5380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [5382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2279),
  [5384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [5386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1705),
  [5388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [5390] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_case_statement, 5),
  [5392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1700),
  [5394] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subroutine_statement, 4, .production_id = 33),
  [5396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1697),
  [5398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1695),
  [5400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [5402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [5404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(462),
  [5406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(461),
  [5408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [5410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [5412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [5414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(531),
  [5416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [5418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [5420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [5422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [5424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(592),
  [5426] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_case_statement, 7, .production_id = 81),
  [5428] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_read_statement, 1),
  [5430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [5432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(39),
  [5434] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_forall_statement, 1),
  [5436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [5438] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_where_statement, 1),
  [5440] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 1),
  [5442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [5444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [5446] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__file_position_spec, 3),
  [5448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [5450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [5452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1288),
  [5454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [5456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [5458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2753),
  [5460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2429),
  [5462] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2426),
  [5464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1136),
  [5466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1538),
  [5468] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_block_construct_statement, 3, .production_id = 48),
  [5470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [5472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1277),
  [5474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [5476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1137),
  [5478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1138),
  [5480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [5482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [5484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(41),
  [5486] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [5488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2512),
  [5490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [5492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2261),
  [5494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1471),
  [5496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1473),
  [5498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_rank_statement, 5, .production_id = 68),
  [5500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1269),
  [5502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [5504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [5506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(77),
  [5508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2920),
  [5510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [5512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1494),
  [5514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_modification, 3),
  [5516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [5518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [5520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1723),
  [5522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1480),
  [5524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2192),
  [5526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1515),
  [5528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1822),
  [5530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [5532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1243),
  [5534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_select_statement, 3, .production_id = 84),
  [5536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2431),
  [5538] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2432),
  [5540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1446),
  [5542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [5544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [5546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1229),
  [5548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [5550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(13),
  [5552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [5554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_type_statement, 5, .production_id = 67),
  [5556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [5558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [5560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [5562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1293),
  [5564] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_if_statement, 11),
  [5566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [5568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1282),
  [5570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1273),
  [5572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2588),
  [5574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [5576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [5578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1426),
  [5580] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_write_statement, 4),
  [5582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [5584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(997),
  [5586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [5588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(29),
  [5590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1236),
  [5592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1311),
  [5594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1924),
  [5596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [5598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [5600] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_associate_statement, 2, .production_id = 85),
  [5602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_associate_statement, 7),
  [5604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(937),
  [5606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1304),
  [5608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [5610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [5612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1323),
  [5614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1318),
  [5616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(11),
  [5618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [5620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [5622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_close_statement, 7),
  [5624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(933),
  [5626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1291),
  [5628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1523),
  [5630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1254),
  [5632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1252),
  [5634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__file_position_spec, 4),
  [5636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [5638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [5640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [5642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1246),
  [5644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2152),
  [5646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [5648] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__file_position_spec, 6),
  [5650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_close_statement, 5),
  [5652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [5654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1995),
  [5656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1510),
  [5658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2109),
  [5660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [5662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2794),
  [5664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2795),
  [5666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [5668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1461),
  [5670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [5672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_declaration, 3),
  [5674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1465),
  [5676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1798),
  [5678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1703),
  [5680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [5682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__block_forall_statement, 3),
  [5684] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_enum_statement, 2, .production_id = 37),
  [5686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1537),
  [5688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [5690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [5692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2834),
  [5694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2835),
  [5696] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_file_line, 4, .production_id = 41),
  [5698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_file_line, 5, .production_id = 41),
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_fortran_external_scanner_create(void);
void tree_sitter_fortran_external_scanner_destroy(void *);
bool tree_sitter_fortran_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_fortran_external_scanner_serialize(void *, char *);
void tree_sitter_fortran_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef _WIN32
#define extern __declspec(dllexport)
#endif

extern const TSLanguage *tree_sitter_fortran(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_fortran_external_scanner_create,
      tree_sitter_fortran_external_scanner_destroy,
      tree_sitter_fortran_external_scanner_scan,
      tree_sitter_fortran_external_scanner_serialize,
      tree_sitter_fortran_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
